#include "clusterMarkers.h"

double sl_main(int numMarkers,int* markersUsed, Bool diseaseUsed, int progCode,double specialThetas, PedigreeDataStruct *currPedigree);
void createDataForCluster(int clusterSize,int *markersUsed);
void createClusterOneMarker(int markerInd);
int checkNumAllelesInCluster(int clusterInd);


void clusterMarkers(int maxClusterSize){
	int pedInd,   locusInd;
	int clusterSize = 0;
	int *markersUsed;
	double prob;
	PedigreeDataStruct *currPedigree;
	Bool maxSize, changeSize;
	Bool flagOfSingeltonMarker = FALSE;

	myMalloc(markersUsed,2*maxClusterSize*sizeof(int));
	ClusterData = NULL;
	printf("Clustering markers: ");

	clusterSize = 0; // current cluster size
	locusInd = 0;  //current marker
	maxSize = FALSE; //If current cluster reached it maximum size (without recomb)
	changeSize = FALSE; // If one of the pedigrees changed cluster size

	// iterate over all loci
	while(locusInd < ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci){

		if (!OkMarkers[locusInd]){
			locusInd++;
			continue;
		}

		//If we want to increase cluster size
		if (!maxSize){
			markersUsed[clusterSize] = locusInd;
			clusterSize++;
		}
		prob = getcMToRec((LociData[markersUsed[clusterSize - 1]].malePosition - LociData[markersUsed[0]].malePosition), ProgramData.haldane);
		for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){
			if ( prob * 2 * Pedigrees.pedigrees[pedInd].numNonFounders >= 1 
				|| prob > 0.02){
				locusInd--;
				clusterSize--;
				maxSize = TRUE;
			}
		}

		// if we want to cluster
		if (maxSize || clusterSize >= maxClusterSize ||
			locusInd == ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci -1){

				pedInd = 0, currPedigree = Pedigrees.pedigrees;
				// iterate over all pedigrees
				while (pedInd < Pedigrees.numOfPedigrees){ 
					// find max possible cluster size
					while (sl_main(clusterSize,markersUsed,FALSE,SUPER_HAPLO,0, currPedigree) == -DBL_MAX ){
						clusterSize--;
						locusInd--;
						maxSize = TRUE;
						changeSize = TRUE;
						if (clusterSize == 1){
							break;
						}
					}
					/* If it's singelton cluster, update all families 
					and continue to next marker */
					if (clusterSize == 1){		
						createClusterOneMarker(markersUsed[0]);
						flagOfSingeltonMarker = TRUE;
						break;
					}

					/* If cluster size decreased and it not first pedigree,
					recalculate all pedigrees */
					if (changeSize ){
						changeSize = FALSE;
						pedInd = 0; currPedigree = Pedigrees.pedigrees;
						continue;
					}
					pedInd++;
					currPedigree++;
				} // end while (pedInd < Pedigrees.numOfPedigrees)

				if ((checkNumAllelesInCluster(ProgramData.numberOfClusters) >= MIN_NUMBER_ALLELES_PER_CLUSTER )
					)
					{

						if (clusterSize > 1) {
							createDataForCluster(clusterSize,markersUsed);
							if (ProgramData.numberOfClusters > 0 && 
								(ClusterData[ProgramData.numberOfClusters].malePosition) ==
								ClusterData[ProgramData.numberOfClusters-1].malePosition){
									printf("!!!same position !!!\n");
								}
							else{
								ProgramData.numberOfClusters++;
							}
						}else {
							//createClusterOneMarker(markersUsed[0]);
							ProgramData.numberOfClusters++;
							flagOfSingeltonMarker = FALSE; // so we wont add twice
						}
						clusterSize = 0;
						maxSize = FALSE;
						changeSize = FALSE;
					}else if  (clusterSize >= 2*maxClusterSize || maxSize){
						clusterSize = 0;
						maxSize = FALSE;
						changeSize = FALSE;
					}
			}// end  if (clusterSize >= maxClusterSize )
			
			if(flagOfSingeltonMarker == TRUE)
			{
				ProgramData.numberOfClusters++;
				flagOfSingeltonMarker = FALSE;
			}
			locusInd++;
	} //end while(locusInd < ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci){
	free(markersUsed);
}

void createClusterOneMarker(int markerInd){
	int pedInd, clusterInd;
	PersonDataStruct *personPtr;
	LocusDataStruct *clusterPtr;
	printf("%d ",ProgramData.numberOfClusters+1);
	myRealloc(ClusterData,(sizeof(LocusDataStruct)*(ProgramData.numberOfClusters+1)));
	clusterPtr = ClusterData + ProgramData.numberOfClusters;
	strcpy(clusterPtr->locusName, LociData[markerInd].locusName);
	clusterPtr->numOfAlleles = 2;
	clusterPtr->malePosition = LociData[markerInd].malePosition ;
	clusterPtr->femalePosition = LociData[markerInd].femalePosition ;
	clusterPtr->type = NUMBERED_LOCUS;
	myMalloc(clusterPtr->alleleFreq, (sizeof(double)*clusterPtr->numOfAlleles) );
	clusterPtr->alleleFreq[0] = LociData[markerInd].alleleFreq[0];
	clusterPtr->alleleFreq[1] = LociData[markerInd].alleleFreq[1];
	clusterInd = ProgramData.numberOfClusters;

	for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){
		for (personPtr = Pedigrees.pedigrees[pedInd].people;
			personPtr < Pedigrees.pedigrees[pedInd].people + Pedigrees.pedigrees[pedInd].numPeople;  personPtr++){
				myRealloc((personPtr->clustersAlleles), ((ProgramData.numberOfClusters+1)*sizeof(AlleleDataStruct)));
				personPtr->clustersAlleles[clusterInd].allele1 =
					personPtr->markersAlleles[markerInd].allele1;
				personPtr->clustersAlleles[clusterInd].allele2 =
					personPtr->markersAlleles[markerInd].allele2;
			}
	}
}

void createDataForCluster(int clusterSize,int *markersUsed){
	int  markerInd, alleleInd;
	int *allelesindex;

	double freq;
	LocusDataStruct *clusterPtr, *markerPtr;


	printf("%d ",ProgramData.numberOfClusters+1);
	fflush(NULL);
	myRealloc(ClusterData,(sizeof(LocusDataStruct)*(ProgramData.numberOfClusters+1)));
	clusterPtr = ClusterData + ProgramData.numberOfClusters;
	clusterPtr->locusName[0] ='\0';
	clusterPtr->numOfAlleles = 1;
	clusterPtr->malePosition = (LociData[markersUsed[0]].malePosition + LociData[markersUsed[clusterSize -1]].malePosition)/2;
	clusterPtr->femalePosition = (LociData[markersUsed[0]].femalePosition + LociData[markersUsed[clusterSize -1]].femalePosition)/2;
	clusterPtr->type = NUMBERED_LOCUS;

	for (markerInd = 0; markerInd < clusterSize; markerInd++ ){
		markerPtr = LociData + markersUsed[markerInd];
		clusterPtr->numOfAlleles   *= markerPtr->numOfAlleles;
		if (strlen(clusterPtr->locusName) + strlen(markerPtr->locusName) < (MAX_CLUSTER_NAME - 3)) {
			strcat( clusterPtr->locusName, markerPtr->locusName);
			strcat( clusterPtr->locusName, " ");
		}
	}
#if (CLUSTER_FREQ == 2)
	clusterPtr->numOfAlleles  = checkNumAllelesInCluster(ProgramData.numberOfClusters) + 1;
#endif 

	myMalloc(allelesindex,(clusterSize*sizeof(int)));
	myMalloc(clusterPtr->alleleFreq, (sizeof(double)*clusterPtr->numOfAlleles) );
	memset(allelesindex,0,clusterSize*sizeof(int));
#if (CLUSTER_FREQ == 1)
	for (alleleInd = 0; alleleInd < clusterPtr->numOfAlleles; alleleInd++){
		clusterPtr->alleleFreq[alleleInd] = 1;

		carry = 1;
		for (markerInd =0; markerInd < clusterSize; markerInd++ ){
			markerPtr = LociData + markersUsed[markerInd];
			clusterPtr->alleleFreq[alleleInd] *= 
				markerPtr->alleleFreq[allelesindex[markerInd]];
			nextCarry = (allelesindex[markerInd] + carry >= markerPtr->numOfAlleles)?1:0;
			allelesindex[markerInd] = (allelesindex[markerInd] + carry)%markerPtr->numOfAlleles;
			carry = nextCarry;
		}
	}
	free(allelesindex);
#elif (CLUSTER_FREQ == 2)

	freq = 1.0 / clusterPtr->numOfAlleles;
	for (alleleInd = 0; alleleInd < clusterPtr->numOfAlleles; alleleInd++){
		clusterPtr->alleleFreq[alleleInd] = freq;
	}
#endif
	free(allelesindex);
}
int findAlleleNum(int *mapAlleles, int num){
	int i;
	for (i =0; i < MAX_DIF_ALLELES; i++ ){
		if (mapAlleles[i] == num || mapAlleles[i] == 0 ){
			mapAlleles[i] = num;
			return i+1;
		}
	}
	printf(stderr, "Reached 100 different haplotypes in cluster\n");
	return MAX_DIF_ALLELES;
}

int checkNumAllelesInCluster(int clusterInd){
	int mapAlleles[MAX_DIF_ALLELES] = {0 };
	int pedInd, personInd;
	PersonDataStruct *personPtr;
	int numDifAlleles = 0;

	for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){
		for (personInd =0, personPtr = Pedigrees.pedigrees[pedInd].people;
			personInd < Pedigrees.pedigrees[pedInd].numPeople; personInd++, personPtr++){
				if (personPtr->clustersAlleles[clusterInd].allele1 > 0 	){
					personPtr->clustersAlleles[clusterInd].allele1 = findAlleleNum(mapAlleles,personPtr->clustersAlleles[clusterInd].allele1);
				}
				/*else if( personPtr->clustersAlleles[clusterInd].allele1 < 0  ){
					personPtr->clustersAlleles[clusterInd].allele1 = 0;
					//reportError?
				}*/
				if (personPtr->clustersAlleles[clusterInd].allele2 > 0 ){
					personPtr->clustersAlleles[clusterInd].allele2 = findAlleleNum(mapAlleles,personPtr->clustersAlleles[clusterInd].allele2);
				}
				/*else if( personPtr->clustersAlleles[clusterInd].allele2 < 0 ){
					personPtr->clustersAlleles[clusterInd].allele2 = 0;
					//reportError?
				}*/
			}
	}
	return findAlleleNum(mapAlleles, -1) - 1;
}
 /*
int checkNumAllelesInCluster(int clusterInd){
	int pedInd, personInd;
	PersonDataStruct *personPtr;
	int numDifAlleles = 0;

	int alleleNewNumbers[MAX_NUM_ALLELES_PER_CLUSTER] = {0};
	for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){

		for (personInd =0, personPtr = Pedigrees.pedigrees[pedInd].people;
			personInd < Pedigrees.pedigrees[pedInd].numPeople; personInd++, personPtr++){

				if (personPtr->clustersAlleles[clusterInd].allele1 > 0 &&
					personPtr->clustersAlleles[clusterInd].allele1 < MAX_NUM_ALLELES_PER_CLUSTER){
						if (alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele1] == 0){
							alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele1]= ++numDifAlleles;
						}
						personPtr->clustersAlleles[clusterInd].allele1 = alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele1];
					}
					if (personPtr->clustersAlleles[clusterInd].allele2 > 0 &&
						personPtr->clustersAlleles[clusterInd].allele2 < MAX_NUM_ALLELES_PER_CLUSTER){
						
							if (alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele2] == 0){
								alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele2]= ++numDifAlleles;
							}
							personPtr->clustersAlleles[clusterInd].allele2 = alleleNewNumbers[personPtr->clustersAlleles[clusterInd].allele2];
						}
			}
	}




	return numDifAlleles;
}

*/
