#ifndef COMMON_H
#define COMMON_H



#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <math.h>



void printStdErr(char *str);
void printStdErrWithParamNoExit(char *message, ...) ;
void printStdErrWithParam(char *message, ...);
void mallocErr(char *message);
void printFile(char *filename);

/* The distance (in Morgans) used to indicate a recombination fraction of 0.5. */

#define		UNLINKED_DIST		1000

/* Computes the distance in cM from the given recombination fraction 
   (probability), using either Haldane's or Kosambi's map function. */

#define	getRecTocM(rec, haldane) \
	(rec > 0.499) ? UNLINKED_DIST : \
	(haldane ? (log(1 - 2 * rec) * (-50)) : (25*log((1+2*rec)/(1-2*rec))));

/* Computes the recombination fraction (probability) from the given distance 
   in Morgans, using Haldane's or Kosambi's map function. */

#define getcMToRec(dist, haldane) \
	haldane ? (0.5 * (1 - exp(-0.02 * dist))) : (0.5*(exp(0.04*dist)-1)/(exp(0.04*dist)+1));

#define myMalloc(pointer, size) \
		pointer = malloc(size); \
		if (pointer == NULL) { \
			printf("Size: %u\n", size); \
			mallocErr("pointer"); \
		} 

#define myCalloc(pointer, num,  size) \
		pointer = calloc(num, size); \
		if (pointer == NULL) { \
			printf("Size: %u\n", num*size); \
			mallocErr("pointer"); \
		} 

#define myRealloc(pointer, size) \
		pointer = realloc(pointer,size); \
		if (pointer == NULL) { \
			printf("Size: %u \n", size); \
			mallocErr("pointer"); \
		} 



#endif
