#include "output.h"

void printClusterPedfile(char *name);
void printClusterDatafile(int numClusters, char *name);
void printMarkerPedfile(char *name);
void printMarkerDatafile(int numClusters, char *name);
void printAffectedLocus(FILE *outstream);
void printComplexLocus(FILE *outstream);
int findClustersAtDist(double dist);
int findMarkersAtDist(double dist);


void printResultClusters(){
	int numClusters;
	//	numClusters = findClustersAtDist(10);
	//	printClusterPedfile("10cM");
	//	printClusterDatafile(numClusters, "10cM");
	//	numClusters = findClustersAtDist(1);
	//	printClusterPedfile("1cM");
	//	printClusterDatafile(numClusters, "1cM");
	numClusters = findClustersAtDist(0);
	printClusterPedfile("All");
	printClusterDatafile(numClusters, "All");

}	

void printOkMarkers(){
	int numMarkers;
	numMarkers = findMarkersAtDist(10);
	printMarkerPedfile("10cM");
	printMarkerDatafile(numMarkers, "10cM");
	numMarkers = findMarkersAtDist(1);
	printMarkerPedfile("1cM");
	printMarkerDatafile(numMarkers, "1cM");
	numMarkers = findMarkersAtDist(0);
	printMarkerPedfile("All");
	printMarkerDatafile(numMarkers, "All");
}


int findMarkersAtDist(double dist){
	int markerInd;
	int countPrintMarkers;
	double curDist;

markerInd = 0;
while (!OkMarkers[markerInd] ){
	markerInd++;
}
LociData[markerInd].printFlag = TRUE;
countPrintMarkers = 1;

for (markerInd++,curDist = 0;markerInd < ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci; ++markerInd){
			curDist = curDist   + LociData[markerInd].femalePosition - 
				LociData[markerInd-1].femalePosition;
			if (!OkMarkers[markerInd] )
				continue;
			if (curDist >= dist){
				LociData[markerInd].printFlag = TRUE;
				curDist = 0;
				countPrintMarkers++;
			}else{
				LociData[markerInd].printFlag = FALSE;
			}
}
		return countPrintMarkers;
}


void printMarkerPedfile(char *name){
	FILE *outstream;
	int pedInd, personInd, markerInd;
	PersonDataStruct *personPtr;
	AlleleDataStruct *markerPtr;
	char fullName[100];

	sprintf(fullName,"%s%s.ped",OK_PED_FILENAME,name);
	outstream = fopen(fullName,"w");

	for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){
		for (personInd =0, personPtr = Pedigrees.pedigrees[pedInd].people;
			personInd < Pedigrees.pedigrees[pedInd].numPeople; personInd++, personPtr++){
				fprintf(outstream,"%d %d %d %d %d %d %d %d 0\t", pedInd+1,
					personPtr->id, personPtr->father,personPtr->mother,
					personPtr->firstChild, personPtr->siblingFather, personPtr->siblingMother,
					personPtr->sex);
				fprintf(outstream,"%d ",personPtr->status);
				if (DiseaseData.numOfLiabClasses > 1){
					fprintf(outstream,"%d",personPtr->liabClass);
				}
				fprintf(outstream,"\t");
				for (markerInd = 0, markerPtr = personPtr->markersAlleles;
					markerInd < ProgramData.numberOfLoci -  ProgramData.numberOfDiseaseLoci; markerInd++, markerPtr++){
						if (OkMarkers[markerInd] && LociData[markerInd].printFlag == TRUE )
							fprintf(outstream,"%d %d\t",markerPtr->allele1, markerPtr->allele2);
					}
					fprintf(outstream,"\n");
			}

	}
	fclose(outstream);
}

void printMarkerDatafile(int numMarkers, char *name){
	FILE *outstream;
	int markerInd, i;
	int tmp, numLoci;
	double rec;
	LocusDataStruct *markerPtr;
	char fullName[100];

	sprintf(fullName,"%s%s.dat",OK_DAT_FILENAME,name);
	outstream = fopen(fullName,"w");

	tmp = (ProgramData.numberOfDiseaseLoci == 1)?0:2;
	numLoci = numMarkers + ProgramData.numberOfDiseaseLoci;
	fprintf(outstream,"%d %d %d 8 %d", numLoci,
						ProgramData.riskLocus, ProgramData.sexLinked, tmp);
	fprintf(outstream,"<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI\n");
	fprintf(outstream,"0 0.0 0.0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n");
	for (markerInd = 0; markerInd < numLoci; markerInd++){
		fprintf(outstream,"%d ",markerInd+1);
	}
	fprintf(outstream,"\n");
	if (ProgramData.numberOfDiseaseLoci == 1){
		 printAffectedLocus(outstream);
	}
	if (ProgramData.numberOfDiseaseLoci == 2){
		 printComplexLocus(outstream);
	}
	//print allele freq
	for (markerInd = 0, markerPtr = LociData;
		markerInd < ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci; markerInd++, markerPtr++){
			if (!OkMarkers[markerInd] || LociData[markerInd].printFlag == FALSE)
				continue;
			fprintf(outstream,"3 %d  #%s#<< NUMBERED LOCUS, NUMBER OF ALLELES\n",
				markerPtr->numOfAlleles, markerPtr->locusName);
			for ( i =0; i < markerPtr->numOfAlleles; i++){
				fprintf(outstream,"%lf ",markerPtr->alleleFreq[i]);
			}
		fprintf(outstream,"\n");
	}
	fprintf(outstream,"%d %d << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n",
				ProgramData.sexDiff, ProgramData.interference);	
	//recombination values - first ignored
	fprintf(outstream,"0 ");
	for (i = 0; i <  ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci ; i++){
		if (!OkMarkers[i])
			continue;
		tmp = i;
		break;
	}
	for (i = tmp+1; i <  ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci ; i++){
		if (!OkMarkers[i] || LociData[i].printFlag == FALSE)
			continue;
		rec = getcMToRec((LociData[i].malePosition - LociData[tmp].malePosition), ProgramData.haldane);
		fprintf(outstream,"%lf ",rec);
		tmp =i;
	}
	fprintf(outstream,"\n");
	fprintf(outstream,"1\n -n 2 1 %d\n",numMarkers);
	fclose(outstream);
	
}
void printClusterPedfile(char *name){
	FILE *outstream;
	int pedInd, personInd, clusterInd;
	PersonDataStruct *personPtr;
	AlleleDataStruct *clusterPtr;
	char fullName[100];
	sprintf(fullName,"%s%s.ped",OUT_PED_FILENAME,name);
	outstream = fopen(fullName,"w");
	for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){
		for (personInd =0, personPtr = Pedigrees.pedigrees[pedInd].people;
			personInd < Pedigrees.pedigrees[pedInd].numPeople; personInd++, personPtr++){
				fprintf(outstream,"%d %d %d %d %d %d %d %d 0\t", pedInd+1,
					personPtr->id, personPtr->father,personPtr->mother,
					personPtr->firstChild, personPtr->siblingFather, personPtr->siblingMother,
					personPtr->sex);
				fprintf(outstream,"%d ",personPtr->status);
				if (DiseaseData.numOfLiabClasses > 1){
					fprintf(outstream,"%d",personPtr->liabClass);
				}
				fprintf(outstream,"\t");
				for (clusterInd = 0, clusterPtr = personPtr->clustersAlleles;
					clusterInd < ProgramData.numberOfClusters; clusterInd++, clusterPtr++){

					if( ClusterData[clusterInd].printFlag == FALSE)
						continue;

                    fprintf(outstream,"%d %d\t",clusterPtr->allele1, clusterPtr->allele2);
				}
				fprintf(outstream,"\n");
			}

	}
	fclose(outstream);
}


int findClustersAtDist(double dist){
	int clusterInd;
	int countPrintClusters;
	LocusDataStruct *clusterPtr;
	double curDist;

	if( ClusterData == NULL ) return 0; // need to fix this case
	ClusterData[0].printFlag = TRUE;
	countPrintClusters = 1;

	for (clusterInd = 1, clusterPtr = ClusterData + 1 , curDist = 0;
		clusterInd < ProgramData.numberOfClusters; clusterInd++, clusterPtr++){
			curDist = curDist   + ClusterData[clusterInd].femalePosition - 
				ClusterData[clusterInd-1].femalePosition;
			if (curDist >= dist){
				ClusterData[clusterInd].printFlag = TRUE;
				curDist = 0;
				countPrintClusters++;
			}else{
				ClusterData[clusterInd].printFlag = FALSE;
			}
		}
		return countPrintClusters;
}


void printClusterDatafile(int numClusters, char *name){
	FILE *outstream;
	int clusterInd, i;
	int tmp, numLoci;
	double rec, lastPosition;
	LocusDataStruct *clusterPtr;
	char fullName[100];
	sprintf(fullName,"%s%s.dat",OUT_DAT_FILENAME,name);
	outstream = fopen(fullName,"w");

	tmp = (ProgramData.numberOfDiseaseLoci == 1)?0:2;
	 numLoci = numClusters + ProgramData.numberOfDiseaseLoci;
	fprintf(outstream,"%d %d %d 8 %d", numLoci,
						ProgramData.riskLocus, ProgramData.sexLinked, tmp);
	fprintf(outstream,"<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI\n");
	fprintf(outstream,"0 0.0 0.0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n");
	for (clusterInd = 0; clusterInd < numLoci; clusterInd++){
		fprintf(outstream,"%d ",clusterInd+1);
	}
	fprintf(outstream,"\n");
	if (ProgramData.numberOfDiseaseLoci == 1){
		 printAffectedLocus(outstream);
	}
	if (ProgramData.numberOfDiseaseLoci == 2){
		 printComplexLocus(outstream);
	}
	//print allele freq
	for (clusterInd = 0, clusterPtr = ClusterData;
		clusterInd < ProgramData.numberOfClusters; clusterInd++, clusterPtr++){
			if (clusterPtr->printFlag == FALSE)
				continue;
			fprintf(outstream,"3 %d  #%s#<< NUMBERED LOCUS, NUMBER OF ALLELES\n",
				clusterPtr->numOfAlleles, clusterPtr->locusName);
			for ( i =0; i < clusterPtr->numOfAlleles; i++){
				fprintf(outstream,"%lf ",clusterPtr->alleleFreq[i]);
			}
		fprintf(outstream,"\n");
	}
	fprintf(outstream,"%d %d << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)\n",
				ProgramData.sexDiff, ProgramData.interference);	
	//recombination values - first ignored
	fprintf(outstream,"0 ");
	if( ClusterData != NULL ){ // fix
		lastPosition = ClusterData[0].femalePosition;
	}
	for (i = 1; i < ProgramData.numberOfClusters; i++){
		if (ClusterData[i].printFlag == TRUE){
			rec = getcMToRec((ClusterData[i].femalePosition - lastPosition), ProgramData.haldane);
			fprintf(outstream,"%lf ",rec);
			lastPosition = ClusterData[i].femalePosition;
		}
	}
	fprintf(outstream,"\n");
	fprintf(outstream,"1\n -n 2 1 %d\n",numClusters);
	fclose(outstream);
	
}

void printAffectedLocus(FILE *outstream){
	int i;

	fprintf(outstream,"1 2\n %lf %lf\n",DiseaseData.freq[0],DiseaseData.freq[1]);
	
	fprintf(outstream,"%d\n",DiseaseData.numOfLiabClasses);
	for (i =0; i < DiseaseData.numOfLiabClasses; i++){
		fprintf(outstream,"%lf %lf %lf\n",DiseaseData.femalePenetrances[i][0],
DiseaseData.femalePenetrances[i][1], DiseaseData.femalePenetrances[i][2]);
		if (ProgramData.sexLinked){
				fprintf(outstream,"%lf %lf %lf\n",DiseaseData.malePenetrances[i][0],
					DiseaseData.malePenetrances[i][1]);
		}
	}
}

void printComplexLocus(FILE *outstream){
	int i,j;

	fprintf(outstream,"4 2\n %lf %lf\n",DiseaseData.freq[0],DiseaseData.freq[1]);
	fprintf(outstream,"4 2\n %lf %lf\n",DiseaseData.freq[2],DiseaseData.freq[3]);
	
	fprintf(outstream,"%d\n",DiseaseData.numOfLiabClasses);
	for (i =0; i < DiseaseData.numOfLiabClasses; i++){
		for (j = 0; j < 3; j++){
			fprintf(outstream,"%lf %lf %lf\n",DiseaseData.femalePenetrances[i][j*3 + 0],
				DiseaseData.femalePenetrances[i][j*3 + 1], DiseaseData.femalePenetrances[i][j*3 + 2]);
		}
		if (ProgramData.sexLinked){
			for (j = 0; j < 2; j++){
				fprintf(outstream,"%lf %lf %lf\n",DiseaseData.femalePenetrances[i][j*2 + 0],
					DiseaseData.femalePenetrances[i][j*2 + 1]);
			}
		}
	}
}
