#include "InformativeMarkers.h"
#include "common.h"

void removeExtremeFreqMarkers(FILE* output);
void removeOneOreExtraAlleleMarkers(FILE* output);

void removeUnfirmotiveMarkers()
{
	FILE* output;
	output = fopen(ERROR_FILE ,"a");
	//fprintf(output,"       ###############################\n");
	//fprintf(output,"       # UNINFORMATIVE_MARKRES_START #\n");
	//fprintf(output,"       ###############################\n");

	removeOneOreExtraAlleleMarkers(output);
	removeExtremeFreqMarkers(output);
	
	//fprintf(output,"        #############################\n");
	//fprintf(output,"        # UNINFORMATIVE_MARKRES_END #\n");
	//fprintf(output,"        #############################\n\n");
	fclose(output);
}

void removeOneOreExtraAlleleMarkers(FILE* output){
	int pedInd, personInd, markerInd, alleleInd;
	int alleleInUseCount;
	PersonDataStruct *personPtr;
	
	Bool *alleleInUse;// = {FALSE};
	
	myMalloc(alleleInUse, (sizeof(Bool)*MAX_ALLELE_NUM));
	for (markerInd = 0; markerInd < ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci; markerInd++){
		if (!OkMarkers[markerInd])
			continue;
		memset(alleleInUse,'\0',sizeof(int)*MAX_ALLELE_NUM);
		for (pedInd = 0; pedInd < Pedigrees.numOfPedigrees; pedInd++){

			for (personInd =0, personPtr = Pedigrees.pedigrees[pedInd].people;
				personInd < Pedigrees.pedigrees[pedInd].numPeople; personInd++, personPtr++){

					if (personPtr->markersAlleles[markerInd].allele1 > 0 &&
						personPtr->markersAlleles[markerInd].allele1 < MAX_ALLELE_NUM){
							alleleInUse[personPtr->markersAlleles[markerInd].allele1] = TRUE;
						}
						if (personPtr->markersAlleles[markerInd].allele2 > 0 &&
							personPtr->markersAlleles[markerInd].allele2 < MAX_ALLELE_NUM){
								alleleInUse[personPtr->markersAlleles[markerInd].allele2] = TRUE;
							}
				}
		}

		for (alleleInd =0, alleleInUseCount = 0; alleleInd < MAX_ALLELE_NUM; alleleInd++){
			if (alleleInUse[alleleInd])
				alleleInUseCount++;
		}
		memset(alleleInUse,FALSE,MAX_ALLELE_NUM);
		if (alleleInUseCount < 2){
			OkMarkers[markerInd] = FALSE;
			printf("Marker #%d (%s) - uninformative - only one allele in use\n",markerInd + 1, LociData[markerInd].locusName);
			//fprintf(output, "Marker #%d (%s) - uninformative - only one allele in use\n",markerInd, LociData[markerInd].locusName);
			fprintf(output, "%s - uninformative - only one allele in use\n", LociData[markerInd].locusName);
			ProgramData.numInformativeMarkers--;
		}
		if (alleleInUseCount > LociData[markerInd].numOfAlleles){
			OkMarkers[markerInd] = FALSE;
			printf("Marker #%d (%s) -  has more alleles in pedfile, then specified in data file \n",markerInd +1, LociData[markerInd].locusName);
			//fprintf(output,"Marker #%d (%s) -  has more alleles in pedfile, then specified in data file \n",markerInd, LociData[markerInd].locusName);
			fprintf(output,"%s -  has more alleles in pedfile, then specified in data file \n", LociData[markerInd].locusName);
			ProgramData.numInformativeMarkers--;

		}
	}

	free(alleleInUse);
}


void removeExtremeFreqMarkers(FILE* output)
{
	int markerInd, alleleInd;
	LocusDataStruct *markerPtr;

	for (markerInd =0, markerPtr = LociData; markerInd < ProgramData.numberOfLoci- ProgramData.numberOfDiseaseLoci; 
		markerInd++, markerPtr++){
			if (!OkMarkers[markerInd])
				continue;
			for (alleleInd = 0; alleleInd < markerPtr->numOfAlleles; alleleInd++){
				if (markerPtr->alleleFreq[alleleInd] < LOW_FREQ_LIMIT ||
					markerPtr->alleleFreq[alleleInd] > HIGH_FREQ_LIMIT ){
						OkMarkers[markerInd] = FALSE;
						printf("Marker #%d (%s) - has too low or to high frequencies\n",markerInd + 1, LociData[markerInd].locusName);
						//fprintf(output, "Marker #%d (%s) - has too low or to high frequencies\n",markerInd, LociData[markerInd].locusName);
						fprintf(output, "%s - has too low or to high frequencies\n", LociData[markerInd].locusName);
						ProgramData.numInformativeMarkers--;
						break;
					} //if
			} // for (allelInd
		} //for markerInd
}
