/*********************************************************************
*  main.c											   			   
*  Main file
*  Author: Anna Tzemach								
**********************************************************************/

#include "parseLociFile.h"
#include "parsePedFile.h"
#include "convertGHtoSL.h"
#include "freeMemory.h"
#include "clusterMarkers.h"
#include "checkGenotypes.h"
#include "output.h"
#include "globals.h"

#include "myDebug.h"

int pc_main();
double sl_main(int numMarkers,int* markersUsed, Bool diseaseUsed, int progCode,double specialThetas, PedigreeDataStruct *currPedigree);
void removeUnfirmotiveMarkers();
void removeUnLikelyGenotypes();
/****************************************************
*       Function        main
*       Parameters		argc - number of parameters
*                       argv - datafile name, pedfile name
*       Output          none
*       Description		Main function of the program
*       Comments		none
/****************************************************/
int main(int argc, char * argv[]) {
	int  start_point = 1;
	int  exit_point = 5;
	int typeOfGenoCheck = 0; // ERRORS_PER_MARKER_TRESHOLD
	
	// Disabled due to lack of usability
	//char eliminatedMarkersOutputFile[100];
	 

	// 1. Read input
	if ( argc < 6 || argc > 10  ){
		printf("\nUsage: format(1 = GH, 2 = SL) datafileName pedfileName typeOfGenoCheck firstMarkerIndexShift [optional exit_point] [optional start_point]  [optional markerseliminatedfileName(input)] [optional markerseliminatedfileName(output)] \n");
		printf(" for start point: \n");
		printf(" 1 from beggining(default)\n");
		printf(" 2 from after pedcheck\n");
		printf(" 3 from after removeUnfirmotiveMarkers\n");
		printf(" 4 from after removeInfinity\n");
		printf(" 5 from after removeUnLikelyGenotypes\n");
		printf(" for exit point: \n");
		printf(" 1 after pedcheck\n");
		printf(" 2 after removeUnfirmotiveMarkers\n");
		printf(" 3 after removeInfinity\n");
		printf(" 4 after removeUnLikelyGenotypes\n");
		printf(" 5 after clustering(default)\n");
		return 0;
	}
	if( argc > 6 ){
		exit_point = atoi(argv[6]);
	}
	if ( argc > 7 ){
		start_point = atoi(argv[7]);
	}
	 
	readLociFile(argv[2],argv[5]);

	if( argc > 8 ) readMarkersEliminatedFile(argv[8]);
	//if( argc == 10 ) sprintf(eliminatedMarkersOutputFile, "%s",argv[9]);
	//else sprintf(eliminatedMarkersOutputFile, "markersEliminated_output.%d.txt",exit_point);
	ProgramData.inputFormat = atoi(argv[1]);
	typeOfGenoCheck = atoi(argv[4]); 
	parsePedFile(argv[3]);
	
	// 2. Update datastructures (family conncetion, distances and etc)
	convertGHtoSL(Pedigrees);

	// 3. Remove mendelian errors
	if ( (start_point == 1) || (start_point == 777)) // 777 == We dealing with raw files, we want to check only the mendelian errors before we are performing clustering 
	{
		// if were errors in pedigree structure, print errors
		if (pc_main() == -1)
		{
			freeMainMemory();
			printFile(CRITICAL_ERROR_FILE);
			return -1;
		}
	}	

	if (exit_point == 1){
		printOkMarkers();
		//writeMarkersEliminatedFile(eliminatedMarkersOutputFile);
		return 0;
	}
	 
	if ( start_point <= 2 ){
		// 4.Remove Uninformative markers
		removeUnfirmotiveMarkers();
	}
	if (exit_point == 2){
		printOkMarkers();
		//writeMarkersEliminatedFile(eliminatedMarkersOutputFile);
		return 0;
	}

	if ( start_point <= 3 ){
		// 5. Remove unlikely genotypes
		removeInfinity();
	}
	if (exit_point == 3){
		printOkMarkers();
		//writeMarkersEliminatedFile(eliminatedMarkersOutputFile);
		return 0;
	}

	if ( start_point <= 4 )
	{	
		if(typeOfGenoCheck != -1) // If typeOfGenoCheck == -1 then we dont perform it
		{
			removeUnLikelyGenotypes(typeOfGenoCheck);
		}
	}
	if (exit_point == 4){
		printOkMarkers();
		//writeMarkersEliminatedFile(eliminatedMarkersOutputFile);
		return 0;
	}


	// 6. (Optional) Remove LD
	// 7. Cluster SNPs to markers
	clusterMarkers(20);

	// 8. Print output
	printResultClusters();
	//writeMarkersEliminatedFile(eliminatedMarkersOutputFile);
	

#ifdef WIN32
	//_ASSERT(_CrtCheckMemory());
#endif
	freeMainMemory();
	return 0;
} 
