#include "parseLociFile.h"

Bool readGeneFrequencies(int locusInd, FILE* stream,int firstMarkerIndexShift);
void readLocus(int locusInd,FILE* stream,  int *diseaseCount,int firstMarkerIndexShift);
void readAffectionLocus(FILE* stream, int femaleTableSize, int maleTableSize);
/****************************************************
*       Function        readLociFile
*       Parameters		lociFileName - name of the loci file
*       Output          none
*       Description		Parses loci file and initialize data structure
*       Comments		none
/****************************************************/
void readLociFile(char *lociFileName ,char *firstMarkerIndexShiftString){
	FILE *stream;
	char tmpStr[100];
	int *lociOrder, locusInd,locusInputInd,firstMarkerIndexShift = 0 ;
	double theta;
	int diseaseCount = 0;
	int i=0;
	firstMarkerIndexShift = atoi(firstMarkerIndexShiftString);

	/* 1.0 Open file */
	if((stream = fopen(lociFileName, "r")) == NULL){
		printStdErr("cannot open loci file\n");
	}

	/* 2.0 Read first line */
	if ((fscanf(stream,"%d%d%d%d",&ProgramData.numberOfLoci, &ProgramData.riskLocus,           &ProgramData.sexLinked, &ProgramData.programCode)) != 4){
		printStdErr("Error on the first line of loci file.\n Format:  No_Loci    Risk_Locus    Sex_Linked    Program_Code   [ No_Complex_Affection_Loci]\n");
	}
	if (fscanf(stream,"%[^\n]", tmpStr+0) != 0) {
		ProgramData.numberOfDiseaseLoci = atoi(tmpStr+0);
		ProgramData.numberOfDiseaseLoci  = (ProgramData.numberOfDiseaseLoci == 2)?2:1;
	}
	else {
		ProgramData.numberOfDiseaseLoci  = 1;
	}
	/* 3.0 Ignore second line */
	getc(stream);
    fscanf(stream, "%d%lf%lf%d%*[^\n]",&theta,&theta,&theta,&theta);
	getc(stream);
	myMalloc(OkMarkers,(sizeof(boolean)*(ProgramData.numberOfLoci-1))); // kfir --> -1
	//memset(OkMarkers,TRUE,sizeof(boolean)*ProgramData.numberOfLoci);
	for (i=0;i<ProgramData.numberOfLoci-1;++i)
	{
		OkMarkers[i]=TRUE;
	}

	/* 4.0 Read loci order */
	myMalloc(lociOrder, sizeof(int)*ProgramData.numberOfLoci);
	for (locusInd = 0; locusInd < ProgramData.numberOfLoci; locusInd++) {
		if (fscanf(stream, "%d", &locusInputInd) != 1){
			printf("error in marker %d\n",locusInd +1);
			printStdErr("Cannot read loci order (loci file, line 3).\n");
		}
		lociOrder[locusInputInd - 1] = locusInd;
	}
	fscanf(stream, "%*[^\n]"); 
	getc(stream);

	/*  5.0 Read loci information */
	myMalloc(LociData,sizeof(LocusDataStruct)*ProgramData.numberOfLoci);
	for (locusInd = 0; locusInd < ProgramData.numberOfLoci; locusInd++) {
		readLocus(locusInd, stream,&diseaseCount,firstMarkerIndexShift);

	}
	ProgramData.numberOfDiseaseLoci = diseaseCount;
	ProgramData.numInformativeMarkers = ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci;


	/* 8.	Read the rest of the values (whether there is sex-difference in recombination 
			fractions and whether interference should be taken into account). */

	fscanf(stream, "%d%d%*[^\n]", &(ProgramData.sexDiff), &(ProgramData.interference));
	getc(stream);

	
	if (ProgramData.interference == 2)  {
		ProgramData.haldane = FALSE;
	}
	else {
		ProgramData.haldane = TRUE;
	}

	/* 11.	Read the male recombination values between each two adjacent loci. */

	LociData[0].malePosition = LociData[0].femalePosition = 0;
	for (locusInd = 0, locusInputInd = 1; locusInputInd < ProgramData.numberOfLoci; locusInputInd++) {
		fscanf(stream, "%lf", &theta);
		theta =  getRecTocM(theta, ProgramData.haldane);
		if (!((ProgramData.numberOfDiseaseLoci >0  && DiseaseData.positions[0] == locusInputInd-1) 
			|| (ProgramData.numberOfDiseaseLoci == 2  && DiseaseData.positions[1] == locusInputInd-1))){
				locusInd++;
				LociData[locusInd].malePosition =   LociData[locusInd-1].malePosition + theta ;
				LociData[locusInd].femalePosition = LociData[locusInd].malePosition;
			}else{
				LociData[locusInd].malePosition =   LociData[locusInd].malePosition + theta ;
				LociData[locusInd].femalePosition = LociData[locusInd].malePosition;
			}
	}


	/* 12.	If interference option 1 has been chosen, get the theta value between 
	the flanking loci. */

	if (ProgramData.sexDiff == 1) {
		fscanf(stream, "%lf", &ProgramData.geneticDist);
		for (locusInd = 1; locusInd < ProgramData.numberOfLoci; locusInd++) {
			LociData[locusInd].femalePosition =   getcMToRec((LociData[locusInd-1].malePosition - LociData[locusInd-1].malePosition),ProgramData.haldane);
			LociData[locusInd].femalePosition = getRecTocM(LociData[locusInd].femalePosition*ProgramData.geneticDist,ProgramData.haldane);
		}

	}
	if (ProgramData.sexDiff == 2) {
		fscanf(stream, "%lf", &theta);
		theta =  getRecTocM(theta, ProgramData.haldane);
		LociData[locusInd].femalePosition =   LociData[locusInd-1].malePosition + theta ;
	}
		
	/* Program specific is ignored, will be print other version*/ 
	fclose(stream);

	ProgramData.numberOfClusters = 0;
	free(lociOrder);
}

/******************************************************************************
*
*       Function        readLocus
*       Parameters		locusInd (the index of the locus being read).
*       Output          none
*       Description		Reads the information on one locus from the locus file.
*       Comments		none
*
******************************************************************************/
void readLocus(int locusInd, FILE* stream, int *diseaseCount , int firstMarkerIndexShift){
	LocusDataStruct	*locusPtr;

	/* 1.	Update a pointer to the position in the loci data-structure that 
	would store the information on this locus. */

	locusPtr =  &(LociData[locusInd - *diseaseCount]);

	/* 2.	Read the type of locus. */

	fscanf(stream,"%d",&locusPtr->type);

	/* 3.	Read the number of alleles, allocate memory for an array of 
	gene-frequenices and initialize it. */
	if (!readGeneFrequencies(locusInd - *diseaseCount, stream , firstMarkerIndexShift)
			&& locusPtr->type == NUMBERED_LOCUS){
		OkMarkers[locusInd - *diseaseCount] = FALSE;
		ProgramData.numInformativeMarkers--;
	}

	/* 4.	Update the locus type, and read the rest of the data regarding the 
	locus according to its type. */

	switch (locusPtr->type) {

		case AFFECTED_LOCUS:
			if (DiseaseData.positions != NULL){
				printStdErr("More then one affection loci was read");
			}
			ProgramData.numberOfDiseaseLoci = 1;
			myMalloc(DiseaseData.positions, sizeof(int));
			DiseaseData.positions[0] = locusInd;
			readAffectionLocus(stream,3,2);
			DiseaseData.freq = locusPtr->alleleFreq;
			*diseaseCount = 1;
			break;
		case COMLEX_LOCUS:
			if (diseaseCount == 0){
				myMalloc(DiseaseData.positions,(2*sizeof(int)));
				DiseaseData.positions[0] = locusInd;
				DiseaseData.freq = locusPtr->alleleFreq;
				*diseaseCount = 1;
			}else{	
				if (*diseaseCount != 1){
					printStdErr("More then one affection loci was read");
				}
				ProgramData.numberOfDiseaseLoci = 2;
				DiseaseData.positions[1] = locusInd;
				readAffectionLocus(stream,9,4);
				myRealloc(DiseaseData.freq,(sizeof(double) * 4));
				memcpy(DiseaseData.freq + 2, locusPtr->alleleFreq, 2);
				(*diseaseCount)++;
			}
			break;
		case NUMBERED_LOCUS:
			break;

		default :
			printStdErrWithParam("Invalid locus type in locus #%d (the only types allowed are 1,3 or 4).\n", 	locusInd + 1);
	}
}


/******************************************************************************
*
*       Function        readGeneFrequencies
*       Parameters		locusInd (the index of the locus whose gene-frequencies
*						are being read currently).
*       Output          none
*       Description		Reads the number of alleles of the current locus, 
*						allocates memory for the array of gene-frequencies and
*						initializes it according to the input file.
*       Comments		none
*
******************************************************************************/
Bool readGeneFrequencies(int locusInd, FILE* stream , int firstMarkerIndexShift){
	int				 count;
	LocusDataStruct	*locusPtr;
	char			tmpStr[1000], ch, *index; // Hidden bug - size of 100 is too short. 1000 should be ok
	double			sumGeneFreq,prob;
	Bool correctFlag = TRUE;

	/* 1.	Update a pointer to the position in the loci data-structure that 
	would store the information on this locus. */

	locusPtr = &(LociData[locusInd]);

	/* 2.	Read the number of alleles. */

	fscanf(stream,"%d",&(locusPtr->numOfAlleles));


	/* 3.	Read the name of the locus, if its specified, and allocate memory to store it. */

	locusPtr->locusName[0] = '\0';;


	while (fscanf(stream,"%c",&ch) == 1) {
		if (ch == ' ' || ch == '\t')
			continue;
		if (ch == '\n')
			break;
		if (ch == '#') {
			fscanf(stream, "%[^\n]\n", tmpStr);
			index = strchr(tmpStr, '#');
			if (index ){
				index[0] = '\0';
				sprintf(locusPtr->locusName, "%s(%d)", tmpStr,locusInd+1+firstMarkerIndexShift);
			}
			break;
		}
		else {
			fscanf(stream, "%*[^\n]\n");
			break;
		}
	}

	/* 4.	Allocate memory for the gene frequencies array. */

	myMalloc(locusPtr->alleleFreq, (sizeof(double) * locusPtr->numOfAlleles));

	/* 5.	Read the gene frequencies. */

	sumGeneFreq = 0.0;

	for (count = 0; count < locusPtr->numOfAlleles; count++) {
		fscanf(stream, "%lf",&prob);

		if ((prob < 0) || (prob > 1.0)) {
			printStdErrWithParamNoExit("Invalid gene frequency for locus #%d, allele #%d. \n",locusInd+1, count+1);
			correctFlag = FALSE;
		}

		(locusPtr->alleleFreq)[count] = prob;
		sumGeneFreq += prob;
	}

	/* 6.	Read the rest of the line. */

	fscanf(stream, "%*[^\n]");
	getc(stream);

	/* 7.	Check that the gene frequencies sum up to ~ 1.0. */

	if ((sumGeneFreq > 1.001) || (sumGeneFreq < 0.999)) {
		printStdErrWithParamNoExit("The gene frequencies of locus #%d (%s), don't sum to 1.0. \n", locusInd+1,locusPtr->locusName);
		correctFlag = FALSE;
	}
	return correctFlag;
}	


/******************************************************************************
*
*       Function        readAffectionLocus
*       Parameters		locusInd (the index of the locus being read).
*       Output          none
*       Description		Reads the data on an affection type locus from the 
*						locus file.
*       Comments		none
*
******************************************************************************/
void readAffectionLocus(FILE* stream, int femaleTableSize, int maleTableSize){

	int				count, i;

	/* 1.	Read the number of liability classes. */
	fscanf(stream,"%d%*[^\n]", &DiseaseData.numOfLiabClasses);
	getc(stream);


	myMalloc(DiseaseData.femalePenetrances, DiseaseData.numOfLiabClasses*sizeof(double*));
	if (ProgramData.sexLinked){
		myMalloc(DiseaseData.malePenetrances, DiseaseData.numOfLiabClasses*sizeof(double*));
	}

	for ( i = 0; i < DiseaseData.numOfLiabClasses; i++){
		myMalloc(DiseaseData.femalePenetrances [i], sizeof(double)*femaleTableSize);
		for (count =0; count < femaleTableSize; count++){
			fscanf(stream,"%lf",&DiseaseData.femalePenetrances[i][count]);
			if ( DiseaseData.femalePenetrances[i][count] < 0
				|| DiseaseData.femalePenetrances[i][count] > 1){
					printStdErr("Penetrances should be between 0 and 1\n");
				}
		}
		if (ProgramData.sexLinked){
			myMalloc(DiseaseData.malePenetrances [i], sizeof(double)*femaleTableSize);
			for (count =0; count < maleTableSize; count++)
				fscanf(stream,"%lf",&DiseaseData.malePenetrances[i][0]);
			if ( DiseaseData.malePenetrances[i][count] < 0
				|| DiseaseData.malePenetrances[i][count] > 1){
					printStdErr("Penetrances should be between 0 and 1\n");
				}
		}
	}
		fscanf(stream, "%*[^\n]"); 
		getc(stream);

}
/****************************************************
*       Function        readMarkersEliminatedFile
*       Parameters		markersEliminatedFileName - name of the markers eliminated file
*       Output          none
*       Description		update data structure with markers already eliminated before
*       Comments		Eliminated markers file format - numbers separated by spaces
/****************************************************/
void readMarkersEliminatedFile(char *markersEliminatedFileName){
	FILE *stream;
	int markerEliminated;

	/* Open file */
	if((stream = fopen(markersEliminatedFileName, "r")) == NULL){
		printStdErr("cannot open markers eliminated file\n");
	}

	while( fscanf( stream, "%d", &markerEliminated ) > 0 ){
		if( OkMarkers[markerEliminated] != FALSE ){
			OkMarkers[markerEliminated] = FALSE;
			ProgramData.numInformativeMarkers--;
		}
	}
	fclose(stream);
	return;
}
/****************************************************
*       Function        writeMarkersEliminatedFile
*       Parameters		markersEliminatedFileName - name of the markers eliminated file
*       Output          none
*       Description		update data structure with markers already eliminated before
*       Comments		Eliminated markers file format - numbers separated by spaces
/****************************************************/
void writeMarkersEliminatedFile(char *markersEliminatedFileName){
	FILE *stream;
	int	markerInd;

	/* Open file */
	if((stream = fopen(markersEliminatedFileName, "w")) == NULL){
		printStdErr("cannot open markers eliminated file for writing\n");
	}
	for( markerInd = 0 ; markerInd < ProgramData.numberOfLoci ; ++markerInd ){
		if( OkMarkers[markerInd] == FALSE )	fprintf( stream , "%d ", markerInd );
	}

	fclose(stream);
	return;
}
