#include "parsePedFile.h"



void coundPeopleAndPedigrees(char *filename);
void readSinglePedigree(PedigreeDataStruct *pedigree,FILE* stream);
void readSinglePerson(PersonDataStruct *person, int personIndex, FILE* stream);
void readSLFields(PersonDataStruct *person, FILE* stream);
void parseLociInformation(PersonDataStruct *person, FILE* stream);
/******************************************************************************
*
*       Function        parsePedFile
*       Parameters		filename 
*       Output          none
*       Description		Parses the pedigree data
*       Comments		none
*
******************************************************************************/
void parsePedFile(char *filename){
	int i;
	FILE *stream;

	/* 1.0 Count the number of pedigrees and allocate memory for them */
	coundPeopleAndPedigrees(filename);
	/* 2.0 Open file */
	if((stream = fopen(filename, "r")) == NULL){
		printStdErr("cannot open ped file\n");
	}
	/* 3.0 Read pedigree information */

	for (i = 0; i < Pedigrees.numOfPedigrees; i++){
		readSinglePedigree(Pedigrees.pedigrees + i,stream);
	}
	fclose(stream);	
}

/******************************************************************************
*
*       Function        coundPeopleAndPedigrees
*       Parameters		filename 
*       Output          none
*       Description		Counts number of families
*       Comments		Allocated: Pedigrees.pedigrees
*
******************************************************************************/
void coundPeopleAndPedigrees(char *filename){
	int currentPed = 0,numPeds = 0, numPeople = 1, tmp;
	FILE *stream; 
	/* 1.0 Open file */
	if((stream = fopen(filename, "r")) == NULL){
		printStdErr("cannot open pedfile\n");
	}
	Pedigrees.pedigrees = NULL;
	fscanf(stream, "%d", &currentPed);
	fscanf(stream,"%*[^\n]\n");
	/*count familes */
	while (fscanf(stream, "%d", &tmp)!= EOF){
		if (tmp != currentPed){
			numPeds++;
			Pedigrees.numOfPedigrees = numPeds;
			myRealloc(Pedigrees.pedigrees,( sizeof(PedigreeDataStruct) * Pedigrees.numOfPedigrees));
			myMalloc(Pedigrees.pedigrees[numPeds-1].people,  sizeof(PersonDataStruct)*numPeople);
			Pedigrees.pedigrees[numPeds-1].numPeople = numPeople;
			Pedigrees.pedigrees[numPeds-1].index = currentPed;
			currentPed = tmp;
			numPeople = 1;
		}else{
			numPeople++;
		}
		fscanf(stream,"%*[^\n]\n");
	}
	Pedigrees.numOfPedigrees = numPeds+1;
			myRealloc((Pedigrees.pedigrees), (sizeof(PedigreeDataStruct) * Pedigrees.numOfPedigrees));
			myMalloc(Pedigrees.pedigrees[Pedigrees.numOfPedigrees-1].people,  sizeof(PersonDataStruct)*numPeople);
			Pedigrees.pedigrees[Pedigrees.numOfPedigrees-1].numPeople = numPeople;
			Pedigrees.pedigrees[Pedigrees.numOfPedigrees-1].index = currentPed;
	fclose(stream);
}

/******************************************************************************
*
*       Function        readSinglePedigree
*       Parameters		Pointer to the pedigree 
*       Output          none
*       Description		Parses pedigree information
*       Comments		
*
******************************************************************************/
void readSinglePedigree(PedigreeDataStruct *pedigree, FILE* stream){
	int i, pedIndex,personIndex;
	pedigree->numNonFounders = 0;
	for (i = 0; i < pedigree->numPeople; i++){
		if (fscanf(stream,"%d%d",&pedIndex,&personIndex) == EOF){
			printStdErrWithParam("Cant' read line #%d in pedigree #%d\n",i,pedigree->index);
		}
		readSinglePerson(pedigree->people + i, personIndex, stream);
		if ((pedigree->people + i)->father == 0 )
			pedigree->numNonFounders++;
	}	
}

/******************************************************************************
*
*       Function        readSinglePerson
*       Parameters		Pointer to the pedigree, person index 
*       Output          none
*       Description		Parses person information
*       Comments		
*
******************************************************************************/
void readSinglePerson(PersonDataStruct *person,int personIndex, FILE * stream){
	int tmp;

	person->id = personIndex;

	if (fscanf(stream,"%d%d",&(person->father),&(person->mother)) == EOF){
		printStdErrWithParam("Cant' read person #%d \n",personIndex);
	}

	if (ProgramData.inputFormat == SL){
		readSLFields(person,stream);
	}else{
		person->firstChild = -1;
	}

	if (fscanf(stream,"%d",&person->sex)== EOF){
		printStdErrWithParam("Cant' read sex of person #%d \n",personIndex);
	}

	if (ProgramData.inputFormat == SL){
		fscanf(stream,"%d",&tmp);
	}
	
	parseLociInformation(person,stream);
	
	fscanf(stream, "%*[^\n]"); 
	getc(stream);
	return;
}

/******************************************************************************
*
*       Function        readSLFields
*       Parameters		Reads 3 SL specific fields
*       Output          none
*       Description		Parses pedigree information
*       Comments		
*
******************************************************************************/
void readSLFields(PersonDataStruct *person, FILE *stream){
	if (fscanf(stream,"%d%d%d",&(person->firstChild),&(person->siblingFather), &(person->siblingMother)) == EOF){
		printStdErrWithParam("Cant' read person #%d in pedigree\n",person->id);
	}

}

/******************************************************************************
*
*       Function        readSLFields
*       Parameters		Reads 3 SL specific fields
*       Output          none
*       Description		Parses pedigree information
*       Comments		
*
******************************************************************************/
void parseLociInformation(PersonDataStruct *person,FILE *stream){
	int i, markerInd;
	person->clustersAlleles = NULL;
	myMalloc((person->markersAlleles),((sizeof(AlleleDataStruct))*ProgramData.numberOfLoci - ProgramData.numberOfDiseaseLoci )); //missing () ?
	for ( i = 0, markerInd =0; i < ProgramData.numberOfLoci; i++){
	 if((ProgramData.numberOfDiseaseLoci == 1 && DiseaseData.positions[0] == i)
			        || (ProgramData.numberOfDiseaseLoci == 2 && DiseaseData.positions[1] == i)){
			if (fscanf(stream,"%d",&(person->status))== EOF){
				printStdErrWithParam("Cant' read disease status of person #%d\n",person->id);
			}
			if (DiseaseData.numOfLiabClasses > 1 ){
				if (fscanf(stream,"%d",&(person->liabClass)) == EOF){
					printStdErrWithParam("Cant' read liability class of person #%d\n",person->id);
				}
			}else{
				person->liabClass = 1;
			}
		}	else if(!(ProgramData.numberOfDiseaseLoci == 2 && DiseaseData.positions[0] == i)){
			if (fscanf(stream,"%d%d",&(person->markersAlleles[markerInd].allele1),&(person->markersAlleles[markerInd].allele2)) == EOF){
				printStdErrWithParam("Cant' read  alleles of locus #%d person #%d \n",i, person->id);
			}
			markerInd++;
		}
	}
}
