/*************************************************************************************
 Error.h
 The file deals with Pedigree errors handlings.
 An error is a struct which includes an error string, that is presented to the user.
*************************************************************************************/
#ifndef _PC_ERROR_H
#define _PC_ERROR_H

#include "pc_NuclearFam.h"
#include "pc_Common.h"




typedef enum {

	/* level 1 errors */
	NO_LEGAL_GENDER_ERROR,
	NO_LEGAL_AFF_STATUS_ERROR,
	FULLY_TYPED_ERROR,
	DETAILS_TOO_LONG_ERROR,
	ODD_ALLELE_NUM_ERROR,
	NO_LEGAL_REMARK_ERROR,
	PARSING_MARKER_ERROR,
	ONE_ALLELE_UNKNOWN_ERROR,
	INVALID_ALLELE_ERROR,
	MARKERS_CNT_DIFF_IN_PERSONS_ERROR,
	EMPTY_FAMILY_ERROR,
	NOT_FULLY_TYPED_ERROR,
	NO_FAMILY_ID_ERROR,
	MAN_ALLELES_VALUE_ERROR,

	/* family connections syntax errors */
	FATHER_IS_FEMALE_ERROR, 
	MOTHER_IS_MALE_ERROR,
	PARENTS_ORDER_ERROR,
	FATHER_OF_HIMSELF_ERROR,
	MOTHER_OF_HIMSELF_ERROR,
	NO_FATHER_FOUND_ERROR,
	NO_MOTHER_FOUND_ERROR,

	/* level 2 errors */
	MORE_THAN_2_HOMO_CHILDREN_ERROR,
	MORE_THAN_4_IN_SIBSHIP_ERROR,
	MORE_THAN_3_IN_SIBSHIP_ERROR,
	MORE_THAN_2_IN_SIBSHIP_ERROR,
	NO_POSSIBLE_GENOTYPING_ERROR,
	CHILD_AND_PARENTS_NOT_COMPATIBLE_ERROR, 
	MORE_THAN_3_IN_SEX_SIBSHIP_ERROR,
	MORE_THAN_1_HOMO_CHILDREN_SEX_ERROR,
	MORE_THAN_2_IN_SIBSHIP_SEX_ERROR

} ErrorType;



typedef struct _Error
{
	char errorString[MAX_ERROR_ID_LEN];
    struct _Error * mNext;

} Error;



typedef struct _ErrorLL
{
    Error * mFirst;
	Error * mLast;
	int     numOfErrors;

} ErrorLL;


ErrorLL * CreateErrorLL();
void InitErrorLL(ErrorLL * aErrorLL);
void FreeErrorLL(ErrorLL * aErrorLL);
ckrResult AddError(ErrorLL * aErrorLL, char * aErrorStr);


void SetErrorInPerson(ErrorLL * aErrorLL, int index, char aID[MAX_PERSON_ID_LEN], ErrorType error);
void SetParentErrorInPerson(ErrorLL * aErrorLL, int index, char aID[MAX_PERSON_ID_LEN], char aParentID[MAX_PERSON_ID_LEN], ErrorType error);
void SetErrorInNuclearFam(ErrorLL * aErrorLL, int index, NuclearFam * aNFam, ErrorType error);
void SetErrorInPersonLine(ErrorLL * aErrorLL, char aID[MAX_PERSON_ID_LEN], ErrorType error);
void SetError(ErrorLL * aErrorLL, ErrorType error);


#endif

