/*************************************************************************************
 Level2Checker.c
 The file deals with level 2 checkings functions. 
 Level 2 checkings are as specified in the project requirements.
*************************************************************************************/
#include <stdio.h>
#include <stdlib.h>

#include "pc_Level2Checker.h"
#include "pc_NuclearFam.h"
#include "pc_Pedigree.h"
#include "pc_Error.h"
#include "pc_LangeGoradia.h"

void SetAllFamUnchecked(Pedigree * aPedigree);

ckrResult CheckNuclearFam(NuclearFam * aNuclearFam, int aMarkerIdx, Format aFormat, ErrorLL * aErrorLL);

/*************************************************************************************
Function name :ChkAllelesInFullSib
The function recieves aChildLL and checks the alleles in that sibship.
************************************************************************************/
ckrResult ChkAllelesInFullSib(Person * aChildLL, int aMarkerIdx, ErrorLL * aErrorLL)
{
	Person * aPerson;
    int aPersonA1,aPersonA2,aAllele1,aAllele2,aAllele3,aAllele4;
    int aHomozFlag=0;
	int aHomozAllele1;
	int aHomozAllele2;
	int aHomozCnt=0;
	int aAlleleCnt=0;
	NuclearFam * parentsFam = NULL;


	aPerson = aChildLL;
	if (aPerson != NULL)
	{
		parentsFam = aPerson->mParentsFam;
	}

	while(aPerson != NULL)
	{
		aPersonA1 = aPerson->mMarkerArr[aMarkerIdx].mAllele1;
		aPersonA2 = aPerson->mMarkerArr[aMarkerIdx].mAllele2;

		// at this stage, it is promised that either none of them, or both of them are 0
		if ((aPersonA1 == 0) || (aPersonA2 == 0)){
			aPerson = aPerson->mSibling;
			continue;
		}
	
		if (aAlleleCnt>4)
		{
			SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_4_IN_SIBSHIP_ERROR);
			return ckrErr; 
		}
	/*if this person is homozegotic */
		if (aPersonA1 == aPersonA2)	
		{
			aHomozFlag = 1; 
			switch (aHomozCnt)
			{
			case 0: aHomozAllele1 = aPersonA1;
					aHomozCnt++;
					break;

			case 1: if (aHomozAllele1 != aPersonA1)
					{
						aHomozAllele2 = aPersonA1;
						aHomozCnt++;
					}
					break;
		
			case 2: if ((aPersonA1 != aHomozAllele1) && (aPersonA1 != aHomozAllele2))
					{
						SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_2_HOMO_CHILDREN_ERROR);
						return ckrErr; 
					}
			}
		
		}  /* found homozygous */	
		else{
			aHomozFlag = 0;
		}
	
		switch (aAlleleCnt)
		{
		case 0: if (aHomozFlag==0) 
				{
					aAllele1 = aPersonA1; 
					aAllele2 = aPersonA2; 
					aAlleleCnt = aAlleleCnt+2;
				}  		
				else 
				{
					aAllele1 = aPersonA1; 
					aAlleleCnt++; /* the two alleles are the same */
				}
				break;
	 
		case 1: if (aHomozFlag==0) 
				{
					if ((aAllele1 == aPersonA1) && (aAllele1 !=  aPersonA2))
					{
						aAllele2 = aPersonA2;
						aAlleleCnt++;
						break;
					}
					if ((aAllele1 != aPersonA1) && (aAllele1 ==  aPersonA2))
					{
						aAllele2 = aPersonA1;
						aAlleleCnt++;
						break;
					}
					else //((aAllele1 != aPersonA1) && (aAllele1 !=  aPersonA2)
					{
						aAllele2 = aPersonA1;
						aAllele3 = aPersonA2;
						aAlleleCnt += 2;
						break;
					}
					// ((aAllele1 == aPersonA1) && (aAllele1 ==  aPersonA2))
					// is not possible since aHomozFlag==0
				}
				else
				{
					if (aAllele1 != aPersonA1)
					{
						aAllele2 = aPersonA1; 
						aAlleleCnt++;
					}
				}
				break;				 
								 
		case 2:	      
				if (aHomozFlag==1)
				{
					if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1)) 
					{
						aAllele3 = aPersonA1; 
						aAlleleCnt++;
					}
					/* add thois
					if ((aAllele1 != aPersonA2) && (aAllele2 != aPersonA2)) 
					{
						aAllele4 = aPersonA1; 
						aAlleleCnt++;
					}
					*/
				}
				else 
				{
					if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1)) 
					{
						aAllele3 = aPersonA1; 
						aAlleleCnt++;
					}
					/* check the second allele of Person */
		            if ((aAllele1 != aPersonA2) && (aAllele2 != aPersonA2)) 
					{
						if (aAlleleCnt == 3)
						{
							aAllele4 = aPersonA2; 
							aAlleleCnt++;
						}
						else /* aAlleleCnt == 2 */
						{
							aAllele3 = aPersonA2; 
							aAlleleCnt++;
						}
					}
				}
			    break;
		  
		case 3:
			if (aHomozFlag==1)
			{
				if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1) && (aAllele3 != aPersonA1))
				{
					aAllele4 = aPersonA1; 
					aAlleleCnt++;
				}	
			}
			else
			{
				if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1) && (aAllele3 != aPersonA1)) 
				{
					aAllele4 = aPersonA1;
					aAlleleCnt++;
				}	
				if ((aAllele1 != aPersonA2) && (aAllele2 != aPersonA2) && (aAllele3 != aPersonA2)) 
				{
					if (aAlleleCnt == 3)
					{
						aAllele4 = aPersonA2;
						aAlleleCnt++;
					}
					else
					{
						SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_4_IN_SIBSHIP_ERROR);
						return ckrErr; 
					}
				}
			}
			break;
		case 4:
			if (((aAllele1 == aPersonA1) || (aAllele2 == aPersonA1) || (aAllele3 == aPersonA1) || (aAllele4 == aPersonA1)) &&
				((aAllele1 == aPersonA2) || (aAllele2 == aPersonA2) || (aAllele3 == aPersonA2) || (aAllele4 == aPersonA2)))
			{
				// a fine state! both allele #1 and #2 of the person are an allready known alleles
			}
			else
			{
				SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_4_IN_SIBSHIP_ERROR);
				return ckrErr; 
			}
		}

		if (aPerson->mSibling != NULL)
		{
			aPerson = aPerson->mSibling;
		}
		else
		{
			break;
		}
	}
 	
	
	
	if (aHomozCnt==1 && aAlleleCnt > 3)
	{
		SetErrorInNuclearFam(aErrorLL, aMarkerIdx, parentsFam, MORE_THAN_3_IN_SIBSHIP_ERROR);
		return ckrErr;
	}
	if (aHomozCnt==2 && aAlleleCnt > 2)
	{
		SetErrorInNuclearFam(aErrorLL, aMarkerIdx, parentsFam, MORE_THAN_2_IN_SIBSHIP_ERROR);
		return ckrErr;	
	}
	
	return ckrOK;
}


int getMaleAllele(int aPersonA1, int aPersonA2){
	
	if (aPersonA1 == aPersonA2)
		return aPersonA1;

	return	(aPersonA2 == 0)? aPersonA1 : aPersonA2;

}

/*************************************************************************************
Function name :ChkAllelesInFullSibSex
The function recieves aChildLL and checks the alleles in that sibship.
************************************************************************************/
ckrResult ChkAllelesInFullSibSex(Person * aChildLL, int aMarkerIdx, ErrorLL * aErrorLL)
{
	Person * aPerson;
    int aPersonA1,aPersonA2,aAllele1,aAllele2,aAllele3,aAlleleTmp;
    int aHomozFlag=0;
	int aHomozAllele1;
	int aHomozCnt=0;
	int aAlleleCnt=0;
	NuclearFam * parentsFam = NULL;


	aPerson = aChildLL;
	if (aPerson != NULL)
	{
		parentsFam = aPerson->mParentsFam;
	}

	while(aPerson != NULL)
	{
		aPersonA1 = aPerson->mMarkerArr[aMarkerIdx].mAllele1;
		aPersonA2 = aPerson->mMarkerArr[aMarkerIdx].mAllele2;

		// if both alleles are unknown
		if ((aPersonA1 == 0) && (aPersonA2 == 0)){
			aPerson = aPerson->mSibling;
			continue;
		}
	
		if (aAlleleCnt>3)
		{
			SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_3_IN_SEX_SIBSHIP_ERROR);
			return ckrErr; 
		}
	/*if this person is homozegotic */
		if ((aPersonA1 == aPersonA2) && (aPerson->mGender != male))
		{
			aHomozFlag = 1; 
			switch (aHomozCnt)
			{
				case 0: aHomozAllele1 = aPersonA1;
						aHomozCnt++;
						break;

				case 1: if (aHomozAllele1 != aPersonA1)
						{
							SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_1_HOMO_CHILDREN_SEX_ERROR);
							return ckrErr; 
						}
						break;
			}
		
		}  /* found homozygous */	
		else
		{
			aHomozFlag = 0;
		}
	
		switch (aAlleleCnt)
		{
		case 0: if (aHomozFlag==0) 
				{
					if ( aPerson->mGender == male){
						aAllele1 = getMaleAllele(aPersonA1, aPersonA2);
						aAlleleCnt++;
					}
					else{ /*  gender is female or unknown (as female)*/
						aAllele1 = aPersonA1; 
						aAllele2 = aPersonA2; 
						aAlleleCnt = aAlleleCnt+2;
					}
				}  		
				else 
				{
					aAllele1 = aPersonA1; 
					aAlleleCnt++; /* the two alleles are the same */
				}
				break;
	 
		case 1: if (aHomozFlag==0) 
				{
					if ( aPerson->mGender == male)
					{
						aAllele2 = getMaleAllele(aPersonA1, aPersonA2);
						if ( (aAllele1 != aAllele2) ) 
						{
							aAlleleCnt++;
							break;
						}
					}
					else
					{
						if ((aAllele1 == aPersonA1) && (aAllele1 !=  aPersonA2))
						{
							aAllele2 = aPersonA2;
							aAlleleCnt++;
							break;
						}
						if ((aAllele1 != aPersonA1) && (aAllele1 ==  aPersonA2))
						{
							aAllele2 = aPersonA1;
							aAlleleCnt++;
							break;
						}
						else //((aAllele1 != aPersonA1) && (aAllele1 !=  aPersonA2)
						{
							aAllele2 = aPersonA1;
							aAllele3 = aPersonA2;
							aAlleleCnt += 2;
							break;
						}
					}
					// ((aAllele1 == aPersonA1) && (aAllele1 ==  aPersonA2))
					// is not possible since aHomozFlag==0
				}
				else // since aHomozFlag==1 it can be only female
				{
					if (aAllele1 != aPersonA1)
					{
						aAllele2 = aPersonA1; 
						aAlleleCnt++;
					}
				}
				break;				 
								 
		case 2:	      
				if (aHomozFlag==0) 
				{
					if ( aPerson->mGender == male)
					{
						aAllele3 = getMaleAllele(aPersonA1, aPersonA2);
						if ( (aAllele3 != aAllele1) && (aAllele3 != aAllele2) ) 
						{
							aAlleleCnt++;
							break;
						}
					}
					else /* person gender is female*/
					{
						if (((aPersonA1 == aAllele1) || (aPersonA1 == aAllele2)) &&
							((aPersonA2 != aAllele1) && (aPersonA2 != aAllele2)) )
						{
							aAllele3 = aPersonA2; 
							aAlleleCnt++;
							break;
						}
						if (((aPersonA2 == aAllele1) || (aPersonA2 == aAllele2)) &&
							((aPersonA1 != aAllele1) && (aPersonA1 != aAllele2)) )
						{
							aAllele3 = aPersonA1; 
							aAlleleCnt++;
							break;
						}
						else
						{
							SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_3_IN_SEX_SIBSHIP_ERROR);
							return ckrErr; 
						}
					}
				}
				else   /*	(aHomozFlag==1) , it should be only a male */
				{
					if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1)) 
					{
						aAllele3 = aPersonA1; 
						aAlleleCnt++;
					}
					/* add thois
					if ((aAllele1 != aPersonA2) && (aAllele2 != aPersonA2)) 
					{
						aAllele4 = aPersonA1; 
						aAlleleCnt++;
					}
					*/
				}
			    break;
		  
		case 3:
			if (aHomozFlag==0)
			{
				if ( aPerson->mGender == male)
				{
					aAlleleTmp = getMaleAllele(aPersonA1, aPersonA2);
						if ((aAlleleTmp != aAllele1) && (aAlleleTmp != aAllele2) && (aAlleleTmp != aAllele3) ) 
						{
							SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_3_IN_SEX_SIBSHIP_ERROR);
							return ckrErr; 
						}
				}
				else /* gender is female */
				{
					if (  ((aPersonA1 != aAllele1) && 
						   (aPersonA1 != aAllele2) &&  
						   (aPersonA1 != aAllele3)) ||
						  ((aPersonA2 != aAllele1) &&
						   (aPersonA2 != aAllele2) &&
						   (aPersonA2 != aAllele3)) )
					{
						SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_3_IN_SEX_SIBSHIP_ERROR);
						return ckrErr; 
					}
				}
			}
			else /* (aHomozFlag==1),  it should be a male*/
			{
				if ((aAllele1 != aPersonA1) && (aAllele2 != aPersonA1) && (aAllele3 != aPersonA1))
				{
					SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aPerson->mParentsFam, MORE_THAN_3_IN_SEX_SIBSHIP_ERROR);
					return ckrErr; 
				}	
			}
			break;

		}

		if (aPerson->mSibling != NULL)
		{
			aPerson = aPerson->mSibling;
		}
		else
		{
			break;
		}
	}
 	
	
	
	if (aHomozCnt==1 && aAlleleCnt > 2)
	{
		SetErrorInNuclearFam(aErrorLL, aMarkerIdx, parentsFam, MORE_THAN_2_IN_SIBSHIP_SEX_ERROR);
		return ckrErr;
	}
	
	return ckrOK;
}
/*************************************************************************************
Function name :Level2Errors, recieves aPedigree.
Level 2 errors are checked, using a few functions.
************************************************************************************/
ckrResult Level2Errors(Pedigree * aPedigree)
{
    int iPerson,iMarker;
	Person * aPerson;
	int aRes;
	int aNumPerson;
	int aMarkerCnt = aPedigree->mMarkerCnt;
	NuclearFam * aNuclearFam;
	int errFlag = 0;
	Format aFormat = aPedigree->mFormat;

	aNumPerson = aPedigree->mPersonCnt;

	for (iMarker=0; iMarker<aMarkerCnt; ++iMarker)
	{
		SetAllFamUnchecked(aPedigree);
		//OkMarkers[iMarker] = TRUE;
		for (iPerson=0; iPerson<aNumPerson; ++iPerson) {
			aPerson = aPedigree->mPersonArray[iPerson];
			aNuclearFam = aPerson->mParentsFam;
			if (NULL == aNuclearFam)
			{
				continue;
			}
			if (1 == aNuclearFam->mIsChecked)
			{
				continue;
			}

			if (ckrOK != CheckNuclearFam(aNuclearFam, iMarker, aFormat, aPedigree->mErrorLL2))
			{
				errFlag = 1;
				//Anna
				if (OkMarkers[iMarker] )
				{
					OkMarkers[iMarker] = FALSE;
					ProgramData.numInformativeMarkers--;
				}
				//end
			}
		    aNuclearFam->mIsChecked = 1;
		 }
	 }
	
	/*Anna
		if (errFlag){
		return ckrErr;
	}
	*/
	/*if (aPedigree->mFormat == SEXLINKED){
		return ckrOK;
	}
	*/
	/* check 10  */

	aPedigree->mCriticalGenotypesLevel = (int *)malloc(sizeof(int)*aMarkerCnt);
	if (NULL == aPedigree->mCriticalGenotypesLevel)
	{
		return ckrMemErr;
	}
	
	for (iMarker = 0; iMarker < aMarkerCnt; ++iMarker)
	{
		if (!OkMarkers[iMarker])
			continue;
		aRes = LangeGoradia(aPedigree, iMarker);
		if (ckrOK == aRes)
		{
			/* points how many people should be zeroded so the opedigree will be consistent */
			aPedigree->mCriticalGenotypesLevel[iMarker] = 0;
		}
		else
		{
			/* the pedigree is not correct, therefor AT LEAST one person is not correct. 
		       (If we knew that exactly one person is not correct, mCriticalGenotypesLevel[i] 
		       would have had the value 1) */
			aPedigree->mCriticalGenotypesLevel[iMarker] = -1;
			//Anna
			if (OkMarkers[iMarker] )
			{
				OkMarkers[iMarker] = FALSE;
				ProgramData.numInformativeMarkers--;
			}
			//end
		}
		RETURN_MEM_ERROR(aRes);
	}

	if (errFlag == 1)
	{
		return ckrErr;
	}

	return ckrOK;
}


/*************************************************************************************
Function name :ChkChildParentsCompatible
This function checks if a marker (which includes two alleles) of child is compatible
to his parents marker.
*************************************************************************************/
Result ChkChildParentsCompatible(int aChAllele1, int aChAllele2,
								 int aFAllele1,  int aFAllele2,
								 int aMAllele1,  int aMAllele2)
{
	/* child's markers are unknown */
	if ((0 == aChAllele1) || (0 == aChAllele2))
	{
		return YES;
	}

	/* mother's && father's markers are unknown */
	if (((0 == aFAllele1) || (0 == aFAllele2)) &&
		((0 == aMAllele1) || (0 == aMAllele2)))
	{
		return YES;
	}

	/* only mother's markers are unknown */
	if ((0 == aMAllele1) || (0 == aMAllele2))
	{
		/* check compatibility to father's markers */
		if ((YES == IsAlleleIn(aChAllele1, aFAllele1, aFAllele2)) ||
			(YES == IsAlleleIn(aChAllele2, aFAllele1, aFAllele2)))
		{
			return YES;
		}
		else
		{
			return NO;
		}
	}

	/* only father's markers are unknown */
	if ((0 == aFAllele1) || (0 == aFAllele2))
	{
		/* check compatibility to mother's markers */
		if ((YES == IsAlleleIn(aChAllele1, aMAllele1, aMAllele2)) ||
			(YES == IsAlleleIn(aChAllele2, aMAllele1, aMAllele2)))
		{
			return YES;
		}
		else
		{
			return NO;
		}
	}


	if (((YES == IsAlleleIn(aChAllele1, aFAllele1, aFAllele2)) &&
		 (YES == IsAlleleIn(aChAllele2, aMAllele1, aMAllele2)))||
	    ((YES == IsAlleleIn(aChAllele2, aFAllele1, aFAllele2)) &&
	     (YES == IsAlleleIn(aChAllele1, aMAllele1, aMAllele2))))
	{
		return YES;
	}
	else
	{
		return NO;
	}
	
}
/*************************************************************************************
Function name :ChkChildParentsCompatibleSex
This function checks if a marker (which includes two alleles) of child in sexlinked format 
is compatible to his mather marker.
*************************************************************************************/
Result ChkChildParentsCompatibleSex(int aChAllele1, int aChAllele2,
								    int aMAllele1,  int aMAllele2)
{
	/* child's markers are unknown */
	if ((0 == aChAllele1) || (0 == aChAllele2))
	{
		return YES;
	}

	
	/* only mother's markers are unknown */
	if ((0 == aMAllele1) || (0 == aMAllele2))
	{
		return YES;
	}

	/* check compatibility to mother's markers */
	if ((YES == IsAlleleIn(aChAllele1, aMAllele1, aMAllele2)) ||
		(YES == IsAlleleIn(aChAllele2, aMAllele1, aMAllele2)))
		{
			return YES;
		}
		
	return NO;
}


/***************************************************************************************
Function name :CheckNuclearFam
The function recieves aNuclearFam and returns ckrOK after checks 6,7,8 and 9. 
/***************************************************************************************/
ckrResult CheckNuclearFam(NuclearFam * aNuclearFam, int aMarkerIdx, 
						  Format aFormat, ErrorLL * aErrorLL)
{
	int aChAllele1,aChAllele2;
	int aFAllele1,aFAllele2;
	int aMAllele1,aMAllele2;
	Person * aChild;
	int errFlag = 0;
	int res;
			
	/* Get parents alleles*/
	GetMarkers(aNuclearFam->mFather,&aFAllele1, &aFAllele2, aMarkerIdx);	
	GetMarkers(aNuclearFam->mMother,&aMAllele1, &aMAllele2, aMarkerIdx);
	 
	/* check 6 - A child and parent's alleles are incompatible*/
	aChild = aNuclearFam->mChild;
	while (NULL != aChild)
	{
		GetMarkers(aChild, &aChAllele1, &aChAllele2, aMarkerIdx);

		if ((aFormat == AUTOSOMAL) || (aChild->mGender != male))/*==my==*/
		{
			if (NO == ChkChildParentsCompatible(aChAllele1, aChAllele2,
											   aFAllele1,  aFAllele2,
											   aMAllele1,  aMAllele2))
			{		
				/* SetErrorInPerson(aErrorLL, aMarkerIdx, aChild->mID, CHILD_AND_PARENTS_NOT_COMPATIBLE_ERROR);*/
				aChild->mProblem = '*';
				errFlag = 1;
			}
		}

		if ( (aFormat == SEXLINKED) && (aChild->mGender == male))/* ==my== */
		{
			if (NO == ChkChildParentsCompatibleSex(aChAllele1, aChAllele2, 
										            aMAllele1,  aMAllele2))
			{
				aChild->mProblem = '*';
				errFlag = 1;
			}
		}

		aChild = aChild->mSibling;
	}
	
	if(errFlag){
        SetErrorInNuclearFam(aErrorLL, aMarkerIdx, aNuclearFam, CHILD_AND_PARENTS_NOT_COMPATIBLE_ERROR);
		return ckrErr;
	}


	/* check 7+8+9 */
	res = ChkAllelesInFullSib(aNuclearFam->mChild, aMarkerIdx, aErrorLL);
	return res;

	/*if (res != ckrOK)
	{
		return ckrErr;
	}
	if (errFlag == 1)
	{
		return ckrErr;
	}

	return ckrOK;
	*/
}
/***************************************************************************************
Function name :SetAllFamUnchecked

/***************************************************************************************/
void SetAllFamUnchecked(Pedigree * aPedigree)
{
	int i;
	NuclearFam * nuclearFam = NULL;

	for (i=0; i<aPedigree->mPersonCnt; ++i)
	{
		nuclearFam = aPedigree->mPersonArray[i]->mParentsFam;
		if (NULL == nuclearFam)
		{
			continue;
		}
		nuclearFam->mIsChecked = 0;

	}
}

