/*************************************************************************************
 Marker.c
 Marker is a struct which represents a marker. It holds the marker's 2 alleles.
*************************************************************************************/
#include "pc_Marker.h"
#include "pc_Common.h"
#include <stdio.h>
#include <stdlib.h>


/*************************************************************************************
Function:CreateMarkerLLNode recieves two alleles.
This function creates a new linked list of aMarker node, it also intializes its fields.
*************************************************************************************/
MarkerLLNode * CreateMarkerLLNode(int aAllele1, int aAllele2)
{
	MarkerLLNode * aMarker;

	aMarker = (MarkerLLNode *)malloc(sizeof(MarkerLLNode));
	if (aMarker == NULL)
		return NULL;
	
	/* set the fields of the newly created node */
	aMarker->mNext = NULL;
	aMarker->mMarker.mAllele1 = aAllele1;
	aMarker->mMarker.mAllele2 = aAllele2;

	aMarker->mIsMarked    = 0;
	aMarker->mTmpIsMarked = 0;
	return aMarker;
}

/*************************************************************************************
Function:DeleteMarkerLLNode
This function deletes aMarker. 
************************************************************************************/		 
void DeleteMarkerLLNode(MarkerLLNode * aMarker)
{
	if (aMarker != NULL)
	{
		free(aMarker);
	}
}


/*************************************************************************************
Function:CreateMarkerArray
This function creates a new MarkerArray, according to input (size of array)
**************************************************************************************/
Marker * CreateMarkerArray(int aSize)
{
	Marker * aMarker;

	if (aSize<=0)
		return NULL;

	aMarker = (Marker *)malloc(sizeof(Marker)*aSize);
	return aMarker;
}

/*************************************************************************************
Function:CopyMarkerMarkerLLN, recieves aMarker and aMarkerLLN.
Copies values of alleles from linked list of markers to aMarker struct.
*************************************************************************************/
		 
void CopyMarkerMarkerLLN(Marker * aMarker, MarkerLLNode * aMarkerLLN)
{
	aMarker->mAllele1 = aMarkerLLN->mMarker.mAllele1;
	aMarker->mAllele2 = aMarkerLLN->mMarker.mAllele2;
}

/*************************************************************************************
Function:DeleteMarkerLL recieves aMarkerLL.
Deletes aMarkerLL by going over the linked list.
*************************************************************************************/		 
void DeleteMarkerLL(MarkerLLNode ** aMarkerLL)
{
	MarkerLLNode * markerLL1 = *aMarkerLL;
	MarkerLLNode * markerLL2 = NULL;

	
	/* go over the aMarkerLL */
	
	while (NULL != markerLL1)
	{
		markerLL2 = markerLL1->mNext;
		free (markerLL1);
		markerLL1 = markerLL2;
	}

	*aMarkerLL = NULL;
}

/*************************************************************************************
Function:AddMarkerToList recieves a aMarkerLLN (=marker) and a aMarkerLLLst.
Adds the new node of the kind marker to linked list.
**************************************************************************************/
void AddMarkerToList(MarkerLLNode * aMarkerLLN, MarkerLLNode ** aMarkerLLLst)
{
	MarkerLLNode * aTmpMarkerLLN;

	aTmpMarkerLLN = *aMarkerLLLst;
	*aMarkerLLLst = aMarkerLLN;
	aMarkerLLN->mNext = aTmpMarkerLLN;
}

/*************************************************************************************
Function:CreateAddMarker recieves two alleles and aMarkerList.
The function creates a new MarkerLLNode and adds it to list.
*************************************************************************************/
ckrResult CreateAddMarker(int aAllele1, int aAllele2, MarkerLLNode ** aMarkerList)
{
	MarkerLLNode * marker = NULL;

	marker = CreateMarkerLLNode(aAllele1, aAllele2);
	if (NULL == marker)
	{
		return ckrMemErr;
	}
	
	/* Go to add the newly created marker to aMarkerList*/
	AddMarkerToList(marker, aMarkerList);
	return ckrOK;
}

