/*************************************************************************************
 Pedigree.h
 Pedigree is a struct which represenets a pedigree. It holds all the information needed
 for the application's run, on the pedigree, and points to its members.
*************************************************************************************/
#ifndef _PC_PEDIGREE_H
#define _PC_PEDIGREE_H

#include <stdio.h>
#include "pc_Common.h"
#include "pc_Person.h"
#include "pc_Marker.h"
#include "pc_Error.h"
#include "pc_PedAlleles.h"
#include "pc_Recommendation.h"
#ifdef WIN32
#include "..\PedData.h"
#include "..\globals.h"
#else
#include "../PedData.h"
#include "../globals.h"
#endif
typedef struct _Pedigree
{
	char mName[MAX_PEDIGREE_NAME_LEN];	/* family name*/
	Format mFormat; /* file format: SEXLINKED or AUTOSOMAL */
	int mPersonCnt; /* number of people*/
	int mMarkerCnt; /*number of markers*/
	
	Person **  mPersonArray; /* list of people*/
	PedAlleles * mPedAlleles; 

	ErrorLL * mErrorLL1; /* parsing*/
	ErrorLL * mErrorLL2; /* medelina + lang. algorithm*/
//	ErrorLL * mErrorLL3;

	struct _RecommendationLL * mRecommendationLL;

	int * mCriticalGenotypesLevel; /* an array that contains for each marker 
								      the number of ZIVUGS that had to be done for 
									  lang algorithm */

	struct _Pedigree * mNext;

} Pedigree;

ckrResult SetRecommendation(struct _Pedigree* aPedigree, int aMarkerIdx, int aI, int * unknownIndices);




Pedigree * CreatePedigree(Person * aLLPerson, 
						  int aPersonCnt, 
						  int aMarkerCnt, 
						  PedAlleles * aPedAlleles, 
						  ErrorLL *  aPedigreeErrorLL);

void SetPedigreeName(Pedigree * aPedigree, const char * aName); 

/*void SetPedigreeFormat(Pedigree * aPedigree, Format aFormat); */

/* TODO [MAX_PEDIGREE_NAME_LEN] */

void DeletePedigree(Pedigree * aPedigree);

int PrintPedigreeErrors(int aLevelNum, Pedigree * aPedigree ,FILE * output);
int PrintPedigreeRecommendations(Pedigree * aPedigree,FILE * output);


#endif
