/*****************************************************************************/
/*  EliminationOrderGraph.h													 */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _SL_ELIMINATION_ORDER_GRAPH_H_
#define _SL_ELIMINATION_ORDER_GRAPH_H_

#include <stdio.h>
#include <stdlib.h>
#include <limits.h>
#include "sl_Auxilary.h"
#include "sl_Node.h"
#include "sl_EliminationOrder.h"


/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* This is a 2-dimensional bit array, which contains information about the graph 
   (adjacency matrix, indicates for every 2 vertices whether they are neighbors). */

typedef struct adjMatrixT {
	unsigned char	a:1, b:1, c:1, d:1, e:1, f:1, g:1, h:1;
} adjMatrixT;

typedef adjMatrixT **adjMatrix;



typedef struct vertex {
	int				numNeighbors;
	boolean			isMarked;
	struct vertex	**neighborList;
	int				numNeighMarked;
	elimOrderVar	*varInProd;
} vertex;


typedef struct graph {
	int				numVertices;
	vertex			*vertexList;
	adjMatrix		adjMat;
} graph;


/* This structure holds the data on one of the vertices in the vertex pool. */
typedef struct vertexInPool {
	vertex				*v;
	vertex				**vPtr;
	int					score;		/* the score of this vertex. */
	double				relProb;	/* the relative probability of this
									   vertex to be chosen. */
	struct vertexInPool	*prev;
	struct vertexInPool	*next;
} vertexInPool;


/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/



#endif		/*	_ELIMINATION_ORDER_GRAPH_H_	*/