/*****************************************************************************/
/*  EliminationOrderStructures.h											 */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _SL_ELIMINATION_ORDER_STRUCTURES_H_
#define _SL_ELIMINATION_ORDER_STRUCTURES_H_


#include <stdio.h>
#include <stdlib.h>
#include "sl_Auxiliary.h"
#include "sl_Node.h"
#include "sl_Settings.h"

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* This structure holds information on one node in the elimination order. */

typedef struct elimOrderNode {
	boolean		isBreakNode;		/* Is TRUE if this is a break node (the variable
									   is to be conditioned on instead of eliminated). */
	node		*originalNode;		/* Points to the original node in the Bayesian
									   network which this node represents. */
} elimOrderNode;


/* This structure holds information on the elimination order for a Bayesian network
   which represnts one pedigree. */

typedef struct elimOrderPed {
	elimOrderNode	*elimOrderArr;	/* The elimination order array. */
	long			currElimNode;	/* The next node that needs to be eliminated. */
	long			numNodes;		/* The number of nodes in the array. */

	double			sumTableSizes;	/* The sum of the sizes of all probability tables created
									   during the inference process. */
	int				numBreakNodes;	/* The number of break nodes in the elimination order. */
	boolean			isLGOrder;		/* A flag that indicates whether the nodes are ordered in
									   a locus by locus order (style Lander-Green). */
#if CALC_MEM_REQ	
	double			maxMemReq;		/* The estimated maximum memory requirement. */
#endif
#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
	int				treeWidth;		/* The treewidth of the elimination order. */
#endif

} elimOrderPed;

/* This structure is an array of elimination orders (one for each pedigree). */

typedef elimOrderPed* elimOrder;


/* Update the elimination-order-node with a pointer to the original node in the
   Bayesian network, and whether to eliminate or condition on this node.  */

#define updateElimNode(elimNode, isBreakN, originalN) \
    elimNode->isBreakNode = isBreakN; \
	elimNode->originalNode = originalN;



#endif	/* _ELIMINATION_ORDER_STRUCTURES_H_ */
