/*****************************************************************************/
/*  Globals.h															     */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _SL_GLOBALS_H_
#define _SL_GLOBALS_H_

#define _SL_CRTDBG_MAP_ALLOC

#ifdef WIN32
	#define _SL_CRTDBG_MAP_ALLOC
#endif
	#include <stdlib.h>
#ifdef WIN32
	#include <crtdbg.h>
#endif
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include "sl_AlleleRecoding.h"
#include "sl_ReadData.h"
#include "sl_LociData.h"
#include "sl_PedValueElimination.h"
#include "sl_EliminationOrder.h"
#include "sl_Settings.h"
#include "sl_Mem.h"

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

typedef char xLabel[100];		/* A label on the X-axis (for generation of a PS-graph). */

/* A data-structure which holds information on the nodes that have been updated during
   the value elimination stage. */

typedef struct updatedNodesPed {
	node		**constNodes;
	int			numConstNodes;
	valEntry	**updatedValEntries;
	int			numUpdatedValEntries;
} updatedNodesPed;


/* A prototype of function which receives a double (x) and other information, and
   returns double. */
typedef double (*funcType)(double *x, void *funcData);

typedef struct evaluateLikeData {
	int				numTraitLoci;
	retValFuncType	retFunc;
	boolean			printResults;
} evaluateLikeData;


/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

/* Force genehunter algorithm */
int RUN_GH ;
/* Force superlink algorithm */
int RUN_SL;

pedData				peds;			/* Holds the data on the input pedigree(s). */
lociData			loci;			/* Holds the data on the loci in the analysis. */
recombinationData	recombData;		/* Holds the recombination data. */
programData			progData;		/* Holds the data required by the analysis programs. */		
results				totalRes;		/* Holds the results of all likelihood evaluations. */

currHeuristicEO		heuristicEO;

FILE *outFile;

/* File descriptor for the statistics file. */

#if PRINT_VARIOUS_STATISTICS_LEVEL
	FILE *statisticsFile;
#endif 

char	*datafileName, *pedfileName;
boolean	readElimOrder;
char	*eliminationOrderFileName;


double	breakProductBottomThreshold, bigBreakThreshold, smallBreakThreshold;

#if PRINT_MATRIX_MULT_STATISTICS
	FILE	*matrixFile;
#endif

#if PRINT_VARIOUS_STATISTICS_LEVEL
	int		numOneUnite, numTwoUnite, numFourUnite;
	FILE	*varsUniteFile;
#endif

pedigree	*currPed;

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

double calcLikelihoodOneLocus();
void calcLikelihoodSuperLinkmap();
void incThetas(thetaCompTheta *maleTheta, thetaCompTheta *femaleTheta, double inc);
void calcLikelihoodSuperMlink();
void calcLikelihoodSuperPos();
void calcLikelihoodSuperGH();
void updateMarkerName(xLabel *markerNamesEntry, locus *loc, int ind);
double superHaploProg();
void calcLikelihoodSuperLinkmapApprox();
void calcLikelihoodSuperOptlink();
double calcLikeSuperOptlinkOneModel(double *xMax, evaluateLikeData *evalData, 
									scanIntervalData *scanData);
void createExportBayesProg();
void OptimizeMarginalLikelihood();
double calcLikelihoodNoDisease();

/* Various functions called by the above functions. */

#if (PRINT_VARIOUS_STATISTICS_LEVEL > 2)
	void calcVariousStatistics(product** prodArr, int numProds);
#endif

void iterateOneLocus(int numMarkersScan, xLabel *markerNames);
void iterateTwoLoci();
boolean performOneIter(elimOrder *elimOrderArrs, boolean *areValsElim, 
					   boolean *isChangedNet, boolean createBN);

void updateJointThetas();
void printResults_OneIter(int numThetas, int numPeds, boolean printLodScore);
void printResults_OneIterLod(int numPeds);
void printResults_OneIterWOLod(int numPeds);
void updateMax(retValFuncType func);

void printResultsSummary_OneLocus(int numMarkers, xLabel *markerNames);
void printResultsSummary_OneLocus_MMLS(int numMarkers, xLabel *markerNames);
void printResultsSummary_TwoLoci();

void freeAfterOrder(product	**prodArr, elimOrder *elimOrderArrs);
void exitAfterOrder(product	**prodArr, elimOrder *elimOrderArrs);


void initResultList();
void freeResultList();
void freeList(resNode *resPtr);
void reverseResList();

boolean calcLikeNoLinkage(elimOrder	*elimOrderArrs, boolean	*areValsElim,
						  boolean *isChangedNet);
void updateLogLikes(int numPeds);

double envEvaluateLike(double *x, void* funcData);


#endif /* _GLOBALS_H_ */
