/*****************************************************************************/
/*  Haplotype.h															     */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _SL_HAPLOTYPE_H_
#define _SL_HAPLOTYPE_H_

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include "sl_LociData.h"
#include "sl_EliminationOrder.h"
#include "sl_Settings.h"
#include "sl_Globals.h"
#include "sl_ProductValueElimination.h"
#include "sl_ArgMax.h"

/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/

#define		UNKNOWN		-1

#define HAPLOTYPE_FILE_NAME  "haplotypes.txt"


/* A type of a function that updates the child's genotype according to his parent's genotype. */

typedef void (updateChLociHaplo)(person* parent, person* child, int locusInd);

typedef void (updateFounderHaplo)(person *founder);

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

typedef struct optValsVar {
	int		optVal;
	int		optValInd;
} optValsVar;

/* This structure stores the list of optimizing values of some variable (represented by 
   the node 'mainNode') for each tuple for the rest of the variables in the function,
   obtained in the first part of the MPE inference. */

typedef	struct optimizingValsFunc {
	boolean			used;				/* A flag that indicates whether this slot is used. */
	node			*mainNode;			/* Pointer to the node in which elimination this 
										   function was created. */
	int				numNodes;			/* The number of nodes this function depends on. */
	node			**nodes;			/* An array of pointers to the nodes of the function. */
	unsigned long	sizeOfTable;		/* The size of the list 'optVals'. */
	optValsVar		*optVals;			/* The list of values of 'mainNode' which maximized 
										   the original function for the assignment . */
	long			baseInd;			/* An auxiliary field that is used when performing
										   different operations on the function. */
} optimizingValsFunc;


typedef struct constNode {
	node		*Node;
	int			val;
	int			valInd;
} constNode;

typedef struct optimizingValsProd {
	optimizingValsFunc	*optimizingValsFuncArr; /* An array that stores all the 
												   optimizingVals functions. */
	int					numFuncs;				/* The number of function slots in the array. */
	int					currNumFuncs;			/* The number of function currently in the array. */
	optimizingValsFunc	*nextFuncPlace;			/* Pointer to the next empty slot in the 
												   function array. */
	constNode			*constNodes;			/* An array which holds information on the
												   nodes that have been updated during the
												   value elimination stage. */
	int					numConstNodes;			/* The number of nodes which have been 
												   determined to be constant during the 
												   value elimination stage. */
} optimizingValsProd;

/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

int						numMarkers;

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void updateAllelesLociHaplo();
double findMostLikelyHaplotypes(product** prodArr, elimOrder *elimOrderArrs, 
							  argMaxData *argMaxArr,PedigreeDataStruct *origPedigree);
void findMostLikelyHaplotypesPed(product *prod, int pedNum, elimOrderPed *elimOrderArr,
									argMaxData argMax);
Probability inferMPEProduct(product *prod, elimOrderPed *elimOrderArr, 
						    argMaxData *head, argMaxData *tail,  
							float problemSize, float solved);
Probability inferMPEProductWithVariableBreaking(product *prod, variable *breakVar,
											    elimOrderPed *elimOrd,
											    argMaxData *head, argMaxData *tail, 
												float problemSize, float solved);
boolean eliminateVarMax(product *prod, variable *elimVar, Probability *tempRes,
						argMaxData *head, boolean *isZeroFunc);
boolean multiplyAllFuncsMaxSimp(function *combinedFunc, variable *elimVar, int *optVals);
boolean multiplyAllFuncsMaxComplex(function *combinedFunc, variable *elimVar, int *optVals);
Probability multiplyAllOneVarFuncsMax(variable *elimVar, double logMultFactor,
									  int *optVals);
void traceMPEProduct(argMaxData head);
void updateOptVarVal(argMaxData argMax);

void traverseUpHaplo(person *currPerson, updateChLociHaplo updateChLociHap,
					 updateFounderHaplo updateFounderHap);
void traverseDownHaplo(person *currPerson, updateChLociHaplo updateChLociHap,
					   updateFounderHaplo updateFounderHap);

void updateFounderHaploAutosomal(person *currPerson);
void updateFounderHaploSexLinked(person *currPerson);
void updateChHaplo(updateChLociHaplo updateChLociHap, person *parent, person *child);
void updateChLociHaploAutosomal(person* parent, person* child, int locusInd);
void updateChLociHaploSexLinked(person* parent, person* child, int locusInd);

void printMPEHaplotypes();
void printMPEHaplotypesSexLink();

void printHaplotype(person *personPtr, char haploSex, node **locusNodes, FILE *haplotypeFile);
void printHaplotypePaternal(person *personPtr, char haploSex, FILE *haplotypeFile);
int printRecomb(person *personPtr, FILE *haplotypeFile, int *numIRecombHaplo);
int printRecombMaternalHaplo(person *personPtr, FILE *haplotypeFile, int *numIRecombHaplo);
void printRecombFounder(person *personPtr, FILE *haplotypeFile);

#endif /* _HAPLOTYPE_H_ */
