/*****************************************************************************/
/*  LociData.c																 */
/*  Author: Ma'ayan Fishelson					                             */
/*****************************************************************************/

#include "sl_LociData.h"
#include "sl_IterateAff.h"
#include <string.h>
#include <math.h>

/******************************************************************************
*
*       Function        fillLociInformation
*       Parameters		number and markers themselves, wheter to use disease
*       Output          none
*       Description		Updates correspondig data stucture.
*       Comments		Instead of readLociSL
*
******************************************************************************/

void  fillLociInformation(int numLoci,int* markersUsed, Bool diseaseUsed){
	locus		*locusPtr;
	LocusDataStruct *origLociPtr; 
	int locusInd;
	
	for (locusInd = 0; locusInd < numLoci; locusInd++) {
		locusPtr =  &(loci.locusArr[locusInd]);
		//Anna
		if (diseaseUsed && locusInd < ProgramData.numberOfDiseaseLoci){
			loci.traitLociPos[locusInd] = locusInd;
			locusPtr->type = (ProgramData.numberOfDiseaseLoci == 1)?affection:null;
			locusPtr->numAlleles = 2;
			myMalloc(locusPtr->geneFrequencies, sizeof(Probability) * locusPtr->numAlleles);
			memcpy(locusPtr->geneFrequencies, DiseaseData.freq + 2*locusInd,(2 *sizeof(double)));
			if (ProgramData.numberOfDiseaseLoci == 1){ 
				readAffectionLocusSL(0);
			}else if (locusInd == 1){
				readNullLocusSL(1);
			}
		}else{
			origLociPtr = LociData+markersUsed[locusInd - diseaseUsed* ProgramData.numberOfDiseaseLoci];
			locusPtr->type = ALLELES;
			locusPtr->numAlleles = origLociPtr ->numOfAlleles;
			myMalloc(locusPtr->geneFrequencies, sizeof(Probability) * locusPtr->numAlleles);
			memcpy(locusPtr->geneFrequencies, origLociPtr->alleleFreq,(locusPtr->numAlleles *sizeof(double)));
		}
		
		/* 7.1.	Initially the data-structure order is the same as the genetic map. */
				 
		loci.mapToData[locusInd] = locusInd;
	}
}
/******************************************************************************
*
*       Function        readLocusFile
*       Parameters		none
*       Output          none
*       Description		Reads the locus file and initializes the variables that
*						hold the data regarding the loci being analyzed. 
*       Comments		none
*
******************************************************************************/
void readLocusFileSL(int numMarkers,int* markersUsed, Bool diseaseUsed, int progCode)
{
	int				autosomal,  locusInd, thetaInd, 
					  riskLocus, numLoci;
	double			maleDist;
	thetaCompTheta	*maleThetas, *femaleThetas;
	Probability		maleThetas2Seg, femaleThetas2Seg;
	int markerInd;
	double tmp;
	
	/* 1.	Read 1st line in locus file. */

	//fscanf(dataFile, "%d%d%d%d", &numLoci, &riskLocus, &autosomal, &(progData.programCode));
numLoci = numMarkers + diseaseUsed*ProgramData.numberOfDiseaseLoci;
riskLocus = ProgramData.riskLocus;
autosomal = ProgramData.sexLinked;
progData.programCode = progCode;
	if ((progData.programCode == BAYES_NET_XML_AFTER_ELIM) ||
		(progData.programCode == BAYES_NET_XBN_AFTER_ELIM) ||
		(progData.programCode == PRINT_ELIM_ORDER)){
			RUN_CREATE_BAYES = 1;
			RUN_SL = 1;
		}else{
			RUN_CREATE_BAYES = 0;
		}
		/* 2.	Read the rest of the line and determine the number of null loci. 
			If the file format is the same as in Fastlink (the number of null loci isn't
			specified), then there are 0 null loci. */

		
	
	

	loci.numTraitLoci = ProgramData.numberOfDiseaseLoci;
	myMalloc(loci.traitLociPos, sizeof(int) * loci.numTraitLoci);
	myMalloc(progData.scanData, sizeof(scanIntervalData) * loci.numTraitLoci);

	numTraitLociRead = 0;

	/* 3.	Determine whether the analysis is sex-linked or not. */

	loci.sexLink = (autosomal == 1) ? TRUE : FALSE;

	/*if (loci.sexLink)
	    printf("\nSEXLINKED DATA\n");
	else
		printf("\nAUTOSOMAL DATA\n");
*/
	/* 4.	Read the 2nd line in locus file (currently these numbers are ignored). */

	//fscanf(dataFile, "%d%lf%lf%d%*[^\n]", &mutLocus, &mutRate, &mutRate, &coupling);
	//getc(dataFile);


	myMalloc(loci.lociOrder, numLoci * sizeof(int));

	/* 5.	Read the 3rd line in locus file (the order of the loci along the chromosome). */

	for (locusInd = 0; locusInd < numLoci; locusInd++) {
		//fscanf(dataFile, "%d", &locusInputInd); 
		loci.lociOrder[locusInd] = locusInd;
	}
//	fscanf(dataFile, "%*[^\n]"); 
//	getc(dataFile);

	/* 6.	Allocate memory for the data-structures that hold the data on the loci being
			analyzed and on the theta values between each the loci. */

	loci.numLoci = numLoci;
	myMalloc(loci.locusArr, sizeof(locus) * numLoci);

	if (numLoci > 1) {
		myMalloc(recombData.maleThetas, sizeof(thetaCompTheta) * (numLoci - 1));
		myMalloc(recombData.femaleThetas, sizeof(thetaCompTheta) * (numLoci - 1));
	}

	maleThetas = recombData.maleThetas;
	femaleThetas = recombData.femaleThetas;

	myMalloc(loci.mapToData, sizeof(int) * numLoci);

	/* 7.	Read the data on the loci. */

		fillLociInformation( numLoci, markersUsed, diseaseUsed);
/*	if ((numNullLoci == 2) && (numTraitLociRead < 2)) {
		inputErr("Invalid no. of complex affection loci (%d instead of 2).\n", numTraitLociRead);
	}*/

	/* 8.	Read the rest of the values (whether there is sex-difference in recombination 
			fractions and whether interference should be taken into account). */

	//fscanf(dataFile, "%d%d%*[^\n]", &(recombData.sexDiff), &(recombData.interference));
	//getc(dataFile);
	recombData.sexDiff = ProgramData.sexDiff;
	recombData.interference = ProgramData.interference;
	/* 9.	If there are more than 3 loci and interference is to be taken into account,
			print an error message. */

	if ((recombData.interference != 0) &&  (numLoci > 3)) {
		inputErr("Currently interference can only be specified for a 3-loci analysis.\n");
	}

	/* 10.	If interference option 2 has been chosen and the Super-Mlink program option
			is being used, update the mapping function used to Kosambi's mapping function.
			Otherwise, set it to Haldane's mapping function. */

	if ((recombData.interference == 2) && (progData.programCode == SUPER_MLINK_PROG)) {
		progData.haldane = FALSE;
	}
	else {
		progData.haldane = TRUE;
	}

	/* 11.	Read the male recombination values between each two adjacent loci. */

	for (thetaInd = 0, markerInd  = 1; thetaInd < numLoci-1; thetaInd++) {
		
		//fscanf(dataFile, "%lf", &maleThetas[thetaInd][0]);
		if (thetaInd  < ProgramData.numberOfDiseaseLoci && diseaseUsed)
			maleThetas[thetaInd][0] = 0;
		else{
			tmp = fabs(LociData[markersUsed[markerInd]].malePosition - LociData[markersUsed[markerInd-1]].malePosition);
			maleThetas[thetaInd][0] = getHaldaneMToRec(tmp/100);
			markerInd++;
		}

		maleThetas[thetaInd][1] = 1 - maleThetas[thetaInd][0];
	}

	/* 12.	If interference option 1 has been chosen, get the theta value between 
			the flanking loci. */

	if ((numLoci > 2) && (recombData.interference == 1)) {
		fscanf(dataFile, "%lf", &maleThetas2Seg);
		recombData.maleThetas2Seg = maleThetas2Seg;

		/* 12.1.	Calculate the probabilities for each combination of existence/lack 
					of recombination in these 2 successive intervals along the chromosome. */

		recombData.maleJointThetas[0] = 1 - 0.5*(maleThetas[0][0] + maleThetas[1][0] + maleThetas2Seg);
		recombData.maleJointThetas[1] = 0.5 * (-maleThetas[0][0] + maleThetas[1][0] + maleThetas2Seg);
		recombData.maleJointThetas[2] = 0.5 * (maleThetas[0][0] - maleThetas[1][0] + maleThetas2Seg);
		recombData.maleJointThetas[3] = 0.5 * (maleThetas[0][0] + maleThetas[1][0] - maleThetas2Seg);
	}

	/* 13.	Read the rest of the line. */

/*	fscanf(dataFile, "%*[^\n]"); 
	getc(dataFile);*/

	/* 14.	Update the female recombination fractions, according to whether there's a 
			sex-difference or not. */

	switch (recombData.sexDiff) {

		case 0: 	/* 14.1.	If there's no sex-difference, the female thetas are
								exactly the same as the male thetas. */

					for (thetaInd = 0; thetaInd < numLoci-1; thetaInd++) {
						femaleThetas[thetaInd][0] = maleThetas[thetaInd][0];
						femaleThetas[thetaInd][1] = maleThetas[thetaInd][1];

						recombData.femaleJointThetas[0] = recombData.maleJointThetas[0];
						recombData.femaleJointThetas[1] = recombData.maleJointThetas[1];
						recombData.femaleJointThetas[2] = recombData.maleJointThetas[2];
						recombData.femaleJointThetas[3] = recombData.maleJointThetas[3];
					}
					break;
		case 1:		
					/* 14.1.	If sex-difference option 1 has been chosen, read the genetic
								distance between the male and female thetas, and multiply the
								male theta by the genetic distance to obtain the female theta. */

				/*	fscanf(dataFile, "%lf%*[^\n]", &(recombData.geneticDist));
					getc(dataFile);*/
					recombData.geneticDist = ProgramData.geneticDist;
					for (thetaInd = 0; thetaInd < numLoci-1; thetaInd++) {
						maleDist = getHaldaneRecToM(maleThetas[thetaInd][0]);
						updateThetaCompTheta(femaleThetas[thetaInd],
											 getHaldaneMToRec(maleDist * recombData.geneticDist));
					}

					/* 14.2.	If interference option 1 has been chosen, compute also the female
								theta value between the flanking loci. */

					if (recombData.interference == 1) {
						maleDist = getHaldaneRecToM(maleThetas2Seg);
						recombData.femaleThetas2Seg = getHaldaneMToRec(maleDist * recombData.geneticDist);
					}
					break;
		case 2:
					/* 14.1.	If sex-difference option 2 has been chosen, read the female thetas. */
		
					for (thetaInd = 0; thetaInd < numLoci-1; thetaInd++) {
						//	fscanf(dataFile, "%lf", &femaleThetas[thetaInd][0]);

						if (thetaInd  < ProgramData.numberOfDiseaseLoci && diseaseUsed)
							femaleThetas[thetaInd][0] = 0;
						else{
							tmp = fabs(LociData[markersUsed[markerInd]].femalePosition - LociData[markersUsed[markerInd-1]].femalePosition);
							maleThetas[thetaInd][0] = getHaldaneMToRec(tmp/100);
							markerInd++;
						}
							femaleThetas[thetaInd][1] = 1 - femaleThetas[thetaInd][0];
					}

					/* 14.2.	If interference option 1 has been chosen, read also the female
								theta value between the flanking loci. */

					if (recombData.interference == 1) {
						fscanf(dataFile, "%lf", &(recombData.femaleThetas2Seg));
					}


					/* 14.2.	Read the rest of the line. */

				//	fscanf(dataFile, "%*[^\n]"); 
				//	getc(dataFile);
	}

	/* 15.	If interfence option 1 has been chosen, and there's a sex-difference */

	if ((numLoci > 2) && ((recombData.sexDiff != 0 ) && (recombData.interference == 1))) {

		femaleThetas2Seg = recombData.femaleThetas2Seg;

		/* 15.1.	Calculate the probabilities for each combination of existence/lack 
					of female recombination in these 2 successive intervals along the chromosome. */

		recombData.femaleJointThetas[0] = 1 - 0.5*(femaleThetas[0][0] + femaleThetas[1][0] + femaleThetas2Seg);
		recombData.femaleJointThetas[1] = 0.5 * (-femaleThetas[0][0] + femaleThetas[1][0] + femaleThetas2Seg);
		recombData.femaleJointThetas[2] = 0.5 * (femaleThetas[0][0] - femaleThetas[1][0] + femaleThetas2Seg);
		recombData.femaleJointThetas[3] = 0.5 * (femaleThetas[0][0] + femaleThetas[1][0] - femaleThetas2Seg);
	}

	/* 16.	Read the program-specific data. */

	switch (progData.programCode) {

		case SUPER_LINKMAP_PROG:		readSuperLinkmapSpecificData();
										return;

		case SUPER_MLINK_PROG:			readSuperMlinkSpecificData();
										return;

		case SUPER_POS_PROG:			readSuperPosSpecificData();
										return;
										
		case SUPER_GH_PROG:				readSuperGHSpecificData();
										return;

		case SUPER_OPTLINK_PROG: 
		case SUPER_OPTMARGINAL_PROG:	readSuperOptLinkSpecificData();
										return;
	}
}


/******************************************************************************
*
*       Function        readLocus
*       Parameters		locusInd (the index of the locus being read).
*       Output          none
*       Description		Reads the information on one locus from the locus file.
*       Comments		none
*
******************************************************************************/
void readLocusSL(int locusInd)
{
	int			type;
	locus		*locusPtr;


	/* 1.	Update a pointer to the position in the loci data-structure that 
			would store the information on this locus. */

	locusPtr =  &(loci.locusArr[locusInd]);

	/* 2.	Read the type of locus. */

	fscanf(dataFile,"%d",&type);

	/* 3.	Read the number of ALLELES, allocate memory for an array of 
			gene-frequenices and initialize it. */

	readGeneFrequenciesSL(locusInd);

	/* 4.	Update the locus type, and read the rest of the data regarding the 
			locus according to its type. */

	switch (type) {

		case AFFECTION_LOCUS:
			locusPtr->type = affection;
			readAffectionLocusSL(locusInd);
			break;

		case ALLELES_LOCUS:
			locusPtr->type = ALLELES;
			break;

		case NULL_LOCUS:
			locusPtr->type = null;
			readNullLocusSL(locusInd);
			numTraitLociRead++;
			break;

		default :
			inputErr("Invalid locus type in locus #%d (the only types allowed are 1,3 or 4).\n", 
					 locusInd + 1);
	}
}


/******************************************************************************
*
*       Function        readGeneFrequenciesSL
*       Parameters		locusInd (the index of the locus whose gene-frequencies
*						are being read currently).
*       Output          none
*       Description		Reads the number of ALLELES of the current locus, 
*						allocates memory for the array of gene-frequencies and
*						initializes it according to the input file.
*       Comments		none
*
******************************************************************************/
void readGeneFrequenciesSL(int locusInd)
{
	int				numAlleles, count;
	Probability		prob;
	locus			*locusPtr;
	char			tmpStr[100], ch, *index;
	double			sumGeneFreq;


	/* 1.	Update a pointer to the position in the loci data-structure that 
			would store the information on this locus. */

	locusPtr = &(loci.locusArr[locusInd]);

	/* 2.	Read the number of ALLELES. */

	fscanf(dataFile,"%d",&numAlleles);
	locusPtr->numAlleles = numAlleles;

	/* 3.	Read the name of the locus, if its specified, and allocate memory to store it. */

	locusPtr->name = NULL;


	while (fscanf(dataFile,"%c",&ch) == 1) {
		if (ch == ' ' || ch == '\t')
			continue;
		if (ch == '\n')
			break;
		if (ch == '#') {
		//  changed on 9/12/2004 by Anna
	   	    fscanf(dataFile, "%[^\n]\n", tmpStr);
			index = strchr(tmpStr, '#');
			if (index ){
				index[0] = '\0';
				myMalloc(locusPtr->name, sizeof(char) * (strlen(tmpStr)+1));
				sprintf(locusPtr->name, "%s", tmpStr);
			}
			//end change
			break;
		}
		else {
			fscanf(dataFile, "%*[^\n]\n");
			break;
		}
	}

	/* 4.	Allocate memory for the gene frequencies array. */

	myMalloc(locusPtr->geneFrequencies, sizeof(Probability) * numAlleles);

	/* 5.	Read the gene frequencies. */

	sumGeneFreq = 0.0;

	for (count = 0; count < numAlleles; count++) {
		fscanf(dataFile, "%lf",&prob);
		
		if ((prob == 0.0) || (prob > 1.0)) {
			inputErr("Invalid gene frequency for locus #%d, allele #%d. \n", 
						locusInd+1, count+1);
		}
		
		(locusPtr->geneFrequencies)[count] = prob;
		sumGeneFreq += prob;
	}

	/* 6.	Read the rest of the line. */

	fscanf(dataFile, "%*[^\n]");
	getc(dataFile);

	/* 7.	Check that the gene frequencies sum up to ~ 1.0. */

	if ((sumGeneFreq > 1.001) || (sumGeneFreq < 0.999)) {
		inputErr("The gene frequencies of locus #%d (%s), don't sum to 1.0. \n", locusInd+1,locusPtr->name);
	}
}


/******************************************************************************
*
*       Function        readAffectionLocus
*       Parameters		locusInd (the index of the locus being read).
*       Output          none
*       Description		Reads the data on an affection type locus from the 
*						locus file.
*       Comments		none
*
******************************************************************************/
void readAffectionLocusSL(int locusInd)
{
	int				count,  i;
	locus			*locusPtr;


	/* 1.	Update a pointer to the position in the loci data-structure that 
			would store the information on this locus. */

	locusPtr =  &(loci.locusArr[locusInd]);

	/* 2.	Read the number of liability classes. */

	/*fscanf(dataFile,"%d%*[^\n]", &numClasses);
	getc(dataFile);
	loci.numClasses = numClasses;
	*/
	loci.numClasses = DiseaseData.numOfLiabClasses;

	/* 3.	Allocate memory for the penetrance probabilities array(s). */

	myMalloc((loci.diseaseModel).oneLocus.fPenetrances, sizeof(femalePenClass) * loci.numClasses);

	if (loci.sexLink) {
		myMalloc((loci.diseaseModel).oneLocus.mPenetrances, sizeof(malePenClass) * loci.numClasses);
	}

	/* 4.	Read the penetrances. */

	for (count = 0; count < loci.numClasses; count++) {

		/* 4.1.	For each liability class, there are three penetrances. */

		for (i = 0; i < 3; i++) { 
			/*fscanf(dataFile,"%lf",
						&((loci.diseaseModel).oneLocus.fPenetrances)[count][i][0]);
			*/
			((loci.diseaseModel).oneLocus.fPenetrances)[count][i][0] =
				DiseaseData.femalePenetrances[count][i];
			((loci.diseaseModel).oneLocus.fPenetrances)[count][i][1] = 
					1.0 - ((loci.diseaseModel).oneLocus.fPenetrances)[count][i][0];
		}

		/* 4.2.	Read the rest of the line. */

	/*	fscanf(dataFile, "%*[^\n]");
		getc(dataFile);
*/
		/* 4.3.	If the data is sex-linked, for each liability class, there are also
				two male penetrances. */

		if (loci.sexLink) {

			/* 4.3.1.	Read the male penetrances. */

			for (i = 0; i < 2; i++) { 
			/*	fscanf(dataFile,"%lf",
							&((loci.diseaseModel).oneLocus.mPenetrances)[count][i][0]);
			*/
				((loci.diseaseModel).oneLocus.mPenetrances)[count][i][0] =
					DiseaseData.malePenetrances[count][i];
				((loci.diseaseModel).oneLocus.mPenetrances)[count][i][1] = 
						1.0 - ((loci.diseaseModel).oneLocus.mPenetrances)[count][i][0];
			}

			/* 4.3.2.	Read the rest of the line. */

			/*fscanf(dataFile, "%*[^\n]");
			getc(dataFile);*/
		}
	}
}



/******************************************************************************
*
*       Function        readNullLocusSL
*       Parameters		locusInd (the index of the locus being read).
*       Output          none
*       Description		Reads the data on a Null-type locus from the locus file.
*       Comments		none
*
******************************************************************************/
void readNullLocusSL(int locusInd)
{
	int			count,  i, j;
	locus		*locusPtr;


	/* 1.	Update a pointer to the position in the loci data-structure that 
			would store the information on this locus. */

	locusPtr =  &(loci.locusArr[locusInd]);

	/* 2.	Update the array that holds the indices of the 2 Null loci. */

	//moved to fillLociInformation
	//loci.traitLociPos[numTraitLociRead] = locusInd;
	
	/* 3.	If this is the last Null Locus, read the data on the liability classes. */

	//if (numTraitLociRead == loci.numTraitLoci - 1) {

		loci.lastNullLocusIndDS = locusInd;

		/* 3.1.	Read the number of liability classes. */

		/*fscanf(dataFile,"%d%*[^\n]", &numClasses);
		loci.numClasses = numClasses;
*/
		loci.numClasses = DiseaseData.numOfLiabClasses;
		/* 3.2.	Allocate memory for the penetrance probabilities array(s). */

		myMalloc((loci.diseaseModel).twoLocus.fPenetrances, 
					sizeof(femalePenClassTwo) * loci.numClasses);

		if (loci.sexLink) {
			myMalloc((loci.diseaseModel).twoLocus.mPenetrances, 
						sizeof(malePenClassTwo) * loci.numClasses);
		}

		/* 3.3.	Read the penetrances. */

		for (count = 0; count < loci.numClasses; count++) {
			for (i = 0; i < 3; i++) {
			//	fscanf(dataFile, "\n");

				for (j = 0; j < 3; j++) { 
				/*	fscanf(dataFile,"%lf",
							&((loci.diseaseModel).twoLocus.fPenetrances)[count][i][j][0]);
*/
					((loci.diseaseModel).twoLocus.fPenetrances)[count][i][j][0]= 
						DiseaseData.femalePenetrances[count][i*3 + j];
					((loci.diseaseModel).twoLocus.fPenetrances)[count][i][j][1] = 
							1.0 - ((loci.diseaseModel).twoLocus.fPenetrances)[count][i][j][0];
				}
			}

			if (loci.sexLink) {

				for (i = 0; i < 2; i++) {
					fscanf(dataFile, "\n");

					for (j = 0; j < 2; j++) { 
						/*fscanf(dataFile,"%lf",
							&((loci.diseaseModel).twoLocus.mPenetrances)[count][i][j][0]);
	*/
						((loci.diseaseModel).twoLocus.mPenetrances)[count][i][j][0] =
							DiseaseData.malePenetrances[count][i*3 + j];
						((loci.diseaseModel).twoLocus.mPenetrances)[count][i][j][1] = 
							1.0 - ((loci.diseaseModel).twoLocus.mPenetrances)[count][i][j][0];
					}
				}
			}
		}

		/* 4.	Read the rest of the line. */

	/*	fscanf(dataFile, "%*[^\n]");
		getc(dataFile);*/
	//} //if
}



/******************************************************************************
*
*       Function        readSuperLinkmapSpecificData
*       Parameters		none
*       Output          none
*       Description		Reads program specific data for the Super-Linkmap 
*						program option.
*       Comments		none
*
******************************************************************************/
void readSuperLinkmapSpecificData() 
{
	struct linkmapData	*linkmapDataPtr;
	scanIntervalData	*scanDataPtr;

	linkmapDataPtr = &(progData.progSpecific).linkmapData;

	/* 1.	Read the program-specific parameters. */

	fscanf(dataFile, "%d%lf%d", &linkmapDataPtr->locusVaried, &linkmapDataPtr->finalVal, 
			&linkmapDataPtr->numEvalPoints);

	/* 2.	Translate the index of the locus to be varied from input order to 
			chromosome order. */

	linkmapDataPtr->locusVaried = loci.lociOrder[linkmapDataPtr->locusVaried - 1];

	scanDataPtr = progData.scanData;
	progData.numScanIntervals = 1;

	scanDataPtr->numIntervals = 1;

	/* 3.	Allocate memory for auxiliry arrays. */

	myMalloc(scanDataPtr->mapDistArr, sizeof(double) * 2);
	myMalloc(scanDataPtr->distFromMarker1, sizeof(double) * 2);
}


/******************************************************************************
*
*       Function        readSuperMlinkSpecificData
*       Parameters		none
*       Output          none
*       Description		Reads program specific data for the Super-Mlink 
*						program option.
*       Comments		none
*
******************************************************************************/
void readSuperMlinkSpecificData() 
{
	struct mlinkData	*mlinkDataPtr;
	scanIntervalData	*scanDataPtr;

	mlinkDataPtr = &(progData.progSpecific).mlinkData;

	/* 1.	Read the program-specific parameters. */

	fscanf(dataFile, "%d%lf%lf", &mlinkDataPtr->recVaried, &mlinkDataPtr->inc, 
			&mlinkDataPtr->finalVal);

	if (mlinkDataPtr->recVaried != 1) {
		inputErr("The recombination to be varied needs to be 1. \n");
	}

	mlinkDataPtr->recVaried--;

	scanDataPtr = progData.scanData;

	progData.numScanIntervals = 1;

	/* 2.	Update the data on the scan interval (start position, end position, 
			number of intervals).*/

	scanDataPtr->startPos = 1;
	scanDataPtr->endPos = 0;
	scanDataPtr->numIntervals = 1;

	/* 4.	Allocate memory for auxiliry arrays, and initialize them. */

	myMalloc(scanDataPtr->mapDistArr, sizeof(double));
	scanDataPtr->mapDistArr[0] = 0.0;
	myMalloc(scanDataPtr->distFromMarker1, sizeof(double));
	scanDataPtr->distFromMarker1[0] = 0.0;
}

/******************************************************************************
*
*       Function        readSuperPosSpecificData
*       Parameters		none
*       Output          none
*       Description		Reads program specific data for the Super-Pos 
*						program option.
*       Comments		none
*
******************************************************************************/
void readSuperPosSpecificData() 
{
	struct posData		*posDataPtr;
	onePosData			*onePosDataPtr, *onePosDataPtrEnd;
	char				ch;
	double				*iterLocPos, *iterLocPosEnd;
	scanIntervalData	*scanDataPtr, *scanDataPtrEnd;
	int					numIntervalsMap;

	posDataPtr = &(progData.progSpecific).posData;

	/* 1.	Read the number of disease loci that are to be iterated (moved). 
			It can be either 1 or 2. Read the number of positions where the likelihood
			will be evaluated. */

	fscanf(dataFile, "%d%d%*[^\n]", &(posDataPtr->numIterLoci), &(posDataPtr->numPos));
	fgetc(dataFile);

	/* 2.	Input consistency checks. */

	if ((posDataPtr->numIterLoci != 1) && (posDataPtr->numIterLoci != 2)) {
		inputErr("You can only iterate 1 or 2 loci.\n");
	}
	else if ((posDataPtr->numIterLoci == 2) && (loci.numTraitLoci == 1)) {
		inputErr("You can only iterate 1 locus in case of a one-locus disease.\n");
	}

	/* 3.	Allocate memory for the array that would store the positions of the
			iterated disease locus (loci) in which the likelihood needs to be 
			evaluated. */

	myMalloc(posDataPtr->posDataArr, sizeof(onePosData) * posDataPtr->numPos);

	/* 4.	Read these positions from the input and update the array with these
			values. */

	for (scanDataPtr = progData.scanData, onePosDataPtr = posDataPtr->posDataArr, 
			onePosDataPtrEnd = onePosDataPtr + posDataPtr->numPos;
			onePosDataPtr < onePosDataPtrEnd; onePosDataPtr++, scanDataPtr++) {

		/* 4.1.	Keep reading until the '(' is reached. */

		while (fscanf(dataFile,"%c",&ch) == 1) {
			
			/* 4.1.1.	While there are white spaces, ignore them and continue. */

			if (ch == ' ' || ch == '\t') {
				continue;
			}
			
			/* 4.1.2.	If the '(' has been reached, allocate memory for the array that 
						would store the position(s) of the iterated disease locus (loci), 
						and initialize it. */

			if (ch == '(') {
				myMalloc(onePosDataPtr->locPos, sizeof(double) * posDataPtr->numIterLoci);

				for (iterLocPos = onePosDataPtr->locPos, 
						iterLocPosEnd = iterLocPos + posDataPtr->numIterLoci;
						iterLocPos < iterLocPosEnd; iterLocPos++) {
					if (fscanf(dataFile,"%lf", iterLocPos) != 1) {
						inputErr("There is an error in position #%d specified for likelihood evaluation.\n", 
								 (onePosDataPtr - posDataPtr->posDataArr + 1));
					}
				}
				fscanf(dataFile, ")");
				break;
			}

			/* 4.1.3.	If end-of-line has been reached stop reading, update the number of
						positions that have been specified and issue a warning message. */

			if (ch == '\n') {
				posDataPtr->numPos = onePosDataPtr - posDataPtr->posDataArr;
				inputWarning("Note that only %d positions have been specified for likelihood evaluation.\n\n", 
							 posDataPtr->numPos);
				break;
			}
		}
	}

	/* 5.	Calculate the number of intervals in the scan interval, allocate memory 
			and save the original recombination values (the ones specified in the input 
			file), between the stationary loci.*/

	numIntervalsMap = loci.numLoci - posDataPtr->numIterLoci - 1;
	myMalloc(thetaMarkers, sizeof(thetaCompTheta) * numIntervalsMap);
	memcpy(thetaMarkers, recombData.maleThetas + posDataPtr->numIterLoci, 
				sizeof(thetaCompTheta) * numIntervalsMap);

	/* 6.	Update the information on the scan intervals of the iterated disease 
			locus (loci). */

	for (scanDataPtr = progData.scanData, 
			scanDataPtrEnd = scanDataPtr + posDataPtr->numIterLoci;
			scanDataPtr < scanDataPtrEnd; scanDataPtr++) {

		scanDataPtr->startPos = 1;
		scanDataPtr->endPos = numIntervalsMap + 1;
		scanDataPtr->numIntervals = numIntervalsMap;
		scanDataPtr->currPosTraitLoc = scanDataPtr - progData.scanData;
		myMalloc(scanDataPtr->mapDistArr, sizeof(double) * (numIntervalsMap + 1));
		myMalloc(scanDataPtr->distFromMarker1, sizeof(double) * (numIntervalsMap + 1));
		updateDistArrs(scanDataPtr, thetaMarkers);
	}
}



/******************************************************************************
*
*       Function        readSuperGHSpecificData
*       Parameters		none
*       Output          none
*       Description		Reads program specific data for the Super-Genehunter 
*						program option.
*       Comments		none
*
******************************************************************************/
void readSuperGHSpecificData() 
{
	int					numMarkers;
	char				tmpSt[500];
	sGHDataLocus		*sGHDataLocusPtr, *sGHDataLocusPtrEnd;
	scanIntervalData	*scanDataPtr;
	superGHData			*sGHData;

	sGHData = &(progData.progSpecific).sGHData;

	/* 1.	Read the number of disease loci that are to be iterated (moved). 
			It can be either 1 or 2. */

	fscanf(dataFile, "%d", &(sGHData->numIterLoci));

	if ((sGHData->numIterLoci != 1) && (sGHData->numIterLoci != 2)) {
		inputErr("You can only iterate 1 or 2 loci.\n");
	}
	else if ((sGHData->numIterLoci == 2) && (loci.numTraitLoci == 1)) {
		inputErr("You can only iterate 1 locus in case of a one-locus disease.\n");
	}

	numMarkers = loci.numLoci - sGHData->numIterLoci;

	/* 2.	Determine whether there's uncertainty regarding the inheritance model.
			If so, two models will be tested and the more likely one - chosen. */

	sGHData->checkModel = ifCheckModel();

	/* 3.	Allocate memory to store all the necessary information on the iterated loci. */

	myMalloc(sGHData->sGHDataLocusArr, sizeof(sGHDataLocus) * sGHData->numIterLoci);

	/* 4.	Read the Super-GH specific data for the iterated disease loci. */

	for (sGHDataLocusPtr = sGHData->sGHDataLocusArr, 
			sGHDataLocusPtrEnd = sGHDataLocusPtr + sGHData->numIterLoci,
			scanDataPtr = progData.scanData; sGHDataLocusPtr < sGHDataLocusPtrEnd; 
			sGHDataLocusPtr++, scanDataPtr++) {

		/* 4.1.	Read the size of the step between the position of the iterated
				disease locus in 2 successive iterations, or the number of 
				requested positions for the iterated disease locus in each
				interval on the map. */

		fscanf(dataFile, "%s", tmpSt);

		if (strcmp(tmpSt, "-s") == 0) {
			fscanf(dataFile, "%lf", &(sGHDataLocusPtr->stepIter));
			sGHDataLocusPtr->isStepIter = TRUE;

			/* 4.1.1.	If the interval step is smaller than 0.5 (--> it is a 
						recomination value), turn it into cM using Haldane's 
						(or Kosambi's) mapping function. */
		
			if (sGHDataLocusPtr->stepIter < 0.5) {
				sGHDataLocusPtr->stepIter = getRecTocM(sGHDataLocusPtr->stepIter, 
													   progData.haldane);
			}
		}
		else if (strcmp(tmpSt, "-n") == 0) {
			fscanf(dataFile, "%d", &(sGHDataLocusPtr->numIterInterval));
			sGHDataLocusPtr->isStepIter = FALSE;
		}

		/* 4.2.	Read the indices of the markers delimiting the scan interval on the
				left and on the right. */

		readIntervalEnds(scanDataPtr, numMarkers, 
						 sGHDataLocusPtr - sGHData->sGHDataLocusArr + 1);
		scanDataPtr->currPosTraitLoc = scanDataPtr - progData.scanData;

		/* 4.3.	Check whether the scan should continue off the sides of the map, and
				if so read the distance. */

		scanDataPtr->offEndDistLeft = 0.0;
		scanDataPtr->offEndDistRight = 0.0;
		updateOffEndDist(scanDataPtr, sGHData->numIterLoci, numMarkers);
	}

	progData.numScanIntervals = sGHData->numIterLoci;
}


/******************************************************************************
*
*       Function        readSuperOptLinkSpecificData
*       Parameters		none
*       Output          none
*       Description		Reads program specific data for the Super-OptLink
*						program option.
*       Comments		none
*
******************************************************************************/
void readSuperOptLinkSpecificData() 
{
	int					numMarkers, numIntervalsMap;
	scanIntervalData	*scanDataPtr;
	struct optlinkData	*optlinkDataPtr;

	scanDataPtr = progData.scanData; 
	numMarkers = loci.numLoci - 1;
	optlinkDataPtr = &(progData.progSpecific).optlinkData;

	/* 1.	Read the number of disease loci that are to be iterated (moved). 
			It has to be 1. */

	fscanf(dataFile, "%*d");

	optlinkDataPtr->searchTol = EPSILON;

	/* 2.	Determine whether there's uncertainty regarding the inheritance model.
			If so, two models will be tested and the more likely one chosen. */
	optlinkDataPtr->checkModel =  ifCheckModel(optlinkDataPtr);

	/* 3.	Read the indices of the markers delimiting the scan interval on the
			left and on the right. */

	readIntervalEnds(scanDataPtr, numMarkers, 1);

	/* 4.	Allocate memory and save the original recombination values 
			(the ones specified in the input file), between the stationary loci.*/

	numIntervalsMap = loci.numLoci - 2;

	myMalloc(thetaMarkers, sizeof(thetaCompTheta) * numIntervalsMap);
	memcpy(thetaMarkers, recombData.maleThetas + 1, sizeof(thetaCompTheta) * numIntervalsMap);

	/* 5.	Calculate the distances between the markers. */

	updateDistArrs(scanDataPtr, thetaMarkers);
	scanDataPtr->currPosTraitLoc = 0;

	/* 6.	Check whether the scan should continue off the sides of the map, and
			if so read the distance. */

	scanDataPtr->offEndDistLeft = 0.0;
	scanDataPtr->offEndDistRight = 0.0;
	updateOffEndDist(scanDataPtr, 1, numMarkers);

	progData.numScanIntervals = 1;
}


/******************************************************************************
*
*       Function        ifCheckModel
*       Parameters		none
*       Output          none
*       Description		Determines whether there's uncertainty regarding the 
*						inheritance model. If so, returns TRUE. Otherwise, 
*						returns FALSE.  Also reads the requested search-tolerance.
*       Comments		none
*
******************************************************************************/
int ifCheckModel()
{
	char		ch;
	boolean		checkModel = FALSE;

	checkModel = FALSE;

	/* 1.	As long as there are still characters in this line, keep reading. */

	while (fscanf(dataFile,"%c",&ch) == 1) {
		
		/* 1.1.	While there are white spaces, ignore them and continue. */

		if (ch == ' ' || ch == '\t') {
			continue;
		}
		/* 1.2.	If end-of-line has been reached stop reading. */

		if (ch == '\n') {
			break;
		}

		/* 1.3.	If a '-' sign has been reached check if this in one of the flags. */

		if (ch == '-') {
			fscanf(dataFile,"%c",&ch);
            
			/* 1.3.1.	If this is the flag '-M' of uncertainty regarding inheritance
						mode, turn on the flag 'checkModel'. */
					
			if (ch == 'M') {
				if (loci.numTraitLoci == 2) {
					inputErr("You can only specify uncertainty in the inheritance model for the case of one disease locus.\n");
				}
				checkModel = TRUE;

				/* 1.3.1.1.	If this is the SUPER_GH program, read the rest of the line
							and return. */

				if (progData.programCode == SUPER_GH_PROG) {
					fscanf(dataFile, "%*[^\n]\n");
					break;
				}
			}
			else if (progData.programCode != SUPER_GH_PROG) {

				/* 1.3.1.1.	If this isn't the SUPER_GH program, check if this is the flag
							'-T' which is used to specify search tolerance. */

				if (ch == 'T') {

					/* 1.3.1.1.1.	If so, read the desired search tolerance. */

					fscanf(dataFile, "%lf", &((progData.progSpecific).optlinkData.searchTol));
					fscanf(dataFile, "%*[^\n]");
				}
			}
			else {
				/* 1.3.1.1.	If this is the SUPER_GH program, and this isn't the flag '-M',
							read the rest of the line and return. */

				fscanf(dataFile, "%*[^\n]\n");
				break;
			}
		}
	}

	return checkModel;
}


/******************************************************************************
*
*       Function        readIntervalEnds
*       Parameters		scanDataPtr (pointer to the data-structure that stores
*						all the necessary information on the scan interval),
*						numMarkers (the number of markers in the scan interval),
*						diseaseLocInd (the index of the disease locus, either 1
*						or 2).
*       Output          none
*       Description		Reads the indices of the markers delimiting the scan 
*						interval on the left and on the right.
*       Comments		none
*
******************************************************************************/
void readIntervalEnds(scanIntervalData	*scanDataPtr, int numMarkers, 
					  int diseaseLocInd)
{
    /* 1.	Read the indices of the markers delimiting the scan interval on the
			left and on the right. */

	fscanf(dataFile, "%d%d", &(scanDataPtr->startPos), &(scanDataPtr->endPos));

	/* 2.	Check that these indices are in the correct range. */

	if (scanDataPtr->startPos < 1) { 
		inputErr("Starting position of scan interval (%d) is out of range.\n", 
				 scanDataPtr->startPos);
	}

	if (scanDataPtr->endPos > numMarkers) {
		inputErr("Ending position of scan interval (%d) is out of range.\n", 
				 scanDataPtr->endPos);
	}

	if (scanDataPtr->endPos < scanDataPtr->startPos) { 
		inputErr("Ending position of scan interval (%d) is smaller than starting position (%d).\n", 
					scanDataPtr->endPos, scanDataPtr->startPos);
	} 
	else if (scanDataPtr->startPos == scanDataPtr->endPos) {
		inputWarning("Note that disease locus #%d won't be iterated.\n\n", diseaseLocInd);
	}

	/* 3.	Update the number of interior intervals in the scan interval and allocate 
			memory for arrays that would store the distances between the markers in the scan
			interval (the sizes of the intervals), and the distance of each
			marker from the leftmost marker in the scan interval. */

	scanDataPtr->numIntervals = scanDataPtr->endPos - scanDataPtr->startPos;
	myMalloc(scanDataPtr->mapDistArr, sizeof(double) * (scanDataPtr->numIntervals + 1));
	myMalloc(scanDataPtr->distFromMarker1, sizeof(double) * (scanDataPtr->numIntervals + 1));
}


/******************************************************************************
*
*       Function        readOffEndDist
*       Parameters		scanDataPtr (pointer to the data-structure that stores
*						all the necessary information on the scan interval),
*						numIterLoci (the number of loci that are to be iterated),
*						numMarkers (the number of markers on the map).
*       Output          none
*       Description		Determines whether an 'offEndDist' was specified for the 
*						current scan interval and returns its value (or 0.0 in
*						case no 'offEndDist' was specified).
*       Comments		none
*
******************************************************************************/
void updateOffEndDist(scanIntervalData *scanDataPtr, int numIterLoci, 
					  int numMarkers)
{
	double		offEndDist = 0.0;
	char		ch;
	int			adjThetaInd;

	/* 1.	Check whether an offEndDist was specified in the input. */

	while (fscanf(dataFile,"%c",&ch) == 1) {
		if (ch == ' ' || ch == '\t') {
			continue;
		}
		if (ch == '\n') {
			break;
		}
		if (ch == '-') {
			fscanf(dataFile,"%c",&ch);
        
			if ((ch == 'o') || (ch == 'O')) {
				fscanf(dataFile, "%lf", &offEndDist);

				/* 1.1.	If the 'offEndDist' is smaller than 0.5 (--> it is 
						recombination value), turn it into cM using Haldane's 
						(or Kosambi's) mapping function. */
				
				if (offEndDist < 0.5) {
					offEndDist = getRecTocM(offEndDist, progData.haldane);
				}
			}
		}
	}

	/* 2.	Update the off-end distance on each side of the scan interval. */

	if (offEndDist > 0) {
		adjThetaInd = scanDataPtr->startPos + numIterLoci - 2;

		/* 2.1.	Check whether the scan interval reaches the left end of the map. If so,
				update the left off-end distance to the distance specified in the input. */

		if ((scanDataPtr->startPos == 1) || 
				(recombData.maleThetas[adjThetaInd][0] == 0.5)) {

			scanDataPtr->offEndDistLeft = offEndDist;
		}
		else {
			scanDataPtr->offEndDistLeft = 0.0;
		}

		adjThetaInd = scanDataPtr->endPos + numIterLoci - 1;

		/* 2.2.	Check whether the scan interval reaches the right end of the map. If so,
				update the right off-end distance to the distance specified in the input. */

		if ((scanDataPtr->endPos == numMarkers) || 
				(recombData.maleThetas[adjThetaInd][0] == 0.5)) {

			scanDataPtr->offEndDistRight = offEndDist;
		}
		else {
			scanDataPtr->offEndDistRight = 0.0;
		}
	}
}


/******************************************************************************
*
*       Function        updateDistArrs
*       Parameters		scanDataPtr (pointer to the data-structure that stores
*						all the necessary information on the scan interval),
*						thetaMarkers (the theta values between the markers).
*       Output          none
*       Description		Updates auxiliary arrays with the distances between the
*						markers in the scan interval, and the distance of each
*						marker from the leftmost marker in the scan interval.
*       Comments		none
*
******************************************************************************/
void updateDistArrs(scanIntervalData *scanDataPtr, thetaCompTheta *thetaMarkers)
{
	int			i;
	double		*mapDistArrPtr, *distFromMarker1Ptr;

	/* 1.	Update an auxiliary array with the distances (in Morgans) of each marker, 
			in the scan interval, from the lefmost marker in the scan interval. */

	i = scanDataPtr->startPos - 1;

	/* 2.	Update the distance of the leftmost marker in the scan interval from
			itself (for consistency). */

	distFromMarker1Ptr = scanDataPtr->distFromMarker1;
	(*distFromMarker1Ptr) = 0.0;
	mapDistArrPtr = scanDataPtr->mapDistArr;
	(*mapDistArrPtr) = 0.0;

	/* 3.	Iterate over the rest of the markers in the scan interval. 
			The distance of each marker from the leftmost marker is equal to the sum of
			its distance from the marker preceeding it and the distance of the marker
			preceeding it from the first marker. Update the distance of each marker 
			from the marker preceeding it and from the leftmost marker. */

	for (i++, mapDistArrPtr++, distFromMarker1Ptr++; i < scanDataPtr->endPos; 
				i++, mapDistArrPtr++, distFromMarker1Ptr++){

		(*mapDistArrPtr) = getRecTocM(thetaMarkers[i-1][0], progData.haldane);
		(*distFromMarker1Ptr) = (*mapDistArrPtr) + *(distFromMarker1Ptr-1);
	}
}



/******************************************************************************
*
*       Function        freeLociDataSL
*       Parameters		none
*       Output          none
*       Description		Frees the data-structures that were allocated 
*						dynamically to hold the data about the loci.
*       Comments		none
*
******************************************************************************/
void freeLociDataSL()
{
	int					locusInd, numTraitLociRead, numLoci;
	locus				*locusPtr;
	scanIntervalData	*scanDataPtr, *scanDataPtrEnd;
	

	numTraitLociRead = 0;
	numLoci = loci.numLoci;

	/* 1.	Iterate over the loci. */

	for (locusInd = 0, locusPtr = loci.locusArr; locusInd < numLoci; locusInd++,
			locusPtr++) {
			
		/* 1.1.	Free the gene frequency array. */
		
		myFree(locusPtr->geneFrequencies);
		//myFree(locusPtr->name);
	}

	/* 2.	Free the arrays of penetrance probabilities, in case of an
			affection type locus or a Null type locus. */

	if (loci.numTraitLoci == 1) {
		myFree(loci.diseaseModel.oneLocus.fPenetrances);

		if (loci.sexLink) {
			myFree(loci.diseaseModel.oneLocus.mPenetrances);
		}
	}
	else if (loci.numTraitLoci == 2) {
		myFree(loci.diseaseModel.twoLocus.fPenetrances);

		if (loci.sexLink) {
			myFree(loci.diseaseModel.twoLocus.mPenetrances);
		}
	}

	/* 3.	Free the arrays that were used to hold the information on the loci, the 
			theta values and the order of the loci. */

	if (loci.numLoci > 1) {
		myFree(recombData.maleThetas);
		myFree(recombData.femaleThetas);
	}

	myFree(loci.locusArr);
	myFree(loci.traitLociPos);
	myFree(loci.lociOrder);
	myFree(loci.mapToData);

	/* 4.	Free the auxliary arrays used to store distances. */

	for (scanDataPtr = progData.scanData, scanDataPtrEnd = scanDataPtr + progData.numScanIntervals;
			scanDataPtr < scanDataPtrEnd; scanDataPtr++) {
		myFree(scanDataPtr->distFromMarker1);
		myFree(scanDataPtr->mapDistArr);
	}
		
	myFree(progData.scanData);
}


/******************************************************************************
*
*       Function        updateModel
*       Parameters		isRecessive (whether the model is recessive or dominant),
*						penetrance (the penetrance value).
*       Output          none
*       Description		Updates the penetrances to match either a recessive or
*						a dominant transmission model.
*       Comments		none
*
******************************************************************************/
void updateTransmissionModel(boolean isRecessive, double penetrance)
{
	femalePenClass	*fPenetrances;		
	malePenClass    *mPenetrances;	
	int				count;

	fPenetrances = loci.diseaseModel.oneLocus.fPenetrances;

	/* 1.	Iterate over the different liability classes and update the penetrances
			according to either a dominant or a recessive disease model, with 
			penetrances of 'penetrance'. */

	for (count = 0; count < loci.numClasses; count++) {	
		if (isRecessive) {
			updateThetaCompTheta(fPenetrances[count][0], 0.0);
			updateThetaCompTheta(fPenetrances[count][1], 0.0);
			updateThetaCompTheta(fPenetrances[count][2], penetrance);
		}
		else {
			updateThetaCompTheta(fPenetrances[count][0], 0.0);
			updateThetaCompTheta(fPenetrances[count][1], penetrance);
			updateThetaCompTheta(fPenetrances[count][1], penetrance);
		}

		if (loci.sexLink) {
			mPenetrances = loci.diseaseModel.oneLocus.mPenetrances;
			updateThetaCompTheta(mPenetrances[count][0], 0.0);
			updateThetaCompTheta(mPenetrances[count][1], penetrance);
		}
	}
}
