/*********************************************************************************/
/*  Main.c												     				     */
/*  Author: Ma'ayan Fishelson					                                 */
/*********************************************************************************/

#include "sl_Globals.h"
/******************************************************************************
*
*       Function        main
*       Parameters		none
*       Output          none
*       Description		The main function of the program.
*       Comments		none
*
******************************************************************************/
double sl_main(int numMarkers,int* markersUsed, Bool diseaseUsed, int progCode,double specialThetas, PedigreeDataStruct *origPedigree)
{
	int			numLoc;
	clock_t		start_time, end_time;
	double		calcLikelihoodTime;
	int thetaInd;
	double res = 0;
	start_time=clock();

#if MANAGE_MEM
	manageMemory = FALSE;
#endif

#ifdef NDEBUG
	mem_avl = availableVirtMem();
#else
	mem_avl = 1000000000;
#endif

	SET_THRESHOLDS();

	/* 2.	Determine whether the optimization of the elimination order would be
			determined automatically by the program, or by the user in the command line. */

	exitAfterFindOrder = FALSE;
	RUN_SL = 0;
	heuristicEO.isAutomatic = TRUE;

	/* 3.	Read the input files and initialize the program's data-structures. */
	
/*	if (markersUsed[0] == 830){
		printf("");
	}*/
	init(numMarkers,markersUsed, diseaseUsed, progCode, origPedigree);

	if (specialThetas >= 0){
		for (thetaInd = 0; thetaInd < loci.numLoci - 1; thetaInd++) {
			recombData.femaleThetas[thetaInd][0] = specialThetas;
			recombData.femaleThetas[thetaInd][1] = 1 - recombData.femaleThetas[thetaInd][0];
			recombData.maleThetas[thetaInd][0] = specialThetas;
			recombData.maleThetas[thetaInd][1] = 1 - recombData.maleThetas[thetaInd][0];
		}
	}

	/* 4.	Perform initial value-elimination on the pedigrees. */

	if (loci.sexLink) {
		valueEliminate(updateChildSexLinked, updateFlagsLociSexLinked,
					   updateParentsSexLinked);
	}
	else {
		valueEliminate(updateChildAutosomal, updateFlagsLociAutosomal,
					   updateParentsAutosomal);
	}

	/* 5.	If allele-recoding is requested, perform allele-recoding. */

	#if	PERFORM_ALLELE_RECODING
		performAlleleRecoding();
	#endif	

	/* 6.	Initialize the random seed of the system. */

	srand((unsigned)time(NULL));

	/* 7.	Open the elimination-order statistics file for writing. */

	#if PRINT_VARIOUS_STATISTICS_LEVEL
		openInitEOStatisticsFile();
	#endif

	/* 8.	Initialize the data-structure that would hold the results of all the
			likelihood calculations. */
			initResultList();

	if ( RUN_CREATE_BAYES){
			createExportBayesProg();
	}



	/* 9.	If there's only 1 locus and this isn't a haplotyping analysis, call the 
	function which calculates the likelihood for 1 locus. */

/*	if ((loci.numLoci == 1) && (progData.programCode != SUPER_HAPLO)
		&& !RUN_CREATE_BAYES) {
			res = calcLikelihoodOneLocus();
			goto end;
		}
*/
	/* 10.	Calculate the likelihood, according to the requested program. */

	switch (progData.programCode) {
		
		case SUPER_LINKMAP_PROG: calcLikelihoodSuperLinkmap();
								 break;

		case SUPER_MLINK_PROG:	calcLikelihoodSuperMlink();
								break;

		case SUPER_POS_PROG:	calcLikelihoodSuperPos();
								break;

		case SUPER_GH_PROG:		calcLikelihoodSuperGH();
								break;

		case SUPER_HAPLO:		res = superHaploProg(origPedigree);
								break;

		case SUPER_LINKMAP_APPROX_PROG: calcLikelihoodSuperLinkmapApprox();
									    break;

		case SUPER_OPTLINK_PROG:	calcLikelihoodSuperOptlink();
									break;

		case SUPER_OPTMARGINAL_PROG:   OptimizeMarginalLikelihood();
                                        break;	

		case FIND_ORDER_PROG:      exitAfterFindOrder = TRUE;
								
		case CALCULATE_LIKELIHOOD: res = calcLikelihoodNoDisease();
								   if (exitAfterFindOrder == TRUE)
										return res;
									break;
	}

end:

	/* 11.	Calculate and print the total computation time. */

	end_time=clock();

	calcLikelihoodTime = ((double)(end_time-start_time))/((double)CLOCKS_PER_SEC);
/*
	myPrintf("\n\nThe time needed for calculating the likelihood of the pedigree is %lf sec.\n",calcLikelihoodTime);
	myPrintf("\nSuperlink v1.5 (Original Citation: Fishelson M. and Geiger D. Bioinformatics,  2002:18, S189-S198).\n");
	myPrintf("(c)2002-2005  Technion - Israel Institute of Technology\n\n");
	*/
	/* 12.	Print the total run time to the statistics files and close them. */

	#if PRINT_VARIOUS_STATISTICS_LEVEL
		fprintf(statisticsFile, "%lf\n", calcLikelihoodTime);
		fclose(statisticsFile);
	#endif 

	/* 13.	Free the memory allocated dynamically. */

	numLoc = (progData.programCode == SUPER_HAPLO) ? numMarkers : loci.numLoci;
//exit(0);
	#if PERFORM_ALLELE_RECODING
		freePedDataAR(numLoc);
	#else
		freePedData(numLoc);
	#endif

	freeLociDataSL();
	freeResultList();

	return res;
}

