/*********************************************************************************/
/*  Marginal.h											     				     */
/*  Author: Dmitry Rusakov  					                                 */
/*********************************************************************************/

#ifndef _SL_MARGINAL_H_
#define _SL_MARGINAL_H_

#include "sl_Globals.h"

#define ABS(a)      (a<0?(-a):a)


/* likelihood evaluation functions */
#define PRIOR_NONE          0
#define PRIOR_UNIFORM       1
#define PEN_MIN             0.001
#define PEN_MAX             0.999
#define MAXLOD_CORRECTION   0.3
#define TEST_POSITION       23
#define MARGINAL_INTEGRAL   quad_gaussian_n5
#define REL_ERROR           0.01
void OptimizeMarginalLikelihood();
void OptimizeFullAvMarginalLikelihood();
void OptimizeDoubleAvMarginalLikelihood();
void OptimizeBestMarginalLikelihood();
double evaluateLnMarginalLikelihood(double loc, boolean isRecessive, double min_pen, double max_pen, int prior);
double evaluateBaseLnLikelihood(elimOrder *elimOrderArrs, boolean *areValsElim, boolean *isChangedNet);


/* generalized superlink style functions */
typedef double (*functype)(double x, void *func_data);
double evaluateLikeV2(double location);
void updateTransmissionModelV2(double penHH, double penHD, double penDD);
double goldenV2(functype func, double ax, double bx, double cx, double ftol, double xtol, void *funcData, double *x_opt);


/* 1-d integration */
#define RECURSIVE_LIMIT 5
#define QUAD_TOL 0.01
typedef double (*quadfunc)(functype func, double xmin, double xmax, double rtol, void *func_data);
double quad_trapezoid(functype func, double xmin, double xmax, double rtol, void *func_data);
double quad_adaptive_simpson(functype func, double xmin, double xmax, double rtol, void *func_data);
double quad_gaussian_n5(functype func, double xmin, double xmax, double rtol, void *func_data);


/* 2-d integration */
typedef double (*functype2)(double x, double y, void *func_data);
double dblquad_general(functype2 func, double xmin, double xmax, double ymin, double ymax, double rtol, void *func_data, quadfunc quad);
double dblquad_gaussian(functype2 func, double xmin, double xmax, double ymin, double ymax, double rtol, void *func_data);
/* 2-d general limit integration */
double dblquad_general2(functype2 func, double xmin, double xmax, functype ymin_fun, functype ymax_fun, double rtol, void *func_data, quadfunc quad);
/* 2-g general limit integration: common limit functions */
double return_zero(double x, void* data);
double return_x(double x, void* data);


/* search */
#define SCAN_STEPS 10
#define SEARCH_XTOL 0.1
/* #define SEARCH_FTOL 0.01 - specified outside by '-T ..'  parameter */
double search_max(functype func, double xmin, double xmax, double ftol, double xtol, void *func_data, double *x_opt);


#endif
