/*********************************************************************************/
/*  Mem.h																	     */
/*  Author: Nickolay Dovgolevsky     											 */
/*********************************************************************************/
/*																				 */
/*  Comment: This file includes functions used for finding the total and         */
/*			 available amounts of virtual and physical memory of the computer.	 */
/*																				 */
/*********************************************************************************/

#ifndef _SL_MEM_H_
#define _SL_MEM_H_

#include <stdlib.h>
#include <math.h>

#if HAVE_CONFIG_H
# include <config.h>
#endif

#if HAVE_UNISTD_H || defined linux || defined sun
# include <unistd.h>
#endif

#if HAVE_SYS_PSTAT_H
# include <sys/pstat.h>
#endif

#if HAVE_SYS_SYSMP_H
# include <sys/sysmp.h>
#endif

#if HAVE_SYS_SYSINFO_H && HAVE_MACHINE_HAL_SYSINFO_H
# include <sys/sysinfo.h>
# include <machine/hal_sysinfo.h>
#endif

#if HAVE_SYS_TABLE_H
# include <sys/table.h>
#endif

#include <sys/types.h>

#if HAVE_SYS_PARAM_H
# include <sys/param.h>
#endif

#if HAVE_SYS_SYSCTL_H || defined __APPLE_CC__
# include <sys/sysctl.h>
#endif

#if HAVE_SYS_SYSTEMCFG_H
# include <sys/systemcfg.h>
#endif

#ifdef _WIN32
# define WIN32_LEAN_AND_MEAN
# include <windows.h>

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/*  MEMORYSTATUSEX is missing from older windows headers, so define */
/*  a local replacement.                                            */
typedef struct
{
  DWORD dwLength;
  DWORD dwMemoryLoad;
  DWORDLONG ullTotalPhys;
  DWORDLONG ullAvailPhys;
  DWORDLONG ullTotalPageFile;
  DWORDLONG ullAvailPageFile;
  DWORDLONG ullTotalVirtual;
  DWORDLONG ullAvailVirtual;
  DWORDLONG ullAvailExtendedVirtual;
} lMEMORYSTATUSEX;
#endif

typedef char _byte;

/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/

#define ARRAY_SIZE(a) (sizeof (a) / sizeof ((a)[0])) /* array size */


/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

double totalPhysMem();
double availablePhysMem();
double availableVirtMem();

#endif /* _MEM_H_ */

