/*********************************************************************************/
/*  MemManage.c																     */
/*  Author: Nickolay Dovgolevsky     											 */
/*********************************************************************************/
#if MANAGE_MEM

#include "sl_MemManage.h"

/******************************************************************************
*
*       Function        zeroFirstMemBlockPointers    
*       Parameters		none
*       Output          none
*       Description     assigns zero values to the first block in the each entry
*                       of memory management DB (hashtable).                   
*       Comments		none
*
*******************************************************************************/
void zeroFirstMemBlockPointers(){
	int		i;

	for(i = 0; i < HASH_SIZE; i++){
		firstMemBlock[i] = 0;
	}
}


/******************************************************************************
*
*       Function        insertBlock    
*       Parameters		data (pointer to a memory block).
*       Output          none
*       Description     Inserts a pointer to a newly allocated memory block into 
*                       the memory management DB. 
*       Comments		none
*
*******************************************************************************/
void insertBlock(void *data){
	int			i;
	MemBlock	mem_blk;

	i = (((unsigned long)data) / 4) % HASH_SIZE;

	mem_blk = (MemBlock)malloc(sizeof(struct MemBlock_str));
	mem_blk->data = data;
	mem_blk->next = firstMemBlock[i];
	mem_blk->prev = NULL;

	if (firstMemBlock[i])  {
		firstMemBlock[i]->prev = mem_blk;
	}

	firstMemBlock[i] = mem_blk;
}


/******************************************************************************
*
*       Function        removeBlock 
*       Parameters		data (pointer to memory block).
*       Output          none
*       Description     Removes the pointer to the removed memory block from the
*                       memory management DB.
*       Comments		none
*
*******************************************************************************/
void removeBlock(void *data){
	int		 i;
	MemBlock mem_blk_it;


	i = (((unsigned long)data) / 4) % HASH_SIZE;
	mem_blk_it = firstMemBlock[i];

	while (mem_blk_it) {
		if (mem_blk_it->data == data) {
			if(mem_blk_it->prev) mem_blk_it->prev->next = mem_blk_it->next;
			if(mem_blk_it->next) mem_blk_it->next->prev = mem_blk_it->prev;
			if(firstMemBlock[i]->data == data) firstMemBlock[i] = firstMemBlock[i]->next;
			free(mem_blk_it);
			return;
		}

		mem_blk_it = mem_blk_it->next;
	}
}


/******************************************************************************
*
*       Function        freeAllBlocks
*       Parameters		none
*       Output          none
*       Description     Frees all memory allocated by the program when
*                       manageMemory flag was on.
*       Comments		none
*
*******************************************************************************/
void freeAllBlocks(){
	int		i;

	for (i = 0; i < HASH_SIZE; i++) {
		while (firstMemBlock[i]) {
			free(firstMemBlock[i]->data);
			removeBlock(firstMemBlock[i]->data);
		}
	}
}

#endif /* MANAGE_MEM */

