/*********************************************************************************/
/*  MemManage.h																     */
/*  Author: Nickolay Dovgolevsky     											 */
/*********************************************************************************/
/*																				 */
/*  Comment: This file includes functions used for handling dynamic memory		 */
/*			 allocations (allocating and freeing memory).						 */
/*																				 */
/*********************************************************************************/

#ifndef _SL_MEM_MANAGE_H_
#define _SL_MEM_MANAGE_H_

#include "sl_Auxiliary.h"


/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/

/* The number of entries in the Memory Management DB (hashtable). */
#define HASH_SIZE 10000 

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* Stores the addresses of dynamically allocated memory blocks. */
typedef struct MemBlock_str{
	void				*data;
	struct MemBlock_str *next;
	struct MemBlock_str *prev;
} *MemBlock;


/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

extern double thr_div_factor, breakProductBottomThreshold, smallBreakThreshold;

MemBlock firstMemBlock[HASH_SIZE];


/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void zeroFirstMemBlockPointers();
void insertBlock(void *data);
void removeBlock(void *data);
void freeAllBlocks();

#endif /* _MEM_MANAGE_H_ */

