/*********************************************************************************/
/*  PSGraph.h												     			     */
/*  Author: Ma'ayan Fishelson					                                 */
/*********************************************************************************/
/*																				 */
/*  Comment: This file includes functions used for creating a Postscript file 	 */
/*			 with a graph of the results.								         */
/*																				 */
/*********************************************************************************/

#ifndef _SL_PS_GRAPH_H_
#define _SL_PS_GRAPH_H_

#include "sl_Auxiliary.h"
#include "sl_Globals.h"
#include "sl_LociData.h"


/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/

/* The default name for the Postscript file that would be generated with the graph of
   the results. */

#define	PS_FILE_NAME  "lod_plot.ps"

/* The different line types. */

#define SOLID_LINE    ""
#define THIN_LINE     "0.5 setLW"
#define EVEN_DASH     "[4 4] 0 setdash"
#define DOTTED2       "[2 2] 0 setdash"

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* Defines a 2-dimensional point. */

typedef struct point {
	double		x;
	double		y;
} point;

/* Defines a data-structure that stores the 4 control points that define a cubic
   Bezier curve. */

typedef struct cubicControlPts {
	point	p0;
	point	p1;
	point	p2;
	point	p3;
}  cubicControlPts;

typedef char yLabel[10];		/* A label on the Y-axis. */

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void createPSGraph(char *psFileName, char *graphTitle, char *yTitle, point *vals, 
				   int numPoints, xLabel *markerNames, int numXLabels, double *mapDistArr,
				   boolean	drawBezCurve);
void psFileStart(FILE *psFile);
void drawAxes(FILE *psFile, int numXVals, double *xVals, xLabel *xLabels, 
			  int numYVals, double *yVals, yLabel *yLabels, char *xTitle, 
			  char *yTitle, double xScale, double yScale);

void drawCurve(FILE *psFile, point *vals, int numPoints, char *lineType, 
			   double xScale, double yScale);

void drawBezier(FILE *psFile, point *vals, int numPoints, double s0, double sn, 
				char *lineType, double xScale, double yScale);
void psPageEnd(FILE *psFile);


#endif	/* _PS_GRAPH_H_ */

