/*****************************************************************************/
/*  SpecialMult.h															 */
/*  Author: Anna Reitmann     											     */
/*****************************************************************************/
/*****************************************************************************/
/*																			 */
/*  Comment: This file includes functions used for performing special		 */
/*			 matrix-multiplication for the case where the elimination is 	 */
/*			 performed locus-by-locus style Lander-Green. In this case, the	 */
/*			 probability tables of the selectors are multiplied in a special */
/*			 way (according to Yate's algorithm).							 */
/*																			 */
/*****************************************************************************/

#ifndef _SL_SPECIAL_MULT_H_
#define _SL_SPECIAL_MULT_H_


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <memory.h>
#include "sl_Product.h"
#include "sl_Probability.h"
#include "sl_LociData.h"
#include "sl_Globals.h"

/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

int		power2_sl[32];

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void initVars();
Probability eliminateSelectorsLocusI(product *prod, elimOrderPed *elimOrd, 
									 boolean *isZeroLike);
variable** getSelectorVarsLocusI(elimOrderPed *elimOrd, int *numSelectors);
boolean checkPrevSelector(variable *selectorPtr, int prevLocusNum, variable* *prevSelector);
boolean checkFuncVars(function *funcPtr, int locusNum, variable **prevSelector);
void createOneFunc(variable *selecVarPtr, product *prod);
void removeTransFunc(variable *selectorPtr, product *prod, int prevLocusNum);
function* findFunc(variable *selectorVar, int prevLocusNum);

/* Functions used for Yate's algorithm. */

void Yate(double *result, double *input, double theta, int size);
void updateFuncsVariables(variable **newVarArr, variable **oldVarArr, int numVars,
						product *prod, variable **mapCurrToPrevArr);

/* Functions used for FFT algorithm. */

void calcFFT(double *result, double *input, double theta, int size);
void calcComplexFFT(double *result, double *input, double theta, int size, int n);
void calcComplexFTrans(double *inp, int size, int n);
void calcSimpleFFT(double *result, double *input, double theta, int size, int n);
void calcSimpleFTrans(double *inp, int size, int *wArr, int n, int *incArr);


#endif /* _SPECIAL_MULT_H_ */

