#include "sl_XMLFormat.h"
#include "sl_Product.h"
#include <string.h>
#include "sl_XMLhelpFunction.h"



char family[5];    // family index
int *indexesVar;   //variable index



void printXBNHead();
void printAllVariablesXBN(int size,function *functions);
void printFunctionsXBN(int size,function *functions);
void printTailXBN();
void printArcsXBN(int size,function *functions);

/******************************************************************************
*
*       Function        convertToXBN
*       Parameters		prodArr - product
*						numProd - number of products in array
*						peds - pedigree dat
*						locusNumber - number of loci
*       Output          none
*       Description		Prints current bayes network in XBN format.
*       Comments		none
*
******************************************************************************/
void convertToXBN(product**	 prodArr ,int numProd,pedData *peds,int locusNumber){
	int i,totalNumVariables=0,totalNumFunc=0;
	int curVarNumber=0;
	pedigree *pedPtr;

	START=10;
	maxPeople=0;
	maxLoci=locusNumber;

	myMalloc(indexesVar,(sizeof(int)*totalNumVariables));
	
	/* find the bigest pedigree */
	for (pedPtr = peds[0].head ;pedPtr != NULL ;pedPtr= pedPtr->nextPed){
		maxPeople=(pedPtr->numPeople>maxPeople)?pedPtr->numPeople:maxPeople;
	}

	addConstantToProd(prodArr[0]);
	bayesNetFile = fopen("bayesNetFile.xbn","w");
	printXBNHead();

	fprintf(bayesNetFile,"\t\t<VARIABLES>\n");
	for(i=0;i<numProd;i++){
		START=maxLoci*2000+2000+START;
		sprintf(family,"_f%d",i);
		printAllVariablesXBN(prodArr[i]->numFunctions,prodArr[i]->functions);
	}
	fprintf(bayesNetFile,"\t\t</VARIABLES>\n");


	fprintf(bayesNetFile,"\t\t<STRUCTURE>\n");
	for(i=0;i<numProd;i++)
		printArcsXBN(prodArr[i]->numFunctions,prodArr[i]->functions);
	fprintf(bayesNetFile,"\t\t</STRUCTURE>");

	fprintf(bayesNetFile,"\t\t<DISTRIBUTIONS>\n");
	for(i=0;i<numProd;i++)
		printFunctionsXBN(prodArr[i]->numFunctions,prodArr[i]->functions);
	fprintf(bayesNetFile,"\t\t</DISTRIBUTIONS>\n");

	printTailXBN();
	fclose(bayesNetFile);
}

/*******************************************************/
/* ANALYSISNOTEBOOK: a collection of belief network models
/* BNMODEL: a single belief network
/* VARIABLES: the random variables (nodes) in the belief network
/* STRUCTURE: the dependency information (edges) in the belief network
/* DISTRIBUTIONS: the conditional distributions for variables in the belief network
/***********************************************************************/
void printXBNHead(int numVar,int numFunc){
	fprintf(bayesNetFile,"<?xml version=\"1.0\"?>\n");
	fprintf(bayesNetFile,"<!--XML Belief Network File Format\n");
	fprintf(bayesNetFile,"Superlink v1.5 (Original Citation: Fishelson M. and Geiger D. Bioinformatics,  2002:18, S189-S198)\n(c)2002-2005 Technion - Israel Institute of Technology-->\n");
	fprintf(bayesNetFile,"<ANALYSISNOTEBOOK NAME=\"Notebook.Pedigree\" ROOT=\"Pedigree\">\n");	
	fprintf(bayesNetFile,"\t<BNMODEL NAME=\"Pedigree\"><STATICPROPERTIES><FORMAT>MSR DTAS XML</FORMAT>\n");
	fprintf(bayesNetFile,"\t<VERSION>1.0</VERSION>\n");
	fprintf(bayesNetFile,"\t<CREATOR>Pedigree Builder</CREATOR>\n");
	fprintf(bayesNetFile,"\t\t </STATICPROPERTIES>\n");

}

/******************************************************************************
*
*       Function        findPositionXML
*       Parameters		name - name of the variable 
*       Output          none
*       Description		finds and print variable location 
*       Comments		none
*
******************************************************************************/
void findPositionXBN(char *name){
	int xPos,yPos;

	findPosition(name,&xPos,&yPos,maxPeople,START);
	fprintf(bayesNetFile,"\t\t\t<VAR NAME=\"%s\" TYPE=\"discrete\" XPOS=\"%d\" YPOS=\"%d\">\n",name,xPos,yPos);
}

/*******************************************************************************/
/*	VAR node:
/*	- it contains the description of a variable/node in the model/network.
/*	- it has four attributes:
/*		* NAME - the name of the variable (unique).
/*		* TYPE - the type of the variable (by default it has the value "discrete").
/*		* XPOS - X coordinate of the center of the graphical object (circle).
/*		* YPOS - Y coordinate of the center of the graphical object (circle).
/* 	- the children of the VAR node are represented by the following tags:
/*		* DESCRIPTION - a detailed description of the variable (as an XML text node).
/*		* STATENAME - the domain of values is represented by a sequence of sibling nodes, every one of them being identified by this tag.
/********************************************************************************/
void 	printVariableXBN(char* name,int numValues){
	int j;
	findPositionXBN(name);
	fprintf(bayesNetFile,"\t\t\t\t<DESCRIPTION></DESCRIPTION>\n");
	for(j=0;j<numValues;j++)
		fprintf(bayesNetFile,"\t\t\t\t<STATENAME>Value%d</STATENAME>\n",j);
	//printf("\t\t\t\t<OBSERVED></OBSERVED>\n");
	fprintf(bayesNetFile,"\t\t\t</VAR>\n");
}

/***************************************************************************/
/*	VARIABLES "root" node:
/*		- its children specify the variables of the model/network.
/*
/****************************************************************************/
void printAllVariablesXBN(int size,function *functions){

	int i;
//	char name[20];
	variable *varPtr;
	for(i=0;i<size;i++){
		if ( (&functions[i])->used==0 || (&functions[i])->probabList[0]>1 || (&functions[i])->probabList[0]<0 )
			continue;
		varPtr=(&functions[i])->owner;
		if ( strcmp((&functions[i])->ownerName,MULTFACTOR)){
			strcat((&functions[i])->ownerName,family);
		}
		if (varPtr==NULL) {// variable has only one value 						 
			printVariableXBN((&functions[i])->ownerName,1);
		}
		else{			
			printVariableXBN((&functions[i])->ownerName,varPtr->numValues);

		}
	}
}

/*****************************************************************/
/* STRUCTURE
/* This section lists the edges or dependency arcs present in the network. 
/* Each ARC element has PARENT and CHILD attributes. 
/* There is also a MEMBER attribute for declaring parentless and childless 
/*        nodes which are to be considered as variables in the network.
/*****************************************************************/

void  printArcsXBN(int size,function *functions){

	int i,j;
	function* funcPtr;
	variable* varPtr;
	char name[30];
	for (i=0;i<size;i++){
		funcPtr=&(functions[i]);
		if (funcPtr->used==0 || funcPtr->probabList[0]>1 || funcPtr->probabList[0]<0 )
			continue;
		varPtr=funcPtr->owner;

		strcpy(name,funcPtr->ownerName);
		for (j=0;j<funcPtr->numVariables;j++){
			if (varPtr==funcPtr->variables[j])
				continue;
			if (funcPtr->variables[j]->used==0)
				continue;
			fprintf(bayesNetFile, "\t\t\t<ARC PARENT=\"%s\" CHILD=\"%s\"/>\n",funcPtr->variables[j]->originalNode->name,name);
		}
	}    

}

/*****************************************************************************/
/*
/* Function print parents of first parameter according to function
/* return value - index of child in variable array,-1 if not exist there
/*
/*	* CONDELEM - identifies a parent.
/*		 - it is specified by the NAME attribute of the node and its value has to be a valid variable name.
/*	* CONDSET - the root node for the parents; each parent is represented by a CONDELEM node.
/*
/***************************************************************************/
int printFuncVariablesXBN(variable *var,function *funcPtr,char *name){

	variable *varPtr;
	int i,varIndex=-1;
	if (funcPtr->numVariables==1 && funcPtr->variables[0]==var )
		return 0;
	//print name of the parents
	fprintf(bayesNetFile,"\t\t\t\t<CONDSET>\n");
	for (i=0;i<funcPtr->numVariables;i++){
		varPtr=funcPtr->variables[i];
		if(varPtr==var){
			varIndex=i;
			continue;
		}
		fprintf(bayesNetFile,"\t\t\t\t\t<CONDELEM NAME=\"%s\"/>\n",varPtr->originalNode->name);
	}
	fprintf(bayesNetFile,"\t\t\t\t</CONDSET>\n");
	return varIndex;
} 

/************************************************************************/
/*
/* print probabilities of varPtr given funPtr.
/* for each probability print indexes of the parents
/*	- the function itself (i.e. data) is represented by the DPIS section.
/*	- each DPI entry represents the probability values for each of the values of the child and a certain combination of values for the parents.
/*		* INDEXES - list of zero-based value indexes for the conditional set (parents).
/*		* the XML text node associated with the DPI tag is a blank ( ) separated list of probability values corresponding to each of the child values (starting from the first one)
/*		* the indexes are used to compute the index entry in the table.
/*******************************************************************************/
void printProbabilities(variable *varPtr,function *funPtr,int varIndex){
	int j,numValues,step=1;
	unsigned int i;
	int *indexes;
	char str[256],tmp[20];
	indexes=(int*)calloc(funPtr->numVariables,sizeof(int));

	if (varPtr==NULL)
		numValues=1;
	else
		numValues=varPtr->numValues;
	for (j=varIndex+1;varIndex!=-1 && j<funPtr->numVariables;j++)
		step*=(funPtr->variables[j])->numValues;

	/* go over all probabilities*/
	for (i=0;i<funPtr->sizeOfTable/numValues;i++){

		if(varIndex!=-1 && indexes[varIndex]!=0)
		{   //already was printed 
			addOneToIndex(indexes,funPtr->numVariables,funPtr);
			continue;
		}
		str[0]='\0';
		if (funPtr->numVariables==1 && varIndex!=-1){
			fprintf(bayesNetFile, "\t\t\t\t\t<DPI>");

		}else{
			fprintf(bayesNetFile,"\t\t\t\t\t<DPI INDEXES=\" ");
			//print current indexes
			for (j=0;j<funPtr->numVariables;j++){
				if (j==varIndex)
					continue;
				sprintf(tmp,"%d ",indexes[j]);
				strcat(str,tmp);
			}
			addOneToIndex(indexes,funPtr->numVariables,funPtr);
			//	str[strlen(str)-2]='\0'; //delete last ','
			fprintf(bayesNetFile,"%s\">",str);
		}
		//print probabilities */
		for (j=0;j<numValues;j++)       
			if (strcmp(funPtr->ownerName,MULTFACTOR)){
			fprintf(bayesNetFile,"%lf ",funPtr->probabList[i+j*step]*exp(funPtr->logMultFactor));
			}else{
				fprintf(bayesNetFile,"%lf ",funPtr->logMultFactor);
			}
		fprintf(bayesNetFile,"</DPI>\n");
	}

}
/*****************************************************************************
/*	DISTRIBUTION "root" node:
/*		-its children specify the functions of the model/network.
/*	DIST node:
/*		- it contains the description of a function/distribution in the model/network.
/*		- it has one attribute:
/*			* TYPE - the type of the function
/*				   		-> discrete = probability table
/*		- the scope of the function/distribution is specified by a PRIVATE node (i.e. the child) and the conditional set (i.e. the parents).
/*		* PRIVATE - identifies the child.
/*			- it is specified by the NAME attribute of the node and its value has to be a valid variable name (see the VAR node).	  
/******************************************************************************/
void  printFunctionsXBN(int size,function *functions){
	int i,varIndex;
	variable *varPtr;
	function *funcPtr;
	char name[44];
	for(i=0;i<size;i++){
		funcPtr=&(functions[i]);
		if (funcPtr->used==0 || funcPtr->probabList[0]>1 || funcPtr->probabList[0]<0 )
			continue;
		varPtr=funcPtr->owner;
		if (varPtr==0 ){
			if (strcmp(funcPtr->ownerName,MULTFACTOR)){
				sprintf(name,"dummy%x",funcPtr);
			}else{
				strcpy(name,funcPtr->ownerName);
			}
		}else
			strcpy(name,varPtr->originalNode->name);

		fprintf(bayesNetFile,"\t\t\t<DIST TYPE=\"discrete\">\n");
		//	varIndex=printFuncVariablesXBN(varPtr,funcPtr,name);
		varIndex=printFuncVariablesXBN(varPtr,funcPtr,funcPtr->ownerName);
		//	printf("\t\t\t\t <PRIVATE NAME=\"%s\"/>\n",name);
	if (strcmp((funcPtr->ownerName),"ms_18_7_f0") == 0  ){
			printf("");
		}
		fprintf(bayesNetFile,"\t\t\t\t <PRIVATE NAME=\"%s\"/>\n",funcPtr->ownerName);
		fprintf(bayesNetFile,"\t\t\t\t<DPIS>\n");

		printProbabilities(varPtr,funcPtr,varIndex);
		fprintf(bayesNetFile,"\t\t\t\t</DPIS>\n");
		fprintf(bayesNetFile,"\t\t\t</DIST>\n\n");
	}
}



/*****************************************************************************/
/*
/* Closes nodes
/******************************************************************************/
void printTailXBN(){
	fprintf(bayesNetFile,"\t</BNMODEL>\n");
	fprintf(bayesNetFile,"</ANALYSISNOTEBOOK>\n");
}
