#include "sl_XMLFormat.h"
#include "sl_Product.h"
#include <string.h>
#include "sl_XMLhelpFunction.h"

int *indexesVar;



void printXMLHead(int numPeds);
void printAllVariablesXML(int size,function *functions,char *family);
void printFunctionsXML(int size,function *functions);
void printTailXML();

/******************************************************************************
*
*       Function        convertToXML
*       Parameters		prodArr - product
*						numProd - number of products in array
*						peds - pedigree dat
*						locusNumber - number of loci
*       Output          none
*       Description		Prints current bayes network in XML(REES) format.
*       Comments		none
*
******************************************************************************/

void convertToXML(product**	 prodArr ,int numProd,pedData *peds,int locusNumber){
	int i,totalNumVariables=0/*,totalNumFunc=0*/;
//	int curVarNumber=0;
	char family[5];
	pedigree	*pedPtr;
	START=10;
	maxPeople=0;
	maxLoci=0;
	indexesVar = (int*)malloc(sizeof(int)*totalNumVariables);
	if (indexesVar== NULL){
		exit(2);
	}
	//printProduct(prodArr[0], 0);
	bayesNetFile = fopen("bayesNetFile.xml","w");
	if( bayesNetFile == NULL){
		//printerror("Cannot open file for write: bayesNetFile.xbn");
		exit(0);
	}
	maxLoci=locusNumber;
	for (pedPtr = peds[0].head ;pedPtr != NULL ;pedPtr= pedPtr->nextPed){
		maxPeople=(pedPtr->numPeople>maxPeople)?pedPtr->numPeople:maxPeople;
	}

	printXMLHead(peds->numPeds);
	addConstantToProd(prodArr[0]);
	fprintf(bayesNetFile,"\t\t<VARIABLES>\n");
	for(i=0;i<numProd;i++){
		sprintf(family,"_f%d",i);
		START=maxLoci*40+40+START;
		printAllVariablesXML(prodArr[i]->numFunctions,prodArr[i]->functions,family);
	}
	fprintf(bayesNetFile,"\t\t</VARIABLES>\n");
	;

	fprintf(bayesNetFile,"\t\t<DISTRIBUTIONS>\n");
	for(i=0;i<numProd;i++)
		printFunctionsXML(prodArr[i]->numFunctions,prodArr[i]->functions);
	fprintf(bayesNetFile,"\t\t</DISTRIBUTIONS>\n");

	printTailXML();
}


/*******************************************************/
/*	REES root node:
/*		- it has only one attribute which depends upon the REES system version (currently 2.0)
/*	MODEL node:
/*		- the node contains the model elements i.e. variables and distributions.
/*		- it has only two attributes:
/*			* NAME - the name of the model.
/*			* TYPE - the type of the model. The only possible values for this attr. are: BAYES (bayes models), COP (constraint models).
/*		- Four property children are available for custom bayesian networks.
/*			* FORMAT 		- this is a predefined value (= "custom").
/*			* CREATOR 		- may be ignored, but it specifies the creator of the model 
/*			* LAYOUT 		- the size of the drawing area, if an actual graphical representation is assigned with the model/network.
/* 		- it may not be present.----(it not)---
/*			* VARIABLES 	- the number of nodes in the network
/*			* DISTRIBUTIONS - the number of distributions (functions) in the model.
/**************************************************************************************************/
void printXMLHead(int numPeds)
{
	fprintf(bayesNetFile,"<?xml version=\"1.0\"?>\n");
	fprintf(bayesNetFile,"<!--Bayesian Network in XML v1.0 format.\n");
	fprintf(bayesNetFile,"Superlink v1.5 (Original Citation: Fishelson M. and Geiger D. Bioinformatics,  2002:18, S189-S198)\n(c)2002-2005 Technion - Israel Institute of Technology-->\n");
	fprintf(bayesNetFile,"<REES VERSION=\"2.0\">\n");
	fprintf(bayesNetFile,"\t<MODEL NAME=\"SuperLink Network\" TYPE=\"BAYES\">\n");
	fprintf(bayesNetFile,"\t\t<PROPERTY FORMAT=\"custom\"/>\n");
	fprintf(bayesNetFile,"\t\t<PROPERTY CREATOR=\"XML convertor\"/>\n");
	fprintf(bayesNetFile,"\t\t<PROPERTY LAYOUT=\"(%d,%d) \"/>\n",maxPeople*55*10,maxLoci*35*numPeds*2);
}

/******************************************************************************
*
*       Function        findPositionXML
*       Parameters		name - name of the variable 
*       Output          none
*       Description		finds and print variable location 
*       Comments		none
*
******************************************************************************/
void findPositionXML(char *name){
	int xPos,yPos;

	findPosition(name,&xPos,&yPos,maxPeople,START);
	fprintf(bayesNetFile,"\t\t\t<VAR NAME=\"%s\" TYPE=\"discrete\" XPOS=\"%d\" YPOS=\"%d\">\n",name,xPos,yPos);

}



/*******************************************************************************/
/*	VAR node:
/*	- it contains the description of a variable/node in the model/network.
/*	- it has four attributes:
/*		* NAME - the name of the variable (unique).
/*		* TYPE - the type of the variable (by default it has the value "discrete").
/*		* XPOS - X coordinate of the center of the graphical object (circle).
/*		* YPOS - Y coordinate of the center of the graphical object (circle).
/* 	- the children of the VAR node are represented by the following tags:
/*		* DESCRIPTION - a detailed description of the variable (as an XML text node).
/*		* OBSERVED - the text field of the node contains the actual value that was selected if the variable has been observed.
/*		* STATENAME - the domain of values is represented by a sequence of sibling nodes, every one of them being identified by this tag.
/********************************************************************************/
void 	printVariable(char* name,int numValues)
{
	int j;
	//	printf("\t\t\t<VAR NAME=\"%s\" TYPE=\"discrete\" XPOS=\"\" YPOS=\"\">\n",name);
	findPositionXML(name);
	fprintf(bayesNetFile,"\t\t\t\t<DESCRIPTION></DESCRIPTION>\n");
	for(j=0;j<numValues;j++)
		fprintf(bayesNetFile,"\t\t\t\t<STATENAME>Value%d</STATENAME>\n",j);
	fprintf(bayesNetFile,"\t\t\t\t<OBSERVED></OBSERVED>\n");
	fprintf(bayesNetFile,"\t\t\t</VAR>\n");
}


/***************************************************************************/
/*	VARIABLES "root" node:
/*		- its children specify the variables of the model/network.
/
/****************************************************************************/
void printAllVariablesXML(int size,function *functions,char *family)
{

	int i;
//	char name[20];
	variable *varPtr; 
	for(i=0;i<size;i++){
		if ( (&functions[i])->used==0 || (&functions[i])->probabList[0]>1 || (&functions[i])->probabList[0]<0 )
			continue;
		varPtr=(&functions[i])->owner;
		if ( strcmp((&functions[i])->ownerName,MULTFACTOR)){
			strcat((&functions[i])->ownerName,family);
		}
		if (varPtr==NULL){ // variable has only one value 		
			printVariable((&functions[i])->ownerName,1);
		}
		else{
			printVariable((&functions[i])->ownerName,varPtr->numValues);

		}



	}

}

/*****************************************************************************/
/*
/* Function print parents of first parameter according to function
/* return value - index of child in variable array,-1 if not exist there
/*
/*	* CONDELEM - identifies a parent.
/*		 - it is specified by the NAME attribute of the node and its value has to be a valid variable name.
/*	* CONDSET - the root node for the parents; each parent is represented by a CONDELEM node.
/*
/***************************************************************************/
int printFuncVariables(variable *var,function *funcPtr,char *name){

	variable *varPtr;
	int i,varIndex=-1;
	if (funcPtr->numVariables==1 && funcPtr->variables[0]==var )
		return 0;

	//print name of the parents
	fprintf(bayesNetFile,"\t\t\t\t<CONDSET>\n");
	for (i=0;i<funcPtr->numVariables;i++){
		varPtr=funcPtr->variables[i];
		if(varPtr==var){
			varIndex=i;
			continue;
		}
		fprintf(bayesNetFile,"\t\t\t\t\t<CONDELEM NAME=\"%s\"/>\n",varPtr->originalNode->name);
	}
	fprintf(bayesNetFile,"\t\t\t\t</CONDSET>\n");
	return varIndex;
} 

/************************************************************************/
/*
/* print probabilities of varPtr given funPtr.
/* for each probability print indexes of the parents
/*	- the function itself (i.e. data) is represented by the DPIS section.
/*	- each DPI entry represents the probability values for each of the values of the child and a certain combination of values for the parents.
/*		* INDEXES - a comma (,) separated list of zero-based value indexes for the conditional set (parents).
/*		* the XML text node associated with the DPI tag is a blank ( ) separated list of probability values corresponding to each of the child values (starting from the first one)
/*		* the indexes are used to compute the index entry in the table.
/*******************************************************************************/
void printProbabilitiesXML(variable *varPtr,function *funPtr,int varIndex){
	int j,numValues,step=1;
	unsigned int i;
	int *indexes;
	char str[500],tmp[100];
	indexes=(int*)calloc(funPtr->numVariables,sizeof(int));

	if (varPtr==NULL)
		numValues=1;
	else
		numValues=varPtr->numValues;
	for (j=varIndex+1;varIndex!=-1 && j<funPtr->numVariables;j++)
		step*=(funPtr->variables[j])->numValues;

	/* go over all probabilities*/
	for (i=0;i<funPtr->sizeOfTable;i++){

		if(varIndex!=-1 && indexes[varIndex]!=0){
		   //already was printed 
			addOneToIndex(indexes,funPtr->numVariables,funPtr);
			continue;
		}
		str[0]='\0';
		fprintf(bayesNetFile,"\t\t\t\t\t<DPI INDEXES=\"");
		//print current indexes
		/*	if (funPtr == 0x004c4b90){
		printf("");
		}*/
		for (j=0;j<funPtr->numVariables;j++){
			if (j==varIndex)
				continue;
			sprintf(tmp,"%d, ",indexes[j]);
			strcat(str,tmp);
		}
		addOneToIndex(indexes,funPtr->numVariables,funPtr);
		if (strlen(str)< 2){
			fprintf(bayesNetFile,"");
		}else{
			str[strlen(str)-2]='\0'; //delete last ','
		}
		fprintf(bayesNetFile,"%s\" ADDRESS=\"\" OFFSET=\"\">",str);

		//print probabilities */
		for (j=0;j<numValues;j++)  
			if (strcmp(funPtr->ownerName,MULTFACTOR)){
				fprintf(bayesNetFile,"%.6lf ",funPtr->probabList[i+j*step]*exp(funPtr->logMultFactor));
			}else{
				fprintf(bayesNetFile,"%.6lf ",funPtr->logMultFactor);
			}
		fprintf(bayesNetFile,"</DPI>\n");
	}

}
/*****************************************************************************
/*	DISTRIBUTION "root" node:
/*		-its children specify the functions of the model/network.
/*	DIST node:
/*		- it contains the description of a function/distribution in the model/network.
/*		- it has one attribute:
/*			* TYPE - the type of the function
/*				   		-> discrete = probability table
/*		- the scope of the function/distribution is specified by a PRIVATE node (i.e. the child) and the conditional set (i.e. the parents).
/*		* PRIVATE - identifies the child.
/*			- it is specified by the NAME attribute of the node and its value has to be a valid variable name (see the VAR node).	  
/******************************************************************************/
void  printFunctionsXML(int size,function *functions)
{
	int i,varIndex;
	variable *varPtr;
	function *funcPtr;
	char name[44];
	for(i=0;i<size;i++){
		funcPtr=&(functions[i]);
		if (funcPtr->used==0)
			continue;
		varPtr=funcPtr->owner;
		if (varPtr==0 ){
			if (strcmp(funcPtr->ownerName,MULTFACTOR)){
				sprintf(name,"dummy%x",funcPtr);
			}else{
				strcpy(name,funcPtr->ownerName);
			}
		}else{
			strcpy(name,varPtr->originalNode->name);
		}

		fprintf(bayesNetFile,"\t\t\t<DIST TYPE=\"discrete\">\n");
		fprintf(bayesNetFile,"\t\t\t\t <PRIVATE NAME=\"%s\"/>\n",funcPtr->ownerName);
		varIndex=printFuncVariables(varPtr,funcPtr,funcPtr->ownerName);
		fprintf(bayesNetFile,"\t\t\t\t<DPIS>\n");
		printProbabilitiesXML(varPtr,funcPtr,varIndex);
		fprintf(bayesNetFile,"\t\t\t\t</DPIS>\n");
		fprintf(bayesNetFile,"\t\t\t</DIST>\n\n");
	}
}



/*****************************************************************************/
/*
/* Closes nodes
/******************************************************************************/
void printTailXML(){
	fprintf(bayesNetFile,"\t</MODEL>\n");
	fprintf(bayesNetFile,"</REES>\n");
}

