#!/usr/bin/perl -w
# dvir 2011-10
use strict;

use FindBin ();
use lib "$FindBin::Bin";

use POSIX qw(strftime); # in order to track the locks time

use Configuration qw(%configfile); 
use General;
use SetupEnviourment;
use SharedConsts;

# die "boooooooooooob";
# if there are no resources indication, than it will exit
&General::makeSureThatSystemIsNotOutOfResources();

# perl AcceptSubmission.pl <projectName> ( tar in stdin with details.txt file and ped/dat files)
if( $#ARGV < 0 ) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"missing argument: projectName");
}
my $projectName = $ARGV[0];

&SetupEnviourment::runSetup();
   
my $projectTaskDir = &General::getTaskDirNameByTaskName($projectName);

# preforming a lock in order that only one script will run at a time
# and by that we avoid submitting the same task twice.
open (ACCEPT_SUBMISSIONS_LOCK, ">$Configuration::ACCEPT_SUBMISSIONS_SCRIPT_LOCK") || &General::reportOnErrorReleaseSemaphoresAndDie($0,`pwd`, sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $Configuration::ACCEPT_SUBMISSIONS_SCRIPT_LOCK ,$!));
flock(ACCEPT_SUBMISSIONS_LOCK, $Configuration::EXCLUSIVE_LOCK) || &General::reportOnErrorReleaseSemaphoresAndDie($0,`pwd`, sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $Configuration::ACCEPT_SUBMISSIONS_SCRIPT_LOCK ,$!));

# this check is when we hold the lock, in order to avoid the case of 2 processes which
# try to submit the same task
if( -d $projectTaskDir) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"project with the name: $projectName already exists");
}

# checking that there aren't too many tasks in WAITING_TO_RUN_DIR
my $howManyTasksAreInWaitingToRunMode = &General::howManyTasksAreInWaitingToRunMode();
if ($howManyTasksAreInWaitingToRunMode > $Configuration::MAX_TASKS_IN_WAITING_TO_RUN_DIR) {
	# o.k to exit here, just marking that t
	close ACCEPT_SUBMISSIONS_LOCK;
	exit $SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS;
}

my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
print ACCEPT_SUBMISSIONS_LOCK "locked by AcceptSubmission $$ at $now_string\n";

mkdir $projectTaskDir || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"couldn't create $projectTaskDir");

# extracting the tar in the standard input
my $command = "tar -C $projectTaskDir -xzf -";
system($command);
my $ret = $? >> 8;
if ($ret != 0) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to preform : $command");
}

my $projectDetailsFile = "$projectTaskDir\/details.txt";
# making sure that the details files exists
unless( -e $projectDetailsFile ) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"missing file in tar: $projectDetailsFile");
}

$command = "tar -C $projectTaskDir -xzf $projectTaskDir\/Files.tar.gz";
system($command);
$ret = $? >> 8;
if ($ret != 0) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to preform : $command");
}

########111
# _STAGE2_ -> delete in future
my $projectDataDir = $configfile{dataDir}."\/$projectName";
my $projectOutputDir  = $configfile{outputDir}."\/$projectName";
my $projectMailDir = $configfile{mailDir}."\/$projectName";  
mkdir $projectMailDir || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"couldn't create $projectMailDir");
mkdir $projectOutputDir || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"couldn't create $projectOutputDir");
mkdir $projectDataDir || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"couldn't create $projectDataDir");

system("cp $projectTaskDir\/*.ped $projectTaskDir\/*.dat $projectTaskDir\/.*.raw $projectDataDir >& /dev/null"); 

my $taskFile = "$projectTaskDir\/task.log";

open( TASK , ">$taskFile" );
print TASK "$projectDataDir\n";	
print TASK "$projectOutputDir\n";
########111

# making sure that all files exists
open DETAILS, $projectDetailsFile;
 
my $doWeHaveAtLeastOneFileInTheTar = 0;
for my $line ( <DETAILS> ){
	$line =~ s/\s+$//; #remove \n
	if( length($line) == 0 ) {
		next;
	}
	$doWeHaveAtLeastOneFileInTheTar = 1;
	my $currentFile = "$projectTaskDir\/" . "$line";
	unless (-e $currentFile) {
		&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"missing file in tar: $currentFile");
	}
	####222
	if ($line =~ /\.dat/) {
		my $chromName = $line;
		$chromName =~ s/\.dat//g;
		print TASK "$chromName\n";
	}
	####222
}
close DETAILS;
close TASK;

# checking if we don't have any files in the tar
if (! $doWeHaveAtLeastOneFileInTheTar) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"tar has no files within");
}

&General::markThatTaskIsSubmmited($projectName);
close ACCEPT_SUBMISSIONS_LOCK;
