# dvir 2011/03
package Analysis;

use strict;
use General;
use Configuration;
use POSIX;
use ProcessingBackend;

# all the possible analysises:
use HaplotypingLars;
use MorganLmMarkers;
use GlAuto;
use Drawing;
use HMM;

# this file is a framework for analysis.

our $projectName; # the name of the task at the tasks dir
our $projectTaskDir; # the task direcory path, treated as an absoulte path
our $workDir;
our $analysisTypeFileIndication;
our $analysisTypeName;
our $isAnalysisAMultiWindowTask;
	
our @ISA; # from which class we are inherting the methods

# returning the type of the analysis
sub getType() {
	return $analysisTypeFileIndication;
}
sub getTypeName() {
	return $analysisTypeName;
}

sub getProjectName() {
	return $projectName;
}

sub getProjectDir() {
	return $projectTaskDir;
}

sub getWorkDir() {
	return $workDir;
}

sub isAnalysisAMultiWindowTask() {
	return $isAnalysisAMultiWindowTask;
}

# returning the name of the original dat, at the project dir
# (it is named the original dat, because we can have new dat-s for range or list)
# doesn't work on Drawing, because we don't have a dat !! and drawing doesn't have the modefile.txt file.
sub getOriginalDatFileName() {
	
	# reading the variables from the user
	&ProcessingBackend::setLogFiles("$projectTaskDir\/");
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	
	# the dat & ped files which were received from web server
	my $datFileName = "$projectTaskDir\/$variables{ datafile }";	
	
	return $datFileName;
}

# Creating a new analysis
# the $analysisWorkDir should contain a indication file that indicates what type of analysis is it
# methods whose routine isn't implemented at this class will be call at runtime
# from the package at @ISA
sub new( $ ) {
	my $class = $_[0]; # the name of this class
	$workDir = $_[1]; # analysisWorkDir
	
	($projectName, $projectTaskDir) = &General::getTaskNameAndTaskDirFromTaskWorkDir($workDir);
	
	if (-e "$workDir\/$Configuration::HAPLOTYPING_ANALYSIS_FILENAME") {
		$analysisTypeFileIndication = "$workDir\/$Configuration::HAPLOTYPING_ANALYSIS_FILENAME";
		$analysisTypeName = $Configuration::HAPLOTYPING_ANALYSIS_FILENAME;
		@ISA = ('HaplotypingLars');
	}
	elsif (-e "$workDir\/$Configuration::MORGAN_LM_MARKERS_ANALYSIS_FILENAME") {
		$analysisTypeFileIndication = "$workDir\/$Configuration::MORGAN_LM_MARKERS_ANALYSIS_FILENAME";
		$analysisTypeName = $Configuration::MORGAN_LM_MARKERS_ANALYSIS_FILENAME;
		@ISA = ('MorganLmMarkers');
	}
	elsif (-e "$workDir\/$Configuration::DRAWING_ANALYSIS_FILENAME") {
		$analysisTypeFileIndication = "$workDir\/$Configuration::DRAWING_ANALYSIS_FILENAME";
		$analysisTypeName = $Configuration::DRAWING_ANALYSIS_FILENAME;
		@ISA = ('Drawing');
	}
	elsif (-e "$workDir\/$Configuration::MORGAN_GL_AUTO_ANALYSIS_FILENAME") {
		$analysisTypeFileIndication = "$workDir\/$Configuration::MORGAN_GL_AUTO_ANALYSIS_FILENAME";
		$analysisTypeName = $Configuration::MORGAN_GL_AUTO_ANALYSIS_FILENAME;
		@ISA = ('GlAuto');
	}
	elsif (-e "$workDir\/$Configuration::HMM_ANALYSIS_FILENAME") {
		$analysisTypeFileIndication = "$workDir\/$Configuration::HMM_ANALYSIS_FILENAME";
		$analysisTypeName = $Configuration::HMM_ANALYSIS_FILENAME;
		@ISA = ('HMM');
	}	
	else {
		&General::reportOnErrorReleaseSemaphoresAndDie('Analysis.pm',`pwd`, "new(): unknown analysis");
	}
	
	$isAnalysisAMultiWindowTask = -e "$workDir\/$Configuration::MULTI_WINDOW_TASK_INDICATION_FILENAME";
	
	return bless {}, $class;
}

1;