package AnalysisSubmissionFile;

use strict;
use Class::Struct;

use Configuration;

our $JOB_LOG_FILENAME = 'AnalysisRunScript.condor.log';

# dvir 2011/08/04
# a good way is for anaylsis to have only one Submission file.
# and if there are many jobs, than all the jobs can be with the same 
# submission file, and each job can have its own param file (simple text file)
# to distinguish the job from other jobs.

struct AnalysisSubmissionFileData =>
[
	executeableName => '$',
	executeableArgumentsList => '@',
	executeableInputFilesList => '@',
	jobUniverse     => '$',
	jobPriority     => '$',
	jobRank         => '$',
	jobSecondsToBeExecuted => '$',
	jobMemoryRequirments => '$',
	canRunOn32BitMachine => '$', # 0 - no, can run on 64bitMacine. 1 - yes
];

sub createAnalysisSubmissionFileAtCwd( $ $ ) {
	my $analysisSubmissionFileData = $_[0];
	my $analysisSubmissionFileName = $_[1];
	
	my $argumentsStr = '';
	if (scalar(@{$analysisSubmissionFileData->executeableArgumentsList}) > 0) {
		my $tmp = @{$analysisSubmissionFileData->executeableArgumentsList};
		$argumentsStr = 'arguments = '.join (' ', @{$analysisSubmissionFileData->executeableArgumentsList});
	}

	my $inputFilesStr = join ',', @{$analysisSubmissionFileData->executeableInputFilesList};

	my $canRunOn32BitMachineStr = '';
	if ($analysisSubmissionFileData->canRunOn32BitMachine) {
		$canRunOn32BitMachineStr = ' ||(Arch == "INTEL" && OpSys == "LINUX")';
	}

	my $executeableName = $analysisSubmissionFileData->executeableName();
	my $jobUniverse = $analysisSubmissionFileData->jobUniverse();
	my $jobPriority = $analysisSubmissionFileData->jobPriority();
	my $jobRank = $analysisSubmissionFileData->jobRank();
	my $jobSecondsToBeExecuted = $analysisSubmissionFileData->jobSecondsToBeExecuted();
	my $jobMemoryRequirments = $analysisSubmissionFileData->jobMemoryRequirments();

	my $condorJobAnalysisTemplate = `cat $Configuration::CONDOR_JOB_SUBMIT_DEFENITIONS_TEMPLATE_FILENAME`;
	$condorJobAnalysisTemplate =~ s/_EXECUTEABLE_NAME_/$executeableName/g;
	$condorJobAnalysisTemplate =~ s/_ARGUMENTS_/$argumentsStr/g;
	$condorJobAnalysisTemplate =~ s/_INPUT_FILES_/$inputFilesStr/g;
	$condorJobAnalysisTemplate =~ s/_UNIVERSE_/$jobUniverse/g;
	$condorJobAnalysisTemplate =~ s/_PRIORITY_/$jobPriority/g;
	$condorJobAnalysisTemplate =~ s/_RANK_/$jobRank/g;
	$condorJobAnalysisTemplate =~ s/_SECONDS_TO_BE_EXECUTED_/$jobSecondsToBeExecuted/g;
	$condorJobAnalysisTemplate =~ s/_MEMORY_REQUIERMENTS_/$jobMemoryRequirments/g;
	$condorJobAnalysisTemplate =~ s/_32_BIT_REQUIERMENTS_/$canRunOn32BitMachineStr/g;
	$condorJobAnalysisTemplate =~ s/_LOG_NAME_/$JOB_LOG_FILENAME/g;
	
	open CONDOR_JOB_SUBMIT_FILE, ">$analysisSubmissionFileName";
	print CONDOR_JOB_SUBMIT_FILE $condorJobAnalysisTemplate;
	close CONDOR_JOB_SUBMIT_FILE;
}
1;
