package DAGSplit;
###########################################################################################################
# 	This file is the library for all the functions which are used while spliting / checking / clustering  #
###########################################################################################################
use Cwd;
use FindBin();
use lib "$FindBin::Bin";
use Configuration qw(%configfile);
use Tie::File;
use POSIX qw(strftime); # in order to track the error time
use Environment;
use General;

		#####################################################################
		#  	 Global Environment Parameters - Defined in DAGConfiguration file 	#
		#####################################################################

my $SNPdistiller  					=$configfile{SNPdistiller}; 

my $mode  							=$configfile{mode};  				# Initialize again in initVariablesFromFile()    
my $programCode  					=$configfile{programCode}; 			# Initialize again in getDatafileInfo()

my $partNum  						=$configfile{partNum}; 				# Number of clustering parts - Initialize with 0 in  EliminatingAndCluster() or Clustering()
																		# after it this value is used in joinFiles() 
my $dataDir 						=$configfile{dataDir};  	
my $errorOutboxReport				=$configfile{errorOutboxReport};	
my $splitInnerSysError				=$configfile{splitInnerSysError};
my $splitRunLog						=$configfile{splitRunLog};
my $cleaningParamToDistiler 		=$configfile{cleaningParam};
my $clusteringParamToDistiler 		=$configfile{clusteringParam};
my $clusteringRawParamToDistiler	=$configfile{clusteringRawParam};
	
				########################
				#    SSH  Parameters   #
				########################
				
$ssh_options = $configfile{ssh_options};               
$ssh_remote = $configfile{ssh_remote};              
$ssh_user = $configfile{ssh_user};              
$ssh_cmd = "ssh $ssh_options $ssh_user" . "@" . "$ssh_remote";
$remote_script = $configfile{remote_script}; 
$currProName = ""; # Initialized in initVariablesFromFile function

#remote info
$remote_user_full = "$ssh_user" . "@" . "$ssh_remote";
$remote_submission_dir = $configfile{remote_dir};

				########################
				#  Global Parameters   #
				########################

# Run time Environment parameters
#################################

my @datfile 				= (); # Current dat file array - initialize in getFiles function
my @pedfile 				= (); # Current ped file array - initialize in getFiles function		
my %variables 				= (); # Environment variables hash table		
my $workDir 				= ""; # Work dir of the current task being performd ("chrom_%d" dir) - see setEnviroment()
my $clusterWorkDir 			= ""; # Clustering dir of the current chromosome 	- initialize in EliminatingAndCluster() or Clustering() 
my $markerListsDir 			= ""; # Dir in which all the markers output lists of the current chromosom will be saved
my $cleaningWorkDir 		= ""; # Dir in which all the ped error files of the current chromosom will be saved
my $currentChromosomName 	= ""; 



# Calculation associate parameters
##################################

my $numMarkers = 0;				# Number of markers	     										 - initialize in getDatafileInfo() 
my $numDiseaseLoci = 0;			# Number of Disease Loci 										 - initialize in getDatafileInfo() 
my @dist = ();					# Array for the distances between each marker to the start point - initialize in getDatafileInfo()  

my $distLastPart = 0;			# Used by createMappingFile for updating distances
my @distBetweenParts = ();		# Distance between files

my $totalLenght =0 ;			# Total distance between the first and the last chromosoms

my $cMorganWindowSize=10; 		# "Big" Window size in centimorgan - Used only by EliminatingAndCluster() at the moment.
								# should be initialize by user parameter ( see initVariablesFromFile() ). Notice - dummy initialization.

my @markersIDStrings = ();		# Saving for each Marker his "rs*" string. 

my $checkGenoParam = 0;			# Type of genotype check ( ERRORS_PER_MARKER_TRESHOLD ) 

				##################################
				#	Public Functions Deleration  #
				##################################
				
# Updated global variables hash according to the given file ("variables.txt" as defined at splitSNPdistiller.pl)
# Can also update some Global Environment Parameters if mention in the file
sub initVariablesFromFile( $ $ $ ); 

# Saving the information of the dat & ped files into the global "datfile" & "pedfile" arrays
sub getFiles( $ );

# Creating the suitable directories for the task (workDir and chrom_%d)
sub setEnviroment();

# Updating the global parameters according to the dat file	(numDiseaseLoci,numMarkers,programCode,dist)
sub getDatafileInfo();

# Return the current run mode  
sub getMode();

# Performing spliting, checking & clustering 
sub EliminatingAndCluster( );

# Performing cleaning only
sub cleaningAllOnly();

# Performing clusteting only
sub clusteringOnly( $ );

# Join all the ped & dat files into 2 output files 
sub joinFiles();

# Log report functions
######################
sub splitLog( $ );

				###############################################
				#		Auxiliary functions Deleration 		  #
				###############################################

# Check legality of the given dat file 
sub checkDataFile( $ $ $ );

# Check legality of the given ped file 
sub checkPedFile( $ $ $ $ );

# Creating new ped & dat files according to the given start & end values.
sub createPart( $ $ $ $ );

# Return the number of markers in the given dat file
sub getAmountOfMarkers( $ ) ;

sub isThereMarkers( $ $ );

# Return the number of liability classes in the dat file
sub getNumberOfLiability( $ );
 
# Create map file according to the given dat file
sub createMappingFile( $ );

# Create map file according to the given dat file THIS DUP IS INLY TMP SOLUTION IN ORDER TO CHECK SOME FUNCTIONS
sub createMappingFileForClusteringOnly( $ );

# Create new error log file according to a given flag and err.txt file.
# after it send to log to the website.
sub createAndSendCurrentReportFile ( $ $ $ $ $ );

# Send cluster report with the given progress percent
sub sendClusterReport( $ );

# $_[0] - final log file 
sub AddSummaryToFinalReportFile( $ );

# Error log functions
######################
sub inputError( $ );
sub fileLog( $ $ );
sub logInnerSysError( $ );
sub logInnerSysMinorError( $ );
			
		
# Conversion functions
######################
sub recToCm( $ );
sub cmToRec( $ );

				######################################
				#	Public Functions Implementation  #
				######################################	

################################################################
#		initVariablesFromFile
#
#	Parameters:
#			$_[0] - filename of file containing directories name and possible extra var
#		    $_[1] - filename of file containing the current chromosom ped and dat file names
#		    $_[2] - name of the project
#			$_[3] - name of chrom
#	Returns:
#			updated global variables hash
################################################################
sub initVariablesFromFile( $ $ $ )
{
	my $filename = $_[0];
	$currProName = $_[2];
	$currentChromosomName=$_[3];
	open( VARIABLES_FILE , $filename ) or logInnerSysError("couldn't open $filename for reading of variables");
	
	my $lineNum = 0;
	
	for $line ( <VARIABLES_FILE> )
	{
		$lineNum++;
		my @tmp = split(/[ \t\n\r\s\f]+/,$line);
		#Omer - there can be fields with no value, so I disabled this check
		#if( $#tmp != 1 ) {inputError("bad line $lineNum in $_[0] file");}		
		#$variables { $tmp[0] } = $tmp[1];
		if( $#tmp == 1 ) {$variables { $tmp[0] } = $tmp[1];}
	}	
	close( VARIABLES_FILE );
	
	$filename = $_[1];
	open( CURRCHR_FILE , $filename ) or logInnerSysError("couldn't open $filename for reading of variables");
	$lineNum = 0;
	
	for $line ( <CURRCHR_FILE> )
	{
		$lineNum++;
		my @tmp = split(/[ \t\n\r\s\f]+/,$line);
		if( $#tmp != 1 )
		{
			inputError("bad line $lineNum in $_[1] file");
		}
		$variables { $tmp[0] } = $tmp[1];
	}	
	close( CURRCHR_FILE );
	
	#checking for needed arguments
	if( !(exists $variables{ 'DatfileDir' }) or 
		!(exists $variables{ 'PedfileDir' }) or 
		!(exists $variables{ 'Datfile' }) or 
		!(exists $variables{ 'Pedfile' } ) )
	{
		
		inputError("missing input file info");
		inputError("variables file should contain:\nDatfileDir\nPedfileDir\nDatfile\nPedfile");
	}	

	# Updating Environment parameters according to the variables file
 
	my $datfileName = $variables{ Datfile };
	$datfileName =~ s/\.dat//;	
	$currentChromosomName=$datfileName;
	
	if( exists $variables{ 'CMWindowSize' } )
	{
		$cMorganWindowSize = $variables{ CMWindowSize };
	}	
	
	if( exists $variables{ 'errorsPerMarkerThreshold' } )
	{
		my $typeOfCheck = $variables{ errorsPerMarkerThreshold };
		if ( $typeOfCheck eq "Strong" )
		{
			$checkGenoParam = 0;
		}
		elsif ( $typeOfCheck eq "Medium" )
		{
			$checkGenoParam = 4;
		}
		elsif ( $typeOfCheck eq "Weak" )
		{
			$checkGenoParam = -1; # Pass the unlikely check
		}
	}	
	if (exists $variables{ 'modeOfRun' } )
	{
		$mode = $variables{ modeOfRun };
	}	
	
	return \%variables; 
}

################################################################
#		getFiles
#
#	Parameters:
#		$_[0] - name of the badfile log ( used for file checking inside)
#		$_[1] - the path to write the datafile result
#		$_[2] - the path to write the pedfile result
#		$_[3] - 1 if the files were created from snp source and 0 otherwise
#	Returns:
#		Returns a ref to array of ref to datlines,pedlines
#		if bad files return 1 else 0
################################################################
sub getFiles( $ )
{
	my ($line,$i,$j);
	
	if (defined($_[3])) {
		$snpSource = pop @_;
	}
	else {
		$snpSource = 0;
	}
	 
	#if ( ($mode eq "EliminatingAndCluster") || ($mode eq "EliminatingOnly") )
	#{
		$res = checkDataFile( "$variables{ DatfileDir }\/$variables{ Datfile }" , "FirstDatFile" , $snpSource);
		if ($res != 0) {
			return $res;
		}
	#}
	
	my $numberOfMarkerInPed = &getAmountOfMarkers("$variables{ DatfileDir }\/$variables{ Datfile }");
	 
	
	my $numberOfLiabilityClassesInPed = &getNumberOfLiability("$variables{ DatfileDir }\/$variables{ Datfile }");
	 
	
	$numberOfMarkerInPed --;  
	
	#if ( ($mode eq "EliminatingAndCluster") || ($mode eq "EliminatingOnly") )
	#{
		$res = checkPedFile( "$variables{ PedfileDir }\/$variables{ Pedfile }" , $numberOfMarkerInPed , $numberOfLiabilityClassesInPed , "FirstDatFile" ); 
		if ($res != 0) {
			return $res;
		}
	#}
	
	open (DATFILE,"$variables{ DatfileDir }\/$variables{ Datfile }") or logInnerSysError("couldn't open $variables{ DatfileDir }\/$variables{ Datfile } for reading");
	open (PEDFILE,"$variables{ PedfileDir }\/$variables{ Pedfile }") or logInnerSysError("couldn't open $variables{ PedfileDir }\/$variables{ Pedfile } for reading");
	$i = 0;
	my $trimLine = "";
	for $line (<PEDFILE>) 
	{
		$trimLine=&trim($line);
        $pedfile[$i] = [ split(/[ \t\n\r\s\f]+/,$trimLine) ];
		 
		$i++;								
	}	
 
	$i = 0;
	for $line (<DATFILE>) 
	{
		$trimLine=&trim($line);
		if ($trimLine eq "") {
			next;
		}
        $datfile[$i] = [ split(/[ \t\n\r\s\f]+/,$trimLine) ];		
		# Add '#' to the end of the marker name in case we are at marker line - the distiller need the '#' 
		if (($datfile[$i][0] ne "-n") && ($datfile[$i][0] eq "3") && ($datfile[$i][2] eq "#"))
		{
			$datfile[$i][3]=&trim($datfile[$i][3]);
			if(defined($datfile[$i][4]))
			{
				unless ($datfile[$i][4] eq "#")
				{
					my $j=0;
					for $j ($#{$datfile[$i]}+2 .. 4)
					{
						$datfile[$i][$j] = $datfile[$i][$j-1];
					}
					
				}
			}
			$datfile[$i][4] = "#";
		}		
		$i++;
	}	
	#removing empty lines
	while(1)
	{
		if( $#{$datfile[$#datfile]} < 1 )
		{
			$#datfile = $#datfile - 1;
		}
		else
		{
			last;
		}	
	}
	while(1)
	{
		if( $#{$pedfile[$#pedfile]} < 1 )
		{
			$#pedfile = $#pedfile - 1;
		}
		else
		{
			last;
		}	
	}
	
	# Write the file into the given paths
	open(FIXEDDATFILE ,">$_[1]" );

	$i = 0;
	my $tmp;
	for $i (0..$#datfile)
	{
		$tmp = join(' ', @{$datfile[$i]});
		print FIXEDDATFILE "$tmp";
		print FIXEDDATFILE "\n";
	}	
	close FIXEDDATFILE;

	open(FIXEDPEDFILE ,">$_[2]" );

	$i = 0;
	for $i (0..$#pedfile)
	{
		$tmp = join(' ', @{$pedfile[$i]});
		print FIXEDPEDFILE "$tmp";
		print FIXEDPEDFILE "\n";
	}	
	close FIXEDPEDFILE;
	
	#return $result; 
	return 0; 
}
 
#########################################################################
#		setEnviroment
#
# Parameters:
#	None
# Description:
#	Creating the suitable directories for the task (workDir and chrom_%d)
# 
#########################################################################
sub setEnviroment(){
	my ($cwd) = cwd();
	my $datfileName = $variables{ Datfile };
	$datfileName =~ s/\.dat//;	
	 # First creating / checking the workDir of the current task 
	$workDir = "$cwd\/workDir";
	( -d $workDir ) or system("mkdir $workDir");
	( -d $workDir ) or logInnerSysError("mkdir $workDir failed");
    
	
	# Creating workDir for the current chromosom.  
	$workDir = "$workDir\/$datfileName";
	splitLog("try to create $workDir");
	if( -d $workDir ){
		splitLog("$workDir already exist, content will be override.");
	}
	else{
		system("mkdir $workDir");
		( -d $workDir ) or logInnerSysError("mkdir $workDir failed");
	}
}

################################################################
#		getDatafileInfo
#
# Parameters:
#	None
# Description:
#	Updating the global parameters according to the dat file		 
################################################################
sub getDatafileInfo( )
{
	
	if( $#{$datfile[0]} == 3){		
		$numDiseaseLoci = 1;
	}
	elsif( $datfile[0][4] == 0 ){
		$numDiseaseLoci = 1;
	}
	else{
		$numDiseaseLoci = 2;
	}
	
	$numMarkers = $datfile[0][0] - $numDiseaseLoci;
	$programCode = $datfile[0][3]; 

	my ($distLine);
	if( $datfile[ $#datfile - 1 ][0] eq "1" ){ 			
		$distLine = $#datfile - 2;
	}
	else{
		$distLine = $#datfile - 3;
	}

	# Initializing the distances array - each distance is the relative distance to the start point.
	# Also initializing the total lenght
	$dist[0] = 0;
	for $i (2..$numMarkers){
		$dist[$i-1] = recToCm($datfile[$distLine][$i-1]) + $dist[$i-2];
	}
	$totalLenght = $dist[$numMarkers-1];
	splitLog("Got markers distance & total lenght");
	
	
	 
}

################################################################
#		getMode
#
#	Parameters:
#				None
#	Return value:
#		 		Current run mode.
################################################################
sub getMode(){
	return $mode;
}

################################################################
#		EliminatingAndCluster
#
#	Parameters:
#			None
#	Description:
#	 Performing spliting, checking & clustering
################################################################
sub EliminatingAndCluster( )
{
	splitLog("Eliminating And Clustering");
	$clusterWorkDir =  "$workDir\/clustering";
	$markerListsDir =  "$workDir\/markersLists";
	$cleaningWorkDir 	=  "$workDir\/cleaningWorkDir";
	
	unless( -d $clusterWorkDir )
	{
		mkdir $clusterWorkDir or logInnerSysError("couldn't create $clusterWorkDir");
	}
	unless( -d $markerListsDir )
	{
		mkdir $markerListsDir or logInnerSysError("couldn't create $markerListsDir");
	}
	unless( -d $cleaningWorkDir )
	{
		mkdir $cleaningWorkDir or logInnerSysError("couldn't create $cleaningWorkDir");
	}
	
	my $avrageDistBetweenMarkers = ($dist[$numMarkers-1]/$numMarkers); # In order to calc suitble small window size
	my $smallWindowSize = ($avrageDistBetweenMarkers * 50); # We want to handle maximum 50 markers in one small window
	my $halfSmallWindowSize = ($smallWindowSize/2); # Dist in centimorgan
	
	# We dont want out small window will be big. so we make a upper bound for him -
	# at most half of big window size
	if($smallWindowSize > ($cMorganWindowSize/2)) 
	{ 
		$smallWindowSize = $cMorganWindowSize/2; 
		$halfSmallWindowSize = $smallWindowSize/2; 
	}
	
	# We will count out distance as we walk along the chromosom
	my $currentDistFromFirstMark=0;
	
	# my	$bigWindowSize = $cMorganWindowSize;
	  my	$bigWindowSize = 2 * $cMorganWindowSize; # we want to have $cMorganWindowSize dist between clusters
	# Current cluster file number
	$partNum = 0;
	
	# Parameters for the inner loop:
	my $foundResult = 0; 
	my $result = 0;
	
	# Initialize values with zeros, but surly the scan will start from chromosom 1
	my $numOfFirstMarkerInCurrentWindow =0; 
	my $numOfLastMarkerInTheCurrentWindow =0;
	my $numOfMiddleMarkerInTheCurrentWindow=0;
	my $numOfFirstMarkerInTheBigWindow =0; # For report perpose
	
	my $smallIterationNum=0;
	my $bigIterationNum = 0;
	
	my $tmpPart=0; # see the handle to unchecked markers creation
	my $needToSendLastRep =0;
	
	# In this loop we will create in each iteration 2 files and run the SNPdistiller on them.
	# The total amount of files is affected by the results of the SNPdistiller - meaning by the presence of markers.
	while( $currentDistFromFirstMark < $totalLenght )  # "Walking" all over the chromosom
	{
	
		# Current cluster file number that will be created in the current iteration
		$partNum++;
		$bigIterationNum++;
		$foundResult = 0;  
		$smallIterationNum=0;   
		
		# First placing the first marker of the current window to be after the last marker of the prev window
		# We do that in order to save some steps in the next loop
		$numOfFirstMarkerInCurrentWindow = ($numOfLastMarkerInTheCurrentWindow +1);
		$numOfFirstMarkerInTheBigWindow = $numOfFirstMarkerInCurrentWindow;
		
		my $curUncheckedMarkerNum =0;
		my $haveUncheckedMarker =0;
		my $firstUncheckedMarker =0;
		
		# Now finding the first marker placed in the current big window
		while	(
				($numOfFirstMarkerInCurrentWindow<$numMarkers) 
				&& 
				($dist[$numOfFirstMarkerInCurrentWindow-1] < ($bigWindowSize * ($bigIterationNum-1)) ) 
				) 
		{
			if ($partNum != 1)
			{
				# Updating the last unchecked marker ID  
				$curUncheckedMarkerNum = $numOfFirstMarkerInCurrentWindow;
				
				# Saving the first unchecked marker
				if ($haveUncheckedMarker == 0)
				{
					$firstUncheckedMarker = $curUncheckedMarkerNum
				}
				$haveUncheckedMarker = 1;
				
			}
			$numOfFirstMarkerInCurrentWindow++;
		}
		
		# Inorder to print report on the big window, we need the first marker from which we start our current check
		my $numOfFirstMarkerInTheFirstLittleWindow = $numOfFirstMarkerInCurrentWindow;
		
		if (($partNum != 1)&&($haveUncheckedMarker == 1))
		{
			# Adding the unchecked markers to the prev report
			
			# in order to creat handles to the prev iteration
			$tmpPart = $partNum-1;
			
			# Creating handle to a file in which all the unchecked markers id of the last iteration will be written
			open (MARKERSUNCHECKEDLIST,">$markerListsDir\/markersUnchecked.WindowNum_$tmpPart.$currentChromosomName.txt") or logInnerSysError("couldn't open markersEliminated.ofWindowNum_$tmpPart.txt for writing");
			print MARKERSUNCHECKEDLIST "All the markers between - \n";
			my $firstUncheckedMarkerMinusOne = $firstUncheckedMarker -1;
			my $curUncheckedMarkerNumMinusOne = $curUncheckedMarkerNum -1;
			print MARKERSUNCHECKEDLIST "$markersIDStrings[$firstUncheckedMarkerMinusOne]($firstUncheckedMarker) (distance = $dist[$firstUncheckedMarkerMinusOne])\n";
			print MARKERSUNCHECKEDLIST "$markersIDStrings[$curUncheckedMarkerNumMinusOne]($curUncheckedMarkerNum) (distance = $dist[$curUncheckedMarkerNumMinusOne])\n\n";
			
			
			close MARKERSUNCHECKEDLIST;
			my $typeOfReport = "AddUncheckedMarkersToExistReport";
			my $percent = int (100 * $currentDistFromFirstMark / $dist[$numMarkers-1]);
			
			&createAndSendCurrentReportFile($tmpPart,$typeOfReport,$numOfFirstMarkerInTheBigWindow,$numOfLastMarkerInTheCurrentWindow,$percent);
		}
		
		# Placing the last marker of the last window to be at least one marker after the first
		# Notice that we calc its true number in the inner loop
		$numOfLastMarkerInTheCurrentWindow=$numOfFirstMarkerInCurrentWindow+1;
		$numOfMiddleMarkerInTheCurrentWindow=$numOfFirstMarkerInCurrentWindow+1;
		
		
		# This loop is performed until we find markers
		while(	( $foundResult == 0 ) 
				&& 
				($numOfFirstMarkerInCurrentWindow <$numMarkers )
				&&
				($numOfLastMarkerInTheCurrentWindow<=$numMarkers)
			)
		{	
			# Finding the last marker found in the current small window
			while(
			     ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
			      &&
				 ($dist[$numOfLastMarkerInTheCurrentWindow-1] <
				 ($bigWindowSize*($bigIterationNum-1) + $smallWindowSize + $halfSmallWindowSize *$smallIterationNum  ))) 
			{	
				if (
					($numOfMiddleMarkerInTheCurrentWindow==$numOfFirstMarkerInCurrentWindow+1)
					&&
					($dist[$numOfLastMarkerInTheCurrentWindow-1] > ($halfSmallWindowSize + $dist[$numOfFirstMarkerInCurrentWindow-1]) )
					)
					{			 
						$numOfMiddleMarkerInTheCurrentWindow = $numOfLastMarkerInTheCurrentWindow;
					}						
				 $numOfLastMarkerInTheCurrentWindow++;
			}
			
			# Check if the search should continue in the next small window
			# == The first marker is already outside this window.
			if ($dist[$numOfFirstMarkerInCurrentWindow-1] >= ($bigWindowSize*($bigIterationNum-1) + $smallWindowSize + $halfSmallWindowSize *$smallIterationNum  )) 
			{
				$smallIterationNum++;
				next;
			}
			
			# Check if the last loop ended when the marker found outside the window.
			# If so, ajusting his number to be inside the window.
			# Also checking the case in which the loop ended at the begining.
			if(	($numOfLastMarkerInTheCurrentWindow != $numMarkers)
				&&
				($numOfLastMarkerInTheCurrentWindow!=$numOfFirstMarkerInCurrentWindow+1))
			{
			 $numOfLastMarkerInTheCurrentWindow--;	
			}
			
			# If we won't find new markers in the last small window, we will continue our search in steps of half small window size
			$smallIterationNum++;
			
			# Now we have the first marker and the last for the current window, we will perform our search using their numbers -
			# Creating new ped & dat files according to the numOfFirstMarkerInCurrentWindow & numOfLastMarkerInTheCurrentWindow values.
			&createPart($numOfFirstMarkerInCurrentWindow,$numOfLastMarkerInTheCurrentWindow,$workDir,$partNum); 
			
			# Saving the names of those files.
			$currentPedfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow . "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Pedfile }";
			$currentDatfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow. "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Datfile }";
			
			# In oreder to get the correct markers index we will send this parameter to the SNPdistiller
			my $numOfLastMarkerInThePrevWindow = $numOfFirstMarkerInCurrentWindow - 1;
			
			# Running the SNPdistiller on the files		
			system("$SNPdistiller 2 $workDir\/$currentDatfileName $workDir\/$currentPedfileName $checkGenoParam $numOfLastMarkerInThePrevWindow >& SNPdistillerErr.txt"); 
			
			my $exit_value  = $? >> 8; 
			# Check if the distiller ended well
			if ( $exit_value != 0 )
			{
				my $deadSignal = ($? & 127);
				open(ERRORLOG, "SNPdistillerErr.txt");
				my $errMsg = "";
				for $line ( <ERRORLOG> ) 
				{
					$errMsg = $errMsg . &trim($line);
				}
				close ERRORLOG;
				logAndReportError("Error with SNPdistiller runnig on $currentDatfileName\nended with $errMsg","cleaning",$currProName, $currentChromosomName);
			}
			
			
			# We want to add the current err file to the prev errors that we found in the current big window
			system("cat err.txt >> window.$partNum.err.txt");
			
			# Checkning the existence of the SNPdistiller output files
			unless( (-e "clusterAll.dat") && (-e "clusterAll.ped") )
			{			 
				logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName");
			}
			
			# After the distiler run we should send the cleaning report
			$needToSendLastRep = 1;
			
			# Checknig if there are some markers in the output file
			$foundResult = isThereMarkers("clusterAll.dat", $numDiseaseLoci); 
			if( $foundResult == 0 ) 
			{
				
				if ($numOfLastMarkerInTheCurrentWindow!=$numMarkers)
				{
					# If not, searching again in the next smaller chunk in the current window 
					$numOfFirstMarkerInCurrentWindow = $numOfMiddleMarkerInTheCurrentWindow; 
					$numOfMiddleMarkerInTheCurrentWindow = $numOfFirstMarkerInCurrentWindow+1;
					$currentDistFromFirstMark = $dist[$numOfLastMarkerInTheCurrentWindow-1];
					next;
				}
				# Its the end of the global loop
				else
				{
					last;
				}
			}
	
			# Create map file
			&createMappingFile("clusterAll.dat");
			
			system("mv clusterAll.dat $clusterWorkDir\/clusterAll.$partNum.dat");
			system("mv clusterAll.ped $clusterWorkDir\/clusterAll.$partNum.ped");
			 
			# Checking the correctness of the dat file
			checkDataFile( "$clusterWorkDir\/clusterAll.$partNum.dat" , "InnerFile" , 0);
		}

		if ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
		{
			$currentDistFromFirstMark= $dist[$numOfLastMarkerInTheCurrentWindow-1]; 
		}
		else # Its the end of the loop
		{
			$currentDistFromFirstMark= $dist[$numMarkers-1];
		}
		
		if( $foundResult == 0 )  
		{
			$partNum--;
		}
		else # We cluster some markers - we finished with this part
		{
			my $percent = int (100 * $currentDistFromFirstMark / $dist[$numMarkers-1]);
			
			
			# Sending the full file as it is, and giving it all the right head line.
			my $typeOfReport = "SimpleAll";
			&createAndSendCurrentReportFile($partNum,$typeOfReport,$numOfFirstMarkerInTheFirstLittleWindow,$numOfLastMarkerInTheCurrentWindow,$percent);
			
			# Now handeling the clustering "report"
			#######################################
			
			&sendClusterReport( $percent );
			$needToSendLastRep = 0 ;
		}
	}
	
	my $totalParts = $partNum;
	# Check the "mikre kazhe" when we didn't got cluster from the last markers. need to create a cleaning report to them as well.
	if ($needToSendLastRep == 1)
	{
			# Sending the full file as it is,and giving it all the right head line.
			my $typeOfReport = "SimpleAll";
			my $lastMarker = @markersIDStrings;
			$totalParts = $partNum + 1;
			&createAndSendCurrentReportFile($totalParts,$typeOfReport,$numOfFirstMarkerInTheBigWindow,$lastMarker,100);
	}
	
	# Sending the final cleaning report - appending all the files
	my $typeOfReport = "FinalRep";
	&createAndSendCurrentReportFile($totalParts,$typeOfReport,0,0,100);
	&sendClusterReport(100);
	
	system("rm results.txt");
	system("rm err.txt");
	system("rm window.*");
}

################################################################
#		cleaningAllOnly
#
#	Parameters:
#			
#	Description:
#		
################################################################
sub cleaningAllOnly( )
{
	splitLog("Cleaning All");
	my $projDatadir = "$dataDir\/$currProName";
	open(FIXEDDATFILE ,">$projDatadir\/$currentChromosomName.dat" );
	
	my $i=0;
	my $tmp;
	for $i (0..$#datfile)
	{
		$tmp = join(' ', @{$datfile[$i]});
		print FIXEDDATFILE "$tmp";
		print FIXEDDATFILE "\n";
	}	
	close FIXEDDATFILE;
	
	open(FIXEDPEDFILE ,">$projDatadir\/$currentChromosomName.ped" );
	
	$i=0;
	for $i (0..$#pedfile)
	{
		$tmp = join(' ', @{$pedfile[$i]});
		print FIXEDPEDFILE "$tmp";
		print FIXEDPEDFILE "\n";
	}	
	close FIXEDPEDFILE;
	
	my $filesToDist = "$projDatadir\/$currentChromosomName.dat $projDatadir\/$currentChromosomName.ped";
	
	# Letting the site know that cleaning has started
	#################################################
	
	# First opening new file for the first report
	open(CLEANING_LOG,">$workDir\/cleaningReport.start.log") or die logInnerSysError("couldn't open $workDir\/cleaningReport.start.log for writing");
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	print CLEANING_LOG "Cleaning started at $now_string\n";
	close CLEANING_LOG;
	
	# Sending the file
	my $scpCommand ="scp $workDir\/cleaningReport.start.log $remote_user_full:\/$remote_submission_dir\/$currProName\/cleaningReport.start.log";
	my $result = &General::perfomrWithExponentialBackoff($scpCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $scpCommand");
	}
	my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent started -log cleaningReport.start.log";
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $sshCommand");
	}
	splitLog("Cleaning All - start report sent to site");
	
	# Running the SNPdistiller on the recived files	- cleaning only !
	system("$SNPdistiller 2 $filesToDist $checkGenoParam 0 $cleaningParamToDistiler > SNPdistillerOutput.txt"); 
	
	my $exit_value  = $? >> 8; 
	# Check if the distiller ended well
	if ( $exit_value != 0 )
	{
		my $deadSignal = ($? & 127);
		logInnerSysError("Error with SNPdistiller runnig on $filesToDist \nreturn value : $? \nsignal : $deadSignal");
	}	
	
	# we del the output because we have it already in the outputfile ( we didnt want it to trash the runlog ) 
	system("rm SNPdistillerOutput.txt");
	
	# We want to send the site the error report 
	############################################
	
	# First opening new file for the final report
	open(CLEANING_REPORT,">$workDir\/cleaningReport.fin.log") or die logInnerSysError("couldn't open cleaningReport.fin.log for writing");
	
	# now working on the err file
	open(RESFILE,"err.txt") or die logInnerSysError("couldn't open err.txt for reading");
	@lines = <RESFILE>;
	@sorted = sort { ($a =~ /\((\d+)/)[0] <=> ($b =~ /\((\d+)/)[0] } @lines;
	my %seen;
	my @uniqed = grep !$seen{$_}++, @sorted;
	
	print CLEANING_REPORT "\nThis file contains error reports that were found during the calculation on the given files\n\n";
	
	# Copy the sorted data from err file to the report.
	print CLEANING_REPORT "@uniqed";
	
	close RESFILE;
	close CLEANING_REPORT;
	
	# Sending the file
	$scpCommand ="scp $workDir\/cleaningReport.fin.log $remote_user_full:\/$remote_submission_dir\/$currProName\/cleaningReport.fin.log";
	$result = &General::perfomrWithExponentialBackoff($scpCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $scpCommand");
	}
	$sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent 100% -log cleaningReport.fin.log";
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $sshCommand");
	}
	
	splitLog("Cleaning All - final errors report sent to site");
	
	# Checkning the existence of the SNPdistiller output files
	unless( (-e "ok_snpAll.dat") && (-e "ok_snpAll.ped") )
	{			 
		logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName");
	}
	
	# Fixing the numbers of markers in the dat file to be in increasing order
	open (DATFILE , "ok_snpAll.dat");
	open (FIXDATFILE , ">ok_snpAll.dat.fixed");
	my $index =1;
	for $line (<DATFILE>)
	{
		$mapFlag = 0;
		
	    my $tmpLine = $line;
		if ($tmpLine =~ s/\((\d+)\)\#/\# \<\< marker $index/)
		{
			$index++;
		}
		
		print FIXDATFILE $tmpLine;
	}		
	
	system("rm ok_snpAll.dat");
	system("mv ok_snpAll.dat.fixed $workDir\/resultFile.$variables{ Datfile }");
	system("mv ok_snpAll.ped $workDir\/resultFile.$variables{ Pedfile }");	 
	
	system("rm results.txt");
	system("rm err.txt");
	system("rm window.*");
}

################################################################
#		cleaningOnlyUNUSED
#
#	Parameters:
#			
#	Description:
#		
################################################################
sub cleaningOnlyUNUSED( )
{
	splitLog("Cleaning");
	$markerListsDir = "$workDir\/markersLists";
	$cleaningWorkDir = "$workDir\/cleaningWorkDir";
	
	unless( -d $markerListsDir )
	{
		mkdir $markerListsDir or logInnerSysError("couldn't create $markerListsDir");
	}
	unless( -d $cleaningWorkDir )
	{
		mkdir $cleaningWorkDir or logInnerSysError("couldn't create $cleaningWorkDir");
	}
	
	# We will count out distance as we walk along the chromosom
	my $currentDistFromFirstMark=0;
	
	my	$bigWindowSize = $cMorganWindowSize;
	
	# Current cluster file number
	$partNum = 0;
	
	# Parameters for the inner loop:
	my $foundResult = 0; 
	my $result = 0;
	
	# Initialize values with zeros, but surly the scan will start from marker 1
	my $numOfFirstMarkerInCurrentWindow =0; 
	my $numOfLastMarkerInTheCurrentWindow =0;
	
	my $bigIterationNum = 0;
	
	
	# In this loop we will create in each iteration 2 files and run the SNPdistiller on them.
	# The total amount of files is affected by the results of the SNPdistiller - meaning by the presence of markers.
	while( $currentDistFromFirstMark < $totalLenght )  # "Walking" all over the chromosom
	{
	
		# Current cluster file number that will be created in the current iteration
		$partNum++;
		$bigIterationNum++; 
		
		# First placing the first marker of the current window to be after the last marker of the prev window
		$numOfFirstMarkerInCurrentWindow = ($numOfLastMarkerInTheCurrentWindow +1);
	
		# Placing the last marker of the last window to be at least one marker after the first
		# Notice that we calc its true number in the inner loop
		$numOfLastMarkerInTheCurrentWindow=$numOfFirstMarkerInCurrentWindow+1;
		
		# Finding the last marker found in the current window
		while(
			 ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
			  &&
			 ($dist[$numOfLastMarkerInTheCurrentWindow-1] < ($bigWindowSize*($bigIterationNum-1)))) 
		{	
			$numOfLastMarkerInTheCurrentWindow++;
		}
		
		# Check if the last loop ended when the marker found outside the window.
		# If so, ajusting his number to be inside the window.
		# Also checking the case in which the loop ended at the begining.
		if(	($numOfLastMarkerInTheCurrentWindow != $numMarkers)
			&&
			($numOfLastMarkerInTheCurrentWindow!=$numOfFirstMarkerInCurrentWindow+1))
		{
		 $numOfLastMarkerInTheCurrentWindow--;	
		}
		
		# Now we have the first marker and the last for the current window, we will perform our search using their numbers -
		# Creating new ped & dat files according to the numOfFirstMarkerInCurrentWindow & numOfLastMarkerInTheCurrentWindow values.
		&createPart($numOfFirstMarkerInCurrentWindow,$numOfLastMarkerInTheCurrentWindow,$workDir,$partNum); 
		
		# Saving the names of those files.
		$currentPedfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow . "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Pedfile }";
		$currentDatfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow. "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Datfile }";
		
		# In oreder to get the correct markers index we will send this parameter to the SNPdistiller
		my $numOfLastMarkerInThePrevWindow = $numOfFirstMarkerInCurrentWindow - 1;
		
		# Running the SNPdistiller on the files	- cleaning only !
		system("$SNPdistiller 2 $workDir\/$currentDatfileName $workDir\/$currentPedfileName $checkGenoParam $numOfLastMarkerInThePrevWindow $cleaningParamToDistiler > SNPdistillerOutput.txt"); 
		
		my $exit_value  = $? >> 8; 
		# Check if the distiller ended well
		if ( $exit_value != 0 )
		{
			my $deadSignal = ($? & 127);
			logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName \nreturn value : $? \nsignal : $deadSignal");
		}
		
		# we del the output because we have it already in the outputfile ( we didnt want it to trash the runlog ) 
		system("rm SNPdistillerOutput.txt");
		
		# We want to add the current err file to the prev errors that we found in the current big window
		system("cat err.txt >> window.$partNum.err.txt");
		
		# Checkning the existence of the SNPdistiller output files
		unless( (-e "ok_snpAll.dat") && (-e "ok_snpAll.ped") )
		{			 
			logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName");
		}
		
		system("mv ok_snpAll.dat $cleaningWorkDir\/ok_snpAll.$partNum.dat");
		system("mv ok_snpAll.ped $cleaningWorkDir\/ok_snpAll.$partNum.ped");
		 
		# Checking the correctness of the dat file
		# checkDataFile( "$cleaningWorkDir\/ok_snpAll.$partNum.dat" , "InnerFile" );
		

		if ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
		{
			$currentDistFromFirstMark= $dist[$numOfLastMarkerInTheCurrentWindow-1]; 
		}
		else # Its the end of the loop
		{
			$currentDistFromFirstMark= $dist[$numMarkers-1];
		}
		
		my $percent = int (100 * $currentDistFromFirstMark / $dist[$numMarkers-1]);
			
		# Sending the full file as it is, and giving it all the right head line.
		my $typeOfReport = "SimpleAll";
		&createAndSendCurrentReportFile($partNum,$typeOfReport,$numOfFirstMarkerInCurrentWindow,$numOfLastMarkerInTheCurrentWindow,$percent);
	}

	# Sending the final cleaning report - appending all the files
	my $typeOfReport = "FinalRep";
	&createAndSendCurrentReportFile($partNum,$typeOfReport,0,0,100);
		
	system("rm results.txt");
	system("rm err.txt");
	system("rm window.*");
}

################################################################
#		clusteringOnly
#
#	Parameters:
#	$_[0] - boolean param, indicate if the current files are raw. if so,
#			then we need to run mandelian error check on them as well.
#			( We will do it by the $clusteringParamToDistiler param to the distiler )
#			
#	Description:
#		
################################################################
sub clusteringOnly( $ )
{
	my $isRaw = $_[0];
	
	splitLog("Clustering");
	$clusterWorkDir =  "$workDir\/clustering";
	$markerListsDir =  "$workDir\/markersLists";
	
	unless( -d $clusterWorkDir )
	{
		mkdir $clusterWorkDir or logInnerSysError("couldn't create $clusterWorkDir");
	}
	unless( -d $markerListsDir )
	{
		mkdir $markerListsDir or logInnerSysError("couldn't create $markerListsDir");
	}
	
	my $avrageDistBetweenMarkers = ($dist[$numMarkers-1]/$numMarkers); # In order to calc suitble small window size
	my $smallWindowSize = ($avrageDistBetweenMarkers * 50); # We want to handle maximum 50 markers in one small window
	my $halfSmallWindowSize = ($smallWindowSize/2); # Dist in centimorgan
	
	# We dont want out small window will be big. so we make a upper bound for him -
	# at most half of big window size
	if($smallWindowSize > ($cMorganWindowSize/2)) 
	{ 
		$smallWindowSize = $cMorganWindowSize/2; 
		$halfSmallWindowSize = $smallWindowSize/2; 
	}
	
	# We will count out distance as we walk along the chromosom
	my $currentDistFromFirstMark=0;
	
	# my	$bigWindowSize = $cMorganWindowSize;
	my      $bigWindowSize = 2 * $cMorganWindowSize;

	# Current cluster file number
	$partNum = 0;
	
	# Parameters for the inner loop:
	my $foundResult = 0; 
	my $result = 0;
	
	# Initialize values with zeros, but surly the scan will start from chromosom 1
	my $numOfFirstMarkerInCurrentWindow =0; 
	my $numOfLastMarkerInTheCurrentWindow =0;
	my $numOfMiddleMarkerInTheCurrentWindow=0;
	my $numOfFirstMarkerInTheBigWindow =0; # For report perpose
	
	my $smallIterationNum=0;
	my $bigIterationNum = 0;
	
	my $tmpPart=0; # see the handle to unchecked markers creation
	my $needToSendLastRep =0;
	
	# In this loop we will create in each iteration 2 files and run the SNPdistiller on them.
	# The total amount of files is affected by the results of the SNPdistiller - meaning by the presence of markers.
	while( $currentDistFromFirstMark < $totalLenght )  # "Walking" all over the chromosom
	{
	
		# Current cluster file number that will be created in the current iteration
		$partNum++;
		$bigIterationNum++;
		$foundResult = 0;  
		$smallIterationNum=0;   
		
		# First placing the first marker of the current window to be after the last marker of the prev window
		# We do that in order to save some steps in the next loop
		$numOfFirstMarkerInCurrentWindow = ($numOfLastMarkerInTheCurrentWindow +1);
		$numOfFirstMarkerInTheBigWindow = $numOfFirstMarkerInCurrentWindow;
		
		my $curUncheckedMarkerNum 	=0;
		my $haveUncheckedMarker 	=0;
		my $firstUncheckedMarker 	=0;
		
		# Now finding the first marker placed in the current big window
		while	(
				($numOfFirstMarkerInCurrentWindow<$numMarkers) 
				&& 
				($dist[$numOfFirstMarkerInCurrentWindow-1] < ($bigWindowSize * ($bigIterationNum-1)) ) 
				) 
		{
			if ($partNum != 1)
			{
				# Updating the last unchecked marker ID  
				$curUncheckedMarkerNum = $numOfFirstMarkerInCurrentWindow;
				
				# Saving the first unchecked marker
				if ($haveUncheckedMarker == 0)
				{
					$firstUncheckedMarker = $curUncheckedMarkerNum
				}
				$haveUncheckedMarker = 1;
				
			}
			$numOfFirstMarkerInCurrentWindow++;
		}
		
		# Placing the last marker of the last window to be at least one marker after the first
		# Notice that we calc its true number in the inner loop
		$numOfLastMarkerInTheCurrentWindow=$numOfFirstMarkerInCurrentWindow+1;
		$numOfMiddleMarkerInTheCurrentWindow=$numOfFirstMarkerInCurrentWindow+1;
		
		
		# This loop is performed until we find markers
		while(	( $foundResult == 0 ) 
				&& 
				($numOfFirstMarkerInCurrentWindow <$numMarkers )
				&&
				($numOfLastMarkerInTheCurrentWindow<=$numMarkers)
			)
		{	
			# Finding the last marker found in the current small window
			while(
			     ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
			      &&
				 ($dist[$numOfLastMarkerInTheCurrentWindow-1] <
				 ($bigWindowSize*($bigIterationNum-1) + $smallWindowSize + $halfSmallWindowSize *$smallIterationNum  ))) 
			{	
				if (
					($numOfMiddleMarkerInTheCurrentWindow==$numOfFirstMarkerInCurrentWindow+1)
					&&
					($dist[$numOfLastMarkerInTheCurrentWindow-1] > ($halfSmallWindowSize + $dist[$numOfFirstMarkerInCurrentWindow-1]) )
					)
					{			 
						$numOfMiddleMarkerInTheCurrentWindow = $numOfLastMarkerInTheCurrentWindow;
					}						
				 $numOfLastMarkerInTheCurrentWindow++;
			}
			
			# Check if the search should continue in the next small window
			# == The first marker is already outside this window.
			if ($dist[$numOfFirstMarkerInCurrentWindow-1] >= ($bigWindowSize*($bigIterationNum-1) + $smallWindowSize + $halfSmallWindowSize *$smallIterationNum  )) 
			{
				$smallIterationNum++;
				next;
			}
			
			# Check if the last loop ended when the marker found outside the window.
			# If so, ajusting his number to be inside the window.
			# Also checking the case in which the loop ended at the begining.
			if(	($numOfLastMarkerInTheCurrentWindow != $numMarkers)
				&&
				($numOfLastMarkerInTheCurrentWindow!=$numOfFirstMarkerInCurrentWindow+1))
			{
			 $numOfLastMarkerInTheCurrentWindow--;	
			}
			
			# If we won't find new markers in the last small window, we will continue our search in steps of half small window size
			$smallIterationNum++;
			
			# Now we have the first marker and the last for the current window, we will perform our search using their numbers -
			# Creating new ped & dat files according to the numOfFirstMarkerInCurrentWindow & numOfLastMarkerInTheCurrentWindow values.
			&createPart($numOfFirstMarkerInCurrentWindow,$numOfLastMarkerInTheCurrentWindow,$workDir,$partNum); 
			
			# Saving the names of those files.
			$currentPedfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow . "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Pedfile }";
			$currentDatfileName = "part" . $partNum . "_from_" . $numOfFirstMarkerInCurrentWindow. "_" . "$numOfLastMarkerInTheCurrentWindow.$variables{ Datfile }";
			
			# In oreder to get the correct markers index we will send this parameter to the SNPdistiller
			my $numOfLastMarkerInThePrevWindow = $numOfFirstMarkerInCurrentWindow - 1;
			
			# Running the SNPdistiller on the files	, according to there status (raw = Yes/No)
			# (Raw files need to be checked first)
			if ($isRaw == 0)
			{
				system("$SNPdistiller 2 $workDir\/$currentDatfileName $workDir\/$currentPedfileName $checkGenoParam $numOfLastMarkerInThePrevWindow $clusteringParamToDistiler > SNPdistillerErr.txt"); 
			}
			else
			{	
				system("$SNPdistiller 2 $workDir\/$currentDatfileName $workDir\/$currentPedfileName $checkGenoParam $numOfLastMarkerInThePrevWindow $clusteringRawParamToDistiler > SNPdistillerErr.txt"); 
			}
			
			my $exit_value  = $? >> 8; 
			# Check if the distiller ended well
			if ( $exit_value != 0 )
			{
				my $deadSignal = ($? & 127);
				open(ERRORLOG, "SNPdistillerErr.txt");
				my $errMsg = "";
				for $line ( <ERRORLOG> ) 
				{
					$errMsg = $errMsg . &trim($line);
				}
				close ERRORLOG;
				logAndReportError("Error with SNPdistiller runnig on $currentDatfileName\nended with $errMsg","clustering",$currProName, $currentChromosomName);
			}
			
			
			# Checkning the existence of the SNPdistiller output files
			unless( (-e "clusterAll.dat") && (-e "clusterAll.ped") )
			{			 
				logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName");
			}
			
			# After the distiler run we should send the cleaning report
			$needToSendLastRep = 1;
			
			# Checknig if there are some markers in the output file
			$foundResult = isThereMarkers("clusterAll.dat", $numDiseaseLoci); 
			if( $foundResult == 0 ) 
			{
				
				if ($numOfLastMarkerInTheCurrentWindow!=$numMarkers)
				{
					# If not, searching again in the next smaller chunk in the current window 
					$numOfFirstMarkerInCurrentWindow = $numOfMiddleMarkerInTheCurrentWindow; 
					$numOfMiddleMarkerInTheCurrentWindow = $numOfFirstMarkerInCurrentWindow+1;
					$currentDistFromFirstMark = $dist[$numOfLastMarkerInTheCurrentWindow-1];
					next;
				}
				# Its the end of the global loop
				else
				{
					last;
				}
			}
			
			# Create map file
			&createMappingFile("clusterAll.dat" );
			
			system("mv clusterAll.dat $clusterWorkDir\/clusterAll.$partNum.dat");
			system("mv clusterAll.ped $clusterWorkDir\/clusterAll.$partNum.ped");
			 
			# Checking the correctness of the dat file
			checkDataFile( "$clusterWorkDir\/clusterAll.$partNum.dat" , "InnerFile" , 0);
		}

		if ($numOfLastMarkerInTheCurrentWindow<$numMarkers)
		{
			$currentDistFromFirstMark= $dist[$numOfLastMarkerInTheCurrentWindow-1]; 
		}
		else # Its the end of the loop
		{
			$currentDistFromFirstMark= $dist[$numMarkers-1];
		}
		
		if( $foundResult == 0 )  
		{
			$partNum--;
		}
		else # We cluster some markers - we finished with this part
		{
			my $percent = int (100 * $currentDistFromFirstMark / $dist[$numMarkers-1]);
			
			# Now handeling the clustering "report"
			#######################################
			
			&sendClusterReport( $percent );
			$needToSendLastRep = 0 ;
		}
	}
	
	&sendClusterReport(100);
	
	system("rm results.txt");
}

################################################################
#		joinFiles
#
#	Parameters:
#			$_[0] - array of distBetweenParts
#			$_[1] - file name reisha (before part number)
#			$_[2] - number of parts
#			$_[3] - number of disease Loci
#	Description:
#		Put it back together 
################################################################
sub joinFiles()
{
	my $numDiseaseLoci = pop @_;
	my $partNum = pop @_;
	my $fileNameReisha = pop @_; #"clusterAll";
	my @distBetweenParts = @_;
	
	my (@newPedlines);   
	
	for $i (1..$partNum)
	{
		my ($line);
		open(CURRENTPED,"$fileNameReisha.$i.ped") or &DAGSplit::logInnerSysError("couldn't open $fileNameReisha.$i.ped for reading");
		if( $i == 1 )
		{	#starting file arrays
			$j = 0;
			for $line (<CURRENTPED>) {
	        	$newPedlines[$j] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
				$j++;
			}	
		}
		else
		{
			$j = 0;
			for $line (<CURRENTPED>)
			{
				my (@tmp) = split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line));
				for $k (10..$#tmp)
				{
					$newPedlines[$j][$#{$newPedlines[$j]}+1] = $tmp[$k];			
				}
				$j++;
			}
		}
		close CURRENTPED;
		#system("rm $fileNameReisha.$i.ped");
	}

	 my (@newDatlines,@newDatlastlines,$inMarkersFlag,$markerFirstLineFlag,$endLinesFlag);
	 for $i (1..$partNum)
	 {
		my ($line);
		my ($lineNum) = 0;
		open(CURRENTDAT,"$fileNameReisha.$i.dat") or &DAGSplit::logInnerSysError("couldn't open $fileNameReisha.$i.dat for reading");
		$inMarkersFlag = 0;
		$markerFirstLineFlag = 1;
		$endLinesFlag = 0;
		if( $i == 1 )
		{	#starting file arrays
			for $line (<CURRENTDAT>) 
			{
				$lineNum++;
				$line =~ s/<< RECOMBINATION VALUES//g;
				my (@tmp) = split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line));
				if( $inMarkersFlag == 0 && $tmp[0] eq 3  && $lineNum > 1 )
				{ #last check is for special case when numLoci = 3
					$inMarkersFlag = 1;
				}
				if( $inMarkersFlag == 0 )
				{
					$newDatlines[$#newDatlines+1] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
				}
				else
				{
					if( $endLinesFlag == 1)
					{
						$newDatlastlines[$#newDatlastlines+1] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
					}
					elsif( $markerFirstLineFlag == 1 && $tmp[0] != 3)
					{
						$endLinesFlag = 1;
						$newDatlastlines[$#newDatlastlines+1] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
					}
					else
					{
						if( $markerFirstLineFlag == 1 )
						{
							$markerFirstLineFlag = 0;
						}
						else
						{
							$markerFirstLineFlag = 1;
						}
						$newDatlines[$#newDatlines+1] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
					}
				}
			}
		}
		else
		{
			for $line (<CURRENTDAT>) 
			{
				$lineNum++;
				$line =~ s/<< RECOMBINATION VALUES//g;
				my (@tmp) = split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line));
				if( $lineNum == 1 )
				{
					$newDatlines[0][0] += $tmp[0] - $numDiseaseLoci;
				}
				if( $inMarkersFlag == 0 && $tmp[0] eq 3 && $lineNum > 1 )
				{ #last check is for special case when numLoci = 3
					$inMarkersFlag = 1;
				}
				if( $inMarkersFlag == 1 )
				{
					if( $endLinesFlag > 0 )
					{
						$endLinesFlag++;
						if( $endLinesFlag == 2)
						{
							for $j (0..$#tmp)
							{
								if( $j == 0 )
								{
									$newDatlastlines[1][$#{$newDatlastlines[1]}+1] = $distBetweenParts[$i-1]; #this need to be calculated better (distance between files)
								}
								else{
									if ($tmp[$j] =~ /\d+/) {
										$newDatlastlines[1][$#{$newDatlastlines[1]}+1] = $tmp[$j];
									}
								}
							}
						}
					}			
					elsif( $markerFirstLineFlag == 1 && $tmp[0] != 3)
					{
						$endLinesFlag = 1;
					}
					else{
						if( $markerFirstLineFlag == 1 )
						{
							$markerFirstLineFlag = 0;
						}
						else{
							$markerFirstLineFlag = 1;
						}
						$newDatlines[$#newDatlines+1] = [ split(/[ \t\n\r\s\f]+/,&DAGSplit::trim($line)) ];
					}
				}
			}	
		}
		close CURRENTDAT;	
		#system("rm $fileNameReisha.$i.dat");
	}
	#repair a line in the datfile
	for $i( 1..$newDatlines[0][0] )
	{
		$newDatlines[2][$i-1] = $i;
	}
	$#{$newDatlines[2]} = $newDatlines[0][0] - 1;
	
	
	open(FINALDAT,">resultFile.datafile.dat") or &DAGSplit::logInnerSysError("couldn't open resultFile.datafile.dat for writing");
	open(FINALPED,">resultFile.pedfile.dat") or &DAGSplit::logInnerSysError("couldn't open resultFile.pedfile.dat for writing");
	#write files
	for $i (0..$#newDatlines)
	{
		$tmp = join(' ', @{$newDatlines[$i]});
		
		#################################
		if ($tmp =~ /3.*#/) {
			@words = split /<</, $tmp;
			$tmp = $words[0]." << ALLELE NUMBERS, NO. OF ALLELES";
		}
		#################################
		
		print FINALDAT "$tmp";
		#for Hadar - should be removed
		if( $i == 2)
		{
			print FINALDAT " << NO. OF ALLELES ";
		}
		#
		print FINALDAT "\n";
	}
	for $i (0..$#newDatlastlines)
	{
		$tmp = join(' ', @{$newDatlastlines[$i]});
		#####################################
		if ($i == $#newDatlastlines - 2) {
			@words = split /<</, $tmp;
			$tmp = $words[0]." << RECOMBINATION VALUES";
		}
		#################################
		if ($tmp =~ /-n/ ) {
			$tmp = "-n 2 1 ".($newDatlines[0][0] - $numDiseaseLoci);
		}
		print FINALDAT "$tmp";
		print FINALDAT "\n";
	}		
	for $i (0..$#newPedlines)
	{
		$tmp = join(' ', @{$newPedlines[$i]});
		print FINALPED "$tmp";
		print FINALPED "\n";
	}	
	close FINALDAT;
	close FINALPED;
	
	return;
}

# Log report functions
######################

################################################################
#		splitLog
#
#	Parameters:
#					$_[0] - the message to be reported
#	Returns:
#					 reports a stage in the running of the script
################################################################
sub splitLog( $ )
{
	if ($Environment::SHOULD_LOG_MESSAGES) {
		my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		my ($report) = $_[0];
		open( SPLITLOG , ">>$splitRunLog") or logInnerSysError("couldn't open $splitRunLog for writing");;
		print SPLITLOG "PID $$ :\n$report\nat $now_string\n";
		close SPLITLOG;
	}
}

							#########################################
							#	Auxiliary functions Implementation  #
							#########################################

###########################################################################
#		checkDataFile
#
#	Parameters:
#					$_[0] - path of datfile
#					$_[1] - string with "FirstDatFile" or "InnerFile", 
#							for initialization of "@markersIDStrings"
#					$_[2] - 1 if the files were created from snp source and 0 otherwise
#	Returns:
#					reports to the badfile log if file is in bad format
#					returns 0 if everything is good. 1 if something is bad.
############################################################################
sub checkDataFile( $ $ $ )
{
	#checking datfile

	my $datafileName = $_[0];
	my $typeOfCheckFile = $_[1];
	
	my $snpSource = 0;
	if (defined($_[2])) {
		$snpSource = $_[2];
	}
	
	my @datfile;
	my $tmpStringOfID;
	
	#reading file
	open (DATFILE,"$datafileName") or logInnerSysError("couldn't open $datafileName for reading");

	my ($i,$j,$line,$numDiseaseLoci,$numOfLinesInFile);
	
	$i = 0;
	for $line (<DATFILE>) 
	{
        $datfile[$i] = [ split(/[ \t\n\r\s\f]+/,&trim($line)) ];
		#removing comments
		for $j (0..$#{$datfile[$i]}){
			if( $datfile[$i][$j] =~ /^<</ )
			{
				$#{$datfile[$i]} = $j - 1;			
				last;
			}	
			elsif ( $datfile[$i][$j] =~ /^#/ )
			{
					# kind of "Piggybacking" - initialize "@markersIDStrings" on the run 
					# Only if the file is the original dat.
					if ($typeOfCheckFile eq "FirstDatFile")
					{	
						$tmpStringOfID=$datfile[$i][$j];
						$tmpStringOfID =~ s/#//g ;
						push(@markersIDStrings,$tmpStringOfID); 
					}	
					
				$#{$datfile[$i]} = $j - 1;			
				last;
			}			
		}
		$i++;
	}	
	$numOfLinesInFile=$i;
	
	#removing empty lines
	while(1)
	{
		if( $#{$datfile[$#datfile]} < 1 )
		{
			$#datfile = $#datfile - 1;
		}
		else
		{
			last;
		}	
	}	
	
	#checking first line	
	$firstLineArgsNum = $#{$datfile[0]};
	if( $firstLineArgsNum != 3 ){
		if( $firstLineArgsNum != 4 )
		{
			fileLog("the number of arguments in the first line is invalid",$typeOfCheckFile);
			return -1;
		}
		if( $datfile[0][4] != 0 && $datfile[0][4] != 2 )
		{
			fileLog("NoComplexAffectionLoci field is invalid" , $typeOfCheckFile);
			return -1;
		}		
	}
	if( $datfile[0][0] < 1 )
	{
		fileLog("NumLoci is invalid",$typeOfCheckFile);
		return -1;
	}
	if( $datfile[0][2] != 0  && $datfile[0][2] != 1 )
	{
		my $val = $datfile[0][2];
		fileLog("SexLinked field is invalid $val",$typeOfCheckFile);
		return -1;
	}	
	if( $datfile[0][3] != 8  )
	{
		fileLog("Program code should be 8 for now",$typeOfCheckFile);
		return -1;
	}	
	#checking second line
	if( $#{$datfile[1]} != 3 )
	{
		fileLog("second line is invalid",$typeOfCheckFile);
		return -1;
	}
	#checking third line
	if( $#{$datfile[2]} != ($datfile[0][0] - 1) )
	{
		fileLog("size of third line is invalid",$typeOfCheckFile);
		return -1;
	}
	for $i ( 1 .. $datfile[0][0] )
	{	
		if( $datfile[2][$i-1] < 1 || $datfile[2][$i-1] > $datfile[0][0] )
		{
			fileLog("field $i in the third line is invalid",$typeOfCheckFile);
			return -1;
		}
	}
	#checking disease markers
	#checking will be added later - for now we'll jump to check the other markers
	$lineNum = 4;
	#while($datfile[$lineNum-1][0] != 3 )
	while (($datfile[$lineNum-1][0] != 3) || (! defined ($datfile[$lineNum-1][1])) || ($datfile[$lineNum-1][1] !~ /\d+/))
	{
		$lineNum++;
		if ($lineNum == $numOfLinesInFile)
		{
			fileLog("Cant find the first 3 in $datafileName",$typeOfCheckFile);
			return -1;
		}
	}
	
	if( $#{$datfile[0]} == 3)
	{
		$numDiseaseLoci = 1;
	}
	elsif( $datfile[0][4] == 0 )
	{
		$numDiseaseLoci = 1;
	}
	else
	{
		$numDiseaseLoci = 2;
	}
	my $numMarkers = $datfile[0][0] - $numDiseaseLoci;
	
	#checking markers
	for $i ( 1 .. $numMarkers ){
		if( $#{$datfile[$lineNum-1]} != 1 )
		{
			fileLog("first line describing markers should have 2 fields",$typeOfCheckFile);
			return -1;			
		}
		if( $datfile[$lineNum-1][0] != 3 )
		{
			fileLog("LociType should be 3 in marker $i in line $lineNum. maybe not enough markers",$typeOfCheckFile);
			return -1;
		}
		$numAlleles = $datfile[$lineNum-1][1];
		if( $numAlleles < 1 )
		{ #might change later to see only 2 alleles
			fileLog("NumAlleles is invalid $numAlleles in marker $i in line $lineNum",$typeOfCheckFile);
			return -1;
		}
		
		#######################################################################################################
		# For now we don't accept dat file with such markers
		if (($snpSource == 1) && ($numAlleles > 2) && ($typeOfCheckFile eq "FirstDatFile" ) )
		{
		 	fileLog("NumAlleles is $numAlleles - bigger than 2 , in marker $i in line $lineNum",$typeOfCheckFile);	
			return -1;
		}
		########################################################################################################
		
		$lineNum++;			
		$numFreqFields = $#{$datfile[$lineNum-1]} + 1;
		if( $numAlleles != $numFreqFields )
		{ #might change later to see only 2 alleles
			fileLog("number of frequencies $numFreqFields doesnt match NumAlleles $numAlleles  is invalid $numAlleles  in marker $i in line $lineNum",$typeOfCheckFile);	
			return -1;
		}
		
		#Check that th freq sum is 1
		$freqSum = 0;
		for $f (0..$#{$datfile[$lineNum-1]}) {
			$freqSum = $freqSum + $datfile[$lineNum-1][$f];
		}
		
		if (($freqSum < 0.9999) || ($freqSum > 1.0001)) {
			fileLog("The sum of the gene frequencies : $freqSum , don't sum to 1.0 in marker $i in line $lineNum at file $datafileName. line is ".$datfile[$lineNum-1],$typeOfCheckFile);
		}
		
		$lineNum++;	
	}
	#checking last lines
	if( $#{$datfile[$lineNum-1]} != 1 )
	{
		fileLog("number of fields in line $lineNum should be 2",$typeOfCheckFile);	
		return -1;
	}
	$lineNum++;	
	#checking thetas
	$numThetaFields = $#{$datfile[$lineNum-1]} + 1;
	$thetaFieldsNeeded = $datfile[0][0] - 1;
	if( $numThetaFields != $thetaFieldsNeeded )
	{
		fileLog("number of thetas is $numThetaFields. should be $thetaFieldsNeeded",$typeOfCheckFile);	
		return -1;
	}
	$lineNum++;	
	
	return 0;
}

###########################################################################
#		checkPedFile
#
#	Parameters:
#					$_[0] - path of pedfile
#					$_[1] - number of markers in the ped (according to the dat file)
#					$_[2] - number of liability classes
#					$_[3] - string with "FirstDatFile" or "InnerFile", 
#							for fileLog error report
#	Returns:
#					reports to the badfile log if file is in bad format
#					returns 0 if everything is good. 1 if something is bad.
############################################################################
sub checkPedFile( $ $ $ $ )
{
	my $line = 0;
	my $i = 0;
	my @pedfile;
	my $numMarkers = $_[1];
	my $numberOfLiabilityClasses = $_[2];
	my $typeOfCheckFile = $_[3];
	my %hashOfIndividualNumbers =();
	my $firstLineNumOfCurrentPed = 1; 
	my $currentLineNum = 1;  
	my $numberOfLinesInLastPed = 0; 
	
	
	open (PEDFILE,"$_[0]") or logInnerSysError("couldn't open $_[0] for reading");

	for $line (<PEDFILE>) 
	{
        #$line =~ s/<<//g;
		@lineData = split /<</, $line;
		$line = $lineData[0];
		
		$pedfile[$i] = [ split(/[ \t\n\r\s\f]+/,&trim($line)) ];
		$i++;
	}	
	 	
	#removing empty lines
	while(1)
	{
		if( $#{$pedfile[$#pedfile]} < 1 )
		{
			$#pedfile = $#pedfile - 1;
		}
		else
		{
			last;
		}	
	}
	
	my $currMaxIndividualNumber = $pedfile[0][1];
	my $currentPedNum = $pedfile[0][0];
	
	# Moves along the lines of the ped file, 
	# get The max index for each ped found in the file, and checking that it equals the number of lines belonging to that ped,
	# meaning that the people in that ped have index from the right range
	for $currentLineNum ( 1 .. $#pedfile+1 )
	{
		####################################
		# Global correctness tests section #
		####################################
		
		# Check if the line size is proper according to the number of markers found in it
		if( $#{$pedfile[$currentLineNum-1]} != (2*$numMarkers+9) + (($numberOfLiabilityClasses > 1) ? 1 : 0) )
		{
			fileLog("line size is invalid in line $currentLineNum in pedfile",$typeOfCheckFile);	
			return -1;
		}
		
		# Check if we move to another ped
		if( $pedfile[$currentLineNum-1][0] != $currentPedNum )
		{
			
			#################################################
			# Check the last ped correctness 
			if ( ($currMaxIndividualNumber != $numberOfLinesInLastPed) || ( keys( %hashOfIndividualNumbers ) != $numberOfLinesInLastPed) )
			{
				fileLog("Individual numbers error in ped number $currentPedNum",$typeOfCheckFile);
				return -1;
			}
			#################################################
			
			# initializing the hash
			%hashOfIndividualNumbers = ();
			
			# Update the first line number of the new ped
			$numberOfLinesInLastPed = 1;
			
			# Update the current ped number
			$currentPedNum = $pedfile[$currentLineNum-1][0];
			
			# Update the current max individual number to be the index of the first one in the ped
			$currMaxIndividualNumber=$pedfile[$currentLineNum-1][1];	
		}
		else
		{
			$numberOfLinesInLastPed ++;
		}
		
		#############################################
		# check wrong indexing of people in the ped #
		#############################################
		
		# insert the current number to the hash (number must be bigger that 0)
		if ($pedfile[$currentLineNum-1][1] > 0 )
		{
			$hashOfIndividualNumbers{$pedfile[$currentLineNum-1][1]}=$pedfile[$currentLineNum-1][1];
		}
		
		# Check if we found bigger index in the current ped
		if (( $pedfile[$currentLineNum-1][1] > $currMaxIndividualNumber ) && ( $pedfile[$currentLineNum-1][0] == $currentPedNum ))
		{
			$currMaxIndividualNumber = $pedfile[$currentLineNum-1][1] ;				 	
		}
		
		#######################################################################
		# Test for Chars inside the data part of the ped (instead of numbers) #
		#######################################################################
		my $numberOfCurrentColumn = 1;
		for $numberOfCurrentColumn ( 1 .. $#{$pedfile[$currentLineNum-1]}+1 )
		{
			unless ($pedfile[$currentLineNum-1][$numberOfCurrentColumn-1] =~ /\d+/)
			{
				fileLog("Bad value - $pedfile[$currentLineNum-1][$numberOfCurrentColumn-1] in line $currentLineNum column $numberOfCurrentColumn",$typeOfCheckFile);
				return -1;
			}
		}
		
	}
	
	###################################
	# finish the wrong indexing check #
	###################################
	if ( ($currMaxIndividualNumber != $numberOfLinesInLastPed) || ( keys( %hashOfIndividualNumbers ) != $numberOfLinesInLastPed) )
	{
		fileLog("Individual numbers error in ped number $currentPedNum",$typeOfCheckFile);
		return -1;
	}
	
	return 0;
}

################################################################################
#		createPart
#
#	Parameters:
#				$_[0] - Start of the chunk size
#			  	$_[1] - End of the chunk size
#				$_[2] - Work directory 
#				$_[3] - Current part num
#	Description:
#	  Creating new ped & dat files according to the smallStart & smallEnd values. 
#################################################################################
sub createPart( $ $ $ $ )
{
	my ($startMarker,$endMarker,$currentPedfileName,$currentDatfileName,$i,$j,$datfileMarkersStartline,$newNumLoci);
	my ($currentDatfileMarkersStartline,$currentDatfileMarkersEndline,$tmp,$flag);
	$startMarker = $_[0];
	$endMarker = $_[1];
	my $currentWorkDir = $_[2];
	my $partNum = $_[3];

	# Creating for the new ped & dat files two new file names, according to the chunk size. 
	$currentPedfileName = "part" . $partNum . "_from_" . $startMarker . "_" . "$endMarker.$variables{ Pedfile }";
	$currentDatfileName = "part" . $partNum . "_from_" . $startMarker . "_" . "$endMarker.$variables{ Datfile }";
	
	open (CURRENTPED,">$currentWorkDir\/$currentPedfileName")  or logInnerSysError("couldn't open $currentWorkDir\/$currentPedfileName for writing");
	open (CURRENTDAT,">$currentWorkDir\/$currentDatfileName")  or logInnerSysError("couldn't open $currentWorkDir\/$currentDatfileName for writing");

	# Creating the ped file
	#######################
	
	# Moving along each line in the original ped file
	for $i (0..$#pedfile)
	{
		# From each line taking the first 10 cells (the ped itself) and then the information 
		# starting at 8+2*$startMarker and ending at 9+2*$endMarker column. for example - the first file will be
		# something like - pedfile[0..14][0..9,10..89] , which is 10 columns of the ped and 80 for the information
		for $j (0..9,8+2*$startMarker..9+2*$endMarker)
		{
			print CURRENTPED "$pedfile[$i][$j] ";
		}
		print CURRENTPED "\n";
	}
	close CURRENTPED;

	
	# Creating the dat file
	#######################
	
	# do some changes to first datfile lines
	$newNumLoci = $endMarker - $startMarker + 1 + $numDiseaseLoci;
	$datfile[0][0] = $newNumLoci;
	
	#loci in order at beggining
	for $i (0..$newNumLoci-1)
	{	
		$datfile[2][$i] = $i+1;	
	}
	$#{$datfile[2]} = $newNumLoci - 1;
	
	# do some changes to last datfile line
	$datfile[$#datfile][1] = 2;
	$datfile[$#datfile][2] = 1;
	$datfile[$#datfile][3] = $newNumLoci - 1;
	
	# Finding the first line in the dat file that represent a marker
	$flag = 2;
	while ($flag != 0)
	{
		$flag++;
		if( $datfile[$flag][0] == 3 )
		{
			$datfileMarkersStartline = $flag;
			$flag = 0;
		}				
	}

	# Calculating the start & end lines of the current chunk (in order to copy the appropriate markers)
	$currentDatfileMarkersStartline = $datfileMarkersStartline + 2*($startMarker-1);
	$currentDatfileMarkersEndline = $datfileMarkersStartline + 2*($endMarker-1) + 1;

	# Copy the first lines that represent the information about the marked found in the file,
	# then copy the window - aka current markers chunk.
	for $i (0..$datfileMarkersStartline-1,$currentDatfileMarkersStartline..$currentDatfileMarkersEndline)
	{
		$tmp = join(' ', @{$datfile[$i]});
		print CURRENTDAT "$tmp";
		print CURRENTDAT "\n";
	}	
	
	# Updating the dat file according to the current program code.
	if( $programCode == 4)
	{
	 	$numEndLines = 3; 
	}
	elsif( $programCode == 8)
	{
		if( $numDiseaseLoci == 1)
		{
		 	$numEndLines = 4; 
		}
		else
		{
			$numEndLines = 5; 
		}
	}
	else
	{
		#error for now
	}

	# This loop is in charged of creating the sifa of the file.
	# notice - the loop dependes on the programCode param. 
	for $i (1..$numEndLines)
	{
		if( $i == 2)
		{
			# The first marker get the 0 place, all his followers will get relative place to their left neighbor
			print CURRENTDAT "0 ";
			
			# Copy the distances
			for $j ($startMarker+1..$endMarker)
			{
				print CURRENTDAT "$datfile[$#datfile - $numEndLines + $i][$j-1] ";
			}
		}
		else
		{
			# Copy the line as it is
			$tmp = join(' ', @{$datfile[$#datfile - $numEndLines + $i]});
			print CURRENTDAT "$tmp";		
		}
		print CURRENTDAT "\n";
	}
	close CURRENTDAT;
}

################################################################
#		getAmountOfMarkers
#
#	Parameters:
#			 $_[0] - dat file
#	Retrun value:
#	  the number of markers in the dat file
################################################################
sub getAmountOfMarkers( $ )
{
	my $datafileName = $_[0];
	open (DATFILE,"$datafileName") or logInnerSysError("couldn't open $datafileName for reading");
	my $line = <DATFILE>;
	my @tmp = split(/[ \t\n\r\s\f]+/,$line);
	return $tmp[0] ;
}

################################################################
#		isThereMarkers
#
#	Parameters:
#			 
#	Retrun value:
#	  1 if there is markers in the dat files, 0 otherwise. 
################################################################
sub isThereMarkers( $ $ ){

	my $datafileName = $_[0];
	my $numDiseaseLoci = $_[1];
	open (DATFILE,"$datafileName") or splitLog("coudn't open $datafileName for reading");
	my $line = <DATFILE>;
	my @tmp = split(/[ \t\n\r\s\f]+/,$line);
	if( $tmp[0] > $numDiseaseLoci ){
		return 1 ;
	}
	else{
		return 0;
	}
}


################################################################
#		getNumberOfLiability
#
#	Parameters:
#			 $_[0] - dat file
#	Retrun value:
#	  the number of liability classes in the dat file
################################################################
sub getNumberOfLiability( $ )
{
 
	#tie @datfile, 'Tie::File', $_[0] or splitLog("couldn't open $_[0] for reading");
	open(FILE,"$_[0]")  or logInnerSysError("couldn't open $_[0] for reading");
	my @datfile = <FILE>;
	close FILE;
	my $line = $datfile[5];
	my @tmp = split(/[ \t\n\r\s\f]+/,$line);
	#untie @datfile;
	return $tmp[0] ;
}

################################################################
#		createMappingFile
#
#	Parameters:
#					$_[0] - array of distBetweenParts
#					$_[1] - path of datfile
#					$_[2] - distance of last part
#	Returns:
#					(@distBetweenParts, $distLastPart)
################################################################
sub createMappingFile( $ )
{
	my($filename,$last,$number,$line,$firstFlag,$mapFlag);
	my $distLastPart = pop @_;
	$filename = pop @_;
	@distBetweenParts = @_;

	open(FILE,"$filename")  or &DAGSplit::logInnerSysError("couldn't open $filename for reading");
	
	#$mapfileName = "$workDir\/resultFile.$variables{ Pedfile }";
	#$mapfileName =~ s/.ped$/.map/g;
	$mapfileName = "resultFile.mapfile.map";
	open(MAPPING,">>$mapfileName") or &DAGSplit::logInnerSysError("couldn't open $mapfileName for writing");
	
	$last = 0;
	$firstFlag = 0;
	for $line (<FILE>)
	{
		$mapFlag = 0;
		
	    my $tmpLine = $line;
		if ( $tmpLine =~ /\((\d+)\)/)
		{
			$mapFlag = 1;
			
			$last = $1;
			if( $firstFlag == 0)
			{
				$firstFlag = 1;
				$distBetweenParts[$#distBetweenParts+1] = &DAGSplit::cmToRec($dist[$1-1] - $distLastPart);
			}
		}
		
		
		if( $mapFlag == 1 )
		{
			$line =~ /.*#(.*)#.*/;
			print MAPPING "$1\n";
		}
	}		
	close FILE;
	close MAPPING;

	$distLastPart = $dist[$last-1];
	
	push(@distBetweenParts, $distLastPart);
	
	return @distBetweenParts;
}

################################################################
#		createMappingFile
#
#	Parameters:
#					$_[0] - path of datfile
#	Returns:
#					
################################################################
# sub createMappingFileForClusteringOnly( $ )
# {
	# my($filename,$last,$number,$line,$firstFlag,$mapFlag);
	# $filename = $_[0];

	# open(FILE,"$filename")  or logInnerSysError("couldn't open $filename for reading");
	
	# $mapfileName = "$workDir\/resultFile.$variables{ Pedfile }";
	# $mapfileName =~ s/.ped$/.map/g;
	# open(MAPPING,">>$mapfileName") or logInnerSysError("couldn't open $mapfileName for writing");
	
	# $last = 0;
	# $firstFlag = 0;
	# for $line (<FILE>)
	# {
		# $mapFlag = 0;
		
	    # my $tmpLine = $line;
		# while ( $tmpLine =~ s/<< marker (\d+)/<< marker TestCase/)
		# {
			# $mapFlag = 1;
			
			# $last = $1;
			# if( $firstFlag == 0)
			# {
				# $firstFlag = 1;
				# $distBetweenParts[$#distBetweenParts+1] = cmToRec($dist[$1-1] - $distLastPart);
			# }
		# }
		
		
		# if( $mapFlag == 1 )
		# {
			# $line =~ /.*#(.*)#.*/;
			# print MAPPING "$1\n";
		# }
	# }		
	# close FILE;
	# close MAPPING;
	# $distLastPart = $dist[$last-1];
# }

################################################################
#		createAndSendCurrentReportFile
#
#	Parameters:
#					$_[0] - $partNum 
#					$_[1] - type of report
#					$_[2] - first marker in the window
#					$_[3] - last marker in the window 
#					$_[4] - percent of progress
#	Returns:
#	 create new file for the current flag error type from err.txt file
#	posible types:
#			 1 SimpleAll - create report from the window files of the cleaning and clustering
#			 2 AddUncheckedMarkersToExistReport - add the unchecked markers found between the windows
#			 3 FinalRep - gather all the reports togheter (in eliminating and clustering)
################################################################
sub createAndSendCurrentReportFile ( $ $ $ $ $ )
{
	my $partNum = $_[0];
	my $percent =$_[4];
	open(CLEANING_REPORT,">>$cleaningWorkDir\/cleaningReport.$partNum.log") or die logInnerSysError("couldn't open $cleaningWorkDir\/cleaningReport.$partNum.log for writing");
	
	if ($_[1] eq "SimpleAll")
	{
		open(RESFILE,"window.$partNum.err.txt") or die logInnerSysError("couldn't open window.$partNum.err.txt for reading");
		@lines = <RESFILE>;
		@sorted = sort { ($a =~ /\((\d+)/)[0] <=> ($b =~ /\((\d+)/)[0] } @lines;
		my %seen;
		my @uniqed = grep !$seen{$_}++, @sorted;
		
		print CLEANING_REPORT "\nThe next part of the file contains error reports that were found during the calculation on one window of markers,\n";
		print CLEANING_REPORT "starting from $_[2] to $_[3].\n\n ";
		
		# Copy the sorted data from err file to the report.
		print CLEANING_REPORT "@uniqed";
		
		close RESFILE;
	}
	elsif ($_[1] eq "AddUncheckedMarkersToExistReport")
	{
		
		open(UNCHECKEDFILE,"$markerListsDir\/markersUnchecked.WindowNum_$partNum.$currentChromosomName.txt") or die logInnerSysError("couldn't open $markerListsDir\/markersUnchecked.WindowNum_$partNum.$currentChromosomName.txt for reading");
		
		print CLEANING_REPORT "\n\nUnchecked markers:\n";
	
		# Copy data from err file to the report.
		while(<UNCHECKEDFILE>)
		{
		   print CLEANING_REPORT $_;
		}
		
		close UNCHECKEDFILE;

	}
	elsif($_[1] eq "FinalRep")
	{
		close CLEANING_REPORT;
		system("for i in \$(seq 1 $partNum); do cat $cleaningWorkDir\/cleaningReport.\$i.* >> $cleaningWorkDir\/cleaningReport.fin.log ; done");
		AddSummaryToFinalReportFile("$cleaningWorkDir\/cleaningReport.fin.log");
		my $scpCommand ="scp $cleaningWorkDir\/cleaningReport.fin.log $remote_user_full:\/$remote_submission_dir\/$currProName\/cleaningReport.fin.log";
		my $result = &General::perfomrWithExponentialBackoff($scpCommand);
		if ( $result != 0 ) 
		{
			logInnerSysError("couldn't perform $scpCommand");
		}
		my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent $percent% -log cleaningReport.fin.log";
		$result = &General::perfomrWithExponentialBackoff($sshCommand);
		if ( $result != 0 ) 
		{
			logInnerSysError("couldn't perform $sshCommand");
		}
		return;
	}
	
	close CLEANING_REPORT;	
	
	my $scpCommand = "scp $cleaningWorkDir\/cleaningReport.$partNum.log $remote_user_full:\/$remote_submission_dir\/$currProName\/cleaningReport.$partNum.log";
	my $result = &General::perfomrWithExponentialBackoff($scpCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $scpCommand");
	}
	my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent $percent% -log cleaningReport.$partNum.log";
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $sshCommand");
	}
}

################################################################
#		sendClusterReport
#
#		$_[0] - progress percent
#
################################################################
sub sendClusterReport( $ )
{
	my $progress = $_[0];
	
	$clusterReportFileName = "$clusterWorkDir\/clusterReport.$currentChromosomName.log";
	open(CLUSTERREPORTFILE,">$clusterReportFileName") or logInnerSysError("couldn't open $clusterReportFileName for writing");
	
	
	$mapfileName = "$workDir\/resultFile.$variables{ Pedfile }";
	$mapfileName =~ s/.ped$/.map/g;
	
	if ( -e $mapfileName )
	{
		open(MAPPING,"$mapfileName") or logInnerSysError("couldn't open $mapfileName for reading");
		
		my $i=1;
		# Copy data from mapfile to the report.
		while(<MAPPING>)
		{
		   print CLUSTERREPORTFILE "Cluster #$i : ";
		   print CLUSTERREPORTFILE $_;
		   $i++;
		}
		
		close MAPPING;
	}
	
	my $scpCommand ="scp $clusterReportFileName $remote_user_full:\/$remote_submission_dir\/$currProName\/clusterReport.$currentChromosomName.log";
	my $result = &General::perfomrWithExponentialBackoff($scpCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $scpCommand");
	}
	
	my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter clustering -percent $progress% -log clusterReport.$currentChromosomName.log";
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	if ( $result != 0 ) 
	{
		logInnerSysError("couldn't perform $sshCommand");
	}
	
	close CLUSTERREPORTFILE;
}

# $_[0] - final log file 
sub AddSummaryToFinalReportFile( $ )
{
	my $amountOfUninformative =0;
	my $amountOfUnlikely =0;
	my $amountOfMendelianError=0;
	
	open (LOGFILE,"$_[0]") or logInnerSysError("couldn't open $_[0] for reading");
	my $line ="";
	my $lastNum=0;
	for $line (<LOGFILE>)
	{
		# we will count each marker only once
		if($line=~m/\((\d+)\)/)
		{
			if($lastNum == $1)
			{
				next;
			}
			else
			{
				$lastNum = $1;
			}
		}
		
		if($line=~m/uninformative/)
		{
			$amountOfUninformative++;	
		}
		elsif($line=~m/Mendelian error/)
		{
			$amountOfUnlikely++;
	
		}
		elsif($line=~m/unlikely genotype/)
		{
			$amountOfMendelianError++;
		}
	}
	close LOGFILE;
	
	my $file = $_[0];
	my $newline = "Cleanning summary:\n\tUninformative: $amountOfUninformative\tUnlikely: $amountOfUnlikely\tMendelianError: $amountOfMendelianError\n\n";
	{
		local @ARGV = ($file);
		local $^I = '.bac';
		while(<>){
		   if ($. == 1) {
			  print "$newline$/";
		   }
		   else {
			  print;
		   }
		}
	}
}


# Error log functions
######################

################################################################
#		inputError
#
#	Parameters:
#		$_[0] - the error to be reported
#	Returns:
#		reports an error in the input of the script and exits
################################################################
sub inputError( $ )
{
	if ( ($mode eq "EliminatingAndCluster") || ($mode eq "EliminatingOnly") )
	{
		&logAndReportError("Input tar file error - $_[0]" , "cleaning",$currProName, $currentChromosomName);
	}
	else
	{
		&logAndReportError("Input tar file error - $_[0]" , "clustering",$currProName, $currentChromosomName);
	}
	
}

################################################################
#		fileLog
#
#	Create error report regarding bad file 
#
#	$_[0] - the error report string
#	$_[1] - string with "FirstDatFile" or "InnerFile" 
#			(in order to know if we need to send a report of the error to the user)
################################################################
sub fileLog( $ $ )
{	
	
	if ($_[1] eq "FirstDatFile")
	{
		if ( ($mode eq "EliminatingAndCluster") || ($mode eq "EliminatingOnly") )
		{
			&logAndReportError("$currentChromosomName : Bad File - $_[0]" , "cleaning",$currProName, $currentChromosomName);
		}
		else
		{
			&logAndReportError("$currentChromosomName : Bad File - $_[0]" , "clustering",$currProName, $currentChromosomName);
		}
	}
	else
	{
		logInnerSysError("Bad File - $_[0]");
	}
	
}

################################################################
#		logAndReportError
#
#	Create report of an error whice isn't related to inner system failure  
#
#	$_[0] - the error report string
################################################################
sub logAndReportError( $ $ )
{
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	my $errorReport = $_[0];
	my $currentStage = $_[1];
	my $currProName = $_[2];
	my $currentChromosomName = $_[3];
	
	my $errorMsg = "$errorReport\nat $now_string\n";
	open( SPLITERRORLOG , ">>$errorOutboxReport") or die  "Cant report error!\n";
	print SPLITERRORLOG "$errorMsg";
	close SPLITERRORLOG;
	
	open( SPLITERRORLOG , ">>$configfile{taskDir}\/$currProName\/workDir\/$currentChromosomName\/reportedError.txt") or die  "Cant report error!\n";
	print SPLITERRORLOG "$errorMsg";
	close SPLITERRORLOG;
	
	# Report to site on the error
	$errorMsg =~ s/\n/ /g;
	$errorMsg =~ s/ /~~/g;
	my $sshCommand ="$ssh_cmd $remote_script error -r $currProName -chr $currentChromosomName -parameter $currentStage -msg $errorMsg";
	
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	if ( $result != 0 ) 
	{
		system("mv $errorOutboxReport $errorOutboxReport.FAILTOSEND");
		logInnerSysError("couldn't perform $sshCommand");
	}
	else 
	{
		system("mv $errorOutboxReport $errorOutboxReport.SENT");
	}
}
 
################################################################
#		logInnerSysError
#
#	Parameters:
#		 $_[0] - the error to be reported
#	Returns:
#		 reports an error during the running of the script
################################################################
sub logInnerSysError( $ )
{
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	my ($errorReport) = $_[0];
	open( SPLITERRORLOG , ">>$splitInnerSysError") or die "Cant report inner error! - $errorReport\n";
	print SPLITERRORLOG "$currentChromosomName : $errorReport\nat $now_string\n";
	close SPLITERRORLOG;
	
	open( SPLITERRORLOG , ">>$configfile{taskDir}\/$currProName\/workDir\/$currentChromosomName\/reportedError.txt") or die  "Cant report error!\n";
	print SPLITERRORLOG "$currentChromosomName : $errorReport\nat $now_string\n";
	close SPLITERRORLOG;
	exit(-1); 
} 

################################################################
#		logInnerSysMinorError
#
#	Parameters:
#		 $_[0] - the error to be reported
#	Returns:
#		 reports an error during the running of the script
################################################################
sub logInnerSysMinorError( $ )
{
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	my ($errorReport) = $_[0];
	open( SPLITERRORLOG , ">>$splitInnerSysError") or die "Cant report inner error! - $errorReport\n";
	print SPLITERRORLOG "$currentChromosomName : $errorReport\nat $now_string\n";
	close SPLITERRORLOG;
} 

# Conversion functions
######################

################################################################
#		recToCm
#
#	Parameters:
#					$_[0] - distance in rec
#	Returns:
#					distance in cM
################################################################
sub recToCm( $ ){
	#my $rec = $_[0];
	#if( $rec == 0.5 ){
	#	$cm = 10000;
	#	return $cm;
	#}	
	#my $cm = (25*log((1+2*$rec)/(1-2*$rec)));
	#return $cm;
	my $theta = $_[0];
	
	if ( $theta >= 0.5 )
	{
		$cM = 100000000;
	}
	elsif ( $theta <= -0.5 )
	{
		$cM = -100000000;
	}
	else
	{
		$cM = -50 * ( log( 1 - 2*$theta ) ) / ( log ( exp(1) ) ) if ( $theta >= 0 );
		$cM = 50 * ( log( 1 + 2*$theta ) ) / ( log ( exp(1) ) ) if ( $theta < 0 );
	}
	
	return sprintf( "%.16f",  $cM  );
}

################################################################
#		cmToRec
#
#	Parameters:
#					$_[0] - distance in cM
#	Returns:
#					 distance in rec
################################################################
sub cmToRec( $ ){
	#my $cm = $_[0];
	#my $rec = (0.5*(exp(0.04*$cm)-1)/(exp(0.04*$cm)+1));
	#return $rec;
	
	my $cM = $_[0];
	
	my $theta = 0.5*( 1 - exp( -2*$cM / 100 ) ) if ( $cM >= 0 );
	$theta = -0.5*( 1 - exp( 2*$cM / 100 ) ) if ( $cM < 0 );

	return sprintf( "%.16f",  $theta  );
}

################################################################
#			getDistFromStart
#	Get distance from the first marker in data 
#
#	Parameters:
#					$_[0] - Marker number
#					$_[1] - original datafile
#	Returns:
#					Distance
################################################################
sub getDistFromStart( $ )
{
	my $marker = $_[0];
	
	open DAT, $_[1]; 

	@data = <DAT>;
	close DAT;
	my $DIST = pop @data;
	$DIST = pop @data;
	$DIST = pop @data;
	my @words = split ' ', $DIST;
	
	&DAGSplit::logInnerSysError("Wrong marker value, no marker number ($marker)!!!", \%variables) unless defined $words[$marker];
	
	my $dist = 0;
	for( my $count = 0; $count <= $marker; $count ++ )
	{
		$dist += &recToCm($words[$count]);
	}
	
	return $dist;
}

################################################################
#		trim
#
#	Parameters:
#					$_[0] - string to trim
#	Returns:
# Trim function to remove leading whitespace
################################################################
sub trim( $ )
{
	my $string = $_[0];
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}


########################################################### FOR DAG #####################################################

################################################################
# 				getMarkerWindowDataFile
#	Get's the Datafile, that will contain only $_[1] markers 
#	starting from given index
#
#	Parameters:
#					$_[0]  - Marker index
#					$_[1]  - Number of markers to be used
#					$_[2]  - Datafile
#	Returns:
#					Variable with datafile
################################################################
sub getMarkerWindowDataFile( $ $ $ )
{
	my $number = $_[0];
	my $count = $_[1];
	my $file = $_[2];
	&DAGSplit::splitLog("Get Marker Data File with index ($number) and file ($file)" );

	my $result = "";
	open IN, $file;

	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	if ($size - $number < $count) {
		$count = $size - $number;
	}
	
	my $datafile = join "\n", @data;
	if ($data[0] =~ /\n/) {
		$datafile = join "", @data;
	}
	
	@data = split "\n", $datafile;

	my $line = shift @data;		#	<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI
	
	$words_first_line[0] = $count + 1;
	$line = join ' ', @words_first_line;
	
	$result = $result.$line."\n";
	$line = shift @data;			#	<< MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
	$result = $result.$line."\n";
	$line = shift @data;			#	<< 1 2
	
	for( my $i = 1; $i <= $count + 1; $i ++ ) {
		$result = $result."$i ";
	}
	$result = $result."\n";
	
	$line = shift @data;	

	#while ( ( (substr($line,0,1) != 3) || (substr($line,4,1) ne "#" ) ) && ( $#data >= 0) )
	while ( ( $line !~ /^3\s+\d+/ ) && ( $#data >= 0) )
	{
		$result = $result.$line."\n";
		$line = shift @data;
	}
	
	for( my $i = 0; $i < $number; $i ++ )
	{
		$line = shift @data;		#	 << GENE FREQUENCIES
		$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
	}
	
	for( my $i = 0; $i < $count; $i ++ )
	{		
		$result = $result.$line."\n";	# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		$line = shift @data;	
		$result = $result.$line."\n";	#	 << GENE FREQUENCIES
		$line = shift @data;
	}
	
	#while ( ( (substr($line,0,1) == 3) && (substr($line,4,1) eq "#" ) ) && ( $#data >= 0) )
	while ( ( $line =~ /^3\s+\d+/ ) && ( $#data >= 0) )
	{
		$line = shift @data;
		$line = shift @data;
	}
	$result = $result.$line."\n";			#		<< SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	$line = shift @data;					#		<< RECOMBINATION VALUES
	
	my $start = $number;
	my $prev = $start;
	for (my $i = $start; $i < $start + $count; $i++ )
	{
		my $distCM = &DAGSplit::getDistFromStart( $i, $file ) - &DAGSplit::getDistFromStart( $prev, $file ); 
		my $distRec = &DAGSplit::cmToRec($distCM);
		$prev = $i;
		$result = $result."$distRec ";
	}
	$result = $result." << RECOMBINATION VALUES\n";
	
	$line = shift @data;		# << number of disease loci
	$result = $result.$line."\n";
	
	$result = $result."-n 2 1 ".($words_first_line[0] - 1)."	 << REC VARIED, INCREMENT, FINISHING VALUE\n";
	
	
	&DAGSplit::splitLog("getMarkerDataFile: Result=($result)");
	return $result;
}
################################################################



################################################################
# 				getMarkerWindowPedFile
#	Get's the Pedfile, that will contain only marker with given index
#
#	Parameters:
#					$_[0]  - Marker index
#					$_[1]  - Number of markers in group
#					$_[2]  - number of markers
#					$_[3]  - Pedfile
#	Returns:
#					Variable with Pedfile
################################################################
sub getMarkerWindowPedFile( $ $ $ )
{
	my $number = $_[0];
	my $count = $_[1];
	my $size = $_[2];
	my $file = $_[3];
	
	&DAGSplit::splitLog("Get Marker Ped File with index ($number) and file ($file)" );
	
	my $result = "";
	open IN, $file;

	my @lines = <IN>;
	close IN;
	
	my @testWords = split "<<", $lines[0];
	@testWords = split /\s+/, $testWords[0];
	
	my $total = 10 + $size*2;
	my $startPos = 10;
	if ( $#testWords % 2 == 0 ) {
		$startPos++;
		$total++;
	}

	#my $leftSize = $#testWords - $startPos - 2*$number + 1;
	my $leftSize = $total - $startPos - 2*$number;
	
	if ($leftSize < 2*$count)
	{
		$count = int ($leftSize/2);
	}

	foreach my $line ( @lines )
	{
		my @words = split /\s+/, $line;
		splice ( @words, $startPos, 2*$number);
		splice( @words, $startPos + 2*$count );
		$line = join ' ', @words;
		$result = $result.$line."\n";
	}

	&DAGSplit::splitLog("getMarkerPedFile: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				getMarkersListDataFile
#	Get's the Datafile, that will contain only the markers 
#	specified by the array
#
#	Parameters:
#					$_[0]  - Markers array
#					$_[1]  - Datafile
#	Returns:
#					Variable with datafile
################################################################
sub getMarkersListDataFile
{
	my $file = pop @_;
	my @markers = @_;
	my $count = $#markers + 1;
	
	&DAGSplit::splitLog("Get Marker Data File with (@markers) and file ($file)" );

	my $result = "";
	open IN, $file;

	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	my $datafile = join "\n", @data;
	if ($data[0] =~ /\n/) {
		$datafile = join "", @data;
	}
	
	@data = split "\n", $datafile;

	my $line = shift @data;		#	<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI
	
	$words_first_line[0] = $count + 1;
	$line = join ' ', @words_first_line;
	
	$result = $result.$line."\n";
	$line = shift @data;			#	<< MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
	$result = $result.$line."\n";
	$line = shift @data;			#	<< 1 2
	
	for( my $i = 1; $i <= $count + 1; $i ++ ) {
		$result = $result."$i ";
	}
	$result = $result."\n";
	
	$line = shift @data;	
	
	#while ( ( (substr($line,0,1) != 3) || (substr($line,4,1) ne "#" ) ) && ( $#data >= 0) )
	while ( ( $line !~ /^3\s+\d+/ ) && ( $#data >= 0) )
	{
		$result = $result.$line."\n";
		$line = shift @data;
	}
	
	my $startFrom = 0;
	
	for (my $index = 0; $index <= $#markers; $index++) {
		
		my $lookFor = $markers[$index] - 1;
		
		for( my $i = $startFrom; $i < $lookFor; $i++ )
		{
			$line = shift @data;		#	 << GENE FREQUENCIES
			$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		}
		
		$startFrom = $lookFor + 1;
		
		$result = $result.$line."\n";	# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		$line = shift @data;	
		$result = $result.$line."\n";	#	 << GENE FREQUENCIES
		$line = shift @data;
	}
	
	#while ( ( (substr($line,0,1) == 3) && (substr($line,4,1) eq "#" ) ) && ( $#data >= 0) )
	while ( ( $line =~ /^3\s+\d+/ ) && ( $#data >= 0) )
	{
		$line = shift @data;
		$line = shift @data;
	}
	$result = $result.$line."\n";			#		<< SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	$line = shift @data;					#		<< RECOMBINATION VALUES
	@words = split ' ', $line;
	
	my $prev = 0;
	for (my $i = 0; $i <= $#markers; $i++) {
		
		my $distCM = &DAGSplit::getDistFromStart( $markers[$i] - 1, $file) - &DAGSplit::getDistFromStart( $markers[ $prev ] - 1, $file );
		my $distRec = &DAGSplit::cmToRec($distCM);
		$prev = $i;
		$result = $result."$distRec ";
	
		#$result = $result."$words[ $markers[$i] - 1 ] ";
	}
	$result = $result." << RECOMBINATION VALUES\n";
	
	$line = shift @data;		#	 << REC VARIED, INCREMENT, FINISHING VALUE
	$result = $result.$line."\n";
	
	$result = $result."-n 2 1 ".($words_first_line[0] - 1)."	 << REC VARIED, INCREMENT, FINISHING VALUE\n";
	
	&DAGSplit::splitLog("getMarkerDataFile: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				getMarkersListPedFile
#	Get's the Pedfile, that will contain only the markers 
#	specified by the array
#
#	Parameters:
#					$_[0]  - Markers array
#					$_[2]  - Pedfile
#	Returns:
#					Variable with Pedfile
################################################################
sub getMarkersListPedFile
{
	my $file = pop @_;
	my @markers = @_;
	my $count = $#markers + 1;
	
	&DAGSplit::splitLog("Get Marker Ped File with (@markers) and file ($file)" );
	
	my $result = "";
	open IN, $file;

	my @lines = <IN>;
	close IN;

	my @testWords = split "<<", $lines[0];
	@testWords = split /\s+/, $testWords[0];
	my $startPos = 10;
	if ( $#testWords % 2 == 0 ) {
		$startPos++;
	}

	foreach my $line ( @lines )
	{
		my @words = split /\s+/, $line;
		$result = $result.$words[0];
		
		for (my $i = 1; $i < $startPos; $i++) {
			$result = $result." ".$words[ $i ];
		}
		
		for (my $i = 0; $i <= $#markers; $i++) {
			my $markerIndex = $startPos + 2*($markers[$i] - 1);
			$result = $result." ".$words[ $markerIndex ]." ".$words[ $markerIndex + 1 ];
		}
		$result = $result."\n";
	}

	&DAGSplit::splitLog("getMarkerPedFile: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				isValidMarkersList
#	Checks whether the given marker exists in the given file
#
#	Parameters:
#					$_[0] - markers array
#					$_[1] - datafile path
#	Returns:
#					"OK" - If the window is OK
#					"clean file size is X" - Otherwise
################################################################
sub isValidMarkersList( $ $ ) {
	my $file = pop @_;
	my @markers = @_;
	
	&DAGSplit::logInnerSysError("Failed opening file ($file) to read") unless ( open IN, $file );
	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	foreach $marker (@markers) {
		if ( ($marker > $size) || ($marker < 1) ) {
			return "Clean file size is ".$size;;
		}
	}
	
	return "OK";
}	

################################################################
#		saveToFile
# Reads upload file and write it in temporary file
#
# Parameters: 
#			 $_[0] - path to file to create
#            $_[1] - data to save in file
################################################################
sub saveToFile( $ $ )
{
	my $name = $_[0];
	my $data = $_[1];
	
	&DAGSplit::splitLog( "Creating file: ($name)" );
	&DAGSplit::splitLog( "With data: ($data)" );
	&DAGSplit::logInnerSysError("Could not open file ($name) for write (in function saveToFile) error: ($!)") unless ( open (FH,">$name") );

  	print FH $data;	
  	close FH;
}	


1;
