# dvir 2010
package DatPedHandler;

use strict;
use General;
use Configuration;
use Class::Struct;
use FileHandle;

#	dvir 2010/09/26
#	checking that the Dat file is valid.
#	The Checks :
#		1. all the markers has names.
#		2. there aren't two markers with the same name in the dat file
#		3. the dat file has one marker less than the number of markers in the first line
#		4. the dat file has markers as at the last line of the dat file
#	parameters : 
#		$datFileName - the name of the dat file (including path, if the dat file ain't at pwd)
sub checkDatFile($) {
	my $datFileName = $_[0];
	
	open (DAT_FILE, $datFileName) || &General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $datFileName ,$!));
	
	my $firstLine = <DAT_FILE>;
	my $numOfMarkers = &numOfMarkersInDatFileByFirstLine($firstLine);
	
	<DAT_FILE> for 1..2; # lines 2&3 in the dat file aren't relevant
	my $currMarker = 1;
		
	my %markersNumByNames = ();
	while (my $line = <DAT_FILE>) {
		# checking if this is a marker line
		if ($line =~ /$Configuration::MARKER_LINE_TEMPLATE/) {
			# checking if the marker line doesn't have a name
			if ($line !~ /$Configuration::MARKER_LINE_WITH_MARKER_NAME_TEMPLATE/) {
				&General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", "marker num : $currMarker, doesn't have a name");
			}
			
			# if we reach to here, than the marker has a name
			
			my @markerLineArr = split ('#', $line);
			my $markerName = $markerLineArr[1];
			# checking if there is a marker with the same name
			if (defined $markersNumByNames{$markerName}) {
				&General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", "there are two markers in the dat file with the name: $markerName");
			}			
			
			# if we reach to here, than there isn't a marker with the same name
			# marking that we have marker with the name : $markerName
			$markersNumByNames{$markerName} = 1;
			
			<DAT_FILE>; # the second line of the marker isn't relevant
			++$currMarker;			
		}
	}
	close DAT_FILE;
	
	# if we havn't read $numOfMarkers markers, than something is wrong
	if ($numOfMarkers != $currMarker - 1) {
		&General::reportOnErrorReleaseSemaphoresAndDie('DatPedHandler.pm',"", "$datFileName claims having $numOfMarkers markers, but it have only ".($currMarker-1)." markers");
	}
}

#	dvir 2010/09/26
#	returning the num of markers in the dat file by the given first line of the dat file
#	parameters : 
#		$firstLine - the first line in the dat file
sub numOfMarkersInDatFileByFirstLine($) {
	my $firstLine = $_[0];
	my @firstLineArr = split (/\s+/,$firstLine);
	my $numOfMarkers = $firstLineArr[0] - 1;
	return $numOfMarkers;
}

#	dvir 2010/09/26
#	returning the num of markers in the dat file by the dat file name (if you are using the datfile for more things,
#	than its more efficent to call the method numOfMarkersInDatFileByFirstLine)
#	parameters : 
#		$datFileName - the name of the dat file (including path, if the dat file ain't at pwd)
sub numOfMarkersInDatFile($) {
	my $datFileName = $_[0];
	open (DAT_FILE, $datFileName) || &General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $datFileName ,$!));
	my $firstLine = <DAT_FILE>;
	close DAT_FILE;
	return numOfMarkersInDatFileByFirstLine($firstLine);
}

# dvir 2011/07/26
#	calculating the distances of all the markers from their previous marker in theta
#	theta is distance written at the dat file recombination values
# 	ASSUMING that the recombination values line should be the third from the end of the dat file
#	(after cleaning & clustering that it is at that postion)
#	parameters : 
#		the dat file name
sub calcDistInTehtaForEachMarkerFromItsPreviousMarkerByDatFile($ \@) {
	my $datFileName = $_[0];
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[1];
	
	my $numOfMarkers = &numOfMarkersInDatFile($datFileName);
	
	# the recombination values line is the third from the end of the dat file
	open DAT_FILE_FOOTER, "tail -3 $datFileName|";
	my $recombinationVaulesLine = <DAT_FILE_FOOTER>; # 0.0000000000000000 0.0058527555703775  << marker distances
	close DAT_FILE_FOOTER;	

	&calcDistInTehtaForEachMarkerFromItsPreviousMarker(\$recombinationVaulesLine, $numOfMarkers, $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);	
}

#	dvir 2010/09/12
#	calculating the distances of all the markers from their previous marker in theta.
#	using the recombination values line from the dat file
#	theta is distance written at the dat file recombination values
#	parameters : 
#		$recombinationVaulesLineRef - a reference to a string that is the line of the recombination values at the dat file
#									for example a reference to the string : "0.0000000000000000 0.2120000000000000 0.2379999999999999 0.2069999999999999 0.1790000000000000    << RECOMBINATION VALUES "
#									the reason that i use reference, is when there are many markers, and than, this string is a long string
#		$numOfMarkers - number of markers according to the data file
#		$distancesInThetaFromFirstMarkerRef - (output param) a reference to an array that will contain the distances in theta.
#			cell i in the array is the distance of marker i from marker i-1,
#			and therefore cell 0 in the array is meaningless
sub calcDistInTehtaForEachMarkerFromItsPreviousMarker(\$ $ \@) {
	my $recombinationVaulesLineRef = $_[0];
	my $numOfMarkers = $_[1];
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[2];
	
	@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = split '\s+', $$recombinationVaulesLineRef;
	
	# dvir: the recombination line may starts with a value which is not 0.
	# the first value is unimportant
	# but for calcs, we will set the first value to 0.
	# the rec value #i means the dist in recombination values between marker i and marker i-1
	$$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef[0] = 0;
	
	while (scalar(@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef) > $numOfMarkers) {
		pop @$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef;
	}
}

#	dvir 2010/09/27
#	calculating the distances in tehta between two given markers acording to the array of recombination values in theta
#	parameters : 
#		$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef - a reference to an array of all the recombination values in theta
#		$firstMarkerIndex, $secondMarkerIndex - the index of the two markers that we want to know the distance between them
#												$firstMarkerIndex is the first marker,
#	NOTE: the indexes are from 0 (should be changed after not using at PostCleaning.pl )
sub calcDistInTehtaBetweenTwoMarkers($ $ \@) {
	my $firstMarkerIndex = $_[0];
	my $secondMarkerIndex = $_[1];
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[2];
	
	my $distInCentiMorganBetweenTwoMarkers = calcDistInCMBetweenTwoMarkers($firstMarkerIndex, $secondMarkerIndex, $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);

	my $distInThetaBetweenTwoMarkers = &cmToRec($distInCentiMorganBetweenTwoMarkers);
	return sprintf( "%.16f", $distInThetaBetweenTwoMarkers);
}

#	dvir 2011/07/27
#	calculating the distances in CM between two given markers acording to the array of recombination values in theta
#	parameters : 
#		$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef - a reference to an array of all the recombination values in theta
#		$firstMarkerIndex, $secondMarkerIndex - the index of the two markers that we want to know the distance between them
#												$firstMarkerIndex is the first marker,
#	NOTE: the indexes are from 0 (should be changed after not using at PostCleaning.pl )
sub calcDistInCMBetweenTwoMarkers($ $ \@) {
	my $firstMarkerIndex = $_[0];
	my $secondMarkerIndex = $_[1];
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[2];
	
	my $numOfMarkers = scalar(@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);
	if ($firstMarkerIndex < 0 || $secondMarkerIndex < 0 || $firstMarkerIndex > $numOfMarkers || $secondMarkerIndex > $numOfMarkers) {
		&General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", "calcDistInTehtaBetweenMarkers: bad indexes: firstMarkerIndex: $firstMarkerIndex, secondMarkerIndex: $secondMarkerIndex, numOfMarkers: $numOfMarkers.");	
	}
	
	my $distInCentiMorganBetweenTwoMarkers = 0;
	for (my $currMarker=$firstMarkerIndex+1;$currMarker<=$secondMarkerIndex;++$currMarker) {
		$distInCentiMorganBetweenTwoMarkers += &recToCm($$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef[$currMarker]);
	}
	return $distInCentiMorganBetweenTwoMarkers;
}

#	dvir 2011/07/27
#	calculating the distances in cM For Each Marker From The First Marker
#	parameters : 
#		$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef - a reference to an array of all the recombination values in theta for each marker from its prev marker
#		$distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef - (output param) a reference to an array of all the distances from the first marker for all markers
#	Note: the indexes are from 0.
sub calcDistInCM_ForEachMarkersFromTheFirstMarker(\@ \@) {
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[0];
	my $distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef = $_[1];

	my $numOfMarkers = scalar(@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);

	my $distInCM_OfCurrMarker_FromTheFirstMarker = 0;
	for (my $currMarker=0;$currMarker<$numOfMarkers;++$currMarker) {
		my $distInCentiMorganBetweenCurrMarkerAndItsPrevMarker = &recToCm($$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef[$currMarker]);
		
		$distInCM_OfCurrMarker_FromTheFirstMarker += $distInCentiMorganBetweenCurrMarkerAndItsPrevMarker;
		$$distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef[$currMarker] = $distInCM_OfCurrMarker_FromTheFirstMarker;
	}	
}

#	dvir 2011/08/24
#	calculating the distances in cM For Each Marker From The begining of the chromosome
#	parameters : 
#		$distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef - a reference to an array of all the distances in cM from the first marker for all markers
#		$distanceOfTheFirstMarkerInDatFile_From_TheBeginingOfChrom
#		$distancesInCM_ForEachMarker_From_TheBeginingOfChromArrRef - (output param) a reference to an array of all the distances in cM from the begining of the chromosome for all markers
#	Note: the indexes are from 0.
sub calcDistInCM_ForEachMarkers_From_TheBeginingOfChrom(\@ $ \@) {
	my $distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef = $_[0];
	my $distanceOfTheFirstMarkerInDatFile_From_TheBeginingOfChrom = $_[1];
	my $distancesInCM_ForEachMarker_From_TheBeginingOfChromArrRef = $_[2];

	my $numOfMarkers = scalar(@$distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef);

	for (my $currMarker=0;$currMarker<$numOfMarkers;++$currMarker) {
		my $distInCM_OfCurrMarker_FromTheFirstMarker = $$distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef[$currMarker];
		
		$$distancesInCM_ForEachMarker_From_TheBeginingOfChromArrRef[$currMarker] = $distanceOfTheFirstMarkerInDatFile_From_TheBeginingOfChrom + $distInCM_OfCurrMarker_FromTheFirstMarker;
	}
}

# 	dvir 2011/10 : use &Distances::recToCm and not this.
#	dvir 2010/09/27
#	dvir : copied from DagSplit.pm
#	converts between recombination value in theta to recombination value in centi morgan
sub recToCm( $ ){
	my $theta = $_[0];
	
	my $cM;
	if ( $theta >= 0.5 ) {
		$cM = 100000000; # to mark that the distance is : infinity
	}
	elsif ( $theta <= -0.5 ) {
		$cM = -100000000; # to mark that the distance is : minus infinity
	}
	elsif ( $theta >= 0 ) {
		$cM = -50 * ( log( 1 - 2*$theta ) ) / ( log ( exp(1) ) );
	}
	else { # $theta < 0
		$cM = 50 * ( log( 1 + 2*$theta ) ) / ( log ( exp(1) ) );
	}
	
	return sprintf( "%.16f",  $cM  );
}

# 	dvir 2011/10 : use &Distances::cmToRec and not this.
#	dvir 2010/09/27
#	dvir : copied from DagSplit.pm
#	converts between recombination value in centi morgan to recombination value in theta
sub cmToRec( $ ){
	my $cM = $_[0];
	
	my $theta;
	if ( $cM >= 0 ) {
		$theta = 0.5*( 1 - exp( -2*$cM / 100 ) ) ;
	} else {
		$theta = -0.5*( 1 - exp( 2*$cM / 100 ) ) if ( $cM < 0 );
	}
	return sprintf( "%.16f",  $theta  );
}

#	dvir 2010/09/28 & 2010/11/29
#		creates files in the input format for madeline & haplo painter, in CWD
#		the files names are 
#		$Configuration::MADELINE_DIR\$Configuration::MADELINE_INPUT_FILENAME, 
#		$Configuration::HAPLO_PAINTER_DIR\$Configuration::HAPLO_PAINTER_INPUT_FILENAME
#
#		description of the format of the file of madeline is in : http://eyegene.ophthy.med.umich.edu/madeline/documentation.php
#		 the other format with spaces cause to the problem : Unrecoverable Error: MadelineTable::_CheckDataBlockRectangularity(): Lengths of rows in the DATA block are inconsistent
#	parameters :
#		$pedFileName - the name of the ped file
#		$peopleNumToNameConversionFileName - the name of the file that contains conversion between people number to their name
#			this file is relevant only if the user uses snp, so if if the file doesn't exists than this parameter should be undef
sub createMadelineAndHaploPainterInputFiles($ $) {
	my $pedFileName = $_[0];
	my $peopleNumToNameConversionFileName = $_[1];

	my %peopleNumToName = ();
	
	# if there is an conversion file
	if ($peopleNumToNameConversionFileName) {
		open (CONVERSION, $peopleNumToNameConversionFileName) || &General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $peopleNumToNameConversionFileName ,$!));
		while (my $line = <CONVERSION>) {
			chomp $line;
			my @line = split /\t+/, $line;
			# $line[0] : family number
			# $line[1] : person in family number,
			# 			 persons in diffrenet families may have the same number
			# $line[2] : the person name
			$peopleNumToName{"$line[0]_$line[1]"} = "$line[2]\[$line[1]\]";
		}
		close CONVERSION;
	}
	
	my $currFamily;
	open (PEDFILE, $pedFileName) || &General::reportOnErrorReleaseSemaphoresAndDie("DatPedHandler.pm", "", sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $pedFileName ,$!));
	
	# making sure that the dir exists
	if (! -d $Configuration::MADELINE_DIR) {
		mkdir $Configuration::MADELINE_DIR;
	}			
	open (MADELINE_OUT, ">$Configuration::MADELINE_DIR\/$Configuration::MADELINE_INPUT_FILENAME");
	
	# making sure that the dir exists
	if (! -d $Configuration::HAPLO_PAINTER_DIR) {
		mkdir $Configuration::HAPLO_PAINTER_DIR;
	}	
	open (HAPLOPAINTER_OUT, ">$Configuration::HAPLO_PAINTER_DIR\/$Configuration::HAPLO_PAINTER_INPUT_FILENAME");

	print MADELINE_OUT "FamilyId\tIndividualId\tFather\tMother\tGender\tAffected\n";
	while (my $line = <PEDFILE>) {
		#my $line =~ s/\t+/' '/; # the line may be seperated by tab or by space
		my @line = split /\s+/, $line;
				
		$currFamily = $line[0];
		my $currFamilyName = "F$currFamily";
		my $currPersonNum = getPersonName($currFamily, $line[1], $peopleNumToNameConversionFileName, \%peopleNumToName);
		my $currPersonFather = getPersonName($currFamily, $line[2], $peopleNumToNameConversionFileName, \%peopleNumToName);
		my $currPersonMother = getPersonName($currFamily, $line[3], $peopleNumToNameConversionFileName, \%peopleNumToName);
		my $sexNum = $line[4];
		my $sex = ("$sexNum" eq "1") ? "M" : "F";
		my $isAffectedNum = $line[5];
		my $isAffected = ("$isAffectedNum" eq "2") ? "A" : ".";
		
		# printing new line + family + id + father + mother + sex + isAffected (status)
		my $currPersonLineForMadeline = "$currFamilyName\t$currPersonNum\t$currPersonFather\t$currPersonMother\t$sex\t$isAffected\n";
		print MADELINE_OUT $currPersonLineForMadeline;

		$currPersonNum = ($currPersonNum eq ".") ? "0" : $currPersonNum;
		$currPersonFather = ($currPersonFather eq ".") ? "0" : $currPersonFather;
		$currPersonMother = ($currPersonMother eq ".") ? "0" : $currPersonMother;
		my $currPersonLineForHaploPainter = "$currFamily\t$currPersonNum\t$currPersonFather\t$currPersonMother\t$sexNum\t$isAffectedNum\n";
		print HAPLOPAINTER_OUT $currPersonLineForHaploPainter;		
	}
	close MADELINE_OUT;
	close HAPLOPAINTER_OUT;
	close PEDFILE;
}

# To be used only from the sub : createMadelineInputFiles
sub getPersonName($ $ $ \%) {
	my $family = $_[0];
	my $personNum = $_[1];
	my $peopleNumToNameConversionFileName = $_[2];
	my $peopleNumToNameHashRef = $_[3];
	
	if ($personNum == 0) {
		return ".";
	}
	elsif ($peopleNumToNameConversionFileName) {
		return $$peopleNumToNameHashRef{"${family}_$personNum"};
	}
	else {
		return $personNum;
	}
}


# dvir 21/10/2010
# superlink executable creates a LOD table, and may add rows between two markers.
#
# we assume that the $datFileName is after cleaning,
# therefore, the line with the attribute of num of Rows between 2 markers at lod table is the last row at the dat file
# example for such a row :
# -n 2 1 1285      << REC VARIED, INCREMENT, FINISHING VALUE
#
# IMPORTANT: this method is called also from the computer on which a job run, so, in case of an error it doesn't send mail
# also, the dat file should be at the cwd (common case when running on a condor tmp dir on a remote computer)
sub getNumOfRowsBetweenTwoMarkersAtLodTable($) {
	my $datFileName = $_[0];
	
	&General::makeSureFileExists($datFileName);
	
	my $numOfRowsBetweenMarkersAtLodTable = `tail -1 $datFileName | cut -d ' ' -f 2`;
	# checking that we got a number
	&General::makeSureVarIsNumeric($numOfRowsBetweenMarkersAtLodTable);
	# the minimum value of numOfRowsBetweenMarkersAtLodTable is 1 (if its 1, than there is no rows between two markers in the Lod Table )
	if ($numOfRowsBetweenMarkersAtLodTable < 1) {
		die "numOfRowsBetweenMarkersAtLodTable=$numOfRowsBetweenMarkersAtLodTable is less than 1.";
	}
	
	# we substract 1, since the actual meaning of this var is the num Of Rows Between Markers At Lod Table + 1
	return $numOfRowsBetweenMarkersAtLodTable - 1;
}

# dvir 08/11/2010
# returns an array with all family numbers that there at least one person of them in the ped file
#	parameters :
#		$pedFileName - the name of the ped file
#		$familiesNumbersArrRef - reference to an array of the familes numbers
sub getFamiliesNumbersInPedFile ($ \@) {
	my $pedFileName = $_[0];
	my $familiesNumbersArrRef = $_[1];
	
	&General::makeSureFileExists($pedFileName);
	
	# awk works when the delimiter in the ped file is space \ tab
	#Omer 1/9/2011 - added grep to discard empty lines
	my $familiesNumbersStr = `awk '{ print \$1 }' $pedFileName | sort | uniq  | grep -v \"\^\$\"`;
	@$familiesNumbersArrRef = split /\s+/,$familiesNumbersStr;
}

# dvir 2011/09/07
# creating a file with one line
# the line will contain all the families numbers by the given arr
# the families numbers will be sepertated by ' '
sub createFamiliesFileByArr(\@) {
	my $familiesNumbersArrRef = $_[0];
	
	# sorting the families numbers
	my @familiesNums = sort {$a <=> $b} @$familiesNumbersArrRef;
	my $familiesNumbersStr = join ' ', @familiesNums;
	
	open FAMILIES_NUMS, ">$Configuration::FAMILIES_NUMBERS_FILENAME";
	print FAMILIES_NUMS $familiesNumbersStr;
	close FAMILIES_NUMS;	
}

# dvir 29/11/2010
# creating a file with one line
# the line will contain all the families numbers which are in the given file
# the families numbers will be sepertated by ' '
sub createFamiliesNumbersFile($) {
	my $pedFileName = $_[0];
	   
	my @familiesNumbers;
	&getFamiliesNumbersInPedFile($pedFileName, \@familiesNumbers);
	
	my $familiesNumbersStr = join ' ', @familiesNumbers;
	
	open FAMILIES_NUMS, ">$Configuration::FAMILIES_NUMBERS_FILENAME";
	print FAMILIES_NUMS $familiesNumbersStr;
	close FAMILIES_NUMS;
}

# dvir 29/11/2010
# getting the families numbers by a families numbers file in CWD
sub getFamiliesNumbersArrByFamiliesNumbersFile( \@ ) {
	
	my $familiesNumbersArrRef = $_[0];
	&getFamiliesNumbersArrByFamiliesNumbersFileAtFolder($familiesNumbersArrRef, '.');	
}

# dvir 20/12/2010
# getting the families numbers by a families numbers file at the given folder
sub getFamiliesNumbersArrByFamiliesNumbersFileAtFolder(\@ $) {
	my $familiesNumbersArrRef = $_[0];
	my $folderOfTheFamiliesNumbersFile = $_[1];
	
	my $familiesNumbersFileFullPath = "$folderOfTheFamiliesNumbersFile\/$Configuration::FAMILIES_NUMBERS_FILENAME";
	&General::makeSureFileExists($familiesNumbersFileFullPath);
		
	open FAMILIES_NUMS, $familiesNumbersFileFullPath;
	my $familiesNumbersStr = <FAMILIES_NUMS>;
	close FAMILIES_NUMS;
	
	@$familiesNumbersArrRef = split(/\s+/, $familiesNumbersStr);
	@$familiesNumbersArrRef = sort {$a <=> $b} @$familiesNumbersArrRef;
}

# dvir 19/12/2010
# getting the number of families by a families numbers file in CWD
sub getNumOfFamiliesByFamiliesNumbersFile() {
	
	&General::makeSureFileExists($Configuration::FAMILIES_NUMBERS_FILENAME);
		
	open FAMILIES_NUMS, "$Configuration::FAMILIES_NUMBERS_FILENAME";
	my $familiesNumbersStr = <FAMILIES_NUMS>;
	close FAMILIES_NUMS;
	
	my @familiesNumbersArr = split(/\s+/, $familiesNumbersStr);
	
	my $numOfFamilies = scalar (@familiesNumbersArr);
	
	if ($numOfFamilies < 0) {
		die "Wrong number of families: $numOfFamilies";
	}
	return $numOfFamilies;
}

# dvir 2011-02-22,
# returning the name of a dat that containts list of markers from the dat file : $datFileName
sub getDatByListFileName( $ ) {
	my $datFileName = $_[0];
	
	$datFileName = &General::removePath($datFileName);
	$datFileName =~ s/.dat$/.listOfMarkers.dat/;
	return $datFileName;
}

# dvir 2011-02-22,
# returning the name of a dat that containts range of markers from the dat file : $datFileName
sub getDatByRangeFileName( $ ) {
	my $datFileName = $_[0];
	
	$datFileName = &General::removePath($datFileName);	
	$datFileName =~ s/.dat$/.rangeOfMarkers.dat/;
	return $datFileName;
}
	
# dvir 2011-02-23
# Returning how many markers are in the given range
# the range includes the markers at position $firstMarker & $lastMarker
# and the markers indexes start from 1
sub getTotalNumOfMarkersByRange( $ $ ) {
	my $firstMarker = $_[0];
	my $lastMarker = $_[1];
	return $lastMarker - $firstMarker + 1;
}

# dvir 2011-02-23
# Returning how many full windows (which are exactlly at the size of $windowSize) are.
# and how many markers are in the last window (may be less than $windowSize)
# for example, windowSize = 3, numOfMarkers = 14 -> numOfFullWindows = 4, numOfMarkersAtLastWindow = 14-3*4=2
# if the last window has exactlly $windowSize markers, than we return 0 at numOfMarkersAtLastWindow
# the range includes the markers at position $firstMarker & $lastMarker
# and the markers indexes start from 1
sub getNumOfFullWindowsAndNumOfMarkersAtLastWindowByWindowSizeAndNumOfMarkers( $ $ ) {
	my $windowSize = $_[0];
	my $totalNumOfMarkers = $_[1];
	
	my $numOfFullWindows = &POSIX::floor($totalNumOfMarkers / $windowSize);
	my $numOfMarkersAtLastWindow = $totalNumOfMarkers - $numOfFullWindows * $windowSize;
	
	return ($numOfFullWindows, $numOfMarkersAtLastWindow);
}

# dvir 2011/08/10
sub getNumOfWindowsByWindowSizeAndNumOfMarkers( $ $ ) {
	my $windowSize = $_[0];
	my $totalNumOfMarkers = $_[1];

	(my $numOfFullWindows, my $numOfMarkersAtLastWindow) = &getNumOfFullWindowsAndNumOfMarkersAtLastWindowByWindowSizeAndNumOfMarkers(
		$windowSize, 
		$totalNumOfMarkers);

	# if there is a window at the end, then there is another window
	if ($numOfMarkersAtLastWindow) {
		return $numOfFullWindows+1;
	}
	else {
		return $numOfFullWindows;
	}
}

#Omer, 5/9/2011
sub createMarkersNamesToNumsHash( $ $ ) {
	my $datFileName = $_[0];
	my $MarkersNamesToNumsHashRef = $_[1];
	
	#Omer - For now, we take the whole range of markers
	my $firstMarker = 1;
	my $lastMarker = numOfMarkersInDatFile($datFileName);	
	
	&General::makeSureFileExists($datFileName);	
	open (DAT_FILE, $datFileName) || &General::reportOnErrorReleaseSemaphoresAndDie("createMarkersNumsToNamesFile.pm", "", sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, $datFileName ,$!));	
		
	<DAT_FILE> for 1..7;

	# moving the cursor to the first marker in the range
	<DAT_FILE> for 1 .. (2*($firstMarker-1));	
	
	for (my $currMarkerIndexInRange = $firstMarker; $currMarkerIndexInRange <= $lastMarker; ++$currMarkerIndexInRange) {	
		# reading the 2 lines of the current marker
		my $markerLine1 = <DAT_FILE>; # 3 2 #rs1474052# << marker 1
		my @markerNameHelp = split '#', $markerLine1;
		my $markerName = $markerNameHelp[1]; # the second (no' 1, starting from 0) field is the marker name
		$$MarkersNamesToNumsHashRef{$markerName} = $currMarkerIndexInRange;		
		<DAT_FILE>; # 0.5 0.5 << marker frequency
	}	
	close DAT_FILE;	
}

1;