# dvir, 2011-09-09
package Distances;

use strict;

#	dvir 2010/09/27
#	dvir : copied from DagSplit.pm
#	converts between recombination value in theta to recombination value in centi morgan
sub recToCm( $ ){
	my $theta = $_[0];
	
	my $cM;
	if ( $theta >= 0.5 ) {
		$cM = 100000000; # to mark that the distance is : infinity
	}
	elsif ( $theta <= -0.5 ) {
		$cM = -100000000; # to mark that the distance is : minus infinity
	}
	elsif ( $theta >= 0 ) {
		$cM = -50 * ( log( 1 - 2*$theta ) ) / ( log ( exp(1) ) );
	}
	else { # $theta < 0
		$cM = 50 * ( log( 1 + 2*$theta ) ) / ( log ( exp(1) ) );
	}
	
	return sprintf( "%.16f",  $cM  );
}

#	dvir 2010/09/27
#	dvir : copied from DagSplit.pm
#	converts between recombination value in centi morgan to recombination value in theta
sub cmToRec( $ ){
	my $cM = $_[0];
	
	my $theta;
	if ( $cM >= 0 ) {
		$theta = 0.5*( 1 - exp( -2*$cM / 100 ) ) ;
	} else {
		$theta = -0.5*( 1 - exp( 2*$cM / 100 ) ) if ( $cM < 0 );
	}
	return sprintf( "%.16f",  $theta  );
}

#find the distance of the first used marker from the beginning of the chromosome modefile.txt
sub calcDistanceOfFirstMarkerFromTheBeginingOfTheChromosomeIn_cM( \% ) {
	my %variables = %{$_[0] };
	
	my $distanceOfFirstMarkerInDatFile_From_TheBeginingOfChrom;
	if (! defined $variables{positions}) {
		return 0;
	}
	if (defined $variables{positions} && 
		$variables{positions} ne '' &&
	    scalar(@{$variables{positions}}) > 0) {
		my $firstMarkerNameAndDist = ${$variables{positions}}[0];
		# getting the dist from $firstMarkerNameAndDist.
		# example for $firstMarkerNameAndDist is "rs123 : 300000"
		$firstMarkerNameAndDist =~ /.+ : (.+)/;
		my $distInMillion_cM = $1; # $1 is the string between the brackets at the reg exp above		
		$distanceOfFirstMarkerInDatFile_From_TheBeginingOfChrom = $distInMillion_cM/1000000;
	}
	else {
		# we don't have the info of the distance of the first marker from the begining of the chromosome
		$distanceOfFirstMarkerInDatFile_From_TheBeginingOfChrom = 0;
	}
	
	return $distanceOfFirstMarkerInDatFile_From_TheBeginingOfChrom;	
}


#	dvir 2010/09/12
#	calculating the distances of all the markers from their previous marker in theta.
#	using the recombination values line from the dat file
#	theta is distance written at the dat file recombination values
#	parameters : 
#		$recombinationVaulesLineRef - a reference to a string that is the line of the recombination values at the dat file
#									for example a reference to the string : "0.0000000000000000 0.2120000000000000 0.2379999999999999 0.2069999999999999 0.1790000000000000    << RECOMBINATION VALUES "
#									the reason that i use reference, is when there are many markers, and than, this string is a long string
#		$numOfMarkers - number of markers according to the data file
#		$distancesInThetaFromFirstMarkerRef - (output param) a reference to an array that will contain the distances in theta.
#			cell i in the array is the distance of marker i from marker i-1,
#			and therefore cell 0 in the array is meaningless
sub calcDistInTehtaForEachMarkerFromItsPreviousMarker(\$ $ \@) {
	my $recombinationVaulesLineRef = $_[0];
	my $numOfMarkers = $_[1];
	my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[2];
	
	@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = split '\s+', $$recombinationVaulesLineRef;
	
	# dvir: the recombination line may starts with a value which is not 0.
	# the first value is unimportant
	# but for calcs, we will set the first value to 0.
	# the rec value #i means the dist in recombination values between marker i and marker i-1
	$$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef[0] = 0;
	
	while (scalar(@$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef) > $numOfMarkers) {
		pop @$distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef;
	}
}

1;