package Drawing;

use strict;
use Configuration;
use General;
use AnalysisTemplateClass;
use AnalysisSubmissionFile;

# this class inherits methods from AnalysisTemplateClass
our @ISA = ('AnalysisTemplateClass');

# madeline and haploPainter are installed only at cbltest
sub getCondorJobMemoryRequirments() {
	return 'Machine == "cbltest.cs.technion.ac.il" &&';
}

# returning an array of files names
# those files will be copied to the machine on which the job will run
# it shouldn't contain the dat & ped files, which will be added by the calling method.
sub getCondorJobInputFiles() {
	my @pmFilesInBinDirArr = &General::getArrayOf_pm_filesNamesInBinDir();
	return @pmFilesInBinDirArr;
}

# dvir
# creates the file $Configuration::TASK_MAIN_DAG_NAME for the drawing task
sub initDag( $ ) {
	my $class = $_[0]; # the name of this class
		
	my $projectTaskDir = $class->getProjectDir();
	my $mainWorkDir = $class->getWorkDir();
	
	my $binDir = $Configuration::configfile{binDir};

	open PAINTER_PARAMS, "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME" || &General::reportOnErrorReleaseSemaphoresAndDie('Drawing.pm',$mainWorkDir,sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME" ,$!));

	my $drawHaploPainter = 0;
	my $drawMadeline = 0;
	my $translateFile = '';
	my $destDir = '';
	my $pedForDrawing = '';
	while (my $line = <PAINTER_PARAMS>) {
		my @lineData = split /\s+/, $line;
		
		if ($lineData[0] =~ /PainterData/) {
			$drawHaploPainter = 1;
		}
		
		if ($lineData[0] =~ /MadelineData/) {
			$drawMadeline = 1;
		}
		
		if ($lineData[0] =~ /DestDir/) {
			$destDir = $lineData[1];
		}
		
		if ($lineData[0] =~ /Translate/) {
			$translateFile = "$projectTaskDir\/$lineData[1]";
		}
		
		if ($lineData[0] =~ /PedNameForDrawing/) {
			$pedForDrawing = $lineData[1];
		}
	}
	close PAINTER_PARAMS;

	if ((!$drawHaploPainter) && (!$drawMadeline)) {
		&General::reportOnErrorReleaseSemaphoresAndDie('Drawing.pm',$mainWorkDir,"both Madeline & haploPainter weren't requested");
	}

	my $drawingJobsDagText = '';
	my $DrawingJobsNames = '';

	my @uniqueInputFilesNames = $class->getCondorJobInputFiles();
	
	if ($drawHaploPainter) {
		my $jobdir = "$mainWorkDir/$Configuration::HAPLO_PAINTER_DIR";
		mkdir $jobdir;

		my @jobInputFilesList = (
			"$jobdir\/$Configuration::HAPLO_PAINTER_INPUT_FILENAME",
			"$mainWorkDir\/$Configuration::FAMILIES_NUMBERS_FILENAME",
			@uniqueInputFilesNames);
		
		# AnalysisSubmissionFileData is from AnalysisSubmissionFile
		my $analysisSubmissionFileData = $class->getAnalysisSubmissionFileData();
		
		$analysisSubmissionFileData->executeableName ( "$binDir\/run_painter\.pl" );
		@{$analysisSubmissionFileData->executeableArgumentsList} = ();
		@{$analysisSubmissionFileData->executeableInputFilesList} = @jobInputFilesList;
		$analysisSubmissionFileData->jobSecondsToBeExecuted ( $class->getCondorJobSecondsToBeExecuted() );
		
		my $submissionFileName = "$jobdir\/Painter.sub";
		# creating the submission file		
		&AnalysisSubmissionFile::createAnalysisSubmissionFileAtCwd($analysisSubmissionFileData, $submissionFileName);

		$drawingJobsDagText.= "JOB HAPLO_PAINTER $submissionFileName DIR $jobdir\n".
							  "SCRIPT POST HAPLO_PAINTER $binDir\/PostDrawing.pl \$RETURN $Configuration::PAINTER_PROGRAM_HAPLO_PAINTER $destDir $mainWorkDir\n\n";
				  
		$DrawingJobsNames.= " HAPLO_PAINTER ";
	}

	if ($drawMadeline) {
		my $jobdir = "$mainWorkDir/$Configuration::MADELINE_DIR";
		mkdir $jobdir;

		my @jobInputFilesList = (
			"$jobdir\/$Configuration::MADELINE_INPUT_FILENAME", 
			"$mainWorkDir\/$Configuration::FAMILIES_NUMBERS_FILENAME", 
			@uniqueInputFilesNames);		
		
		# AnalysisSubmissionFileData is from AnalysisSubmissionFile
		my $analysisSubmissionFileData = $class->getAnalysisSubmissionFileData();

		$analysisSubmissionFileData->executeableName ( "$binDir\/run_madeline\.pl" );
		@{$analysisSubmissionFileData->executeableArgumentsList} = ();
		@{$analysisSubmissionFileData->executeableInputFilesList} = @jobInputFilesList;
		$analysisSubmissionFileData->jobSecondsToBeExecuted ( $class->getCondorJobSecondsToBeExecuted() );
		
		my $submissionFileName = "$jobdir\/Madeline.sub";
		# creating the submission file		
		&AnalysisSubmissionFile::createAnalysisSubmissionFileAtCwd($analysisSubmissionFileData, $submissionFileName);
		
		$drawingJobsDagText.= "JOB MADELINE $submissionFileName DIR $jobdir\n".
							  "SCRIPT POST MADELINE $binDir\/PostDrawing.pl \$RETURN $Configuration::PAINTER_PROGRAM_MADELINE $destDir $mainWorkDir\n\n";
		
		$DrawingJobsNames.= " MADELINE ";			  
	}
	
	&DatPedHandler::createFamiliesNumbersFile("$projectTaskDir\/$pedForDrawing");

	&DatPedHandler::createMadelineAndHaploPainterInputFiles("$projectTaskDir\/$pedForDrawing", $translateFile);
	
	my $projectName = $class->getProjectName();
	
	# there is only one DAG, in order to have less condor_dagman jobs
	open (GLOBAL, ">$mainWorkDir\/$Configuration::TASK_MAIN_DAG_NAME");
	print GLOBAL $drawingJobsDagText.
				 "JOB SIMPLE2 $Configuration::SIMPLE_SHORT_CONDOR_JOB\n".
				 "SCRIPT POST SIMPLE2 $binDir\/postJob.pl $mainWorkDir determine_job_status_of_the_task_by_sub_folders\n". 
				 "\n".
				 "PARENT $DrawingJobsNames CHILD SIMPLE2";
	close GLOBAL;
	
	# in order to mark that no input error was detected
	return undef	
}

# this method is called from "checkTasks.pl"
# and the method should report to the web server that the analysis has failed.
# the type of failure is that the analysis is still marked as running (at the running folder)
# but doesn't exists at the condor.
# each analysis has its own way to handle error.
sub reportOnFailureToWebServer( $ ) {
	my $class = $_[0]; # the name of this class
	my $errorText = $_[1];
	
	my $projectTaskDir = $class->getProjectDir();

	open PAINTER_PARAMS, "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME" || &General::reportOnErrorReleaseSemaphoresAndDie('Drawing.pm',$projectTaskDir,sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME" ,$!));

	my $destDir = '';
	while (my $line = <PAINTER_PARAMS>) {
		my @lineData = split /\s+/, $line;		
		
		if ($lineData[0] =~ /DestDir/) {
			$destDir = $lineData[1];
		}
	}
	close PAINTER_PARAMS;
		
	if ($destDir eq '') {
		&General::reportOnError('Drawing.pm','',"No Dest dir in painter param files, couldn't report draw pedigree error to web server");
	}
	else {
		my @commands = ("touch $destDir\/error");
	
		&General::executeSSHCommandsOnWebServer(\@commands);
	}
}

1;