# dvir , 31/07/2011
package GenomeBrowser;

use strict;

use Class::Struct;

use General;
use ProcessingBackend;

my $CUBE_BASE_SIZE = $ProcessingBackend::CUBE_BASE_SIZE;
our $BED_OUTPUT_FILENAME = $ProcessingBackend::BED_OUTPUT_FILE;
our $COLLECT_DATA_FILENAME = $ProcessingBackend::COLLECT_DATA_FILE;

# adds a link to genome browser to the current file handle.
# and creates the $BED_OUTPUT_FILENAME file at CWD
# this file will be created only if the analysis is on SNP input.

# assumption: "List of markers and their positions" contains positions of all markers in the dat file, in an ascending order.
# and may contain positions of markers which aren't at use at the analysis.

# another assumption is that the "Project Name" . for example Project Name: _(_[_(_PROJ=chrom_19_)_]_)_
# contains the chromozone number, when the analysis is on SNP input
sub addLinkToGenomeBrowser( $ \% \@ ) {
	my $htmlFileHandleRef = $_[0];
	my $variablesHashRef = $_[1];
	my $aggregateLodForAllFamiliesArrRef = $_[2];
	
	my $doesTheAnalysisIsOn_SNP_Input = &General::doesTheAnalysisIsOn_SNP_Input($variablesHashRef);
	
	my $linkText = "";
	if ($doesTheAnalysisIsOn_SNP_Input) {
		# we are using the absoulteWebDir in order that even that the user downloads 
		# the results data, than if he will have a connection to the web,
		# than he will be able to press the link
		my $releativeWebDir = $$variablesHashRef{newDir};
		my $absoulteWebDir = $ProcessingBackend::HTTP_HOST.$releativeWebDir;
		
		$linkText = "<a href=\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&pix=800&hgt.customText=$absoulteWebDir\/$BED_OUTPUT_FILENAME\" target=\"_blank\">View in genome browser</a>";
		
		# creating the $BED_OUTPUT_FILENAME file
		
		# figuring what chromozone we are running on. the chromozome name is part of the project name
		my $projName = $$variablesHashRef{ PROJ };
		my @chromNameData = split /chrom_/, $projName;
		my $chromName = $chromNameData[1];
		
		# figuring what is the postion of the first marker and the end marker
		my @markersArray = sort {$a <=> $b} @{$$variablesHashRef { usedMarkersId }};
		
		# the "-1" is because the indexes in perl arrays are from 0 
		# and in "usedMarkersId" are from 1.
		my $startMarkerIndex = $markersArray[0] - 1;
		my $endMarkerIndex = $markersArray[$#markersArray] - 1;
		
		my @allMarkersPositions = @{$$variablesHashRef { positions }};
		
		my $startPosText = $allMarkersPositions[$startMarkerIndex];
		my $endPosText = $allMarkersPositions[$endMarkerIndex];

		my @tempArr = split / : /, $startPosText; # splitting "rscFiveM3 : 34034740"
		my $startPos = $tempArr[1];
		@tempArr = split / : /, $endPosText; # splitting "rscFiveM3 : 34034740"
		my $endPos = $tempArr[1];
		
		open OUT_FILE, ">$BED_OUTPUT_FILENAME";
		
		print OUT_FILE 
			"browser position chr${chromName}:$startPos-$endPos\n".
			"browser hide all\n".
			"browser dense ruler\n".
			"browser pack recombRate\n".
			"browser full knownGene\n".
			"track name=\"LODscore\" description=\"LOD Score\" visibility=3 itemRgb=\"On\"\n";			
		
		my $currMarkerLodIndex = 0;
		# passing over all the markers that participate in the analysis
		foreach my $currMarkerIndex ( @markersArray ) {
			my $lod = $$aggregateLodForAllFamiliesArrRef[$currMarkerLodIndex];
			
			if (defined $lod) {
			
				# the indexes in the positions array are from 0 and not from 1
				my $currMarkerIndexFrom0 = $currMarkerIndex - 1;
				
				my $markerPosText = $allMarkersPositions[$currMarkerIndexFrom0];
				@tempArr = split / : /, $markerPosText; # splitting "rscFiveM3 : 34034740"
				my $markerPos = $tempArr[1];			
				
				my $markerPosEnd = $markerPos + $CUBE_BASE_SIZE;	
				
				my $color = &ProcessingBackend::getLODcolor($lod);
				
				print OUT_FILE "chr${chromName}\t$markerPos\t$markerPosEnd\t$lod\t0\t.\t$markerPos\t$markerPosEnd\t$color\n";
			}
			
			++ $currMarkerLodIndex;
		}
		close OUT_FILE;
	}
	# here the analysis is not on SNP input
	else {
		$linkText = "<font color=\"#999999\"><u>View in genome browser</u></font>&nbsp;(No data. Available for snp input only)";
	}
	
	print $htmlFileHandleRef $linkText;
}

struct lodDataForCollectData =>
[
	markerNum => '$',
	lod	=> '$', # the lod of the marker
	stdErr => '$', # the standard error of the computation
];

# dvir 2011/08/09
# this method will be used for the "data browser" button in the web server
sub create_collect_date_txt( \@ ) {
	open(COL_DATA,">$COLLECT_DATA_FILENAME");
	
	#print COL_DATA "$id\t$name\t$distInCm\t$LOD\n";
	close COL_DATA;
}

1;