package GlAuto;

use strict;
use AnalysisWith_modefile;
use Configuration;
use General;
use ProcessingBackend;
use SharedConsts;

# this class inherits methods from AnalysisWith_modefile
our @ISA = ('AnalysisWith_modefile');

# those files will be copied to the web server
my $HTML_OUTPUT_FILENAME = $ProcessingBackend::OUTPUT_FILE;
my @RESULTS_FILE_LIST = ($HTML_OUTPUT_FILENAME);

# increase the number of iterations to the next step
sub getNextNumOfIterations($) {
	my $currNumOfIterations = $_[0];
	return $currNumOfIterations * 2;
}

# methods we override from AnalysisTemplateClass.pm :

# returning an array of files names
# those files will be copied to the machine on which the job will run
# it shouldn't contain the dat & ped files, which will be added by the calling method.
sub getCondorJobInputFiles() {
	my $class = $_[0];
	
	my $projectDir = $class->getProjectDir();
	my $binDir = $Configuration::configfile{ binDir };
	return 
		("$Configuration::MORGAN_LM_MARKERS_PROGRAM_DIR\/gl_auto.64bit.version3.0.3",
		"$binDir\/superlink2lnkg.pm",
		"$Configuration::MORGAN_LM_MARKERS_PROGRAM_DIR\/lnkg2lmm.pl",
		"$projectDir\/$SharedConsts::MORGAN_MORE_PARAMS_FILENAME", # more prams for executeable
		"$projectDir\/$Configuration::ANALYSIS_PARAMETERS_MODEFILE_FILENAME");
	# copying $Configuration::ANALYSIS_PARAMETERS_MODEFILE_FILENAME in order to access those params:
	# $SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_BURN_IN_PERCENTAGE
	# $SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_SEQUENTIAL_IMPUTATION_REALI
}

# mnorgan al_auto runs only on 64-bit machines
sub canRunOn32BitMachine() { return 0; }

# returning an array of files names which contain the results of the analysis, 
# those files should be copied to the web server
# TODO: fill the function after there will be a script creating the result.
sub getFileListToBeCopiedToWebServer() {
	my $class = $_[0];
	
	my $projectTaskDir = $class->getProjectDir();
	my $workDir = $class->getWorkDir();
	
	return @RESULTS_FILE_LIST;
}


# returning an array of the names of the output files of a multi window per job run
sub getJobOutputFiles() { return $Configuration::GL_AUTO_MULTI_WIN_RES; }

sub getJobErrorFile() { return $Configuration::GL_AUTO_MULTI_WIN_ERR; }

sub getCondorJobSecondsToBeExecuted() {
	# morgan should not use this method. TODO: cehck if leave the function or delete it's context!!!
	&General::reportOnErrorReleaseSemaphoresAndDie('GlAuto.pm',`pwd`,
					"getCondorJobSecondsToBeExecuted: this method shouldn't be used.\n".
					"the run time should be given as parameter at the modefile.txt");
}


######
sub preformUniqueChecks( \% ) {
	my $class = $_[0]; # the name of this class
	my $variablesHashRef = $_[1];
	
	if (! (defined $$variablesHashRef{num_dups_per_job} && defined $$variablesHashRef{run_time_of_window} && 
		   defined $$variablesHashRef{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS} && 
		   defined $$variablesHashRef{$SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS} )) {
		&General::reportOnErrorReleaseSemaphoresAndDie('GlAuto.pm', `pwd`, 
			"At least one of following params is missing: num_dups_per_job run_time_of_window $SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS $SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS.\n");
	}
	
	# since we compute the lod between two markers, than if the jump is 1
	# than it is an error
	if (defined $$variablesHashRef{jump} && $$variablesHashRef{jump} eq 1) {
			&General::reportOnErrorReleaseSemaphoresAndDie('MorganLmMarkers.pm',`pwd`,
			"since we compute the lod between two markers, than the jump must be at least 2, in order to have 2 markers in the dat file.\n");
	}
}

our $INITIAL_OUTPUT_CSV_FILENAME = 'initial.out.csv';
our $FINAL_OUTPUT_CSV_FILENAME = 'final.out.csv';
our $MAX_NUM_OF_ITERATIONS_FILENAME = 'maxNumOfIterations.txt';
our $OUTPUT_DIRNAME = 'outputData';
our $FAMILIES_DIR_LIST_SUFFIX = '.outputDirList.txt';

# creating the initial output files for each job
# will run once from the function that creates the dag
#TODO: for now do nothing - fill after the GlAuto output project will be finished.
sub createInitialOutputFiles(\$ \$ \%) { }

# that will contain the results of the run of the current job.
# the cwd is the job dir
#TODO: for now do nothing - fill after the GlAuto output project will be finished.
sub createFinalOutputFileForCurrentJob() { }
1;