# dvir 2011/03
# dvir 2011/11
package HaplotypingHtml;

use strict;

use Cwd;
use Class::Struct;

use Configuration;
use ProcessingBackend;
use DatWrapper;

struct PersonHaploData =>
[
	paterna	=> '@', # the haplotype of chromoze, the person recevied from its father
	materna => '@', # the haplotype of chromoze, the person recevied from its mother
];

our $RECOMBINATION_CHAR = 'x';

# output files names
our $HTML_HAPLO_TABLES_HTML_FILE_NAME = $ProcessingBackend::OUTPUT_RESULTS_FILE;
my $HTML_HAPLO_TABLES_PDF_FILE_NAME = $ProcessingBackend::OUTPUT_RESULTS_FILE_FOR_PDF;
my $HTML_MARKERS_LIST_HTML_FILE_NAME = $ProcessingBackend::OUTPUT_NAVIGATION_FILE;

# those files will be copied to the web server
our @RESULTS_FILE_LIST = ($HTML_HAPLO_TABLES_HTML_FILE_NAME, $HTML_HAPLO_TABLES_PDF_FILE_NAME, $HTML_MARKERS_LIST_HTML_FILE_NAME);

my $CSS_FILE_NAME = 'haplotypingResults.css';
my $JAVASCRIPT_FILE_NAME = 'haplotypingResults.js';
my $HAPLOTYPIMG_RESULTS_FOLDER_NAME = 'haplotypingResults';
my $JAVASCRIPT_FILE_REL_PATH = "$HAPLOTYPIMG_RESULTS_FOLDER_NAME/$JAVASCRIPT_FILE_NAME";
my $CSS_FILE_REL_PATH = "$HAPLOTYPIMG_RESULTS_FOLDER_NAME/$CSS_FILE_NAME";

my $FRAMESET_HTML_FILE_NAME = $ProcessingBackend::OUTPUT_FILE;

# the html file is hidden in order that when the user presses on download, 
# than the downloaded zip won't contatin this html file, in the 'haplotypingResults/' folder.
my $FRAMESET_HTML_FILE_REL_PATH = "$HAPLOTYPIMG_RESULTS_FOLDER_NAME/.$FRAMESET_HTML_FILE_NAME";
	
# those commands will be executed on webserver
sub getCommandsToBeExecutedOnWebServer($) {
	my $analysisDirAtWebServer = $_[0];
	
	# the commands are to copy a directory with the css, js to the Project Dir
	# and to create a soft link to the frameset html file.
	# the rm is for cases when the files were allready copied,
	# so in order that the current copy won't fail, than we are deleting the to be copied files
	my @COMMANDS_TO_BE_EXECUTED_ON_WEB_SERVER = 
	("rm -rf $analysisDirAtWebServer/$HAPLOTYPIMG_RESULTS_FOLDER_NAME",
	 "rm -rf $analysisDirAtWebServer/$FRAMESET_HTML_FILE_NAME",
	 "cp -R $Configuration::configfile{web_main_dir}/staticHTML/$HAPLOTYPIMG_RESULTS_FOLDER_NAME $analysisDirAtWebServer",
	 "ln -s $analysisDirAtWebServer/$FRAMESET_HTML_FILE_REL_PATH $analysisDirAtWebServer/$FRAMESET_HTML_FILE_NAME");
}

				
# in order that the HTML FOR THE PDF won't contain links, so we will delete all links  in that html ($htmlResult2)
# (all links = all the text between $HIDDEN_PDF_START and $HIDDEN_PDF_END)
my $HIDDEN_PDF_START = "#HS#";
my $HIDDEN_PDF_END = "#HE#";

# short unique names of the current affection status, will be used for the "hide" options:
# ta = typed_affected, th = typed_healthy, uta = untyped_affected, uth = untyped_healthy, unk = unknown
my $AFFECTION_STATUS_TYPED_AFFECTED_UNIQUE_NAME = "ta";
my $AFFECTION_STATUS_TYPED_HEALTHY_UNIQUE_NAME = "th";
my $AFFECTION_STATUS_UNTYPED_AFFECTED_UNIQUE_NAME = "uta";
my $AFFECTION_STATUS_UNTYPED_HEALTHY_UNIQUE_NAME = "uth";
my $AFFECTION_STATUS_UNKNOWN_UNIQUE_NAME = "unk";

my $ROW1 = "R1"; # indicate a paterna row
my $ROW2 = "R2"; # indicate a materna row
	
my $resultStr = '';
	
# creating HTML pages at CWD, from the haplotype data structre
sub createHtmlPagesWithHaplotypeData( \@ \@ \% \% \%) {

	my $headline = $_[0]; # an headline
	my $familiesWithSuccessfulAnalysisArrRef = $_[1]; # list of families numbers, which the analysis was successful for them.
	
	# A data structrue that will hold the results for people from all the successful families
	# each entry in the hash will be "famNum personNum" -> struct PersonHaploData (resides at HaplotypingHtml.pm)
	my $peopleHaploDataHashRef = $_[2];
	my $familiesPerMarkersHashRef = $_[3];
	my $markersPerFamiliesHashRef = $_[4];

	my $analysis = new Analysis(getcwd());
	my $projectTaskDir = $analysis->getProjectDir();
	&ProcessingBackend::setLogFiles("$projectTaskDir\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	
	# if the run was on windows, than this parameter will get the window size
	# if the run wasn't on windows, than this parameter can be undef
	my $windowSize = $variables{jump};
	
	my $datFileName = "$projectTaskDir\/$variables{datafile}";
	my $pedFileName = "$projectTaskDir\/$variables{pedfile}";
	my $peopleNamesFileName = "$projectTaskDir\/$Configuration::PEOPLE_NAMES_FILE_NAME";
	
	my $projectTitle = $variables{PROJ};

	struct PeopleByAffectionStatus =>
	[
		typed_affected    => '@',
		untyped_affected  => '@',
		typed_healthy     => '@',
		untyped_healthy   => '@',
		unknown           => '@',
	];

	struct PersonPedigreeData =>
	[
		name => '$',
		fatherNum => '$',
		motherNum  => '$',
		fatherName => '$',
		motherName  => '$',		
		sex  => '$',
	];
	my %peoplePedigreeData = ();	
	my $datWrapper = new DatWrapper($datFileName);
	my $numOfMarkers = $datWrapper->getNumOfMarkers();

	# the minus 1, is because we start counting from the first index of allel
	my $indexOfSecondAllelOfLastMarker = $Configuration::PED_FILE_COL_INDEX_FIRST_MARKER + $numOfMarkers*2 - 1;

	# dvir 27/08/2010
	# since that a person is typed in snp&ped files, if he appears in the snp file
	# so in order to make the check more quiqer, we will check if the first 10 allels appears.
	my $minNumOfKnownAllelesInPedFileToBeConsideredTyped;
	if ($numOfMarkers > 5) {
		$minNumOfKnownAllelesInPedFileToBeConsideredTyped = 10; # each marker has 2 allels (2*5=10)
	}
	else {
		$minNumOfKnownAllelesInPedFileToBeConsideredTyped = $numOfMarkers * 2; # total num of allels
	}

	# checking if there is a file with names of the people
	my %peopleNames = ();
	if (-e $peopleNamesFileName) {
		# updating the people names
		open PEOPLE_NAMES, $peopleNamesFileName;
		while (my $line = <PEOPLE_NAMES>) {
			my @line = split(/\s+/,$line);
			my $family = $line[0];
			my $personNum = $line[1];
			my $personName = $line[2];
			$peopleNames{"$family $personNum"} = $personName;
		}
		close $peopleNamesFileName;
	}
	
	
	# the keys of the hash is the family number
	# each hash value will be a struct PeopleByAffectionStatus (4 arrays of the people of the current family)
	# for example, if a person is type affected, than he will be in the typed affected array.
	my %peopleByFamily = ();
	# initiating %peopleByFamily , %peoplePedigreeData
	my ($currPersonFamily, $currPersonId, $currPersonAffectionStatus, $currPersonIsTyped, @currPerson, @currPersonAllels, $currPersonNumOfKnownAllels,
		$personHashKey, $currPersonFatherNum, $currPersonMotherNum);
		
	open PED_FILE, $pedFileName;
	while (my $currPersonLine = <PED_FILE>) {
		@currPerson = split(/\s+/,$currPersonLine);
		# calc the person relevant fields
		$currPersonFamily = $currPerson[$Configuration::PED_FILE_COL_INDEX_FAMILY];
		$currPersonId = $currPerson[$Configuration::PED_FILE_COL_INDEX_ID];
		$currPersonAffectionStatus = $currPerson[$Configuration::PED_FILE_COL_INDEX_AFFECTION_STATUS];
		$currPersonFatherNum = $currPerson[$Configuration::PED_FILE_COL_FATHER_ID];
		$currPersonMotherNum = $currPerson[$Configuration::PED_FILE_COL_MOTHER_ID];
				
		my $personPedigreeData = PersonPedigreeData->new();
		$personPedigreeData->sex($currPerson[$Configuration::PED_FILE_COL_SEX]);
		$personPedigreeData->fatherNum($currPersonFatherNum);
		$personPedigreeData->motherNum($currPersonMotherNum);
		
		# checking with exists, cause I want to be sure that we have the name
		if (exists $peopleNames{"$currPersonFamily $currPersonId"}) {
			#print 'a: '.$currPersonId."\n";
			#print 'b: '.$peopleNames{"$currPersonFamily $currPersonId"}."\n";
			$personPedigreeData->name($peopleNames{"$currPersonFamily $currPersonId"}."($currPersonId)");
			#print "c\n";
		}
		else {
			$personPedigreeData->name($currPersonId);
		}		
		
		if ($currPersonFatherNum != $Configuration::UNKNOWN_PERSON_NUM && exists $peopleNames{"$currPersonFamily $currPersonFatherNum"}) {
			$personPedigreeData->fatherName($peopleNames{"$currPersonFamily $currPersonFatherNum"}."($currPersonFatherNum)");
		}
		else {
			$personPedigreeData->fatherName($currPersonFatherNum);
		}		
		
		if ($currPersonFatherNum != $Configuration::UNKNOWN_PERSON_NUM && exists $peopleNames{"$currPersonFamily $currPersonMotherNum"}) {
			$personPedigreeData->motherName($peopleNames{"$currPersonFamily $currPersonMotherNum"}."($currPersonMotherNum)");
		}
		else {
			$personPedigreeData->motherName($currPersonMotherNum);
		}
		
		$personHashKey = $currPersonFamily.' '.$currPersonId;
				
		# adding the person to the list , acording to his affection status
		
		# if the person is the first from his family, than we need to create (by new) the entry in the hash
		if (! exists $peopleByFamily{$currPersonFamily} ) {
			$peopleByFamily{$currPersonFamily} = PeopleByAffectionStatus->new();
		}
		
		# we need to check if the current person is typed, only if the current person is healthy or affected.
		if ($currPersonAffectionStatus == $Configuration::AFFECTION_STATUS_HEALTHY or
			$currPersonAffectionStatus == $Configuration::AFFECTION_STATUS_AFFECTED) {

			$currPersonIsTyped = 0;
			$currPersonNumOfKnownAllels = 0;
			# count how many known allels the person has
			for (my $currAllel=$Configuration::PED_FILE_COL_INDEX_FIRST_MARKER;$currAllel<= $indexOfSecondAllelOfLastMarker;++$currAllel) {
				# checking if the current person allel is unknown
				if ($currPerson[$currAllel] != $Configuration::ALLEL_VALUE_UNKNOWN) {
					++$currPersonNumOfKnownAllels;
				}
				if ($minNumOfKnownAllelesInPedFileToBeConsideredTyped <= $currPersonNumOfKnownAllels) {
					$currPersonIsTyped = 1;
					last;
				}
			}
		}

		# adding the person to the relevant array by its family & affection status
		# the adding is done by push, so that the order of the people in the table will be as the order of the people in the
		# ped file
		if ($currPersonAffectionStatus == $Configuration::AFFECTION_STATUS_HEALTHY) {
			if ($currPersonIsTyped) {
				push @{ $peopleByFamily{$currPersonFamily}->typed_healthy }, $currPersonId;
			}
			else {
				push @{ $peopleByFamily{$currPersonFamily}->untyped_healthy }, $currPersonId;
			}			
		}
		elsif ($currPersonAffectionStatus == $Configuration::AFFECTION_STATUS_AFFECTED) {
			if ($currPersonIsTyped) {
				push @{ $peopleByFamily{$currPersonFamily}->typed_affected }, $currPersonId;
			}
			else {
				push @{ $peopleByFamily{$currPersonFamily}->untyped_affected }, $currPersonId;
			}
		}
		else {
			push @{ $peopleByFamily{$currPersonFamily}->unknown }, $currPersonId;	
		}
		$peoplePedigreeData{$personHashKey} = $personPedigreeData;
	}
	close PED_FILE;	
	
	# start building the htmls
	
	# adding some javascript vars, for the dynamic actions
	my $javaScriptVars = "var peoplePerFamily = [];\n";
	my $familiesAndPeople = '';
	my $familiesNums;
	foreach my $currFamily (@$familiesWithSuccessfulAnalysisArrRef) {
		my $peopleByAffectionStatusInFamily = $peopleByFamily{$currFamily};

		$familiesNums .= $currFamily.',';
		my @peopleInFamily =
			(@{$peopleByAffectionStatusInFamily->typed_affected},
			 @{$peopleByAffectionStatusInFamily->untyped_affected},
			 @{$peopleByAffectionStatusInFamily->typed_healthy},
			 @{$peopleByAffectionStatusInFamily->untyped_healthy},
			 @{$peopleByAffectionStatusInFamily->unknown});
		
		# var per family with the family people
		$javaScriptVars .= "peoplePerFamily[$currFamily] = [".join (",", @peopleInFamily)."];\n";
		
		my $peopleInFamilyStr .= join "],[$currFamily,", @peopleInFamily;
		$familiesAndPeople .= "[$currFamily,$peopleInFamilyStr],";
	}
	
	# making sure that there are families with successful analysis
	if (scalar(@$familiesWithSuccessfulAnalysisArrRef) > 0) {
		# removing the last ','
		$familiesAndPeople =~ s/,$//;
		$javaScriptVars .= 'peopleAndFamilies = ['.$familiesAndPeople."];\n";
		
		# removing the last ','
		$familiesNums =~ s/,$//;
		$javaScriptVars .= 'families = ['.$familiesNums."];\n";
	}
	
	$javaScriptVars .= "var familiesPerMarker = [];\n";
	while ( my ($markerNum, $familiesArrRef) = each(%$familiesPerMarkersHashRef) ) {
	   $javaScriptVars .= "familiesPerMarker[$markerNum] = [".join(',',@$familiesArrRef)."];\n";
	 }
	$javaScriptVars .= "var markersPerFamily = [];\n";
	while ( my ($familyNum, $markersArrRef) = each(%$markersPerFamiliesHashRef) ) {
	   my @familyMarkers = sort {$a <=> $b} @$markersArrRef;
	   $javaScriptVars .= "markersPerFamily[$familyNum] = [".join(',',@familyMarkers)."];\n";
	 }	 
	
	# end with the javascript vars		
	$resultStr .= 
		"<html>
		<head>
			<script type=\"text/javascript\">
				var row1Suffix = '$ROW1';
				var row2Suffix = '$ROW2';
				var uniqueAffectionNames = [
					'$AFFECTION_STATUS_TYPED_AFFECTED_UNIQUE_NAME',
					'$AFFECTION_STATUS_TYPED_HEALTHY_UNIQUE_NAME',
					'$AFFECTION_STATUS_UNTYPED_AFFECTED_UNIQUE_NAME',
					'$AFFECTION_STATUS_UNTYPED_HEALTHY_UNIQUE_NAME',
					'$AFFECTION_STATUS_UNKNOWN_UNIQUE_NAME'];
				$javaScriptVars
			</script>
			<script type=\"text/javascript\" src=\"$JAVASCRIPT_FILE_REL_PATH\">
			</script>
			<link rel=\"stylesheet\" type=\"text/css\" href=\"$CSS_FILE_REL_PATH\"> 
		</head>
		<body>
		<center><p><h1><u><b>Results of Haplotyping for project $projectTitle</b></u></h1></p></center>";	
	
	$resultStr .= $headline;
	
	# checking if there are families with Successful jobs
	if (scalar(@$familiesWithSuccessfulAnalysisArrRef) > 0) {
		
		$resultStr .= "<p align=\"ltr\">";
		if (scalar(@$familiesWithSuccessfulAnalysisArrRef) > 1) {
			$resultStr .= "Haplotypes for each family appear separately.<br/>";
		}
		$resultStr .= 
			"The included haplotypes are of:
			<OL style=\"padding-top:0em;margin-top:0em;margin-bottom:0em\"><LI>typed affected persons</LI>
			<LI>typed healthy persons</LI>
			<LI>untyped affected persons</LI>
			<LI>untyped healty persons</LI>
			<LI>persons with unknown affection status</LI>
			</OL>";


		foreach my $currFamily (@$familiesWithSuccessfulAnalysisArrRef) {
			# checking if there is more than one family with Successful jobs
			if (scalar(@$familiesWithSuccessfulAnalysisArrRef) > 1) {
				$resultStr .= "<h2>Family: $currFamily</h2>";
			}
			
			# defaultHeaderString of columns
			# the num of markers in the analysis
			my $markersTableHeaderStr = '';
			
			foreach my $currMarkerNum (@{$$markersPerFamiliesHashRef{$currFamily}}) {
				my $markerName = $datWrapper->getMarkerName($currMarkerNum);
				$markersTableHeaderStr .= 
					"<th id=\"^^FAM^^_^^AFF^^_COL_$currMarkerNum\" title=\"$markerName\">M$currMarkerNum<br/>".
					$HIDDEN_PDF_START.
					"<a href=\"javascript:void(0);\" onclick=\"hideCol($currMarkerNum);\">hide</a><br/>".
					"<a href=\"javascript:void(0);\" onclick=\"hideLeftCols($currMarkerNum);\">L</a>".
					' '.
					"<a href=\"javascript:void(0);\" onclick=\"hideRightCols($currMarkerNum);\">R</a>".
					$HIDDEN_PDF_END.
					"</th>";			
			}
		
			printTable($peopleHaploDataHashRef, \%peoplePedigreeData, $AFFECTION_STATUS_TYPED_AFFECTED_UNIQUE_NAME,"Typed Affected Persons","typed affected people", \@{ $peopleByFamily{$currFamily}->typed_affected }, $currFamily, \$markersTableHeaderStr,$markersPerFamiliesHashRef);
			printTable($peopleHaploDataHashRef, \%peoplePedigreeData, $AFFECTION_STATUS_TYPED_HEALTHY_UNIQUE_NAME,"Typed Healthy Persons","typed healthy people", \@{ $peopleByFamily{$currFamily}->typed_healthy }, $currFamily, \$markersTableHeaderStr,$markersPerFamiliesHashRef);
			printTable($peopleHaploDataHashRef, \%peoplePedigreeData, $AFFECTION_STATUS_UNTYPED_AFFECTED_UNIQUE_NAME,"Untyped Affected Persons","untyped affected people", \@{ $peopleByFamily{$currFamily}->untyped_affected }, $currFamily, \$markersTableHeaderStr,$markersPerFamiliesHashRef);
			printTable($peopleHaploDataHashRef, \%peoplePedigreeData, $AFFECTION_STATUS_UNTYPED_HEALTHY_UNIQUE_NAME,"Untyped Healthy Persons","untyped healthy people", \@{ $peopleByFamily{$currFamily}->untyped_healthy }, $currFamily, \$markersTableHeaderStr,$markersPerFamiliesHashRef);
			printTable($peopleHaploDataHashRef, \%peoplePedigreeData, $AFFECTION_STATUS_UNKNOWN_UNIQUE_NAME,"Persons with Unknown Affection Status", "people with unknown affection status" , \@{ $peopleByFamily{$currFamily}->unknown }, $currFamily, \$markersTableHeaderStr,$markersPerFamiliesHashRef);
		}
	}
		
	my $resultStrCopy = $resultStr;
	$resultStr =~ s/$HIDDEN_PDF_START//g;
	$resultStr =~ s/$HIDDEN_PDF_END//g;
	my $strToDelete = $HIDDEN_PDF_START.'.+?'.$HIDDEN_PDF_END;
	$resultStrCopy =~ s/$strToDelete//gs;
	open HTML_HAPLO_TABLES, ">$HTML_HAPLO_TABLES_HTML_FILE_NAME";
	print HTML_HAPLO_TABLES $resultStr;
	close HTML_HAPLO_TABLES;
	open PDF_HAPLO_TABLES, ">$HTML_HAPLO_TABLES_PDF_FILE_NAME";
	print PDF_HAPLO_TABLES $resultStrCopy;
	close PDF_HAPLO_TABLES;

	# here we have finished with $HTML_HAPLO_TABLES_HTML_FILE_NAME	
	
	# the frameset (for HTML_MARKERS_MAP, HTML_HAPLO_TABLES) is allwayes the same ,so there isn't a need to create it uniuqely
}

sub printTable(\% \% $ $ $ \@ $ \$ ) {
	my $peopleHaploDataHashRef = $_[0];
	my $peoplePedigreeDataHashRef = $_[1];
	my $affectionStatusUniqueName = $_[2];
	my $tableHeadline = $_[3];
	my $headlineWhenPeopleArrayIsEmpty = $_[4];
	my $peopleArrRef = $_[5]; # a reference to an array of people at the current affection status
	# @$peopleArrRef may be :
	# 	1. all the typed affected people in the family
	# 	2. all the untyped affected people in the family
	# 	3. all the typed healthy people in the family
	# 	4. all the untyped healthy people in the family	
	# 	5. all the people in the family	
	# each entry in the array is the id of the person
	# the entries are ordered acording to the person position in the ped file	
	my $currFamily = $_[6];
	my $markersTableHeaderStrRef = $_[7];
	my $markersPerFamiliesHashRef = $_[8];	
	
	# checking if at the current affection status, the people list is empty
	if (scalar(@$peopleArrRef) == 0) {
		$resultStr .= 
			"<div>There were no $headlineWhenPeopleArrayIsEmpty in pedigree.</div>";
		return;
	}
	
	# if we reach to here, than there are people at the current affextion status

	my $markersTableHeaderStr = $$markersTableHeaderStrRef;
	$markersTableHeaderStr =~ s/\^\^FAM\^\^/$currFamily/g;
	$markersTableHeaderStr =~ s/\^\^AFF\^\^/$affectionStatusUniqueName/g;
	my $peopleInFamiliyWithCurrAffectionStatus = join ",", @$peopleArrRef;
	
	# printing the table header
	$resultStr .= 
		"<h3 align=center>Computed Haplotypes of $tableHeadline</h3>\n".
        "<table border=1 align=center>\n".
		"<div style=\"visibility:hidden;color:red;\" id=\"MES_${currFamily}_$affectionStatusUniqueName\">Some parts of the table are hidden.</div>\n".		
		"<THEAD>\n".
		"${HIDDEN_PDF_START}<th style=\"font-weight:normal;\">".
		"<a href=\"javascript:void(0);\" onclick=\"showEntireTable($currFamily,'$affectionStatusUniqueName',[$peopleInFamiliyWithCurrAffectionStatus]);\">".
		"show<br/>all</a></th>${HIDDEN_PDF_END}\n".
		"<th>Person</th><th>Father</th><th>Mother</th><th>M/F</th><th>A/H</th>\n".
		"$markersTableHeaderStr\n".
		"</THEAD><TBODY>\n";
		
	my %AFFECTION_STATUS = ($AFFECTION_STATUS_UNKNOWN_UNIQUE_NAME          => "Unknown",
							$AFFECTION_STATUS_TYPED_HEALTHY_UNIQUE_NAME    => "Healthy",
							$AFFECTION_STATUS_UNTYPED_HEALTHY_UNIQUE_NAME  => "Healthy",
							$AFFECTION_STATUS_TYPED_AFFECTED_UNIQUE_NAME   => "Affected",
							$AFFECTION_STATUS_UNTYPED_AFFECTED_UNIQUE_NAME => "Affected");

	my %SEX_NAMES = ($Configuration::SEX_MALE => "M",
				 	 $Configuration::SEX_FEMALE => "F");

	my $affStatusStr = $AFFECTION_STATUS{$affectionStatusUniqueName};
	
	foreach my $currPerson (@$peopleArrRef) {
		my $presonPedigreeData = $$peoplePedigreeDataHashRef{"$currFamily $currPerson"};
		
		$resultStr .= 
			"<tr class=\"m1\" id=r1_${currFamily}_$currPerson>\n".
			"${HIDDEN_PDF_START}<td class=\"per\" rowspan=2><a href=\"javascript:void(0);\" onclick=\"".
				"hideRow($currFamily,$currPerson,'$affectionStatusUniqueName');\">".
			"hide</a></td>\n${HIDDEN_PDF_END}".  
			"<td class=\"per\" rowspan=2>".$presonPedigreeData->name."</td>\n".
			"<td class=\"per\" rowspan=2>".$presonPedigreeData->fatherName."</td>\n".
			"<td class=\"per\" rowspan=2>".$presonPedigreeData->motherName."</td>\n".
			"<td class=\"per\" rowspan=2>".$SEX_NAMES{$presonPedigreeData->sex}."</td>\n".
			"<td class=\"per\" rowspan=2>$affStatusStr</td>\n";
			
		# adding the person allels
		my $presonHaploData = $$peopleHaploDataHashRef{"$currFamily $currPerson"};
		
		# 
			# my $markerName = $datWrapper->getMarkerName($currMarkerNum);
			# $markersTableHeaderStr .= 
				# "<th id=\"^^FAM^^_^^AFF^^_COL_$currMarkerNum\" title=\"$markerName\">M$currMarkerNum<br/>".
				# $HIDDEN_PDF_START.
				# "<a href=\"javascript:void(0);\" onclick=\"hideCol($currMarkerNum);\">hide</a><br/>".
				# "<a href=\"javascript:void(0);\" onclick=\"hideLeftCols($currMarkerNum);\">L</a>".
				# ' '.
				# "<a href=\"javascript:void(0);\" onclick=\"hideRightCols($currMarkerNum);\">R</a>".
				# $HIDDEN_PDF_END.
				# "</th>";
		# }
			
		my @allels = @{$presonHaploData->paterna};
		my $allelIndex = 0;
		foreach my $currMarkerNum (@{$$markersPerFamiliesHashRef{$currFamily}}) {
			my $allelValue = $allels[$allelIndex];
			$resultStr .= "<td id=\"${currFamily}_${currPerson}_COL_${currMarkerNum}_$ROW1\" >$allelValue</td>\n";
			++$allelIndex;
		}
		$resultStr .= 
			"</tr>\n".
			"<tr class=\"m2\" id=\"r2_${currFamily}_$currPerson\">\n";
		
		@allels = @{$presonHaploData->materna};		
		
		$allelIndex = 0;
		foreach my $currMarkerNum (@{$$markersPerFamiliesHashRef{$currFamily}}) {
			my $allelValue = $allels[$allelIndex];
			$resultStr .= "<td id=\"${currFamily}_${currPerson}_COL_${currMarkerNum}_$ROW2\" >$allelValue</td>\n";
			++$allelIndex;
		}
		$resultStr .= "</tr>\n";			
	}
	
	$resultStr .= "</TBODY></table>\n";
}

# printing the markerList to $HTML_MARKERS_LIST_HTML_FILE_NAME
sub createMarkersMap( $ ) {
	my $datWrapperRef = $_[0];
	my $markersNumsArrRef = $_[1];

	open HTML_MARKERS_MAP,">$HTML_MARKERS_LIST_HTML_FILE_NAME";
	print HTML_MARKERS_MAP
		"<html>\n".
		"<body bgcolor=\"#F0FFFF\">\n".
		"<b><h3 align=center> Mapping table:</h3></b>\n".
		"<table border=\"1\" align=\"center\" cellpadding=\"5\">";
	foreach my $currMarkerNum (@$markersNumsArrRef) {
		my $markerName = $$datWrapperRef->getMarkerName($currMarkerNum);
		print HTML_MARKERS_MAP
			"<tr>".
			"<td align=\"center\"> M$currMarkerNum </td>".
			"<td align=\"center\">$markerName</td>".
			"</tr>";
		}
	print HTML_MARKERS_MAP "</table>\n</body>\n</html>";
	close HTML_MARKERS_MAP;
}
1;