# dvir 2011/03 2011/10
package HaplotypingLars;

use strict;
use AnalysisWith_modefile;
use Configuration;
use HaplotypingHtml;
use General;
use JobStatus;
use POSIX qw(strftime); # for strftime

# this class inherits methods from AnalysisWith_modefile
our @ISA = ('AnalysisWith_modefile');

# methods we override from AnalysisTemplateClass.pm :

# returning an array of files names which contain the results of the analysis, 
# those files should be copied to the web server
sub getFileListToBeCopiedToWebServer() {
	# if there was an error with an html, than although it was copied allready, 
	# I'm coping it again in order not to make an error at the General::sendFilesToWebServer command
	# at postJob.pl
	if (-e $HaplotypingHtml::HTML_HAPLO_TABLES_HTML_FILE_NAME) {
		return @HaplotypingHtml::RESULTS_FILE_LIST;
	}
	
	if (-e "$Configuration::REPORT_HTML_FILENAME") {
		return ("$Configuration::REPORT_HTML_FILENAME");
	}
	
	if (-e "$Configuration::RUN_REPORT_HTML_FOLDERNAME\/$Configuration::REPORT_HTML_FILENAME") {
		return ("$Configuration::RUN_REPORT_HTML_FOLDERNAME\/$Configuration::REPORT_HTML_FILENAME");
	}	
}

# returning an array of commands,
# which will be executed on the web server when the task is about to end
sub getCommandsToBeExecutedOnWebServer($) {
	my $class = $_[0]; # the name of this class
	my $analysisDirAtWebServer = $_[1];
	# if there isn't an html table, than no commands should be executed
	if (-e $HaplotypingHtml::HTML_HAPLO_TABLES_HTML_FILE_NAME) {
		return &HaplotypingHtml::getCommandsToBeExecutedOnWebServer($analysisDirAtWebServer);
	}
	else {
		return ();
	}
}

# the program code needed in order that the executeable will run
# the program code will be planted at the first line of the dat file
sub getDatFileProgramCode() {
	return 10;
}

# returning an array of the names of the output files of the job
sub getJobOutputFiles() {
	my @outputFiles = ($Configuration::RESULT_FILE_HAPLOTYPE_LARS, $Configuration::RESULT_FILE_ASSIGNMENT_HAPLOTYPE_LARS);
	return @outputFiles;
}

# if a single job failes, than we will expect that the error will be in this file
sub getJobErrorFile() {
	return 'err.txt';
}

# a limit of the num of windows for the task
# in order not to make too large computations
# if returns undef, than there is no limit
sub maxNumOfWindowsAllowed() {
	my $class = $_[0]; # the name of this class
	return 8;
}

# a limit of the num of markers in window (window size) for the task
# in order not to make too large computations
# if returns undef, than there is no limit
sub maxNumOfMarkersAllowedInWindow() {
	my $class = $_[0]; # the name of this class
	return 120;
}

# dvir 2011/12/11
# this method is called from postJob.pl with determine_job_status_of_leaf_job_with_one_exec
# parameters:
#	the folder name of a job
# this function should check if there was an acceptaable error.
# if the error is acceptable, than no mail should be sent to the developers.
# this method is called with a full path of the jobDir and you cannot assume nothing about the cwd
#
# for now, only a complex input is a reasonable error.
# an example for a message inidicating such an error: 
# "DAOOPT: Problem instance with width w=84 is too complex, aborting (limit is w=50)"
sub doesAnAcceptableErrorOcurredAtJob($ $) {
	my $class = $_[0]; # the name of this class
	my $jobDir = $_[1]; # full path or relative to the calling scope
	my $jobExitCode = $_[2];
	
	my $executeableErrFileName = $class->getJobErrorFile();
	my $errFileName = "$jobDir\/$executeableErrFileName";
	if (-e $errFileName) {
		open ERR_FILE, $errFileName;
		my $firstLineInErrorFile = <ERR_FILE>;
		close ERR_FILE;
		
		if ($firstLineInErrorFile =~ /is too complex/) {
			return 1;
		}
	}

	return 0;
}

# dvir 14/11/2011
sub setCurrentWindowEndTime() {
	my $class = $_[0]; # the name of this class
	my $windowRunTimeInSec = $_[1];
	my $totalNumOfWindowsForAllFamilies = $_[2];
	
	my $workDir = $class->getWorkDir();
	open END_TIME,">$workDir\/$Configuration::WINDOW_END_TIME_FILENAME";
	open END_TIME2,">$workDir\/${Configuration::WINDOW_END_TIME_FILENAME}.backup.txt";
	my $lastEndTime = time;
	for (my $currWindow=0;$currWindow<$totalNumOfWindowsForAllFamilies; ++$currWindow) {
		$lastEndTime += $windowRunTimeInSec;
		print END_TIME "$lastEndTime\n";
		print END_TIME2 "$lastEndTime\n";
	}
	close END_TIME;
	close END_TIME2;
}

# dvir 14/11/2011
sub getCurrentWindowEndTime() {
	my $class = $_[0]; # the name of this class
	
	my $workDir = $class->getWorkDir();
	open END_TIME,"$workDir\/$Configuration::WINDOW_END_TIME_FILENAME";	
	my $timeInSec = <END_TIME>;
	chomp($timeInSec); # removing the '\n'
	close END_TIME;
	return $timeInSec;
}

# dvir 14/11/2011
sub updateCurrentWindowEndTime() {
	my $class = $_[0]; # the name of this class
	
	my $workDir = $class->getWorkDir();
	open END_TIME,"$workDir\/$Configuration::WINDOW_END_TIME_FILENAME";	
	my @endTimes = <END_TIME>;
	close END_TIME;
	open END_TIME,">$workDir\/$Configuration::WINDOW_END_TIME_FILENAME";	
	# the remaining time contatins '\n' at each time (=line) end
	# so we only need to concat them
	print END_TIME join ('',@endTimes[1..$#endTimes]);
	close END_TIME;
}

# the names that the program expect to get
my $EXPECTED_DAT_FILENAME = 'datafile.dat';
my $EXPECTED_PED_FILENAME = 'pedfile.dat';
my $HAPLO_LARS_SUPPLIED_DAG_FILENAME = 'haplotyping.dag';

sub getLeafJob_CondorLogFileName() {
	my $class = $_[0]; # the name of this class
	
	return $HAPLO_LARS_SUPPLIED_DAG_FILENAME.'.dagman.log';
}

# dvir 2011/09/6
# create dag, dirs dat files & ped files for the multi window per job run for all the families.
# each family can have some jobs.
# each job can have some duplications.
sub createDagForMultiWindowPerJobAnalysis(\$ \$ $ \% $ $ ) {
	my $class = $_[0]; # the name of this class
	
	my $datWrapperRef = $_[1];
	my $pedWrapperRef = $_[2];
	# numOfDuplications is the num of duplications for each window
	# or the num of duplications for each 'multi window job'
	my $numOfDuplications = $_[3];
	# each family will have a list of jobs, 
	# and each job will have a list of windows to be run in that job.
	# a window is a list of markers.
	my $windowsPerJobPerFamilyHashRef = $_[4];
	my $maxTimePerJob = $_[5]; # in seconds, unused

	# time to compute haplotyping for each familiy
	my $timeToRunSingleWindow = $_[6]; # in seconds
	# can we create only one dat file for all the families jobs, and use soft link to this dat file
	# when using estimation, than this should be 0, cause diffrenet families can have a diffrenet partition of windows to jobs.
	my $allFamiliesJobsDatFilesAreTheSame = $_[7];
	
	# $allFamiliesJobsDatFilesAreTheSame should be cause for now, each job in haplo has only 1 window,
	# therefore jobs #x for family 1 & family 2 will both be on the same window: window #x.
	if (!$allFamiliesJobsDatFilesAreTheSame) {
		&General::reportOnErrorReleaseSemaphoresAndDie('HaplotypingLars.pm',`pwd`,
					"createDagForMultiWindowPerJobAnalysis: allFamiliesJobsDatFilesAreTheSame should be 1 in haplotyping");
	}
	
	my $binDir = $Configuration::configfile{ binDir };
	
	my @families = $$pedWrapperRef->getFamiliesNumbers();

	my $workDir = $class->getWorkDir();		
	
	#### making sure that each job has only 1 window
	foreach my $currFamily (@families) {
		
		# passing over the familiy's jobs
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
				
			my $numOfWindowsPerJob = scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}[$currJobNum]});
		
			# making sure that the job has only 1 window
			if ($numOfWindowsPerJob != 1) {
				&General::reportOnErrorReleaseSemaphoresAndDie('HaplotypingLars.pm',`pwd`,
					"createDagForMultiWindowPerJobAnalysis: family $currFamily, job $currJobNum has $numOfWindowsPerJob windows (and not only 1 window)");
			}
		}
	}
	

	#### creating the job dirs & ped with no allels for each family
	
	# the value of this list isn't important, cause we are only writing the person data
	my @tempMarkersList = (); 
	foreach my $currFamily (@families) {
		my $familyRunDir = &General::getFamilyRunDir($currFamily);		
		
		# passing over the familiy's jobs & creating them
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);
			mkdir $jobDir;
		}
	}
	
	
	#### creating a dat file for each window
	# each window is a unique job
	my $datfilesDirName = "$workDir\/datfiles";
	mkdir $datfilesDirName;
		
	my $someFamilyNum = $families[0];
	# passing over the 'some familiy' jobs	
	for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$someFamilyNum}}); ++$currJobNum) {
	
		# creating the datfile
		my $currDatFileName = "$datfilesDirName\/job_$currJobNum.$Configuration::MULTI_DAT_FILE_NAME";

		# the job has only 1 window (window no' 0)
		# $windowMarkers is a ref to an array with markers nums that belong to window no' 0
		my $windowMarkers = $$windowsPerJobPerFamilyHashRef{$someFamilyNum}[$currJobNum][0];
		open DATFILE_FH, ">$currDatFileName";		
		$$datWrapperRef->writeSubDatFileByListOfMarker(\*DATFILE_FH, $windowMarkers);
		close DATFILE_FH;
		
		# creating a soft link to the dat files for all the families		
		foreach my $currFamily (@families) {
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);
			
			symlink $currDatFileName, "$jobDir\/$EXPECTED_DAT_FILENAME";

			# extracting the tar for each job
			my $command = "tar -xf $Configuration::HAPLOTYPE_LARS_PROGRAM_DIR\/parallelHaplo-v06.tgz -C $jobDir";
			system($command);
			my $ret = $? >> 8;
			if ($ret != 0) {
				&General::reportOnErrorReleaseSemaphoresAndDie('HaplotypingLars.pm',`pwd`,
					"createDagForMultiWindowPerJobAnalysis: failed to preform : $command");
			}
						
			# making sure that the dag was created
			my $currJobDagFileName = "$jobDir\/$HAPLO_LARS_SUPPLIED_DAG_FILENAME";
			&General::makeSureFileExists($currJobDagFileName);			
		}
	}
	
	#### creating the ped file for each window
	# each window is a unique job
	foreach my $currFamily (@families) {
		
		my @familiesInDestPedFile = ($currFamily);
				
		# passing over the familiy's jobs
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
			
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);
			
			# the job has only 1 window (window no' 0)
			# $windowMarkers is a ref to an array with markers nums that belong to window no' 0
			my $windowMarkers = $$windowsPerJobPerFamilyHashRef{$someFamilyNum}[$currJobNum][0];
			# creating the pedfile
			open PEDFILE_FH, ">$jobDir\/$EXPECTED_PED_FILENAME";			
			$$pedWrapperRef->writeSubPedFileByListOfMarker(\*PEDFILE_FH, $windowMarkers, \@familiesInDestPedFile, 1, $PedWrapper::WRITE_PERSON_DATA_AND_ALLELS);
			close PEDFILE_FH;
		}
	}
	
	#### Creating the DAG	
	open DAG_FILE, ">$Configuration::TASK_MAIN_DAG_NAME";
	my $prevJobName = undef;
	my $dagEdgesStr = "";
	my $totalNumOfWindowsForAllFamilies = 0;
	foreach my $currFamily (@families) {
						
		# passing over the familiy's jobs
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
			++$totalNumOfWindowsForAllFamilies;
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);
			
			my $currFamilyJobName = "Family_${currFamily}_Job_${currJobNum}";
			
			print DAG_FILE 
				# DIR is the place to which the output of the run will be placed, 
				# and from which it will try to locate the job submission file (if that file is without a absulte path)
				#
				# $condorJobParameterName_FamDir is used for the files to be copied to execution machine.
				"SUBDAG EXTERNAL $currFamilyJobName $HAPLO_LARS_SUPPLIED_DAG_FILENAME DIR $jobDir\n".
				"SCRIPT POST     $currFamilyJobName $binDir\/postJob.pl $workDir determine_job_status_of_leaf_job_with_one_exec____and____create_html \$RETURN\n\n";
				
			# in order that no more than 1 window will run at any time.
			# casue each window is a dag with at least 1200 jobs
			if (defined $prevJobName) {
				$dagEdgesStr .= "PARENT  $prevJobName  CHILD  $currFamilyJobName\n";
			}
			
			$prevJobName = $currFamilyJobName;
		}
	}
	
	print DAG_FILE "\n\n".$dagEdgesStr;
	
	# adding a job that will mark that the task is finished.
	# a successful task is when all families are successful.
	print DAG_FILE 
		"\nJOB       LAST_JOB $Configuration::SIMPLE_SHORT_CONDOR_JOB\n".
		# there is here a creation of html, for cases when all the jobs are failed, then the results.html file
		# that will be sent to hap will be a failure results.html
		"SCRIPT POST LAST_JOB $binDir\/postJob.pl $workDir determine_job_status_of_families_dirs_by_their_sub_jobs____and____create_html____and____send_indication_to_web_server_to_mark_that_task_is_done____and____determine_job_status_of_the_task_by_families_dirs\n\n".
		"PARENT  $prevJobName  CHILD  LAST_JOB\n";
	close DAG_FILE;
	
	$class->setCurrentWindowEndTime($timeToRunSingleWindow, $totalNumOfWindowsForAllFamilies);
}

our $OUTPUT_DIRNAME = 'outputData';
our $FAMILIES_DIR_LIST_SUFFIX = '.outputDirList.txt';
our $FAMILIY_MEMBERS_SUFFIX = '.members.txt';

# creating the initial output files for each job
sub createInitialOutputFiles(\$ \$ \%) {
	my $class = $_[0]; # the name of this class
	
	my $datWrapperRef = $_[1];
	my $pedWrapperRef = $_[2];
	# each family will have a list of jobs, 
	# and each job will have a list of windows to be run in that job.
	# a window is a list of markers.
	my $windowsPerJobPerFamilyHashRef = $_[3];
	
	my $workDir = $class->getWorkDir();	
	
	my $outputDataDir = "$workDir\/$OUTPUT_DIRNAME";
	mkdir "$workDir\/outputData";
	
	# passing over the families	
	my @families = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@families);
	
	foreach my $currFamily (@families) {
		open FAM_OUT_DIR_LIST, ">$outputDataDir\/fam$currFamily$FAMILIES_DIR_LIST_SUFFIX";
		
		# passing over the familiy's jobs
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);			
			
			# each job has only 1 window, writing the num of markers in that window
			# (cause the last window may contain less markers).
			# 0 -> first window
			my @windowOfJob = @{$$windowsPerJobPerFamilyHashRef{$currFamily}[$currJobNum][0]};
			print FAM_OUT_DIR_LIST "$jobDir ".join(' ',@windowOfJob)."\n";
		}
		close FAM_OUT_DIR_LIST;
		
		my @familiesMembers = $$pedWrapperRef->getFamiliyMembers($currFamily);
		open FAM_MEMBERS, ">$outputDataDir\/fam$currFamily$FAMILIY_MEMBERS_SUFFIX";
		print FAM_MEMBERS join(' ', @familiesMembers);
		close FAM_MEMBERS;		
	}
	
	my @markersNums = ();
	# creating a map of markers
	my $someFamilyNum = $families[0];
	# passing over the 'some familiy' jobs	
	for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$someFamilyNum}}); ++$currJobNum) {
	
		# the job has only 1 window (window no' 0)
		# $windowMarkers is a ref to an array with markers nums that belong to window no' 0
		my $windowMarkers = $$windowsPerJobPerFamilyHashRef{$someFamilyNum}[$currJobNum][0];
		
		push @markersNums, @$windowMarkers;
	}
	&HaplotypingHtml::createMarkersMap($datWrapperRef, \@markersNums);
}

# do nothing special
sub createFinalOutputFileForCurrentJob() {
	my $class = $_[0]; # the name of this class
	my $numOfDuplications = $_[1];
	
	# updating the excpected end run time of the next window
	$class->updateCurrentWindowEndTime();
}

# dvir 2011/11/10
# creating the final output file
sub createOutputFiles( \% ) {
	my $class = $_[0]; # the name of this class
	my $variablesHashRef = $_[1];
	
	my $workDir = $class->getWorkDir();	
	chdir $workDir;
	
	my $projectTaskDir = $class->getProjectDir();
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	
	my $outputDataDir = "$workDir\/$OUTPUT_DIRNAME";
	# getting the families
	my @families = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@families);
		
	my %familiesWithAtLeastOneSuccessfulWindow = ();
	my %familiesWithAtLeastOneFailedWindow = ();
	my %familiesWithAllWindowsFailed = ();	
	my %familiesWithAllWindowsTimeout = ();
	my %familiesWithAllWindowsTimeoutOrFailed = ();
	my %familiesWithAllWindowsNotComputed = ();
	my %familiesWithNotYetComputedWindows = ();
	my %familiesWithAllWindowsComputedTillNowAsFailed_andHaveNotYetComputedWindow = ();
	my %familiesWithAllWindowsComputedTillNowAsTimeout_andHaveNotYetComputedWindow = ();
	my %familiesWithAllWindowsComputedTillNowAsTimeoutOrFailed_andHaveNotYetComputedWindow = ();
	my %familiesWithTooMuchComplexComputations = ();
	# passing over the families   
	foreach my $familyNum (@families) {
		# examples for entries at FAM_OUT_DIR_LIST
		# 1.familyJobDir/0.job 3
		# 1.familyJobDir/1.job 3
		# 1.familyJobDir/2.job 1
		# 2.familyJobDir/0.job 3
		# 2.familyJobDir/1.job 3
		# 2.familyJobDir/2.job 1		
		open FAM_OUT_DIR_LIST, "$outputDataDir\/fam$familyNum$FAMILIES_DIR_LIST_SUFFIX";
		my @currfamiJobsStr = <FAM_OUT_DIR_LIST>;
		close FAM_OUT_DIR_LIST;
				
		my $numOfSuccessful = 0;
		my $numOfFailed = 0;
		my $numOfTimedOut = 0;
		my $numOfNotComputed = 0;
		my $numOfTooMuchComplexJobs = 0;
		# passing over the curr family jobs
		for my $tempStr (@currfamiJobsStr) {
			my @tempArr = split ' ', $tempStr;
			my $currJobDir = "$workDir\/$tempArr[0]";
			
			if (&JobStatus::doesJobHasA_Successful_Indication($currJobDir)) {
				++$numOfSuccessful;
			}
			if (&JobStatus::doesJobHasA_TimedOut_Indication($currJobDir)) {
				++$numOfTimedOut;
			}
			if (&JobStatus::doesJobHasA_Failed_Indication($currJobDir) ||
			    &JobStatus::doesJobHasA_NoStatus_Indication($currJobDir)) {
				++$numOfFailed;
			}
			if (! &JobStatus::isJobFinished($currJobDir)) {
				++$numOfNotComputed;
			}
			if (&JobStatus::doesJobHasA_FailedWithAcceptableReason_Indication($currJobDir)) {
				++$numOfTooMuchComplexJobs;
			}
		}
		
		my $totalNumOfWindows = scalar(@currfamiJobsStr);
		if ($numOfTooMuchComplexJobs > 0) {
			$familiesWithTooMuchComplexComputations{$familyNum} = 1;
		}
		if ($numOfNotComputed > 0) {
			$familiesWithNotYetComputedWindows{$familyNum} = 1;
		}
		if ($numOfFailed > 0) {
			$familiesWithAtLeastOneFailedWindow{$familyNum} = 1;
		}
		if ($numOfNotComputed == $totalNumOfWindows) {
			$familiesWithAllWindowsNotComputed{$familyNum} = 1;
		}
		elsif ($numOfSuccessful > 0) {
			$familiesWithAtLeastOneSuccessfulWindow{$familyNum} = 1;
		}		
		elsif ($numOfFailed == $totalNumOfWindows) {
			$familiesWithAllWindowsFailed{$familyNum} = 1;
		}
		elsif ($numOfTimedOut == $totalNumOfWindows) {
			$familiesWithAllWindowsTimeout{$familyNum} = 1;
		}
		elsif ($numOfTimedOut + $numOfFailed == $totalNumOfWindows) {
			$familiesWithAllWindowsTimeoutOrFailed{$familyNum} = 1;
		}
		elsif ($numOfSuccessful == 0 && $numOfNotComputed > 0) {
			if ($numOfFailed + $numOfNotComputed == $totalNumOfWindows) {
				$familiesWithAllWindowsComputedTillNowAsFailed_andHaveNotYetComputedWindow{$familyNum} = 1;
			}
			elsif ($numOfTimedOut + $numOfNotComputed == $totalNumOfWindows) {
				$familiesWithAllWindowsComputedTillNowAsTimeout_andHaveNotYetComputedWindow{$familyNum} = 1;
			}
			elsif ($numOfFailed + $numOfTimedOut + $numOfNotComputed == $totalNumOfWindows) {
				$familiesWithAllWindowsComputedTillNowAsTimeoutOrFailed_andHaveNotYetComputedWindow{$familyNum} = 1;
			}
		}		      
	}
		
	my $headline = "<div>\n";
	if (scalar (keys %familiesWithNotYetComputedWindows) > 0 && scalar(keys %familiesWithAtLeastOneSuccessfulWindow) > 0) {
		my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		$headline.="last update: $now_string</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsFailed) > 0) {
		$headline.="The analysis had failed for these families: ".join(' ',keys %familiesWithAllWindowsFailed)."</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsTimeout) > 0) {
		$headline.="The analysis had timed out for these families: ".join(' ',keys %familiesWithAllWindowsTimeout)."</br>\n";
	}	
	if (scalar (keys %familiesWithAllWindowsTimeoutOrFailed) > 0) {
		$headline.="The analysis had failed/timed out for these families: ".join(' ',keys %familiesWithAllWindowsTimeoutOrFailed)."</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsNotComputed) > 0) {
		$headline.="families that wait to be computed: ".join(' ',keys %familiesWithAllWindowsNotComputed)."</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsComputedTillNowAsFailed_andHaveNotYetComputedWindow) > 0) {
		$headline.="families with failed & unstarted window computations: ".join(' ',keys %familiesWithAllWindowsComputedTillNowAsFailed_andHaveNotYetComputedWindow)."</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsComputedTillNowAsTimeout_andHaveNotYetComputedWindow) > 0) {
		$headline.="families with timed out & unstarted window computations: ".join(' ',keys %familiesWithAllWindowsComputedTillNowAsTimeout_andHaveNotYetComputedWindow)."</br>\n";
	}
	if (scalar (keys %familiesWithAllWindowsComputedTillNowAsTimeoutOrFailed_andHaveNotYetComputedWindow) > 0) {
		$headline.="families with failed/timed out  & unstarted window computaions: ".join(' ',keys %familiesWithAllWindowsComputedTillNowAsTimeoutOrFailed_andHaveNotYetComputedWindow)."</br>\n";
	}
	if (scalar (keys %familiesWithTooMuchComplexComputations) > 0) {
		$headline.="families with too much complex computations: ".join(' ',keys %familiesWithTooMuchComplexComputations)."</br>\n";
	}	
	$headline.= "</div>\n";
	
	# A data structrue that will hold the results for people from all the successful families
	# each entry in the hash will be "famNum personNum" -> struct PersonHaploData (resides at HaplotypingHtml.pm)	
	my %peopleResultsHash = ();
	my $totalNumOfMarkers;
	# for each marker, we will save to which family it was computed
	# hash of arrays
	# m1 -> [f1,f4]
	my %familiesPerMarkers = ();
	my %markersPerFamilies = ();
	
	# checking if there is at least one successful computation:
	if (scalar(keys %familiesWithAtLeastOneSuccessfulWindow) > 0) {
	
		# passing over all the families with successful indications in of their window
		foreach my $familyNum (keys %familiesWithAtLeastOneSuccessfulWindow) {
			$totalNumOfMarkers = 0; # the total num of markers is the same for all families
			
			# getting the curr family members
			open FAM_MEMBERS, "$outputDataDir\/fam$familyNum$FAMILIY_MEMBERS_SUFFIX";
			my $familiesMembersStr = <FAM_MEMBERS>;
			close FAM_MEMBERS;
			my @familiesMembers = split ' ', $familiesMembersStr;
			
			# Initializing the data structure for the current family
			for my $personNum (@familiesMembers) {
				# adding the data of the current person to the data structure
				$peopleResultsHash{"$familyNum $personNum"} = PersonHaploData->new(); # PersonHaploData is from HaplotypingHtml
			}
			
			# examples for entries at FAM_OUT_DIR_LIST
			# 1.familyJobDir/0.job 3
			# 1.familyJobDir/1.job 3
			# 1.familyJobDir/2.job 1
			# 2.familyJobDir/0.job 3
			# 2.familyJobDir/1.job 3
			# 2.familyJobDir/2.job 1		
			open FAM_OUT_DIR_LIST, "$outputDataDir\/fam$familyNum$FAMILIES_DIR_LIST_SUFFIX";
			my @currfamiJobsStr = <FAM_OUT_DIR_LIST>;
			close FAM_OUT_DIR_LIST;				
					
			my $numOfSuccessful = 0;
			my $numOfFailed = 0;
			my $numOfTimedOut = 0;
			my $numOfNotComputed = 0;
			for my $tempStr (@currfamiJobsStr) {
				my @tempArr = split ' ', $tempStr;
				my $currJobDir = "$workDir\/$tempArr[0]";
				my @windowMarkers = @tempArr[1..$#tempArr];
				my $numOfMarkerInCurrJob = scalar(@windowMarkers);
				
				# when we are getting to an uncomputed window, than since all the windows are computed one after the other, 
				# than we can be sure that there are no computed windows afterwads.
				if (! &JobStatus::isJobFinished($currJobDir)) {			
					last;
				}
				
				if (&JobStatus::doesJobHasA_Successful_Indication($currJobDir)) {
					my $currOutputFile = "$currJobDir\/$Configuration::RESULT_FILE_HAPLOTYPE_LARS";
					
					open RESULTS_FILE, $currOutputFile;
					<RESULTS_FILE> for 1..2; # the first two lines of the file are headlines
					# "\*RESULTS_FILE" : passing a reference to the file.
					&readResultsFromFile($familyNum, \%peopleResultsHash, \*RESULTS_FILE);
					close RESULTS_FILE;
				}
				elsif (&JobStatus::doesJobHasA_TimedOut_Indication($currJobDir)) {
					# printing 'timed out' str to the result data structure
					$class->addEmptyAlleles($familyNum,
						\@familiesMembers,
						$numOfMarkerInCurrJob,
						\%peopleResultsHash,
						'Timed out',
						'TO');
				}
				elsif (&JobStatus::doesJobHasA_Failed_Indication($currJobDir) ||
					&JobStatus::doesJobHasA_NoStatus_Indication($currJobDir) ||
					&JobStatus::doesJobHasA_FailedWithAcceptableReason_Indication($currJobDir)) {
					# printing failed str to the result data structure
					$class->addEmptyAlleles($familyNum,
						\@familiesMembers,
						$numOfMarkerInCurrJob,
						\%peopleResultsHash,
						'Failed',
						'F');
				}						
				
				$totalNumOfMarkers += $numOfMarkerInCurrJob;
				for my $markerNum (@windowMarkers) {
					push (@{$familiesPerMarkers{$markerNum}}, $familyNum);
					push (@{$markersPerFamilies{$familyNum}}, $markerNum);
				}
			}
		}
		
		if (scalar(keys %familiesWithAtLeastOneFailedWindow) > 0) {
			$headline.= "<div>please contact sys admin<div></br>\n";
		}
	
		# creating the htmls				
		my @familiesWithAtLeastOneSuccessfulWindowArr = keys %familiesWithAtLeastOneSuccessfulWindow;
		&HaplotypingHtml::createHtmlPagesWithHaplotypeData($headline, \@familiesWithAtLeastOneSuccessfulWindowArr, \%peopleResultsHash, \%familiesPerMarkers, \%markersPerFamilies);
	}
	else {
		my $shouldNotContactSysAdmin = 1;
		if (scalar(keys %familiesWithAtLeastOneFailedWindow) > 0) {
			# undef to mark that there is a need to contact sys admin
			$shouldNotContactSysAdmin = undef;			
		}	
		
		# if there is a chance to one of the computations to success, than 
		if (scalar (keys %familiesWithNotYetComputedWindows) > 0) {
			# replacing each "failed" string with "fail_ed" in order that hap won't mark this as a failed task.
			$headline =~ s/failed/fail_ed/g;
			$class->setReportText($headline);
			$class->reportOnRunningToWebServer();
		}
		# no uncomputed windows, than sending an error file.
		else {
			$class->reportOnFailureToWebServer($headline,$shouldNotContactSysAdmin);
		}
	}
}

# adding empty cells for a non successful (timed out / failed / no status) window job
sub addEmptyAlleles () {
	my $class = $_[0];
	my $currFamilyNum = $_[1];
	my $familiesMembersArrRef = $_[2];
	my $numOfMarkers = $_[3];
	my $peopleResultsHashRef = $_[4]; # data structre that holds the results 
	my $strForFirstMarker = $_[5];
	my $strForRestMarkers = $_[6];
		
	for my $personNum (@$familiesMembersArrRef) {
		# in order not to access the hash for each allel
		# saving a reference to the curr person Haplo Data
		my $personHaploData = $$peopleResultsHashRef{"$currFamilyNum $personNum"};
		
		push @{$personHaploData->paterna}, $strForFirstMarker;
		push @{$personHaploData->materna}, $strForFirstMarker;
		for my $currMarker (2..$numOfMarkers) {
			push @{$personHaploData->paterna}, $strForRestMarkers;
			push @{$personHaploData->materna}, $strForRestMarkers;
		}
	}
} 

# dvir 2011/03/13
# not a method!
# will be used to read the results from the given filehandle,
# and updaing the data structre that holds the results 
#
# Returning an indication about have we read a results.
sub readResultsFromFile( \% $ $) {
	my $currFamilyNum = $_[0];
	my $peopleResultsHashRef = $_[1]; # data structre that holds the results 
	my $resultsFileHandleRef = $_[2];
	
	my $haveWeReadResults = 0;
	while (my $currLine = <$resultsFileHandleRef>) {
		# if the current line is not a Paterna line, than we have finished reading the file
		if ($currLine !~ /^P/) { # example for a paterna line "P 1     1  -  2  -  1  -  1  -  2  -  1  -  1  -  1  -  1  "
			return $haveWeReadResults;
		}
		
		my $paternaLine = $currLine;
		my $maternaLine = <$resultsFileHandleRef>;
		
		my @paternaArr = split /\s+/, $paternaLine;
		my @maternaArr = split /\s+/, $maternaLine;
					
		my $personNum = $paternaArr[1];
		
		# in order not to access the hash for each allel
		# saving a reference to the curr person Haplo Data
		my $personHaploData = $$peopleResultsHashRef{"$currFamilyNum $personNum"};
		
		# passing over all the fields of the markers
		# and adding them to the person Haplo Data
		# if there where recombinations, than adding that too
		# an odd rec will be before the allel after it
		# an even rec will be after the allel before it
		# for example:
		# P 1     1  -  2  x  1  -  1  -  2  x  1  -  1  -  1  x  1  
		# will be:
		# 1 2  x1 1 2x 1 1 1 x1  
		my $doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna = 1;
		my $doesNextRecombinationIsAnOddOneAtChromosomeFromMaterna = 1;
		# the arrays : $#paternaArr == $#maternaArr shoud be on equal size
		for (my $currMarker = 2;$currMarker<=$#paternaArr;$currMarker+=2) {
			my $currRecSignIndex = $currMarker - 1;

			# for the paterna
			my $recValueSign = $paternaArr[$currRecSignIndex];
			my $allelVal = $paternaArr[$currMarker];
			
			# checking if this is a recombination
			if ($recValueSign eq $HaplotypingHtml::RECOMBINATION_CHAR) {
				if ($doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna) {
					$allelVal = $HaplotypingHtml::RECOMBINATION_CHAR.' '.$allelVal;
				}
				# changing the previous allel, to end with the sign $HaplotypingHtml::RECOMBINATION_CHAR
				else {
					${$personHaploData->paterna}[$#{$personHaploData->paterna}] .= ' '.$HaplotypingHtml::RECOMBINATION_CHAR;
				}
				
				# changing $doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna to the oppozite
				$doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna = !$doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna;
			}	
			push @{$personHaploData->paterna}, $allelVal;
			
			
			# for the materna
			$recValueSign = $maternaArr[$currRecSignIndex];
			$allelVal = $maternaArr[$currMarker];
			
			# checking if this is a recombination
			if ($recValueSign eq $HaplotypingHtml::RECOMBINATION_CHAR) {
				if ($doesNextRecombinationIsAnOddOneAtChromosomeFromMaterna) {
					$allelVal = $HaplotypingHtml::RECOMBINATION_CHAR.' '.$allelVal;
				}
				# changing the previous allel, to end with the sign $HaplotypingHtml::RECOMBINATION_CHAR
				else {
					${$personHaploData->materna}[$#{$personHaploData->materna}] .= ' '.$HaplotypingHtml::RECOMBINATION_CHAR;
				}
				
				# changing $doesNextRecombinationIsAnOddOneAtChromosomeFromPaterna to the oppozite
				$doesNextRecombinationIsAnOddOneAtChromosomeFromMaterna = !$doesNextRecombinationIsAnOddOneAtChromosomeFromMaterna;
			}	
			push @{$personHaploData->materna}, $allelVal;
		}				
							
		<$resultsFileHandleRef>; # reading an empty line at the end of the current person
		
		# marking that we have read results
		$haveWeReadResults = 1;
	}
}
1;		
