#!/usr/bin/perl -w
use General;


$mainWorkDir = $ARGV[0];
$workDir = $ARGV[1];
$line = $ARGV[2]; 	#chrom
$project = $ARGV[3];
$mode = $ARGV[4];
$binDir = $ARGV[5];

$i = $ARGV[6];
# $ARGV[7] is $projectDataDir which is not used here
$projectTaskDir = $ARGV[8];
$projectRunlog = $ARGV[9];
$projectMailDir = $ARGV[10];
$currentlyRunningDir = $ARGV[11];
$projectOutputDir = $ARGV[12];
$semid = $ARGV[13];

my $MonitorTmpl = `cat $binDir\/Tmpl\/MonitorReduction.sub.tmpl`;
$MonitorTmpl =~ s/_CONFIGDIR_/$binDir/g;
$MonitorTmpl =~ s/_PROJECT_/$project/g;
$MonitorTmpl =~ s/_CHROM_/$line/g;
$MonitorTmpl =~ s/_MODE_/$mode/g;
$MonitorTmpl =~ s/_WORKDIR_/$workDir/g;
open( MONITOR , ">$workDir\/Monitor_$line.sub" );
print MONITOR "$MonitorTmpl";
close MONITOR;


chdir("$mainWorkDir");

open (GLOBAL, ">$workDir\/Global_$line.dag");

chdir("$mainWorkDir");

if ( $mode ne "ClusteringOnly" ) {
	print GLOBAL "JOB  A  $workDir\/CleaningSubJobs_$line.dag.condor.sub
	VARS A  EXECDIR=\"$workDir\"  JOBWORKDIR=\"$workDir\"
	SCRIPT PRE  A $binDir\/PreCleaning.pl $workDir $project $line $semid
	SCRIPT POST  A $binDir\/PostCleaning.pl $workDir $project $line $semid
	";
}

if ( $mode ne "EliminatingOnly" ) {
	print GLOBAL "JOB  B  $workDir\/ClusteringSubJobs_$line.dag.condor.sub
	VARS B  EXECDIR=\"$workDir\"  JOBWORKDIR=\"$workDir\"
	SCRIPT PRE  B $binDir\/PreClustering.pl $workDir $project $line $semid
	SCRIPT POST  B $binDir\/PostClustering.pl $workDir $project $line $semid
	";
	if ( $mode ne "ClusteringOnly" ) { #otherwise clustering was the first and only action
		print GLOBAL "PARENT A CHILD B";
	}
}

close GLOBAL;

open (WRAPPER, ">>Global.dag");
print WRAPPER "JOB  G$i  $workDir\/Global_$line.dag.condor.sub
SCRIPT POST  G$i $binDir\/PostGlobal.pl $workDir $project $projectTaskDir $projectRunlog $projectMailDir $currentlyRunningDir $projectOutputDir
JOB  GM$i $workDir\/Monitor_$line.sub
VARS GM$i TASKNUM=\"$i\" JOBWORKDIR=\"$workDir\"
";
close WRAPPER;

open (CLEANING, ">$workDir\/CleaningSubJobs_$line.dag");
print CLEANING "JOB A $mainWorkDir\/hostname.sub";
close CLEANING;

open (CLUSTERING, ">$workDir\/ClusteringSubJobs_$line.dag");
print CLUSTERING "JOB A $mainWorkDir\/hostname.sub";
close CLUSTERING;

system(". \$HOME/.bash_profile; cd $workDir; condor_submit_dag  -f -oldrescue 1 -autorescue 0 -no_recurse -no_submit -config $binDir\/dagman_config $workDir\/CleaningSubJobs_$line.dag >> $workDir\/CondorSubmitCleaning.txt 2>> $workDir\/CondorSubmitCleaningErr.txt");
system(". \$HOME/.bash_profile; cd $workDir; condor_submit_dag  -f -oldrescue 1 -autorescue 0 -no_recurse -no_submit -config $binDir\/dagman_config $workDir\/ClusteringSubJobs_$line.dag >> $workDir\/CondorSubmitClustering.txt 2>> $workDir\/CondorSubmitClusteringErr.txt");

system(". \$HOME/.bash_profile; cd $workDir; condor_submit_dag  -f -oldrescue 1 -autorescue 0 -no_recurse -no_submit -config $binDir\/dagman_config $workDir\/Global_$line.dag >> $workDir\/CondorSubmit.txt 2>> $workDir\/CondorSubmitErr.txt");

while ((! -e "$workDir\/CleaningSubJobs_$line.dag.condor.sub") || (! -e "$workDir\/ClusteringSubJobs_$line.dag.condor.sub") || (! -e "$workDir\/Global_$line.dag.condor.sub")) {
	sleep(5);
}
