# dvir 2011/11/09
package JobStatus;

use strict;
use Configuration;
use General;

# all the subs : "markThat..." are for "leaf" jobs (=jobs that doesn't have a sub job).

# in theory a job can be at those statues at the same time:
# 1. Successful
# 2. Failed
# 3. Timed out
# 4. Without status
# 5. a combination of the above.

#	dvir 2010/12/19
sub markThatJobSucceeded($) {
	my $jobFolder = $_[0];
	`touch $jobFolder\/$Configuration::JOB_SUCCEEDED_FILENAME`;
}

#	dvir 2010/12/19
sub doesJobHasA_Successful_Indication($) {
	my $jobFolder = $_[0];
	return -e "$jobFolder\/$Configuration::JOB_SUCCEEDED_FILENAME";
}

#	dvir 2010/12/19
sub markThatJobFailed($) {
	my $jobFolder = $_[0];
	`touch $jobFolder\/$Configuration::JOB_FAILED_FILENAME`;
}

#	dvir 2011/11/09
# returning an indication whether or not the job or one of its sub jobs had failed.
sub doesJobHasA_Failed_Indication($) {
	my $jobFolder = $_[0];
	return -e "$jobFolder\/$Configuration::JOB_FAILED_FILENAME";
}

#	dvir 2011/11/09
sub markThatJobTimedOut($) {
	my $jobFolder = $_[0];
	`touch $jobFolder\/$Configuration::JOB_TIMEOUT_FILENAME`;
}

#	dvir 2011/11/09
# returning an indication whether or not the job or one of its sub jobs has timed out
sub doesJobHasA_TimedOut_Indication($) {
	my $jobFolder = $_[0];
	return -e "$jobFolder\/$Configuration::JOB_TIMEOUT_FILENAME";
}

#	dvir 2011/11/09
sub markThatJobHasNoStatus($) {
	my $jobFolder = $_[0];
	`touch $jobFolder\/$Configuration::JOB_WITHOUT_STATUS_FILENAME`;
}

#	dvir 2011/11/09
sub doesJobHasA_NoStatus_Indication($) {
	my $jobFolder = $_[0];
	return -e "$jobFolder\/$Configuration::JOB_WITHOUT_STATUS_FILENAME";
}

#	dvir 2010/11/16
sub markThatJobFailedWithAcceptableReason($) {
	my $jobFolder = $_[0];
	`touch $jobFolder\/$Configuration::JOB_FAILED_WITH_ACCEPTABLE_REASON_FILENAME`;
}

#	dvir 2011/11/16
# returning an indication whether or not the job or one of its sub jobs had failed with an acceptable reason.
sub doesJobHasA_FailedWithAcceptableReason_Indication($) {
	my $jobFolder = $_[0];
	return -e "$jobFolder\/$Configuration::JOB_FAILED_WITH_ACCEPTABLE_REASON_FILENAME";
}

#	dvir 2010/12/20
# getting all the sub folders of the current dir, this sub is for internal use of the package
sub getSubFolders( \@ ) {
	my $jobFolder = $_[0];
	my $subFoldersArrRef = $_[1];
	
	my $subFoldersStr = `ls -d $jobFolder/*/`;
	@$subFoldersArrRef = split "\/\n", $subFoldersStr;
}

#	dvir 2011/11/09
# this method is for a non "leaf" job, meaning job that has sub jobs.
# it will update the status of the job by its sub jobs
sub updateJobStatusByItsSubJobs() {
	my $jobFolder = $_[0];
	my $subJobFoldersArrRef = $_[1];
	
	my @subJobFolders;
	if (defined $subJobFoldersArrRef) {
		@subJobFolders = @$subJobFoldersArrRef;
	}
	else {
		&getSubFolders($jobFolder, \@subJobFolders);
	}
	
	# making sure that the job has indeed sub jobs
	if (scalar(@subJobFolders) == 0 ) {
		&General::reportOnErrorReleaseSemaphoresAndDie('JobStatus.pm', '', "failed: updateJobStatusByItsSubJobs, job doesn't have sub jobs. \$jobFolder: $jobFolder");
	}
	
	# for each output sub job folder, we will check its status.
	foreach my $subJobFolder ( @subJobFolders ) {
		if (! -d $subJobFolder) {
			&General::reportOnErrorReleaseSemaphoresAndDie('JobStatus.pm', '', "failed: updateJobStatusByItsSubJobs, folder doesn't exists \$subJobFolder: $subJobFolder");
		}
		my $subJobHasAStatus = 0;
		# here the folder exists
		# making the indications
		if (&doesJobHasA_Successful_Indication($subJobFolder)) {
			&markThatJobSucceeded($jobFolder);
			$subJobHasAStatus = 1;
		}
		if (&doesJobHasA_Failed_Indication($subJobFolder)) {
			&markThatJobFailed($jobFolder);
			$subJobHasAStatus = 1;
		}
		if (&doesJobHasA_TimedOut_Indication($subJobFolder)) {
			&markThatJobTimedOut($jobFolder);
			$subJobHasAStatus = 1;
		}
		if (&doesJobHasA_FailedWithAcceptableReason_Indication($subJobFolder)) {
			$subJobHasAStatus = 1;
			&markThatJobFailedWithAcceptableReason($jobFolder);
		}
		if (&doesJobHasA_NoStatus_Indication($subJobFolder) || !$subJobHasAStatus) {
			&markThatJobHasNoStatus($jobFolder);
		}

	}	
}

#	dvir 2010/12/19
# returning an indication whether or not the job finshed with success or failure
sub isJobFinished($) {
	my $jobFolder = $_[0];
	return &doesJobHasA_Successful_Indication($jobFolder) ||
		   &doesJobHasA_Failed_Indication($jobFolder) ||
		   &doesJobHasA_TimedOut_Indication($jobFolder) ||
		   &doesJobHasA_NoStatus_Indication($jobFolder) ||
		   &doesJobHasA_FailedWithAcceptableReason_Indication($jobFolder);
}

#	dvir 2011/11/09
sub isJobOnlySuccessful( $ ) {
	my $jobFolder = $_[0];
	return &doesJobHasA_Successful_Indication($jobFolder) &&
		   (!&doesJobHasA_Failed_Indication($jobFolder)) && 
		   (!&doesJobHasA_TimedOut_Indication($jobFolder)) &&
		   (!&doesJobHasA_NoStatus_Indication($jobFolder)) &&
		   (!&doesJobHasA_FailedWithAcceptableReason_Indication($jobFolder));
}

#	dvir 2011/11/09
sub isJobOnlySuccessfulAndMayBeAlsoTimedOut( $ ) {
	my $jobFolder = $_[0];
	return &doesJobHasA_Successful_Indication($jobFolder) &&
		   (!&doesJobHasA_Failed_Indication($jobFolder)) && 
		   (!&doesJobHasA_NoStatus_Indication($jobFolder)) &&
		   (!&doesJobHasA_FailedWithAcceptableReason_Indication($jobFolder));		   
}

#	dvir 2011/11/09
#sub isJobSuccessfulAndMayBeAlsoFailed_TimedOut_NoStatus( $ ) {
#}
# this method is actually : doesJobHasA_Successful_Indication

#	dvir 2011/11/09
sub isJobOnlyTimedOut( $ ) {
	my $jobFolder = $_[0];
	return (!&doesJobHasA_Successful_Indication($jobFolder)) &&
		   (!&doesJobHasA_Failed_Indication($jobFolder)) && 
		   &doesJobHasA_TimedOut_Indication($jobFolder) &&
		   (!&doesJobHasA_NoStatus_Indication($jobFolder));
}

1;