# dvir, 2011-05-26
package MorganLmMarkers;

use strict;

use Class::Struct;
use Scalar::Util;
use List::Util;

use AnalysisWith_modefile;
use Configuration;
use General;
use DatPedHandler;
use ProcessingBackend;
use SharedConsts;
use GenomeBrowser;
use Plotter;
use JobStatus;

struct MarkerLodData =>
[
	lod	=> '$', # the lod of the marker
	stdErr => '$', # the standard error of the computation
];

# those files will be copied to the web server
my $HTML_OUTPUT_FILENAME = $ProcessingBackend::OUTPUT_FILE;
my @RESULTS_FILE_LIST = ($HTML_OUTPUT_FILENAME);


# for now the num of iterations increases exponentially
sub getNextNumOfIterations($) {
	my $currNumOfIterations = $_[0];
	
	return $currNumOfIterations * 2;
}

# this class inherits methods from AnalysisWith_modefile
our @ISA = ('AnalysisWith_modefile');

# methods we override from AnalysisTemplateClass.pm :

# returning an array of files names
# those files will be copied to the machine on which the job will run
# it shouldn't contain the dat & ped files, which will be added by the calling method.
sub getCondorJobInputFiles() {
	my $class = $_[0];
	
	my $projectDir = $class->getProjectDir();
	my $binDir = $Configuration::configfile{ binDir };
	return 
		("$Configuration::MORGAN_LM_MARKERS_PROGRAM_DIR\/$Configuration::MORGAN_LM_LINKAGE_EXE",
		"$binDir\/superlink2lnkg.pm",
		"$Configuration::MORGAN_LM_MARKERS_PROGRAM_DIR\/lnkg2lmm.pl",
		"$projectDir\/$SharedConsts::MORGAN_MORE_PARAMS_FILENAME", # more prams for executeable
		"$projectDir\/$Configuration::ANALYSIS_PARAMETERS_MODEFILE_FILENAME");
	# copying $Configuration::ANALYSIS_PARAMETERS_MODEFILE_FILENAME
	# in order to access those params:
	# $SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_BURN_IN_PERCENTAGE
	# $SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_SEQUENTIAL_IMPUTATION_REALI
}

# morgan runs only on 64-bit machines
sub canRunOn32BitMachine() {
	return 0;
}

# returning an array of files names which contain the results of the analysis, 
# those files should be copied to the web server
sub getFileListToBeCopiedToWebServer() {
	my $class = $_[0];
	
	# if there exists an error report, than it will the only one that will be sent.
	if ($class->doesAReportExists()) {
		return "$Configuration::REPORT_HTML_FILENAME";
	}
	
	my $projectTaskDir = $class->getProjectDir();
	my $workDir = $class->getWorkDir();
	&ProcessingBackend::setLogFiles("$projectTaskDir\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	
	my $doesTheAnalysisIsOn_SNP_Input = &General::doesTheAnalysisIsOn_SNP_Input(\%variables);
		
	my @plottingFiles = &Plotter::getPlottingFilesNamesToBeSentToWebServer();
	
	my @fileList = (@RESULTS_FILE_LIST, @plottingFiles);
		
	# if the input is SNP, then adding the file for the genome browser
	if ($doesTheAnalysisIsOn_SNP_Input) {
		push @fileList, $GenomeBrowser::BED_OUTPUT_FILENAME;
	}
	
	# for now, we don't add the input file for the data browser, 
	# since that we have lod score between the markers and not at the markers
	#push @fileList, $GenomeBrowser::COLLECT_DATA_FILENAME;
	
	return @fileList;
}


# returning an array of the names of the output files of a multi window per job run
sub getJobOutputFiles() {
	return $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_OUTPUT_FILENAME;
}

sub getJobErrorFile() {
	return $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_ERROR_FILENAME;
}

sub getCondorJobSecondsToBeExecuted() {
	# morgan should not use this method.
	&General::reportOnErrorReleaseSemaphoresAndDie('MorganLmMarkers.pm',`pwd`,
					"getCondorJobSecondsToBeExecuted: this method shouldn't be used.\n".
					"the run time should be given as parameter at the modefile.txt");
}

sub getWindowsList( $ $ $ ) {
	my $class = $_[0];
	my $startAt = $_[1];
	my $endAt = $_[2];
	my $jump = $_[3];
	
	my @windowList = ();
	for (my $currMarker = $startAt; $currMarker < $endAt;) {
		my $startMarker = $currMarker;
		my $currWindowLastMarker = $currMarker + $jump - 1;
		if ($currWindowLastMarker > $endAt) {
			$currWindowLastMarker = $endAt;
			#Omer 16/9/2011 - make all windows the same size
			$startMarker = $currWindowLastMarker - $jump + 1;
			if ($startMarker < $startAt) {$startMarker = $startAt;}
		}
		# I'm using '[' and not '(' to create array of arrays
		push @windowList, [$startMarker..$currWindowLastMarker];
		
		# the first marker of the next window, is the last marker of our window
		$currMarker = $currWindowLastMarker;
	}	
	return @windowList;
}
######
sub preformUniqueChecks( \% ) {
	my $class = $_[0]; # the name of this class
	my $variablesHashRef = $_[1];
	
	if (! (defined $$variablesHashRef{num_dups_per_job} &&
		   defined $$variablesHashRef{run_time_of_window} && 
		   defined $$variablesHashRef{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS} && 
		   defined $$variablesHashRef{$SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS} )) {
		&General::reportOnErrorReleaseSemaphoresAndDie('MorganLmMarkers.pm',`pwd`,
			"At least one of following params is missing: num_dups_per_job run_time_of_window $SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS $SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS.\n");
	}
	
	# since we compute the lod between two markers, than if the jump is 1
	# than it is an error
	if (defined $$variablesHashRef{jump} &&
		$$variablesHashRef{jump} eq 1) {
			&General::reportOnErrorReleaseSemaphoresAndDie('MorganLmMarkers.pm',`pwd`,
			"since we compute the lod between two markers, than the jump must be at least 2, in order to have 2 markers in the dat file.\n");
	}
	
}

our $INITIAL_OUTPUT_CSV_FILENAME = 'initial.out.csv';
our $FINAL_OUTPUT_CSV_FILENAME = 'final.out.csv';
our $MAX_NUM_OF_ITERATIONS_FILENAME = 'maxNumOfIterations.txt';
our $OUTPUT_DIRNAME = 'outputData';
our $FAMILIES_DIR_LIST_SUFFIX = '.outputDirList.txt';

# dvir 2011/09/7
# creating the initial output files for each job
# will run once from the function that creates the dag
sub createInitialOutputFiles(\$ \$ \%) {
	my $class = $_[0]; # the name of this class
	
	my $datWrapperRef = $_[1];
	my $pedWrapperRef = $_[2];
	# each family will have a list of jobs, 
	# and each job will have a list of windows to be run in that job.
	# a window is a list of markers.
	my $windowsPerJobPerFamilyHashRef = $_[3];

	my $workDir = $class->getWorkDir();	
	
	my $outputDataDir = "$workDir\/$OUTPUT_DIRNAME";
	mkdir "$workDir\/outputData";
	
	# passing over the families	
	my @families = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@families);
	
	foreach my $currFamily (@families) {
		open FAM_OUT_DIR_LIST, ">$outputDataDir\/fam$currFamily$FAMILIES_DIR_LIST_SUFFIX";
		
		# passing over the familiy's jobs
		for (my $currJobNum=0;$currJobNum<scalar (@{$$windowsPerJobPerFamilyHashRef{$currFamily}}); ++$currJobNum) {
			my $jobDir = &General::getFamilyJobNumRunDir($currFamily, $currJobNum);
			print FAM_OUT_DIR_LIST "$jobDir\n";
			
			# a file that will help for creating the output
			my $outputDataFile = "$jobDir\/$INITIAL_OUTPUT_CSV_FILENAME";
			open OUT_DATA_FH, ">$outputDataFile";
			# passing over the job's windows, and writing the output data for them
			foreach my $currWindow (@{$$windowsPerJobPerFamilyHashRef{$currFamily}[$currJobNum]}) {
				# passing over the markers of the window
				my $prevMarkerNum = $$currWindow[0];
				my $prevMarkerName = $$datWrapperRef->getMarkerName($prevMarkerNum);				
				foreach my $currMarkerNum (@$currWindow[1..$#$currWindow]) {
					my $currMarkerName = $$datWrapperRef->getMarkerName($currMarkerNum);
					print OUT_DATA_FH "$prevMarkerName($prevMarkerNum) - $currMarkerName($currMarkerNum)";
										
					# calc the distance in CM of the middle postion between the prev marker and the next marker,
					# from the first marker in the dat file.
					my $distIn_cM_OfCurrMarkerFromPrevMarker = $$datWrapperRef->getDistIn_cM_FromPrevMarker($currMarkerNum);
					my $distIn_cM_OfPrevMarkerFromFirstMarker = $$datWrapperRef->getDistIn_cM_FromFirstMarker($prevMarkerNum);
										
					# dist FromTheFirstMarker in the dat file
					my $distanceIn_cM_OfMiddlePointBetweenPrevAndCurrMarker_FromTheFirstMarker = $distIn_cM_OfPrevMarkerFromFirstMarker + $distIn_cM_OfCurrMarkerFromPrevMarker/2;
					$distanceIn_cM_OfMiddlePointBetweenPrevAndCurrMarker_FromTheFirstMarker = sprintf("%.6f",$distanceIn_cM_OfMiddlePointBetweenPrevAndCurrMarker_FromTheFirstMarker);
					print OUT_DATA_FH ",$distanceIn_cM_OfMiddlePointBetweenPrevAndCurrMarker_FromTheFirstMarker\n";
										
					$prevMarkerNum = $currMarkerNum;
					$prevMarkerName = $currMarkerName;
				}
				print OUT_DATA_FH $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR;
			}
			close OUT_DATA_FH;
		}
		close FAM_OUT_DIR_LIST;
	}
}

# dvir 2011/09/7
# that will contain the results of the run of the current job.
# the cwd is the job dir
sub createFinalOutputFileForCurrentJob() {
	my $class = $_[0]; # the name of this class
	my $numOfDuplications = $_[1];
	
	my $projectTaskDir = $class->getProjectDir();
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	my $minNumOfIterations = $variables{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS};
		
	if ($class->doesAReportExists()) {
		return;
	}
	
	my $maxNumOfIterations = 0;	
	# level 1 is an array of the windows, window num starting from window 0
	# level 2 is the an array of the duplication, duplication num starting from window 0
	# level 3 is the an array of the points nums starting from point 0
	# level 4 is an hash of the numOfIterations
	# level 5 is a struct of MarkerLodData
	my @LodScoreByPointByDupByWindow = ();	
	
	
	#### reading data from the initial csv into the array @pointsDataByWindow[][].
    #### This array contain information about the point (name, location ,etc.) - not LOD scores.
	my @pointsDataByWindow = ();
	# how many points are at each window (each window may contain more/less markers than points)
	my @windowSizes = ();
	open INITIAL_CSV, "$INITIAL_OUTPUT_CSV_FILENAME";
	my $currWindow = 0;
	my $currPointNum = 0;
	while (my $currLineInitial = <INITIAL_CSV>) {
		# moving to the next window
		if ($currLineInitial eq $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR) {
			$windowSizes[$currWindow] = scalar(@{$pointsDataByWindow[$currWindow]});
			++$currWindow;
			$currPointNum = 0;
			next;
		}
		chomp($currLineInitial);
		$pointsDataByWindow[$currWindow][$currPointNum] = $currLineInitial;
		++$currPointNum;
	}
	close INITIAL_CSV;
	
	
	#### passing over all the duplications and filling the hash
    #### %{LodScoreByPointByDupByWindow[$currWindow][$dupNum][$currPointNum]} that maps a point number
    #### to a LOD score and to a stderr estimation
	for (my $currDup = 1; $currDup <= $numOfDuplications; ++$currDup) {
		my $duplicateDir = &General::getDuplicateDir($currDup);
		
		### dvir 2011/12/11
		#my $wasJobSuccessful = &JobStatus::doesJobHasA_Successful_Indication($duplicateDir);
		#if (!$wasJobSuccessful) {next;}	
		# here the job have succeed (at least one window succeed)
		###
		
		# the curr dup result file name
		my $resultFileName = "$duplicateDir\/$Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_OUTPUT_FILENAME";
		open RES_FH, $resultFileName;
		
		# passing over all the window to check which one of them was successful
		my $currWindow = 0;
		my $returnValuesFileName = "$duplicateDir\/$Configuration::MULTI_WINDOW_RUN_RETURN_VALUES_FILENAME";
		open RET_VAL_FH, $returnValuesFileName;
		while (my $currRetValLine=<RET_VAL_FH>) {
			chomp($currRetValLine);
			
            #If the return value indicates failure
			if ($currRetValLine != 0) {
				my $errorFile = "$duplicateDir\/".$class->getJobErrorFile();
				my $posibleErrorText = `head -20 $errorFile`;
				
				# dvir 2011/12/11
				# the error will only be reported once (at other times, the check : doesAReportExistsabout will be positive)
				$posibleErrorText =~ s/$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR//g;
				$class->reportOnFailureToWebServer($posibleErrorText);
				exit 0;
				
				# moving in the output file to the output part of the next window
				while (my $currLine = <RES_FH>) {
					# if we have finished reading the results of the curr window
					if ($currLine eq $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR) {last;}
				}
			}
			
            # here the current window has succeeded
			else {
				# reading the results.
                #Each entry in @currWindow_LodScoreByPoint corresponds to a point.
                #Each value is a hash whose keys are iteration numbers and values are a MarkerLodData object,
                #which contains a LOD score and a stderr estimation.
				(my $currWindow_MaxNumOfIterations, my @currWindow_LodScoreByPoint) = &readResultsFromFile(\*RES_FH, $windowSizes[$currWindow]);
				if (defined $currWindow_MaxNumOfIterations) {
					push @{$LodScoreByPointByDupByWindow[$currWindow]}, \@currWindow_LodScoreByPoint;
					$maxNumOfIterations = List::Util::max($maxNumOfIterations, $currWindow_MaxNumOfIterations);
				}
			}
			<RET_VAL_FH>; # reading the line :###multi end###
			if (!defined @{$LodScoreByPointByDupByWindow[$currWindow]}) {
				push @{$LodScoreByPointByDupByWindow[$currWindow]}, ();	#Create an empty list for this window if it has not results
			}
			++$currWindow;
		}
		close RET_VAL_FH;
		close RES_FH;
	}
	
	
	#### creating the $MAX_NUM_OF_ITERATIONS_FILENAME file
	open MAX_ITER, ">$MAX_NUM_OF_ITERATIONS_FILENAME";
	print MAX_ITER $maxNumOfIterations;
	close MAX_ITER;
	
    #Creating an array called @numOfIterationsArr with all the iteration numbers that were computed
	my @numOfIterationsArr = ();
	for (my $currNumOfIterations = $minNumOfIterations;
		 $currNumOfIterations <= $maxNumOfIterations;
		 $currNumOfIterations = &getNextNumOfIterations($currNumOfIterations)) {
		push @numOfIterationsArr, $currNumOfIterations;
	}

	#### creating the final csv	
	open FINAL_CSV, ">$FINAL_OUTPUT_CSV_FILENAME";
	# passing over all the windows
	#for (my $currWindow=0;$currWindow<=$#LodScoreByPointByDupByWindow; ++$currWindow) {
	for (my $currWindow=0;$currWindow<=$#pointsDataByWindow; ++$currWindow) {
		
		# does the current window have results in some duplication
		my $currWindowHasResults = 1;
		# checking if we don't have a single duplication with results for the current window
		if (not defined $LodScoreByPointByDupByWindow[$currWindow]) {$currWindowHasResults = 0;}
		
        #Create an array called @dupNumWithResults  with the duplication numbers that succeeded for this window
        my @dupNumWithResults = ();
		my $currDupNum = 0;
		foreach my $currDupResults (@{$LodScoreByPointByDupByWindow[$currWindow]}) {
			if (scalar (@{$currDupResults}) > 0) {push @dupNumWithResults, $currDupNum;}
			++$currDupNum;
		}
		if (scalar (@dupNumWithResults) == 0) {$currWindowHasResults = 0;}
		
		#If no computations succeeded for this window, print skipped messages
		if (!$currWindowHasResults) {
            my $numOfStringToPrint = 4 * List::Util::max(1, scalar(@numOfIterationsArr));
            my $csvLine = (",$ProcessingBackend::SKIPPED_INFORMATION") x $numOfStringToPrint;
			for my $currPointData (@{$pointsDataByWindow[$currWindow]}) {
                print FINAL_CSV $currPointData.$csvLine."\n";
            }
			next;
		}
		
        # here some duplication have results for the current window
		# passing over the points for the current window
		my $currPointNum = 0;
		for my $currPointData (@{$pointsDataByWindow[$currWindow]}) {
			# passing over all the successful duplications
			
			# Creating an array of the lod and stderr of the current point in the current window
			my %lodByNumOfIterationsAndDup = ();			
			my $pointLod = 0;
			for my $dupNum (@dupNumWithResults) {
				while ( my ($numOfIterations, $lodData) = each(%{$LodScoreByPointByDupByWindow[$currWindow][$dupNum][$currPointNum]}) ) {
					push @{$lodByNumOfIterationsAndDup{$numOfIterations}}, $lodData;
				}
			}
			
			print FINAL_CSV "$currPointData";			
			for my $currNumOfIterations (@numOfIterationsArr) {
                my $fieldsStr = (",$ProcessingBackend::SKIPPED_INFORMATION") x 4;
				if ( exists $lodByNumOfIterationsAndDup{$currNumOfIterations}) {
                    $fieldsStr = ",".(&getLodFieldsString($lodByNumOfIterationsAndDup{$currNumOfIterations}, $currNumOfIterations));
                }
				print FINAL_CSV $fieldsStr;	
			}
			print FINAL_CSV "\n";
			++$currPointNum;
		}
	}
	close FINAL_CSV;
	
	
	#Omer 16/9/2011 - there may be two lines of the same marker in the csv file if there were two overlapping windows.
	#This can happen if the last window was "too small", and then more markers were added to the beginning
	#of it in the function getWindowsList(). We overcome this problem by only keeping the first line
	#with any marker data, just like in regular multipoint.
	my @csvLines = ();
	my %pointsHash = ();
	open FINAL_CSV, "$FINAL_OUTPUT_CSV_FILENAME";
	while (my $currLine = <FINAL_CSV>) {
		my @currLineArr = split ',', $currLine;
		my $markerName = $currLineArr[0];
		if (!defined($pointsHash{$markerName})) {
			$pointsHash{$markerName} = 1;
			push @csvLines, $currLine;
		}
	}
	close FINAL_CSV;
	open FINAL_CSV, ">$FINAL_OUTPUT_CSV_FILENAME";
	foreach my $csvLine (@csvLines) {print FINAL_CSV $csvLine;}
	close FINAL_CSV;
}

##################
# dvir 2011/09/7
# compute the lod and other fields using the results from the computations
# return a string of all the fields seperare with ','
sub getLodFieldsString( \@ $ ) {
	# each cell is the lod result from diffrenet computations
	# the num of iterations that was used to compute this lod data at all the computations is the same
	my $lodDataArrRef = $_[0];
	my $currNumOfIterations = $_[1];	
	my $numOfSuccessfulComputations = 0;
	
	#Omer 11/9/2011 - the fields below are needed to compute GR scores
	my $lodSum = 0;
	my $reportedStdErrSum = 0;
	my $reportedVarianceSum = 0;
	my $reportedVarianceSquaredSum = 0;
	my $lodSumPower2 = 0;
	
	# the lod data for the given point for all the computations (all the computations are with the same number of iterations)
	for my $lodData (@$lodDataArrRef) {
		if (!Scalar::Util::looks_like_number($lodData->lod())) {next;}
		if (!Scalar::Util::looks_like_number($lodData->stdErr())) {next;}		
		++$numOfSuccessfulComputations;
		
		#Omer 11/9/2011 - update GR-related fields
		$lodSum += $lodData->lod();
		$reportedStdErrSum += $lodData->stdErr();
		$reportedVarianceSum += ($lodData->stdErr())**2;
		$reportedVarianceSquaredSum += ($lodData->stdErr())**4;
		$lodSumPower2 += ( $lodData->lod() )**2;		
	}
	
	#Omer 11/9/2011 - compute the average stderr and LOD score reported by Morgan
	my $avgLOD = "NA";
	my $standardDeviationLod = "NA";
	my $avgReportedStdErr = "NA";
	my $GRScore = "NA";
	if ($numOfSuccessfulComputations > 0) {
		$avgLOD = $lodSum / $numOfSuccessfulComputations;
		$avgReportedStdErr = $reportedStdErrSum / $numOfSuccessfulComputations;	
	};
	
	#If more than one computer performed a computation, compute GR statistics
	if ($numOfSuccessfulComputations > 1) {
		my $lodVariance = 1/($numOfSuccessfulComputations-1) * ($lodSumPower2 - $numOfSuccessfulComputations * ($avgLOD**2));
        if ($lodVariance < 0) {$lodVariance = 0;}   #Fix for small numeric errors
		$standardDeviationLod = sqrt($lodVariance);
		my $avgReportedVariance = $reportedVarianceSum / $numOfSuccessfulComputations;		
		$GRScore = &computeGelmanRubinScore(
			$numOfSuccessfulComputations,
			$currNumOfIterations,
			$reportedVarianceSum,
			$lodVariance,
			$reportedVarianceSquaredSum,
			$avgReportedVariance,
			$lodSumPower2,
			$avgLOD,
			\@$lodDataArrRef);
			
            #Handle special case of GRScore<1 due to numerical errors
            if (Scalar::Util::looks_like_number($GRScore) and $GRScore < 1) {$GRScore = 1;}
        }
	
	return"$avgLOD,$standardDeviationLod,$avgReportedStdErr,$GRScore";
}

##################




# dvir 2011/09/7
# creating the final output file
sub createOutputFiles( \% ) {
	my $class = $_[0]; # the name of this class
	my $variablesHashRef = $_[1];
	
	if ($class->doesAReportExists()) {
		return;
	}
	
	my $workDir = $class->getWorkDir();	
	chdir $workDir;
	
	my $projectTaskDir = $class->getProjectDir();
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	my $minNumOfIterations = $variables{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS};	
	
	my $outputDataDir = "$workDir\/$OUTPUT_DIRNAME";
	# getting the families
	my @families = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@families);
	
	my $progName = $$variablesHashRef{program};
	$progName =~ s/lm_markers/lm_linkage/g;
	my $title = "Results of $progName Analysis for Project $$variablesHashRef{PROJ}";
	
	open HTML_FILE, ">$HTML_OUTPUT_FILENAME";
	print HTML_FILE
		"<html>\n".
		"<head>\n".
		'<script type="text/javascript">
			function hideOrShow(idElement,idButtonLabel) {
				if (document.getElementById(idElement).style.display==\'none\') {
					document.getElementById(idElement).style.display=\'\';
					document.getElementById(idButtonLabel).innerHTML = \'Hide results\';
				}
				else {
					document.getElementById(idElement).style.display=\'none\';
					document.getElementById(idButtonLabel).innerHTML  = \'Show results\';
				}				
			}
		</script>'."\n".
		"<title>$title</title>\n".
		"<STYLE TYPE=\"text/css\">\n".
		"TD, TH {white-space: nowrap; overflow: hidden}\n".
		"TH {color:#0000FF; padding:2px;}\n".
		"TD {text-align:center;padding:3px;}\n".
    	"</STYLE>\n".
		"</head>\n".
		"<body bgcolor=\"#F0FFFF\">\n".
		"<center><p><h1><u>$title</u></h1></p></center>\n";
	
	# passing over the families
    my %aggregateLodPerFamilyHash = ();
    my @distances = ();    
    my $arePlotterArraysInitialized = 0;
    
	foreach my $familyNum (@families) {
	
		#Create handles markers array for this family		
		my %handledMarkersHash = ();
	
		open FAM_OUT_DIR_LIST, "$outputDataDir\/fam$familyNum$FAMILIES_DIR_LIST_SUFFIX";
		my @famiDirs = <FAM_OUT_DIR_LIST>;
		close FAM_OUT_DIR_LIST;
		
		# finding the max numOfIterations for this family
		my $maxNumOfIterations = 0;
		for my $dir (@famiDirs) {
			chomp($dir);
			my $file = "$dir\/$MAX_NUM_OF_ITERATIONS_FILENAME";
            unless (-e $file) {next;}			
            open MAX, $file;
            my $currJobMaxNumOfIterations = <MAX>;
            chomp($currJobMaxNumOfIterations);
            close MAX;
            $maxNumOfIterations = List::Util::max($maxNumOfIterations, $currJobMaxNumOfIterations);			
		}
        
        #Print HTML table title for this familiy
		my $htmlLodTableDivId = "family_${familyNum}";
		my $currFamGraphFileName = &Plotter::getGraphFileNameByFamilyNum($familyNum);
		my $htmlGraphImageText = "<center><img src=\"${currFamGraphFileName}\" /></center>";
		my $hiderShowerId = "hiderShowerButtonFam_".$familyNum;
		my $htmlFamilyHiderShowerText = "<a id=\"$hiderShowerId\" href=\"javascript:void(0);\" onclick=\"hideOrShow('$htmlLodTableDivId', '$hiderShowerId');\">Show results</a>";
		my $htmlFamilyDivStyle = 'style="display:none"';
		print HTML_FILE 
			"<p>Results of all iterations for family $familyNum: $htmlFamilyHiderShowerText</p>\n".
			"<div id=\"$htmlLodTableDivId\" $htmlFamilyDivStyle>\n".
			$htmlGraphImageText."\n".
			"<table align=\"center\" border=\"1\" cellpadding=\"0\" cellspacing=\"0\">\n".
			"<thead>\n";

        #Create an array of iteration numbers
		my @numOfIterationsArr = ();
		for (my $currNumOfIterations = $minNumOfIterations;
			 $currNumOfIterations <= $maxNumOfIterations;
			 $currNumOfIterations = &getNextNumOfIterations($currNumOfIterations)) {
			push @numOfIterationsArr, $currNumOfIterations;
		}		
		#The array must have at least one number of iterations
		if (scalar(@numOfIterationsArr) == 0) {push @numOfIterationsArr, $minNumOfIterations;}
        
        #Create the top of the HTML table for this family (titles row)
		my $iterationsHeadline = '';		
		$iterationsHeadline = '<th colspan="4">'.join(' iterations</th><th colspan="4">', @numOfIterationsArr).' iterations</th>';
		        
		print HTML_FILE '<tr><th rowspan="2">Marker 1 - Marker 2</th><th rowspan="2">Distance(CM)</th>'."$iterationsHeadline</tr>\n";
		my $numberOfTitleRepeatitions = scalar(@numOfIterationsArr);
		my $secondHeader = '<th>LodScore</th><th>StdErr</th><th>ReportedStdErr</th><th>GRScore</th>' x $numberOfTitleRepeatitions;
		print HTML_FILE "<tr>$secondHeader</tr>\n";
		print HTML_FILE "</thead>\n";
		
		# printing the lod scores of the family
		for my $dir (@famiDirs) {
			my $file = "$dir\/$FINAL_OUTPUT_CSV_FILENAME";
            
            #If the current directory does not have a final output file yet, then print "unknown" messages only
			if (! -e $file) {
				open JOB_RESULT, "$dir\/$INITIAL_OUTPUT_CSV_FILENAME";
				while (my $currLine = <JOB_RESULT>) {
					if ($currLine eq $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR) {next;} # to the next point data
					chomp($currLine);
					my @currLineArr = split ',', $currLine;
					
					#Format the line
					for my $arrElem (@currLineArr) {
						if (Scalar::Util::looks_like_number($arrElem)) {$arrElem = sprintf("%.4f", $arrElem); $arrElem =~ s/\.0*$|0*$//;}
					}
					my $lineHtmlText = "<td>".join ("</td><td>", @currLineArr)."</td>";
                    my $unknownMessage = &ProcessingBackend::printColoredNumber($ProcessingBackend::UNCALCULATED_INFORMATION);
                    my $numberOfMissingIterations = 4 * scalar(@numOfIterationsArr) - scalar(@currLineArr) + 2;
                    my $restOfLine = ("<td>".$unknownMessage."</td>") x $numberOfMissingIterations;
					print HTML_FILE "<tr>".$lineHtmlText.$restOfLine."</tr>\n";
				}
				close JOB_RESULT;				
				next; # moving to the next dir
            }
			
			# If we reached here then the computations for this directory is finished
			open JOB_RESULT, $file;
			while (my $currLine = <JOB_RESULT>) {
				chomp($currLine);
				my @currLineArr = split ',', $currLine;
				
				#Format the line
				for my $arrElem (@currLineArr) {
					if (Scalar::Util::looks_like_number($arrElem)) {$arrElem = sprintf("%.4f", $arrElem); $arrElem =~ s/\.0*$|0*$//;}
				}								
				
				my $lineHtmlText = '<tr>';
				my $colNum = -2; # -1 because the first column is the distance
				for my $currCellText (@currLineArr) {
					my $currCellTextForHtml = $currCellText;					
                    if ($currCellText eq $ProcessingBackend::ERROR_IN_COMPUTATION) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}
					elsif ($currCellText eq $ProcessingBackend::SKIPPED_INFORMATION) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}
                    # each 4-th cell is a cell with a lod score
					elsif ($colNum % 4 == 0) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}
					$lineHtmlText.= "<td>$currCellTextForHtml</td>";
					++$colNum;
				}
                
                #Print skipped message for computations that weren't made because of time-out
                my $skippedMessage = &ProcessingBackend::printColoredNumber($ProcessingBackend::SKIPPED_INFORMATION);
                my $numberOfMissingIterations = 4 * scalar(@numOfIterationsArr) - scalar(@currLineArr) + 2;
                my $restOfLine = ("<td>".$skippedMessage."</td>") x $numberOfMissingIterations;
                $lineHtmlText .= $restOfLine;
                
                #Finish the HTML row and print it to the HTML file
				$lineHtmlText .= '</tr>'."\n";
				print HTML_FILE $lineHtmlText;
			}
			close JOB_RESULT;
		}
		
		print HTML_FILE "</tbody></table><br/><br/></div>\n";
        
        
               
        ###############################Omer - start of new table code#####################################        
        #Determine the window size
        my $windowSize = $variables{jump};
        if (Scalar::Util::looks_like_number($windowSize)) {$windowSize--;}
        else {
            if (Scalar::Util::looks_like_number($variables{from}) and Scalar::Util::looks_like_number($variables{to})) {
                $windowSize = $variables{to} - $variables{from} + 1;
            }
            else {$windowSize = scalar(@{$variables{usedMarkersId}});}
        }
		
        #Create the top of the HTML table for this family (titles row)        
		$htmlLodTableDivId = "family_${familyNum}_2";		
		$hiderShowerId = "hiderShowerButtonFam_".$familyNum."_2";	
		$htmlFamilyHiderShowerText = "<a id=\"$hiderShowerId\" href=\"javascript:void(0);\" onclick=\"hideOrShow('$htmlLodTableDivId', '$hiderShowerId');\">Hide results</a>";
        $htmlFamilyDivStyle = 'style="display:block"';
        print HTML_FILE 
			"<p>Results of longest run for family $familyNum: $htmlFamilyHiderShowerText</p>\n".
			"<div id=\"$htmlLodTableDivId\" $htmlFamilyDivStyle>\n".
            $htmlGraphImageText."\n".
			"<table border=\"1\" align=\"center\" cellpadding=\"0\" cellspacing=\"0\">\n".
			"<thead>\n";
        
		print HTML_FILE '<tr><th>Group id</th><th>Marker 1 - Marker 2</th><th>Distance(CM)</th><th>#Iterations</th><th>LOD</th><th>GR</th></tr>'."\n";
		print HTML_FILE "</thead>\n";
        
        #Define plotter-related arrays
        @{$aggregateLodPerFamilyHash{$familyNum}} = ();        
		
		# printing the lod scores of the family
        my $rowCounter = 0;        
		for my $dir (@famiDirs) {
			my $file = "$dir\/$FINAL_OUTPUT_CSV_FILENAME";            
            
            #If the current directory does not have a final output file yet, then print "unknown" messages only
			if (! -e $file) {
				open JOB_RESULT, "$dir\/$INITIAL_OUTPUT_CSV_FILENAME";
				while (my $currLine = <JOB_RESULT>) {
					if ($currLine eq $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR) {next;} # to the next point data
					chomp($currLine);
					my @currLineArr = split ',', $currLine;
					
					#If this is the second time we handle this marker (due to overlapping last window), don't print it
					#and don't add it to the plotter array
					my $currMarkersNames = $currLineArr[0];
					if (defined($handledMarkersHash{$currMarkersNames})) {next;}
					$handledMarkersHash{$currMarkersNames} = 1;									
                    
                    #Add fields for the plotter
                    unless ($arePlotterArraysInitialized) {push(@distances, $currLineArr[1])};                    
                    push (@{$aggregateLodPerFamilyHash{$familyNum}}, $ProcessingBackend::UNCALCULATED_INFORMATION);
                    
					#Format the line
					for my $arrElem (@currLineArr) {
						if (Scalar::Util::looks_like_number($arrElem)) {$arrElem = sprintf("%.4f", $arrElem); $arrElem =~ s/\.0*$|0*$//;}
					}
					my $lineHtmlText = "<td>".join ("</td><td>", @currLineArr)."</td>";
                    #Add group id cell
                    my $colorString = "";
                    my $windowNum = int($rowCounter / $windowSize)+1;
                    if ($windowNum %2 == 0) {$colorString = " bgcolor=#D3D3D3";}
                    if ($rowCounter % $windowSize == 0) {
                        my $rowSpan = $windowSize;
                        $lineHtmlText = "<td rowspan=$rowSpan><b><font color=#0000ff>$windowNum<b></font></td>".$lineHtmlText;
                    }
                    
                    my $unknownMessage = &ProcessingBackend::printColoredNumber($ProcessingBackend::UNCALCULATED_INFORMATION);
                    my $restOfLine = "<td>0</td><td>$unknownMessage</td><td>NA</td>";                    
                    print HTML_FILE "<tr".$colorString.">".$lineHtmlText.$restOfLine."</tr>\n";
                    $rowCounter++;
				}
				close JOB_RESULT;				
				next; # moving to the next dir
            }
			
			# If we reached here then the computations for this directory is finished
			open JOB_RESULT, $file;
			while (my $currLine = <JOB_RESULT>) {
				
				chomp($currLine);
				my @currLineArr = split ',', $currLine;
                my $numberResults = int((scalar(@currLineArr)-2)/4);
                my $numberOfIterations = $numOfIterationsArr[$numberResults-1];
				
				#If this is the second time we handle this marker (due to overlapping last window), don't print it
				#and don't add it to the plotter array
				my $currMarkersNames = $currLineArr[0];
				if (defined($handledMarkersHash{$currMarkersNames})) {next;}
				$handledMarkersHash{$currMarkersNames} = 1;				
                
                #Omer - don't take the last results, if the GR score for them is unknown				
                if ($numberResults > 1 and !Scalar::Util::looks_like_number($currLineArr[-1]) and Scalar::Util::looks_like_number($currLineArr[-5])) {
                    $numberOfIterations = $numOfIterationsArr[$numberResults-2];
                    @currLineArr = ($currLineArr[0],$currLineArr[1], $numberOfIterations ,$currLineArr[-8],$currLineArr[-5]);
                }
                else {
                    @currLineArr = ($currLineArr[0],$currLineArr[1], $numberOfIterations ,$currLineArr[-4],$currLineArr[-1]);
                }
                
				#Format the line
				for my $arrElem (@currLineArr) {
					if (Scalar::Util::looks_like_number($arrElem)) {$arrElem = sprintf("%.4f", $arrElem); $arrElem =~ s/\.0*$|0*$//;}
                }                
                
                #Add group id cell
                my $lineHtmlText = "";
                my $colorString = "";
                my $windowNum = int($rowCounter / $windowSize)+1;
                if ($windowNum %2 == 0) {$colorString = " bgcolor=#D3D3D3";}
                if ($rowCounter % $windowSize == 0) {
                    my $rowSpan = $windowSize;
                    $lineHtmlText .= "<td rowspan=$rowSpan><b><font color=#0000ff>$windowNum</b></font></td>".$lineHtmlText;
                }                
                $lineHtmlText = "<tr".$colorString.">".$lineHtmlText;
				
				my $colNum = 0;
                my $grScore = $currLineArr[4];
				for my $currCellText (@currLineArr) {
					my $currCellTextForHtml = &General::trim($currCellText);
					if ($colNum == 0) {  #Markers name
						my $trimmedString = $currCellTextForHtml;
						my $numWhiteSpaces = ($trimmedString =~ s/\s+/\<br\>/g); #Replace whitespaces with line breaks
						if ($numWhiteSpaces > 2) {$currCellTextForHtml = $trimmedString;}						
					}					
                    if ($colNum <= 1) {$currCellTextForHtml = "<font color=#0000FF><b>".$currCellTextForHtml."</b></font>";}
                    elsif ($currCellText eq $ProcessingBackend::ERROR_IN_COMPUTATION) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}
					elsif ($currCellText eq $ProcessingBackend::SKIPPED_INFORMATION) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}                    
					elsif ($colNum == 3) {  #LOD score
                        if (!Scalar::Util::looks_like_number($grScore) or $grScore < 10) {$currCellTextForHtml = &ProcessingBackend::printColoredNumber($currCellText);}                        
                        else {$currCellTextForHtml = '<font color="SaddleBrown"><b>Unreliable</b></font>';}
                    }
                    elsif ($colNum == 4 and Scalar::Util::looks_like_number($grScore)) {  #GR score
                        if ($grScore < 5) {$currCellTextForHtml = "<font color=#006600><b>".$currCellText."</font></b>";}       #Good GR score
                        elsif ($grScore > 10) {$currCellTextForHtml = "<font color=#FF6666>".$currCellText."</font>";}   #Bad  GR score
                    }                    
					$lineHtmlText.= "<td>$currCellTextForHtml</td>";
					++$colNum;
				}
                
                #Add fields for the plotter
                unless ($arePlotterArraysInitialized) {push(@distances, $currLineArr[1])};
                if (Scalar::Util::looks_like_number($grScore) and $grScore < 10) {push(@{$aggregateLodPerFamilyHash{$familyNum}}, $currLineArr[3]);}
                else {push(@{$aggregateLodPerFamilyHash{$familyNum}}, undef);}
                
                #Finish the HTML row and print it to the HTML file
				$lineHtmlText .= '</tr>'."\n";
				print HTML_FILE $lineHtmlText;
                $rowCounter++;
			}
			close JOB_RESULT;            
		}
        $arePlotterArraysInitialized = 1;
		print HTML_FILE "</tbody></table><br/><br/></div>\n";        
        ###############################Omer - end of new table code#####################################
	}
	print HTML_FILE '</body></html>';
	close HTML_FILE;
    
	
    #Create graphs
    my @lodScoresTotal = ();
    foreach my $dist (@distances) {push(@lodScoresTotal, 0);}
    foreach my $familyNum (keys %aggregateLodPerFamilyHash) {
        my $i = 0;
        foreach my $lod (@{$aggregateLodPerFamilyHash{$familyNum}}) {
            if (Scalar::Util::looks_like_number($lod)) {$lodScoresTotal[$i] += $lod;}            
            $i++;
        }
    }
    &Plotter::plotLodGraphs(\@distances, \@lodScoresTotal, \%aggregateLodPerFamilyHash);
	
	# dvir, change this, in order that the script will succeed
	`touch $GenomeBrowser::BED_OUTPUT_FILENAME`;	
}

# dvir 2011/09/07
# will be used to read the results from the given filehandle,
# returns:
# 1. max
# and updaing the data structre that holds the results 
#
# Returning an indication about have we read a results.
sub readResultsFromFile($) {
	my $resultsFileHandleRef = $_[0];
	# the num of points in which we have computed the lod
	my $numOfPoints = $_[1];
	
	# for now, point isn't a marker, but the place between the markers
	# but this is abstract for this method
	my @lodScoreByPoint = ();
	
	my $currNumOfIterations = undef;
	# looking for another sub job ("new Num Of Iterations")
	while (my $currLine = <$resultsFileHandleRef>) {		
		# if we have finished reading the results of the curr window
		if ($currLine eq $Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR) {return ($currNumOfIterations, @lodScoreByPoint);}
		$currLine = &General::trim($currLine);
		
		if ($currLine eq $Configuration::MORGAN_LM_MARKERS_SEPERATOR) {
			$currLine = <$resultsFileHandleRef>;
			$currNumOfIterations = &General::trim($currLine);
			next;
		}
		
		# if the current line is not the begining of a lod score, than moving to the next line
		# because $currLine was trim, than it was compared to "or marker" and not to "   or marker"
		if (defined $currLine and $currLine !~ /^or marker/) {next;} # "   or marker        male     female       LodScore    StdErr"
		
		<$resultsFileHandleRef>; # reading an empty line
		
		# reading the point lod score lines
		for (my $currPoint = 0; $currPoint < $numOfPoints; ++$currPoint) {
			# reading an uninformative line (NA data)
			# example for that line : "marker-1       0.000      0.000             NA        NA"
			<$resultsFileHandleRef>; 
			
			$currLine = <$resultsFileHandleRef>; # the first marker line
			
			my @lineData = split(/\s+/,$currLine);
			
			my $pointLodData = MarkerLodData->new(); # MarkerLodData is at the current package
			# example for the lines: 
			#   or marker        male     female       LodScore    StdErr
			#           1       0.000      0.000         0.0000    0.0000
			# setting the values
			$pointLodData->lod( $lineData[4] );
			$pointLodData->stdErr( $lineData[5] );

			$lodScoreByPoint[$currPoint]{$currNumOfIterations} = $pointLodData;
		}		
	}
	# if we return from here, than it means, that we havn't read the lod table.
	return ($currNumOfIterations, @lodScoreByPoint);
}

my $NA_STR = 'NA';

#Compute Gelman-Rubin Statistic, taken from Gelman and Rubin (1992b)
#Using notations from the paper (even though variable names are not clear and possibly redundant)
sub computeGelmanRubinScore($ $ $ $ $ $ $ $ \@) {
	my $numOfSuccessfulComputations = $_[0];
	my $currNumOfIterations = $_[1];
	my $reportedVarianceSum = $_[2];
	my $lodVariance = $_[3];
	my $reportedVarianceSquaredSum = $_[4];
	my $avgReportedVariance = $_[5];
	my $lodSumPower2 = $_[6];
	my $avgLod = $_[7];
	my $lodDataArrByComputationNum = $_[8];
    
    #Handle special side-cases
	if ($numOfSuccessfulComputations < 2) {return $NA_STR;}
    elsif ($reportedVarianceSum == 0 and $lodVariance < 0.0001) {return 1;}
    elsif ($reportedVarianceSum == 0) {return "infinity";}
	
    #Define variables in the notation of the paper
	my $m = $numOfSuccessfulComputations;    #Definition of m
	my $n = $currNumOfIterations;		 #Definition of n
	my $W = $reportedVarianceSum / $m;	 #Definition of W, from bottom of page 460
	my $B_n = $lodVariance;			 #Definition of B/n, from bottom of page 460
	my $B = $B_n * $n;			 #Definition of B, from bottom of page 460

	#Compute degrees of freedom - all the following computations are taken from page 464
	my $var_W = 1/($m-1) * ($reportedVarianceSquaredSum / $m - ($avgReportedVariance**2));	#Taken from bottom left of page 464	
	if ($var_W < 0) {$var_W = 0;}
	my $var_B = 2*($B**2) / ($m-1);	#Taken from bottom left page 464
	my $avgLodSquared = $lodSumPower2 / $m;		#Average of xi^2

	#Compute covariances
	my $covSi2Xi2 = 0;
	my $covSi2Xi = 0;
	for my $lodData (@$lodDataArrByComputationNum) {
		if (!(defined $lodData)) {next;}
		$covSi2Xi2 += (($lodData->stdErr())**2 - $avgReportedVariance) * ((($lodData->lod())**2) - $avgLodSquared);  #cov(si^2,xi^2)
		$covSi2Xi  += (($lodData->stdErr())**2 - $avgReportedVariance) * ((($lodData->lod()) - $avgLod));	     #cov(si^2, xi)
	}
	$covSi2Xi2 /= ($m-1);	#Fix covariance
	$covSi2Xi /= ($m-1);    #Fix covariance

	my $covW_B = $n/$m * ($covSi2Xi2 - 2*$avgLod*$covSi2Xi);     #Equation 18
	my $var_V_hat = ((($n-1)/$n)**2) * $var_W + (($m+1)/($m*$n))**2 * $var_B + 2*($m-1)*($n-1)/($m*($n**2))*$covW_B;   #Equation 17
	my $sigmaHatSquared = ($n-1)/$n*$W + (1/$n)*$B;      #Equation 3
	my $vHat = $sigmaHatSquared + $B/($m*$n);	#Equation 12
    #Finished computing covariances

    if ($var_V_hat > 0) {
        my $d = 2 * $vHat**2 / $var_V_hat;	#Between Equations 16 and 17    
        my $GelmanRubinScore = sqrt((($n-1)/$n + ($m+1)/($m) * $B_n/$W) * ($d+3)/($d+1));      #Equation 20 + fix for d, taken from Brooks and Gelman (1998)
        return $GelmanRubinScore;
    }
    
    else {
        my $GelmanRubinScore = sqrt(($n-1)/$n + ($m+1)/($m) * $B_n/$W);      #Equation 20 + fix for d, taken from Brooks and Gelman (1998)
        return $GelmanRubinScore;    
    }
}

1;		
