# dvir, 2011-07-28
# plotting using gnu plotter
package Plotter;

use strict;

use General;
use Configuration;
use SharedConsts;

# it is called "initial" because if there is a lod value greater than it, than we will update it.
my $INITIAL_MAX_LOD_VALUE = 4;

my $GRAPH_WIDTH_IN_PIXELS = 640;
my $GRAPH_HEIGHT_IN_PIXELS = 480;

my $GRAPH_FOR_VIEW_RESULTS_BUTTON_WIDTH_IN_PIXELS = 333;
my $GRAPH_FOR_VIEW_RESULTS_BUTTON_HEIGHT_IN_PIXELS = 250;

my $NO_TITLE = '';

my $FS = $SharedConsts::LOD_SCORES_DATA_FIELD_SEPEREATOR;

# this is the radius of the distance to cluster the markers in TwoPoint avarage graph
my $CLUSTERING_RADIUS = 1;
# parameters of graphs of avarage TwoPoint results
my $AvgLodScoreFilename = 'avg_graphs_data.xls';

# dvir, 2011-08-01
# if there is more than 1 family, then 
# it will create a unique graph file for each the families
# and a graph for all the families.
# else it will only create a graph for the single family
#
# in the graphs the x-axis is the distance from the first marker (which is usually a distance from the begining of the chromozome).
#
# the terminology is lod score at position and not at marker,
# because there are cases in which the lod score is in the middle of two markers ..
sub plotLodGraphs( \@ \% \@) {
		
	# position is an alias to "distance from the first marker"
	# first marker position is at index 0
	my $postionsArrRef = $_[0];

	# array, for each marker, its average lod score over all families
	# first marker lod is at index 0	
	my $aggregateLodForAllFamiliesArrRef = $_[1];
	
	# hash, for each family we will have an array of all the lod scores.
	# first marker lod is at index 0
	my $aggregateLodPerFamilyHashRef = $_[2];
	
	my @familiesNums = keys %$aggregateLodPerFamilyHashRef;
	
	my $familiesStr = join $FS, @familiesNums;
	
	my $allFamiliesColumnLabel = "all";
	my $positionsColumnLabel = "postions";	
	
	my %maxLodValuePerFamily = ();
	for my $currFamily (@familiesNums) {
		$maxLodValuePerFamily{$currFamily} = $INITIAL_MAX_LOD_VALUE;
	}
	my $maxLodValueForAllFamilies = $INITIAL_MAX_LOD_VALUE;
	
	# creating the graph data file
	open GRAPHS_DATA, ">$SharedConsts::LOD_SCORES_DATA_FILENAME";
	# the order of the columns is important for the drawing part "using 1:__"
	print GRAPHS_DATA $positionsColumnLabel.$FS.$allFamiliesColumnLabel.$FS.$familiesStr."\n";
	
	for (my $currPositionIndex = 0; $currPositionIndex < scalar(@$postionsArrRef); ++$currPositionIndex) {
		my $currPosition = $$postionsArrRef[$currPositionIndex];
		
		my $currMarkerAllFamiliesLod = $$aggregateLodForAllFamiliesArrRef[$currPositionIndex];
		# making sure that the lod is defined and its numeric
		if (defined $currMarkerAllFamiliesLod && $currMarkerAllFamiliesLod =~ /\d/) {
			if ($currMarkerAllFamiliesLod < $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH) {
				$currMarkerAllFamiliesLod = $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH;
			}		
			if ($currMarkerAllFamiliesLod > $maxLodValueForAllFamilies) {
				$maxLodValueForAllFamilies = $currMarkerAllFamiliesLod;
			}
		}
		else {
			$currMarkerAllFamiliesLod = '';
		}
		
		my $currPositionData = $currPosition.$FS.$currMarkerAllFamiliesLod;

		for my $currFamily (@familiesNums) {
			
			my $lod = $$aggregateLodPerFamilyHashRef{$currFamily}[$currPositionIndex];

			if (defined $lod && $lod =~ /\d/) {
				if ($lod < $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH) {
					$lod = $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH;
				}		
				if ($lod > $maxLodValuePerFamily{$currFamily}) {
					$maxLodValuePerFamily{$currFamily} = $lod;
				}
			}
			else {
				$lod = '';
			}

			$currPositionData .= $FS.$lod;
		}
		print GRAPHS_DATA $currPositionData."\n";
	}
	
	close GRAPHS_DATA;
	
	my $startPos = $$postionsArrRef[0];
	my $endPos = $$postionsArrRef[$#$postionsArrRef];
	if (scalar(@$postionsArrRef) == 1) {
		$startPos = 0;
		$endPos = 2*$endPos;
		# dvir, 2011/11/16 : there is an error when trying to print with a range of [0:0]
		if ($endPos == 0) {
			$endPos = 1;
		}
	}
	#print $startPos.' '.$endPos."\n";
	
	# creating the graph
	my $gnuPlotCommandsFileName = 'gnu_plot_commnads.txt';
	my @shouldBeOutputFiles = ();
	
	open GNU_PLOT_CMD, ">$gnuPlotCommandsFileName";
	open GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE, ">$SharedConsts::GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE_FILENAME";
	
	my $currFamilyColumnOfLodScoreInDataFile = 3;
	for my $currFamily (@familiesNums) {
		my $currFamGraphFileName = &getGraphFileNameByFamilyNum($currFamily);
		push @shouldBeOutputFiles ,$currFamGraphFileName;
		
		# the main graph of the pedigree, which will be presented
		# at the html with the lod table
		my $gnuPlotCommand = getGnuPlotCommandToPlotAGraphOfLodScore(
			$currFamGraphFileName,
			"Pedigree $currFamily",
			$startPos,
			$endPos,
			$maxLodValuePerFamily{$currFamily},
			$currFamilyColumnOfLodScoreInDataFile,
			$GRAPH_WIDTH_IN_PIXELS,
			$GRAPH_HEIGHT_IN_PIXELS,
			undef,
			$SharedConsts::LOD_SCORES_DATA_FILENAME);
			
		print GNU_PLOT_CMD $gnuPlotCommand;
		
		# antoher graph, just smaller, and with a max y coord which is no bigger
		# than $INITIAL_MAX_LOD_VALUE
		# these graphs will be presented when user presses on the "View results" button
		# at the expirement page.
		# at that page, the user sees graphs for all the chromozomes in his initial snp file
		# (if he had uploaded a dat&ped file, than there is only one chromozome and one graph)
		# and in order that all the graphes will have the same graph skails at the y axis.	
		# than we are using a condition on the y-axis, to be no more than $INITIAL_MAX_LOD_VALUE
		# and now the user can simply compare between the results of the lod scores
		# at the html with the lod table
		$currFamGraphFileName = &getGraphFileNameByFamilyNum_For_ViewResults_Button($currFamily);
		push @shouldBeOutputFiles ,$currFamGraphFileName;
		my $gnuPlotCommandForShowResults_Template;
		($gnuPlotCommand, $gnuPlotCommandForShowResults_Template) = getGnuPlotCommandToPlotAGraphOfLodScore(
			$currFamGraphFileName,
			$NO_TITLE,
			$startPos,
			$endPos,
			$INITIAL_MAX_LOD_VALUE,
			"(\$$currFamilyColumnOfLodScoreInDataFile > $INITIAL_MAX_LOD_VALUE \? $INITIAL_MAX_LOD_VALUE-0.01 \: \$$currFamilyColumnOfLodScoreInDataFile)",
			$GRAPH_FOR_VIEW_RESULTS_BUTTON_WIDTH_IN_PIXELS,
			$GRAPH_FOR_VIEW_RESULTS_BUTTON_HEIGHT_IN_PIXELS,
			$currFamily,
			$SharedConsts::LOD_SCORES_DATA_FILENAME);
			
		print GNU_PLOT_CMD $gnuPlotCommand;		
		print GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE $gnuPlotCommandForShowResults_Template;
		
		++$currFamilyColumnOfLodScoreInDataFile;
	}
	
	my $allFamGraphFileName = &getGraphFileNameForAllFamiliesCombined();
	$currFamilyColumnOfLodScoreInDataFile = 2;
	
	push @shouldBeOutputFiles ,$allFamGraphFileName;
			
	my $gnuPlotCommand= getGnuPlotCommandToPlotAGraphOfLodScore(
		$allFamGraphFileName,
		"All analyzed pedigrees combined",
		$startPos,
		$endPos,
		$maxLodValueForAllFamilies,
		$currFamilyColumnOfLodScoreInDataFile,
		$GRAPH_WIDTH_IN_PIXELS,
		$GRAPH_HEIGHT_IN_PIXELS,
		undef,
		$SharedConsts::LOD_SCORES_DATA_FILENAME);
	print GNU_PLOT_CMD $gnuPlotCommand;
	
	# antoher graph, just smaller, and with a max y coord which is no bigger
	# than $INITIAL_MAX_LOD_VALUE
	# these graphs will be presented when user presses on the "View results" button
	# at the expirement page.
	$allFamGraphFileName = &getGraphFileNameForAllFamiliesCombined_For_ViewResults_Button();
	push @shouldBeOutputFiles ,$allFamGraphFileName;
	
	($gnuPlotCommand, my $gnuPlotCommandForShowResults_Template) = getGnuPlotCommandToPlotAGraphOfLodScore(
		$allFamGraphFileName,
		$NO_TITLE,
		$startPos,
		$endPos,
		$INITIAL_MAX_LOD_VALUE,
		"(\$$currFamilyColumnOfLodScoreInDataFile > $INITIAL_MAX_LOD_VALUE \? $INITIAL_MAX_LOD_VALUE-0.1 \: \$$currFamilyColumnOfLodScoreInDataFile)",
		$GRAPH_FOR_VIEW_RESULTS_BUTTON_WIDTH_IN_PIXELS,
		$GRAPH_FOR_VIEW_RESULTS_BUTTON_HEIGHT_IN_PIXELS,
		$allFamiliesColumnLabel,
		$SharedConsts::LOD_SCORES_DATA_FILENAME);	
	print GNU_PLOT_CMD $gnuPlotCommand;
	print GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE $gnuPlotCommandForShowResults_Template;	
		
	close GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE;
	close GNU_PLOT_CMD;
	
	
	my $errorFileName = 'gnuplot_exe.error';
	my $executableExitCode  = system ("gnuplot $gnuPlotCommandsFileName >> gnuplot_exe.out 2>> $errorFileName");
		
	my $areSomeFilesMissing = &General::checkExecutionExitCodeAndExistenceOfOutputFiles("Plotting",'',$executableExitCode, \@shouldBeOutputFiles, $errorFileName);
	
	return $areSomeFilesMissing;
}

###
# create a graph that each point is an avarage of all the points
# in a given radius ($CLUSTERING_RADIUS).
####
sub plotLodAvgGraphs( \@ \% \@) {
	# position is an alias to "distance from the first marker"
	# first marker position is at index 0
	my $postionsArrRef = $_[0];

	# array, for each marker, its average lod score over all families
	# first marker lod is at index 0	
	my $aggregateLodForAllFamiliesArrRef = $_[1];
	
	# hash, for each family we will have an array of all the lod scores.
	# first marker lod is at index 0
	my $aggregateLodPerFamilyHashRef = $_[2];
	
	my @familiesNums = keys %$aggregateLodPerFamilyHashRef;
	
	my $familiesStr = join $FS, @familiesNums;
	
	my $allFamiliesColumnLabel = "all";
	my $positionsColumnLabel = "postions";	
	
	my %maxLodValuePerFamily = ();
	for my $currFamily (@familiesNums) { $maxLodValuePerFamily{$currFamily} = $INITIAL_MAX_LOD_VALUE; }
	my $maxLodValueForAllFamilies = $INITIAL_MAX_LOD_VALUE;
	
	# creating the graph data file
	open GRAPHS_DATA, ">$AvgLodScoreFilename";
	# the order of the columns is important for the drawing part "using 1:__"
	print GRAPHS_DATA $positionsColumnLabel.$FS.$allFamiliesColumnLabel.$FS.$familiesStr."\n";
	
	my $currPositionIndex = 0;
	my $numOfFamilies = @familiesNums;
	while ($currPositionIndex < scalar(@$postionsArrRef)) {
		my $basePosition = $$postionsArrRef[$currPositionIndex];
		
		my @localAvgLOD = (); # index 0 is for all the famileis avg, index 1 is for family 1 and so on...
		for (my $i = 0; $i <= $numOfFamilies; ++$i) { $localAvgLOD[$i] = 0; }
		
		my $step = 1; # +1 to move to the right marker, -1 to move to the left marker
		my $numOfMarkersInRadius = 0;
		my $smoothingMarkerIndex = $currPositionIndex;
		for (my $TwoSides = 1; $TwoSides <= 2; ++$TwoSides) {
			while ($smoothingMarkerIndex < scalar(@$postionsArrRef) && $smoothingMarkerIndex >= 0 &&
						abs($$postionsArrRef[$smoothingMarkerIndex]-$basePosition) <= $CLUSTERING_RADIUS) {
				++$numOfMarkersInRadius;
				my $currMarkerAllFamiliesLod = $$aggregateLodForAllFamiliesArrRef[$smoothingMarkerIndex];
				# making sure that the lod is defined and its numeric
				if (defined $currMarkerAllFamiliesLod && $currMarkerAllFamiliesLod =~ /\d/) {
					if ($currMarkerAllFamiliesLod < $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH) {
						$currMarkerAllFamiliesLod = $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH;
					} elsif ($currMarkerAllFamiliesLod > $maxLodValueForAllFamilies) {
						$maxLodValueForAllFamilies = $currMarkerAllFamiliesLod;
					}
				} else { #the marker's LOD is illigal
					$currMarkerAllFamiliesLod = 0;
				}
				$localAvgLOD[0] += $currMarkerAllFamiliesLod; #index num 0 - all families LOD

				my $familyNum = 1;
				for my $currFamily (@familiesNums) {
					my $lod = $$aggregateLodPerFamilyHashRef{$currFamily}[$smoothingMarkerIndex];
					if (defined $lod && $lod =~ /\d/) {
						if ($lod < $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH) {
							$lod = $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH;
						} elsif ($lod > $maxLodValuePerFamily{$currFamily}) {
							$maxLodValuePerFamily{$currFamily} = $lod;
						}
					} else { #the marker's LOD is illigal
						$lod = 0;
					}
					$localAvgLOD[$familyNum] += $lod;
					++$familyNum;
				}
				$smoothingMarkerIndex += $step;
			} # inner while
			$step = -1;
		} # for loop
		#calculating the avrages
		for (my $i = 0; $i <= $numOfFamilies; ++$i) { $localAvgLOD[$i] /= $numOfMarkersInRadius; }
		
		my $currPositionData = $basePosition.$FS;
		$currPositionData .= join $FS, @localAvgLOD;
		print GRAPHS_DATA $currPositionData."\n";
		++$currPositionIndex;
	} # outer while
	
	close GRAPHS_DATA;
	
	my $startPos = $$postionsArrRef[0];
	my $endPos = $$postionsArrRef[$#$postionsArrRef];
	if (scalar(@$postionsArrRef) == 1) {
		$startPos = 0;
		$endPos = 2*$endPos;
		# there is an error when trying to print with a range of [0:0]
		if ($endPos == 0) { $endPos = 1; }
	}
	
	# creating the graph
	my $gnuPlotCommandsFileName = 'gnu_avg_plot_commnads.txt';
	my @shouldBeOutputFiles = ();
	
	open GNU_PLOT_CMD, ">$gnuPlotCommandsFileName";
	
	my $currFamilyColumnOfLodScoreInDataFile = 3;
	for my $currFamily (@familiesNums) {
		my $currFamGraphFileName = 'LodAvgGraph.pedfile_'.$currFamily.'.'.$Configuration::LOD_SCORE_GRAPH_FILES_EXTENTION;
		push @shouldBeOutputFiles ,$currFamGraphFileName;
		
		# the main graph of the pedigree, which will be presented
		# at the html with the lod table
		my $gnuPlotCommand = getGnuPlotCommandToPlotAGraphOfLodScore(
			$currFamGraphFileName,
			"Pedigree $currFamily - smoothing graph ($CLUSTERING_RADIUS cM radius)",
			$startPos,
			$endPos,
			$maxLodValuePerFamily{$currFamily},
			$currFamilyColumnOfLodScoreInDataFile,
			$GRAPH_WIDTH_IN_PIXELS,
			$GRAPH_HEIGHT_IN_PIXELS,
			undef,
			$AvgLodScoreFilename);
			
		print GNU_PLOT_CMD $gnuPlotCommand;

		++$currFamilyColumnOfLodScoreInDataFile;
	}
	
	my $allFamGraphFileName = 'LodAvgGraph.pedfile_main.'.$Configuration::LOD_SCORE_GRAPH_FILES_EXTENTION;
	$currFamilyColumnOfLodScoreInDataFile = 2;
	
	push @shouldBeOutputFiles ,$allFamGraphFileName;
			
	my $gnuPlotCommand= getGnuPlotCommandToPlotAGraphOfLodScore(
		$allFamGraphFileName,
		"All analyzed pedigrees combined - smoothing graph ($CLUSTERING_RADIUS cM radius)",
		$startPos,
		$endPos,
		$maxLodValueForAllFamilies,
		$currFamilyColumnOfLodScoreInDataFile,
		$GRAPH_WIDTH_IN_PIXELS,
		$GRAPH_HEIGHT_IN_PIXELS,
		undef,
		$AvgLodScoreFilename);
	print GNU_PLOT_CMD $gnuPlotCommand;

	close GNU_PLOT_CMD;
	
	my $errorFileName = 'gnuAvgPlotExe.error';
	my $executableExitCode  = system ("gnuplot $gnuPlotCommandsFileName >> gnuplot_exe.out 2>> $errorFileName");
		
	my $areSomeFilesMissing = &General::checkExecutionExitCodeAndExistenceOfOutputFiles("Plotting",'',$executableExitCode, \@shouldBeOutputFiles, $errorFileName);
	
	return $areSomeFilesMissing;
}

sub getGraphFileNameByFamilyNum( $ ) {
	my $familyNum = $_[0];
	
	return 'LODgraph.pedfile_'.$familyNum.'.'.$Configuration::LOD_SCORE_GRAPH_FILES_EXTENTION;
}

sub getGraphFileNameForAllFamiliesCombined() {
	return getGraphFileNameByFamilyNum("main");
}

sub getGraphFileNameByFamilyNum_For_ViewResults_Button( $ ) {
	my $familyNum = $_[0];
	
	# reducing 1 from the family number, since that the web server
	# treats image with the name "0" as the image of family no' 1
	# note: we assume that the pedigrees are starting from 1 at the ped file.
	--$familyNum;
	return getGraphFileNameByFamilyNum("${familyNum}$SharedConsts::SHOW_RESULTS_FILENAME_TEXT");
}

sub getGraphFileNameForAllFamiliesCombined_For_ViewResults_Button() {
	return getGraphFileNameByFamilyNum("main$SharedConsts::SHOW_RESULTS_FILENAME_TEXT");
}

# dvir, 2011-08-02
sub getGnuPlotCommandToPlotAGraphOfLodScore( $ $ $ $ $ $ $ $ $) {
	my $graphFileName = $_[0];
	my $graphTitle = $_[1];
	my $xAxisStartCoordinate = $_[2];
	my $xAxisEndCoordinate = $_[3];
	my $yAxisEndCoordinate = $_[4];
	# the column number in the file $SharedConsts::LOD_SCORES_DATA_FILENAME, which contains the
	# data of the drawing for the y coordinate 
	# (the x corrdinate is the first column (col no' 1)),
	# so $colNumOfYCoordData should be 2 or more.
	#
	# Note: it can be also a condition on that column such as ($3>4 ? 4 : $3).	
	my $colNumOfYCoordData = $_[5];
	my $imageWidthInPixels = $_[6];
	my $imageHeightInPixels = $_[7];
	my $familyNumForShowResults_Template = $_[8]; # can be "2" -> family no' 2 or "all" -> all families
	my $dataFileName = $_[9];
	
	my $gnuPlotCommand = 
		"reset\n".
		"set terminal png nocrop font \",8\" size $imageWidthInPixels,$imageHeightInPixels \n".
		"set output \"_CCCC_$graphFileName\"\n".
		"f(x) = 3.3\n".
		"g(x) = -2\n".
		"set xrange [$xAxisStartCoordinate:$xAxisEndCoordinate]\n".
		"set yrange [$SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH:$yAxisEndCoordinate]\n".
		"set pointsize 1\n".
		"set xlabel \"distance (cM)\"\n".
		"set ylabel \"LOD score\"\n";
	
	if ($graphTitle ne $NO_TITLE) {
		$gnuPlotCommand .= "set title \"$graphTitle\"\n";
	}
	
	# "lt" is linetype. possible values : "2" -> green, "3" -> blue
	$gnuPlotCommand .=
		"_AAAA_".
		"set multiplot\n".
		"plot \"_CCCC_$dataFileName\" using 1:$colNumOfYCoordData with linespoints pointtype 26 notitle\n".
		"plot f(x) with lines lt 3 notitle\n".
		"plot g(x) with lines lt 3 notitle\n".
		"_BBBB_".
		"unset multiplot\n\n";
	
	my $gnuPlotCommandForShowResults_Template;
	if (defined $familyNumForShowResults_Template) {	
		$gnuPlotCommandForShowResults_Template = $gnuPlotCommand;
		
		# g -> because _CCCC_ appears twice and I don't want it to stop the substituion
		# after the first substitute
		$gnuPlotCommandForShowResults_Template =~ s/_AAAA_/m(x) = $SharedConsts::MAX_LOD_SCORE_PER_FAMILY_STR_PREFIX$familyNumForShowResults_Template\n/g;
		$gnuPlotCommandForShowResults_Template =~ s/_BBBB_/plot m(x) with lines lt 2 notitle\n/g;
		$gnuPlotCommandForShowResults_Template =~ s/_CCCC_/$SharedConsts::LOD_SCORE_GRAPH_DIR_TEXT\//g;
	}
	
	# g -> because _CCCC_ appears twice and I don't want it to stop the substituion
	# after the first substitute
	$gnuPlotCommand =~ s/_AAAA_//g;
	$gnuPlotCommand =~ s/_BBBB_//g;
	$gnuPlotCommand =~ s/_CCCC_//g;

	if (defined $familyNumForShowResults_Template) {	
		return $gnuPlotCommand, $gnuPlotCommandForShowResults_Template;
	}
	else {
		return $gnuPlotCommand;
	}
}

# dvir 2011/08/23
# returning an array with all the files which are relevant for plotting
# that needs to be sent to the web server.
# the files are in the cwd.
sub getPlottingFilesNamesToBeSentToWebServer() {
	my @fileList = ();
	# opening the cwd
	opendir(WORKDIR,'.'); 
	for my $fileName ( readdir(WORKDIR) ) {
		#print "$fileName\n";
		if( $fileName =~ /\.$Configuration::LOD_SCORE_GRAPH_FILES_EXTENTION$/ ) {
			push @fileList, $fileName;
		}
		if( $fileName =~ /$SharedConsts::GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE_FILENAME$/ ) {
			push @fileList, $fileName;
		}
		if( $fileName =~ /$SharedConsts::LOD_SCORES_DATA_FILENAME$/ ) {
			push @fileList, $fileName;
		}				
	}
	closedir WORKDIR;
	
	return @fileList;
}

1;