#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";

use strict;
use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use General;
use DatPedHandler;
use DatWrapper;
use PedWrapper;

#######
#
#start
########
my $workDir = $ARGV[0];
chdir($workDir);

my $projName = $ARGV[1];
my $chromName = $ARGV[2];
my $semid = $ARGV[3];

#Initial preparations
#dvir 2011/12/04 - no more use in semaphores
#&General::catchSemaphore($semid);
my %variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$projName, $chromName) };
$workDir = "$workDir\/cleaning";
( -d $workDir ) or &General::reportOnErrorReleaseSemaphoresAndDie('PostCleanig.pl',$workDir,"Post isn't running where Pre did!- A BUG!");

#Find the original dat and ped files (from which markers need to be removed)
my $datFileName;
my $pedFileName;
if (-e "$workDir\/datafileTmp.dat") {
	$datFileName = "$workDir\/datafileTmp.dat";
	$pedFileName = "$workDir\/pedfileTmp.dat";
}
else {
	$datFileName = "$workDir\/$variables{ Datfile }";
	$pedFileName = "$workDir\/$variables{ Pedfile }";	
}

#Create a hash that maps marker names to their numbers
my %markerNamesToNumsHash = ();
&DatPedHandler::createMarkersNamesToNumsHash($datFileName, \%markerNamesToNumsHash);

#Create the removed marker numbers array
my @removedMarkerNumbers = ();

# Setting the total cleaning log, with the markers that were removed
my $mask = "$workDir/datafile_*";
my @partDirs = <$mask*>;
open (CLEANING_LOG_FH, ">$workDir\/cleaningReport.fin.total.log") || 
	&General::reportOnErrorReleaseSemaphoresAndDie('PostCleanig.pl',$workDir,sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, "$workDir\/cleaningReport.fin.total.log" ,$!));
print CLEANING_LOG_FH "\nThis file contains error reports that were found during the calculation on the given files\n\n";
for my $i (0..$#partDirs) {
	my $currPath = "$workDir/datafile_".$i."\/cleaningReport.fin.log";
	open CURRLOG_FH, $currPath;
	for my $logLine (<CURRLOG_FH>) {
		if ($logLine =~ / \- /) {   #If this is a marker line
			# dvir 2011/04/13
			$logLine =~ s/\([^\(\)]*\)//g;			
			my @markerLine1 = split / \- /, $logLine;			
			my $markerName = &DAGSplit::trim($markerLine1[0]);
			my $markerNumber = $markerNamesToNumsHash{$markerName};			
			push(@removedMarkerNumbers, $markerNumber);						
			print CLEANING_LOG_FH $markerLine1[0].'('.$markerNumber.') - '.join ' - ', @markerLine1[1..$#markerLine1];
		}
	}
	close CURRLOG_FH;
}
close CLEANING_LOG_FH;

#Remove duplicates from the list by creating a hash and keeping its keys (dirty trick from the perl FAQ...)
my %uniqueHash = map { $_, 1 } @removedMarkerNumbers;
@removedMarkerNumbers = keys %uniqueHash;

#Sort the removed markers array
@removedMarkerNumbers = sort {$a <=> $b} @removedMarkerNumbers;

#Create a list of the markers that will be KEPT in the dat file
my @usedMarkersNumbers = ();
my $numOfMarkersInOriginalDatFile = &DatPedHandler::numOfMarkersInDatFile($datFileName);
my @markersArray = (1 .. $numOfMarkersInOriginalDatFile);
my $nextMarkerNumberToRemove = -1;
my $removedMarkerNumbersIndex = 0;
if (defined($removedMarkerNumbers[$removedMarkerNumbersIndex])) {
	$nextMarkerNumberToRemove = $removedMarkerNumbers[$removedMarkerNumbersIndex];
}
foreach my $i (@markersArray) {
	if ($i ne $nextMarkerNumberToRemove) {push(@usedMarkersNumbers, $i);}
	else {		
		$removedMarkerNumbersIndex++;
		if (defined($removedMarkerNumbers[$removedMarkerNumbersIndex])) {
			$nextMarkerNumberToRemove = $removedMarkerNumbers[$removedMarkerNumbersIndex];
		}
	}
}

#If the used markers array is empty, we will keep a single (arbitrary marker)
if (scalar(@usedMarkersNumbers) == 0) {push(@usedMarkersNumbers, 1);}


#Create new dat and ped files, with only the relevant markers that were not removed
my $datWrapper = new DatWrapper($datFileName);
my $numOfMarkers = $datWrapper->getNumOfMarkers();
my $pedWrapper = new PedWrapper($pedFileName, $numOfMarkers);
my @familiesList = $pedWrapper->getFamiliesNumbers();
#Print the dat file
open (DATFILE_FH,">$workDir\/resultFileTotal.$variables{ Datfile }") || &General::reportOnErrorReleaseSemaphoresAndDie('PostCleanig.pl',$workDir,sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, "$workDir\/resultFileTotal.$variables{ Datfile }" ,$!));
$datWrapper->writeSubDatFileByListOfMarker(\*DATFILE_FH, \@usedMarkersNumbers);
close DATFILE_FH;
#Print the ped file
open (PEDFILE_FH,">$workDir\/resultFileTotal.$variables{ Pedfile }") || &General::reportOnErrorReleaseSemaphoresAndDie('PostCleanig.pl',$workDir,sprintf($Configuration::ERR_MSG_FAILED_OPENING_FILE, "$workDir\/resultFileTotal.$variables{ Pedfile }" ,$!));
$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@usedMarkersNumbers, \@familiesList);
close PEDFILE_FH;

#dvir 2011/12/04 - no more use in semaphores
#&General::releaseSemaphore($semid);
exit(0);

