#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";

use Cwd;
use DAGSplit;
use File::Copy;
use Configuration qw(%configfile);
use General;
use DatPedHandler;
use DatWrapper;
use PedWrapper;

##########################################
##########################################

my $SNPdistiller  			=$configfile{SNPdistiller};
my $cleaningParamToDistiler =$configfile{cleaningParam};
my $binDir 					= $configfile{binDir};
my $configDir = $binDir;

$workDir = $ARGV[0];
$projName = $ARGV[1];
$chromName = $ARGV[2];
#$semid = $ARGV[3];
chdir($workDir);


#dvir 2011/12/04 - no more use in semaphores
#&General::catchSemaphore($semid);

%variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$projName, $chromName) };	

$mode = "EliminatingOnly";
if (exists $variables{ 'modeOfRun' } ) {$mode = $variables{ modeOfRun };}
my ($cwd) = cwd();

$cleaningWorkDir = "$workDir\/cleaning";
$workDir = "$workDir\/clustering";
( -d $workDir ) or &DAGSplit::logInnerSysError("Post isn't running where Pre did!- A BUG!");

#Read the group ranges from the Ranges.txt file
open(RANGES, "$workDir\/Ranges.txt");
@ranges = <RANGES>;
close RANGES;

open DISTILLEROUT, ">$workDir\/SNPdistillerOutput.txt";
$mapIndex = 1;

my $mask = "$workDir/datafile_*";
my @partDirs = <$mask*>;

for $i (0..$#partDirs) {
	my $currPath = "$workDir/datafile_".$i;
	my @rangeLine = split ' ', $ranges[$i];
	my $partPath = "$workDir\/part.".($i+1);
	
	#Create new dat and ped files with the real marker names
	my $datafile = $partPath.".dat";
	my $pedfile = $partPath.".ped";	
	system("cp $currPath\/resultFile.pedfile.dat $pedfile");
    system("cp $currPath\/resultFile.datafile.dat $datafile");
	
	#update (with updated cluster numbers) the file SNPdistillerOutput.txt in the workDir
	open (CURRDISTOUT, "$currPath\/SNPdistillerOutput.txt");	
	for $line (<CURRDISTOUT>) {
		if ($line =~ /Mapping of cluster/) {
			$line = "Mapping of cluster $mapIndex is finished\n";
			$mapIndex++;
		}
		print DISTILLEROUT $line;
	}
	close CURRDISTOUT;
}
close DISTILLEROUT;

my $originalDatFileName;
if ( $mode eq "ClusteringOnly" ) {$originalDatFileName =  "$workDir\/$variables{ Datfile }";}
else {$originalDatFileName = "$cleaningWorkDir\/$variables{ Datfile }"};

#Create unified dat and ped files
my $partsNum = scalar(@partDirs);
my $targetDatfile = "$workDir\/resultFileTotal.datafile.dat";
my $targetPedfile = "$workDir\/resultFileTotal.pedfile.dat";
my $fileNameReisha = "$workDir\/part";
&joinDatFiles($partsNum, $fileNameReisha, $targetDatfile, $originalDatFileName);
&joinPedFiles($partsNum, $fileNameReisha, $targetPedfile);


&createMapfile( "$workDir\/resultFileTotal.datafile.dat", "$workDir\/SNPdistillerOutput.txt" );
if (-d "$ARGV[0]\/multiPedClustering") {
	system("export PERLLIB=$binDir; perl $binDir\/PostMultiPedClustering.pl $ARGV[0] $projName $chromName");
}
#dvir 2011/12/04 - no more use in semaphores
#&General::releaseSemaphore($semid);
exit(0);






#This function receives DatWrapper object and a dat file.
#It concatnates all the markers in the dat file to the DatWrapper object
sub concatDatFileToDatWrapperObject( $ $ $ $ ) {
    my $datWrapper = $_[0];
    my $datFile = $_[1];
    my $markerNamesToNumsHashRef = $_[2];    
    my $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = $_[3];
    
    #Receive an array of marker objects from the dat file
    my $newdatWrapper = new DatWrapper($datFile);
    my @datFileMarkers = @{$$newdatWrapper{MARKERS}};
    shift @datFileMarkers;  #The first entry in the markers array is not used
	
    #Change marker numbers in the datFileMarkers array to match their real number in the original dat file
    my $firstMarkerNumInDatFile = 0;
    foreach my $markerData (@datFileMarkers) {
        my @clusterMarkerNames = split ' ', $markerData->name();
        my @newClusteredMarkersNames = ();
        foreach my $markerNameAndNum (@clusterMarkerNames) {            
            if ($markerNameAndNum !~ /(^.*)\(\d+\)$/) {
                push(@newClusteredMarkersNames, $markerNameAndNum);
                next;
            }
            my $markerName = $1;            
            my $realMarkerNum = $$markerNamesToNumsHashRef{$markerName};            
            my $newMarkerNameAndNum = $markerName."(".$realMarkerNum.")";
            push(@newClusteredMarkersNames, $newMarkerNameAndNum);
			if ($firstMarkerNumInDatFile == 0) {$firstMarkerNumInDatFile = $realMarkerNum;}	
        }
        my $newMarkerName = join ' ', @newClusteredMarkersNames;
        $markerData->name($newMarkerName);
    }
    if ($firstMarkerNumInDatFile == 0) { #error
        &General::reportOnErrorReleaseSemaphoresAndDie("PostClustering.pl", "", "Failed to read marker numbers from dat file");
    }
	
    
    #Compute the distance between the last marker in the datWrapper and the first marker in the dat file
    my @lastClusterInDatWrapperMarkers = split ' ',$$datWrapper{MARKERS}[-1]->name();
	my $FirstMarkerNameAndNumInLastClusterInDatWrapper = $lastClusterInDatWrapperMarkers[0];
    my $lastMarkerNumInDatWrapper;
    if ($FirstMarkerNameAndNumInLastClusterInDatWrapper =~ /^.*\((\d+)\)$/) {$lastMarkerNumInDatWrapper = $1;}
    else {$lastMarkerNumInDatWrapper = $$markerNamesToNumsHashRef{$FirstMarkerNameAndNumInLastClusterInDatWrapper};}	
	my $distanceInCm =
		&DatPedHandler::calcDistInCMBetweenTwoMarkers($lastMarkerNumInDatWrapper-1,
		$firstMarkerNumInDatFile-1, $distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);
	my $distanceInTheta = &DatPedHandler::cmToRec($distanceInCm);
	$datFileMarkers[0]->distIn_Theta_FromPrevMarker($distanceInTheta);
	$datFileMarkers[0]->distIn_cM_FromPrevMarker($distanceInCm);
	
	#Adjust the distance from the first marker field
	my $distInCmFromFirstMarkerDelta = $distanceInCm + $$datWrapper{MARKERS}[-1]->distIn_cM_FromFirstMarker();
	foreach my $markerData (@datFileMarkers) {
		my $distInCmFromFirstMarker = $markerData->distIn_cM_FromFirstMarker();
		$distInCmFromFirstMarker += $distInCmFromFirstMarkerDelta;
		$markerData->distIn_cM_FromFirstMarker($distInCmFromFirstMarker);
	}
    
    #Concatenate the markers to the datWrapper object markers array
    push(@{$$datWrapper{MARKERS}}, @datFileMarkers);
}


#This function creates a joined dat file out of all the dat files created in the clustering
sub joinDatFiles( $ $ $ $ ) {
	my $partsNum = $_[0];    
	my $fileNameReisha = $_[1];
    my $targetDatfile = $_[2];
	my $originalDatFileName = $_[3];
    
    #Get a hash that maps markers names to their numbers in the original dat file
    my %MarkersNamesToNumsHash = ();
    &DatPedHandler::createMarkersNamesToNumsHash($originalDatFileName, \%MarkersNamesToNumsHash);

	#Get an array of markers distances (in cM) from each other
	my @distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = ();
	&DatPedHandler::calcDistInTehtaForEachMarkerFromItsPreviousMarkerByDatFile($originalDatFileName, \@distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);
    
    #Create a dat file wrapper for the first part
    my $datPartFileName = "$fileNameReisha.1.dat";
    unless (-e $datPartFileName) {&DAGSplit::logInnerSysError("couldn't open $datPartFileName for reading");}
    my $datWrapper = new DatWrapper($datPartFileName);
	
    #Change marker numbers in the datWrapper object to match their real number in the original dat file	
    my $firstMarkerNumInDatFile = 0;
    my @datFileMarkers = @{$$datWrapper{MARKERS}};
    shift @datFileMarkers;  #The first entry in the markers array is not used	
    foreach my $markerData (@datFileMarkers) {
        my @clusterMarkerNames = split ' ', $markerData->name();
        my @newClusteredMarkersNames = ();
        foreach my $markerNameAndNum (@clusterMarkerNames) {            
            if ($markerNameAndNum !~ /(^.*)\(\d+\)$/) {
                push(@newClusteredMarkersNames, $markerNameAndNum);
                next;
            }
            my $markerName = $1;            
            my $realMarkerNum = $MarkersNamesToNumsHash{$markerName};            
            my $newMarkerNameAndNum = $markerName."(".$realMarkerNum.")";
            push(@newClusteredMarkersNames, $newMarkerNameAndNum);
			if ($firstMarkerNumInDatFile == 0) {$firstMarkerNumInDatFile = $realMarkerNum;}	
        }
        my $newMarkerName = join ' ', @newClusteredMarkersNames;
        $markerData->name($newMarkerName);
    }
    if ($firstMarkerNumInDatFile == 0) { #error
        &General::reportOnErrorReleaseSemaphoresAndDie("PostClustering.pl", "", "Failed to read marker numbers from dat file");
    }
	
	#Concatenate the other dat files
	for $i (2..$partsNum) {
		my $datPartFileName = "$fileNameReisha.$i.dat";
		unless (-e $datPartFileName) {&DAGSplit::logInnerSysError("couldn't open $datPartFileName for reading");}		
		&concatDatFileToDatWrapperObject($datWrapper, $datPartFileName, \%MarkersNamesToNumsHash, \@distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);
	}	
    
    #Write the datWrapper contents to a file
    my $numOfMarkers = $datWrapper->getNumOfMarkers();
    open TARGET_DAT_FILE_FH, ">$targetDatfile";
    my @markersArray = (1 .. $numOfMarkers);
    $datWrapper->writeSubDatFileByListOfMarker(\*TARGET_DAT_FILE_FH, \@markersArray);
    close TARGET_DAT_FILE_FH;
}



#Create a joined ped file, where each line is a concatenation of all the lines in all the partial ped files
#that were created in the clustering.
sub joinPedFiles( $ $ $ ) {
	my $partsNum = $_[0];
	my $fileNameReisha = $_[1];	
	my $targetPedfile = $_[2];	
	
	#Create 10 initial columns of new ped file
	my $pedPartFileName = "$fileNameReisha.1.ped";
	unless (-e $pedPartFileName) {&DAGSplit::logInnerSysError("couldn't open $pedPartFileName for reading");}
    my $awkCommand = "'\{print \$1, \$2, \$3, \$4, \$5, \$6, \$7, \$8, \$9, \$10\}'";
	system("awk $awkCommand $pedPartFileName > $targetPedfile");
    my $tempFileName = "tempFile.ped";
	
	#Add markers from all partial ped files to the new ped file
	for $i (1..$partsNum) {        
		my $pedPartFileName = "$fileNameReisha.$i.ped";
		unless (-e $pedPartFileName) {&DAGSplit::logInnerSysError("couldn't open $pedPartFileName for reading");}
        my $cutCommand = "cut -d\" \" -f11\- $pedPartFileName | paste -d\" \" $targetPedfile \-";
		system("$cutCommand > $tempFileName");
        system("cp -f $tempFileName $targetPedfile"); 
	}    
    system("rm -f $tempFileName");
}





################################################################
#		createMapfile
#
# Parameters:
#			$_[0] - datafile
#			$_[1] - distiller output
# Description:
#	
################################################################
sub createMapfile( $ ) {
	my $file = $_[0];
	my $distFile = $_[1];
	
	#Get an array of cluster-markers distances (in cM) from each other
	my @distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef = ();
	&DatPedHandler::calcDistInTehtaForEachMarkerFromItsPreviousMarkerByDatFile($file, \@distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef);
	
	#Get the distance of each marker from the first marker
	my @distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef = ();
	&DatPedHandler::calcDistInCM_ForEachMarkersFromTheFirstMarker(\@distancesInThetaForEachMarkerFromItsPreviousMarkerArrRef,
																  \@distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef);
	
	open DATFILE, $file;
	open MAPFILE, ">$workDir\/resultFileTotal.mapfile.dat";
	open MAPREPORT, ">$workDir\/resultFileTotal.mapReport.dat";
	print MAPREPORT "<table border=\"1\" cellpadding=\"0\" cellspacing=\"0\" width=\"100%\" id=\"AutoNumber1\">
  <tr>
    <td width=\"15%\" align=\"center\"><b>Cluster id</b></td>
    <td width=\"70%\" align=\"center\"><b>Markers names</b></td>
	<td width=\"15%\" align=\"center\"><b>Distance (cM)</b></td>
  </tr>";
  
	@clusters = ();	
	
	my $clusterCounter = 0;
	for $line (<DATFILE>) {
		$trimLine=&DAGSplit::trim($line);
		unless ( defined($trimLine) && ($trimLine ne "")) {next;}
		unless ($trimLine =~ /#.+#/ ) {next;}
		my @markerLine = split /#/ , $trimLine;
		print MAPFILE "$markerLine[1]\n";
		push(@clusters,"$markerLine[1]");		
		my $ClusterdistanceFromStart = $distancesInCM_ForEachMarkerFromTheFirstMarkerArrRef[$clusterCounter];
		push(@clDist, $ClusterdistanceFromStart);
		$clusterCounter++;
	}
	
	for $c (0..$#clusters) {
		print MAPREPORT "<tr>
			<td width=\"15%\" align=\"center\"><b>c".($c+1)."</b></td>
			<td width=\"70%\" align=\"center\"><b>".$clusters[$c]."</b></td>
			<td width=\"15%\" align=\"center\"><b>".sprintf ("%.4f",$clDist[$c])."</b></td>
		</tr>";
	}
	
	print MAPREPORT "</table>";
	
	close DATFILE;
	close MAPREPORT;
	
	print MAPFILE "[VALUE MAPPING]\n";
	
	open DISTFILE, $distFile;
	
	for $line (<DISTFILE>) {print MAPFILE "$line";}
	
	close DISTFILE;	
	close MAPFILE;
}
