#!/usr/bin/perl -w
# dvir 2010-12-19

use strict;

use FindBin ();
use lib "$FindBin::Bin";

use DatPedHandler;
use Configuration;
use General;
use DrawingGeneral;
use Analysis;
use JobStatus;

my $executableExitCode = $ARGV[0];
my $painterProgram = $ARGV[1];
my $destDir = $ARGV[2];
my $workDir = $ARGV[3];

my @familiesNumbers;
&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFileAtFolder(\@familiesNumbers,'..');

# checking that all the families has a pdf&png files of the family pedigree
my @shouldBeOutputFiles = ();
# passing over all the families, and painting acording to the svg file
foreach my $currFamilyNum (@familiesNumbers) {
	my $pdfOutputFileName = &DrawingGeneral::getPainterProgramOutputFileName($Configuration::OUTPUT_FORMAT_PDF, $painterProgram, $currFamilyNum);
	my $pngOutputFileName = &DrawingGeneral::getPainterProgramOutputFileName($Configuration::OUTPUT_FORMAT_PNG, $painterProgram, $currFamilyNum);
	
	push(@shouldBeOutputFiles, $pdfOutputFileName);
	push(@shouldBeOutputFiles, $pngOutputFileName);
}
my $areSomeFilesMissing = &General::checkExecutionExitCodeAndExistenceOfOutputFiles("Drawing - $painterProgram",'',$executableExitCode, \@shouldBeOutputFiles, '');

# we can reach here even if the check has found wrong stuff such as missing files,
# the programmer will recive a mail. and the files that were created will be sent to the web server

# sending the resuls to the web server

# the echo -n is important since without it, each element in the array has \n char at the end
# of it, but there isn't a file with a \n char actually,
# so the try to send a file with a \n at the end of it will fail since the file doesn't exists
my @drawingsOutputFiles = `echo -n \`ls *.pdf *.png\``;

if (@drawingsOutputFiles > 0) {
	
	# dvir, 2011/05/15, I'm deleting the old pedigrees,
	# so that copying the new pedigree, won't failed
	my $outputFiles = join ' ',@shouldBeOutputFiles;
	my @commands = ("cd $destDir", "rm -rf $outputFiles");	
	&General::executeSSHCommandsOnWebServer(\@commands);
	
	&General::sendFilesToWebServer(\@drawingsOutputFiles, $destDir);
}

# if the check hasn't fond missing files, than we will mark that the job succeeded.
if (! $areSomeFilesMissing) {
	&JobStatus::markThatJobSucceeded('.');
}
else {
	# the workDir is the contating folder
	my $analysis = new Analysis($workDir);
	$analysis->reportOnFailureToWebServer('');
	
	&JobStatus::markThatJobFailed('.');
}