#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";

use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use General;
use POSIX qw(strftime); # in order to track the locks time

##########################################

##########################################

$ssh_options = $configfile{ssh_options};               
$ssh_remote = $configfile{ssh_remote};              
$ssh_user = $configfile{ssh_user};              
$ssh_cmd = "ssh $ssh_options $ssh_user" . "@" . "$ssh_remote";
$remote_script = $configfile{remote_script}; 

#remote info
$remote_user_full = "$ssh_user" . "@" . "$ssh_remote";
$remote_submission_dir = $configfile{remote_dir};

#######
#
#start
########
my $binDir 	  = $configfile{binDir};

my $configDir = $binDir;

$workDir = $ARGV[0];
chdir($workDir);

$project = $ARGV[1];

my $projectTaskDir		= $ARGV[2];
my $projectRunlog 		= $ARGV[3];
my $projectMailDir 		= $ARGV[4];
my $currentlyRunningDir = $ARGV[5];
my $projectOutputDir 	= $ARGV[6];

%variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$project) };

$mode = "EliminatingOnly";
if (exists $variables{ 'modeOfRun' } )
{
	$mode = $variables{ modeOfRun };
}	

my ($cwd) = cwd();
my $datfileName = $variables{ Datfile };
$datfileName =~ s/\.dat//;	

if ( $mode ne "ClusteringOnly" ) {		#Cleaning was done
	@cleaningFinish = `grep "All jobs Completed!" CleaningSubJobs_$datfileName.dag.dagman.out`;
	if ($#cleaningFinish == -1) {
		`touch TASK_FAILED.txt`;
		$errData = "";
		my $currErrPath = "$cwd\/cleaning\/datafile_0\/err.txt";
		if (-e $currErrPath) {
			open CURRERR, $currErrPath;
			for $errLine (<CURRERR>) {
				if ($errLine =~ /Error in/) {
					$errData = $errData.$errLine;
				}
			}
		}
		&DAGSplit::logAndReportError("Cleaning failed.\n $errData", "cleaning", $project, $datfileName);
		# DVIR, 07-11-2010, in order that other cleanings of other dat files will continue to execute
		# even if one cleaning had failed
		#system("perl $binDir\/killProjByName.pl $project");
		exit(0);
	}
	
	if ( $mode eq "EliminatingOnly" ) {  #cleaning was the last action
		system("cp $cwd\/cleaning\/resultFileTotal.$datfileName.dat $cwd\/resultFile.$datfileName.dat ");
		system("cp $cwd\/cleaning\/resultFileTotal.$datfileName.ped $cwd\/resultFile.$datfileName.ped");
	}
	
	system("cp $cwd\/cleaning\/cleaningReport.fin.total.log $cwd\/cleaningReport.fin.log");
}
if ( $mode ne "EliminatingOnly" )  	#Clustering was done
{
	@clusteringFinish = `grep "All jobs Completed!" ClusteringSubJobs_$datfileName.dag.dagman.out`;
	if ($#clusteringFinish == -1) {
		`touch TASK_FAILED.txt`;
		&DAGSplit::logAndReportError("Clustering failed.", "clustering", $project, $datfileName);
		# DVIR, 07-11-2010, in order that other cleanings \clustering of other dat files will continue to execute
		# even if one clustering had failed
		# system("perl $binDir\/killProjByName.pl $project");
		exit(0);
	}

	system("cp $cwd\/clustering\/resultFileTotal.datafile.dat $cwd\/resultFile.$datfileName.dat ");
	system("cp $cwd\/clustering\/resultFileTotal.pedfile.dat $cwd\/resultFile.$datfileName.ped");
	system("cp $cwd\/clustering\/resultFileTotal.mapfile.dat $cwd\/resultFile.$datfileName.map ");
}

@globalError = `grep "ERROR: the following job(s) failed" Global_$datfileName.dag.dagman.out`;
if ($#globalError > -1) {
	`touch TASK_FAILED.txt`;
	&DAGSplit::logAndReportError("Job execution failed.", "cleaning", $project, $datfileName);
	# DVIR, 07-11-2010, in order that other cleanings \clustering of other dat files will continue to execute
	# even if one clustering had failed
	# system("perl $binDir\/killProjByName.pl $project");
	exit(0);
}

`touch TASK_COMPLETE.txt`;

for (0..9) { 	# We give the monitor a chance to show completion before we send the results 
				# but if it doesn't finish in the next 5 min we assume it failed and send the results without further waiting
	if (! -e "MONITOR_DONE.txt") {
		sleep(30);
	}
	else {
		last;
	}
}

# Copy the output of the splitscript run
system("cp $workDir\/resultFile.* $projectOutputDir");

open (FILELIST , ">$projectOutputDir\/FilesList_$datfileName.txt");

##############################################################
# Sending the output of the current chromosom to the website #
##############################################################

	# if map file is missing and we perfomed clustering we need to create dummy one for the site
	############################################################################################# 
	my %variables = (); 
	my $filename = "variables.txt";
	open( VARIABLES_FILE , $filename ) or logInnerSysError("couldn't open $filename for reading of variables");
	my $lineNum = 0;
	for $line ( <VARIABLES_FILE> )
	{
		$lineNum++;
		my @tmp = split(/[ \t\n\r\s\f]+/,$line); 
		if( $#tmp != 1 ) {next;}
		$variables { $tmp[0] } = $tmp[1];
	}	
	close( VARIABLES_FILE );
	
	if ( !( $mode eq "EliminatingOnly") && !( -e "$projectOutputDir\/resultFile.$datfileName.map" ) )
	{
	  system ("echo \"\" >$projectOutputDir\/resultFile.$datfileName.map"); 
	}
	################################################################################################

if ( $mode eq "EliminatingOnly")
{
	print FILELIST "resultFile.$datfileName.dat\nresultFile.$datfileName.ped";
}
else
{
	print FILELIST "resultFile.$datfileName.dat\nresultFile.$datfileName.ped\nresultFile.$datfileName.map";
}
close FILELIST;

system("tar -C $projectOutputDir -T $projectOutputDir\/FilesList_$datfileName.txt -czf $projectMailDir\/report_$datfileName.tar.gz");
my $scpCommand = "scp $projectMailDir\/report_$datfileName.tar.gz $remote_user_full:\/$remote_submission_dir\/$project\/report_$datfileName.tar.gz"; 
my $result = &General::perfomrWithExponentialBackoff($scpCommand);

if ($result==-1)
{
	open( RUNLOG , ">>$projectRunlog" );
	flock(RUNLOG,2) or die "Can't lock $projectRunlog file\n";
	$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	print RUNLOG "fail to copy result file of $datfileName to site! at $now_string\n";	
	close RUNLOG;
}
else
{
	my $sshCommand = "$ssh_cmd -vvv $remote_script finish $project report_$datfileName.tar.gz >> /tmp/sshLogFile.log 2>&1"; # Tmp -vvv , log
	$result = &General::perfomrWithExponentialBackoff($sshCommand);
	
	if ($result==-1)
	{
		open( RUNLOG , ">>$projectRunlog" );
		flock(RUNLOG,2) or die "Can't lock $projectRunlog file\n";
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print RUNLOG "fail to send 'finish' of $datfileName to site! at $now_string\n";	
		close RUNLOG;
		
		open( STATUSUPDATE , ">>$projectTaskDir\/status.log" );
		flock(STATUSUPDATE,2) or die "Can't lock $projectTaskDir\/status.log file\n";
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print STATUSUPDATE "$datfileName task is done but not reported to site. at $now_string\n";
		close STATUSUPDATE;
	}
	else
	{
		open( RUNLOG , ">>$projectRunlog" );
		flock(RUNLOG,2) or die "Can't lock $projectRunlog file\n";				
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print RUNLOG "sent output report of $datfileName at $now_string\n";	
		print RUNLOG "finished $datfileName\n";
		close RUNLOG;

		open( STATUSUPDATE , ">>$projectTaskDir\/status.log" );
		flock(STATUSUPDATE,2) or die "Can't lock $projectTaskDir\/status.log file\n";
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print STATUSUPDATE "$datfileName task is done at $now_string\n";
		close STATUSUPDATE;
	}
}

exit(0);
