#!/usr/bin/perl -w
use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use POSIX qw(strftime); # in order to track the locks time

##########################################

##########################################

#######
#
#start
########
my $binDir 	  = $configfile{binDir};

my $configDir = $binDir;

$workDir = $ARGV[0];
chdir($workDir);

$project = $ARGV[1];

%variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$project) };

my $datfileName = $variables{ Datfile };
$datfileName =~ s/\.dat//;

my ($cwd) = cwd();

$inPedfile = "$workDir\/..\/..\/$variables{ Pedfile }";

#DVIR, 2010-10-28 this var is unused: $maxLen = 0;
#DVIR, 2010-10-28 this var is unused: $maxPed = 0;
@families = `awk ' { print \$1 } ' $inPedfile |sort | uniq`;
foreach $fam (@families) {
	$fam =~ s/\s+//sig;
	system("grep ^$fam $inPedfile > $cwd\/multiPedClustering\/input_$fam.ped");
}

system("cp $cwd\/clustering\/resultFileTotal.pedfile.dat $cwd\/multiPedClustering\/");
system("cp $cwd\/clustering\/resultFileTotal.mapfile.dat $cwd\/multiPedClustering\/");

chdir("$cwd\/multiPedClustering\/");

open MAP, "resultFileTotal.mapfile.dat";
@linesMap = <MAP>;
close MAP;

@clusters = ();
my $line = shift @linesMap;
while (($#linesMap > -1) && ($line !~ /\[VALUE MAPPING\]/)) {
	@lineMarkers = split /\(/ , $line;
	shift @lineMarkers;
	for $i (0..$#lineMarkers) {
		@tmp = split /\)/ , $lineMarkers[$i];
		$lineMarkers[$i] = $tmp[0];
	}
	$lineMarkersStr = join ",", @lineMarkers;
	push(@clusters, $lineMarkersStr);
	$line = shift @linesMap;
}

$pedNum = 1;
$usedPed = `cat usedPed.txt`;
$pedAddition = "";

$nextPedNum = 2;

while (-e "input_$pedNum.ped") {
	if ($pedNum != $usedPed) {
		open CURRPED, "input_$pedNum.ped";
		@pedData = <CURRPED>;
		close CURRPED;
		
		for $k (0..$#pedData) {
			@words = split /\s+/, $pedData[$k];
			$pedAddition = $pedAddition.$nextPedNum;
		
			for (my $i = 1; $i < 10; $i++) {
				$pedAddition = $pedAddition." ".$words[ $i ];
			}
			
			for $j (0..$#clusters) {
				@currMarkers = split /\,/, $clusters[$j];
				@val =();
				
				$val1 = "";
				$val2 = "";
				for $i (0..$#currMarkers) {
					my $markerIndex = 10 + 2*($currMarkers[$i] - 1);
					$val1 = $val1." ".$words[ $markerIndex ]." ";
					$val2 = $val2." ".$words[ $markerIndex + 1 ]." ";
				}
				
				@val1vars = split /\s+/ , $val1;
				@val2vars = split /\s+/ , $val2;
				
				if ($#val1vars <= 1) {	#the cluster has only one value
					$pedAddition = $pedAddition." ".&DAGSplit::trim($val1)." ".&DAGSplit::trim($val2);
					next;
				}
				
				$prevCluster = $j;
				$currCluster = $j + 1;
				$mapIndex = 0;
				if ($prevCluster > 0) {
					while ($linesMap[ $mapIndex ] !~ /Mapping of cluster $prevCluster is finished/ ) {
						$mapIndex++;
					}
				}
				
				$changeInd = 0;
				while ($changeInd < $#val1vars) {
					$currVal = $val1;
					$valSearch = $mapIndex;
					for (1..2) {
						while (($linesMap[ $valSearch ] !~ /Mapping of cluster $currCluster is finished/ ) && ($linesMap[ $valSearch ] !~ /original.*$currVal/)) { 
							$valSearch++;
						}
						
						if ($linesMap[ $valSearch ] =~ /original.*$currVal/) {
							@lineData = split /result allele:/ , $linesMap[ $valSearch ];
							$code = &DAGSplit::trim($lineData[1]);
							
							while (($linesMap[ $valSearch ] !~ /Mapping of cluster $currCluster is finished/ ) && ($linesMap[ $valSearch ] !~ /mapCode $code is/ )) { 
								$valSearch++;
							}
							
							if ($linesMap[ $valSearch ] =~ /mapCode $code is/ ) {
								@codeLineData = split /is/, $linesMap[ $valSearch ];
								push(@val, (&DAGSplit::trim($codeLineData[1]) + 1) );	
							}
							else {
								last; #We haven't found the first value - no need to look for the second
							}
						}
						$currVal = $val2;
						$valSearch = $mapIndex;
					}
					if ($#val == 1) { #found a match
						last;
					}
					else {
						$changeInd++;
						$tmp = $val1vars[$changeInd];
						$val1vars[$changeInd] = $val2vars[$changeInd];
						$val2vars[$changeInd] = $tmp;
						$val1 = join "  ", @val1vars;
						$val2 = join "  ", @val2vars;
						$val1 = $val1." ";
						$val2 = $val2." ";
					}
				}
				
				if ($#val == 1) {$pedAddition = $pedAddition." ".$val[0]." ".$val[1];}				
				else {$pedAddition = $pedAddition." 0 0";}
			}
			$pedAddition = $pedAddition."\n";
		}
		$nextPedNum++;
	}	
	$pedNum++;
}

open PED, ">>resultFileTotal.pedfile.dat";
print PED "$pedAddition";
close PED;

system("export PERLLIB=$binDir; perl $binDir\/pedcheck.pl resultFileTotal.pedfile.dat $cwd\/multiPedClustering");
$returnVal = $? >> 8;
if ($returnVal != 0) {
	if ($returnVal == $configfile{HighRemoveRate}) {
		&DAGSplit::logAndReportError("Multiple pedigrees clustering produced a very high information removal rate.", "clustering", $project, $datfileName);
		exit($returnVal);
	}
	if ($returnVal == $configfile{RecheckNotEmpty}) {
		system("export PERLLIB=$binDir; perl $binDir\/pedcheck.pl resultFileTotal.pedfile.dat $cwd\/multiPedClustering");
		$secondReturnVal = $? >> 8;
		if ($secondReturnVal != 0) {
			if ($secondReturnVal == $configfile{HighRemoveRate}) {
				&DAGSplit::logAndReportError("Multiple pedigrees clustering produced a very high information removal rate.", "clustering", $project, $datfileName);
				exit($secondReturnVal);
			}
			&DAGSplit::logAndReportError("Multiple pedigrees clustering failed.", "clustering", $project, $datfileName);
			exit($secondReturnVal);
		}
	}
	else {
		&DAGSplit::logAndReportError("Multiple pedigrees clustering failed.", "clustering", $project, $datfileName);
		exit($returnVal);
	}
}

system("cp $cwd\/multiPedClustering\/resultFileTotal.pedfile.dat $cwd\/clustering\/");

exit(0);
