#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";

use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use General;
use DatPedHandler;
use DatWrapper;
use PedWrapper;
use List::Util;

#######
#
#start
########

my $SNPdistiller  			=$configfile{SNPdistiller};
my $binDir 					= $configfile{binDir};

$workDir = $ARGV[0];
chdir($workDir);

$projName = $ARGV[1];
$chromName = $ARGV[2];
#$semid = $ARGV[3];

#dvir 2011/12/04 - no more use in semaphores
#&General::catchSemaphore($semid);

%variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$projName, $chromName) };	


&DatPedHandler::checkDatFile("$variables{ DatfileDir }\/$variables{ Datfile }");

my $typeOfCheck = $variables{ errorsPerMarkerThreshold };

# Building the distiler command
# "  ./snpDistiller 2 "
#                   ^
# this is the format of the dat&ped it can be : "1" GH (gene hunter) or "2" SL (superlink)
my $distilerCleaningCommand = './snpDistiller 2 datafile.dat pedfile.dat ';

# the parameters are : "typeOfGenoCheck firstMarkerIndexShift [optional exit_point] [optional start_point]"
# if typeOfGenoCheck == -1, than we won't preform unlikeliy Genotypes.
if ( $typeOfCheck eq "Strong" ) {$distilerCleaningCommand.= '0 0 4  > SNPdistillerOutput.txt';}
# notice the diffrence:
# REMOVE_mendelian_infinity_unLikelyGenotypes___DO_NOT_REMOVE_uninformative
# vs
# REMOVE_mendelian_infinity___DO_NOT_REMOVE_uninformative
elsif ( $typeOfCheck eq "REMOVE_mendelian_infinity_unLikelyGenotypes___DO_NOT_REMOVE_uninformative" )
{
	# dvir 2010/04/13,
	# in order just to make all cleaning except uninformative markers,
	# than we need to call the executeable twice, since of its command options structure.
	$distilerCleaningCommand.= "-1 0 1 > SNPdistillerOutput2\.txt\n".
							   "mv ok_snpAll\.dat ok_snpAll\.no_mendelian_errors\.dat\n".
							   "mv ok_snpAll\.ped ok_snpAll\.no_mendelian_errors\.ped\n".
	# here err.txt will contain markers that were removed from first executeable run
								"mv err\.txt err\.mendelian_errors\.txt\n".
								"./snpDistiller 2 ok_snpAll\.no_mendelian_errors\.dat ok_snpAll\.no_mendelian_errors\.ped 0 0 4 3  > SNPdistillerOutput\.txt\n".
	# THE DIFF :                                                                                                            ^								
	# here err.txt will contain markers that were removed from second executeable run
							   "cat err\.mendelian_errors\.txt >> err\.txt";
	# here err.txt will contain markers that were removed from both executeable runs
}

elsif ( $typeOfCheck eq "Medium" ) {$distilerCleaningCommand.= '4 0 4 > SNPdistillerOutput.txt';}
elsif ( $typeOfCheck eq "Weak" ) {$distilerCleaningCommand.= '-1 0 4 > SNPdistillerOutput.txt';} # Pass the unlikely check
elsif ( $typeOfCheck eq "MendelianOnly" ) {$distilerCleaningCommand.= '-1 0 1 > SNPdistillerOutput.txt';} # only mendelian errors
elsif ( $typeOfCheck eq "REMOVE_mendelian_infinity___DO_NOT_REMOVE_uninformative" ) {
	# dvir 2010/04/13,
	# in order just to make all cleaning except uninformative markers,
	# than we need to call the executeable twice, since of its command options structure.
	$distilerCleaningCommand.= "-1 0 1 > SNPdistillerOutput2\.txt\n".
							   "mv ok_snpAll\.dat ok_snpAll\.no_mendelian_errors\.dat\n".
							   "mv ok_snpAll\.ped ok_snpAll\.no_mendelian_errors\.ped\n".
	# here err.txt will contain markers that were removed from first executeable run
								"mv err\.txt err\.mendelian_errors\.txt\n".
								"./snpDistiller 2 ok_snpAll\.no_mendelian_errors\.dat ok_snpAll\.no_mendelian_errors\.ped -1 0 4 3  > SNPdistillerOutput\.txt\n".
	# THE DIFF :                                                                                                           ^
	# here err.txt will contain markers that were removed from second executeable run
							   "cat err\.mendelian_errors\.txt >> err\.txt";
	# here err.txt will contain markers that were removed from both executeable runs
}
else {&General::reportOnErrorReleaseSemaphoresAndDie('PreCleanig.pl',$workDir,"unknown errorsPerMarkerThreshold: $typeOfCheck");}

my $datfileName = $variables{ Datfile };
$datfileName =~ s/\.dat//;
 # First creating / checking the workDir of the current task 
$workDir = "$workDir\/cleaning";
&DAGSplit::splitLog("try to create $workDir");
if( -d $workDir ){&DAGSplit::splitLog("$workDir already exist, content will be overriden.");}
else {
	system("mkdir $workDir");
	( -d $workDir ) or &DAGSplit::logInnerSysError("mkdir $workDir failed");
}

$snpSource = 0;
if ((defined $variables{ snpSource }) && ($variables{ snpSource } eq "true")) {$snpSource = 1;}

$res = &DAGSplit::getFiles( "badFile.log","$workDir\/$variables{ Datfile }","$workDir\/$variables{ Pedfile }", $snpSource);
if ($res != 0) {
	#dvir 2011/12/04 - no more use in semaphores
	#&General::releaseSemaphore($semid);
	exit(-1);
}

#*****************************
$mainDatafile = "$workDir\/$variables{ Datfile }";
$mainPedfile = "$workDir\/$variables{ Pedfile }";
if (defined($variables{ReductionMarkersList}) or defined($variables{FromMarker} or defined($variables{ToMarker}))) {
	my @markersArray = ();
	if (defined($variables{ReductionMarkersList})) {@markersArray = split ',', $variables { ReductionMarkersList };}
	else {
		my $from = 1;
		my $to = `head -1 $mainDatafile | cut -d\" \" -f1` - 1;
		if (defined($variables{FromMarker})) {$from = List::Util::max($from, $variables{FromMarker});}
		if (defined($variables{ToMarker}))   {$to = List::Util::min($to, $variables{ToMarker});}
		@markersArray = ($from .. $to);
	}
	@markersArray  = sort {$a <=> $b} @markersArray;
	my $validList = &DAGSplit::isValidMarkersList(@markersArray, $mainDatafile);
	if ($validList =~ /size/ ) {%variables = %{&DAGSplit::logInnerSysError("Invalid parameters! ".$validList)};}
	$mainDatafileText = &DAGSplit::getMarkersListDataFile( @markersArray, $mainDatafile); 
	$mainPedfileText = &DAGSplit::getMarkersListPedFile( @markersArray, $mainPedfile); 
	
	open DAT ,">$workDir\/datafileTmp.dat";
	print DAT "$mainDatafileText";
	close DAT;
	
	open PED ,">$workDir\/pedfileTmp.dat";
	print PED "$mainPedfileText";
	close PED;
	
	$mainDatafile = "$workDir\/datafileTmp.dat";
	$mainPedfile = "$workDir\/pedfileTmp.dat";
}
open IN, "$mainDatafile";

my @data = <IN>;
close IN;

my @words_first_line = split ' ', $data[0];
my $size = $words_first_line[0] - 1;

#******************************************

# dvir 17/11/2011 - coping all pm files
my @pmFilesInBinDirArr = &General::getArrayOf_pm_filesNamesInBinDir();
my $pmFilesInBinDirStr = join ',', @pmFilesInBinDirArr;
#Omer 4/9/2011 - moved these lines out of the loop (don't think they need to be inside)
my $CleaningTmpl = `cat $binDir\/Tmpl\/Cleaning.sub.tmpl`;
$CleaningTmpl =~ s/_CONFIGDIR_/$binDir/g;
$CleaningTmpl =~ s/_DISTILLER_/$SNPdistiller/g;
$CleaningTmpl =~ s/_WORKDIR_/$workDir/g;
$CleaningTmpl =~ s/_PM_FILES_LIST_/$pmFilesInBinDirStr/g;

#Omer 4/9/2011 - read contents of the dat and ped files using the DatWrapper and PedWrapper classes
my $datWrapper = new DatWrapper($mainDatafile);
my $numOfMarkers = $datWrapper->getNumOfMarkers();
my $pedWrapper = new PedWrapper($mainPedfile, $numOfMarkers);
my @familiesList = $pedWrapper->getFamiliesNumbers();

my $count = 500; 				#Decide on size
my $currIndex = 0;
my $DAGContent = "";
my $i = 1;

while ($i < $size)
{
	my $currDir = "$workDir\/datafile_".($currIndex);
	if( -d $currDir ) {&DAGSplit::splitLog("$currDir already exist, content will be overriden.");}
	else {mkdir "$currDir" or &DAGSplit::logInnerSysError("mkdir $currDir failed");}
	&DAGSplit::saveToFile( "$currDir\/Cleaning.sub",$CleaningTmpl);
	$DAGContent = $DAGContent."JOB ".$datfileName."A".$currIndex." ".$currDir."\/Cleaning.sub\n"."VARS ".$datfileName."A".$currIndex." TASKNUM=\"".$currIndex."\" JOBWORKDIR=\"".$currDir."\"\n";
	my @currentMarkersList = ($i .. List::Util::min($i + $count-1, $numOfMarkers));
	
	#Print the dat file
	open DATFILE_FH, ">$currDir\/datafile.dat";	
	$datWrapper->writeSubDatFileByListOfMarker(\*DATFILE_FH, \@currentMarkersList);
	close DATFILE_FH;
	
	#Print the ped file
	open PEDFILE_FH, ">$currDir\/pedfile.dat";
	$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@currentMarkersList, \@familiesList);
	close PEDFILE_FH;
	
	#Print the distiller command file
	&DAGSplit::saveToFile( "$currDir\/DistillerCommand.txt", $distilerCleaningCommand);
	
	#Omer - this is the old code for creating the dat and ped files...
	#$datafile = &DAGSplit::getMarkerWindowDataFile( $i, $count, $mainDatafile); 
	#$pedfile = &DAGSplit::getMarkerWindowPedFile( $i, $count, $size, $mainPedfile);	
	#&DAGSplit::saveToFile( "$currDir\/datafile.dat", $datafile );
	#&DAGSplit::saveToFile( "$currDir\/pedfile.dat", $pedfile );	
	
	$currIndex++;
	$i += $count;
}
&DAGSplit::saveToFile( "CleaningSubJobs_$datfileName.dag", $DAGContent );

#dvir 2011/12/04 - no more use in semaphores
#&General::releaseSemaphore($semid);

exit(0);

