#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";

use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use List::Util;
use General;
use DatWrapper;
use PedWrapper;
use DatPedHandler;
#######
#
#start
########
my $SNPdistiller  				=$configfile{SNPdistiller};
my $clusteringParamToDistiler 	=$configfile{clusteringParam};
my $binDir 						= $configfile{binDir};

my $workDir = $ARGV[0];
my $projName = $ARGV[1];
my $chromName = $ARGV[2];
my $semid = $ARGV[3];

chdir($workDir);

#dvir 2011/12/04 - no more use in semaphores
#&General::catchSemaphore($semid);

%variables = %{ &DAGSplit::initVariablesFromFile("variables.txt","currentChromosom.txt",$projName, $chromName) };

my $checkGenoParam = 0;
my $typeOfCheck = $variables{ errorsPerMarkerThreshold };
if ( $typeOfCheck eq "Strong" ) {$checkGenoParam = 0;}
elsif ( $typeOfCheck eq "Medium" ) {$checkGenoParam = 4;}
elsif ( $typeOfCheck eq "Weak" ) {$checkGenoParam = -1;} # Pass the unlikely check

my $mode = "EliminatingOnly";
if (exists $variables{ 'modeOfRun' } ) {$mode = $variables{ modeOfRun };}

my $datfileName = $variables{ Datfile };
$datfileName =~ s/\.dat//;	
 # First creating the workDir of the current task 
my $cleaningWorkDir = "$workDir\/cleaning";
$workDir = "$workDir\/clustering";

&DAGSplit::splitLog("try to create $workDir");
if( -d $workDir ){&DAGSplit::splitLog("$workDir already exist, content will be overriden.");}
else{
	system("mkdir $workDir");
	( -d $workDir ) or &DAGSplit::logInnerSysError("mkdir $workDir failed");
}

my $inDatafile;
my $inPedfile;
if ( $mode eq "ClusteringOnly" ) {
	#If there was no cleaning we are working on input files and should check them
	my $snpSource = 0;
	if ((defined $variables{ snpSource }) && ($variables{ snpSource } eq "true")) {$snpSource = 1;}	
	my $res = &DAGSplit::getFiles( "badFile.log","$workDir\/$variables{ Datfile }","$workDir\/$variables{ Pedfile }" , $snpSource);
	if ($res != 0) {
		#dvir 2011/12/04 - no more use in semaphores
		#&General::releaseSemaphore($semid);
		exit(-1);
	}
	$inDatafile = "$workDir\/$variables{ Datfile }";
	$inPedfile = "$workDir\/$variables{ Pedfile }";
}
else {
	$inDatafile = "$cleaningWorkDir\/resultFileTotal.".$datfileName.".dat";
	$inPedfile = "$cleaningWorkDir\/resultFileTotal.".$datfileName.".ped";
}

#Create a list of family numbers in the ped file
my @families = ();
&DatPedHandler::getFamiliesNumbersInPedFile($inPedfile, \@families);

#If there is more than one family, create a dat/ped pair for each family
if ($#families > 0) {
	system("export PERLLIB=$binDir; perl $binDir\/PreMultiPedClustering.pl $ARGV[0] $projName $chromName");
	$inDatafile = "$ARGV[0]\/multiPedClustering\/datafile.dat";
	$inPedfile = "$ARGV[0]\/multiPedClustering\/pedfile.dat";
}
#*****************************
if ($mode eq "ClusteringOnly") {
	if (defined($variables{ ReductionMarkersList}) or defined($variables{FromMarker} or defined($variables{ToMarker}))) {
		my @markersArray = ();
		if (defined($variables{ ReductionMarkersList})) {@markersArray = split ',', $variables { ReductionMarkersList };}
		else {
			my $from = 1;
			my $to = `head -1 $inDatafile | cut -d\" \" -f1` - 1;
			if (defined($variables{FromMarker})) {$from = List::Util::max($from, $variables{FromMarker});}
			if (defined($variables{ToMarker}))   {$to = List::Util::min($to, $variables{ToMarker});}
			@markersArray = ($from .. $to);
		}
			
		@markersArray  = sort {$a <=> $b} @markersArray;
		my $validList = &DAGSplit::isValidMarkersList(@markersArray, $inDatafile);
		if ($validList =~ /size/ ) {%variables = %{&DAGSplit::logInnerSysError("Invalid parameters! ".$validList)};}
		my $mainDatafileText = &DAGSplit::getMarkersListDataFile( @markersArray, $inDatafile); 
		my $mainPedfileText = &DAGSplit::getMarkersListPedFile( @markersArray, $inPedfile); 
		
		open DAT ,">$workDir\/datafileTmp.dat";
		print DAT "$mainDatafileText";
		close DAT;
		
		open PED ,">$workDir\/pedfileTmp.dat";
		print PED "$mainPedfileText";
		close PED;
		
		$inDatafile = "$workDir\/datafileTmp.dat";
		$inPedfile = "$workDir\/pedfileTmp.dat";
	}
}
#******************************************
#Determine the size of a "Big" window
my $cMorganWindowSize=10; 		# "Big" Window size in centimorgan
if( exists $variables{ 'CMWindowSize' } ) {$cMorganWindowSize = $variables{ CMWindowSize };}
my $bigWindowSize = 2 * $cMorganWindowSize * 10;

# dvir 17/11/2011 - coping all pm files
my @pmFilesInBinDirArr = &General::getArrayOf_pm_filesNamesInBinDir();
my $pmFilesInBinDirStr = join ',', @pmFilesInBinDirArr;
#Omer 6/9/2011 - read clustering sub file from condor (moved this out of the loop)
my $ClusteringTmpl = `cat $binDir\/Tmpl\/Clustering.sub.tmpl`;
$ClusteringTmpl =~ s/_CONFIGDIR_/$binDir/g;
$ClusteringTmpl =~ s/_DISTILLER_/$SNPdistiller/g;
$ClusteringTmpl =~ s/_PM_FILES_LIST_/$pmFilesInBinDirStr/g;
$ClusteringTmpl =~ s/_WORKDIR_/$workDir/g;

#Create datWrapper and pedWrapper objects for the dat and ped file
my $datWrapper = new DatWrapper($inDatafile);
my $numOfMarkers = $datWrapper->getNumOfMarkers();
my $pedWrapper = new PedWrapper($inPedfile, $numOfMarkers);
my @familiesList = $pedWrapper->getFamiliesNumbers();

open(RANGES, ">>$workDir\/Ranges.txt");
my $currIndex = 0;
my $DAGContent = "";
my $i = 1;
my $MAX_MARKERS_IN_SINGLE_CLUSTERING_JOB = 500;

while ($i <= $numOfMarkers) {
	my $lastMarkerInGroup = $i;
	my $numMarkersInGroup = 1;
	my $totalDist = 0;
	
	#Add markers to the group while they are close enough
	while ($lastMarkerInGroup < $numOfMarkers and $numMarkersInGroup <= $MAX_MARKERS_IN_SINGLE_CLUSTERING_JOB) {
		my $dist = $datWrapper->calcDistIn_cM_BetweenTwoMarkers($lastMarkerInGroup, $lastMarkerInGroup+1);
		if ($totalDist + $dist >= $bigWindowSize) {last;}
		$totalDist += $dist;
		$numMarkersInGroup++;
		$lastMarkerInGroup++;		
	}
	
	#Create a new directory for this group if needed
	my $currDir = "$workDir\/datafile_".($currIndex);
	if( -d $currDir ) {&DAGSplit::splitLog("$currDir already exist, content will be overriden.");}
	else {mkdir "$currDir" or &DAGSplit::logInnerSysError("mkdir $currDir failed");}
	
	#Save dat and ped files with only the relevant markers
	my @currentMarkersList = ($i .. List::Util::min($i + $numMarkersInGroup-1, $numOfMarkers));
	open DATFILE_FH, ">$currDir\/datafile.dat";	
	$datWrapper->writeSubDatFileByListOfMarker(\*DATFILE_FH, \@currentMarkersList);
	close DATFILE_FH;	
	open PEDFILE_FH, ">$currDir\/pedfile.dat";
	$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@currentMarkersList, \@familiesList);
	close PEDFILE_FH;
	
	#update RunningParams and ranges file
	&DAGSplit::saveToFile( "$currDir\/RunningParams.txt", "./snpDistiller 2 datafile.dat pedfile.dat $checkGenoParam _X_ $clusteringParamToDistiler >> SNPdistillerOutput.txt\n$cMorganWindowSize");   # A parameter for window size calc
	print RANGES $i." ".($i + $numMarkersInGroup - 1)."\n"; 	
	
	#Create clustering.sub file
	my $ClusteringTmplForWorkDir = $ClusteringTmpl;
	$ClusteringTmplForWorkDir =~ s/_WORKDIR_/$workDir/g;
	&DAGSplit::saveToFile( "$currDir\/Clustering.sub",$ClusteringTmplForWorkDir);
	
	#Update DAG contents
	$DAGContent = $DAGContent."JOB ".$datfileName."B".$currIndex." ".$currDir."\/Clustering.sub\n"."VARS ".$datfileName."B".$currIndex." TASKINUM=\"1".$currIndex."\" JOBWORKDIR=\"".$currDir."\"\n";
	
	$currIndex++;
	$i += $numMarkersInGroup;
}

&DAGSplit::saveToFile( "ClusteringSubJobs_$datfileName.dag", $DAGContent );
close RANGES;

#dvir 2011/12/04 - no more use in semaphores
#&General::releaseSemaphore($semid);
exit(0);
