package ProcessingBackend;
################################################################
# 		This file is the library for all the function 
#	which are common in the preparing of pedfile and datafile 
#	for every point process.
################################################################

$YES = 1;     														
$NO = -1;    														 
$SOMETIME = 0;   				
$DEBUG = $NO;
$SUCCESS = 777;
$ERROR = -1;
$SYSTEM = "SYSTEM";
$INPUT = "INPUT";

$LINKED = "LINKED";
$UNLINKED = "UNLINKED";
$INFINITY = "-Infinity";
$UNCALCULATED_INFORMATION = "Unknown"; 
$SKIPPED_INFORMATION = "Skipped";
$LOCAL_RESULT_FILE = "result_local";
$CONDOR_RESULT_FILE = "result_condor";
$SKIPPED_RESULT_FILE = "skipped";
$ERROR_RESULT_FILE = "error";
$CONDOR_COMPLEXITY_FILE = "complexity";
$RESULT_HTML_FILE_FROM_CONDOR = "results.html";
$EXCEPTION_FILE = "exception.txt";

$OS = "UNIX";
$RUN_DIR = ".";
$HTTP_HOST = "http://cbl-hap.cs.technion.ac.il/";
$OUTPUT_FILE = "formatted_output.html";
$OUTPUT_NAVIGATION_FILE = "nav_output.html";
$OUTPUT_RESULTS_FILE = "res_output.html";
$OUTPUT_RESULTS_FILE_FOR_PDF = "res_output_pdf.html";
$DEFAULT_DEBUG_FILE = "debug.txt";
$DEBUG_FILE = "";
$DEFAULT_ERROR_FILE = "error.txt";
$ERROR_FILE = "";
$COMPLEXITY_CONVERSION_TABLE_FILE = "Texts\/ComplexityConversionTable.txt";    							
$LOD_HEADER_FILE = "Unknown.lod.file";
$CMPLX_HEADER_FILE = "Unknown.cmplx.file";
$COLLECT_DATA_FILE = "collect_data.txt";
$ERROR_IN_COMPUTATION = "Error in computation";
# dvir 2010-08-30 according to the dat format, a line of a marker name should start with "3" and the 
# marker name if specified is between "#"
$MARKER_ROW_REG_EXP_PREFIX = "3 .*#.*#";

$TwoPointNUM = 7;
$clusteringPost = 0;
#graph info
$graphDivisionSize = 50;
$graphNumMarkers = 0;
$graphLastPed  = -1;

################################################################
#			Segregation specific parameters
################################################################
@frequencesQuick =  ( "0.9 0.1",  "0.999 0.001");
@frequencesMedium = ( "0.9 0.1", "0.95 0.05", "0.99 0.01","0.995 0.005", "0.999 0.001" );
@frequencesLong =   ( "0.9 0.1", "0.95 0.05", "0.99 0.01","0.995 0.005", "0.999 0.001" );


@penetranceDomQuick = ("0 1 1");
@penetranceRecQuick = ("0 0 1");

@penetranceDomMedium = ("0 1 1", "0 0.99 0.99", "0 0.95 0.95", "0 0.9 0.9", "0 0.8 0.8", "0 0.7 0.7", "0 0.6 0.6");
@penetranceRecMedium = ("0 0 1", "0 0 0.99", "0 0 0.95", "0 0 0.9", "0 0 0.8", "0 0 0.7", "0 0 0.6");

@penetranceDomLong = ("0 1 1", "0 0.99 0.99", "0 0.95 0.95", "0 0.9 0.9", "0 0.8 0.8",
		      "0 0.7 0.7", "0 0.6 0.6","0 0.5 0.5", "0 0.4 0.4", "0 0.3 0.3",
		      "0 0.2 0.2", "0 0.1 0.1");
@penetranceRecLong = ("0 0 1", "0 0 0.99", "0 0 0.95", "0 0 0.9", "0 0 0.8",
		      "0 0 0.7", "0 0 0.6", "0 0 0.5", "0 0 0.4", "0 0 0.3",
		      "0 0 0.2", "0 0 0.1");
			  
############################################
#	Segregation Project specific test points
############################################

@frequencies = ();
@peneAll = ();

###############################################
#		BEDcreator constants
###############################################

$MAX_LOD = 3;
$CUBE_BASE_SIZE = 10;
$BED_OUTPUT_FILE = "custom_tracks.txt";

################################################################
################################################################
# 			PUBLIC FUNCTIONS:
################################################################
sub setRunDir( $ );
sub setLogFiles( $ );
sub setHeaderFiles( $ $ );
sub debug_print( $ );
sub makeSystemError( $ \% );
sub makeInputError( $ \% );
sub saveToFile( $ $ \% );
sub getThetaFromCM( $ );
sub getCMfromTheta( $ );
sub makeLink( $ $ \% );
sub initVariablesFromPrePoint( $ );
sub twoLocusDivideToPoints( $ \@ \@ );
sub divideToPedigrees( $ );
sub changeLociAndCode( $ $ \% );
sub getCalculatedChrNumber();
sub createFamilyResult( $ $ $ \% );
sub updateTotalLOD( \@ \@ \% );
sub createTotalResult( \@ $ $ \% );
sub printTablesToOutput( \% );
sub runEachMarker( \% $ );
sub runBetweenTwoMarkers( \% $ );
sub runOnPoint( \% $ );
sub customRun( \% $ );
sub jumpsRun( \% $ );
sub convertChromosomesToMarkers( \@ \@ );
sub twoLocusDivideToChromosomes( \@ \@ \@ \@ \% );
sub multiPointDivision( \@ \@ \% $ );
sub printOutputHeader( $ \% );
sub divideToPoints( $ \@ );
sub getPointInfoFromDirectory( $ \% );
sub calculatePoint( \% \% );
sub divideToChromosomes( \@ \@ \@ \% );
sub makeGraphs( );
sub makeOuputFrameset();

#####		BED PUBLIC FUNCTIONS:    #####
sub createBEDfile( $ $ $ );
sub addMarkerLODscore( $ $ $ $ );
sub addClusterLODscore( $ $ $ $ $ );

##### 		COLLECT DATA             #####
sub addMarkerData( $ $ $ $ );

################################################################
#			PRIVATE FUNCTIONS:
################################################################
sub initTotalLOD( $ $ \% );
sub printProjectOutputButtom( \% );
sub finishOutputFile( \% );
sub getPointMarker( $ $ $ );
sub addTheta( $ $ );
sub getComplexityLevel($ \%);
sub pushOffMap( $ $ $ \@ \% );
sub getDistTillMarker( $ \% );
sub initPoint( $ $ );
sub getInfoFromResultFile( $ \% );
sub getPointDist( $ );
sub printLinetoTable( \% $ $ $ $ $ $ $ $ $ );
sub printColoredNumber( $ );
sub printCmplxToTable( \% $ $ );
sub getChromosomes( $ );
sub getHeaderLine( $ $ \% );

#####		BED PRIVATE FUNCTIONS:    #####
sub getLODcolor( $ );

################################################################


################################################################
#		setRunDir
#	Sets RUN_DIR parameter
#
#	Parameters:
#					$_[0] - Directory
#	Returns:
#					NOTHING
################################################################
sub setRunDir( $ )
{
	$RUN_DIR = $_[0];
	return;
}
################################################################

################################################################
#		setLogFiles
#	Sets DEBUG_FILE parameter
#
#	Parameters:
#					$_[0] - Path and start of filename
#	Returns:
#					NOTHING
################################################################
sub setLogFiles( $ )
{
	$DEBUG_FILE = $_[0].$DEFAULT_DEBUG_FILE;
	$ERROR_FILE = $_[0].$DEFAULT_ERROR_FILE;
	return;
}
################################################################

################################################################
#		setHeaderFiles
#	Sets DEBUG_FILE parameter
#
#	Parameters:
#					$_[0] - Path and start of LOD table header filename
#					$_[1] - Path and start of Complexity table header filename
#	Returns:
#					NOTHING
################################################################
sub setHeaderFiles( $ $ )
{
	$LOD_HEADER_FILE = $_[0];
	$CMPLX_HEADER_FILE = $_[1];
	return;
}
################################################################



################################################################
# 				printOutputHeader
#	Prints OUTPUT file header
#
#	Parameters:
#					$_[0] - Initial message file
#					$_[1] - Hash with variables
#	Returns:
#					NOTHING
################################################################
sub printOutputHeader( $ \% )
{
	my $messageFile =  $_[0];
	%variables =  %{ $_[1] };
	&debug_print( "Print OutputHeader with message file=($messageFile)" );
	
	my $message = "";
	if ( -e $messageFile )
	{
		open IN, $messageFile;
		$message = <IN>;
		close IN;
	}
	
	%variables =  %{&makeSystemError("Failed creating output file ($OUTPUT_FILE)", \%variables )}
		unless ( open OUT, ">$OUTPUT_FILE" );
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
		
	print OUT "<html>\n<body bgcolor=\"#F0FFFF\">\n<body>\n<center>\n<p><h1><u><b>\n";
	$variables{ program } =~ s/\./\ /;
	print OUT "Results of $variables{ program } Analysis for Project $variables{ PROJ }\n</b></u></h1></p>\n</center>\n";
	
	### add main graphs
	if ( $clusteringPost == 1 ){			
		my $pedInd = 0;
		my $projectDir = $RUN_DIR;
		print OUT "<a name=\"Graph\"></a>\n";
		while( -e "$projectDir\/LODgraph.pedfile_$pedInd.png" ){
			print OUT "<center>Graph for Pedigree $pedInd</center><br>\n";
			print OUT "<center><img src=\"LODgraph.pedfile_$pedInd.png\" /></center><br><br>\n";
			$pedInd++;
		}
	}
	###
	
	######## Genome browser link ######
	$fullAddress = $HTTP_HOST;
	if ( $variables{ webDir } =~ /^.*var\/www\/html\/(.*)$/ ) {
		$fullAddress = $HTTP_HOST.$1;
	}
	@positions = @{$variables { positions }}; 
	if ($#positions > -1) {
		print OUT "<a href=\"http://genome.ucsc.edu/cgi-bin/hgTracks?clade=mammal&org=Human&db=hg18&pix=800&hgt.customText=$fullAddress\/$BED_OUTPUT_FILE\" target=\"_blank\">View in genome browser</a>";
	}
	else {
		print OUT "<font color=\"#999999\"><u>View in genome browser</u></font>&nbsp;(No data. Available for \".snp\" input file format only)";
	}
	
	print OUT $message;
	close OUT;
}
################################################################


################################################################
#		twoLocusDivideToChromosomes
#	Removes all the chromosomes wich are unlinked to the disease loci
#
#	Parameters:
#					$_[0] - Array with datafile content
#					$_[1] - Array with pedfile content
#					$_[2] - Array with points1
#					$_[3] - Array with points2
#					$_[4] - Hash with variables
#	Returns:
#			Hash with chromosomes in the format:
#					$chromosomes{ NUMBER } =			Number of chromosomes pairs
#					@{ $chromosomes{ DATA } } = 		Array of datafiles, one per chromosome
#					@{ $chromosomes{ PED } } = 			Array of pedfiles, one per chromosome
#					@{ $chromosomes{ POINTS1 } } = 		Array of arrays of points, one point's array per chromosome
#					@{ $chromosomes{ POINTS2 } } = 		Array of arrays of points, one point's array per chromosome
#
################################################################
sub twoLocusDivideToChromosomes( \@ \@ \@ \@ \% )
{
	my @data = @{ $_[0] };
	my @ped = @{ $_[1] };
	my @points1 = @{ $_[2] };
	my @points2 = @{ $_[3] };
	%variables = %{ $_[4] };
	&debug_print("twoLocusDivideToChromosomes, Number of chromosomes is ($#{ $variables{ chromosomes } })");
	&debug_print("twoLocusDivideToChromosomes: data=(@data)");
	
	my %chromosomes = %{ &divideToChromosomes( \@data, \@ped, \@points1, \%variables ) };
	my %tempChromosomes = %{ &divideToChromosomes( \@data, \@ped, \@points2, \%variables ) };

	@{ $chromosomes{ POINTS1 } } = @{ $chromosomes{ POINTS } };
	@{ $chromosomes{ POINTS2 } } = @{ $tempChromosomes{ POINTS } };

#	Pointers problem - have to do assign cycle
	for( my $i=0; $i<= $#{ $chromosomes{ POINTS1 } }; $i++ )
	{
		@{ ${ $tempChromosomes{ POINTS1 } }[ $i ] } = (@{ ${ $chromosomes{ POINTS1 } }[ $i ] });
		&debug_print("twoLocusDivideToChromosomes: tempChromosome[p1][$i]=(@{ ${ $chromosomes{ POINTS1 } }[ $i ] })");
	}
	for( my $i=0; $i<= $#{ $chromosomes{ POINTS2 } }; $i++ )
	{
		@{ ${ $tempChromosomes{ POINTS2 } }[ $i ] } = (@{ ${ $chromosomes{ POINTS2 } }[ $i ] });
		&debug_print("twoLocusDivideToChromosomes: tC[p2][$i]=(@{ ${ $chromosomes{ POINTS2 } }[ $i ] })");
	}

	my $nextTempChromosome = 0;
	$chromosomes{ NUMBER } = 0;


	my @firstLineArr = split /\s+/, $data[0];
	my $firstLine = join " ", @firstLineArr[1..$#firstLineArr];
	my $colBase = 9;
	$colBase ++ if ( $variables{mode} > 1 );
	for( my $chNum1 = 0; $chNum1 <= $#{ $variables{ chromosomes } }; $chNum1 ++ )
	{
		for( my $chNum2 = 0; $chNum2 <= $#{ $variables{ chromosomes } }; $chNum2 ++ )
		{
			&debug_print("twoLocusDivideToChromosomes: Iteration chnum1=($chNum1), chnum2=($chNum2)");
			my $chNum = $chromosomes{ NUMBER };
			$chromosomes{ NUMBER } ++;
			${ $chromosomes{ DATA } }[ $chNum ] = "";
			${ $chromosomes{ PED } }[ $chNum ] = "";
			@{ ${ $chromosomes{ POINTS1 } }[ $chNum ] } = ();
			@{ ${ $chromosomes{ POINTS2 } }[ $chNum ] } = ();
			
			if ( $chNum1 == $chNum2 )
			{
				&debug_print("twoLocusDivideToChromosomes: Loci on the same chrmsm=($chNum1)");
				
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $tempChromosomes{ DATA } }[ $nextTempChromosome ];
				&debug_print("Chromosome ($chNum) datafile created, =(${ $chromosomes{ DATA } }[ $chNum ])" );

				${ $chromosomes{ PED } }[ $chNum ] = ${ $tempChromosomes{ PED } }[ $nextTempChromosome ];
				&debug_print("Chromosome pedfile created, =(${ $chromosomes{ PED } }[ $chNum ])" );
				
				@{ ${ $chromosomes{ POINTS1 } }[ $chNum ] } = @{ ${ $tempChromosomes{ POINTS1 } }[ $nextTempChromosome ] };
				&debug_print("Chromosome points1 arrays updated, Points1=(@{ ${ $chromosomes{ POINTS1 } }[ $chNum ] })" );

				@{ ${ $chromosomes{ POINTS2 } }[ $chNum ] } = @{ ${ $tempChromosomes{ POINTS2 } }[ $nextTempChromosome ] };
				&debug_print("Chromosome points2 arrays updated, Points2=(@{ ${ $chromosomes{ POINTS2 } }[ $chNum ] })" );

#	Make the same starting point
				for( my $strtMrkr = 0 ; $strtMrkr < $#{ ${ $chromosomes{ POINTS2 } }[ $chNum ] }; $strtMrkr += 2 )
				{
					${ ${ $chromosomes{ POINTS2 } }[ $chNum ] }[$strtMrkr] = ${ ${ $chromosomes{ POINTS1 } }[ $chNum ] }[0];
				}
				
				$nextTempChromosome ++;
				next;
			}
			&debug_print("twoLocusDivideToChromosomes: Loci on different chromosomes");
			
			my $firstMarker1 = ${ $variables{ chromosomes } }[$chNum1];
			my $lastMarker1 = 0;
			if ( $chNum1 < $#{ $variables{ chromosomes } } )
			{
				$lastMarker1 = ${ $variables{ chromosomes } }[$chNum1+1] - 1;
			}
			else
			{
				$lastMarker1 = $firstLineArr[0] - 2;
			}
			
			my $firstMarker2 = ${ $variables{ chromosomes } }[$chNum2];
			my $lastMarker2 = 0;
			if ( $chNum2 < $#{ $variables{ chromosomes } } )
			{
				$lastMarker2 = ${ $variables{ chromosomes } }[$chNum2+1] - 1;
			}
			else
			{
				$lastMarker2 = $firstLineArr[0] - 2;
			}
			&debug_print("Chromosomes Num ($chNum1,$chNum2),place=($chNum) First=($firstMarker1,$lastMarker1), Second=($firstMarker2,$lastMarker2)");
			
			my ( $min1, $max1, $min2, $max2 ) = ( $firstMarker1, $lastMarker1, $firstMarker2, $lastMarker2 );
			( $min1, $max1, $min2, $max2 ) = ( $min2, $max2, $min1, $max1 ) if ( $min1 > $min2 );
			
			&debug_print("Create Chromosome datafile" );
			my $numOfMarkers = ($lastMarker1 - $firstMarker1 + 1) + ($lastMarker2 - $firstMarker2 + 1);
			$numOfMarkers += 2;
			&debug_print("twoLocusDivideToChromosomes: numOfMarkers=($numOfMarkers)");
			
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$numOfMarkers." ".$firstLine."\n";
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[1]."\n";
			for( my $i = 1; $i <= $numOfMarkers; $i ++ )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."$i ";
			}
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."\n";
			my $index = 3;
			while ( ( $index <= $#data ) && ( $data[$index] !~ /ALLELE NUMBERS, NO. OF ALLELES/ ) )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
				$index ++;
			}
			for( my $mark = 1; $mark < $min1; $mark ++ )
			{
				$index += 2;
			}
			for( my $mark = $min1; $mark <= $max1; $mark ++ )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index+1]."\n";
				$index +=2;
			}
			for( my $mark = ($max1+1); $mark < $min2; $mark ++ )
			{
				$index += 2;
			}
			for( my $mark = $min2; $mark <= $max2; $mark ++ )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index+1]."\n";
				$index +=2;
			}
			while ( ( $index <= $#data ) && ( $data[$index] !~ /SEX DIFFERENCE, INTERFERENCE/ ) )
			{
				$index ++;
			}
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."0 0 ";
			for( my $th = $min1; $th < $max1; $th ++ )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].${$variables{thetas}}[$th-1]." ";
			}
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."0.5 ";
			for( my $th = $min2; $th < $max2; $th ++ )
			{
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].${$variables{thetas}}[$th-1]." ";
			}
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."<< RECOMBINATION VALUES\n";
			$index ++;
			while ( $index < $#data )
			{
				$index ++;
				${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
			}
			&debug_print("Chromosome ($chNum) datafile created, =(${ $chromosomes{ DATA } }[ $chNum ])" );

			&debug_print("Create Chromosome pedfile" );
			my $colStrt1 = $colBase + 2*$min1-1;
			my $colFnsh1 = $colBase + 2*$max1;
			my $colStrt2 = $colBase + 2*$min2-1;
			my $colFnsh2 = $colBase + 2*$max2;

			for( $i = 0; $i <= $#ped; $i++ )
			{
				my @words = split /\s+/, $ped[$i];
				@words = @words[ 0..$colBase, $colStrt1..$colFnsh1, $colStrt2..$colFnsh2 ];
				my $remained = join " ", @words;
				${ $chromosomes{ PED } }[ $chNum ] = ${ $chromosomes{ PED } }[ $chNum ]."\n".$remained;
			}
			&debug_print("Chromosome pedfile created, =(${ $chromosomes{ PED } }[ $chNum ])" );

			&debug_print("Create Chromosome updated point array" );
			&debug_print("p1=(@points1), p2=(@points2), first=($firstMarker1), sec=($firstMarker2)" );

			my $mark = 1;
			$mark = $max1 - $min1 + 2 if ( $firstMarker1 > $firstMarker2 );
			my $p = 1;
			for( ; $p <= $#points1; $p+=2)
			{
				last if ( $firstMarker1 eq $points1[$p-1] );
			}
			for( ; $p <= $#points1; $p+=2)
			{
				&debug_print("p=($p), s=($#points1), firstMarker1=($firstMarker1), points[p-1]=($points1[$p-1])");
				last if ( $firstMarker1 ne $points1[$p-1] );
				push @{ ${ $chromosomes{ POINTS1 } }[ $chNum ] }, ( $mark, $points1[$p] );
			}
			&debug_print("Chromosome points1 arrays updated, Points1=(@{ ${ $chromosomes{ POINTS1 } }[ $chNum ] })" );

			$mark = $max1 - $min1 + 2;
			$mark = 1 if ( $firstMarker1 > $firstMarker2 );
			$p = 1;
			for( ; $p <= $#points2; $p+=2)
			{
				last if ( $firstMarker2 eq $points2[$p-1] );
			}
			for( ; $p <= $#points2; $p+=2)
			{
				last if ( $firstMarker2 ne $points2[$p-1] );
				push @{ ${ $chromosomes{ POINTS2 } }[ $chNum ] }, ( $mark, $points2[$p] );
			}
			&debug_print("Chromosome points2 arrays updated, Points2=(@{ ${ $chromosomes{ POINTS2 } }[ $chNum ] })" );
		}
	}
	
	return \%chromosomes;
}
################################################################




################################################################
#		divideToChromosomes
#	Removes all the chromosomes which are unlinked to the disease loci
#
#	Parameters:
#					$_[0] - Array with datafile content
#					$_[1] - Array with pedfile content
#					$_[2] - Array with points
#					$_[3] - Hash with variables
#	Returns:
#			Hash with chromosomes in the format:
#					$chromosomes{ NUMBER } =			Number of chromosomes
#					@{ $chromosomes{ DATA } } = 		Array of datafiles, one per chromosome
#					@{ $chromosomes{ PED } } = 			Array of pedfiles, one per chromosome
#					@{ $chromosomes{ POINTS } } = 		Array of arrays of points, one point's array per chromosome
#
################################################################
sub divideToChromosomes( \@ \@ \@ \% )
{
	my @data = @{ $_[0] };
	my @ped = @{ $_[1] };
	my @points = @{ $_[2] };
	%variables = %{ $_[3] };
	&debug_print("divideToChromosomes:	Number of chromosomes is ($#{ $variables{ chromosomes } })");
	
	my %chromosomes = ();
	$chromosomes{ NUMBER } = $#{ $variables{ chromosomes } } + 1;
	@{ $chromosomes{ DATA } } = ();
	@{ $chromosomes{ PED } } = ();
	@{ $chromosomes{ POINTS } } = ();
	
	my @firstLineArr = split /\s+/, $data[0];
	my $firstLine = join " ", @firstLineArr[1..$#firstLineArr];
	my $colBase = 9;
	$colBase ++ if ( ( $variables{ mode } =~ /\d+/ ) && ( $variables{ mode } > 1 ) );
	for( my $chNum = 0; $chNum <= $#{ $variables{ chromosomes } }; $chNum ++ )
	{
		my $firstMarker = ${ $variables{ chromosomes } }[$chNum];
		my $lastMarker = 0;
		if ( $chNum < $#{ $variables{ chromosomes } } )
		{
			$lastMarker = ${ $variables{ chromosomes } }[$chNum+1] - 1;
		}
		else
		{
			$lastMarker = $firstLineArr[0] - 1 if ( $variables{ program } !~ /TwoLocus/ );
			$lastMarker = $firstLineArr[0] - 2 if ( $variables{ program } =~ /TwoLocus/ );
		}
		&debug_print("divideToChromosomes:	Chromosome Num ($chNum), First Marker=($firstMarker), Last Marker=($lastMarker)");
#	Create Chromosome datafile
		my $numOfMarkers = $lastMarker - $firstMarker + 1;
		$numOfMarkers += 1 if ( $variables{ program } !~ /TwoLocus/ );
		$numOfMarkers += 2 if ( $variables{ program } =~ /TwoLocus/ );
		&debug_print("divideToChromosomes:	numOfMarkers=($numOfMarkers)");
		
		${ $chromosomes{ DATA } }[ $chNum ] = $numOfMarkers." ".$firstLine."\n";
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[1]."\n";
		for( my $i = 1; $i <= $numOfMarkers; $i ++ )
		{
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."$i ";
		}
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."\n";
		my $index = 3;
		while ( ( $index <= $#data ) && ( $data[$index] !~ /ALLELE NUMBERS, NO. OF ALLELES/ ) )
		{
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
			$index ++;
		}
		for( my $mark = 1; $mark < $firstMarker; $mark ++ )
		{
			$index += 2;
		}
		for( my $mark = 0; $mark <= ($lastMarker - $firstMarker); $mark ++ )
		{
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index+1]."\n";
			$index +=2;
		}
		while ( ( $index <= $#data ) && ( $data[$index] !~ /SEX DIFFERENCE, INTERFERENCE/ ) )
		{
			$index ++;
		}
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."0 " if ( $variables{ program } !~ /TwoLocus/ );
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."0 0 " if ( $variables{ program } =~ /TwoLocus/ );
		for( my $th = $firstMarker; $th < $lastMarker; $th ++ )
		{
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].${$variables{thetas}}[$th-1]." ";
		}
		${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ]."<< RECOMBINATION VALUES\n";
		$index ++;
		while ( $index < $#data )
		{
			$index ++;
			${ $chromosomes{ DATA } }[ $chNum ] = ${ $chromosomes{ DATA } }[ $chNum ].$data[$index]."\n";
		}
		&debug_print("divideToChromosomes:	Chromosome datafile created" );

#	Create Chromosome pedfile
		my $colStrt = $colBase + 2*$firstMarker-1;
		my $colFnsh = $colBase + 2*$lastMarker;
		${ $chromosomes{ PED } }[ $chNum ] = "";
		for( $i = 0; $i <= $#ped; $i++ )
		{
			my @words = split /\s+/, $ped[$i];
			@words = @words[0..$colBase, $colStrt..$colFnsh];
			my $remained = join " ", @words;
			${ $chromosomes{ PED } }[ $chNum ] = ${ $chromosomes{ PED } }[ $chNum ]."\n".$remained;
		}
		&debug_print("divideToChromosomes:	Chromosome pedfile created" );

#	Create Chromosome updated point array	
		my $p = 1;
		@{ ${ $chromosomes{ POINTS } }[ $chNum ] } = ();
		for( ; $p <= $#points; $p+=2)
		{
			last if ( $firstMarker eq $points[$p-1] );
		}
		for( ; $p <= $#points; $p+=2)
		{
			last if ( $firstMarker ne $points[$p-1] );
			push @{ ${ $chromosomes{ POINTS } }[ $chNum ] }, ( 1, $points[$p] );
		}
		&debug_print("divideToChromosomes:	Chromosome points array updated, points at ($chNum) are (@{ ${ $chromosomes{ POINTS } }[ $chNum ] })" );
	}

	return \%chromosomes;
}
################################################################



################################################################
#			printProjectOutputButtom
#	Prints the buttom part of result form
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					Message to print
################################################################
sub printProjectOutputButtom( \% )
{
	%variables = %{ $_[0] };
	&debug_print( "Creating Project details for user" );
	my $result = "";
	
	$result = $result."<p align=\"left\"><b>WARNING:</b> 
					All pairs of markers in the table(s) are assumed to be unlinked.
					Use the option LINKED to specify otherwise.</p>" 
	if ( ( $variables{ program } =~ /TwoLocus.TwoPoint/ ) && ( $variables{ linked } =~ /Unlinked/ ) );

	$result = $result."<p align=\"left\">";
	$result = $result."<b>The input used by Superlink-online to generate these results are:<br></b>\n";
	$result = $result."Number of markers = ". ( $#{$variables{ markersNames }} + 1 )."<br>\n";
	$result = $result."Marker Names = @{$variables{ markersNames }}<br>\n";

	if ( $variables{ program } =~ /TwoLocus/ )
	{
		$result = $result."Number of used markers for First Locus= ".( $#{$variables{ usedMarkersNames1 }} + 1 )."<br>\n";
		$result = $result."Used Markers for First Locus = @{$variables{ usedMarkersNames1 }}<br>\n";
		$result = $result."First Locus gene frequency = $variables{ mutantFreq1 } <br>\n";

		$result = $result."Number of used markers for Second Locus= ".( $#{$variables{ usedMarkersNames2 }} + 1 )."<br>\n";
		$result = $result."Used Markers for Second Locus = @{$variables{ usedMarkersNames2 }}<br>\n";
		$result = $result."Second Locus gene frequency = $variables{ mutantFreq2 } <br>\n";

		my $s = "";$s = "es" if ( $variables{ mode } > 1 );										#	Spell check :)
  		$result = $result."Mode of inheritance (MOI) = Complex, $variables{ mode } penetrance class$s <br>\n";
   		for($i=0; $i<= $#{$variables{ penetrances }}; $i ++ )
   		{
			$result = $result." ${$variables{ penetrances }}[$i] <br>\n";
		}
  		$result = $result."Disease Loci are ";
  		if ( $variables{ linked } !~ /Unlinked/ )
  		{
			$result = $result."Linked <br>\n";
		}
  		else
  		{
			$result = $result."Unlinked <br>\n";
		}
	}
	else
	{
		$result = $result."Number of used markers = ".( $#{$variables{ usedMarkersNames }} + 1 )."<br>\n";
		$result = $result."Used Markers = @{$variables{ usedMarkersNames }}<br>\n";
		$result = $result."Disease mutant gene frequency = $variables{ mutantFreq } <br>\n";
	
		if ( $variables{ mode  } =~ /dom/)
		{
			$result = $result."Mode of inheritance (MOI) = Dominant ";
			for $pene (0.. $#{$variables{ penetrances }}) {
				$result= $result."${$variables{ penetrances }}[$pene] ";
			}
			$result = $result."<br>\n";
		}
		elsif ($variables{ mode  } =~ /rec/)
		{
			$result = $result."Mode of inheritance (MOI) = Recessive ";#${$variables{ penetrances }}[0] <br>\n";
			for $pene (0.. $#{$variables{ penetrances }}) {
				$result= $result."${$variables{ penetrances }}[$pene] ";
			}
			$result = $result."<br>\n";
		}
		elsif ($variables{ mode  } =~ /z/ )
		{
			# Do nothing - input in the files
		}
		else
		{
			$result = $result."Mode of inheritance (MOI) = Complex ${$variables{ penetrances }}[0] <br>\n";
		}
 	}
	
	$result = $result."The data is X-linked<br>\n" if ( $variables{ sexLinked }  > 0 );

	$result = $result."All pedigrees analyzed together<br>\n";
	
	if ( ( ( $variables{ program } =~ /MultiPoint/ ) || ( $variables{ program } =~ /Haplotyping/ ) )
		&& ( defined($variables{ jump }) && ( $variables{ jump } !~ /Unknown/ ) ) )   {
		
		$program = "MultiPoint";
		if ( $variables{ program } =~ /Haplotyping/ ) {
			$program = "Haplotyping";
		}

		$result = $result."<br/><b>The ".$program." Analysis was performed using the following parameters: <br></b>\n";
		$result = $result."Start Marker: ".$variables{ from }."<br>";
		$result = $result."End Marker: ".$variables{ to }."<br>";
		$result = $result."Group Size: ".$variables{ jump }."<br>";
	}
	else {
		if ( $variables{ program } =~ /Haplotyping/ ) {  # A list was defined
			$result = $result."<br/><b>The Haplotyping Analysis was performed using the following markers: <br></b>\n";
			my @markers = sort {$a <=> $b} @{$variables{ usedMarkersId }};
			$result = $result."@markers<br>\n";
		}
	}
	
	$result = $result."</p>\n</body>\n</html>\n";
	
	return $result;
}
################################################################



################################################################
#			finishOutputFile
#	Connect all the output files into one
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#				$SUCCESS 	- if success
#				$ERROR	- if error
################################################################
sub finishOutputFile( \% )
{
	%variables = %{ $_[0] };
	&debug_print("finishOutputFile: Preparing final html");
	
	%variables = %{&makeSystemError("Cannot append to ($OUTPUT_FILE)", \%variables )}
		unless (open OUT, ">>$OUTPUT_FILE");
	return $ERROR if ( $NO !~ $variables{ errorExist } );
	
	print OUT &printProjectOutputButtom( \%variables );
	close OUT;

	if ( -e $variables{ OUTPUT_FILE } )
	{
		%variables = %{&makeSystemError("Failed delete old OUTPUT FILE ($variables{ OUTPUT_FILE })", \%variables )}
			unless (unlink ($variables{ OUTPUT_FILE }));
		return $ERROR if ( $NO !~ $variables{ errorExist } );
	}
		
	%variables = %{&makeSystemError("Failed copy ($OUTPUT_FILE) to ($variables{ OUTPUT_FILE })", \%variables )}
		unless (link ( "$OUTPUT_FILE", $variables{ OUTPUT_FILE }));
		
	return $ERROR if ( $NO !~ $variables{ errorExist } );
	return $SUCCESS;
}
################################################################



################################################################
#		saveToFile
# Reads upload file and write it in temporary file
#
# Parameters: 
#			 $_[0] - path to file to create
#            $_[1] - data to save in file
#			 $_[2] - Hash with variables for error purpose
# Returns:
#					$SUCCESS	- 	Success
#					$ERROR	-	Failure
################################################################
sub saveToFile( $ $ \% )
{
	my $name = $_[0];
	my $data = $_[1];
	%variables = %{$_[2]};
	
	&debug_print ( "Creating file: ($name)" );
	&debug_print ( "With data: ($data)" );
	%variables = %{&makeSystemError("Could not open file ($name) for write (in function saveToFile) error: ($!)", \%variables)}
		unless ( open (FH,">$name") );
	return $ERROR if ( $NO !~ $variables{ errorExist } );

  	print FH $data;	
  	close FH;
  	return $SUCCESS;
}
################################################################



################################################################
# 				getPointMarker
#		Return the name of marker on the chromosome with given distance 
#	from the first marker on this chromosome or blank if there is
#	no marker on this point	
#
#	Parameters:
#					$_[0] - Chromosome number
#					$_[1] - Distance
#					$_[2] - Serial number of locus
#	Returns:

#					Marker name on this point
################################################################
sub getPointMarker( $ $ $ )
{
	my $dir = $_[0];
	my $distance = $_[1];
	my $serial = $_[2];
	&debug_print("getPointMarker: Looking for marker name in dir=($dir) with distance ($distance) from first marker with serial ($serial)" );

	return $UNCALCULATED_INFORMATION 
		unless (open DAT, "$dir\/$variables{ datafile }");
	my @data = <DAT>;
	close DAT;
	my $line = pop @data;
	my $marker = 1;
	
	my @words = split '\(', $line;
	@words = split '\)', $words[1];
	@words = split ' ', $words[0];
	my $twoLocus = 0;
	$twoLocus = 1 if ( $#words > 0 );
	
	if ( 1 == $twoLocus )
	{
		$marker = $words[0] if ( $serial =~ /1/ );
		$marker = $words[1] if ( $serial =~ /2/ );
	}
	else
	{
		$marker = 1;
	}
	
	while ( $line !~ /RECOMBINATION VALUES/ )
	{
		$line = pop @data;
	}

	@words = split "<<", $line;
	@words = split /\s+/, $words[0];
	shift @words if ( 1 == $twoLocus );
	
	my $tempM = 0;
	for ( ; $tempM < $marker; $tempM++ )
	{
		shift @words;
	}

	my $dist = 0;
	my $found = 0;
	&debug_print("getPointMarker: marker=($marker), tempM=($tempM), updating distance to distance=($distance)" );
	while( $#words >= -1 ) 
	{
		$found = 1 if ( abs( $distance - $dist ) < 0.000001 );
		last if ( 1 == $found );
		last if ( $#words < 0 );
		$dist += &getCMfromTheta( shift @words );
		$tempM ++;
	}

	$marker = $tempM if ( 1 == $found );
	&debug_print("getPointMarker: marker=($marker), distance updated to dist=($dist)" );
	
	my $name = "&nbsp;";
	return $name if ( 0 == $found );
	
	while ( $line !~ /ALLELE NUMBERS, NO. OF ALLELES/ )
	{
		$line = shift @data;
	}
	my $current = 1;
	while ( $current < $marker )
	{
		$line = shift @data;
		$line = shift @data;
		$current ++;
	}
	@words = split '#', $line;
	$name = $words[1] if ( $#words > 0 );

	&debug_print("Name found!!! =($name)");
	return $name;
}
################################################################




################################################################
#		convertChromosomesToMarkers
# Converts the chromosome numbers to the first marker numbers on them
#
# Parameters: 
#			 $_[0] - Array with points to be calculated in format: ( chrom1#, point1_Val, ..) 
#            $_[1] - Array @{ $variables{ chromosomes } } - contains the first markers
# Returns:
#			Updated points array
################################################################
sub convertChromosomesToMarkers( \@ \@ )
{
	my @points = @{ $_[0] };
	my @markers = @{ $_[1] };
	
	&debug_print ( "convertChromosomesToMarkers: Points=(@points), First markers are=(@markers)" );

	for( my $i=0; $i < $#points; $i+= 2 )
	{
		$points[$i] = $markers[ $points[ $i ] - 1 ];
	}
	return \@points;
}
################################################################




################################################################
# 		makeLink
#	Makes symbolic link to the existing file, if can. 
#	Otherwise makes regular copy
#
#	Parameters:
#					$_[0] - source filename
#					$_[1] - link/copy name
#					$_[2] - hash with variables
#	Returns:
#					Hash (updated) with variables
################################################################
sub makeLink( $ $ \% )
{
	my $source = $_[0];
	my $newFile = $_[1];
	%variables = %{$_[2]};
	&debug_print("Creating link with name ($newFile) to the file ($source)");
	
	return &makeSystemError("No source file ($source) exist", \%variables)
		unless ( -e $source);

	if ( $OS =~ /WINDOWS/ )
	{
		return &makeSystemError("Failed to create link ($newFile) to the file ($source)", \%variables)
			unless( link( $source, $newFile ) );
	}
	else
	{
		return &makeSystemError("Failed to create Symbolic link ($newFile) to the file ($source)", \%variables )
			unless( symlink( $source, $newFile ) );
	}
	return \%variables;
}
################################################################


################################################################
#			makeSystemError	
#	Gets the first variable (line) from the array or exits if empty
#
#	Parameters:
#					$_[0] - error message
#					$_[1] - hash with variables
#	Returns:
#					Hash (updated) with variables
################################################################
sub makeSystemError( $ \% )
{
	my $error = $_[0];
	%variables = %{ $_[1] };
	
	&debug_print( "makeSystemError, error=($error)"  );
	
	if ( $NO !~ $variables{ errorExist } )		# NO ERROR OVERWRITING
	{
		&debug_print( "Error already exists, it is ($variables{ errorMsg })"  );
		$variables{ errorMsg } = $variables{ errorMsg }."\n<p>\n</p>\n".$error;
		return $variables{ errorExist };
	}
	
	$variables{ errorExist } = $SYSTEM;
	$variables{ errorMsg } = $error;
	
	unlink $ERROR_FILE if ( -e $ERROR_FILE );
	
	
	&debug_print( "Failed to create ERROR_FILE ($ERROR_FILE) to write SYSTEM message ($error)")
		unless ( open ERR, ">$ERROR_FILE" );
	print ERR "SYSTEM\n";
	print ERR $error;
	close ERR;
	
	return \%variables;
}
################################################################

################################################################
#			makeInputError
#	Gets the first variable (line) from the array or exits if empty
#
#	Parameters:
#					$_[0] - error message
#					$_[1] - hash with variables
#	Returns:
#					Hash (updated) with variables
################################################################
sub makeInputError( $ \% )
{
	my $error = $_[0];
	%variables = %{ $_[1] };
	
	&debug_print( "makeInputError, error=($error)"  );
	if ( $INPUT =~ $variables{ errorExist } )		# NO INPUT ERROR OVERWRITING
	{
		&debug_print( "Error already exists, it is ($variables{ errorMsg })"  );
		$variables{ errorMsg } = $variables{ errorMsg }."\n<p>\n</p>\n".$error;
		return $variables{ errorExist };
	}

	$variables{ errorExist } = $INPUT;
	$variables{ errorMsg } = $error;
	
	&debug_print( "Failed to create ERROR_FILE ($ERROR_FILE) to write INPUT message ($error)")
		unless ( open ERR, ">$ERROR_FILE" );
	print ERR "INPUT\n";
	print ERR $error;
	close ERR;
	
	return \%variables;
}
################################################################

################################################################
#			debug_print
#	If $DEBUG is on - prints the message. 
#	In any case saves the message into $DEBUG_FILE
#
#	Parameters:
#					$_[0]  <= Message to print
#	Returns:
#					NOTHING
################################################################
sub debug_print( $ )
{
	my $message = $_[0];
	
	print "<p>$message</p>" if ( $DEBUG == $YES);

	unless( -e "$DEBUG_FILE")
	{
		open DBG, ">$DEBUG_FILE";
		print "<p>Debug file created = ($DEBUG_FILE) created</p>" if ( $DEBUG == $YES);
		print DBG"Debug file created = ($DEBUG_FILE) created\n";
		close DBG;					
	}

	if ($DEBUG == $YES){
		open DBG, ">>$DEBUG_FILE";
		print DBG "$message\n";
		close DBG;
	}
}
################################################################



################################################################
# 			initVariablesFromPrePoint
#	Initializes hash with variables from the modfile
#
#	Parameters:
#					$_[0] - Run directory
#	Returns:
#					Hash with variables
################################################################
sub initVariablesFromPrePoint( $ ) 
{
	my $dir = $_[0];
	my %hash = ();
	die( "No such file: ($dir\/modefile.txt)" ) if ( ! -s "$dir\/modefile.txt" );
	&debug_print("Initializing array of input data from prePoint.pl" );

	my $key = "";
	my $value = "";
	die("Failed to open ($dir\/modefile.txt) for reading")
		unless ( open MODE, "$dir\/modefile.txt" );
		
	foreach $line ( <MODE> )
	{
		my @words = split '_\(_\[_\(_', $line;
		next if ( $#words < 1 );

		@words = split '_\)_\]_\)_', $words[1];
		@words = split '=', $words[0];
		
		$key = $words[0];
		if ( $#words > 0 ) { $value = $words[1]; }
		else { 
			$value = "";
			&debug_print( "Empty value" );
		}
		if ( $value =~ /\[/ )
		{
			@array = split /\[/, $value;
			shift @array ;
			for( $i =0; $i <= $#array; $i ++ )
			{
				@words = split /\]/, $array[$i];
				$array[$i] = $words[0];
			}
			
			@{$hash{$key}} = @array;
			
			if (  $#array >= 0 ) 
			{
				#*********************** find out who needs $paramLine
				#if (  $#array > 0 ) 
				#{
				#	$paramLine = join ",", @array;
				#}
				#else
				#{
				#	$paramLine = $array[0];
				#}
			}
			else { &debug_print("Empty array, array=(@array)"); }
		}
		else { $hash{ $key } = $value; }
	}
	close MODE;

  	$hash{ currentPed } = " ";
  	$hash{ currentDat } = " ";
  	$hash{ currentOut } = " ";
  	$hash{ currentTime }  = " ";
	$hash{ errorExist } = $NO;			#	int value on purpose to return errors
										#	Possible values:	$NO, INPUT, SYSTEM
	$hash{ errorMsg } = "";				#	Error content
	
	&debug_print("Array of input data initialized successfully" );

	return \%hash;
}
################################################################



################################################################
#			changeLociAndCode
#	Gets the first variable (line) from the array or exits if empty
#
#	Parameters:
#					$_[0] - Datafile content
#					$_[1] - Program code to execute
#					$_[2] - Hash with variables
#	Returns:
#					Updated Datafile content or $ERROR
################################################################
sub changeLociAndCode( $ $ \%)
{
	my $datafile = $_[0];
	my $code = $_[1];
	%variables = %{ $_[2] };
	my $result = "";
	
	&debug_print("changeLociAndCode:  Code=($code)");
	
	my @data = split "\n", $datafile;
	my $line = "";
	while ($line !~ /NO. OF LOCI/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (NO. OF LOCI) in the data: ($datafile)", \%variables )}
			if ( $#data < 0 );
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift ( @data );
	}
	my @words = split ' ', $line;
	$words[0] = "2" if ( ( $variables{program} =~ /TwoPoint/ ) && ( $variables{program} !~ /TwoLocus/ ) );
	my $loci = $words[0];
	$words[3] = $code;
	$words[4] = "2" if ( $variables{program} =~ /TwoLocus/ );
	$line = join ' ',@words;
	$result = $result.$line."\n";

	while ($line !~ /MUT LOCUS/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (MUT LOCUS) in the data: ($datafile)", \%variables )}
			if ( $#data < 0);
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift( @data );
	}
	$result = $result.$line."\n";

	while ($line !~ /NO. OF ALLELES/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (NO. OF ALLELES) in the data: ($datafile)", \%variables )}
			if ( $#data < 0);
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift( @data );
	}
	for(my $i=1; $i<=$loci; $i++)
	{
		$result = $result.$i." ";
	}
	$result = $result."\n".$line."\n";

	for(my $j=0; $j<=$#data; $j++)
	{
		$result = $result.$data[$j]."\n" if ( $data[$j] =~ /\S/);
	}
	&debug_print("changeLociAndCode: Result=($result)");
	return $result;
}
################################################################

################################################################
#			changeLociNumber
#	Gets the first variable (line) from the array or exits if empty
#	Used for multipoint
#
#	Parameters:
#					$_[0] - Datafile content
#					$_[1] - Number of loci
#					$_[2] - Hash with variables
#	Returns:
#					Updated Datafile content or $ERROR
################################################################
sub changeLociNumber( $ $ \%)
{
	my $datafile = $_[0];
	my $num = $_[1];
	%variables = %{ $_[2] };
	my $result = "";
	
	#&debug_print("changeLociNumber:  Number=($num)");
	
	my @data = split "\n", $datafile;
	my $line = "";
	while ($line !~ /NO. OF LOCI/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (NO. OF LOCI) in the data: ($datafile)", \%variables )}
			if ( $#data < 0 );
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift ( @data );
	}
	my @words = split ' ', $line;
	$words[0] = $num;
	my $loci = $words[0];
	$line = join ' ',@words;
	$result = $result.$line."\n";

	while ($line !~ /MUT LOCUS/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (MUT LOCUS) in the data: ($datafile)", \%variables )}
			if ( $#data < 0);
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift( @data );
	}
	$result = $result.$line."\n";

	while ($line !~ /NO. OF ALLELES/)
	{
		%variables = %{&makeSystemError("Failed to find the comment: (NO. OF ALLELES) in the data: ($datafile)", \%variables )}
			if ( $#data < 0);
		return $ERROR if ( $NO !~ $variables{ errorExist } );
		$line = shift( @data );
	}
	for(my $i=1; $i<=$loci; $i++)
	{
		$result = $result.$i." ";
	}
	$result = $result."\n".$line."\n";

	for(my $j=0; $j<=$#data; $j++)
	{
		$result = $result.$data[$j]."\n" if ( $data[$j] =~ /\S/);
	}
	#&debug_print("changeLociAndCode: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				divideToPedigrees
#	Divides the given pedfile to several pedigree's pedfiles
#
#	Parameters:
#					$_[0]  - Pedfile data
#	Returns:
#					Array of pedfiles
################################################################
sub divideToPedigrees( $ )
{
	my $data = $_[0];
	&debug_print("divideToPedigrees data=(\n$data)" );
	
	my @result =();
	
	my @lines = split '\n', $data;
	my $maxPed = 0;
	
	while ( $#lines >= 0 )
	{
		my $line = shift @lines;
		my @words = split /\s+/, $line;
		next if ( 0 > $#words );
		if ( $words[0] > $maxPed )
		{
			&debug_print("divideToPedigrees result[maxPed]=($result[$maxPed])" ) if ( defined ($result[$maxPed]) );
			$maxPed = $words[0];
			$result[$maxPed] = "";
		}
		$words[0] = "1";
		$line = join " ", @words;
		$result[ $maxPed ] = $result[ $maxPed ].$line."\n";
	}
	&debug_print("divideToPedigrees result[maxPed]=($result[$maxPed])" );
	
	shift @result;				# To make the first index be 0 and not 1.
	
	return \@result;
}
################################################################



################################################################
# 				divideToPoints
#	Divides the given datafile to several datafiles, one per calculation point
#
#	Parameters:
#					$_[0]  - Datafile content
#					$_[1]  - Points to make
#	Returns:
#					Array of datafiles
################################################################
sub divideToPoints( $ \@ )
{
	my $data = $_[0];
	my @points = @{ $_[1] };
	
	&debug_print("divideToPoints: data=(\n$data), Points = (@points)" );
	
	my @result =();
	
	my @lines = split "\n", $data;
	my $poped = pop @lines;
	while ($poped !~ /RECOMBINATION VALUES/)
	{
		return @lines if ($#lines < 0);
		$poped = pop @lines;
	}

	$data = join "\n", @lines;
	&debug_print("divideToPoints: Cutted date=($data)" );
	my $datafile = 0;
	my $cM = 0;
	for( my $p=0; $p <= $#points; $p+=2 )
	{
		&debug_print("The point is (".$points[$p+1].") cM from marker ($points[$p])" );
		$result[ $datafile++ ] = $data."\n".$poped."\n1 1\n"."(".$points[$p+1].")\n";
	}
	return \@result;
}
################################################################



################################################################
# 				twoLocusDivideToPoints
#	Divides the given datafile to several datafiles, one per calculation point
#
#	Parameters:
#					$_[0]  - Datafile content
#					$_[1]  - Points to make for first locus
#					$_[2]  - Points to make for second locus
#	Returns:
#					Array of datafiles
################################################################
sub twoLocusDivideToPoints( $ \@ \@ )
{
	my $data = $_[0];
	my @points1 = @{ $_[1] };
	my @points2 = @{ $_[2] };
	&debug_print("twoLocusDivideToPoints: data=(\n$data), Points1 = (@points1), Points2 = (@points2)" );
	my @result =();
	
	my @lines = split "\n", $data;
	my $poped = pop @lines;
	while ($poped !~ /RECOMBINATION VALUES/)
	{
		return @lines if ($#lines < 0);
		$poped = pop @lines;
	}

	$data = join "\n", @lines;
	$data = $data."\n".$poped;
	
	my $datafile = -1;
	for ( my $p1=0; $p1 <= $#points1; $p1+=2 )
	{
		my $marker1 = $points1[$p1];
		for ( my $p2=0; $p2 <= $#points2; $p2+=2 )
		{
			my $marker2 = $points2[$p2];

			$datafile ++;
			my $cM1 = $points1[$p1+1];
			my $cM2 = $points2[$p2+1];
			
			&debug_print("Distances for datafile ($datafile) are ($cM1 $cM2) cM" );
			$result[ $datafile ] = $data."\n2 1\n"."($cM1 $cM2)\n($marker1 $marker2)\n";
			&debug_print("Result[ $datafile ] is ($result[ $datafile ])" );
		}
	}
	return \@result;
}
################################################################



################################################################
#			addTheta
#	Adds two thetas
#
#	Parameters:
#					$_[0] - theta1
#					$_[1] - theta2
#	Returns:
#					sum
################################################################
sub addTheta( $ $ )
{
	my $first = $_[0];
	my $second = $_[1];
	
	my $result = $first + $second - 2*$first*$second;

	return $result;
}
################################################################


################################################################
#			getCMfromTheta
#	Gets centiMorgans from Theta
#
#	Parameters:
#					$_[0] - theta
#	Returns:
#					cM
################################################################
sub getCMfromTheta( $ )
{
	my $theta = $_[0];
	
	if ( $theta >= 0.5 )
	{
		$cM = 100000000;
	}
	elsif ( $theta <= -0.5 )
	{
		$cM = -100000000;
	}
	else
	{
		$cM = -50 * ( log( 1 - 2*$theta ) ) / ( log ( exp(1) ) ) if ( $theta >= 0 );
		$cM = 50 * ( log( 1 + 2*$theta ) ) / ( log ( exp(1) ) ) if ( $theta < 0 );
	}
	&debug_print("getCMfromTheta: theta=($theta), cM = ($cM)" );
	
	return sprintf( "%.16f",  $cM  );
}
################################################################



################################################################
#			getThetaFromCM
#	Gets Theta from centiMorgans
#
#	Parameters:
#					$_[0] - cM
#	Returns:
#					Theta
################################################################
sub getThetaFromCM( $ )
{
	my $cM = $_[0];
	
	my $theta = 0.5*( 1 - exp( -2*$cM / 100 ) ) if ( $cM >= 0 );
	$theta = -0.5*( 1 - exp( 2*$cM / 100 ) ) if ( $cM < 0 );
	&debug_print("getThetaFromCM: cM = ($cM), theta=($theta)" );

	return sprintf( "%.16f",  $theta  );
}
################################################################



################################################################
# 				getComplexityLevel
#	Returns the complexity level of such number of multiplications
#
#	Parameters:
#					$_[0] - Number of multiplications
#					$_[1] - Hash with variables
#	Returns:
#					Complexity level
################################################################
sub getComplexityLevel( $ \% )
{
	my $mult = $_[0];
	%variables = %{ $_[1] };
	&debug_print("Getting Complexity level for complexity ($mult) ");
	
	return $UNCALCULATED_INFORMATION if ( $mult =~ $UNCALCULATED_INFORMATION );
	return "Too Hard" if ( $mult =~ $SKIPPED_INFORMATION );
	
	%variables ={ &makeSystemError("Failed reading Complexity Conversion table from file ($RUN_DIR\/$COMPLEXITY_CONVERSION_TABLE_FILE)", \%variables )}
		unless ( open TBL, "$RUN_DIR\/$COMPLEXITY_CONVERSION_TABLE_FILE" );
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	my @levels = <TBL>;
	close TBL;

	foreach $level (@levels)
	{
		if ( $level =~ /$mult/ )
		{
			@words = split ' ', $level;
			return $words[1];
		}
	}
	return $UNCALCULATED_INFORMATION;		#	NO SUCH COMPLEXITY LEVEL
}
################################################################



################################################################
#			runEachMarker
#	Run each marker with "jump" points in the middle and offset "offmap"
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Locus number
#	Returns:
#					Array with points to run
################################################################
sub runEachMarker( \% $ )
{
	%variables = %{ $_[0] };
	my $locus = $_[1];
	my @result = ();
	my $from = $variables{ "from".$locus };
	my $to = $variables{ "to".$locus };
	my $offmap = $variables{ "offmap".$locus };
	my $jump = $variables{ "jump".$locus };
	&debug_print("Running each marker For Locus ($locus) from ($from) to ($to)".
					"with jumps ($jump) and offset ($offmap)");
	
	my ( $chrom, $dist ) = &getDistTillMarker( $from, \%variables );
	for ( ; $chrom <= ( $#{ $variables{ chromosomes } } + 1 ); )
	{
		
		@result = @{ &pushOffMap( $from, -$offmap, $jump, \@result, \%variables ) } if ( $offmap > 0 );
		
		push @result, ( $chrom, $dist );
		&debug_print("Before iterations result array is (@result), from=($from), to=($to)");
		for( ; $from < $to; $from ++ )
		{
			if ( ! defined ${ $variables{ thetas } }[ $from-1 ] )
			{	#	TwoLocus.MultiPoint Linked - miss last theta
				$from ++;
				$dist = 0;
				last;
			}
			if ( abs( ${ $variables{ thetas } }[ $from-1 ] ) >= 0.49999 )
			{
				&debug_print("Chromosome finished, going to next");
				$from ++;
				$dist = 0;
				last;
			}
			next if ( ${ $variables{ thetas } }[ $from-1 ] == 0 );
			
			$cM = &getCMfromTheta( ${ $variables{ thetas } }[ $from-1 ] );
			
			if ( $variables{ "jump".$locus } =~ /one/ )
			{
				push @result, ( $chrom, $dist + $cM/2 );
				push @result, ( $chrom, $dist + $cM );
			}
			elsif ( $variables{ "jump".$locus } =~ /two/ )
			{
				push @result, ( $chrom, $dist + $cM/3 );
				push @result, ( $chrom, $dist + 2*$cM/3 );
				push @result, ( $chrom, $dist + $cM );
			}
			else
			{
				push @result, ( $chrom, $dist + $cM );
			}
			&debug_print("After this iteration result array is (@result)");
			$dist += $cM;
		}
		
		if ( $offmap > 0 )
		{
			if ( $variables{ program } !~ /TwoLocus/ )
			{
				if ( ( $chrom == 1 ) && ( $#{ $variables{ chromosomes } } > 0 )  )
				{
					@result = @{ &pushOffMap( $from-1, $offmap,  $jump, \@result, \%variables ) };
				}
				else
				{
					@result = @{ &pushOffMap( $from, $offmap,  $jump, \@result, \%variables ) };
				}
			}
			else
			{
				@result = @{ &pushOffMap( $from-1, $offmap,  $jump, \@result, \%variables ) };
			}
		}	
		
		last if (  $from == $to );
		$chrom ++;
		&debug_print("chrom=($chrom), up to (".( $#{ $variables{ chromosomes } } + 1 ).")");
	}
	return \@result; 
}
################################################################



################################################################
#			runOnPoint
#	Prepare the files to one point run
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Locus number
#	Returns:
#					Array with points to run
################################################################
sub runOnPoint( \% $ )
{
	%variables = %{ $_[0] };
	my $locus = $_[1];
	my $from = $variables{ "from".$locus };
	my $jump = $variables{ "jump".$locus };
	&debug_print("Running on point For Locus ($locus) with jump=($jump) from marker=($from)");
	my @result = ();
	my ( $chrom, $cM ) = &getDistTillMarker( $from, \%variables );

	push @result, ( $chrom, $cM + $jump );
	&debug_print("The result array is (@result)");

	return \@result; 
}
################################################################



################################################################
#			runBetweenTwoMarkers
#	Calculate one point between marker "from" to marker "to"
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Locus number
#	Returns:
#					Array with points to run
################################################################
sub runBetweenTwoMarkers( \% $ )
{
	%variables = %{ $_[0] };
	my $locus = $_[1];
	my @result = ();
	my $from = $variables{"from".$locus};
	my $to = $variables{"to".$locus};

	&debug_print("Calculating middle point For Locus ($locus) between marker ($from) and ($to)");
	
	my $middle = 0;
	my $dist = $to - $from;
	my ( $chrom, $cm ) = &getDistTillMarker( $from, \%variables );

	while ( $dist > 0 )
	{
		$dist -= 1;
		%variables = %{&makeInputError("Wrong middle point!!!", \%variables)}
			if ( abs( ${ $variables{ thetas } }[ $from + $dist - 1 ] ) >= 0.49999 );
		exit $ERROR if ( $NO !~ $variables{ errorExist } );

		next if ( ${ $variables{ thetas } }[ $from + $dist - 1 ] == 0 );
		$middle += &getCMfromTheta( ${ $variables{ thetas } }[ $from + $dist - 1 ] );
	}
	$middle /= 2;
	&debug_print("Middle Cm between FROM and TO is (".( $cm + $middle ).")");

	push @result, ( $chrom, $cm + $middle );
	&debug_print("The result array is (@result)");
	
	return \@result; 
}
################################################################



################################################################
#			customRun
#	Run each marker between marker "from" to marker "to" with jump "jump" with offset "offmap"
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Locus number
#	Returns:
#					Array with points to run
################################################################
sub customRun( \% $ )
{
	%variables = %{ $_[0] };
	my $locus = $_[1];

	my @result = ();
	my $from = $variables{"from".$locus};
	my $to = $variables{"to".$locus};
	my $jump = $variables{ "jump".$locus };
	my $offmap = $variables{ "offmap".$locus };

	&debug_print("Running each marker from ($from) to ($to) with jumps ($jump) and offset ($offmap)");
	
	my ( $chrom, $cm ) = &getDistTillMarker( $from, \%variables );
	@result = @{ &pushOffMap( $from, -$offmap, $jump, \@result, \%variables ) } if ( $offmap > 0 );
	
	if ( $#result > 0 )
	{
		my $step = $chrom - $result[0];
		for( my $i=0; $i < $#result; $i += 2 )
		{
			$result[$i] += $step;
		}
		$chrom = $result[ $#result - 1 ];
	}
	
	push @result, ( $chrom, $cm );
	&debug_print("Pushed ( $chrom, $cm ) for the first marker");

	while( $from < $to )
	{
		if ( abs( ${ $variables{ thetas } }[ $from - 1 ] ) >= 0.4999 )
		{
			$chrom ++;
			$cm = 0;
			push @result, ( $chrom, $cm );
			next;
		}
		next if ( ${ $variables{ thetas } }[ $from - 1 ] == 0 );

		my $dist = &getCMfromTheta( ${ $variables{ thetas } }[ $from - 1 ] );
		%variables = %{&makeInputError("Input error: There is no distance betweet marker ($from) and marker (".($from+1).")", \%variables)}
			unless defined $dist;
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
		&debug_print("Working on distance ($dist) between marker ($from) and (".($from+1).")");
		my $added = 0;
		
		while ( $added + $jump < $dist )
		{
			$added += $jump;
			$cm += $jump;
			push @result, ( $chrom, $cm );
			&debug_print("Pushed ( $chrom, $cm )");
		}
		
		$dist -= $added;
		$cm += $dist;
		push @result, ( $chrom, $cm ) if ( $dist > 1e-10 );
		&debug_print("Pushed ( $chrom, $cm )") if ( $dist > 1e-10 );
		
		$from ++;
	}
	
	@result = @{ &pushOffMap( $from, $offmap, $jump, \@result, \%variables ) } if ( $offmap > 0 );

	return \@result; 
}
################################################################



################################################################
#			jumpsRun
#	Mode 5: Run each jumps from position on marker "from" until distance "to" with jumps "jump"
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Locus number
#	Returns:
#					Array with points to run
################################################################
sub jumpsRun( \% $ )
{
	%variables = %{ $_[0] };
	my $locus = $_[1];

	my @result = ();
	my $from = $variables{"from".$locus};
	my $to = $variables{"to".$locus};
	my $jump = $variables{ "jump".$locus };

	&debug_print("Running jumps marker from position on marker ($from) until distance ($to) with jumps ($jump) cM");
	
	my ( $chrom, $cm ) = &getDistTillMarker( $from, \%variables );
	@result = @{ &pushOffMap( $from, -$offmap, $jump, \@result, \%variables ) } if ( $offmap > 0 );
	
	if ( $#result > 0 )
	{
		my $step = $chrom - $result[0];
		for( my $i=0; $i < $#result; $i += 2 )
		{
			$result[$i] += $step;
		}
		$chrom = $result[ $#result - 1 ];
	}
	
	my $counter = 0;
	push @result, ( $chrom, $cm );
	&debug_print("Pushed ( $chrom, $cm ) for the first marker");

	while( $counter < $to )
	{
		$counter += $jump;
		my $cm = &getCMfromTheta( $counter );
		push @result, ( $chrom, $cm );
		&debug_print("Pushed ( $chrom, $cm )");
	}

	return \@result; 
}
################################################################



################################################################
#			pushOffMap
#	Update point array with offmap points
#
#	Parameters:
#					$_[0] - Start Position
#					$_[1] - Offmap
#					$_[2] - Jump
#					$_[3] - Array with points
#					$_[4] - Hash with variables
#	Returns:
#					Updated array
################################################################
sub pushOffMap( $ $ $ \@ \% )
{
	my $marker = $_[0];
	my $off = $_[1];
	my $jump = $_[2];
	my @result = @{ $_[3] };
	%variables = %{ $_[4] };
	&debug_print("Push Off MAP with OFFSET = ($off) and JUMP = ($jump) from marker=($marker)");

	my ( $chrom, $cm ) = &getDistTillMarker( $marker, \%variables );

	if ( $jump =~ /one/ )
	{
		push @result, ( $chrom, $cm + $off ) if ( $off < 0 );
		push @result, ( $chrom, $cm + $off/2 );
		push @result, ( $chrom, $cm + $off ) if ( $off > 0 );
	}
	elsif ( $jump =~ /two/ )
	{
		if ( $off < 0 )
		{
			push @result, ( $chrom, $cm + $off );
			push @result, ( $chrom, $cm + 2*$off / 3 );
			push @result, ( $chrom, $cm + $off / 3 );
		}
		else
		{
			push @result, ( $chrom, $cm + $off / 3 );
			push @result, ( $chrom, $cm + 2*$off / 3 );
			push @result, ( $chrom, $cm + $off );
		}
	}
	elsif ( $jump !~ /zero/ )
	{
		if ( $off < 0 )
		{
			while ( $off < 0 )
			{
				push @result, ( $chrom, $cm + $off );
				$off += $jump;
			}
		}
		else
		{
			while ( $off > 0 )
			{
				$cm += $jump;
				push @result, ( $chrom, $cm );
				$off -= $jump;
			}
		}
	}

	&debug_print("Updated array is (@result)");
	return \@result;
}
################################################################




################################################################
#			getDistTillMarker
#	Get distance from the first marker in data till the first marker in calculation ( $variables{from} )
#
#	Parameters:
#					$_[0] - Marker number
#					$_[1] - Hash with variables
#	Returns:
#					Distance
################################################################
sub getDistTillMarker( $ \% )
{
	my $target = $_[0];
	%variables = %{ $_[1] };
	&debug_print("Calculating Distance from First marker to First Calculated marker=($target)");

	my $dist = 0;
	my $count = 0;
	my $marker = ${ $variables{ usedMarkersId } }[ $count ];
	my $chrom = 1;
	&debug_print("Started from marker ($marker), used markers are (@{ $variables{ usedMarkersId } }), looking for serial ($target)");
	
	%variables = %{&makeInputError("Wrong start point!!!", \%variables)}
		if ( $count > $target-1 );
	exit $ERROR if ( $NO !~ $variables{ errorExist } );

	while ( $count < $target-1 )
	{
		&debug_print("Count = ($count), dist=($dist), marker=($marker), target=($target)");
		%variables = %{&makeInputError("Wrong start point, no point number ($count)!!!", \%variables)}
			unless defined ${ $variables{ usedMarkersId } }[ $count ];

#		%variables = %{&makeInputError("Wrong theta values, no theta number (".(${ $variables{ usedMarkersId } }[ $count ] - 1).")!!!", \%variables)}
#			unless defined ${ $variables{ thetas } }[   ${ $variables{ usedMarkersId } }[ $count ] - 1 ];

		&debug_print("Started from marker ($marker), thetas are (@{ $variables{ thetas } }), count is $count");
		%variables = %{&makeInputError("Wrong theta values, no theta number (".(${ $variables{ usedMarkersId } }[ $count ] - 1).")!!!", \%variables)}
			unless defined ${ $variables{ thetas } }[  $count ];
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
#		my $theta = ${ $variables{ thetas } }[   ${ $variables{ usedMarkersId } }[ $count ] - 1 ];
		my $theta = ${ $variables{ thetas } }[  $count  ];

		if ( abs($theta) >= 0.4999 )
		{
			$dist = 0;
			$chrom ++;
		}
		else
		{
			$dist += &getCMfromTheta($theta );
		}
		
		$count ++;
#		$marker = ${ $variables{ usedMarkersId } }[ $count ];
		$marker ++;
	}
	&debug_print("Cm till marker FROM is ($dist) ");
	
	return ( $chrom, $dist );
}
################################################################


################################################################
#			getDistFromStart
#	Get distance from the first marker in data 
#
#	Parameters:
#					$_[0] - Marker number
#					$_[1] - Datafile
#	Returns:
#					Distance
################################################################
sub getDistFromStart( $ )
{
	my $marker = $_[0];
	my $datafile = $_[1];
	
	&debug_print ( "Error opening $RUN_DIR\/$datafile")
		unless (open DAT, "$RUN_DIR\/$datafile"); 

	@data = <DAT>;
	close DAT;
	
	my @words_first_line = split ' ', $data[0];
	$total_markers = $words_first_line[0] - 1;
	
	# Get to penetrances
	for $l (1..7) {
		$line = shift @data;
	}
	
	# Get to first marker line
	while ($line !~ /^3/) {
		$line = shift @data;
	}
	
	# Now we have the first marker line
	$line = shift @data; # GENE FREQUENCIES - for the first marker
	
	for $l (2..$total_markers) {
		$line = shift @data; # ALLELE NUMBERS, NO. OF ALLELES 
		$line = shift @data; # GENE FREQUENCIES
	}
	
	$line = shift @data; # SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	$line = shift @data; # RECOMBINATION VALUES
	
	my @words = split ' ', $line;
	
	%variables = %{&makeInputError("Wrong marker value, no marker number ($marker)!!!", \%variables)}
			unless defined $words[$marker];
	
	$words[0] = 0; # The first recombination value has no meaning
	
	my $dist = 0;
	for( my $count = 0; $count <= $marker; $count ++ )
	{
		$dist += &getCMfromTheta($words[$count]);
	}
	
	return $dist;
}
################################################################

################################################################
#			getName
#	Get name of the marker in the datafile 
#
#	Parameters:
#					$_[0] - Marker number
#	Returns:
#					Name
################################################################
sub getName( $ )
{
	my $marker = $_[0];
	
	$datfile = "$RUN_DIR\/datafile.dat";
	
	if (! -e "$RUN_DIR/multiParams.dat") {  #then we got a list of markers
		$datfile = "$RUN_DIR\/datafile_1_0/pedfile_0/datafile.dat";
	}
	
	&debug_print ( "Error opening $RUN_DIR\/datafile.dat")
		unless (open DAT, $datfile); 	
		
	@data = <DAT>;
	close DAT;
	my $line = shift @data;
	while ( ( $line !~ /ALLELE NUMBERS, NO. OF ALLELES/ ) && ( $#data >= 0) )
	{
		$line = shift @data;
	}
	
	if ( open IN, "$RUN_DIR/multiParams.dat" )
	{
		my @multiData = <IN>;
		close IN;

		my @words = split ' ', $multiData[0];
		my $start = $words[0];		
		
		if ($start > 1) {
			for( my $i = 0; $i < $start - 1; $i ++ ) {
				$line = shift @data;		#	 << GENE FREQUENCIES
				$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
			}
		}
	}
	
	for( my $i = 0; $i < $marker; $i ++ )
	{
		$line = shift @data;		#	 << GENE FREQUENCIES
		$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
	}
	
	my @words = split '#', $line;
	return $words[1]; 	
	
}
################################################################

################################################################
#			multiPointDivision
#	Divides the Multi Point input to files
#
#	Parameters:
#					$_[0] - Datafiles array
#					$_[1] - Pedfiles array
#					$_[2] - hash with variables
#					$_[3] - Chromosome number
#
#	Returns:
#					SUCCESS or ERROR
################################################################
sub multiPointDivision( \@ \@ \% $ )
{
	my @datafiles = @{ $_[0] };
	my @pedfiles = @{ $_[1] };
	%variables = %{ $_[2] };
	my $chromosome = $_[3];
	&debug_print ( "multiPointDivision");

	%variables = %{&makeSystemError("Failed creating directory ($variables{ newDir }\/pedfiles)", \%variables)}
		unless ( -e "$variables{ newDir }\/pedfiles" || mkdir "$variables{ newDir }\/pedfiles" );
	%variables = %{&makeSystemError("Failed creating directory ($variables{ newDir }\/datafiles)", \%variables)}
			unless ( -e "$variables{ newDir }\/datafiles" || mkdir "$variables{ newDir }\/datafiles" );
	%variables = %{&makeSystemError("Failed changing to project directory ($variables{ newDir })", \%variables)}
		unless ( chdir $variables{ newDir } );
	
	%variables = %{&makeSystemError("Failed to open datafile ($variables{ datafile })", \%variables)}
		unless ( open DATA, $variables{ datafile } );
	return $ERROR if ( $NO !~ $variables{ errorExist } );	
	
	my @lines = <DATA>;
	close DATA;
	my $data = join "\n",@lines;
	
	%variables = %{&makeSystemError("Failed changing to directory (datafiles)", \%variables)}
		unless ( chdir "datafiles" );
	return $ERROR if ( $NO !~ $variables{ errorExist } );	
	
	for( my $count = 0; $count <= $#datafiles; $count ++ )
	{
		$datafiles[ $count ] = &changeLociAndCode($datafiles[ $count ], 6, \%variables) if ( $variables{ program } !~ /TwoLocus/ );
		$datafiles[ $count ] = &changeLociAndCode($datafiles[ $count ], 6, \%variables) if ( $variables{ program } =~ /TwoLocus/ );
		return $ERROR if ( $ERROR eq $datafiles[ $count ] );
		return $ERROR if ( $ERROR =~ &saveToFile( "datafile_Chr".$chromosome."_point".($count).".dat",$datafiles[ $count ], \%variables ) );
	}
	%variables = %{&makeSystemError("Failed changing back to project directory ($variables{ newDir })", \%variables)}
		unless ( chdir ".." );
	%variables = %{&makeSystemError("Failed to open pedfile ($variables{ pedfile })", \%variables)}
		unless ( open DATA, $variables{ pedfile } );
	return $ERROR if ( $NO !~ $variables{ errorExist } );	
	
	@lines = <DATA>;
	close DATA;
	my $ped = join "\n",@lines;
	
	%variables = %{&makeSystemError("Failed changing to directory (pedfiles)", \%variables)}
		unless ( chdir "pedfiles" );
	return $ERROR if ( $NO !~ $variables{ errorExist } );	
	
	for( $count = 0; $count <= $#pedfiles; $count ++ )
	{
		return $ERROR if ( $ERROR =~ &saveToFile( "pedfile_Chr".$chromosome."_family".($count+1).".dat",$pedfiles[ $count ], \%variables ) );
	}
	%variables = %{&makeSystemError("Failed changing back to project directory ($variables{ newDir })", \%variables)}
		unless ( chdir ".." );
	return $ERROR if ( $NO !~ $variables{ errorExist } );	
	
	for( my $point = 0; $point <= $#datafiles; $point ++ )
	{
		my $dataDir = "datafile_".$chromosome."_".$point;
		%variables = %{&makeSystemError("Failed creating directory ($dataDir)", \%variables)}
			unless (  -e  $dataDir || mkdir $dataDir );
		%variables = %{&makeSystemError("Failed changing to directory ($dataDir)", \%variables)}
			unless ( chdir $dataDir );
		return $ERROR if ( $NO !~ $variables{ errorExist } );	
		
		for( my $fam = 0; $fam <= $#pedfiles; $fam ++ )
		{
			my $pedDir = "pedfile_$fam";
			&debug_print ( "Creating directory: $pedDir" );
			%variables = %{&makeSystemError("Failed creating directory ($pedDir)", \%variables)}
				unless (  -e  $pedDir || mkdir $pedDir);
			%variables = %{&makeSystemError("Failed changing to directory ($pedDir)", \%variables)}
				unless (chdir $pedDir);
			return $ERROR if ( $NO !~ $variables{ errorExist } );
			
			%variables = %{&makeLink( "../../datafiles/datafile_Chr".$chromosome."_point".($point).".dat", "datafile.dat", \%variables )}
				unless ( -e "datafile.dat" );
			%variables = %{&makeLink( "../../pedfiles/pedfile_Chr".$chromosome."_family".($fam+1).".dat", "pedfile.dat", \%variables )}
				unless (  -e "pedfile.dat" );
			&debug_print ( "Datafile & Pedfile creation in directory ($dataDir\/$pedDir) was successfull" );
				
			%variables = %{&makeSystemError("Failed come back to directory ($dataDir)", \%variables )}
				unless ( chdir ".." );
			return $ERROR if ( $NO !~ $variables{ errorExist } );
		}
	
		%variables = %{&makeSystemError("Failed come back to project directory ($variables{ newDir })", \%variables )}
			unless ( chdir ".." );
		return $ERROR if ( $NO !~ $variables{ errorExist } );
	}
	
	&debug_print ( "Directories creation FINISHED with SUCCESS" );
	%variables = %{&makeSystemError("Failed come back to project directory ($variables{ newDir })", \%variables )}
		unless ( chdir $RUN_DIR );
	return $ERROR if ( $NO !~ $variables{ errorExist } );
	return $SUCCESS;
}
################################################################


################################################################
# 				sub initPoint()
#	Init Hash with distances
#
#	Parameters:
#					$_[0] - Program
#					$_[1] - Number of LODs
#	Returns:
#					Hash with point info
################################################################
sub initPoint( $ $ )
{
	my $program = $_[0];
	my $LODs = $_[1];
	&debug_print( "Init Point with prorgam=($program) with # of LODs=($LODs)" );

	%point_hash = ();
	$point_hash{ CHROM } = $UNCALCULATED_INFORMATION;
	$point_hash{ DIST } = $UNCALCULATED_INFORMATION;
	$point_hash{ NAME } = $UNCALCULATED_INFORMATION;
	$point_hash{ DIST1 } = $UNCALCULATED_INFORMATION;
	$point_hash{ NAME1 } = $UNCALCULATED_INFORMATION;
	$point_hash{ DIST2 } = $UNCALCULATED_INFORMATION;
	$point_hash{ NAME2 } = $UNCALCULATED_INFORMATION;
	$point_hash{ PROGRAM } = $program;

	for ( my $l = 0; $l < $LODs; $l ++ )
	{
		${ $point_hash{ LN } }[$l] = $UNCALCULATED_INFORMATION;
		${ $point_hash{ LOD } }[$l] = $UNCALCULATED_INFORMATION;
		${ $point_hash{ MULT } }[$l] = $UNCALCULATED_INFORMATION;
		${ $point_hash{ CMPLX } }[$l] = $UNCALCULATED_INFORMATION;
		${ $point_hash{ FINISHED } }[$l] = $YES;
	}
	
	return \%point_hash;
}
################################################################




################################################################
# 				initTotalLOD
#	Init Array of total calculation result
#
#	Parameters:
#					$_[0] - Number of points
#					$_[1] - Hash with variables
#	Returns:
#					Initialized Array
################################################################
sub initTotalLOD( $ $ \% )
{
	my $points = $_[0];
	my $LODs = $_[1];
	%variables = %{ $_[2] };
	&debug_print( "Multi Point Init Total LOD with ($points) points" );
	
	my @total = ();
	for ( my $point = 0; $point < $points; $point ++ )
	{
		%{ $total[ $point ] } = %{ &initPoint($variables{ program }, $LODs ) };
	}
	
	return \@total;
}
################################################################




################################################################
# 				calculatePoint
#	Init Hash with distances
#
#	Parameters:
#					$_[0] - Hash with asked point Info
#					$_[1] - Hash with unlinked information
#	Returns:
#					Hash with total point LOD info
################################################################
sub calculatePoint( \% \% )
{
	my %point = %{ $_[0] };
	my %unlinked = %{ $_[1] };
	&debug_print( "Calculate Point LOD Information" );

	%point_LOD = ();

	foreach my $k ( keys %point )
	{
		next if ( $k =~ /LN|LOD/ );
		$point_LOD{ $k } = $point{ $k };
	}
	@{ $point_LOD{ LN } } = ();
	@{ $point_LOD{ LOD } } = ();

	for ( $p=0; $p <= $#{ $unlinked{ LOD } }; $p ++ )
	{
		if ( ( ${ $point{ LOD } }[$p] eq $INFINITY ) || ( ${ $unlinked{ LOD } }[$p] eq $INFINITY ) )
		{
			${ $point_LOD{ LN } }[$p] = $INFINITY;
			${ $point_LOD{ LOD } }[$p] = $INFINITY;
			next;
		}
		if ( ( ${ $point{ LOD } }[$p] eq $LINKED	) || ( ${ $unlinked{ LOD } }[$p] eq $LINKED ) )
		{
			${ $point_LOD{ LN } }[$p] = $LINKED;
			${ $point_LOD{ LOD } }[$p] = $LINKED;
			next;
		}
		if ( ( ${ $point{ LOD } }[$p] =~ /$SKIPPED_INFORMATION/ ) || ( ${ $unlinked{ LOD } }[$p] =~ /$SKIPPED_INFORMATION/ ) )
		{
			${ $point_LOD{ LN } }[$p] = $SKIPPED_INFORMATION;
			${ $point_LOD{ LOD } }[$p] = $SKIPPED_INFORMATION;
			next;
		}
		if ( ( ${ $point{ LOD } }[$p] =~ /$UNCALCULATED_INFORMATION/ ) || ( ${ $unlinked{ LOD } }[$p] =~ /$UNCALCULATED_INFORMATION/ ) )
		{
			${ $point_LOD{ LN } }[$p] = $UNCALCULATED_INFORMATION;
			${ $point_LOD{ LOD } }[$p] = $UNCALCULATED_INFORMATION;
			next;
		}

		if ( ( ${ $point{ LOD } }[$p] =~ /$ERROR_IN_COMPUTATION/ ) || ( ${ $unlinked{ LOD } }[$p] =~ /$ERROR_IN_COMPUTATION/ ) )
		{
			${ $point_LOD{ LN } }[$p] = $ERROR_IN_COMPUTATION;
			${ $point_LOD{ LOD } }[$p] = $ERROR_IN_COMPUTATION;
			next;
		}
		${ $point_LOD{ LN } }[$p] = sprintf( "%.4f",  ${ $point{ LN } }[$p] );
		${ $point_LOD{ LOD } }[$p] = sprintf( "%.4f",  ( ${ $point{ LOD } }[$p] - ${ $unlinked{ LOD } }[$p] )  );
	}

	return \%point_LOD;
}
################################################################




################################################################
# 				updateTotalLOD
#	Updates total LODscores with family LODscores
#
#	Parameters:
#					$_[0] - Array with family LODscores
#					$_[1] - Array with total LODscores in the same format - look at &initTotalLOD( $ $ \% )
#	Returns:
#					Updated Array
################################################################
sub updateTotalLOD( \@ \@ \% )
{
	my @family = @{ $_[0] };
	my @total = @{ $_[1] };
	%variables = %{ $_[2] };
	&debug_print( "Update Total LOD, size of total=($#total), size of family=($#family)" );

	@total =  @{ &initTotalLOD( scalar @family, scalar @{ ${ $family[0] }{LOD} }, \%variables ) }
		if ( $#total < 0 );

	for ( my $point = 0; $point <= $#total; $point ++ )
	{
		my %pointHash = %{ $family[ $point ] };

		${ $total[ $point ] }{ CHROM } = $pointHash{ CHROM };

# Distance		
		if ( $pointHash{DIST2} eq $UNCALCULATED_INFORMATION )		#	Single Locus
		{
			${ $total[ $point ] }{DIST} = $pointHash{DIST}
				if ( ${ $total[ $point ] }{DIST} eq $UNCALCULATED_INFORMATION );
			${ $total[ $point ] }{NAME} = $pointHash{NAME}
				if ( ${ $total[ $point ] }{NAME} eq $UNCALCULATED_INFORMATION );
		}
		else														#	Two Locus
		{	
			${ $total[ $point ] }{DIST1} = $pointHash{DIST1}
				if ( ${ $total[ $point ] }{DIST1} eq $UNCALCULATED_INFORMATION );
			${ $total[ $point ] }{NAME1} = $pointHash{NAME1}
				if ( ${ $total[ $point ] }{NAME1} eq $UNCALCULATED_INFORMATION );
		
			${ $total[ $point ] }{DIST2} = $pointHash{DIST2}
				if ( ${ $total[ $point ] }{DIST2} eq $UNCALCULATED_INFORMATION );
			${ $total[ $point ] }{NAME2} = $pointHash{NAME2}
				if ( ${ $total[ $point ] }{NAME2} eq $UNCALCULATED_INFORMATION );
		}

# LN & LOD
		for ( my $p = 0; $p <= $#{ $pointHash{LOD} }; $p ++ )
		{
			next if ( $NO == ${ ${ $total[ $point ] }{FINISHED} }[$p] );
			
			if ( ${$pointHash{LOD}}[$p] eq $UNCALCULATED_INFORMATION ) 
			{
				${ ${ $total[ $point ] }{LN} }[$p] = $UNCALCULATED_INFORMATION;
				${ ${ $total[ $point ] }{LOD} }[$p] =	$UNCALCULATED_INFORMATION;
				${ ${ $total[ $point ] }{FINISHED} }[$p] = $NO;			
				next;
			}
	
			if ( ( ${ ${ $total[ $point ] }{LOD} }[$p] eq $INFINITY ) 
				|| ( ${ ${ $total[ $point ] }{LOD} }[$p] eq $SKIPPED_INFORMATION )
				|| ( ${ ${ $total[ $point ] }{LOD} }[$p] eq $LINKED ) )
			{
				next;
			}
	
			if ( ( ${$pointHash{LOD}}[$p] eq $SKIPPED_INFORMATION ) || ( ${$pointHash{LOD}}[$p] eq $LINKED ) )
			{
				${ ${ $total[ $point ] }{LN} }[$p] = ${$pointHash{LN}}[$p];
				${ ${ $total[ $point ] }{LOD} }[$p] =	${$pointHash{LOD}}[$p];
				next;
			}
	
			if ( ${ ${ $total[ $point ] }{LOD} }[$p] eq $UNCALCULATED_INFORMATION )
			{
				${ ${ $total[ $point ] }{LN} }[$p] = ${$pointHash{LN}}[$p];
				${ ${ $total[ $point ] }{LOD} }[$p] =	${$pointHash{LOD}}[$p];
				next;
			}
	
			${ ${ $total[ $point ] }{LN} }[$p] += ${$pointHash{LN}}[$p];
			${ ${ $total[ $point ] }{LOD} }[$p] += ${$pointHash{LOD}}[$p];
		}
	}
	return \@total;
}
################################################################





################################################################
# 				createTotalResult
#	Sums all the family results into one file
#
#	Parameters:
#					$_[0] - Total LODs array. For format look in &initTotalLOD( $ $ \% )
#					$_[1] - Output LOD file
#					$_[2] - Output CMPLX file
#					$_[3] - Hash with variables
#	Returns:
#					NOTHING
################################################################
sub createTotalResult( \@ $ $ \% )
{
	my @totalLOD = @{ $_[0] };
	my $outputLODfile = $_[1];
	my $outputCMPLXfile = $_[2];
	%variables = %{ $_[3] };
	&debug_print( "createTotalResult with LOD file ($outputLODfile), CMPLX file ($outputCMPLXfile), size of totalLOD is ($#totalLOD)" );
	
	%variables = %{&makeSystemError("Failed creating output file ($outputLODfile)", \%variables )}
		unless (open LOD, ">$outputLODfile");
	%variables = %{&makeSystemError("Failed creating output file ($outputCMPLXfile)", \%variables )}
		unless (open CMPLX, ">$outputCMPLXfile");
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	
	my $prev1 = -100000;
	my $prev2 = -100000;
	my $rows = 0;
	my $outputLOD = "";
	my $outputCMPLX = "";
	my $firstDist = "DIST1";
	$firstDist = "DIST" if ( $variables{ program } !~ /TwoLocus/ );
	
	my $chrom = 1;
	my $dir = "datafile_".$chrom."_1\/pedfile_0";

	my $headerLODString = &getHeaderLine( "$RUN_DIR\/$LOD_HEADER_FILE", $dir, \%variables );
	my $headerCMPLXString = &getHeaderLine( "$RUN_DIR\/$CMPLX_HEADER_FILE", $dir, \%variables );

	print LOD $headerLODString;
	print CMPLX $headerCMPLXString;
	&debug_print( "LOD & CMPLX file headers were created" );

	my @spanning = ();
	my $rowSpan = 1;
		
	for( my $point = 0; $point <= $#totalLOD; $point ++ ) 
	{
		if ( 0 == $point )
		{
			push @spanning, $rowSpan if ( $#totalLOD == $point );
			$prev2 = ${ $totalLOD[ $point ] }{DIST2} if ( $variables{program} =~ /TwoLocus.MultiPoint/ );
			next;
		}

		if ( ($point < $#totalLOD) && ( $variables{program} =~ /TwoLocus.MultiPoint/ ) 
			&& ( ${ $totalLOD[ $point ] }{$firstDist} == ${ $totalLOD[ $point-1 ] }{$firstDist} )
			&& ( $prev2 < ${ $totalLOD[ $point ] }{DIST2} )
			)
		{
			$rowSpan ++;
		}
		elsif ( ($point < $#totalLOD) && ($variables{program} =~ /TwoLocus.TwoPoint/)
				&&  ( ${ $totalLOD[ $point ] }{NAME1} eq ${ $totalLOD[ $point-1 ] }{NAME1} )
				)
		{
			$rowSpan ++;
		}
		elsif ($point < $#totalLOD)
		{
			push @spanning, $rowSpan;
			$rowSpan = 1;
		}
		elsif ( ( $variables{program} =~ /TwoLocus.MultiPoint/ ) 
			&& ( ${ $totalLOD[ $point ] }{$firstDist} == ${ $totalLOD[ $point-1 ] }{$firstDist} )
			&& ( $prev2 < ${ $totalLOD[ $point ] }{DIST2} )
			)
		{
			push @spanning, $rowSpan + 1;
		}
		elsif ( ($variables{program} =~ /TwoLocus.TwoPoint/) 
				&&  ( ${ $totalLOD[ $point ] }{NAME1} eq ${ $totalLOD[ $point-1 ] }{NAME1} )
				)
		{
			push @spanning, $rowSpan + 1;
		}
		else
		{
			push @spanning, $rowSpan;
			push @spanning, 1;
		}
		$prev2 = ${ $totalLOD[ $point ] }{DIST2} if ( $variables{program} =~ /TwoLocus.MultiPoint/ );
	}
	&debug_print("Table spanning=(@spanning)" );

	$rowSpan = 0;
	$prev2 = -100000;
	my $changed = $spanning[ $rowSpan ];
	my $span = $spanning[ $rowSpan ];
	
	
	my $multiKey = 0;
	
	my $extraString = "";
	my $extra = 0;
	my $extraIndex = -1;
	
	if ( ( $variables{ program } =~ /MultiPoint/ ) && (-e "$RUN_DIR/multiParams.dat")){
		exit &makeSystemError("Failed opening file ($RUN_DIR/multiParams.dat) to read", \%variables )
		unless ( open IN, "$RUN_DIR/multiParams.dat" );

		my @multiData = <IN>;
		close IN;

		my @words = split ' ', $multiData[0];
		$multiKey = $words[2];
				
		my $totalSize = $#totalLOD + 1;
		my $groupNum = int ($totalSize / $multiKey);
		my $endOfFullGroups = $groupNum * $multiKey - 1;
		if (($endOfFullGroups + 1) < $totalSize)
		{
			$extra = $multiKey - ($totalSize - $endOfFullGroups - 1);
		}
		$extraIndex = $endOfFullGroups - $extra + 1;
	}
	
	if ( ( $variables{ program } =~ /MultiPoint/ ) && ( ( ! defined($variables{ jump }) ) || ( $variables{ jump } =~ /Unknown/ ) ) )
	{
		$multiKey = $#{ $variables{ usedMarkersId } } + 1;
	}
			
	for( $point = 0; $point <= $#totalLOD; $point ++ ) 
	{
		&debug_print("Working on point: ($point), prev1=($prev1), prev2=($prev2), Dist1=(${ $totalLOD[ $point ] }{DIST1}), Dist2=(${ $totalLOD[ $point ] }{DIST2}), changed=($changed)" );
		if ( ( ${ $totalLOD[ $point ] }{$firstDist} !~ /TwoPoint/ ) && 
			( ( ($UNCALCULATED_INFORMATION ne ${ $totalLOD[ $point ] }{DIST2}) && ( $prev2 >= ${ $totalLOD[ $point ] }{DIST2} ) )
			&& ( $variables{program} =~ /TwoLocus.MultiPoint/ ) ) )
		{
			if ( ($UNCALCULATED_INFORMATION ne ${ $totalLOD[ $point ] }{$firstDist}) && ( $prev1 >= ${ $totalLOD[ $point ] }{$firstDist} ) )
			{
				&debug_print("For first marker New chromosome started. Prev1=($prev1), Dist1=(${ $totalLOD[ $point ] }{DIST1}),Prev2=($prev2), Dist2=(${ $totalLOD[ $point ] }{DIST2}) " );
		
				print LOD $outputLOD;
				print CMPLX $outputCMPLX;
				
				$chrom ++;
				$dir = "datafile_".$chrom."_1\/pedfile_0";
				
				$headerLODString = &getHeaderLine( "$RUN_DIR\/$LOD_HEADER_FILE", $dir, \%variables );				
				$headerCMPLXString = &getHeaderLine( "$RUN_DIR\/$CMPLX_HEADER_FILE", $dir, \%variables );

				print LOD "</table>\n".$headerLODString;
				print CMPLX "</table>\n".$headerCMPLXString;
	
				$rows = 0;
				$outputLOD = "";
				$outputCMPLX = "";
			}
		}
		$prev1 = ${ $totalLOD[ $point ] }{$firstDist};
		$prev2 = ${ $totalLOD[ $point ] }{DIST2} if ( $variables{ program } =~ /TwoLocus.MultiPoint/ );
		
		$span = 0 if ( $changed < $spanning[ $rowSpan ] );

		my $loci = 1;
		$loci = 2 if ( $variables{ program } =~ /TwoLocus/ );
		my $secKey = $NO;
		my $multiColor = "WHITE";

		$secKey = "LN";
		if ( $variables{ program } =~ /MultiPoint/ ) 
		{
			$multiColor = getRowColor($point, $multiKey);
		}	
		
		$extraString = "";
		#if (($extraIndex != -1) && ($point >= $extraIndex) && ($point < $extraIndex + $extra)){
		#	my $nextGroupNum = int ($point / $multiKey) + 2;
		#	$extraString = "  (Additional to group $nextGroupNum)";			
		#}
		if ($extraIndex != -1) {
			my %info = %{ $totalLOD[ $point ] };
			my $totalSize = $#totalLOD + 1;
			my $groupNum = int ($totalSize / $multiKey);
			if ($info{ CHROM } == $groupNum + 1) { #If it's the last group
				$first = &getMultiMarkerID($extraIndex);
				$last = &getMultiMarkerID($extraIndex + $extra - 1);
				if (! -z "mapfile.dat"){
					$first = "c".$first;
					$last = "c".$last;
				}
				if ($first eq $last) {
					$extraString = "<br/>To this group the marker <br/> $first was added";
				}
				else {
					$extraString = "<br/>To this group markers <br/> $first-$last were added";
				}
			}
		}
		
		$outputLOD = $outputLOD.&printLinetoTable( $totalLOD[ $point ], $loci, $span, "LOD", $secKey, "MULT" , $multiKey, $multiColor, $point, $extraString);
		$secKey = "MULT" if ( $variables{ program } =~ /MultiPoint/ );
		$outputCMPLX = $outputCMPLX.&printLinetoTable( $totalLOD[ $point ], $loci, $span, "CMPLX", $secKey ,"MULT" , $multiKey, $multiColor, $point, $extraString);
	
		$rows++;
		$changed--;
		if ( 0 == $changed )
		{
			$rowSpan++;
			$changed = $spanning[ $rowSpan ];
			$span = $spanning[ $rowSpan ];
		}
	}
	print LOD $outputLOD."</table>\n";
	print CMPLX $outputCMPLX."</table>\n";
	
	close LOD;
	close CMPLX;
}
################################################################



################################################################
# 				getHeaderLine
#	Prints headline of table of marker
#
#	Parameters:
#					$_[0] - Header Template
#					$_[1] - Calculation Directory 
#					$_[2] - Hash with variables
#	Returns:
#					String with header
################################################################
sub getHeaderLine( $ $ \% )
{
	my $headerFile = $_[0];
	my $workDir = $_[1];
	%variables = %{$_[2]};
	&debug_print("Getting getHeaderLine with template=($headerFile) from dir=($workDir)");
	
	%variables = %{&makeSystemError("Failed reading from file ($headerFile)", \%variables )}
		unless (open TEMPLATE, $headerFile );
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	
	my @header = <TEMPLATE>;
	close TEMPLATE;
	my $headerString = join "\n", @header;

	my $firstName = &getPointMarker( $workDir, 0, 1 );
	$headerString =~ s/\#1/$firstName/;
	if ( $variables{ program } =~ /TwoLocus.MultiPoint/ )
	{
		my $secondName = &getPointMarker( $workDir, 0, 2 );
		$headerString =~ s/\#1/$secondName/;
	}
	
	return $headerString;
}
################################################################



################################################################
# 				getLIKEfromResultFile
#	Prints headline of table of marker
#
#	Parameters:
#					$_[0] - File with result ( for one point calculation )
#					$_[1] - Hash with variables
#	Returns:
#					Info hash - look for format in the &initPoint()
################################################################
sub getInfoFromResultFile( $ \% )
{
	my $resultFile = $_[0];
	%variables = %{$_[1]};
	&debug_print("Getting Likelihood from local file ($resultFile)");
	
	my %result = %{ &initPoint($variables{ program }, 1) };
	%variables = %{&makeSystemError("Failed reading from result file ($resultFile)", \%variables )}
		unless (open RES, $resultFile);
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	${ $result{ MULT } }[0] = 0;
	foreach my $line ( <RES> )
	{
		if ( $line =~ /The complexity coefficient of the elimination order found/ )
		{
#			Line format is:
#	The complexity coefficient of the elimination order found for pedigre #1 is 4 .

			my @words = split /\s+/, $line;
			${ $result{ MULT } }[0] = $words[$#words-1];
		}
		if ( $line =~ /TOTAL/ )
		{
			@words = split /\s+/, $line;
			
			${$result{ LN }}[0] = $words[1];
			${$result{ LOD }}[0] = $words[2];
		}
	}
	close RES;
	&debug_print("Complexity=(${ $result{ MULT }}[0]), LN=(${$result{ LN }}[0]), LOG=(${$result{ LOD }}[0]) ");
	
	return \%result;
}
################################################################



################################################################
# 				getPointDist
#	Returns the distance from the first marker in the current calculation
#
#	Parameters:
#					$_[0] - directory name
#	Returns:
#					Array with Distance(s)
################################################################
sub getPointDist( $ )
{
	my $dir = $_[0];
	&debug_print( "getPointDist: dir=($dir)" );
	
	return $UNCALCULATED_INFORMATION 
		unless (open DAT, "$dir\/$variables{ datafile }");
	my @data = <DAT>;
	close DAT;
	my $DIST = pop @data;
	&debug_print( "getPointDist: Parsing line ($DIST)" );
	my @words = split ' ', $DIST;
	if ( $#words < 1 )
	{
		@words = split '\)', $words[0];
		@words = split '\(', $words[0];
		$DIST = sprintf( "%.16f",  $words[1] );
		&debug_print( "getPointDist: Single locus, DIST=($DIST)" );
		return ( $DIST );
	}
	$DIST = pop @data;
	&debug_print( "getPointDist: Two locus, Parsing line ($DIST)" );
	@words = split '\)', $DIST;
	@words = split '\(', $words[0];
	@words = split ' ', $words[1];
	my $dist1 = sprintf( "%.5f",  $words[0] );
	my $dist2 = sprintf( "%.5f",  $words[1] );

	&debug_print( "getPointDist: Two locus, dist1=($dist1), dist2=($dist2)" );
	return ( $dist1, $dist2 );
}
################################################################



################################################################
# 				printLinetoTable
#	Prints LOD to table Cell
#
#	Parameters:
#					$_[0] - Hash in the format defined at the top
#					$_[1] - Number of loci
#					$_[2] - Span of first markers on
#					$_[3] - Main key in %INFO to print
#					$_[4] - Secondary key in %INFO to print. If it is $NO - not printed
#					$_[5] - Third key in %INFO to print. If it is $NO - not printed
#
#					For multiPoint:	
#					$_[5] - The size of each group of markers
#					$_[6] - The color of the line. See getRowColor( $ $ )
#					$_[7] - The number of the line in the table
#					$_[8] - Extra information to be presented with the marker
#
#	Returns:
#					String with output
################################################################
sub printLinetoTable( \% $ $ $ $ $ $ $ $ $ )
{
	my %info = %{ $_[0] };
	my $locus = $_[1];
	my $rowSpan = $_[2];
	my $firstKey = $_[3];
	my $secondKey = $_[4];
	my $thirdKey = $_[5];
	my $multiKey = $_[6];
	my $multiColor = $_[7];
	my $multiLineNum = $_[8];
	my $extraString = $_[9];

	my @infoKeys = keys %info;
	my @infoValues = values %info;
	&debug_print( "Print Lod to Table for ($locus) locus with keys (@infoKeys) and values (@infoValues)" );
	&debug_print( "Print Lod to Table: First Key=($firstKey), second Key=($secondKey), third Key=($thirdKey), Num of values for First Key=($#{ $info{$firstKey} })" );
	
	if (! defined $complexityMap{ $info{NAME} } ) {
		my @complexity_line = `cat $COMPLEXITY_CONVERSION_TABLE_FILE | grep "^${ $info{$thirdKey} }[0]"`;
		if ($#complexity_line > -1) {
			@tmp = split ' ', $complexity_line[0];
			$complexityMap { $info{NAME} } = $tmp[1];
		}
		else {
			$complexityMap { $info{NAME} } = $UNCALCULATED_INFORMATION;
		}
	}

	my $color = "";
	if ( ($info{PROGRAM} =~ /MultiPoint/) && ($multiColor =~/GRAY/) )
	{
		$color =  " bgcolor=\"#D3D3D3\"";
	}
	
	my $result = "<tr".$color.">\n";
	my $cellS1 = "<td width=";
	my $cellS1_1 = $cellS1;
	$cellS1_1 = "<td rowspan=$rowSpan width=" if ( $rowSpan > 0 );
	my $cellS2 = "\% align=middle>";
	my $cellE = "</td>\n";
	my $f_width = 15;
	my $s_width = 20;

	if ( $locus =~ /1/ )
	{
		#if ( $info{PROGRAM} =~ /TwoPoint/ )
		#{
			$f_width = 8;
			if ( $info{PROGRAM} =~ /TwoPoint/ )
			{
				$name = $info{ CHROM };
				if ( $variables{ numMarkers } != $#{$variables { usedMarkersId }} + 1 ) {
					$name = ${ $variables{ usedMarkersId } }[ $info{ CHROM } - 1 ];
				}
				
				if (! -z "mapfile.dat") {
					$name = "c".$name;
				}
			}
			else
			{
				if ( ($multiLineNum / $multiKey) eq  (int ($multiLineNum / $multiKey)) )  #If the marker is the first in its group
				{
					my $cellS1_2 = "<td rowspan=\"$multiKey\" width=";

					$result = $result.$cellS1_2.$f_width.$cellS2."<b><font color=#0000FF>".$info{ CHROM }.$extraString."</font></b>".$cellE;
				}
				
				$name = &getMultiMarkerID($multiLineNum);
				if (! -z "mapfile.dat") {
					$name = "c".$name;
				}
			}
			
			$result = $result.$cellS1.$f_width.$cellS2."<b><font color=#0000FF>".$name."</font></b>".$cellE;
			
			if  ( $info{PROGRAM} =~ /MultiPoint/ ) {
				my $correctName = &getName($multiLineNum);			
				if ( $correctName !~ $info{NAME} ) {
					$info{NAME} = $correctName;
				}
			}
			$result = $result.$cellS1.$f_width.$cellS2."<b><font color=#0000FF>".$info{NAME}."</font></b>".$cellE;
			
			my $dist = 0.0;
			
			if ( $info{PROGRAM} =~ /TwoPoint/ )
			{
				if ( $variables{ numMarkers } != $#{$variables { usedMarkersId }} + 1 ) {
					$num = $info{ CHROM };
					$num = ${ $variables{ usedMarkersId } }[ $info{ CHROM } - 1 ];
					$dist = sprintf ("%.3f", &getDistFromStart($num - 1, $variables{datafile}));
				}
				else {
					$dist = sprintf ("%.3f", &getDistFromStart($info{ CHROM } - 1, $variables{datafile}));
				}
			}
			else 
			{			
				$distFromStart = &getDistFromStart(&getMultiMarkerID($multiLineNum) - 1, $variables{datafile});
				$dist = sprintf ("%.3f", $distFromStart);
			}
			
			$result = $result.$cellS1.$f_width.$cellS2."<b><font color=#0000FF>".$dist."</font></b>".$cellE;
		#}
		#else
		#{
		#	$result = $result.$cellS1.$f_width.$cellS2.$info{NAME}.$cellE;
		#	$result = $result.$cellS1.$s_width.$cellS2.$info{DIST}.$cellE;
		#}
		$LODdata = ${ $info{$firstKey} }[0];
		if ($LODdata =~ /^[+-]?\d+\.?\d*$/){
			&addMarkerData($name, $info{NAME}, $dist, ${ $info{$firstKey} }[0]);
		}
	}
	elsif ( $locus =~ /2/ )
	{
		$result = $result.$cellS1_1.$f_width.$cellS2.$info{NAME1}.$cellE if ( $rowSpan > 0 );
		$result = $result.$cellS1_1.$s_width.$cellS2.$info{DIST1}.$cellE if ( ( $rowSpan > 0 ) && ( $info{PROGRAM} !~ /TwoPoint/ ) );
		$result = $result.$cellS1.$f_width.$cellS2.$info{NAME2}.$cellE;
		$result = $result.$cellS1.$s_width.$cellS2.$info{DIST2}.$cellE if ( $info{PROGRAM} !~ /TwoPoint/ );
	}
	
	for ( my $p = 0; $p <= $#{ $info{$firstKey} }; $p ++ )
	{
		$result = $result.$cellS1.$f_width.$cellS2.${ $info{$secondKey} }[$p].$cellE unless ( $secondKey eq $NO );
		$result = $result.$cellS1.$f_width.$cellS2;
		$result = $result.&printColoredNumber( ${ $info{$firstKey} }[$p] );
		$result = $result.$cellE;
		$result = $result.$cellS1.$f_width.$cellS2.$complexityMap{ $info{NAME} }.$cellE unless ( $thirdKey eq $NO );
	}
	
	$result = $result."</tr>\n";

	&debug_print( "printLinetoTable: result=($result)" );
	
	$projName = $variables{ PROJ };
	@chromNameData = split /chrom_/, $projName;
	
	my @positions = @{$variables { positions }};
	
	if ($#positions > -1) {
		$LODdata = ${ $info{$firstKey} }[0];
		if ($LODdata =~ /^[+-]?\d+\.?\d*$/){
			@markersNames = split /\([0-9]+\)[ ]*/, $info{NAME};
			$markerName = $markersNames[0];
			@markerData = grep(/$markerName :/, @positions);
			@splitData = split / : /, $markerData[0];
			$startPos = $splitData[1];
			if ($name =~ /c/) {
				$endPos = $startPos;
				if ($#markersNames > 0) {
					$markerName = $markersNames[$#markersNames];
					@markerData = grep(/$markerName :/, @positions);
					@splitData = split / : /, $markerData[0];
					$endPos = $splitData[1];
				}			
				addClusterLODscore("$RUN_DIR\/$BED_OUTPUT_FILE", $chromNameData[1]+0, $startPos, $endPos, $LODdata );
			}	
			else {
				addMarkerLODscore("$RUN_DIR\/$BED_OUTPUT_FILE", $chromNameData[1]+0, $startPos, $LODdata);
			}
		}
	}
	
	return $result;
}
################################################################


################################################################
# 				printColoredNumber
#	Receives the number and return it's colored variant in the HTML:
#		If the number below 0 			- RED
#		If the number between 0 and 1 	- BLACK
#		If the number above 1			- GREEN BOLD
#
#	Parameters:
#					$_[0] - Number
#	Returns:
#					String with output
################################################################
sub printColoredNumber( $ )
{
	my $number = $_[0];
	
	&debug_print( "printColoredNumber for ($number) Number" );
	
	my $result = "";
	if ( $number eq $UNCALCULATED_INFORMATION )
	{
		$result = $result."<font color=#33FF33>".$number;
    }
	elsif ( ( $number eq $SKIPPED_INFORMATION) || ( $number eq $INFINITY ) || ( $number eq $LINKED ) )
	{
		$result = $result."<font color=#660066><b>".$number."</b>";
    }
	elsif ( $number eq $ERROR_IN_COMPUTATION )
	{
		$result = $result."<font color=#AA0078>".$number;
    }
	elsif ( $number =~ /Trivial|Simple|Average|Hard/ )				#	Complexity status
	{
		return $number;
    }
    else
    { 
		$number = sprintf( "%.4f",  $number );
		if ($number < 0) 
		{
			$result = $result."<font color=#FF6666>".$number;
		} 
		elsif ( $number < 1) 
		{
			$result = $result."<font color=#000000>".$number;
		}
		else
		{
			$result = $result."<font color=#006600><b>".$number."</b>";
		}
	}
      
	$result = $result."</font>";
	
	return $result;
}
################################################################

################################################################
# 				getRowColor
#	Receives a row number and return it's color according to the 
#	group it belongs when the data was divided to groups of the 
#	given number of markers
#
#	Parameters:
#					$_[0] - Line number
#					$_[1] - Number of markers according to which 
#							the file was divided
#	Returns:
#		Every odd group		 			- "GRAY"
#		Every even group		 		- "WHITE"
################################################################
sub getRowColor( $ $ )
{
	my $lineNum = $_[0];
	my $groupSize = $_[1];
	
	my $groupNum = int ($lineNum / $groupSize);
	
	if (($groupNum % 2) eq 0) 
	{
		return "GRAY";
	}
	
	return "WHITE";
}

################################################################
# 				isFirstInGroup
#	Receives a row number and return whether it is starting a new 
#	color according to the group it belongs when the data was 
#	divided to groups of the given number of markers
#
#	Parameters:
#					$_[0] - Line number
#					$_[1] - Number of markers according to which 
#							the file was divided
#	Returns:
#			boolean indicator
################################################################
sub isFirstInGroup( $ $ )
{
	my $lineNum = $_[0];
	my $groupSize = $_[1];
	
	my $groupIndication = $lineNum / $groupSize;
	my $groupNum = int ($lineNum / $groupSize);
	
	if ($groupIndication eq $groupNum ) 
	{
		return 1;
	}
	
	return 0;
}

################################################################
# 				printCmplxToTable
#	Prints Complexities to table Cell
#
#	Parameters:
#					$_[0] - Hash with point properties
#					$_[1] - One or Two locus
#					$_[2] - Row span
#	Returns:
#					String with output
################################################################
sub printCmplxToTable( \% $ $ )
{
	my %info = %{ $_[0] };
	my $locus = $_[1];
	my $rowSpan = $_[2];
	my @infoKeys = keys %info;
	my @infoValues = values %info;
	&debug_print( "printCmplxToTable for ($locus) locus with keys (@infoKeys) and values (@infoValues)" );
	
	my $result = "<tr>\n";
	my $cellS1 = "<td width=";
	my $cellS1_1 = $cellS1;
	$cellS1_1 = "<td rowspan=$rowSpan width=" if ( $rowSpan > 0 );
	my $cellS2 = "\% align=middle>";
	my $cellE = "</td>\n";
	if ( $locus =~ /1/ )
	{
		$result = $result.$cellS1."15".$cellS2.$info{NAME}.$cellE;
		$result = $result.$cellS1."20".$cellS2.$info{DIST}.$cellE;
	}
	elsif ( $locus =~ /2/ )
	{
		$result = $result.$cellS1_1."15".$cellS2.$info{NAME1}.$cellE if ( $rowSpan > 0 );
		$result = $result.$cellS1_1."20".$cellS2.$info{DIST1}.$cellE if ( ( $rowSpan > 0 ) && ( $info{DIST1} !~ /TwoPoint/ ) );
		$result = $result.$cellS1."15".$cellS2.$info{NAME2}.$cellE;
		$result = $result.$cellS1."20".$cellS2.$info{DIST2}.$cellE if ( $info{DIST2} !~ /TwoPoint/ );
	}
	for ( my $p = 0; $p <= $#{ $info{MULT} }; $p ++ )
	{
		$result = $result.$cellS1."15".$cellS2.${ $info{MULT} }[$p].$cellE;
		$result = $result.$cellS1."15".$cellS2.${ $info{CMPLX} }[$p].$cellE;
	}
	$result = $result."</tr>\n";
	
	&debug_print( "printCmplxToTable: result=($result)" );
	return $result;
}
################################################################




################################################################
# 				printTablesToOutput
#	Prints all the "*.result" files into PROJECT_NAME.$OUTPU_FILE
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					NOTHING
################################################################
sub printTablesToOutput( \% )
{
	%variables = %{ $_[0] };
	&debug_print("printTablesToOutput");

	%variables = %{&makeSystemError("No total result calculated!!!", \%variables )}
		unless ( -e "total\.result");
	
	%variables = %{&makeSystemError("Failed appending to file ($OUTPUT_FILE)", \%variables )}
		unless (open OUT, ">>$OUTPUT_FILE");

	%variables = %{&makeSystemError("Failed reading from result file ($total\.result)", \%variables )}
		unless (open RES, "total\.result");

	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	&debug_print("Printing total table");
		
		if ( $clusteringPost == 1 ){
			print OUT "<a name=\"Table\"></a>\n";
		}
		
		my $title = "All analyzed pedigrees combined";
		print OUT "\n<font size=4><div align=center><center><b>$title</b></font>\n";

		print OUT "\n<br><font size=4><div align=center><center>".
				"(LOD score is given for requested Multi Point calculation per each chromosome)".
				"</font>\n"
			if ( ( $#{ $variables{ chromosomes } } > 0 ) && ( $variables{ program } =~ /MultiPoint/ ) );
   		print OUT <RES>;
	close RES;
	
	
	
	
	unless ( $#{ $variables{ pednames } } == 0 )
	{
		foreach my $family ( @{ $variables{ pednames } } )
		{
			unless ( -e "$family\.result" )
			{
				&debug_print("No Family result calculated for family ($family)!!!");
				next;
			}
			&debug_print("Printing family ($family) table");
			%variables = %{&makeSystemError("Failed reading from ($family\.result)")}
				unless (open RES, "$family\.result");
			exit $ERROR if ( $NO !~ $variables{ errorExist } );
	
				$title = "Family $family";
				print OUT "\n<p><font size=4><div align=center><center><b>$title</b></font>\n";
				print OUT <RES>;
				print OUT "</p>\n";
			close RES;
		}
	}
	
	
	### add mapping and small graphs
	if ( $clusteringPost == 1 ){			
		my $projectDir = $RUN_DIR;
		print OUT "<br><a name=\"Mapping\"></a>\n";
		print OUT "Mapping for clustering :<br><br>\n";
		my $mappingFile = "mapfile.dat";
		open(MAPPING,"$mappingFile");
		$markerNum = 1;
		print OUT "<table border=\"1\">\n";
		print OUT "<tr>\n<td><div align=center><center> Marker Num </td>\n<td><div align=center><center> Cluster </td>\n</tr>\n";
		for $line ( <MAPPING> ){
			if ($line =~ /\[VALUE MAPPING\]/){
				last;
			}
			chomp $line;
			print OUT "<tr>\n<td><div align=center><center>$markerNum</td>\n<td><div align=center><center>$line</td>\n</tr>\n";
			$markerNum++
		}
		print OUT "</table>\n";
		close(MAPPING);
		#while( -e "$projectDir\/LODgraph.pedfile_$pedInd.png" ){
		#	print OUT "<center>Graph for Pedigree $pedInd</center><br>\n";
		#	print OUT "<center><img src=\"LODgraph.pedfile_$pedInd.png\" /></center><br><br>\n";
		#	$pedInd++;
		#}
		
		if ( $variables{ program } =~ /TwoPoint/ ) {
			print OUT "<br><a name=\"Zoom\"></a>\n";
			print OUT "Zooming Graphs :<br><br>\n";
			for $pedInd (0..$graphLastPed){
				my $start = 1;
				my $end = $graphDivisionSize;
				if( $end > $graphNumMarkers ){
					$end = $graphNumMarkers;
				}
							
				print OUT "<a href=\"#p{$pedInd}_m${start}_${end}\">Pedigree $pedInd</a>   :   \n";
				while( $start < $graphNumMarkers ){
					print OUT "<a href=\"#p{$pedInd}_m${start}_${end}\">${start} - ${end}</a>   \n";
					$start += $graphDivisionSize;
					$end += $graphDivisionSize;
					if( $end > $graphNumMarkers ){
						$end = $graphNumMarkers;
					}
				}
				print OUT "<br><br>\n";
			}		
			for $pedInd (0..$graphLastPed){
				my $start = 1;
				my $end = $graphDivisionSize;
				if( $end > $graphNumMarkers ){
					$end = $graphNumMarkers;
				}
				
				while( $start < $graphNumMarkers ){
					print OUT "<a name=\"p{$pedInd}_m${start}_${end}\"></a>\n";
					print OUT "<center><img src=\"LODgraph_${start}_${end}.pedfile_$pedInd.png\" /></center><br><br>\n";
					$start += $graphDivisionSize;
					$end += $graphDivisionSize;
					if( $end > $graphNumMarkers ){
						$end = $graphNumMarkers;
					}
				}
			}
		}
		
		# $start = 1;
		# $end = $graphDivisionSize;
		# $smallGraphFilename = "$projectDir\/LODgraph_${start}_${end}.pedfile_$pedInd.png";	
		# while( -e $smallGraphFilename  ){
			# while( -e "$projectDir\/LODgraph.pedfile_$pedInd.png" ){
				# print OUT "<center>Graph for Pedigree $pedInd</center><br>\n";
				# print OUT "<center><img src=\"LODgraph.pedfile_$pedInd.png\" /></center><br><br>\n";
				# $pedInd++;
			# }
		# }

	}
	###
	
	close OUT;
	
	exit $ERROR if ( $ERROR =~ &finishOutputFile( \%variables ) );
	if ( $clusteringPost == 1 ){
		return;
	}

	%variables = %{&makeSystemError("Failed appending to file ($OUTPUT_FILE)", \%variables )}
		unless (open OUT, ">>$OUTPUT_FILE");
	exit $ERROR if ( $NO !~ $variables{ errorExist } );
	foreach $family ( @{ $variables{ pednames } } )
	{
		unless ( -e "$family\.complexity" )
		{
			&debug_print("No Family Complexity calculated for family ($family)!!!");
			next;
		}
		&debug_print("Printing family ($family) table");
		%variables = %{&makeSystemError("Failed reading from ($family\.complexity)", \%variables )}
			unless (open RES, "$family\.complexity");
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
			
			$title = "Family $family";
			print OUT "\n<p><font size=4><div align=center><center><b>$title</b></font>\n";
  	 		print OUT <RES>;
			print OUT "</p>\n";
		close RES;
	}
	close OUT;
}
################################################################



################################################################
# 				createFamilyResult
#	Put's all the result files of given family into one file.
#
#	Parameters:
#					$_[0] - pedigree number
#					$_[1] - Output LODscore file
#					$_[2] - Output Complexity file
#					$_[3] - Hash with variables
#	Returns:
#					Array with LODscore of family. For format look at &initTotalLOD( $ $ \% );
################################################################
sub createFamilyResult( $ $ $ \% )
{
	my $pedNumber = $_[0];
	my $outputLOD = $_[1];
	my $outputCMPLX = $_[2];
	%variables = %{ $_[3] };
	&debug_print("A Create Family number ($pedNumber) result into file ($outputLOD) and complexity file ($outputCMPLX)" );
	
	my $place = 0;
	my $rows = 0;
	my @familyLOD = ();
	
	my $realChromCounter = &getCalculatedChrNumber();
	
	for( my $chrom = 1; $chrom <= $realChromCounter; $chrom++ )
	{
		my $lastPlace = $#familyLOD + 1;

# Unlinked LOD calculation
		my $unlinkedDir = "datafile_".$chrom."_0\/pedfile_".$pedNumber;
		my %unlinkedPointInfo = %{ &getPointInfoFromDirectory($unlinkedDir, \%variables) };
		if ( $variables{ program } =~ /TwoLocus.TwoPoint/ )
		{
			${ $unlinkedPointInfo{LOD} }[1] = ${ $unlinkedPointInfo{LOD} }[0];
			${ $unlinkedPointInfo{LN} }[1] = ${ $unlinkedPointInfo{LN} }[0];
		}
		elsif ( $variables{ program } =~ /SingleLocus.TwoPoint/ )
		{
			for ( my $p =1; $p < $TwoPointNUM; $p ++ ) 
			{
				${ $unlinkedPointInfo{LOD} }[$p] = ${ $unlinkedPointInfo{LOD} }[0];
				${ $unlinkedPointInfo{LN} }[$p] = ${ $unlinkedPointInfo{LN} }[0];
			}
		}

# Linked LOD calculation
		my $mask = "datafile_".$chrom."_";
		my @pointDirs = <$mask*>;
		%variables = %{&makeSystemError("Output does not exist", \%variables )}
			if ( $#pointDirs < 1 );
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
	
		&debug_print("Directories are: (@pointDirs)" );
		my $familyDir = "pedfile_".$pedNumber;
		for( my $i = 1; $i <= $#pointDirs; $i++ )
		{
			my %pointInfo = %{ &initPoint($variables{ program }, 1 ) };
			my %singlePointInfo;
			my $dir;
			for ( $p = 0; $p <= $#{ $unlinkedPointInfo{LOD} }; $p ++ )
			{
				$dir = $mask.$i;
				&debug_print("Working on directory ($dir), it's place in array is ($place). LOD # ($p) out of  ($#{ $unlinkedPointInfo{LOD} }) LODs" );
				
				%variables = %{&makeSystemError("Output directory ($dir\/$familyDir) for family ($pedNumber) does not exist!!!", \%variables )}
					unless ( -d "$dir\/$familyDir");
				exit $ERROR if ( $NO !~ $variables{ errorExist } );
		
				%singlePointInfo = %{ &getPointInfoFromDirectory("$dir\/$familyDir", \%variables) };
				${ $pointInfo{ LOD } }[$p] = ${$singlePointInfo{LOD}}[0];
				${ $pointInfo{ LN } }[$p] = ${$singlePointInfo{LN}}[0];
				${ $pointInfo{ MULT } }[$p] = ${$singlePointInfo{MULT}}[0];
				${ $pointInfo{ CMPLX } }[$p] = ${$singlePointInfo{CMPLX}}[0];
				
				$i ++;
			}
			$pointInfo{ CHROM } = $chrom;
			$i--;
			$dir = $mask."1" unless ( $variables{ program } =~ /MultiPoint/ );
				
			if ( $variables{ program } =~ /TwoLocus/ )
			{
				($pointInfo{DIST1}, $pointInfo{DIST2}) = &getPointDist("$dir\/$familyDir");
				$pointInfo{NAME1} = &getPointMarker("$dir\/$familyDir", $pointInfo{DIST1}, 1);
				$pointInfo{NAME2} = &getPointMarker("$dir\/$familyDir", $pointInfo{DIST2}, 2);
			}
			else
			{
				($pointInfo{DIST}) = &getPointDist("$dir\/$familyDir");
				$pointInfo{NAME} = &getPointMarker("$dir\/$familyDir", $pointInfo{DIST}, 1);
			}

			%{ $familyLOD[ $place ] } = %{ &calculatePoint(\%pointInfo, \%unlinkedPointInfo) };

			$place++;
		}
	}
	&createTotalResult( \@familyLOD, $outputLOD, $outputCMPLX, \%variables );
	
	return \@familyLOD;
}
################################################################


################################################################
# 				getCalculatedChrNumber
#	Prints OUTPUT file header
#
#	Parameters:
#					NOTHING
#	Returns:
#					Number of chromosomes calculated
################################################################
sub getCalculatedChrNumber()
{
	&debug_print("getCalculatedChrNumber" );
	my $ch = 1;
	while( $ch > 0 )
	{
		my $unlinkedDir = "datafile_".$ch."_0";
		last if ( ! -e $unlinkedDir );
		$ch++;
	}
	$ch--;
	&debug_print("There are ($ch) of chromosome calculations" );
	return $ch;
}
################################################################




################################################################
# 				getPointInfoFromDirectory
#	Gets the Information hash for current datafile_*/pedfile_*  directory
#
#	Parameters:
#					$_[0] - Data directory name
#					$_[1] - Hash with variables
#	Returns:
#					NOTHING
################################################################
sub getPointInfoFromDirectory( $ \% )
{
	my $dataDir = $_[0];
	%variables = %{ $_[1] };

	my %pointInfo = %{ &initPoint($variables{ program }, 1 ) };
	&debug_print("Parsing info from directory ($dataDir)" );
	if ( -s $dataDir."\/$ERROR_RESULT_FILE" )
	{
		open ERR, $dataDir."\/$ERROR_RESULT_FILE";
		my $errMsg = join "\n", <ERR>;
		close ERR;
		%variables = %{&makeInputError($errMsg, \%variables )};
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
	}
	elsif ( -e $dataDir."\/$EXCEPTION_FILE")
	{
		%variables = %{&makeSystemError("Failed to open RESULT file ", \%variables )}
			unless ( open RES, $dataDir."\/$EXCEPTION_FILE" );
		exit $ERROR if ( $NO !~ $variables{ errorExist } );
		my @res = <RES>;
		close RES;
		${ $pointInfo{LOD} }[0] = shift @res;
		chomp ${ $pointInfo{LOD} }[0];
		if ( ${ $pointInfo{LOD} }[0] =~ /$LINKED/ )
		{
			${ $pointInfo{LN} }[0] = $LINKED;
			${ $pointInfo{LOD} }[0] = $LINKED;
		}
		else
		{
			%variables = %{&makeSystemError("Unknown exception type!", \%variables )};
			exit $ERROR;
		}
	}
	elsif ( -e $dataDir."\/$LOCAL_RESULT_FILE")
	{
		%pointInfo =  %{ &getInfoFromResultFile( $dataDir."\/$LOCAL_RESULT_FILE", \%variables ) };
	}
	else
	{
		&debug_print("Looking for Condor result files..." );
		if ( -e $dataDir."\/$CONDOR_COMPLEXITY_FILE" )
		{
			%variables = %{&makeSystemError("Failed to open COMPLEXITY file ", \%variables )}
				unless ( open RES, $dataDir."\/$CONDOR_COMPLEXITY_FILE" );
			exit $ERROR if ( $NO !~ $variables{ errorExist } );
			@res = <RES>;
			close RES;
			${ $pointInfo{MULT} }[0] = shift @res;
			chomp ${ $pointInfo{MULT} }[0];
		}
		else
		{
			${ $pointInfo{MULT} }[0] = $UNCALCULATED_INFORMATION;
		}

		if ( -e $dataDir."\/$CONDOR_RESULT_FILE")
		{
			%variables = %{&makeSystemError("Failed to open RESULT file ", \%variables )}
				unless ( open RES, $dataDir."\/$CONDOR_RESULT_FILE" );
			exit $ERROR if ( $NO !~ $variables{ errorExist } );
			@res = <RES>;
			close RES;
			${ $pointInfo{LOD} }[0] = shift @res;
			chomp ${ $pointInfo{LOD} }[0];
			if ( ${ $pointInfo{LOD} }[0] =~ /Infinity/ )
			{
				${ $pointInfo{LN} }[0] = $INFINITY;
				${ $pointInfo{LOD} }[0] = $INFINITY;
			}
			elsif (  ${ $pointInfo{LOD} }[0] =~ /\d+/ )
			{
				${ $pointInfo{LN} }[0] = ${ $pointInfo{LOD} }[0] * log ( 10 );
			}
			else
			{
				${ $pointInfo{LN} }[0] = $UNCALCULATED_INFORMATION;
				${ $pointInfo{LOD} }[0] = $UNCALCULATED_INFORMATION;
			}
			
		}
		elsif ( -e $dataDir."\/$SKIPPED_RESULT_FILE")
		{
			${ $pointInfo{LN} }[0] = $SKIPPED_INFORMATION;
			${ $pointInfo{LOD} }[0] = $SKIPPED_INFORMATION;
		}
		elsif ( -e $dataDir."\/$ERROR_RESULT_FILE")
		{
			${ $pointInfo{LN} }[0] = $ERROR_IN_COMPUTATION;
			${ $pointInfo{LOD} }[0] = $ERROR_IN_COMPUTATION;
		}
		else
		{
			${ $pointInfo{LN} }[0] = $UNCALCULATED_INFORMATION;
			${ $pointInfo{LOD} }[0] = $UNCALCULATED_INFORMATION;
		}
	}
	${ $pointInfo{CMPLX} }[0] = &getComplexityLevel( ${ $pointInfo{MULT} }[0], \%variables );

	&debug_print("LOD=(${ $pointInfo{LOD} }[0]), CMPLX=(${ $pointInfo{CMPLX} }[0])" );
	
	return \%pointInfo;
}
################################################################




################################################################
# 				makeGraphs
#
# dvir 26/7/2010 : This method is probably unused.
#	Parameters:
#
#	Returns:
#					NOTHING
################################################################

sub makeGraphs(  )
{
	my ($projectDir,$mappingFile,$pedInd,$lastPedFlag,$graphInputFilename,$graphFilename,$SmallGraphInputFilename,$smallGraphFilename,$markerNum,$start,$end,$maxValue,$smallMaxValue,$resultFilename,$nullResultFilename,$curResult,$nullResult,$LODscore);
	
	#print "Creating Graphs in $projectDir \n";
	$projectDir = $RUN_DIR;
	
	$mappingFile = "mapfile.dat";
	my $minValue = -3;
	my $graphMin = -3;
	my $graphMax = 4;
	my $updateFlag = 0; #for updating $graphNumMarkers
	
	
	# Get last marker number
	open IN, "$projectDir/$variables{ datafile }";
	my @data = <IN>;
	close IN;
	my @words_first_line = split ' ', $data[0];
	my $lastMarker = $words_first_line[0] - 1;
	my $firstMarker = 1;
	my $totalMarkers = 0;
	
	my $multiKey = 1;  										
	if ( open IN, "$projectDir/multiParams.dat" )
	{
		my @multiData = <IN>;
		close IN;

		my @words = split ' ', $multiData[0];
		$multiKey = $words[2];
		$lastMarker = $words[1];
		$firstMarker = $words[0];
	}
	else {
		if ( $variables{ program } =~ /MultiPoint/ ) { 			#It was a list
			$lastMarker = $#{$variables { usedMarkersId }} + 1;
		}
	}
	
	$resFile = $CONDOR_RESULT_FILE;
	my $testpath = "$projectDir\/datafile_*/*\/$CONDOR_RESULT_FILE";
	
	my @resDirs = `ls $testpath`;
	
	if ($#resDirs <= 0)
	{
		$resFile = $LOCAL_RESULT_FILE;
	}
	
	$pedInd = 0;
	$lastPedFlag = 0;
	while ( $lastPedFlag == 0 ){
		$start = 1;
		$end = $graphDivisionSize;
		$graphInputFilename = "$projectDir\/graphInput.pedfile_$pedInd.txt";
		$graphFilename = "$projectDir\/LODgraph.pedfile_$pedInd.png";	
		$SmallGraphInputFilename = "$projectDir\/graphInput_${start}_${end}.pedfile_$pedInd.txt";
		$smallGraphFilename = "$projectDir\/LODgraph_${start}_${end}.pedfile_$pedInd.png";	
		$lastPedFlag = 1;	
		$markerNum = 1;
		$resultDir = "$projectDir\/datafile_${markerNum}_1\/pedfile_$pedInd";
		$nullResultDir = "$projectDir\/datafile_${markerNum}_0\/pedfile_$pedInd";
		$resultFilename = "$resultDir\/$resFile";
		$nullResultFilename = "$nullResultDir\/$resFile";
		$maxValue = $graphMax;
		$smallMaxValue = $graphMax;
		#print "$resultFilename $nullResultFilename\n";
		my $multiCount = 1;
		
		while ( (-d $resultDir) && (-d $nullResultDir) ){
			&debug_print("Result dirs: regular = $resultDir null = $nullResultDir");
			my $correctMarkerNum = (1 + ($markerNum - 1)*$multiKey) + $multiCount - 1;
			if (( $variables{ program } =~ /TwoPoint/ ) && ( $variables{ numMarkers } != $#{$variables { usedMarkersId }} + 1 )) {
				$correctMarkerNum = ${$variables { usedMarkersId }}[ $markerNum - 1 ];
				`echo "$markerNum is $correctMarkerNum" >> /tmp/testMarker`;
			}
			if (( $correctMarkerNum == $end + 1 ) && ( $variables{ program } =~ /TwoPoint/ ) && ( $variables{ numMarkers } == $#{$variables { usedMarkersId }} + 1 )){
				#run gnuplot commands
				&debug_print("Print divisionGraph: [$start:$end] to creat graph $smallGraphFilename by commands gnuplotCommandsSmall.txt");
				open( COMMANDS_FILE , ">gnuplotCommandsSmall.txt" );
				print COMMANDS_FILE "set terminal png\n";
				print COMMANDS_FILE "set output \"$smallGraphFilename\"\n";
				print COMMANDS_FILE "f(x) = 3.3\n";
				print COMMANDS_FILE "g(x) = -2\n";
				print COMMANDS_FILE "set xrange [$start:$end]\n";
				print COMMANDS_FILE "set yrange [$graphMin:$smallMaxValue]\n";
				print COMMANDS_FILE "set multiplot\n";
				print COMMANDS_FILE "set pointsize 1\n";
				print COMMANDS_FILE "plot \"$SmallGraphInputFilename\" with linespoints pt 5 notitle\n";
				print COMMANDS_FILE "plot f(x) with lines lt 3 notitle\n";
				print COMMANDS_FILE "plot g(x) with lines lt 3 notitle\n";
				print COMMANDS_FILE "unset multiplot\n";
				close( COMMANDS_FILE );
				system("gnuplot gnuplotCommandsSmall.txt");	
				$start += $graphDivisionSize;
				$end += $graphDivisionSize;
				$smallMaxValue = 4;
				$SmallGraphInputFilename = "$projectDir\/graphInput_${start}_${end}.pedfile_$pedInd.txt";
				$smallGraphFilename = "$projectDir\/LODgraph_${start}_${end}.pedfile_$pedInd.png";	
			}		
			unless( (-e $resultFilename) && (-e $nullResultFilename) ){
				$markerNum++;
				$resultDir = "$projectDir\/datafile_${markerNum}_1\/pedfile_$pedInd";
				$nullResultDir = "$projectDir\/datafile_${markerNum}_0\/pedfile_$pedInd";
				$resultFilename = "$resultDir\/$resFile";
				$nullResultFilename = "$nullResultDir\/$resFile";
				&debug_print("Restarting loop for $resultFilename $nullResultFilename");
				next;
			}
			&debug_print("Init lastPedFlag");
			$lastPedFlag = 0;
			
			open( CUR_RESULT , $resultFilename );
			$curResult = <CUR_RESULT>;	
			$curResult =~ s/\s+$//; 				
			close(CUR_RESULT);			
			open( NULL_RESULT , $nullResultFilename );
			$nullResult = <NULL_RESULT>;
			$nullResult =~ s/\s+$//; 		
			close(NULL_RESULT);
			#create input to gnuplot
			open( GRAPH_INPUT , ">>$graphInputFilename");
			open( SMALL_GRAPH_INPUT , ">>$SmallGraphInputFilename");
			$LODscore = $curResult - $nullResult;							
			if( $LODscore < $minValue ){
				$LODscore = $minValue;
			}
			
			if ($curResult eq "Infinity") {
				$LODscore = $minValue;
			}
				
			if (-e "$projectDir/extra.dat")
			{
				my @markerLinesInExtra = `grep ^"$correctMarkerNum has" $projectDir/extra.dat`;
				if ($#markerLinesInExtra > -1) 
				{
					my @markerExtraData = split " ", $markerLinesInExtra[0];
					if ( $multiCount <= $markerExtraData[2] ) {
						$multiCount++;
						next;
					}
				}
			}
			if ( $variables{ program } =~ /MultiPoint/ ) {
					$markerID = &getMultiMarkerID($correctMarkerNum - 1);
					@pointInFile = `grep ^"$markerID " $graphInputFilename`;
					$prevValue = &getMultiMarkerID($correctMarkerNum - 2);
			}
			else {
				@pointInFile = `grep ^"$correctMarkerNum " $graphInputFilename`;
				if (( $variables{ program } =~ /TwoPoint/ ) && ( $variables{ numMarkers } != $#{$variables { usedMarkersId }} + 1 )) {
					if ($markerNum > 1) {
						$prevValue = ${$variables { usedMarkersId }}[ $markerNum - 2];
					}
					else {
						$prevValue = -1;
					}
				}
				else {
					$prevValue = $correctMarkerNum - 1;
				}
			}
			my @prevPointInFile = `grep ^"$prevValue " $graphInputFilename`;
			if ( ($#pointInFile <= -1) && ( ($correctMarkerNum == 1) || ($#prevPointInFile > -1) || ($prevValue == -1)) ){
				#print GRAPH_INPUT "$correctMarkerNum $LODscore\n";
				#print SMALL_GRAPH_INPUT "$correctMarkerNum $LODscore\n";
				if ( $variables{ program } =~ /MultiPoint/ ) {
					$markerID = &getMultiMarkerID($correctMarkerNum - 1);
					print GRAPH_INPUT "$markerID $LODscore\n";
					print SMALL_GRAPH_INPUT "$markerID $LODscore\n";
				}
				else {
					print GRAPH_INPUT "$correctMarkerNum $LODscore\n";
					print SMALL_GRAPH_INPUT "$correctMarkerNum $LODscore\n";
				}
			}
			close( GRAPH_INPUT );	
			
			if( $LODscore > $maxValue ){
				$maxValue = $LODscore + 0.1;
			}	
			if( $LODscore > $smallMaxValue ){
				$smallMaxValue = $LODscore + 0.1;
			}
			
			$multiCount++;
			$totalMarkers++;
			
			$resultDir = "$projectDir\/datafile_${markerNum}_${multiCount}\/pedfile_$pedInd";
			$nullResultDir = "$projectDir\/datafile_${markerNum}_0\/pedfile_$pedInd";
			$resultFilename = "$resultDir\/result_condor";
			$nullResultFilename = "$nullResultDir\/result_condor";
			
			unless ( (-e $resultFilename) && (-e $nullResultFilename) ) {	
				$markerNum++;
				$resultDir = "$projectDir\/datafile_${markerNum}_1\/pedfile_$pedInd";
				$nullResultDir = "$projectDir\/datafile_${markerNum}_0\/pedfile_$pedInd";
				$resultFilename = "$resultDir\/result_condor";
				$nullResultFilename = "$nullResultDir\/result_condor";
				
				$multiCount = 1;
			}
		}
		
		if ($totalMarkers > $markerNum) { 		#May happen only for multi
			#$markerNum = $totalMarkers + 2;		#In two point we add an extra 1 to the number - so we simulate it here
			$markerNum = $lastMarker -$firstMarker + 2;
		}
		&debug_print("Decide if a graph should be printed : flag = $lastPedFlag first = $firstMarker last = $lastMarker marker = $markerNum total = $totalMarkers");
		if ( ( $lastPedFlag == 0 ) && (( $firstMarker - 1 + $markerNum - 1 == $lastMarker) || ($markerNum >= $#{$variables { usedMarkersId }} + 1) ) ){
			#run gnuplot commands for last small graph
			$markerNum--;
			if ( $variables{ program } =~ /MultiPoint/ ) {
				$start = &getMultiMarkerID($start - 1);
				$markerNum = &getMultiMarkerID($markerNum - 1);
			}
			$smallGraphFilename = "$projectDir\/LODgraph_${start}_${markerNum}.pedfile_$pedInd.png";
			open( COMMANDS_FILE , ">gnuplotCommandsSmall_$pedInd.txt" );
			print COMMANDS_FILE "set terminal png\n";
			print COMMANDS_FILE "set output \"$smallGraphFilename\"\n";
			print COMMANDS_FILE "f(x) = 3.3\n";
			print COMMANDS_FILE "g(x) = -2\n";
			print COMMANDS_FILE "set xrange [$start:$markerNum]\n";
			if( $updateFlag == 0 ){
				$graphNumMarkers = $markerNum;
				$updateFlag = 1;
			}
			print COMMANDS_FILE "set yrange [$graphMin:$smallMaxValue]\n";
			print COMMANDS_FILE "set multiplot\n";
			print COMMANDS_FILE "set pointsize 1\n";
			print COMMANDS_FILE "plot \"$SmallGraphInputFilename\" with linespoints pt 5 notitle\n";
			print COMMANDS_FILE "plot f(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "plot g(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "unset multiplot\n";
			close( COMMANDS_FILE );
			system("gnuplot gnuplotCommandsSmall_$pedInd.txt");				
			#run gnuplot commands for big graph
			open( COMMANDS_FILE , ">gnuplotCommands_$pedInd.txt" );
			print COMMANDS_FILE "set terminal png\n";
			print COMMANDS_FILE "set output \"$graphFilename\"\n";
			print COMMANDS_FILE "f(x) = 3.3\n";
			print COMMANDS_FILE "g(x) = -2\n";
			if ( $variables{ program } =~ /MultiPoint/ ) {
				print COMMANDS_FILE "set xrange [$start:$markerNum]\n";
			}
			else {
				print COMMANDS_FILE "set xrange [0:$markerNum]\n";
			}
			print COMMANDS_FILE "set yrange [$graphMin:$maxValue]\n";
			print COMMANDS_FILE "set multiplot\n";
			print COMMANDS_FILE "set pointsize 1\n";
			print COMMANDS_FILE "plot \"$graphInputFilename\" with linespoints pt 5 notitle\n";
			print COMMANDS_FILE "plot f(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "plot g(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "unset multiplot\n";
			close( COMMANDS_FILE );
			system("gnuplot gnuplotCommands_$pedInd.txt");	

			#*******************************************************************************
			#   Small graph for preview
			#*******************************************************************************
			$graphInputSmallfilename = "$projectDir\/graphInputSmall.pedfile_$pedInd.txt";
			open INPUTSMALL, ">$graphInputSmallfilename";
			open INPUT, "$graphInputFilename";
			for $inputLine (<INPUT>) {
				@lineData = split /\s+/ , $inputLine;
				if ($lineData[1] > $graphMax) {
					print INPUTSMALL "$lineData $graphMax\n";
				}
				else {
					print INPUTSMALL "$inputLine";
				}
			}
			close INPUT;
			close INPUTSMALL;
			
			open( COMMANDS_FILE , ">gnuplotCommands_$pedInd.txt" );
			print COMMANDS_FILE "set terminal png\n";
			print COMMANDS_FILE "set size 0.5,0.5\n";
			$previewGraphName = $graphFilename;
			$previewGraphName =~ s/.png/_preview.png/g;
			print COMMANDS_FILE "set output \"$previewGraphName\"\n";
			print COMMANDS_FILE "f(x) = 3.3\n";
			print COMMANDS_FILE "g(x) = -2\n";
			if ( $variables{ program } =~ /MultiPoint/ ) {
				print COMMANDS_FILE "set xrange [$start:$markerNum]\n";
			}
			else {
				print COMMANDS_FILE "set xrange [0:$markerNum]\n";
			}
			print COMMANDS_FILE "set yrange [$graphMin:$graphMax]\n";
			print COMMANDS_FILE "set multiplot\n";
			print COMMANDS_FILE "set pointsize 1\n";
			print COMMANDS_FILE "plot \"$graphInputSmallfilename\" with lines pt 5 notitle\n";
			print COMMANDS_FILE "plot f(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "plot g(x) with lines lt 3 notitle\n";
			print COMMANDS_FILE "unset multiplot\n";
			close( COMMANDS_FILE );
			system("gnuplot gnuplotCommands_$pedInd.txt");
			#*******************************************************************************
			
			#create HTML file
			open(HTMLFILE,">$projectDir\/graphData.pedfile_$pedInd.html");
			print HTMLFILE "<html>";
			print HTMLFILE "<head>";
			print HTMLFILE "<title> Graph and Clustering Mapping </title>";
			print HTMLFILE "</head>";
			print HTMLFILE "<body>";
			print HTMLFILE "<h1><u><b> Results of $variables{ program } Analysis for Project $variables{ PROJ } </b></u></h1><br>\n";			
			print HTMLFILE "<center><img src=\"$graphFilename\" /></center><br><br>\n";
			if( -e $mappingFile ){
				open(MAPPING,"$mappingFile");
				for $mapline (<MAPPING>){
					if ($mapline =~ /\[VALUE MAPPING\]/){
						last;
					}
					print HTMLFILE "$mapline<br>\n";
				}				
			}
			else{
				print HTMLFILE "mapping file is missing<br>\n";
			}
			print HTMLFILE "</body>";
			print HTMLFILE "</html>";
			close(HTMLFILE);
		}
		$pedInd++;
		$totalMarkers = 0;
	}
	$graphLastPed = $pedInd - 2;
}

################################################################
sub setClusteringVariables(){
	$TwoPointNUM = 1;
	$clusteringPost = 1;
}

################################################################
sub makeOuputFrameset(  ){
	my $projectDir = $RUN_DIR;
	
	my $fileList = "fileList.txt";

	system("mv $OUTPUT_FILE $OUTPUT_RESULTS_FILE");
	
	open( OUTPUT_FILE,">$OUTPUT_FILE");
	print OUTPUT_FILE "<html>\n";
	print OUTPUT_FILE "<frameset cols=\"20%, 80%\">\n";
	print OUTPUT_FILE "\t<frame src=\"$OUTPUT_NAVIGATION_FILE\" name=\"menu\">\n";
	print OUTPUT_FILE "\t<frame src=\"$OUTPUT_RESULTS_FILE\" name=\"main\">\n";
	print OUTPUT_FILE "\t<noframes>\n";
	print OUTPUT_FILE "\t\tyou do not support the use of frames. please press the following link to see the results<br>\n";
	print OUTPUT_FILE "\t\t<a href=\"$OUTPUT_RESULTS_FILE\">Results</a>\n";
	print OUTPUT_FILE "\t</noframes>\n";
	print OUTPUT_FILE "</frameset>\n";
	print OUTPUT_FILE "</html>\n";
	close(OUTPUT_FILE);
	
	open( NAV_FILE,">$OUTPUT_NAVIGATION_FILE");
	print NAV_FILE "<a href=\"$OUTPUT_RESULTS_FILE#Graph\" target =\"main\">Graph</a><br>\n";
	print NAV_FILE "<a href=\"$OUTPUT_RESULTS_FILE#Table\" target =\"main\">Table Results</a><br>\n";
	print NAV_FILE "<a href=\"$OUTPUT_RESULTS_FILE#Mapping\" target =\"main\">ClusteringMapping</a><br>\n";
	print NAV_FILE "<a href=\"$OUTPUT_RESULTS_FILE#Zoom\" target =\"main\">Graph Zooming</a><br>\n";
	close(NAV_FILE);
	
	system("ls $projectDir | grep .png > $fileList");
	system("ls $projectDir | grep graphInput.pedfile_*.txt >> $fileList");
	#add frameset and also add it to the fileList
	open(FILELIST,">>$fileList");
	print FILELIST "$OUTPUT_NAVIGATION_FILE\n";
	print FILELIST "$OUTPUT_RESULTS_FILE\n";
	print FILELIST "$OUTPUT_FILE\n";
	print FILELIST "$BED_OUTPUT_FILE\n";
	print FILELIST "$COLLECT_DATA_FILE\n";
	close(FILELIST);	
	system("tar -C $projectDir -T $fileList -czf formatted_output.tar.gz");	
}
################################################################

######################################################################## Sliding Window Functions ################################################

################################################################
# 				isValidWindow
#	Checks whether the sliding window can be performed in the given file
#
#	Parameters:
#					$_[0] - start marker
#					$_[1] - end marker
#					$_[2] - window size
#					$_[3] - datafile path
#	Returns:
#					"OK" - If the window is OK
#					"clean file size is X" - Otherwise
################################################################
sub isValidWindow( $ $ $ $ ) {
	my $start = $_[0];
	my $end = $_[1];
	my $jump = $_[2];
	my $file = $_[3];
	
	exit &makeSystemError("Failed opening file ($file) to read", \%variables )
		unless ( open IN, $file );
	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	if ( ($start > $size) || ($end > $size) || ($start + $jump - 1 > $end) || ($start > $end) || ($start < 1) || ($end < 1) ) {
		return "Clean file size is ".$size;
	}
	
	return "OK";
}

################################################################
# 				getMarkerWindowDataFile
#	Get's the Datafile, that will contain only $_[1] markers 
#	starting from given index
#
#	Parameters:
#					$_[0]  - Marker index
#					$_[1]  - Number of markers to be used
#					$_[2]  - requested end marker
#					$_[3]  - Datafile
#					$_[4]  - Project directory 
#							(to place data that will be needed in the post)
#					$_[5]  - program code
#	Returns:
#					Variable with datafile
################################################################
sub getMarkerWindowDataFile( $ $ $ )
{
	my $number = $_[0];
	my $count = $_[1];
	my $end = $_[2];
	my $file = $_[3];
	my $projectDir = $_[4];
	my $code = $_[5];
	&Processing::debug_print("Get Marker Data File with index ($number) and file ($file)" );

	my $result = "";
	open IN, $file;

	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	#my $size = $end; #$words_first_line[0] - 1;
	
	my $datafile = join "\n", @data;
	if ($data[0] =~ /\n/) {
		$datafile = join "", @data;
	}
	exit $Processing::ERROR if ( $Processing::ERROR =~ $datafile );
	@data = split "\n", $datafile;

	my $line = shift @data;		#	<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI
	
	$words_first_line[0] = $count + 1;
	$words_first_line[3] = $code;
	$line = join ' ', @words_first_line;
	
	$result = $result.$line."\n";
	$line = shift @data;			#	<< MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
	$result = $result.$line."\n";
	$line = shift @data;			#	<< 1 2
	
	for( my $i = 1; $i <= $count + 1; $i ++ ) {
		$result = $result."$i ";
	}
	$result = $result."\n";
	
	$line = shift @data;

	my $extra = 0;
	if ($end - $number < $count)
	{
		$extra = $count - ($end - $number);
	}
	
	if ($extra > 0) { 	
		open (EXTRADATA, ">>$projectDir/extra.dat") or die "Can't open log file 3\n";
		my $fileMarker = $number + 1;
		print EXTRADATA "$fileMarker has $extra \n";
		close EXTRADATA;
	}	
	
	while ( ( $line !~ /ALLELE NUMBERS, NO. OF ALLELES/ ) && ( $#data >= 0) )
	{
		$result = $result.$line."\n";
		$line = shift @data;
	}
	
	for( my $i = 0; $i < $number - $extra; $i ++ )
	{
		$line = shift @data;		#	 << GENE FREQUENCIES
		$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
	}
	
	for( my $i = 0; $i < $count; $i ++ )
	{		
		$result = $result.$line."\n";	# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		$line = shift @data;	
		$result = $result.$line."\n";	#	 << GENE FREQUENCIES
		$line = shift @data;
	}
	
	while ( ( $line =~ /ALLELE NUMBERS, NO. OF ALLELES/ ) && ( $#data >= 0) )
	{
		$line = shift @data;
		$line = shift @data;
	}
	$result = $result.$line."\n";			#		<< SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	$line = shift @data;					#		<< RECOMBINATION VALUES
	@words = split ' ', $line;
	
	my $start = $number - $extra;
	my $prev = $start;
	for (my $i = $start; $i < $start + $count; $i++ )
	{
		my $distCM = &getDistFromStart( $i, $file) - &getDistFromStart( $prev, $file); 
		my $distRec = &getThetaFromCM($distCM);
		$prev = $i;
		$result = $result."$distRec ";
	}
	$result = $result." << RECOMBINATION VALUES\n";
	
	$line = shift @data;		#	 << REC VARIED, INCREMENT, FINISHING VALUE
	$result = $result.$line."\n";
	
	&Processing::debug_print("getMarkerDataFile: Result=($result)");
	return $result;
}
################################################################



################################################################
# 				getMarkerWindowPedFile
#	Get's the Pedfile, that will contain only marker with given index
#
#	Parameters:
#					$_[0]  - Marker index
#					$_[1]  - Number of markers in group
#					$_[2]  - Requested end marker
#					$_[3]  - Pedfile
#	Returns:
#					Variable with Pedfile
################################################################
sub getMarkerWindowPedFile( $ $ $ )
{
	my $number = $_[0];
	my $count = $_[1];
	my $end = $_[2];
	my $file = $_[3];
	
	&Processing::debug_print("Get Marker Ped File with index ($number) and file ($file)" );
	
	my $result = "";
	open IN, $file;

	my @lines = <IN>;
	close IN;
	
	my @testWords = split "<<", $lines[0];
	@testWords = split /\s+/, $testWords[0];
	
	my $total = 10 + $end*2;
	my $startPos = 10;
	if ( $#testWords % 2 == 0 ) {
		$startPos++;
		$total++;
	}
		

	#my $leftSize = $#testWords - $startPos - 2*$number + 1;
	my $leftSize = $total - $startPos - 2*$number;
	
	my $extra = 0;
	if ($leftSize < 2*$count)
	{
		$extra = 2*$count - $leftSize;
	}

	foreach my $line ( @lines )
	{
		my @words = split /\s+/, $line;
		splice ( @words, $startPos, 2*$number-$extra);
		splice( @words, $startPos + 2*$count );
		$line = join ' ', @words;
		$result = $result.$line."\n";
	}

	&Processing::debug_print("getMarkerPedFile: Result=($result)");
	return $result;
}
################################################################	

################################################################
# 				getMarkersListDataFile
#	Get's the Datafile, that will contain only the markers 
#	specified by the array
#
#	Parameters:
#					$_[0]  - Markers array
#					$_[1]  - Datafile
#					$_[2]  - program code
#	Returns:
#					Variable with datafile
################################################################
sub getMarkersListDataFile
{
	my $code = pop @_;
	my $file = pop @_;
	my @markers = @_;
	my $count = $#markers + 1;
	
	&Processing::debug_print("Get Marker Data File with (@markers) and file ($file)" );

	my $result = "";
	open IN, $file;

	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	my $datafile = join "\n", @data;
	if ($data[0] =~ /\n/) {
		$datafile = join "", @data;
	}
	exit $Processing::ERROR if ( $Processing::ERROR =~ $datafile );
	@data = split "\n", $datafile;

	my $line = shift @data;		#	<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI
	
	$words_first_line[0] = $count + 1;
	$words_first_line[3] = $code;
	$line = join ' ', @words_first_line;
	
	$result = $result.$line."\n";
	$line = shift @data;			#	<< MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
	$result = $result.$line."\n";
	$line = shift @data;			#	<< 1 2
	
	for( my $i = 1; $i <= $count + 1; $i ++ ) {
		$result = $result."$i ";
	}
	$result = $result."\n";
	
	$line = shift @data;	
	
	while ( ( $line !~ /ALLELE NUMBERS, NO. OF ALLELES/ ) && ( $#data >= 0) )
	{
		$result = $result.$line."\n";
		$line = shift @data;
	}
	
	my $startFrom = 0;
	
	for (my $index = 0; $index <= $#markers; $index++) {
		
		my $lookFor = $markers[$index] - 1;
		
		for( my $i = $startFrom; $i < $lookFor; $i++ )
		{
			$line = shift @data;		#	 << GENE FREQUENCIES
			$line = shift @data;		# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		}
		
		$startFrom = $lookFor + 1;
		
		$result = $result.$line."\n";	# 	<<  ALLELE NUMBERS, NO. OF ALLELES
		$line = shift @data;	
		$result = $result.$line."\n";	#	 << GENE FREQUENCIES
		$line = shift @data;
	}
	
	while ( ( $line =~ /ALLELE NUMBERS, NO. OF ALLELES/ ) && ( $#data >= 0) )
	{
		$line = shift @data;
		$line = shift @data;
	}
	$result = $result.$line."\n";			#		<< SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	$line = shift @data;					#		<< RECOMBINATION VALUES
	@words = split ' ', $line;
	
	my $prev = 0;
	for (my $i = 0; $i <= $#markers; $i++) {
		
		my $distCM = &Processing::getDistFromStart( $markers[$i] - 1, $file) - &Processing::getDistFromStart( $markers[ $prev ] - 1, $file );
		my $distRec = &Processing::getThetaFromCM($distCM);
		$prev = $i;
		$result = $result."$distRec ";
	
		#$result = $result."$words[ $markers[$i] - 1 ] ";
	}
	$result = $result." << RECOMBINATION VALUES\n";
	
	$line = shift @data;		#	 << REC VARIED, INCREMENT, FINISHING VALUE
	$result = $result.$line."\n";
	
	$result = $result."-n 2 1 ".($words_first_line[0] - 1)."	 << REC VARIED, INCREMENT, FINISHING VALUE\n";
	
	&Processing::debug_print("getMarkerDataFile: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				getMarkersListPedFile
#	Get's the Pedfile, that will contain only the markers 
#	specified by the array
#
#	Parameters:
#					$_[0]  - Markers array
#					$_[2]  - Pedfile
#	Returns:
#					Variable with Pedfile
################################################################
sub getMarkersListPedFile
{
	my $file = pop @_;
	my @markers = @_;
	my $count = $#markers + 1;
	
	&Processing::debug_print("Get Marker Ped File with (@markers) and file ($file)" );
	
	my $result = "";
	open IN, $file;

	my @lines = <IN>;
	close IN;

	my @testWords = split "<<", $lines[0];
	@testWords = split /\s+/, $testWords[0];
	my $startPos = 10;
	$startPos++ if ( $#testWords % 2 == 0 );

	foreach my $line ( @lines )
	{
		my @words = split /\s+/, $line;
		$result = $result.$words[0];
		
		for (my $i = 1; $i < $startPos; $i++) {
			$result = $result." ".$words[ $i ];
		}
		
		for (my $i = 0; $i <= $#markers; $i++) {
			my $markerIndex = $startPos + 2*($markers[$i] - 1);
			$result = $result." ".$words[ $markerIndex ]." ".$words[ $markerIndex + 1 ];
		}
		$result = $result."\n";
	}

	&Processing::debug_print("getMarkerPedFile: Result=($result)");
	return $result;
}
################################################################

################################################################
# 				isValidMarkersList
#	Checks whether the given marker exists in the given file
#
#	Parameters:
#					$_[0] - markers array
#					$_[1] - datafile path
#	Returns:
#					"OK" - If the window is OK
#					"clean file size is X" - Otherwise
################################################################
sub isValidMarkersList( $ $ ) {
	my $file = pop @_;
	my @markers = @_;
	
	exit &makeSystemError("Failed opening file ($file) to read", \%variables )
		unless ( open IN, $file );
	my @data = <IN>;
	close IN;

	my @words_first_line = split ' ', $data[0];
	my $size = $words_first_line[0] - 1;
	
	foreach $marker (@markers) {
		if ( ($marker > $size) || ($marker < 1) ) {
			return "Clean file size is ".$size;;
		}
	}
	
	return "OK";
}	

################################################################
# 				getMultiMarkerID
#	Returns the correct marker id according to the number of output line
#
#	Parameters:
#					$_[0] - line number in output table
#	Returns:
#					The correct ID
################################################################
sub getMultiMarkerID ( $ ) {
	my $multiLineNum = $_[0];
	
	if ( open IN, "$RUN_DIR/multiParams.dat" )  #It was a window
	{
		my @multiData = <IN>;
		close IN;
		my @words = split ' ', $multiData[0];					
		$markerID = $multiLineNum + $words[0];
	}
	else {
		if ( open IN, "$RUN_DIR/multiList.dat" ) #It was a list
		{
			my @multiData = <IN>;
			close IN;

			my @words = split ' ', $multiData[0];
			$markerID = $words[$multiLineNum];
		}
	}

	return $markerID;
}



##########################################################################

####################################################################################
#
# 			Segregation functions
#
####################################################################################

################################333
#
# create pene and freq for test
####################################
sub setTestPoints ( $ ) {
	my $program = $_[0];
	my $i =0;
	if ( $program =~ /LongFreq/) {
		@frequences = @frequencesLong;
		for ($i = 0; $i < $#penetranceDomLong + 1; $i++) {
			push @peneAll,$penetranceDomLong[$i];
			push @peneAll,$penetranceRecLong[$i];
		}
	} elsif ( $program =~ /MediumFreq/) {
		@frequences = @frequencesMedium;
		for ($i = 0; $i < $#penetranceDomMedium + 1; $i++) {
			push @peneAll,$penetranceDomMedium[$i];
			push @peneAll,$penetranceRecMedium[$i];
		}
	} elsif ($program =~ /QuickFreq/) {
		@frequences = @frequencesQuick;
		for ($i = 0; $i < $#penetranceDomQuick + 1; $i++) {
			push @peneAll,  $penetranceDomQuick[$i], ;
			push @peneAll,$penetranceRecQuick[$i];
			for (my $k = 0; $k < $#frequences + 1 ; $k++) {
				$freqPrev{"pen = $penetranceDomQuick[$i] prev = $frequencesQuick[$k]"} = &getFreqFromPrev($penetranceDomQuick[$i],$frequencesQuick[$k],"1");
				$freqPrev{"pen = $penetranceRecQuick[$i] prev = $frequencesQuick[$k]"} = &getFreqFromPrev($penetranceRecQuick[$i],$frequencesQuick[$k],"0");
			}
		}
	}
	if ( $program =~ /LongPrev/) {
		@frequences = @frequencesLong;
		for ($i = 0; $i < $#penetranceDomLong + 1; $i++) {
			push @peneAll,$penetranceDomLong[$i];
			push @peneAll,$penetranceRecLong[$i];
			for (my $k = 0; $k < $#frequences + 1 ; $k++) {
				$freqPrev{"pen = $penetranceDomLong[$i] prev = $frequencesLong[$k]"} = &getFreqFromPrev($penetranceDomLong[$i],$frequencesLong[$k],"1");
				$freqPrev{"pen = $penetranceRecLong[$i] prev = $frequencesLong[$k]"} = &getFreqFromPrev($penetranceRecLong[$i],$frequencesLong[$k],"0");
		  }
		}
	} elsif ( $program =~ /MediumPrev/) {
		@frequences = @frequencesMedium;
		for ($i = 0; $i < $#penetranceDomMedium + 1; $i++) {
			push @peneAll,$penetranceDomMedium[$i];
			push @peneAll,$penetranceRecMedium[$i];
			for (my $k = 0; $k < $#frequences + 1 ; $k++) {
				$freqPrev{"pen = $penetranceDomMedium[$i] prev = $frequencesMedium[$k]"} = &getFreqFromPrev($penetranceDomMedium[$i],$frequencesMedium[$k],"1");
				$freqPrev{"pen = $penetranceRecMedium[$i] prev = $frequencesMedium[$k]"} = &getFreqFromPrev($penetranceRecMedium[$i],$frequencesMedium[$k],"0");
			}
		}
	} elsif ($program =~ /QuickPrev/) {
		@frequences = @frequencesQuick;
		for ($i = 0; $i < $#penetranceDomQuick + 1; $i++) {
			push @peneAll,$penetranceDomQuick[$i];
			push @peneAll,$penetranceRecQuick[$i];
			for (my $k = 0; $k < $#frequences + 1 ; $k++) {
				$freqPrev{"pen = $penetranceDomQuick[$i] prev = $frequencesQuick[$k]"} = &getFreqFromPrev($penetranceDomQuick[$i],$frequencesQuick[$k],"1");
				$freqPrev{"pen = $penetranceRecQuick[$i] prev = $frequencesQuick[$k]"} = &getFreqFromPrev($penetranceRecQuick[$i],$frequencesQuick[$k],"0");
			}
		}
	}
}

#########################################33
#
# Get frequency
#
#  q - probability of risk allele
#  f -  penetrance
#  k - prevalance
#Dominant
#       f +- sqrt(f^2 - f*k)
#  q = ----------------------
#              f
#
#Reccesive
# q = sqrt(k/f)
#$#########################################3
sub getFreqFromPrev{
	my ($pene, $prev, $dom) = @_;
	my @tmp = split(/ /,$pene);
	$pene =  $tmp[2];
	@tmp =  split(/ /, $prev);
	$prev = $tmp[1];
	my $freqAff = 0;
	if ($dom eq "1") {
		$freqAff =  ($pene + sqrt( $pene*$pene - $prev*$pene))/ $pene;
		if ($freqAff > 1) {
			$freqAff =  ($pene - sqrt( $pene*$pene - $prev*$pene))/ $pene;
		}
	} else {
		$freqAff = sqrt($prev / $pene);
	}
	return "".(1-$freqAff)." ".$freqAff;
}
##############################################


####################################################################################
#
# 			BEDcreator functions
#
####################################################################################

################################################################
#		getBEDPositions
#	Returns the position to be used in the BED file
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					An array with the start and end positions
################################################################
sub getBEDPositions( \% )
{
	%variables = %{ $_[0] };
	
	@result = ();
	@positions = @{$variables { positions }}; 
	if ($#positions == -1) {
		return @result;
	}
	
	@clustersNamesLines = `grep "$MARKER_ROW_REG_EXP_PREFIX" ..\/$variables{datafile}`;
	
	if ( ( defined($variables{ jump }) && ( $variables{ jump } !~ /Unknown/ ) ) 
	  && ( defined($variables{ from }) && ( $variables{ from } !~ /Unknown/ ) ) ) {
		$start = $variables{ from } - 1;
		$end = $variables{ to } - 1;	
	}
	else {
		@markersArray  = sort {$a <=> $b} @{$variables { usedMarkersId }};
		$start = $markersArray[0] - 1;
		$end = $markersArray[$#markersArray] - 1;
	}
	
	@clustersNames = split /#/, $clustersNamesLines[$start];
	@markersNames = split /\([0-9]+\)[ ]*/, $clustersNames[1];
	$markerName = $markersNames[0];
	@markerData = grep(/$markerName :/, @positions);
	@splitData = split / : /, $markerData[0];
	push(@result, $splitData[1]);
	
	@clustersNames = split /#/, $clustersNamesLines[$end];
	@markersNames = split /\([0-9]+\)[ ]*/, $clustersNames[1];
	if ($#markersNames > 0) {
		$markerName = $markersNames[$#markersNames];
	}
	else {
		$markerName = $markersNames[0];
	}
	@markerData = grep(/$markerName :/, @positions);
	@splitData = split / : /, $markerData[0];
	push(@result, $splitData[1]);
	
	return @result;
}
################################################################

################################################################
#		createBEDfile
#	Creates a new BED file and returns its name
#
#	Parameters:
#					$_[0] - chromosome number
#					$_[1] - start position
#					$_[2] - end position
#	Returns:
#					BED file name
################################################################
sub createBEDfile( $ $ $ ){
	$chromNum	= $_[0];
	$from		= $_[1];
	$to			= $_[2];#+51000;
	
	open(BED,">$RUN_DIR/$BED_OUTPUT_FILE");
	print BED "browser position chr$chromNum:$from-$to\n";
	print BED "browser hide all\n";
	print BED "browser dense ruler\n";
	print BED "browser pack recombRate\n";
	print BED "browser full knownGene\n";
	print BED "track name=\"LODscore\" description=\"LOD Score\" visibility=3 itemRgb=\"On\"\n";
	close (BED);
	
	return $BED_OUTPUT_FILE;
}
################################################################

################################################################
#  addMarkerLODscore
# Adds a marker and its LOD score to an existing BED file
#
# Parameters:
#     $_[0] - BED file name
#     $_[1] - chromosome number
#     $_[2] - marker position
#     $_[3] - marker's LOD score
# Returns:
#     0 on success, -1 otherwise
################################################################
sub addMarkerLODscore( $ $ $ $ ){
	$BEDfile		= $_[0];
	$chromNum		= $_[1];
	$markerPos		= $_[2];
	$markerPosEnd	= $markerPos + $CUBE_BASE_SIZE;
	$LOD			= $_[3];
	$color			= getLODcolor($LOD);

	if (!(-e $BEDfile)) { print "$BEDfile does not exists, run createBEDfile(...) first"; return -1; }
	open(BED,"<$BEDfile");
	@lines = <BED>;
	close(BED);
	@exists = grep (/chr$chromNum\t$markerPos\t$markerPosEnd\t$LOD\t0\t.\t$markerPos\t$markerPosEnd\t$color/, @lines);
	if ($#exists >= 0) {
		print "line already exists\n";
		return -1;
	}
	open(BED,">>$BEDfile");
	#print BED "chr$chromNum $markerPos $markerPosEnd $LOD 0 . $markerPos $markerPosEnd $color\n";
	print BED "chr$chromNum\t$markerPos\t$markerPosEnd\t$LOD\t0\t.\t$markerPos\t$markerPosEnd\t$color\n";
	close (BED);

	return 0;
}
################################################################

################################################################
#  addClusterLODscore
# Adds a cluster and its LOD score to an existing BED file
#
# Parameters:
#     $_[0] - BED file name
#     $_[1] - chromosome number
#     $_[2] - cluster start position
#     $_[3] - cluster end position
#     $_[4] - cluster's LOD score
# Returns:
#     0 on success, -1 otherwise
################################################################
sub addClusterLODscore( $ $ $ $ $ ){
	$BEDfile			= $_[0];
	$chromNum			= $_[1];
	$clusterPosStart	= $_[2];
	#$clusterPosEnd		= ($_[3] == $clusterPosStart) ? $clusterPosStart + $CUBE_BASE_SIZE : $_[3];
	$clusterPosEnd		= $_[3];
	$LOD				= $_[4];
	$color				= getLODcolor($LOD);

	if (!(-e $BEDfile)) { print "$BEDfile does not exists, run createBEDfile(...) first"; return -1; }
	open(BED,"<$BEDfile");
	@lines = <BED>;
	close(BED);
	@exists = grep (/chr$chromNum\t$clusterPosStart\t$clusterPosEnd\tC:$LOD\t0\t.\t$clusterPosStart\t$clusterPosEnd\t$color/, @lines);
	if ($#exists >= 0) {
		print "line already exists\n";
		return -1;
	} 
	open(BED,">>$BEDfile");

	print BED "chr$chromNum\t$clusterPosStart\t$clusterPosEnd\tC:$LOD\t0\t.\t$clusterPosStart\t$clusterPosEnd\t$color\n";
	close (BED);

	return 0;
}
################################################################

################################################################
#		getLODcolor
#	returns color of a LOD score in an RGB format
#
#	Parameters:
#					$_[0] - LOD score
#	Returns:
#					color in RGB format
################################################################
sub getLODcolor( $ ){
	$LOD = $_[0];

	$color = int(255 * ( $LOD / $MAX_LOD ));
	if ($color > 255) { $color = 255; }
	if ($color < -255) { $color = -255; }
	
	$RGBcolor = "50,".int($color/1.7).",50";
	if ( $color < 0 ) {
		$color = $color*(-1);
		$RGBcolor = "$color,100,100";
	}
	if ( $LOD > 2 ) {	$RGBcolor = "0,".int($color/1.7).",0"; }
	
	return $RGBcolor;
}
################################################################

################################################################
#  addMarkerData
# Adds a marker and its LOD score to the data collection file
#
# Parameters:
#     	$_[0] - marker (or cluster) id 
#     	$_[1] - marker name (or cluster string)
#     	$_[2] - distance from first marker
#		$_[3] - LOD score
################################################################
sub addMarkerData( $ $ $ $ ){
	$id			= $_[0];
	$name		= $_[1];
	$dist		= $_[2];
	$LOD		= $_[3];

	open(DATA,"<$RUN_DIR\/$COLLECT_DATA_FILE");
	@lines = <DATA>;
	close(DATA);
	@exists = grep (/$id\t$name\t$dist\t$LOD\n/, @lines);
	if ($#exists >= 0) {
		return -1;
	} 
	open(DATA,">>$RUN_DIR\/$COLLECT_DATA_FILE");
	print DATA "$id\t$name\t$dist\t$LOD\n";
	close (DATA);

	return 0;
}
################################################################


1;
