# dvir 2011/10/26
package Segregation;

use strict;
use Configuration;
use General;
use DatWrapper;
use PedWrapper;
use ProcessingBackend;

my $CURR_SEGREGATION_COMPUTATION_OUTPUT_FILENAME = 'segregation.curr.out';
my $CURR_SEGREGATION_COMPUTATION_ERROR_FILENAME = 'segregation.curr.err';
my $MULTI_SEGREGATION_COMPUTATIONS_OUTPUT_FILENAME = 'segregation.multi.out';

my $HTML_OUTPUT_FILENAME = 'formatted_output.html';

my $pedFileDirName = 'pedfiles';

sub run() {
	my $projectName = $_[0];
	my $projectTaskDir = $_[1];
	my $workDir = $_[2];
	
	my $binDir = $Configuration::configfile{ binDir };
		
	chdir $workDir || &General::reportOnErrorReleaseSemaphoresAndDie('Segregation.pm',`pwd`, "run: \"chdir $workDir\" failed");
			
	# reading the variables from the user
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
	
	# the dat & ped files which were received from web server
	my $datFileName = "$projectTaskDir\/$variables{ datafile }";
	my $pedFileName = "$projectTaskDir\/$variables{ pedfile }";

	&General::makeSureFileExists($datFileName);
	&General::makeSureFileExists($pedFileName);
	
	my $datWrapper = new DatWrapper($datFileName);
	my $numOfMarkersInDatFile = $datWrapper->getNumOfMarkers();
	my $pedWrapper = new PedWrapper($pedFileName, 0);
	
	&ProcessingBackend::setTestPoints( $variables{ program } );
	
	# creating the dat files and running the executeable on them
	my $datFileDirName = 'datafiles';
	mkdir $datFileDirName || &General::reportOnErrorReleaseSemaphoresAndDie('Segregation.pm',`pwd`, "run: \"mkdir $datFileDirName\" failed");

	# ped for each family
	mkdir $pedFileDirName || &General::reportOnErrorReleaseSemaphoresAndDie('Segregation.pm',`pwd`, "run: \"mkdir $pedFileDirName\" failed");
	
	# creating a ped file for each family
	my @families = $pedWrapper->getFamiliesNumbers();
	
	# creating the estimation dir for each family and a ped file in each dir
	my @tempMarkersList = ();
	foreach my $currFamily (@families) {
		my @familiesList = ($currFamily);
		
		my $currFamilyPedFileName = &getPedNameByFamilyNumber($currFamily);
		open PEDFILE_FH, ">$currFamilyPedFileName";
		$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@tempMarkersList, \@familiesList, 1, $PedWrapper::WRITE_PERSON_DATA_AND_2_DUMMY_0_ALLELS);
		close PEDFILE_FH;		
	}
	
	my $peneIndex = -1;
	my $freqIndex = -1;
	
	my %lodByFamilyPenetranceAndFrequencyHash = ();
	foreach my $penetrance (@ProcessingBackend::peneAll) {
		foreach my $pene ($penetrance) {	
			$peneIndex++;
			$freqIndex = -1;
			foreach my $freq (@ProcessingBackend::frequences) {
				$freqIndex++;
				my $freqT;
				if ( $variables{ program } =~ /Prev/) {
					$freqT = $ProcessingBackend::freqPrev{"pen = $pene prev = $freq"};
				}
				else {
					$freqT = $freq;
				}
				
				my $currDatafileStr = &getDataFile( $pene, $freqT );
								
				my $currDatFileName = "$datFileDirName\/datafile_pene_${peneIndex}_freq_${freqIndex}.dat";
				open DATFILE_FH, ">$currDatFileName";
				print DATFILE_FH $currDatafileStr;
				close DATFILE_FH;
				
				# for each family we will run the results.
				foreach my $currFamily (@families) {
					my $currFamilyPedFileName = &getPedNameByFamilyNumber($currFamily);
					# running the segeregation over the current dat for the current family					
					my $command = "$Configuration::SEGREGATION_EXECUTEABLE $currDatFileName $currFamilyPedFileName > $CURR_SEGREGATION_COMPUTATION_OUTPUT_FILENAME 2> $CURR_SEGREGATION_COMPUTATION_ERROR_FILENAME";
					system($command);
					my $ret = $? >> 8;
					if ($ret != 0) {
						&General::reportOnErrorReleaseSemaphoresAndDie('Segregation.pm',`pwd`,"Segregation run : failed to preform : $command `cat $CURR_SEGREGATION_COMPUTATION_ERROR_FILENAME`");
					}
					
					$lodByFamilyPenetranceAndFrequencyHash{$currFamily}{$peneIndex}{$freqIndex} = &getLOD($CURR_SEGREGATION_COMPUTATION_OUTPUT_FILENAME);
					
					# backuping the results of the computation, for debug purpose
					system("cat $CURR_SEGREGATION_COMPUTATION_OUTPUT_FILENAME >> $MULTI_SEGREGATION_COMPUTATIONS_OUTPUT_FILENAME");
					system("echo \"$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR\" >> $MULTI_SEGREGATION_COMPUTATIONS_OUTPUT_FILENAME");
				}
			}
		}
	}
	
	&createOutput(\%variables, \@families, $workDir, \%lodByFamilyPenetranceAndFrequencyHash);
	
	
	my $dirAtWebServer = $variables{ newDir };
	my @outputFiles = ($HTML_OUTPUT_FILENAME);
	&General::sendFilesToWebServer(\@outputFiles, $dirAtWebServer);
	
	
	# this command will cause hap to recognize that the task has ended.
	# and the task will be removed from the file "TaskWaitingList" at hap.
	my @commands = ("touch $dirAtWebServer\/$Configuration::HAP_INDICATION_FILE_FOR_ANALYSIS_END");
	&General::executeSSHCommandsOnWebServer(\@commands);
	
	&JobStatus::markThatJobSucceeded($workDir);
	
	my @subJobsFolders = (); # there is no sub job
	# first 1 -> in order not to obtain lock
	# second 1 -> in order not to try yo update status.
	&General::markThatTaskIsFinished($projectName, \@subJobsFolders, 1, 1);
}
			
sub getPedNameByFamilyNumber($) {
	my $familyNum = $_[0];
	return "$pedFileDirName\/${familyNum}.ped";
}

################################################################
# 				getDataFile
#	Get's the Datafile, that will contain the speciefied frequency 
#	and penetrace 
#
#	Parameters:
#					$_[0]  - Penetrace
#					$_[1]  - Frequency
#	Returns:
#					Variable with datafile
################################################################
# dvir : copied from prepareSegFiles.pl
sub getDataFile( $ $ )
{
	my $pene = $_[0];
	my $freq = $_[1];
	&ProcessingBackend::debug_print("Get Data File with PENETRANCES ($pene) and FREQUENCIES ($freq)" );

	my $result = 
		"2 0 0 5 0 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI\n".
		"0 0.0 0.0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n".
		"1 2 \n".
		"1 2 << AFFECTION, NO. OF ALLELES\n".
		$freq ."<< GENE FREQUENCIES\n".
		"1 << NO. OF LIABILITY CLASSES \n".
		$pene ."<< PENETRANCES \n".
		"3 2 #m1# << ALLELE NUMBERS, NO. OF ALLELES \n".
		"0.5 0.5  << GENE FREQUENCIES \n".
		"0 0 << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2) \n".
		"0.5 << RECOMBINATION VALUES \n".
		"1 0.1 0.45  << REC VARIED, INCREMENT, FINISHING VALUE \n";
		
	&ProcessingBackend::debug_print("getDataFile: Result=($result)");
	return $result;
}



# dvir : copied from postSnpSeg.pl
sub getLOD ( $ ) {
	my $resultFile = $_[0];
	
	my $LODtotal =  "-Infinity";
	
	open(INFO, "< $resultFile" ) || return $LODtotal; # Open the file 
	my @lines = <INFO> ;		# Read it into an array
	close(INFO) ;			# Close the file
	
	
	for (my $numLine = 0; $numLine <= $#lines; $numLine++ ) {	      # assign @lines to $line, one at a time
		
		if ($lines[$numLine] =~ /TOTAL/ ) {
			my @LOD_LINE = split(/\s+/,$lines[$numLine]); 		#format is "TOTAL   LN_LIKE    LOG_LIKE" 
			$LODtotal =  $LOD_LINE[2];						# If the result has more than one "TOTAL" line (more than one iteration) 
															# it will have a summary from which we should take the final result
		}
		
		if ($lines[$numLine] =~ /Trait/ ) {
			my @LOD_LINE = split(/\s+/,$lines[$numLine + 4]); 	#format is "  Marker | Trait Position (cM) |   LN(LIKE)  | LOD_SCORE
															# 			   Name  |  (from marker #x)   |             |
			$LODtotal =  $LOD_LINE[3];
		}
	}
	
	# If we didn't return we didn't have a summary line and there was only one "TOTAL" line whose value is returned here
	return $LODtotal;
}

# dvir: code was originally located at postSnpSeg.pl
# I have made few changes
sub createOutput () {
	
	my $variablesHashRef = $_[0];
	my $familiesArrRef = $_[1];
	my $workDir = $_[2];
	my $lodByFamilyPenetranceAndFrequencyHashRef = $_[3];
	
	my $program = $$variablesHashRef{ program };
	# &ProcessingBackend::setTestPoints( $$variablesHashRef{ program } ); # no need, cause it is done at run()

	open (OUT," >$HTML_OUTPUT_FILENAME") || die "Cannot open file output html\n";

	my $type = "<center><p><h3><b>Analysis By: "; 
	if ( $program =~ /Freq/  ) {
		$type = $type."Mutant gene frequency";
	}
	else {
		$type = $type."Population prevalence";
	}
	$type = $type."</br>Total likelihood evaluations: "; 
	if ( $program =~ /Quick/  ) {
		$type = $type."4";
	}
	if ( $program =~ /Medium/  ) {
		$type = $type."50";
	}
	if ( $program =~ /Long/  ) {
		$type = $type."120";
	}
	$type = $type."</b></h3></p></center>";

	print OUT '
	<html><head>
	<meta http-equiv="Content-Language" content="en-us">
	<meta http-equiv="Content-Type" content="text/html">
	<title>Segregation analysis</title>
	</head>

	<body bgcolor="#F0FFFF">
	<center><p><h1><u><b>Results of Segregation Analysis for Project '.$$variablesHashRef{ PROJ }.'</b></u></h1></p></center>'.$type.'
	<p align="left">Each entry in the table is the log<sub>10</sub> likelihood of the data (for all analyzed pedigrees together and for each pedigree alone) for a specific Mode Of Inheritance (MOI) and specific penetrance probabilities.When comparing two lines, one is checking the likelihood of a dominant MOI versus a recessive MOI for a specific penetrance value. The highest value, namely when the negative number is closest to zero, is the more likely MOI. A difference of 3 indicates that one MOI is more likely than the other by a factor of 1000. If this difference persists on the entire line, ';
	if ( $program =~ /Prev/ ) {
		print OUT 'meaning that it holds for all assumed possibilities for population prevalence, ';
	} else {
		print OUT 'meaning that it holds for all assumed frequencies for the mutant allele, ';
	}
	print OUT 'the one MOI is preferred over the other in somewhat robust way for the specified penetrance.</p>';
	my @familyMembers = sort {$a <=> $b} @$familiesArrRef;
	&drawTable( 0 , $program , $lodByFamilyPenetranceAndFrequencyHashRef, $familiesArrRef); # Draw all pedigrees

	if (scalar(@familyMembers) > 1) {
		print OUT '<br/><br/><br/>';
		for ( my $i = 0; $i <= $#familyMembers; $i++ ) {
			&drawTable( $familyMembers[ $i ] , $program , $lodByFamilyPenetranceAndFrequencyHashRef, $familiesArrRef);
			print OUT '<br/><br/><br/>';
		}
	}	
	close(OUT);
}

sub drawTable ( $ $ \% \@ ){
	my $pedNum = $_[0];
	my $program = $_[1];
	my $lodByFamilyPenetranceAndFrequencyHashRef = $_[2];
	my $familiesArrRef = $_[3];
	
	print OUT '<div align="center">
	<center>';
	
	if ($pedNum == 0) {
		print OUT '
		<b>All analyzed pedigrees combined</b>';
	}
	else {
		print OUT "
		<b> Pedigree $pedNum </b>";
	}
	print OUT '
		<div align="center">
		<center>
	   <table border="2" cellpadding="0" cellspacing="0" width="70%" id="AutoNumber1">
		<tr align="center" style="font-weight: normal; color: black; border-style: none; background-color: white" >
		  <td><b>penetrance \ ';
	  if ( $program =~ /Prev/ ) {
		print OUT "population prevelance";
	  } else {
		print OUT "mutant frequency";
	  }
	  print OUT '
	  <b></td>'."\n";
	  foreach my $freq ( @ProcessingBackend::frequences) {
		my @mutant = split(/ /,$freq);
		print OUT '<td><b>'.$mutant[1].'</b></td>'."\n";
	  }
	  print OUT '</tr> ';


	my $peneIndex = -1;
	my $freqIndex = -1;

	foreach my $penetrance (@ProcessingBackend::peneAll) {
		foreach my $pene ($penetrance) {	
			$peneIndex++;
			$freqIndex = -1;
			if (($peneIndex %4 ) < 2) {
				print OUT '<tr align="center"  style="font-weight: normal; color: black; border-style: none; background-color: white" ><td><b>';
			} else {
				print OUT '<tr align="center"  style="font-weight: normal; color: black; border-style: none; background-color: #D3D3D3"><td><b>'; # #EFFFEF
			}
			if (($peneIndex %2) == 0) {
				print OUT "dominant  ".$pene.'</b></td>';
			} else {
				print OUT "recessive ".$pene.'</b></td>';
			}
			foreach my $freq (@ProcessingBackend::frequences) {
				$freqIndex++;
				my $LODscore = 0.0;
				
				if ( $pedNum > 0 ) {					
					$LODscore = $$lodByFamilyPenetranceAndFrequencyHashRef{$pedNum}{$peneIndex}{$freqIndex};
				}
				# for all the families
				else {
					foreach my $fam (@$familiesArrRef) {

						my $currLOD = $$lodByFamilyPenetranceAndFrequencyHashRef{$fam}{$peneIndex}{$freqIndex};
						if ($currLOD !~ /-Infinity/) {
							$LODscore = $LODscore + $currLOD;
						}
						else {
							$LODscore = $currLOD;
							last;
						}
					}
				}
				$LODscore = sprintf ("%.1f", $LODscore);
				print OUT '<td  width=10% NOWRAP>'."$LODscore</td>\n";
			}
			print OUT '</tr> ';
		}
	}
	print OUT '</table> ';
}

1;