package SetupEnviourment;

# dvir 2011/10/10
use strict;

use FindBin ();
use lib "$FindBin::Bin";

use File::Path;

use Configuration qw(%configfile); 
use General;

# each key is the script name, and its value x is the defintion of how often to run the script.
my %crontabScriptsNames = (
	'TaskHandler.pl'    => '*/1 * * * *',
	'cleanWorkspace.pl' => '0 0 1,15 * *',
	'spaceMonitor.pl'   => '*/60 * * * *',
	'checkTasks.pl'     => '*/5 * * * *');

# those dirs should be deleted when all analyises will work with Analysis.pm
# but now, they are required. _STAGE2_
my @dirsToBeDeleted = (
	$configfile{runlogDir},
	$configfile{mailDir},
	$configfile{dataDir},
	$configfile{outputDir},
	$configfile{runlogDir},
	$configfile{currentlyRunningDir}
);
	
# the script will make sure that each dir in the list exists (and if not, it will create it
# the script can create a path.
my @dirList = (
	$Configuration::workDir,
	$Configuration::TASKS_DIR,
	$Configuration::ERRORS_DIR,
	$Configuration::LOCKS_DIR,
	$Configuration::LOGS_DIR,
	$Configuration::TASKS_BY_STATUS_DIR,
	$Configuration::TASKS_BY_WAIT_TO_RUN_STATUS_DIR,
	$Configuration::TASKS_BY_RUN_STATUS_DIR,
	@dirsToBeDeleted,
);

my $binDir = $configfile{binDir};

# this function makes sure that all we need to the enviourment exists.
# can be checked by running : 
# perl -e "use SetupEnviourment; &SetupEnviourment::runSetup();" ; crontab -l
sub runSetup() {

	# making sure that the needed dirs exists (or creating them)
	for my $dir (@dirList) {
		unless( -d $dir) {
			mkpath $dir || &General::reportOnErrorReleaseSemaphoresAndDie('SetupEnviourment.pm','',"failed to create dir : $dir");
		}
	}	
	############
	
	# setting crontab if needed
	my $crontabShouldBeText = '';
	while ( my ($scriptName, $scriptRunFrequencyDefinition) = each(%crontabScriptsNames) ) {
		$crontabShouldBeText .=
		"$scriptRunFrequencyDefinition $binDir\/$scriptName 2>> $Configuration::ERRORS_DIR\/$scriptName\.crontab\.error\n";
	}
		
	# getting the current crontab jobs
	# we use the grep to filter only relevant crontab lines.
	my $currentRelevantCrontabText = `crontab -l | grep $binDir`;
	# checking if crontab is not as it should be.
	if ($currentRelevantCrontabText ne $crontabShouldBeText) {
		# fixing crontab
		my $currentNonRelevantCrontabText = `crontab -l | grep -v $binDir`;
		
		my $TEMP_CRONTAB_ENTRIES_FILENAME = "crontabEntries.log";
		open TEMP_CRONTAB, ">$TEMP_CRONTAB_ENTRIES_FILENAME";
		print TEMP_CRONTAB $crontabShouldBeText.$currentNonRelevantCrontabText;
		close TEMP_CRONTAB;
		
		my $returnVal = system ("crontab $TEMP_CRONTAB_ENTRIES_FILENAME");
		if ($returnVal != 0) {
			&General::reportOnErrorReleaseSemaphoresAndDie('SetupEnviourment.pm','','failed updating crontab');
		}
		
		unlink $TEMP_CRONTAB_ENTRIES_FILENAME;
	}
}

1;