# dvir 2010 - 2011
use strict;
use Configuration;
use General;
use superlink2lnkg;
use SharedConsts;
use ProcessingBackend;
use File::Copy;
use MorganLmMarkers;
use GlAuto;

package SuperlinkWrapper;
BEGIN {
	our @ISA; # from which class we are inherting the methods

	sub new() {	
				
		my $class = $_[0]; # the name of this class

		if (-e "$Configuration::HAPLOTYPING_ANALYSIS_FILENAME") {
			@ISA = ('HaplotypeLarsWrapper');
		}
		elsif (-e "$Configuration::MORGAN_LM_MARKERS_ANALYSIS_FILENAME") {
			@ISA = ('MorganLmMarkersWrapper');
		}
		elsif (-e "$Configuration::MORGAN_GL_AUTO_ANALYSIS_FILENAME") {
			@ISA = ('GlAutoWrapper');
		}
		else {
			die 'SuperlinkWrapper: new(): unknown analysis';
		}
		
		return bless {}, $class;
	}
}

package MorganLmMarkersWrapper;
	my $winNum = 0;
	my $currWinRetrunValFileName = 'WIN_RET_VAL.txt'; # this file will use to transfer the return value
	# of the son process to the parent process.
	my $pid;	
BEGIN {
	sub runAnalysis($ $ $) {
		my $class = $_[0]; # the name of this class
		my $datFileName = $_[1];
		my $pedFileName = $_[2];
		my $timeLimitOfRunInSeconds = $_[3];		
		
		&General::makeSureFileExists($datFileName);
		&General::makeSureFileExists($pedFileName);		
		
		if ($timeLimitOfRunInSeconds <= 0) {die "MorganLmMarkersWrapper, runAnalysis 0 time to run;"}			
		
		# for saftey, I'll reduce 1 seconds
		# only if $timeLimitOfRunInSeconds > 1
		if ($timeLimitOfRunInSeconds > 1) {$timeLimitOfRunInSeconds -= 1;}
		
		# raising the window num (relevant for multi window run).
		++$winNum;
		
		# in order to perform clean exit, than I'm adding a signal when to terminate the run,
		# and since the signal doesn't stop a parent process that waits for the "system()" call
		# to end, then I'm adding a fork.
		# if $pid is not 0, than we are at the parent process
		if ($pid = fork()) {
			# the father process will be responsible for terminating the son process			
		
			# in order to perform clean exit, than I'm adding a signal when to terminate the run
			$SIG{ALRM} = \&MorganLmMarkersWrapper::stopMorganAfterTimePreiod;
			alarm($timeLimitOfRunInSeconds); #create an alarm that will be invoked at the end of allocated time		
			waitpid $pid, 0; # waiting for the child to end. Note that just "wait" is not good becuase the father sees children that died in the past
			alarm(0); # turning off the alarm			
			
			# checking if there was an error
			if (-e $currWinRetrunValFileName) {
				my $retVal = `cat $currWinRetrunValFileName`;
				chomp($retVal);
				
				# deleting the file, in order that other runs won't think that an error ocurred
				unlink $currWinRetrunValFileName;				
				return $retVal;
			}
			return 0;
		}
		
		
		# we are at the child process
		else {
            #Omer - we set process group to be able to kill both this process and its children (lm_markers)
            setpgrp(0, 0);        
			
			# only one time the conversion should be made
			(my $datFileNameInLnkgFormat, my $pedFileNameInLnkgFormat) = &superlink2lnkg::convertSuperlink2lnkg($datFileName,$pedFileName);
			
			#Define the names of the paramaters file - with and without numbers of iterations
			my $PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS = $Configuration::MORGAN_LM_MARKERS_PARAM_FILENAME;
			my $PARAM_FILE_WITH_NUM_OF_ITERATIONS = $Configuration::MORGAN_LM_MARKERS_PARAM_WITH_NUM_OF_ITERATIONS_FILENAME;

			# getting arguments by whom we will add parameters (the parameters that we will add depends on the number of iterations)
			&ProcessingBackend::setLogFiles('.'); # needed in order that the initVariablesFromPrePoint() will work at debug mode
			my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( '.' ) };		
			# the arguments
			my $burnInPercenage = $variables{$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_BURN_IN_PERCENTAGE};
			my $sequentialImputationRealizationsPercentage = $variables{$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_SEQUENTIAL_IMPUTATION_REALI};
			my $minNumOfIterations = $variables{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS};
			my $maxNumOfIterations = $variables{$SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS};
			my $numOfIterations = $minNumOfIterations;
			
			my $runCommand = "perl lnkg2lmm.pl --datafile=$datFileNameInLnkgFormat --pedfile=$pedFileNameInLnkgFormat --out_par=$Configuration::MORGAN_LM_MARKERS_PARAM_FILENAME --iter=$numOfIterations";
			my $returnVal  = system($runCommand);			
			if ($returnVal != 0) {
				system("echo $returnVal > $currWinRetrunValFileName");				
				warn "\n$runCommand\nRet val:$returnVal\n\n";				
				exit($returnVal);
			}			
			
			
			# adding the given parameters from the parameters file from the web server
			# to our param file
            system("cat $SharedConsts::MORGAN_MORE_PARAMS_FILENAME >> $PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS");
			
			# we will perform a new run with a new num of iterations, only if we haven't passed half of the time 
			# that was dedicated to all of the runs with diffrenet num of iterations,
			# since that the raising method of $numOfIterations is by 2, than the last run will take a little bit more
			# than all the previous runs.			
            my $totalRunTime = times();
			while (2*$totalRunTime < $timeLimitOfRunInSeconds and $numOfIterations <= $maxNumOfIterations) {			
				
				# morgan has a max iterations limit,
				# so in order not to exceed the limit, we are checking it here.
				if ($numOfIterations > 9000000000) {last;}
                
				#Omer 18/9/2011 - we use perl copy instead of Unix copy to avoid creating a subprocess that may be killed in the middle
                unlink $PARAM_FILE_WITH_NUM_OF_ITERATIONS;
				File::Copy::copy($PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS, $PARAM_FILE_WITH_NUM_OF_ITERATIONS);                
				
				# adding parameters to the param file. most important param is the num of iterations
                system("echo \"set MC iterations $numOfIterations\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				if (defined $burnInPercenage && $burnInPercenage ne '') {                    
					# the "%.f" is for rounding
					my $numOfBurnInIterations = sprintf("%.f",$burnInPercenage*$numOfIterations);
                    system("echo \"set burn-in iterations $numOfBurnInIterations\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				}				
				if (defined $sequentialImputationRealizationsPercentage && $sequentialImputationRealizationsPercentage ne '') {					
					my $numOfSequentialImputationRealizations = sprintf("%.f",$sequentialImputationRealizationsPercentage*$numOfIterations);
                    system("echo \"use $numOfSequentialImputationRealizations sequential imputation realizations for setup\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				}
				
				#Omer - Morgan will write its output to a temp file to not create "garbage" information in the output file
                #If it is killed while running
				my $tempFileName = "temp_file.txt";
				unlink $tempFileName;
                $runCommand = "./$Configuration::MORGAN_LM_LINKAGE_EXE $PARAM_FILE_WITH_NUM_OF_ITERATIONS > $tempFileName 2>> $Configuration::MORGAN_LM_MARKERS_STANDARD_ERROR_FILE";
				$returnVal = system($runCommand);				
				if ($returnVal != 0) {
					system("echo $returnVal > $currWinRetrunValFileName");					
					warn "\n$runCommand\nRet val:$returnVal\n\n";					
					exit($returnVal);
				}
                
                #Adding the run output to the Morgan output file
				#Also, Print separator and number of iterations
                system("echo \"$Configuration::MORGAN_LM_MARKERS_SEPERATOR\" >> $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE");
                system("echo $numOfIterations >> $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE");
                system("cat $tempFileName >> $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE");
                                             
				# raising the num of iterations
				$numOfIterations = &MorganLmMarkers::getNextNumOfIterations($numOfIterations);
				
				#Getting updated time
                (my $userTime, my $systemTime, my $childrenUserTime, my $childrenSystemTime) = times();
                $totalRunTime = $userTime+$systemTime+$childrenUserTime+$childrenSystemTime;
			}			
			exit(0); # the son process exits, only the parent process returns to the calling method
		}
	}
		
	sub stopMorganAfterTimePreiod() {	
		# here we are killing the child process that runs several iterations of morgan.
		#Hopefully, since the child is a group leader it should also kill all running instances of Morgan.
		&General::markThatJobHasStoppedAfterTimePeriod("Stopping Win No' $winNum\n");
		warn "\n$$ Killing $pid on window num: $winNum\n";		
		kill -9, $pid;		
	}
	
	# dvir 2011/03/09
	sub openMultiWindowRunOutputFiles() {
	}
	
	# dvir 2011/03/09
	# only deleting the single run output files, in order not to copy them back to
	# c b l - l i n k 0 1 from the execution machine
	sub closeMultiWindowRunOutputFiles() {
	
		# deleting the files of the last window, in order that when the condor job ends
		# than, those files won't be copied to execution server (cbl - link 01 )
		unlink $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE;
		unlink $Configuration::MORGAN_LM_MARKERS_STANDARD_ERROR_FILE;
		unlink $Configuration::MORGAN_LM_MARKERS_PARAM_WITH_NUM_OF_ITERATIONS_FILENAME;
#		unlink more output files ?
		
		# deleting files that were created by the call superlink2lnkg::convertSuperlink2lnkg
		`rm -rf $superlink2lnkg::FILES_PREFIX\*`;
	}
	
	# dvir 2011/03/09
	# updating the files in which the mutli output will be saved
	# with the current window results
	sub updateMultiWindowRunOutputFiles( $ ) {
		my $class = $_[0];
	
		# standard output
        if (-e $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE) {
            system("cat $Configuration::MORGAN_LM_MARKERS_STANDARD_OUTPUT_FILE >> $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_OUTPUT_FILENAME");
        }
        system("echo \"$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR\" >> $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_OUTPUT_FILENAME");

		# standard error
		# standard output
        if (-e $Configuration::MORGAN_LM_MARKERS_STANDARD_ERROR_FILE) {
            system("cat $Configuration::MORGAN_LM_MARKERS_STANDARD_ERROR_FILE >> $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_ERROR_FILENAME");
        }
        system("echo \"$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR\" >> $Configuration::MULTI_WINDOW_RUN_MORGAN_LM_MARKERS_ERROR_FILENAME");
	}
}


package GlAutoWrapper;
	my $gl_winNum = 0;
	# this file will use to transfer the return value of the son process to the parent process.
	my $retrunValFileName = 'WIN_RET_VAL.txt';
	my $childPid;
BEGIN {
	sub runAnalysis($ $ $) {
		my $class = $_[0]; # the name of this class
		my $datFileName = $_[1];
		my $pedFileName = $_[2];
		my $timeLimitOfRunInSeconds = $_[3];		
		
		&General::makeSureFileExists($datFileName);
		&General::makeSureFileExists($pedFileName);		
		
		if ($timeLimitOfRunInSeconds <= 0) { die "GlAutoWrapper, runAnalysis got non-positive time to run;" }			
		
		# for saftey, I'll reduce 1 seconds
		# only if $timeLimitOfRunInSeconds > 1
		if ($timeLimitOfRunInSeconds > 1) { $timeLimitOfRunInSeconds -= 1; }
		
		# raising the window num (relevant for multi window run).
		++$gl_winNum;
		
		# in order to perform clean exit, than I'm adding a signal when to terminate the run,
		# and since the signal doesn't stop a parent process that waits for the "system()" call
		# to end, then I'm adding a fork.
		# if $childPid is not 0, than we are at the parent process
		if ($childPid = fork()) {
			# the father process will be responsible for terminating the son process			

			# in order to perform clean exit, than I'm adding a signal when to terminate the run
			$SIG{ALRM} = \&MorganLmMarkersWrapper::stopMorganAfterTimePreiod;
			alarm($timeLimitOfRunInSeconds); #create an alarm that will be invoked at the end of allocated time		
			waitpid $childPid, 0; # waiting for the child to end. Note that just "wait" is not good becuase the father sees children that died in the past
			alarm(0); # turning off the alarm
			
			# checking if there was an error
			if (-e $retrunValFileName) {
				my $retVal = `cat $currWinRetrunValFileName`;
				chomp($retVal);
				
				# deleting the file, in order that other runs won't think that an error ocurred
				unlink $retrunValFileName;				
				return $retVal;
			}
			return 0;
		}
		
		else {
            #we set process group to be able to kill both this process and its children (gl_auto)
            setpgrp(0, 0);
			
			# only one time the conversion should be made
			(my $datFileNameInLnkgFormat, my $pedFileNameInLnkgFormat) = &superlink2lnkg::convertSuperlink2lnkg($datFileName,$pedFileName);
			
			#Define the names of the paramaters file - with and without numbers of iterations
			my $PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS = $Configuration::GL_AUTO_PARAM_FILENAME;
			my $PARAM_FILE_WITH_NUM_OF_ITERATIONS = $Configuration::GL_AUTO_PARAM_WITH_NUM_OF_ITERATIONS_FILENAME;
			
			# getting arguments by whom we will add parameters (the parameters that we will add depends on the number of iterations)
			&ProcessingBackend::setLogFiles('.'); # needed in order that the initVariablesFromPrePoint() will work at debug mode
			my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( '.' ) };	#getting hash from the modfile.txt file
			# the arguments
			my $burnInPercenage = $variables{$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_BURN_IN_PERCENTAGE};
			my $sequentialImputationRealizationsPercentage = $variables{$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_SEQUENTIAL_IMPUTATION_REALI};
			my $minNumOfIterations = $variables{$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS};
			my $maxNumOfIterations = $variables{$SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS};
			my $numOfIterations = $minNumOfIterations;

			#Create Morgan parameters files
			my $runCommand = "perl lnkg2lmm.pl --datafile=$datFileNameInLnkgFormat --pedfile=$pedFileNameInLnkgFormat --out_par=$Configuration::GL_AUTO_PARAM_FILENAME --iter=$numOfIterations"; # TODO: fix and correct to the real one
			my $returnVal  = system($runCommand);
			if ($returnVal != 0) {
				system("echo $returnVal > $retrunValFileName");
				warn "\n$runCommand\nRet val:$returnVal\n\n";
				exit($returnVal);
			}
			
			# adding the given parameters from the parameters file from the web server to our param file
            system("cat $SharedConsts::MORGAN_MORE_PARAMS_FILENAME >> $PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS");
			
			#add special commands used only by gl_auto to the param.GlAuto file.
			open PARAM_FILE_FH, ">>$PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS";
			print PARAM_FILE_FH "map tloc 1 unlinked\n";
			print PARAM_FILE_FH "output founder genome labels\n";
			print PARAM_FILE_FH "output scores file '$Configuration::GL_AUTO_SINGLE_RUNNING_OUTPUT_FILE'\n";
			close PARAM_FILE_FH;
			
			# we will perform a new run with a new num of iterations, only if we haven't passed half of the time 
			# that was dedicated to all of the runs with diffrenet num of iterations,
			# since that the raising method of $numOfIterations is by 2, than the last run will take a little bit more
			# than all the previous runs.			
            my $totalRunTime = times();
			while (2*$totalRunTime < $timeLimitOfRunInSeconds and $numOfIterations <= $maxNumOfIterations) {
				# morgan has a max iterations limit, so in order not to exceed the limit, we are checking it here.
				if ($numOfIterations > 9000000000) {last;}
				
				#we use perl copy instead of Unix copy to avoid creating a subprocess that may be killed in the middle
                unlink $PARAM_FILE_WITH_NUM_OF_ITERATIONS;
				File::Copy::copy($PARAM_FILE_WITHOUT_NUM_OF_ITERATIONS, $PARAM_FILE_WITH_NUM_OF_ITERATIONS);
				
				# adding parameters to the param file. most important param is the num of iterations
                system("echo \"set MC iterations $numOfIterations\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				if (defined $burnInPercenage && $burnInPercenage ne '') {
					my $numOfBurnInIterations = sprintf("%.f",$burnInPercenage*$numOfIterations); # the "%.f" is for rounding
                    system("echo \"set burn-in iterations $numOfBurnInIterations\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				}				
				if (defined $sequentialImputationRealizationsPercentage && $sequentialImputationRealizationsPercentage ne '') {					
					my $numOfSequentialImputationRealizations = sprintf("%.f",$sequentialImputationRealizationsPercentage*$numOfIterations);
                    system("echo \"use $numOfSequentialImputationRealizations sequential imputation realizations for setup\" >> $PARAM_FILE_WITH_NUM_OF_ITERATIONS");
				}
				
				#Omer - Morgan will write its output to a temp file to not create "garbage" information in the output file
                #If it is killed while running
				unlink $Configuration::GL_AUTO_SINGLE_RUNNING_OUTPUT_FILE;
                $runCommand = "./gl_auto.64bit.version3.0.3 $PARAM_FILE_WITH_NUM_OF_ITERATIONS 2>> $Configuration::GL_AUTO_STANDARD_ERROR_FILE";
				$returnVal = system($runCommand);				
				if ($returnVal != 0) {
					system("echo $returnVal > $currWinRetrunValFileName");					
					warn "\n$runCommand\nRet val:$returnVal\n\n";					
					exit($returnVal);
				}
				
				#Adding the run output to the Morgan output file. Also, Print separator and number of iterations
                system("echo \"$Configuration::GL_AUTO_SEPERATOR\" >> $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE");
                system("echo $numOfIterations >> $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE");
                system("cat $Configuration::GL_AUTO_SINGLE_RUNNING_OUTPUT_FILE >> $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE");
				
				# raising the num of iterations
				$numOfIterations = &GlAuto::getNextNumOfIterations($numOfIterations);
				
				#Getting updated time
                (my $userTime, my $systemTime, my $childrenUserTime, my $childrenSystemTime) = times();
                $totalRunTime = $userTime+$systemTime+$childrenUserTime+$childrenSystemTime;
			}
			exit(0); # the son process exits, only the parent process returns to the calling method
		}
	}

	sub openMultiWindowRunOutputFiles() { }
	
	# only deleting the single run output files, in order not to copy them back to
	# c b l - l i n k 0 1 from the execution machine
	sub closeMultiWindowRunOutputFiles() {
		# deleting the files of the last window, in order that when the condor job ends
		# than, those files won't be copied to execution server (cbl - link 01 )
		unlink $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE;
		unlink $Configuration::GL_AUTO_STANDARD_ERROR_FILE;
		unlink $Configuration::GL_AUTO_PARAM_WITH_NUM_OF_ITERATIONS_FILENAME;
		# TODO: check if more unnececary files are need to be deleted?
		
		# deleting files that were created by the call superlink2lnkg::convertSuperlink2lnkg
		`rm -rf $superlink2lnkg::FILES_PREFIX\*`;
	}
	
	# updating the files in which the mutli output will be saved
	# with the current window results
	sub updateMultiWindowRunOutputFiles( $ ) {
		my $class = $_[0];
		
		# standard output
        if (-e $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE) {
            system("cat $Configuration::GL_AUTO_STANDARD_OUTPUT_FILE >> $Configuration::GL_AUTO_MULTI_WIN_RES");
        }
        system("echo \"$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR\" >> $Configuration::GL_AUTO_MULTI_WIN_RES");

		# standard error
		# standard output
        if (-e $Configuration::GL_AUTO_STANDARD_ERROR_FILE) {
            system("cat $Configuration::GL_AUTO_STANDARD_ERROR_FILE >> $Configuration::GL_AUTO_MULTI_WIN_ERR");
        }
        system("echo \"$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR\" >> $Configuration::GL_AUTO_MULTI_WIN_ERR");
	}
	
	###
	# this function is the signal to the parent in runAnalysis after the time the analysis should run
	# is passed. this function kill the child that runs the analysis.
	###
	sub stopMorganAfterTimePreiod() {	
		# here we are killing the child process that runs several iterations of morgan.
		#Hopefully, since the child is a group leader it should also kill all running instances of Morgan.
		&General::markThatJobHasStoppedAfterTimePeriod("Stopping Win No' $gl_winNum\n");
		warn "\n$$ Killing $childPid on window num: $gl_winNum\n";		
		kill -9, $childPid;		
	}
}

# dvir 2011/11/15 not in use. new ver of haplo works with a dag suplied by lars
#our $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_OUTPUT_FILENAME = 'haplotypeLars.multi.out';
#our $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_ERROR_FILENAME = 'haplotypeLars.multi.err';
# package HaplotypeLarsWrapper;
# BEGIN {
	# sub runAnalysis($ $) {	
		# my $class = $_[0]; # the name of this class
		# my $datFileName = $_[1];
		# my $pedFileName = $_[2];
		# my $timeLimitOfRunInSeconds = $_[3]; # not relevant, was added for MorganLmMarkersWrapper
		
		# &General::makeSureFileExists($datFileName);
		# &General::makeSureFileExists($pedFileName);		
		
		# my $returnVal = system("./haplotyping.sh $datFileName $pedFileName > $Configuration::STANDARD_OUTPUT_FILE_HAPLOTYPE_LARS 2> $Configuration::STANDARD_ERROR_FILE_HAPLOTYPE_LARS");
		
		# return $returnVal;
	# }
	
	# # dvir 2011/03/09
	# sub openMultiWindowRunOutputFiles() {
	# }
	
	# # dvir 2011/03/09
	# # only deleting the single run output files, in order not to copy them back to
	# # c b l - l i n k 0 1 from the execution machine
	# sub closeMultiWindowRunOutputFiles() {
	
		# # deleting the files of the last window, in order that when the condor job ends
		# # than, those files won't be copied to execution server (cbl - link 01 )
		# unlink $Configuration::STANDARD_OUTPUT_FILE_HAPLOTYPE_LARS;
		# unlink $Configuration::STANDARD_ERROR_FILE_HAPLOTYPE_LARS;
		# unlink $Configuration::RESULT_FILE_HAPLOTYPE_LARS;
	# }
	
	# # dvir 2011/03/09
	# # updating the files in which the mutli output will be saved
	# # with the current window results
	# sub updateMultiWindowRunOutputFiles() {
		# # standard output
		# `cat $Configuration::STANDARD_OUTPUT_FILE_HAPLOTYPE_LARS >> $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_OUTPUT_FILENAME`;
		# `echo -n "$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR" >> $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_OUTPUT_FILENAME`;

		# # standard error
		# `cat $Configuration::STANDARD_ERROR_FILE_HAPLOTYPE_LARS >> $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_ERROR_FILENAME`;
		# `echo -n "$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR" >> $MULTI_WINDOW_RUN_HAPLOTYPE_LARS_ERROR_FILENAME`;
		
		# # result file
		# `cat $Configuration::RESULT_FILE_HAPLOTYPE_LARS >> $Configuration::MULTI_WINDOW_RESULT_FILE_HAPLOTYPE_LARS`;
		# `echo -n "$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR" >> $Configuration::MULTI_WINDOW_RESULT_FILE_HAPLOTYPE_LARS`;		
	# }
# }
1;
