#!/usr/bin/perl -w

use FindBin ();
use lib "$FindBin::Bin";
use Configuration qw(%configfile);
use Fcntl ':flock'; # import LOCK_* constants
use POSIX qw(strftime); # in order to track the locks time
use Environment;
use General;
use IPC::SysV qw(IPC_PRIVATE S_IRWXU IPC_CREAT);
use ProcessingBackend;
use Analysis;
use SharedConsts;
use Segregation;

#use IPC::Semaphore;
#use POSIX::RT::Semaphore;

#general parameters
$workDir				= $Configuration::workDir;
$mailDir				= $configfile{mailDir};     
$runlogDir 				= $configfile{runlogDir};
$jobsErrorLog 			= $configfile{jobsErrorLog};
$errorOutboxReport		= $configfile{errorOutboxReport};	
$splitInnerSysError		= $configfile{splitInnerSysError};	
$binDir 				= $configfile{binDir};
$currentlyRunningDir 	= $configfile{currentlyRunningDir};           
$maxSimTasksInSystem    = $configfile{maxNumberOfParallelTasks};

# if there are no resources indication, than it will exit
&General::makeSureThatSystemIsNotOutOfResources();

open (HNDLRLOCKFILE, ">$Configuration::TASK_HANDLER_SCRIPT_LOCK") or die "Can't open $Configuration::TASK_HANDLER_SCRIPT_LOCK file\n";
# We want only one handler to run in simultaneously (LOCK_EX == 2)
flock(HNDLRLOCKFILE,2 | 4) or die "Can't lock file\n";
my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
print HNDLRLOCKFILE "locked by TaskHandler $$ at $now_string\n";
										
	# Global environment directories check -
	########################################

# checking all the submmited tasks, but not yet running
my @projectsWaitningToRunArr = &General::getTasksNamesArr_ofTaskswhichWaitToRun_SortedByDateOfSubmission();
for $project ( @projectsWaitningToRunArr ) {
	my $projectTaskDir = &General::getTaskDirNameByTaskName($project);
		
	if (( -e "$projectTaskDir\/status.log" ) || (-e "$projectTaskDir\/TASK_FAILED.txt")) { 
		next;
	}
	
	# Check current load (last 1 min)
	$uptime = `/usr/bin/uptime`;
	$uptime = &trim($uptime);
	@loadData = split /load average:/ , $uptime;
	@loadDataInner = split /\,/, $loadData[1];
	$local_load = 100 * $loadDataInner[0] + 1;
    if ( $local_load == 0) {
        $local_load = 400; # We couldn't estimate the load
    }

	if ($local_load > $configfile{MAX_CPU_LOAD_THRESHOLD}) {
		next;
	}

	# First we check if we can activate more tasks according to the limit
	my $counterVal = &General::getCurrentRunningCounter($projectTaskDir);
	
	if ($counterVal == 0) {
		#dvir 2011/12/04 - no more use in semaphores
		##
		# # All the semaphores are supposed to be available - check that it is so
		# $semValString = `$binDir\/Semaphore\/$Environment::PROCESSES_SEMAPHORE_GETVAL`;
		# @semData = split /semval:/ , $semValString;
		# $semVal = $semData[1];
		
		# if ($semVal < $configfile{semNum}) { # The total number of semaphores got ruined somehow - here we fix it
			# $missing = $configfile{semNum} - $semVal;
			# for (1 .. $missing) {
				# system("$binDir\/Semaphore\/$Environment::PROCESSES_SEMAPHORE_PLUS");
			# }
		# }
		
		# # All the net semaphores are supposed to be available - check that it is so
		# $netSemValString = `$binDir\/Semaphore\/$Environment::NETWORK_CONNECTIONS_SEMAPHORE_GETVAL`;
		# @semData = split /semval:/ , $netSemValString;
		# $semVal = $semData[1];
		
		# if ($semVal < $configfile{semNetNum}) { # The total number of semaphores got ruined somehow - here we fix it
			# $missing = $configfile{semNetNum} - $semVal;
			# for (1 .. $missing) {
				# system("$binDir\/Semaphore\/$Environment::NETWORK_CONNECTIONS_SEMAPHORE_PLUS");
			# }
		# }
	}

	if ($counterVal >= $maxSimTasksInSystem) {
		# Queue is full
		close(HNDLRLOCKFILE);
		exit(0);
	}
	
	open (STSFILE,">$projectTaskDir\/status.log");
	flock(STSFILE,2) or die "Can't lock $projectTaskDir\/status.log file\n";
	$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	print STSFILE "##########\n# Status #\n##########\nStart at $now_string\n";
	close STSFILE;
	
	my ($projectRunlog) = "$runlogDir\/runlog.$project.log";
	my ($projectMailDir) = "$mailDir\/$project";
	
	my %variables = ();
	#choose the type of the analysis
	my $taskIsHaplotyping = 0;
	my $taskIsMorganLmMarkers = 0;
	my $taskIsMorganGlAuto = 0;
	my $taskIsSegregation = 0;
	my $taskIsHMM = 0;
	if (-e "$projectTaskDir\/modefile.txt") {
		&ProcessingBackend::setLogFiles("$projectTaskDir\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
		%variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };
		if ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/) { $taskIsHaplotyping = 1; }
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_LM_MARKERS/) { $taskIsMorganLmMarkers = 1; }
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_GL_AUTO/) { $taskIsMorganGlAuto = 1; }
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_SEGREGATION/) { $taskIsSegregation = 1; }
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HMM/) { $taskIsHMM = 1; }		
	}
	
	# dvir 2011/01/06
	# here we decided to start run the tasks, so removing the indication that the task waits for start running.
	# should be deleted, when all the tasks will be working with : markThatTaskIsRunning.
	# 2011/01/06: at drawing task, this is done in "markThatTaskIsRunning"
	# 2011/02/27: at haplotyping task, this is done in "markThatTaskIsRunning"
	# 2011/05/26: at morgan task, this is done in "markThatTaskIsRunning"	
	if (! (-e "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME"
	       || $taskIsHaplotyping
		   || $taskIsMorganLmMarkers
		   || $taskIsSegregation
		   || $taskIsMorganGlAuto
		   || $taskIsHMM)) {
		&General::removeTaskFromSubmittedDir($project);
	}

	
	my $pid = fork();
	if (not defined $pid) {
		# in order not to do an endless while with forks
		&General::markThatSystemIsOutOfResources();
	}
	
	if ( $pid == 0)	{ # the son process
		close(HNDLRLOCKFILE);
		&General::increaseCurrentRunningCounter($projectTaskDir);				
		&newProcessTaskHandle ( $project , $projectTaskDir , $projectRunlog  , $projectMailDir , $currentlyRunningDir, $taskIsHaplotyping, $taskIsMorganLmMarkers, $taskIsSegregation, $taskIsMorganGlAuto, $taskIsHMM);
		&General::decreaseCurrentRunningCounter($projectTaskDir); 
		exit(0);
	}  
	
} 

# Release the handler lock
close(HNDLRLOCKFILE);

exit(0);


											##############################
											# Auxilary Functions Section #
											##############################

# Handle one new task -
# This function is currently working in serial order , taking each chromosom and working on it until its done
#
# Parameters:
# $_[0] - project name
# $_[1] - project Task Diretory
# $_[2] - project Runlog file
# $_[3] - project Mail Directory
# $_[4] - currently Running Directory

sub newProcessTaskHandle ( $ $ $ $ $ $ $ $ ) 
{
	my $project 			= $_[0];
	my $projectTaskDir		= $_[1];
	my $projectRunlog 		= $_[2];
	my $projectMailDir 		= $_[3];
	my $currentlyRunningDir = $_[4];
	my $taskIsHaplotyping 	= $_[5];
	my $taskIsMorganLmMarkers = $_[6];
	my $taskIsSegregation   = $_[7];
	my $taskIsMorganGlAuto = $_[8];
	my $taskIsHMM = $_[9];
	
	my ($projectDataDir,$projectOutputDir);
	my $i=0;
	
	&General::addMeToPIDList($project);
	
	open( RUNLOG , ">>$projectRunlog" )or die "Can't open $projectRunlog\n";
	flock(RUNLOG,2) or die "Can't lock $projectRunlog file\n";
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;	
	print RUNLOG "runnin task $project at $now_string\n";
	close RUNLOG;
	
	open( TASK, "$projectTaskDir\/task.log" );
	
	# create variables file
	open( VAR_FILE , ">$projectTaskDir\/variables.txt" );
	
	my @lines = <TASK>;
	close TASK;
	
	$line = shift @lines;
	$line =~ s/\s+$//;  #remove \n
	$projectDataDir = $line;
	print VAR_FILE "DatfileDir $projectDataDir\n";
	
	$line = shift @lines;
	$line =~ s/\s+$//;  #remove \n
	$projectOutputDir = $line;
	( -d $projectDataDir ) or &TaskError("data dir $projectDataDir doesnt exist for $project", $projectTaskDir);
	( -d $projectOutputDir ) or &TaskError("output dir $projectOutputDir doesnt exist for $project", $projectTaskDir);
	
	$mode = "EliminatingOnly";
	print VAR_FILE "PedfileDir $projectDataDir\n";
	if( -e "$projectTaskDir\/moreVariables.txt" ) {
		open( MOREVAR_FILE , "$projectTaskDir\/moreVariables.txt" );
		for $varLine ( <MOREVAR_FILE> ) {
			print VAR_FILE "$varLine";
			if ($varLine =~ /modeOfRun/) {
				my @tmp = split(/[ \t\n\r\s\f]+/,$varLine);
				$mode = $tmp[1];
			}
		}
		close(MOREVAR_FILE);			
	}
	close(VAR_FILE);
	
	
	if (-e "$projectTaskDir\/modefile.txt") { $mode = "Analysis"; }	
	
	$mainWorkDir = "$projectTaskDir\/workDir";
	( -d $mainWorkDir ) or system("mkdir -p $mainWorkDir");
	
	chdir("$mainWorkDir");
	
	# there isn't a need for a monitor for TaskHandler.pl, because the task will decrease the counters by itself.
	# dvir 2011-02-27, haplotping
	# dvir 2010-12-20, drawing
	# dvir 2011-07, morgan
	# dvir 2011-10, segregation
	if (-e "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME" || 
	    $taskIsHaplotyping || 
		$taskIsMorganLmMarkers || 
		$taskIsSegregation ||
		$taskIsMorganGlAuto ||
		$taskIsHMM) {
		
		my $analysisTypeIndicationFileName; 
		if ($taskIsHaplotyping) {
			$analysisTypeIndicationFileName = $Configuration::HAPLOTYPING_ANALYSIS_FILENAME;
		}
		elsif ($taskIsMorganLmMarkers) {
			$analysisTypeIndicationFileName = $Configuration::MORGAN_LM_MARKERS_ANALYSIS_FILENAME;
		}
		elsif ($taskIsMorganGlAuto) {
			$analysisTypeIndicationFileName = $Configuration::MORGAN_GL_AUTO_ANALYSIS_FILENAME;
		}
		elsif (-e "$projectTaskDir\/$Configuration::PAINTER_PARAMETERS_FILENAME") {
			$analysisTypeIndicationFileName = $Configuration::DRAWING_ANALYSIS_FILENAME;
		}
		elsif ($taskIsSegregation) {
			$analysisTypeIndicationFileName = $Configuration::SEGREGATION_ANALYSIS_FILENAME;
		}
		elsif ($taskIsHMM) {
			$analysisTypeIndicationFileName = $Configuration::HMM_ANALYSIS_FILENAME;
		}		
		else {die;}
		
		# marking with a file that the task is haplotyping.
		open ANALYSIS_IND_FILE, ">$projectTaskDir\/workDir\/$analysisTypeIndicationFileName";
		close ANALYSIS_IND_FILE;
		
		# getting the task lock, in order that no delete will run at the same time we submit the dag of the task
		# and in order that checkTask.pl won't see that segregation is in the running folder without a condor dag
		# (cause segregation is running locally)
		open TASK_LOCK_FILE, ">$mainWorkDir\/$Configuration::TASK_LOCK_FILENAME";
		flock TASK_LOCK_FILE, $Configuration::EXCLUSIVE_LOCK;
		
		# if the task wasn't killed then exit without submitting the dag
		if ($taskIsSegregation) {
			if (-e "$mainWorkDir\/$Configuration::TASK_KILLED_FILENAME") {
				# decreasing the counters
				&General::markThatTaskIsKilled($project, $projectTaskDir,$mainWorkDir, "Task hasn't run, since the task was killed.", 1, 0);
			}
			else { # runnig & checkTask.pl cannot see, cause we have got the lock
				&General::markThatTaskIsRunning($project);
				&Segregation::run($project, $projectTaskDir, $mainWorkDir);
			}
		}
		else { # haplo, drawing, morgan
			my $analysis = new Analysis($mainWorkDir);
			my $errorMsg = $analysis->initDag();
				
			# if there was an error, than reporting on it.
			# and decreasing the counters.
			if (defined $errorMsg) {			
				&General::markThatTaskIsKilled($project, $projectTaskDir,$mainWorkDir, 
					"Killed due to Dag creation faliure.\nerror message: $errorMsg", 1, 0);
			}
			else {
				# if the task was killed then exit without submitting the dag
				if (-e "$mainWorkDir\/$Configuration::TASK_KILLED_FILENAME") {
					# decreasing the counters
					&General::markThatTaskIsKilled($project, $projectTaskDir,$mainWorkDir, "Dag wasn't submitted, since the task was killed.", 1, 0);
				}
				# the task wasn't killed
				else {	
					&General::markThatTaskIsRunning($project);
					# submitting the dag
					# dvir 2011/11/15: I have removed the option : "-no_recurse", in order to allow condor_submit_dag on sub dag inside the main dag.
					# relevat for haplotyping (the inner dag is a dag supplied from lars)
					system(". \$HOME/.bash_profile; condor_submit_dag -f -oldrescue 1 -autorescue 0 -debug 7 -config $binDir\/dagman_config $Configuration::TASK_MAIN_DAG_NAME > $Configuration::CONDOR_DAG_SUBMISSION_OF_MAIN_TASK_OUTPUT_FILENAME 2>> $Configuration::CONDOR_DAG_SUBMISSION_OF_MAIN_TASK_ERROR_FILENAME");
					
					# dvir 2011/11/16, sending an html update to the web server saying the analysis is running
					$analysis->reportOnRunningToWebServer();
				}
			}
		}
		close TASK_LOCK_FILE;
		exit(0);
	}
	else {
		# Temp sub to be overwriten
		my $hostnameTmpl = `cat $binDir\/Tmpl\/hostname.sub.tmpl`;
		$hostnameTmpl =~ s/_WORKDIR_/$mainWorkDir/g;
		
		open (HOSTNAME, ">$mainWorkDir\/hostname.sub");
		print HOSTNAME "$hostnameTmpl";
		close HOSTNAME;
			
		$semid = 0;
		if ( ($mode =~ /Eliminating/) || ($mode =~ /Clustering/ ) ) {
			#dvir 2011/12/04 - no more use in semaphores
			# $semid = semget($configfile{semKey}, 1, S_IRWXU|IPC_CREAT) || die "$!";
		}

		if ($#lines == -1) {
			push(@lines, "NO_DAT_FILES");
		}
		
		for $line ( @lines ) 
		{
			$i++;
			$line =~ s/\s+$//;  #remove \n
			
			if ( ($mode =~ /Eliminating/) || ($mode =~ /Clustering/ ) ) {
				my $currentDatFile = "$projectDataDir\/" . "$line.dat";
				my $currentPedFile = "$projectDataDir\/" . "$line.ped";
			
				( ( -e $currentDatFile ) && ( -e $currentPedFile ) ) or &TaskError("missing ped or dat files for $line in $project.", $projectTaskDir);
			
				my $rawFileFlag = "$projectDataDir\/" . ".$line.dat.raw";
				my $isRaw = 0 ; # Flag that indicate if the current files are raw files
				if ( -e $rawFileFlag )
				{
					$isRaw = 1;
				}
				
				chdir("$projectTaskDir");
				
				$workDir = "$mainWorkDir\/$line";
				( -d $workDir ) or system("mkdir -p $workDir");
				
				open( CHRFILE , ">$workDir\/currentChromosom.txt" );
				print CHRFILE "Datfile $line.dat \n";
				print CHRFILE "Pedfile $line.ped";
				close( CHRFILE );
				
				system("cp $projectTaskDir\/variables.txt $workDir\/variables.txt");
			}
			
			chdir("$projectTaskDir");
			
			if ( ($mode =~ /Eliminating/) || ($mode =~ /Clustering/) ) {$initScript = "InitDagsReduction.pl";}			
			else {
				if ($mode =~ /Analysis/ ) {					
					# dvir 2011/11/10, this section was copy from the old script InitDagsAnalysis.pl
					if ($line ne "NO_DAT_FILES") {
						system("cp $projectDataDir\/$line.dat $projectTaskDir\/workDir\/$line.dat");
						system("cp $projectDataDir\/$line.ped $projectTaskDir\/workDir\/$line.ped");
					}
					system("cp $projectDataDir\/mapfile.dat $projectTaskDir\/workDir\/mapfile.dat");
					system("cp $projectTaskDir\/modefile.txt $projectTaskDir\/workDir\/modefile.txt");
					if (-e "$projectTaskDir\/.pedigree_translate_1.txt") {
						system("cp $projectTaskDir\/.pedigree_translate_1.txt $projectTaskDir\/workDir\/pedigree_translate_1.txt");
					}
					&ProcessingBackend::setLogFiles( "$mainWorkDir\/" );
					my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $mainWorkDir ) };
					
					$analysis_name = $variables{ program };

					if ($analysis_name =~ /HZT/ ) {
						$initScript = "InitDagsHZT.pl";					
					}
					# 2011/11/10 end of section
				}
			}
			system("export PERLLIB=$binDir;perl $binDir\/$initScript $mainWorkDir $workDir $line $project $mode $binDir $i $projectDataDir $projectTaskDir $projectRunlog $projectMailDir $currentlyRunningDir $projectOutputDir $semid");
		}
	}
	# Calling the split script for the current chromosom
	@resStopped = `ls $mainWorkDir\/TASK_STOPPED.txt 2> /dev/null`;
	if ($#resStopped == -1) {
		system(". \$HOME/.bash_profile; cd $mainWorkDir; condor_submit_dag -f -oldrescue 1 -autorescue 0 -no_recurse -debug 7 -config $binDir\/dagman_config Global.dag >> $mainWorkDir/CondorSubmit.txt 2>> $mainWorkDir/CondorSubmitErr.txt");
		# system(". \$HOME/.bash_profile; cd $mainWorkDir; condor_submit_dag -debug 7 -config $binDir\/dagman_config -no_submit Global.dag >> $mainWorkDir/CondorSubmitDag.txt 2>> $mainWorkDir/CondorSubmitDagErr.txt");
		# open GLOBAL, "$mainWorkDir\/Global.dag.condor.sub";
		# @globalDag = <GLOBAL>;
		# close GLOBAL;
		# open GLOBAL, ">$mainWorkDir\/Global.dag.condor.sub";
		# $result = "";
		# for $i (0..$#globalDag-1) {
			# $result = $result.$globalDag[$i];
		# }
		# $result = $result."periodic_remove = (RemoteWallClockTime > 24*60*60)\n"."queue\n";
		# print GLOBAL "$result";
		# close GLOBAL;
		# system(". \$HOME/.bash_profile; cd $mainWorkDir; condor_submit Global.dag.condor.sub >> $mainWorkDir/CondorSubmit.txt 2>> $mainWorkDir/CondorSubmitErr.txt");
	}
	
	open (MONITOR, ">$mainWorkDir\/MonitorStatus.log");
	while (1) {
		sleep (60);
		@resFailed = `ls $mainWorkDir\/*\/TASK_FAILED.txt 2> /dev/null`;
		if ($#resFailed > -1) {
			print MONITOR "There are failed tasks\n";
			close MONITOR;
			last;
		}
		@resStopped = `ls $mainWorkDir\/TASK_STOPPED.txt 2> /dev/null`;
		if ($#resStopped > -1) {
			print MONITOR "The task was stopped\n";
			close MONITOR;
			last;
		}
		@resMainComplete = `ls $mainWorkDir\/TASK_COMPLETE.txt 2> /dev/null`;
		if ($#resMainComplete > -1) {
			print MONITOR "Task complete\n";
			close MONITOR;
			last;
		}
		@resMainFailed = `ls $mainWorkDir\/TASK_FAILED.txt 2> /dev/null`;
		if ($#resMainFailed > -1) {
			print MONITOR "Task failed\n";
			close MONITOR;
			last;
		}
		
		#@resMainCondorSubmitErr = `cat $mainWorkDir\/CondorSubmitErr.txt`;
		#if ($#resMainCondorSubmitErr > -1) {
		#	print MONITOR "Task failed\n";
		#	close MONITOR;
		#	last;
		#}
		
		@resTotal = `ls -l $mainWorkDir\/ \| grep ^d 2> /dev/null`;
		print MONITOR "resTotal : $#resTotal\n";
		@resComplete = `ls $mainWorkDir\/*\/TASK_COMPLETE.txt 2> /dev/null`;
		print MONITOR "resComplete : $#resComplete\n";
		if ( ($#resTotal > -1)  && ($#resTotal == $#resComplete) ) {
			close MONITOR;
			last;
		}
		else {
			system(". \$HOME/.bash_profile; condor_reschedule -name \$HOSTNAME");
		}
	}
	
	#if ($semid > 0) {
	#	system("ipcrm -s $semid");
	#}
	
	if (-e $errorOutboxReport || -e $splitInnerSysError  )
	{
		# Need to check the error and act according to it
		open( DONE , ">>$projectTaskDir\/status.log" );
		flock(DONE,2) or die "Can't lock $projectTaskDir\/status.log file\n";
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print DONE "job is done at $now_string - please see error logs\n";
		close DONE;
	}
	else 
	{
		open( DONE , ">>$projectTaskDir\/status.log" );
		flock(DONE,2) or die "Can't lock $projectTaskDir\/status.log file\n";
		$now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print DONE "job is done at $now_string\n";
		close DONE;
	}	
	
	#system("rm $projectTaskDir\/variables.txt");
}

											
# Report Errors functions #
###########################
sub Error( $ $ ) {
	&General::removeMeFromPIDList( $project );
	my ($errorReport) = $_[0];
	my $errorType = $_[1];
	$errorLog = $jobsErrorLog;
	$returnVal = $configfile{jobsErrorCode};
	
	open( ERRORFILE, ">>$errorLog");
	flock(ERRORFILE,2) or die "Can't lock $errorLog file\n";
	my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
	print ERRORFILE "$errorReport \n	at $now_string\n";
	close ERRORFILE;
	exit($returnVal);
}

sub TaskError( $ $ ) {
	my $projectTaskDir = $_[1];
	&General::removeMeFromPIDList();
	&General::decreaseCurrentRunningCounter($projectTaskDir); 
	&Error($_[0], "JOB")
}

sub trim( $ ) {
        my $string = $_[0];
        $string =~ s/^\s+//;
        $string =~ s/\s+$//;
        return $string;
}