#!/usr/bin/perl -w
# perl killProjByName <projectName>

# because we are running the script from another dir (the workDir),
# than this 2 uses, will tell the script from where to take the packages: from the dir in which the script resides
use FindBin ();
use lib "$FindBin::Bin";

use FindBin ();
use lib "$FindBin::Bin";

use File::Path;

use Configuration qw(%configfile); 
use POSIX qw(strftime); 
use General;
use ProcessingBackend;
use SharedConsts;
use JobStatus;

if( $#ARGV < 0 ) {
	&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"missing argument: projectName");
}

$receivedProjectName = $ARGV[0];

my $killAllFamilyScript = $configfile{killAllFamilyScript};
my $binDir 				= $configfile{binDir};
my $tasksDir			= $configfile{taskDir};	
my $runlogDir			= $configfile{runlogDir};	
my $currentlyRunningDir = $configfile{currentlyRunningDir};	
my $deathScript = $binDir . "\/" . $killAllFamilyScript;

@projects = `ls $tasksDir | grep $receivedProjectName`;

for $projectName (@projects){
	chomp($projectName);
	
	if( -d "$tasksDir\/$projectName\/workDir" ){
		#The directory was already created
	}
	else{
		my $dir = "$tasksDir\/$projectName\/workDir";
		mkpath $dir || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to create dir : $dir");
	}	

	my $taskIsHaplotyping = 0;
	my $taskIsMorganLmMarkers = 0;
	my $taskIsSegregation = 0;
	my $taskIsGlAuto = 0;
	my $taskIsHMM = 0;
	if (-e "$tasksDir\/$projectName\/modefile.txt") {
		&ProcessingBackend::setLogFiles("$tasksDir\/$projectName\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
		%variables = %{ &ProcessingBackend::initVariablesFromPrePoint( "$tasksDir\/$projectName" ) };
		if ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/){
			$taskIsHaplotyping = 1;
		}
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_LM_MARKERS/){
			$taskIsMorganLmMarkers = 1;		
		}
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_SEGREGATION/){
			$taskIsSegregation = 1;
		}
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_GL_AUTO/){
			$taskIsGlAuto = 1;
		}
		elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HMM/){
			$taskIsHMM = 1;
		}
	}
			
	# Dvir, 2010/12/29, special treatment to a pedigree killing / morgan killing / haplo killing
	if (-e "$tasksDir\/$projectName\/$Configuration::PAINTER_PARAMETERS_FILENAME" ||
		$taskIsMorganLmMarkers ||
	    $taskIsHaplotyping ||
		$taskIsSegregation ||
		$taskIsGlAuto ||
		$taskIsHMM) {
	
		my $taskLockFileName = "$tasksDir\/$projectName\/workDir\/$Configuration::TASK_LOCK_FILENAME";
		open TASK_LOCK_FILE, ">$taskLockFileName" || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to open file : $taskLockFileName");
		flock TASK_LOCK_FILE, $Configuration::EXCLUSIVE_LOCK || &General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to lock file : $taskLockFileName");
		
		# checking if the task was allready killed
		my $theTaskWasAllreadyKilled = (-e "$tasksDir\/$projectName\/workDir\/$Configuration::TASK_KILLED_FILENAME");
		
		# There are 3 stages of the an unkilled task:
		# 1. the main dag of the task han't been submitted yet
		# 2. the task is running (the main dag of the task was submitted).
		# 3. the task is finished.
		# and because we hold the lock than the task cannot move between thoses stages.
		# because for finish \ submitting the dag, than the task need to hold the lock.
		
		# checking if the task was allready killed
		if ($theTaskWasAllreadyKilled) {
			&General::markThatTaskIsKilled($projectName, "$tasksDir\/$projectName","$tasksDir\/$projectName\/workDir", "The task had allready been killed earlier.", 0, 0);
		}
		# checking if the task is finished (stage 3)
		elsif (&JobStatus::isJobFinished("$tasksDir\/$projectName\/workDir")) {
			&General::markThatTaskIsKilled($projectName, "$tasksDir\/$projectName","$tasksDir\/$projectName\/workDir", "The main task dag had allready finished, when we received a kill command for the task.", 0, 0);
		}
		# the task isn't finished
		# checking if the main DAG of the task had allready been submited (stage 2)
		elsif (-e "$tasksDir\/$projectName\/workDir\/$Configuration::CONDOR_DAG_SUBMISSION_OF_MAIN_TASK_OUTPUT_FILENAME") {
			# Getting the id of the main dag of the task
			my $taskMainDagId = &General::getClusterIdOfMainDagOfTask($projectName);
				
			my $command = "condor_rm $taskMainDagId >> $tasksDir\/$projectName\/workDir\/$Configuration::CONDOR_DAG_REMOVING_OF_MAIN_TASK_OUTPUT_FILENAME 2>> $tasksDir\/$projectName\/workDir\/$Configuration::CONDOR_DAG_REMOVING_OF_MAIN_TASK_ERROR_FILENAME";
			system($command);
			my $ret = $? >> 8;
			if ($ret != 0) {
				&General::reportOnErrorReleaseSemaphoresAndDie($0,'',"failed to preform : $command");
			}

			# decreasing the counters (CurrentRunning & TotalSubmitedTasksCounter) & marking that the task is killed
			&General::markThatTaskIsKilled($projectName, "$tasksDir\/$projectName","$tasksDir\/$projectName\/workDir", "Dag $taskMainDagId was removed from condor.", 1, 1);
		}
		# the main DAG hadn't been submited yet (stage 1)
		# so by creating the TASK_KILL_FILE we will mark not to create this DAG
		else {
			# we aren't decreasing the counters (CurrentRunning & TotalSubmitedTasksCounter) now
			# like we are doing on stage 2, 
			# because if the CurrentRunning = 0, than the decreasement will leave it as 0,
			# and when TaskHandler.pl will raise the counter, 
			# than there won't be no one who will decrease it afterwads.
			# so the decreasment is at TaskHanldler, when we choose not to submit the dag.
			&General::markThatTaskIsKilled($projectName, "$tasksDir\/$projectName","$tasksDir\/$projectName\/workDir", "Dag shouldn't be submitted, since the task was killed. (if its segregation, than we won't preform the computation)", 0, 0);
		}		
		close TASK_LOCK_FILE;
	}
	
	# the old kill for tasks which aren't as in the if part.
	else {
		`touch $tasksDir\/$projectName\/workDir\/TASK_STOPPED.txt`;

		if (-e "$tasksDir\/$projectName\/workDir\/CondorSubmit.txt") {
			open SUBMIT, "$tasksDir\/$projectName\/workDir\/CondorSubmit.txt";
			my @sub_data = <SUBMIT>;
			close SUBMIT;

			for $sub_line ( @sub_data ) {
				if ($sub_line =~ /submitted to cluster/) {
					@id_line_tmp = split /submitted to cluster/, $sub_line;
					@id_line = split /\./, $id_line_tmp[1];
					system(". \$HOME/.bash_profile; condor_rm $id_line[0]");
					last;
				}
			}
		}

		my $ppid = &General::getPIDNumberByProjectName($projectName);

		open (KILLEDFILE, ">>$runlogDir\/killerCalls.log") or die "Can't open $tasksDir\/$projectName\/killed.log file\n"; 
		my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
		print KILLEDFILE "killed at $now_string\n";
		close KILLEDFILE;	
			
		if ($ppid > 0)
		{
			open (KILLEDFILE, ">$tasksDir\/$projectName\/killed.log") or die "Can't open $tasksDir\/$projectName\/killed.log file\n";
			my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
			print KILLEDFILE "killed at $now_string\n";
			close KILLEDFILE;	
			
			system("bash $deathScript $ppid > /dev/null");
			&General::decreaseCurrentRunningCounter();
			system("find $currentlyRunningDir -type f -name \"$projectName.*\" -exec rm -f \{\} \\;"); # removing all the running leftovers
		}
	}
	# elsif($ppid == -1) # meaning that the task has ended - status exists but pid doesn't
	# {
	#	nothing left to do..
		# exit(-1);
	# }
	# elsif($ppid == -2) # hasn't started yet - so we create status file inorder to trick the system to think it was created already.
	# {
		# system("touch $tasksDir\/$projectName\/status.log");
	# }
	# elsif($ppid == -3) # No such project
	# {
		# exit(-1);
	# }
}

exit(0);