#!/usr/bin/perl -w
use Cwd;
use DAGSplit;
use Configuration qw(%configfile);
use General;

use FindBin ();
use lib "$FindBin::Bin";

$ssh_options = $configfile{ssh_options};               
$ssh_remote = $configfile{ssh_remote};              
$ssh_user = $configfile{ssh_user};              
$ssh_cmd = "ssh $ssh_options $ssh_user" . "@" . "$ssh_remote";
$remote_script = $configfile{remote_script}; 

$currProName = $ARGV[0];
$currentChromosomName = $ARGV[1];
$mode = $ARGV[2];

#remote info
$remote_user_full = "$ssh_user" . "@" . "$ssh_remote";
$remote_submission_dir = $configfile{remote_dir};


$workDir = cwd();

$cleaningStartSent = 0;
$cleaningEndSent = 0;
$firstCleaningReport = "";

$clusteringStartSent = 0;
$clusteringEndSent = 0;
$firstClusteringReport = "";

if ($mode eq "EliminatingOnly") {  # We should not wait for clustering
	$clusteringEndSent = 1;
}

if ( $mode eq "ClusteringOnly" )  # We should not wait for cleaning
{
	$cleaningEndSent = 1;
}

$sleepTime = 60;

open (MONITOR, ">MonitorStatus.log");
while (1) {
	sleep ($sleepTime);
	if ((($cleaningEndSent == 1) && ($clusteringEndSent == 1)) || (-e "TASK_FAILED.txt")){
		last;
	}
	else {
		@globalError = `grep "ERROR: the following job(s) failed" $workDir\/Global_$currentChromosomName.dag.dagman.out`;
		if ($#globalError > -1) {
			`touch TASK_FAILED.txt`;
			&DAGSplit::logAndReportError("Job execution failed.", "cleaning", $currProName, $currentChromosomName);
			last;
		}
		
		@Timeout = `cat $workDir\/*.log | grep "The job attribute PeriodicRemove"`;
		if ($#Timeout > -1) {
			`touch TASK_FAILED.txt`;
			&DAGSplit::logAndReportError("Job execution failed because it reached execution timeout.", "cleaning", $currProName, $currentChromosomName);
			last;
		}
		
		# Check current load (last 1 min)
		$uptime = `/usr/bin/uptime`;
		$uptime = &DAGSplit::trim($uptime);
		@loadData = split /load average:/ , $uptime;
		@loadDataInner = split /\,/, $loadData[1];
		$local_load = 100 * $loadDataInner[0] + 1;
		if ( $local_load == 0) {
			$local_load = $configfile{MAX_MONITOR_CPU_LOAD_THRESHOLD} + 100; # We couldn't estimate the load
		}
		
		if ($local_load > $configfile{MAX_MONITOR_CPU_LOAD_THRESHOLD}) {
			$sleepTime = $sleepTime + int(rand(30));
			next;
		}
		
		print MONITOR "Local load is: $local_load\n";
		
		@resTotalCleaning = `ls $workDir\/cleaning \| grep datafile_ 2> /dev/null`;
		print MONITOR "resTotalCleaning : $#resTotalCleaning\n";
		
		if ($cleaningStartSent == 0) {
			@resStart = `ls $workDir\/cleaning\/datafile_*\/cleaningReport.start* 2> /dev/null`;
			print MONITOR "resStart : $#resStart\n";
			if ( $#resStart > -1 ) {
				# Sending the file
				$cleaningStartSent = 1;
				$firstCleaningReport = &DAGSplit::trim($resStart[0]);
				my $scpCommand ="scp $firstCleaningReport $remote_user_full:\/$remote_submission_dir\/$currProName\/".$currentChromosomName."_cleaningReport.start.log";
				print MONITOR "$scpCommand\n";
				my $result = &General::perfomrWithExponentialBackoff($scpCommand);
				if ( $result != 0 ) 
				{
					&DAGSplit::logInnerSysError("couldn't perform $scpCommand");
				}
				my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent started -log ".$currentChromosomName."_cleaningReport.start.log";
				print MONITOR "$sshCommand\n";
				$result = &General::perfomrWithExponentialBackoff($sshCommand);
				if ( $result != 0 ) 
				{
					&DAGSplit::logInnerSysError("couldn't perform $sshCommand");
				}
				&DAGSplit::splitLog("Cleaning All - start report sent to site");
			}
		}
		
		if (( $cleaningEndSent == 0 ) && ($cleaningStartSent == 1)) {
			@resCompleteCleaning = `ls $workDir\/cleaning\/datafile_* \| grep ^resultFile. 2> /dev/null`;
			print MONITOR "resCompleteCleaning : $#resCompleteCleaning\n";
			if ($#resCompleteCleaning > -1) {
				$percent = &round(($#resCompleteCleaning+1)*100/(2*($#resTotalCleaning+1)));
				if ($percent > 100) {
					&DAGSplit::logInnerSysError("Got $percent completion in cleaning");
				}
				$percent = $percent."%";
				print MONITOR "percent : $percent\n";
				$cleaningReport = $firstCleaningReport;
				$cleaningReportName = "cleaningReport.start.log";
				if ($percent eq "100%") {
					$cleaningReport = "$workDir\/cleaning\/cleaningReport.fin.total.log";
					if (! -e $cleaningReport) {
						next;
					}
					$cleaningReportName = "cleaningReport.fin.total.log";
					$cleaningEndSent = 1;
				}
				
				# Sending the file
				$scpCommand ="scp $cleaningReport $remote_user_full:\/$remote_submission_dir\/$currProName\/".$currentChromosomName."_".$cleaningReportName;
				print MONITOR "$scpCommand\n";
				$result = &General::perfomrWithExponentialBackoff($scpCommand);
				if ( $result != 0 ) 
				{
					&DAGSplit::logInnerSysError("couldn't perform $scpCommand");
				}
				$sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter cleaning -percent $percent -log ".$currentChromosomName."_".$cleaningReportName;
				print MONITOR "$sshCommand\n";
				$result = &General::perfomrWithExponentialBackoff($sshCommand);
				if ( $result != 0 ) 
				{
					&DAGSplit::logInnerSysError("couldn't perform $sshCommand");
				}
				
				&DAGSplit::splitLog("Cleaning All - final errors report sent to site");
			}
		}
		
		if ($mode ne "EliminatingOnly") {
			# Clustering Monitor
			@resTotalClustering = `ls $workDir\/clustering \| grep datafile_ 2> /dev/null`;
			print MONITOR "resTotalClustering : $#resTotalClustering\n";
			
			if ($clusteringStartSent == 0) {
				@resStart = `ls $workDir\/clustering\/datafile_*\/clusteringReport.start* 2> /dev/null`;
				print MONITOR "resStart : $#resStart\n";
				if ( $#resStart > -1 ) {
					# Sending the file
					$clusteringStartSent = 1;
					$firstClusteringReport = &DAGSplit::trim($resStart[0]);
					my $scpCommand ="scp $firstClusteringReport $remote_user_full:\/$remote_submission_dir\/$currProName\/".$currentChromosomName."_clusteringReport.start.log";
					print MONITOR "$scpCommand\n";
					my $result = &General::perfomrWithExponentialBackoff($scpCommand);
					if ( $result != 0 ) 
					{
						&DAGSplit::logInnerSysError("couldn't perform $scpCommand");
					}
					my $sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter clustering -percent started -log ".$currentChromosomName."_clusteringReport.start.log";
					print MONITOR "$sshCommand\n";
					$result = &General::perfomrWithExponentialBackoff($sshCommand);
					if ( $result != 0 ) 
					{
						&DAGSplit::logInnerSysError("couldn't perform $sshCommand");
					}
					&DAGSplit::splitLog("Clustering All - start report sent to site");
				}
			}
			
			if (( $clusteringEndSent == 0 ) && ($clusteringStartSent == 1)) {
				@resCompleteClustering = `ls $workDir\/clustering\/datafile_* \| grep ^resultFile. 2> /dev/null`;
				print MONITOR "resCompleteClustering : $#resCompleteClustering\n";
				if ($#resCompleteClustering > -1) {
					$percent = &round(($#resCompleteClustering+1)*100/(3*($#resTotalClustering + 1)));
					if ($percent > 100) {
						&DAGSplit::logInnerSysError("Got $percent completion in clustering");
					}
					$percent = $percent."%";
					print MONITOR "percent : $percent\n";
					$clusteringReport = $firstClusteringReport;
					$clusteringReportName = "clusteringReport.start.log";
					if ($percent eq "100%") {
						$clusteringReport = "$workDir\/clustering\/resultFileTotal.mapReport.dat";
						if (! -e $clusteringReport) {
							next;
						}
						$clusteringReportName = "resultFileTotal.mapReport.dat";
						$clusteringEndSent = 1;
					}
					
					# Sending the file
					$scpCommand ="scp $clusteringReport $remote_user_full:\/$remote_submission_dir\/$currProName\/".$currentChromosomName."_".$clusteringReportName;
					print MONITOR "$scpCommand\n";
					$result = &General::perfomrWithExponentialBackoff($scpCommand);
					if ( $result != 0 ) 
					{
						&DAGSplit::logInnerSysError("couldn't perform $scpCommand");
					}
					$sshCommand ="$ssh_cmd $remote_script update -r $currProName -chr $currentChromosomName -parameter clustering -percent $percent -log ".$currentChromosomName."_".$clusteringReportName;
					print MONITOR "$sshCommand\n";
					$result = &General::perfomrWithExponentialBackoff($sshCommand);
					if ( $result != 0 ) 
					{
						&DAGSplit::logInnerSysError("couldn't perform $sshCommand");
					}
					
					&DAGSplit::splitLog("Clustering All - final errors report sent to site");
				}
			}
		}
	}	
}

chdir("$workDir");
`touch MONITOR_DONE.txt`;

exit(0);


sub round {
    my($number) = shift;
    return int($number + .5);
}