#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";
use CGI qw(:standard);
use Configuration qw(%configfile);
use DAGSplit;

$PEDCHECK = "$configfile{binDir}\/pedcheck";

$pedFile = $ARGV[0];
$workDir = $ARGV[1];

chdir($workDir);
&checkPedigree($pedFile);

if (-z "Out.txt") {
	exit(0);
}

open RES, "Out.txt";
@res = <RES>;
close RES;

$pedNum = -1;
$markerNum = -1;
%markersToFix = ();
@allChildren = ();
$foundStar = 0;

for $i (0..$#res) {
	if ($res[$i] =~ /had the following Mendelian errors/) {
		if (($foundStar == 0) && ($pedNum != -1) && ($markerNum != -1)) { #The previous marker had no error for specific children
			for $child (@allChildren) {
				if (defined($markersToFix{ $pedNum.".".$child })) {
					$markersToFix{ $pedNum.".".$child } = $markersToFix{ $pedNum.".".$child }.",".$markerNum;
				}
				else {
					$markersToFix{ $pedNum.".".$child } = $markerNum;
				}
			}
		}
		@tmpData1 = split /had/, $res[$i];
		@tmpData2 = split /Pedigree/, $tmpData1[0];
		$pedNum = &DAGSplit::trim($tmpData2[1]);
	}
	
	if ($res[$i] =~ /Mendelian error in marker/) {
		if (($foundStar == 0) && ($pedNum != -1) && ($markerNum != -1)) { #The previous marker had no error for specific children
			for $child (@allChildren) {
				if (defined($markersToFix{ $pedNum.".".$child })) {
					$markersToFix{ $pedNum.".".$child } = $markersToFix{ $pedNum.".".$child }.",".$markerNum;
				}
				else {
					$markersToFix{ $pedNum.".".$child } = $markerNum;
				}
			}
		}
		@tmpData1 = split /Mendelian error in marker/, $res[$i];
		@tmpData2 = split /:/, $tmpData1[1];
		$markerNum = &DAGSplit::trim($tmpData2[0]);
		$foundStar = 0;
	}
	
	if ($res[$i] =~ /\(\*\)/) {
		$foundStar = 1;
		@tmpData1 = split /\s+/, $res[$i];
		$personNum = $tmpData1[0];
		if (defined($markersToFix{ $pedNum.".".$personNum })) {
			$markersToFix{ $pedNum.".".$personNum } = $markersToFix{ $pedNum.".".$personNum }.",".$markerNum;
		}
		else {
			$markersToFix{ $pedNum.".".$personNum } = $markerNum;
		}
	}
	
	if ($res[$i] =~ /children/) {
		@allChildren = ();
	}
	
	if ($res[$i] =~ /^\d+\t\d+\/\d+/)  {
		@tmpData1 = split /\s+/, $res[$i];
		push(@allChildren, $tmpData1[0]);
	}
}

if (($foundStar == 0) && ($pedNum != -1) && ($markerNum != -1)) { #The previous marker had no error for specific children
	for $child (@allChildren) {
		if (defined($markersToFix{ $pedNum.".".$child })) {
			$markersToFix{ $pedNum.".".$child } = $markersToFix{ $pedNum.".".$child }.",".$markerNum;
		}
		else {
			$markersToFix{ $pedNum.".".$child } = $markerNum;
		}
	}
}

open PED, "$pedFile";
@pedLines = <PED>;
close PED;

$countRemoved = 0;
@words = split /\s+/, $pedLines[0];
$countTotal = $#pedLines*($#words - 10);

for $i (0..$#pedLines) {
	@words = split /\s+/, $pedLines[$i];
	if (defined($markersToFix{ $words[0].".".$words[1] })) {
		@markers = split /\,/, $markersToFix{ $words[0].".".$words[1] };
		for $j (0..$#markers){
			my $markerIndex = 10 + 2*($markers[$j] - 1);
			$words[ $markerIndex ] = 0;
			$words[ $markerIndex + 1 ] = 0;
			$countRemoved++;
		}
		$pedLines[$i] = join " ", @words;
		if ($i < $#pedLines) {
			$pedLines[$i] = $pedLines[$i]."\n";
		}
	}
}

$newData = join "", @pedLines;

system("cp $pedFile orig_$pedFile");

open PED, ">$pedFile";
print PED "$newData\n";
close PED;

if ($countRemoved/$countTotal > 0.5) {
	exit($configfile{HighRemoveRate})
}

system("cp Out.txt Out_1.txt");
&checkPedigree($pedFile);

if (! -z "Out.txt") {
	exit($configfile{RecheckNotEmpty});
}

exit(0);

###########################################
#removeEmptyLines:
# get a file and remove from it all empty lines
##########################################
sub removeEmptyLines{
  my ($file) = @_;
#  &debug_print("IN Remove empty lines");
  open II, "<$file" || die "cannot open file $_";
  open OO, ">1$file" || die "cannot open file $_";
  my $line;
  my @arr = <II>;
  close I;
  foreach $line( @arr){
    if ($line !~ /^\s*$/){
      $line =~ s/^\s+//sig;
      print OO $line;
    }
  }
  close OO;
  system ("cp 1$file $file");
  system ("rm 1$file");
}

sub editOutput{
 my ($names_line, $num) = @_;
 if ($names_line =~ /no/){
   return;
 }
 my @names = split(/,/,$names_line);
 if ($#names < $num - 1){
   #error return;
 }
 open I, "<Out.txt" || die "cannot open file $_";
 my @lines = <I>;
 close (I);
 open O, ">Out1.txt" || die "cannot open file $_";
 my $line;
 foreach $line (@lines){
   if ($line =~ /marker/){
     my @words = split(/marker/, $line);
     my @words1 = split ':', $words[1];
     print O "$words[0] marker XXX: $words1[1]\n";
   }
   else{
     print O "$line";
   }
 }
 close O;
 system("mv Out1.txt Out.txt");
}


###########################################
# checkPedigree:
# get a file , checks it with pedcheck programm
# and if there were some errors send them to mail.
##########################################
sub checkPedigree{

  my ($ped) = @_;
  open D, ">o.dat" || die "cannot open file $_";
  close D;
  open D, ">myOUT.txt" || die "cannot open file $_";
  close D;
 
  &removeEmptyLines("$ped");
    
	$res = &checkFamilyID($ped, "myOUT.txt");
	if ($res =~/stop/){
	  system("cat myOUT.txt >> Out.txt");
	  system("cp Out.txt err.txt");
	  return;
	}    

	&convertToSimpif($ped, "myOUT.txt");
  

  #&debug_print("CHECKING PEGIREE ...\n");
  system("$PEDCHECK o.dat 1$ped 2>err-ped.txt");

  #check if myOUT not empty
  my $size = -s "myOUT.txt";
  if ($size > 0){
    system("cat myOUT.txt >> Out.txt");
  }

## here check connectivity
# and concatinate all warnings to end of file

 system("cp Out.txt err.txt");

}


sub checkFamilyID{
  my ($file, $out) = @_;

  #&debug_print("in checkFamilyID\n");
  open O, ">>$out" || die "cannot open file $_";
  my ($line, @w, @ids);
  my $ids_size=0;
  open I, "<$file" || die "cannot open file $_";
  my $line_cnt = 0;
  while ($line = <I>){
    $line_cnt++;
    @w = split(/\s+/,$line);

    if (($#w > 0) and ($#w < 9)){
      print O "Error in superlink format line $line_cnt: \nThe line should contain minimum 10 fields (fam, person, father, mother ...)\n";
      return "stop";
    }

    $w[0] =~ s/\s+//sig;

    if ($w[0] !~ /\b[0-9]+\b/){
      print O "Error in superlink format line  $line_cnt: \nFamily Id should be a number\n";
      last;
    }

    if ($w[0] <= 0){
      print O "Error: family id <= 0\nIn superlink format family ids should start from 1,\nand contain sequential numbers\n";
      last;
    }

    if (not $ids[$w[0]]){
      $ids_size++;
    }
    $ids[$w[0]]=1;
  }
  close I;

  for $i (1 .. $ids_size){
    if ( not $ids[$i]){
 print O "Error: Family ids are not fitting supelink format \nIn superlink format family ids should start from 1,\nand contain sequential numbers\n";
      last;
    }
  }

  close O;
  return "continue";
}

sub convPersonSimplyf{
  my $line = shift;
  my @w1 = split(/<</, $line);
  my @w = split(/\s+/, $w1[0]);
  my ($i, $st)= (0,10);
  if ($#w % 2 == 0){ #there is a penetrance class
    $st = 11;
  }
  my $result = "$w[1]\t$w[2]\t$w[3]\t$w[7]\t$w[9]\t";
  foreach $i ($st .. $#w){
    $result = "$result $w[$i]  ";
  }
  return "$result\n";
}

sub convertToSimpif{
  my ($file, $out) = @_;

  open OUT, ">>$out";

  system("sort -k 1n -k 2n $file > 1$file");

  open I, "<1$file";
  my @arr = <I>;
  close I;

  my @words = split(/\s+/, $arr[0]);
  my $famID = $words[0];

  open O, ">1$file";
  print O "pedigree:  $famID\n";

  my $personID = 1;
  for $i (0 .. $#arr){

    @words = split(/\s+/, $arr[$i]);
    if ($famID !~ $words[0]){
      $famID = $words[0];
      $personID = 1;
      print O "\npedigree:  $famID\n";
    }

#check the persons numbers of superlink format
    if ($personID !~ $words[1]){
      print OUT "\nError in family $famID person $words[1]:\nIn superlink format persons id's shoud start from 1 and be sequential numbers \n";
$personID = $words[1];
    }

    print O &convPersonSimplyf("$arr[$i]");

    $personID++;
  }
  close O;
  close OUT;
}

1;
