#!/usr/bin/perl -w
# dvir, 2011/11/13
use strict;

# because we are running the script from another dir (the job Dir),
# than this 2 uses, will tell the script from where to take the packages: from the dir in which the script resides
use FindBin ();
use lib "$FindBin::Bin";

use Cwd;

use General;
use Analysis;
use JobStatus;
use DatPedHandler;
use ProcessingBackend;

my $workDirPath = $ARGV[0];
# several actions may be requested
# create_html -> preform html update to the web server.
# determine_job_status_of_leaf_job_with_one_exec -> check Leaf job (job which doesn't have sons at the DAG), there isn't a return values file.
# determine_job_status_of_leaf_job_with_possible_more_than_one_exec -> check Leaf job (job which doesn't have sons at the DAG), there is a return values file.
# determine_job_status_of_an_inner_job -> check Inner Job (job which have sons at the DAG).
# determine_job_status_of_families_dirs_by_their_sub_jobs
# send_indication_to_web_server_to_mark_that_task_is_done -> sends the file $Configuration::HAP_INDICATION_FILE_FOR_ANALYSIS_END to the dest dir at the web server
# determine_job_status_of_the_task_by_families_dirs -> determine the status that appears at the workDir by families dirs only
# 				it also: reduce counters, remove indication (running indication of the task)
# determine_job_status_of_the_task_by_sub_folders -> determine the status that appears at the workDir, when each sub folder of the workDir represnt a sub job
# 				it also: reduce counters, remove indication (running indication of the task)

#
# for example : the script will run after all the duplications of a job. will update the status of the duplication dir (inner job)
# it will create the html's with the intermediate result (the results of the windows that have been completed) and send it to the web server.
# so $actionsRequested will be 'ad' (without the ' char)
#
# if the checks of a leaf job results in a failure or a missing file, than a mail will be sent to the developers.
my $actionsRequested = $ARGV[1];

my $jobDir = &getcwd(); # this is the full path

my $analysis = new Analysis($workDirPath);

if (($actionsRequested =~ 'determine_job_status_of_leaf_job_with_one_exec' && $actionsRequested =~ 'determine_job_status_of_leaf_job_with_possible_more_than_one_exec') ||
    ($actionsRequested =~ 'determine_job_status_of_leaf_job_with_one_exec' && $actionsRequested =~ 'determine_job_status_of_an_inner_job') ||
	($actionsRequested =~ 'determine_job_status_of_leaf_job_with_possible_more_than_one_exec' && $actionsRequested =~ 'determine_job_status_of_an_inner_job') ||
	($actionsRequested =~ 'determine_job_status_of_the_task_by_sub_folders' && $actionsRequested =~ 'determine_job_status_of_the_task_by_families_dirs')) {
	die 'bad combination of actions requested';
}

############

# checking if there is a need to check Leaf job (job which doesn't have sons at the DAG), there isn't a return values file.
if ($actionsRequested =~ 'determine_job_status_of_leaf_job_with_one_exec') {
	my $executableExitCode = $ARGV[2];

	my @jobOutputFiles = $analysis->getJobOutputFiles();
	my $errorFile = $analysis->getJobErrorFile();	
	my $areSomeFilesMissing = 0;
	my $shouldPreformExecCheck = 1;
	my $doesAnAcceptableErrorOcurredAtJob;
	
	my $does_A_TimedOutOcurredAtJob = $analysis->does_A_TimedOutOcurredAtJob($jobDir);
	if ($does_A_TimedOutOcurredAtJob) {
		&JobStatus::markThatJobTimedOut('.');
	}
	
	# Asuring that no acceptable error has ocurr.
	# if an acceptable error has ocurred than we don't need to check the Exec.
	$doesAnAcceptableErrorOcurredAtJob = $analysis->doesAnAcceptableErrorOcurredAtJob($jobDir, $executableExitCode);
	
	if ($doesAnAcceptableErrorOcurredAtJob) {
		&JobStatus::markThatJobFailedWithAcceptableReason('.');
		$shouldPreformExecCheck = 0;		
	}
	
	if ($shouldPreformExecCheck) {
		# this call won't cause an exit, only infrom the system admin about an error.
		$areSomeFilesMissing = &General::checkExecutionExitCodeAndExistenceOfOutputFiles($analysis->getTypeName(), $analysis->getProjectName(), $executableExitCode, \@jobOutputFiles, $errorFile);

		if ($executableExitCode==0 && !$areSomeFilesMissing) {
			&JobStatus::markThatJobSucceeded('.');
		}
		else {
			&JobStatus::markThatJobFailed('.');
		}
	}	
}

############

# checking if there is a need to check Leaf job (job which doesn't have sons at the DAG), there is a return values file.
if ($actionsRequested =~ 'determine_job_status_of_leaf_job_with_possible_more_than_one_exec') {
	my $executableExitCode = $ARGV[2];
	
	#Omer 15/9/2011 - sleep for 3 seconds to prevent possible synchronization bugs
	sleep 3;
	
	# the return values file is a file with the return values of each seperate window run
	# example for the file for 3 windows : 
	#
	#0
	###multi end###
	#0
	###multi end###
	#0
	###multi end###
	# now I'll look for a return value which is not "0"
	my $returnVal = $executableExitCode;
	my $isThereAnSuccessfulComputation = 0;
	if ($returnVal == 0) {
		open RET_VALUES_FILE, $Configuration::MULTI_WINDOW_RUN_RETURN_VALUES_FILENAME;
		while (my $currLine=<RET_VALUES_FILE>) {
			chomp($currLine);		
			if ($currLine != 0) {
				$returnVal = $currLine;
			}
			else {
				$isThereAnSuccessfulComputation = 1;
			}
			<RET_VALUES_FILE>; # reading the line :###multi end###
		}
		close RET_VALUES_FILE;
	}

	my @jobOutputFiles = $analysis->getJobOutputFiles();
	my $errorFile = $analysis->getJobErrorFile();
	# this call won't cause an exit, only infrom the system admin about an error.
	my $areSomeFilesMissing = &General::checkExecutionExitCodeAndExistenceOfOutputFiles($analysis->getTypeName(), $analysis->getProjectName(), $returnVal, \@jobOutputFiles, $errorFile);
	
	if ($isThereAnSuccessfulComputation && ! $areSomeFilesMissing) {
		&JobStatus::markThatJobSucceeded('.');
	}
	else {
		&JobStatus::markThatJobFailed('.');
	}
}

############

# checking if there is a need to check Inner Job (job which have sons at the DAG).
if ($actionsRequested =~ 'determine_job_status_of_an_inner_job') {

	# checking if at least one of the duplications was successful.
	# if so, than the job will be marked as successful.
	# a duplication of the job is considered successful, if one of its window is successful.
	&JobStatus::updateJobStatusByItsSubJobs($jobDir);
}

############

# checking if there is a need to preform html update to the web server
if ($actionsRequested =~ 'create_html') {
	# the chdir is in order that the html file will be created at the workDir of the task.
	chdir $workDirPath;

	my $projectTaskDir = $analysis->getProjectDir();
	&ProcessingBackend::setLogFiles("$projectTaskDir\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };

	my $numOfDuplications = 1;
	if (defined $variables{num_dups_per_job}) {
		$numOfDuplications = $variables{num_dups_per_job};
	}

	# the lock is in order that other postJob.pl post scripts won't run at same time
	# that this script run
	open TASK_LOCK_FILE, ">$workDirPath\/$Configuration::TASK_LOCK_FILENAME";
	flock TASK_LOCK_FILE, $Configuration::SHARED_LOCK;

	chdir $jobDir;
	$analysis->createFinalOutputFileForCurrentJob($numOfDuplications);
	chdir $workDirPath;
	$analysis->createOutputFiles(\%variables);

	my @fileListToBeCopiedToWebServer = $analysis->getFileListToBeCopiedToWebServer();

	# checking that all the files to be copied really exists
	foreach my $fileToBeCopiedToWebServer (@fileListToBeCopiedToWebServer) {
		if (! -e $fileToBeCopiedToWebServer) {
			&General::reportOnErrorReleaseSemaphoresAndDie('postJob.pl','',"file: $fileToBeCopiedToWebServer doesn't exists.");
		}
	}

	my $dirAtWebServer = $variables{newDir};

	# if @fileListToBeCopiedToWebServer is empty, than this method will alert on that
	&General::sendFilesToWebServer(\@fileListToBeCopiedToWebServer, $dirAtWebServer);

	# executing several commands
	my @commands = $analysis->getCommandsToBeExecutedOnWebServer($dirAtWebServer);
	# only if there are commands, than we will try to execute them
	if (scalar(@commands) > 0) {
		&General::executeSSHCommandsOnWebServer(\@commands);
	}

	close TASK_LOCK_FILE; # free the lock
}

############

if ($actionsRequested =~ 'determine_job_status_of_families_dirs_by_their_sub_jobs') {

	my @familiesNumbers = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@familiesNumbers);
	# the markup of the families dir is needed in order to mark the work dir as successful or failed

	foreach my $currFamily (@familiesNumbers) {
		my $currFamilyDir = &General::getFamilyRunDir($currFamily);
		# marking the family dir as successful or failed		
		&JobStatus::updateJobStatusByItsSubJobs($currFamilyDir);
	}
}

############

if ($actionsRequested =~ 'send_indication_to_web_server_to_mark_that_task_is_done') {
	my $projectTaskDir = $analysis->getProjectDir();
	&ProcessingBackend::setLogFiles("$projectTaskDir\/"); # needed in order that the initVariablesFromPrePoint() will work at debug mode
	my %variables = %{ &ProcessingBackend::initVariablesFromPrePoint( $projectTaskDir ) };

	my $dirAtWebServer = $variables{newDir};

	# this command will cause hap to recognize that the task has ended.
	# and the task will be removed from the file "TaskWaitingList" at hap.
	my @commands = ("touch $dirAtWebServer\/$Configuration::HAP_INDICATION_FILE_FOR_ANALYSIS_END");
	&General::executeSSHCommandsOnWebServer(\@commands);
}

############

if ($actionsRequested =~ 'determine_job_status_of_the_task_by_families_dirs') {
	
	my @familiesNumbers = ();
	&DatPedHandler::getFamiliesNumbersArrByFamiliesNumbersFile(\@familiesNumbers);
	# the markup of the families dir is needed in order to mark the work dir as successful or failed

	my @familiesDirsNames = ();
	foreach my $currFamily (@familiesNumbers) {
		my $currFamilyDir = &General::getFamilyRunDir($currFamily);
		push @familiesDirsNames, $currFamilyDir;
	}
	
	# marking that the task is finished, reducing counters, removing indication
	&General::markThatTaskIsFinished($analysis->getProjectName(), \@familiesDirsNames);
}

############

if ($actionsRequested =~ 'determine_job_status_of_the_task_by_sub_folders') {
	# marking that the task is finished, reducing counters, removing indication
	&General::markThatTaskIsFinished($analysis->getProjectName());
}
# mark_to_web_server_that_task_is_done

# in order that the rest of the dag will not be affected by a possible failure of this run
# we are exiting with code 0
exit(0);