#!/usr/bin/perl -w

# dvir 2011/03
use strict;
use Configuration;
use SuperlinkWrapper;
use General;

# not required, required only when the method runAnalysis need this parameter
my $timeLimitOfRunInSeconds	= $ARGV[0]; # dvir 2011/06/01, was added for MorganLmMarkers

&General::makeSureFileExists($Configuration::MULTI_DAT_FILE_NAME);
&General::makeSureFileExists($Configuration::MULTI_PED_FILE_NAME);
&General::makeSureFileExists($Configuration::PED_WITH_NO_ALLELS_FILE_NAME);

my $superlinkWrapper = SuperlinkWrapper -> new();
$superlinkWrapper -> openMultiWindowRunOutputFiles();

my $prevRetVal = 0;

open MULTI_DAT, $Configuration::MULTI_DAT_FILE_NAME;
open MULTI_PED, $Configuration::MULTI_PED_FILE_NAME;

open RET_VALUES_FILE, ">$Configuration::MULTI_WINDOW_RUN_RETURN_VALUES_FILENAME";

open CURRENT_WINDOW_DAT, ">$Configuration::CURRENT_WINDOW_DAT_FILE_NAME";
open CURRENT_WINDOW_PED, ">$Configuration::CURRENT_WINDOW_PED_FILE_NAME";
while (my $currMultiDatLine = <MULTI_DAT>) {
	# checking if we have finsihed passing over the dat file of the CURRENT window
	if ($currMultiDatLine =~ /$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR/) {
	
		open NO_ALLELS_PED, $Configuration::PED_WITH_NO_ALLELS_FILE_NAME;
		
		while (my $currMultiPedLine = <MULTI_PED>) {
			# checking if we have finsihed passing over the ped file of the CURRENT window
			if ($currMultiPedLine =~ /$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR/) {last;}							
			
			# the line belongs to the current window dat
			else {
				# getting the person data with no allels (the first 10 cols)
				my $currNoAllelsPedLine = <NO_ALLELS_PED>;
				# removing the \n at the end of the line
				$currNoAllelsPedLine =~ s/\n//; 
				print CURRENT_WINDOW_PED $currNoAllelsPedLine.' '.$currMultiPedLine;
			}
		}
		
		# closing the current window dat&ped
		close CURRENT_WINDOW_DAT;
		close CURRENT_WINDOW_PED;
		close NO_ALLELS_PED;
		
		# running the analysis on the current window
		my $retVal = $superlinkWrapper -> runAnalysis(
			$Configuration::CURRENT_WINDOW_DAT_FILE_NAME,
			$Configuration::CURRENT_WINDOW_PED_FILE_NAME,
			$timeLimitOfRunInSeconds);		
		
		# saving the retVal
		print RET_VALUES_FILE "$retVal\n$Configuration::MULTI_FILES_IN_SINGLE_FILE_SEPERATOR";
		
		# saving the output of the run and deleting files from the current window, to not interfere
		#with the next window
		$superlinkWrapper -> updateMultiWindowRunOutputFiles();
		$superlinkWrapper -> closeMultiWindowRunOutputFiles();
		
		# if the last two windows had failed, than stop executeing the rest of the windows
		if ($retVal != 0 && $prevRetVal != 0) {
			exit $retVal;
		}
		$prevRetVal = $retVal;
		
		# reopening the current window dat&ped
		open CURRENT_WINDOW_DAT, ">$Configuration::CURRENT_WINDOW_DAT_FILE_NAME";
		open CURRENT_WINDOW_PED, ">$Configuration::CURRENT_WINDOW_PED_FILE_NAME";		
	}
	# the line belongs to the current window dat
	else {
		print CURRENT_WINDOW_DAT $currMultiDatLine;
	}
}

close CURRENT_WINDOW_DAT;
close CURRENT_WINDOW_PED;

close RET_VALUES_FILE;

close MULTI_DAT;
close MULTI_PED;

# deleting the dat&ped files that were used to run the current window
unlink $Configuration::CURRENT_WINDOW_DAT_FILE_NAME;
unlink $Configuration::CURRENT_WINDOW_PED_FILE_NAME;