#!/usr/bin/perl -w
#perl splitSNPdistiller.pl
use Cwd;
use FindBin ();
use lib "$FindBin::Bin";
use DAGSplit;
use Configuration qw(%configfile);
use POSIX qw(strftime); # in order to track the error time
use POSIX ":sys_wait_h";

my $SNPdistiller  			=$configfile{SNPdistiller};
my $cleaningParamToDistiler =$configfile{cleaningParam};

my ($cwd) = cwd();

# First opening new file for the first report
open(CLEANING_LOG,">cleaningReport.start.log") or die &DAGSplit::logInnerSysError("couldn't open cleaningReport.start.log for writing");
my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
print CLEANING_LOG "Cleaning started at $now_string\n";
close CLEANING_LOG;

# Running the SNPdistiller on the recived files	- cleaning only !
#open IN, "DistillerCommand.txt";
#my @data = <IN>;
#close IN;
system("chmod 700 snpDistiller DistillerCommand.txt");
system("./DistillerCommand.txt");
my $exit_value  = $? >> 8; 
# Check if the distiller ended well
if ( $exit_value != 0 )
{
	my $deadSignal = ($? & 127);
	&DAGSplit::logInnerSysError("Error with SNPdistiller running in $cwd \nreturn value : $? \nsignal : $deadSignal");
}	

# we del the output because we have it already in the outputfile ( we didnt want it to trash the runlog ) 
system("rm SNPdistillerOutput.txt");

# We want to send the site the error report 
############################################

# First opening new file for the final report
open(CLEANING_REPORT,">cleaningReport.fin.log") or die &DAGSplit::logInnerSysError("couldn't open cleaningReport.fin.log for writing");

# now working on the err file
open(RESFILE,"err.txt") or die &DAGSplit::logInnerSysError("couldn't open err.txt for reading");
@lines = <RESFILE>;
@sorted = sort { ($a =~ /\((\d+)/)[0] <=> ($b =~ /\((\d+)/)[0] } @lines;
my %seen;
my @uniqed = grep !$seen{$_}++, @sorted;

print CLEANING_REPORT "\nThis file contains error reports that were found during the calculation on the given files\n\n";

# Copy the sorted data from err file to the report.
print CLEANING_REPORT "@uniqed";

close RESFILE;
close CLEANING_REPORT;

# Checkning the existence of the SNPdistiller output files
unless( (-e "ok_snpAll.dat") && (-e "ok_snpAll.ped") )
{			 
	&DAGSplit::logInnerSysError("Error with SNPdistiller running on the current datafile");
}

# Fixing the numbers of markers in the dat file to be in increasing order
open (DATFILE , "ok_snpAll.dat");
open (FIXDATFILE , ">ok_snpAll.dat.fixed");
my $index =1;
for $line (<DATFILE>)
{	
	my $tmpLine = $line;
	if ($tmpLine =~ s/\((\d+)\)\#/\# \<\< marker $index/)
	{
		$index++;
	}
	
	print FIXDATFILE $tmpLine;
}		

system("rm ok_snpAll.dat");
system("mv ok_snpAll.dat.fixed resultFile.datafile");
system("mv ok_snpAll.ped resultFile.pedfile");	 

system("rm results.txt");
system("rm err.txt");

