#!/usr/bin/perl -w
use FindBin ();
use lib "$FindBin::Bin";
use DAGSplit;
use General;
use Configuration qw(%configfile);
use POSIX qw(strftime); # in order to track the error time
use List::Util;
use DatWrapper;
use PedWrapper;
use strict;

# First opening new file for the first report
open(CLUSTERING_LOG,">clusteringReport.start.log") or die &DAGSplit::logInnerSysError("couldn't open clusteringReport.start.log for writing");
my $now_string = strftime "%a %b %e %H:%M:%S %Y", localtime;
print CLUSTERING_LOG "Clustering started at $now_string\n";
close CLUSTERING_LOG;

#Init command and window param
open IN, "RunningParams.txt";
my @data = <IN>;
close IN;

my $DistillerCommand = $data[0];
my $cMorganWindowSize = $data[1];
my $MAX_MARKERS_IN_SINGLE_CLUSTERING_RUN = 50;

#################################################################
#Create datWrapper and pedWrapper objects for the dat and ped file
my $inDatafile = "datafile.dat";
my $inPedfile = "pedfile.dat";
my $datWrapper = new DatWrapper($inDatafile);
my $numOfMarkers = $datWrapper->getNumOfMarkers();
my $pedWrapper = new PedWrapper($inPedfile, $numOfMarkers);
my @familiesList = $pedWrapper->getFamiliesNumbers();
my $numDiseaseLoci = $$datWrapper{NUM_OF_DISEASE_LOCI};
if ($numDiseaseLoci==0) {$numDiseaseLoci++;}	#0 means 1 disease locus

my $currIndex = 0;
my $firstMarkerInGroup = 1;
my @distBetweenParts = (0);
my $indexOfFirstMarkerInLastClusterOfLastGroup = 1;
while ($firstMarkerInGroup <= $numOfMarkers) {
	$currIndex++;
	my $lastMarkerInGroup = $firstMarkerInGroup;
	my $numMarkersInGroup = 1;
	my $totalDist = 0;
	
	#Add markers to the group while they are close enough
	while ($lastMarkerInGroup < $numOfMarkers and $numMarkersInGroup <= $MAX_MARKERS_IN_SINGLE_CLUSTERING_RUN) {
		my $dist = $datWrapper->calcDistIn_cM_BetweenTwoMarkers($lastMarkerInGroup, $lastMarkerInGroup+1);
		if ($totalDist + $dist >= $cMorganWindowSize) {last;}
		$totalDist += $dist;
		$numMarkersInGroup++;
		$lastMarkerInGroup++;		
	}	
    
    #Save dat and ped files with only the relevant markers
	my @currentMarkersList = ($firstMarkerInGroup .. List::Util::min($firstMarkerInGroup + $numMarkersInGroup-1, $numOfMarkers));
    my $currentDatfileName = "part" . $currIndex . "_from_" . $firstMarkerInGroup. "_" . "$lastMarkerInGroup.datafile.dat";    
    my $currentPedfileName = "part" . $currIndex . "_from_" . $firstMarkerInGroup . "_" . "$lastMarkerInGroup.pedfile.dat";    	
	open DATFILE_FH, ">$currentDatfileName";	
	$datWrapper->writeSubDatFileByListOfMarker(\*DATFILE_FH, \@currentMarkersList);
	close DATFILE_FH;	
	open PEDFILE_FH, ">$currentPedfileName";
	$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@currentMarkersList, \@familiesList);
	close PEDFILE_FH;
    
    # In order to get the correct markers index we will send this parameter to the SNPdistiller
    my $numOfLastMarkerInThePrevWindow = $firstMarkerInGroup - 1;
    
    # Running the SNPdistiller on the files	
    my $currCommand = $DistillerCommand;
    $currCommand =~ s/_X_/$numOfLastMarkerInThePrevWindow/;
    $currCommand =~ s/datafile.dat/$currentDatfileName/;
    $currCommand =~ s/pedfile.dat/$currentPedfileName/;
    open(DISTCOMM, ">DistillerCommand.txt");
    print DISTCOMM "$currCommand";
    close DISTCOMM;		
    system("chmod 700 snpDistiller DistillerCommand.txt");		
    system("./DistillerCommand.txt");
    my $exit_value  = $? >> 8; 
    # Check if the distiller ended well
    if ( $exit_value != 0 ) {			
        open(ERRORLOG, "SNPdistillerErr.txt");
        my $errMsg = "";
        for my $line ( <ERRORLOG> ) {$errMsg = $errMsg . &trim($line);}
        close ERRORLOG;
        &DAGSplit::logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName\nended with $errMsg","clustering");
    }		
    
    #Checking the existence of the SNPdistiller output files
    unless (-e "clusterAll.dat" && -e "clusterAll.ped") {		
        &DAGSplit::logInnerSysError("Error with SNPdistiller runnig on $currentDatfileName");
    }	
	
	#Update the array @distBetweenParts
	if ($currIndex > 1) {
		my $dist = $datWrapper->calcDistIn_Theta_BetweenTwoMarkers($indexOfFirstMarkerInLastClusterOfLastGroup, $firstMarkerInGroup);		
		push(@distBetweenParts, $dist);
		
		#Update the field $indexOfFirstMarkerInLastClusterOfLastGroup		
		my $groupDatWrapper = new DatWrapper("clusterAll.dat");		
		my $lastMarkerName = ${$$groupDatWrapper{MARKERS}}[-1]->name();
		$lastMarkerName =~ /\((\d+)\)/;
		$indexOfFirstMarkerInLastClusterOfLastGroup = $1;		
	}
	
    system("mv clusterAll.dat clusterAll.$currIndex.dat");
    system("mv clusterAll.ped clusterAll.$currIndex.ped");        
	$firstMarkerInGroup += $numMarkersInGroup;	
}
system("rm -f results.txt");
&DAGSplit::joinFiles(@distBetweenParts, "clusterAll", $currIndex, $numDiseaseLoci);

`touch resultFile.mapfile.dat`;	#The dumb monitor requires this file to exist
exit(0);
#################################################################
