#!/usr/bin/perl -w
use strict;

use FindBin ();
use lib "$FindBin::Bin";

use General;

# dvir 2010/11/21
# when there is a LOCAL mount that more than $USAGE_THRESHOLD of its capacity is taken,
# than we will send a mail, warning about it (since that we expect that the mountes will be clean from older files in scripts like cleanWorkspace.pl)
my $USAGE_THRESHOLD = 66; # 66 is 2/3 mount

my $mountsStatusCommand = "df -h | grep '%.*/' | grep -v ':'";

# df -h shows how much space is in use in each drive (local drive or network drive).
# The grep '%.*/' is added in order :
# 1. not to refer to the headline -
# "Filesystem            Size  Used Avail Use% Mounted on" 
# which have the char "%" but doesn't have the char "/"
# 2. and to refer only to lines which shows the percentage of the usage (containes the char "%")
# and not to lines such as "/dev/mapper/vg_dsgpu02-lv_root".
# The grep -v ':' is added in order not to refer to network disks (disks which aren't on this computer)
# example for network disks: 
# cbl-disk01:/home      466G  312G  155G  67% /home
# cbl-disk01:/shared    219G   20G  197G  10% /shared
#           ^
my @mountUsageLocal = `$mountsStatusCommand`;
&checkMounts(\@mountUsageLocal);

my @mountUsageWebServer;
&General::executeSSHCommandOnWebServerAndCaptureOutput($mountsStatusCommand, \@mountUsageWebServer);
&checkMounts(\@mountUsageWebServer);

sub checkMounts( \@ )  {
	my $mountUsageArrRef = $_[0];
	
	my $message = '';
	foreach my $currMountUseStr (@$mountUsageArrRef) {
		print $currMountUseStr."\n";
		my @currMountUse = split(/\s+/,$currMountUseStr);
		my $percentageUseStr = $currMountUse[4];
		my $mountName = $currMountUse[5];
		if ($percentageUseStr !~ /%/) {
			warn "Wrong line of mount use: $currMountUseStr, it doesn't have % at the 5 field\n";
			next;
		}
		# removing the '#' from the percentage of use
		$percentageUseStr =~ s/%//;
		if ($percentageUseStr >= $USAGE_THRESHOLD) {
			$message .= "Mount: $mountName, Use: $percentageUseStr%\n";		
		}
	}
	
	if ($message ne '') {
		&General::sendMailToSuperlinkDeveloper("Mounts use exceed threshold ($USAGE_THRESHOLD%)", $message);
	}
}