#!/bin/bash
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
RedirOutToGlobalLog
# this script is called by the remote computer to submit a task.
# a task is submitted as a directory in a queue_dir, which is handled afterwards by the QueueHandler
ignore_load=0
[ -z "$1" ] && Die "Missing argument - <taskname>" 
[ -z "$2" ] || ignore_load=1

counter=20
while [ 0 -eq 0 ]; do
	ValidateLockfile $ACCEPT_SUBMISSION_LOCK || break;
	sleep 5
	let counter--
	[ $counter -eq 0 ] && exit -1 
done
clear(){
        rm ${ACCEPT_SUBMISSION_LOCK}  > /dev/null 2>&1
}
trap clear EXIT
	
taskdir="$QUEUE_DIR/$1"
workdir="$WORK_DIR/$1"

[ -e $taskdir -o -e $workdir ] && { 
	Error "Task with such name already exists: $1";
	exit 100;
}

# taskname is a UNIQUE name created by portal 

Log "Registrating new job $1"
LogSubmit $1
TestNoLogin && { 
	Log "NO_LOGIN detected, exiting";
	echo "The system is either experiencing major failure or is under maintenance. Try again later." 
	LogReject "NO_LOGIN"
	exit ${NO_LOGIN_DETECTED}; 
}

CountTasks 
total=$?
[ $total -gt ${MAX_ALLOWED_TASKS_IN_QUEUE} ] && {
	[ $ignore_load -eq 0 -o  $total -gt  ${MAX_ALLOWED_TASKS_IN_QUEUE_BACKGROUND} ] && {
		Log "There are too many tasks in the queue. Try again later."
		echo "There are too many tasks in the queue. Try again later."
		LogReject "LOAD_TOO_HIGH"
		exit ${LOAD_TOO_HIGH};
	}
}

# check the load on the gateway
EvalCPULoad
load=$CPU_local_load

[ $load -gt $QUEUE_HANDLER_MAX_CPU_LOAD -a $ignore_load -eq 0 ] && {
	Log "The gateway computer is overloaded. Try again later."
	echo "The gateway computer is overloaded. Try again later."
	LogReject "GW_CPU_HIGH"
	exit ${LOAD_TOO_HIGH};
}

# THIS is standard setup operation in case something happened to these dirs 
createDir $QUEUE_DIR || DieNoLogin "Failed to create $QUEUE_DIR"
createDir $WORK_DIR  || DieNoLogin "Failed to create $WORK_DIR"
createDir $MAIL_DIR || DieNoLogin "Failed to create $MAIL_DIR"
createDir $DONE_DIR || DieNoLogin "Failed to create $DONE_DIR"
createDir $LOG_DIR || DieNoLogin "Failed to create $LOG_DIR"

#


createDir $taskdir || DieNoLogin "Failed to create $taskdir"
# create lock to make sure that the queue handler checks only valid tasks
touch $taskdir/NOT_READY

# read the tar file and open it up into the taskdir

tar -C $taskdir -z -x -f - || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Was unable to untar the data"; }

[ -r ${taskdir}/pedfile.dat ] || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Missing pedfile.dat"; }
[ -r ${taskdir}/datafile.dat ] || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Missing datafile.dat"; }
[ -r ${taskdir}/${JOB_DETAILS_FILE} ] || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Missing ${JOB_DETAILS_FILE}"; }
[ -r ${taskdir}/all_execs.tar ] || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Missing all_execs.tar"; }

tar -C $taskdir -x -v -f ${taskdir}/all_execs.tar ||  { rm -rf $taskdir > /dev/null 2>&1 ; Die "Failed to untar all_execs.tar"; }
[ -e ${taskdir}/$FRONTEND_PRE_SCRIPT -a -e ${taskdir}/$FRONTEND_POST_SCRIPT ] || { rm -rf $taskdir > /dev/null 2>&1 ; DieNoLogin "Failed to find Pre and Post files in the all_execs.tar"; }
chmod +x  ${taskdir}/$FRONTEND_PRE_SCRIPT;
chmod +x  ${taskdir}/$FRONTEND_POST_SCRIPT;

#### add queue entry time#####
echo "TASK_QUEUE_ENTRY=\"`date`\"" >>  ${taskdir}/${JOB_DETAILS_FILE}
##############################

Log "Reading details file : $taskdir/${JOB_DETAILS_FILE}"
source $taskdir/${JOB_DETAILS_FILE} || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Was unable to read task parameters"; }
CheckJobSubmitDetails || { rm -rf $taskdir > /dev/null 2>&1 ; Die "Wrong task parameters"; }
[ -z "$TASK_TYPE" ] && TASK_TYPE="LINKAGE"
CheckTypeSupported || { rm -rf $taskdir > /dev/null 2>&1; Die "Task type $TASK_TYPE is not supported"; }


Log "Task ${TASK_PROJECT_NAME} enqueued"
rm $taskdir/NOT_READY > /dev/null 2>&1

exit 0
