#!/bin/bash

# Script constants
SCRIPT_NAME=$(basename $0)

# Exit codes
USAGE_EXIT=1
INVALID_INPUT=2
# End of exit codes

declare -r INVALID_INPUT
declare -r USAGE_EXIT
declare -r SCRIPT_NAME
# End of script constants

# Includes
. "${PEDTOOL_ROOT}/BoincNodeConstants.sh"
# End of includes

# Script variables
directory="."
dataped=""
datafile=""
pedfile="0"
location=""
monitoring_id=""
set_id=""
# End of script variables

# Auxiliary functions

# Print the help screen
function usage {
        cat << EOF
--dataped    <datafile number:pedfile number> : specifies the numbers of datafile and pedfile directories
                                                (e.g. if the path looks like datafile_T_X:pedfile_Y_Z, then the value of the flag should be _T_X:_Y_Z)
--route      <route number>                   : specifies the number of Superlink route, must be non-negative
[--directory <task working directory>]        : specifies the working directory of the task (the default is the current directory)
[--debug]                                     : turns on debugging information
[--help|--usage|-h]                           : displays this message
Usage example:
./${SCRIPT_NAME} --directory /p/125 --dataped 11:0 --route 0
EOF
}
# End of auxiliary functions
# Parsing the input
while [ $# -ge 1 ]; do
        case "$1" in
				"--directory" )         		shift; directory=$1;;
				"--dataped" )					shift; dataped=$1; datafile=$(echo ${dataped} | cut -f1 -d':'); pedfile=$(echo ${dataped} | cut -f2 -d':');;
				"--route" )						shift; monitoring_id=$1;;
                "--debug" )                     set -x;;
                "--help" )                      usage; exit ${USAGE_EXIT};;
                "-h" )                          usage; exit ${USAGE_EXIT};;
                "--usage" )                     usage; exit ${USAGE_EXIT};;
        esac
        shift
done
# End of parsing the input

# Validating the input
[[ -z "${directory}" || ! ( -d "${directory}" ) ]] && \
	echo "Invalid or missing task working directory" && echo && usage && exit ${INVALID_INPUT}
#[[ -z "${datafile}" || ! ( "${datafile}" =~ "^[0-9]+$" ) || ! ( -d "${directory}/datafile${datafile}" ) ]] && \
[[ -z "${datafile}" || ! ( -d "${directory}/datafile${datafile}" ) ]] && \
	echo "Invalid or missing datafile directory number" && echo && usage && exit ${INVALID_INPUT}

location="${directory}/datafile${datafile}/pedfile${pedfile}"

#[[ -z "${pedfile}" || ! ( "${pedfile}" =~ "^[0-9]+$" ) || ! ( -d "${location}" ) ]] && \
[[ -z "${pedfile}" || ! ( -d "${location}" ) ]] && \
	echo "Invalid or missing pedfile directory number" && echo && usage && exit ${INVALID_INPUT}
[[ -z "${monitoring_id}" || ! ( "${monitoring_id}" =~ "^[0-9]+$" ) ]] && 
	echo "Invalid or missing route number" && echo && usage && exit ${INVALID_INPUT}
# End of validating the input

cd "${directory}"

set_id=$(grep "${directory}/datafile${datafile}/pedfile${pedfile}" "${BOINC_PARAMETERS_FILE_NAME}" | cut -f1 -d' ')

find_debug_info $monitoring_id $set_id $location
exit 0
