#!/bin/bash

# For debugging purposes only
set -x

# Script constants
DIRECTORY_NAME=$(dirname $0)
SCRIPT_NAME=$(basename $0)

# Includes
. "$DIRECTORY_NAME/JobUtils.sh"
. "$DIRECTORY_NAME/BoincNodeConstants.sh"
# End of includes

FRONTEND=$(boinc_frontend)

declare -r DIRECTORY_NAME
declare -r SCRIPT_NAME
declare -r FRONTEND

# End of script constants

# Auxiliary functions

#submit_boinc_monitoring_job()
#{
#	local set_id=$1
#	local job_id=$2
#
#	#ssh $SSH_FLAGS $FRONTEND << EOF
#	execute_ssh_frontend << EOF
#		. \$HOME/.bash_profile; \
#		mysql $MYSQL_TABLE_FLAGS -e "insert into $MONITORING_ID_TABLE (set_id, id, mod_time) values ($set_id, $job_id, unix_timestamp())";
#EOF
#	return $?
#}
#
#submit_boinc_job()
#{
#	local directory=$1
#	local job_id=$2
#	local depth=$3
#	local priority=$4
#
#	local formatted_date=`date +%d-%m-%Y-%H-%M-%S`
#	local job_name="Point-$directory-job-$job_id-date-$formatted_date"
#	local superlink_parameters="datafile.dat pedfile.dat -e order.txt $job_id $depth"
#
##	ssh $SSH_FLAGS $FRONTEND ". \$HOME/.bash_profile; create_work_superlink.sh -path $directory -appname superlink -wu_name $job_name \
##				-priority $priority -command \"$superlink_parameters\" datafile.dat pedfile.dat order.txt"
#	execute_ssh_frontend << EOF
#		. \$HOME/.bash_profile; \
#		create_work_superlink.sh -path $directory -appname superlink -wu_name $job_name \
#			-priority $priority -command "$superlink_parameters" datafile.dat pedfile.dat order.txt
#EOF
#	local exit_code=$?
#	[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "SSH command failed (exit code ${exit_code}), unable to create a BOINC job"
#}

submit_boinc_monitoring_jobs()
{
	local set_id=$1
	local starting_id=$2
	local jobs_number=$3
	local submission_file_name=mysql_submission_file.${set_id}

	declare -r submission_file_name

	touch ${submission_file_name}

	# Opening the transaction
	echo "set autocommit=0; start transaction;" > ${submission_file_name}

	# We do not want to monitor the transaction preparation, as it might be a very long transaction
	set +x
	while read job_id
	do
		# Committing transaction when it reaches the pre-defined size
		if [[ $(($job_id % $BOINC_TRANSACTION_SIZE)) = 0 ]]
		then
			echo "commit; start transaction; " >> ${submission_file_name}
		fi

		echo "insert into $MONITORING_ID_TABLE (set_id, id, mod_time) values ($set_id, $job_id, unix_timestamp());" >> ${submission_file_name}
	done < "${ESTIMATION_PARAMETERS_FILE_NAME}.${set_id}"
	set -x

	# Closing the transaction
	echo "commit;" >> ${submission_file_name}
	echo "quit" >> ${submission_file_name}

	# Waiting until the MySQL process finishes and removing the named pipe.
	# The file path is identical, that's why we can use $PWD
#	ssh $SSH_FLAGS $FRONTEND ". \$HOME/.bash_profile; mysql $MYSQL_TABLE_FLAGS < $PWD/$submission_file_name" 
	execute_ssh_frontend << EOF
		. \$HOME/.bash_profile; \
		mysql $MYSQL_TABLE_FLAGS < "$PWD/${submission_file_name}"
EOF
	exit_code=$?
	[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "SSH command failed (exit code ${exit_code}), unable to create monitoring ids"
	
	#rm $submission_file_name
}
set -x
submit_boinc_jobs()
{
	local watermark=$(check_watermark ${TASKID})
	local exit_code=$?
	[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to find the number of currently running workunits (exit code ${exit_code})"
	[[ ${watermark} -gt ${lower_watermark} ]] && return 1

	local load_output

	load_output=$(load_submission_state_file ${lower_watermark})
	exit_code=$?
	[[ ${exit_code} != 0 && ${exit_code} != 1 ]] && \
		clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to read set id and line from the gradual submission state file (exit code ${exit_code})"
	# No more jobs to submit
	[[ ${exit_code} == 1 ]] && return 0
	local -a sequence_file_set_id=( ${load_output} )
	local sequence_file_path=${sequence_file_set_id[0]}
	local set_id=${sequence_file_set_id[1]}
	local set_id_counter=$(find_array_index "${set_ids[*]}" ${set_id})
	local directory=${working_directories[${set_id_counter}]}
	local jobs_number=${jobs_numbers[${set_id_counter}]}
	local depth=${depths[${set_id_counter}]}
	local priority=${priorities[${set_id_counter}]}
	local rsc_fpops_est=0;
	
	rsc_fpops_est=$(floating_point_estimate "${directory}")
	update_bundle_size

	local bundle=$(bundle_size)
	local bundle_flag="--bundle ${bundle}"
	#local sequence_file_path=$(pwd)"/${ESTIMATION_PARAMETERS_FILE_NAME}.${set_id}"

	rsc_fpops_est=$(echo "scale = 0; ${rsc_fpops_est} * ${bundle} / ${BOINC_INITIAL_BUNDLE_SIZE}" | bc -l)

	execute_ssh_frontend << EOF
	. \$HOME/.bash_profile; template_file=\$(mktemp); cat \$PROJECT_ROOT/templates/${WU_TEMPLATE_FILE_NAME} | sed 's/<rsc_fpops_est>.*<\/rsc_fpops_est>/<rsc_fpops_est>${rsc_fpops_est}<\/rsc_fpops_est>/' > \$template_file; make_work_superlink_transaction.sh ${bundle_flag} --template_file \$template_file --superlink_path "${directory}" \
		--sequence_file "${sequence_file_path}" --depth ${depth} --priority ${priority} --transaction_size ${BOINC_TRANSACTION_SIZE} --task_id ${TASKID}; exit_code=\$?; rm -f \$template_file; exit \$exit_code
EOF
	exit_code=$?
	#[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "SSH command failed (exit code ${exit_code}), unable to create BOINC jobs"
	[[ ${exit_code} != 0 ]] && Error "SSH command failed (exit code ${exit_code}), unable to create BOINC jobs" && return 1
	#rm -f $sequence_file_path
	update_submission_state_file ${lower_watermark}
	exit_code=$?
	[[ ${exit_code} != 0 && ${exit_code} != 1 ]] && \
        clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to update the gradual submission state file (exit code ${exit_code})"
	# Updating the total jobs file for tracking down the progress status is done in the post-estimation stage
	# Updating the completed jobs file for tracking down the progress status
	[ ! -f "${directory}/${TOTAL_DONE_JOBS_FILE}" ] && echo 0 > "${directory}/${TOTAL_DONE_JOBS_FILE}"
}

submit_jobs()
{
	local set_id=$1
	local directory=$2
	local starting_id=$3
	local jobs_number=$4
#	local depth=$5
#	local priority=$6

	# Transactional submission of monitoring and BOINC jobs
	submit_boinc_monitoring_jobs ${set_id} ${starting_id} ${jobs_number} 
	#submit_boinc_jobs "${set_id}" "$directory" $starting_id $jobs_number $depth $priority
 	submit_boinc_jobs 
	# Give transitioner a time to create workunits
	sleep 5
}
# Given a finished Condor cluster job and set id of the BOINC job, removes the Condor job from the Condor queue or restarts it (if it finished unsuccessfully)
# and updates the job's status in the monitoring tables 
#remove_condor_job()
#{
#	local condor_job=$1
#	local set_id=$2
#	local working_directory=$3
#
#	# The job id is fifth argument
#	local args=$(condor_q ${condor_job} -format "%s" Args)
#	local job_id=$(echo ${args} | awk '{ print $5 }' )
#	local depth=$(echo ${args} | awk '{ print $6 }' )
#	local condor_q_output=( $(condor_q $condor_job -format "%s " JobStatus -format "%d " JobPrio -format "%d " ExitCode -format "%d" ExitBySignal ) )
#	local job_status=${condor_q_output[0]}
#	local priority=${condor_q_output[1]}
#	local condor_exit_code=${condor_q_output[2]}
#	local condor_exit_by_signal=${condor_q_output[3]}
#	local exit_code=0
#
#	if [[ "x$job_status" == "x$CONDOR_JOB_COMPLETED" && "x${condor_exit_by_signal}" == "x0" && "x${condor_exit_code}" == "x0" ]]
#	then
#		change_jobs_status $set_id $job_id $FINISHED_STATUS 
#		exit_code=$?
#	elif [[ ( "x$job_status" == "x$CONDOR_JOB_COMPLETED" && "x${condor_exit_by_signal}" == "x1" ) || "x${job_status}" == "x${CONDOR_JOB_REMOVED}" || \
#			"x${job_status}" == "x${CONDOR_JOB_HELD}" ]]
#	then
#		restart_condor_job ${set_id} "${working_directory}" ${job_id} ${depth} ${priority}
#		exit_code=$?
#	# Condor job ended successfully, but the exit code was non-zero
#	else
#		report_progress_log "${working_directory}" ${job_id} "Fatal error" "no more restarts left and Condor job failed"
#		#finish_set $set_id "$FAILED_STATUS"
#		clean_sets ${set_id} ${TASKID}
#		exit_code=$?
#
#		MarkError "${working_directory}"
#		exclude_failed_jobs_total "${working_directory}"
#	fi
#	# If the job status changing was unsuccessful, we do not remove the respective Condor job from the queue and try the next time
#	[[ ${exit_code} != 0 ]] && return ${exit_code}
#	# Verifying that the job is purged from the local queue
#	# condor_rm -forcex causes problems to schedd due to an internal bug, that is why we switched to use condor_qedit instead
#	#condor_qedit $condor_job LeaveJobInQueue false 
#	#condor_rm $condor_job
#
#	# Appending the job to the purger's list
#	echo "ClusterId integer ${job_id}" >> "${PURGE_DIR}/${TASKID}"
#	"${DIRECTORY_NAME}/Purger.sh"
#}

# Given finished Condor cluster jobs, ids and set ids of the BOINC job, removes all the Condor jobs from the Condor queue or restarts them
# and updates the jobs' statuses in the monitoring tables
# Remark: upon success updates the monitoring tables in one SQL statement
remove_condor_jobs()
{
	# Arguments, passed as arrays
	local -a condor_jobs=( $1 )
	local -a set_ids=( $2 )
	local -a job_ids=( $3 )
	local -a working_directories=( $4 )
	local -a job_statuses=( $5 )
	local -a priorities=( $6 )
	local -a signals=( $7 )
	local -a exit_codes=( $8 ) 

	# Local variables
	local finished_set_ids=""
	local finished_job_ids=""
	local -a restarted_set_ids=()
	local -a restarted_job_ids=()
	local -a restarted_directories=()
	local -a restarted_priorities=()
	local -a restarted_condor_jobs=()
	# End of local variables
	
	for index in $(seq 0 $(( ${#condor_jobs[@]} - 1 )) )
	do
		local condor_job=${condor_jobs[$index]}
		local set_id=${set_ids[$index]}
		local job_id=${job_ids[$index]}
		local working_directory=${working_directories[$index]}
		local job_status=${job_statuses[${index}]}
		local priority=${priorities[${index}]}
		local condor_exit_by_signal=${signals[${index}]}
		local condor_exit_code=${exit_codes[${index}]}
		local restarts=""

		restarts=$(execute_ssh_frontend << EOF
			. \$HOME/.bash_profile; \
			mysql $MYSQL_FLAGS -e "select mi.restarts from $MONITORING_ID_TABLE mi where mi.id = ${job_id} and mi.set_id = ${set_id}"
EOF
		)
		exit_code=$?
		[[ ${exit_code} != 0 ]] && return ${exit_code}

		if [[ "x${job_status}" == "x${CONDOR_JOB_COMPLETED}" && "x${condor_exit_code}" != "x0" && ${restarts} -ge ${RESTARTS_LIMIT} ]]
		then
			report_progress_log "$working_directory" $job_id "Fatal error" "no more restarts left and Condor job failed"
			#clean_sets ${set_id} ${TASKID}
			finish_set  ${set_id} ${FINISHED_STATUS}
			# If changing the jobs' status was unsuccessful, we do not remove Condor jobs from the queue and try the next time
			exit_code=$?
			#[[ ${exit_code} != 0 ]] && return ${exit_code}
			
			# Creating the error file for tracking down the progress of the directory
			MarkError "${working_directory}"
			exclude_failed_jobs_total "${working_directory}"
		# Restarting Condor jobs that have not exhausted their restarts limit
		elif [[ "x${job_status}" == "x${CONDOR_JOB_COMPLETED}" && "x${condor_exit_code}" != "x0" ]]
		then
			#local set_id_counter=$(find_set_id_counter ${set_id})
			local set_id_counter=$(find_array_index "${set_ids[*]}" ${set_id})

			restart_condor_job_monitoring ${set_id} "${working_directory}" ${job_id} ${depths[${set_id_counter}]} ${priority} ${restarts}
			exit_code=$?
			[[ ${exit_code} != 0 ]] && Warn "Unable to restart job id ${job_id} from set id ${set_id} from directory ${working_directory} (exit code ${exit_code})"
		elif [[ "x${job_status}" == "x${CONDOR_JOB_COMPLETED}" && "x${condor_exit_by_signal}" == "x0" && "x${condor_exit_code}" == "x0" ]]
		then
			finished_set_ids="$finished_set_ids $set_id"
			finished_job_ids="$finished_job_ids $job_id"
		else
			restarted_set_ids=( ${restarted_set_ids[@]} ${set_id} )
			restarted_job_ids=( ${restarted_job_ids[@]} ${job_id} )
			restarted_directories=( ${restarted_directories[@]} ${working_directory} )
			restarted_priorities=( ${restarted_priorities[@]} ${priority} )
			restarted_condor_jobs=( ${restarted_condor_jobs[@]} ${condor_job} )
		fi
	done

	if [[ "${restarted_set_ids}" != "x" ]]
	then
		# To have the Condor jobs printed out in the same order, we have to append the ClusterId
		job_index=-1

		for set_id in ${restarted_set_ids[@]}
		do
			let job_index++
			working_directory=${restarted_directories[${job_index}]}
			job_id=${restarted_job_ids[${job_index}]}
			priority=${restarted_priorities[${job_index}]}
			depth=$(condor_q ${restarted_condor_jobs[${job_index}]} -format "%s" Args | awk '{print $6}')

			restart_condor_job ${set_id} "${working_directory}" ${job_id} ${depth} ${priority}
		done
	fi

	if [[ "x$finished_set_ids" != "x" ]]
	then
		change_jobs_status "$finished_set_ids" "$finished_job_ids" $FINISHED_STATUS
		# If changing the jobs' status was unsuccessful, we do not remove Condor jobs from the queue and try the next time
		exit_code=$?
		[[ ${exit_code} != 0 ]] && return ${exit_code} 
	fi
	# Appending the set to the purger's list
	echo "ClusterId integer ${condor_jobs[@]}" >> "${PURGE_DIR}/${TASKID}"
	"${DIRECTORY_NAME}/Purger.sh"
}

# Given set id, working directory and failed job id submits the corresponding job to the local Condor queue, updating the
# monitoring status of the job
submit_condor_job()
{
	local set_id=$1
	local directory=$2
	local job_id=$3
	local depth=$4
	local priority=$5

	local escaped_directory_name=$(echo ${DIRECTORY_NAME} | sed 's/\//\\\//g')
	local escaped_directory=$(echo ${directory}           | sed 's/\//\\\//g')

	cat ${SUBFILES}/${CONDOR_NODE_NAME}.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_EXECDIR_/${escaped_directory_name}/g" | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_JOB_ID_/${job_id}/g"                  | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_DEPTH_/${depth}/g"                    | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_PRIORITY_/${priority}/g"              | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_INITIAL_DIR/${escaped_directory}/g"   | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_SET_ID_/${set_id}/g"                  | \
	                                           sed "s/_PEDTOOL_SUBSTITUTE_TASKID_/${TASKID}/g" > condor_node.sub
#	cat > condor_node.sub << EOF
#executable = ${DIRECTORY_NAME}/Superlink/Superlink_Vanilla.\$\$(Arch).\$\$(OpSys)
#arguments = datafile.dat pedfile.dat -e order.txt ${job_id} ${depth}
#
#transfer_input_files = pedfile.dat, datafile.dat, order.txt
#universe = vanilla
#should_transfer_files = YES
#when_to_transfer_output = ON_EXIT
#priority = ${priority}
#
#InitialDir = ${directory}
#output     = output.condor_node.${set_id}.${job_id}
#error      = error.condor_node.${set_id}.${job_id}
#log        = log.condor_node.${set_id}.${job_id}
#
#Requirements = Memory >= 450 && \\
#		((Arch == "INTEL" && OpSys == "LINUX")   || \\
#		(Arch == "X86_64" && OpSys == "LINUX")   || \\
#		(Arch == "ppc64"  && OpSys == "LINUX")   || \\
#		(Arch == "INTEL"  && OpSys == "WINNT51") || \\
#		(Arch == "INTEL"  && OpSys == "WINNT50"))
#
#ImageSize     = 100
#Rank          = KFlops 
#copy_to_spool = False
#
#getenv = true
#notification = never
#+MonitoringSetId = "${set_id}"
#+MonitoringId = "${job_id}"
#+TaskId = "${TASKID}"
#leave_in_queue = true
#queue
#EOF
	safe_condor_submit condor_node.sub && change_jobs_status $set_id $job_id $CONDOR_STATUS 
	exit_code=$?

	#[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "Condor submission of set id ${set_id}, job id ${job_id} from ${directory} or its status changing failed"
	# If we did not succeed to start the job, mark it as failed and hope to start it the next time
	[[ ${exit_code} != 0 ]] && Error "Unable to submit Condor job of set id ${set_id}, job id ${job_id} from ${directory} or unable to change its status" && \
							   change_jobs_status $set_id $job_id $FAILED_STATUS
	return ${exit_code}
}

#find_set_id_counter()
#{
#	local requested_set_id=$1
#	local set_id_counter=-1
#
#	for set_id in ${set_ids[*]}
#	do
#		let set_id_counter++
#		[[ "x$set_id" == "x${requested_set_id}" ]] && break
#	done
#	echo ${set_id_counter}
#}
# End of auxiliary functions

# Main
Log "Starting $SCRIPT_NAME"

# Script parameters processing
parameters_file_path=$1
lower_watermark=$(submission_lower_watermark)

declare -a set_ids
declare -a working_directories
declare -a depths
declare -a jobs_numbers
declare -a priorities

[ -s  $parameters_file_path ] || Die "Expected BOINC node parameter file, but did not find one: $parameters_file_path";

read_parameters_file $parameters_file_path

[ -z "${set_ids[*]}" ] && Die "Zero sets in BOINC node";


[ -e $UBNORMAL_TERMINATION_FILE ] && {
	Log "Abnormal termination file detected - exiting"
	exit -1;
}
# Guaranteeing the set_ids are unique 
# For debugging purposes only
#set_ids=(1)
#jobs_numbers=(1)
#depths=(19)
#working_directories=(/tmp)
# End of script parameters processing

if [ ! -z "$TASK_CHECKPOINT" ]; then
	Log " TASK_CHECKPOINT request found in Details.txt - exiting and check pointing";
	Log " To recover from the checkpoint, delete TASK_CHECKPOINT line from Details.txt";
	touch $BOINC_CHECKPOINT;
	exit -1;
fi	

# TERM signal from the DAG root to clean up all its children details
trap 'clean_sets "${set_ids[*]}" ${TASKID}; exit 1' USR1 
trap '. "$DIRECTORY_NAME/JobUtils.sh"; . "$DIRECTORY_NAME/BoincNodeConstants.sh"; update_classad ${TASKID}' USR2

# Create monitoring tables in the database if they still do not exist there. 
Log "Creating monitoring tables if they do not exist there"
create_monitoring_tables
exit_code=$?
[[ ${exit_code} != 0 ]] && Warn "Unable to creating monitoring tables (exit code ${exit_code})"

# With this priority the new monitoring sets will be created
new_priority=$(default_priority ${TASKID})
#new_priority=$(( ${DEFAULT_PRIORITY} - ${TASKID} ))
Log "The sets priority is chosen to be $new_priority"

# Since we are not going to unset none of the arrays' members, the arrays indices are successive natural numbers
# Adding classads after the monitoring tables modifications; cannot be done before that, since the db_purge might have deleted the unmatching classads
Log "Inserting new classad"
add_classad ${TASKID}

set_id_counter=-1

for set_id in ${set_ids[*]}
do
	let set_id_counter++

	jobs_number=${jobs_numbers[$set_id_counter]}
	depth=${depths[$set_id_counter]}
	working_directory=${working_directories[$set_id_counter]}
	priorities[$set_id_counter]=$new_priority

	# Obtaining the status of our monitoring set
	mysql_status=$(execute_ssh_frontend << EOF
				. \$HOME/.bash_profile; \
				mysql $MYSQL_FLAGS -e "select status from $MONITORING_SET_TABLE where id = $set_id"
EOF
		       ) 
	exit_code=$?
	[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to obtain the status of set id ${set_id} (exit code ${exit_code})"
	greatest_id=0
	# Status of the monitoring set is initialized to "Filling", if the set has not been created
	if [[ "x$mysql_status" == "x" ]]
	then
		Log "Filling the $MONITORING_SET_TABLE table with new set no. $set_id and priority $new_priority"

		execute_ssh_frontend << EOF
			. \$HOME/.bash_profile; \
			mysql $MYSQL_TABLE_FLAGS -e "insert into $MONITORING_SET_TABLE (id, directory, priority, task_id) \
						     values ($set_id, '$working_directory', $new_priority, ${TASKID})";
EOF
		exit_code=$?
		[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && \
			Die "Unable to insert a new set id ${set_id} from ${working_directory} to the ${MONITORING_SET_TABLE} table (exit code ${exit_code})"
		mysql_status="$FILLING_STATUS"
#	elif [[ "x$mysql_status" == "x$FILLING_STATUS" ]]
#	then
#		# Checking the greatest id, pertaining to this monitoring set, if existent
#		max_id=$(execute_ssh_frontend << EOF
#				. \$HOME/.bash_profile; \
#				mysql $MYSQL_FLAGS -e "select max(mi.id) from $MONITORING_ID_TABLE mi where mi.set_id = $set_id"
#EOF
#			)
#		exit_code=$?
#		[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to find the greatest job id, pertaining to set id ${set_id} (exit code ${exit_code})"
#
#		# Examining if the BOINC job, corresponding to the greatest id, has been launched. Launching it, if it hasn't
#		if [[ "x${max_id}" != "xNULL" ]]
#		then
#			greatest_id=${max_id}
#			#submit_if_boinc_job_not_exists "$working_directory" $greatest_id $depth ${priorities[$set_id_counter]} 
#			Log "Submitting BOINC jobs, corresponding to the orphan monitoring records"
#			submit_if_boinc_jobs_not_exist ${set_id} "${working_directory}" ${greatest_id} ${jobs_number} ${depth} ${priorities[$set_id_counter]} 
#			let greatest_id++
#		fi
	fi

	# Alternatively filling the monitoring tables with data and submitting jobs to BOINC, starting from the found greatest id. 
	if [[ "x${mysql_status}" == "x${FILLING_STATUS}" ]]
	then
		Log "Submitting BOINC jobs and auditing monitoring information"
		submit_jobs ${set_id} "${working_directory}" ${greatest_id} $(( $jobs_number - $greatest_id )) #$depth ${priorities[$set_id_counter]}
		change_sets_status ${set_id} "${MONITORING_STATUS}"
		exit_code=$?
		[[ ${exit_code} != 0 ]] && clean_sets "${set_ids[*]}" ${TASKID} && Die "Unable to change the status of set id ${set_id} to ${MONITORING_STATUS}"
	fi
done

set_ids_list=$(echo ${set_ids[@]} | tr ' ' ',')
iwd_constraint=""
operator=""

for working_directory in ${working_directories[*]}
do
	iwd_constraint="$iwd_constraint $operator Iwd==\"$working_directory\""
	operator="||"
done
regexp_constraint="regexp(\".*Superlink_Vanilla.*\",Cmd)"
submission_counter=0
tail=0
# In this part of the script the status of the monitoring set is "Monitoring"
# Here we monitor several events, the rest of the time we just sleep:
#
# 1) Gradual submission of yet unsubmitted BOINC jobs
# 2) Monitoring jobs, forwarded to the local Condor pool and accomplished there
# 3) Monitoring failed BOINC jobs
# 4) Monitoring stuck Condor jobs
# 5) Monitoring finished BOINC jobs
# 6) Monitoring the tail proximity
# 7) Running outliers' analysis
while [[ $(are_all_finished "${set_ids[*]}") == 0 ]]
do
	sleep $BOINC_ROUND_TIME
	# 1) Gradual submission of yet unsubmitted BOINC jobs
	[[ ${submission_counter} == ${BOINC_SUBMISSION_FREQUENCY} ]] && 
	{
		submit_boinc_jobs
		submission_counter=0
	}
	# End of gradual submission

	let submission_counter++
	set_id_counter=0

	# 2) Monitoring jobs, forwarded to the local Condor pool
	declare -a condor_jobs_details=()
	condor_jobs=""
	remove_set_ids=""
	remove_job_ids=""
	remove_job_statuses=""
	remove_job_priorities=""
	remove_job_signals=""
	remove_working_directories=""
	remove_job_exit_codes=""

	# ExitCode comes last as it might be missing for various job statuses at all
	condor_q_output=$(condor_q -format "%s" ClusterId -format " %s" MonitoringSetId -format " %s" MonitoringId \
							   -format " %d" JobStatus -format " %d" JobPrio -format " %d" ExitBySignal -format " %s" Iwd -format " %d\n" ExitCode \
							   -constraint "${iwd_constraint}" -constraint "JobStatus > 2" \
							   -constraint "${regexp_constraint}" )
	exit_code=$?
	[[ ${exit_code} != 0 ]] && Warn "Unable to query the Condor queue (exit code ${exit_code})"
	[[ ${exit_code} == 0 ]] && condor_jobs_details=( ${condor_q_output} )
	# If there was an outage of condor_schedd, this loop is not executed
	regular_query_size=8
	index=0
	#for index in $(seq 0 $(( ${#condor_jobs_details[@]} - 1)) )
	while [[ ${index} -lt ${#condor_jobs_details[@]} ]]
	do
		condor_jobs="$condor_jobs ${condor_jobs_details[ ${index} ]}"
		remove_set_ids="$remove_set_ids ${condor_jobs_details[ $(( ${index} + 1 )) ]}"
		remove_job_ids="$remove_job_ids ${condor_jobs_details[ $(( ${index} + 2 )) ]}"
		remove_job_statuses="$remove_job_statuses ${condor_jobs_details[ $(( ${index} + 3 )) ]}"
		remove_job_priorities="${remove_job_priorities} ${condor_jobs_details[ $(( ${index} + 4 )) ]}"
		remove_job_signals="${remove_job_signals} ${condor_jobs_details[ $(( ${index} + 5 )) ]}"
		remove_working_directories="$remove_working_directories ${condor_jobs_details[ $(( ${index} + 6 )) ]}"
		if [[ "${condor_jobs_details[ $(( ${index} + 3 )) ]}" == ${CONDOR_JOB_COMPLETED} ]]
		then 
			remove_job_exit_codes="${remove_job_exit_codes} ${condor_jobs_details[ $(( ${index} + 7 )) ]}" 
			let index+=${regular_query_size}
		elif [[ "${condor_jobs_details[ $(( ${index} + 3 )) ]}" != ${CONDOR_JOB_COMPLETED} ]]
		then
			remove_job_exit_codes="${remove_job_exit_codes} 0"
			let index+=${regular_query_size}-1
		fi
	done

	if [[ "x$condor_jobs" != "x" ]]
	then
		Log "Removing Condor jobs $condor_jobs and updating the corresponding monitoring information"
		remove_condor_jobs "$condor_jobs" "$remove_set_ids" "$remove_job_ids" "$remove_working_directories" "$remove_job_statuses" \
						   "${remove_job_priorities}" "${remove_job_signals}" "${remove_job_exit_codes}"
		exit_code=$?
		[[ ${exit_code} != 0 ]] && \
			Warn "Unable to remove Condor jobs (${condor_jobs}), whose set ids are ${remove_set_ids} and job ids are ${remove_job_ids} "\
				 "from ${remove_working_directories} (exit code ${exit_code})"
	fi
	# End of monitoring jobs, forwarded to the local Condor pool

	# 3) Monitoring failed jobs
	declare -a failed_jobs_and_sets=() failed_jobs=() failed_jobs_restarts=() failed_jobs_sets=()
	declare total_condor_jobs=0

	failed_jobs_and_sets_output=$(execute_ssh_frontend << EOF
			. \$HOME/.bash_profile; \
			mysql $MYSQL_FLAGS -e "select mi.id, mi.set_id \
			from $MONITORING_ID_TABLE mi \
			where mi.status = '$FAILED_STATUS' and mi.set_id in ($set_ids_list) order by mi.id"
EOF
	)
	exit_code=$?
	[[ ${exit_code} != 0 ]] && Warn "Unable to obtain a data about failed jobs (exit code ${exit_code})"
	# We calculate the number of running and idle Superlink jobs only
set -x
	[[ ${exit_code} == 0 ]] &&
	{
		failed_jobs_and_sets=( ${failed_jobs_and_sets_output} )
		total_condor_jobs=$(( $(condor_q -constraint "JobStatus <= 2" -constraint "${regexp_constraint}" | wc -l ) - 4 ))

		if [ ! -z "${failed_jobs_and_sets[@]}" ]; then
			[[ ${total_condor_jobs} -ge ${CONDOR_MAXIMAL_QUEUE_LENGTH} ]] &&
			{ 
				Warn "Local cluster is overloaded, postponing the transfer of Condor jobs ${failed_jobs_and_sets[@]}"
				continue
			}
		fi
	}
	# If there was an outage of the database server, this loop is not executed
	# The denominator in the expression for the upper bound of the sequence and the multiplicative coefficient of the index is 
	# the number of columns, returned from the above query
	columns_number=2
# set +x
	for index in $(seq 0 $(( ${#failed_jobs_and_sets[@]} / ${columns_number} - 1 )) )
	do
		failed_job=${failed_jobs_and_sets[ $(( ${columns_number} * $index )) ]}
		#failed_job_restart=${failed_jobs_and_restarts[ $(( ${columns_number} * $index + 1 )) ]}
		failed_job_set=${failed_jobs_and_sets[ $(( ${columns_number} * $index + 1 )) ]}

		# We need the counter to determine the specific failed job directory, depth and priority	
		#set_id_counter=0
		#set_id_counter=$(find_set_id_counter ${failed_job_set})
		set_id_counter=$(find_array_index "${set_ids[*]}" ${failed_job_set})
		#for set_id in ${set_ids[*]}
		#do
		#	[[ "x$set_id" == "x$failed_job_set" ]] && break
		#	let set_id_counter++
		#done
		failed_job_directory=${working_directories[$set_id_counter]}
		failed_job_depth=${depths[$set_id_counter]}
		failed_job_priority=${priorities[$set_id_counter]}
	
		# If we hit the restarts limit for any of the jobs, route it to the local Condor pool when it is not full
		#if [[ "${failed_job_restart}" == "${RESTARTS_LIMIT}" ]]
		#then
		
		report_progress_log "${failed_job_directory}" ${failed_job} "Transfer" "moving to the local Condor pool"
		submit_condor_job ${failed_job_set} "${failed_job_directory}" ${failed_job} ${failed_job_depth} "${failed_job_priority}"
		exit_code=$?
		[[ ${exit_code} == 0 ]] && let total_condor_jobs++
		# Otherwise restart it and report about the restart
		#else
		#	report_progress_log "$failed_job_directory" $failed_job "Restart" "restarted $(($failed_job_restart+1)) time"
		#	restart_job $set_id "$failed_job_directory" $failed_job "$failed_job_depth" $failed_job_priority
		#	exit_code=$?
		#	[[ ${exit_code} != 0 ]] && Warn "Unable to restart job id ${failed_job} from set id ${set_id} from directory ${failed_job_directory} (exit code ${exit_code})"
		#fi
	done
	# End of monitoring failed jobs

	# 4) Monitoring stuck Condor jobs
	declare -a stuck_jobs=()
	# We are restarting all the jobs that are stuck in the $CONDOR_STATUS state for several times the $ESTIMATION_UPPER_BOUND
	stuck_jobs_output=$(execute_ssh_frontend << EOF
		. \$HOME/.bash_profile; \
		mysql $MYSQL_FLAGS -e "select mi.id, mi.set_id, ms.directory, ms.priority from ${MONITORING_ID_TABLE} mi, ${MONITORING_SET_TABLE} ms \
							   where mi.set_id = ms.id and mi.status = '${CONDOR_STATUS}' and mi.set_id in ($set_ids_list) and \
			  				   mi.mod_time < unix_timestamp() - ${ESTIMATION_UPPER_BOUND} * ${BOINC_UPPER_BOUND_SAFETY_COEFFICIENT}"
EOF
	)
	exit_code=$?
	[[ ${exit_code} != 0 ]] && Warn "Unable to obtain a data about stuck jobs (exit code ${exit_code})"
	[[ ${exit_code} == 0 ]] && stuck_jobs=( ${stuck_jobs_output} )
	# If there was an outage of the database server, this loop is not executed
	# The denominator in the expression for the upper bound of the sequence and the multiplicative coefficient of the index is 
	# the number of columns, returned from the above query
	for index in $(seq 0 $(( ${#stuck_jobs[@]} / 4 - 1 )) )
	do
		job_id=${stuck_jobs[ $(( 4 * $index )) ]}
		set_id=${stuck_jobs[ $(( 4 * $index + 1 )) ]}
		directory=${stuck_jobs[ $(( 4 * $index + 2 )) ]}
		priority=${stuck_jobs[ $(( 4 * $index + 3)) ]}

		[[ ${total_condor_jobs} -ge ${CONDOR_MAXIMAL_QUEUE_LENGTH} ]] && { 
			Warn "Local cluster is overloaded, postponing the transfer of Condor job ${job_id} from set ${set_id} and directory ${directory}"
			continue
		}
		#set_id_counter=$(find_set_id_counter ${set_id})
		set_id_counter=$(find_array_index "${set_ids[*]}" ${set_id})
		#for set_id_counter in $(seq 0 $(( ${#set_ids[@]} - 1 )) )
		#do
		#	[[ "x${set_ids[$set_id_counter]}" == "x$set_id" ]] && break
		#done
		depth=${depths[${set_id_counter}]}
		#priority=${priorities[${set_id_counter}]}
		restart_condor_job ${set_id} "${directory}" ${job_id} ${depth} ${priority}
		exit_code=$?
		[[ ${exit_code} == 0 ]] && let total_condor_jobs++ 
	done
	# End of monitoring stuck Condor jobs

	# 5) Monitoring finished jobs
	declare -a finished_jobs_and_status=()
	# If the entire jobs' set is finished, that is, all of its jobs are finished, we change its status to "Finished" too
	finished_jobs_and_status_output=$(execute_ssh_frontend << EOF
			. \$HOME/.bash_profile; \
			mysql $MYSQL_FLAGS -e "select count(*), ms.id, ms.status \
			from $MONITORING_ID_TABLE mi , $MONITORING_SET_TABLE ms \
			where ms.id = mi.set_id and mi.status = '$FINISHED_STATUS' \
			and ms.status <> '$FINISHED_STATUS' and ms.id in ($set_ids_list)\
			group by ms.id, ms.status"
EOF
	)
	exit_code=$?
	[[ ${exit_code} != 0 ]] && Warn "Unable to obtain a data about finished jobs (exit code ${exit_code})"
	[[ ${exit_code} == 0 ]] && finished_jobs_and_status=( ${finished_jobs_and_status_output} )
	finished_set_ids=""
	# Determines the number of unfinished routes; this is a means of controlling the number of resources in progress by the gradual submission mechanism
	#declare total_unfinished_routes_number=0
	# If there was an outage of the database server, this loop is not executed
	# The denominator in the expression for the upper bound of the sequence and the multiplicative coefficient of the index is 
	# the number of columns, returned from the above query
	for index in $(seq 0 $(( ${#finished_jobs_and_status[@]} / 3 - 1 )) )
	do
		jobs_number=${finished_jobs_and_status[ $(( 3 * $index )) ]}
		set_id=${finished_jobs_and_status[ $(( 3 * $index + 1 )) ]}
		status=${finished_jobs_and_status[ $(( 3 * $index + 2 )) ]}

		set_id_counter=$(find_array_index "${set_ids[*]}" ${set_id})
		routes_number=$( valid_routes_number $set_id )
		[[ "x${routes_number}" == "x${jobs_number}" ]] && finished_set_ids="$finished_set_ids $set_id"

		#let total_unfinished_routes_number+=${routes_number}-${jobs_number}
		# Updating the completed jobs file for tracking down the progress status of the directory.
		# Sets with outliers are pointed by symbolic links, hence lest override the content of ${TOTAL_DONE_JOBS_FILE},
		# we check the name of the directory
		directory_name=$(basename "${working_directories[$set_id_counter]}")
		[[ "${directory_name}" != "${OUTLIERS_EXTENSION}-${set_id}" ]] && echo $jobs_number > "${working_directories[$set_id_counter]}/$TOTAL_DONE_JOBS_FILE"
	done

	[[ "x$finished_set_ids" != "x" ]] &&
	{
		Log "Changing the monitoring status of sets $finished_set_ids to $FINISHED_STATUS"
		[ -e ${working_directories[$set_id_counter]}/GATHERED_RESULTS ] && rm ${working_directories[$set_id_counter]}/GATHERED_RESULTS

		change_sets_status "$finished_set_ids" "$FINISHED_STATUS"
		exit_code=$?
		[[ ${exit_code} != 0 ]] && Warn "Unable to change the status of set ids ${finished_set_ids} (exit code ${exit_code})"
	}
	# End of monitoring finished jobs

	# 6) Monitoring the tail proximity
	[[ "${tail}" -ne 1 && $(is_tail_close "${set_ids[*]}") -eq 1 ]] &&
	{
		tail=1
		Log "Detected task tail, modifying the classad"
		cp ${BOINC_CLASSADS} ${BOINC_CLASSADS}.before_tail
		cat "${BOINC_CLASSADS}" | sed 's/'${TAIL}'[ ]*=\([^=][^;]*\);/'${TAIL}' = true;/' | sed 's/'${TAIL}'[ ]*=\([^=][^;]*\)\]/'${TAIL}' = true \]/' > "${BOINC_CLASSADS}.new"
#		# Enforce the maximal possible concurrency level and minimal possible number of duplications
#		cat "${BOINC_CLASSADS}" | sed 's/'${CONCURRENCY}'[ ]*=\([^=][^;]*\);/'${CONCURRENCY}' = 1;/' | sed 's/'${CONCURRENCY}'[ ]*=\([^=][^;]*\)\]/'${CONCURRENCY}' = 1 \]/' | \
#								  sed 's/'${SCHEDULER_REQUIREMENTS}'[ ]*=\([^=][^;]*\);/'${SCHEDULER_REQUIREMENTS}' = \1 \&\& Host.ErrorRate < 0.001;/' | \
#								  sed 's/'${SCHEDULER_REQUIREMENTS}'[ ]*=\([^=][^;]*\)\]/'${SCHEDULER_REQUIREMENTS}' = \1 \&\& Host.ErrorRate < 0.001 \]/' | \
#								  sed 's/'${TAIL}'[ ]*=\([^=][^;]*\);/'${TAIL}' = true;/' | sed 's/'${TAIL}'[ ]*=\([^=][^;]*\)\]/'${TAIL}' = true \]/' > "${BOINC_CLASSADS}.new"
		mv -f "${BOINC_CLASSADS}.new" "${BOINC_CLASSADS}"
		update_classad ${TASKID}
	}
	# End of monitoring the tail proximity

	# 7) Running outliers' analysis
	[[ "x${OUTLIERS_ENABLED}" != "x0" ]] &&
	{
		declare -a set_ids_percentages

		set_ids_percentages=( $(percentage_done "${set_ids[*]}") )
		for index in $(seq 0 $(( ${#set_ids_percentages[@]} / 2 - 1 )) )
		do
			set_id=${set_ids_percentages[ $(( 2 * $index )) ]}
			percentage=${set_ids_percentages[ $(( 2 * $index + 1 )) ]}
			set_id_counter=$(find_array_index "${set_ids[*]}" ${set_id})
			directory=${working_directories[${set_id_counter}]}
			depth=${depths[${set_id_counter}]}
			priority=${priorities[${set_id_counter}]}

			[[ $(start_outliers_analysis "${directory}" ${percentage}) == 1 ]] &&
			{
				outliers_file="${directory}/${OUTLIERS_FILE}"
				# Fills the outliers file with routes and outlying results
				"${DIRECTORY_NAME}/${GATHER_RESULTS_TOOL_NAME}" "${directory}/${BOINC_SET_OUTPUT_FILE_NAME}" --avgstd ${OUTLIERS_ANALYSIS_DEVIATION} "${outliers_file}"
				# Writing down the percentage of accomplished workunits to be able to run outliers' analysis after some predefined delta of them
				# has been crunched since the last analysis
				[[ ! -s "${outliers_file}" ]] && echo "${percentage}" > "${directory}/${OUTLIERS_SUBMITTED}" && continue
				# The outliers file, whose line format is <ln-likelihood> <route>, might be unsorted, so we sort it by the number of route
				jobs_number=$(wc -l "${outliers_file}" | awk '{ print $1 }')
				set_id=$(update_maximal_set_id 1)
				exit_code=$?
				[[ ${exit_code} != 0 ]] && Warn "Unable to obtain a new offset for the outliers analysis in ${directory} (exit code ${exit_code})" && break

				# Creating a symbolic link in order to avoid duplicate entry for the 'directory' field
				old_directory="${directory}"
				directory="$PWD/${OUTLIERS_EXTENSION}-${set_id}"
				ln -sf "${old_directory}" "${directory}" 

				execute_ssh_frontend << EOF
					. \$HOME/.bash_profile; \
					mysql $MYSQL_TABLE_FLAGS -e "insert into $MONITORING_SET_TABLE (id, status, directory, priority, task_id) \
							     				 values ($set_id, '$MONITORING_STATUS', '$directory', $priority, ${TASKID})";
EOF
				exit_code=$?
				[[ ${exit_code} != 0 ]] && \
					Warn "Unable to insert a new set id ${set_id} from ${directory} to the ${MONITORING_SET_TABLE} table (exit code ${exit_code})" && continue
				numeric_sort_by_key "${outliers_file}" 2
				cat "${outliers_file}" | awk '{ print $2 }' > "${ESTIMATION_PARAMETERS_FILE_NAME}.${set_id}"
				submit_boinc_monitoring_jobs ${set_id} 0 ${jobs_number}
				rsc_fpops_est=$(floating_point_estimate "${old_directory}")
				bundle_flag="--bundle ${BOINC_INITIAL_BUNDLE_SIZE}"
				sequence_file_path="$PWD/${ESTIMATION_PARAMETERS_FILE_NAME}.${set_id}"
	
				execute_ssh_frontend << EOF
				. \$HOME/.bash_profile; template_file=\$(mktemp); \
				cat \$PROJECT_ROOT/templates/${WU_TEMPLATE_FILE_NAME} | \
				sed 's/<rsc_fpops_est>.*<\/rsc_fpops_est>/<rsc_fpops_est>${rsc_fpops_est}<\/rsc_fpops_est>/' > \$template_file; \
				make_work_superlink_transaction.sh ${bundle_flag} --template_file \$template_file --superlink_path "${directory}" \
				--sequence_file "${sequence_file_path}" --depth ${depth} --priority ${priority} --transaction_size ${BOINC_TRANSACTION_SIZE} \
				--task_id ${TASKID} --outliers_analysis; exit_code=\$?; rm -f \$template_file; exit \$exit_code
EOF
				exit_code=$?
				[[ ${exit_code} != 0 ]] && Warn "SSH command failed (exit code ${exit_code}), unable to create BOINC jobs for outliers analysis in ${directory}" && break 
				echo $set_id ${jobs_number} ${depths[$set_id_counter]} ${directory} >> ${BOINC_PARAMETERS_FILE_NAME}
				# Updating the in-memory monitoring set ids
				set_ids=( ${set_ids[@]} ${set_id} )
				working_directories=( ${working_directories[@]} ${directory} )
				depths=( ${depths[@]} ${depth} )
				jobs_numbers=( ${jobs_numbers[@]} ${jobs_number} )
				priorities=( ${priorities[@]} ${priority} )
				set_ids_list=$(echo ${set_ids[@]} | tr ' ' ',')
				# To mark that the analysis in the directory has already started and there is no need to relaunch it anymore
				echo 100 > "${old_directory}/${OUTLIERS_SUBMITTED}"
			}
		done
	}
	# End of running outliers' analysis
done

# In this part of the script the status of all the monitoring sets is "Finished"

# If all of the jobs are finished successfully or if due to another reason we have to remove all the monitoring information,
# we do it transactionally
Log "Cleaning up all the monitoring information"
clean_sets "${set_ids[*]}" ${TASKID}

# Touching the completion file to track down the progress status of the directory
touch $(dirname $parameters_file_path)/$BOINC_TASK_COMPLETE

Log "Ending $SCRIPT_NAME"
# End of main
