#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
SCRIPT_NAME=$(basename $0)

# Includes
. "${PEDTOOL_ROOT}/CONFIG.SH"
#. "${PEDTOOL_ROOT}/BoincNodeConstants.sh"
# End of includes

# Exit codes
USAGE_EXIT=1
INVALID_INPUT=2
INVALID_CLASSAD=3
# End of exit codes

declare -r INVALID_CLASSAD
declare -r INVALID_INPUT
declare -r USAGE_EXIT
# End of script constants

# Script variables
declare task_id
# End of script variables

# Auxiliary functions

# Print the help screen
function usage {
        cat << EOF
--task <task id>       : specifies, which task to reconfigure                    
[--debug]              : turns on debugging information
[--help|--usage|-h]    : displays this message
Usage example:
./${SCRIPT_NAME} --task 184
EOF
}
# End of auxiliary functions

# Parsing the input
while [ $# -ge 1 ]; do
	case "$1" in
		"--task" )                      shift; task_id=$1;; 
		"--debug" )						set -x;;
		"--help" )                      usage; exit $USAGE_EXIT;;
		"-h" )                          usage; exit $USAGE_EXIT;;
		"--usage" )                     usage; exit $USAGE_EXIT;;
	esac
	shift
done
# End of parsing the input

# Validating the input
task_path="${WORK_DIR}/${task_id}"

[ -z "${task_id}" ] && echo "Missing task id" && echo && usage && exit ${INVALID_INPUT}
[[ ! ( "${task_id}" =~ "^[0-9]+$" ) ]] && echo "Task id should be a non-negative number" && echo && usage && exit ${INVALID_INPUT}
[[ ! -d "${task_path}" ]] && echo "Task has got no corresponding directory ${task_path}" && echo && usage && exit ${INVALID_INPUT}
# End of validating the input

# Main

# Validating the classad
classads_file_path="${task_path}/${BOINC_CLASSADS}"
error_file_path=$(mktemp)

[ -s "${classads_file_path}" ] &&
{
	"${PEDTOOL_ROOT}/xml_to_classad" "${classads_file_path}" classad >& "${error_file_path}" > /dev/null

	exit_code=$?
	[[ ${exit_code} -ne 0 ]] &&
	{
		echo "Invalid classad file ${classads_file_path}, please, review its syntax. Here is the error:"
		cat "${error_file_path}"
		rm -f "${error_file_path}"
		exit ${INVALID_CLASSAD}
	}
}
rm -f "${error_file_path}"
# End of validating the classad

lsof_path=$(whereis lsof | cut -f2 -d' ')
grep_pattern="^n.*/${task_id}\$"

index=()
while [[ ${#index[@]} -ne 1 ]]
do
	lsof_output=()
	lsof_output=( $( ${lsof_path} -c /^BoincNode\.sh$/ -a -d cwd -F pn) )
	index=( $( echo ${lsof_output[@]} | tr ' ' '\n' | grep -n ${grep_pattern} | cut -f1 -d':') )
done
let index=index-2

process_number=${lsof_output[${index}]}
process_number=${process_number:1}

kill -s SIGUSR2 ${process_number}
exit 0 
# End of main
