#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
SCRIPT_NAME=$(basename $0)
SLEEP_INTERVAL=3

# Includes
# For WORK_DIR
. "${PEDTOOL_ROOT}/CONFIG.SH"
#. "${PEDTOOL_ROOT}/BoincNodeConstants.sh"
# End of includes

# Exit codes
USAGE_EXIT=1
INVALID_INPUT=2
# End of exit codes

declare -r INVALID_INPUT
declare -r USAGE_EXIT
declare -r SLEEP_INTERVAL
# End of script constants

# Script variables
declare task_id
# End of script variables

# Auxiliary functions

# Print the help screen
function usage {
        cat << EOF
--task <task id>       : specifies, which task to edit the classad for 
[--debug]              : turns on debugging information
[--help|--usage|-h]    : displays this message
Usage example:
./${SCRIPT_NAME} --task 184
EOF
}
# End of auxiliary functions

# Parsing the input
while [ $# -ge 1 ]; do
	case "$1" in
		"--task" )                      shift; task_id=$1;; 
		"--debug" )						set -x;;
		"--help" )                      usage; exit $USAGE_EXIT;;
		"-h" )                          usage; exit $USAGE_EXIT;;
		"--usage" )                     usage; exit $USAGE_EXIT;;
	esac
	shift
done
# End of parsing the input
task_path="${WORK_DIR}/${task_id}"

# Validating the input
[ -z "${task_id}" ] && echo "Missing task id" && echo && usage && exit ${INVALID_INPUT}
[[ ! ( "${task_id}" =~ "^[0-9]+$" ) ]] && echo "Task id should be a non-negative number" && echo && usage && exit ${INVALID_INPUT}
[[ ! -d "${task_path}" ]] && echo "Task has got no corresponding directory ${task_path}" && echo && usage && exit ${INVALID_INPUT}
# End of validating the input

# Main
classads_file_path="${task_path}/${BOINC_CLASSADS}"
classads_temporary_file_path=$(mktemp)
error_file_path=$(mktemp)
[ -e $classads_file_path ] && cp -f "${classads_file_path}" "${classads_temporary_file_path}" 
# Editing and validating the classad
# First trying the VISUAL environment variable, then the EDITOR variable, then moving to the default 'vi' editor
editor=${VISUAL:-${EDITOR:-vi}}
exit_code=1

while [[ ${exit_code} -ne 0 ]]
do
	"${editor}" "${classads_temporary_file_path}" 

	"${PEDTOOL_ROOT}/xml_to_classad" "${classads_temporary_file_path}" classad >& "${error_file_path}" > /dev/null

	exit_code=$?
	[[ ${exit_code} -ne 0 ]] &&
	{
		echo "You have just edited an invalid classad file ${classads_temporary_file_path}, please, review its syntax. Here is the error:"
		cat "${error_file_path}" 
		echo "Reloading after ${SLEEP_INTERVAL} seconds."
		sleep ${SLEEP_INTERVAL}
		continue
	}
	cp -f "${classads_temporary_file_path}" "${classads_file_path}"
done
rm -f "${error_file_path}"
rm -f "${classads_temporary_file_path}"
# End of editing and validating the classad

exit 0 
# End of main
