#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
SCRIPT_NAME=$(basename $0)

# Includes
#. "${PEDTOOL_ROOT}/CONFIG.SH"
. "${PEDTOOL_ROOT}/BoincNodeConstants.sh"
# End of includes

# Exit codes
USAGE_EXIT=1
INVALID_INPUT=2
# End of exit codes

declare -r INVALID_INPUT
declare -r USAGE_EXIT
declare -r SCRIPT_NAME
# End of script constants

# Script variables
sets_list=""
directory="."
# End of script variables

# Auxiliary functions

# Print the help screen
function usage {
        cat << EOF
--sets "<space-separated list of monitoring sets' ids to clean>" : specifies which monitoring sets (and, hence, workunits and results) to remove, 
                                                                   sets' ids are non-negative numbers
[--directory <task working directory>]                           : specifies the working directory of the task (the default is the current directory)
[--debug]                                                        : turns on debugging information
[--help|--usage|-h]                                              : displays this message
Usage example:
./$SCRIPT_NAME --sets "103 104 105"
EOF
}
# End of auxiliary functions

# Parsing the input
while [ $# -ge 1 ]; do
	case "$1" in
		"--sets" )			shift; sets_list=$1;;
		"--directory" )		shift; directory=$1;;
		"--debug" )			set -x;;
		"--help" ) 			usage; exit $USAGE_EXIT;;
		"-h" )				usage; exit $USAGE_EXIT;;
		"--usage" )			usage; exit $USAGE_EXIT;;
	esac
	shift
done
# End of parsing the input

# Validating the input
[ -z "${sets_list}" ] && echo "Invalid or missing list of sets to remove" && echo && usage && exit $INVALID_INPUT 
[[ ! ( "${sets_list}" =~ "^[0-9 ]+$" ) ]] && echo "Sets' ids are non-negative numbers" && echo && usage && exit $INVALID_INPUT
[[ -z "${directory}" || ! ( -d "${directory}" ) ]] && echo "Invalid or missing task working directory" && echo && usage && exit $INVALID_INPUT
# End of validating the input

# Main
cd "${directory}"
task_id=${SCRIPT_NAME}.${RANDOM}
clean_sets "${sets_list}" ${SCRIPT_NAME} 
exit 0
# End of main
