#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh


# we don't support two point yet
[ $TASK_MULTIPOINT -ne 1 ] && exit 0

alldirs=` find  -maxdepth 2 -mindepth 2 -type d  -name 'pedfile*'`

# let's find the number of pedigrees
numOfPeds=`ls datafile0/ |wc -l`

[ $numOfPeds -eq 0 ] && exit 0

# now we'll sort things according to the pedigree
let numOfPeds--
for k in `seq 0 $numOfPeds`; do
	peddirs=`echo $alldirs | xargs -n1 | grep pedfile${k}`
	[ -z "$peddirs" ] && continue
	printed_header=""
	for i in $peddirs; do
	# fetch the point
		[ -e $i/datafile.dat ] || continue
		[ ! -e $i/ComplexityEstimate.txt -a -e $i/ComplexityEstimate0.txt ] && cp -f $i/ComplexityEstimate0.txt $i/ComplexityEstimate.txt
		[ -e $i/ComplexityEstimate.txt ] || continue
		   position=`tail -1 $i/datafile.dat | awk '{ match($0,/((-*[[:digit:]]+[[:punct:]][[:digit:]][[:digit:]][[:digit:]][[:digit:]]))/,k); print k[1]}'`
		[ "x$position" = "x-100000000.0000" ] && position="UNLINKED"
		[ -z "$position" ] && position=0
		ParseComplexityEstimateFile $i/ComplexityEstimate.txt
		complexity=$task_complexity
	
		systemComplexity=`${BIN}/TranslateComplexity.sh $complexity`
		[ -e $i/${DO_NOT_COMPUTE_FILE} ] && {
				[ $complexity -gt $LOW_PARALLELIZATION_THRESHOLD ] && systemComplexity="Skipped-too hard";
		}

		table_format="<td align=middle>"
		[ -z "$printed_header" ] && {
			printed_header=1;
			let current_ped="$k"+1;
			echo "<center><p><h2><u><b>Computation complexity for pedigree number $current_ped</b></u></h2></p></center>"
			cat ${REPORT_COMPLEXITY}.pre;
		}
cat <<EOF
<tr>
	$table_format $position </td> $table_format $complexity </td>$table_format $systemComplexity</td>
</tr>
EOF
	
	done

	[ -z "$printed_header" ] || cat ${REPORT_COMPLEXITY}.post
done
