#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog

# THIS SCRIPT WILL PRODUCE RESULTS ONLY IF IT DETECTS DATA DIRECTORIES 

# In case of resubmit we want to regenerate all DAGs, BUT take into account
# all jobs with the state DONE
# however we assume that Job can migrate only before submission,
# after SecondEstimate, or in the process of ParallelDag. 
# Thus we can overwrite First and Second estimate since even if they were 
# completed, GlobalDag will contain the results.


# if we were invoked for restart -> we will have to regenerate all DAGs here
# otherwize we will for sure exit after this
alldirs=`FindAllDataDirs`
# nothing found -> we are not restarting -> just exit
[ -z "$alldirs" ] && {
	exit 0;
}

# let's keep old rescue file
for i in *.dag.rescue; do
	mv -f $i $i.old
done


cwd=`pwd`
sed_cwd=`pwd | sed 's/\//\\\\\\//g'`


# start SecondEstimate.dag. This all will be overwritten by  PreSecondEstimate
cat > SecondEstimate.dag <<EOF
JOB A SecondE_singleJob.sub
VARS A TASKID="$TASKID" EXECDIR="$BIN"  JOBWORKDIR="$cwd"
EOF


#create SecondEstimate from template
 
logfile=$sed_cwd\\/SecondEstimate.log

cat $SUBFILES/SecondE_singleJob.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > SecondE_singleJob.sub
cat $SUBFILES/SecondE_localJob.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > SecondE_localJob.sub

CondorSubmitDag SecondEstimate.dag || Die " Failed to create SecondEstimate.dag:  $command_output"

#start ParallelDag.dag - all this will be overwritten by PreParallelDag
cat > ParallelDag.dag <<EOF
JOB Parallel Parallel_single.sub
VARS Parallel EXECDIR="${BIN}" TOTAL="0" PERMNUM="0" DEPTH="0" WORKDIR="."  PRIO="0" TASKID="1"
EOF

#create Parallel_single.sub from template. The important point is to specify the right log file
 
logfile=$sed_cwd\\/ParallelDag.log
cat $SUBFILES/Parallel_single.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > Parallel_single.sub

#create ParallelDag.dag.condor.sub

CondorSubmitDag ParallelDag.dag  || Die "Failed to create ParallelDag.dag"

# move back all rescue files
for i in *.dag; do
	[ -e $i.rescue.old ] && mv -f $i.rescue.old $i.rescue
done

exit 0
