#!/bin/bash
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
RedirOutToGlobalLog
# this script goes over all dirs which it is provided and decides which one to migrate to another pool


[ -z "$1" ] && exit 0

alldirs=(`echo $* |xargs -n1| sort -n`)
totaldirs=${#alldirs[*]}

peer_migrate="no"
bf_migrate=""

while [ 0 -eq 0 ]; do
	peer_perf_file=$LOG_DIR/${PEER_ACCOUNT}_${PEER_HOST_PERF_FILE_SUFFIX}
# can we migrate to peer at all?
	[ -z "$PEER_ACCOUNT" -o ! -s "$peer_perf_file" ] && break
# do we WANT to migrate to peer?
	peer_perf=`cat $peer_perf_file`
	[ -z "$peer_perf" ] && break;
# if our peer performance is 0 -> we don't migrate
	[ $peer_perf -le 0 ] && break;

# OK, we can migrate if we want. 
	peer_migrate=""
	break;
done

[ -z "$peer_migrate" ] && Log "Decided to consider migration -> remote performance reported : $peer_perf" 

# get our own performance metrics
[ -s $LOG_DIR/$SELF_PERF_FILE ] && {
	my_perf=`cat $LOG_DIR/$SELF_PERF_FILE`
	if [ ! -z "$my_perf" -a $my_perf -gt 0 ]; then
		# we don't migrate if we have resources
		Log "No migration required -> our own performance is $my_perf"
		exit	
	fi
}

[ -s $LOG_DIR/$SELF_RESOURCES_FILE ] && self_resources=`cat $LOG_DIR/$SELF_RESOURCES_FILE`
[ -z "$self_resources" ] && self_resources=0;


queue_counter=0;
for i in ${alldirs[*]}; do
	i="$WORK_DIR/$i"
	second_migration_attempt=""
	[ -e $i/${MIGRATION_MARKER_FILE}_1 ] && {
		second_migration_attempt=true;
		rm -f  $i/${MIGRATION_MARKER_FILE}_1;
	} 
	# check if the timer requires to move to backfill
	[ -s $i/$TOTAL_COMPUTATION_TIME_FILE ] || continue;
	
	total_time=`cat $i/$TOTAL_COMPUTATION_TIME_FILE`
	[ ! -z "$total_time" ] || continue;

	# move to backfill 
	if [ $total_time -gt $BACKFILL_TIMEOUT ]; then 
		if [ $totaldirs -gt 1 -a -z "$bf_migrate" -a ! -z "$BACKFILL_ACCOUNT" ]; then
	# only if there are other tasks waiting and we didn't move any other task to bf in this session
			bf_migrate=true
		else if [ $FORCE_BACKFILL -eq 1 ]; then
		# if this site prescribes to FORCE backfill migration -> do it whatever load we have now
			bf_migrate=true
		     fi
		fi
		
		[ -z "$bf_migrate" ] || {
			Log "Detected backfill run for the task $i"
			echo $BACKFILL_ACCOUNT $BACKFILL_ASKPASS $BACKFILL_ACCEPT_SCRIPT_PATH > $i/$MIGRATION_MARKER_FILE
			bf_migrate=true;
			break;
		}
	fi
	# if already one task migrated - skip to next round
	[ ! -z "$peer_migrate" ] && continue;

	# let's check if we may want to migrate this task?
	# if task has no jobs pending, or we don't know about them - leave locally
	[ -e $i/$TASK_STATE_FILE ] || continue;

	# how many jobs are running for this task?
	[ -s $i/$CUR_NUM_RUNNING_JOBS ] && num_of_running=`cat $i/$CUR_NUM_RUNNING_JOBS`
	[ -z "$num_of_running" ] && num_of_running=0

	# how many jobs total for this task?
	total=0
	if [ -s $i/$TOTAL_JOBS_FILE -a -s $i/$TOTAL_DONE_JOBS_FILE ]; then
		# if we already have statistics about number of jobs - use it
		total_jobs=`cat $i/$TOTAL_JOBS_FILE`
		total_done_jobs=`cat $i/$TOTAL_DONE_JOBS_FILE`
		
		[ -z "$total_jobs" -o -z "$total_done_jobs" ] || {
			let total="$total_jobs"-"$total_done_jobs"

			# we don't want to move it around if less than 10% left
			let tmp="$total*10"
			[ $tmp -lt $total_jobs ] && {
				Log "less than 10% left -> we don't move";
 				continue;
			}
		}
			
	else
		# if we don't have info about global num of jobs - try to guess
		SecondEstimate=`grep ORDER_ESTIMATE $i/$TASK_STATE_FILE | tail -1 | cut -f2 -d"="`
		ParallelOrdering=`grep PARALLEL_ORDERING $i/$TASK_STATE_FILE | tail -1 | cut -f2 -d"="`
		
		[ ! -z "$SecondEstimate" ] && total=$SecondEstimate;
		[ ! -z "$ParallelOrdering" ] && total=$ParallelOrdering; 
	fi

	Log "Found $total jobs requests"
	# if only 20 jobs - we don't migrate
	[ $total -lt 20 ] && {
		Log "Total jobs less than 20 -> no need to migrate";
		continue;
	}

	# how many jobs idle?
	let idle="$total"-"$num_of_running"
	# if there are trice as many idle than running -> we migrate
	let tmp="$num_of_running*3"
	[ $idle -lt $tmp ] && { 
		Log "idle: $idle running: $num_of_running => only $idle idle jobs left -> we don't migrate"; continue;
	} 
	# if we have too many running -> let's skip migration
	[ $num_of_running -ge $MAX_RUNNING_TASKS_FOR_LOAD_BALANCE ] && {
		Log "Too many running jobs found : $MAX_RUNNING_TASKS_FOR_LOAD_BALANCE - we don't wont to migrate"
		continue;
	}
	
	let queue_counter+="$idle"
	Log "total jobs in queue = $queue_counter"	
	if [ $self_resources -eq 0 ]; then
		Log "No local resources available - migration is worthy"
		peer_migrate="YES"	
	else 
		# let's compare where it will run faster
		# if queued tasks/local resources > current number of jobs/remote resources => we want to migrate
		Log "Migration considered: $queue_counter $self_resources $total $peer_perf"
		migrate=`echo "$queue_counter $self_resources $total $peer_perf" | awk '{ if( $1/$2 > $3/$4 ) print 1 }' `
		[ -z "$migrate" ] && continue
		Log "We decide to migrate"
		peer_migrate="YES"
	fi
	
	[ -z "$peer_migrate" ] && continue

	if [ -z "$second_migration_attempt" ]; then
		Log "This is our first migration attempt"
		touch $i/${MIGRATION_MARKER_FILE}_1
	else
		Log "Task $i should be migrated, marking it for migration"
		echo $PEER_ACCOUNT $PEER_ASKPASS $PEER_ACCEPT_SCRIPT_PATH > $i/$MIGRATION_MARKER_FILE
	fi
	# if we are here -> migration is going to start. 
	# we allow only SINGLE migration EVENT per iteration
	exit 0
	
done

