#!/bin/bash
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
# this script  attempts to estimate the number of available resources, combining info
# from the queue with the amount of potential resources from condor_status
find_num_resources(){
	# first create the statistics about the amount of resources we have
	# logic - if we have idle jobs, the number of running jobs is the monitor. This, smoothed with the history does the trick.
	current_resources=0
	if [ $idle -gt 0 ]; then 
		current_resources=$busy
	fi
	#let's try to find with condor_status
	#detect CONDOR_HOST
#	collector=`$CONDOR_CONFIG_VAL_EXEC $COLLECTOR_VAR`
	
	flock_to=`$CONDOR_CONFIG_VAL_EXEC $FLOCK_TO_VAR`

#	[ -z "$collector" ] && {
#		Log "Failed to detect collector of this pool";
#		return;
#	}
	
	[ -z "$flock_to" ] || {
		flock_to=`echo $flock_to| awk '{gsub(/,/," ");print $0}'`
	}
	all_collectors="$flock_to";
	
	total_avail=0	
	for i in 1 $all_collectors; do
		if [ "$i" = "1" ]; then
			$CONDOR_STATUS_EXEC -total -avail -const Memory'>='$MAX_AVAILABLE_MEMORY > $LOG_DIR/LAST_CONDOR_STATUS 2>/dev/null
		else
			$CONDOR_STATUS_EXEC -total -avail -const Memory'>='$MAX_AVAILABLE_MEMORY -pool $i > $LOG_DIR/LAST_CONDOR_STATUS 2>/dev/null
		fi
		[ $? -ne 0 ] && continue;
		for k in ${SUPPORTED_PLATFORMS[*]}; do
#set -x
			current_avail=`cat $LOG_DIR/LAST_CONDOR_STATUS | grep -i $k | awk '{print $5}' `
			[ -z "$current_avail" ] && continue
			let total_avail+="$current_avail"
		done
	done

	echo "Found $total_avail hosts which can run Superlink, while estimation accroding to busy jobs is: $current_resources. idle jobs is $idle"
	# we count condor_status only if $idle is 0
	if [ $idle -eq 0 ]; then
		let current_resources+="$total_avail"
	fi
	if [ $busy -eq 0 -a ! -z "$RESOURCE_ESTIMATION_BOOST" ]; then
		[ $current_resources -lt $RESOURCE_ESTIMATION_BOOST ] && current_resources=$RESOURCE_ESTIMATION_BOOST
	fi
}

if [ -s $LOG_DIR/Utilization.log ]; then
	performance=`tail -1 $LOG_DIR/Utilization.log`
	idle=`echo $performance| awk '{ match($0,/IDLE=([[:digit:]]+)/,t); print t[1]; }'`
	[ -z "$idle" ] && idle=0
	busy=`echo $performance| awk '{ match($0,/BUSY=([[:digit:]]+)/,t); print t[1]; }'`
	tasks=`echo $performance| awk '{ match($0,/TASKS=([[:digit:]]+)/,t); print t[1]; }'`
	[ -z "$idle" -o -z "$busy" -o -z "$tasks" ] && {
		Die "Performance data contains empty variables -  idle=$idle, busy=$busy, tasks=$tasks - too bad";
	}
else
	Die "Performance data unavailable - too bad"
fi

# first lest get our number of resources estimate:
find_num_resources


echo "Before smoothing - Result - $current_resources available for execution"

# because of high variability of this parameter it should be smoothed with history
[ -s $LOG_DIR/$SELF_RESOURCES_FILE ] && {
	oldres=`cat $LOG_DIR/$SELF_RESOURCES_FILE`
	# we use old results with the ration 0.6 
	current_resources=`echo $oldres $current_resources | awk '{ print int($1*0.7+$2*0.3+1)}'`;
	echo "After smoothing - result $current_resources"
}
echo $current_resources > $LOG_DIR/$SELF_RESOURCES_FILE

# first, lets create performance data for all others: SELF_PERFORMANCE_FILE
# if there idle jobs or if both idle and busy is 0, but tasks is not 0 => we don't want anyone here
echo "we decide about SELF_PERFORMANCE according to the data : idle=$idle, busy=$busy, tasks=$tasks"
perf=""

[ $idle -gt 0 ] && perf=0
[ $idle -eq 0 -a $busy -eq 0 -a $tasks -ne 0 ] && perf=0;	

# we didn't decide 
if [ -z "$perf" ]; then
	perf=$current_resources	
fi

echo "Our SELF_PERFORMANCE is $perf"
echo $perf > $LOG_DIR/$SELF_PERF_FILE

