#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog
#set -x
UpdateTaskState ${FIRST_ESTIMATE_STATE}
# this script performs fast and very approximate estimate of the task complexity
Log "Starting FirstEstimate stage"

# we currently restrict only 3200 points, sorry
#############################################
num_of_df=`cat ${NUM_OF_DATAFILES_FILE}`
num_of_pf=`cat ${NUM_OF_PEDFILES_FILE}`
let total=$num_of_df

isFeasibleSL(){
 [ -e datafile.dat ] || return 0
 local numMarkers=`head -1  datafile.dat | cut -f1 -d" "`
 [ -z "$numMarkers" ] && return 0
 local programCode=`head -1 datafile.dat | cut -f4 -d" "`
 [ -z "$programCode" ] && return 0
 
 [ $programCode -eq 6 -a $numMarkers -gt $MAX_FEASIBLE_MARKERS_SL ] && {
    Log "Infeasible"
	touch INFEASIBLE
    return -1;
 }
 return 0;
}


[ $total -gt $MAX_NUM_MARKERS ] && {
	Log "Too high - the task is too hard -> too many points: $total"
	echo ${TOO_MANY_POINTS} > ${FIRST_ESTIMATE_RESULT_FILE};
	
	exit ${TOO_MANY_POINTS};
}
#############################################
alldirs=`FindAllDataDirs`

[ -z "$alldirs" ] && Die "Not found any DATA dir after initial parallelization - incorrect"

cwd=`pwd`
lastload=0
# how many points do we have:
fix=`${BIN}/log10 $total`
let threshold="${LOCAL_RUN_APPROX_COMPLEXITY_THRESHOLD}"-"$fix"
[ -z "$SL_MAX_AVAILABLE_MEMORY_BOUND" ] && let SL_MAX_AVAILABLE_MEMORY_BOUND=$MAX_AVAILABLE_MEMORY/2
for i in $alldirs; do
	cd $i || DieNoLogin "Failed to chdir to $i"
	[ -x ${BIN}/Superlink/Superlink_Vanilla ] || DieNoLogin "${BIN}/Superlink/Superlink_Vanilla  not found"
	
   if isFeasibleSL;  then 

        ${BIN}/Superlink/Superlink_Vanilla datafile.dat pedfile.dat -s -maxmem $SL_MAX_AVAILABLE_MEMORY_BOUND 2> ${SUPERLINK_INPUT_ERROR_FILE}.tmp || {
			[ -s ${SUPERLINK_INPUT_ERROR_FILE}.tmp ] && {
				cp ${SUPERLINK_INPUT_ERROR_FILE}.tmp $cwd/${SUPERLINK_INPUT_ERROR_FILE} ;
				cp ${SUPERLINK_INPUT_ERROR_FILE}.tmp  cp ${SUPERLINK_INPUT_ERROR_FILE}
			}
			cd $cwd
			echo "$SUPERLINK_INPUT_ERROR" > ${FIRST_ESTIMATE_RESULT_FILE}
			Log "Failed to run complexity estimation in dir $i - `cat $SUPERLINK_INPUT_ERROR_FILE`";
			exit $SUPERLINK_INPUT_ERROR;
 		}
    fi   
	# let's try to check if this constellation is valid at all
    [ -e INFEASIBLE ] && { echo  COMPLEXITY 100 >> ComplexityEstimate0.txt; echo TABLESUM 14666900430626725275906296553110849454080 >> ComplexityEstimate0.txt; }
	[ -e ComplexityEstimate0.txt ] || Die "Expected to find ComplexityEstimate0.txt"
	# if we find in this file only if the following is true
	ParseComplexityEstimateFile ComplexityEstimate0.txt
	tbl=$task_ts;
	cmplx=$task_complexity;
	echo $cmplx > $FRONTEND_COMPLEXITY_FILE
	# maybe we can run this right away at this stage? 
	# we don't want to run uEvalCPULoad more than once in 15 sec
	let now++
	[ $lastload -eq 0 -o $lastload -lt $now ] && {
		EvalCPULoad;
		let lastload="$now+30";
	}
	[ $cmplx -le ${threshold} -a $CPU_local_load -le $MAX_CPU_LOAD_THRESHOLD ] && {
		# we can run right away!
		# Log "Found really low complexity -> we can run right away"
		${BIN}/Superlink/Superlink_Vanilla datafile.dat pedfile.dat -e order0.txt 0 0 > output.0 || Die "Failed to run Superlink_Vanilla in degenerate case"
		# gather the results
		echo 1 > $TOTAL_JOBS_FILE
		${BIN}/${COLLECT_SINGLE_POINT} || Die "Failed to run ${BIN}/${COLLECT_SINGLE_POINT}"
		# set up num of jobs for TaskMonitor.sh
		MarkAlreadyDone .
		cp -f ComplexityEstimate0.txt ComplexityEstimate.txt || Die "Failed to copy complexity estimate file"
		cd $cwd
		continue;
	}
 
	cd $cwd 
done

Log "Checking the first estimation"

parallel_run=0
condor_single_run=0
local_single_run=0

complexity_array=""
table_sum=0

too_heavy_array=""
too_heavy_for_report=""

alldirs=`FindAllDataDirs`
[ -z "$alldirs" ] && {
	# we've got ALL jobs done locally
	Log "All jobs completed On the first estimate! - locally - bailing out"
	UpdateTaskState "${FIRST_ESTIMATE_STATE} TOO_SIMPLE"
	echo ${JOB_TOO_SIMPLE} > ${FIRST_ESTIMATE_DECISION_FILE}
	exit ${JOB_TOO_SIMPLE}
}


for i in $alldirs; do
	if [ -e $i/ComplexityEstimate0.txt ]; then
		ParseComplexityEstimateFile $i/ComplexityEstimate0.txt
		TABLESUM=$task_ts;
		COMPLEXITY=$task_complexity;
		complexity_array="${complexity_array} $COMPLEXITY"
		if [ ! -z "$COMPLEXITY" ]; then
			table_sum=`echo "$table_sum+$TABLESUM" | bc -l`
			if IsNumber $COMPLEXITY ; then
					[ $COMPLEXITY -ge $APPROX_ESTIMATION_COMPLEXITY_THRESHOLD ] && {
						Log "Found run with too high complexity - $i";
						too_heavy_array="$too_heavy_array $i";
						too_heavy_for_report="$too_heavy_for_report $i $COMPLEXITY";
						# mark to ignore this directory
						MarkDoNotCompute $i
						continue;
					}
					
					[ $COMPLEXITY -ge $LOW_PARALLELIZATION_THRESHOLD ] && {
						Log "The run will be parallelized - found complexity above $LOW_PARALLELIZATION_THRESHOLD";
						[ ${parallel_run} -lt $COMPLEXITY ] && 	parallel_run=${COMPLEXITY} 
						continue;
					}
					
					[ $COMPLEXITY -gt $LOCAL_RUN_APPROX_COMPLEXITY_THRESHOLD ] && {
						Log "Detected low complexity run - single condor job - $i ";
						[ ${condor_single_run} -lt $COMPLEXITY ] && condor_single_run=${COMPLEXITY} 
						continue;
					}
					
					[ $COMPLEXITY -le $LOCAL_RUN_APPROX_COMPLEXITY_THRESHOLD ] && {
						Log "Detected very low complexity run -  for local job - $i ";
						[ ${local_single_run} -lt $COMPLEXITY ] && 	local_single_run=${COMPLEXITY} 
						continue;
					}
			else 
				Log "Grabage in  ComplexityEstimate.txt - $COMPLEXITY"
				# keep it safe -> go remote
				[ $parallel_run -lt 1 ] && parallel_run=1
			fi
		else 
				Log "No complexity found in ComplexityEstimate.txt"
				# Keep it safe -> go remote
				[ $parallel_run -lt 1 ] && parallel_run=1
		fi
	else
		Log "No ComplexityEstimate.txt file"
		[ $local_single_run -lt 1 ] && local_single_run=1
	fi 
done
UpdateTaskState "${FIRST_ESTIMATE_STATE} COMPLEXITY_ARRAY=\"${complexity_array}\""
total_complexity=`${BIN}/log10 $table_sum`
UpdateTaskState "${FIRST_ESTIMATE_STATE} TOTAL_COMPLEXITY=${total_complexity}"
Log "Total complexity is $total_complexity"

echo $total_complexity > ${TOTAL_COMPLEXITY_FILE}



output=`basename ${TASK_OUTPUT_FILE}`
RunCommand ${BIN}/$GATHER_RESULT_SCRIPT "ordering" || Die "Failed to run $FRONTEND_POST_SCRIPT"
[ -s $output ] && UpdateFileForUser $output

if [ ! -z "$too_heavy_array" ]; then
	# in case of BEST effort run/single point run we may even survive this stage
	HandleTooHeavyJobs ${FIRST_ESTIMATE_RESULT_FILE} ${FIRST_ESTIMATE_STATE} $too_heavy_for_report
	#if we are still alive -> we probably should continue in parallel
	parallel_run=1
fi
# we are here only if there are jobs which are NOT too heavy

# now let's try to decide: if we have parallel_run marked -> this job is parallel
[ $parallel_run -ne 0 ] && { 
	Log "Decided to run Full parallel run"
	echo ${FULL_PARALLEL_RUN} > ${FIRST_ESTIMATE_DECISION_FILE}; 
	exit 0; 
}

[ $condor_single_run -ne 0 ] && { 
	[ $total_complexity -ge ${LOW_PARALLELIZATION_THRESHOLD} ] &&
		echo ${FULL_PARALLEL_RUN} > ${FIRST_ESTIMATE_DECISION_FILE} || 
		echo ${CONDOR_SINGLE_RUN} > ${FIRST_ESTIMATE_DECISION_FILE}
	exit 0;
}

[ $local_single_run -ne 0 ] && { 
	[ "$total_complexity" = "-inf " ] && total_complexity=0
	[ $total_complexity -gt ${LOCAL_RUN_APPROX_COMPLEXITY_THRESHOLD} ] &&
		echo ${CONDOR_SINGLE_RUN} > ${FIRST_ESTIMATE_DECISION_FILE} ||
		echo ${LOCAL_SINGLE_RUN} > ${FIRST_ESTIMATE_DECISION_FILE}
 	exit 0;
}

Die "Failed to estimate correctly job complexity"


