#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
DIRECTORY_NAME=$(dirname $0)
#NON_BOINC_OUTPUTS_FILE_NAME=non_boinc_outputs.txt
ALL_RESULTS_FILE_NAME=all_results.txt
COLLECT_LIKELIHOODS_TOOL_NAME=collect_likelihoods
COMPARE_OUTLIERS_TOOL_NAME=compare_outliers
#GATHER_RESULTS_TOOL_NAME=gather_single_point_results
# The value is equal to DBL_MAX like in the tool for collecting likelihoods 
INFINITY=10000000000000000000000000000000000000
#INFINITY_STRING="Infinity"

# Exit codes
# The code ought to be alike the code for infinity in the tool for collecting likelihoods
INFINITY_RESULT=4
# End of exit codes

declare -r INFINITY_RESULT
#declare -r INFINITY_STRING
declare -r INFINITY
#declare -r GATHER_RESULTS_TOOL_NAME
declare -r COLLECT_LIKELIHOODS_TOOL_NAME
#declare -r NON_BOINC_OUTPUTS_FILE_NAME 
declare -r DIRECTORY_NAME

# if called without params assume local dir => return two dirs up and set the cwd as a parameter
required_dir=`pwd`
[ -z "$1" ] && cd ../../

. "$DIRECTORY_NAME/JobUtils.sh"
. "$DIRECTORY_NAME/BoincNodeConstants.sh"
# End of script constants

# Auxiliary functions

function write_result_file()
{
	local result_file_path=$1
	local result=$2

	echo $2 > $result_file_path
}
# End of auxiliary functions

# Script parameters processing 
#parameters_file_path=$1
task_directory=${1:-$required_dir}

[ ! -d "$task_directory" ] && Log "Non-existent working directory for the post-BOINC node"

cd "$task_directory"
# End of script parameters processing

declare -a working_directories;


#working_directories=( $(FindAllDataDirs) )

# Upon empty result, we resort to the last attempt and seek in the current directory
[[ "x${working_directories[@]}" == "x" ]] && working_directories=( . )
incomplete=0;

lockfile_name=gather_results_lock;

for working_directory in ${working_directories[@]}
do
#	Log "Processing: ${working_directory}"
    count=0
	to_skip=0
	while ! ${BIN}/lockfile $working_directory/$lockfile_name; do
			sleep 5
			let count+=5
			[ $count -gt 720 ] && { 
					incomplete=1;
					to_skip=1;
					break;
			}
	done
	[ $to_skip -eq 1 ] && continue;

	result_file_path="${working_directory}/${BOINC_RESULT_FILE_NAME}"
	# If the result has been calculated, skip the directory
	
	non_boinc_outputs_file_path="${working_directory}/${NON_BOINC_OUTPUTS_FILE_NAME}"
	all_results_file_path="${working_directory}/${ALL_RESULTS_FILE_NAME}"
	outliers_file_path="${working_directory}/${BOINC_SET_OUTPUT_FILE_NAME}.${OUTLIERS_EXTENSION}"

#	[ -f "$all_results_file_path" ] && {
#		res=`cat ${all_results_file_path}`
#		[ -z "$res" ] && rm -f  ${result_file_path};
#	}
			
	[ -f "${result_file_path}" -a ! -e ${working_directory}/${OUTLIERS_SUBMITTED} ] && { 
		res=`cat ${result_file_path}`
		[ -z "$res" ] || {
#			Log "Result file found, skipping the directory"
			rm -f $working_directory/$lockfile_name
			continue;	
		}
	}
	# Finding all non-BOINC outputs

	find $working_directory -name 'output.condor_node.*' -size +0 -o -name 'output.[0-9]*' -size +0 | \
		while read line; do [[ $(grep TOTAL ${line}) != "" ||  $(grep '@@@\#\#\#' ${line} ) != "" ]] && echo ${line}; done > "${non_boinc_outputs_file_path}"

	# We extract the last field, delimited with '.', using awk since there are two possible types of outputs: output.condor_node.[0-9]* and output.[0-9]*
	non_boinc_ids=( $(cat "${non_boinc_outputs_file_path}" | while read line; do  basename $line; done | awk 'BEGIN{ FS="."} {print $NF}' ) )
	non_boinc_outputs_number=$(wc -l "${non_boinc_outputs_file_path}" | cut -f1 -d' ')
	# Invocation of tool that extracts the unextracted ln-likelihoods
	rm -f "${all_results_file_path}"
    touch "${all_results_file_path}"

	[[ $non_boinc_outputs_number != 0 ]] && {
		id_index=0

		"$DIRECTORY_NAME/$COLLECT_LIKELIHOODS_TOOL_NAME" "$non_boinc_outputs_file_path" | \
			while read line; do echo $line ${non_boinc_ids[$id_index]}; let id_index++; done > "$all_results_file_path"
	}
	# End of extracting tool invocation
	
	# Finding all BOINC outputs: sorting out possible duplicates
	[ -f "$working_directory/${BOINC_SET_OUTPUT_FILE_NAME}" ] && cat "$working_directory/${BOINC_SET_OUTPUT_FILE_NAME}" >> "$all_results_file_path"
	#set -x
	
 	[ -s $all_results_file_path ] && {
		cat "$all_results_file_path" | awk '{print $2" "$1}' | sort -n | awk 'BEGIN{prev=1000}{if (prev!=$1){ prev=$1; print $2;}}'  > "${all_results_file_path}.old"
		mv "${all_results_file_path}.old" "$all_results_file_path"
	}
	all_likelihoods_number=$(cat "$all_results_file_path" | wc -l)
	#set +x
	[[ -f "${working_directory}/${GATHER_TOTAL_JOBS_FILE}"       ]] && total_jobs=$(cat "${working_directory}/${GATHER_TOTAL_JOBS_FILE}")
	[[ ! ( -f "${working_directory}/${GATHER_TOTAL_JOBS_FILE}" ) ]] && total_jobs=$(cat "${working_directory}/${TOTAL_JOBS_FILE}")

	[[ $all_likelihoods_number != $total_jobs ]] && {
#		Log "Number of BOINC and non-BOINC jobs altogether ($all_likelihoods_number) in $working_directory is not equal to the number, declared in $TOTAL_JOBS_FILE ($total_jobs)"
		incomplete=1
		rm -f $working_directory/$lockfile_name
		continue
	}
	# Invocation of tool that aggregates results of both BOINC and local Condor calculations
	max_ln_likelihood=-$INFINITY
	
	[[ $all_likelihoods_number == 0 ]] && {
		Log "Found no outputs in $working_directory"
		write_result_file "$result_file_path" "$INFINITY_STRING"
		rm -f $working_directory/$lockfile_name
		continue
	}
	final_result=$( "${DIRECTORY_NAME}/${GATHER_RESULTS_TOOL_NAME}" "${all_results_file_path}" )
	gather_results_exit_code=$?

	[[ $gather_results_exit_code == ${INFINITY_RESULT} ]] &&
	{
		write_result_file "$result_file_path" "$INFINITY_STRING"
		rm -f $working_directory/$lockfile_name
		continue
	}
	[[ $gather_results_exit_code != 0 ]] &&
	{
		Log "${GATHER_RESULTS_TOOL_NAME} exited with invalid code (${gather_results_exit_code})"
		MarkError "$working_directory"; 
		rm -f $working_directory/$lockfile_name
		#incomplete=1
		continue
	}
	# End of aggregating tool invocation
	
	# Checking for improbable results
	improbable_results=$( grep -E "^1" "${all_results_file_path}" | wc -l )
	[[ "${improbable_results}" -gt 0 ]] && { 
		Warn "There are ${improbable_results} improbable results in ${all_results_file_path}"; 
		MarkError "$working_directory"; 
		rm -f $working_directory/$lockfile_name
		continue; 
	}
	# End of checking for improbable results

	# Checking for discrepancies in the outliers' recomputation
	[[ -s "${outliers_file_path}" ]] &&
	{
		discrepancies=( $("${DIRECTORY_NAME}/${COMPARE_OUTLIERS_TOOL_NAME}" "${working_directory}/${BOINC_SET_OUTPUT_FILE_NAME}" "${outliers_file_path}") )

		[[ ${#discrepancies[@]} -ne 0 ]] && { 
			Warn "${COMPARE_OUTLIERS_TOOL_NAME} has found discrepancies ${discrepancies[*]} in ${outliers_file_path} after the analysis of outliers";
			MarkError "$working_directory";
			rm -f $working_directory/$lockfile_name
    	    continue;
		}
		
	}
	# End of checking for discrepancies in the outliers' recomputation
	#echo $final_result
	write_result_file "${result_file_path}" ${final_result}
	rm -f $working_directory/$lockfile_name
	
done

[ $incomplete -eq 0 ] && exit 0
exit 1
