#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog
# this simple script takes the file with directories and complexities 
# and grouppes the runs according to the threshold.

# this function generates submission file and environment for "groupped" job.
# such job is actually a group of sequential invocations of a program
# grouped for better efficiency

###############################################################

GenerateGrouppedJob(){

	[ -z "$3" ] && Die "Missing parameters - jobID , type(order OR linkage), DAG filename, list of dirs"
	jobid=$1
	shift
	type=$1
	shift
	dagfile=$1
	shift
	alldirs="$*"

	[ "$type" = "order" -o "$type" = "linkage" ] || Die "Unrecognized type"
	
	mydir="${GROUPPED_JOB_DIR_PREFIX}.$jobid"
	[ -e $mydir ] && rm -rf $mydir
	
	mkdir -p $mydir || Die "Failed to create $mydir for groupped job"
	
	work_idx=""
	files="${BIN}/Superlink/Superlink_Vanilla.\$\$(Arch).\$\$(OpSys)"
	local i
	for i in $alldirs; do
		[ -e $i/pedfile.dat -a -e $i/datafile.dat ] || {
			Error "Specified wrong data/pedfile $i";
			continue;
		}
		GetPedigreeNumberFromPath $i
		GetDatapointNumberFromPath $i
		work_idx="${datapointNum}.${pedNum} $work_idx"
		
		cp  $i/pedfile.dat $mydir/pedfile.${datapointNum}.${pedNum} || Die "Failed to create $mydir/pedfile.${datapointNum}.${pedNum}"
		cp $i/datafile.dat $mydir/datafile.${datapointNum}.${pedNum} || Die "Failed to create $mydir/datafile.${datapointNum}.${pedNum}"
		files="$files, datafile.${datapointNum}.${pedNum}, pedfile.${datapointNum}.${pedNum}"
		[ "$type" = "linkage" ] && {
			cp $i/order.txt $mydir/order.${datapointNum}.${pedNum} || Die "Failed to create $mydir/order.${datapointNum}.${pedNum}";
			files="$files, order.${datapointNum}.${pedNum}";
		}
	done
	
	# now we crete a submission file and a dagnode
	# we need to parse subfile in order to transfer all files safely - problem with DAGman parameter parsing
	
	cwd=`pwd`
	sed_cwd=`pwd | sed 's/\//\\\\\\//g'`
	logfile=$sed_cwd\\/GrouppedJobRun.log 
	
	subfile=GrouppedJobRun${jobid}.sub
	
	cat ${SUBFILES}/GrouppedJobRun.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > ${subfile}
	
	# output all indexes into the appropriate file
	echo $work_idx > $mydir/indices
	

[ -z "$SL_MAX_AVAILABLE_MEMORY_BOUND" ] && let SL_MAX_AVAILABLE_MEMORY_BOUND=$MAX_AVAILABLE_MEMORY/2
cat >> ${subfile} <<EOF
transfer_input_files = $files
arguments = $type Superlink_Vanilla.\$\$(Arch).\$\$(OpSys) $SL_MAX_AVAILABLE_MEMORY_BOUND $work_idx
Rank = KFlops
Priority = $TASK_PRIO_IN_CONDOR
InitialDir = ${cwd}/$mydir
Queue
EOF
	
	#now we create the appropriate DAGman node
cat >> $dagfile  <<EOF
JOB $mydir ${subfile}
VARS $mydir EXECDIR="${BIN}" WORKDIR="${cwd}"  TASKID="$TASKID"
SCRIPT POST $mydir ${BIN}/PostGrouppedJobRun.sh \$RETURN $type $mydir indices
RETRY $mydir 10
EOF
	Log "Done creation of groupped job for $alldirs"
	
	return 0
}

[ -z "$5"  ] && Die "Missing arguments: file_with_dirs type[order|linkage] threshold DAGfilename StageName"

mapfile=$1
[ -e $mapfile ] || Die "Can't find $mapfile"

type=$2
[ "$type" = "order" -o "$type" = "linkage" ] || Die "Unrecognized type"

threshold=$3

dagfile=$4

stage=$5

# we now sort the file according to the complexity
sortedArr=(`cat $mapfile | sort -k1 -n`)

[ ${#sortedArr[*]} -eq 0 ] && Die "Can't find anything in the sorted array"

acc=0
mydirs=""
counter=0
for (( idx=0 ; idx<${#sortedArr[*]} ; idx+=2 )); do
	tmp=$acc
	let tmp+="${sortedArr[idx]}"
	log=`${BIN}/log10 $tmp`
	[ -z "$log" ] && Die "Failure in Log "
 
	if [ $log -lt ${threshold} ]; then
		acc=$tmp
		
		mydirs="${sortedArr[idx+1]} $mydirs"
	else
		[ -z "$mydirs" ] || {
		GenerateGrouppedJob $counter $type $dagfile $mydirs  || Die "Failed to execute GenerateGrouppedJob.sh";
		let counter++;
		}
		mydirs="${sortedArr[idx+1]}"
		acc="${sortedArr[idx]}"

	fi
done
 
[ -z "$mydirs" ] ||  { 
	GenerateGrouppedJob $counter $type $dagfile $mydirs
	let counter++;
}

UpdateTaskState "$stage GROUP=$counter"
Log "Created $counter groups"

exit 0
