#!/bin/bash

# this script runs a group of  Superlink runs, according to the type of the run -> 
# we can't use Utils and CONFIG.SH, since it runs remotely
# 
die(){
	echo "$1"
	exit -1;
}


[ -z "$4" ] && die "Missing parameters: type[order|linkage]  ExecutableName MaxMemory indices[e.g 1.1 2.2.....]"
type=$1
[ "$type" = "order" -o "$type" = "linkage" ] || die "Unrecognized type"
shift
command=$1
[ -e $command ] && mv -f $command Superlink_Vanilla
shift
maxmem=$1
shift
indices="$*"
[ -z "$indices" ] && die "No indices specified..., too bad"

# we are running remotely - so we chmod Superlink_Vanilla

[ -e Superlink_Vanilla ] || die " Superlink_Vanilla not found!!! "

chmod +x Superlink_Vanilla || die "Can't chmod for executables"

SL="./Superlink_Vanilla";
case "$type" in
	order)
	# we are supposed to run Superlink, and copy  the iportant files to the names with the suffix
	for i in $indices; do
		[ -e datafile.$i -a -e pedfile.$i ] || die " failed to find files datafile.$i  pedfile.$i"

		${SL} datafile.$i  pedfile.$i -idx 0 600 -maxmem $maxmem > output.$i 2>&1 || die "failed to run Superlink on index $i"
		# we now move the created files to the naming convention with idx
		[ -e ComplexityEstimate0.txt -a -e order0.txt -a -e NumOfJobs0.txt ] || die "failed to find the appriate Superlink files : ComplexityEstimate0.txt, order0.txt, NumOfJobs0.txt"
		[ -e ComplexityEstimate.$i -o -e order.$i -o -e NumOfJobs.$i ] && die "Found previous files - ComplexityEstimate.$i order.$i NumOfJobs.$i - and we don't want to overwrite them"

		mv ComplexityEstimate0.txt ComplexityEstimate.$i || die "Failed to move ComplexityEstimate0.txt ComplexityEstimate.$i"
		mv order0.txt order.$i || die "Failed to move order0.txt order.$i"
		mv NumOfJobs0.txt NumOfJobs.$i || die "Failed to move NumOfJobs0.txt NumOfJobs.$i"

	done
	;;

	linkage)
	for i in $indices; do
		[ -e order.$i -a -e datafile.$i -a -e pedfile.$i ] || die " failed to find files order.$i  datafile.$i  pedfile.$i"
		${SL} datafile.$i  pedfile.$i -e order.$i 0 0 > output.$i 2>&1 || die "failed to run Superlink on index $i"
	done
	;;
	*)
		die "Unexpected type"
	;;
esac

exit 0
