#!/bin/bash
basepath=`dirname $0`
basepath=`dirname $basepath`
. ${basepath}/CONFIG.SH ${basepath}
. ${basepath}/Utils.sh

RedirOutToGlobalLog

DecideLanderGreen(){
	return 1
	[ "x$TASK_ORIGIN" = "xSL_ONLINE" ] || return 1
	[ $TASK_MULTIPOINT -eq 0 ] && return 1
	$BIN/Superlink/Superlink_1.5_Vanilla datafile.dat pedfile.dat -alg  
	ret=$?
	[ $ret -eq 200 -o $ret -eq 1 ] && return 0
	return 1
}

[ $# -ne 2 ] && DieNoLogin "Wrong parameters: <job working dir> <TaskID>"

WorkDir=$1
TaskID=$2

cd $WorkDir || DieNoLogin "Could not chdir to $WorkDir"

[ -e $JOB_DETAILS_FILE ] || DieNoLogin "Failed to load $JOB_DETAILS_FILE - probably wrong invocation sequence"

source ${WorkDir}/$JOB_DETAILS_FILE

BIN="$DAG_HOME/bin"
SUBFILES="$DAG_HOME/subfiles";

[ "x$TASK_TWO_LOCI" = "x1" ] && {
        BIN="$TWO_LOCI_DAG_HOME/bin"
        SUBFILES="$TWO_LOCI_DAG_HOME/subfiles";
}


cwd=`pwd`
sed_cwd=`pwd | sed 's/\//\\\\\\//g'`
sed_execdir=`echo $BIN | sed 's/\//\\\\\\//g'`


#create JOB SUBMIT DAG

cat ${SUBFILES}/${JOB_SUBMIT_DAG}.tmpl | sed "s/_PEDTOOL_EXECDIR_/${sed_execdir}/" | sed "s/_PEDTOOL_WORKDIR_/${sed_cwd}/" > ${JOB_SUBMIT_DAG}

#create GlobalDag

#Here we decide if we run Lander-Green
if DecideLanderGreen ; then 
	cat ${SUBFILES}/GlobalDag_LanderGreen.dag.tmpl | sed "s/_PEDTOOL_EXECDIR_/${sed_execdir}/" | sed "s/_PEDTOOL_TASKID_/$TaskID/" | sed "s/_PEDTOOL_JOB_TYPE_DIR_/./" | sed "s/_PEDTOOL_DAGMAN_RETRIES_/$MAX_DEAD_DAGMAN_RETRIES/" > GlobalDag.dag
#	# copy LanderGreen submit
	cp ${SUBFILES}/LanderGreen.sub.tmpl  LanderGreen.sub || DieNoLogin "Problem while creating LanderGreen.sub"
	Log "Found lander green run"
else

	#At this point we decide if we use parallel ordering
	if [ -z "$TASK_SPECIFIC_POINT" ]; then
	# we create DAG without parallel ordering stage
		cat ${SUBFILES}/GlobalDag_no_parallel_ordering.dag.tmpl | sed "s/_PEDTOOL_EXECDIR_/${sed_execdir}/" | sed "s/_PEDTOOL_TASKID_/$TaskID/" | sed "s/_PEDTOOL_JOB_TYPE_DIR_/$JobTypeDir/" | sed "s/_PEDTOOL_DAGMAN_RETRIES_/$MAX_DEAD_DAGMAN_RETRIES/" > GlobalDag.dag
		else
	# we create DAG with parallel ordering
		cat ${SUBFILES}/GlobalDag_parallel_ordering.dag.tmpl | sed "s/_PEDTOOL_EXECDIR_/${sed_execdir}/" | sed "s/_PEDTOOL_TASKID_/$TaskID/" | sed "s/_PEDTOOL_JOB_TYPE_DIR_/$JobTypeDir/" | sed "s/_PEDTOOL_DAGMAN_RETRIES_/$MAX_DEAD_DAGMAN_RETRIES/" > GlobalDag.dag
	fi
fi
# in case we are restarting - use JOB entry from the exiting dag, since some jobs maybe already DONE
MergeRescueDag "GlobalDag.dag"

[ -e $BOINC_CHECKPOINT ] && { 
	Log "BOINC checkpoint file detected - exiting";
	MergeRescueDag "ParallelDag.dag"

	CondorSubmitDag "GlobalDag.dag" || DieNoLogin "Problem creating GlobalDag.dag"
	
	CondorSubmitDag "PedtoolSubmit.dag" || DieNoLogin "Problem while checking the validity of PedtoolSubmit.dag"
	
	CondorSubmitDag "ParallelDag.dag" || DieNoLogin "Problem while checking the validity of ParalleDag.dag"

	# rm GlobalDag log since in case it existed, since it would confuse PreAccept handler
	rm -f GlobalDag.dag.dagman.log

	 exit 0; 
}
#copy Accept.sub
ln -fs ${SUBFILES}/Accept.sub . || DieNoLogin "Problem while creating Accept.sub"

#copy TaskMonitor.sub
ln -fs ${SUBFILES}/TaskMonitor.sub . || DieNoLogin "Problem while creating TaskMonitor.sub"

#copy FirstEstimate.sub
ln -fs ${SUBFILES}/FirstEstimate.sub . || DieNoLogin "Problem while creating FirstEstimate.sub"

#copy SimpleJobRun.sub
cp -f ${SUBFILES}/SimpleJobRun.sub . || DieNoLogin "Problem while creating SimpleJobRun.sub"

#copy FirstEstimate.dag.condor.sub, SecondEstimate.dag.condor.sub, ParallelDag.dag.condor.sub
cp -f ${SUBFILES}/*.condor.sub . || DieNoLogin "Problem while copying all condor.sub files"

CondorSubmitDag "GlobalDag.dag" || DieNoLogin "Problem creating GlobalDag.dag"

CondorSubmitDag "PedtoolSubmit.dag" || DieNoLogin "Problem while checking the validity of PedtoolSubmit.dag"

# rm GlobalDag log since in case it existed, since it would confuse PreAccept handler
rm -f GlobalDag.dag.dagman.log


