#!/bin/bash

base=`dirname $0`
. $base/CONFIG.SH
. $base/Utils.sh
exec 3> $LOG_DIR/Maint.log
exec 1>&3
exec 2>&3
			
#TestNoLogin && { Log "NO_LOGIN detected, exiting"; exit; }

current_time=date+%s
lockfile=/tmp/status_lock_${CONDOR_GW_USER}
ValidateLockfile $lockfile && exit;


clean(){
	rm /tmp/status_lock_${CONDOR_GW_USER}	
}
trap clean EXIT

# just in case someone took our computer...- we don't want it to happen
#condor_vacate

AddCellData(){
 filename=$1
 shift
 [ -z "$*" ] && echo "<td align=middle><font size=1 color=\"$bgcolor\">---</font></td>" >> $filename ||
	echo "<td align=middle><font size=1 color=\"$bgcolor\">$*</font></td>" >> $filename
}

ParseState(){
	grep "$1" $2 | sed "s/$1//" | sed 's/\"//g' | awk '{ if ($0~/DONE/) { print "DONE<br>" } else { if ($0~/FAILED/) { print $0 "<br>"; } else { if ($0!~/^[[:space:]]*$/) print $0 "<br>"; };}}' | sort | uniq 
}

CreateStateStatHtml(){
	out=$1
	old_out=$out
	shift
	done=$1
	[ "$done" = "RUN" ] && done="" 
	shift
	rundirs=$*
		for i in $rundirs; do
			[ -e $WORK_DIR/$i/TASK_STATE.txt ] || continue
			if [ -e $WORK_DIR/$i/CACHED_STATUS.HTML -a ! -z "$done" ]; then
				 cat $WORK_DIR/$i/CACHED_STATUS.HTML >> $old_out
				 continue;
			else
				[ ! -z "$done" ] && out=$WORK_DIR/$i/CACHED_STATUS.HTML
			fi

			echo "<tr>" >> $out
			
			toParse=$WORK_DIR/$i/TASK_STATE.txt
			bgcolor="#00CC00"
			res=`grep BACKFILL $toParse`
			[ -z "$res" ] || bgcolor="#000099" 
			res=`grep ${FAILED} $toParse | grep -v  SECONDESTIMATE.FAILED`
			[ -z "$res" ] || bgcolor="#CC0033"
			#col_1 ID
			AddCellData $out $i
			#col_2 PEDTOOL TASK_
			AddCellData $out `grep "PEDTOOL TASK_" $toParse | cut -f2- -d"_" | sort | uniq | awk '{print $0 "<br>"}'` 
			#col_3 TASK_STATE
			htmls=`ls  $WORK_DIR/$i/*.html`
			[ ! -z "$htmls" ] && unknowns=`grep -i Unknown $WORK_DIR/$i/*.html |wc -l`
			AddCellData $out `tail -1 $toParse | awk '{ if ($0~/./) {print $0} else {print $1} }'` "<br>UNKNOWNS=$unknowns<br>"
			#col_4 TIME it takes -> get it from TOTAL_COMPUTATION_TIME_FILE
			AddCellData $out `cat $WORK_DIR/$i/$TOTAL_COMPUTATION_TIME_FILE`
			#col_5 Current res usage -> from statistics gatherd by condor_q
			av_par=" "
			[ -e $WORK_DIR/$i/PARALLELISM_STAT.txt ] && av_par=`cat $WORK_DIR/$i/PARALLELISM_STAT.txt`
			[ -z "$done" ] && AddCellData $out "`cat $WORK_DIR/$i/$RESOURCE_USAGE_STAT_FILE` $av_par"
			[ ! -z "$done" ] && AddCellData $out "`cat $WORK_DIR/$i/MAX_NUM_OF_RUNNING_JOBS` $av_par"
			#col_6 Done stat:
			data=`cat $WORK_DIR/$i/$TOTAL_DONE_JOBS_FILE`
			data="$data/`cat $WORK_DIR/$i/$TOTAL_JOBS_FILE`"
			[ -z "$done" ] && AddCellData $out $data
			#coll_7 Data points - ${INIT_PARALLEL_STATE} TOTAL_SUBTASKS
			AddCellData $out `ParseState "${INIT_PARALLEL_STATE} TOTAL_SUBTASKS" $toParse | cut -f2- -d"=" `
			#col_8 First estimate:
			AddCellData $out `ParseState "$FIRST_ESTIMATE_STATE" $toParse`
 			#col_9 Second estimate:
			AddCellData $out `ParseState "$SECOND_ESTIMATE_STATE" $toParse`
			#col_10 Simple run:
			AddCellData $out `ParseState  "$SIMPLEJOBRUN" $toParse `
			#col_11 Parallel:
			AddCellData $out `ParseState  "$PARALLEL_STATE" $toParse`
			echo "</tr>" >> $out

			 [ ! -z "$done" ] && cat $out >> $old_out
	 	done	
}

UpdateRunJobStatHtml(){
	rundirs=$*
	runout=${PEDTOOL_HTML_STAT_FILE}.RUN
	cat $REPORT_TEMPLATE_DIR/StatRun.pre >  $runout
	echo "<h3>Condor queue status: </h3>" >>$runout
	if [ ! -e $CONDOR_Q_OUTPUT ]; then 
		echo "<b>FAILED</b><br>" >> $runout
	else
		cat $CONDOR_Q_OUTPUT | awk 'BEGIN{total_count=0;run_count=0;idle_count=0;}{total_count++; split($2,status,/=/); if (status[2]==2) { run_count++ }else{idle_count++}}END{print "Total jobs in the queue: " total_count ", Running jobs: " run_count ", Idle/Held jobs: " idle_count "<br><br>" }' >> $runout
	fi
	
	if [ -s $LOG_DIR/SELF_RESOURCES.txt ]; then
		RES=`cat $LOG_DIR/SELF_RESOURCES.txt`
		echo `date +%s` $RES >> $LOG_DIR/AvailableResourcesLog
	else
		RES="<b>NOT AVAILABLE</b>" >> $runout
	fi
	echo "<h3>Number of resources available: $RES</h3>">>$runout
	CreateStateStatHtml $runout RUN $rundirs
	cat $REPORT_TEMPLATE_DIR/StatRun.post >> $runout
}

UpdateDoneJobStatHtml(){
	rundirs=$*
	doneout=${PEDTOOL_HTML_STAT_FILE}.DONE
	cat $REPORT_TEMPLATE_DIR/StatDone.pre | sed "s/PEDTOOL_LAST_DONE_TASKS_NUM/$MAX_DONE_FILES_STAT/" >  $doneout
	CreateStateStatHtml $doneout DONE $rundirs
	cat $REPORT_TEMPLATE_DIR/StatDone.post >> $doneout
}
UpdateHealthStatHtml(){
	healthout=${PEDTOOL_HTML_STAT_FILE}.HEALTH
	cat $REPORT_TEMPLATE_DIR/StatHealth.pre > $healthout
	echo "<h3>uptime</h3>: `uptime`<br>" >> $healthout
	echo "<h3>disk usage</h3><br>: ">> $healthout
	df | awk '{ print $0 "<br>"}' >> $healthout
	if [ -e $LOG_DIR/SanityCheck.log ]; then
		echo "<h3>Sanity check log</h3>" >> $healthout
		cat $LOG_DIR/SanityCheck.log |  awk '{ print $0 "<br>"}' >> $healthout
	fi
	echo "<hr>" >> $healthout
	echo "<h3>Local config file dump:</h3>" >> $healthout
	cat $base/CONFIG.LOCAL.SH | awk 'BEGIN{publish=0}{ if ($0~/STOP_PUBLISH_HERE/) {publish=0}; if (publish==1) {print $0 "<br>";}; if ($0~/START_PUBLISH_HERE/) {publish=1} }' >> $healthout
	cat $REPORT_TEMPLATE_DIR/StatHealth.post >> $healthout
}
#===========Let's check the load average, and if too high -> send mail to admin
EvalCPULoad
if [ $CPU_local_load -gt 3000 ]; then
	[ -e ${LOG_DIR}/LOCAL_LOAD_TOO_HIGH ] || {
		touch ${LOG_DIR}/LOCAL_LOAD_TOO_HIGH;
		CreateMailForAdmin "Load on `hostname` is very high!!! $CPU_local_load ";
	}
else
	 [ -e ${LOG_DIR}/LOCAL_LOAD_TOO_HIGH ] && {
	 	rm -f ${LOG_DIR}/LOCAL_LOAD_TOO_HIGH;
		CreateMailForAdmin "Load on `hostname` became normal: $CPU_local_load ";
	 }
fi


# let's do logrotate in log dir
logs=`ls ${LOG_DIR} | grep -v old`
for k in $logs ; do
	i=${LOG_DIR}/$k
	[ -e $i ] || continue;
	size=`stat  -c %s  $i`
	[ -z "$size" ] && continue;
	[ $size -lt 1000000 ] && continue;
	# this is the rotation
	cp -f $i $i.old
	cat /dev/null > $i
	gzip $i.old
done

#========== Find all done/run dirs=========#

count_done=0
count_run=0
declare -a done_tasks
declare -a run_tasks

#detect if nothing changed
ls $WORK_DIR > $LOG_DIR/old_tasks.new
ls $DONE_DIR >> $LOG_DIR/old_tasks.new
[ -e $LOG_DIR/old_tasks ] || touch $LOG_DIR/old_tasks
same=0
dout=`diff $LOG_DIR/old_tasks.new $LOG_DIR/old_tasks`
[ -z "$dout" ] || same=1
mv $LOG_DIR/old_tasks.new $LOG_DIR/old_tasks
# find them in the reverse order of modification time
for id in `ls -t $WORK_DIR`; do
	if [ -e $DONE_DIR/$id ]; then
		done_tasks[$count_done]=$id;
		let count_done++;
	else
		[ -d $WORK_DIR/$id -a ! -L $WORK_DIR/$id ] && {
			run_tasks[$count_run]=$id;
			let count_run++;
		};
	fi
done
# remove all old DONE jobs
if [ $count_done -gt $MAX_DONE_FILES_STAT ]; then
	let toRemove="$count_done"-"$MAX_DONE_FILES_STAT"
	
	while [ $toRemove -gt 0 ]; do
		let removeTask="$count_done"-"$toRemove"
		taskname=$WORK_DIR/${done_tasks[$removeTask]}
		[ -d $taskname ] && cat $taskname/TASK_STATE.txt >> $GLOBAL_STAT_FILE
		[ -z "$ARCHIVE_DIR" ] || {
			mkdir -p $ARCHIVE_DIR
			tar -C $WORK_DIR -z -c -f $ARCHIVE_DIR/${done_tasks[$removeTask]}.tar.gz  $taskname  
			rm -rf $taskname ;
			rm -f $DONE_DIR/$id;
			done_tasks[$removeTask]=""
		}
		let toRemove--
	done
fi

#==========================================#

[ -e $CONDOR_Q_OUTPUT -a $count_run -ne 0 ] && {
# take the condor_q output and put the relative statistics about jobs into each running dir
	[ -x ${BIN}/CreateRunJobsQStat.sh ] && ${BIN}/CreateRunJobsQStat.sh $CONDOR_Q_OUTPUT ${run_tasks[*]}
# Run sanity check for every running process 
	[ -x ${BIN}/RunSatityCheck.sh ] && ${BIN}/RunSatityCheck.sh $CONDOR_Q_OUTPUT ${run_tasks[*]} > $LOG_DIR/SanityCheck.log
# Run rescheduling
	[ -x ${BIN}/EnforceFIFO.sh ] && {
		[ -x ${BIN}/SetupPrios.sh ] && ${BIN}/SetupPrios.sh  $CONDOR_Q_OUTPUT

			${BIN}/EnforceFIFO.sh $CONDOR_Q_OUTPUT > $LOG_DIR/ClustersToHold.log
			[ -s $LOG_DIR/ClustersToHold.log ] && {
				to_hold=`cat $LOG_DIR/ClustersToHold.log`;
				Log "Would issue condor_hold on clusters $to_hold"
		#		Log "issuing condor_hold on clusters $to_hold"
		#		condor_hold $to_hold 
		#		sleep 5
		#		condor_release -a;
			};
	}
# Run migration scripts
	[ -x $BIN/DecideTaskMigration.sh ] && $BIN/DecideTaskMigration.sh ${run_tasks[*]};
}

#create stat file
mv $PEDTOOL_HTML_STAT_FILE $PEDTOOL_HTML_STAT_FILE.old

cat $REPORT_TEMPLATE_DIR/PedtoolReport.html.pre |sed "s/PEDTOOL_GATEWAY/${CONDOR_GW_USER}@${CONDOR_GW}/" > $PEDTOOL_HTML_STAT_FILE

stat_name=$LOG_DIR/global_stat/`date +"%d_%m_%y"`
[ -e $stat_name ] && {
	echo "Total CPU time for the last day: `grep ${CONDOR_GW_USER} $stat_name` <br>" >>  $PEDTOOL_HTML_STAT_FILE ;
}


[ -z "$BACKFILL_GW" ] || echo "For viewing backfill statistics please press <a href=${BACKFILL_GW_USER}.${BACKFILL_GW}.html> here <a/>" >> $PEDTOOL_HTML_STAT_FILE
echo "<p><b> Last update on:</b> `date` <p>" >> $PEDTOOL_HTML_STAT_FILE 
[ -e $QUEUE_DIR/NO_LOGIN ] && {
	echo "<h1>ATTENTION: NO_LOGIN found - Pedtool HALTED!!!!!</h1>" >> $PEDTOOL_HTML_STAT_FILE
	echo "Last lines of the log file: <br>" >> $PEDTOOL_HTML_STAT_FILE
	tail -30 $GLOBAL_LOG | awk '{print $0 "<br>"}' >> $PEDTOOL_HTML_STAT_FILE;
}
[ -e ${LOG_DIR}/LOCAL_LOAD_TOO_HIGH ] && {
	echo "<h1><font color="#ff0000">ATTENTION: Local load on the gateway is too high! <br> `uptime`</h1>" >> $PEDTOOL_HTML_STAT_FILE;
}
UpdateRunJobStatHtml ${run_tasks[*]} 
cat ${PEDTOOL_HTML_STAT_FILE}.RUN >> $PEDTOOL_HTML_STAT_FILE 

[ $same -ne 0 ] && UpdateDoneJobStatHtml ${done_tasks[*]}
cat ${PEDTOOL_HTML_STAT_FILE}.DONE >> $PEDTOOL_HTML_STAT_FILE 

UpdateHealthStatHtml
cat ${PEDTOOL_HTML_STAT_FILE}.HEALTH >> $PEDTOOL_HTML_STAT_FILE

cat $REPORT_TEMPLATE_DIR/PedtoolReport.html.post >>  $PEDTOOL_HTML_STAT_FILE

rm -f $PUBLISH_STATUS_LOCATION
cp $PEDTOOL_HTML_STAT_FILE $PUBLISH_STATUS_LOCATION
