#!/bin/bash
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
exec 3>> $LOG_DIR/PerformanceEstimationHandler.log
exec 1>&3 
exec 2>&3

TestNoLogin && { Log "NO_LOGIN detected, exiting"; exit; }

perf_lock_file=/tmp/PerfEstimationHandlerLock_${CONDOR_GW_USER}_${CONDOR_GW}
clear(){
	rm $perf_lock_file  > /dev/null 2>&1
}
ValidateLockfile $perf_lock_file && exit

trap clear EXIT
echo "------------Starting Performance estimation---------"
# this scripts goes to the remote peer to fetch its performance information

${BIN}/CondorQ_Fetcher.sh
# Analyze condor_q output to create statistics re/ our system usage
[ -x ${BIN}/CreateUtilizationStat.sh ] && ${BIN}/CreateUtilizationStat.sh $CONDOR_Q_OUTPUT >> ${LOG_DIR}/Utilization.log 
tail -1 ${LOG_DIR}/Utilization.log >> $PUBLISH_RUN_STAT_LOCATION

# create our own peformance stuff
if [ "x$ENABLE_RESOURCE_SHARING" = "x1" ]; then 
	$BIN/DetermineAvailableResources.sh

	if [ ! -z "$PEER_ACCOUNT" -a ! -z "$PEER_PERFORMANCE_DATA_SCRIPT" ]; then 

		peer_perf_file=$LOG_DIR/${PEER_ACCOUNT}_${PEER_HOST_PERF_FILE_SUFFIX} 
		rm -f $peer_perf_file 
		$BIN/Passwordless_ssh.sh $PEER_ACCOUNT $PEER_ASKPASS $PEER_PERFORMANCE_DATA_SCRIPT > $peer_perf_file
		res=$?
		[ $res -ne 0 ] && {
			Log "Failed to fetch performance info from peer ${PEER_ACCOUNT}"
			echo -1 > $peer_perf_file;
		}
	
		echo " Peer performance -> `cat $peer_perf_file` "
	fi
fi
