#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
DIRECTORY_NAME=$(dirname $0)
SCRIPT_NAME=$(basename $0)

declare -r DIRECTORY_NAME
declare -r SCRIPT_NAME
#declare -r RUNNING_TIME_SAFETY_FACTOR
# End of script constants

# Includes
. "$DIRECTORY_NAME/JobUtils.sh"
. "$DIRECTORY_NAME/BoincNodeConstants.sh"
# End of includes

# Processing arguments
current_directory=${1:-.}
return_value=$2

[ -z "${return_value}" ] && Die "Missing return value from the estimation node"

WaitForAccept || { Log "Accept returned failure -> can't continue"; 
				   exit ${retval}; 
				 }

[ "${return_value}" -eq $JOB_TOO_HEAVY ] && {
	echo $JOB_TOO_HEAVY > ESTIMATION_NODE_RESULT.txt
	exit $JOB_TOO_HEAVY;
}

[ "${return_value}" -ne 0 ] && { 
        Log "Abnormal termination of the estimation node"
        UpdateTaskState "ESTIMATION_NODE.${FAILED}"
        echo "${return_value}" > ${UBNORMAL_TERMINATION_FILE}
        exit "${return_value}";
}

parameters_file_path=$BOINC_PARAMETERS_FILE_NAME
new_parameters_file_path="${parameters_file_path}.new"

declare -a set_ids
declare -a working_directories
declare -a depths
declare -a jobs_numbers
#declare -a valid_routes
declare -a all_data_directories;  #=( $(FindAllDataDirs) )
declare -a all_data_directories_inodes; #=( $(stat --format="%i" ${all_data_directories[@]} ) )
declare -a working_directories_inodes; #=( $(stat --format="%i" ${working_directories[@]} ) )

read_parameters_file $parameters_file_path
# End of processing arguments

countDataDirs(){
	all_data_directories=( $(FindAllDataDirs) )
	all_data_directories_inodes=( $(stat --format="%i" ${all_data_directories[@]} ) )
	working_directories_inodes=( $(stat --format="%i" ${working_directories[@]} ) )

	total_jobs=0

	cd $current_directory

	Log "Starting $SCRIPT_NAME in $PWD"

	Log "Traversing BOINC directories to set totals: ${working_directories[@]}"

	set_id_counter=-1

	# First pass - setting the BOINC directories' totals
	for working_directory in ${working_directories[@]}
	do
		let set_id_counter++

		inode=$(stat --format="%i" ${working_directory})
		# Ignore directories that errored out on the previous stages
		[[ ! ( " ${all_data_directories_inodes[@]} " =~ " ${inode} " ) ]] && continue
		set_id=${set_ids[$set_id_counter]}
		working_directory_jobs=$(valid_routes_number $set_id)
		echo $working_directory_jobs > "$working_directory/$TOTAL_JOBS_FILE"

		Log "Set the total of $working_directory_jobs in $working_directory"
	done
	# End of first pass

	directories_totals_file_path=$(mktemp)

	# Second pass - collecting the information about total amount of jobs
	#working_directories=( $(FindAllDataDirs) )
	Log "Traversing working directories to collect all the totals: ${working_directories[@]}"
	rm -f "${new_parameters_file_path}"
	touch "${new_parameters_file_path}"

	estimate_results=""
	set_id_counter=-1
	for working_directory in ${working_directories[@]}
	do
		let set_id_counter++
	
		inode=$(stat --format="%i" ${working_directory})
		# Ignore directories that errored out on the previous stages
		[[ ! ( " ${all_data_directories_inodes[@]} " =~ " ${inode} " ) ]] && continue
		[ ! -f "$working_directory/${TOTAL_JOBS_FILE}" ] && continue

		working_directory_jobs=$(cat ${working_directory}/${TOTAL_JOBS_FILE})
		let total_jobs+=${working_directory_jobs}

		Log "Added ${working_directory_jobs} from ${working_directory} to the total"
		echo ${working_directories[$set_id_counter]} ${working_directory_jobs} >> "${directories_totals_file_path}"
		echo ${set_ids[$set_id_counter]} ${jobs_numbers[$set_id_counter]} ${depths[$set_id_counter]} ${working_directories[$set_id_counter]} >> "${new_parameters_file_path}"
	done

	data_directory_counter=-1
	for data_directory in ${all_data_directories[@]} 
	do
		let data_directory_counter++
		data_directory_inode=${all_data_directories_inodes[${data_directory_counter}]}
	
		[[ " ${working_directories_inodes[@]} " =~ " ${data_directory_inode} " ]] && continue
		[ ! -f "${data_directory}/${TOTAL_JOBS_FILE}" ] && continue

		data_directory_jobs=$(cat ${data_directory}/${TOTAL_JOBS_FILE})
		let total_jobs+=${data_directory_jobs}

		Log "Added ${data_directory_jobs} from ${data_directory} to the total"
		estimate_results="$estimate_results+${data_directory_jobs}"
		echo ${working_directories[$data_directory_counter]} ${data_directory_jobs} >> "${directories_totals_file_path}"
	done
	# End of second pass
	UpdateTaskState "$POST_ESTIMATION_STATE PRELIM=$estimate_results" 

}

countDataDirs

[[ ${total_jobs} -eq 0 ]] && 
{
	 echo $JOB_TOO_HEAVY > ESTIMATION_NODE_RESULT.txt
     exit $JOB_TOO_HEAVY;
}

	
# Third pass - getting rid of the biggest directories when the total number of jobs is excessive
# [[ ${total_jobs} -gt ${MAX_JOBS_PER_TASK} ]] && Die "The total number of jobs is more than ${MAX_JOBS_PER_TASK}, unable to process a task that complex"
[[ ${total_jobs} -gt ${COMMUNITY_GRID_LIMIT} ]] &&
{
	Log "The total of ${total_jobs} jobs before marking off the directories with the biggest number of jobs"
	removed_directories=( $(sort -r -k 2 -n "${directories_totals_file_path}" | awk '{ print $1 }' ) )
	removed_totals=( $(sort -r -k 2 -n "${directories_totals_file_path}" | awk '{ print $2 }' ) )
	removed_directory_counter=-1
	TooComplex=""
	for removed_directory in ${removed_directories[*]}
	do
		let removed_directory_counter++
		removed_total=${removed_totals[${removed_directory_counter}]}

		Log "Marking off ${removed_directory} with ${removed_total} jobs"
		let total_jobs-=${removed_total}
	    
        TooComplex="${TooComplex} ${removed_directory} ${removed_total}"
		MarkDoNotCompute ${removed_directory}

		[[ ${total_jobs} -le ${COMMUNITY_GRID_LIMIT} ]] && break
	done
	UpdateTaskState "$POST_ESTIMATION_STATE $total_jobs"
    Log "Checking if there are too complex tasks: TooComplex- ${TooComplex} "

	if [ ! -z "${TooComplex}" ]; then
    # let's check if we are not doing BEST_EFFORT or Single point.
	    HandleTooHeavyJobs ${SECOND_ESTIMATE_RESULT_FILE} ${SECOND_ESTIMATE_STATE} $TooComplex 
		rm -f "${directories_totals_file_path}"
		countDataDirs
	fi
}
# End of third pass

rm -f "${directories_totals_file_path}"

Log "Updating the total jobs file of the task directory with the total of ${total_jobs} jobs"

# Updating the total number of jobs
#echo ${total_jobs} > ${TOTAL_JOBS_FILE}
OverrideTotalJobs ${total_jobs}

# Calculating the grand average
running_time_estimate_files=( $(find . -name "${RUNNING_TIME_ESTIMATE_FILE}") )
# we assume that without running time we have rather short jobs

running_time_estimate=3600

[ -z "$TASK_BYPASS_PRUNING_ESTIMATION" ] || running_time_estimate=1500

update_bundle_size

bundle=$(bundle_size)
[[ ${#running_time_estimate_files[@]} != 0 ]] &&
{
	sum=$(echo $(cat ${running_time_estimate_files[@]} | tr '\n' '+')0 | bc -l )
	running_time_estimate=$(echo "scale = 0; ${sum} / ${#running_time_estimate_files[@]}" | bc -l)
	for future_bundle_size in `seq 1 ${bundle}`; do 
		tmp=$(( $running_time_estimate * ${future_bundle_size} ))
		[[ $tmp -gt $ESTIMATION_UPPER_BOUND ]] && { 
			let future_bundle_size--;  update_bundle_size $future_bundle_size; bundle=$future_bundle_size; break;
		}
	done
	running_time_estimate=$(( ${RUNNING_TIME_SAFETY_FACTOR} * ${bundle} * ${running_time_estimate} ));
}

Log "Generating a classad for the task ${TASKID}"
boinc_priority=$(default_priority ${TASKID})
cat "${DIRECTORY_NAME}/${BOINC_CLASSADS}" | sed 's/'${BOINC_PRIORITY}'[ ]*=\([^=][^;]*\);/'${BOINC_PRIORITY}' = '${boinc_priority}';/'   | \
                                            sed 's/'${BOINC_PRIORITY}'[ ]*=\([^=][^;]*\)\]/'${BOINC_PRIORITY}' = '${boinc_priority}'\]/' | \
                                            sed 's/CLUSTERS_LIMIT/'${CLUSTERS_LIMIT}'/'                                                  | \
											sed 's/RUNTIME_ESTIMATE/'${running_time_estimate}'/'                                         | \
											sed 's/BUNDLE_SIZE/'${bundle}'/'                                         | \
                                            sed 's/LOCAL_CLUSTER_LIMIT/'${LOCAL_CLUSTER_LIMIT}'/' > "${BOINC_CLASSADS}"
#echo $total_jobs
mv -f "${new_parameters_file_path}" "${parameters_file_path}"
[ ! -s "${parameters_file_path}" ] && Die "No sets left to compute, the parameters file is empty"
	
UpdateTaskState "${ESTIMATE_STATE}.${DONE}"
UpdateTaskState "PARALLEL_COMPUTATION_BOINC BUNDLE=$bundle RUNTIME=${running_time_estimate} "

ReportWebUpdateToUser

Log "Ending $SCRIPT_NAME"
exit 0
