#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog
#set -x
# this script has to complete the groupped job by putting the result files into all
# the appropriate data dirs

getPathFromIdx(){
	idx=$1
	d_idx=`echo $idx | cut -d"." -f1`
	p_idx=`echo $idx | cut -d"." -f2`
	result="datafile_${d_idx}/pedfile_${p_idx}"
	[ -e $result ] || Die "Failed to find $result"
}

[ -z "$4" ] && Die "Wrong parameters: RETURN type[order|linkage] workdir_with_results indexfile"

retval=$1
type=$2
workdir=$3
idxfile=$4

# let's check the parameters:
# we pass through only if we succeed
[ $retval -ne 0 ] && {
	Log "Grouped job $type $workdir $idxfile returned $retval, exiting with $retval"
	exit $retval;
}
[ "$type" = "order" -o "$type" = "linkage" ] || Die "Unrecognized type $type"
[ -e $workdir -a -e $workdir/$idxfile ] || Die "Failed to find either  $workdir or $workdir/$idxfile"
indices=`cat $workdir/$idxfile`
[ -z "$indices" ] && Die "Index file $workdir/$idxfile empty"

# for "order" we should copy: 
# ComplexityEstimate0.txt, order0.txt, NumOfJobs0.txt
# for "linkage" we should copy:
# output.0

case "$type" in
	order)
		for i in $indices; do
			getPathFromIdx $i
			cp -f $workdir/ComplexityEstimate.$i $result/ComplexityEstimate0.txt || Die "Failed to copy $workdir/ComplexityEstimate.$i $result/ComplexityEstimate0.txt"
			cp -f $workdir/order.$i $result/order0.txt || Die "Failed to copy $workdir/order.$i $result/order0.txt "
			cp -f $workdir/NumOfJobs.$i $result/NumOfJobs0.txt || Die "Failed to copy $workdir/NumOfJobs.$i $result/NumOfJobs0.txt"
		done
	;;
	linkage)
		for i in $indices; do
			getPathFromIdx $i
			cp -f $workdir/output.$i $result/tmp_grp_out || Die "Failed to copy  $workdir/output.$i $result/output.0"
			mv $result/tmp_grp_out $result/output.0  || Die "Failed to copy  $result/output.$i $result/output.0"
			echo 1 > $result/${TOTAL_DONE_JOBS_FILE}
		done
	;;
	*)
esac
	
# we can safely remove the directory:
#rm -rf $workdir
Log "Done multiplexing results for GrouppedJobRun on $workdir"
exit 0
