#!/bin/bash
basepath=`dirname $0`
. "${basepath}/JobUtils.sh"
. "${basepath}/BoincNodeConstants.sh"
RedirOutToTaskLog

# invoked after all Parallel runs are complete

# we get here only after ParallelDag is complete. 

# we are bailing out in case it's checkpoint
[ ! -z "$TASK_CHECKPOINT" ] && { 
	Log "exiting since TASK_CHECKPOINT found"
	exit -1;
} 

[ -z "$1" ] && Die "Missing parameters: return status"
ret=$1
[ -e ${UBNORMAL_TERMINATION_FILE} ] && ret=`cat ${UBNORMAL_TERMINATION_FILE}`

# Checking the consistency of BOINC-related results
[ $ret -eq 0 ] && rm -f ParallelDag.dag.rescue 

if [[ $ret -eq 0  && "x1" -ne "x$BOINC_RUN" ]]
then 
		# first we check that all the output files are full
		find -mindepth 2 -name "output.*" -size 0 > zero

		res=`cat zero`
		[ -z "$res" ] && {
			# the run is complete 
			touch ${TASK_COMPLETE_FILE}
			Log "Normal termination of parallel run detected - consistency check OK - terminating"
			exit 0;
		}

		Error "Consistency check failed - regenerating the relevant DAG nodes"
		rm -f names;	touch names;
		for i in `cat zero| cut -d"/" -f2|uniq` ; do
			 name=`grep $i\/ ParallelDag.dag | head -1| cut -f2 -d" "| cut -f1,2 -d"_"` ; 
		 	for j in `grep $i zero | cut -f3 -d"."`;do 
				echo ${name}_${j} >> names; 
			done;  
		done
#		#
		${BIN}/../auxilary/FixDagNotReady.pl || Die  "Failed to fix ParallelDag - restarting the whole DAG"
		cp ParallelDag.dag ParallelDag.dag.old
		mv -f ParallelDag.dag.result ParallelDag.dag || Die "Failed to copy new ParallelDag onto old - restarting the whole DAG"
fi

if [[ "x${BOINC_RUN}" == "x1" && ${ret} == 0 ]]
#if [[ "x${BOINC_RUN}" == "x1" ]]
then
	for xxixx in `seq 0 5`; do  
		"${basepath}/$GATHER_RESULT_SCRIPT"
		exit_code=$?
		# The task accomplished successfully
		[[ ${exit_code} == 0 ]] && {
			touch ${TASK_COMPLETE_FILE}
			Log "Normal termination of parallel run detected - consistency check OK - terminating"
			exit 0
		}
		sleep 10
	done
	Error "Consistency check failed - regenerating the relevant DAG nodes"
	# we first MUST check that there was boinc node ad all in the parallel dag
    isBoinc=`grep  Parallel_single_boinc_condor ParallelDag.dag`
	[ -z "$isBoinc" ] && Die "Inconsistency in the run"
	
	# Deleting the gradual submission state file to start the submission anew
	rm -f "${BOINC_SUBMISSION_STATE_FILE}"
		
	declare -a finished_directories=( $(find "${WORK_DIR}/${TASKID}" -name "${BOINC_RESULT_FILE_NAME}" | while read line; do dirname $line; done) )
	declare -a valid_data_inodes=( $(stat --format="%i" $(FindAllDataDirs) ) )
	declare -a set_ids=() working_directories=() depths=() jobs_numbers=()

	read_parameters_file "${BOINC_PARAMETERS_FILE_NAME}"
	directory_index=-1
	new_parameters_file="${BOINC_PARAMETERS_FILE_NAME}.new"
	old_parameters_file="${BOINC_PARAMETERS_FILE_NAME}.old"

	for directory in ${working_directories[@]}
	do
		let directory_index++

		[[ ! ( " ${finished_directories[@]} " =~ " ${directory} " ) && ( " ${valid_data_inodes[@]} " =~ " $(stat --format "%i" ${directory}) " ) ]] && {
			# New content of the BOINC parameters file
			echo "${set_ids[$directory_index]}" "${jobs_numbers[$directory_index]}" "${depths[$directory_index]}" "${directory}"

			current_estimation_parameters_file="${ESTIMATION_PARAMETERS_FILE_NAME}.${set_ids[$directory_index]}"
			finished_routes_file="${current_estimation_parameters_file}.finished"
			primary_estimation_parameters_file="${current_estimation_parameters_file}.primary"
			new_estimation_parameters_file="${current_estimation_parameters_file}.new"
			old_estimation_parameters_file="${current_estimation_parameters_file}.old"	

			# This is the estimation parameters file, we will be comparing ourselves to after each restart
			[[ ! ( -f "${primary_estimation_parameters_file}" ) ]] && cp -f "${current_estimation_parameters_file}" "${primary_estimation_parameters_file}"

			# New content of estimation parameters file
			cat "${directory}/output" | cut -f2 -d' ' > "${finished_routes_file}"
			cat "${directory}/${NON_BOINC_OUTPUTS_FILE_NAME}" | \
				while read line; do ln_like=""; ln_like=$(grep TOTAL $line | awk '{ print $2 }' ); [[ -n "${ln_like}" ]] && echo $line; done | \
				awk 'BEGIN { FS="." } { print $NF }' >> "${finished_routes_file}"
			cat "${finished_routes_file}" | sort -n | uniq > "${new_estimation_parameters_file}"
			cp -f "${new_estimation_parameters_file}" "${finished_routes_file}"
			diff -c -b -B "${finished_routes_file}" "${primary_estimation_parameters_file}" | grep -e '^+' | cut -f2 -d' ' > "${new_estimation_parameters_file}"
			cp -f "${current_estimation_parameters_file}" "${old_estimation_parameters_file}"
			mv -f "${new_estimation_parameters_file}" "${current_estimation_parameters_file}"

			gather_total_jobs_file="${directory}/${GATHER_TOTAL_JOBS_FILE}"
			total_jobs_file="${directory}/${TOTAL_JOBS_FILE}"

			[[ ! ( -f "${gather_total_jobs_file}" ) ]] && cp -f "${total_jobs_file}" "${gather_total_jobs_file}" 
			wc -l "${current_estimation_parameters_file}" | cut -f1 -d' ' > "${total_jobs_file}"
		}
	done > "${new_parameters_file}"

	cp -f "${BOINC_PARAMETERS_FILE_NAME}" "${old_parameters_file}"
	mv -f "${new_parameters_file}" "${BOINC_PARAMETERS_FILE_NAME}"
	# Regenerating the parallel DAG nodes, at this stage only BOINC node has to remain
	current_dag_file=ParallelDag.dag
	new_dag_file=${current_dag_file}.new
	old_dag_file=${current_dag_file}.old
	job_id="SLJob_${TASKID}"
	incarnation_number=0

	[ -s ${DEAD_DAGMAN_FILE} ] && let incarnation_number+=$(cat ${DEAD_DAGMAN_FILE})
	instantiate_boinc_node_template ".${incarnation_number}"

	cat > ${new_dag_file} << EOF
# Dag for all parallel jobs
JOB ${job_id} ${BOINC_CONDOR_NODE_NAME} 
RETRY ${job_id} 10
VARS ${job_id} EXECDIR="${basepath}" WORKDIR="${WORK_DIR}/${TASKID}" TOTAL="1" TASKID="${TASKID}" ARGUMENTS="boinc_node_parameters.txt" PRIO="${TASK_PRIO_IN_CONDOR}"
EOF

	cp -f ${current_dag_file} ${old_dag_file} 
	mv -f ${new_dag_file} ${current_dag_file} || Die "Failed to copy a new ParallelDag onto the old - restarting the whole DAG"
fi

# we are here since DAGman died several times in line -> check how many times and
# update UBNORMAL_TERMINATION if too many
[ -e  ${DEAD_DAGMAN_FILE} ] && 	num=`cat ${DEAD_DAGMAN_FILE}`
[ -z "$num" ] && num=0
[ $num -gt $MAX_DEAD_DAGMAN_RETRIES ] && echo $ret > ${UBNORMAL_TERMINATION_FILE}
let num++;
echo $num > ${DEAD_DAGMAN_FILE}
Log "Retrying parallel DAG. Either DAGman died or consistency check failed"
exit 1 
