#!/bin/bash 
# this script is called to assert that the GlobalDag.dag is being started
# and to output its cluster ID to be able to condor_rm it whenever we want
basepath=`dirname $0`
. ${basepath}/JobUtils.sh

RedirOutToTaskLog

# find cluster Id and put it into GLOBALDAG_CLUSTERID_FILE: check submit entry in the GlobalDag log file
# if Schedd is overloaded, it will not be able to update the log right away. try 3 times
retries=0
clusterId=""
Log "Starting Pedtool task - detecting cluster ID"

UpdateTaskState "PEDTOOL"
UpdateTaskState "PEDTOOL TASKID=${TASKID}"
[ -z "$TASK_QUEUE_ENTRY" ] || UpdateTaskState "PEDTOOL TASK_QUEUE_ENTRY=${TASK_QUEUE_ENTRY}"

UpdateTaskState "PEDTOOL TASK_EMAIL=${TASK_EMAIL}"
UpdateTaskState "PEDTOOL TASK_PROJECT_NAME=${TASK_PROJECT_NAME}"
[ ! -z $TASK_TWO_LOCI ]  && UpdateTaskState "PEDTOOL TASK_TWO_LOCI=$TASK_TWO_LOCI"

if [ -z "${TASK_TYPE}" -o "${TASK_TYPE}" = "LINKAGE" ]; then 
	UpdateTaskState "PEDTOOL TASK_IS_MULTIPOINT=${TASK_MULTIPOINT}"
	UpdateTaskState "PEDTOOL TASK_TYPE=LINKAGE" 
else
	UpdateTaskState "PEDTOOL TASK_TYPE=${TASK_TYPE}"
fi
# check the number of loci and num of peds and max num of persons
[ -e pedfile.dat ] || Die "Failed to find pedfile.dat"
peds=(`cat pedfile.dat | awk '{ if ( max<$2 ) max=$2; if (max2<$1) max2=$1 }END{print max2 " " max}'`)
[ -e datafile.dat ] || Die "Failed to find datafile.dat"
loci=`grep -e [[:digit:]] datafile.dat | head -1 | cut -f1 -d" "`

UpdateTaskState "PEDTOOL TASK_NUM_OF_PEDIGREES=${peds[0]}"
UpdateTaskState "PEDTOOL TASK_MAX_NUM_PERSONS=${peds[1]}"
UpdateTaskState "PEDTOOL TASK_NUM_OF_LOCI=${loci}"

while [ $retries -lt 3 -a -z "$clusterId" ]; do
	clusterId=`cat GlobalDag.dag.dagman.log |  awk 'BEGIN{res=-1}{p=match($0, /^000[[:blank:]]+\(([[:alnum:]]*)\..*\)/,k); if (p!=0) res=k[1]}END{if (res!=-1) print res}'`
	[ ! -z ${clusterId} ] && break
	let retries++
	sleep ${retries}
done
[ -z "$clusterId" ] && { 
	DieNoLogin "BUG!!! Failed to find clusterId from the GlobalDag.dag.dagman.log";
}

echo $clusterId > $GLOBALDAG_CLUSTERID_FILE
Log "Cluster ID detected: $clusterId"

# IMPORTANT - we don't output 0 to the ${ACCEPT_RESULT_FILE} since 
# we continue to Accept 
exit 0
