#!/bin/bash

DIRECTORY_NAME=$(dirname $0)
SCRIPT_NAME=$(basename $0)
TEMPORARY_ROUTES_FILE_NAME=routes.old

declare -r TEMPORARY_ROUTES_FILE_NAME
declare -r SCRIPT_NAME
declare -r DIRECTORY_NAME

. "$DIRECTORY_NAME/JobUtils.sh"
. "$DIRECTORY_NAME/BoincNodeConstants.sh"
WaitForAccept || { Log "Accept returned failure -> can't continue"; 
				   exit ${retval}; 
				 }

[ -s ESTIMATION_NODE_RESULT.txt ] && {
	retval=`cat ESTIMATION_NODE_RESULT.txt`
	exit $retval;
}

UpdateTaskState "PRUNING.DONE"

# Arguments processing
parameters_file_path=$1

[ ! -f $parameters_file_path ] && Log "Non-existent parameters file for the pre-estimation node" && exit 1 

declare -a set_ids depths jobs_numbers working_directories complexities accomplished_sets

read_parameters_file $parameters_file_path
#task_directory=${1:-.}
#
#[ ! -d $task_directory ] && Log "Non-existent working directory for the pre-estimation node" && exit 1
#
#cd $task_directory
Log "Starting $SCRIPT_NAME"

#declare -a working_directories=( $(FindAllDataDirs) )
# End of arguments processing

set_index=0

Log "Removing estimation parameters file $ESTIMATION_PARAMETERS_FILE_NAME"
rm -f "$ESTIMATION_PARAMETERS_FILE_NAME"

Log "Traversing working directories: ${working_directories[*]}"

unique_set_id_base=$(update_maximal_set_id ${#set_ids[@]})
exit_code=$?
[[ ${exit_code} != 0 ]] && Die "Unable to obtain a new offset for all the set ids (exit code ${exit_code})"
pruning_results=""

for working_directory in ${working_directories[@]}
do
	#declare -a valid_routes=( $(grep 'valid=>' $working_directory/output.pruning_node.* | cut -d'>' -f2 | tr -d ' ' | sort -n | uniq | tr '\n' ' ') )
	# The elimination of ^M characters is intended to get rid of the Windows CLRF characters at the end of each line, which might
	# appear there, as the pruning job might have ran on a Windows machine. In order to edit the ^M character properly, use vi.
	# In vi press Ctrl+V, then Ctrl+M. Vim sometimes remaps all Ctrl+V commands to Visual Block commands, so it is not trustworthy.
	# The proper ^M is usually marked as bold both in vi and vim
	find $working_directory -name 'output.pruning_node.*' -exec grep 'valid=>' {} \; | cut -d'>' -f2 | tr -d ' 
' | sort -n | uniq > "$TEMPORARY_ROUTES_FILE_NAME"

	routes_number=$(wc -l "$TEMPORARY_ROUTES_FILE_NAME" | cut -f1 -d' ' )
	# When no valid routes are found, skip creating the directory
	[[ $routes_number == 0 ]] && {
		Log "Directory $working_directory has been pruned out completely"
		echo 0 > "$working_directory/$TOTAL_JOBS_FILE"
		let set_index++
		continue
	}
	# Making set id unique, this is crucial for the interaction with the database node
	unique_set_id=$(( $unique_set_id_base + $set_index ))

	echo $unique_set_id ${jobs_numbers[$set_index]} ${depths[$set_index]} $working_directory >> $ESTIMATION_PARAMETERS_FILE_NAME
	mv "$TEMPORARY_ROUTES_FILE_NAME" "${ESTIMATION_PARAMETERS_FILE_NAME}.${unique_set_id}"
	Log "There were ${jobs_numbers[$set_index]} jobs before the pruning and ${routes_number} after it in $working_directory"
	pruning_results="$pruning_results ${jobs_numbers[$set_index]}->${routes_number}"
	let set_index++
done

UpdateTaskState "PRUNING_RESULTS=$pruning_results"

Log "Ending $SCRIPT_NAME"
