#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog


[ -z "$1" ] && Die "Missing parameter: PedtoolTaskID"

#First of all -> check Lander green
[ -e $TASK_COMPLETE_FILE ] && exit -1
# if LANDER_GREEN is not empty -> error already handled
[ -s $LANDER_GREEN_RUN_FILE ] && {
	cat $LANDER_GREEN_RUN_FILE > ${FIRST_ESTIMATE_RESULT_FILE};
	exit -1;
}
	

# The following is a workaround for DAGman which cannot handle DONE children for READY parent
# we enforce dependencies manually here

WaitForAccept || { Log "Accept returned failure -> can't continue"; 
				   echo ${retval} > ${FIRST_ESTIMATE_RESULT_FILE}; 
				   exit ${retval}; 
				 }


Log "Running inital preparallelization"
UpdateTaskState "${INIT_PARALLEL_STATE}"

[ -x ${BIN}/${INITIAL_PARALLELIZATION_SCRIPT} ] || { 
	DieNoLogin "No ${INITIAL_PARALLELIZATION_SCRIPT} available/runnable";
}

RunCommand "${BIN}/${INITIAL_PARALLELIZATION_SCRIPT}" || {
	[ -e error.txt ] && {
		mv error.txt $SUPERLINK_INPUT_ERROR_FILE;
		ret=$SUPERLINK_INPUT_ERROR;
	}
	echo ${ret} > ${FIRST_ESTIMATE_RESULT_FILE}
	[ $ret = $SUPERLINK_INPUT_ERROR ] && exit $SUPERLINK_INPUT_ERROR;
	TaskDie "Failed during invocation of pre-parallelization state. Command output: ${command_output}";

}

[ -e ${NUM_OF_DATAFILES_FILE} ] || TaskDie "Expected ${NUM_OF_DATAFILES_FILE} => not found here"
total=`cat ${NUM_OF_DATAFILES_FILE}`
echo $total
[ "x${total}" = "x0" ] && {
	TaskDie "Inital parallelization produced 0 parallel instances. This may occur either due to a bug, or wrong input";
}

Log "Initial parallelization successfull - created $total initial instances"
UpdateTaskState "${INIT_PARALLEL_STATE} TOTAL_SUBTASKS=$total"
UpdateTaskState "${INIT_PARALLEL_STATE}.${DONE}"

RunCommand "${BIN}/CreateInitDags.sh" || { 
	TaskDie "Failed to create inital DAGs";
}

Log "DAG generation successfull"

