#!/bin/bash
basepath=`dirname $0`
#BOINC_CONDOR_NODE_NAME=Parallel_single_boinc_condor.sub
ESTIMATION_NODE_NAME=Parallel_single_estimation.sub
PRUNING_NODE_NAME=Parallel_single_pruning.sub
POST_ESTIMATION_NODE_NAME=Parallel_single_post_estimation.sub
DAG_FILENAME=ParallelDag.dag
POST_PRUNING_SCRIPT_NAME=PostPruningNode.sh
PRE_ESTIMATION_SCRIPT_NAME=PreEstimationNode.sh
POST_ESTIMATION_SCRIPT_NAME=PostEstimationNode.sh
#POST_BOINC_SCRIPT_NAME=PostBoincNode.sh
#set -x

#declare -r POST_BOINC_SCRIPT_NAME
#declare -r BOINC_CONDOR_NODE_NAME
declare -r POST_ESTIMATION_NODE_NAME
declare -r ESTIMATION_NODE_NAME
declare -r PRUNING_NODE_NAME
declare -r DAG_FILENAME
declare -r POST_ESTIMATION_SCRIPT_NAME
declare -r PRE_ESTIMATION_SCRIPT_NAME 
declare -r POST_PRUNING_SCRIPT_NAME

. ${basepath}/JobUtils.sh
. ${basepath}/BoincNodeConstants.sh
RedirOutToTaskLog
createDagNodes(){
        NumOfJobs=$1
        Depth=$2
        ParallelTaskId=$3
        WorkDir=$4
        TaskId=$TASKID
		new_priority=$TASK_PRIO_IN_CONDOR

        # we create large DAG here
        [ -e $WorkDir ] || Die "$0 - Cannot find directory $WorkDir"

        if [ ! -r $WorkDir/${PRIORITY_FILE} ]; then
                Prio=0
                Prio=$new_priority
				echo $Prio >  $WorkDir/${PRIORITY_FILE}
        else
                Prio=`cat $WorkDir/${PRIORITY_FILE}`
        fi

        [ -z "$Prio" ] && { Warn "Condor prio cannot be detected. Assuming 0"; Prio=0; }

        #fill up parallel dag with jobs
        DagFilename=ParallelDag.dag
        SingleSubFilename=Parallel_single.sub
        FinalSubFilename=Parallel_single_done.sub
        FinalJobId=SumUp${ParallelTaskId}

        ii=0

        while [ "x$ii" != "x$NumOfJobs" ]; do
                SLJobId=SLJob_${ParallelTaskId}_${ii}
cat >> $DagFilename <<EOF
JOB $SLJobId $SingleSubFilename
RETRY $SLJobId 1000
VARS $SLJobId EXECDIR="${BIN}" TOTAL="$NumOfJobs" PERMNUM="$ii" DEPTH="$Depth" WORKDIR="$WorkDir"  PRIO="$Prio" TASKID="$TaskId"
EOF

touch $WorkDir/output.$ii
let ii++
done
}

# Removes relative '.' and '..' symbols from the directory path
beautifyDirectory()
{
	local directory=$1

	cd $directory
	local beautified_directory=$(pwd)
	cd - > /dev/null

	echo $beautified_directory
}

createPruningNodeParameters()
{
	local NumOfJobs=$1
	local Depth=$2
	local ParallelTaskId=$3
	local WorkDir=$4
	local TaskId=${TASKID}
	# we create large DAG here
	[ -e ${WorkDir} ] || Die "$0 - Cannot find directory $WorkDir"

	# Canonize the working directory
	WorkDir=$(beautifyDirectory "$WorkDir")

	# Creating/updating the parameters file for this BOINC monitoring node
	cat >> ${PRUNING_PARAMETERS_FILE_NAME} << EOF
${ParallelTaskId} ${NumOfJobs} ${Depth} ${WorkDir}
EOF
}

createBoincDagNodes(){
	local NumOfJobs=$1
	local Depth=$2
	local ParallelTaskId=$3
	local WorkDir=$4
	local TaskId=$TASKID
	# we create large DAG here
	[ -e $WorkDir ] || Die "$0 - Cannot find directory $WorkDir"

	# Canonize the working directory
	WorkDir=$(beautifyDirectory "$WorkDir")
	#fill up parallel dag with jobs

	# Calculating the number of pruning nodes
	local approximate_pruning_jobs_number=$(ceiling $(echo "$NumOfJobs / $ESTIMATION_SAMPLES_PER_BIN" | bc -l) )
	local -a pruning_jobs_number_depth=( $(read_jobs_number_depth "$WorkDir/NumOfJobs.txt" $approximate_pruning_jobs_number) )
	local pruning_jobs_priority=$((${HIGHEST_PRIORITY} - ${TaskId} - ${approximate_pruning_jobs_number}))

	# Creating a definition of pruning node
	local EstimationJobId=EstimationJob_${TaskId}

	Log "Creating ${pruning_jobs_number_depth[0]} pruning jobs for directory ${WorkDir}"

	local pruning_node_working_directory=$(echo $WorkDir | sed "s/\//\\\\\\//g")

	cat $SUBFILES/$PRUNING_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_WORKDIR_/$pruning_node_working_directory/" > $PRUNING_NODE_NAME.$ParallelTaskId
	local periodic_remove_interval=$(( ${ESTIMATION_UPPER_BOUND_SAFETY_COEFFICIENT} * ${ESTIMATION_UPPER_BOUND} ))
	local range_size=$(( ${NumOfJobs} / ${pruning_jobs_number_depth[0]} ))

	for dead_end_permutation in $( seq 0 $((${pruning_jobs_number_depth[0]} - 1)) )
	do
		local range_start=$(( ${range_size} * ${dead_end_permutation} ))
		local range_end=$(( ${range_start} + ${range_size} - 1))

		PruningJobId=PruningJob_${TaskId}_${ParallelTaskId}_${dead_end_permutation}
		cat >> $DAG_FILENAME << EOF
JOB $PruningJobId $PRUNING_NODE_NAME.$ParallelTaskId
RETRY $PruningJobId 10
VARS $PruningJobId EXECDIR="${BIN}" WORKDIR="$WorkDir" DEPTH="$Depth" DEAD_END_PERMUTATION="$dead_end_permutation" DEAD_END_DEPTH="${pruning_jobs_number_depth[1]}" PRIO="${pruning_jobs_priority}" PERIODIC_REMOVE_INTERVAL="${periodic_remove_interval}"
SCRIPT POST ${PruningJobId} ${BIN}/${POST_PRUNING_SCRIPT_NAME} ${WorkDir}/output.pruning_node.${dead_end_permutation} ${range_start} ${range_end}

PARENT $PruningJobId CHILD $EstimationJobId
EOF
	done

	# Creating/updating the parameters file for this BOINC monitoring node
	cat >> $PRUNING_PARAMETERS_FILE_NAME << EOF
$ParallelTaskId $NumOfJobs $Depth $WorkDir
EOF
}

createBoincNodeBypassDefinition()
{
	local NumOfJobs=$1
	local WorkDir=$2
	local TaskId=$TASKID

	local SLJobId=SLJob_${TaskId}
	local boinc_job_priority=$TASK_PRIO_IN_CONDOR

	# Creating a definition of BOINC monitoring node without preceding estimation and pruning nodes
	
	# Make sure it is the first in the DAG
	[ -e $DAG_FILENAME ] && mv $DAG_FILENAME ${DAG_FILENAME}.update
	
	# Repeating part of the pre-estimation node functionality: adding the greatest monitoring set id to all the set ids
	Log "Seeking for a new set id base for ${NumOfJobs} sets"
	local unique_set_id_base=$(update_maximal_set_id ${NumOfJobs})

	Log "Found new set id base (${unique_set_id_base}) for task ${TaskId}"

	while read line
	do
		local -a line_array=( ${line} )
		local set_id=${line_array[0]}
		local unique_set_id=$(( ${unique_set_id_base} + ${set_id} ))

		echo ${unique_set_id} ${line_array[@]:1} >> "${BOINC_PARAMETERS_FILE_NAME}"
	done < "${PRUNING_PARAMETERS_FILE_NAME}"
	# End of adding the greatest monitoring set id to all the set ids
	# Repeating part of the pre-estimation node functionality: creating the entire range of routes
	while read line
	do
		local -a line_array=( ${line} )
		local unique_set_id=${line_array[0]}
		local routes_number=${line_array[1]}

		Log "Creating ${routes_number} routes for set id ${unique_set_id}"

		for route_index in $(seq 0 $(( ${routes_number} - 1 )) )
		do
			echo ${route_index} >> ${ESTIMATION_PARAMETERS_FILE_NAME}.${unique_set_id}
		done
	done < "${BOINC_PARAMETERS_FILE_NAME}"
	# End of creating the entire range of routes
	cat >> $DAG_FILENAME << EOF
JOB $SLJobId $BOINC_CONDOR_NODE_NAME
RETRY $SLJobId 10
VARS $SLJobId EXECDIR="${BIN}" WORKDIR="$WorkDir" TOTAL="$NumOfJobs" TASKID="$TaskId" ARGUMENTS="$BOINC_PARAMETERS_FILE_NAME" PRIO="${boinc_job_priority}"
EOF

	touch output.boinc_node

	local post_estimation_job_priority=$TASK_PRIO_IN_CONDOR
	# Creating a definition of the post-estimation node, which calculates totals in all the working directories
	local PostEstimationJobId=PostEstimationJob_${TaskId}

	Log "Creating post estimation job for task ${TaskId} in directory ${WorkDir}"

	cat >> ${DAG_FILENAME} << EOF
JOB ${PostEstimationJobId} ${POST_ESTIMATION_NODE_NAME}
RETRY ${PostEstimationJobId} 10
VARS ${PostEstimationJobId} EXECDIR="${BIN}" WORKDIR="${WorkDir}" TOTAL="${NumOfJobs}" TASKID="${TaskId}" ARGUMENTS="${WorkDir} 0" PRIO="${post_estimation_job_priority}"

PARENT $PostEstimationJobId CHILD $SLJobId
EOF

	[ -e ${DAG_FILENAME}.update ] && { 
		cat ${DAG_FILENAME}.update >>  $DAG_FILENAME
		rm -f  ${DAG_FILENAME}.update;
	}
}

createBoincNodeDefinition()
{
	local NumOfJobs=$1
	local WorkDir=$2
	local TaskId=$TASKID

	local SLJobId=SLJob_${TaskId}
	local boinc_job_priority=$TASK_PRIO_IN_CONDOR

	# Creating a definition of BOINC monitoring node
	
	# make sure it's the first in the DAG
	
	[ -e $DAG_FILENAME ] && mv $DAG_FILENAME ${DAG_FILENAME}.update
	
	cat >> $DAG_FILENAME << EOF
JOB $SLJobId $BOINC_CONDOR_NODE_NAME
RETRY $SLJobId 10
VARS $SLJobId EXECDIR="${BIN}" WORKDIR="$WorkDir" TOTAL="$NumOfJobs" TASKID="$TaskId" ARGUMENTS="$BOINC_PARAMETERS_FILE_NAME" PRIO="${boinc_job_priority}"
EOF
#SCRIPT POST $SLJobId $BIN/$POST_BOINC_SCRIPT_NAME $WorkDir

	touch output.boinc_node

	local estimation_job_priority=$TASK_PRIO_IN_CONDOR
	# Creating a definition of the estimation node
	local EstimationJobId=EstimationJob_${TaskId}

	Log "Creating estimation job for task ${TaskId} in directory ${WorkDir}"

	cat >> $DAG_FILENAME << EOF
JOB $EstimationJobId $ESTIMATION_NODE_NAME
RETRY $EstimationJobId 10
VARS $EstimationJobId EXECDIR="${BIN}" WORKDIR="$WorkDir" TOTAL="$NumOfJobs" TASKID="$TaskId" ARGUMENTS="$ESTIMATION_PARAMETERS_FILE_NAME" PRIO="${estimation_job_priority}"
SCRIPT PRE $EstimationJobId $BIN/$PRE_ESTIMATION_SCRIPT_NAME $PRUNING_PARAMETERS_FILE_NAME 
SCRIPT POST $EstimationJobId $BIN/$POST_ESTIMATION_SCRIPT_NAME $WorkDir \$RETURN

PARENT $EstimationJobId CHILD $SLJobId
EOF

	[ -e ${DAG_FILENAME}.update ] && { 
		cat ${DAG_FILENAME}.update >>  $DAG_FILENAME
		rm -f  ${DAG_FILENAME}.update;
	}
}

# first thing we do - we wait for accept. Since otherwise we should return.
WaitForAccept || { Log "Accept returned failure -> can't continue"; 
				   exit ${retval}; 
				 }


# if BOINC_CHECKPOINT file exists - we already did all that. Go streight to BOINC node  

[ ! -z "$BOINC_CHECKPOINT" -a -e $BOINC_CHECKPOINT ] && {
	[ ! -z "$TASK_CHECKPOINT" ] && {
		Log "Detected intentional checkpoint state - bailing out"
		echo 0 > ${SECOND_ESTIMATE_RESULT_FILE}
		exit -1;
	}
	instantiate_boinc_node_template
	Log "Regenerating Boinc Params"
	[ -e $BOINC_PARAMETERS_FILE_NAME ] || Die "Failed to find $BOINC_PARAMETERS_FILE_NAME"
    oldPath=`head -1 $BOINC_PARAMETERS_FILE_NAME | cut -f4 -d" "`
    oldPath=`dirname $oldPath`
    oldPath=`dirname $oldPath`
    oldPath=`basename $oldPath`

    newPath=$TASKID

    cat $BOINC_PARAMETERS_FILE_NAME  | sed "s/\/$oldPath\//\/$newPath\//" >  ${BOINC_PARAMETERS_FILE_NAME}.tmp
	
	mv ${BOINC_PARAMETERS_FILE_NAME}.tmp ${BOINC_PARAMETERS_FILE_NAME}
	[ ! -s ${BOINC_PARAMETERS_FILE_NAME} ] && Die "Parameters file empty - cant restart"
	Log "Regenerating all submission files"
	for i in *.sub; do 
		cat $i | sed "s/\/$oldPath\//\/$newPath\//" > $i.tmp
		mv -f $i.tmp $i;
		[ ! -s ${i} ] && Die "Parameters file empty - cant restart"
	done
	
	Log "Regenerating all set ids"

	total_sets=`cat $BOINC_PARAMETERS_FILE_NAME | wc -l `
	[ -z "$total_sets" -o $total_sets -eq 0 ] && Die "Failed to read set ids";

	unique_set_id_base=$(update_maximal_set_id ${total_sets})
	[ $? -eq 0 ] || Die "Failed to regenerate set_ids while accessing DB"

	Log "Got set_id base: $unique_set_id_base"
	counter=0;
	touch ${BOINC_PARAMETERS_FILE_NAME}.new
	while read line; do
		set_id=`echo $line | cut -d' ' -f1`;
		route=`echo $line | cut -d' ' -f2-`;
		# now we move all the relevant files
		let new_set_id=$unique_set_id_base+$counter;
		Log "Mapping $set_id to $new_set_id";
		let counter++
		echo $new_set_id $route >> ${BOINC_PARAMETERS_FILE_NAME}.new
		
		[ -e  $ESTIMATION_PARAMETERS_FILE_NAME.$set_id ] || Die "Expected to see $ESTIMATION_PARAMETERS_FILE_NAME.$set_id file but noone found"
		
		mv $ESTIMATION_PARAMETERS_FILE_NAME.$set_id  $ESTIMATION_PARAMETERS_FILE_NAME.$new_set_id

	done < $BOINC_PARAMETERS_FILE_NAME
	
	mv $BOINC_PARAMETERS_FILE_NAME.new $BOINC_PARAMETERS_FILE_NAME

	Log "Exiting and going straight to BOINC node: $BOINC_CHECKPOINT file found";	
	rm $BOINC_CHECKPOINT;
	exit 0;
}

# first check that we may have backfill 
val=0
[ -e ${SECOND_ESTIMATE_RESULT_FILE} ] && val=`cat ${SECOND_ESTIMATE_RESULT_FILE}`

if [ ${val} -eq ${BACKFILL_JOB} ]; then 
	# we die here to be able to restore from this point in the backfill
	Log "BACKFILL job detected -> exiting with status ${BACKFILL_JOB}"
	exit ${BACKFILL_JOB}
fi

# if for some reason SECOND_ESTIMATE_RESULT_FILE contains error -> just exit
[ ${val} -ne 0 ] && exit ${val}

[ -e ESTIMATION_NODE_RESULT.txt ] && {
	Log "Estimation node failed - skipping"
	exit `cat ESTIMATION_NODE_RESULT.txt`;
}
# if we are here because of dead DAGMAN -> let's restart
[ -e ${DEAD_DAGMAN_FILE} ] && {
	Log "Dead DAGMAN BUG or inconsistency bug !!!! -> retrying"
	rm -f ParallelDag.dag.lock
	[ -e ParallelDag.dag.rescue ] && mv -f ParallelDag.dag.rescue ParallelDag.dag
	exit 0;
}
# we rerun DecideTaskComplexity here if we are restarted
# Check if the job already has been in the parallel state before migration
# the logic is to rerun this script in all cases of restart, except for the case
# when this task is checkpointed from another computer. 

[ -e TASK_STATE.txt ] && isParallelState=`grep -w ${PARALLEL_STATE} TASK_STATE.txt`
if [ -e $RESTART_JOB -a -z "${isParallelState}" ]; then
	${BIN}/DecideTaskComplexity.sh ||  {
		retval=$?
		echo $retval > ${SECOND_ESTIMATE_RESULT_FILE}
		exit $retval;
	}
fi

# let's load all the settings which were used in the latest DecideTaskComplexity
# this is important to be able to regenerate correct Dag, in case of checkpoint-restart
# this file contains the settings of all thresholds used in parallelization
# it overrides the local settings
[ -s ${CHECKPOINT_CONFIG_FILE} ] && source ${CHECKPOINT_CONFIG_FILE}


[ -e ${SPLIT_RESULT_FILE} -a -e $TASK_COMPLEXITY_MAP_FILE -a -e ${TASK_COMPLEXITY_MAP_FILE}.order ] || 
		DieNoLogin "Didn't find $TASK_COMPLEXITY_MAP_FILE,or its order file, or split file, which was supposed to be generated by SecondE_Final"

UpdateTaskState "$PARALLEL_STATE"

# this file is invoked after all SecondEstimation jobs are done. We have to decide where we stand with complexity
cwd=`pwd`
alldirs=`cat ${TASK_COMPLEXITY_MAP_FILE}.order`
[ -z "$alldirs" ] && DieNoLogin "This is a BUG - no DATA dirs found: $alldirs. I should have detected it much earlier!"

# this file is invoked after we know that Second estimation finished successfully
# here we decide the priority of a bunch of tasks, the number of parallelization for each job
##############################################################################
Log "Starting creation of Parallel jobs"

#start again ParallelDag.dag ( after we created it already in CreateInitDags, we want to start from clean )

# create Sub files for the DAG

cwd=`pwd`
sed_cwd=`pwd | sed 's/\//\\\\\\//g'`
logfile=$sed_cwd\\/ParallelDag.log 

cat $SUBFILES/Parallel_single.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > Parallel_single.sub
cat $SUBFILES/$ESTIMATION_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_WORKDIR_/$sed_cwd/" > $ESTIMATION_NODE_NAME
cat $SUBFILES/${POST_ESTIMATION_NODE_NAME}.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_WORKDIR_/$sed_cwd/" > ${POST_ESTIMATION_NODE_NAME}
#cat $SUBFILES/$BOINC_CONDOR_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_WORKDIR_/$sed_cwd/" > $BOINC_CONDOR_NODE_NAME
#cat $SUBFILES/$BOINC_CONDOR_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_ERROR_/error\.boinc_node/" > $BOINC_CONDOR_NODE_NAME
#cat $SUBFILES/$BOINC_CONDOR_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_OUTPUT_/output\.boinc_node/" > $BOINC_CONDOR_NODE_NAME
#cat $SUBFILES/$BOINC_CONDOR_NODE_NAME.tmpl | sed "s/_PEDTOOL_SUBSTITUTE_LOG_/log\.boinc_node/" > $BOINC_CONDOR_NODE_NAME

instantiate_boinc_node_template

cat > ParallelDag.dag <<EOF
#Dag for all parallel jobs
EOF



splits=( `cat $SPLIT_RESULT_FILE` )
Log "Task will be able to run - it fits the max number of jobs allowed per task"
counter=-1
TotalJobs=0
[ -e ParallelComplexity.sort ] && rm -f ParallelComplexity.sort
touch ParallelComplexity.sort

declare -a alldeps
declare -a allsplits

pruning_generated=0

# First counting the total number of jobs, as this information will later be used to decide, whether to split the task into more jobs and start a BOINC run,
# or to proceed as a local cluster run
max_num_jobs=0;
for directory in ${alldirs}
do
	let counter++
	[ -z "${splits[$counter]}" ] && continue 

	[[ ${splits[$counter]} -eq 0 ]] && let TotalJobs+=1
	[[ ${splits[$counter]} -ne 0 ]] && let TotalJobs+="${splits[$counter]}"
  	[ ${splits[$counter]} -gt $max_num_jobs ] && max_num_jobs=${splits[$counter]}
done
# End of counting the total number of jobs

#[[ "${TotalJobs}" -gt "${MAX_JOBS_PER_TASK}" ]] && Die "The total number of jobs is more than ${MAX_JOBS_PER_TASK}, unable to process a task that complex"
[ "${TotalJobs}" -lt ${BYPASS_PRUNING_LIMIT} -o $max_num_jobs -lt ${BYPASS_PRUNING_LIMIT_PER_JOB} ] && echo "TASK_BYPASS_PRUNING_ESTIMATION=1" >> $JOB_DETAILS_FILE 
source $JOB_DETAILS_FILE	

counter=0

[ -e $PRUNING_PARAMETERS_FILE_NAME ] && rm $PRUNING_PARAMETERS_FILE_NAME
[ -e $BOINC_PARAMETERS_FILE_NAME ] &&  rm $BOINC_PARAMETERS_FILE_NAME
total_grouped_jobs=0;

for i in $alldirs; do
	[ -z "${splits[$counter]}" ] && DieNoLogin "Split command produced garbage - not found output for given dir"
	if [ ${splits[$counter]} -eq 0 ]; then
		echo 0 > $i/${DEPTH_FILE}
		alldeps[$counter]=0
		allsplits[$counter]=1
		echo 1 > $i/${TOTAL_JOBS_FILE}
		new_priority=$TASK_PRIO_IN_CONDOR

		echo  $new_priority > $i/${PRIORITY_FILE}
		#let TotalJobs+=1
		let total_grouped_jobs++
		touch $i/${GROUPPED_JOB_MARKER}
	# we DON't create special node for this job, since we know that it will be groupped
	# enqueue that for groupping
		[ -e $i/ComplexityEstimate.txt ] || Die "Expecting $i/ComplexityEstimate.txt file"
		ParseComplexityEstimateFile $i/ComplexityEstimate.txt
		TABLESUM=$task_ts
		[ -z "$TABLESUM" ] && Die "Failed to find TABLESUM in $i/ComplexityEstimate.txt "
		echo ${TABLESUM} $i >> ParallelComplexity.sort
	else
		allsplits[$counter]=${splits[$counter]}
		#let TotalJobs+="${splits[$counter]}"
		echo ${splits[$counter]} > $i/${TOTAL_JOBS_FILE}
		[ -r ${i}/${COMPLEXITY_TABLE_FILE} ] || DieNoLogin "BUG: ${i}/${COMPLEXITY_TABLE_FILE} not found"
		depth=`cat ${i}/${COMPLEXITY_TABLE_FILE} | awk "{ if ( \\$2 == ${splits[$counter]} ) print \\$1 }" `
		[ -z "$depth" ] && DieNoLogin "BUG: ${i}/${COMPLEXITY_TABLE_FILE} doesn't contain ${splits[$counter]} jobs - very strange"
		echo $depth > $i/${DEPTH_FILE}
		alldeps[$counter]=$depth
		# The bypassing parameter TASK_BYPASS_PRUNING_ESTIMATION is defined inside ${JOB_DETAILS_FILE}, which is defined at the startup of every run and
		# is included in all the scripts that include 'JobUtils.sh'
		if [[ "x$BOINC_RUN" == "x1" && "x${TASK_BYPASS_PRUNING_ESTIMATION}" != "x1" && "${TotalJobs}" -gt "${LOCAL_CLUSTER_LIMIT}" ]]
		then
			createBoincDagNodes ${allsplits[$counter]} ${alldeps[$counter]} $counter $WORK_DIR/$TASKID/$i
			pruning_generated=1
		elif [[ "x$BOINC_RUN" == "x1" && "${TotalJobs}" -gt "${LOCAL_CLUSTER_LIMIT}" ]]
		then
			# Only creating parameters for the pruning node to later derive parameters for the BOINC node
			createPruningNodeParameters ${allsplits[$counter]} ${alldeps[$counter]} $counter $WORK_DIR/$TASKID/$i
		else
			# Occurs either when ${BOINC_RUN} == 0 or when ${TotalJobs} < ${LOCAL_CLUSTER_LIMIT}
			createDagNodes ${allsplits[$counter]} ${alldeps[$counter]} $counter $i	
			# we enforce non-boinc run by definition here
			echo BOINC_RUN=0 >> $JOB_DETAILS_FILE
		fi

###OPEN ISSUE-> how to manage priorities
	fi

	let counter++
done

alldirs_array=( ${alldirs} )

# check that there are jobs left
if [ ${TotalJobs} -ne  ${total_grouped_jobs} ]; then

	Log "${TotalJobs} note equal  ${total_grouped_jobs} - generating BOINC node"
	# Create BOINC node definition
	[[ "x$BOINC_RUN" == "x1"                        && "x$pruning_generated" == "x1"                  && \
	   "x${TASK_BYPASS_PRUNING_ESTIMATION}" != "x1" && "${TotalJobs}" -gt "${LOCAL_CLUSTER_LIMIT}" ]] && \
		createBoincNodeDefinition ${#alldirs_array[@]} $WORK_DIR/$TASKID 
	# When the bypassing feature is on, we create a shorter version of the parallel DAG
	[[  "x$BOINC_RUN" == "x1" && "x${pruning_generated}" != "x1"  && \
	   "${TotalJobs}" -gt "${LOCAL_CLUSTER_LIMIT}" ]] && createBoincNodeBypassDefinition ${#alldirs_array[@]} $WORK_DIR/$TASKID

fi
	# we necessarilly regenerate groupped nodes, since otherwise all the DAGs specifications
	# are incorrect (pathes are wrong, since absolute passes used. Note, that we do that with 
	# the thresholds settings, as specified in the CHECKPOINT_CONFIG_FILE, so that 
# the DAG created is consistent with the DAG created ( maybe ) before the checkpoint

if [ -s ParallelComplexity.sort ]; then
	touch GrouppedDag.dag
	# if this file is not empty -> we should do groupping. 
	# Group small jobs
	${BIN}/GroupJobsByThreshold.sh ParallelComplexity.sort linkage ${SINGLE_JOB_LINKAGE_THRESHOLD} GrouppedDag.dag ${PARALLEL_STATE} || Die "Failed to group jobs"
	# we want small jobs to be first
	cat ParallelDag.dag >> GrouppedDag.dag
	mv GrouppedDag.dag ParallelDag.dag
fi

#done with DAG creation. 
# we will try Merging of DAGs - this will work if a job was checkpointed
MergeRescueDag "ParallelDag.dag" 


# Lock on complexity lock to make sure that this Task is the ONLY one deciding now about
# the number of jobs, taking into account the number of jobs in the system.

Log "Deciding on number of jobs"
#wait in the loop till 
c="1"
count=0

while [ ! -z "$c" ]; do 
	touch ${JOB_COMPLEXITY_ESTIMATE_LOCK}
	c=""
	for i in `find ${WORK_DIR} -type d -maxdepth 1 -mindepth 1`; do
		id=`basename $i`
		[ "${id}" != "${TASKID}" -a -e ${i}/${JOB_COMPLEXITY_ESTIMATE_LOCK} ] && c=1
	done
# sleep random time
	[ ! -z "$c" ] && { 
						rm ${JOB_COMPLEXITY_ESTIMATE_LOCK};
					    r=$RANDOM; 
						let count++; 
						let tosleep="$r"%"$count"; 
						echo "this is the time we will sleep - $tosleep"
						sleep $tosleep; 
					 }
done

Log "Final evaluation of the load on the system"
#eval "${BIN}/${LOAD_ESTIMATION_SCRIPT}" || { Log "Load too high, can't continue" ; 
#											 UpdateTaskState ${ACCEPT_STATE}.${FAILED} ; 
#											 echo ${LOAD_TOO_HIGH} > ${SECOND_ESTIMATE_RESULT_FILE}; 
#											 rm ${JOB_COMPLEXITY_ESTIMATE_LOCK};
#											 exit $LOAD_TOO_HIGH;
#											}
#we put it into total jobs file
echo $TotalJobs > ${TOTAL_JOBS_FILE}
rm ${JOB_COMPLEXITY_ESTIMATE_LOCK}

isParalllelState=""
[ -e TASK_STATE.txt ] && isParallelState=`grep "${PARALLEL_STATE} TOTAL_JOBS" TASK_STATE.txt`

if [  -z  "$isParallelState" ]; then
# if user never got any update about this stage

	UpdateTaskState "$PARALLEL_STATE SPLIT=\"${allsplits[*]}\""
	UpdateTaskState "$PARALLEL_STATE SPLIT_DEPTH=\"${alldeps[*]}\""

	Log "Task is parallelized to $TotalJobs condor jobs"
	UpdateTaskState "$PARALLEL_STATE TOTAL_JOBS=$TotalJobs"

	let single_cpu_runtime="15*$TotalJobs"
	# let's see how many others are waiting in queue
	CountJobsAndTasksBeforeMe ${TASKID}

	# if we are the first in queue -> perfect
	if [ $tasks -eq 0 ]; then
		Log "Task is supposed to take $single_cpu_runtime minutes"
		ReportUpdateToUser "Update Task $TASK_PROJECT_NAME has been parallelized and enqued to be executed on thousands of computers. The computation would take about $single_cpu_runtime minutes on single computer. While we hope to use many computers, the exact number may vary depending on their availability. Please be patient."
	else
		# we get $jobs from CountJobsAndTasks...
		[ $jobs -ge 1 ]  && jobsInHours="some"
		[ $jobs -ge 100 ] && jobsInHours="quite many"
		[ $jobs -ge 500 ] && jobsInHours="a lot of"
		
		Log "We have $jobs in the queue before this task"
		ReportUpdateToUser "Update Task $TASK_PROJECT_NAME has been parallelized and enqued to be executed. However, there are $jobsInHours ($jobs) jobs in the queue, which have to be computed before your task. \
 Your computation would take about $single_cpu_runtime minutes on single computer. We hope to use many computers (up to 500), but the exact number may vary depending on their availability, with the average being about 30. Please be patient."
	fi
	
	UpdateTaskState "${PARALLEL_STATE}.${DONE}"
	UpdateTaskState "PARALLEL_COMPUTATION"
fi 

ReportWebUpdateToUser

#MARK that we can skip this stage if repeated by RETRY
touch ${DEAD_DAGMAN_FILE}

exit 0
