#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog
generateNopJob(){
	Log "Generating NOOP job"
	cp $SUBFILES/NOOP.sub ParallelOrdering.dag.condor.sub
}

[ -z "$SL_MAX_AVAILABLE_MEMORY_BOUND" ] && let SL_MAX_AVAILABLE_MEMORY_BOUND=$MAX_AVAILABLE_MEMORY/2
createDagNodes(){
	NumOfJobs=$1
	WorkDir=$2
	ParallelTaskId=$3
	TaskId=$TASKID
	# we create large DAG here
	[ -e $WorkDir ] || Die "$0 - Cannot find directory $WorkDir"

	#fill up  dag with jobs
	DagFilename=ParallelOrdering.dag
	SingleSubFilename=SecondE_singleJob.sub

	ii=0
	new_priority=$TASK_PRIO_IN_CONDOR
	while [ "x$ii" != "x$NumOfJobs" ]; do
		SLJobId=ParOrderJob_${ParallelTaskId}_${ii}
cat >> $DagFilename <<EOF
JOB $SLJobId $SingleSubFilename
RETRY $SLJobId 10
VARS $SLJobId EXECDIR="${BIN}" TOTAL="$NumOfJobs" IDX="$ii" WORKDIR="$WorkDir"  TASKID="$TaskId" UNIVERSE="Vanilla" MAXMEM="$SL_MAX_AVAILABLE_MEMORY_BOUND" PRIO="$new_priority"
EOF

let ii++
done

}

# this script checks if we need to do anything about ordering. 
# if no -> this job will become a "NOOP"
# if yes -> create a new DAG of nodes for every datadir

# first we should wait for ACCEPT, since we might be called after restart:
# in case we are started by Regular DAG - we'll just return immediately from this function
WaitForAccept || { Log "Accept returned failure -> can't continue"; 
				   echo ${retval} > ${PARALLEL_RESULT_FILE}; 
				   exit ${retval}; 
				 }

# first check that we may have backfill 
val=0
[ -e ${SECOND_ESTIMATE_RESULT_FILE} ] && val=`cat ${SECOND_ESTIMATE_RESULT_FILE}`

# this is the only value whihc could be found in SECOND_ESTIMATE_RESULT_FILE, since all
# other values are filtered by PostSecondEstimate.sh
if [ ${val} -eq ${BACKFILL_JOB} ]; then 
	# we die here to be able to restore from this point in the backfill
	Log "BACKFILL job detected -> exiting with status ${BACKFILL_JOB}"
	exit ${BACKFILL_JOB}
fi


### We attempt to do something in case 
# The job is special job: TASK_SPECIFIC_POINT AND each of the datapoints is above Configurable threshold

##11/08.05 - this part will never be invoked because of new mechanism for enforcing TASK_SPECIFIC_POINT 
# via generating of different GlobalDags for this case
#[ -z "$TASK_SPECIFIC_POINT" ] && {
#	touch ORDERING_NOOP
#	Log "Skipping Parallel Ordering stage"
#	generateNopJob
#	exit 0;
#}

# check for each dir that it's above the lowest threshold
alldirs=`FindAllDataDirs`

[ -z "$alldirs" ] && DieNoLogin "Failed to find any directory for further ordering calculation"

################### start creating of OrderingDag
Log "Generating ParallelOrdering.dag"
cwd=`pwd`
sed_cwd=`pwd | sed 's/\//\\\\\\//g'`
logfile=$sed_cwd\\/ParallelOrdering.log 

cat $SUBFILES/SecondE_singleJob.sub.tmpl | sed "s/_PEDTOOL_LOG_SUBSTITUTE_/$logfile/" > SecondE_singleJob.sub

cat > ParallelOrdering.dag <<EOF
#Dag for all ordering jobs
EOF



parallel_jobs=""
counter=0
for i in $alldirs; do
	[ -e $i/ComplexityEstimate.txt ] || Die "Expected $i/ComplexityEstimate.txt file - but not found"
	ParseComplexityEstimateFile $i/ComplexityEstimate.txt
	COMPLEXITY=$task_complexity
	
	let j="${#PARALLEL_ORDERING_THRESHOLDS[*]}"-1
	# for every threshold select the number appropriate number of jobs
	while [ $j -ge 0 ] ; do
		[ $COMPLEXITY -ge ${PARALLEL_ORDERING_THRESHOLDS[${j}]} ] && { 
			numOfJobs=${PARALLEL_ORDERING_SPLITS[${j}]}
			createDagNodes $numOfJobs $i $counter
			Log "Creating parallel order estimation for dir $i into $numOfJobs jobs"
			parallel_jobs="$i $numOfJobs $parallel_jobs"
			let counter++
			break;
		}
		let j--
	done
done


[ -z "$parallel_jobs" ] && {
	Log "No parallel jobs created - skipping"
	touch ORDERING_NOOP
	generateNopJob
	exit 0;
}

# Let's try the dagman to produce a dag
num_of_jobs=`grep -w JOB ParallelOrdering.dag | wc -l`

CondorSubmitDag "ParallelOrdering.dag" || DieNoLogin "Problem while checking the validity of ParallelOrdering.dag"

UpdateTaskState "${SECOND_ESTIMATE_STATE} PAR_ORDERING=$num_of_jobs"
echo $num_of_jobs > ${TOTAL_JOBS_FILE}
exit 0
