#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog

WaitForAccept || { Log "Accept returned failure -> can't continue";
                       echo ${retval} > ${FIRST_ESTIMATE_RESULT_FILE};
		   exit ${retval};
		   }
												       
# first we should see if the DAG is going our way:
[ -e ${FIRST_ESTIMATE_DECISION_FILE} ] || DieNoLogin "Expecting ${FIRST_ESTIMATE_DECISION_FILE} to appear - no good"

dec=`cat ${FIRST_ESTIMATE_DECISION_FILE}`
[ -z "$dec" ] && DieNoLogin "Failed to read ${FIRST_ESTIMATE_DECISION_FILE} correctly - no good"

# the following means that we don't continue with second estimate -> skip forward to LocalRun
[ $dec -ne ${FULL_PARALLEL_RUN} ] && { 
	Log "Detected that parallelization is not needed-exiting with $dec"
	exit $dec;
}
UpdateTaskState "${SECOND_ESTIMATE_STATE}"
# let's check again that we still can run the second stage
#eval "${BIN}/${LOAD_ESTIMATION_SCRIPT}" || { Log "Load too high, can't continue" ; 
#											 echo ${LOAD_TOO_HIGH} > ${FIRST_ESTIMATE_RESULT_FILE}; 
#											 exit $LOAD_TOO_HIGH;
#											}



# this file is invoked before second estimation is started. New Dags for second estimation
cwd=`pwd`
alldirs=`FindAllDataDirs`
# important -> we get here only directories which can be computed. All TOO-HEAVY are ignored, if exist
[ -z "$alldirs" ] && { 
	DieNoLogin "This is a BUG - no DATA dirs found: $alldirs. I should have detected it much earlier!";
}

Log "Regenerating SecondEstimate.dag to optimize low-complexity runs"
# start SecondEstimate.dag
cat > SecondEstimate.dag <<EOF
EOF

k=0

Universe=""
SecondEstimateVector=""
SchedulerUnivDirs=""
VanillaUnivDirs=""

EvalCPULoad
cpu_load=$CPU_local_load

[ -e InitComplexity.sort ] && rm -f InitComplexity.sort
touch InitComplexity.sort
dir_counter=0
for i in $alldirs; do

#let's copy what we need to have for SecondEstimate:

        cp -f $i/ComplexityEstimate0.txt $i/ComplexityEstimate1.txt || Die "Failed to copy complexity estimate"
        cp -f $i/ComplexityEstimate0.txt $i/ComplexityEstimate.txt || Die "Failed to copy complexity estimate file"
        cp -f $i/order0.txt $i/order1.txt || Die "Failed to copy order0.txt"
        cp -f $i/NumOfJobs0.txt $i/NumOfJobs1.txt || Die "Failed to copy NumOfJobs0.txt"


	#This is an optimization - don't submit condor Vanilla job unless it's above LOCAL_COMPLEXITY_ESTIMATION_THRESHOLD
	[ $cpu_load -gt ${MAX_CPU_LOAD_THRESHOLD} -o $dir_counter -gt 30 ] && Universe="Vanilla" || Universe="Scheduler"

	[ "$Universe" = "Scheduler" ] && let dir_counter++;

	[ -e $i/ComplexityEstimate.txt ] || Die "Expecting ComplexityEstimate file"

	ParseComplexityEstimateFile $i/ComplexityEstimate.txt
	COMPLEXITY=$task_complexity
	TABLESUM=$task_ts
	
	if [ ! -z "$COMPLEXITY" ]; then
		if IsNumber $COMPLEXITY ; then
				[ $COMPLEXITY -ge $LOCAL_COMPLEXITY_ESTIMATION_THRESHOLD ] && Universe="Vanilla"
		else 
			Die "Garbage in ComplexityEstimate in $i"
		fi
	fi
	
	# we remove it to NOT confuse in the next stages
	# rm -f  $i/ComplexityEstimate.txt > /dev/null 2>&1

	if [ "$Universe" = "Vanilla" ]; then
		SecondEstimateVector="${SecondEstimateVector} C"
		# we want to know if we will leave this job for grouping or not
		if [ $COMPLEXITY -ge ${SINGLE_JOB_COMPLEXITY_ESTIMATION_THRESHOLD} ]; then
			#This job is large enough - Fill up SecondEstimate.
			[ -z "$SL_MAX_AVAILABLE_MEMORY_BOUND" ] && let SL_MAX_AVAILABLE_MEMORY_BOUND=$MAX_AVAILABLE_MEMORY/2
			new_priority=$TASK_PRIO_IN_CONDOR
			
cat >> SecondEstimate.dag <<EOF
JOB SecondE$k SecondE_singleJob.sub
VARS SecondE$k TASKID="$TASKID" WORKDIR="$i" EXECDIR="$BIN"  JOBWORKDIR="$cwd" UNIVERSE="$Universe" MAXMEM="$SL_MAX_AVAILABLE_MEMORY_BOUND" IDX="0" PRIO="$new_priority"
RETRY SecondE$k 10
EOF
		else
			VanillaUnivDirs="$i ${VanillaUnivDirs}"
			echo $TABLESUM $i >> InitComplexity.sort
		fi
	else
			SchedulerUnivDirs="$i ${SchedulerUnivDirs}"
			SecondEstimateVector="${SecondEstimateVector} L"
	fi
let k++
done

if [ ! -z "${SchedulerUnivDirs}" ]; then
# create file with all directories to work on
file_with_dirs=SchedUniv_SecondEstimate.txt
echo "${SchedulerUnivDirs}" > $file_with_dirs || Die "Failed to create $file_with_dirs for Local run of second estimate"
#Fill up SecondEstimate.
[ -z "$SL_MAX_AVAILABLE_MEMORY_BOUND" ] && let SL_MAX_AVAILABLE_MEMORY_BOUND=$MAX_AVAILABLE_MEMORY/2
cat >> SecondEstimate.dag <<EOF
JOB SecondE_local SecondE_localJob.sub
VARS SecondE_local TASKID="$TASKID" WORKDIR="${file_with_dirs}" EXECDIR="$BIN" MAXMEM="$SL_MAX_AVAILABLE_MEMORY_BOUND" IDX="0"
EOF
fi	

if [ ! -z "${VanillaUnivDirs}" ]; then
	# let's try to group small jobs together
	[ -e InitComplexity.sort ] || Die "Expecting InitComplexity.sort here"
	# Group small jobs
	${BIN}/GroupJobsByThreshold.sh InitComplexity.sort order ${SINGLE_JOB_COMPLEXITY_ESTIMATION_THRESHOLD} SecondEstimate.dag ${SECOND_ESTIMATE_STATE} || Die "Failed to group jobs"
fi

# check how many jobs we generated
total_jobs=`grep -w JOB SecondEstimate.dag | wc -l`

Log "Done regeneration of SecondEstimate.dag"
Log "Starting SecondEstimate.dag"
UpdateTaskState "${SECOND_ESTIMATE_STATE} ORDER_ESTIMATE=$total_jobs"
ReportWebUpdateToUser
exit $SUCCESS
