#!/bin/bash

# For debugging purposes only
#set -x

# Script constants
PURGE_LOCKFILE=.purge
#ATTEMPTS=3
#SLEEP_INTERVAL=10

# Includes
# For $PURGE_DIR
#. "${PEDTOOL_ROOT}/CONFIG.SH"
# For $BOINC_ATTEMPTS
. "${PEDTOOL_ROOT}/BoincNodeConstants.sh"
# End of includes

declare -r PURGE_LOCKFILE
#declare -r ATTEMPTS
# End of script constants

# Script variables
declare -a purge_files=()
declare purge_lockfile_path="${PURGE_DIR}/${PURGE_LOCKFILE}"
# End of script variables

# Auxiliary functions
function compare_arrays()
{
	local -a first=($1)
	local -a second=($2)
	local -a sorted_first=( $(echo ${first[@]} | tr " " "\n" | sort -n) )
	local -a sorted_second=( $(echo ${second[@]} | tr " " "\n" | sort -n) )

	[[ "${sorted_first[@]}" == "${sorted_second[@]}" ]] && return 0
	return 1
}

function mark_drop_jobs_from_queue()
{
	local -a job_ids=($1)
	local constraint="$2"

	for job_id in ${job_ids[@]}
	do
		echo condor_qedit -constraint "${constraint}" LeaveJobInQueue false
		[[ $? != 0 ]] && return 1
	done

	return 0
}
# End of auxiliary functions

# Main

# Checking that this is the only running instance of purger
[ -f "${purge_lockfile_path}" ] && exit 1
touch "${purge_lockfile_path}"

# Ensuring that the purge directory is created
mkdir -p "${PURGE_DIR}"

# Passing through all the files to purge
purge_files=( $(ls "${PURGE_DIR}") )

for purge_file in ${purge_files[@]}
do
	purge_file_path="${PURGE_DIR}/${purge_file}"
	new_purge_file_path="${purge_file}.new"

	while read line
	do
		classad_attribute=$(echo ${line} | cut -d' ' -f1)
		classad_type=$(echo ${line} | cut -d' ' -f2 | awk '{print tolower($0)}')
		purge_ids=( $(echo ${line} | cut -d' ' -f3-) )
		failed_ids=()
		successful_ids=()

		[[ -z "${classad_type}" || -z "${classad_attribute}" || -z "${purge_ids[@]}" ]] && continue
		# Try to remove Condor jobs several times
		for attempts in `seq 1 5`
		do
			case "${classad_type}" in
				"string" )		constraint="stringListMember(${classad_attribute}, \"${purge_ids[@]}\", \" \")"; \
								#mark_drop_jobs_from_queue "${purge_ids[*]}" "${constraint}"; \
								condor_qedit -constraint "${constraint}" LeaveJobInQueue false
								[[ $? != 0 ]] && continue; \
								condor_rm -constraint "${constraint}"; \
								if [[ $? == 0 ]]; then successful_ids=( ${purge_ids[@]} ); else failed_ids=( ${purge_ids[@]} ); fi ;;
				"integer" )		for purge_id in ${purge_ids[@]}; do \
									constraint="${classad_attribute} == ${purge_id}"; \
									#mark_drop_jobs_from_queue "${purge_ids[*]}" "${constraint}"; \
									condor_qedit -constraint "${constraint}" LeaveJobInQueue false; \
									[[ $? != 0 ]] && continue; \
									condor_rm -constraint "${constraint}"; \
									if [[ $? == 0 ]]; then successful_ids=( ${successful_ids[@]} ${purge_id} ); else failed_ids=( ${failed_ids[@]} ${purge_id} ); fi \
								done;;
				"string_pair" )	first_attribute=$(echo "${classad_attribute}" | cut -d'+' -f1 ); \
								second_attribute=$(echo "${classad_attribute}" | cut -d'+' -f2 ); \
								for purge_index in $(seq 0 $(( ${#purge_ids[@]} / 2 - 1 )) ); do \
									first_id=${purge_ids[ $(( 2 * ${purge_index} )) ]}; \
									second_id=${purge_ids[ $(( 2 * ${purge_index} + 1 )) ]}; \
									first_constraint="${first_attribute} == \"${first_id}\""; \
									second_constraint="${second_attribute} == \"${second_id}\""; \
									condor_qedit -constraint "${first_constraint}" -constraint "${second_constraint}" LeaveJobInQueue false; \
									[[ $? != 0 ]] && continue; \
									condor_rm -constraint "${first_constraint}" -constraint "${second_constraint}"; \
									if [[ $? == 0 ]]; then successful_ids=( ${successful_ids[@]} ${first_id} ${second_id} ); \
									else failed_ids=( ${failed_ids[@]} ${first_id} ${second_id} ); fi \
								done;;
	        esac
			compare_arrays "${purge_ids[*]}" "${successful_ids[*]}"
			compare_arrays_return_value=$?

			[[ ${compare_arrays_return_value} == 0 ]] && break
			purge_ids=( ${failed_ids[@]} )
			#sleep ${BOINC_SLEEP_TIME}
		done

		# If all the attempts are not successful, do not delete the line in the file
		#[[ $attempts == ${BOINC_ATTEMPTS} && $compare_arrays_return_value != 0 ]] && echo "${classad_attribute} ${classad_type} ${purge_ids[@]}" >> "${new_purge_file_path}"
	done < "${purge_file_path}"

	# If all the tasks accomplished successfully, we get rid of the file, otherwise, we only leave the delta that we didn't manage to accomplish
	[ ! -f "${new_purge_file_path}" ] && rm -f "${purge_file_path}"
	[ -f "${new_purge_file_path}" ] && mv -f "${new_purge_file_path}" "${purge_file_path}"
done

# Releasing the lock
rm -f "${purge_lockfile_path}"
# End of main
