#!/bin/bash 
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
pid=$$
RedirOutToGlobalLog
#set -x
clear(){
	rm ${LOCK_FILE}  > /dev/null 2>&1
}
# create new job
# Parameter should be a non-fully qualified name: ex :asdf
createNewJob(){
	
	BIN="$DAG_HOME/bin"  
	SUBFILES="$DAG_HOME/subfiles";
	
	CleanJobSubmitDetails
	JobFile=$1
	Log "---------------Creating new task for task for $JobFile---------------"
# insert all vars from the DETAILS file
	SubmitJobDir=${QUEUE_DIR}/${JobFile}
	[ -e ${SubmitJobDir}/${JOB_DETAILS_FILE} ] || { 
		Error "Failed to read ${JOB_DETAILS_FILE}"; 
		CreateMailForAdmin "Failed to read  ${SubmitJobDir}/${JOB_DETAILS_FILE}"
		rm -rf ${SubmitJobDir}
		return; 
	}

	source ${SubmitJobDir}/$JOB_DETAILS_FILE || { 
			Error "Was unable to read $JobWorkDir/$JOB_DETAILS_FILE with the task parameters"; 
			CreateMailForAdmin "Was unable to read $JobWorkDir/$JOB_DETAILS_FILE with the task parameters";
			rm -rf ${SubmitJobDir}
			return;
	}
#check that all the parameters are present as we expect
# error reported from CheckJobSubmitDetails
# this should never happen, since we check it on every stage of submission
	
	CheckJobSubmitDetails || { 
		rm -rf ${SubmitJobDir}
		Error "The parameters specified during job submission are wrong"; 
		CreateMailForAdmin "The parameters specified during job submission are wrong";
		return;
	}

	jobnum=0
	[ -s ${WORK_DIR}/${LAST_HANLED_JOB_MARKER} ] && jobnum=`cat ${WORK_DIR}/${LAST_HANLED_JOB_MARKER}`
# update job num and remove the original file from the QUEUE
	let newjobnum="$jobnum"+1
	echo $newjobnum >  ${WORK_DIR}/${LAST_HANLED_JOB_MARKER}

	TASKID=$jobnum

	Log "Assigned TASKID $TASKID"
	[ -z "$jobnum" ] && DieNoLogin "Critical error: failed to detect jobnum!!!, System is halted"
# create new dir for a job
	JobWorkDir=${WORK_DIR}/${jobnum}
	createDir ${JobWorkDir} || DieNoLogin "Critical error: failed to create execution directory. System is halted"

# clean it up
# this can happen only if the task is being resubmitted due to System failure
	rm -rf ${JobWorkDir} > /dev/null 2>&1
	[ -e ${JobWorkDir} ] && {
#		rm -rf ${JobWorkDir}; rm -rf ${SubmitJobDir};
		DieNoLogin "Was unable to clean up ${WORK_DIR}/${jobnum}. System is halted";
	}

# copy all the files	
	mv -f  ${SubmitJobDir} $JobWorkDir || {
	 	rm -rf ${JobWorkDir}; rm -rf ${SubmitJobDir};
	 	DieNoLogin "Was unable to copy  ${QUEUE_DIR}/${JobFile} to $JobWorkDir";
	}

# create soft link to be able to issue job remove request 
	[ -e ${WORK_DIR}/$JobFile ] && {
		 ReportUpdateToUser "Failure The system failed to start your job due to internal error. Error is reported to the administrator. Please try again later";
		 CreateMailForAdmin "The job with such name already exists!! should have been caught earlier";
		 rm -rf  ${JobWorkDir};
		 return -1;
	}
	ln -sf ${JobWorkDir} ${WORK_DIR}/$JobFile || {
		 rm -rf ${JobWorkDir};
		DieNoLogin "Was unable to create softlink  ${WORK_DIR}/${jobnum} ${WORK_DIR}/$JobFile";
	}
	echo "$JobFile" > ${JobWorkDir}/${JOB_UID_FILE}

# we decide here what DAG to create according to the TASK_TYPE. 
# if NO TASK_TYPE defined -> it's regular two/multipoint SUPERLINK

	[ -z "$TASK_TYPE" ] && TASK_TYPE=LINKAGE
	CheckTypeSupported || {
		# unsupported type
		ReportUpdateToUser "Rejected The task type $TASK_TYPE is not supported"
		CreateMailForAdmin "Found task with unspported type -> $TASK_TYPE"
		#cleanup
		rm -rf ${JobWorkDir}
		return 1;
	}

	
	orig_cwd=`pwd`
	cd ${JobWorkDir} || {
#	           rm -rf ${JobWorkDir}
		DieNoLogin "Failed to chdir to ${WORK_DIR}/${jobnum} in order to submit dag";
	}
	# init the environment: create all the necessary files and $JOB_SUBMIT_DAG

	# we first create the common part for all types of tasks -> $JOB_SUBMIT_DAG

	[ "x$TASK_TWO_LOCI" = "x1" ] && {
		BIN="$TWO_LOCI_DAG_HOME/bin"  
		SUBFILES="$TWO_LOCI_DAG_HOME/subfiles";
	}

	[ -e $BIN ] || {
	    rm -rf ${JobWorkDir}
        DieNoLogin "Critical failure: $BIN does not exist";
	}

	sed_cwd=`pwd | sed 's/\//\\\\\\//g'`
	sed_execdir=`echo $BIN | sed 's/\//\\\\\\//g'`

	#create JOB SUBMIT DAG : Pedtool
	cat ${SUBFILES}/${JOB_SUBMIT_DAG}.tmpl | sed "s/_PEDTOOL_EXECDIR_/${sed_execdir}/" | sed "s/_PEDTOOL_WORKDIR_/${sed_cwd}/" > ${JOB_SUBMIT_DAG}
	
	# now create TYPE-specific part: Every type creates file GlobalDag.dag.condor.sub which
	# submits its own GlobalDag.dag 
	RunCommand "${BIN}/${TASK_TYPE}/InitSubmissionDir.sh ${JobWorkDir} ${TASKID}" || {
	           rm -rf ${JobWorkDir}
		DieNoLogin "Was unable to initialize submission dags";
	}

	# we have it all set, just submit
	CondorSubmitDag "$JOB_SUBMIT_DAG" || {
	           rm -rf ${JobWorkDir}
		DieNoLogin "Failed to execute condor_submit_dag ${WORK_DIR}/${jobnum}/$JOB_SUBMIT_DAG";
	}
	RunCommand "condor_submit ${JOB_SUBMIT_DAG}.condor.sub" || {
	           rm -rf ${JobWorkDir}
		DieNoLogin "Failed to execut condor_submit  ${JOB_SUBMIT_DAG}.condor.sub";
	}

	cd $orig_cwd

	Log "Submission successfull"

	return 0
}



# we want to ensure no REENTRANCE here 
# check lock
# first touch the lock of PID

ValidateLockfile $LOCK_FILE && exit

pid=$$
echo $pid > ${LOCK_FILE} || DieNoLogin "Cannot touch ${LOCK_FILE}"
touch  ${LOCK_FILE} || DieNoLogin "Cannot touch ${LOCK_FILE}"

trap clear EXIT 

# THIS is standard setup operation in case something happened to these dirs 
createDir $QUEUE_DIR || DieNoLogin "Failed to create $QUEUE_DIR"
createDir $WORK_DIR  || DieNoLogin "Failed to create $WORK_DIR"
createDir $MAIL_DIR || DieNoLogin "Failed to create $MAIL_DIR"
createDir $DONE_DIR || DieNoLogin "Failed to create $DONE_DIR"
createDir $LOG_DIR || DieNoLogin "Failed to create $LOG_DIR"
createDir $ARCHIVE_DIR || DieNoLogin "Failed to create $ARCHIVE_DIR"

TestNoLogin && { Log "NO_LOGIN detected, exiting"; exit; }

cd ${QUEUE_DIR} || DieNoLogin "Failed to chdir ${QUEUE_DIR}"

sleep_counter=0
# we want QueueHandler to exit every 10 mins
cpu_load_counter=0
Log "New queue handler started $pid"
all_queue_handlers=`ps auxwww | grep QueueHandler.sh | grep 50505 | grep -v PEDTOOL | grep -v grep`
Log "All handlers $all_queue_handlers"
while [ $sleep_counter -lt 600 ]; do
	cd ${QUEUE_DIR}

	[ -e NO_LOGIN ] && exit
	
	# if nothing - exit
	jobs=`ls -I NO_LOGIN`

	[ -z "$jobs" ] && {
		sleep ${QUEUE_HANDLER_RATE}
		let sleep_counter+=${QUEUE_HANDLER_RATE}
		continue;
	}
	
	EvalCPULoad 
	cpu_load=$CPU_local_load
	
	[ -z "$cpu_load" ] && {
		Error "Failed to evaluate CPU load on this machine, assuming 100"
		cpu_load=100;
	}
	
	sleepon_cpu_load="false"

	if [ $cpu_load -gt $QUEUE_HANDLER_MAX_CPU_LOAD ]; then
		Log "Computer overload detected - $cpu_load -gt $QUEUE_HANDLER_MAX_CPU_LOAD - queue handler exiting"
		# we just exit
		exit
		########################################
		#sleepon_cpu_load=""
		#let cpu_load_counter++
		# we make submission 10 times slower
		#[ $cpu_load_counter -gt 10 ] && {
	#		Log "CPU load is too high -> $cpu_load , QueueHandler decreased submission rate 10 times";
#			cpu_load_counter=0
#			sleepon_cpu_load="false";
#		};
	else 
		cpu_load_counter=0;
	fi
		
	[ -z "$jobs" -o -z "$sleepon_cpu_load" ] && {
		sleep ${QUEUE_HANDLER_RATE}
		let sleep_counter+=${QUEUE_HANDLER_RATE}
		continue;
	}
	#create and submit
	for job in $jobs; do
		cd ${QUEUE_DIR}
	# if the submission is not ready -> just skip it, it will become ready after complete transfer of data
		[ ! -e $job/${JOB_DETAILS_FILE} ] && continue
		[ -e $job/NOT_READY ] && continue
		[ -e $job/ERROR ] && continue
		createNewJob "$job" || Warn "Problem with creating task for file $job" 
		# we submit one job at a time, each 5 secs
		condor_reschedule
		sleep 5
		condor_reschedule
		sleep 5
		condor_reschedule
		sleep 10
		condor_reschedule
		break;
	done
done
