#!/bin/bash
basepath=`dirname $0`
. ${basepath}/CONFIG.SH
. ${basepath}/Utils.sh
exec 3>> ${LOG_DIR}/NotificationLog
exec 1>&3
exec 2>&3
#set -x
		      
# Thi script will handle updates and send them to the portal
# there are 3 types of updates: 
#  status.* -> status updates of the job, created by ReportWebUpdateToUser
#  mail.* -> mail updates, created by CreateMailForUser
#  any other file which will be transferred to user

# It looks up the MAIL_DIR, and if it finds there something, 
# Check if ADMIN dir is empty and first send stuff from it if there is
# For each dir other than ADMIN
 # + check that there is no lock in it or sleep for 1 sec to check again. If still locked after 3 attempts - rm lock, and send mail
 # + contact server and script specified in "Details.txt" of that dir and do transmit each update. rm file sent. 
 # + check lock and if exists -> get to next dir. Otherwise rm 
 # + if LAST present -> remove DIR forcefully

pid=$$
lockfile=/tmp/MAIL_SENDER_LOCK.${CONDOR_GW_USER}.${CONDOR_GW}

ValidateLockfile $lockfile && exit
echo "+++++++++++++++++++++++++Starting new notification+++++++++++++++++++++++++++"
clean(){
	rm ${lockfile}  > /dev/null 2>&1
}

trap clean EXIT



cd ${MAIL_DIR}
if [ -e ${MAIL_DIR}/ADMIN/mail ]; then
	echo "Sending mail to admin"
	cat ${MAIL_DIR}/ADMIN/mail | ${ADMIN_MAIL_SENDING_SCRIPT} && {
		rm -f ${MAIL_DIR}/ADMIN/mail ;
	}
	[ -e ${MAIL_DIR}/ADMIN/mail ] && Log "failed to send mail to ADMIN - very bad"
fi
dirs=`ls -I ADMIN`
[ -z "$dirs" ] && exit

for i in $dirs; do
	counter=1
	failure=0
	while [ -e ${i}/lock -a $counter -lt 4 ]; do
		sleep $counter
		let counter++
 	done
# self heal if lock is stuck there for some reason
	rmdir ${i}/lock > /dev/null 2>&1
	
	[ -e ${i}/${JOB_DETAILS_FILE} ] || { Log "Can't send mail for ${i} - ${JOB_DETAILS_FILE} missing"; rm -rf ${i}; continue; }

	unset TASK_PORTAL_RUNDIR
	unset TASK_PORTAL_HOST

	source ${i}/${JOB_DETAILS_FILE}
# let's check if this is the last result
	send_final_mail=""
	remove_dir=""
	[ -e ${i}/LAST -a -e ${i}/SUCCESS ] && send_final_mail="yes"
	[ -e ${i}/LAST ] && remove_dir="yes"

	[ -z "${i}/${TASK_PORTAL_RUNDIR}" -o -z "${i}/${TASK_PORTAL_HOST}" ] && { 
		rm -rf ${i};
		Log "Can't send mail for ${i} - details are missing in ${JOB_DETAILS_FILE}"; continue; 
	}

########## we start sending results first - it will be referenced in all mails	
	result_files=`ls $i -I status -I status.* -I mail -I mail.* -I ${JOB_DETAILS_FILE} -I LAST -I SUCCESS | tr '\n' ' '`
	if [ ! -z "${result_files}" ]; then 
		cd $i;
		echo "Found result files for $i - ${result_files}"
		$scpcmd ${result_files} ${PORTAL_USER}@${TASK_PORTAL_HOST}:${TASK_PORTAL_RUNDIR} 
		ret=$?
		if [ $ret -eq 0 ]; then
			rm -f ${result_files}
		else
			Log "Failed to move files ${result_files} from "$USER" at "$(pwd)" to ${PORTAL_USER}@${TASK_PORTAL_HOST}:${TASK_PORTAL_RUNDIR}   - will try again later"
			failure=1
		fi
		cd -;
	fi

########## now send status update, it will have all the required info on the job status
	
	status_file=status
	
#if last -> send last results, else - send update
	if [ -e ${i}/${status_file} -a $failure -ne 1 ]; then
		echo "Found status file for $i"
		cat  ${i}/${status_file} | ${sshcmd} ${PORTAL_USER}@${TASK_PORTAL_HOST} ${TASK_PORTAL_RUNDIR}/${REMOTE_STATUS_UPDATER_SCRIPT}
		ret=$?
		# if $ret==126 -> no such file or dir at the remote side => something bad happened, but updates will never arrive
		[ $ret -eq 126 -o $ret -eq 127 ] && touch ${i}/FORSE
		if [ $ret -eq 0 ]; then
			rm -f ${i}/${status_file}
		else
			Log "Failed to update a user - will try again later"
			failure=1
		fi
	fi

####################and now send mail ##########
	if [ ! -z "$send_final_mail" ]; then
		remote_mail_flag=2
		status_file=SUCCESS
	else
		remote_mail_flag=1
		status_file=mail
	fi
	if [ -e ${i}/${status_file} -a $failure -eq 0 ]; then
		echo "found mail file"
		cat  ${i}/${status_file} | ${sshcmd} ${PORTAL_USER}@${TASK_PORTAL_HOST} ${TASK_PORTAL_RUNDIR}/${REMOTE_MAIL_SENDING_SCRIPT} $remote_mail_flag
		ret=$?
		# if $ret==126 -> no such file or dir at the remote side => something bad happened, but updates will never arrive
		[ $ret -eq 126 -o $ret -eq 127 ] && touch ${i}/FORSE
		if [ $ret -eq 0 ]; then
			rm -f ${i}/${status_file}
		else
			Log "Failed to send user a mail - will try again later"
			failure=1
		fi
	fi

# we can safely remove the dir if all sends are successfull and 
	[ -e ${i}/FORSE ] && rm -rf ${i}
	[ $failure -eq 0 -a ! -z "$remove_dir" ] && rm -rf ${i} 

done

