#!/bin/bash
#this script handles any failure/success of single Pedtool taks
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog

[ $# -ne 3 ] && {
	echo "Missing parameter - <username@server> <Path to askpass> <Remote AcceptSubmission script path>"
	exit -1;
}
account=$1
path_to_askpass=$2
remote_accept=$3

# first - remove all state files so we are sure don't mess up in the next submission
for i in ${STATE_FILES[*]}; do
	rm -f $i
done
# second - remove files which are used for condor_rm
rm -f ${GLOBALDAG_CLUSTERID_FILE}
# remove timeout file
rm -f ${TIMEOUT_MARKER_FILE}
# remove DEAD_DAGMAN_FILE 
rm -f ${DEAD_DAGMAN_FILE}
# remove MIGRATION_MARKER_FILE
rm -f ${MIGRATION_MARKER_FILE}
# now we touch to mark that we are to be restarted:
touch ${RESTART_JOB}
Log "Preprocessing rescue dags for BACKFILL jobs"

# regenerate GlobalDag.dag.rescue to have Accept job NOT done, despite that it probably is DONE by now
[ -e GlobalDag.dag.rescue ] && {
	cat  GlobalDag.dag.rescue | awk '{ if ($0~"JOB Accept Accept.sub" ){ print "JOB Accept Accept.sub" } else { print $0} }' > GlobalDag.dag.tmp;
	mv -f GlobalDag.dag.tmp GlobalDag.dag.rescue;
}


# get the job's name:
[ -e ${JOB_UID_FILE} ] || DieNoLogin "Failed to find ${JOB_UID_FILE}. Something went wrong"

job_uid=`cat ${JOB_UID_FILE}`

# and finally start ssh to the next submit machine
Log "Moving job to ${account}"
set -x
count=0
while [ 1 -eq 1 ]; do
	let count++
	success=1
	tar --exclude ParallelDag.dag.dagman.out --exclude ParallelDag.log -z -c -f resubmit.tar.gz * && break; 
	success=0
	Log "Attempt number $count for tarring failed"
	[ $count -gt 10 ] && break
	sleep 60
	Log "trying again to tar"
	rm -f resubmit.tar.gz
done
[ $success -eq 0 ] && {
		Log "Problem with tarring of the directory content" 
		exit -1;
}
cat resubmit.tar.gz | ${BIN}/Passwordless_ssh.sh ${account} ${path_to_askpass} ${remote_accept} ${job_uid} || {
	rm -f resubmit.tar.gz
	Log " Failed to move a task to backfill";
	exit -1;
}
rm -f resubmit.tar.gz
