#!/bin/bash

# this script can run remotely -> we can't use Utils and CONFIG.SH
# very simple script to try and run everything in one single job. 

set -x
die(){
	echo "$1"
	exit -1;
}

die_to_retry(){
	echo "$1";
	exit 1;
}


# first we wanna detect if we are running locally, or we are moved as a condor job
[ -z "$1" -o -z "$2" ] && die "Missing parameter - <full path to directory where all executables reside> <universe>"
execdir=$1
universe=$2
datadirs=(` find  -maxdepth 2 -mindepth 2 -type d  -name 'pedfile*'`)

source Details.txt || die "Failed to find Details.txt"
run_location="";
if [ "0" = "${#datadirs[*]}" ]; then
# we are running remotely - so we should do checks and rerun creation of single jobs
	if [ -e job_archive.tar ]; then
		tar xfv job_archive.tar || {
			tar --help || die_to_retry "Problem with running tar on this computer..."
			die "Failed to open up the job_archive.tar";
		}
	else
		die "Failed to find job_archive.tar"

	fi

	[ -e Details.txt -a -e datafile.dat -a -e pedfile.dat -a -e Superlink_Vanilla ] || { die " `ls -l` Some files are not present, can't start"; }
	run_location="REMOTE";
	chmod +x Superlink_Vanilla || die "Can't chmod for executables"

	SUPERLINK="./Superlink_Vanilla"
	[ -x ${SUPERLINK} ] || die "Failed to find/execute ${SUPERLINK}"
	SUPERLINK="../../Superlink_Vanilla"
else
	run_location="LOCAL";
	SUPERLINK="$execdir/Superlink/Superlink_Vanilla"
	[ -x ${SUPERLINK} ] || die "Failed to find/execute ${SUPERLINK}"

fi

datadirs=(` find  -maxdepth 2 -mindepth 2 -type d  -name 'pedfile*'`)

[ "0" = "${#datadirs[*]}" ] && die "No data directories found"

rm to_pack_back.txt
touch to_pack_back.txt
for i in ${datadirs[*]}; do
	echo "$i" >> to_pack_back.txt
	cd $i || die "Cannot cd to $i"
	echo "Running superlink in $i"
	echo "1" > TotalJobs.txt
	${SUPERLINK} || die "failed to Run first run of superlink"
	${SUPERLINK} datafile.dat pedfile.dat -e order0.txt 0 0 > output.0 || die "failed to run second run of superlink"
	cd - > /dev/null
done

# if we're running remotely -> let's try and tar all the results. 
if [ "$run_location" != "LOCAL" ]; then
	tar cvf SIMPLE_JOB_RUN_RESULTS.tar -T to_pack_back.txt  || die_to_retry "Failed to create the back-result archive"
fi

echo "Success"
exit 0
