#!/bin/bash
basepath=`dirname $0`
. ${basepath}/JobUtils.sh
RedirOutToTaskLog
#set -x
# this file is invoked from within the job directory to monitor Job execution
done_counter=0
progress_counter=0

CreateResultFile(){
	IsParallel=`grep ${PARALLEL_STATE} ${TASK_STATE_FILE}`
	[  -z "${IsParallel}" ] && return
	[ -x ${BIN}/${GATHER_RESULT_SCRIPT} ] || DieNoLogin "Failed to find ${GATHER_RESULT_SCRIPT}"

	${BIN}/${GATHER_RESULT_SCRIPT}  ||  Error "Failed to run ${GATHER_RESULT_SCRIPT} "
}


DoMonitoring(){
	IsParallel=`grep ${PARALLEL_STATE} ${TASK_STATE_FILE}`
	[  -z "${IsParallel}" ] && return

	alldirs=`FindAllDataDirs`
	[ -z "${alldirs}" ] && return -1
	
# if ${TOTAL_JOBS_FILE} still doesn't exists -> we shouldn't bother 
	[ -e ${TOTAL_JOBS_FILE} ] || return
	
	total_jobs=`cat ${TOTAL_JOBS_FILE}`
	total_excluded_jobs=0
    pid=$$
	[ -s $EXCLUDED_NUM_JOBS_FILE ] && mv $EXCLUDED_NUM_JOBS_FILE $EXCLUDED_NUM_JOBS_FILE.$pid
	[ -e $EXCLUDED_NUM_JOBS_FILE.$pid ] &&  total_excluded_jobs=`cat $EXCLUDED_NUM_JOBS_FILE.$pid`;
	rm -f $EXCLUDED_NUM_JOBS_FILE.$pid
	[ -z "$total_excluded_jobs" ] && total_excluded_jobs=0
	let total_jobs="$total_jobs"-"$total_excluded_jobs"
	echo $total_jobs > $TOTAL_JOBS_FILE

	[ -z "$total_jobs" ] && return
	
	[ $total_jobs -eq 0 ] && return

# let's make it a bit less frequent
	[ $total_jobs -gt 300 ] && MONITOR_PERIOD=600
	
# we should be able to run it here, since all datafile* dirs are present
	
	EvalCPULoad
	[ $CPU_local_load -lt 400 ] && CreateResultFile

# now start monitoring
	total_done=0
	#[ -e ${TOTAL_DONE_JOBS_FILE} ] && total_done=`cat ${TOTAL_DONE_JOBS_FILE}`
	
	done_jobs=("")
	mycounter=-1
	for k in ${alldirs}; do
		let mycounter++
		total_jobs_file="${k}/${TOTAL_JOBS_FILE}"
		total_done_jobs_file="${k}/${TOTAL_DONE_JOBS_FILE}"
		gather_total_jobs_file="${k}/${GATHER_TOTAL_JOBS_FILE}"
		# this datafile/pedfile pair is already complete
	#	[ -e ${k}/${TASK_COMPLETE_FILE} ] && continue;
		
		# check all output files : this takes into account that the list can be very long
		output=0
		#ready_output=`ls ${k}/${OUTPUT_PREFIX}.*`
		#if [ "x$BOINC_RUN" == "x0" ]; then 
		files=`ls ${k}/${OUTPUT_PREFIX}.* | grep -v node`
		[ -z "$files" ] || 	output=`echo $files | xargs grep  \\\\$\\\\$\\\\$ | wc -l` ;
		#fi
		
		[ -e "${total_done_jobs_file}" -a -s "${total_done_jobs_file}" ] && output=`cat "${total_done_jobs_file}"` 
		[ -z "$output" ] && output=0 
		
		[ $output -eq 0 ] && continue;
		
		# detect completion
		[ -e "${total_jobs_file}" ] && {
			total_estimate=`cat "${total_jobs_file}"`
			[  ! -z "${total_estimate}" -a ${total_estimate} -eq $output ] && {
				touch ${k}/${TASK_COMPLETE_FILE};
			};
		}
		[[ ! ( -e "${gather_total_jobs_file}" ) ]] && done_jobs[$mycounter]=$output 
		# If the GatherTotalJobs.txt exists, TotalJobs.txt exists too, so we do not check its existence
		[[ -e "${gather_total_jobs_file}" ]] && done_jobs[$mycounter]=$(( $(cat "${gather_total_jobs_file}") - $(cat "${total_jobs_file}") + ${output} ))

		let total_done+="${done_jobs[$mycounter]}"
	done

# we get to status file update
	tmp_numjobs=0
    tmp_numjobs=`cat ${TOTAL_DONE_JOBS_FILE}`
# lets check the progress
	[ $total_jobs -gt 0 ] && {
		let total_left="$total_jobs-$total_done"
#		let ratio=$total_left*5
#		if [ ${total_done} -ne $tmp_numjobs -o $ratio -gt $total_jobs ]; then
		if [ ${total_done} -ne $tmp_numjobs -o $progress_counter -eq 0 ]; then
			  progress_counter=`date +%s`
		else
			Log "NO PROGRESS  progress counter: $progress_counter: old $tmp_numjobs new: $total_jobs"
		fi;
	}


	echo ${total_done} > ${TOTAL_DONE_JOBS_FILE}
	[ -e $CUR_NUM_RUNNING_JOBS -a ${total_jobs} -gt 0 ] && {
		# compute total_left
		let total_left="$total_jobs-$total_done"
		cur_num=`cat $CUR_NUM_RUNNING_JOBS`
		echo $total_left $cur_num >> AVERAGE_RESOURCES_STAT;
	}
	# let's check average parallelism
	[ -e ParallelDag.dag.dagman.out -a "x$BOINC_RUN" = "x0" ] && {
		let dag_counter++
		[ $dag_counter -gt 3 ]  && {
			dag_counter=0
			${BIN}/ComputeAverageParallelism.sh ./ParallelDag.dag.dagman.out;
		}
	}

	[ ${total_done} -eq ${total_jobs} ] && {
		let done_counter++
		Log "Task has been completed, but Condor doesn't know about it... $done_counter";
	}
	
	[ ${total_done} -eq 0 ] && { 
		# let's report it only once
		[ -z "$null_report" ] && { 
			null_report="DONE"; ReportWebUpdateToUser ; return 0; 
		};
	}

	percent=`echo | awk "{ print 100*($total_done/$total_jobs) }"`
	# we don't want to report things twice
	[ "$prev_done" != "$total_done" ] && { 
		Log "${total_done} jobs done out of ${total_jobs}"
		ReportWebUpdateToUser
		expected_output=`basename $TASK_OUTPUT_FILE`
		UpdateFileForUser ${expected_output};
		UpdateFileForUser $FRONTEND_ARCHIVE_FILE;
		prev_done=$total_done;
	}
}


CheckStateFiles(){
	status=0
		[ -e $BOINC_CHECKPOINT ] && {
			Log "Bailing out due to checkpoint request"
			exit -1;
		}
	for i in ${STATE_FILES[*]}; do
		[ "${i}" = "${TASK_STATE_FILE}" ] && continue
		if [ "${i}" = "${TASK_COMPLETE_FILE}" ]; then
			[ -e ${i} ] && { 
				CreateResultFile
				[ -e ${TOTAL_JOBS_FILE} -a  -e ${TOTAL_DONE_JOBS_FILE} ] && { 
					DoMonitoring
					total_jobs=`cat ${TOTAL_JOBS_FILE}`
					total_done=`cat ${TOTAL_DONE_JOBS_FILE}`
					[ $total_jobs != $total_done ] && {
			CreateMailForAdmin "Found inconsistency: task complete, but there are still empty output files!!!";
			Die "Found inconsistency: task compelte, but there are still empty output files!!!!!!!!!!!!!!!!";
		};
	}

				Log "${TASK_COMPLETE_FILE} detected - Monitoring job is exiting with status 0"; exit 0; 
			}
		fi
		[ -e ${i} ] && status=`cat ${i}`
	done
	[ "x$status" != "x0" ] && { Log "Monitoring job is exiting with status -1 due to the status ${status}"; exit -1; }
}

clean(){
	echo "$now" > TOTAL_COMPUTATION_TIME.txt
}

trap clean EXIT


# here are the monitoring stuff we should do:
# 0) condor_reschedule
# 1) check whether task is still alive: all state files should contain non-zero
# 2) +if we are running PARALLEL_STATE check all data dirs to find the total number of completed files. 
#    +update result file
# 3) check Global CONDOR_Q querrier for jobs for this job
# 4) check that total running time is not more than X

RESCHEDULE_INC=60
RESCHEDULE_PERIOD=30
# how many times to ignore that the task has been completed?
TIME_SKIP_COMPLETION_DETECTED=40
[ ${RESCHEDULE_PERIOD} -gt ${MONITOR_PERIOD} ] && MONITOR_PERIOD=${RESCHEDULE_PERIOD}
sleep_counter=0
cached_result_file=0
# get time in secs
begin=`date +%s`
[ -s  TOTAL_COMPUTATION_TIME.txt ] && {
	in_system=`cat  TOTAL_COMPUTATION_TIME.txt`
	[ -z "$in_system" ] || {
		IsNumber $in_system && let begin-="$in_system";
	};
}

resched_counter=1
resched_sleep_counter=0
# let's check state files every second during first minute
counter=0;
now=0
t=0
while [ $counter -lt 60 ]; do
	[ $now -lt 30 ] && condor_reschedule
	CheckStateFiles
	sleep 1
	let counter++
	let now++
done	
while [ 1 -eq 1 ]; do

	sleep 15
	let sleep_counter+=15
	let resched_sleep_counter+=15
	
	# this method handles all exits inside
	CheckStateFiles

	# check how much time the TASK is running
	now=`date +%s`
	
	# make sure that progress is made at reasonable rate
	let progress_timer=0;
	[ $progress_counter -gt 0 ] && let progress_timer="$now"-"$progress_counter"
	
	[ $progress_timer -gt $MAX_ALLOWED_TASK_NO_PROGRESS ] && touch ${TIMEOUT_MARKER_FILE}
	[ -e ${TIMEOUT_MARKER_FILE} ] && Log "Marked task too slow progress"	

	let now-="$begin"
	if [ ! -z "$now" -o -e ${TIMEOUT_MARKER_FILE} ]; then
		if [ $now -gt ${MAX_ALLOWED_TASK_DURATION} -o $done_counter -gt $TIME_SKIP_COMPLETION_DETECTED -o -s $MIGRATION_MARKER_FILE -o -e ${TIMEOUT_MARKER_FILE} ]; then
			Log "Migration/timeout/complete marker found - removing everything"
			# we should condor_rm everything
			cluster=`cat ${GLOBALDAG_CLUSTERID_FILE}`
			# this condor_rm removes this job as well, so it's like suicide
			# if it failes - it will be repeated in 15 secs
			[ $now -gt ${MAX_ALLOWED_TASK_DURATION} ] && touch ${TIMEOUT_MARKER_FILE}
			counter=0
			while [ $counter -lt 10 ]; do
				[ -z "$cluster" ] && break;
				condor_rm $cluster && break
				let counter++
				sleep 5
			done
			
			[ $done_counter -ge 1 -a $counter -ne 100 ] && touch ${TASK_COMPLETE_FILE}
		fi
	fi
	# we want reschedule to stop sending so frequently and reduce it every 1 min twice
	let cur_resched="120"*"$resched_counter"
	[ $cur_resched -lt $now ] && {
		let resched_counter++;
		let RESCHEDULE_PERIOD+="${RESCHEDULE_INC}";
#		Log "Decreasing RESCHEDULE_PERIOD to $RESCHEDULE_PERIOD"
	}
	# run resched either every reschedule_period, or every 5 sec when we only start
	
	[ $resched_sleep_counter -ge ${RESCHEDULE_PERIOD} -o $now -lt 60 ] && { 
		condor_reschedule
		resched_sleep_counter=0;
	}
	
	[ $sleep_counter -ge ${MONITOR_PERIOD} ] && { 
		sleep_counter=0; 
		DoMonitoring ; 
		echo "$now" > TOTAL_COMPUTATION_TIME.txt;
	}

done
