/* $Id: EliminationOrder.c,v 1.2 2008/08/03 08:15:45 superlink Exp $ */
/*********************************************************************************/
/*  EliminationOrder.c												     	     */
/*  Author: Ma'ayan Fishelson					                                 */
/*********************************************************************************/

#include "EliminationOrder.h"
#include "Globals.h"
#include <errno.h>


#ifdef PARALLEL

int perm(int i, long permuteId,int* max_array, int* res_array, int arr_size){
        static long counter=0;
        for ( res_array[i]=0;res_array[i]<max_array[i];res_array[i]++){
                if ( i<arr_size-1){
                        if (perm(i+1,permuteId,max_array,res_array,arr_size)){
                                return 1;
                        }
                }else{
                        counter++;
                        if (counter>permuteId){
                                counter=0;
                                return 1;
                        }
                }
        }
        return 0;
}

/* Function to find the permutation of the conditioning assignments 
   according to the serial number of the permutation. For istance, if we have
   4 conditioned vars, each with 2 values, and we want permutation number 3, 
   this function retunrs resultPermutation array with [0,0,1,1]
   parameters:
   permuteId - serial number of the permutation
   maxValues - array of number of values for each variable in the permutation
   resultPermutation - array of assignment
   arraySize - number of values in the array
   returns -1 if something went wrong
*/
int permutation(long permuteId, int* maxValues, int* resultPermutation, int arraySize){
  if (arraySize==0) return 0;
  return -1+perm(0,permuteId,maxValues,resultPermutation,arraySize);
}

void printProduct_Mark(elimOrderProduct* prod){

        FILE* out;
        elimOrderFunc* func,*funcEnd;
        elimOrderVar** var,**varEnd;


        out=fopen("Network.txt","w");
        for (func = prod->functions, funcEnd= func + prod->numFunctions; func<funcEnd; func++) {
          if (func->used) {
                fprintf(out,"(");
                for ( var=func->variables,varEnd=var+func->numVariables;var<varEnd;var++){
                  if ((*var)->used){
                        fprintf(out,"%s",(*var)->originalNode->name);
                        if (var+1!=varEnd){
                          fprintf(out,",");
                        }
                  }
                }
                fprintf(out,")\n");
          }
        }
        fclose(out);
}
void setBreakThreshold(char* threshold){
        // set bigBreakThreshold to be as specified
        double maxAllowed;
        double minAllowed;
        double maxMem=strtod(threshold,NULL);
        maxMem*=(1024.0*1024.0);
        maxAllowed=2048.0*1024.0*1024.0;
        minAllowed=100.0*1024.0*1024.0;
        if (maxMem>maxAllowed){
          fprintf(stderr,"WRONG SPECIFICATION: memory threshold is too large >2Gb - %lf\n",maxMem/1024/1024);
          exit(-1);
        }
        if (maxMem<minAllowed){
          fprintf(stderr,"WRONG SPECIFICATION: memory threshold is too small >100Mb - %lf\n",maxMem/1024/1024);
          exit(-1);
        }
        bigBreakThreshold=floor(maxMem/16);
}

/**compute complexity taking into account split nodes***/
double  parallelSumTableSizes(double *tableSizes, elimOrderPed *elimOrd, 
                                                          long firstElimInd, int splitDepth, int totalBreakNodes)
{
        int                             numBreakNodes;
        double                  sumSizes;
        elimOrderNode   *elimNode;
        long                    elimInd;

        /* 1.   Initialize the sum of the table sizes and the number of break nodes
                        to zero. */

        sumSizes = 0;
        numBreakNodes = 0;

        /* 2.   Update the sum of the table sizes, and the number of break nodes. */
	
	numBreakNodes = 0;
        elimNode = elimOrd->elimOrderArr + elimOrd->numNodes - 1;

        for (elimInd = elimOrd->numNodes - 1; elimInd >= firstElimInd; elimInd--,
                        elimNode--) {

                if (elimNode->isBreakNode) {
                        if (totalBreakNodes-splitDepth>numBreakNodes){
                                sumSizes *= ((elimNode->originalNode)->varInProd)->numValues;
                        }
                        numBreakNodes++;
                }
                else {
                        sumSizes += tableSizes[elimInd];
                }
        }

        return sumSizes;

}

#endif
/******************************************************************************
*
*       Function        setEOOptimizationPolicy
*       Parameters		argc, argv (the command line paramteres).
*       Output          none
*       Description		Determines whether the optimization policy of the 
*						elimination order would be determined automatically, 
*						by the program, according to the problem at hand, or by 
*						the user (in the command line) - advanced option.
*       Comments		none
*
******************************************************************************/
void setEOOptimizationPolicy(int argc, char* argv[])
{
#ifdef PARALLEL

  int cur_argv=0;
  printf("Possible args: [exportBN - exports BN without breaking]  [-s -maxmem memory]: to stop after few iterations and set max memory threshold,\n\t [-idx number time(sec) -maxmem memory ]: to attach number to metadata files (integer) and restrict max time (should be >60) of execution. memory is the max memory threshold(in MB) assumed to be available for the program, no_breaking - no conditioning is performed \n\t [-e orderfilename permnumer depth] - read order file and execute superlink conditioned on DEPTH variables, on permutation number PERMNUMBER  \n");
	
  if (!strcmp(argv[3], "exportBN")){
	 heuristicEO.isAutomatic = TRUE;
	PERFORM_PRODUCT_BREAKING=0;
	RUN_CREATE_BAYES=1;	
  } 
#endif 

  if (strcmp(argv[3], "-e") == 0) {
	
	/* 1.	If the flag '-e' appears as the 3rd argument, then the elimination
	   order needs to be read from an external file. */
	
	readElimOrder = TRUE;
	
	/* 2.	If there's another argument, the file from which the 
	   elimination order needs to be read is specified. Otherwise,
	   the elimination order is read from a file with the default name. */ 
	
	if (argc > 4) {
	  eliminationOrderFileName = argv[4];
	}
	else {
	  eliminationOrderFileName = DEFAULT_ELIMINATION_ORDER_NAME;
	}
#ifdef PARALLEL
	
	/* we want to specify the current number of the permutation */
	currentPermuationSid=-1;
	if ( argc > 6){
	  currentPermuationSid=atoi(argv[5]);
	  numOfSplitNodes=atoi(argv[6]);
	}else{
	  printf("FATAL: syntax: -e [order filename] [perm numer] [numof split vars]\n");
	  exit(-1);
	}
#endif
  }
#ifdef PARALLEL
  else   if ( !strcmp(argv[3],"-s")){
	/*stop after first iteration to set up complexity*/
	heuristicEO.stopAfterEstimate=TRUE;
	heuristicEO.isAutomatic = TRUE;
	cur_argv=4;
	if (argc>cur_argv+1 && !strcmp(argv[cur_argv],"-maxmem")){
	  setBreakThreshold(argv[cur_argv+1]);
	}
  }else if (!strcmp(argv[3],"-idx") && argc>4){

	heuristicEO.isAutomatic = TRUE;
	heuristicEO.indexNum = atoi ( argv[4]);
	if (heuristicEO.indexNum < 0 ){
	  fprintf(stderr,"WRONG SPECIFICATION: index is less that zero - %d\n",heuristicEO.indexNum);
	  exit(-1);
	}
	// if the runtime is not constrained->read it
	cur_argv=5;
	if (argc>5 && strcmp(argv[5],"-maxmem")){
	  heuristicEO.totalTime =  atoi (argv[5]);
	  cur_argv++;
	  if (heuristicEO.totalTime <= 60){
		fprintf(stderr,"WRONG SPECIFICATION: time is less that 60 - %d\n",heuristicEO.totalTime);
		exit(-1);
	  }
	}

	if (argc>cur_argv+1 && !strcmp(argv[cur_argv],"-maxmem")){
	  setBreakThreshold(argv[cur_argv+1]);
	  cur_argv++;
	}
  }
#endif
  else if ((strcmp(argv[3], "-c") == 0) || (strcmp(argv[3], "-C") == 0)) {
	
	/* 1.	If the flag '-c' appears as the 3rd argument, then the program exits
	   after finding an elimination order. */
	
	exitAfterFindOrder = TRUE;
	heuristicEO.isAutomatic = TRUE;
  }
  else if (strcmp(argv[3], "-gh") == 0){
	RUN_GH = 1;
	heuristicEO.isAutomatic = TRUE;
  }
  else if (strcmp(argv[3], "-sl") == 0){
	RUN_SL = 1;
	heuristicEO.isAutomatic = TRUE;
  }else{
	/* 1.	The elimination order will be determined internally. */
	
	readElimOrder = FALSE;
	
	/* 2.	The number of iterations for each optimization algorithm is 
	   determined by the user (and not automatically by the program). */
	if (argc<=4){
		heuristicEO.isAutomatic = TRUE;
		return;
	}
	heuristicEO.isAutomatic = FALSE;
	

		/* 3.	Initialize the number of iterations for the Min-Weight algorithm. */

	heuristicEO.numIterAlgs[MW_IND] = atoi(argv[4]);
		
	/* 4.	Initialize the number of iterations for the weighted-min-fill
	   algorithm. */
	
	if (argc > 5)
	  heuristicEO.numIterAlgs[WMF_IND] = atoi(argv[5]);
	else {
	  heuristicEO.numIterAlgs[WMF_IND] = 0;
	}

	/* 5.	Initialize the number of iterations for the min-fill algorithm. */
	
	if (argc > 6)
	  heuristicEO.numIterAlgs[MF_IND] = atoi(argv[6]);
	else {
	  heuristicEO.numIterAlgs[MF_IND] = 0;
	}
  }
}


/******************************************************************************
*
*       Function        openInitEOStatisticsFile
*       Parameters		none
*       Output          none
*       Description		Opens the elimination-order statistics file for writing.
*       Comments		none
*
******************************************************************************/
#if PRINT_VARIOUS_STATISTICS_LEVEL
void openInitEOStatisticsFile()
{
	boolean		fileExists = FALSE;

	/* 1.	Check if the statistics file exists already. If so, close it. */

	if ((statisticsFile = fopen(STATISTICS_FILE_NAME, "r")) != NULL) {
		fileExists = TRUE;
		fclose(statisticsFile);
	}

	/* 2.	Open the file for appending. */

	printf("Opening %s\n", STATISTICS_FILE_NAME);
	statisticsFile = fopen(STATISTICS_FILE_NAME, "a");

	if (statisticsFile == NULL) {
		error("Can't open file %s", statisticsFile);
	}

	/* 3.	If the file didn't exist, print the column names on the first line of the file. */

	if (fileExists == FALSE) {
		fprintf(statisticsFile,"%s,%s, %s,%s,","datafileName", "pedfileName", "#People", "#Loci");
		fprintf(statisticsFile,"%s,%s,%s,%s,%s,","sumTableSizesDG", "sumTableSizesMW", 
					"sumTableSizesWMF","sumTableSizesMF", "numBreakVars");
		
		#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
			fprintf(statisticsFile,"%s,%s,%s,%s,","treeWidthDG", "treeWidthMW", 
					"treeWidthWMF", "treeWidthMF");
		#endif

		fprintf(statisticsFile,"%s, %s,","timeIter(sec)", "#IterSG");

		#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
			fprintf(statisticsFile,"%s,%s,%s,%s,","#VarsElim", "#VarsConst", "#VarsTrim", "#VarsInBN");
		#endif

		#if (PRINT_VARIOUS_STATISTICS_LEVEL > 2) 
			fprintf(statisticsFile,"%s,%s,%s,","min#ValsVar", "max#ValsVar", "avg#ValsVar");
		#endif

		#if (PRINT_VARIOUS_STATISTICS_LEVEL > 3) 
			fprintf(statisticsFile,"%s,%s,%s,","min#Selectorslocus", "max#Selectorslocus", "avg#Selectorslocus");
		#endif

		fprintf(statisticsFile,"%s, ", "findOrderTime(sec)");
		fprintf(statisticsFile,"%s, %s, %s,", "numOneUnite", "numTwoUnite", "numFourUnite");
		fprintf(statisticsFile,"%s\n", "totalTime(sec)");
	}
}
#endif



/******************************************************************************
*
*       Function        determineEliminationOrder
*       Parameters		prodArr (the array of products for which an 
*						elimination-order must be determined), numLoci (the 
*						number of loci in the analysis), updateElimVarsLSegFunc
*						(pointer to the function that needs to be used for 
*						determining a locus-by-locus order), 
*						determineOptHeuristicFunc (pointer to the function that
*						needs to be used for determining an optimization heuristic).
*       Output          none
*       Description		Determines an eliminationOrder for each one of the 
*						Bayesian-networks (which represent the input pedigrees).
*       Comments		none
*
******************************************************************************/
elimOrder determineEliminationOrder(product** prodArr, int numLoci,
									updateElimVarsLSeg updateElimVarsLSegFunc,
									determineOptHeuristic determineOptHeuristicFunc)
{
	int						numPeds, elimInd, complexityCoefficient, numSeg, 
							algInd, firstElimInd;
	long					*numVarsSegArr, *elimVarsSegArr, sumTableSizesPre;
	elimOrder				elimOrderArrs;
	pedigree				*currPed;
	elimOrderPed			*elimOrderPedPtr, *elimOrderPedPtrEnd, tempElimOrderPed;
	product					**prodPtr, *prod;
	elimOrderProduct		*elimProd, *tempElimProd;
	elimOrderNode			*tmpPtr, *tmpPtrEnd;
	clock_t					start_time, end_time;

	/* The time needed for finding the elimination order, using the DG/SG algorithm. */
	double					findElimOrderTimeIter, *tableSizes;
	boolean					foundBetterOrd, *isZeroLikePed;

	numPeds = peds.numPeds; 
	breakProductBottomThreshold = smallBreakThreshold;

	/* 1.	If the optimization policy of the elimination order is determined 
			automatically, by the program. */

	if (heuristicEO.isAutomatic) {

		/* 1.1.	Configure an optimization policy array which defines a 
				specific optimization policy for each possible problem complexity. */

		optimizationPolicyArr = configureOptimizationPolicy();
	}

	/* 2.	Allocate memory for the array of elimination-order arrays. */

	myMalloc(elimOrderArrs, sizeof(elimOrderPed) * numPeds);

	/*	3.	Iterate over the pedigrees.  For each pedigree: */

	for (elimOrderPedPtr = elimOrderArrs, elimOrderPedPtrEnd = elimOrderPedPtr + numPeds,
			currPed = peds.head, prodPtr = prodArr, isZeroLikePed = totalRes.isZeroLikeArr; 
			elimOrderPedPtr < elimOrderPedPtrEnd; elimOrderPedPtr++, prodPtr++, 
			currPed = currPed->nextPed, isZeroLikePed++) {

		prod = *prodPtr;

		/* 3.1.	If the likelihood of this pedigree is zero, or there are no variables in 
				the product, initialize the elimination order to NULL, and continue to 
				the next pedigree. */

		if ((*isZeroLikePed) || (prod->currNumVars == 0)) {
			elimOrderPedPtr->elimOrderArr = NULL;
			continue;
		}

		foundBetterOrd = FALSE;
		numVarsSegArr = NULL;
		elimVarsSegArr = NULL;
		findElimOrderTimeIter = 0;
		firstElimInd = 0;
		sumTableSizesPre = 0;

		/* 3.1.	Allocate memory for the elimination-order array, and initialize its fields. */

		myMalloc(elimOrderPedPtr->elimOrderArr, sizeof(elimOrderNode) * prod->currNumVars);

		elimOrderPedPtr->numNodes = prod->currNumVars;

		/* 3.2.	Create the elimination-order-product from the "real" product. */

		elimProd = createElimOrderProduct(prod);

		/* 3.3.	Allocate memory for the temp elimination-order array, and initialize
				its fields. */

		myMalloc(tempElimOrderPed.elimOrderArr, sizeof(elimOrderNode) * prod->currNumVars);

		tempElimOrderPed.numNodes = prod->currNumVars;

		/* 3.4.	Allocate memory for an array that would store the sizes of the probability
				tables created during the elimination. */

		myMalloc(tableSizes, sizeof(double) * elimOrderPedPtr->numNodes);

		/* 3.5.	Duplicate the elimination-order product. */

		tempElimProd = copyElimOrderProduct(elimProd, prod->numVariables);

		/* 3.6.	Determine a general elimination order in a determinstic-greedy way 
				(min-weight), to estimate the problem's complexity. */

		elimOrderPedPtr->isLGOrder = FALSE;
		tempElimOrderPed.isLGOrder = FALSE;

		start_time = clock();

		MinWeightSearch(tempElimProd, elimOrderPedPtr, currPed, numLoci, 
						firstElimInd, FALSE, tableSizes, elimVarsSegArr, numVarsSegArr,DBL_MAX);

		end_time = clock();

		findElimOrderTimeIter = ((double)(end_time-start_time))/((double)CLOCKS_PER_SEC)*1.2;

		/* 3.7.	Determine the elimination complexity of the problem. */

		heuristicEO.sumTableSizesDG = elimOrderPedPtr->sumTableSizes;
		
		for (algInd = 0; algInd < NUM_ALGS; algInd++) {
			heuristicEO.sumTableSizesAlgs[algInd] = heuristicEO.sumTableSizesDG;
		}

		complexityCoefficient = (int)(log10(heuristicEO.sumTableSizesDG));

		#if PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER
			printf("General greedy heuristics results: sumOfTableSizes=%.f, complexityCoefficient=%d.\n", 
					heuristicEO.sumTableSizesDG, complexityCoefficient);
		#endif

		/* 3.8.	If the number of non-founders in the pedigree is below the minimum
				for which a general-greedy elimination order is determined,	allocate
				memory for data-structures that would store the variables according
				to the locus-by-locus order, and initialize them. */

		if (currPed->numNonFounders < MIN_NUM_NF_GENERAL) {
			elimOrderPedPtr->isLGOrder = TRUE;
			tempElimOrderPed.isLGOrder = TRUE;

			/* 3.8.1.	Allocate memory for data-structures that would store the 
						variables of the network in an order of locus by locus (first
						all the genetic-loci variables of the first locus, then the
						selector variables of the first locus, and so on). */

			numSeg = numLoci * 2;
			myMalloc(numVarsSegArr, sizeof(long) * numSeg);
			myMalloc(elimVarsSegArr, sizeof(long) * elimOrderPedPtr->numNodes);

			/* 3.8.2.	Duplicate the elimination-order-product. */
			
			tempElimProd = copyElimOrderProduct(elimProd, prod->numVariables);

			/* 3.8.3.	Update these data-structures according to the type of analysis	
						being performed. */

			updateElimVarsLSegFunc(elimVarsSegArr, numVarsSegArr, tempElimProd, 
								   currPed, numLoci);

			/* 3.8.4.	Free the memory allocated for the elimination-order-product. */

			freeElimOrderProduct(tempElimProd);

			/* 3.8.5.	Duplicate the elimination-order product. */

			tempElimProd = copyElimOrderProduct(elimProd, prod->numVariables);

			MinWeightSearch(tempElimProd, &tempElimOrderPed, currPed, numLoci, 
							firstElimInd, FALSE, tableSizes, elimVarsSegArr, numVarsSegArr,DBL_MAX);
				
			/* 3.8.6.	If the cost of the locus-by-locus elimination order found better 
						than the cost of the general elimination order found, save it 
						and calculate the new complexity coefficient. */

			if (tempElimOrderPed.sumTableSizes < elimOrderPedPtr->sumTableSizes) {
				tmpPtr = elimOrderPedPtr->elimOrderArr;
				(*elimOrderPedPtr) = tempElimOrderPed;
				tempElimOrderPed.elimOrderArr = tmpPtr;
				
				heuristicEO.sumTableSizesDG = elimOrderPedPtr->sumTableSizes;
				complexityCoefficient = (int)(log10(heuristicEO.sumTableSizesDG));
				
				#if PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER
					printf("Locus-by-Locus Greedy heuristics results: sumOfTableSizes=%.f, complexityCoefficient=%d.\n",
							heuristicEO.sumTableSizesDG, complexityCoefficient);
				#endif
			}	
			else {
				elimOrderPedPtr->isLGOrder = FALSE;
				tempElimOrderPed.isLGOrder = FALSE;
			}
		}
		
		/* 3.9.	If the complexity is too small for further optimization, continue to 
				the next pedigree. */

		if (complexityCoefficient <= OPTIMIZATION_EO_BOTTOM_THRESHOLD){
			goto continueToNextPed;
		}

		/* 3.10.	Determine the heuristic that is going to be used for this pedigree,
					i.e., whether big-threshold or small-threshold is going to be used,
					and whether a general greedy order is going to be found or a locus-by-locus
					greedy order. */

		if (determineOptHeuristicFunc(&complexityCoefficient, currPed, elimProd, 
								      elimOrderPedPtr, &tempElimOrderPed, prod->numVariables, 
									  numLoci, tableSizes, elimVarsSegArr, numVarsSegArr)) {
			goto continueToNextPed;
		}
			
		/* 3.11.	If the optimization policy of the elimination order is determined 
					automatically, by the program. */

		if (heuristicEO.isAutomatic) {

			sumTableSizesPre =
				performAdditionalIterAuto(&complexityCoefficient, findElimOrderTimeIter, elimOrderPedPtr,
										  elimProd, currPed, &tempElimOrderPed, prod->numVariables,
										  numLoci, tableSizes, elimVarsSegArr, numVarsSegArr);

		}
		else {
			sumTableSizesPre = 
				performAdditionalIterManual(&complexityCoefficient, findElimOrderTimeIter, elimOrderPedPtr,
										    elimProd, currPed, &tempElimOrderPed, prod->numVariables,
										    numLoci, tableSizes, elimVarsSegArr, numVarsSegArr);
		}
	
	continueToNextPed:	

		#if PRINT_COMPLEXITY_CLASS
			myPrintf("\nThe complexity coefficient of the elimination order found for pedigre #%d is %d .\n\n",
						currPed->pedNum, (int)(log10(elimOrderPedPtr->sumTableSizes)));
		#endif

		/* 3.12.	Free the memory allocated dynamically. */

		myFree(tempElimOrderPed.elimOrderArr);
		freeElimOrderProduct(elimProd);
		myFree(tableSizes);

		if (currPed->numNonFounders < MIN_NUM_NF_GENERAL) {
			myFree(numVarsSegArr);
			myFree(elimVarsSegArr);
		}

		/* 3.13.	Print the best results to the statistics files. */

		#if PRINT_VARIOUS_STATISTICS_LEVEL

			heuristicEO.numBreakNodes = elimOrderPedPtr->numBreakNodes;

			#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
				heuristicEO.treeWidthAlgs[MW_IND] = heuristicEO.treeWidthAlgs[MCS_IND] = 
					heuristicEO.treeWidthDG = elimOrderPedPtr->treeWidth;
			#endif

			printStatisticsResults(currPed, numLoci, findElimOrderTimeIter, prod, sumTableSizesPre);
		#endif

		/* 3.14.	Update the elimInd of the nodes of the Bayesian network. */

		for (elimInd = 0, tmpPtr = elimOrderPedPtr->elimOrderArr, 
				tmpPtrEnd = tmpPtr + elimOrderPedPtr->numNodes; tmpPtr < tmpPtrEnd; 
				tmpPtr++, elimInd++) {

			(tmpPtr->originalNode)->elimInd = elimInd;
		}


	}

	myFree(optimizationPolicyArr);

	return elimOrderArrs;
}

	
/******************************************************************************
*
*       Function        determineElimOrderHeuristic
*       Parameters		complexityCoefficient (the complexity coefficient of the
*						order found), currPed (the pedigree for which an order
*						is being determined), elimOrderPedPtr (will store the
*						elimination order found for this pedigree), 
*						tempElimOrderPed (used to store temporary orders found),
*						numVariables (the number of variables in the product),
*						numLoci (the number of loci in the analysis), 
*						tableSizes (a data structure that stores the sizes of
*						the probability tables created during the inference),
*						elimVarsSegArr (holds the variables ordered in segments, 
*						the variables in each segment need to be ordered, but the
*						order of the segments between themselves needs to be kept),
*						numVarsSegArr (holds the number of variables in each 
*						segment).
*       Output          none
*       Description		determines the heuristic according to which an elimination
*						order would be seeked (either a general order or a 
*						locus-by-locus order, and a small or big memory threshold
*						size).
*       Comments		returns TRUE if the complexity of the elimination order
*						found is smaller for further optimization. Otherwise,
*						returns FALSE.
*
******************************************************************************/
boolean determineElimOrderHeuristic(int *complexityCoefficient, pedigree *currPed, 
									elimOrderProduct *elimProd, elimOrderPed *elimOrderPedPtr, 
									elimOrderPed *tempElimOrderPed, int numVariables, 
									int numLoci, double *tableSizes, long *elimVarsSegArr, 
									long *numVarsSegArr)
{
	int				i, j;			
	double			minSumTableSizesSmall, minSumTableSizesBig;
	boolean			isGeneralOrder;
	elimOrderPed	elimOrderPedArr[4], *elimOrderPedArrPtr;
	elimOrderNode			*tmpPtr;


	/* 1.	Determine the best elimination heuristics for a pedigree with a medium
			number of nonfounders, either a general order or a locus-by-locus order. */

	if ((currPed->numNonFounders >= MAX_NUM_NF_LOCUS_LOCUS) && 
			(currPed->numNonFounders < MIN_NUM_NF_GENERAL)) {

		/* 1.1.	First treat the case where the complexity class is smaller then 9. 
				In this case, a small memory threshold is used. */

		if (*complexityCoefficient < BIG_THRESHOLD_BOTTOM_OPTIMIZATION) {

			/* 	1.1.1.	Execute 3 iterations of the MW (Min-Weight) heuristic to 
						find a a locus-by-locus greedy elimination sequence. */
			
			tempElimOrderPed->isLGOrder = TRUE;

			determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed, 0, numVariables,
						numLoci, tableSizes, elimVarsSegArr, numVarsSegArr, SMALL_NUM_TEST_ITER);
			
			/* 1.1.2.	If the complexity is too small for further optimization , return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {

				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;
			}

			/* 	1.1.3.	Execute 3 iterations of the MW (Min-Weight) heuristic to 
						find a a general greedy elimination sequence. */
			
			tempElimOrderPed->isLGOrder = FALSE;
			
			determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed, 0, numVariables,
						numLoci, tableSizes, elimVarsSegArr, numVarsSegArr, SMALL_NUM_TEST_ITER);
			
			/* 1.1.4.	If the complexity is too small for further optimization , return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {
				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;
			}

			/*	1.1.5.	Decide on the type of order (general-greedy or locus-by-locus), 
						according to the sum of table sizes. */
			
			if (elimOrderPedPtr->isLGOrder == FALSE) {
				tempElimOrderPed->isLGOrder = FALSE;
			}
			else {
				tempElimOrderPed->isLGOrder = TRUE;
			}

			goto endFalse;
		}
		
		/* 1.1.	The complexity class is larger than 9. In this case the 
				threshold size also needs to be determined. */

		/* 1.1.1.	For each combination of order type (LL/general) and threshold
					size (small/big) determine an elimination sequence and cost. */

		for (i = 0, elimOrderPedArrPtr = elimOrderPedArr; i < 4; i++, elimOrderPedArrPtr++) {
			
			myMalloc(elimOrderPedArrPtr->elimOrderArr, sizeof(elimOrderNode) * elimOrderPedPtr->numNodes);
			elimOrderPedArrPtr->numNodes = elimOrderPedPtr->numNodes;
			elimOrderPedPtr->sumTableSizes = DBL_MAX;
		
			/* 1.1.1.1.	Set the memory threshold size (small/big). */

			breakProductBottomThreshold = (i%2 ? bigBreakThreshold : smallBreakThreshold);

			/* 1.1.1.2.	Set the order type (LL/General). */				

			tempElimOrderPed->isLGOrder = (i<2 ? FALSE : TRUE);				

			/* 1.1.1.3. Execute 5 iterations of the Min-Weight heuristic 
						to find an elimination sequence. */
			
			determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
							0, numVariables, numLoci, tableSizes,
							elimVarsSegArr, numVarsSegArr, NUM_TEST_ITER);
			
			/* 1.1.1.4.	If the complexity of the elimination order found smaller for further
						optimization free the memory allocated dynamically and return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {

				for (j = 0; j < i; j++) {
					myFree(elimOrderPedArr[j].elimOrderArr);
				}

				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;
			}

			/* 1.1.1.5. Execute 5 iterations of the Weighted Min-Fill heuristic 
						to find an elimination sequence. */

			determineEO_MF(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
						   0, numVariables, numLoci, tableSizes, 
						   elimVarsSegArr, numVarsSegArr, NUM_TEST_ITER, calcWMinFillCost, TRUE);
			
			/* 1.1.1.4.	If the complexity of the elimination order found smaller for further
						optimization free the memory allocated dynamically and return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {

				for (j = 0; j < i; j++) {
					myFree(elimOrderPedArr[j].elimOrderArr);
				}

				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;
			}
			/* 1.1.1.5.	Store the best elimination order found for this combination of
						order type and threshold size. */

			tmpPtr = elimOrderPedArrPtr->elimOrderArr;
			*elimOrderPedArrPtr = *elimOrderPedPtr;
			elimOrderPedPtr->elimOrderArr = tmpPtr;
		}

		tmpPtr = elimOrderPedPtr->elimOrderArr;

		/*	1.1.2.	Decide on the size of threshold (big or small), and on the 
					order type (general or locus-by-locus), and update the 
					elimination order with the sequence found according to this 
					combination. */

		minSumTableSizesSmall = 
				MIN(elimOrderPedArr[0].sumTableSizes, elimOrderPedArr[2].sumTableSizes);

		minSumTableSizesBig = 
				MIN(elimOrderPedArr[1].sumTableSizes, elimOrderPedArr[3].sumTableSizes);

		isGeneralOrder = FALSE;

		if (minSumTableSizesSmall < 3 * minSumTableSizesBig) {		
			breakProductBottomThreshold = smallBreakThreshold;

			if (elimOrderCost(elimOrderPedArr[2]) > elimOrderCost(elimOrderPedArr[0])) {
				isGeneralOrder = TRUE;
				tempElimOrderPed->isLGOrder = FALSE;

				(*elimOrderPedPtr) = elimOrderPedArr[0];
				elimOrderPedArr[0].elimOrderArr = tmpPtr;
			}
			else {
				(*elimOrderPedPtr) = elimOrderPedArr[2];
				elimOrderPedArr[2].elimOrderArr = tmpPtr;
			}
		}
		else if (elimOrderCost(elimOrderPedArr[3]) > elimOrderCost(elimOrderPedArr[1])) {
			isGeneralOrder = TRUE;
			tempElimOrderPed->isLGOrder = FALSE;

			(*elimOrderPedPtr) = elimOrderPedArr[1];
			elimOrderPedArr[1].elimOrderArr = tmpPtr;
		}
		else {
			(*elimOrderPedPtr) = elimOrderPedArr[3];
			elimOrderPedArr[3].elimOrderArr = tmpPtr;
		}

		for (i = 0; i < 4; i++) {
			myFree(elimOrderPedArr[i].elimOrderArr);
		}

		goto endFalse;
	}
	
	/* 2.	Determine the best elimination heuristics for a pedigree with either a number of 
			nonfounders in the pedigree less than MAX_NUM_NF_LOCUS_LOCUS 
			or more than MIN_NUM_NF_GENERAL, and a complexity coefficient bigger than 9. */

	if (*complexityCoefficient >= BIG_THRESHOLD_BOTTOM_OPTIMIZATION) {

		/* 2.1.	For each threshold size (small/big) determine an elimination sequence and cost. */

		for (i = 0, elimOrderPedArrPtr = elimOrderPedArr; i < 2; i++, elimOrderPedArrPtr++) {

			myMalloc(elimOrderPedArrPtr->elimOrderArr, sizeof(elimOrderNode) * elimOrderPedPtr->numNodes);
			elimOrderPedArrPtr->numNodes = elimOrderPedPtr->numNodes;
			elimOrderPedPtr->sumTableSizes = DBL_MAX;

			/* 2.1.1.	Set the memory threshold size (small/big). */

			breakProductBottomThreshold = (i%2 ? bigBreakThreshold : smallBreakThreshold);

			/* 2.1.2. Execute 5 iterations of the Min-Weight heuristic to find an
					  elimination sequence. */

			determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed, 0,
						   numVariables, numLoci, tableSizes, elimVarsSegArr, numVarsSegArr,
						   NUM_TEST_ITER);

			/* 2.1.3.	If the complexity of the elimination order found smaller for further
						optimization free the memory allocated dynamically and return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD){

				for(j = 0; j < i; j++) {
					myFree(elimOrderPedArr[j].elimOrderArr);
				}
				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;			
			}
			/* 2.1.4. Execute 5 iterations of the Weighted Min-Fill heuristic to find an
					  elimination sequence. */

			determineEO_MF(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
						   0, numVariables, numLoci, tableSizes, elimVarsSegArr,
						   numVarsSegArr, NUM_TEST_ITER, calcWMinFillCost, TRUE);

			/* 2.1.5.	If the complexity of the elimination order found smaller for further
						optimization free the memory allocated dynamically and return. */

			if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD){
				
				for (j = 0; j < 2; j++) {
					myFree(elimOrderPedArr[j].elimOrderArr);
				}
				heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
				return TRUE;
			}

			/* 2.1.6.	Store the best elimination order found for this threshold size. */

			tmpPtr = elimOrderPedArrPtr->elimOrderArr;
			*elimOrderPedArrPtr = *elimOrderPedPtr;
			elimOrderPedPtr->elimOrderArr = tmpPtr;
		}

		/*	2.2.	Decide on the size of threshold (big or small), and update the 
					elimination order with the sequence found according to this combination. */
		
		tmpPtr = elimOrderPedPtr->elimOrderArr;

		if (elimOrderPedArr[0].sumTableSizes / elimOrderPedArr[1].sumTableSizes > 3.0) {
			(*elimOrderPedPtr) = elimOrderPedArr[1];
			elimOrderPedArr[1].elimOrderArr = tmpPtr;	
		}
		else {
			(*elimOrderPedPtr) = elimOrderPedArr[0];
			elimOrderPedArr[0].elimOrderArr = tmpPtr;	
			breakProductBottomThreshold = smallBreakThreshold;
		}

		for (j = 0; j < i; j++){
			myFree(elimOrderPedArr[j].elimOrderArr);
		}
	}
		
endFalse:
	
	/*	3.	Calculate the complexity coefficient of the problem based 
			on the best result found. */
			
	*complexityCoefficient = (int)(log10(elimOrderPedPtr->sumTableSizes));

	if (*complexityCoefficient >= MAX_ELIMINATION_COMPLEXITY) {
		*complexityCoefficient = MAX_ELIMINATION_COMPLEXITY - 1;
	}

	heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
	
	return FALSE;
}

/******************************************************************************
*
*       Function        determineElimOrderHeuristicHaplo
*       Parameters		complexityCoefficient (the complexity coefficient of the
*						order found), currPed (the pedigree for which an order
*						is being determined), elimOrderPedPtr (will store the
*						elimination order found for this pedigree), 
*						tempElimOrderPed (used to store temporary orders found),
*						numVariables (the number of variables in the product),
*						numLoci (the number of loci in the analysis), 
*						tableSizes (a data structure that stores the sizes of
*						the probability tables created during the inference),
*						elimVarsSegArr (holds the variables ordered in segments, 
*						the variables in each segment need to be ordered, but the
*						order of the segments between themselves needs to be kept),
*						numVarsSegArr (holds the number of variables in each 
*						segment).
*       Output          none
*       Description		determines the heuristic according to which an elimination
*						order would be seeked (either a general order or a 
*						locus-by-locus order, and a small or big memory threshold
*						size).
*       Comments		returns TRUE if the complexity of the elimination order
*						found is smaller for further optimization. Otherwise,
*						returns FALSE.
*						Used for haplotyping (Super_Haplo program).
*
******************************************************************************/
boolean determineElimOrderHeuristicHaplo(int *complexityCoefficient, pedigree *currPed, 
										 elimOrderProduct *elimProd, 
										 elimOrderPed *elimOrderPedPtr,  
										 elimOrderPed *tempElimOrderPed, int numVariables, 
									     int numLoci, double *tableSizes, 
										 long *elimVarsSegArr, long *numVarsSegArr)
{
	double			 sumTableSizesLG;

	/* 1.	Determine the best elimination heuristics for a pedigree with a medium
			number of nonfounders, either a general order or a locus-by-locus order. */

	if ((currPed->numNonFounders >= MAX_NUM_NF_LOCUS_LOCUS) && 
			(currPed->numNonFounders < MIN_NUM_NF_GENERAL)) {

		/* 1.1.	Execute 3 iterations of the MW (Min-Weight) heuristic to 
				find a a locus-by-locus greedy elimination sequence. */
		
		tempElimOrderPed->isLGOrder = TRUE;

		determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed, 0, numVariables,
					numLoci, tableSizes, elimVarsSegArr, numVarsSegArr, SMALL_NUM_TEST_ITER);
		
		/* 1.2.	If the complexity is too small for further optimization , return. */

		if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {

			heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
			return TRUE;
		}

		/* 1.3.	Store the sum of table sizes of the locus-by-locus greedy
				elimination order found. */
		
		sumTableSizesLG = elimOrderPedPtr->sumTableSizes;
			
		/* 	1.4.	Execute 3 iterations of the MW (Min-Weight) heuristic to 
					find a a general greedy elimination sequence. */
		
		tempElimOrderPed->isLGOrder = FALSE;
		
		determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed, 0, numVariables,
					numLoci, tableSizes, elimVarsSegArr, numVarsSegArr, SMALL_NUM_TEST_ITER);
		
		/* 1.5.	If the complexity is too small for further optimization , return. */

		if ((int)log10(elimOrderPedPtr->sumTableSizes) <= OPTIMIZATION_EO_BOTTOM_THRESHOLD) {
			heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
			return TRUE;
		}

		/*	1.6.	Decide on the type of order (general-greedy or locus-by-locus), 
					according to the sum of table sizes. */
		
		if (elimOrderPedPtr->isLGOrder == FALSE) {
			tempElimOrderPed->isLGOrder = FALSE;
		}
		else {
			tempElimOrderPed->isLGOrder = TRUE;
		}

		/*	1.7.	Calculate the complexity coefficient of the problem based 
					on the best result found. */
			
		*complexityCoefficient = (int)(log10(elimOrderPedPtr->sumTableSizes));

		if (*complexityCoefficient >= MAX_ELIMINATION_COMPLEXITY) {
			*complexityCoefficient = MAX_ELIMINATION_COMPLEXITY - 1;
		}
	}

	heuristicEO.sumTableSizesAlgs[MW_IND] = elimOrderPedPtr->sumTableSizes;
	
	return FALSE;
}

/******************************************************************************
*
*       Function        performAdditionalIterAuto
*       Parameters		complexityCoefficient (the complexity coefficient of the
*						order found), findElimOrderTimeIter (the estimated time
*						of one optimization iteration), elimOrderPedPtr (will store the
*						elimination order found for this pedigree), 
*						elimProd (the elimination order Product that represents 
*						this pedigree), currPed (the pedigree for which an order
*						is being determined), tempElimOrderPed (used to store 
*						temporary orders found), numVariables (the number of variables 
*						in the product), numLoci (the number of loci in the analysis), 
*						tableSizes (a data structure that stores the sizes of
*						the probability tables created during the inference),
*						elimVarsSegArr (holds the variables ordered in segments, 
*						the variables in each segment need to be ordered, but the
*						order of the segments between themselves needs to be kept),
*						numVarsSegArr (holds the number of variables in each 
*						segment).
*       Output          none
*       Description		Determines an eliminationOrder for each one of the 
*						Bayesian-networks (which represent the input pedigrees).
*       Comments		returns the sum of table sizes created during the 
*						preprocessing stage.
*						Used in the case when the elimination order is determined
*						automatically by the program.
*
******************************************************************************/
long performAdditionalIterAuto(int *complexityCoefficient, double findElimOrderTimeIter, 
	 						   elimOrderPed *elimOrderPedPtr, elimOrderProduct *elimProd,
							   pedigree *currPed, elimOrderPed *tempElimOrderPed, 
							   long numVariables, int numLoci, double *tableSizes, 
							   long *elimVarsSegArr, long *numVarsSegArr)
{

	elimOrderPed	*tempElimOrderPedPre;
	int				firstElimInd;
	long			sumTableSizesPre;
	boolean			foundBetterOrd, foundBetterOrdTmp;
	double			multFactor;
	
	long 	iterationsLeft;

#if PERFORM_PREPROCESSING
	elimOrderNode			*preProcessPtr, *tmpPtr, *tmpPtrEnd;
#endif

	tempElimOrderPedPre = NULL;
	sumTableSizesPre = 0;
	firstElimInd = 0;
	
	/* 1.	Find the appropriate optimization policy in the optimization policy array,
			according to the complexity coefficient of the problem. */

	optimizationPolicyDefPtr = &(optimizationPolicyArr[*complexityCoefficient]);

	if (findElimOrderTimeIter > 0.0) {
	  
		/* 1.1.	Determine the maximum number of optimization iterations that will
				be performed - it is determined as the minimum between the number of 
				iterations that are enforced by the maximum time that can be spent on 
				optimization, and the maximum number of iterations for this problem complexity. */

	  heuristicEO.maxNumIter = 
			(int)(optimizationPolicyDefPtr->maxOptimizationTime / findElimOrderTimeIter);

		if (optimizationPolicyDefPtr->maxNumIter <= heuristicEO.maxNumIter) {
		  heuristicEO.maxNumIter = optimizationPolicyDefPtr->maxNumIter;
		}
		else if (heuristicEO.maxNumIter == 0) {
			heuristicEO.maxNumIter = 2;
		}
	}
	else {
		heuristicEO.maxNumIter = optimizationPolicyDefPtr->maxNumIter;
		findElimOrderTimeIter = 0.001;
	}

#if PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER
	if (heuristicEO.maxNumIter == 0) {
	  printf("Complexity too small for optimization (%d).\n", *complexityCoefficient);
	}
	else {
	  printf("Decided to run %d optimization iterations - approximated time for optimization is %lf.\n",
			 heuristicEO.maxNumIter, findElimOrderTimeIter*heuristicEO.maxNumIter);
	}
#endif
	
#ifdef PARALLEL
	/* exit if need to stop right after the initial estimate*/
	if (heuristicEO.stopAfterEstimate){
	  printProduct_Mark( elimProd);
	  printElimOrder(elimOrderPedPtr);
	  exit(0);
	}
#endif

	/* 2.	If the problem is simple enough, then there's no need to apply additional 
			stochastic-greedy iterations. Continue to the next pedigree. */

	if (heuristicEO.maxNumIter == 0) {
	  findElimOrderTimeIter = 0;
	  return sumTableSizesPre;
	}
	
	/* 3.	Perform preprocessing of the Bayesian network if required. */
	
#if PERFORM_PREPROCESSING
	if (!elimOrderPedPtr->isLGOrder) {
	  
	  myMalloc(tempElimOrderPedPre, sizeof(elimOrderPed));
	  (*tempElimOrderPedPre) = (*elimOrderPedPtr);
	  myMalloc(elimOrderPedPtr->elimOrderArr, sizeof(elimOrderNode) * elimOrderPedPtr->numNodes);
	  
	  firstElimInd = preprocessEOPed(elimProd, elimOrderPedPtr, &sumTableSizesPre);
	  
	  /* 3.1.	Copy the information on all the elimination-order nodes that 
		 have been determined during the preprocessing, to the temporary
		 elimination order array. */
	  
	  for (tmpPtr = tempElimOrderPed->elimOrderArr, tmpPtrEnd = tmpPtr + firstElimInd, 
			 preProcessPtr = elimOrderPedPtr->elimOrderArr; 
		   tmpPtr < tmpPtrEnd; tmpPtr++, preProcessPtr++) {
		
		tmpPtr->isBreakNode = preProcessPtr->isBreakNode;
		tmpPtr->originalNode = preProcessPtr->originalNode;
	  }
	}
#endif

	/* 4.	Execute the MW (Min-Weight) heuristic to find an elimination sequence. */
	/** Addition by Mark - bias the algorithm according to the heuristics which results in better order **/
	
	foundBetterOrd=FALSE;
	
	for(iterationsLeft=heuristicEO.maxNumIter;iterationsLeft>0;){
	  
#ifdef PARALLEL
	  /* we may decide to stop if we spent too much time */        
	  if (time(NULL)-heuristicEO.startTime >heuristicEO.totalTime) {
		break;
	  }
#endif          
	  srand(time(NULL));

	  foundBetterOrdTmp = determineEO_MW_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
											  firstElimInd, numVariables, numLoci, tableSizes,
											  elimVarsSegArr, numVarsSegArr, iterationsLeft, &iterationsLeft);
	  

	  foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;
	  
	  /* 5.	Execute the weighted-min-fill heuristic to find an elimination sequence. */
	  srand(time(NULL));
	  if ((int)(log10(elimOrderPedPtr->sumTableSizes)) > 7.0) {
		
		foundBetterOrdTmp = determineEO_MF_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
												firstElimInd, numVariables, numLoci, tableSizes, 
												elimVarsSegArr, numVarsSegArr, calcWMinFillCost, 
												TRUE,iterationsLeft,&iterationsLeft);

		
		foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;
	  }	
	  
	  /* 6.	Execute the min-fill heuristic to find an elimination sequence. */
	  srand(time(NULL));
	  
	  if ((int)(log10(elimOrderPedPtr->sumTableSizes)) > 7.0) {
		
		foundBetterOrdTmp = determineEO_MF_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
												firstElimInd, numVariables, numLoci, tableSizes, 
												elimVarsSegArr, numVarsSegArr, calcMinFillCost, 
												FALSE,iterationsLeft,&iterationsLeft);

		
		foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;
	  }
	  
	  /* 7.	If the haplotyping program is being run: */
	  
	  if (progData.programCode == SUPER_HAPLO){
		
		/* 7.1.	Calculate the complexity coefficient of the best order found. */
		
		(*complexityCoefficient) = (int)(log10(elimOrderPedPtr->sumTableSizes));
		
		/* 7.2.	If the complexity coefficient is llarger than 9: */
		
		if (*complexityCoefficient > 9){
		  
		  optimizationPolicyDefPtr = &(optimizationPolicyArr[*complexityCoefficient]);
		  heuristicEO.maxNumIter = optimizationPolicyDefPtr->maxNumIter;
		  
		  /* 7.2.1.	Calculate a factor by which the memory threshold size is increased. */ 
		  
		  multFactor = mem_avl / elimOrderPedPtr->maxMemReq;
		  
		  /* 7.2.2.	If the factor is smaller than 2, there's no point in this increase. */
		  
		  if (multFactor < 2) 
			goto restore;
		  
		  /* 7.2.3.	Update the threshold size. */
		  
		  breakProductBottomThreshold *= multFactor;
		  
		  /* 7.2.4.	Execute the MW (Min-Weight) heuristic to find an elimination sequence. */
		  
		  foundBetterOrd = determineEO_MW_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
											   firstElimInd, numVariables, numLoci, tableSizes,
											   elimVarsSegArr, numVarsSegArr,
											   iterationsLeft,&iterationsLeft);
		  
		  /* 7.2.5.	Execute the weighted-min-fill heuristic to find an elimination sequence. */
		  
		  if ((int)(log10(elimOrderPedPtr->sumTableSizes)) > 7.0) {
			
			foundBetterOrdTmp = determineEO_MF_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
													firstElimInd, numVariables, numLoci, tableSizes, 
													elimVarsSegArr, numVarsSegArr, calcWMinFillCost, 
													TRUE,iterationsLeft,&iterationsLeft);
			
			foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;
		  }	
		  
		  /* 7.2.6.	Execute the min-fill heuristic to find an elimination sequence. */
		  
		  if ((int)(log10(elimOrderPedPtr->sumTableSizes)) > 7.0) {
			
			foundBetterOrdTmp = determineEO_MF_Auto(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
													firstElimInd, numVariables, numLoci, tableSizes, 
													elimVarsSegArr, numVarsSegArr, calcMinFillCost, 
													FALSE,iterationsLeft,&iterationsLeft);
			
			foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;
		  }
		}
	  }
	}
	restore:
	  /* 8.	If no better elimination order was found during the additional
		 optimization iterations, restore the elimination order found
		 during the initial optimization, and free the memory allocated
		 to store it temporarily. */
	  
#if PERFORM_PREPROCESSING
	if (!elimOrderPedPtr->isLGOrder)  {
	  if (foundBetterOrd == FALSE) {
		myFree(elimOrderPedPtr->elimOrderArr);
		(*elimOrderPedPtr) = (*tempElimOrderPedPre);
		myFree(tempElimOrderPedPre);
	  }
	  else {
		if (tempElimOrderPedPre) {
		  myFree(tempElimOrderPedPre->elimOrderArr);
		  myFree(tempElimOrderPedPre);
		}
	  }
	}
#endif	
	
	return sumTableSizesPre;
}


/******************************************************************************
*
*       Function        performAdditionalIterManual
*       Parameters		complexityCoefficient (the complexity coefficient of the
*						order found), findElimOrderTimeIter (the estimated time
*						of one optimization iteration), elimOrderPedPtr (will store the
*						elimination order found for this pedigree), 
*						elimProd (the elimination order Product that represents 
*						this pedigree), currPed (the pedigree for which an order
*						is being determined), tempElimOrderPed (used to store 
*						temporary orders found), numVariables (the number of variables 
*						in the product), numLoci (the number of loci in the analysis), 
*						tableSizes (a data structure that stores the sizes of
*						the probability tables created during the inference),
*						elimVarsSegArr (holds the variables ordered in segments, 
*						the variables in each segment need to be ordered, but the
*						order of the segments between themselves needs to be kept),
*						numVarsSegArr (holds the number of variables in each 
*						segment).
*       Output          none
*       Description		Determines an eliminationOrder for each one of the 
*						Bayesian-networks (which represent the input pedigrees).
*       Comments		returns the sum of table sizes created during the 
*						preprocessing stage.
*						Used in the case when the elimination order is determined
*						by the used in the command line.
*
******************************************************************************/
long performAdditionalIterManual(int *complexityCoefficient, double findElimOrderTimeIter, 
								 elimOrderPed *elimOrderPedPtr, elimOrderProduct *elimProd,
								 pedigree *currPed, elimOrderPed *tempElimOrderPed, 
								 long numVariables, int numLoci, double *tableSizes, 
								 long *elimVarsSegArr, long *numVarsSegArr)
{
	int				firstElimInd;
	long			sumTableSizesPre;
	boolean			foundBetterOrd, foundBetterOrdTmp;

#if PERFORM_PREPROCESSING
	elimOrderNode			*preProcessPtr, *tmpPtr, *tmpPtrEnd;
	elimOrderPed			*tempElimOrderPedPre=NULL;
#endif


	sumTableSizesPre = 0;
	firstElimInd = 0;
	
	/* 1.	Perform preprocessing of the Bayesian network if required. */

	#if PERFORM_PREPROCESSING
		if (!elimOrderPedPtr->isLGOrder) {
			myMalloc(tempElimOrderPedPre, sizeof(elimOrderPed));
			(*tempElimOrderPedPre) = (*elimOrderPedPtr);
			myMalloc(elimOrderPedPtr->elimOrderArr, sizeof(elimOrderNode) * elimOrderPedPtr->numNodes);

			firstElimInd = preprocessEOPed(elimProd, elimOrderPedPtr, &sumTableSizesPre);

			/* 1.1.	Copy the information on all the elimination-order nodes that 
					have been determined during the preprocessing, to the temporary
					elimination order array. */

			for (tmpPtr = tempElimOrderPed->elimOrderArr, tmpPtrEnd = tmpPtr + firstElimInd, 
					preProcessPtr = elimOrderPedPtr->elimOrderArr; 
					tmpPtr < tmpPtrEnd; tmpPtr++, preProcessPtr++) {
			
				tmpPtr->isBreakNode = preProcessPtr->isBreakNode;
				tmpPtr->originalNode = preProcessPtr->originalNode;
			}
		}
	#endif

	/* 2.	Execute the MW (Min-Weight) heuristic to find an elimination sequence. */

	foundBetterOrd = determineEO_MW(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
									firstElimInd, numVariables, numLoci, tableSizes,
									elimVarsSegArr, numVarsSegArr, heuristicEO.numIterAlgs[MW_IND]);

	/* 3.	Execute the weighted-min-fill heuristic to find an elimination sequence. */

	foundBetterOrdTmp = determineEO_MF(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
										firstElimInd, numVariables, numLoci, tableSizes, 
										elimVarsSegArr, numVarsSegArr, heuristicEO.numIterAlgs[WMF_IND],
										calcWMinFillCost, TRUE);
	foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;

	/* 4.	Execute the min-fill heuristic to find an elimination sequence. */

	foundBetterOrdTmp = determineEO_MF(elimProd, currPed, elimOrderPedPtr, tempElimOrderPed,
										firstElimInd, numVariables, numLoci, tableSizes, 
										elimVarsSegArr, numVarsSegArr, heuristicEO.numIterAlgs[MF_IND],
										calcMinFillCost, FALSE);
	foundBetterOrd = foundBetterOrd || foundBetterOrdTmp;

	/* 5.	If no better elimination order was found during the additional
			optimization iterations, restore the elimination order found
			during the initial optimization, and free the memory allocated
			to store it temporarily. */

	#if PERFORM_PREPROCESSING
		if (!elimOrderPedPtr->isLGOrder)  {
			if ((foundBetterOrd == FALSE) && (firstElimInd > 0)) {
				myFree(elimOrderPedPtr->elimOrderArr);
				(*elimOrderPedPtr) = (*tempElimOrderPedPre);
				myFree(tempElimOrderPedPre);
			}
			else {
				if (tempElimOrderPedPre) {
					myFree(tempElimOrderPedPre->elimOrderArr);
					myFree(tempElimOrderPedPre);
				}
			}
		}
	#endif	

	return sumTableSizesPre;
}



/******************************************************************************
*
*       Function        updateNewOrd
*       Parameters		finalElimOrd (will be updated with the final elimination
*						order), tempElimOrd (holds the temporary elimination 
*						order found in each iteration), firstElimInd (the index
*						of the first node which will be eliminated), 
*						foundBetterOrd (will be updated with TRUE, since a better
*						elimination order has been found), complexityCoefficient
*						(will be updated with the complexity coefficient of the
*						new order found), iterCnt (the current iteration no.), 
*						alg (the name of the algorithm currently being run).
*       Output          none
*       Description		updates the elimination order according to the order 
*						found in the current iteration.
*       Comments		none.
*
******************************************************************************/
void updateNewOrd(elimOrderPed *finalElimOrd, elimOrderPed *tempElimOrd, 
				  boolean *foundBetterOrd, int *complexityCoefficient,
				  int iterCnt, char *alg, int algInd)
{
	elimOrderNode			*tmpPtr;

	/* 1.	Turn on the flag that indicates that a better elimination
			order has been found. */

	*foundBetterOrd = TRUE;

	heuristicEO.sumTableSizesAlgs[algInd] = tempElimOrd->sumTableSizes;

	/* 2.	Replace the old elimination sequence with the new one. */

	tmpPtr = finalElimOrd->elimOrderArr;
	(*finalElimOrd) = (*tempElimOrd);
	tempElimOrd->elimOrderArr = tmpPtr;

	/* 3.	Recalculate the complexity coefficient. */
	
	*complexityCoefficient = (int)(log10(finalElimOrd->sumTableSizes));

	/* 4.	If requested, print the information on the new elimination sequence found. */

	#if PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER 
		printf("%s:sumOfTableSizes=%.f, complexityCoefficient=%d, numBreakNodes =%d ",
				alg, finalElimOrd->sumTableSizes, *complexityCoefficient, 
				finalElimOrd->numBreakNodes);
		printf("iteration %d.\n", iterCnt);
	#endif
}



/******************************************************************************
*
*       Function        preprocessEOPed
*       Parameters		elimProd (the elimination order product which represents 
*						the Bayes-net for which an elimination order needs to be found), 
*						elimOrd (will hold the elimination-order found for the 
*						pedigree pointed to by currPed), sumTableSizesPre (will
*						be updated with the sum of table sizes created during the
*						elimination of the variables that are eliminated during
*						the preprocessing step).
*       Output          none
*       Description		performs preprocessing on the graph represented by the 
*						Bayes-Net and determines part of the elimination order.
*       Comments		Returns the next empty index in the elimination-order
*						array (that is, the index of the 1st element of the array
*						that hasn't been determined yet).
*
******************************************************************************/
#if PERFORM_PREPROCESSING
int preprocessEOPed(elimOrderProduct *elimProd, elimOrderPed *elimOrd,
					long *sumTableSizesPre)
{
	long			numElimVarsTot, numSimp, numASimp;
	double			low;

#if PRINT_ELIMINATION_STATISTICS
	double			preprocessTime;
	clock_t			start_time, end_time;

	start_time=clock();
#endif

	/* 1.	Initializations. */

	numElimVarsTot = 0;
	elimOrd->currElimNode = 0;
	*sumTableSizesPre = 0;

	/* 2.	As long as simplicial/almost simplicial nodes are found, continue. */ 

	do {
		/* 2.1.	Check for simplicial nodes. */

		numSimp = simplicialElim(elimProd, elimOrd, sumTableSizesPre, &low);

		/* 2.2.	Check for almost simplicial nodes. */

		numASimp = almostSimplicialElim(elimProd, elimOrd, sumTableSizesPre, low);

		/* 2.3.	Add the number of nodes eliminated in this round to the total number
				of nodes eliminated up to now. */

		numElimVarsTot += (numSimp + numASimp);

	} while (numSimp || numASimp);

#if PRINT_ELIMINATION_STATISTICS
	end_time=clock();
	preprocessTime = ((double)(end_time-start_time))/((double)CLOCKS_PER_SEC);

	printf("Preprocess time: %lf\n", preprocessTime);
	printf("Num nodes eliminated in preprocessing: %ld\n", numElimVarsTot);
#endif

	/* 3.	Return the index in the elimination order of the next 
			node that would be eliminated. */

	return elimOrd->currElimNode;
}
#endif


/******************************************************************************
*
*       Function        simplicialElim
*       Parameters		elimProd (the elimination order product which represents 
*						the Bayesian netwok for which an elimination order needs 
*						to be found), elimOrd (will hold the elimination-order 
*						found for the pedigree pointed to by currPed), 
*						sumTableSizesPre (will be updated with the sum of table 
*						sizes created during the elimination of the variables that 
*						are eliminated during the preprocessing step), low (holds 
*						the upper known bound on the size of a probability table
*						that is created during the elimination process.).
*       Output          none
*       Description		executes the simplicial rule on the graph represented by the 
*						Bayesian network (finds simplicial vertices) and determines
*						part of the elimination order.
*       Comments		Returns the number of variables eliminated by the 
*						function. Also updates low, each time a simplicial vertex
*						is found.
*
******************************************************************************/
#if PERFORM_PREPROCESSING
int simplicialElim(elimOrderProduct *elimProd, elimOrderPed *elimOrd,
				   long *sumTableSizesPre, double *low)
{
	elimOrderVar		*var, *varEnd;
	elimOrderNode		*elimNode;
	long				elimInd, numElimVarsTot, numElimVarsIter;
	int					numVars;


	/* 1.	Initializations. */

	numElimVarsTot = 0;
	elimInd = elimOrd->currElimNode;
	elimNode = &(elimOrd->elimOrderArr[elimInd]);

	/* 2.	As long as simplicial nodes have been found, continue. */

	do {
		/* 2.1.	Initialize the number of simplicial nodes found in the current iteration
				to zero. */

		numElimVarsIter = 0;

		/* 2.2.	Iterate over the variables of the elimOrder product. */

		for (var = elimProd->variables, varEnd = var + elimProd->numVariables;
				var < varEnd; var++) {

			/* 2.2.1.	If the variable is used and appears in functions,
						check if it is simplicial. */

			if ((var->used) && (var->currNumFunctions)) {

				/* 2.2.1.1.	If the elimination complexity of the variable is invalid
							recalculate it. */

				if (var->elimComplexity == INVALID) {
                	calcElimComplexity(var);
				}

				/* 2.2.1.2.	If the elimination complexity of the variable is zero (i.e.,
							all the neighbors of the variable form a clique), then this 
							is a simplicial vertex. */

				if (var->elimComplexity == 0) {

					/* 2.2.1.2.1.	If necessary, update low. */

					if (*low < var->tableSize)
						*low = var->tableSize;

					/* 2.2.1.2.2.	Update the elimination order array, the sum of
									table sizes calculation and the elimination
									order product. */

					#if UNITE_VARS
						if (var->originalNode->uniteT) {
							numVars = var->originalNode->numVarsUnite;
						}
						else {
					#endif
						numVars = 1;
					#if UNITE_VARS
						}
					#endif

					*sumTableSizesPre += (long)var->tableSize;

					eliminateElimOrderVarsPre(var, numVars, &elimInd, &elimNode, elimProd);

					numElimVarsIter += numVars;
				}
			}
		}

		/* 2.2.2.	Update the total number of simplicial vertices found up to 
					now with the number of simplicial vertices found in the 
					current iteration. */

		numElimVarsTot += numElimVarsIter;
	} while (numElimVarsIter);

	elimOrd->currElimNode = elimInd;

	/* 3.	Return the number of simplicial nodes found. */

	return numElimVarsTot;
}
#endif



/******************************************************************************
*
*       Function        almostSimplicialElim
*       Parameters		elimProd (the elimination order product which represents 
*						the Bayes-net for which an elimination order needs to be found), 
*						elimOrd (will hold the elimination-order found for the 
*						pedigree pointed to by currPed), sumTableSizesPre (will
*						be updated with the sum of table sizes created during the
*						elimination of the variables that are eliminated during
*						the preprocessing step), low (holds the upper known bound 
*						on the size of a probability table that is created during 
*						the elimination process.).
*       Output          none
*       Description		executes the almost simplicial rule on the graph represented 
*						by the Bayesian network (finds almost simplicial vertices) 
*						and determines part of the elimination order.
*       Comments		Returns the number of variables eliminated by the 
*						function.
*
******************************************************************************/
#if PERFORM_PREPROCESSING
int almostSimplicialElim(elimOrderProduct *elimProd, elimOrderPed *elimOrd,
						 long *sumTableSizesPre, double low)
{
	elimOrderVar		*var, *varEnd;
	elimOrderNode		*elimNode;
	long				elimInd, numElimVarsTot, numElimVarsIter;


	/* 1.	Initializations. */

	numElimVarsTot = 0;
	elimInd = elimOrd->currElimNode;
	elimNode = &(elimOrd->elimOrderArr[elimInd]);

	/* 2.	As long as almost simplicial nodes are found, continue. */

	do {
		/* 2.1.	Initialize the number of almost simplicial nodes found in the 
				current iteration to zero. */

		numElimVarsIter = 0;

		/* 2.2.	Iterate over the variables of the elimOrder product. */

		for (var = elimProd->variables, varEnd = var + elimProd->numVariables;
				var < varEnd; var++) {

			/* 2.2.1.	If the variable is used and appears in functions,
						check if it is simplicial. */

			if ((var->used) && (var->currNumFunctions)) {

				/* 2.2.1.1.	If the variable isn't updated regarding almost-simpliciality,
							check whether it is almost simplicial. */

				if (var->isUpdatedASimp == FALSE) {

					/* 2.2.1.1.1.	If the vertex is almost simplicial, update the 
									elimination order, the sum of table sizes calculation 
									and the elimination order product. */

					if (checkASimp(var, low)) {
						*sumTableSizesPre += (long)var->tableSize;
						eliminateElimOrderVarsPre(var, 1, &elimInd, &elimNode, elimProd);
						numElimVarsIter++;
					}
				}
			}
		}

		/* 2.2.2.	Update the total number of simplicial vertices found up to 
					now with the number of simplicial vertices found in the 
					current iteration. */

		numElimVarsTot += numElimVarsIter;
	} while (numElimVarsIter);

	elimOrd->currElimNode = elimInd;

	/* 3.	Return the number of almost simplicial nodes found. */

	return numElimVarsTot;
}
#endif



/******************************************************************************
*
*       Function        checkASimp
*       Parameters		var (the variable which is checked if it is an 
*						almost simplicial vertex), low (holds the upper known bound 
*						on the size of a probability table that is created during 
*						the elimination process.).
*						is calculated).
*       Output          none
*       Description		checks if 'var' is almost simplicial and if so, returns
*						TRUE.
*       Comments		none
*
******************************************************************************/
#if PERFORM_PREPROCESSING
boolean checkASimp(elimOrderVar *var, double low)
{
	elimOrderFunc	**funcPtr, **funcPtrEnd, *func, *maxVarsFunc = NULL;
	elimOrderVar	**varPtr, **varPtrEnd, *currVar, **neighArr;
	double			elimComplexity = 1;		/* the multiplication of the number 
											   of possible values of each of var's 
											   neighbors. */
	int				numNeighbors = 0;		/* the number of neighbors of var. */
	int				maxVarsInFunc = 0;		/* will hold the maximum number of 
											   variables in a function, among the 
											   functions in which var appears. */
	#if UNITE_VARS
		if ((var->originalNode)->uniteT) {
			return FALSE;
		}
	#endif

	/* 1.	Zero all the 'seen' flags of the variable's neighbors. */

	zeroSeenFuncsVars(var->funcs, var->currNumFunctions);

	var->alreadySeen = TRUE;

	/* 2.	Iterate over the functions in which 'var' appears. */

	for (funcPtr = var->funcs, funcPtrEnd = funcPtr + var->currNumFunctions;
			funcPtr < funcPtrEnd; funcPtr++) {
		
		func = *funcPtr;
		
		/* 2.1.	Trace the function with the maximum number of variables among the
				functions that 'var' appears in and store the maximum number of 
				variables and a pointer to the function. */

		if (func->numVariables > maxVarsInFunc) {
			maxVarsInFunc = func->numVariables;
			maxVarsFunc = func;
		}

		/* 2.2.	Iterate over all of the variables in each one of the functions. */

		for (varPtr = func->variables, varPtrEnd = varPtr + func->numVariables;
				varPtr < varPtrEnd; varPtr++) {

			currVar = *varPtr;

			/* 2.2.1.	If we haven't seen the variable yet, mark that it's been 
						seen, update the 'number of neighbors' count and update the
						elimination complexity calculation. */

			if (currVar->alreadySeen == FALSE) {
				currVar->alreadySeen = TRUE;
				elimComplexity *= currVar->numValues;
				numNeighbors++;
			}
		}
	}

	var->elimComplexity = elimComplexity;

	/* 3.	If there's a function of 'var' that contains all its neighbors except one and 
			the neighborhood weight of 'var' is below or equal to low: */

	if ((numNeighbors == maxVarsInFunc) && ((elimComplexity * var->numValues) <= low)) {

		/* 3.1.	Allocate memory for an array of pointers to the neighbors of 'var',
				and update it. All the neighbors of 'var' are marked as seen. */

		myMalloc(neighArr, sizeof(elimOrderVar*) * numNeighbors);

		getVarsInCombinedElimOrderFunc(var->funcs, var->currNumFunctions, neighArr, 
									   &numNeighbors, var);

		/* 3.2.	Mark all the variables in the function with the maximum number of
				variables as unseen. */

		for (varPtr = maxVarsFunc->variables, varPtrEnd = varPtr + maxVarsFunc->numVariables;
				varPtr < varPtrEnd; varPtr++) {
			(*varPtr)->alreadySeen = FALSE;
		}

		/* 3.3.	Iterate over all the neighbors of 'var' and look for the one who is 
				marked as seen. This is the neighbor that isn't connected to the rest
				of the neighbors. */

		for (varPtr = neighArr, varPtrEnd = varPtr + numNeighbors; varPtr < varPtrEnd;
				varPtr++) {

			if ((*varPtr)->alreadySeen == TRUE) {

				/* 3.3.1.	If the number of possible values of this variable is lower
							than that of 'var', then 'var' is an almost simplicial
							vertex. Free the memory allocated and return TRUE. */

				if ((*varPtr)->numValues <= var->numValues) {
					myFree(neighArr);
					return TRUE;
				}
				break;
			}
		}

		myFree(neighArr);
	}

	/* 4.	'var' isn't an almost simplicial vertex. Return FALSE. */

	return FALSE;
}
#endif



/******************************************************************************
*
*       Function        updateSumTableSizes
*       Parameters		elimProd (the elimination order product which represents 
*						the Bayes-net for which an elimination order needs to be found), 
*						elimOrd (will hold the elimination-order found for the 
*						pedigree pointed to by currPed), sumTableSizesPre (will
*						be updated with the sum of table sizes created during the
*						elimination of the variables that are eliminated during
*						the preprocessing step).
*       Output          none
*       Description		performs preprocessing on the graph represented by the 
*						Bayes-Net and determines part of the elimination order.
*       Comments		Returns the next empty index in the elimination-order
*						array (that is, the index of the 1st element of the array
*						that hasn't been determined yet).
*
******************************************************************************/
void updateSumTableSizes(double *tableSizes, elimOrderPed *elimOrd, 
						 long firstElimInd)
{
	int				numBreakNodes;
	double			sumSizes;
	elimOrderNode	*elimNode;
	long			elimInd;

	/* 1.	Initialize the sum of the table sizes and the number of break nodes
			to zero. */

	sumSizes = 0;
	numBreakNodes = 0;

	/* 2.	Update the sum of the table sizes, and the number of break nodes. */

	numBreakNodes = 0;
	elimNode = elimOrd->elimOrderArr + elimOrd->numNodes - 1;

	for (elimInd = elimOrd->numNodes - 1; elimInd >= firstElimInd; elimInd--, 
			elimNode--) {

		if (elimNode->isBreakNode) {
			sumSizes *= ((elimNode->originalNode)->varInProd)->numValues;
			numBreakNodes++;
		}
		else {
			sumSizes += tableSizes[elimInd];
		}
	}

	elimOrd->sumTableSizes = sumSizes;
	elimOrd->numBreakNodes = numBreakNodes;

	#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
		elimOrd->treeWidth--;
	#endif
}


/******************************************************************************
*
*       Function        updateElimVarsLSegAutosomal
*       Parameters		elimVarsSegArr (an array that holds the indices of the
*						variables according to the "order" in which they need to
*						be eliminated), numVarsSegArr (the number of nodes in
*						each segment), elimProd (the elimination-order product), 
*						currPed (the pedigree that this product represents),
*						numLoci (the number of loci in the analysis).
*       Output          none
*       Description		Initializes data structures that store the variables
*						for the Locus-by-Locus elimination order.
*       Comments		for autosomal analysis.
*
******************************************************************************/
void updateElimVarsLSegAutosomal(long *elimVarsSegArr, long *numVarsSegArr,
								 elimOrderProduct *elimProd, pedigree *currPed, 
								 int numLoci)	
{
	int				locusIndMap, locusIndDS, numVarsSeg;
	person			*personPtr, *personPtrEnd;
	long			*elimVarsArrPtr, *numVarsSegArrPtr;
	node			*currNode;


	/* 1.	Initialize pointers to the beginning of the arrays. */

	elimVarsArrPtr = elimVarsSegArr;
	numVarsSegArrPtr = numVarsSegArr;


	/* 2.	Iterate over all loci (from the last one on the map to the first one) 
			and update the array with the indices of the variables
			corresponding to each locus, first insert the variables corresponding to the
			phenotpye, & genetic-loci nodes of the locus and then the variables 
			corresponding to selector variables of the locus. */

	for (locusIndMap = numLoci - 1; locusIndMap >= 0; locusIndMap--) {
		locusIndDS = loci.mapToData[locusIndMap];

		numVarsSeg = 0;

		/* 2.1.	Initialize the array with the indices of the variables corresponding to
				all the relevant phenotype & genetic-locus nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->phenotypeVar[locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			if ((personPtr->pa != NULL) || (locusIndMap != 0)) {
				currNode = personPtr->locusVar[0][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}

				currNode = personPtr->locusVar[1][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
			else {
				currNode = personPtr->genotypeVar;

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;

		numVarsSeg = 0;

		/* 2.2.	Initialize the array with the indices of all the relevant
				selector nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->selectorVar[0][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			currNode = personPtr->selectorVar[1][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;
	}
}



/******************************************************************************
*
*       Function        updateElimVarsLSegSexlink
*       Parameters		elimVarsSegArr (an array that holds the indices of the
*						variables according to the "order" in which they need to
*						be eliminated), numVarsSegArr (the number of nodes in
*						each segment), elimProd (the elimination-order product), 
*						currPed (the pedigree that this product represents),
*						numLoci (the number of loci in the analysis).
*       Output          none
*       Description		Initializes data structures that store the variables
*						for the Locus-by-Locus elimination order.		 
*       Comments		for sex-linked analysis.
*
******************************************************************************/
void updateElimVarsLSegSexlink(long *elimVarsSegArr, long *numVarsSegArr,
							   elimOrderProduct *elimProd, pedigree *currPed, 
							   int numLoci)	
{
	int				locusIndMap, locusIndDS, numVarsSeg;
	person			*personPtr, *personPtrEnd;
	long			*elimVarsArrPtr, *numVarsSegArrPtr;
	node			*currNode;


	/* 1.	Initialize pointers to the beginning of the arrays. */

	elimVarsArrPtr = elimVarsSegArr;
	numVarsSegArrPtr = numVarsSegArr;


	/* 2.	Iterate over all loci (from the last one on the map to the first one) 
			and update the array with the indices of the variables
			corrseponding to each locus, first insert the variables corrsponding to the
			phenoptye, & genetic-loci nodes of the locus and then the variables 
			corresponding to selector variables of the locus. */

	for (locusIndMap = numLoci - 1; locusIndMap >= 0; locusIndMap--) {
		locusIndDS = loci.mapToData[locusIndMap];

		numVarsSeg = 0;

		/* 2.1.	Initialize the array with the indices of the variables corresponding to
				all the relevant phenotype & genetic-locus nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->phenotypeVar[locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			if (personPtr->personSex == Male) {
				currNode = personPtr->locusVar[1][locusIndDS];

				if (needToElimNode(currNode, elimProd)) { 
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
			else if ((personPtr->pa != NULL) || (locusIndMap != 0)) {
				currNode = personPtr->locusVar[0][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}

				currNode = personPtr->locusVar[1][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
			else {
				currNode = personPtr->genotypeVar;

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;

		numVarsSeg = 0;

		/* 2.2.	Initialize the array with the indices of all the relevant
				selector nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->selectorVar[1][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;
	}
}



/******************************************************************************
*
*       Function        updateElimVarsLSegAutosomalHaplo
*       Parameters		elimVarsSegArr (an array that holds the indices of the
*						variables according to the "order" in which they need to
*						be eliminated), numVarsSegArr (the number of nodes in
*						each segment), elimProd (the elimination-order product), 
*						currPed (the pedigree that this product represents),
*						numLoci (the number of loci in the analysis).
*       Output          none
*       Description		Initializes data structures that store the variables
*						for the Locus-by-Locus elimination order.		 
*       Comments		for autosomal haplotyping analysis.			 
*       Comments		none
*
******************************************************************************/
void updateElimVarsLSegAutosomalHaplo(long *elimVarsSegArr, long *numVarsSegArr,
									  elimOrderProduct *elimProd, pedigree *currPed, 
									  int numLoci)	
{
	int				locusIndMap, locusIndDS, numVarsSeg;
	person			*personPtr, *personPtrEnd;
	long			*elimVarsArrPtr, *numVarsSegArrPtr;
	node			*currNode;


	/* 1.	Initialize pointers to the beginning of the arrays. */

	elimVarsArrPtr = elimVarsSegArr;
	numVarsSegArrPtr = numVarsSegArr;


	/* 2.	Iterate over all loci (from the last one on the map to the first one) 
			and update the array with the indices of the variables
			corresponding to each locus, first insert the variables corresponding to the
			phenotpye, & genetic-loci nodes of the locus and then the variables 
			corresponding to selector variables of the locus. */

	for (locusIndMap = numLoci - 1; locusIndMap >= 0; locusIndMap--) {
		locusIndDS = loci.mapToData[locusIndMap];

		numVarsSeg = 0;

		/* 2.1.	Initialize the array with the indices of the variables corresponding to
				all the relevant phenotype & genetic-locus nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->phenotypeVar[locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			currNode = personPtr->locusVar[0][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			currNode = personPtr->locusVar[1][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;

		numVarsSeg = 0;

		/* 2.2.	Initialize the array with the indices of all the relevant
				selector nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->selectorVar[0][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			currNode = personPtr->selectorVar[1][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;
	}
}



/******************************************************************************
*
*       Function        updateElimVarsLSegSexlinkHaplo
*       Parameters		elimVarsSegArr (an array that holds the indices of the
*						variables according to the "order" in which they need to
*						be eliminated), numVarsSegArr (the number of nodes in
*						each segment), elimProd (the elimination-order product), 
*						currPed (the pedigree that this product represents),
*						numLoci (the number of loci in the analysis).
*       Output          none
*       Description		Initializes data structures that store the variables
*						for the Locus-by-Locus elimination order.		 
*       Comments		for sex-linked haplotyping analysis.			 
*       Comments		none
*
******************************************************************************/
void updateElimVarsLSegSexlinkHaplo(long *elimVarsSegArr, long *numVarsSegArr,
									elimOrderProduct *elimProd, pedigree *currPed, 
									int numLoci)	
{
	int				locusIndMap, locusIndDS, numVarsSeg;
	person			*personPtr, *personPtrEnd;
	long			*elimVarsArrPtr, *numVarsSegArrPtr;
	node			*currNode;


	/* 1.	Initialize pointers to the beginning of the arrays. */

	elimVarsArrPtr = elimVarsSegArr;
	numVarsSegArrPtr = numVarsSegArr;

	/* 2.	Iterate over all loci (from the last one on the map to the first one) 
			and update the array with the indices of the variables
			corrseponding to each locus, first insert the variables corrsponding to the
			phenoptye, & genetic-loci nodes of the locus and then the variables 
			corresponding to selector variables of the locus. */

	for (locusIndMap = numLoci - 1; locusIndMap >= 0; locusIndMap--) {
		locusIndDS = loci.mapToData[locusIndMap];

		numVarsSeg = 0;

		/* 2.1.	Initialize the array with the indices of the variables corresponding to
				all the relevant phenotype & genetic-locus nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->phenotypeVar[locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			if (personPtr->personSex == Male) {
				currNode = personPtr->locusVar[1][locusIndDS];

				if (needToElimNode(currNode, elimProd)) { 
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
			else {
				currNode = personPtr->locusVar[0][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}

				currNode = personPtr->locusVar[1][locusIndDS];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;

		numVarsSeg = 0;

		/* 2.2.	Initialize the array with the indices of all the relevant
				selector nodes of the current locus. */

		for (personPtr = currPed->people, personPtrEnd = personPtr + currPed->numPeople; 
				personPtr < personPtrEnd; personPtr++) {

			currNode = personPtr->selectorVar[1][locusIndDS];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;
	}
}



/******************************************************************************
*
*       Function        updateElimVarsPSeg
*       Parameters		elimVarsSegArr, numVarsSegArr, personOrder, numSeg, 
*						elimProd
*       Output          none
*       Description					 
*       Comments		none
*
******************************************************************************/
void updateElimVarsPSeg(long *elimVarsSegArr, long *numVarsSegArr, int *personOrder,
					    elimOrderProduct *elimProd, pedigree *currPed, int numLoci)	
{
	int				locusInd, numVarsSeg, *personPtr, *personPtrEnd;
	person			*currPerson;
	long			*elimVarsArrPtr, *numVarsSegArrPtr;
	node			*currNode;

	/* 1.	Initialize pointers to the beginning of the arrays. */

	elimVarsArrPtr = elimVarsSegArr;
	numVarsSegArrPtr = numVarsSegArr;


	/* 2.	Iterate over all people in the pedigree acording to the order in
			the array 'personOrder', and update the array with the indices of the
			variables corrseponding to each person. */

	for (personPtr = personOrder, personPtrEnd = personPtr + currPed->numPeople; 
			personPtr < personPtrEnd; personPtr++) {

		currPerson = &(currPed->people[*personPtr]);

		numVarsSeg = 0;

		/* 2.1.	Iterate over the loci from the last one on the map to the first one
				and update the  array with the indices of the variables corresponding to
				all the relevant phenotype, genetic-locus and selector nodes of the 
				the current person at the current locus. */

		for (locusInd = numLoci - 1; locusInd >= 0; locusInd--) {

			currNode = currPerson->phenotypeVar[locusInd];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			if ((currPerson->pa != NULL) || (locusInd != 0)) {
				currNode = currPerson->locusVar[0][locusInd];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}

				currNode = currPerson->locusVar[1][locusInd];

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}
			else {
				currNode = currPerson->genotypeVar;

				if (needToElimNode(currNode, elimProd)) {
					(*elimVarsArrPtr) = 
						elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
						elimProd->variables;
					elimVarsArrPtr++;
					numVarsSeg++;
				}
			}

			currNode = currPerson->selectorVar[0][locusInd];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}

			currNode = currPerson->selectorVar[1][locusInd];

			if (needToElimNode(currNode, elimProd)) {
				(*elimVarsArrPtr) = 
					elimProd->oldToNewVarMapping[currNode->varInProd - elimProd->firstVar] -
					elimProd->variables;
				elimVarsArrPtr++;
				numVarsSeg++;
			}
		}

		(*numVarsSegArrPtr) = numVarsSeg;
		numVarsSegArrPtr++;
	}
}






/******************************************************************************
*
*       Function        configureOptimizationPolicy
*       Parameters		none
*       Output          none
*       Description		configures the optimization policy array.			 
*       Comments		none
*
******************************************************************************/
optimizationPolicyDef* configureOptimizationPolicy()
{
	int						complexity;
	optimizationPolicyDef	*optimizationPolicyArr;


	/* 1.	Allocate memory for the optimization policy array. */

	myMalloc(optimizationPolicyArr, sizeof(optimizationPolicyDef) * 
								    MAX_ELIMINATION_COMPLEXITY);

	/* 2.	If the complexity is lower than 8, than there's
			no need to apply stochastic-greedy methods. */

	for (complexity = 0; complexity < 7; complexity++) {
		optimizationPolicyArr[complexity].maxNumIter = 0;
		optimizationPolicyArr[complexity].maxOptimizationTime = 0;
	}

	if (complexity ==7) {
		optimizationPolicyArr[complexity].maxNumIter = 5;
		optimizationPolicyArr[complexity].maxOptimizationTime = 2;
	}
	
	/* 3.	Configure the optimization policy for medium-complex files. */ 

	for (complexity = 8; complexity < 12; complexity++) {

		/* 3.1.	Determine the number of iterations performed. */

		if (complexity == 8) {
			optimizationPolicyArr[complexity].maxNumIter = 75; /* 50 */
			optimizationPolicyArr[complexity].maxOptimizationTime = 30; /* 30 */
		}
		else if (complexity == 9) {
			optimizationPolicyArr[complexity].maxNumIter = 200; /* 150 */
			optimizationPolicyArr[complexity].maxOptimizationTime = 60;
		}
		else if (complexity == 10) {
			optimizationPolicyArr[complexity].maxNumIter = 750;
			optimizationPolicyArr[complexity].maxOptimizationTime = 120;
		} else {
			optimizationPolicyArr[complexity].maxNumIter = 2000;
			optimizationPolicyArr[complexity].maxOptimizationTime = 300;
		}
	}

	/* 4.	Configure the optimization policy for very-complex files. */
	
	for (complexity = 12; complexity < MAX_ELIMINATION_COMPLEXITY - 1; 
			complexity++) {

		/* 4.1.	Determine the number of iterations performed, the maximum time
				for optimization and the number of different sized pools used. */

		if (complexity == 12) {
			optimizationPolicyArr[complexity].maxNumIter = 4000; /* 1000 */
			optimizationPolicyArr[complexity].maxOptimizationTime = 500;
		}
		else {
			optimizationPolicyArr[complexity].maxNumIter = 10000;
			optimizationPolicyArr[complexity].maxOptimizationTime = 1800;
		}
	}

	/* 5.	Configure the optimization policy for the most complex file 
			possible. */

	optimizationPolicyArr[complexity].maxNumIter = 25000;
	optimizationPolicyArr[complexity].maxOptimizationTime = 5000;
	
	/* 6.	Return the configured optimization policy array. */

	return optimizationPolicyArr;
}



/******************************************************************************
*
*       Function        printStatisticsResults
*       Parameters		peds, numLoci, heuristic, sumTableSizes, 
*       Output          none
*       Description		prints statistics on the elimination order to the 
*						statistics files.			 
*       Comments		none
*
******************************************************************************/
#if PRINT_VARIOUS_STATISTICS_LEVEL
void printStatisticsResults(pedigree *ped, int numLoci, 
							double findElimOrderTimeIter, product *prod, 
							double sumTableSizesPre)
{

	/* 1. Print the general information and the name of the function. */

   fprintf(statisticsFile,"%s,%s, %d,%d,",datafileName, pedfileName, ped->numPeople, 
		   numLoci);
   
   /* 2.  Print the deterministic-greedy results */

   fprintf(statisticsFile,"%.f,",heuristicEO.sumTableSizesDG + sumTableSizesPre);

	/* 3.  Print the results. */

	fprintf(statisticsFile,"%.f,", heuristicEO.sumTableSizesAlgs[MW_IND] + sumTableSizesPre);
    fprintf(statisticsFile,"%.f,", heuristicEO.sumTableSizesAlgs[WMF_IND] + sumTableSizesPre);
    fprintf(statisticsFile,"%.f,", heuristicEO.sumTableSizesAlgs[MF_IND] + sumTableSizesPre);

   fprintf(statisticsFile,"%d,", heuristicEO.numBreakNodes);

   #if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
		fprintf(statisticsFile,"%d,",heuristicEO.treeWidthDG);

		fprintf(statisticsFile,"%d,%d,%d,%d,",heuristicEO.treeWidthAlgs[MW_IND], heuristicEO.treeWidthAlgs[WMF_IND],
						heuristicEO.treeWidthAlgs[MF_IND]);
   #endif

   /* 4.  Print the time for one iteration of the SG algorithm, and the maximum number
		  of iterations performed for finding an elimination order for this pedigree. */

	fprintf(statisticsFile,"%lf,%d,",findElimOrderTimeIter, heuristicEO.maxNumIter);

   	#if (PRINT_VARIOUS_STATISTICS_LEVEL > 1)
		fprintf(statisticsFile,"%ld,%ld,%ld,%ld,",prod->numVarsElim, prod->numVarsConst, 
				prod->numVarsTrim, prod->currNumVars);
	#endif

	#if (PRINT_VARIOUS_STATISTICS_LEVEL > 2) 
		fprintf(statisticsFile,"%d,%d,%lf,",prod->minNumValsVar, prod->maxNumValsVar, 
				prod->avgNumValsVar);
	#endif

	#if (PRINT_VARIOUS_STATISTICS_LEVEL > 3) 
		fprintf(statisticsFile,"%ld,%ld,%lf,",prod->minNumSelectorsInterval, 
				prod->maxNumSelectorsInterval, prod->avgNumSelectorsInterval);
	#endif
}
#endif




/******************************************************************************
*
*       Function        freeEliminationOrderStructs
*       Parameters		elimOrderArrs (pointer to the elimination-order structure
*						that needs to be freed), numPeds (number of pedigrees).
*       Output          none
*       Description		frees the memory allocated for the elimination-order
*						structures.				 
*       Comments		none
*
******************************************************************************/
void freeEliminationOrderStructs(elimOrder *elimOrderArrs, int numPeds)
{
	elimOrderPed	*elimOrderPtr;
	int				currPed;

	for (elimOrderPtr = *elimOrderArrs, currPed = 0; currPed < numPeds;
			elimOrderPtr++, currPed++) {

		myFree(elimOrderPtr->elimOrderArr);
	}

	myFree(*elimOrderArrs);
}


/******************************************************************************
*
*       Function        moveSelectorBreakNodesToStart
*       Parameters		elimOrderPtr (pointer to the elimination-order structure
*						that needs to be rearranged).
*       Output          none
*       Description		
*       Comments		none
*
******************************************************************************/
void moveSelectorBreakNodesToStart(elimOrderPed *elimOrderPedPtr, int numSeg, 
								   long *numVarsSegArr, int numNonFounders)
{
	int				currSeg, numBreakNodesSeg;
	elimOrderNode	*elimNode, *elimNodeStart, *elimNodeEnd, *breakNodeArr,
					*readNode, *writeNode;
	long			*numVarsSegArrPtr;


	elimNode = elimOrderPedPtr->elimOrderArr;
	numVarsSegArrPtr = numVarsSegArr;
	
	/* 1.	Allocate memory for a temporary array which would store the information
			on the break nodes in the segment. */

	myMalloc(breakNodeArr, sizeof(elimOrderNode) * numNonFounders * 2);

	/* 2.	Iterate over the segments. */

	for (currSeg = 0; currSeg < numSeg; currSeg++) {

		/* 2.1.	If this is a segment of selectors, */

		if (((elimNode->originalNode)->nodeType == SELECTOR) && 
				((elimNode->originalNode)->lNum > 0)) {
			
			elimNodeStart = elimNode;
			numBreakNodesSeg = 0;
						
			/* 2.1.1.	Iterate over the nodes in the segment. */

			for (elimNodeEnd = elimNodeStart + *numVarsSegArr; 
					elimNode < elimNodeEnd; elimNode++) {
				
				/* 2.1.1.1.	Count the number of break nodes in the segment, and update 
							the array with information on them. */

				if (elimNode->isBreakNode) {
					breakNodeArr[numBreakNodesSeg].originalNode = elimNode->originalNode;
					numBreakNodesSeg++;
				}
			}

			/* 2.1.2.	If there were break nodes in the segment, */

			if (numBreakNodesSeg > 0) {
				readNode = elimNodeEnd-1;
				writeNode = readNode;

				/* 2.1.2.1.	Iterate over all the nodes in the segment from the end of the
							segment and shift all the non-break nodes to the end of the segment. */

				while (readNode >= elimNodeStart) {
					if (readNode->isBreakNode == FALSE) {
						if (writeNode != readNode) {
							writeNode->originalNode = readNode->originalNode;
							writeNode->isBreakNode = FALSE;
						}
						writeNode--;
					}
					readNode--;
				}

				/* 2.1.2.2.	Copy the break nodes to the beginning of the segment. */

				for (readNode = breakNodeArr, elimNodeEnd = readNode + numBreakNodesSeg,
						writeNode = elimNodeStart; readNode < elimNodeEnd; readNode++, writeNode++) {
					writeNode->originalNode = readNode->originalNode;
					writeNode->isBreakNode = TRUE;
				}
			}
		}
		else {
			elimNode += (*numVarsSegArr);
		}
	}

	myFree(breakNodeArr);
}



/******************************************************************************
*
*       Function        updateEliminationOrder
*       Parameters		prodArr (the array of products for which an 
*						elimination-order must be determined), elimOrderFileName 
*						(the name of the file from which the elimination order
*						needs to be read).
*       Output          none
*       Description		reads the eliminationOrder for each one of the 
*						Bayesian-networks (which represent the input pedigrees),
*						from a file and updates the data-structures of the 
*						program with this elimination order.
*       Comments		none
*
******************************************************************************/
elimOrder updateEliminationOrder(product** prodArr, char *elimOrderFileName)
{
  int						numPeds, elimInd;
	elimOrder				elimOrderArrs;
	pedigree				*currPed;
	elimOrderPed			*elimOrderPedPtr, *elimOrderPedPtrEnd;
	product					**prodPtr, *prod;
	elimOrderNode			*elimNode;
	FILE					*elimOrderFile;
	char					line[MAX_LINE];
	node                    *currNode = NULL;
	elimOrderVar			**elimOrderVarArr, **elimOrderVarArrPtr, 
							**elimOrderVarArrPtrEnd, *elimVarPtr, *elimVarPtrEnd, *elimVar;
	elimOrderProduct		*elimProd;
	double					*tableSizes, eliminateComplexity;

	char* origLine;
	int i;
	boolean isBreakNode=FALSE;

	char* c_numOfVals;
	char* c_isBreakNode;

#ifdef PARALLEL

	double totalElimComplexity=0;
	/* 1000 - max broken nodes possible */
	int brokenNodesNumOfValues[1000];
	int* ptrBrokenNodesNumOfValues=brokenNodesNumOfValues;
	int * assignments;
	int numBreakNodes=0;
	int k;
	int breakNodes=0;


   
#endif

	/* 1.	Open the elimination order file for reading. */
	
	elimOrderFile = fopen(elimOrderFileName, "r");
    
	if (elimOrderFile == NULL) {
	  error("Elimination order file (%s) not found", elimOrderFileName);
	}

	numPeds = peds.numPeds;
	
	/* 2.	Allocate memory for the array of elimination-order arrays. */

	myMalloc(elimOrderArrs, sizeof(elimOrderPed) * numPeds);

	/*	3.	Iterate over the pedigrees.  For each pedigree: */

	for (elimOrderPedPtr = elimOrderArrs, elimOrderPedPtrEnd = elimOrderPedPtr + numPeds,
		   currPed = peds.head, prodPtr = prodArr; elimOrderPedPtr < elimOrderPedPtrEnd;
			elimOrderPedPtr++, currPed = currPed->nextPed, prodPtr++) {

	  prod = (*prodPtr);

	  /* 3.1.	Allocate memory for the elimination-order array. */

	  myMalloc(elimOrderPedPtr->elimOrderArr, sizeof(elimOrderNode) * prod->currNumVars);
	  elimOrderPedPtr->numNodes = prod->currNumVars;
#ifdef PARALLEL		
	  /* boundary case, there is no elimination order and further information */
	  /* is useless for this pedigree */
	  if(strstr(fgets(line, MAX_LINE, elimOrderFile), "!@#$")){
		elimOrderPedPtr->elimOrderArr = NULL;
		continue;
	  }
	  /* get the size of small threshold */
	  smallBreakThreshold = strtod(line,NULL);
	  /* get the size of big threshold */
	  bigBreakThreshold = strtod(fgets(line, MAX_LINE, elimOrderFile),NULL);
	  /* if the big theshold has been chosen or not */
	  pedThreshold[ped_num] = (atoi(fgets(line, MAX_LINE, elimOrderFile)));
	  /* set the memory threshold */
	  breakProductBottomThreshold = (pedThreshold[ped_num] ? bigBreakThreshold : smallBreakThreshold);
	  /* set the number of nodes in the order */
	  elimOrderPedPtr->numNodes = prod->currNumVars;
	  /* if the LG order (or greedy) has been chosen */
	  elimOrderPedPtr->isLGOrder = atoi(fgets(line, MAX_LINE, elimOrderFile));
	  /* virtual memory size required by the given order */
	  elimOrderPedPtr->maxMemReq = strtod(fgets(line, MAX_LINE, elimOrderFile),NULL);
	  /* number of break nodes in the given order */
	  elimOrderPedPtr->numBreakNodes = atoi(fgets(line, MAX_LINE, elimOrderFile));
	  /* the sum of table sizes (complexity) of the given order */
	  elimOrderPedPtr->sumTableSizes = strtod(fgets(line, MAX_LINE, elimOrderFile),NULL);
	  
	  fprintf(stdout,"+++++>Memory threshold set to %f<+++++\n",breakProductBottomThreshold);
#endif
	

	  elimNode = elimOrderPedPtr->elimOrderArr;
	  elimInd = 0;

	  /* 3.2.	While there is still information in the file, read the next line 
		 from the file. */
	  
	  while (fgets(line, MAX_LINE, elimOrderFile) != NULL && line[0]!= '#') {

		/* 3.2.1.	find the node according to its name */

		i=-1;
		origLine=strtok(line, " ");
		c_numOfVals=strtok(NULL," ");
		c_isBreakNode=strtok(NULL," ");

		while(strcmp((&(prod->variables[++i]))->originalNode->name, origLine));
		currNode = (&(prod->variables[i]))->originalNode;

		isBreakNode=FALSE;

		/* 3.2.2.	If the variable corresponding to this node exists,
		   then update the elimination order with this node. */
		
		if ((currNode != NULL) && (currNode->varInProd != NULL) &&
			((currNode->varInProd)->used == TRUE)) {
#ifdef PARALLEL
		  /* we need to get the total num of vals for the var here and add it if it's a break node*/
		  
		  int numOfVals=-1;
		  if (c_numOfVals!=NULL && c_isBreakNode!=NULL){
			numOfVals=atoi(c_numOfVals);
		  }
		  //                      printf("%s, %d, %c \n",line,numOfVals,c_isBreakNode[0]);
		  if (c_isBreakNode!=NULL){
			//printf("breaknode:%s\n",c_isBreakNode);
			if(c_isBreakNode[0]=='1'&& numOfVals!=-1){
			  /* it is a break node */
			  (*ptrBrokenNodesNumOfValues)=numOfVals;
			  ptrBrokenNodesNumOfValues++;
			  printf("Found conditioning node %s, %d, %c\n",currNode->name,numOfVals,c_isBreakNode[0]);
			  isBreakNode=TRUE;
			}
		  }
		  
		  
#endif
		  
		  updateElimNode(elimNode, isBreakNode, currNode);
		  (elimNode->originalNode)->elimInd = elimInd;
		  elimNode++;
		  elimInd++;
		}
		else{
		  printf("BUG!!!!!!!!!!!!!!!!!!HERE");
		  exit(-1);
		}

	  }
		
#ifdef PARALLEL
	  /* current number of split break nodes is 0 */
	  maxBreakStack[ped_num] = 0;
      
	  /* we will use permutation in order to get the required permutation */
	  numBreakNodes=ptrBrokenNodesNumOfValues-brokenNodesNumOfValues;
	  if (numBreakNodes<numOfSplitNodes){
		printf("Error: number of split nodes exceeds the number of conditioning nodes %d vs %d\n ",numOfSplitNodes,numBreakNodes);
		exit(-1);
	  }
	  myMalloc(assignments,sizeof(int)*numOfSplitNodes);
	  
	  if (permutation(currentPermuationSid,brokenNodesNumOfValues,assignments,numOfSplitNodes)){
		printf("Error: the requested permutation exceeds the total possible number of permutations for these split nodes\n");
		exit(-1);
	  }
	  k=0;
	  for (k=0;k<numOfSplitNodes;k++){
		printf("%d\n",assignments[k]);
		breakNodeAssignment[ped_num][k]=assignments[k];
		maxBreakStack[ped_num]++;
	  }
	  myFree(assignments);
	  
	  breakNodes=0;
#endif
	  
	  
	  /* 3.3.	Create the elimination-order-product from the "real" product. */
	  
	  elimProd = createElimOrderProduct(prod);
	  
	  /* 3.4.	Allocate memory for an array that would hold pointers to the variables 
		 of the elimination-order product according to the elimination order. */
	  
	  myMalloc(elimOrderVarArr, sizeof(elimOrderVar*) * prod->currNumVars);
	  
	  /* 3.5.	Update the array with pointers to the variables. */
	  
	  for (elimVarPtr = elimProd->variables, elimVarPtrEnd = elimVarPtr + elimProd->currNumVars;
		   elimVarPtr < elimVarPtrEnd; elimVarPtr++) {
		elimOrderVarArr[(elimVarPtr->originalNode)->elimInd] = elimVarPtr;
	  }
	  
	  myMalloc(tableSizes, sizeof(double) * prod->currNumVars);
	  
	  elimInd = 0;
	  
	  /* 3.6.	Iterate over the variables in the array. For each variable: */
	  
	  for (elimNode = elimOrderPedPtr->elimOrderArr, elimOrderVarArrPtr = elimOrderVarArr, 
			 elimOrderVarArrPtrEnd = elimOrderVarArrPtr + prod->currNumVars;
		   elimOrderVarArrPtr < elimOrderVarArrPtrEnd; elimOrderVarArrPtr++, elimInd++,
			 elimNode++) {
		
		elimVar = *elimOrderVarArrPtr;
		
		/* 3.6.1.	Calculate its elimination complexity. */
		
		eliminateComplexity = calcElimComplexity(elimVar);
		
		/* 3.6.2.	If the elimination complexity of the variable is too
		   large, perform product-breaking on it. Otherwise,
		   update the size of the probability table that would be
		   created when eliminating it. */
		if (eliminateComplexity > breakProductBottomThreshold && PERFORM_PRODUCT_BREAKING) {
		  elimNode->isBreakNode = TRUE;
		  setBreakValElimOrder(elimProd, elimVar);
		}
		else {
		  tableSizes[elimInd] = elimVar->tableSize;
		  
		  eliminateElimOrderVar(elimProd, elimVar);
		}
	  }
	  
	  /* 4.	Update the sum of the table sizes. */
	  
	  updateSumTableSizes(tableSizes, elimOrderPedPtr, 0);
#ifdef PARALLEL
	  
	  totalElimComplexity=parallelSumTableSizes(tableSizes, elimOrderPedPtr,0,numOfSplitNodes,numBreakNodes);
	  fprintf(stdout,"Complexity is: %f versus %f\n",totalElimComplexity,elimOrderPedPtr->sumTableSizes);
	  fprintf(stdout,"ratio: %f\n",elimOrderPedPtr->sumTableSizes/totalElimComplexity);
	  fflush(stdout);
	  
#endif
	  
	  /* 5.	Free the memory allocated dynamically. */
	  
	  myFree(elimOrderVarArr);
	  myFree(tableSizes);
	}
	
	fclose(elimOrderFile);
	return elimOrderArrs;
}



/******************************************************************************
*
*       Function        parseLine
*       Parameters		line (holds one line from the elimination order file,
*						each line specifies one node in the Bayesian network), 
*						nodeSex (will be update with the sex of the node 
*						specified in the specific line, whether it is paternal 
*						or maternal), nodeType (will hold the type of the node
*						specified in the specific line), personNum (will hold
*						the number of the person that this node describes),
*						locusNum (will be updated with the index of the locus
*						that this node describes).
*       Output          none
*       Description		reads one line from the eliminationOrder file (this line
*						holds the name of a single node of the Bayesian network)
*						and updates the relevant fields with the details of 
*						this node.
*		Comments		none
*
******************************************************************************/
void parseLine(char *line, NodeSex *nodeSex, NodeType *nodeType, int *personNum, 
			   int *locusNum)
{
	char numP[MAX_LINE], numL[MAX_LINE];

	switch(line[0]){
		case 'm': 
			*nodeSex = MATERNAL;
			goto determine_type;
			break;
		case 'p':
			switch(line[1]) {
				case 'h':
					*nodeType = PHENOTYPE;
					goto determineNumbers;
					break;
				default:
					*nodeSex = PATERNAL;
					goto determine_type;
					break;
				}
				break;
		case 'g': 
			*nodeType = GENOTYPE;
			goto determineNumbers;
			break;
	}

	determine_type:
		switch(line[1]) {
			case 'l': 
				*nodeType = LOCUS;
				break;
			case 's':
				*nodeType = SELECTOR;
				break;
		}

	determineNumbers: 
		strtok(line,"_");
		strcpy(numP, strtok(NULL,"_"));
		strcpy(numL, strtok(NULL," \t\n"));
		*personNum = atoi(numP);
		*locusNum = atoi(numL);
}



#ifdef PARALLEL

/* function printElimOrder calculates split_size of a given project */
/* and prints the appropriate number of order files: */
/* order0.txt, order1.txt, etc. */
/* format of order file is described in */
/* Superlink parallel version README file */

/*arguments: elimOrderPedPtr - contains the elimination order calculated */

/* at first order0.txt file is created, then it is */
/* duplicated "split_size" times and then for each order file */
/* printStack function gives a unique conditioning assignment */
void printElimOrder(elimOrderPed *elimOrderPedPtr)
{

        /* f_numOfJobs - pointer to "NumOfJobs.txt" file, */
        /* which contains the split_size of a project */
  char numJobsFile[100];
	/* outFileName - current order file name */
  char outFileName[100];
  /* OrderEstimate file name */
  char orderEstimate[100];
  

        /* elimOrderNode, elimOrderNodeEnd - variables to iterate */
        /* over the elimination order array */
        elimOrderNode   *elimOrderNode, *elimOrderNodeEnd;


        /* breakNodeNum - number of break nodes in the given order */
        /* numOfJobs - split_size of the project */
        /* depth - number of break nodes to be splitted */
        /* breakNodeNumVal - array, containing number of values for */
        /*                                       each break node in the given order */
        int i, breakNodeNum = 0,  breakNodeNumVal[100];
        long  numOfJobs = 1;

  	FILE *f_numOfJobs;
  	FILE* fce;

  if (heuristicEO.indexNum >=0){
        sprintf(numJobsFile,"NumOfJobs%d.txt",heuristicEO.indexNum);
        sprintf(outFileName,"order%d.txt",heuristicEO.indexNum);

        sprintf(orderEstimate,"ComplexityEstimate%d.txt",heuristicEO.indexNum);
  }else{
        sprintf(numJobsFile,"NumOfJobs0.txt");
        sprintf(outFileName,"order0.txt");
        sprintf(orderEstimate,"ComplexityEstimate0.txt");
  }
  f_numOfJobs=NULL;
  if (!(f_numOfJobs = fopen(numJobsFile, "w"))){
        fprintf(stderr,"FATAL: Failed to open %s, %s\n",numJobsFile, strerror(errno));
        exit(-1);
  }
  fce=fopen(orderEstimate,"w");
  if (fce==NULL){
	fprintf(stderr,"Failed to opent file %s, %s\n",
			orderEstimate,strerror(errno));
	exit(-1);
  }
  

  outFile = fopen(outFileName, "w");
  if (outFile==NULL){
	fprintf(stderr,"Failed to opent file %s, %s\n",
			outFileName,strerror(errno));
	exit(-1);
  }


  /* boundary case  - there is no elimination order */
  /* because the project is impossible (ln-likelihood = -Infinity) */
  if(elimOrderPedPtr->elimOrderArr == NULL){
	
	/* !@#$ - special symbol for this case */
	fprintf(outFile, "!@#$\n");
	fclose(outFile);
	
	/* print split_size (here - 1), to the "NumOfJobs.txt" file */
	fprintf(f_numOfJobs, "%d\n", 1);
	fclose(f_numOfJobs);
	/*print COMPLEXITY and TABLESUME**/
	fprintf(fce,"COMPLEXITY %.0lf\n",0.0);
	fprintf(fce,"TABLESUM %.0lf\n",1.0);
	fclose(fce);
	/* exit the program - ordering has been calculated */
	exit(0);
  }
  
  
  /* print to the order file the size of small threshold */
  fprintf(outFile, "%.0lf\n", smallBreakThreshold);
  /* print to the order file the size of big threshold */
  fprintf(outFile, "%.0lf\n", bigBreakThreshold);
  
  /* print to the order file if the big theshold has been */
  /* chosen or not */
  if(breakProductBottomThreshold == smallBreakThreshold){
	fprintf(outFile, "0\n");
  }else{
	fprintf(outFile, "1\n");
  }
  
  /* print to the order file if the LG order (or greedy) */
  /* has been chosen */
  fprintf(outFile, "%d\n", elimOrderPedPtr->isLGOrder);
  
  /* print to the order file virtual memory size required */
  /* by the given order */
  fprintf(outFile, "%.0lf\n", elimOrderPedPtr->maxMemReq);
  
  /* print to the order file number of break nodes */
  /* in the given order */
  fprintf(outFile, "%d\n", elimOrderPedPtr->numBreakNodes);
  
  /* print to the order file sum of table sizes (complexity) */
  /* of the given order */
  fprintf(outFile, "%.0lf\n", elimOrderPedPtr->sumTableSizes);
  
  fprintf(fce,"COMPLEXITY %d\n",(int)log10(elimOrderPedPtr->sumTableSizes));
  fprintf(fce,"TABLESUM %.0lf\n", elimOrderPedPtr->sumTableSizes);
  fclose(fce);
  /* iterate over the calculated elimination order */
  for(i=0,elimOrderNode = elimOrderPedPtr->elimOrderArr, 
		elimOrderNodeEnd = elimOrderPedPtr->elimOrderArr + elimOrderPedPtr->numNodes; 
	  elimOrderNode < elimOrderNodeEnd;
	  elimOrderNode++, i++){
	/* print current node name to the order file */
	fprintf(outFile, "%s %d %d\n", elimOrderNode->originalNode->name,elimOrderNode->originalNode->numValues, elimOrderNode->isBreakNode);
	
	/* if current node is a break node */
	if(elimOrderNode->isBreakNode == TRUE){
	  /* update the appropriate entry of breakNodeNumVal variable */
	  breakNodeNumVal[breakNodeNum] = elimOrderNode->originalNode->numValues;
	  
	  /* if number of current break node is less then max. number */
	  /* of break nodes that can be splitted, increment the split_size */
	  /* of the project */
	  /** we want to know the what number of break nodes corresponds to which 
		  number of splits **/
	  numOfJobs *= breakNodeNumVal[breakNodeNum];
	  fprintf(f_numOfJobs,"%d %d %d\n",breakNodeNum+1,numOfJobs,(int)log10(elimOrderPedPtr->sumTableSizes/numOfJobs));
	  
	  /* increment current number of break nodes */
	  breakNodeNum++;
	}
  }
  if ( 0 == breakNodeNum ){
	// if no breakNodes found -> no parallelization possible
	fprintf(f_numOfJobs,"%d %d %d\n",breakNodeNum,1,(int)log10(elimOrderPedPtr->sumTableSizes));
  }
  fclose(f_numOfJobs);
  
  /* # symbol corresponds to the break node assignments - see printStack function */
  fprintf(outFile, "#");
  fclose(outFile);
  
  exit(0);
}
 
#else

void printElimOrder(elimOrderPed *elimOrderPedPtr, char *family)
{
	elimOrderNode	*elimOrderNode, *elimOrderNodeEnd;
	int i;

	outFile = fopen("order.txt", "a");

	for (i=0, elimOrderNode = elimOrderPedPtr->elimOrderArr, 
			elimOrderNodeEnd = elimOrderPedPtr->elimOrderArr + elimOrderPedPtr->numNodes; 
			elimOrderNode < elimOrderNodeEnd; elimOrderNode++, i++) {

	/*	fprintf(outFile, "%s,%d  %d\n", elimOrderNode->originalNode->name,elimOrderNode->originalNode->numValues,
				elimOrderNode->isBreakNode);
	*/	fprintf(outFile, "%d %s%s %d ", i, elimOrderNode->originalNode->name,family,elimOrderNode->originalNode->numValues);
		if (elimOrderNode->isBreakNode) 
			fprintf(outFile, "breakNode\n");
		else
			fprintf(outFile, "\n");
	}

	fclose(outFile);
}

#endif

