/* $Id: EliminationOrderSG.h,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*****************************************************************************/
/*  EliminationOrderSG.h													 */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _ELIMINATION_ORDER_SG_H_
#define _ELIMINATION_ORDER_SG_H_

#include <stdio.h>
#include <stdlib.h>
#include "Auxiliary.h"
#include "EliminationOrder.h"


/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* This structure holds the information on one of the variables in the variable pool. */
typedef struct varInPool {
	elimOrderVar		*var;			/* Pointer to the variable. */
	elimOrderVar		**varPtr;
	double				complexity;		/* The elimination complexity of this
										   variable. */
	double				relProb;		/* The relative probability of this
										   variable to be chosen. */
	struct varInPool	*prev;			/* Pointer to the previous structure in the list. */
	struct varInPool	*next;			/* Pointer to the next structure in the list. */
} varInPool;


/* A type of a function that updates part of the elimination order. It determines an
   order for the variables in the array 'elimVarsArr'. */

typedef long (updatePartElimOrderFunc)(elimOrderVar **elimVarsArr, long numNodesInArr, 
									   long elimOrdInd, elimOrderPed *elimOrd, 
									   elimOrderProduct *elimProd, double *tableSizes);

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void MinWeightSearch(elimOrderProduct *elimProd, elimOrderPed *elimOrd, 
					 pedigree *currPed, int numLoci, int firstElimInd,
					 boolean isStochastic, double *tableSizes, long *elimVarsSegArr, 
					 long *numVarsSegArr,double bestSize);

boolean MinWeightSearchG(elimOrderProduct *elimProd, elimOrderPed *elimOrd,
					  int firstElimInd, double *tableSizes, double bestSize);
boolean MinWeightSearchG_Stoch(elimOrderProduct *elimProd, elimOrderPed *elimOrd,
						    int firstElimInd, double *tableSizes, double bestSize);


elimOrderVar* getNextVar(elimOrderProduct *elimProd, int *numVars, 
						 boolean *isBreakVar);
elimOrderVar* getNextVarProb(elimOrderProduct *elimProd, int *numVars, 
							 boolean *isBreakVar, varInPool **bestVarPtr, 
							 varInPool **worstVarPtr);

#if	UNITE_VARS
	boolean updateUniteVars(elimOrderVar *elimVar, double *tableSizes, long *elimInd, 
	  						elimOrderNode **elimNodePtr, elimOrderProduct *elimProd);
#endif

elimOrderVar* getBreakVar(elimOrderProduct *elimProd, int *numVars);


void MinWeightSearchLG(elimOrderPed *elimOrd, elimOrderProduct *elimProd,
								 pedigree *currPed, int numLoci, 
								 int firstElimInd, double *tableSizes, 
								 long *elimVarsSegArr, long *numVarsSegArr,
								 updatePartElimOrderFunc updatePartElimOrderPtr);

void minWeightSearchInSegs(elimOrderPed *elimOrd, elimOrderProduct *elimProd,
						   long maxSegSize, int numSegs, int firstElimInd, 
						   double *tableSizes, long *elimVarsSegArr, 
						   long *numVarsSegArr,
						   updatePartElimOrderFunc updatePartElimOrderPtr);

long updatePedElimOrderPart(elimOrderVar **elimVarsArr, long numNodesInArr, 
							long elimOrdInd, elimOrderPed *elimOrd, 
							elimOrderProduct *elimProd, double *tableSizes);
long updatePedElimOrderPart_Stoch(elimOrderVar **elimVarsArr, long numNodesInArr, 
								  long elimOrdInd, elimOrderPed *elimOrd, 
								  elimOrderProduct *elimProd, double *tableSizes);

elimOrderVar* getNextVarLGPart(elimOrderVar **elimVarsArr, long numNodesInArr, 
							   elimOrderVar ***nextElimVar, int *numVars, 
							   boolean *isBreakVar);
elimOrderVar* getNextVarLGPartProb(elimOrderVar **elimVarsArr, long numNodesInArr, 
								   elimOrderVar ***nextElimVar, 
								   varInPool **bestVarPtr, varInPool **worstVarPtr,
								   int *numVars, boolean *isBreakVar);
elimOrderVar* getBreakVarLGPart(elimOrderVar **elimVarsArr, long numNodesInArr,
								int *numVars);


#endif	/* _ELIMINATION_ORDER_SG_H_ */
