/* $Id: Globals.h,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*****************************************************************************/
/*  Globals.h															     */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _GLOBALS_H_
#define _GLOBALS_H_

#define _CRTDBG_MAP_ALLOC

#ifdef WIN32
	#define _CRTDBG_MAP_ALLOC
#endif
	#include <stdlib.h>
#ifdef WIN32
	#include <crtdbg.h>
#endif
#include <stdio.h>
#include <time.h>
#include <math.h>
#include <string.h>
#include "AlleleRecoding.h"
#include "ReadData.h"
#include "LociData.h"
#include "PedValueElimination.h"
#include "EliminationOrder.h"
#include "Settings.h"
#include "Mem.h"
#include "UseBoincIO.h"

/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

typedef char xLabel[100];		/* A label on the X-axis (for generation of a PS-graph). */

/* A data-structure which holds information on the nodes that have been updated during
   the value elimination stage. */

typedef struct updatedNodesPed {
	node		**constNodes;
	int			numConstNodes;
	valEntry	**updatedValEntries;
	int			numUpdatedValEntries;
} updatedNodesPed;


/* A prototype of function which receives a double (x) and other information, and
   returns double. */
typedef double (*funcType)(double *x, void *funcData);

typedef struct evaluateLikeData {
	int				numTraitLoci;
	retValFuncType	retFunc;
	boolean			printResults;
} evaluateLikeData;


/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

/* Force genehunter algorithm */
int RUN_GH ;
/* Force superlink algorithm */
int RUN_SL;

pedData				peds;			/* Holds the data on the input pedigree(s). */
lociData			loci;			/* Holds the data on the loci in the analysis. */
recombinationData	recombData;		/* Holds the recombination data. */
programData			progData;		/* Holds the data required by the analysis programs. */		
results				totalRes;		/* Holds the results of all likelihood evaluations. */

currHeuristicEO		heuristicEO;

FILE *outFile;

boolean PERFORM_PRODUCT_BREAKING;

/* File descriptor for the statistics file. */

#if PRINT_VARIOUS_STATISTICS_LEVEL
	FILE *statisticsFile;
#endif 

char	*datafileName, *pedfileName;
boolean	readElimOrder;
char	*eliminationOrderFileName;
#ifdef PARALLEL
long currentPermuationSid;
int numOfSplitNodes;

/* maxBreakStack - number of splitted break nodes in a given order */
/*                 (for each pedigree) */
int maxBreakStack[100];

/* breakNodeAssignment - contains break assignments for each pedigree and */
/* for each break node in appropriate order calculated */
int breakNodeAssignment[100][100];

/* pedThreshold - determines wether big or small threshold has been chosen */
/*                                0 - for small, 1 - for big (for each pedigree) */
int pedThreshold[100];

/* stackDepth - number of splitted break nodes, or current splitted break node, */
/* depending on context */
/* ped_num - current number of pedigree */
int     stackDepth, ped_num;

#endif

double	breakProductBottomThreshold, bigBreakThreshold, smallBreakThreshold;

#if PRINT_MATRIX_MULT_STATISTICS
	FILE	*matrixFile;
#endif

#if PRINT_VARIOUS_STATISTICS_LEVEL
	int		numOneUnite, numTwoUnite, numFourUnite;
	FILE	*varsUniteFile;
#endif

pedigree	*currPed;

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

void calcLikelihoodOneLocus();
void calcLikelihoodSuperLinkmap();
void incThetas(thetaCompTheta *maleTheta, thetaCompTheta *femaleTheta, double inc);
void calcLikelihoodSuperMlink();
void calcLikelihoodSuperPos();
void calcLikelihoodSuperGH();
void updateMarkerName(xLabel *markerNamesEntry, locus *loc, int ind);
void superHaploProg();
void calcLikelihoodSuperLinkmapApprox();
void calcLikelihoodSuperOptlink();
double calcLikeSuperOptlinkOneModel(double *xMax, evaluateLikeData *evalData, 
									scanIntervalData *scanData);
void OptimizeMarginalLikelihood();

/* Various functions called by the above functions. */

#if (PRINT_VARIOUS_STATISTICS_LEVEL > 2)
	void calcVariousStatistics(product** prodArr, int numProds);
#endif

void iterateOneLocus(int numMarkersScan, xLabel *markerNames);
void iterateTwoLoci();
boolean performOneIter(elimOrder *elimOrderArrs, boolean *areValsElim, 
					   boolean *isChangedNet, boolean createBN);

void updateJointThetas();
void printResults_OneIter(int numThetas, int numPeds, boolean printLodScore);
void printResults_OneIterLod(int numPeds);
void printResults_OneIterWOLod(int numPeds);
void updateMax(retValFuncType func);

void printResultsSummary_OneLocus(int numMarkers, xLabel *markerNames);
void printResultsSummary_OneLocus_MMLS(int numMarkers, xLabel *markerNames);
void printResultsSummary_TwoLoci();

void exitAfterOrder(product	**prodArr, elimOrder *elimOrderArrs);


void initResultList();
void freeResultList();
void freeList(resNode *resPtr);
void reverseResList();

boolean calcLikeNoLinkage(elimOrder	*elimOrderArrs, boolean	*areValsElim,
						  boolean *isChangedNet);
void updateLogLikes(int numPeds);

double envEvaluateLike(double *x, void* funcData);


#endif /* _GLOBALS_H_ */
