/* $Id: Bucket.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_BUCKET_H)
#define _BUCKET_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <vector>
using namespace std;

#include <string.h>
#include "DiscreteVar.h"
#include "FuncMatrix.h"
#include "DiscreteFunc.h"
#include "InferEng_Settings.h"
#include "ValHolder.h"

class Bucket 
{
	friend class BucketEliminator; 
public:
	Bucket();
	Bucket(int index);
	virtual ~Bucket();

	inline FuncMatrix*		getMatrixAt(int i)			{return this->matrixs[i];}
	inline int				getNumMatrices()			{return numMatrices;}
	inline void				addMatrix(FuncMatrix* func)	{matrixs.push_back(func);++numMatrices;}
	inline void				removeMatrix(int place)		
	{vector<FuncMatrix*>::iterator result = matrixs.begin();
	result += place;
	if (result != matrixs.end())  matrixs.erase(result); --numMatrices;}
	inline int				getIndex()					{return index;}
	inline bool				isEmpty()					{return (numMatrices==0);}
	
	void					init(ValHolder* vh);
	void					emptyBucket();
	int						getNextBucket();
	FuncMatrix*				eliminateVariable(int var, int totalVars, bool mode);
	FuncMatrix*				combineAllMatrices();
	void					combineTwoMatrices(int mat1, int mat2);
	void					combineTwoLastMatrices(int var, int numVals, int totalVars);
	void					combineTwoLastMatricesMAP(int var, int numVals, int totalVars);
	void					eliminateBucket();
	void					copyBucket(Bucket* buck, int numVars);

	


protected:
	vector<FuncMatrix*>	matrixs;	/* a vector of matrices in the bucket */
	int							numMatrices; /* number of matrices in matrixs */
	int							index;		/* ID number of the bucket */
	ValHolder*					valholder;
};

//*********************************************************************************88
// Implementation

Bucket::Bucket() {
	
	this->numMatrices = 0;
}


Bucket::Bucket(int index) {

	this->index = index;
	this->numMatrices = 0;
}

Bucket::~Bucket() {

	for (int i=0; i<numMatrices ;i++) {
		if (this->matrixs[i] != NULL) {
			delete matrixs[i];
		}
	}
}



void Bucket::init(ValHolder* vh) {
	valholder = vh;
}


void Bucket::emptyBucket() {

	matrixs.clear();
	numMatrices = 0;
}


/*int	Bucket::getNextBucket() {

	int highPriorBucket = MAX_INDEX, currBucket;

	for (int m=0; m<this->numMatrices ;m++) {
		currBucket = matrixs[m]->getHighPriorIndex();
		if (currBucket < smallestBucket) {
			smallestBucket = currBucket;
		}
	}

	return smallestBucket;
}*/


/* eliminate a variable and combine the functionMatrices */
/* that contains this variable (usually all of them) */
/* to a single functionMatrix */
FuncMatrix* Bucket::eliminateVariable(int var, int totalVars, bool mode) {
	
	/* combines the matrices so that var is the last (LSB) variable in the new function */
//	FuncMatrix* func = this->combineAllMatrices();
//	func->removeVariable(var);

	for (int i=this->numMatrices-2; i>0 ;i--) {
		combineTwoMatrices(0,i);
	}

	int varIndx = matrixs[0]->getVarPlace(var);
//	int numVals = matrixs[0]->variables[matrixs[0]->varIndices[var]]->getNumVals();
	int numVals = matrixs[0]->variables[varIndx]->getNumVals();

	if (numMatrices == 1) {
		if (mode == INFERENCE_MODE) {
			matrixs[0]->removeVariable(var);
		}
		else {
			matrixs[0]->removeVariableMAP(var);
		}
	}
	else if (numVals == 1) {
		combineTwoMatrices(0,1);

		if (mode == INFERENCE_MODE) {
			matrixs[0]->removeVariable(var);
		}
		else {
			matrixs[0]->removeVariableMAP(var);
		}
	}
	else {

		/* there are 2 last matrices left. combine them while removing var */
		/* to save space of the combined matrix with var */
		if (mode == INFERENCE_MODE) {
			combineTwoLastMatrices(var, numVals, totalVars);
		}
		else {
			combineTwoLastMatricesMAP(var, numVals, totalVars);
		}
	}

//	return func;
	return matrixs[0];
}


/* combine all the matrices in the bucket to a single FuncMatrix */
/* combination is done iteratively by multiplying two matrices at a time */
/* an entry called Enew in the new matrix, which is a result of the combination */
/* of two matrices, is the multiplication of the values at the entries E1 and E2 */
/*  which are the projections of Enew on the two matrices */ 
/* the work done when combining two matrices A and B is |A|*|B|/common */
/* where common is the number of values of the variables that appear in both matrices */
FuncMatrix* Bucket::combineAllMatrices() {
	
	/* in each iteration combine the first matrix with the last one */
	for (int i=this->numMatrices-1; i>0 ;i--) {
		combineTwoMatrices(0,i);
	}

	return matrixs[0];
}


/* combine the matrices at positions mat1 and mat2 into a matrix at mat1 */
/* the other one is deleted from the bucket */
/* the MSB variables in the new matrix (the last ones) */
/* are the variables that are unique to mat2 */
void Bucket::combineTwoMatrices(int mat1, int mat2) {
	
	FuncMatrix* func1 = matrixs[mat1];
	FuncMatrix* func2 = matrixs[mat2];

	func1->operator *=(func2);

	/* delete mat2 from the bucket */
	delete matrixs[mat2];
	this->removeMatrix(mat2);
//	this->matrixs.erase(&matrixs[mat2]);
}



/* copy all the information in this to the Bucket buck */
void Bucket::copyBucket(Bucket* buck, int numVars) {

	buck->index = this->index;
	buck->numMatrices = this->numMatrices;

	FuncMatrix* func;

	for (int i=0; i<numMatrices ;i++) {

		func = new FuncMatrix(numVars, valholder);
	/*	if (func[i].varIndices == NULL) {
			func[i].varIndices = new int[numVars];
		}
		if (func[i].instantVals == NULL) {
			func[i].instantVals = new int[numVars]; 
		}
		for (int j=0; j<numVars ;j++) {
			func[i].varIndices[j] = NO_VALUE;
		}*/
		
		this->matrixs[i]->copyMatrix(func);
		buck->matrixs.push_back(func);
	}
	
}


void Bucket::combineTwoLastMatrices(int var, int numVals, int totalVars) {

	FuncMatrix* func0;
	FuncMatrix* func1;


	int  varIndx;


	/* for efficiency put in func0 the larger function */
	if (matrixs[0]->numberOfEntries > matrixs[1]->numberOfEntries) {
		func0 = matrixs[0];
		func1 = matrixs[1];
	}
	else {
		func0 = matrixs[1];
		func1 = matrixs[0];
	}

	/* to avoid very small numbers (below MIN_DBL) */
	/* get the smallest number out and add it to each value (which is a log value) */
	_prob outFactor;

	FuncMatrix* newFunc = new FuncMatrix(totalVars, valholder);

	/* count the number of variables to be in the new function */
	int numNewVariables = func0->numVariables-1;
	DiscreteVar* nVar;
	int numFirstVars = 0;
	for (int v=0; v<func1->numVariables ;v++) {
		 nVar = func1->variables[v];

		/* if the variable is not in func0 count it */

//		if (func0->varIndices[nVar->index] == NO_VALUE) {
		varIndx = func0->getVarPlace(nVar->index);
		if (varIndx == NO_VALUE) {	
			if (nVar->index != var) {
				++numNewVariables;
				++numFirstVars;
			}
		}
	}

	newFunc->variables = new DiscreteVar*[numNewVariables];

	/* additional arrays to efficiently fill the matrix */
	int* firstVals = new int[numFirstVars];
	int* firstNumVals = new int[numFirstVars];
	int* firstNumValsNew = new int[numFirstVars];
	int* firstNumVals1 = new int[func1->numVariables-1];
	int* indicesInFirst = new int[func1->numVariables-1];

	 
	/* create a new probability matrix  - check its size */
	int newTableSize = 1;
	int firstTableSize=1, lastTableSize=1;

	for (int v0=0; v0<func0->numVariables ;v0++) {
		nVar = func0->variables[v0];

		if (nVar->index != var) {

			/* fill the variables in newFunc from the end */
			++newFunc->numVariables;
			newFunc->variables[numNewVariables-newFunc->numVariables] = nVar;
//			newFunc->varIndices[nVar->index] = numNewVariables-newFunc->numVariables;
			newTableSize *= nVar->numVals;
		}
	}
	for (int v1=0, seenVars=0; v1<func1->numVariables ;v1++) {
		 nVar = func1->variables[v1];

		/* if the variable is not in newFunc, add it at the end of the vector */
//		if (newFunc->varIndices[nVar->index] == NO_VALUE) {	
		varIndx = newFunc->getVarPlace(nVar->index, numNewVariables-newFunc->numVariables);
		if (varIndx == NO_VALUE) {	
			if (nVar->index != var) {

				/* fill the variables in newFunc from the end */
				++newFunc->numVariables;
				newFunc->variables[numNewVariables-newFunc->numVariables] = nVar;
//				newFunc->varIndices[nVar->index] = numNewVariables-newFunc->numVariables;
				firstTableSize *= nVar->numVals;

				/* mark where the variables of func1 are in newFunc to fill the matrix */
				firstNumVals[numNewVariables-newFunc->numVariables] = nVar->numVals;
				indicesInFirst[numNewVariables-newFunc->numVariables] = nVar->index;
			}
		}
		else {
			indicesInFirst[numFirstVars+seenVars] = nVar->index;
			++seenVars;
			
		}
	}

	lastTableSize = newTableSize;
	newTableSize *= firstTableSize;

	int numVars0 = func0->numVariables;
	int numVars1 = func1->numVariables;
	
	/* initialize firstNumVals1 with the magnitute of every variable in firstVars */
	int indx1;
	for (int a=0; a<numVars1-1 ;a++) {

//		indx1 = func1->varIndices[indicesInFirst[a]];
		indx1 = func1->getVarPlace(indicesInFirst[a]);

		firstNumVals1[a] = 1;
		for (int c=0; c<indx1 ;c++) {
			firstNumVals1[a] *= func1->variables[c]->numVals;	
		}
	}

	/* initialize firstNumValsNew with the magnitute of every variable in firstVars */
	for (int b=0; b<numFirstVars ;b++) {

//		indx1 = newFunc->varIndices[indicesInFirst[b]];
		indx1 = newFunc->getVarPlace(indicesInFirst[b]);

		firstNumValsNew[b] = 1;
		for (int c=0; c<indx1 ;c++) {
			firstNumValsNew[b] *= newFunc->variables[c]->numVals;	
		}
	}

	if (newFunc->probs != NULL) {
		delete[] newFunc->probs;
	}

	newFunc->probs = new _prob[newTableSize];
	newFunc->numberOfEntries = newTableSize;

	/* multiply the value at func0 by the value at func1 for the projection of the entry */
	_prob prob0, prob1;
	
	

	/* first do it for val=0 and then add the rest */
	int nVals;
	int indx;
	int numVars = newFunc->numVariables;
//	short* vals = newFunc->instantVals;
	short* vals = valholder->vals;
	
	DiscreteVar** funcVars = func0->variables;;
	DiscreteVar* tmpVar;
	int startingValue;

//	newFunc->instantVals[var] = 0;
	vals[var] = 0;

	/* calculate the increment in func1 when the value of var increases */
	int varIncrement=1;
//	indx1 = func1->varIndices[var];
	indx1 = func1->getVarPlace(var);

	for (int d=0; d<indx1 ;d++) {
		varIncrement *= func1->variables[d]->numVals;
	}

	/* calculate the magnitude of the first variable in newFunc not unique to func1 */
	startingValue = 1;
	for (int e=0; e<numFirstVars ;e++) {
		startingValue *= firstNumVals[e];
	}

	int preIndx; 
	
	

	for (int j=0, pos=0; j<lastTableSize ;j++) {

		pos = j*firstTableSize;
		/* get probability in func0 */
		indx = pos/startingValue;
			
		for (int v=numFirstVars; v<numVars ;v++) {
				
			tmpVar = newFunc->variables[v];
			nVals = tmpVar->numVals;
			vals[tmpVar->index] = indx % nVals;
			indx /= nVals;
		}

		/* inline the equivalent of toIndex(newFunc->instantVals) */
		/* for func0 */
		indx=0;
		
		for (int v0=numVars0-1; v0>0 ;v0--) {
			indx += vals[funcVars[v0]->index];
			indx *= funcVars[v0-1]->numVals;
		}
		indx += vals[funcVars[0]->index];
		prob0 = func0->probs[indx];

		/* no use of making the computations for this entry if it's 0 */
		if (prob0 > MINUS_INF) {
			

			/* compute the preIndx for variables of func1 not in the first variables */
			preIndx = 0;
			for (int v1=numFirstVars; v1<numVars1-1 ;v1++) {
				preIndx += firstNumVals1[v1] * vals[indicesInFirst[v1]];
			}
			
			for (int k=0; k<firstTableSize ;k++) {

				/* inline the equivalent of toVarsVals(j) */
				/* update only the values of the first variables in newFunc */
				indx = pos;
				
				for (int v=0; v<numFirstVars ;v++) {
					firstVals[v] = (indx/firstNumValsNew[v]) % firstNumVals[v];
				}
				/* end of inline toVarsVals(j) */

				indx = preIndx;
				for (int v1=0; v1<numFirstVars ;v1++) {
					indx += firstNumVals1[v1] * firstVals[v1];
				}
				prob1 = func1->probs[indx];
				/* end of inline toIndex() */

				newFunc->probs[pos] = prob0 + prob1;

				++pos;
			}
		}
		else {
			for (int k=0; k<firstTableSize ;k++) {
				newFunc->probs[pos] = MINUS_INF;
				++pos;
			}
		}
	}

	for (int val=1; val<numVals ;val++) {

	//	newFunc->instantVals[var] = val;
		vals[var] = val;

		for (int j=0, pos=0; j<lastTableSize ;j++) {

			pos = j*firstTableSize;
			/* get probability in func0 */
			indx = pos/startingValue;
				
			for (int v=numFirstVars; v<numVars ;v++) {
					
				tmpVar = newFunc->variables[v];
				nVals = tmpVar->numVals;
				vals[tmpVar->index] = indx % nVals;
				indx /= nVals;
			}

			/* inline the equivalent of toIndex(newFunc->instantVals) */
			/* for func0 */
			indx=0;
			
			for (int v0=numVars0-1; v0>0 ;v0--) {
				indx += vals[funcVars[v0]->index];
				indx *= funcVars[v0-1]->numVals;
			}
			indx += vals[funcVars[0]->index];
			prob0 = func0->probs[indx];

			if (prob0 > MINUS_INF) {
			
				/* compute the preIndx for variables of func1 not in the first variables */
				preIndx = val * varIncrement;
				for (int v1=numFirstVars; v1<numVars1-1 ;v1++) {
					preIndx += firstNumVals1[v1] * vals[indicesInFirst[v1]];
				}
				
				for (int k=0; k<firstTableSize ;k++) {

					/* inline the equivalent of toVarsVals(j) */	
					/* update only the values of the first variables in newFunc */
					indx = pos;
					
					for (int v=0; v<numFirstVars ;v++) {
						firstVals[v] = (indx/firstNumValsNew[v]) % firstNumVals[v];
					}
					/* end of inline toVarsVals(j) */

					indx = preIndx;
					for (int v1=0; v1<numFirstVars ;v1++) {
						indx += firstNumVals1[v1] * firstVals[v1];
					}
					prob1 = func1->probs[indx] + prob0;
					/* end of inline toIndex() */

					/* check if computations can be avoided */
					if (prob1 <= MINUS_INF) {
						++pos;
						continue;
					}
					if (newFunc->probs[pos] <= MINUS_INF) {
						newFunc->probs[pos] = prob1;
						++pos;
						continue;
					}

					/* choose the biggest of the two */
					/* this is the common multiplier */
					outFactor = prob1;

					if (newFunc->probs[pos] > outFactor) {
						outFactor = newFunc->probs[pos];
						newFunc->probs[pos] = log(exp(prob1 - outFactor)+1)+outFactor;
					}
					else {
						newFunc->probs[pos] = log(exp(newFunc->probs[pos] - outFactor)+1) + outFactor;
					}
					
					++pos;
				}
			}
			else {
				pos += firstTableSize;
			}
		}

	}
	
	/* delete func0 and func1 from the bucket and add newFunc instead */
	delete matrixs[1];
	this->removeMatrix(1);
	delete matrixs[0];
	this->removeMatrix(0);
	this->addMatrix(newFunc);

	/* delete additional arrays */
	delete[] firstVals;
	delete[] firstNumVals;
	delete[] firstNumValsNew;
	delete[] firstNumVals1;
	delete[] indicesInFirst;
}



/* The same as combineTwoLastMatrices, but removes var by taking a maximum 
/* over its vals */
void Bucket::combineTwoLastMatricesMAP(int var, int numVals, int totalVars) {

	FuncMatrix* func0;
	FuncMatrix* func1;


	int  varIndx;


	/* for efficiency put in func0 the larger function */
	if (matrixs[0]->numberOfEntries > matrixs[1]->numberOfEntries) {
		func0 = matrixs[0];
		func1 = matrixs[1];
	}
	else {
		func0 = matrixs[1];
		func1 = matrixs[0];
	}

	/* to avoid very small numbers (below MIN_DBL) */
	/* get the smallest number out and add it to each value (which is a log value) */
	_prob outFactor;

	FuncMatrix* newFunc = new FuncMatrix(totalVars, valholder);

	/* count the number of variables to be in the new function */
	int numNewVariables = func0->numVariables-1;
	DiscreteVar* nVar;
	int numFirstVars = 0;
	for (int v=0; v<func1->numVariables ;v++) {
		 nVar = func1->variables[v];

		/* if the variable is not in func0 count it */

//		if (func0->varIndices[nVar->index] == NO_VALUE) {
		varIndx = func0->getVarPlace(nVar->index);
		if (varIndx == NO_VALUE) {	
			if (nVar->index != var) {
				++numNewVariables;
				++numFirstVars;
			}
		}
	}

	newFunc->variables = new DiscreteVar*[numNewVariables];

	/* additional arrays to efficiently fill the matrix */
	int* firstVals = new int[numFirstVars];
	int* firstNumVals = new int[numFirstVars];
	int* firstNumValsNew = new int[numFirstVars];
	int* firstNumVals1 = new int[func1->numVariables-1];
	int* indicesInFirst = new int[func1->numVariables-1];

	 
	/* create a new probability matrix  - check its size */
	int newTableSize = 1;
	int firstTableSize=1, lastTableSize=1;

	for (int v0=0; v0<func0->numVariables ;v0++) {
		nVar = func0->variables[v0];

		if (nVar->index != var) {

			/* fill the variables in newFunc from the end */
			++newFunc->numVariables;
			newFunc->variables[numNewVariables-newFunc->numVariables] = nVar;
//			newFunc->varIndices[nVar->index] = numNewVariables-newFunc->numVariables;
			newTableSize *= nVar->numVals;
		}
	}
	for (int v1=0, seenVars=0; v1<func1->numVariables ;v1++) {
		 nVar = func1->variables[v1];

		/* if the variable is not in newFunc, add it at the end of the vector */
//		if (newFunc->varIndices[nVar->index] == NO_VALUE) {	
		varIndx = newFunc->getVarPlace(nVar->index, numNewVariables-newFunc->numVariables);
		if (varIndx == NO_VALUE) {	
			if (nVar->index != var) {

				/* fill the variables in newFunc from the end */
				++newFunc->numVariables;
				newFunc->variables[numNewVariables-newFunc->numVariables] = nVar;
//				newFunc->varIndices[nVar->index] = numNewVariables-newFunc->numVariables;
				firstTableSize *= nVar->numVals;

				/* mark where the variables of func1 are in newFunc to fill the matrix */
				firstNumVals[numNewVariables-newFunc->numVariables] = nVar->numVals;
				indicesInFirst[numNewVariables-newFunc->numVariables] = nVar->index;
			}
		}
		else {
			indicesInFirst[numFirstVars+seenVars] = nVar->index;
			++seenVars;
			
		}
	}

	lastTableSize = newTableSize;
	newTableSize *= firstTableSize;

	int numVars0 = func0->numVariables;
	int numVars1 = func1->numVariables;
	
	/* initialize firstNumVals1 with the magnitute of every variable in firstVars */
	int indx1;
	for (int a=0; a<numVars1-1 ;a++) {

//		indx1 = func1->varIndices[indicesInFirst[a]];
		indx1 = func1->getVarPlace(indicesInFirst[a]);

		firstNumVals1[a] = 1;
		for (int c=0; c<indx1 ;c++) {
			firstNumVals1[a] *= func1->variables[c]->numVals;	
		}
	}

	/* initialize firstNumValsNew with the magnitute of every variable in firstVars */
	for (int b=0; b<numFirstVars ;b++) {

//		indx1 = newFunc->varIndices[indicesInFirst[b]];
		indx1 = newFunc->getVarPlace(indicesInFirst[b]);

		firstNumValsNew[b] = 1;
		for (int c=0; c<indx1 ;c++) {
			firstNumValsNew[b] *= newFunc->variables[c]->numVals;	
		}
	}

	if (newFunc->probs != NULL) {
		delete[] newFunc->probs;
	}

	newFunc->probs = new _prob[newTableSize];
	newFunc->numberOfEntries = newTableSize;

	/* multiply the value at func0 by the value at func1 for the projection of the entry */
	_prob prob0, prob1;
	
	

	/* first do it for val=0 and then take the maximum from the rest */
	int nVals;
	int indx;
	int numVars = newFunc->numVariables;
//	short* vals = newFunc->instantVals;
	short* vals = valholder->vals;
	
	DiscreteVar** funcVars = func0->variables;;
	DiscreteVar* tmpVar;
	int startingValue;

//	newFunc->instantVals[var] = 0;
	vals[var] = 0;

	/* calculate the increment in func1 when the value of var increases */
	int varIncrement=1;
//	indx1 = func1->varIndices[var];
	indx1 = func1->getVarPlace(var);

	for (int d=0; d<indx1 ;d++) {
		varIncrement *= func1->variables[d]->numVals;
	}

	/* calculate the magnitude of the first variable in newFunc not unique to func1 */
	startingValue = 1;
	for (int e=0; e<numFirstVars ;e++) {
		startingValue *= firstNumVals[e];
	}

	int preIndx; 
	
	

	for (int j=0, pos=0; j<lastTableSize ;j++) {

		pos = j*firstTableSize;
		/* get probability in func0 */
		indx = pos/startingValue;
			
		for (int v=numFirstVars; v<numVars ;v++) {
				
			tmpVar = newFunc->variables[v];
			nVals = tmpVar->numVals;
			vals[tmpVar->index] = indx % nVals;
			indx /= nVals;
		}

		/* inline the equivalent of toIndex(newFunc->instantVals) */
		/* for func0 */
		indx=0;
		
		for (int v0=numVars0-1; v0>0 ;v0--) {
			indx += vals[funcVars[v0]->index];
			indx *= funcVars[v0-1]->numVals;
		}
		indx += vals[funcVars[0]->index];
		prob0 = func0->probs[indx];

		/* no use of making the computations for this entry if it's 0 */
		if (prob0 > MINUS_INF) {
			

			/* compute the preIndx for variables of func1 not in the first variables */
			preIndx = 0;
			for (int v1=numFirstVars; v1<numVars1-1 ;v1++) {
				preIndx += firstNumVals1[v1] * vals[indicesInFirst[v1]];
			}
			
			for (int k=0; k<firstTableSize ;k++) {

				/* inline the equivalent of toVarsVals(j) */
				/* update only the values of the first variables in newFunc */
				indx = pos;
				
				for (int v=0; v<numFirstVars ;v++) {
					firstVals[v] = (indx/firstNumValsNew[v]) % firstNumVals[v];
				}
				/* end of inline toVarsVals(j) */

				indx = preIndx;
				for (int v1=0; v1<numFirstVars ;v1++) {
					indx += firstNumVals1[v1] * firstVals[v1];
				}
				prob1 = func1->probs[indx];
				/* end of inline toIndex() */

				newFunc->probs[pos] = prob0 + prob1;

				++pos;
			}
		}
		else {
			for (int k=0; k<firstTableSize ;k++) {
				newFunc->probs[pos] = MINUS_INF;
				++pos;
			}
		}
	}

	// now take the maximum over the rest of the vals
	for (int val=1; val<numVals ;val++) {

	//	newFunc->instantVals[var] = val;
		vals[var] = val;

		for (int j=0, pos=0; j<lastTableSize ;j++) {

			pos = j*firstTableSize;
			/* get probability in func0 */
			indx = pos/startingValue;
				
			for (int v=numFirstVars; v<numVars ;v++) {
					
				tmpVar = newFunc->variables[v];
				nVals = tmpVar->numVals;
				vals[tmpVar->index] = indx % nVals;
				indx /= nVals;
			}

			/* inline the equivalent of toIndex(newFunc->instantVals) */
			/* for func0 */
			indx=0;
			
			for (int v0=numVars0-1; v0>0 ;v0--) {
				indx += vals[funcVars[v0]->index];
				indx *= funcVars[v0-1]->numVals;
			}
			indx += vals[funcVars[0]->index];
			prob0 = func0->probs[indx];

			if (prob0 > MINUS_INF) {
			
				/* compute the preIndx for variables of func1 not in the first variables */
				preIndx = val * varIncrement;
				for (int v1=numFirstVars; v1<numVars1-1 ;v1++) {
					preIndx += firstNumVals1[v1] * vals[indicesInFirst[v1]];
				}
				
				for (int k=0; k<firstTableSize ;k++) {

					/* inline the equivalent of toVarsVals(j) */	
					/* update only the values of the first variables in newFunc */
					indx = pos;
					
					for (int v=0; v<numFirstVars ;v++) {
						firstVals[v] = (indx/firstNumValsNew[v]) % firstNumVals[v];
					}
					/* end of inline toVarsVals(j) */

					indx = preIndx;
					for (int v1=0; v1<numFirstVars ;v1++) {
						indx += firstNumVals1[v1] * firstVals[v1];
					}
					prob1 = func1->probs[indx] + prob0;
					/* end of inline toIndex() */

					/* check if computations can be avoided */
					if (prob1 <= MINUS_INF) {
						++pos;
						continue;
					}
					if (newFunc->probs[pos] <= MINUS_INF) {
						newFunc->probs[pos] = prob1;
						++pos;
						continue;
					}

					/* choose the biggest of the two */
					outFactor = prob1;

					if (newFunc->probs[pos] < prob1) {
						newFunc->probs[pos] = prob1;
					}
					
					++pos;
				}
			}
			else {
				pos += firstTableSize;
			}
		}

	}
	
	/* delete func0 and func1 from the bucket and add newFunc instead */
	delete matrixs[1];
	this->removeMatrix(1);
	delete matrixs[0];
	this->removeMatrix(0);
	this->addMatrix(newFunc);

	/* delete additional arrays */
	delete[] firstVals;
	delete[] firstNumVals;
	delete[] firstNumValsNew;
	delete[] firstNumVals1;
	delete[] indicesInFirst;
}	




#endif // !defined(_BUCKET_H)


