/* $Id: BucketsEliminator.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_BUCKETELIMINATOR_H)
#define _BUCKETELIMINATOR_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <vector>
using namespace std;
#include <string.h>
#include "DiscreteVar.h"
#include "DiscreteFunc.h"
#include "Bucket.h"
#include "InferEng_Settings.h"



class BucketEliminator 
{
	friend class CInferEngine;
public:
	BucketEliminator(ValHolder*	vh);
	BucketEliminator(int numBuckets, ValHolder*	vh);
	virtual ~BucketEliminator();

	inline Bucket*	getBucketAt(int i)				{return &buckets[i];}
	inline int		getNumBucket()					{return numBuckets;}
	inline void		setPriority(int var, int p)		{priority[var] = p;}
	inline void		setLowerPriority(int var, int p){if(p<priority[var]) priority[var]=p;}
	inline int		getPriority(int var)			{return priority[var];}
	inline int*		getPriority()					{return priority;}
	inline _prob	getFactor()						{return exp(multFactor);}
	inline void		setExactPriority(int var, int p){priority[var]=p;}
	inline void		setPriorityTo(int v, int p)		{if (priority[v]>p) priority[v]=p;}
	

//	void			Init(SimpleModel* sm);
	void			Init(BayesNet* bn);
	void			copyBE(BucketEliminator* be);
	void			eliminateBucket(int buc);
	void			eliminateAllPossiblesCond(bool mode);
	void			eliminateAllPossibles(bool mode);
	void			eliminateAllPossiblesLimited(bool mode);
	void			eliminateAllPossiblesFull(bool mode);
	FuncMatrix*		combineAllTo1Matrix();
	void			setPrioritiesTo(Cluster *cl, int p);
	void			setExactPriorities(Cluster *cl, int p);
	void			setPrioritiesTo(DiscreteFunc *func, int p);
	void			setExactPriorities(DiscreteFunc *func, int p);
	void			setPrioritiesTo(Cluster *cl, int* order);
	void			setExactPriorities(Cluster *cl, int* order);
	void			setPrioritiesTo(DiscreteFunc *func, int* order);
	void			setExactPriorities(DiscreteFunc *func, int* order);
	void			arrangeMatricesInBuckets();

	
	


protected:
	Bucket*			buckets;
	int				numBuckets;
	int*			priority;
	_prob			multFactor;
	int				totalNumMatrices;
	int				maxMatrixSize;
	int				sumMatrixSize;
	int*			varsInBucket;
	bool*			limited;
	ValHolder*		valholder;
};

//*********************************************************************************88
// Implementation

BucketEliminator::BucketEliminator(ValHolder*	vh) {

	this->buckets = NULL;
	this->priority = NULL;
	this->numBuckets = 0;
	this->multFactor = 0;

	this->varsInBucket = NULL;
	this->limited = NULL;

	/* initialize extra information members */
	totalNumMatrices = 0;
	maxMatrixSize = 0;
	sumMatrixSize = 0;

	valholder = vh;

	
}

BucketEliminator::BucketEliminator(int numBuckets, ValHolder*	vh) {

	this->buckets = new Bucket[numBuckets];

	for (int k=0; k<numBuckets ;k++) {
		buckets[k].init(vh);
	}

	varsInBucket = new int[numBuckets];
	limited = new bool[numBuckets];


	this->priority = new int[numBuckets];
	for (int i=0; i<numBuckets ;i++) {
		this->priority[i] = MAX_PRIORITY;
		this->varsInBucket[i] = 1;
		this->limited[i] = false;
	}

	this->multFactor = 0;

	this->numBuckets = numBuckets;

	/* initialize extra information members */
	totalNumMatrices = 0;
	maxMatrixSize = 0;
	sumMatrixSize = 0;

	valholder = vh;
}

BucketEliminator::~BucketEliminator() {

	if (buckets) {
		delete[] buckets;
	}
	if (this->priority) {
		delete[] this->priority;
	}

	if (this->varsInBucket) {
		delete[] this->varsInBucket;
	}
	if (this->limited) {
		delete[] this->limited;
	}
}


/* initializes the data structures (if weren't allocated in the constructor) */
/* and turns every potential of a cluster in sm to a FuncMatrix */
/* puts each matrix in a bucket according to the priority of the variables */
void BucketEliminator::Init(BayesNet* bn) {

	int highPrior;

	if (buckets == NULL) {
		this->buckets = new Bucket[bn->numVariables];
		this->numBuckets = bn->numVariables;
		for (int k=0; k<numBuckets ;k++) {
			buckets[k].init(valholder);
		}
	}
	if (priority == NULL) {
		this->priority = new int[bn->numVariables];
	}

	/* create a funcmatrix from the potential of every cluster */
	/* insert it to the bucket of the highest priority variable */
	for (int c=0; c<bn->numFunctions ;c++) {

		if (bn->functions[c]->getNumInstants() > 1) {
			/* create the funcMatrix and fill it */
			FuncMatrix* fm = new FuncMatrix(bn->numVariables, bn->functions[c], valholder);
			
			/* insert the funcMatrix to the proper bucket */
			highPrior = fm->getHighPriorIndex(this->priority);
			buckets[highPrior].addMatrix(fm);

			/* update the information members */
			++this->totalNumMatrices;
			this->sumMatrixSize += fm->numberOfEntries;
			if (this->maxMatrixSize < fm->numberOfEntries) {
				maxMatrixSize = fm->numberOfEntries;
			}
		}
		else {
			this->multFactor += log(bn->functions[c]->getProbabilityAt(0,0));
		}
	}
}


/* makes the BE be the copy of this */
void BucketEliminator::copyBE(BucketEliminator* be) {

	be->numBuckets = this->numBuckets;
	be->buckets = new Bucket[this->numBuckets];
	for (int k=0; k<numBuckets ;k++) {
		be->buckets[k].init(valholder);
	}
	be->priority = new int[numBuckets];
	be->varsInBucket = new int[numBuckets];
	be->limited = new bool[numBuckets];

	for (int i=0; i<numBuckets ;i++) {
		this->buckets[i].copyBucket(&be->buckets[i], this->numBuckets);
		be->priority[i] = this->priority[i];
		be->varsInBucket[i] = 1;
		be->limited[i] = false;
	}

	be->multFactor = this->multFactor;

	/* copy the extra information beside max that we have to compute */
	int numMats;
	for (int b=0; b<numBuckets ;b++) {
		numMats = buckets[b].numMatrices;
		for (int m=0; m<numMats ;m++) {
			if (be->maxMatrixSize < buckets[b].matrixs[m]->numberOfEntries) {
				be->maxMatrixSize = buckets[b].matrixs[m]->numberOfEntries;
			}
		}
	}
//	be->maxMatrixSize = this->maxMatrixSize;
	be->totalNumMatrices = this->totalNumMatrices;
	be->sumMatrixSize = this->sumMatrixSize;
}


void BucketEliminator::eliminateBucket(int buc) {};


/* eliminates all the variables with a non-negative priority */
void BucketEliminator::eliminateAllPossibles(bool mode) {
	
	
	int bucket=NO_BUCKET;
	int numVars, bucketTableSize;
	DiscreteVar* var;
	FuncMatrix* matrix;
	int nMats;

	/* find the bucket with highest priority */
	for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
		if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0)) {
			bucket=b;
			prior = this->priority[b];
		}
	}

	/* while there are buckets with non-negative priority */
	while(bucket >= 0) {
	
		/* check the size of the bucket */
		nMats = buckets[bucket].numMatrices;
		for (int m=0; m<nMats ;m++) {
			matrix = buckets[bucket].matrixs[m];
			numVars = matrix->numVariables;
			for (int v=0; v<numVars ;v++) {
				var = matrix->variables[v];
				this->varsInBucket[var->getIndex()] = var->getNumVals();
			}
		}
		
		int numVals = varsInBucket[bucket];
		bucketTableSize = 1;
		for (int j=0; j<numBuckets ;j++) {

			/* if the size will exceed INT_MAX set the size to maximum */ 
			if (INT_MAX/varsInBucket[j] < bucketTableSize) {
				bucketTableSize = (MAX_MATRIX_SIZE*numVals) + numVals;
				break;
			}
			bucketTableSize *= varsInBucket[j];
			varsInBucket[j] = 1;
		}
		bucketTableSize /= numVals;


		/* update the BE information */
		int nMats = buckets[bucket].numMatrices;
		for (int i=0; i<nMats ;i++) {
			this->sumMatrixSize -= buckets[bucket].matrixs[i]->numberOfEntries;
			--this->totalNumMatrices;
		}
//		printf("elimstart  numMats: %d   sum: %d   max: %d\n",totalNumMatrices,sumMatrixSize,maxMatrixSize);

		/* eliminate the variable of the bucket */
		FuncMatrix* func = this->buckets[bucket].eliminateVariable(bucket,numBuckets, mode);


/* debugging section */

//		for (int h=0; h<func->numberOfEntries ;h++) {
//			if (exp(func->probs[h]) < 0) {
//				printf("**\n");
//			}
//		}



/* **************** */

		this->priority[bucket] = EMPTY_BUCKET;
		this->buckets[bucket].emptyBucket();

		/* move the remaining function to the highest priority bucket */
		/* if there is no remaining function, but only a factor */
		/* multiply the member factor */
		if (func->getNumEntries() == 1) {
			this->multFactor += func->getProbabilityAt(0);
			delete func;
		}
		else {
			int nextBucket = func->getHighPriorIndex(this->priority);
			this->buckets[nextBucket].addMatrix(func);

			/* update BE information */
			++this->totalNumMatrices;
			this->sumMatrixSize += func->numberOfEntries;
			if (this->maxMatrixSize < func->numberOfEntries) {
				maxMatrixSize = func->numberOfEntries;
			}
//			printf("elimend numMats: %d   sum: %d   max: %d\n",totalNumMatrices,sumMatrixSize,maxMatrixSize);
		}
		
		/* find the next bucket with highest priority */
		bucket=NO_BUCKET;
		for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
			if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0)) {
				bucket=b;
				prior = this->priority[b];
			}
		}
	}
}



/* eliminates all the variables with a non-negative priority */
/* except those that create a too big matrix */
/* these are not removed till higher priority variables */
/* get a non-negative priorrity */
void BucketEliminator::eliminateAllPossiblesLimited(bool mode) {
	
	int bucket=NO_BUCKET;
	int numVars, bucketTableSize;
	DiscreteVar* var;
	FuncMatrix* matrix;
	int nMats;

	/* find the bucket with highest priority */
	for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
		if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0)) {
			bucket=b;
			prior = this->priority[b];
		}
	}

	/* while there are buckets with non-negative priority */
	while(bucket >= 0) {
	
		/* check if the bucket is limited */
		nMats = buckets[bucket].numMatrices;
		for (int m=0; m<nMats ;m++) {
			matrix = buckets[bucket].matrixs[m];
			numVars = matrix->numVariables;
			for (int v=0; v<numVars ;v++) {
				var = matrix->variables[v];
				this->varsInBucket[var->getIndex()] = var->getNumVals();
			}
		}
		
		int numVals = varsInBucket[bucket];
		bucketTableSize = 1;
		for (int j=0; j<numBuckets ;j++) {

			/* if the size will exceed INT_MAX set the size to maximum so it is limited */ 
			if (INT_MAX/varsInBucket[j] < bucketTableSize) {
				bucketTableSize = (MAX_MATRIX_SIZE*numVals) + numVals;
				break;
			}
			bucketTableSize *= varsInBucket[j];
			varsInBucket[j] = 1;
		}
		bucketTableSize /= numVals;

		/* if table size will not exceed MAX_MATRIX_SIZE do a regular elimination */
		if (bucketTableSize <= MAX_MATRIX_SIZE) {

			/* update the BE information */
			nMats = buckets[bucket].numMatrices;
			for (int i=0; i<nMats ;i++) {
				this->sumMatrixSize -= buckets[bucket].matrixs[i]->numberOfEntries;
				--this->totalNumMatrices;
			}

			/* eliminate the variable of the bucket */
			FuncMatrix* func = this->buckets[bucket].eliminateVariable(bucket,numBuckets, mode);


			this->priority[bucket] = EMPTY_BUCKET;
			this->buckets[bucket].emptyBucket();

			/* move the remaining function to the highest priority bucket */
			/* if there is no remaining function, but only a factor */
			/* multiply the member factor */
			if (func->getNumEntries() == 1) {
				this->multFactor += func->getProbabilityAt(0);
				delete func;
			}
			else {
				int nextBucket = func->getHighPriorIndex(this->priority, limited);

				/* if not all the remaining variables are limited */
				if (nextBucket != MAX_INDEX) {
					this->buckets[nextBucket].addMatrix(func);
				}
				else {
					nextBucket = func->getHighPriorIndex(this->priority);
					this->buckets[nextBucket].addMatrix(func);
				}

				/* update BE information */
				++this->totalNumMatrices;
				this->sumMatrixSize += func->numberOfEntries;
				if (this->maxMatrixSize < func->numberOfEntries) {
					maxMatrixSize = func->numberOfEntries;
				}
//				printf("elimend %d numMats: %d   sum: %d   max: %d\n",bucket,totalNumMatrices,sumMatrixSize,maxMatrixSize);
			}
		}
		else {

			int nextBucket;
			FuncMatrix* func;
			/* mark this bucket as limited so not to attempt to eliminate it again */
			/* and send matrices to the next bucket they should be in (if any) */
			limited[bucket] = true;
	
			for (int mat=nMats-1; mat>=0 ;mat--) {

				nextBucket = buckets[bucket].matrixs[mat]->getHighPriorIndexBut(priority, 
					bucket, limited);
				if (nextBucket != bucket) {
					func = buckets[bucket].matrixs[mat];
					this->buckets[bucket].removeMatrix(mat);
					this->buckets[nextBucket].addMatrix(func);
				}
			}
		}
		/* find the next bucket with highest priority */
		bucket=NO_BUCKET;
		for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
			if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0) &&
				(limited[b] == false)) {
				bucket=b;
				prior = this->priority[b];
			}
		}

	}

	/* reset the limited flags for reuse */
	for (int k=0; k<numBuckets ;k++) {
		limited[k] = false;
	}
}


/* eliminates all the variables with a non-negative priority */
/* while conditioning on some variables in order to avoid */
/* high space capacity */
void BucketEliminator::eliminateAllPossiblesCond(bool mode) {

	this->eliminateAllPossiblesLimited(mode);
	this->arrangeMatricesInBuckets();

	int bucket=NO_BUCKET;
	int numVars, bucketTableSize;
	DiscreteVar* var;
	FuncMatrix* matrix;
	int nMats;

	/* find the bucket with highest priority */
	for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
		if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0)) {
			bucket=b;
			prior = this->priority[b];
		}
	}

	/* while there are buckets with non-negative priority */
	while(bucket >= 0) {
	
		/* check the size of the bucket */
		nMats = buckets[bucket].numMatrices;
		for (int m=0; m<nMats ;m++) {
			matrix = buckets[bucket].matrixs[m];
			numVars = matrix->numVariables;
			for (int v=0; v<numVars ;v++) {
				var = matrix->variables[v];
				this->varsInBucket[var->getIndex()] = var->getNumVals();
			}
		}
		
		int numVals = varsInBucket[bucket];
		bucketTableSize = 1;
		for (int j=0; j<numBuckets ;j++) {

			/* if the size will exceed INT_MAX set the size to maximum */ 
			if (INT_MAX/varsInBucket[j] < bucketTableSize) {
				bucketTableSize = (MAX_MATRIX_SIZE*numVals) + numVals;
				break;
			}
			bucketTableSize *= varsInBucket[j];
			varsInBucket[j] = 1;
		}
		bucketTableSize /= numVals;


		/* update the BE information */
		int nMats = buckets[bucket].numMatrices;
		for (int i=0; i<nMats ;i++) {
			this->sumMatrixSize -= buckets[bucket].matrixs[i]->numberOfEntries;
			--this->totalNumMatrices;
		}

		/* no conditioning is needed here */
		if (bucketTableSize <= MAX_MATRIX_SIZE) {
	//		printf("elimstart  numMats: %d   sum: %d   max: %d\n",totalNumMatrices,sumMatrixSize,maxMatrixSize);

			/* eliminate the variable of the bucket */
			FuncMatrix* func = this->buckets[bucket].eliminateVariable(bucket,numBuckets, mode);

			this->priority[bucket] = EMPTY_BUCKET;
			this->buckets[bucket].emptyBucket();

			/* move the remaining function to the highest priority bucket */
			/* if there is no remaining function, but only a factor */
			/* multiply the member factor */
			if (func->getNumEntries() == 1) {
				this->multFactor += func->getProbabilityAt(0);
				delete func;
			}
			else {
				int nextBucket = func->getHighPriorIndex(this->priority);
				this->buckets[nextBucket].addMatrix(func);

				/* update BE information */
				++this->totalNumMatrices;
				this->sumMatrixSize += func->numberOfEntries;
				if (this->maxMatrixSize < func->numberOfEntries) {
					maxMatrixSize = func->numberOfEntries;
				}
	//			printf("elimend numMats: %d   sum: %d   max: %d\n",totalNumMatrices,sumMatrixSize,maxMatrixSize);
			}
		}
		// conditioning is needed
		else {

			/* for every value create a new BE and add the resulting factor */

			_prob combinedResult=0;
			int mat, varPos;
			DiscreteVar* var;

			for (int val=0; val<numVals; val++) {

				BucketEliminator* tmpBE = new BucketEliminator(valholder);
				this->copyBE(tmpBE);
				tmpBE->priority[bucket] = EMPTY_BUCKET;

				
				/* change the variable in bucket to be the MSB */
				/* for easier truncation */
				Cluster* cl = new Cluster();
				cl->numVariables = 1;
				cl->Init(numBuckets, 0);

				for (mat=0; mat<nMats ;mat++) {

					matrix = tmpBE->buckets[bucket].matrixs[mat];

					if (matrix->numVariables > 1) {
						
//						varPos = matrix->varIndices[bucket];
						varPos = matrix->getVarPlace(bucket);

						var = matrix->variables[varPos];
						
						cl->variables[0] = var;
						cl->varIndices[bucket] = 0;
						cl->numInstants = var->getNumVals();
						
						matrix->changeMSB2(cl);

						cl->varIndices[bucket] = NO_VALUE;
					}
				}
				delete cl;

				// empty the bucket and put truncated matrixes in the
				// next highest priority bucket
				for (mat=0; mat<nMats ;mat++) {

					matrix = tmpBE->buckets[bucket].matrixs[mat];

					// generate a matrix where the only val is considered
					// and that does not contain the variable bucket 
					if (matrix->numVariables > 1) {
						FuncMatrix* newMatrix = matrix->truncate(bucket, val, numBuckets);
						int nextBucket = newMatrix->getHighPriorIndex(tmpBE->priority);
						tmpBE->buckets[nextBucket].addMatrix(newMatrix);
					}
					else {
						tmpBE->multFactor += matrix->probs[val];
					}
				}

				for (int m=0; m<tmpBE->buckets[bucket].numMatrices ;m++) {
					if (tmpBE->buckets[bucket].matrixs[m] != NULL) {
						delete tmpBE->buckets[bucket].matrixs[m];
					}
				}

				tmpBE->buckets[bucket].emptyBucket();

				// call elimination in the resulted BE
				tmpBE->eliminateAllPossiblesCond(mode);
			//	tmpBE->eliminateAllPossibles();
			//	resMatrix = tmpBE->combineAllTo1Matrix();
			//	int resBucket = resMatrix->getHighPriorIndex(this->priority);


				// gather the information from tmpBE
				if (mode == 0) {
					combinedResult += tmpBE->getFactor();
				}
				// MAP
				else {
					if (combinedResult < tmpBE->getFactor()) {
						combinedResult = tmpBE->getFactor();
					}
				}

				delete tmpBE;
			}

			// the conditioning already eliminated the rest of the variables
			// gather the results and return
			this->multFactor = log(combinedResult);
			return;
		}
		
		/* find the next bucket with highest priority */
		bucket=NO_BUCKET;
		for (int b=0, prior=MIN_PRIOR; b<this->numBuckets ;b++) {
			if ((this->priority[b] > prior) && (this->buckets[b].numMatrices > 0)) {
				bucket=b;
				prior = this->priority[b];
			}
		}
	}
}


/* an elimination procedure that combines eliminateAll and eliminateAllLimited */
/* to efficiently eliminate all buckets with non-negative priority */
/* this function is used in-case we have some buckets with negative priorities */
/* that can affect the efficiency of the chosen elimination order */
void BucketEliminator::eliminateAllPossiblesFull(bool mode) {

	this->eliminateAllPossiblesLimited(mode);
	this->arrangeMatricesInBuckets();
	this->eliminateAllPossibles(mode);
}


/* a function which is called after eliminateAllPossibles() */
/* combines all the matrices to one (those with DONT_TOUCH_PRIORITY are not necessarily */
/* in a single matrix so they should be combined into one) */
/* multiplies the entries of the resulting matrix with multFactor */
/* removes this function from its bucket and returns it */
FuncMatrix*	BucketEliminator::combineAllTo1Matrix() {

	FuncMatrix* func = new FuncMatrix(this->numBuckets, valholder);
	Bucket* buck;

	/* find the next bucket with highest priority */
	for (int b=0; b<this->numBuckets ;b++) {
		if (this->buckets[b].numMatrices > 0) {

			buck = &this->buckets[b];

			/* it's the first matrix found */
			if (func->getNumVars() == 0) {
				buck->combineAllMatrices()->copyMatrix(func);
			}
			else {
				FuncMatrix* tmpFunc = buck->combineAllMatrices();
				func->operator *= (tmpFunc);
			}

			/* update the BE information */
			int nMats = buck->numMatrices;
			for (int i=0; i<nMats ;i++) {
				this->sumMatrixSize -= buck->matrixs[i]->numberOfEntries;
				--this->totalNumMatrices;
			}

			/* delete the remaining matrix in buck */
			delete buck->matrixs[0];
			buck->matrixs.clear();
			buck->numMatrices = 0;
		}
	}

	func->operator *= (this->multFactor);
	this->multFactor = 0;

	return func;
}


/* set the priorities in priority to p but not higher than they already are */
void BucketEliminator::setPrioritiesTo(Cluster* cl, int p) {

	int indx;
	int numVars = cl->getNumVars();

	for (int i=0; i<numVars ;i++) {

		indx = cl->getVarAt(i)->getIndex();

		if(p < priority[indx]) {
			priority[indx] = p;
		}
	}
}


/* set the priorities in priority to p */
void BucketEliminator::setExactPriorities(Cluster* cl, int p) {

	int numVars = cl->getNumVars();

	for (int i=0; i<numVars ;i++) {

		priority[cl->getVarAt(i)->getIndex()] = p;
	
	}
}


/* set the priorities in priority according to the elimination order */
/* but not higher than they already are */
void BucketEliminator::setPrioritiesTo(Cluster* cl, int* order) {

	int indx;
	int numVars = cl->getNumVars();

	for (int i=0; i<numVars ;i++) {

		indx = cl->getVarAt(i)->getIndex();

		if(order[indx] < priority[indx]) {
			priority[indx] = order[indx];
		}
	}
}


/* set the priorities in priority according to the elimination order */
void BucketEliminator::setExactPriorities(Cluster* cl, int* order) {

	int numVars = cl->getNumVars();

	for (int i=0; i<numVars ;i++) {

		priority[cl->getVarAt(i)->getIndex()] = order[cl->getVarAt(i)->getIndex()];
	
	}
}


/* set the priorities in priority to p but not higher than they already are */
void BucketEliminator::setPrioritiesTo(DiscreteFunc* func, int p) {

	int indx = func->getVar()->getIndex();

	if(p < priority[indx]) {
		priority[indx] = p;
	}

	int numPars = func->getNumParents();

	for (int i=0; i<numPars ;i++) {

		indx = func->getParent(i)->getIndex();

		if(p < priority[indx]) {
			priority[indx] = p;
		}
	}
}


/* set the priorities in priority to p */
void BucketEliminator::setExactPriorities(DiscreteFunc* func, int p) {

	priority[func->getVar()->getIndex()] = p;
	
	for (int i=0; i<func->getNumParents() ;i++) {

		priority[func->getParent(i)->getIndex()] = p;
	}
}



/* set the priorities in priority according to the elimination order */
/* but not higher than they already are */
void BucketEliminator::setPrioritiesTo(DiscreteFunc* func, int* order) {

	int indx = func->getVar()->getIndex();

	if(order[indx] < priority[indx]) {
		priority[indx] = order[indx];
	}

	int numPars = func->getNumParents();

	for (int i=0; i<numPars ;i++) {

		indx = func->getParent(i)->getIndex();

		if(order[indx] < priority[indx]) {
			priority[indx] = order[indx];
		}
	}
}


/* set the priorities in priority according to the elimination order */
void BucketEliminator::setExactPriorities(DiscreteFunc* func, int* order) {

	priority[func->getVar()->getIndex()] = order[func->getVar()->getIndex()];
	
	for (int i=0; i<func->getNumParents() ;i++) {

		priority[func->getParent(i)->getIndex()] = order[func->getParent(i)->getIndex()];
	}
}


/* shift matrices in the buckets to the bucket of variable with highest priority */
void BucketEliminator::arrangeMatricesInBuckets() {

	Bucket* buck;
	FuncMatrix* func;
	int numVars;
	int buckIndx;
	
	/* for every function in every bucket */
	for (int b=0; b<this->numBuckets ;b++) {


		buck = &this->buckets[b];
		for (int f=0; f<buck->numMatrices ;) {
			

			func = buck->matrixs[f];
			numVars = func->numVariables;

			/* get the highest priority among the variables in the function */
			/* if it's not the variable of this bucket - shift bucket */
			int newBucket=b;

			for (int v=0,highestPriority=MIN_PRIOR; v<numVars ;v++) {

				buckIndx = func->getVarAt(v)->getIndex();
				if ((priority[buckIndx] > highestPriority) ||
					(((priority[buckIndx] == highestPriority)) && (buckIndx < newBucket))) {
					highestPriority = priority[buckIndx];
					newBucket = buckIndx;
				}
			}

			if (newBucket != b) {
				buck->removeMatrix(f);
				this->buckets[newBucket].addMatrix(func);
			}
			else {
				++f;
			}
		}
	}
}



#endif // !defined(_BUCKETELIMINATOR_H)


