/* $Id: Cluster.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_CLUSTER_H)
#define _CLUSTER_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <vector>
using namespace std;
#include <string.h>
#include "DiscreteFunc.h"
#include "DiscreteVar.h"
#include "InferEng_Settings.h"



class Cluster 
{

	friend class FuncMatrix; 
	friend class BucketEliminator; 
	friend class Vip; 
public:
	Cluster();
	Cluster(int numVars, int numFuncs, int indx);
	Cluster(int numVars, int numFuncs, int indx, string name);

	virtual ~Cluster();

	inline string			getName()				{return this->name;}
	inline int				getNumVars()			{return this->numVariables;}
	inline DiscreteVar*		getVarAt(int indx)		{return this->variables[indx];}
	inline DiscreteFunc*	getFuncAt(int indx)		{return this->functions[indx];}
	inline Cluster*			getSubClusters()		{return this->subClusters;}
	inline Cluster*			getSubClusterAt(int indx)	{return &this->subClusters[indx];}
	inline int				getNumSubClusters()		{return this->numSubClusters;}
	inline _prob			getPotentialAt(int indx){return this->potential[indx];}
	inline _prob*			getPotential()			{return potential;}
	inline int				getIndex()				{return this->index;}
	inline int				getNumInstants()		{return this->numInstants;}
	inline int*				getInstantVals()		{return instantVals;}
	inline void				setVarVal(int place, int value) {this->instantVals[place] = value;}
	inline void				addNeighborFunc(int i)	{neighborFuncs.push_back(i);}
	inline void				addNeighborCluster(int i)	{neighborClusters.push_back(i);}
	inline int				numNeighborFuncs()		{return neighborFuncs.size();}
	inline int				numNeighborClusters()	{return neighborClusters.size();}
	inline int				getNeighborFunc(int i)	{return neighborFuncs[i];}
	inline int				getNeighborCluster(int i){return neighborClusters[i];}
	inline _prob*			getGammas()				{return gammas;}
	inline void				setGammaAt(int i, _prob val){gammas[i] = val;}
	inline DiscreteVar**	getVariables()			{return variables;}
	inline int				getMultValAt(int v)			{return multVals[v];}


	int				toIndex();
	int*			toVarsVals(int indx);
	bool			isInCluster(DiscreteVar* var);
	bool			isInCluster(int indx);
	int				getSubClusterIndx(DiscreteVar* var);
	int				getSubClusterIndx(int indx);
	bool			areParentsInCluster(DiscreteVar* var);
	bool			areParentsInCluster(int indx);
	bool			areParentsInSingleSubCluster(DiscreteVar* var);
	bool			areParentsInSingleSubCluster(int indx);
	bool			isFuncInSingleSubCluster(DiscreteVar* var);
	bool			isFuncInSingleSubCluster(int indx);
	relation		isFuncInCluster(DiscreteFunc* func);
	int				getVarIndxInCluster(DiscreteVar* var);
	int				getVarIndxInCluster(int indx);
	bool			isNeighborCluster(int cl);
	bool			isNeighborFunc(int fn);
	

	void			setPotential(int indx, _prob val);
//	bool			addVariable(DiscreteVar* var);
//	bool			addFunction(DiscreteFunc* func);
	void			calibratePotentials(_prob factor);
	void			Init(int totalNumVars, int index);
	void			normalizePotential();
	





protected:
	int				index;			/* index ID of the cluster */
	string			name;		/* optional name of the network */
	int				numVariables;	/* number of variables */
	DiscreteVar**	variables;		/* array of pointers to variables */
	int				numFunctions;	/* number of functions */
	DiscreteFunc**	functions;		/* array of pointers to functions */
	Cluster*		subClusters;	/* array of sub-clusters */
	int				numSubClusters;	/* number of sub-clusters */
	_prob*			potential;		/* values of potential for each instantiation */
	int				numInstants;	/* number of instantiations */
	int*			varIndices;		/* the index in the variable array of the cluster of every variable */
	int*			instantVals;	/* a helping array to avoid mutiple allocations in toVarvals and toIndex */
	vector<int>neighborFuncs;		/* array of indices of partially contained functions */
	vector<int>neighborClusters;	/* array of indices of neighboring clusters */
	_prob*			gammas;			/* an array with the gamma values that determine the potentials */
	int*			multVals;		/* holds the change in entries with every increase of 1 in a variable */ 
};

//*********************************************************************************88
// Implementation

Cluster::Cluster() {
	this->index = -1;
	this->numVariables = 0;
	this->variables = NULL;
	this->numFunctions = 0;
	this->functions = NULL;
	this->subClusters = NULL;
	this->numSubClusters=0;
	this->potential = NULL;
	this->numInstants = 0;
	this->name = "";
	this->gammas = NULL;
	this->multVals = NULL;
}


Cluster::Cluster(int numVars, int numFuncs, int indx) {
	this->index = indx;
	this->numVariables = numVars;
	this->variables = new DiscreteVar*[numVars];
	this->numFunctions = numFuncs;
	this->functions = new DiscreteFunc*[numFuncs];
	this->subClusters = NULL;
	this->numSubClusters=0;
	this->potential = NULL;
	this->gammas = NULL;
	this->numInstants = 0;
	this->name = "";
	this->multVals = NULL;
}

Cluster::Cluster(int numVars, int numFuncs, int indx, string name) {
	this->index = indx;
	this->numVariables = numVars;
	this->variables = new DiscreteVar*[numVars];
	this->numFunctions = numFuncs;
	this->functions = new DiscreteFunc*[numFuncs];
	this->subClusters = NULL;
	this->numSubClusters=0;
	this->potential = NULL;
	this->gammas = NULL;
	this->numInstants = 0;
	this->name = name;
	this->multVals = NULL;
}

Cluster::~Cluster() {
	
	if (this->variables != NULL) {
		delete[] this->variables;
	}
	if (this->functions != NULL) {
		delete[] this->functions;
	}

	if (this->varIndices != NULL) {
		delete[] this->varIndices;
	}

	if (this->instantVals != NULL) {
		delete[] this->instantVals;
	}

	if (this->potential != NULL) {
		delete[] this->potential;
	}
	if (this->gammas != NULL) {
		delete[] this->gammas;
	}
	if (this->multVals != NULL) {
		delete[] this->multVals;
	}

}

void Cluster::Init(int totalNumVars, int index) {
	

	this->index = index;

	this->varIndices = new int[totalNumVars];
	for (int i=0; i<totalNumVars ;i++) {
		varIndices[i] = NO_VALUE;
	}

	this->instantVals = new int[this->numVariables]; 

	this->variables = new DiscreteVar*[this->numVariables];

	this->functions = new DiscreteFunc*[this->numVariables];


	this->numInstants = 1;
			
}


void Cluster::setPotential(int indx, _prob val) {

	this->potential[indx] = val;
}

/*
bool Cluster::addVariable(DiscreteVar* var) {

	if (numAddedVariables >= numVariables)
		return false;
	
	this->variables[numAddedVariables] = var;
	++numAddedVariables;
	return true;
}


bool Cluster::addFunction(DiscreteFunc* func) {

	if (numAddedFunctions >= numFunctions)
		return false;
	
	this->functions[numAddedFunctions] = func;
	++numAddedFunctions;
	return true;
}*/



int	Cluster::getVarIndxInCluster(DiscreteVar* var) {

	return this->varIndices[var->index];
}

int	Cluster::getVarIndxInCluster(int indx) {

	return this->varIndices[indx];
}


bool Cluster::isInCluster(DiscreteVar* var) {

	if (this->varIndices[var->index] == NO_VALUE)
		return false;
	return true;
}


bool Cluster::isInCluster(int var) {
	if (this->varIndices[var] == NO_VALUE)
		return false;
	return true;
}


/* is the function in the cluster */
/* return contained / partially contained / not-contained */
relation Cluster::isFuncInCluster(DiscreteFunc* func) {

	int cont=0,p;
	
	if (this->varIndices[func->getVar()->index] != NO_VALUE) {
		++cont;
	}

	DiscreteVar** pars = func->getParents();

	for (p=0; p<func->getNumParents() ;p++) {
		if (this->varIndices[pars[p]->index] != NO_VALUE) {
			++cont;
		}
	}

	/* now p = numParents so p+1 = num variables in func */
	if (cont == 0) {
		return _not_contained;
	}
	else if (cont <= p) {
		return _partial;
	}
	
	return _contained;
}



/* converts values of variables to index */
/* the MSB are of the last variables */
int	Cluster::toIndex() {
	
	int index=0;

	for (int v=numVariables-1; v>0 ;v--) {
		index += this->instantVals[v];
		index *= variables[v-1]->numVals;
	}

	index += this->instantVals[0];
	return index;
}

/* converts an index to an array of variables values */
/* the first variables are LSB in index */
int* Cluster::toVarsVals(int indx) {

	int nVals;

	for (int v=0; v<numVariables ;v++) {
		
		nVals = variables[v]->numVals;
		this->instantVals[v] = indx % nVals;
		indx /= nVals;
	}

	return  this->instantVals;
}




void Cluster::calibratePotentials(_prob factor) {

	for (int i=0; i<this->numInstants ;i++) {
		potential[i] *= factor;
	}
}


bool Cluster::isNeighborCluster(int cl) {

	int cSize = neighborClusters.size();

	for (int i=0; i<cSize ;i++) {
		if (neighborClusters[i] == cl) {
			return true;
		}
	}

	return false;
}


bool Cluster::isNeighborFunc(int fn) {

	int cSize = this->neighborFuncs.size();

	for (int i=0; i<cSize ;i++) {
		if (neighborFuncs[i] == fn) {
			return true;
		}
	}

	return false;
}


/* normalizes the potential to sum up to 1 */
void Cluster::normalizePotential() {

	_prob sum=0;

	for (int i=0; i<this->numInstants ;i++) {
		sum += this->potential[i];
	}

	_prob norm = 1/sum;
	for (int ins=0; ins<this->numInstants ;ins++) {
		this->potential[ins] *= norm;
	}
}



#endif // !defined(_CLUSTER_H)


