/* $Id: DiscreteFunc.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_DFUNC_H)
#define _DFUNC_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <vector>
using namespace std;
#include <string.h>
#include "DiscreteVar.h"
#include "InferEng_Settings.h"




/* A structure which hold the probability function P(var | parents(var)) */
class DiscreteFunc
{
	friend class BayesNet; 
	friend class RMF;
	friend class Elim; 
	
public:
	DiscreteFunc();
	virtual ~DiscreteFunc();

	inline int				getNumVars()			{return (numParents+1);}
	inline int				getNumParents()			{return this->numParents;}
	inline _prob			getProbabilityAt(int val, int pval)	{return probability[val][pval];}
	inline _prob**			getProbability()		{return probability;}
	inline DiscreteVar*		getVar()				{return this->var;}
	inline DiscreteVar*		getParent(int pNum)		{return this->parents[pNum];}
	inline DiscreteVar**	getParents()			{return this->parents;}
	inline bool				isConstrained()			{return this->constrained;}
	inline int				getNumInstants()		{return this->numInstants;}
	inline int*			getInstantVals()			{return pInstantVals;}
	inline void			setVarVal(int place, int value) {this->pInstantVals[place] = value;}
	inline	void		setVar(DiscreteVar* var)	{this->var = var;}
	inline	void		setParentAt(DiscreteVar* par, int pos)	{this->parents[pos] = par;}
	inline int			getParentsDim()					{return this->parentsDim;}
	inline int			getVarDim()					{return this->varDim;}
	inline int			getMultValAt(int p)			{return pMultVals[p];}


	bool			isVarInFunc(int indx);
	bool			isVarInFunc(DiscreteVar* var);
	void			setProbabilities(_prob** probs);
	void			removeParent(int pIndx);
	_prob			getProbabilityOf(int val);
	_prob			getProbabilityAt(int ins);
	int*			toParentsVals(int ins);
	int				toVarVal(int ins);
	int				toParIndex();
	

//protected:
	DiscreteVar*	var;			/* pointer to the variable */
	DiscreteVar**	parents;		/* array of pointers to the parents of var */
	int				numParents;		/* number of parents */
	_prob**			probability;	/* matrix of probabilities */
	bool			constrained;	/* a flag which true if func=0 for some instantiation */
	int				varDim;			/* number of values of var */
	int				parentsDim;		/* number of values of parents */
	int				numInstants;	/* number of possible instantiations parentDim*varDim */
	int*			pInstantVals;	/* a helping array to avoid mutiple allocations in toParentsVals */
	int*			pMultVals;		/* holds the change in entries with every increase of 1 in a variable */ 
};

//*********************************************************************************88
// Implementation


DiscreteFunc::DiscreteFunc() {
	this->var = NULL;
	this->parents = NULL;
	this->numParents = 0;
	this->probability = NULL;
	this->pMultVals = NULL;
	this->constrained = false;
}

DiscreteFunc::~DiscreteFunc() {

	if (this->probability != NULL) {
		for (int k=0; k<varDim ;k++) {
			delete[] this->probability[k];
		}
		delete[] this->probability;	
	}

	if (this->pInstantVals) {
		delete[] this->pInstantVals;
	}

	if (this->parents) {
		delete[] this->parents;
	}

	if (this->pMultVals) {
		delete[] this->pMultVals;
	}
}

/* is the variable with index indx a member of the function */
bool DiscreteFunc::isVarInFunc(int indx) {

	if (var->index == indx) {
		return true;
	}
	else {
		for (int i=0; i<numParents ;i++) {
			if (parents[i]->index == indx)
				return true;
		}
	}

	return false;
}

/* is the variable a member of the function */
bool DiscreteFunc::isVarInFunc(DiscreteVar* var) {

	if (this->var == var) {
		return true;
	}
	else {
		for (int i=0; i<numParents ;i++) {
			if (parents[i] == var)
				return true;
		}
	}

	return false;
}

/* insert the probability matrix */
void DiscreteFunc::setProbabilities(_prob** probs) {
	
	/* if it's the first time, allocate the matrix */
	if (this->probability == NULL) {
		this->probability = new _prob*[varDim];
		for (int k=0; k<varDim ;k++) {
			this->probability[k] = new _prob[parentsDim];
		}
	}
	for (int d1=0; d1<varDim ;d1++) {
		for (int d2=0; d2<parentsDim ;d2++) {
			this->probability[d1][d2] = probs[d1][d2];
		}
	}
}


/* get the probability of a possibly partial instantiation of the function */
/* that includes the value of the main variable. the function sums over instantiations */
/* which generalize the given partial instantiation */
_prob	DiscreteFunc::getProbabilityOf(int val) {
	
	_prob res = 0;
	bool projects;
	int remain;

	if (val != NO_VALUE) {

		/* for each instantiation see if it's a projection of the given variables */
		/* the values of the given parents are in pInstantVals */
		/* and val is the value of the main variable */
		for (int i=0; i<this->parentsDim ;i++) {
			
			projects = true;
			remain = i;
			
			for (int p=0; p<this->numParents ;p++) {
				if (this->pInstantVals[p] != NO_VALUE) {		
					if ((remain % this->parents[p]->numVals) != this->pInstantVals[p]) {
						projects = false;
						break;
					}
				}
				remain /= this->parents[p]->numVals;
			}
			if (projects == true) {
				res += this->probability[val][i];
			}
		}
	}
	else {

		for (int j=0; j<this->varDim ;j++) {

			/* for each instantiation see if it's a projection of the given variables */
			/* the values of the given parents are in pInstantVals */
			/* and val is the value of the main variable */
			for (int i=0; i<this->parentsDim ;i++) {
				
				projects = true;
				remain = i;
				
				for (int p=0; p<this->numParents ;p++) {
					if (this->pInstantVals[p] != NO_VALUE) {		
						if ((remain % this->parents[p]->numVals) != this->pInstantVals[p]) {
							projects = false;
							break;
						}
					}
					remain /= this->parents[p]->numVals;
				}
				if (projects == true) {
					res += this->probability[j][i];
				}
			}
		}
	}
	/* see how many projections we have */
	int numPriors = 1;
	for (int p=0; p<this->numParents ;p++) {
		if (this->pInstantVals[p] == NO_VALUE) {
			numPriors *= this->parents[p]->numVals;
		}
	}

	res /= numPriors;

	return res;
}


/* returns the probability of an instantiation */
/* where the main variable's value is the MSB */
_prob DiscreteFunc::getProbabilityAt(int ins) {

	int vDim = ins / parentsDim;
	int pDim = ins % parentsDim;

	return getProbabilityAt(vDim, pDim);
}


/* puts in pInstantVals the corresponding values */
int* DiscreteFunc::toParentsVals(int ins) {

	int pIns = ins % parentsDim;

	int nVals;

	for (int p=0; p<numParents ;p++) {
		
		nVals = parents[p]->numVals;
		this->pInstantVals[p] = pIns % nVals;
		pIns /= nVals;
	}


	return this->pInstantVals;
}

int	DiscreteFunc::toVarVal(int ins) {

	return (ins / parentsDim);
}

/* get the place in parDim for the values in pInstantVals */
int	DiscreteFunc::toParIndex() {

	int index=0;

	for (int p=numParents-1; p>=0 ;p--) {
		index += this->pInstantVals[p];
		if (p > 0)
			index *= parents[p-1]->numVals;
	}

	return index;

}


#endif // !defined(_DFUNC_H)



