/* $Id: DiscreteVar.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_DVAR_H)
#define _DVAR_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <string>
#include <iostream>
#include <vector>
using namespace std;
//#include "DiscreteFunc.h"
#include "InferEng_Settings.h"
#include "PedInfo.h"


class DiscreteVar 
{
	friend class BayesNet; 
	friend class RMF; 
	friend class DiscreteFunc; 
	friend class FuncMatrix; 
	friend class Bucket; 
	friend class CInferEngine; 
	friend class Cluster; 
	
public:
	DiscreteVar();
	DiscreteVar(int indx);
	DiscreteVar(string name, int numVals);
	DiscreteVar(int indx, string name, int numVals);
	DiscreteVar(string name, int numVals, int* values);
	DiscreteVar(int indx, string name, int numVals, int* values);

	virtual ~DiscreteVar();

	inline void			addInfo(void* info)		{this->info = info;}
	inline int			getIndex()				{return this->index;}
	inline string		getName()				{return this->name;}
	inline void			setName(string name)	{this->name = name;}
	inline void*		getInfo()				{return this->info;}
	inline int			getNumVals()			{return this->numVals;}
	inline int*			getValues()				{return this->values;}
	inline int			getValueAt(int indx)	{return this->values[indx];}
	inline int			getNumParents()			{return parents.size();}
	inline int			getNumChildren()		{return children.size();}
	inline DiscreteVar* getParentAt(int p)		{return parents[p];}
	inline DiscreteVar* getChildAt(int p)		{return children[p];}
	inline void			addChild(DiscreteVar* c){children.push_back(c);}
	inline void			addParent(DiscreteVar* p){parents.push_back(p);}
	inline bool			getIndication() { return indication; };

	void				setValues(int numVals);
	void				setValuesArray(int numVals);
	void				setIndex(int indx) { index = indx; };
	void				setIndication(bool ind) { indication = ind; };


protected:
	int		index;		/* optional indexing of variables in the network */
	string	name;		/* the name of the variable */
	void*	info;		/* additional information about the variable */
	int		numVals;	/* number of possible values */
	int*	values;		/* the array of possible values */
	vector<DiscreteVar*> parents;
	vector<DiscreteVar*> children;
	
	//	Andrei Anisenia added this field on 16/08/2009 for auxiliary indications
	bool indication;
};

//*********************************************************************************88
// Implementation

DiscreteVar::DiscreteVar() {
	this->index = -1;
	this->info = NULL;
	this->numVals = 0;
	this->values = NULL;
	this->name = "";
}

DiscreteVar::DiscreteVar(int indx) {
	this->index = indx;
	this->info = NULL;
	this->numVals = 0;
	this->values = NULL;
	this->name = "";
}


DiscreteVar::DiscreteVar(string name, int numVals) {
	this->index = -1;
	this->name = name;
	this->info = NULL;
	this->numVals = numVals;
	this->values = NULL;
}

DiscreteVar::DiscreteVar(int indx, string name, int numVals) {
	this->index = indx;
	this->name = name;
	this->info = NULL;
	this->numVals = numVals;
	this->values = NULL;
}

DiscreteVar::DiscreteVar(string name, int numVals, int* values) {
	this->index = -1;
	this->name = name;
	this->info = NULL;
	this->numVals = numVals;
	this->values = new int[numVals];
	for (int i=0; i<numVals ;i++) {
		this->values[i] = values[i];
	}
}

DiscreteVar::DiscreteVar(int indx, string name, int numVals, int* values) {
	this->index = indx;
	this->name = name;
	this->info = NULL;
	this->numVals = numVals;
	this->values = new int[numVals];
	for (int i=0; i<numVals ;i++) {
		this->values[i] = values[i];
	}
}

DiscreteVar::~DiscreteVar() {
	if (this->values != NULL) {
		delete this->values;
	}
	if (this->info) {
		delete this->info;
	}
}


void DiscreteVar::setValues(int numVals) {
	this->numVals = numVals;
}


void DiscreteVar::setValuesArray(int numVals) {
	this->numVals = numVals;
	for (int i=0; i<numVals ;i++) {
		values[i] = i;
	}
}





#endif // !defined(_DVAR_H)


