/* $Id: elimination.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#ifndef GENERAL_ELIMINATION_H
#define GENERAL_ELIMINATION_H

#include "set.h"
#include "graph.h"

#define NDEBUG 0

#define PREPROCESS 0
#define MAX_VERTICES_NUM 100000 /* maximal number of vertices a graph may contain.*/
#define MAX_PARENTS 5
#define RAND_WINDOW_SIZE 5
#define REMOVED -1
#define IS_SWITCHED_OFF ==-1
#define IS_SWITCHED_ON  !=-1
#define CLIQUE_MAX_SIZE 20000000
#define GRAPH_FILE "graph_file.txt"
#define ELIM_FILE "final_elimination_order.txt"
#define TEMP_ELIM_FILE "temporary_elimination_order.txt"
#define NUM_ALGORITHMS 4

typedef enum{ALL_CLQ, MAX_CLQ, LIM_CLQ}COST_MODE;

typedef enum{SG, WSG, MF, WMF}ELIM_ALG;

/* type of function that counts elimination cost of a vertex i in graph gr.*/
typedef double (Cost_func)(Graph gr, int i);

/****************************************************************************************/
/* Brief description: connects all the neighbors of the given vertex (and the given     */
/* vertex itself) into a clique.                                                        */
/* Parameters:                                                                          */
/*		1. i - number of vertex.                                                        */
/*		2. gr - graph.                                                                  */
/* Return value: none.                                                                  */
/* Remarks: none.                                                                       */
/****************************************************************************************/
void create_clique(Graph gr, int i);

/****************************************************************************************/
/* Brief description: calculates the product of the given vertex and its neighbour's    */
/* weights.                                                                             */
/* Parameters:                                                                          */
/*		1. i - number of vertex.                                                        */
/*		2. gr - graph.                                                                  */
/* Return value: product of weights.                                                    */
/* Remarks: none.                                                                       */
/****************************************************************************************/
double neighborhood_weight(Graph gr, int i);

//////////////////////////////////////////////////////////////////////////////////////////////
/*     struct CostSet contains information about elimination cost of each vertex.           */
/*     fields:       size - number of vertices in eliminated graph.                         */
/*                   vertex_cost - current elimination cost of each vertex.                 */
//////////////////////////////////////////////////////////////////////////////////////////////
struct CostSet{
	int size;
	double vertex_cost[MAX_VERTICES_NUM];
};

//////////////////////////////////////////////////////////////////////////////////////////////
/* cost_set - global variable that contains current elimination cost of all vertices.       */
/* It is updated after elimination of each vertex by updateCostSet(...) function.           */
//////////////////////////////////////////////////////////////////////////////////////////////
struct CostSet cost_set;


//////////////////////////////////////////////////////////////////////////////////////////////
/*   createCostSet - initializes global variable cost_set.Sets cost_set size to the number  */
/*                   of vertices in graph and counts elimination cost for each vertex.      */
/*   parameters:     gr - pointer to the eliminated graph.                                  */
/*                   cost_func - pointer to function that counts elimination cost.          */
/*   return value:   none.                                                                  */
////////////////////////////////////////////////////////////////////////////////////////////// 
void createCostSet(Graph gr, Cost_func cost_func);

//////////////////////////////////////////////////////////////////////////////////////////////
/*   removeFromCostSet - removes vertex i from cost_set after it is eliminated from graph   */
/*   parameters:         i - number of eliminated vertex.                                   */
/*   return value:   none.                                                                  */
//////////////////////////////////////////////////////////////////////////////////////////////
void removeFromCostSet(int i);

//////////////////////////////////////////////////////////////////////////////////////////////
/*   updateCostSet - updates cost_set variable after elimination of each vertex with new    */
/*                   elimination cost for vertices which cost has changed.                  */
/*   parameters:     i - number of vertex, which cost changed.                              */
/*                   cost - new cost of vetex i.                                            */
/*   return value:   none.                                                                  */
//////////////////////////////////////////////////////////////////////////////////////////////
void updateCostSet(int i, double cost);

void createNeighborsSets(Graph gr, int vertex, Set nearNeighbors, Set farNeighbors);

void updateNeighborsCost(Graph gr, Cost_func cost_func, Set nearNeighbors, Set farNeighbors);

double calcOrderCost(COST_MODE c_mode);

double calcElimOrder(ELIM_ALG alg, COST_MODE c_mode, double it_num, double time);

int findNumNonElimDescendents(int var);

int elimOrder_main(ELIM_ALG el_alg, COST_MODE c_mode, double num, char* it_time);

void check_input_parameters(int argc, char *argv[]);

void remove_tmp_files();

void SuperLink_statistics(double weight, double opt_time);

#endif
