/* $Id: elimination_old.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
//#include <assert.h>
#include <float.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "costFunctions.h"
#include "elimination.h"
#include "graph.h"
#include "set.h"
#include "sumMaxCliques.h"


#if PREPROCESS
	#include "preprocessing.h"
#endif

#define MAXLINE 10000
#define MAX_BEST_VERTEX 100000

static void insertBest(int best_vert[], int v_lbl, int best_vert_size);
static void changeBest(int best_vert[], int v_lbl);
static void extendBest(int best_vert[], int v_lbl, int best_vert_size);
static int randBest(int best_vert[], int best_vert_size);
static Cost_func *getAlgFunc(ELIM_ALG alg);
static int vrtx2remove(Graph gr, COST_MODE c_mode);
static int vrtx2break(Graph gr);

double calcOrderCost(COST_MODE c_mode){
	
	char line[MAXLINE];
	double weight = 0.0, n_w, w_factor = 1.0;
	int rem_vrt;
	Graph gr;	
	FILE *fin;	
	
	CreateGraph(GRAPH_FILE, &gr);

	if((fin = fopen(ELIM_FILE, "r")) == NULL) error_elim(2);
	
	while(fgets(line, MAXLINE, fin)){
		if(line[0] == '#') continue;
		rem_vrt = atoi(line) - 1;
		n_w = neighborhood_weight(gr, rem_vrt);
		if(c_mode == ALL_CLQ || c_mode == MAX_CLQ){
			weight += n_w;
			create_clique(gr, rem_vrt);
		}else if(c_mode == LIM_CLQ && n_w <= CLIQUE_MAX_SIZE){			
			//assert(gr->list[rem_vrt].weight);
			weight += w_factor * (n_w / gr->list[rem_vrt].weight);
			create_clique(gr, rem_vrt);
		}else if(c_mode == LIM_CLQ && n_w > CLIQUE_MAX_SIZE){
			//assert(gr->list[rem_vrt].weight);
			w_factor *= gr->list[rem_vrt].weight;
		}

		if(c_mode == MAX_CLQ) print_clique(gr, rem_vrt);
		if(gr->list[rem_vrt].ifRemoved == TRUE) error_elim(3);
		removeVertex(rem_vrt, gr);
	}
	
	if(gr->numOfRemovedVertices != gr->numOfVertices) error_elim(3);
	
	if(c_mode == MAX_CLQ){
		weight -=calcNotMaxCliquesCost(gr->numOfVertices);
	}

	fclose(fin);
	DestroyGraph(gr);
	printf("%.0lf\n", weight);
	return weight;	
}

double calcSingleElimOrder(Graph gr, ELIM_ALG alg, COST_MODE c_mode){
	
	double weight = gr->prepr_cost, n_w, w_factor = 1.0;
	int rem_vrt;
	Cost_func *cost_func;
	FILE *fout;
	Set nearNeighbors, farNeighbors;

	if(gr->numOfVertices > MAX_VERTICES_NUM) error_elim(5);	
	if((fout = fopen(TEMP_ELIM_FILE, "a")) == NULL) error_elim(2);

	cost_func = getAlgFunc(alg);
	createCostSet(gr, cost_func);
	nearNeighbors = createEmptySet();
	farNeighbors = ((alg == SG || alg == WSG) ? NULL : createEmptySet());

	
	while(gr->numOfRemovedVertices < gr->numOfVertices){
	
		rem_vrt = vrtx2remove(gr, c_mode);
		
		//assert(rem_vrt >= 0);
		//assert(rem_vrt < gr->numOfVertices);
		//assert(gr->list[rem_vrt].ifRemoved == FALSE);
		
		createNeighborsSets(gr, rem_vrt, nearNeighbors, farNeighbors);
		fprintf(fout, "%d\n", rem_vrt + 1);
		n_w = neighborhood_weight(gr, rem_vrt);

		if(c_mode == ALL_CLQ || c_mode == MAX_CLQ){
			weight += n_w;
			create_clique(gr, rem_vrt);
		
		}else if(c_mode == LIM_CLQ && n_w <= CLIQUE_MAX_SIZE){			
			
			//assert(gr->list[rem_vrt].weight);
			
			weight += w_factor * (n_w / gr->list[rem_vrt].weight);
			create_clique(gr, rem_vrt);
		}else if(c_mode == LIM_CLQ && n_w > CLIQUE_MAX_SIZE){
			w_factor *= gr->list[rem_vrt].weight;
		}
		
		if(c_mode == MAX_CLQ) print_clique(gr, rem_vrt);

		removeFromCostSet(rem_vrt);
		removeVertex(rem_vrt, gr);	
		updateNeighborsCost(gr, cost_func, nearNeighbors, farNeighbors);
	}

	//assert(gr->numOfRemovedVertices = gr->numOfVertices);

	destroySet(nearNeighbors);
	destroySet(farNeighbors);
	fprintf(fout,"%0.lf", weight);
	fclose(fout);

	if(c_mode == MAX_CLQ){
		weight -= calcNotMaxCliquesCost(gr->numOfVertices);
	}

	return weight;
} 

void execOrderingIteration(Graph gr, ELIM_ALG alg, COST_MODE c_mode, double *best_cost){
	double curr_cost = calcSingleElimOrder(gr, alg, c_mode);
	if(curr_cost < *best_cost){
		*best_cost = curr_cost;
		remove(ELIM_FILE);
		rename(TEMP_ELIM_FILE, ELIM_FILE);
	}
}

double calcElimOrder(ELIM_ALG alg, COST_MODE c_mode, double it_num, double time){
	
	double best_cost = DBL_MAX;
	clock_t begin = clock();
	Graph gr;

	#if PREPROCESS
		Graph init_gr;
		remove(INIT_ELIM_FILE);
		remove(INIT_CLIQUE_FILE);
		CreateGraph(GRAPH_FILE, &init_gr);	
		preprocessing(init_gr, c_mode);
	#endif	

	while(it_num-- > 0 && time > (double)(clock() - begin) / CLOCKS_PER_SEC){
		#if PREPROCESS
			gr = copyGraph(init_gr);
			copy_file(TEMP_ELIM_FILE, INIT_ELIM_FILE);
			if(c_mode == MAX_CLQ) copy_file(CLIQUE_FILE, INIT_CLIQUE_FILE);
		#else
			CreateGraph(GRAPH_FILE, &gr);
		#endif

		execOrderingIteration(gr, alg, c_mode, &best_cost);
		DestroyGraph(gr);
	}

	remove(TEMP_ELIM_FILE);

	#if PREPROCESS
		DestroyGraph(init_gr);
		remove(INIT_ELIM_FILE);
	#endif	

	return best_cost;
}

/****************************************************************************************/
/* Connects all the neighbors of the given vertex (and the given vertex itself) into    */
/* a clique.                                                                            */
/****************************************************************************************/
void create_clique(Graph gr, int i){
	
	Edge e1, e2;
	
	if(gr->list[i].edgesNum < 2)
		return;
	else{
		e1 = gr->list[i].neighbors;
		e2 = e1->next;
		while(e1->next != NULL){
			while(e2 != NULL){
				AddEdge(e1->neighbor, e2->neighbor, gr);
				e2 = e2->next;
			}
			e1 = e1->next;
			e2 = e1->next;
		}
	}
	return;
}

/****************************************************************************************/
/* Calculates the product of the given vertex and its neighbour's weights.              */
/****************************************************************************************/
double neighborhood_weight(Graph gr, int i){
	
	double neigh_wt = gr->list[i].weight;
	Edge it = gr->list[i].neighbors;
	
	while(it != NULL){
		neigh_wt *= gr->list[it->neighbor].weight;
		it = it->next;
	}
	return neigh_wt;
}

void createCostSet(Graph gr, Cost_func cost_func){
	
	int i;
	
	for(i = 0; i < gr->numOfVertices; i++){
		if(gr->list[i].ifRemoved == TRUE)
			cost_set.vertex_cost[i] = REMOVED;
		else
			cost_set.vertex_cost[i] = cost_func(gr, i);
	}
	cost_set.size = gr->numOfVertices;
}

void removeFromCostSet(int i){
	
	//assert(cost_set.vertex_cost[i] != REMOVED);
	cost_set.vertex_cost[i] = REMOVED;
}

void updateCostSet(int i, double cost){
	
	//assert(cost_set.vertex_cost[i] >= 0);
	cost_set.vertex_cost[i] = cost;
}

void createNeighborsSets(Graph gr, int vertex, Set nearNeighbors, Set farNeighbors){
	
	int i, j;
	Edge e = gr->list[vertex].neighbors;
	
	for(i = 0; i < gr->list[vertex].edgesNum; i++){
		addElem(nearNeighbors, e->neighbor);
		e = e->next;
	}
	
	if(farNeighbors){
		for(i = 0; i < nearNeighbors->size; i++){
			e = gr->list[nearNeighbors->elem[i]].neighbors;
			for(j = 0; j < gr->list[nearNeighbors->elem[i]].edgesNum; j++){
				if(e->neighbor != vertex){
					addElem(farNeighbors, e->neighbor);
					e = e->next;
				}
			}
		}
	}
}

void updateNeighborsCost(Graph gr, Cost_func cost_func, Set nearNeighbors, Set farNeighbors){

	while(nearNeighbors->size){
		updateCostSet(nearNeighbors->elem[nearNeighbors->size - 1], cost_func(gr, nearNeighbors->elem[nearNeighbors->size - 1]));
		removeTail(nearNeighbors);
	}	
	
	if(farNeighbors){
		while(farNeighbors->size){
			updateCostSet(farNeighbors->elem[farNeighbors->size - 1], cost_func(gr, farNeighbors->elem[farNeighbors->size - 1]));
			removeTail(farNeighbors);
		}
	}
}

int vrtx2remove(Graph gr, COST_MODE c_mode){
	
	int cnt = 0, i, best_vert[MAX_BEST_VERTEX], best_vert_size = 0, possibly_removed;
	
	while(cnt < cost_set.size && cost_set.vertex_cost[cnt] == REMOVED) cnt++;
	//assert(cnt != cost_set.size);
	best_vert[best_vert_size++] = cnt;

	for(i = cnt + 1; i < cost_set.size; i++){
		if(i%100 == 0){
			i += 0;
		}
		if(best_vert_size < RAND_WINDOW_SIZE && cost_set.vertex_cost[i] != REMOVED){
			insertBest(best_vert, i, best_vert_size++);
		}else if(best_vert_size == RAND_WINDOW_SIZE && cost_set.vertex_cost[i] != REMOVED && cost_set.vertex_cost[i] < cost_set.vertex_cost[best_vert[RAND_WINDOW_SIZE - 1]]){
			insertBest(best_vert, i, best_vert_size);
		}else if(best_vert_size >= RAND_WINDOW_SIZE && cost_set.vertex_cost[i] != REMOVED && cost_set.vertex_cost[i] < cost_set.vertex_cost[best_vert[0]]){
			changeBest(best_vert, i);
			best_vert_size = RAND_WINDOW_SIZE;
		}else if(best_vert_size >= RAND_WINDOW_SIZE && cost_set.vertex_cost[i] != REMOVED && cost_set.vertex_cost[i] == cost_set.vertex_cost[best_vert[0]] && cost_set.vertex_cost[i] == cost_set.vertex_cost[best_vert[best_vert_size - 1]]){
			extendBest(best_vert, i, best_vert_size++);
		}
		//assert(best_vert_size < MAX_SET_SIZE);
	}
	
	possibly_removed = randBest(best_vert, best_vert_size);
	//assert(possibly_removed >= 0);
	//assert(possibly_removed < gr->numOfVertices);
	//assert(gr->list[possibly_removed].ifRemoved == FALSE);

	if(c_mode == ALL_CLQ || c_mode == MAX_CLQ || (c_mode == LIM_CLQ && neighborhood_weight(gr, possibly_removed) <= CLIQUE_MAX_SIZE)){
		return possibly_removed;
	}
	else{
		return vrtx2break(gr);
	}
	

}

int vrtx2break(Graph gr){
	
	double best_score = 0.0, curr_score;
	int i, best_vert = -1;

	for(i = 0; i < gr->numOfVertices; i++){
		if(gr->list[i].ifRemoved == FALSE){
			curr_score = (double)gr->list[i].weight * gr->list[i].edgesNum;
			if(curr_score > best_score){
				best_score = curr_score;
				best_vert = i;
			}
		}
	}
	//assert(best_vert != -1);
	return best_vert;
}

static void insertBest(int best_vert[], int v_lbl, int best_vert_size){
	
	int i;
	
	for(i = 0; i < best_vert_size; i++){
		if(cost_set.vertex_cost[v_lbl] < cost_set.vertex_cost[best_vert[i]]){
			memmove(best_vert + i + 1, best_vert + i, sizeof(double) * RAND_WINDOW_SIZE);
			best_vert[i] = v_lbl;
			return;
		}
	}
	best_vert[best_vert_size++] = v_lbl;
}

static void changeBest(int best_vert[], int v_lbl){
	memmove(best_vert + 1, best_vert, sizeof(double) * RAND_WINDOW_SIZE);
	best_vert[0] = v_lbl;
}

static void extendBest(int best_vert[], int v_lbl, int best_vert_size){
	best_vert[best_vert_size] = v_lbl;	
}

static int randBest(int best_vert[], int best_vert_size){
	
	double sum = 0.0, coin;
	int i, next;
	
	if(cost_set.vertex_cost[best_vert[0]] == 0.0){
		return best_vert[0];
	}
	else if(best_vert_size > RAND_WINDOW_SIZE){
		 next = best_vert_size * (int)(abs((rand() - 1)) / ((double)RAND_MAX));	
		 //assert(next != best_vert_size);
		 //assert(next >= 0);
		 return best_vert[next];
	}
	else{
		for(i = 0; i < best_vert_size; i++){
			sum += 1.0 / cost_set.vertex_cost[best_vert[i]];
		}
		coin = (((double)rand()) * sum / ((double)RAND_MAX));
		for(i = 0; i < best_vert_size; i++){
			if(coin <= 1.0 / cost_set.vertex_cost[best_vert[i]]){
				//assert(i >= 0 && i < best_vert_size);
				return best_vert[i];
			}
			else
				coin -= 1.0 / cost_set.vertex_cost[best_vert[i]];
		}
		//assert(0);
		return 0;
	}
}

Cost_func *getAlgFunc(ELIM_ALG alg){

	Cost_func *fnc;
	
	switch(alg){
	case SG:
		fnc = gr_cost;
		break;
	case WSG:
		fnc = wgr_cost;
		break;		
	case MF:
		fnc = min_fill_cost;
		break;	
	case WMF:
		fnc = wmin_fill_cost;
		break;
	//default:
		//assert(0);
	}
	return fnc;
}



int elimOrder_main(ELIM_ALG el_alg, COST_MODE c_mode, double num, char* it_time){
	double weight;
//	time_t begin = clock();
//	remove_tmp_files();

/*	if(!strcmp(argv[1], "estim_order")){
		copy_file(GRAPH_FILE, argv[2]);
		copy_file(ELIM_FILE, argv[3]);
		calcOrderCost(c_m);
	}else{
*///		srand( (unsigned)time( NULL ) );
//		copy_file(GRAPH_FILE, graphFilename);
//		copy_file(ELIM_FILE, outFilename);
		if(!strcmp(it_time, "it"))
		{
			weight = calcElimOrder(el_alg, c_mode, num, DBL_MAX);
		}
		else
		{
			weight = calcElimOrder(el_alg, c_mode, DBL_MAX, num);
		}
		
//		if(c_mode == ALL_CLQ || c_mode == MAX_CLQ) printf("%.0lf\n", weight);
//		else if(c_mode == LIM_CLQ){
//			SuperLink_statistics(weight, (clock() - begin)/(double)CLOCKS_PER_SEC);
//		}
//	}
//	remove_tmp_files();
	return (int)weight;
}

void check_input_parameters(int argc, char *argv[]){
	if(argc != 5 && argc != 7) error_elim(4);
	if(!strcmp(argv[1], "estim_order") && argc != 5) error_elim(4);
	if(strcmp(argv[1], "estim_order") && strcmp(argv[3], "SG") && strcmp(argv[3], "WSG") && strcmp(argv[3], "MF") && strcmp(argv[3], "WMF") && strcmp(argv[3], "WSG")) error_elim(4);
	if(strcmp(argv[4], "ALL_CLQ") && strcmp(argv[4], "MAX_CLQ") && strcmp(argv[4], "LIM_CLQ")) error_elim(4);
	if(argc == 7  && atoi(argv[6]) <= 0 && (strcmp(argv[5], "it") || strcmp(argv[5], "time"))) error_elim(4);
}

void remove_tmp_files(){
	remove(GRAPH_FILE);
	remove(ELIM_FILE);
	remove(TEMP_ELIM_FILE);
	remove(CLIQUE_FILE);
	remove(INIT_CLIQUE_FILE);
}

void SuperLink_statistics(double weight, double opt_time){
	double ordering_time;
	int time[3] = {0};
	int compl_class = (int)log10(weight);

	printf("SuperLink ordering process imitation:\n\n");
	printf("Sum of table sizes: %.0lf\n", weight);
	printf("Complexity class: %d\n", compl_class);
	if(compl_class < 7) ordering_time = 0.1;
	else ordering_time = pow(8, (log10(weight) - 7.0));
	printf("Apprx. likelihood calculation time: ");
	time[0] = ((int)ordering_time) / 3600;
	time[1] = ((int)ordering_time - 3600 * time[0]) / 60;
	time[2] = ((int)ordering_time - 3600 * time[0] - 60 * time[1]);
	if(time[0]) printf("%d hrs ", time[0]);
	if(time[1]) printf("%d min ", time[1]);
	printf("%d sec\n", time[2]);
	printf("Ordering calculation time: %.2lf sec\n", opt_time);
}


