/* $Id: graph.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "elim_auxiliary.h"
#include "graph.h"

#define MAXLINE 10000

static void parse_line(char line[], int *param1, int *param2){
	
	char num1[MAXLINE], num2[MAXLINE];
	
	strtok(line," \t");
	strcpy(num1, strtok(NULL," \t"));
	strcpy(num2, strtok(NULL," \t"));
	*param1 = atoi(num1);
	*param2 = atoi(num2);
	return;
}

//////////////////////////////////////////////////////////////////////////////////////////////
/*                        createEmptyGraph - creates new graph.                             */
/*     parameters:   numOfVertices - Number of vertices in new graph,                       */
/*                   gr            - Double pointer to the new graph.                       */
/*     return value: none.                                                                  */
/*     comments:     creates new empty graph with specified number of vertices and returns  */
/*                   it by double pointer.                                                  */
//////////////////////////////////////////////////////////////////////////////////////////////
static void createEmptyGraph(int numOfVertices, Graph *gr){
	
	int i;
	
	if( ((*gr) = (Graph)malloc(sizeof(struct Graph_t))) == NULL) error_elim(1);
	if( ((*gr)->list = (AdjList)calloc(numOfVertices, sizeof(struct Vertex_t))) == NULL) error_elim(1);
	if( ((*gr)->matrix = (AdjMatrix)calloc(numOfVertices, sizeof(struct AdjMatrix *))) == NULL) error_elim(1);
	
	for(i = 0; i < numOfVertices; i++){
		if( (((*gr)->matrix)[i] = (struct AdjMatrix_t *)calloc((numOfVertices - i)/8 + 1, sizeof(struct AdjMatrix_t))) == NULL) error_elim(1);
		(*gr)->list[i].ifMarked = FALSE;
		(*gr)->list[i].ifRemoved  = FALSE;
	}
	(*gr)->numOfVertices = numOfVertices;
	(*gr)->numOfRemovedVertices = 0;
	(*gr)->edgesNum = 0;
	(*gr)->prepr_cost = 0.0;
	return;
}

//////////////////////////////////////////////////////////////////////////////////
/*                      creates new  graph from Koster file                     */
//////////////////////////////////////////////////////////////////////////////////
void CreateGraph(char filename[], Graph *gr){

	char line[MAXLINE], num1[100], num2[100];
	int param1, param2, i = 0, j;	
	FILE *fin;
	
	if( (fin = fopen(filename, "r")) == NULL) error_elim(2);
	while(fgets(line,MAXLINE,fin) != NULL){
		switch(line[0]){
		case 'c': 
			break;
		case 'p':
			parse_line(line, &param1, &param2);
			createEmptyGraph(param2, gr);
			break;
		case 'e': 
			parse_line(line, &param1, &param2);
			AddEdge(param1 - 1, param2 - 1, *gr);
			break;
		case 'v':
			parse_line(line, &param1, &param2);
			(*gr)->list[param1 - 1].weight = param2;
			break;
			
		case 'a':
			strtok(line," \t");
			strcpy(num1, strtok(NULL," \t"));
			strcpy(num2, strtok(NULL," \t"));
			strcpy((*gr)->list[atoi(num2) - 1].name, num1);
			break;

		}	
	}
	for(j = 0; j < (*gr)->numOfVertices; j++){
		if((*gr)->list[j].edgesNum == 0)
			i++;
	}
	fclose(fin);
	return;
}	


//////////////////////////////////////////////////////////////////////////////////
/*                                 copies graph                                 */
//////////////////////////////////////////////////////////////////////////////////
Graph copyGraph(Graph oldGr){

	int i;
	Graph gr;
	
	if( (gr = (Graph )malloc(sizeof(struct Graph_t))) == NULL) error_elim(1);
	createEmptyGraph(oldGr->numOfVertices, &gr);
	for(i = 0; i < oldGr->numOfVertices; i++){
		Edge e_it = oldGr->list[i].neighbors;
		while(e_it != NULL){
			AddEdge(i, e_it->neighbor, gr);	
			e_it = e_it->next;
		}
		gr->list[i].ifMarked = oldGr->list[i].ifMarked;
		gr->list[i].ifRemoved = oldGr->list[i].ifRemoved;
		gr->list[i].weight = oldGr->list[i].weight;
		strcpy(gr->list[i].name, oldGr->list[i].name);
	}	
	gr->edgesNum = oldGr->edgesNum;
	gr->numOfRemovedVertices = oldGr->numOfRemovedVertices;
	gr->prepr_cost = oldGr->prepr_cost;
	return gr;
}



//////////////////////////////////////////////////////////////////////////////////
/* INNER FUNCTION: adds edge to adjacent matrix only. Returns SUCCESS if the    */
/* edge was added, FALSE - if it already exists.                                */
//////////////////////////////////////////////////////////////////////////////////
static Result addEdgeToAdjMatrix(int vertex1Num, int vertex2Num, Graph gr){
	
	int i, re;
	struct AdjMatrix_t *unit;

	if(vertex1Num < vertex2Num){
		i = (vertex2Num - vertex1Num)/8;
		re = (vertex2Num - vertex1Num)%8;
		unit = &(gr->matrix[vertex1Num][i]);
	}
	else{
		i = (vertex1Num - vertex2Num)/8;
		re = (vertex1Num - vertex2Num)%8;	
		unit = &(gr->matrix[vertex2Num][i]);
	}
	switch(re){
	case 0: return (unit->b1 ? FAILURE : unit->b1++);
	case 1: return (unit->b2 ? FAILURE : unit->b2++);
	case 2: return (unit->b3 ? FAILURE : unit->b3++);
	case 3: return (unit->b4 ? FAILURE : unit->b4++);
	case 4: return (unit->b5 ? FAILURE : unit->b5++);
	case 5: return (unit->b6 ? FAILURE : unit->b6++);
	case 6: return (unit->b7 ? FAILURE : unit->b7++);
	case 7: return (unit->b8 ? FAILURE : unit->b8++);
	default: return FAILURE;
	}
}


//////////////////////////////////////////////////////////////////////////////////
/*                             Adds edge to the graph.                          */
//////////////////////////////////////////////////////////////////////////////////
Result AddEdge(int vertex1Num, int vertex2Num, Graph gr){
	
	Result res = addEdgeToAdjMatrix(vertex1Num, vertex2Num, gr);
	
	if(res == SUCCESS){
		Edge e1, e2;
		gr->edgesNum ++;
		if( (e1 = (Edge)malloc(sizeof(struct Edge_t))) == NULL) error_elim(1);
		if( (e2 = (Edge)malloc(sizeof(struct Edge_t))) == NULL) error_elim(1);
		
		gr->list[vertex1Num].edgesNum++;
		e1->neighbor = vertex2Num;
		e1->next = gr->list[vertex1Num].neighbors;
		gr->list[vertex1Num].neighbors = e1;
		gr->list[vertex2Num].edgesNum++;
		e2->neighbor = vertex1Num;
		e2->next = gr->list[vertex2Num].neighbors;
		gr->list[vertex2Num].neighbors = e2;
	}
	return res;
}



////////////////////////////////////////////////////////////////////////////////////
/* INNER FUNCTION: removes edge from adjacent matrix only. Returns SUCCESS if the */
/* edge was not removed, FALSE - if it does not exist.                            */
////////////////////////////////////////////////////////////////////////////////////
static Result removeEdgeFromAdjMatrix(int vertex1Num, int vertex2Num, Graph gr){
	
	int i, re;
	struct AdjMatrix_t *unit;
	
	if(vertex1Num < vertex2Num){
		i = (vertex2Num - vertex1Num)/8;
		re = (vertex2Num - vertex1Num)%8;
		unit = &(gr->matrix[vertex1Num][i]);
	}
	else{
		i = (vertex1Num - vertex2Num)/8;
		re = (vertex1Num - vertex2Num)%8;	
		unit = &(gr->matrix[vertex2Num][i]);
	}
	switch(re){
	case 0: return (unit->b1 ? --unit->b1 : FAILURE);
	case 1: return (unit->b2 ? --unit->b2 : FAILURE);
	case 2: return (unit->b3 ? --unit->b3 : FAILURE);
	case 3: return (unit->b4 ? --unit->b4 : FAILURE);
	case 4: return (unit->b5 ? --unit->b5 : FAILURE);
	case 5: return (unit->b6 ? --unit->b6 : FAILURE);
	case 6: return (unit->b7 ? --unit->b7 : FAILURE);
	case 7: return (unit->b8 ? --unit->b8 : FAILURE);
	default:return FAILURE;
	}	
}



////////////////////////////////////////////////////////////////////////////////////
/* INNER FUNCTION: removes edge from the list of specified vertex only. Returns   */
/* SUCCESS if the edge was not removed, FALSE - if it does not exist.             */
////////////////////////////////////////////////////////////////////////////////////
static void removeEdgeFromVertex(int vertexNum, int numOfNeighbor, Graph gr){

	Edge tmp, e = gr->list[vertexNum].neighbors;

	if(e->neighbor == numOfNeighbor){
		gr->list[vertexNum].neighbors = gr->list[vertexNum].neighbors->next;
		free(e);
	}
	else{
		while(e->next){
			if(e->next->neighbor == numOfNeighbor){
				tmp = e->next;
				e->next = e->next->next;
				free(tmp);
				break;
			}
			e = e->next;
		}
	}
	gr->list[vertexNum].edgesNum--;
	return;
}


//////////////////////////////////////////////////////////////////////////////////
/*                        Removes edge from the graph.                          */
//////////////////////////////////////////////////////////////////////////////////
Result removeEdge(int vertex1Num, int vertex2Num, Graph gr){
	
	Result res = removeEdgeFromAdjMatrix(vertex1Num, vertex2Num, gr);
	
	if(res == SUCCESS){
		gr->edgesNum--;
		removeEdgeFromVertex(vertex1Num, vertex2Num, gr);
		removeEdgeFromVertex(vertex2Num, vertex1Num, gr);
	}
	return res;
}



//////////////////////////////////////////////////////////////////////////////////
/*                        Removes vertex from the graph.                        */
//////////////////////////////////////////////////////////////////////////////////
void removeVertex(int vertexNum, Graph gr){
	
	int vNum;
	Result res;
	
	while(gr->list[vertexNum].edgesNum){
		Edge e;
		e = gr->list[vertexNum].neighbors;
		vNum = e->neighbor;
		res = removeEdge(e->neighbor, vertexNum, gr);
	}	
	gr->list[vertexNum].ifRemoved = TRUE;
	gr->numOfRemovedVertices++;

	return;
}



//////////////////////////////////////////////////////////////////////////////////
/*                                 Destroys graph.                              */
//////////////////////////////////////////////////////////////////////////////////
void DestroyGraph(Graph gr){
	
	int i;

	if(!gr) return;
	for(i = 0; i < gr->numOfVertices; i++){
		removeVertex(i, gr);
		free(gr->matrix[i]);
	}
	if(gr->list) free(gr->list);
	if(gr->matrix) free(gr->matrix);
	free(gr);
	return;
}


boolean ifConnected(Graph gr, int v1, int v2){
	
	int i, re;
	struct AdjMatrix_t *unit;
	
	if(v1 < v2){
		i = (v2 - v1)/8;
		re = (v2 - v1)%8;
		unit = &(gr->matrix[v1][i]);
	}
	else{
		i = (v1 - v2)/8;
		re = (v1 - v2)%8;	
		unit = &(gr->matrix[v2][i]);
	}
	switch(re){
	case 0: return (!unit->b1 ? FALSE : TRUE);
	case 1: return (!unit->b2 ? FALSE : TRUE);
	case 2: return (!unit->b3 ? FALSE : TRUE);
	case 3: return (!unit->b4 ? FALSE : TRUE);
	case 4: return (!unit->b5 ? FALSE : TRUE);
	case 5: return (!unit->b6 ? FALSE : TRUE);
	case 6: return (!unit->b7 ? FALSE : TRUE);
	case 7: return (!unit->b8 ? FALSE : TRUE);
	default: return FALSE;
	}
}
