/* $Id: preprocessing.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#include <stdio.h>
#include "elim_auxiliary.h"
#include "preprocessing.h"
#include "set.h"
#include "sumMaxCliques.h"

#define REMOVED_VERTICES	(smpl_removed + asmp_removed)

static boolean if_clique(Graph gr, int vertex_num, double *neigh_product);

static boolean if_almost_clique(Graph gr, int vertex_num, double *neigh_product);

static void update_prepr_cost(Graph gr, int v_num, double add_cost, COST_MODE c_mode);

void preprocessing(Graph gr, COST_MODE c_mode){
	double low = 0.0;
	int prev_removed, smpl_removed = 0, asmp_removed = 0;

	do{
		prev_removed = REMOVED_VERTICES;
		smpl_removed = simpl(gr, &low, c_mode);
		asmp_removed = alm_simpl(gr, low, c_mode);
	}while(prev_removed != REMOVED_VERTICES);
	
	if(c_mode == MAX_CLQ) rename(CLIQUE_FILE, INIT_CLIQUE_FILE);
}

int simpl(Graph gr, double *low, COST_MODE c_mode){
	double nw;
	int i, curr_removed = -1;
	static int tot_removed = 0;
	FILE *fout;
	if((fout = fopen(INIT_ELIM_FILE, "a")) == NULL) error_elim(2);

	while(curr_removed){
		curr_removed = 0;
		for(i = 0; i < gr->numOfVertices; i++){
			if(if_clique(gr, i, &nw) == TRUE && gr->list[i].ifRemoved == FALSE){
				if(*low < nw){
					*low = nw;
				}
				if(gr->list[i].edgesNum == 1 && gr->list[gr->list[i].neighbors->neighbor].edgesNum == 1){
					fprintf(fout, "%d\n", gr->list[i].neighbors->neighbor + 1);
					removeVertex(gr->list[i].neighbors->neighbor, gr);
					curr_removed++;
				}
				update_prepr_cost(gr, i, nw, c_mode);
				if(c_mode == ALL_CLQ || c_mode == MAX_CLQ || (c_mode == LIM_CLQ && nw <= CLIQUE_MAX_SIZE)){
					if(c_mode == MAX_CLQ) print_clique(gr, i);
					removeVertex(i, gr);
					curr_removed++;
					fprintf(fout, "%d\n", i + 1);
				}
			}
		}
		tot_removed += curr_removed;
	}
	fclose(fout);
	return tot_removed;
}

int alm_simpl(Graph gr, double low, COST_MODE c_mode){
	int i, curr_removed = -1;
	static int tot_removed = 0;
	double clique_cost, nw;
	FILE *fout;

	if((fout = fopen(INIT_ELIM_FILE, "a")) == NULL) error_elim(2);

	while(curr_removed){
		curr_removed = 0;
		for(i = 0; i < gr->numOfVertices; i++){
			clique_cost = neighborhood_weight(gr, i);
			if(clique_cost <= low && if_almost_clique(gr, i, &nw) == TRUE){
				create_clique(gr, i);
				update_prepr_cost(gr, i, nw, c_mode);
				if(c_mode == ALL_CLQ || c_mode == MAX_CLQ || (c_mode == LIM_CLQ && nw <= CLIQUE_MAX_SIZE)){
 					if(c_mode == MAX_CLQ) print_clique(gr, i);
					removeVertex(i, gr);
					curr_removed++;
					fprintf(fout, "%d\n", i + 1);
				}
			}
		}
		tot_removed += curr_removed;
	}
	fclose(fout);
	return tot_removed;

}


static boolean if_clique(Graph gr, int vertex_num, double *neigh_product){
	Edge e1 = gr->list[vertex_num].neighbors, e2;
	*neigh_product = (double)gr->list[vertex_num].weight;
	while(e1 != NULL){
		*neigh_product *= gr->list[e1->neighbor].weight;
		e2 = e1;
		while(e2->next != NULL){
			e2 = e2->next;
			if(ifConnected(gr, e1->neighbor, e2->neighbor) == FALSE){
				return FALSE;
			}
		}
		e1 = e1->next;
	}		
	return TRUE;
}

static boolean if_almost_clique(Graph gr, int vertex_num, double *neigh_product){
	int not_in_clique1 = -1, not_in_clique2 = -1, not_in_clique = -1;
	Edge e1 = gr->list[vertex_num].neighbors, e2;
	*neigh_product = gr->list[vertex_num].weight;
	while(e1 != NULL){
		*neigh_product *= gr->list[e1->neighbor].weight;
		e2 = e1;
		while(e2->next != NULL){
			e2 = e2->next;
			if(ifConnected(gr, e1->neighbor, e2->neighbor) == FALSE){
				if(not_in_clique == -1){
					not_in_clique1 = e1->neighbor;
					not_in_clique2 = e2->neighbor;
					not_in_clique = -2;
				}
				else if(not_in_clique == -2){
					if(e1->neighbor == not_in_clique1 || e2->neighbor == not_in_clique1){
						not_in_clique = not_in_clique1;
					}
					else if(e1->neighbor == not_in_clique2 || e2->neighbor == not_in_clique2){
						not_in_clique = not_in_clique2;
					}
					else{
						return FALSE;
					}
				}
				else if(not_in_clique != e1->neighbor && not_in_clique != e2->neighbor){
					return FALSE;
				}
			}
		}
		e1 = e1->next;
	}
	if(not_in_clique > 0 && gr->list[not_in_clique].weight > gr->list[vertex_num].weight){
		return FALSE;
	}
	else if(not_in_clique == -1){
		return FALSE;
	}
	else if(gr->list[not_in_clique1].weight > gr->list[vertex_num].weight && gr->list[not_in_clique2].weight > gr->list[vertex_num].weight){
		return FALSE;
	}
	return TRUE;
}


static void update_prepr_cost(Graph gr, int v_num, double nw, COST_MODE c_mode){
	if(c_mode == ALL_CLQ || c_mode == MAX_CLQ){
		gr->prepr_cost += nw;
	}
	else if(c_mode == LIM_CLQ && nw < CLIQUE_MAX_SIZE){
		gr->prepr_cost += nw / gr->list[v_num].weight;
	}
}
