/* $Id: preprocessing.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#ifndef _PREPROCESSING_H
#define _PREPROCESSING_H

#include "graph.h"
#include "elimination.h"

#define INIT_ELIM_FILE "prepr_ordering.txt"

//////////////////////////////////////////////////////////////////////////////////////
/*  simpl - removes vertices from graph according to the Simplicial Rule.           */
/*  parameters:  gr - graph,                                                        */
/*               elim_order - already existing partial elimination order file,      */
/*               low - pointer to lower bound of the maximal clique.                */
/*  return value:total number of vertices removed by the rule, including previous   */
/*               passes.                                                            */
//////////////////////////////////////////////////////////////////////////////////////
int simpl(Graph gr, double *low, COST_MODE c_mode);

//////////////////////////////////////////////////////////////////////////////////////
/*  alm_simpl - removes vertices from graph according to the Almost Simplicial Rule.*/
/*  parameters:  gr - graph,                                                        */
/*               elim_order - already existing partial elimination order file,      */
/*               low - lower bound of the maximal clique.                           */
/*  return value:total number of vertices removed by the rule, including previous   */
/*               passes.                                                            */
//////////////////////////////////////////////////////////////////////////////////////
int alm_simpl(Graph gr, double low, COST_MODE c_mode);


//////////////////////////////////////////////////////////////////////////////////////
/*  preprocessing - preprocesses graph according to preprocessing reduction rules:  */
/*                  Simplicial, Almost Simplicial, Buddy(ies) & Cube rules.         */
/*  parameters:     graph_file, name of file, containing graph.                     */
/*			        elim_order - already existing partial elimination order file,   */
/*                  print_statistics - in case the variable is "TRUE" preprocessing */
/*                                     statistics is printed.                       */
/*                  num_of_rules - number of rules that are activated:              */
/*                                 Simplicial rule is activated first, Almost       */
/*                                 Simplicial - second, Buddy - third and so on.    */
/*  return value:   number of vertices in graph after preprocessing.                */
//////////////////////////////////////////////////////////////////////////////////////
void preprocessing(Graph gr, COST_MODE c_mode);

#endif
