/* $Id: FuncMatrix.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_FUNCMATRIX_H)
#define _FUNCMATRIX_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <vector>
using namespace std;
#include <string.h>
#include "DiscreteVar.h"
#include "Cluster.h"
#include "InferEng_Settings.h"
#include "ValHolder.h"


class FuncMatrix 
{
	friend class Bucket; 
	friend class BucketEliminator; 
	friend class CInferEngine;
public:
	FuncMatrix(ValHolder* vh);
	FuncMatrix(int totalNumVars, ValHolder* vh);
	FuncMatrix(int totalNumVars, Cluster* cl, ValHolder* vh);
	FuncMatrix(int totalNumVars, DiscreteFunc* func, ValHolder* vh);
	virtual ~FuncMatrix();

	inline int			getNumVars()				{return this->numVariables;}
	inline int			getNumEntries()				{return this->numberOfEntries;}
	inline _prob		getProbabilityAt(int ins)	{return this->probs[ins];}
//	inline void			setVarVal(int var, int val)	{instantVals[var] = val;}
	inline DiscreteVar* getVarAt(int i)				{return this->variables[i];}
//	inline short*		getInstantVals()			{return instantVals;}







	bool				isVarInFunc(int var);	
	bool				isVarInFunc(DiscreteVar* v);												
	void				removeVariable(int var);
	void				removeVariableMAP(int var);
	int					getHighPriorIndex(int* priority);
	int					getHighPriorIndex(int* priority, bool* limited);
	int					getHighPriorIndexBut(int* priority, int var, bool* limited);
//	int					toIndex();
	int					toIndex(int* vals);
	short*				toVarsVals(int indx);
	void				toVarsVals(int indx, int* vals);
	void				toVarsVals(int indx, short* vals);
	void				addVariable(DiscreteVar* var);
	void				copyCluster(Cluster* cl);
	void				copyDiscreteFunc(DiscreteFunc* func);
	void				copyMatrix(FuncMatrix* func);
	void				operator+=(_prob d);
	void				operator*=(_prob d);
	void				operator*=(FuncMatrix* func);
	void				operator*=(DiscreteFunc* func);
	void				operator*=(Cluster* cl);
	_prob				sumOfFunc();
	void				convertFuncToProb();
	_prob				largestEntryProb();
	void				changeMSB2(Cluster* cl);
	void				changeMSB2(DiscreteVar** vars, int numVars);
	_prob				sumOfFuncNoLog();
	FuncMatrix*			truncate(int bucket, int val, int totalNumVars);
	int					getVarPlace(int var);
	int					getVarPlace(int var, int offset);

	bool markNotPresentVars( int* notElimVars, int numNotElimVars);			
	void sumOverSignedVars(bool ind);
	

protected:
	DiscreteVar**				variables;		/* pointers to the variables in the function */
	int							numVariables;	/* number of variables */
	_prob*						probs;			/* values in every entry */
	int							numberOfEntries;	/* number of values in probs */
//	short*						varIndices;		/* contains the place in the vector variables of all variables */
//	short*						instantVals;	/* help to avoid mutiple allocations in toVarvals and toIndex */
	int							actualSize;		/* the actual allocated size */
	ValHolder*					valholder;

private:
	//	Sum the entries corresponding to the given variable blocks, reducing the "probs" size
	void sumProbBlocks(int varStateBlock, int numVarStates, int probsSize);
};

//*********************************************************************************88
// Implementation


FuncMatrix::FuncMatrix(ValHolder* vh) {
	
	this->variables = NULL;
//	this->varIndices = NULL;
//	this->instantVals = NULL;
	this->probs = NULL;
	this->actualSize = 0;
	valholder = vh;
}

FuncMatrix::FuncMatrix(int totalNumVars, ValHolder* vh) {

	this->variables = NULL;
	this->numVariables = 0;
	this->numberOfEntries = 1;
	this->probs = new _prob[1];
	this->probs[0] = 0;
	this->actualSize = 0;
	valholder = vh;
	

//	this->varIndices = new short[totalNumVars];
//	this->instantVals = new short[totalNumVars]; 

	/* initialize the function to contain no variables */
//	for (int i=0; i<totalNumVars ;i++) {
//		this->varIndices[i] = NO_VALUE;
//		this->instantVals[i] = NO_VALUE;
//	}	
}


FuncMatrix::FuncMatrix(int totalNumVars, Cluster* cl, ValHolder* vh) {

	this->variables = NULL;
	this->actualSize = 0;
//	this->varIndices = new short[totalNumVars];
//	this->instantVals = new short[totalNumVars]; 

	/* initialize the function to contain no variables */
//	for (int i=0; i<totalNumVars ;i++) {
//		this->varIndices[i] = NO_VALUE;
//		this->instantVals[i] = NO_VALUE;
//	}

	valholder = vh;

	

	this->copyCluster(cl);

}

FuncMatrix::FuncMatrix(int totalNumVars, DiscreteFunc* func, ValHolder* vh) {
	
	this->variables = NULL;
	this->actualSize = 0;
//	this->varIndices = new short[totalNumVars];
//	this->instantVals = new short[totalNumVars]; 

	/* initialize the function to contain no variables */
//	for (int i=0; i<totalNumVars ;i++) {
//		this->varIndices[i] = NO_VALUE;
//		this->instantVals[i] = NO_VALUE;
//	}

	valholder = vh;


	this->copyDiscreteFunc(func);

}


FuncMatrix::~FuncMatrix() {

//	if (this->varIndices) {
//		delete[] this->varIndices;
//	}
//	if (this->instantVals) {
//		delete[] this->instantVals;
//	}
	if(this->probs) {
		delete[] this->probs;
	}
	if (this->variables) {
		delete[] this->variables;
	}
}

bool FuncMatrix::isVarInFunc(int var) {
//	if (this->varIndices[var] == NO_VALUE) {
	if (getVarPlace(var) == NO_VALUE) {
		return false;
	}
	return true;
}


bool FuncMatrix::isVarInFunc(DiscreteVar* v) {
//	if (this->varIndices[v->index] == NO_VALUE) {
	if (getVarPlace(v->index) == NO_VALUE) {
		return false;
	}
	return true;
}
	

/* remove the variable and sum on the others to get the reduced function */
/* the last variables are the MSB */
void FuncMatrix::removeVariable(int var) {

//	int pos = varIndices[var];
	int pos = getVarPlace(var);
	
	/* the variable is not in the function */
	if (pos == NO_VALUE) {
		return;
	}

	

	int vVals = variables[pos]->numVals;
	if (vVals > 1) {

		_prob outFactor = 0;
		_prob smallestExp;

		int newTableSize = this->numberOfEntries/vVals;
	//	_prob* newTable = new _prob[newTableSize];
		

		int before=1;	/* the offset by the LSB variables before var */
		int after=1;	/* the number of steps in each offset by the MSB variables after var */

		for (int i=0; i<pos ;i++) {
			before *= variables[i]->numVals;
		}
		
		for (int j=pos+1; j<this->numVariables ;j++) {
			after *= variables[j]->numVals;
		}
		int stepSize = before * vVals;

		/* iterate over the entries of the matrix */
		/* sum every vVal entries with the same values of variables except var */
		/* into the first entry of these values (where var has value 0) */
		/* the use of baseLine and secondLine is to save multiplications */
		/* which are more expensive and do summing instead */
		for (int step=0, baseLine=numberOfEntries-1; step<after; step++, baseLine-=stepSize) {
			for (int val=1, secondLine= baseLine-before; val<vVals ;val++, secondLine-=before) {
				
				for (int ins=0,curr=secondLine; ins<before ;ins++,curr--) {
					
					/* no need to do computations */
					if (probs[curr+before] <= MINUS_INF) {
						continue;
					}
					if (probs[curr] <= MINUS_INF) {
						probs[curr] = probs[curr+before];
						continue;
					}


					/* choose the biggest of the two */
					/* this is the common multiplier */
					if (probs[curr] > probs[curr+before]) {
						outFactor = probs[curr];
						smallestExp = probs[curr+before] - probs[curr];
					}
					else {
						outFactor = probs[curr+before];
						smallestExp = probs[curr] - probs[curr+before];
					}
					probs[curr] = log(exp(smallestExp)+1) + outFactor;

				//	probs[curr] += probs[curr+before];
				}
			}
		}

		/* copy the results to newTable */
		if (actualSize >= 5*newTableSize) {
			
			_prob* newTable = new _prob[newTableSize];

			for (int s=0,base=0,cnt=0; s<after; s++,base+=stepSize) {
				for (int ins=0,old=base; ins<before ;ins++,old++) {

					newTable[cnt] = probs[old];
					++cnt;
				}
			}

			delete[] this->probs;
			this->probs = newTable;
			this->actualSize = newTableSize;
		}
		else {
			for (int s=0,base=0,cnt=0; s<after; s++,base+=stepSize) {
				for (int ins=0,old=base; ins<before ;ins++,old++) {

				//	newTable[cnt] = probs[old];
					probs[cnt] = probs[old];
					++cnt;
				}
			}
		}
		
		/* update the members */
		this->numberOfEntries = newTableSize;
	//	delete[] this->probs;
	//	this->probs = newTable;
	}


	/* now we are left only with the entries where var has value 0 */
	/* remove var from the list of variables of the function */
	/* update the position indices of the rest */
	for (int v=pos+1; v<this->numVariables ;v++) {
//		--varIndices[variables[v]->index];
		variables[v-1] = variables[v];
	}
//	this->varIndices[var] = NO_VALUE;
	
	--this->numVariables;
}


/* remove the variable by taking maximum over its vals to get the reduced function */
/* the last variables are the MSB */
void FuncMatrix::removeVariableMAP(int var) {

//	int pos = varIndices[var];
	int pos = getVarPlace(var);
	
	/* the variable is not in the function */
	if (pos == NO_VALUE) {
		return;
	}

	

	int vVals = variables[pos]->numVals;
	if (vVals > 1) {

		_prob outFactor = 0;
		
		int newTableSize = this->numberOfEntries/vVals;
	//	_prob* newTable = new _prob[newTableSize];
		

		int before=1;	/* the offset by the LSB variables before var */
		int after=1;	/* the number of steps in each offset by the MSB variables after var */

		for (int i=0; i<pos ;i++) {
			before *= variables[i]->numVals;
		}
		
		for (int j=pos+1; j<this->numVariables ;j++) {
			after *= variables[j]->numVals;
		}
		int stepSize = before * vVals;

		/* iterate over the entries of the matrix */
		/* sum every vVal entries with the same values of variables except var */
		/* into the first entry of these values (where var has value 0) */
		/* the use of baseLine and secondLine is to save multiplications */
		/* which are more expensive and do summing instead */
		for (int step=0, baseLine=numberOfEntries-1; step<after; step++, baseLine-=stepSize) {
			for (int val=1, secondLine= baseLine-before; val<vVals ;val++, secondLine-=before) {
				
				for (int ins=0,curr=secondLine; ins<before ;ins++,curr--) {
					
					/* no need to do computations */
					if (probs[curr+before] <= MINUS_INF) {
						continue;
					}
					if (probs[curr] <= MINUS_INF) {
						probs[curr] = probs[curr+before];
						continue;
					}


					/* choose the biggest of the two */
					if (probs[curr] < probs[curr+before]) {
						probs[curr] = probs[curr+before];
					}
				}
			}
		}

		/* copy the results to newTable */
		if (actualSize >= 5*newTableSize) {
			
			_prob* newTable = new _prob[newTableSize];

			for (int s=0,base=0,cnt=0; s<after; s++,base+=stepSize) {
				for (int ins=0,old=base; ins<before ;ins++,old++) {

					newTable[cnt] = probs[old];
					++cnt;
				}
			}

			delete[] this->probs;
			this->probs = newTable;
			this->actualSize = newTableSize;
		}
		else {
			for (int s=0,base=0,cnt=0; s<after; s++,base+=stepSize) {
				for (int ins=0,old=base; ins<before ;ins++,old++) {

				//	newTable[cnt] = probs[old];
					probs[cnt] = probs[old];
					++cnt;
				}
			}
		}
		
		/* update the members */
		this->numberOfEntries = newTableSize;
	//	delete[] this->probs;
	//	this->probs = newTable;
	}


	/* now we are left only with the entries where var has value 0 */
	/* remove var from the list of variables of the function */
	/* update the position indices of the rest */
	for (int v=pos+1; v<this->numVariables ;v++) {
//		--varIndices[variables[v]->index];
		variables[v-1] = variables[v];
	}
//	this->varIndices[var] = NO_VALUE;
	
	--this->numVariables;
}


/* returns the highest priority index among the variables in the function */
/* if several variables have the same highest priority - return the one with */
/* with minimal index */
int	FuncMatrix::getHighPriorIndex(int* priority) {

	int indx;
	int highPriorIndx = MAX_INDEX, highPrior=MIN_PRIOR, currVarPrior;
	
	for (int i=0; i<this->numVariables ;i++) {

		indx = this->variables[i]->index;
		currVarPrior = priority[indx];

		if ((currVarPrior > highPrior) || 
			((currVarPrior == highPrior) && (highPriorIndx > indx))) {
			highPrior = currVarPrior;
			highPriorIndx = indx;
		}
	}

	return highPriorIndx;
}


/* returns the highest priority index among the variables in the function */
/* if several variables have the same highest priority - return the one with */
/* with minimal index */
int	FuncMatrix::getHighPriorIndex(int* priority, bool* limited) {

	int indx;
	int highPriorIndx = MAX_INDEX, highPrior=MIN_PRIOR, currVarPrior;
	
	for (int i=0; i<this->numVariables ;i++) {

		indx = this->variables[i]->index;
		if (limited[indx] == false) {
			currVarPrior = priority[indx];

			if ((currVarPrior > highPrior) || 
				((currVarPrior == highPrior) && (highPriorIndx > indx))) {
				highPrior = currVarPrior;
				highPriorIndx = indx;
			}
		}
	}

	return highPriorIndx;
}


/* returns the highest priority index among the variables in the function */
/* if several variables have the same highest priority - return the one with */
/* with minimal index which is not var*/
int	FuncMatrix::getHighPriorIndexBut(int* priority, int var, bool* limited) {

	int indx;
	int highPriorIndx = MAX_INDEX, highPrior=MIN_PRIOR, currVarPrior;
	
	for (int i=0; i<this->numVariables ;i++) {
		
		indx = this->variables[i]->index;
		if ((indx != var) && (limited[indx] == false)) {
			currVarPrior = priority[indx];

			if ((currVarPrior > highPrior) || 
				((currVarPrior == highPrior) && (highPriorIndx > indx))) {
				highPrior = currVarPrior;
				highPriorIndx = indx;
			}
		}
	}

	/* if all but var have MIN_PRIOR priority */
	if (highPrior == MIN_PRIOR) {
		return var;
	}

	return highPriorIndx;
}


/* converts values of variables to index */
/* the MSB are of the last variables */
/*int	FuncMatrix::toIndex() {
	
	int index=0;

	for (int v=numVariables-1; v>0 ;v--) {
		index += this->instantVals[variables[v]->index];
		index *= variables[v-1]->numVals;
	}
	index += this->instantVals[variables[0]->index];

	return index;
}*/


/* converts values of variables in vals to index */
/* the MSB are of the last variables */
int	FuncMatrix::toIndex(int* vals) {
	
	int index=0;

	for (int v=numVariables-1; v>0 ;v--) {
		index += vals[variables[v]->index];
		index *= variables[v-1]->numVals;
	}
	index += vals[variables[0]->index];

	return index;
}

/* converts an index to an array of variables values */
/* the first variables are LSB in index */
short* FuncMatrix::toVarsVals(int indx) {

	int nVals;

	for (int v=0; v<numVariables ;v++) {
		
		nVals = variables[v]->numVals;
	//	this->instantVals[variables[v]->index] = indx % nVals;
		valholder->vals[variables[v]->index] = indx % nVals;
		indx /= nVals;
	}

//	return  this->instantVals;
	return valholder->vals;
}


/* converts an index to an array of variables values */
/* puts the values in vals */
/* the first variables are LSB in index */
void FuncMatrix::toVarsVals(int indx, int* vals) {
	
	int nVals;

	for (int v=0; v<numVariables ;v++) {
		
		nVals = variables[v]->numVals;
		vals[variables[v]->index] = indx % nVals;
		indx /= nVals;
	}
}

/* converts an index to an array of variables values */
/* puts the values in vals */
/* the first variables are LSB in index */
void FuncMatrix::toVarsVals(int indx, short* vals) {
	
	int nVals;

	for (int v=0; v<numVariables ;v++) {
		
		nVals = variables[v]->numVals;
		vals[variables[v]->index] = indx % nVals;
		indx /= nVals;
	}
}


/* add a new variable as the MSB - the last variable */
/* expand the probs matrix to hold numVals duplications of the original probs */
void FuncMatrix::addVariable(DiscreteVar* var) {
	
	DiscreteVar** newVariables = new DiscreteVar*[this->numVariables+1];

	for (int v=0; v<numVariables ;v++) {
		newVariables[v] = this->variables[v];
	}
	newVariables[numVariables+1] = var;

//	varIndices[var->index] = numVariables;

	if (this->numVariables > 0) {
		delete[] this->variables;
	}
	this->variables = newVariables;
	++numVariables;

	
	int numVals = var->numVals;

	if (numVals > 1) {
		int newTableSize = this->numberOfEntries * numVals;
		_prob* newTable = new _prob[newTableSize];
		
		/* duplicate the content of probs (numVals-1) times */
		int cnt=0;
		for (int i=0; i<numVals ;i++) {
			for (int j=0; j<this->numberOfEntries ;j++) {
				newTable[cnt] = probs[j];
				cnt++;
			}
		}

		/* update the members */
		this->numberOfEntries = newTableSize;
		delete[] this->probs;
		this->probs = newTable;
		this->actualSize = newTableSize;
	}
}


/* copy the cluster into this empty matrix */
void FuncMatrix::copyCluster(Cluster* cl) {

	DiscreteVar* var;

	this->numVariables = cl->getNumVars();
	if (this->variables == NULL) {
		this->variables = new DiscreteVar*[numVariables];
	}

	for (int v=0; v<numVariables ; v++) {
		var = cl->getVarAt(v);
		this->variables[v] = var;
//		this->varIndices[var->index] = v;
	}

	this->numberOfEntries = cl->getNumInstants();
	this->actualSize = this->numberOfEntries;
	this->probs = new _prob[numberOfEntries];

	_prob* potential = cl->getPotential();

	for (int i=0; i<numberOfEntries ;i++) {
/* debugging section */

//		if (potential[i] < 0) {
//				printf("**\n");
//		}



/* **************** */
		this->probs[i] = log(potential[i]);
	}

}


/* copy the function into this empty matrix */
void FuncMatrix::copyDiscreteFunc(DiscreteFunc* func) {

	DiscreteVar* var;

	this->numVariables = func->getNumParents() + 1;
	if (this->variables == NULL) {
		this->variables = new DiscreteVar*[numVariables];
	}

	var = func->getVar();
	this->variables[numVariables-1] = var;
//	this->varIndices[var->index] = numVariables-1;

	for (int v=0; v<numVariables-1 ; v++) {
		var = func->getParent(v);
		this->variables[v] = var;
//		this->varIndices[var->index] = v;
	}

	this->numberOfEntries = func->getNumInstants();
	this->actualSize = this->numberOfEntries;
	this->probs = new _prob[numberOfEntries];

	_prob** probability = func->getProbability();

	int varDim = func->getVarDim();
	int parDim = func->getParentsDim();

	for (int j=0, ins=0; j<varDim ;j++) {
		for (int i=0; i<parDim;i++) {
/* debugging section */

//			if (probability[j][i] < 0) {
//					printf("**\n");
//			}



	/* **************** */
			this->probs[ins] = log(probability[j][i]);
			++ins;
		}
	}
}



/* copy all the information in this to the empty FuncMatrix func */
void FuncMatrix::copyMatrix(FuncMatrix* func) {

	func->numberOfEntries = this->numberOfEntries;
	func->actualSize = this->numberOfEntries;
	func->numVariables = this->numVariables;

	if (func->probs != NULL) {
		delete[] func->probs;
	}
	func->probs = new _prob[numberOfEntries];

	/* copy probs and variables */
	for (int p=0; p<this->numberOfEntries ;p++) {
		func->probs[p] = this->probs[p];
	}

	if (func->variables != NULL) {
		delete[] func->variables;
	}
	func->variables = new DiscreteVar*[numVariables];
	for (int v=0; v<this->numVariables ;v++) {
		func->variables[v] = this->variables[v];
	}
	
//	for (int i=0; i<this->numVariables ;i++) {
//		func->varIndices[variables[i]->index] = i;
//	}
}


void FuncMatrix::operator+=(_prob d) {

	_prob logD = log(d);

	for (int i=0; i<this->numberOfEntries ;i++) {
		this->probs[i] = log(exp(this->probs[i] - logD)+1) + logD;
	}
}

	
/* d=log(x) so we infact multiply the matrix by x */
void FuncMatrix::operator*=(_prob d) {

	for (int i=0; i<this->numberOfEntries ;i++) {
		this->probs[i] += d;
	}
}


/* combine this and func into this */
/* the MSB variables in the new matrix (the last ones) */
/* are the variables that are unique to func */
/* this is the elementary operation in the bucket elimination */
/* and should be very efficient */
void FuncMatrix::operator*=(FuncMatrix* func) {

	
	int addedValues = 1;
	int numAddedVars = 0;
	vector<DiscreteVar*> addedVariables;

	/* find which variables in func also appear in this */
	/* expand the matrix by the number of values of the unique variables in func */
	/* in each new entry the values equal the original value from this */
	DiscreteVar* var;
	for (int v=0; v<func->numVariables ;v++) {
		 var = func->variables[v];

		/* if the variable is not in this, add it at the end of the vector */
//		if (this->varIndices[var->index] == NO_VALUE) {
		 if (getVarPlace(var->index) == NO_VALUE) {
			
			/* addVariable() already expands the matrix */
			++numAddedVars;
			addedVariables.push_back(var);
			addedValues *= var->numVals;
		}
	}

	/* make a new variables array from the original array of this and addedVariables */
	DiscreteVar** newVariables = new DiscreteVar*[this->numVariables+numAddedVars];
	for (int newK=0; newK<numAddedVars ;newK++) {
		newVariables[newK] = addedVariables[newK];
//		this->varIndices[newVariables[newK]->index] = newK;
	}
	for (int k=0,newPlace=numAddedVars; k<this->numVariables ;k++,newPlace++) {
		newVariables[newPlace] = this->variables[k];
//		this->varIndices[newVariables[newPlace]->index] = newPlace;
	}
	

	if (this->numVariables > 0) {
		delete[] this->variables;
	}
	this->variables = newVariables;
	this->numVariables += numAddedVars;
	

	/* allocate a new matrix to be replaced with the original one */
	int newTableSize = this->numberOfEntries * addedValues;
	_prob* newProbs = new _prob[newTableSize];

	/* multiply the value by the value at func for the projection of the entry */
	int origValues = this->numberOfEntries;
	int indx, nVals;
	DiscreteVar* tmpVar;
//	short* vals = func->instantVals;
	short* vals = valholder->vals;
	DiscreteVar** funcVars = func->variables;
;
	int numVars0 = func->numVariables;

	int numOriginalVars = numVariables - numAddedVars;
	int startingValue = 1;
	for (int a=0; a<numAddedVars ;a++) {
		startingValue *= variables[a]->numVals;
	}



	for (int i=0,ins=0; i<origValues ;i++) {

		indx = ins/startingValue;
		
		for (int v=numAddedVars; v<numVariables ;v++) {
			
			tmpVar = this->variables[v];
			nVals = tmpVar->numVals;
			vals[tmpVar->index] = indx % nVals;
			indx /= nVals;
		}

		/* check if there's any use doing the rest of the computations for this i */
		if (this->probs[i] > MINUS_INF) {
			
			for (int j=0; j<addedValues ;j++) {
			

				/* inline toVarsVals(ins) */
				//this->toVarsVals(ins, func->instantVals);
				indx = ins;
			
				for (int v=0; v<numAddedVars ;v++) {
					
					tmpVar = this->variables[v];
					nVals = tmpVar->numVals;
					vals[tmpVar->index] = indx % nVals;
					indx /= nVals;
				}
				
				/* find the corresponding index in func */
				indx=0;
				
				for (int v0=numVars0-1; v0>0 ;v0--) {
					indx += vals[funcVars[v0]->index];
					indx *= funcVars[v0-1]->numVals;
				}
				indx += vals[funcVars[0]->index];
				
				newProbs[ins] = this->probs[i] + func->probs[indx];
				++ins;
			}
		}
		else {
			for (int j=0; j<addedValues ;j++) {
				newProbs[ins] = MINUS_INF;
				++ins;
			}
		}
	}

	/* update the matrix and delete the old one */
	this->numberOfEntries = newTableSize;
	this->actualSize = newTableSize;
	if (this->probs != NULL) {
		delete[] this->probs;
	}
	this->probs = newProbs;
}


/* multiplying the entries by log(func) */
void FuncMatrix::operator*=(DiscreteFunc* func) {
	

	int numPars = func->getNumParents();

	/* a check of correctness with all parents included */
	/* parents as evidence not supported at the moment */
	for (int p=0; p<numPars ;p++) {
//		if (this->varIndices[func->getParent(p)->index] == NO_VALUE) {
		if (getVarPlace(func->getParent(p)->index) == NO_VALUE) {
			printf("An error while *=DiscreteFunc.\n");
			printf("Not all parents are in the FuncMatrix - possibly evidence parents.\n");
			exit(1);
		}
	}

	int* parsInds = new int[numPars];

	for (int v=0; v<numPars ;v++) {
		parsInds[v] = func->getParent(v)->index;
	}

	_prob funcProb;
	int varVal, varIndx = func->getVar()->index;

	/* we set the main variable to NO_value to be update in toVarsVals() */
	/* if the variable exists in the FuncMatrix */
//	this->instantVals[varIndx] = NO_VALUE;

	short* vals = valholder->vals;
	vals[varIndx] = NO_VALUE;

	for (int ins=0; ins<this->numberOfEntries ;ins++) {
		
		/* no work to do if probs is already 0 */
		if (this->probs[ins] > MINUS_INF) {

			/* fill in the values of ins in func */
			this->toVarsVals(ins, vals);

			for (int p=0; p<numPars ;p++) {
//				func->setVarVal(p,this->instantVals[parsInds[p]]);
				func->setVarVal(p,vals[parsInds[p]]);
			}
//			varVal = this->instantVals[varIndx];
			varVal = vals[varIndx];

			/* if the variable is an evidence (is known) set it to its only value - 0 */
			if (varVal == NO_VALUE) {
				varVal = 0;
			}

			/* find the index and probability there */
			funcProb = func->getProbabilityAt(varVal,func->toParIndex());

			/* multiply by log of the corresponding entry in func */
			if (funcProb == 0) {
				printf("error - Q does not equal 0 for P=0");
				exit(1);
			}
			else {
/* debugging section */

//		if (funcProb < 0) {
//				printf("**\n");
//		}



/* **************** */
				funcProb = log(funcProb);
				this->probs[ins] = exp(this->probs[ins]) * funcProb;
			}
		}
		else {
			this->probs[ins] = 0;
		}
	}
	delete[] parsInds;
}


void FuncMatrix::operator*=(Cluster* cl) {
	
	int numVars = cl->getNumVars();
	for (int p=0; p<numVars ;p++) {
//		if (this->varIndices[cl->getVarAt(p)->index] == NO_VALUE) {
		if (getVarPlace(cl->getVarAt(p)->index) == NO_VALUE) {
			printf("error - multiplying with a not contained cluster\n");
			exit(1);
		}
	}

	int* varsInds = new int[numVars];
	short* vals = valholder->vals;

	for (int v=0; v<numVars ;v++) {
		varsInds[v] = cl->getVarAt(v)->index;
	}

	_prob clProb;
	_prob* potential = cl->getPotential();
	
	for (int ins=0; ins<this->numberOfEntries ;ins++) {
		
		/* no work to do if probs is already 0 */
		if (this->probs[ins] > MINUS_INF) {

			/* fill in the values of ins in cl */
			this->toVarsVals(ins);

			for (int v=0; v<numVars ;v++) {
//				cl->setVarVal(v,this->instantVals[varsInds[v]]);
				cl->setVarVal(v,vals[varsInds[v]]);
			}
			
			/* find the index and probability there */
			clProb = potential[cl->toIndex()];

			/* multiply by log of the corresponding entry in cl */
			if (clProb == 0) {
				printf("error - matrix and cluster hold different variables or wrong elimination");
				exit(1);
			}
			else {
/* debugging section */

//		if (clProb < 0) {
//				printf("**\n");
//		}



/* **************** */
				clProb = log(clProb);
				this->probs[ins] = exp(probs[ins]) * clProb;
			}
		}
		else {
			this->probs[ins] = 0;
		}
	}
}


/* returns the sum of entries in the matrix */
_prob	FuncMatrix::sumOfFunc() {

	_prob res=0;

	for (int i=0; i<this->numberOfEntries ;i++) {
		res += exp(this->probs[i]);
	}

	return res;
}


//	Converts all the entries to probabilities (calcs exp)
void	FuncMatrix::convertFuncToProb() {
	for (int i=0; i<this->numberOfEntries ;i++) {
		this->probs[i] = exp(this->probs[i]);
	}
}


/* returns the maximum probability over all the entries in the matrix */
_prob	FuncMatrix::largestEntryProb() {

	_prob res = MINUS_INF;

	for (int i=0; i<this->numberOfEntries ;i++) {
		if (res < probs[i]) {
			res = probs[i];
		}
	}

	return res;
}


/* returns the sum of entries in the matrix */
/* when these are not logs */
_prob	FuncMatrix::sumOfFuncNoLog() {

	_prob res=0;

	for (int i=0; i<this->numberOfEntries ;i++) {
		res += this->probs[i];
	}

	return res;
}


/* changes the variables in cl to be the MSB variables in the FuncMatrix */
/* All the variables in cl as assumed to be in the matrix */
void FuncMatrix::changeMSB2(Cluster* cl) {

	int clNumVars = cl->getNumVars();
	int restNumVars = numVariables - clNumVars;

	/* check if the variables of cluster are not already the MSB */
	bool organized = true;
	for (int a=restNumVars,v1=0; a<numVariables ;a++,v1++) {
		if (cl->getVarAt(v1)->index != this->variables[a]->index) {
			organized = false;
			break;
		}
	}
	if (organized == true) {
		return;
	}

	_prob* newProbs = new _prob[this->numberOfEntries];
	DiscreteVar** newVars = new DiscreteVar*[this->numVariables];

	
	
	int filledVars=0;
	for (int var1=0; var1<this->numVariables ;var1++) {
		if (cl->isInCluster(variables[var1]->index) == false) {
			newVars[filledVars] = variables[var1];
			++filledVars;
		}
	}
	for (int var=0; var<clNumVars ;var++) {
		newVars[filledVars] = cl->getVarAt(var);
		++filledVars;
	}


	int* numVals = new int[numVariables];
	numVals[0] = 1;
	for (int j=1; j<numVariables ;j++) {
		numVals[j] = numVals[j-1] * variables[j-1]->numVals;
	}

	int* newVals = new int[numVariables];
	newVals[0] = 1;
	for (int j1=1; j1<numVariables ;j1++) {
		newVals[j1] = newVals[j1-1] * newVars[j1-1]->numVals;
	}

	/* for each entry in the new probs find the entry in the existing matrix */
	int clInstants = cl->getNumInstants();
	int restInstants = this->numberOfEntries / clInstants;
	int oldPlace;
	int prePlace;
	int tmpVals;
	int* clVals = cl->getInstantVals();

	for (int k=0, entry=0; k<clInstants ;k++) {

		prePlace = 0;
		cl->toVarsVals(k);

		for (int v=0; v<clNumVars ;v++) {
//			prePlace += clVals[v] * numVals[varIndices[cl->getVarAt(v)->index]];
			prePlace += clVals[v] * numVals[getVarPlace(cl->getVarAt(v)->index)];
		}

		/* iterate over the LSB entries for a MSB=k */
		for (int i=0; i<restInstants ;i++) {

			oldPlace = prePlace;

			for (int r=0; r<restNumVars ;r++) {

//				tmpVals = numVals[varIndices[newVars[r]->index]];
				tmpVals = numVals[getVarPlace(newVars[r]->index)];
				oldPlace += ((i/newVals[r]) % newVars[r]->numVals) * tmpVals; 
							
			}

			newProbs[entry] = probs[oldPlace];
			++entry;
		}
	}

	/* replace variables and probs and update the varIndices */
	delete[] this->variables;
	delete[] this->probs;

	this->variables = newVars;
//	for (int v=0; v<this->numVariables ;v++) {
//		this->varIndices[variables[v]->index] = v;	
//	}

	this->probs = newProbs;

	delete[] numVals;
	delete[] newVals;
}





/* changes the variables in cl to be the MSB variables in the FuncMatrix */
/* All the variables in cl as assumed to be in the matrix */
void FuncMatrix::changeMSB2(DiscreteVar** vars, int numVars) {

	int restNumVars = numVariables - numVars;
	int clInstants = 1;
	
	/* check if the variables of vars are not already the MSB */
	bool organized = true;
	for (int a=restNumVars,v1=0; a<numVariables ;a++,v1++) {
		if (vars[v1]->index != this->variables[a]->index) {
			organized = false;
			break;
		}
	}
	if (organized == true) {
		return;
	}
	
	_prob* newProbs = new _prob[this->numberOfEntries];
	DiscreteVar** newVars = new DiscreteVar*[this->numVariables];

	int filledVars=0;
	for (int var1=0; var1<this->numVariables ;var1++) {
		bool inArr = false;
		for(int v1=0; v1<numVars; v1++) {
			if (vars[v1]->index == variables[var1]->index) {
				inArr = true;
				clInstants *= vars[v1]->numVals;
				break;
			}
		}
		if (inArr == false) {
			newVars[filledVars] = variables[var1];
			++filledVars;
		}
	}
	
	for (int var=0; var<numVars ;var++) {
		newVars[filledVars] = vars[var];
		++filledVars;
	}

	
	int* numVals = new int[numVariables];
	numVals[0] = 1;
	for (int j=1; j<numVariables ;j++) {
		numVals[j] = numVals[j-1] * variables[j-1]->numVals;
	}
	
	int* newVals = new int[numVariables];
	newVals[0] = 1;
	for (int j1=1; j1<numVariables ;j1++) {
		newVals[j1] = newVals[j1-1] * newVars[j1-1]->numVals;
	}
		
	/* for each entry in the new probs find the entry in the existing matrix */
	int restInstants = this->numberOfEntries / clInstants;
	int oldPlace;
	int prePlace;
	int tmpVals;
	int* clVals = new int[numVars];
	for (int k=0, entry=0; k<clInstants ;k++) {

		prePlace = 0;
		

		// replace cl->toVarVals(k)
		int nVals;
		int tmpIndx = k;

		for (int v=0; v<numVars ;v++) {
			
			nVals = vars[v]->numVals;
			clVals[v] = tmpIndx % nVals;
			tmpIndx /= nVals;
		}


		for (int v=0; v<numVars ;v++) {
//			prePlace += clVals[v] * numVals[varIndices[cl->getVarAt(v)->index]];
			prePlace += clVals[v] * numVals[getVarPlace(vars[v]->index)];
		}

		/* iterate over the LSB entries for a MSB=k */
		for (int i=0; i<restInstants ;i++) {

			oldPlace = prePlace;

			for (int r=0; r<restNumVars ;r++) {

//				tmpVals = numVals[varIndices[newVars[r]->index]];
				tmpVals = numVals[getVarPlace(newVars[r]->index)];
				oldPlace += ((i/newVals[r]) % newVars[r]->numVals) * tmpVals; 
							
			}

			newProbs[entry] = probs[oldPlace];
			++entry;
		}
	}

	/* replace variables and probs and update the varIndices */
	delete[] this->variables;
	delete[] this->probs;

	this->variables = newVars;
//	for (int v=0; v<this->numVariables ;v++) {
//		this->varIndices[variables[v]->index] = v;	
//	}

	this->probs = newProbs;

	delete[] numVals;
	delete[] newVals;
}

/* the function creates a new FuncMatrix which contains */
/* all the variables but the one in bucket */
/* and only the entries where this variable equals val */ 
/* it is assumed that the variable bucket is the MSB variable in the matrix */
FuncMatrix*	FuncMatrix::truncate(int bucket, int val, int totalNumVars) {


	FuncMatrix* func = new FuncMatrix(totalNumVars, valholder);

//	int varPos = varIndices[bucket];
	int varPos = getVarPlace(bucket);

	DiscreteVar *var = this->variables[varPos];

	func->numberOfEntries = this->numberOfEntries/var->numVals;
	func->numVariables = this->numVariables-1;
	
	func->probs = new _prob[func->numberOfEntries];

	/* copy probs and variables */
	for (int p=0, p1=val*func->numberOfEntries; p<func->numberOfEntries ;p++) {
		func->probs[p] = this->probs[p1];
		++p1;
	}

	
	func->variables = new DiscreteVar*[func->numVariables];
	for (int v=0, v1=0; v<this->numVariables ;v++) {
//		if (v != varIndices[bucket]) {
		if (v != getVarPlace(bucket)) {
			func->variables[v1] = this->variables[v];
			++v1;
		}
	}
	
//	for (int i=0; i<func->numVariables ;i++) {
//		func->varIndices[func->variables[i]->index] = i;
//	}

	return func;
}


/* replaces the use of varIndices */
int	FuncMatrix::getVarPlace(int var) {

	for (int i=0; i<numVariables ;i++) {
		if (variables[i]->index == var) {
			return i;
		}
	}
	return NO_VALUE;
}


/* similar to getVarPlace but get an offset when the variables */
/* are added in reversed order when combining two matrices */
int	FuncMatrix::getVarPlace(int var, int offset) {

	for (int i=0; i<numVariables ;i++) {
		if (variables[i+offset]->index == var) {
			return i;
		}
	}

	return NO_VALUE;
}


//	Mark all variables of "numNotElimVars" as present/ not present at this function
//	Return true if there is at least 1 variable that was marked as "not present"
//	Else return false
bool FuncMatrix::markNotPresentVars( int* notElimVars, int numNotElimVars) {
	bool res = false;
	for(int v = 0; v < numVariables; ++v) {
		//	Get the index of the variable
		int index = variables[v]->getIndex();
		//	Check that current function's variable is present in "notElimVars" array
		bool present = false;
		for(int i = 0; i < numNotElimVars; ++i) {
			if (notElimVars[i] == index) {
				present = true;
				break;
			}
		}
		variables[v]->setIndication(present);
		//	If till now all variables were present, but current variable is not, then return true
		if ((res == false) && (present == false)) {
			res = true;
		}
	}
	return res;
}


//	All variables that have indication = "ind" are summed out from the function
void FuncMatrix::sumOverSignedVars(bool ind) {
	//	Count the number of non-indicated variables and the new size of the probs table
	int numRealVars = 0;
	int realSize = 1;
	for(int v = 0; v < numVariables; ++v) {
		DiscreteVar* var = variables[v];
		if (var->getIndication() != ind) {
			++numRealVars;
		}
	}
	if (numRealVars != numVariables) {
		//	Allocate new vars array
		DiscreteVar** newVars = new DiscreteVar*[numRealVars];
		//	Traverse all variables of the function and sum entries for non-indicated variables
		int varStateBlock = 1;
		int probsSize = numberOfEntries;
		for(int v = 0, newV = 0; v < numVariables; ++v) {
			DiscreteVar* var = variables[v];
			int numStates = var->getNumVals();
			if (var->getIndication() != ind) {
				//	This variable must stay in the function
				newVars[newV] = var;
				++newV;
				varStateBlock *= numStates;
			}
			else {
				//	This is indicated variable, so sum it out in the "probs" array
				sumProbBlocks(varStateBlock, numStates, probsSize);
				probsSize /= numStates;
			}
		}
		//	Update function fields with newly created structures
		delete[] variables;
		variables = newVars;
		numVariables = numRealVars;
		numberOfEntries = probsSize;
	}
}


//	Sum the entries corresponding to the given variable blocks
void FuncMatrix::sumProbBlocks(int varStateBlock, int numVarStates, int probsSize) {
	if (numVarStates > 1) {
		int varBlock = varStateBlock * numVarStates;
		int sumTo = 0;
		for (int currVarBlock = 0; currVarBlock < probsSize / varBlock; ++currVarBlock) {
			int currState = (currVarBlock == 0) ? 1 : 0;
			int sumFrom = currVarBlock * varBlock + currState * varStateBlock;
			for(; currState < numVarStates; ++currState) {
				for(int entry = 0; entry < varStateBlock; ++entry) {
					probs[sumTo + entry] += probs[sumFrom + entry];
					probs[sumFrom + entry] = 0;
				}
				sumFrom += varStateBlock;
			}
			sumTo += varStateBlock;
		}
	}
}



#endif // !defined(_FUNCMATRIX_H)


