/* $Id: InferEng_BayesNet.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#if !defined(_BAYESNET_H)
#define _BAYESNET_H

#include<stdio.h>
#include<stdlib.h>
#include <iostream>
#include <string>
#include <vector>
using namespace std;
#include "DiscreteFunc.h"
#include "DiscreteVar.h"
#include "InferEng_Settings.h"
#include "PedInfo.h"

#ifdef  __cplusplus
extern "C" {
#endif

	#include "../Product.h" 

#ifdef  __cplusplus
}
#endif

class BayesNet 
{
	friend class CInferEngine; 
	friend class Elim; 
	friend class BucketEliminator; 
public:
	BayesNet();
	virtual ~BayesNet();

	inline string			getName()				{return name;}
	inline void				setName(string name)	{this->name = name;}
	inline int				getNumVars()			{return numVariables;}
	inline DiscreteVar*		getVarAt(int indx)		{return variables[indx];}
	inline DiscreteFunc*	getFuncAt(int indx)		{return functions[indx];}
	inline DiscreteFunc*	getFuncOf(int varIndx)	{return functions[funcPointers[varIndx]];}
	inline int				getFunctionNum(int varIndx)	{return funcPointers[varIndx];}
	inline int				getMaxFunctionSize()	{return maxFunctionSize;}
	inline vector<DiscreteVar*>*	getVarArr()	{return &variables;}	

	void			virtual Init(product *prod);
	void			virtual addVariable(DiscreteVar* var);
	void			virtual addFunction(DiscreteFunc* func);
	//DiscreteVar*			getVarNamed(string str);
	//void			virtual printBN(string filename);



protected:
	string			name;		/* optional name of the network */
	int				numVariables;	/* number of variables */
	int				numFunctions;	/* number of variables */
		;		
	vector<DiscreteVar*>	variables;		/* array of variables */
	vector<DiscreteFunc*>	functions;		/* array of functions */
	vector<int>			funcPointers;	/* array of pointers from variables to their functions */
	int				maxFunctionSize;	/* number of variables in the largest function */
	int				maxValues;		/* maximum number of values in a variable; */
	_prob			initialProb;	/* the probability from the input for removing some founders */

};



//*******************************************************************************
// Implementation

BayesNet::BayesNet() {
	this->name = "";
	this->numVariables = 0;
	this->numFunctions = 0;
	this->maxFunctionSize = 0;
	this->maxValues = 0;
	this->initialProb = 0;
}




BayesNet::~BayesNet() {

	for (int i=0; i< this->numVariables ;i++) {
		delete this->variables[i];
	}

	for (int k=0; k< this->numFunctions ;k++) {
		delete this->functions[k];
	}
}



void BayesNet::Init(product *prod) {
	
	/* add all the variables in the product */
	for (int i=0; i<prod->numVariables ;i++) {

		if (strcmp(prod->variables[i].originalNode->name,"log_multFactor")!=0) {
			DiscreteVar* var = new DiscreteVar(this->numVariables, prod->variables[i].originalNode->name,
				prod->variables[i].numValues);
			
		
			addVariable(var);

		

			/* for convenience we change the original nodes pNum (not in use) */
			/* to the BayesNet index of the corresponding DiscreteVar */
			prod->variables[i].originalNode->pNum = this->numVariables-1;
		}
	}

	/* add all the functions in the product */
	for (int j=0; j<prod->numFunctions ;j++) {

		if (strcmp(prod->functions[j].variables[0]->originalNode->name,"log_multFactor") != 0) {

			DiscreteFunc* func = new DiscreteFunc();

			/* insert a pointer to the main variable */
			func->var = variables[prod->functions[j].variables[0]->originalNode->pNum];
			func->varDim = func->var->getNumVals();	

			if (func->varDim > this->maxValues) {
				this->maxValues = func->varDim;
			}
			
			/* insert pointers to the parent variables */
			/* to adhere to the convention of first variables are the LSB */
			/* we add the parents in a reverse order */
			func->numParents = prod->functions[j].numVariables-1;
			func->parents = new DiscreteVar*[func->numParents];
			func->pMultVals = new int[func->numParents];
			func->parentsDim = 1;

			for (int p=1, place=func->numParents-1; p<prod->functions[j].numVariables ;p++,place--) {
				func->parents[place] = variables[prod->functions[j].variables[p]->originalNode->pNum];
				func->parentsDim *= func->parents[place]->getNumVals();

				/* add parent-child pointers between the variable and the parent */
				func->var->addParent(func->parents[place]);
				func->parents[place]->addChild(func->var);
			}

			/* fill pMultVals */
			if (func->numParents > 0) {

				func->pMultVals[0] = 1;
				for (int p1=1; p1<func->numParents ;p1++) {
					func->pMultVals[p1] = func->pMultVals[p1-1] * func->parents[p1-1]->numVals;
				}
			}

			

			func->numInstants = func->varDim * func->parentsDim;
			func->pInstantVals = new int[func->numParents];

			if (func->numInstants > this->maxFunctionSize) {
				this->maxFunctionSize = func->numInstants;
			}

			/* fill the probability matrix */

			if (func->numInstants != prod->functions[j].sizeOfTable) {
				cout<<"error in filling prob list" << func->numInstants << prod->functions[j].sizeOfTable <<endl;
				exit(1);
			}

			func->probability = new _prob*[func->varDim];
			for (int k=0; k<func->varDim ;k++) {
				func->probability[k] = new _prob[func->parentsDim];
			}

			for (int vd=0, ins=0; vd<func->varDim ;vd++) {
				for (int pd=0; pd<func->parentsDim ;pd++) {
					func->probability[vd][pd] = prod->functions[j].probabList[ins];
					if (prod->functions[j].probabList[ins] == 0) {
						func->constrained = true;
					}
					++ins;
				}
			}
			
			
			addFunction(func);
		}
		else {
		//	this->initialProb = log(prod->functions[j].probabList[0]);
			this->initialProb = prod->functions[j].probabList[0];
		}
	}

	/* fill the linking vector funcPointers to allow for easy search in the network */
	int vSize = variables.size();
	int fSize = functions.size();

	for (int v=0; v<vSize ; v++) {
		for (int f=0; f<fSize ; f++) {
			if (variables[v] == functions[f]->getVar()) {
				funcPointers.push_back(f);
				break;
			}
		}
	}

	/* produce a clustering file according to the clustering scheme */
//	produceClusteringFile(prod);
}


void BayesNet::addVariable(DiscreteVar* var) {
	
	variables.push_back(var);
	++this->numVariables;
	++this->numFunctions;
}


void BayesNet::addFunction(DiscreteFunc* func) {
	
	functions.push_back(func);
}

/*
DiscreteVar* BayesNet::getVarNamed(string str) {

	for (int i=0; i<this->numVariables ;i++) {
		if (variables[i]->getName() == str) {
			return variables[i];
		}
	}

	return NULL;
}
*/


/*
void BayesNet::printBN(string filename) {

	ofstream outFile(filename.c_str());
	int numPars;
	PedInfo* inf;

	outFile << "id  ind  loc  type  side  numVals  name" << endl;

	for (int i=0; i<this->numVariables; i++) {
		outFile << "v " << this->variables[i]->getIndex() << "   ";
		inf = (PedInfo*)this->variables[i]->getInfo();
		if (inf != NULL) {
			outFile << inf->getIndividual() << " " << inf->getLocus() << " ";
			outFile << inf->getType() << " " << inf->getSide() << " "; 
		}
		outFile << this->variables[i]->getNumVals() << " ";
		outFile << this->variables[i]->getName();
		outFile << endl;
	}
	for (int f=0; f<this->numVariables ;f++) {
		numPars = this->functions[f]->getNumParents();
		outFile << "f " << numPars+1 << " " << this->functions[f]->getVar()->getIndex() << " ";
		for (int p=0; p<numPars ;p++) {
			outFile << functions[f]->getParent(p)->getIndex() << " ";
		}
		outFile << endl;
	}

	outFile.close();
}
*/




#endif // !defined(_BAYESNET_H)


