/* $Id: Node.h,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*****************************************************************************/
/*  Node.h																     */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/

#ifndef _NODE_H_
#define _NODE_H_


#include <stdio.h>
#include <stdlib.h>


/*------------------------------------------*/
/*		DATA-STRUCTURES	& DEFINITIONS		*/
/*------------------------------------------*/

/* The different type of nodes in the Bayesian network. */
typedef char		NodeType;
#define	SELECTOR	0
#define LOCUS		1
#define	PHENOTYPE	2
#define	GENOTYPE	3
#define GENOTYPE_MALE_SEXL	4
#define DUMMY		10;


/* Some of the nodes can be either paternal or maternal. */
typedef	char		NodeSex;
#define	PATERNAL	0
#define MATERNAL	1

/* Codes for the different types of variable unions possible. Either there's no unite 
   (noUnite), or the two genetic loci of a person at some locus are united (in case one 
   allele of the person is known for certain (oneAll), or in case the two alleles of 
   the person are known without phase (twoAll)), or the four genetic loci variables of 
   two parents at some locus are united in the case  where the four alleles of the 
   parents are known but their distribution to the two genotypes aren't known (fourAll). */
typedef char	uniteType;
#define	noUnite	0
#define	oneAll	1
#define	twoAll	2
#define fourAll	3


typedef struct variable variable;

typedef struct valEntry {
	int			val;
	boolean		isPossible;
} valEntry;


/* Holds information on a single node in the Bayesian network which represents the pedigree. */

typedef struct node {
	int				numParents;		/* The number of parents of the node, in the network. */
	struct node		**parents;		/* An array of pointers to the node's parents. */
	int				numChildren;	/* The number of children of the node, in the network. */
	int				numValues;		/* The number of possible values of the node. */
	variable		*varInProd;		/* Pointer to the variable that represents this node 
									   in the product. */
	char			name[15];		/* A name given to the node (used in development). */
	int				pNum;			/* The number of the person that this node represents. */
	int				lNum;			/* The number of locus that this node represents. */		
	NodeType		nodeType;		/* The type of this node. */
	NodeSex			nodeSex;		/* Indicates whether this is a paternal or a maternal node. */
	boolean			isKnown;		
	long			elimInd;		/* The index in the elimination order of this node. */
	int				currVal;		/* Holds the value of the node, used in the backtracing 
									   phase of the (MPE) haplotyping algorithm. */
#if UNITE_VARS
	uniteType		uniteT;			/* Indicates whether the variable corresponding to this node
									   needs to be united with other variables during the variable 
									   elimination. */
	int				*certainVal;	/* In case 'uniteT' is equal to 'oneAll', this field holds the
									   value of the variable (the index of the allele) which is known 
									   for certain in the genotype of this person at this locus. 
									   In case 'uniteT' is equal to 'fourAll', this field holds the 
									   possible values for this node. */
	struct node		**uniteVars;	/* In case the variable corresponding to this node needs to 
									   be united with other variables during the variable 
									   elimination, this array holds pointers to the nodes corresponding
									   to these variables. */
	boolean			isMyUniteVars;
	int				numVarsUnite;	/* The number of nodes in the 'uniteVars' array. */
#endif
} node;



#endif		/*	_NODE_H_	*/

