/* $Id: ProductValueElimination.h,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*****************************************************************************/
/*  ProductValueElimination.h												 */
/*  Author: Ma'ayan Fishelson												 */
/*****************************************************************************/
/*																			 */
/*  Comment: This file includes functions used for eliminating impossible    */
/*			 values from the probability tables of the Bayesian networks.	 */
/*																			 */
/*****************************************************************************/

#ifndef _PRODUCT_VALUE_ELIMINATION_H_
#define _PRODUCT_VALUE_ELIMINATION_H_


#include <stdio.h>
#include <stdlib.h>
#include "PedData.h"
#include "Product.h"
#include "Globals.h"
#include "ArgMax.h"


/*------------------------------*/
/*		DATA-STRUCTURES			*/
/*------------------------------*/

/* A struct which holds information on a node whose unite status needs to be changed. */

typedef struct changedUniteTNode {
	node						*Node;	/* Pointer to the node whose unite status has changed. */
	uniteType					uniteT;	/* The original unite status of the node. */
	struct changedUniteTNode	*next;	/* Pointer to the struct of the next node whose unite 
										   status has changed. */
} changedUniteTNode;

/* A struct which holds information on a list of nodes whose unite status needs to be changed. */

typedef struct changeUniteTList {
	changedUniteTNode	*head;
	changedUniteTNode	*next;
	int					numNodes;
} changeUniteTList;



/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/


typedef int (updateVarUsageInFuncs)(product *prod, pedigree *Ped, boolean *varValueUsageFlagsGen, 
								   boolean *varValueUsageFlagsFunc, int maxValuesVar, 
							       boolean *elimFuncs, int *numValsVarsNew, 
								   updatedNodesPed*	updatedNodesPedArr);

/* A type of a function that updates the uniteT status of the nodes of the Bayesian 
   network's nodes for a single pedigree. */

typedef void (updateUniteVarsPedFunc)(pedigree *pedPtr, changeUniteTList *listPtr);

#define INVALID_CURR		-2
#define INVALID_PRE			-1

#define	INVALID_VAL			FALSE
#define	VALID_VAL			TRUE



/*------------------------------*/
/*		GLOBAL VARIABLES		*/
/*------------------------------*/

#if (PRINT_ELIMINATION_STATISTICS || (PRINT_VARIOUS_STATISTICS_LEVEL > 1))
	int countTrimmedVars;
#endif

/*------------------------------*/
/*		FUNCTIONS				*/
/*------------------------------*/

/* The following functions deal with eliminating impossible values from the variables of
   the product. */

void eliminateImpossibleValsProducts(product** prodArr, boolean *areValsElim);
void eliminateImpossibleValsProductsHaplo(product** prodArr, boolean *areValsElim, 
 										  argMaxData *argMaxArr);
Probability eliminateImpossibleValues(product *prod, boolean *isZeroLike, boolean *areValsElim);
Probability eliminateImpossibleValuesHaplo(product *prod, boolean *isZeroLike, 
										   boolean *areValsElim, argMaxData *argMax);
int updateVariableUsageInFuncs(product *prod, boolean *varValueUsageFlagsGen, 
							   boolean *varValueUsageFlagsFunc, int maxValuesVar, 
							   boolean *elimFuncs, int *numValsVarsNew);

int getConstVal(boolean *arr, int arrSize);

/* The following functions deal with updating the unite status of nodes of the pedigrees. */

#if UNITE_VARS
	changeUniteTList* updateUniteVarsPeds(updateUniteVarsPedFunc updateUniteVars);
	changeUniteTList* updateUniteVarsPed(pedigree *currPed);
	void updateUniteVarsPedAutosomal(pedigree *currPed, changeUniteTList *listPtr);
	void updateUniteVarsPedSexLinked(pedigree *currPed, changeUniteTList *listPtr);
	void restoreUniteVals(changeUniteTList *listPtr);
#endif

#endif		/*	_PRODUCT_VALUE_ELIMINATION_H_	*/

