/* $Id: ReadData.c,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*********************************************************************************/
/*  ReadData.c																     */
/*  Author: Ma'ayan Fishelson													 */
/*********************************************************************************/

#include "ReadData.h"


/******************************************************************************
*
*       Function        init
*       Parameters      datafileName (the name of the locus file),
*						pedfileName (the name of the pedigree file).
*       Output          none
*       Description		Opens files, reads the locus file and the pedigree file,
*						initializes the appropriate data-structures, and
*						closes the files.
*       Comments		none
*
******************************************************************************/
void init(char *datafileName, char *pedfileName)
{
	pedFile = NULL;
	dataFile = NULL;
	progType = 0;

	/* 1.	Open the input files for reading. */

	openFiles(datafileName, pedfileName);

	/* 2.	Read the locus file. */

	readLocusFile();

	/* 3.	Read the pedigree file. */

	readPedFile(pedfileName);

	/* 4.	Close the files. */

	closeFiles(datafileName, pedfileName);

}


/******************************************************************************
*
*       Function        openFiles
*       Parameters      datafileName (the name of the locus file),
*						pedfileName (the name of the pedigree file).
*       Output          none
*       Description		Opens the files. If one of the files doesn't exist the  
*                       program exits and prints an error message.
*       Comments
*
******************************************************************************/
void openFiles(char *datafileName, char *pedfileName)
{
	/* 1.	Open the pedfile. */

	if (pedFile != NULL) {
		fclose(pedFile);
		printf("Reopening %s\n", pedfileName);
		pedFile = fopen(pedfileName, "r");
    }
	else {
		//printf("Opening %s\n", pedfileName);
		pedFile = fopen(pedfileName, "r");
    }

	if (pedFile == NULL) {
		error("File %s was not found", pedfileName);
	}

	/* 2.	Open the datafile. */

	if (dataFile != NULL) {
      fclose(dataFile);
      printf("Reopening %s\n", datafileName);
      dataFile = fopen(datafileName, "r");
    }
	else {
      printf("Opening %s\n", datafileName);
      dataFile = fopen(datafileName, "r");
    }

	if (dataFile == NULL) {
		error("File %s was not found", datafileName);
	}

	/* 3.	Check if one of the files is empty. */

	if (P_eof(pedFile)) {
      printf("ERROR: File empty or nonexistent. Press <Enter> to continue or <Ctrl-C> to abort\n");
      scanf("%*[^\n]");
      getchar();
    }

	if (P_eof(dataFile)) {
      printf("ERROR: File empty or nonexistent. Press <Enter> to continue or <Ctrl-C> to abort\n");
      scanf("%*[^\n]");
      getchar();
    }
}


/******************************************************************************
*
*       Function        closeFiles
*       Parameters      datafileName (the name of the locus file),
*						pedfileName (the name of the pedigree file).
*       Output          none
*       Description		Closes the files.
*       Comments		none
*
******************************************************************************/
void closeFiles(char *datafileName, char *pedfileName)
{
	/* 1.	Close the pedfile. */

	if (pedFile != NULL) {
		printf("Closing %s\n", pedfileName);
		fclose(pedFile);
	}
	pedFile = NULL;

	/* 2.	Close the datafile. */

	if (dataFile != NULL) {
		printf("Closing %s\n\n", datafileName);
	    fclose(dataFile);
	}
	dataFile = NULL;
}







