/* $Id: Settings.h,v 1.2 2008/08/03 08:15:46 superlink Exp $ */
/*****************************************************************************/
/*  Settings.h																 */
/*  Author: Ma'ayan Fishelson							                     */
/*****************************************************************************/

#ifndef _SETTINGS_H_
#define _SETTINGS_H_

#include <stdio.h>
#include <stdlib.h>
#include <float.h>


/*------------------------------*/
/*		DEFINITIONS				*/
/*------------------------------*/




#define INVALID		-1.0

/* Prints various statistics into the file "statistics.csv" according to the 
   level requested:
   0) No statistics are printed.
   1) Statistics regarding the elimination-order-finding-process are printed,
   2) and statistics regarding the treewidth of the graph, the number 
      of edges in the graph, the number of vertices, the number of constant vertices,
   3) and the minimum, average and maximum number of values of each vertex, 
   4) and the minimum, average and maximum number of selectors between each 2 
      consecutive loci.*/
#define		PRINT_VARIOUS_STATISTICS_LEVEL		0

/* Prints messages of how many variables have been trimmed from the graph, how 
   many values have been eliminated as a result of the value-elimination process,
   and how many nodes have been created. Also prints the number of nodes that are
   determined in the preprocessing phase. */
#define		PRINT_ELIMINATION_STATISTICS		0

/* Prints the percentage of the total computation that has been performed already. */
#define		PRINT_PROGRESS_REPORT				1

/* Prints messages regarding the optimization of the elimination order (for debugging). */
#define		PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER	1

/* Prints the complexity class of the final elimination-order found. */
#define		PRINT_COMPLEXITY_CLASS						1

/* A constant which determines when it's not worth to continue searching for a 
   variable to eliminate next (if the size of the combined table is equal to 
   this constant or below it, then we stop the search and return the current 
   variable to be eliminated.) */
#define		ELIMINATION_BOTTOM_THRESHOLD		15

/* A constant that determines when a more simple function will be used for 
   multiplying several probability functions (if the product of the size of the
   combined table and the number of functions that need to be combined is below
   this threshold, then the simple function is used).*/
#define		ELIMINATION_SIMPLE_THRESHOLD		24


/* Determines the maximum size of a probability table that would be processed,
   without product-breaking.  If the table that would be created equals this 
   size or exceeds it, product breaking shall be performed on some variable 
   that hasn't been eliminated yet. */

#define		BREAK_PRODUCT_BOTTOM_THRESHOLD		2000000

#define		MIN_DBL								-DBL_MAX

/* The number of times a function can be handled before it needs to be rescaled. */
#define		MAX_RESCALE_UNITS					8

/* The highest complexity coefficient for which only 2 iterations
   (LL and Greedy) are performed. */
#define		OPTIMIZATION_EO_BOTTOM_THRESHOLD	4

/* The smallest complexity for which big threshold can be applied */
#define		BIG_THRESHOLD_BOTTOM_OPTIMIZATION	9

/* The number of iterations that are performed for each order (general-greedy &
   locus-by-locus) to determine which order should be searched for. */
#define		NUM_TEST_ITER						5
#define     SMALL_NUM_TEST_ITER					3

/* The maximum number of non-founder individuals for which an order of 
   locus-after-locus is determined automatically (without testing which order is better). */
#define		MAX_NUM_NF_LOCUS_LOCUS				10

/* The minimum number of non-founder individuals for which a general greedy order is 
   determined automatically (without testing which order is better). */
#define		MIN_NUM_NF_GENERAL					30

#define		EPSILON								1e-5 

/* Determines whether approximate or exact calculations would be performed. */

#define		PERFORM_EXACT_COMPUTATIONS			1
  
/* Determines whether allele-recoding would be performed. */

#define		PERFORM_ALLELE_RECODING				1 // was 1

/* Determines whether all the selectors would be eliminated together (style Genehunter),
   or one by one. */

#define		ELIMINATE_SELECTORS_TOGETHER		1 // was 1

#define		USE_FFT								0

/* Determines whether a preprocessing phase would be applied on the Bayesian network 
   when determining the elimination-order for the variables of the Bayesian network. */

#define		PERFORM_PREPROCESSING				0  // was 1

#define		LOG_10								log(10.0)

/* Determines whether a Postscript graph with the results would be drawn. */

#define		PRINT_PS_GRAPH						1

/* Determines whether addresses of allocated memory blocks will be stored. */
#define		MANAGE_MEM							0

#define		CALC_MEM_REQ						1

#define		PRINT_MATRIX_MULT_STATISTICS		0

/* Determines whether this is a debug status. */
#define		DEBUG_STAT							1


#ifndef PARALLEL
/*We allow UNITE_VARS only if PARALLEL is disabled*/
/* Determines whether variable uniting will be performed during the computations. */
#define		UNITE_VARS							0
#else
#define		UNITE_VARS							0
#undef      DEBUG_STAT
#define     DEBUG_STAT                          1
#endif


/* Determines whether the edges between the selector variables will be removed when the recombination
   fraction between the two loci is 0.5 (the loci are unlinked). */
#define		DISCONNECT_UNLINKED					1 // was 1



#endif /* _SETTINGS_H_ */
