/* $Id: MapWithPositiveRange.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/************************************************************************************/
//	MapWithPositiveRange.c:
//	Implementation of MapWithPositiveRange: map with predefined positive range of keys (positive integers)
//	The values are also positive integers
//	Is implemented using array = range of keys
//	Written by Anisenia Andrei on 16/07/2008
/************************************************************************************/


#include "MapWithPositiveRange.h"



struct MapWithPositiveRange_t {
	int* values;
	int size;
};


//	Creation of the map: the keys will be [0..range] - and not [0..range-1]!!!
//	If range less or equal 0 => return NULL 
MapWithPositiveRange* createMapWithPositiveRange(int range) {

	MapWithPositiveRange* map = NULL;
	if (range > 0) {
		int i;
		sl_myMalloc(map, sizeof(MapWithPositiveRange));
		map->size = range + 1;
		sl_myMalloc(map->values, (map->size)*sizeof(int));
		for(i = 0; i < map->size; i++) {
			map->values[i] = -1;
		}
	}
	return map;
}



//	Setting the new value in the map: 
//	if mapWithPositiveRange is NULL or key is out of range or value is negative => do nothing
void setValue(MapWithPositiveRange* mapWithPositiveRange, int key, int value) {
	
	if ((mapWithPositiveRange != NULL) && (key >= 0) && (key < mapWithPositiveRange->size) && (value >= 0)) {
		mapWithPositiveRange->values[key] = value;
	}
}


//	Getting value from the map:
//	if mapWithPositiveRange is NULL or key wasn't stored in the map (or out of range) => return -1
int getValue(MapWithPositiveRange* mapWithPositiveRange, int key) {
	
	int value = -1;
	if ((mapWithPositiveRange != NULL) && (key >= 0) && (key < mapWithPositiveRange->size)) {
		value = mapWithPositiveRange->values[key];
	}
	return value;
}


void deleteMapWithPositiveRange(MapWithPositiveRange* mapWithPositiveRange) {
	if (mapWithPositiveRange != NULL) {
		free(mapWithPositiveRange->values);
		free(mapWithPositiveRange);
	}
}












