/* $Id: CheckHaplotyping.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
#include "gdata.h"
#include "sl_Auxilary.h"

#define GENEHUNTER_HAPLO_FILE "haplo.dump"
#define HAPLO_OUT_FILE		"haplo.dump.out"


//	internal functions (for internal use)
void parseGenehunterAlleles(Locus** lociAlleles,  int numInfMarkers);
void translateVector(int vectorInd, int* vector, int numBits);
Bool checkLocusHaplotypes(AllPedigrees* PedigreePtr, int* vector, Locus** lociAlleles,
						  int locusInd);
Bool checkParentAlleleCompatibility(AlleleData* paternalBitInfo, Locus* paternalLocus,
									int personAllele, int* vector);
void writeResult(Bool rightHaplotypes);


//	Checking compatibility of haplotypeVectors with "haplo.dump" genehunter haplotypes
void checkHaplotyping(int* haplotypeVectors, int numInfMarkers, AllPedigrees* PedigreePtr,
					  int numBits)
{
	Bool rightHaplotypes = true;
	int i, j, *vector;

	//	Create the structure for storing genehunter haplotypes
	Locus** lociAlleles;
	sl_myMalloc(lociAlleles, (PedigreePtr->size)*sizeof(Locus*));
	for(i = 0; i < PedigreePtr->size; i++) 
	{
		sl_myMalloc(lociAlleles[i], numInfMarkers*sizeof(Locus));
		PedigreePtr->PedigeeGraph[i].fixed = true;
	}
	//	Create vector
	sl_myMalloc(vector, numBits*sizeof(int));

	//	Parse GENEHUNTER_HAPLO_FILE alleles into lociAlleles structure
	parseGenehunterAlleles(lociAlleles, numInfMarkers);

	//	for each vector in haplotypeVectors:
	for(i = 0; i < numInfMarkers; i++)
	{
		for(j = 0; j < PedigreePtr->size; j++) 
		{
			if (PedigreePtr->PedigeeGraph[j].founder)
			{	
				PedigreePtr->PedigeeGraph[j].fixed = false;
			}
		}
		//	translate vector into bit array
		translateVector(haplotypeVectors[i], vector, numBits);
		//	check that vector is compatible with alleles of the persons
		rightHaplotypes *= checkLocusHaplotypes(PedigreePtr, vector, lociAlleles, i);
	}

	//	writing the result to the "haplo.dump"
	writeResult(rightHaplotypes);

	free(vector);
	for(i = 0; i < PedigreePtr->size; i++) 
	{
		free(lociAlleles[i]);
	}
	free(lociAlleles);
	free(haplotypeVectors);
}


//	writing the result to the "haplo.dump"
void writeResult(Bool rightHaplotypes)
{
	FILE* ghFile;

	ghFile = fopen(HAPLO_OUT_FILE, "w");
	if (rightHaplotypes)
	{
		fprintf(ghFile, "Yes\n");
	}
	else
	{
		fprintf(ghFile, "No\n");
	}
}


//	Parse GENEHUNTER_HAPLO_FILE alleles into lociAlleles structure
void parseGenehunterAlleles(Locus** lociAlleles, int numInfMarkers)
{
	FILE* ghFile;
	char string[100];
	int personIndex, garbage, locusInd;

	ghFile = fopen(GENEHUNTER_HAPLO_FILE, "r");
	//	go to the 2nd line
	fgets(string, 100, ghFile);
	
	//	for each line read the person's index
	while(!feof(ghFile)) 
	{
		//	get person index in the pedigree
		fscanf(ghFile,"%d",&personIndex);
		personIndex--;

		//	ignore person's parents and disease status
		fscanf(ghFile,"%d",&garbage);
		fscanf(ghFile,"%d",&garbage);
		fscanf(ghFile,"%d",&garbage);
		
		//	parse allele1 at each locus of the person
		for(locusInd = 0; locusInd < numInfMarkers; locusInd++)
		{
			fscanf(ghFile,"%d",&(lociAlleles[personIndex][locusInd].allele1));
		}
		//	parse allele2 at each locus of the person
		for(locusInd = 0; locusInd < numInfMarkers; locusInd++)
		{
			fscanf(ghFile,"%d",&(lociAlleles[personIndex][locusInd].allele2));
		}
	}
	fclose(ghFile);
}

//	translate vector into bit array
void translateVector(int vectorInd, int* vector, int numBits)
{
	int i;

	for(i = 0; i < numBits; i++)
	{
		vector[i] = vectorInd%2;
		vectorInd = vectorInd>>1;
	}
}


//	check that vector is compatible with alleles of the persons
Bool checkLocusHaplotypes(AllPedigrees* PedigreePtr, int* vector, Locus** lociAlleles,
						  int locusInd)
{
	int personInd, tmpAllele;
	Bool result = true;

	for(personInd = 0; personInd < PedigreePtr->size; personInd++)
	{
		Bool resultTmpFather;
		Bool resultTmpMother;
		//	check only informative non-founders
		if (PedigreePtr->PedigeeGraph[personInd].informative && 
				(!PedigreePtr->PedigeeGraph[personInd].founder))
		{
			int fatherInd, motherInd;
			//	check the compatibility of paternal allele
			fatherInd = PedigreePtr->PedigeeGraph[personInd].father - 1;
			resultTmpFather = checkParentAlleleCompatibility(
						PedigreePtr->PedigeeGraph[personInd].paternalAllele,
						&(lociAlleles[fatherInd][locusInd]),
						lociAlleles[personInd][locusInd].allele1,
						vector);
			if (PedigreePtr->PedigeeGraph[fatherInd].fixed == false)
			{
				PedigreePtr->PedigeeGraph[fatherInd].fixed = true;
				if (resultTmpFather == false)
				{
					tmpAllele = lociAlleles[fatherInd][locusInd].allele1;
					lociAlleles[fatherInd][locusInd].allele1 = 
						lociAlleles[fatherInd][locusInd].allele2;
					lociAlleles[fatherInd][locusInd].allele2 = tmpAllele;
					resultTmpFather = checkParentAlleleCompatibility(
						PedigreePtr->PedigeeGraph[personInd].paternalAllele,
						&(lociAlleles[fatherInd][locusInd]),
						lociAlleles[personInd][locusInd].allele1,
						vector);
				}
			}
			result *= resultTmpFather;

			//	check the compatibility of maternal allele
			motherInd = PedigreePtr->PedigeeGraph[personInd].mother - 1;
			resultTmpMother = checkParentAlleleCompatibility(
						PedigreePtr->PedigeeGraph[personInd].maternalAllele,
						&(lociAlleles[motherInd][locusInd]),
						lociAlleles[personInd][locusInd].allele2,
						vector);
			if (PedigreePtr->PedigeeGraph[motherInd].fixed == false)
			{
				PedigreePtr->PedigeeGraph[motherInd].fixed = true;
				if (resultTmpMother == false)
				{
					tmpAllele = lociAlleles[motherInd][locusInd].allele1;
					lociAlleles[motherInd][locusInd].allele1 = 
						lociAlleles[motherInd][locusInd].allele2;
					lociAlleles[motherInd][locusInd].allele2 = tmpAllele;
					resultTmpMother = checkParentAlleleCompatibility(
						PedigreePtr->PedigeeGraph[personInd].maternalAllele,
						&(lociAlleles[motherInd][locusInd]),
						lociAlleles[personInd][locusInd].allele2,
						vector);
				}
			}
			result *= resultTmpMother;

		}
	}
	return result;
}

//	check the compatibility of parental allele
Bool checkParentAlleleCompatibility(AlleleData* paternalBitInfo, Locus* paternalLocus,
									int personAllele, int* vector)
{
	Bool result;
	int *paternalAllelePtr, vectorBit;
	
	if (paternalBitInfo->bitType == NotUsedBit)
	{
		return true;
	}

	switch(paternalBitInfo->bitType)
	{
		case ReductionBit:	paternalAllelePtr = &(paternalLocus->allele1);
							break;
		case ConstBit:		(paternalBitInfo->value == 0) ?
								(paternalAllelePtr = &(paternalLocus->allele1)):
								(paternalAllelePtr = &(paternalLocus->allele2));
							break;
		case RegularBit:	vectorBit = vector[paternalBitInfo->number];
							(vectorBit == 0) ?
								(paternalAllelePtr = &(paternalLocus->allele1)):
								(paternalAllelePtr = &(paternalLocus->allele2));
							break;
		default:			break;
	}

	if ( (*paternalAllelePtr == 0) || (personAllele == 0) )
	{
		result = true;
	}
	else
	{
		(*paternalAllelePtr == personAllele) ? (result = true) : (result = false);
	}

	return result;
}