/* $Id: MemoryManagerInputOutputFormat.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
#ifndef MEMORY_MANAGER_INPUT_OUTPUT_FORMAT_H
#define MEMORY_MANAGER_INPUT_OUTPUT_FORMAT_H

//	Type of information pointer
typedef void* InfoPtr;

//	Pointers to C-style functions definition
#ifdef __cplusplus 
extern "C" { 
#endif
	//	Type of pointer to C function for deleting info (provided by C client)
	typedef void (*FuncDeleteInfoFromRAM) (InfoPtr infoPtr);
	//	Type of pointer to C function for writing info to file (provided by C client)
	//	Returns success indication: 0 - failed, 1 - succed
	typedef int (*FuncWriteInfoToFile) (InfoPtr infoPtr, const char* filePath, int slotInfoSize);
	//	Type of pointer to C function for reading info from file (provided by C client)
	//	Returns pointer to info if succed, else NULL (error!)
	typedef InfoPtr (*FuncReadInfoFromFile) (const char* filePath, int slotInfoSize);

#ifdef __cplusplus 
} 
#endif


//---------------------------------------------------------------------------------------------------------------------------------------------------
//	Format of MemoryManager Output:


//	Result possibilities of the memory manager operation	
typedef enum{	NoSuchInfoType,							//	setting/retrieving non-existing type of slot information
							SlotOutOfRange,							//	slot identifier is out of slots range [0..m-1]
							WindowCorruption,						//	operating on slot which is not on the boundary between 2 windows
							SlotInfoExists,								//	storing into the memory manager slot which was already stored 
							SlotInfoDoesntExist,						//	retrieving from the memory manager slot which is not in it's storage
							MemoryManagerInternalError,		//	some internal error in memory manager: DEBUG needed!!!!
							MemoryManagerNotNeeded,		//	indicates the case when window size is bigger than number of slots
							NullInfoPtr,									//	info pointer for storing operation is null
							Success} 
MemoryOperationResult;



//---------------------------------------------------------------------------------------------------------------------------------------------------
//	Format of MemoryManager Input:


//	Structure for slot info type
typedef struct SlotInfoType {
	//	size of slot info in bytes
	int slotInfoSize;		
	//	pointer to C function for deleting this type of slot info from RAM
	FuncDeleteInfoFromRAM funcDeleteSlotInfoFromRAM;		
	//	pointer to C function for writing this type of slot info to file
	FuncWriteInfoToFile funcWriteInfoToFile;
	//	pointer to C function for reading this type of slot info from file
	FuncReadInfoFromFile funcReadInfoFromFile;
}
SlotInfoType;


//	Input structure for constructor of MemoryManager
typedef struct MemoryManagerCreateInput {
	double availMemForWindowCalc;		//	available RAM memory for window calculation
	double availExternalMemForWindowCalc;		//	available external memory for window calculation
	int numSlots;											//	overall number of slots
	double sizeOfRunningSlotInfo;			//	sum of sizes of slot info for running in one forward phase  		
	SlotInfoType* slotsInfoTypesArray;	//	array of all slots info types	
	int numSlotsInfoTypes;						//	number of slots information types to be stored (size of slotsInfoTypesArray)
}
MemoryManagerCreateInput;












#endif

