/* $Id: NPLstudents.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  NPLstudents.c										   					*/
/*  Finds NPL score for each testPoint									 	*/
/*  Author: Andrei Anisenia													*/
/****************************************************************************/
#include "NPLstudents.h"
#include "likehood.h"
#include "fdata.h"
#include "allocfile.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include "fft.h"
#include "programSpec.h"
//#include "haplotype.h"
//#include "viterbi.h"
#include "sequence.h"


/****************************************************************************
*
*       Function		getRealFounders
*       Parameters	  	none	
*       Output			array of indexes of founders in PedigreeGraph   
*       Description		these are indexes of founders which have children
*						(not childless founders)
*       Comments		none
*
*****************************************************************************/
int* getRealFoundersArray()
{
	int* Real, i, ind;
	UnlinkedFounder* unf;

	Real = (int*)malloc(f*sizeof(int));
	ind = 0;
	for(i=0; i<AllPedigreesList->numOfFounders; i++)
	{
		unf = unlinkedFounders;
		while(unf != NULL)
		{
			if (unf->founder == PedigreeData.Founders[i])
				break;
			else unf = unf->next;
		}
		if (unf == NULL)
		{
			Real[ind] = PedigreeData.Founders[i];
			ind++;
		}
	}
	return Real;
}


/****************************************************************************
*
*       Function		InitPed
*       Parameters	  	none	
*       Output			none    
*       Description		intiates in each node in "ped" the next fields:
*						"af", "mother" and "father" according to the 
*						information in Pedigree
*       Comments		none
*
*****************************************************************************/
void InitPedigree()
{
	Pedigree* Ped;
	int* help, *RealFounders;
	/*	i - index in "ped"; P_index - index in "Ped"						*/
	int i, P_index;

	Ped = AllPedigreesList->PedigeeGraph;
	f = AllPedigreesList->numOfFounders - PedigreeData.ChildlessFounders;
	RealFounders = getRealFoundersArray();
	n = AllPedigreesList->numOfNonFounders;
	help = (int*)malloc((AllPedigreesList->size)*sizeof(int));
	/*	allocating "ped" array												*/
	ped = (ped_Node**)malloc((f+n)*sizeof(ped_Node*));
	for(i=0; i<f+n; i++)
		ped[i] =(ped_Node*)malloc(sizeof(ped_Node));
	/*	initiation for founders	and nonfounders								*/
	for(i=0; i<f+n; i++)
	{
		if (i<f)
		{
			P_index = RealFounders[i];
			ped[i]->father = -1;
			ped[i]->mother = -1;
		}
		else{
			P_index = PedigreeData.NonFounders[i-f];
			ped[i]->father = help[Ped[P_index].father-1];
			ped[i]->mother = help[Ped[P_index].mother-1];
		}
		ped[i]->PedIndex = P_index;
		/*	if the person is affected										*/
		if (Ped[P_index].myLocus->allele1 == 2)
				ped[i]->af = true;
		/*	if the person is not affected									*/
		else	ped[i]->af = false;
		help[P_index] = i;
	}
	free(help);
}


/****************************************************************************
*
*       Function		InitFounderAlleles  
*       Parameters	  	none	
*       Output			none    
*       Description		parses the alleles {0,...,2*f-1} between 
*						all the founders in ped, allocates f_alleles
*       Comments		none
*
*****************************************************************************/
void InitFounderAlleles()
{
	int i;
	Pedigree* Ped;

	Ped = AllPedigreesList->PedigeeGraph;
	numRealFAlleles = 0;
	for(i = 0; i < f; i++)
	{
		/*	for each founder:	*/
		/*	check that his paternal allele is used	*/
		if (Ped[ped[i]->PedIndex].paternalAllele->bitType != NotUsedBit)
		{	
			ped[i]->allele1 = numRealFAlleles;
			numRealFAlleles++;
		}
		else
		{
			ped[i]->allele1 = ALLELE_DOESNT_EXIST;
		}
		/*	check that his maternal allele is used	*/
		if (Ped[ped[i]->PedIndex].maternalAllele->bitType != NotUsedBit)
		{	
			ped[i]->allele2 = numRealFAlleles;
			numRealFAlleles++;
		}
		else
		{
			ped[i]->allele2 = ALLELE_DOESNT_EXIST;
		}
	}

	f_alleles = (int*)malloc((numRealFAlleles)*sizeof(int));
	for (i = 0; i < numRealFAlleles; i++)
	{
		f_alleles[i] = 0;
	}
}


/****************************************************************************
*
*       Function		InitAffected  
*       Parameters	  	none	
*       Output			none    
*       Description		creates "affected" array and links it to 
*						affected individuals in ped,
*						allocates "aff" array  
*       Comments		none
*
*****************************************************************************/
void InitAffected()
{
	int i, pushFront, pushBack;
	Pedigree* Ped;
	ped_Node *first, *last, *node_ptr; 
	
	af_num = 0;
	first = NULL;
	last = NULL;
	Ped = AllPedigreesList->PedigeeGraph;
	/* creating linked list of affected individuals and calculating af_num	*/
	for(i = 0; i < n+f; i++)
	{
		ped[i]->next_af = NULL;
		/*	check that person is affected and has some used allele	*/
		if (	(ped[i]->af == true) && 
				((Ped[ped[i]->PedIndex].paternalAllele->bitType != NotUsedBit) ||
				 (Ped[ped[i]->PedIndex].maternalAllele->bitType != NotUsedBit))  )
		{
			af_num++;
			if (first == NULL)
			{
				first = ped[i];
				last = first;
			}
			else
			{
				last->next_af = ped[i];
				last = last->next_af;
			}
		}
	}
		
	/* creating and initiating the array "affected"								*/	
	affected = (ped_Node**)malloc(af_num*sizeof(ped_Node*));
	/* converting linked list to array "affected" of size "af_num"				*/
	af_2allelesNum = 0;
	pushFront = 0;
	pushBack = af_num - 1;
	node_ptr = first;
	for(i = 0; i < af_num; i++)
	{
		/*	check if affected person has 2 used alleles		*/
		if ( (Ped[node_ptr->PedIndex].paternalAllele->bitType != NotUsedBit) &&
			 (Ped[node_ptr->PedIndex].maternalAllele->bitType != NotUsedBit) )
		{
			/*	if he has 2 used alleles => insert him to the front of "affected"	*/
			affected[pushFront] = node_ptr;
			pushFront++;
			af_2allelesNum++;
		}
		else
		{
			/*	if he has one allele used => insert him to the bask of "affected"	*/
			affected[pushBack] = node_ptr;
			pushBack--;
			/*	pointing "existingAllele" of the person to the allele != NotUsedBit	*/
			if (Ped[node_ptr->PedIndex].paternalAllele->bitType != NotUsedBit)
			{
				node_ptr->existingAllele = &(node_ptr->allele1);
			}
			else
			{
				node_ptr->existingAllele = &(node_ptr->allele2);
			}
		}
		node_ptr = node_ptr->next_af;
	}
	/* creating and initiating the array "aff"	for the future Sall calculation	*/
	aff = (int*)malloc(af_2allelesNum*sizeof(int));
}



/****************************************************************************
*
*       Function		InitUsefull  
*       Parameters	  	none	
*       Output			none    
*       Description		for each person in ped: if he is affected 
*						or has affected children then 
*						"usefull" is set to true, else false 
*       Comments		none
*
*****************************************************************************/
void InitUsefull()
{
	int i;
	/*	initiating all the "usefull"s to false								*/
	for(i = 0; i < n + f; i++)
	{
		ped[i]->usefull = false;
	}
	/*	going through "ped" in reverse (from end to start), from children 
		to parents, and changing "usefull" to true in affected individuals
		and all their relatives upstream in "ped"							*/
	for(i = n + f - 1; i >= 0; i--)
	{
		if ( (ped[i]->af == true) && (ped[i]->usefull == false) )
		{
			RecursiveUsefullTrue(i);
		}
	}
}


/****************************************************************************
*
*       Function		RecursiveUsefullTrue  
*       Parameters	  	ind - index of person in ped	
*       Output			none    
*       Description		recursive function, set "usefull" of the person "ind"
*						and all of his relatives upstream to true
*       Comments		help function of InitUsefull
*
*****************************************************************************/
void RecursiveUsefullTrue(int ind)
{
	int moth, fath;
	BitType paternalBit, maternalBit;
	Pedigree* Ped;

	Ped = AllPedigreesList->PedigeeGraph;
	/*	checking that person has some used alleles							*/
	paternalBit = Ped[ped[ind]->PedIndex].paternalAllele->bitType;
	maternalBit = Ped[ped[ind]->PedIndex].maternalAllele->bitType;
	if ( (paternalBit == NotUsedBit) && (maternalBit == NotUsedBit) )
	{
		return;
	}

	/*	changing "usefull" of the person									*/
	ped[ind]->usefull = true;

	moth = ped[ind]->mother;
	fath = ped[ind]->father;
	/*	checking if he is not a founder										*/
	if (moth == -1)	return;
	/*	recursive call: changing "usefull" of each parent if his bit is used*/
	if ( (ped[fath]->usefull == false) && (paternalBit != NotUsedBit) )
	{
		RecursiveUsefullTrue(fath);
	}
	if ( (ped[moth]->usefull == false) && (maternalBit != NotUsedBit) )
	{
		RecursiveUsefullTrue(moth);
	}
	return;
}


/****************************************************************************
*
*       Function		InitVector  
*       Parameters	  	none	
*       Output			none    
*       Description		allocates and initiates "Ivector", 
*						parses 2*f founder alleles between n nonfounders
*						according to initial Ivector "inheritance"(zero vector)
*       Comments		should be called after calling InitUsefull
*
*****************************************************************************/
void InitVector()
{
	int i,j, IvectorIndex, mappingVectorIndex, *associatedAllele;
	Pedigree* person;
	ped_Node* parent;
	BitType paternalBit, maternalBit;
	AlleleData* parentalBit;
	Pedigree* Ped;

	Ped = AllPedigreesList->PedigeeGraph;
	/*	go through all non-founders in the pedigree and count number		*/
	/*	of const + regular bits (= sizeOfIvector)							*/
	sizeOfIvector = 0;
	for(i = f; i < n + f; ++i)
	{
		person = &(Ped[ped[i]->PedIndex]);
		paternalBit = person->paternalAllele->bitType;
		if ( (paternalBit == ConstBit) || (paternalBit == RegularBit) )
		{
			++sizeOfIvector;
		}
		maternalBit = person->maternalAllele->bitType;
		if ( (maternalBit == ConstBit) || (maternalBit == RegularBit) )
		{
			++sizeOfIvector;
		}
	}
	/*	allocating Ivector and mappingVector								*/
	sl_myMalloc(Ivector, sizeOfIvector*sizeof(vector_node));
	sl_myMalloc(mappingVector, (PedigreeData.SpeedUpNumOfMeioses)*sizeof(int));
	
	/*	go through all non-founders in the pedigree and init inheritance bits	*/
	for(i = f, IvectorIndex = 0, mappingVectorIndex = 0; i < n + f; ++i)
	{
		/*	treat paternal allele of the person		*/
		parent = ped[ped[i]->father];
		parentalBit = Ped[ped[i]->PedIndex].paternalAllele;
		associatedAllele = &(ped[i]->allele1);
		for(j = 0; j < 2; j++)
		{
			switch(parentalBit->bitType)
			{
				case NotUsedBit:	*associatedAllele = ALLELE_DOESNT_EXIST;
									break;
				case ReductionBit:	*associatedAllele = parent->allele1;
									break;
				case ConstBit:		Ivector[IvectorIndex].inheritance = 
										parentalBit->value;
									Ivector[IvectorIndex].usefull = ped[i]->usefull;
									Ivector[IvectorIndex].person = i;
									Ivector[IvectorIndex].parent = 
										Ped[parent->PedIndex].sex;
									++IvectorIndex;
									break;
				case RegularBit:	Ivector[IvectorIndex].inheritance = 0;
									Ivector[IvectorIndex].usefull = ped[i]->usefull;
									Ivector[IvectorIndex].person = i;
									Ivector[IvectorIndex].parent = 
										Ped[parent->PedIndex].sex;
									mappingVector[mappingVectorIndex] = IvectorIndex;
									++IvectorIndex;
									++mappingVectorIndex;
									break;
				default:			break;
			}
			if (j == 1) break;
			/*	treat maternal allele of the person		*/
			parent = ped[ped[i]->mother];
			parentalBit = Ped[ped[i]->PedIndex].maternalAllele;
			associatedAllele = &(ped[i]->allele2);
		}	
	}	
	RearrangeAlleles(0);
}



/****************************************************************************
*
*       Function		RearrangeAlleles  
*       Parameters	  	vect_ind : index of person's allele in Ivector
*						from which we should compute arrangement of alleles	
*						downstream in Ivector for each person
*       Output			none    
*       Description		none
*       Comments		help function of InitVector (but can be used anywhere)
*
*****************************************************************************/
void RearrangeAlleles(int vect_ind)
{
	int i, person;
	for(i = vect_ind; i < sizeOfIvector; i++)
	{
		person = Ivector[i].person;
		/* if we want to set father's allele of the person					*/
		if (Ivector[i].parent == 0)
			/* if we need the paternal allele of father						*/
			if (Ivector[i].inheritance == 0)
					ped[person]->allele1 = ped[ped[person]->father]->allele1;
			/* if we need the maternal allele of father						*/
			else	ped[person]->allele1 = ped[ped[person]->father]->allele2;
		/* if we want to set mother's allele of the person					*/
		else
			/* if we need the paternal allele of mother						*/
			if (Ivector[i].inheritance == 0)
					ped[person]->allele2 = ped[ped[person]->mother]->allele1;
			/* if we need the maternal allele of mother						*/
			else	ped[person]->allele2 = ped[ped[person]->mother]->allele2;
	}
}


/****************************************************************************
*
*       Function		Destroy  
*       Parameters	  	none
*       Output			none    
*       Description		frees all the allocated memory
*       Comments		none
*
*****************************************************************************/
void Destroy()
{
	int i;

	for(i=0; i<f+n; i++)
		free(ped[i]);
	free(ped);
	free(affected);
	free(aff);
	free(f_alleles);
	free(Ivector);
	free(mappingVector);
	free(powersOf2);
}


/****************************************************************************
*
*       Function		getSpairs()
*       Parameters	  	none
*       Output			none    
*       Description		computes Spairs score for the current Ivector
*						saves it in "Spairs" variable and in "SpairsArray"
*						at "next" cell
*       Comments		sets to zero "f_alleles" before computing score
*
*****************************************************************************/
void getSpairs()
{
	int i, al1, al2, al;

	Spairs = 0;
	for (i = 0; i < numRealFAlleles; i++)
	{
		f_alleles[i] = 0;
	}
	for(i = 0; i < af_2allelesNum; i++)
	{
		al1 = affected[i]->allele1;
		al2 = affected[i]->allele2;
		Spairs += f_alleles[al1];
		Spairs += f_alleles[al2];
		f_alleles[al1]++;
		f_alleles[al2]++;
	}
	for(i = af_2allelesNum; i < af_num; ++i)
	{
		al = *(affected[i]->existingAllele);
		Spairs += f_alleles[al];
		f_alleles[al]++;
	}
	SpairsArray[next] = Spairs;
}


/****************************************************************************
*
*       Function		getNewScores  
*       Parameters	  	i - index in Ivector
*       Output			none    
*       Description		after changing "inheritance" at index "i" 
*						computes Spairs and Sall scores (by using
*						functions getSpairs and getSall)
*       Comments		none
*
*****************************************************************************/
void getNewScores(int index)
{
	int i = mappingVector[index];

	/* converting new vector (with changed "inheritance" at index "i")		*/ 
	/* to a new int number and saves it in "next"							*/
	if ( Ivector[i].inheritance == 0)
		next += powersOf2[index];
	else next -= powersOf2[index];

	/*	changing inheritance of index "i" in Ivector (0 -> 1; 1 -> 0)	*/
	Ivector[i].inheritance = 1 - Ivector[i].inheritance;
	if (Ivector[i].usefull == false)
	{
		SpairsArray[next] = Spairs;
		SallArray[next] = Sall;
	}
	else
	{
		RearrangeAlleles(i);
		getSpairs();
		getSall();
	}
}


/****************************************************************************
*
*       Function		computeScores  
*       Parameters	  	none
*       Output			none    
*       Description		computes Spairs and Sall scores 
*						for all inheritance vectors and saves them
*						in SpairsArray and SallArray
*       Comments		none
*
*****************************************************************************/
void computeScores()
{
	int i, tmp;
	grayCodeGenerator* generator;

	numCollections = pow(2,af_2allelesNum);
	SpairsArray = (double*)malloc(PedigreeData.Possibilities*sizeof(double));
	SallArray = (double*)malloc(PedigreeData.Possibilities*sizeof(double));	
	next = 0;
	getSpairs();
	getSall();
	powersOf2 = (int*)malloc((PedigreeData.SpeedUpNumOfMeioses)*sizeof(int));
	tmp = 1;
	for(i = 0; i < PedigreeData.SpeedUpNumOfMeioses; i++)
	{
		powersOf2[i] = tmp;
		tmp = tmp*2;
	}

	generator = initiateGrayCodeGenerator(PedigreeData.SpeedUpNumOfMeioses);
	for(i = 1; i < PedigreeData.Possibilities; i++)
	{
		getNewScores(getNextGrayCodeNumber(generator));
	}
	destroyGrayCodeGenerator(generator);
}


/****************************************************************************
*
*       Function		getSall()
*       Parameters	  	none
*       Output			none    
*       Description		computes Sall score for the current Ivector
*						saves it in "Sall" variable and in "SallArray"
*						at "next" cell
*       Comments		sets to zero "f_alleles" and to 1 "aff"
*						before computing score	
*					
*****************************************************************************/
void getSall()
{
	int i, allele;

	SallTmp = 1;
	for (i = 0; i < numRealFAlleles; i++)
	{
		f_alleles[i] = 0;
	}
	/*	 collecting allele1 from each affected with 2 used alleles			*/
	for (i = 0; i < af_2allelesNum; i++)
	{
		aff[i] = 1;
		allele = affected[i]->allele1;
		f_alleles[allele]++;
		SallTmp = SallTmp*( f_alleles[allele] );
	}
	/*	collecting existing allele from each affected with 1 used allele	*/
	for(i = af_2allelesNum; i < af_num; ++i)
	{
		allele = *(affected[i]->existingAllele);
		f_alleles[allele]++;
		SallTmp = SallTmp*( f_alleles[allele] );
	}
	Sall = SallTmp;
	compute_algorithm(af_2allelesNum, getSallTmp);
	Sall = Sall/numCollections;
	SallArray[next] = Sall;
}


/****************************************************************************
*
*       Function		getSallTmp
*       Parameters	  	i - index in "aff"
*       Output			none    
*       Description		recomputes the value of SallTmp by using it's
*						previous value and the index of the affected person
*						for whom allele's collection was changed
*       Comments		none
*
*****************************************************************************/
void getSallTmp(int i)
{
	if (aff[i] == 1)
	{
		aff[i] = 2;
		SallTmp = SallTmp/( f_alleles[affected[i]->allele1] );
		f_alleles[affected[i]->allele1]--;
		f_alleles[affected[i]->allele2]++;
		SallTmp = SallTmp*( f_alleles[affected[i]->allele2] );
	}
	else
	{
		aff[i] = 1;
		SallTmp = SallTmp/( f_alleles[affected[i]->allele2] );
		f_alleles[affected[i]->allele2]--;
		f_alleles[affected[i]->allele1]++;
		SallTmp = SallTmp*( f_alleles[affected[i]->allele1] );
	}
	Sall += SallTmp;
}


/****************************************************************************
*
*       Function		normalizeScores
*       Parameters	  	none
*       Output			none    
*       Description		brings Sall and Spairs scores to standart normal form
*       Comments		none
*
*****************************************************************************/
void normalizeScores()
{
	normalizeS(S_PAIRS);
	normalizeS(S_ALL);
}


/****************************************************************************
*
*       Function		normalizeS
*       Parameters	  	stype - type of the score (Sall or Spairs) 
*						to be normalized
*       Output			none  
*       Description		help function of normalizeScores
*       Comments		none
*
*****************************************************************************/
void normalizeS(SCORE stype)
{
	VECTOR SArray;
	int i;
	double sum,mean, var, sd;

	if (stype == S_PAIRS)
		SArray = SpairsArray;
	else SArray = SallArray;
	/*	computing sum of scores												*/
	sum = 0;
	for (i=0; i<PedigreeData.Possibilities; i++)
		sum += SArray[i];
	/*	computing variance of scores										*/
	mean = sum/PedigreeData.Possibilities;
	var = 0.00;
	for (i=0; i<PedigreeData.Possibilities; i++)
		var += ( (SArray[i] - mean)*(SArray[i] - mean) );
	var = var/(PedigreeData.Possibilities);
	/*	computing standart deviation of scores								*/
	if (var == 0)
		/*	the case when there is no variance								*/
			sd = 1;	
	else	sd = sqrt(var);
	/*	normalizing scores using mean and sd								*/
	for (i=0; i<PedigreeData.Possibilities; i++)
		SArray[i] = ((SArray[i] - mean)/sd);
}




/******************************************************************************
*
*       Function        computeNPL
*       Parameters		none
*       Output          none
*       Description		compute NPL for all points 
*       Comments		is used in "gh_main.c"
*
*****************************************************************************/
void computeNPL()
{
	InitPedigree();
	InitFounderAlleles();
	InitAffected();
	InitUsefull();
	InitVector();
	computeScores();
	normalizeScores();
	Destroy();
}



