#include "ReadInputFiles.h"
#include "programSpec.h"
#include "../LociData.h"



/******************************************************************************
*
*       Function        Read_Pedegree_File
*       Parameters		none
*       Output          none
*       Description		reads the sl_pedigree file. 
*       Comments		none
*
******************************************************************************/
void Read_Pedegree_File(){
	FILE *stream;
	int i = 0, locusInd , countEffective = 0, pedNum,pid;
	Locus *locusPtr;
	AllPedigrees *currPed;
	Pedigree *pedPtr;

	PedigreeData.PedigreeSize = countNumPeople(PEDEGREE_FILE);

	if((stream = fopen(PEDEGREE_FILE, "r")) == NULL){
		sl_inputErr("cannot open sl_pedigree file\n");
	}
	for (currPed = AllPedigreesList; currPed != NULL; currPed = currPed->next){
		currPed->numOfFounders = 0;
		currPed->numOfNonFounders = 0;
		sl_myMalloc(currPed->PedigeeGraph,(sizeof(Pedigree)*currPed->size));
		for (i =0; i < currPed->size;	i++){
			
			fscanf(stream,"%d%d",&pedNum,&pid);
			
			pedPtr = currPed->PedigeeGraph + pid -1;
			pedPtr->index = pid;
			sprintf(pedPtr->personName, "%d", pid);
			pedPtr->pedNum = pedNum;
			pedPtr->informative = false;
			pedPtr->markersInformative = false;
			pedPtr->diseaseInformative = false;
			pedPtr->temp = false;
			pedPtr->multiplicity = 1;
			fscanf(stream,"%d%d%d",&(pedPtr->father),&(pedPtr->mother),&(pedPtr->child));
			
			if (pedPtr->child != 0){
				pedPtr->informative = true;
			}

			fscanf(stream,"%d%d%d%d",&(pedPtr->NeSiPa),&(pedPtr->NeSiMa),&(pedPtr->sex),&pid);
			pedPtr->sex = (pedPtr->sex == 1)?Male:Female;

			sl_myMalloc(pedPtr->paternalAllele, sizeof(AlleleData));
			pedPtr->paternalAllele->bitType = RegularBit;
			sl_myMalloc(pedPtr->maternalAllele, sizeof(AlleleData));
			pedPtr->maternalAllele->bitType = RegularBit;
			sl_myMalloc(pedPtr->myLocus,(sizeof(Locus)*NumOfLocuses));
			for (locusInd = 0, locusPtr = pedPtr->myLocus ; locusInd < NumOfLocuses; locusInd++, locusPtr++){
				fscanf(stream,"%d",&(locusPtr->allele1));
				if (locusInd == DesiaseLocus
					&& lociDataArray[DesiaseLocus].num_of_liab == 1){
					locusPtr->allele2 = 1;
				}else{
					fscanf(stream,"%d",&(locusPtr->allele2));		
				}
			
				if ( (ProgramSpecificData.sexLinked) &&
						(locusInd != DesiaseLocus) && (pedPtr->sex == Male) )
				{
					locusPtr->allele2  = locusPtr->allele1;

				}
				if (locusInd == DesiaseLocus && locusPtr->allele1){
					pedPtr->diseaseInformative =true;
				}
				if (locusInd != DesiaseLocus && 
					(locusPtr->allele1 | locusPtr->allele1)){
					pedPtr->markersInformative = true;
				}
				if ( locusInd != DesiaseLocus){
					if	((locusPtr->allele1 == 0 && locusPtr->allele1 != 0) ||
						(locusPtr->allele1 != 0 && locusPtr->allele1 == 0)){
							sl_inputErr("There's an error in sl_locus #%d of sl_person #%d, sl_pedigree #%d. (1 allele is known and the other unknown)\n", 
								locusInd+1, pedPtr->index, pedPtr->pedNum);
						}
				}

			}
			pedPtr->informative = pedPtr->markersInformative || pedPtr->diseaseInformative
				||	pedPtr->informative;

			if (pedPtr ->informative){
				if ((pedPtr->father == 0) && (pedPtr->mother == 0)){
					pedPtr->num= currPed->numOfFounders++;
					pedPtr->founder=1;

				}
				else {
					pedPtr->num = currPed->numOfNonFounders++;
					pedPtr->founder=0;			
				}
			}else{ 
				pedPtr->num = -1;
				printf("Uniformative person was found: pedigree %d person %d\n",pedPtr->pedNum, pedPtr->index);
			}
			// read the rest of the line
			fscanf(stream, "%*[^\n]\n");
		}
	}

	fclose(stream);				
}


/******************************************************************************
*
*       Function        Read_Loci_File
*       Parameters		none
*       Output          none
*       Description		reads the sl_locus file and initializes the variables that
*						hold the data regarding the sl_loci being analyzed. 
*       Comments		none
*
******************************************************************************/
void Read_Loci_File(){
	FILE *stream;
	LOCUS_DATA *locusPtr;
	double *freqPtr,*freqPtrEnd;
	char line[LineLength], *token;
	double maxFreq = 0, sumFreq;
	char *name;
	int i, j, k, type_of_locus, num_of_freq, num_of_liab, *lociOrder;
	double recombVal;


	/* 1.0 Open file */
	if((stream = fopen(LOCI_FILE, "r")) == NULL){
		sl_inputErr("cannot open data(sl_loci) file\n");
	}

	/* 2.0 get number of sl_loci and allocate meory for it */
	fgets( line, LineLength, stream );
	token = strtok(line," \t");
	NumOfLocuses = atoi(token);
	token = strtok(NULL," \t");
	token = strtok(NULL," \t");
	ProgramSpecificData.sexLinked = (atoi(token) == 1)?true:false;

	sl_myMalloc(lociDataArray ,(sizeof(LOCUS_DATA) * NumOfLocuses));

	/* 3.0 Curently ignored */
	fgets( line, LineLength, stream );
	/* 4.0 - ignored */
	sl_myMalloc(lociOrder, NumOfLocuses * sizeof(int));
	//	Perform fscanf operations till the end of loci numbers
	lociOrder[0] = 0;
	for(i = 1 ; i < NumOfLocuses; i++){
		fscanf(stream, "%d", lociOrder + i);
	}
	fgets( line, LineLength, stream );

	/* 5.0 read information for each locus */
	for(i = 0, locusPtr = lociDataArray + lociOrder[i] ; i < NumOfLocuses; i++){
		locusPtr = lociDataArray + lociOrder[i];
		fgets( line, LineLength, stream );
		/* find name of the locus */
      sprintf(locusPtr->locusName,"loc%d",i);
		name = strchr(line,'#');
		if ( name ){
			name = strtok(name,"#");
			if (name){
				strcpy(locusPtr->locusName,name);
			}
		}
		type_of_locus = atoi(strtok(line," \t"));
		if(type_of_locus == 3 || type_of_locus == 1){
			/* 5.1 Read information about the sl_locus */
		    num_of_freq = atoi( strtok(NULL," \t"));
			locusPtr->locus_type = type_of_locus;
			locusPtr->num_of_freq = num_of_freq;
			sl_myMalloc(locusPtr->frequencies,(sizeof(double)*num_of_freq));
		

            /* 5.2 Read Frequencies of the sl_locus */
			fgets( line, LineLength, stream );
			freqPtr = locusPtr->frequencies;
		
			(*freqPtr) = atof(strtok(line," \t"));
		
	
			maxFreq = (*freqPtr);

			sumFreq = (*freqPtr);
			for(freqPtrEnd = freqPtr + num_of_freq, freqPtr++, j = 2;
				freqPtr < freqPtrEnd ; freqPtr++,j++){
					(*freqPtr) = atof(strtok(NULL," \t"));
				
					sumFreq += (*freqPtr);
					if (maxFreq < (*freqPtr)){
						maxFreq = *freqPtr;
					}
					
				}
				locusPtr->alleleFreqDifFromOne = 1 - sumFreq;
					if (sumFreq < 0.99999 && sumFreq > 1.00001) {
						sl_inputErr("The gene frequencies of locus #%d sums to %.10lf instead of 1.0. \n", i+1, sumFreq);
					}
				

		
		}
		/* 5.3 If sl_locus is affectted*/ 
		if(type_of_locus == 1){
			DesiaseLocus=i;
			fgets( line, LineLength, stream );
			num_of_liab = atoi(strtok(line," \t"));
			lociDataArray[i].num_of_liab = num_of_liab;
			for(k = 0; k < 4; k++){
				sl_myMalloc((locusPtr->penetrances[k]),(sizeof(double) * num_of_liab));
			}
			if (ProgramSpecificData.sexLinked){
				for(k = 0; k < 2; k++){
					sl_myMalloc((locusPtr->malePenatrences[k]),(sizeof(double) * num_of_liab));
				}
			}
			for(j = 0; j < num_of_liab; j++){
				char* str;
				fgets( line, LineLength, stream );
				locusPtr->penetrances[0][j] = atof(strtok(line," \t"));
				for(k = 1; k < 4; k++){
					//	added by Anisenia Andrei
					str = strtok(NULL," \t");
					if ((str == NULL) || (strcmp(str,"<<") == 0) ){
						imprinting = FALSE;		
					}
					else{
						locusPtr->penetrances[k][j] = atof(str);
					}
				}
				//	added by Anisenia Andrei 
				if (imprinting == FALSE) {
					//	the non-imprinting case (only 3 penetrances)
					locusPtr->penetrances[3][j] = locusPtr->penetrances[2][j];
					locusPtr->penetrances[2][j] = locusPtr->penetrances[1][j];
				}
				/////////////////////////////////////////
				
				if (ProgramSpecificData.sexLinked){
					fgets( line, LineLength, stream );
					locusPtr->malePenatrences[0][j] = atof(strtok(line," \t"));
					for(k = 1; k < 2; k++){
						locusPtr->malePenatrences[k][j] = atof(strtok(NULL," \t"));		
					}
				}
			}
		}
		if(type_of_locus != 1 && type_of_locus != 3){
		 printf("ERROR: Current version can handle loci of type 1 and 3 only\n");

			exit(1);
		}
	}
	/* 6.0  Sex difference sl_interference - Ignored */
	fgets( line, LineLength, stream );
	/* 7.0  Recombination values */
	sl_myMalloc(Recomb_values,(sizeof(double) * (NumOfLocuses - 1)));
	freqPtr = Recomb_values - 1;
	fscanf(stream, "%lf", &recombVal);
	for(freqPtr++ ,freqPtrEnd = Recomb_values + NumOfLocuses - 1 ; 
														freqPtr < freqPtrEnd ;  freqPtr++){
		int res = fscanf(stream, "%lf", freqPtr);
		if (res == 0) {
			*freqPtr = 0;
		}
	}
	fgets( line, LineLength, stream );
	/* 8.0 Get program specific information */
	readProgSpec(8,stream);
	fclose(stream);
	free(lociOrder);
}



/******************************************************************************
*
*       Function        countNumPeople
*       Parameters		fileName (the name of the sl_pedigree file for which 
*						data-structures need to be allocated).
*       Output          Number of people in the file
*       Description		reads the sl_pedigree file, counts the number of 
*						pedigrees and the number of people in each sl_pedigree
*       Comments		none
*
******************************************************************************/
int countNumPeople(char *fileName){
	int numPeople,nextPed, currPed;
	AllPedigrees *currPedPtr = NULL;
	FILE*  pedFile;

	NumPedigrees = 0;

	if((pedFile = fopen(fileName, "r")) == NULL){
		sl_inputErr("cannot open sl_pedigree file\n");
	}

	fscanf(pedFile, "%d", &nextPed);

	/* 2.	While EOF hasn't been reached, read the data regarding the next sl_pedigree. */


	while (!sl_P_eof(pedFile)) {		
		currPed = nextPed;
		numPeople = 0;
		if (currPedPtr == NULL){
			sl_myMalloc(currPedPtr,sizeof(AllPedigrees));
			AllPedigreesList = currPedPtr;
		}else{
			sl_myMalloc(currPedPtr->next,sizeof(AllPedigrees));
			currPedPtr = currPedPtr->next;
		}
		/* 2.1.	While EOF hasn't been reached and it's still the same sl_pedigree. */

		while (!sl_P_eof(pedFile) && (currPed == nextPed)) {

			/* 2.1.1.	Increase the count of the number of people. */

			numPeople++;

			/* 2.1.2.	Read the rest of the line. */

			fscanf(pedFile,"%*[^\n]\n");

			/* 2.1.3.	Read the ped id field in the next line. */

			if (!sl_P_eof(pedFile))
				fscanf(pedFile, "%d", &nextPed);
		}
		currPedPtr->size = numPeople;
		/* 2.2.	Increase the count of the number of pedigrees in the file. */

		NumPedigrees++;

	}
	currPedPtr->next = NULL;
	fclose(pedFile);
	return numPeople;
}


