/* $Id: allocfile.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  allocfile.c 											   				*/
/*  Allocation and initializing big data structures							*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/

#include "allocfile.h"
#include "fdata.h"
#include "gdata.h"
#include  "likehood.h"
#include <stdio.h>
#include <stdlib.h>
#include "sl_Auxilary.h"
#include "AllelesInheritance/AllelesInheritance.h"


int inhVector;


/******************************************************************************
*
*       Function        printerror
*       Parameters		str - error message
*       Output          none
*       Description		Prints error and exit program 
*       Comments		
*
******************************************************************************/
void printerror(char *str){
	printf("alloc error %s",str);
	exit(1);
}


/*****************************************************************************
*       Function        findFounders
*       Parameters		pedPtr - pointer to the sl_person data
*						founder1 - from which founder patenal allele was inhereted
*						founder2 - from which founder matenal allele was inhereted
*       Output          none
*       Description		finds founders sl_alleles from which sl_person sl_alleles were inhereted
*       Comments		does NOT eliminate all unpossible assignments
*
******************************************************************************/
void findFounders(Pedigree *pedPtr,int *founder1, int *founder2, int curVector){
	#if (ALLELES_INH_USE == 0) 
		*founder1 = AlleleInh[curVector][pedPtr->num*2];
		*founder2 = AlleleInh[curVector][pedPtr->num*2 + 1];
	#else
		*founder1 = getAllele(allelesInhObj, curVector, pedPtr->num*2);
		*founder2 = getAllele(allelesInhObj, curVector, pedPtr->num*2 + 1);
	#endif
}



/******************************************************************************
*
*       Function        allocData
*       Parameters		none
*       Output          none
*       Description		Allocates memory for array(size==NumOfLocuses)of Vectors(size=numOfPossibilities)
*       Comments		
*
******************************************************************************/
VECTOR* allocData(){
	VECTOR* data;
	int i;
#if DEBUG_PRINT == 1
	printf("function allocData() NumOfLocuses = %d,PedigreeData.Possibilities = %d\n",NumOfLocuses,PedigreeData.Possibilities);
#endif
	sl_myCalloc(data,NumOfLocuses,(sizeof(double*)));
	for (i=0;i<NumOfLocuses - 1;++i){ 
			 sl_myMalloc(data[i],((PedigreeData.Possibilities)*sizeof(double)));
		//	myBigMalloc(data[i],(Possibilities),sizeof(double));
	}
	return data;
}


void allocDataNew(VECTOR* data, int startLocus, int endLocus){
	int i;
	for (i = startLocus; i <= endLocus; ++i){ 
			 sl_myMalloc(data[i],((PedigreeData.Possibilities)*sizeof(double)));
	}
}


/******************************************************************************
*
*       Function        allocVector
*       Parameters		none
*       Output          none
*       Description		allocation of array (size = Possibilities)and initialize it to zero
*       Comments		
*
******************************************************************************/
VECTOR allocVector(){
	VECTOR data;
#if DEBUG_PRINT == 1
	printf("function allocVector() Possibilities = %d\n",PedigreeData.Possibilities);
#endif
	sl_myCalloc(data,PedigreeData.Possibilities,sizeof(double));
	return data;

}


/******************************************************************************
*
*       Function        findAllele
*       Parameters		index - index of person(starting zero)
*						paternal - paternal or maternal allele
*       Output          from which founder allele was inhereted 
*       Description		finds founder allele for nonfounder allele
*       Comments		recursive, using global variable inhVector
*
******************************************************************************/
int findAllele(int index,   int paternal) {

	int paternalNextGen;  /* grandpaternal=0 ,grandMaternal = 1 */
	int parentIndex; // from whom allele was inhereted
	int allele;

	// paternal alleles always have even index
	allele = PedigreeGraph[index].num*2 + !paternal;

	/* if founder, stop recursion */
	if (PedigreeGraph[index].founder)
		return  allele;

	
	/* Already find value for this allele */
	if (AlleleInh[inhVector][allele] != 0){
		return AlleleInh[inhVector][allele];
	}


	
	/* allele%2  => 1-if paternal 0 - if maternal */
	if (paternal){
		if (PedigreeGraph[index].paternalAllele->bitType == RegularBit)
			paternalNextGen = (inhVector&(MaskLSB[PedigreeGraph[index].paternalAllele->number]))?0:1;

		else
			paternalNextGen = !PedigreeGraph[index].paternalAllele->value;
	}else{
		if (PedigreeGraph[index].maternalAllele->bitType == RegularBit)
			paternalNextGen = (inhVector&(MaskLSB[PedigreeGraph[index].maternalAllele->number]))?0:1;
		else
			paternalNextGen = !PedigreeGraph[index].maternalAllele->value;			
	}

	/* find if greatpaternal or greatMaternal   */
	/* allele%2  => 1-if paternal 0 - if maternal */
	
	




	/* find parent */
	parentIndex = (paternal)?
		(PedigreeGraph[index].father - 1):
	(PedigreeGraph[index].mother - 1);

	/* recusrsive call */
	AlleleInh[inhVector][allele] = findAllele(parentIndex,paternalNextGen);

	/* if parent is founder update list of founder children */
	if (PedigreeGraph[parentIndex].founder){
		if (!(ProgramSpecificData.sexLinked && paternal)){
				paternalNextGen = AlleleInh[inhVector][allele]/2;
				if (paternal) {
					index = 1<< PedigreeGraph[index].paternalAllele->noSpeedUpNumber;
				}
				else {
					index = 1<< PedigreeGraph[index].maternalAllele->noSpeedUpNumber;
				}
				FoundersChildren[paternalNextGen]|= index;
				//printf("FoundersChildren[%d] |= %d\n", paternalNextGen, index);
			}
	}
	return AlleleInh[inhVector][allele];
}


/******************************************************************************
*
*       Function        initAlleles
*       Parameters		none
*       Output          none
*       Description		for vectors , for each nonfounder finds from which founder allele was inhereted
*       Comments		none
*
******************************************************************************/
void  initAlleles(){  
	int j, paternal;
	sl_myMalloc(AlleleInh,(PedigreeData.Possibilities*sizeof(int*)));
	sl_myCalloc(FoundersChildren,(PedigreeData.NumOfFounders - PedigreeData.ChildlessFounders),(sizeof(int)));
	for (inhVector=0; inhVector < PedigreeData.Possibilities; inhVector++){
		
		sl_myCalloc(AlleleInh[inhVector], 2*PedigreeData.NumOfNonFounders,(sizeof( int)));
		
		for (j = 0; j < 2*PedigreeData.NumOfNonFounders; j++ ){
			paternal = !(j%2);
			if (ProgramSpecificData.sexLinked &&
				PedigreeGraph[PedigreeData.NonFounders[j/2]].sex == Male
				&& paternal)
				continue;
			if (AlleleInh[inhVector][j]==0)
				AlleleInh[inhVector][j]=findAllele(PedigreeData.NonFounders[j/2],paternal);
		}
	}

}


/******************************************************************************
*
*       Function        initVertex
*       Parameters		none
*       Output          none
*       Description		allocate memory for founders graph
*       Comments		none
*
******************************************************************************/
void initVertex(){
	int i;
	sl_myMalloc(Alleles,(2*PedigreeData.NumOfFounders*sizeof(Assign)));
	sl_myMalloc(FoundersGraph,(2*PedigreeData.NumOfFounders*sizeof(Neighbours*)));
	sl_myMalloc(NumFounderNeighbours,(2*PedigreeData.NumOfFounders*sizeof(int)));
	for (i = 0; i < 2*PedigreeData.NumOfFounders; i++){
		sl_myMalloc((FoundersGraph[i]),(2*PedigreeData.NumOfFounders* sizeof(Neighbours)));
	}
}

/******************************************************************************
*
*       Function        freeIfFailedComplex
*       Parameters		none
*       Output          none
*       Description		free all used memory
*       Comments		none
*
******************************************************************************/
void freeIfFailedComplex(){
	int i;
	free(RegularToSpeedArray);
	freeAtTheEndOfProgram();
	free(PedigreeData.Founders);
	free(PedigreeData.NonFounders);
	for(i=0;i<PedigreeData.PedigreeSize;i++){
		free(PedigreeGraph[i].myLocus);
		free(PedigreeGraph[i].maternalAllele);
		free(PedigreeGraph[i].paternalAllele);
	}
	free(PedigreeGraph);

	for (i = 0; i < PedigreeData.Possibilities; i++){
		free(AlleleInh[i]);
	}
	free(AlleleInh);
	free(ProbabilityDesiase);
}

/******************************************************************************
*
*       Function        freeAtTheEndOfProgram
*       Parameters		none
*       Output          none
*       Description		free all used memory
*       Comments		none
*
******************************************************************************/
void freeAtTheEndOfProgram(){
	AllPedigrees *curNode, *nextNode;
	int i,k;

	curNode = AllPedigreesList;
	while (curNode != NULL){
		nextNode = curNode->next;
		free(curNode);
		curNode = nextNode;
	}
	for (i = 0; i< NumOfLocuses; i++){
		free(lociDataArray[i].frequencies);
		if (i==DesiaseLocus){
			for(k = 0; k < 4; k++)
				free(lociDataArray[i].penetrances[k]);
		}
	}


	free(Recomb_values);
	free(lociDataArray);
	free(SpeedToRegularArray);


}



/******************************************************************************
*
*       Function        freeFoundersGraphAndAlleleInh
*       Parameters		none
*       Output          none
*       Description		free all unnecessary memory after calculating propabilities of vectors
*       Comments		none
*
******************************************************************************/
void freeFoundersGraphAndAlleleInh(){
	int i;
	
	free(NumFounderNeighbours);
	free(Alleles);
	
	for (i=0; i<2*PedigreeData.NumOfFounders; i++){
		free(FoundersGraph[i]);
	}
	free(FoundersGraph);
	#if (ALLELES_INH_USE == 0) 
		for (i = 0; i < PedigreeData.Possibilities; i++){
		free(AlleleInh[i]);
		}
		free(AlleleInh);
	#else
		deleteAllelesInheritance(allelesInhObj);
	#endif
}

/******************************************************************************
*
*       Function        freeAfterTransformCreation
*       Parameters		none
*       Output          none
*       Description		frees all memory after creating transformation for fft
*       Comments		none
*
******************************************************************************/

void freeAfterTransformCreation(){
	int i;
	free(PedigreeData.Founders);
	free(PedigreeData.NonFounders);
	free(FirstFoundersChild);
	free(FoundersChildren);
	free(WeightArray);
	free(TransformSpeedTheta);
	for(i=0;i<PedigreeData.PedigreeSize;i++){
		free(PedigreeGraph[i].myLocus);
		free(PedigreeGraph[i].maternalAllele);
		free(PedigreeGraph[i].paternalAllele);
	}
	free(PedigreeGraph);
	free(MaskLSB);
}

/******************************************************************************
*
*       Function        freeBeforeMult
*       Parameters		none
*       Output          none
*       Description		free all unnecessary memory after calculating likelihood
*       Comments		none
*
******************************************************************************/
void freeAll(double** LeftToRightEndProb, double** RightToLeftEndProb){
	int i;
	
	for ( i = 0; i < NumInfMarkers; i++){
		free(LeftToRightEndProb[i]);
		free(RightToLeftEndProb[i]);	
	}
	free(LeftToRightEndProb);
	free(RightToLeftEndProb);
}

/******************************************************************************
*
*       Function        initMask
*       Parameters		none
*       Output          none
*       Description		inits array, which return corresponding bit at integer
*						Mask [i] - bit i from msb is non zero
*       Comments		data for meioses 0 is lsb 
						Function noot in use
*
******************************************************************************/
/*void initMask(){ 
	int i,j;
	j=1<<(PedigreeData.NumOfMeioses-1);  // 1 and (numOfmeioses -1) zeroes 
	sl_myMalloc(Mask,(sizeof(int)*PedigreeData.NumOfMeioses));
	for(i = 0; i < PedigreeData.NumOfMeioses; i++){
		Mask[i] = j;
		j = j >> 1;
	}
}
*/
/******************************************************************************
*
*       Function        initMaskLSB
*       Parameters		none
*       Output          none
*       Description		inits array, which return corresponding bit at integer
*						Mask [i] - bit i from lsb is non zero
*       Comments		data for meioses 0 is msb
*
******************************************************************************/
/*void initMaskLSB2(){ 
	int i,j;
	j=1<<(PedigreeData.NumOfMeioses);  /* 1 and (numOfmeioses -1) zeroes *
	sl_myMalloc(MaskLSB,(sizeof(int)*(PedigreeData.NumOfMeioses+1)));
	for(i = PedigreeData.NumOfMeioses; i >= 0; i--){
		MaskLSB[i] = j;
		j = j >> 1;
	}
}*/

void initMaskLSB(){ 
	int i,j;
	j=1<<(PedigreeData.SpeedUpNumOfMeioses);  /* 1 and (speedUpNumOfmeioses -1) zeroes */
	sl_myMalloc(MaskLSB,(sizeof(int)*(PedigreeData.SpeedUpNumOfMeioses+1)));
	for(i = PedigreeData.SpeedUpNumOfMeioses; i >= 0; i--){
		MaskLSB[i] = j;
		j = j >> 1;
	}
}
