/* $Id: component.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  component.c											   				    */
/*  Finds  probability of the connected component(created from founders     */ 
/*  alleles in  founders sl_alleles 										*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#include "fdata.h"
#include "gdata.h"
#include "stdlib.h"
#include "stdio.h"

double ComputeInsideComp(int id,int allele,int curLocus);


/******************************************************************************
*
*       Function        resetUsedCoponents
*       Parameters		none
*       Output          none
*       Description		sets all vertexes as unused
*       Comments		none
*
******************************************************************************/
void resetUsedCoponents(){
	int i;
	for (i=0;i < 2*PedigreeData.NumOfFounders  ;i++)
		Alleles[i].inUse = false;
}

/******************************************************************************
*
*       Function        ComputeComp
*       Parameters		i - vertex id
*       Output          none
*       Description		Computes probability of whole component 
*       Comments		none
*
******************************************************************************/
double ComputeComp(int i,int curLocus){
	int allele,currNeigh;
	double prob1,prob2;
	Neighbours *it;
	/* Mark sl_alleles as visited with both values */
	Alleles[i].if_visitedA = true;
	Alleles[i].if_visitedB = true;
	Alleles[i].inUse = true;
	/* If current allele doesn't connencted to any other , the probability of
	whole sl_component is 1 (can except any value )                         */
	
	if (NumFounderNeighbours[i]==0){
		if (Alleles[i].A == 0)
			return 1;
		if (Alleles[i].A > 0 && Alleles[i].B == -1)
			return lociDataArray[curLocus].frequencies[Alleles[i].A-1];
		else
			return lociDataArray[curLocus].frequencies[Alleles[i].A-1]+
			lociDataArray[curLocus].frequencies[Alleles[i].B-1];

	}
	

	/* If there is possible assignment to this allele */
	if (Alleles[i].A>0)
		prob1=lociDataArray[curLocus].frequencies[Alleles[i].A-1];
	else /* no possible assignment */ 
		prob1=0;

	/* If there are two possible assignments to this allele */
	if (Alleles[i].B>0) 
		prob2=lociDataArray[curLocus].frequencies[Alleles[i].B-1];
	else 
		prob2=0;


	//for (it=FoundersGraph[i];it!=NULL;it=it->next){
	for (currNeigh=0;(currNeigh<NumFounderNeighbours[i]) ;currNeigh++){
		it = &(FoundersGraph[i][currNeigh]);
		// no possible values for this sl_component
		if (Alleles[i].A==-1)
			prob1*=ComputeInsideComp(it->id,-1, curLocus); 

		else if (Alleles[i].A>0){ // one possible value for A
			Alleles[i].currentValue = Alleles[i].A;
			if (Alleles[i].A == it->allele1)
				allele=it->allele2;
			else
				allele=it->allele1;
			// assignment  for neigbout it->id is allele
			// continue to it->id neigbour

			prob1*=ComputeInsideComp(it->id,allele, curLocus); 
			resetUsedCoponents();
		}

		if (Alleles[i].B==-1)
			prob2*=0/*ComputeInsideComp(it->id,-1)*/; 

		else if (Alleles[i].B>0){
			Alleles[i].currentValue = Alleles[i].B;
			if (Alleles[i].B == it->allele1)
				allele=it->allele2;
			else
				allele=it->allele1;
			prob2*=ComputeInsideComp(it->id,allele, curLocus);
			resetUsedCoponents();
		}
	}
	return prob1+prob2;
}



/******************************************************************************
*
*       Function        ComputeInsideComp
*       Parameters		i - vertex id
*						allele - curent assignment to the vertex
*       Output          none
*       Description		computes possible values for this sl_component
*       Comments		if allele == -1 no possible values 
*
******************************************************************************/
double ComputeInsideComp(int i,int allele, int curLocus){
	double prob;
	Neighbours *it;
	int all,currNeigh;
	// Check if we already visit this sl_vertex
	if (Alleles[i].inUse == true ){
		if (allele == Alleles[i].currentValue)
			return 1;     //with same value
		else
			return 0.0;	 //with another value
	}

	// no possible asignmnets for this neighbour
	if (Alleles[i].A==-1 && Alleles[i].B==-1){
		//already have been here
		if (Alleles[i].if_visitedA==true)
			return 0;
		//mark visited
		Alleles[i].if_visitedA = true;
		Alleles[i].if_visitedB = true;		 
		// set unpossible for all neigbours
		//  for (it=FoundersGraph[i];it!=NULL;it=it->next)
		for(currNeigh=0;currNeigh<NumFounderNeighbours[i];currNeigh++){
			it = &(FoundersGraph[i][currNeigh]);
			ComputeInsideComp(it->id,-1, curLocus);
		}
		return 0;
	}


	// no possible asignment for the sl_component
	if (allele==-1 ){
		if (Alleles[i].if_visited==true)
			return 0;
		Alleles[i].if_visited=true;
		// set unpossible for all neigbours
		//		for (it=FoundersGraph[i];it!=NULL;it=it->next)
		for(currNeigh=0;currNeigh<NumFounderNeighbours[i];currNeigh++){
			it = &(FoundersGraph[i][currNeigh]);
			ComputeInsideComp(it->id,-1, curLocus); 
		}
		return 0;
	}


	Alleles[i].inUse = true;
	Alleles[i].currentValue=allele;

	//there is a possible assignment so far
	// only one according to algorithm
	prob=lociDataArray[curLocus].frequencies[allele-1];
	if (allele==Alleles[i].A){		
		//already been here

		if (Alleles[i].if_visitedA==true)
			return 1;


		Alleles[i].if_visitedA=true;
		//	for (it=FoundersGraph[i];it!=NULL;it=it->next){
		for(currNeigh=0;currNeigh<NumFounderNeighbours[i];currNeigh++){
			it = &(FoundersGraph[i][currNeigh]);	

			if (Alleles[i].A == it->allele1)
				all=it->allele2;
			else
				all=it->allele1;
			prob*=ComputeInsideComp(it->id,all, curLocus); 
		}
		//    Alleles[i].inUse = false;
		return prob;

	}

	if (allele==Alleles[i].B)
	{

		if (Alleles[i].if_visitedB==true)
			return 1;

		Alleles[i].if_visitedB=true;
		//		for (it=FoundersGraph[i];it!=NULL;it=it->next){
		for(currNeigh=0;currNeigh<NumFounderNeighbours[i];currNeigh++){
			it = &(FoundersGraph[i][currNeigh]);		
			if (Alleles[i].B == it->allele1)
				all=it->allele2;
			else
				all=it->allele1;
			prob*=ComputeInsideComp(it->id,all, curLocus); 
		}
		//	Alleles[i].inUse = false;
		return prob;

	}
	Alleles[i].inUse = false;
	return 0.0;
}
