/* $Id: gdata.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/*************************************************************************/
/* gdata.c												     			 */
/* Global definitions													 */
/* Author: Anna Tzemach 												 */	
/*************************************************************************/                              
#ifndef GDATA_H
#define GDATA_H
#define _CRT_SECURE_NO_DEPRECATE
#include <stdio.h>
#include "gh_settings.h"

typedef enum Bool{false = 0, true = 1}Bool;
#define VECTOR  double*
#define MATRIX  double**
Bool failedBreak;

int NumPedigrees;
int symetryMask;
double logMultFactor;
double logResultFactor;

int DesiaseLocus;     /* sl_start from zero*/

int *MarkerOrder; /* Order of the informaive loci */
int NumOfLocuses;
int NumInformativeLoci;
int NumInfMarkers;

/****************************************************************/
/* added by Anisenia Andrei										*/
typedef enum {S_PAIRS, S_ALL} SCORE; 
#define ALLELE_DOESNT_EXIST -1

/*	added by Anisenia Andrei for the computation of NPL score	*/
/*  linked list of indexes of childless founders				*/
/*  created in "normalize.c"
/*	used in "NPLstudents.c"										*/
typedef struct UnlinkedFounder
{
	int founder;	/* index of childless founder in the Pedigree	*/
	struct UnlinkedFounder* next;
}UnlinkedFounder;

UnlinkedFounder* unlinkedFounders;



/****************************************************************/
/* added by Anisenia Andrei	for MemoryManager initiation										*/
#define NUM_LOCI_INFO_TYPES 4

#define LIKELIHOOD_PROBS_IND				0
#define LIKELIHOOD_FACTORS_IND			1
#define HAPLO_BACK_POINTERS_IND		2
#define HAPLO_PROBS_IND						3

/****************************************************************/



double * Recomb_values; /*Array of Teta */


int *FoundersChildren;  // Si in the article
int *FirstFoundersChild; // Oi in the article
int *TransformSpeedTheta;  //pi-wave in article


#define RegularToSpeedArrayMacro(i)  RegularToSpeedArray[i]  

/*(i < (1 << 17))? (RegularToSpeedArray[0][i]) :	(RegularToSpeedArray[1][i >> 16])*/



int* RegularToSpeedArray; //[2]; // array[i] - number without first generation bits
int* SpeedToRegularArray; //array[i] - insertion on first generation bits
int *WeightArray;         // array[i] - number of 1 in i(binary representation)
int *WeightArrayTransform; // array[i] - WeightArray[ TransformSpeedTheta[SpeedToRegularArray[i]]]




//void InitThetas();          /* For each theta calculate all possible values */
char *Hmm_Dst;


typedef struct pedigreeData{
	int FoundersBitOneMask;
	int PedigreeSize;
	int NumOfFounders;
	int *Founders;           /* array with numbers of founders */
	int NumOfNonFounders;
	int *NonFounders;         /* array with numbers of founders */
	int ChildlessFounders;
	int NumOfMeioses;        /* 2*NumOfNonFounders or NumOfNonfounders(SexLinked) */
	int SpeedUpNumOfMeioses; 
	int NumFoundersAlleles; /* 2*NumOfFounders or NumOFFemales*2+NumOFMAles(SexLinked) */
	int Possibilities;       /*pow(2,NumOfMeioses-NumOfFounders) */
	//int MaxPoss;			/*pow(2,NumOfMeioses) */
}PedigreeDataStruct;
PedigreeDataStruct PedigreeData;

typedef struct programSpecificData{
	int progCode;
	Bool sexLinked;
	int type; /* 0 -n 1 -s */
	int start;
	int end;
	double offMap;
	
	union {
		double step;
		int numIntervals;
	};
}ProgramSpecificDataStruct;
ProgramSpecificDataStruct ProgramSpecificData;


/*Data about all sl_loci */

typedef struct LOCUS_DATA{
	int locus_type;        /* 1-sl_affected 3- numerated */
	double *frequencies;   /* gene frequencies */
	Bool *used;
	int num_of_freq;
	int num_of_liab;       /* for sl_affected allele */
	//double *penetrances[3];/* for sl_affected allele */
	//	for imprinting version
	double *penetrances[4];
	double *malePenatrences[2];
	char locusName[1000];
	double alleleFreqDifFromOne;
}LOCUS_DATA;

LOCUS_DATA * lociDataArray;

/* sl_alleles in sl_locus - not ordered */
typedef struct Locus_t
{
	int allele1;
    int allele2;
}Locus;

typedef enum  bitsType { ReductionBit = 1, ConstBit, RegularBit, NotUsedBit} BitType;

typedef struct AlleleData{
	int number; /*	starting 0	*/
	/* serial number out off all sl_alleles of the same type - founders, firstGen, regular*/
	int noSpeedUpNumber;
	/* Const but can be of to reasons:
		1. Sex Linked - all paternal bits are 1
		2. Speed up bits - always zero. 
		Important: Speedup + sexlinked: only maternal bits are include in speedUp
	*/
	BitType bitType; 
	int value; // 0 or 1 (used only for const bits)
}AlleleData;


/*Data about sl_pedigree */

typedef enum Gender{ Male = 0, Female } Gender;
typedef struct Pedigree_t{
 Bool informative;
 Bool markersInformative;
 Bool diseaseInformative;
 int multiplicity; 
 int temp; // for different operation
 Bool fixed; // for different operation
 int index;
 int pedNum;
 int father;
 int mother;
 Bool founder;  /* 1- founder 0-nonFounder */

 AlleleData  *paternalAllele;
 AlleleData  *maternalAllele;
 int num;      /* out of founders/nonfounders,starting 0  */
 Gender sex;      /* 0-male,1-female */
 int child;
 int NeSiPa;
 int NeSiMa;
 Locus* myLocus;
  char personName[10];

}Pedigree;





Pedigree* PedigreeGraph; /*All relevant input data */

typedef struct AllPedigrees{
	Pedigree* PedigeeGraph;
	int size;
	int numOfFounders;

	int numOfNonFounders;
	struct AllPedigrees *next;
}AllPedigrees;

AllPedigrees* AllPedigreesList;



//	added by Anisenia Andrei for the forward phase iteration of window loop calculation

//	Structure with data for haplotyping result output printing 
typedef struct HaplotypingPrintData {
	FILE* haplotypesFile;						//	pointer to the haplotypes output file
	Bool indicationFirstPrint;					//	indication that this is the first window print
	Bool indicationLastPrint;					//	indication that this is the last window print
	double logLikelihoodHaplo;				//	the log-likelihood of the haplotypes found
	int* recombCountersArray;				//	counters for specific numbers of recombinations
	int* recombHaplotypesArray;			//	recombinatons between specific haplotypes
	int numRecombinations;					//	counter for number of recombinations
	double sumRecombValues;				//	sum of all recombination values
}
HaplotypingPrintData;


//	Structure with the intermediate information for the forward phase of loop calculation
typedef struct ForwardPhaseIterationInput {
	int startLocus;				//	start locus of the iteration
	int endLocus;				//	end locus of the iteration
	double* leftPriorProbs;		//	prior probabilities vector at the start locus
	double* rightPriorProbs;	//	prior probabilities vector at the end locus
	double* rightToLeftEndProbs;	//	result vector of right-to-left calculation at the start locus 
	double const* offMapLeftPtr;			//	off map before start locus
	double const* offMapRightPtr;			//	off map right after end locus
	double* distFromFirstMarker;	//	distance between the first locus of current window to the first input locus 
	double sumDiseaseProbs;			//	sum of disease probabilities in disease locus vector
	double stdUnlinked;				//	standart unlinked likelihood
	double FactorLeft;				//	normalizing factor during left-to-right calculation
	double FactorRight;				//	normalizing factor during right-to-left calculation
	int* haploLeftLocusBackPointers;	//	haplotyping back pointers at the first window locus
	int* haploMaxVectorLeftLocus;		//	pointer to haplotyping max probability Viterbi vector at the first window locus
	double* haploRightPriorProbs;		//	prior haplotyping probabilities vector at the end locus
	HaplotypingPrintData* haplotypingPrintData;		//	haplotyping print specific information
} 
ForwardPhaseIterationInput;

//	DEBUG prints of i/o operations time 
#define PRINT_FILES_IO_TIME 0	//	0 - no print; 1 - print
#define IO_TIME_PRINTS_FILENAME	"ioTimes.txt"
FILE* timesFile;

//	DEBUG prints of overall FFT transition times
#define PRINT_FFT_TIME 0//	0 - no print; 1 - print
#define FFT_TIMES_FILE "fftTimes.txt"
FILE* fftFile;

//	DEBUG prints of inference engine calc on disease locus
#define PRINT_HAPLO_TIME 0	//	0 - no print; 1 - print
#define HAPLO_TIMES_FILE "haploTimes.txt"
FILE* haploFile;

//	DEBUG prints of single locus calculations
#define PRINT_SINGLE_LOCUS_TIME 0	//	0 - no print; 1 - print
#define SINGLE_LOCUS_TIMES_FILE "singleLocusTimes.txt"
FILE* singleLocusFile;

//	DEBUG prints of vectors multiplications (with both result in vector or in scalar)
#define PRINT_VECTORS_MULT_TIME 0	//	0 - no print; 1 - print
#define VECTORS_MULT_TIMES_FILE "vectorsMultTimes.txt"
FILE* vectorsMultFile;

//	DEBUG prints of NPL time 
#define PRINT_NPL_CALC_TIME 0
#define NPL_CALC_TIMES_FILE "nplCalcTime.txt"
FILE* nplCalcFile;

//	Structure for the backward phase iteration of window loop calculation
typedef struct BackwardPhaseIterationInput{
	int startLocus;				//	start locus of the iteration
	int endLocus;				//	end locus of the iteration
	double* rightPriorProbs;	//	prior probabilities vector at the end locus
	double FactorFromRight;		//	normalizing factor during right-to-left calculation
	int* haplotypingBackPointers;	//	haplotyping back pointers at the last window locus
	double* haplotypingProbs;			//	resulting probabilities  at the last window locus
}
BackwardPhaseIterationInput;


/*
PiWave transformation -see: Kruglyak and Lander 1998 p4
*/
int *linearSpeedTransform(int size);

int *createSpeedWeightTransform(int size);


/* number of 1 in number*/
void createWeightArray();

/*	checking that number is not in it's scope	*/
Bool isOutOfScope(int number, int lower, int upper);


#endif
