/* $Id: gh_main.c,v 1.8 2008/09/28 06:58:47 sandanis Exp $ */
/****************************************************************************/
/*  main.c											   						*/
/*  reads data from files												 	*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#define _CRT_SECURE_NO_DEPRECATE


#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gdata.h"
#include "fdata.h"
#include <time.h>
#include "allocfile.h"
#include "sl_Auxilary.h"
#include "programSpec.h"
#include  <math.h>
#include <limits.h>
#include "likehood.h"
#include "desiase.h"

#include "initiatePedigree.h"
#include "NPLstudents.h"

#include "haplotypingSimple.h"
#include "haplotypingAlleles.h"
#include "haplotypingLevelsTree.h"
#include "haplotypingReversedTree.h"
#include "haplotypingTreesAlgorithms.h"

//#include "diseaseLocus/diseaseLocusCalculation.h"
#include "restrictionsGraph.h"

#include "../Product.h"
#include "../Mem.h"
#include "MemoryManager/MemoryManager.h"

#include "ReadInputFiles.h"


#define TOTAL_MEM_PERCENT_TO_USE 0.8

#define NUM_SELECTORS 1

void normalizeInput();
void prepareInput();
void finishPrepaireInput();

int sl_main();
void freeIfFailedComplex();

product* initProduct(AllPedigrees* PedPtr, LOCUS_DATA* diseaseLocusData, Bool sexLinked);
void initElimOrderArray(inferenceInput* inf);
void initElimOrderArrayNumSelectors(inferenceInput* inf, int numSelectors);
void destroyProduct(product* prod);
int checkRegularBitIndex(int personNumber, NodeSex parentalMode);
void normalizeDiseaseProbs(double* diseaseProbs, double probForCheck);

void checkHaplotyping(int* haplotypeVectors, int numInfMarkers, AllPedigrees* PedigreePtr,
					  int numBits);
Bool isRequiredMemoryTooBig(int givenMem);
//void compressToFile();

TestPoint* performForwardIteration(ForwardPhaseIterationInput* iterationInput, TestPoint* maxNode, FILE *resFile);
void performBackwardIteration(BackwardPhaseIterationInput* backwardPhaseIterationInput);
ForwardPhaseIterationInput* 
	createForwardPhaseIterationInput(BackwardPhaseIterationInput* firstForwardIterationInput);
BackwardPhaseIterationInput* createBackwardPhaseIterationInput();
void deleteForwardPhaseIterationInput(ForwardPhaseIterationInput* forwardIterationInput);
HaplotypingPrintData* haplotypingPrintDataInitiation();
void deleteHaplotypingPrintData(HaplotypingPrintData* haplotypingPrintData);
double getSumRecombValues();
void* getVectorCopy(void* vector, int size);
double getLeftResultFactor(MemoryManager* memManager, int locus);

double getMemForWindowCalc(int givenMem);
double getSizeOfRunningLocus();
void initiateLociTypes(MemoryManagerCreateInput* managerInput);

int writeArrayToBinaryFile(void* array, int elementSize, int numElements, char* filePath);
void* readArrayFromBinaryFile(int elementSize, int numElements, char* filePath);

void* readIntArrayFromFile(char* filePath, int arraySize);
void* readDoubleArrayFromFile(char* filePath, int arraySize);

int writeIntArrayToFile(void* array, char* filePath, int arraySize);
int writeDoubleArrayToFile(void* array, char* filePath, int arraySize);

BackwardPhaseIterationInput* performBackwardPhase(MemoryManager* memManager);
TestPoint* performForwardPhase(BackwardPhaseIterationInput* firstForwardIterationInput, 
							   MemoryManager* memManager, TestPoint* maxNode, FILE *resFile);
void* operateMemoryManagerAndPrintLog(MemoryManager* manager, void* dataToStore,
																			int dataType, int locus, int operationType);


/******************************************************************************
*
*       Function        gh_main
*       Parameters		diseaseProbs
*       Output          none
*       Description		main function
*       Comments		none
*
******************************************************************************/
int gh_main(double* diseaseProbs, MemoryManager* memManager, double probForCheck) {
	FILE *resFile;
	TestPoint* maxNode = NULL;
	int endLocus;
	BackwardPhaseIterationInput* firstForwardIterationInput;
	int windowSize = getWindowSize(memManager);
	
	//	1st section: prepairing data before the window loop calculations
	finishPrepaireInput();
	normalizeDiseaseProbs(diseaseProbs, probForCheck);

	//	DEBUG section start: TODO remove this section
	/*{
		int i;
		FILE* out;
		int minimum = (PedigreeData.Possibilities > 100) ? (100) : (PedigreeData.Possibilities);
		out = fopen("idoProblem.txt", "w");
		printf("\n");
		for(i = 0; i < minimum; ++i) {  
			double diff;
			double prob = desiaseProb(0, i);
			diff = prob - diseaseProbs[i];
			if //(prob == diseaseProbs[i]) {
				((diff < 0.000001) && (diff > -0.000001)) {
				fprintf(out, "RIGHT!\n");
			}
			else {
				fprintf(out, "WRONG: ido=%.17lf, disease=%.17lf, diff=%.17lf \n", diseaseProbs[i], prob, diff);
			}
		}
		fclose(out);
		exit(1);
	}*/
	//	DEBUG section end

	#if DISEASE == 3
		ProbabilityDesiase = diseaseProbs;
	#endif
	printf("\nNUMBER OF REDUCED VECTORS = %d\n",PedigreeData.Possibilities);

	#if (PRINT_NPL_CALC_TIME == 1) 
		nplCalcFile = fopen(NPL_CALC_TIMES_FILE, "w");
		fprintf(nplCalcFile,  "Start writing = %lf\t\t", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif

	computeNPL();
	
	#if (PRINT_NPL_CALC_TIME == 1) 
		fprintf(nplCalcFile,  "End writing = %lf\n", ((double)clock())/(double)CLOCKS_PER_SEC);
		fclose(nplCalcFile);
	#endif

	
	sl_myCalloc(ProbabilityLocus,NumOfLocuses,(sizeof(double*)));
	sl_myMalloc(MarkerOrder, sizeof(int)*windowSize);	
	initVertex();
	createWeightArray();
	linearSpeedTransform(PedigreeData.Possibilities);
	createSpeedWeightTransform(PedigreeData.Possibilities);
	resFile = printTitle();
	sl_myMalloc(haplotypeVectors, ((windowSize)*sizeof(int)) );
	
	//	2nd section: window loop calculation, backward phase
	firstForwardIterationInput = performBackwardPhase(memManager);
		
	//	3rd section: window loop calculation, forward phase
	maxNode = performForwardPhase(firstForwardIterationInput, memManager, maxNode, resFile);
	
	//	4th section: freeing allocated memory and printing the results  
	free(ProbabilityLocus);
	free(MarkerOrder);
	free(ProbabilityDesiase);
	free(SpairsArray);
	free(SallArray);
	freeAfterTransformCreation();
	free(haplotypeVectors);
	freeFoundersGraphAndAlleleInh();
	deleteUnlinkedFounders();
	freeAtTheEndOfProgram();
	printSummary(resFile, maxNode);
	free(maxNode);
	printf("\n Total run time  %.3lf sec\n\n",((double)clock())/(double)CLOCKS_PER_SEC);
	printf("\n Superlink v1.7 (Original Citation: Fishelson M. and Geiger D. Bioinformatics,  2002:18, S189-S198)\n");
	printf("(c)2002-2008 Technion - Israel Institute of Technology\n\n");

	return 0;
}


BackwardPhaseIterationInput* performBackwardPhase(MemoryManager* memManager) {
	int locus, endLocus;
	
	int windowSize = getWindowSize(memManager);
	BackwardPhaseIterationInput* backwardPhaseIterationInput = createBackwardPhaseIterationInput();
	for(locus = NumOfLocuses - 2; locus + 1 > windowSize; locus = locus - windowSize + 1) {
		//	Prepairing input for the backward phase step
		backwardPhaseIterationInput->startLocus = locus - windowSize + 1;
		backwardPhaseIterationInput->endLocus = locus;
		printf("Treating loci backward [%d %d]\n", 
			backwardPhaseIterationInput->startLocus, backwardPhaseIterationInput->endLocus);
		
		//	Performing backward phase step and saving intermediate results in the backwardPhaseIterationInput
		performBackwardIteration(backwardPhaseIterationInput);
		
		if (backwardPhaseIterationInput->startLocus > windowSize - 1) {
			//	Storing the results of the backward phase step in the memory manager
			operateMemoryManagerAndPrintLog(memManager, 
				getVectorCopy(backwardPhaseIterationInput->rightPriorProbs, PedigreeData.Possibilities*sizeof(double)), 
				LIKELIHOOD_PROBS_IND, backwardPhaseIterationInput->startLocus, 0);

			operateMemoryManagerAndPrintLog(memManager, 
				getVectorCopy(&(backwardPhaseIterationInput->FactorFromRight), sizeof(double)), 
				LIKELIHOOD_FACTORS_IND, backwardPhaseIterationInput->startLocus, 0);
			
			operateMemoryManagerAndPrintLog(memManager, 
				getVectorCopy(backwardPhaseIterationInput->haplotypingProbs, PedigreeData.Possibilities*sizeof(double)), 
				HAPLO_PROBS_IND, backwardPhaseIterationInput->startLocus, 0);
		}
		operateMemoryManagerAndPrintLog(memManager, 
			getVectorCopy(backwardPhaseIterationInput->haplotypingBackPointers, PedigreeData.Possibilities*sizeof(int)), 
			HAPLO_BACK_POINTERS_IND, backwardPhaseIterationInput->startLocus, 0);
		free(backwardPhaseIterationInput->haplotypingBackPointers);
		backwardPhaseIterationInput->haplotypingBackPointers = NULL;
	}
	backwardPhaseIterationInput->endLocus = locus;
	return backwardPhaseIterationInput;
}



TestPoint* performForwardPhase(BackwardPhaseIterationInput* firstForwardIterationInput, 
							   MemoryManager* memManager, TestPoint* maxNode, FILE *resFile) {
	
	int startLocus = 0;
	int endLocus = firstForwardIterationInput->endLocus;
	int windowSize = getWindowSize(memManager);

	//	For the first forward iteration take the saved data from the firstForwardIterationInput structure
	ForwardPhaseIterationInput* forwardIterationInput = 
		createForwardPhaseIterationInput(firstForwardIterationInput);
	free(firstForwardIterationInput);

	while (endLocus <= NumOfLocuses - 2) {
		//	Prepairing input for the forward step
		forwardIterationInput->startLocus = startLocus;
		forwardIterationInput->endLocus = endLocus;
		printf("Treating loci forward [%d %d]\n", 
			forwardIterationInput->startLocus, forwardIterationInput->endLocus);
		if (startLocus == 0) {
			forwardIterationInput->offMapLeftPtr = &(ProgramSpecificData.offMap);
		}
		else {
			forwardIterationInput->offMapLeftPtr = NULL;
			forwardIterationInput->haplotypingPrintData->indicationFirstPrint = false;

			//	It's not the first forward iteration => take the saved data from the memory manager
			forwardIterationInput->rightPriorProbs = 
				operateMemoryManagerAndPrintLog(memManager, NULL, 
				LIKELIHOOD_PROBS_IND, endLocus, 1);

			forwardIterationInput->FactorRight = getLeftResultFactor(memManager, endLocus);

			forwardIterationInput->haploLeftLocusBackPointers = 
				operateMemoryManagerAndPrintLog(memManager, NULL, 
				HAPLO_BACK_POINTERS_IND, startLocus, 1);

			forwardIterationInput->haploRightPriorProbs = 
				operateMemoryManagerAndPrintLog(memManager, NULL, 
				HAPLO_PROBS_IND, endLocus, 1);

			printf("\n");
		}
		if (endLocus == NumOfLocuses - 2) {
			forwardIterationInput->offMapRightPtr = &(ProgramSpecificData.offMap);
			forwardIterationInput->haplotypingPrintData->indicationLastPrint = true;
		}
		else {
			forwardIterationInput->offMapRightPtr = NULL;
		}
		//	Performing forward phase step
		maxNode = performForwardIteration(forwardIterationInput, maxNode, resFile);
		startLocus = endLocus;
		endLocus = endLocus + windowSize - 1;
	}
	deleteForwardPhaseIterationInput(forwardIterationInput);
	return maxNode;
}


int* getDummyIntArray(int size) {
	int* array;
	int i;
	sl_myMalloc(array, size*sizeof(int));
	for (i = 0; i < size; i++) {
		array[i] = i;
	}
	return array;
}

double* getDummyDoubleArray(int size) {
	double* array;
	int i;
	sl_myMalloc(array, size*sizeof(double));
	for(i = 1, array[0] = 1/size; i< size; i++) {
		array[i] = array[i-1] + 1/size;
	}
	return array;
}

void testMemoryManager(MemoryManager* memManager) {
	int locus, endLocus, startLocus;
	
	int windowSize = getWindowSize(memManager);
	printf("DEBUG MODE\n");
	
	//	Backward phase
	for(locus = NumOfLocuses - 2; locus + 1 > windowSize; locus = locus - windowSize + 1) {
		startLocus = locus - windowSize + 1;
		endLocus = locus;
		if (startLocus > windowSize - 1) {
			//	Storing the results of the backward phase step in the memory manager
			operateMemoryManagerAndPrintLog(memManager, getDummyDoubleArray(PedigreeData.Possibilities), 
				LIKELIHOOD_PROBS_IND, startLocus, 0);

			operateMemoryManagerAndPrintLog(memManager, getDummyDoubleArray(1), 
				LIKELIHOOD_FACTORS_IND, startLocus, 0);
			
			operateMemoryManagerAndPrintLog(memManager, getDummyDoubleArray(PedigreeData.Possibilities), 
				HAPLO_PROBS_IND, startLocus, 0);
		}
		operateMemoryManagerAndPrintLog(memManager, getDummyIntArray(PedigreeData.Possibilities), 
				HAPLO_BACK_POINTERS_IND, startLocus, 0);
	}
	printf("\n");
	
	//	forward phase
	endLocus = locus;
	startLocus = 0;
	while (endLocus <= NumOfLocuses - 2) {
			void* array;	
			if (startLocus != 0) {
				array = 
					operateMemoryManagerAndPrintLog(memManager, NULL, 
					LIKELIHOOD_PROBS_IND, endLocus, 1);
				free(array);

				getLeftResultFactor(memManager, endLocus);

				array = 
					operateMemoryManagerAndPrintLog(memManager, NULL, 
					HAPLO_BACK_POINTERS_IND, startLocus, 1);
				free(array);

				array = 
					operateMemoryManagerAndPrintLog(memManager, NULL, 
					HAPLO_PROBS_IND, endLocus, 1);
				free(array);
			}
			startLocus = endLocus;
			endLocus = endLocus + windowSize - 1;

			printf("\n");
	}
	
	printf("\n Total run time  %.3lf sec\n\n",((double)clock())/(double)CLOCKS_PER_SEC);
	fclose(timesFile);
}





/******************************************************************************
*
*       Function        createForwardPhaseIterationInput
*       Parameters		firstForwardIterationInput - initial data struct from the backward phase
*       Output          ForwardPhaseIterationInput
*       Description		creates ForwardPhaseIterationInput
*       Comments		none
*
******************************************************************************/
ForwardPhaseIterationInput* 
createForwardPhaseIterationInput(BackwardPhaseIterationInput* firstForwardIterationInput) {
	ForwardPhaseIterationInput* forwardIterationInput;
	int i;

	sl_myMalloc(forwardIterationInput, sizeof(ForwardPhaseIterationInput));
	forwardIterationInput->startLocus = 0;
	forwardIterationInput->endLocus = 0;
	sl_myMalloc(forwardIterationInput->distFromFirstMarker, sizeof(double));
	*(forwardIterationInput->distFromFirstMarker) = 0;
	forwardIterationInput->leftPriorProbs = NULL;
	forwardIterationInput->rightPriorProbs = firstForwardIterationInput->rightPriorProbs;
	forwardIterationInput->offMapLeftPtr = NULL;
	forwardIterationInput->offMapRightPtr = NULL;
	forwardIterationInput->sumDiseaseProbs = 0;
	for(i = 0; i < PedigreeData.Possibilities; i++) {
		forwardIterationInput->sumDiseaseProbs += ProbabilityDesiase[i];
	}
	forwardIterationInput->FactorLeft = 0;
	forwardIterationInput->FactorRight = firstForwardIterationInput->FactorFromRight;
	forwardIterationInput->stdUnlinked = 0;
	forwardIterationInput->rightToLeftEndProbs = NULL;
	forwardIterationInput->haploLeftLocusBackPointers = firstForwardIterationInput->haplotypingBackPointers;
	sl_myCalloc(forwardIterationInput->haploMaxVectorLeftLocus, 1, sizeof(int));
	*(forwardIterationInput->haploMaxVectorLeftLocus) = 0;
	forwardIterationInput->haploRightPriorProbs = firstForwardIterationInput->haplotypingProbs;
	forwardIterationInput->haplotypingPrintData = haplotypingPrintDataInitiation();

	firstForwardIterationInput->haplotypingBackPointers = NULL;
	firstForwardIterationInput->haplotypingProbs = NULL;
	firstForwardIterationInput->rightPriorProbs = NULL;

	return forwardIterationInput;
}



/******************************************************************************
*
*       Function        haplotypingPrintDataInitiation
*       Parameters		none
*       Output          HaplotypingPrintData
*       Description		initiates HaplotypingPrintData
*       Comments		none
*
******************************************************************************/
HaplotypingPrintData* haplotypingPrintDataInitiation() {
	HaplotypingPrintData* haplotypingPrintData;

	sl_myMalloc(haplotypingPrintData, sizeof(HaplotypingPrintData));
	haplotypingPrintData->haplotypesFile = fopen(HAPLOTYPES_FILE_NAME, "w");
	if (haplotypingPrintData->haplotypesFile == NULL)
	{
		printf("File %s could not be opened.", HAPLOTYPES_FILE_NAME);
		exit(1);
	}
	haplotypingPrintData->indicationFirstPrint = true;
	haplotypingPrintData->indicationLastPrint = false;
	haplotypingPrintData->logLikelihoodHaplo = 0;
	haplotypingPrintData->recombCountersArray = NULL;
	haplotypingPrintData->recombHaplotypesArray = NULL;
	haplotypingPrintData->numRecombinations = 0;
	haplotypingPrintData->sumRecombValues = getSumRecombValues();

	return haplotypingPrintData;
}


//	HELP FUNCTIONS

double getSumRecombValues() {
	int i;
	double sum = 0;

	for(i = 0; i < NumOfLocuses - 1; i++) {
		sum += Recomb_values[i];
	}
	return sum;
}


void* getVectorCopy(void* vector, int size) {
	void* vectorCopy;
	
	if (vector != NULL) {
		sl_myMalloc(vectorCopy, size);
		memcpy(vectorCopy, vector, size);
	}
	else {
		vectorCopy = NULL;
	}
	return vectorCopy;
}


double getLeftResultFactor(MemoryManager* memManager, int locus) {
	double factor;
	double* pFactor = operateMemoryManagerAndPrintLog(memManager, NULL, LIKELIHOOD_FACTORS_IND, locus, 1);
	if (pFactor == NULL) {
		factor = 0;
	}
	else {
		factor = *(pFactor);
		free(pFactor);
	}
	return factor;
}


/******************************************************************************
*
*       Function        operateMemoryManagerAndPrintLog
*       Parameters	manager - pointer to memory manager
*								dataToStore - data to be stored in the case of storing operation (else ignored)
*								dataType - should be one of the 
*									{LIKELIHOOD_PROBS_IND, LIKELIHOOD_FACTORS_IND, HAPLO_BACK_POINTERS_IND, HAPLO_PROBS_IND}
*								locus - locus index in the range [0.. (NumOfLocuses - 2)]
*								operationType: 0 - store; 1 - restore;	else do nothing
*       Output			in the case of restore operation - pointer to the restored data, in any other case NULL
*       Description	operates data storing/restoring of MemoryManager
*								and checks the status of the performed operation, 
*								for each possible status prints to stdout the appropriate message
*       Comments		none
*
******************************************************************************/
void* operateMemoryManagerAndPrintLog(MemoryManager* manager,
																			void* dataToStore,
																			int dataType,
																			int locus,
																			int operationType) {
	void* data = NULL;
																				
	//	Perform the operation
	switch (operationType) {
		case 0:	storeSlotInfo(manager, dataToStore, dataType, locus);
						//printf("Store operation: ");
						break;
		case 1:	data = restoreSlotInfo(manager, dataType, locus);
						//printf("Restore operation: ");
						break;
		default:	break;
	}
	//	Print the result
	/*printf("data type %d, locus number %d: ", dataType, locus);
	switch(getLastOperationResult(manager)) {
		case NoSuchInfoType:			printf("error - datatype not found!\n");
														break;
		case SlotOutOfRange:			printf("error - locus is out of range!\n");
														break;
		case WindowCorruption:		printf("error - trying to store data not due to the window frame!\n");
														break;
		case SlotInfoExists:				printf("some data with the same type was already stored in this locus\n");
														break;
		case	SlotInfoDoesntExist:		printf("there is no data with the specified locus number and data type stored\n");
														break;
		case	MemoryManagerNotNeeded:	
														printf("no need in memory manager\n");
														break;
		case	MemoryManagerInternalError:
														printf("error - internal memory manager error, needed DEBUG!\n");
														break;
		case NullInfoPtr:					printf("can't store the info null pointer\n");
														break;
		case Success:							printf("operation succeeded\n");
														break;
		default:									break;
	}*/	

	return data;
}


/******************************************************************************
*
*       Function        createBackwardPhaseIterationInput
*       Parameters		none
*       Output          BackwardPhaseIterationInput
*       Description		creates BackwardPhaseIterationInput
*       Comments		none
*
******************************************************************************/
BackwardPhaseIterationInput* createBackwardPhaseIterationInput() {
	BackwardPhaseIterationInput* backwardPhaseIterationInput;

	sl_myMalloc(backwardPhaseIterationInput, sizeof(BackwardPhaseIterationInput));
	backwardPhaseIterationInput->startLocus = 0;
	backwardPhaseIterationInput->endLocus = 0;
	backwardPhaseIterationInput->FactorFromRight = 0;
	backwardPhaseIterationInput->rightPriorProbs = NULL;
	backwardPhaseIterationInput->haplotypingBackPointers = NULL;
	backwardPhaseIterationInput->haplotypingProbs = NULL;
	
	return backwardPhaseIterationInput;
}



/******************************************************************************
*
*       Function        deleteForwardPhaseIterationInput
*       Parameters		ForwardPhaseIterationInput
*       Output          none
*       Description		deletes ForwardPhaseIterationInput
*       Comments		none
*
******************************************************************************/
void deleteForwardPhaseIterationInput(ForwardPhaseIterationInput* forwardIterationInput) {
	
	if (forwardIterationInput->distFromFirstMarker != NULL) {
		free(forwardIterationInput->distFromFirstMarker);
	}
	if (forwardIterationInput->leftPriorProbs != NULL) {
		free(forwardIterationInput->leftPriorProbs);
	}
	if (forwardIterationInput->rightPriorProbs != NULL) {
		free(forwardIterationInput->rightPriorProbs);
	}
	if (forwardIterationInput->rightToLeftEndProbs != NULL) {
		free(forwardIterationInput->rightToLeftEndProbs);
	}
	if (forwardIterationInput->haploLeftLocusBackPointers != NULL) {
		free(forwardIterationInput->haploLeftLocusBackPointers);
	}
	if (forwardIterationInput->haploRightPriorProbs != NULL) {
		free(forwardIterationInput->haploRightPriorProbs);
	}
	free(forwardIterationInput->haploMaxVectorLeftLocus);
	deleteHaplotypingPrintData(forwardIterationInput->haplotypingPrintData);
	free(forwardIterationInput);
}


/******************************************************************************
*
*       Function        deleteHaplotypingPrintData
*       Parameters		HaplotypingPrintData
*       Output          none
*       Description		deletes HaplotypingPrintData
*       Comments		none
*
******************************************************************************/
void deleteHaplotypingPrintData(HaplotypingPrintData* haplotypingPrintData) {
	
	fclose(haplotypingPrintData->haplotypesFile);
	free(haplotypingPrintData->recombCountersArray);
	free(haplotypingPrintData->recombHaplotypesArray);
	free(haplotypingPrintData);
}



/******************************************************************************
*
*       Function        performForwardIteration
*       Parameters		iterationInput - input structure for the current iteration
*       Output          test point with maximal LOD score
*       Description		performs one iteration of the forward phase in window loop calculation
*						given the range of loci [startLocus..endLocus]
*       Comments		none
*
******************************************************************************/
TestPoint* performForwardIteration(ForwardPhaseIterationInput* iterationInput, TestPoint* maxNode, FILE *resFile) {
	TestPoint* currIterationMaxNode;
	double* leftPriorForNextIteration;

	//	Preparing single locus probabilities and test points 
	PrepareMatrix(iterationInput->startLocus, iterationInput->endLocus);
	createTestPoints(iterationInput->startLocus, iterationInput->endLocus, 
		iterationInput->offMapLeftPtr, iterationInput->offMapRightPtr, iterationInput->distFromFirstMarker);
	
	//	Performing forward-backward calculation
	leftPriorForNextIteration = likelihood(iterationInput);
	if (iterationInput->leftPriorProbs != NULL) {
		free(iterationInput->leftPriorProbs);
		iterationInput->leftPriorProbs = NULL;
	}
	if (iterationInput->rightPriorProbs != NULL) {
		if (iterationInput->rightToLeftEndProbs != NULL) {
			free(iterationInput->rightToLeftEndProbs);
		}
		iterationInput->rightToLeftEndProbs = iterationInput->rightPriorProbs;
		iterationInput->rightPriorProbs = NULL;
	}
	iterationInput->leftPriorProbs = leftPriorForNextIteration;

	//	Performing haplotyping Viterbi backward calculation 
	haplotypeCalculationForwardPhase(iterationInput->haploLeftLocusBackPointers, 
			iterationInput->haploMaxVectorLeftLocus, iterationInput->haploRightPriorProbs);
	if (iterationInput->haploLeftLocusBackPointers != NULL) {
		free(iterationInput->haploLeftLocusBackPointers);
		iterationInput->haploLeftLocusBackPointers = NULL;
	}
	if (iterationInput->haploRightPriorProbs != NULL) {
		free(iterationInput->haploRightPriorProbs);
		iterationInput->haploRightPriorProbs = NULL;
	}
	// Find haplotyping alleles and printing the result
	setHaplotypeAlleles(iterationInput->haplotypingPrintData);

	//	Printing test points and finding test node with maximal LOD score, returning it
	currIterationMaxNode = printTestPoints(resFile);
	if (maxNode == NULL) {
		sl_myMalloc(maxNode, sizeof(TestPoint));
		copyTestNode(currIterationMaxNode, maxNode);
	}
	else if (currIterationMaxNode->LOD > maxNode->LOD){
		copyTestNode(currIterationMaxNode, maxNode);	
	}
	freeTestPoints();
	deleteProbabilityLocusForWindow(iterationInput->startLocus, iterationInput->endLocus);

	return maxNode;
}



/******************************************************************************
*
*       Function        performBackwardIteration
*       Parameters		iterationInput - input structure for the current iteration
*       Output          left likelihood result probabilities
*       Description		performs one iteration of the backward phase in window loop calculation
*						given the range of loci [startLocus..endLocus]
*       Comments		none
*
******************************************************************************/
void performBackwardIteration(BackwardPhaseIterationInput* backwardPhaseIterationInput) {
	double* leftResult;
	
	PrepareMatrix(backwardPhaseIterationInput->startLocus, backwardPhaseIterationInput->endLocus);
	leftResult = likelihoodBackward(backwardPhaseIterationInput->rightPriorProbs, 
										&(backwardPhaseIterationInput->FactorFromRight));
	free(backwardPhaseIterationInput->rightPriorProbs);
	backwardPhaseIterationInput->rightPriorProbs = leftResult;
	haplotypeCalculationBackwardPhase(backwardPhaseIterationInput);
	deleteProbabilityLocusForWindow(backwardPhaseIterationInput->startLocus, backwardPhaseIterationInput->endLocus);
}





/******************************************************************************

/******************************************************************************/
void normalizeDiseaseProbs(double* diseaseProbs, double probForCheck) {
	int i;
	double diff = 1;
	if (probForCheck != -1) {
		diff = probForCheck / diseaseProbs[0];
	}
	for(i = 0; i < PedigreeData.Possibilities; i++) {
		diseaseProbs[i] *= (diff * PedigreeData.Possibilities);
	}
}



/*	checking that number is not in it's scope	*/
Bool isOutOfScope(int number, int lower, int upper)
{
	if ( (number < lower) || (number > upper) )
	{
		return true;
	}
	else
	{
		return false;
	}
}



/******************************************************************************
*
*       Function        initPedigreeAndElimOrderArray
*       Parameters		pedfile name, datafile name				
*       Output          infer - product of Bayes Net for disease locus and
*						elimOrderArray - elimination order array to be filled
*						with order on variables of "prod", when selectors that
*						correspond to regular bits of inheritance vector
*						will be put in the end of "elimOrderArray" in the same
*						order as they appear in inheritance vector
*       Description		main function
*       Comments		none
*
******************************************************************************/
void initStructuresForHMM(char *pedfileName,char *datafileName) {		
	AllPedigrees* currPed;
	FILE* resFile;
	
	//	parsing input files and initiating pedigree
	#if	(PRINT_FILES_IO_TIME == 1)
		timesFile = fopen(IO_TIME_PRINTS_FILENAME, "w");
	#endif
	strcpy(PEDEGREE_FILE,pedfileName);
	strcpy(LOCI_FILE,datafileName);
	Read_Loci_File();
	Read_Pedegree_File();
	
	//	for (currPed = AllPedigreesList; currPed != NULL; currPed = currPed->next){
	currPed = AllPedigreesList;
	PedigreeGraph = currPed->PedigeeGraph;
	PedigreeData.PedigreeSize = currPed->size;
	PedigreeData.NumOfFounders = currPed->numOfFounders;
	PedigreeData.NumOfNonFounders = currPed ->numOfNonFounders;
	prepareInput();
}


inferenceInput* initInferAndElimOrderArray(int givenMem, int shouldExit) {
	inferenceInput* inf;

	//	Checking that there is enoght memory to run the program
	if (isRequiredMemoryTooBig(givenMem)) {
		printf("Memory required for HMM calculation is much too big for current computer\n");
		if (!shouldExit) {
			if ((resFile = fopen(RESULTS_FILE_NAME, "w")) == NULL) {
				printf("File %s could not be opened.", RESULTS_FILE_NAME);
				exit(1);
			}
			fprintf(resFile,"Memory required for HMM calculation is much too big for current computer\n");
			fclose(resFile);
		}
		return NULL;
	}	
	
	//	DEBUG prints
	#if (PRINT_FFT_TIME == 1)
		fftFile = fopen(FFT_TIMES_FILE, "w");
	#endif
	#if (PRINT_HAPLO_TIME == 1)
		haploFile = fopen(HAPLO_TIMES_FILE, "w");
	#endif
	#if (PRINT_SINGLE_LOCUS_TIME == 1)
		singleLocusFile = fopen(SINGLE_LOCUS_TIMES_FILE, "w");
	#endif
	#if (PRINT_VECTORS_MULT_TIME == 1)
		vectorsMultFile = fopen(VECTORS_MULT_TIMES_FILE, "w");
	#endif

	//	initiation of inference input structure:
	sl_myMalloc(inf, sizeof(inferenceInput));
	inf->productPtr = initProduct(AllPedigreesList, lociDataArray, 
		ProgramSpecificData.sexLinked);
	//	initiation of arrays (not eliminated in "inf")
	initElimOrderArray(inf);
	//andreiPrintProduct(inf->productPtr);
	return inf;
}


inferenceInput* initInferAndElimOrderArrayForSelectors() {
	inferenceInput* inf;
	//	initiation of inference input structure:
	sl_myMalloc(inf, sizeof(inferenceInput));
	inf->productPtr = initProduct(AllPedigreesList, lociDataArray, 
		ProgramSpecificData.sexLinked);
	//	initiation of arrays (not eliminated in "inf")
	initElimOrderArrayNumSelectors(inf, NUM_SELECTORS);
	return inf;
}


//	Checking that there is enoght memory to run the program)
//	Calculation is made in the log-scale in order to prevent big numbers storage problems
//	givenMem - is in MegaBytes!
Bool isRequiredMemoryTooBig(int givenMem) {
	Bool isMemoryTooBig;
	double availableMemory, logAvailableMemory;
	double requiredMemory, logRequiredMemory;
	int numCols;

	isMemoryTooBig = false;
	// instead of using only available memory: availablePhysMem();
	if (givenMem == 0) {
		availableMemory = TOTAL_MEM_PERCENT_TO_USE * totalPhysMem(); 
	}
	else {
		availableMemory = givenMem * 1024.00 * 1024.00;
	}

	//	1st step: AlleleInh (if used, that means if ALLELES_INH_USE equals 0) + 
	//	6 vectors {ProbabilityDesiase, SpairsArray, SallArray, Vector, fromLeftRes, fromRightRes}
	//	+ SpeedToRegularArray
	logAvailableMemory = log(availableMemory)/log(2);
	#if (ALLELES_INH_USE == 0)
		numCols = (2*(PedigreeData.NumOfNonFounders) + 1)*sizeof(int) + 6*sizeof(double);
	#else
		numCols = sizeof(int) + 6*sizeof(double);
	#endif
	logRequiredMemory = PedigreeData.SpeedUpNumOfMeioses + log(numCols)/log(2);
	if (logAvailableMemory <= logRequiredMemory) {
		isMemoryTooBig = true;
	}
	else {
		availableMemory -= PedigreeData.Possibilities * numCols;
	}

	if (isMemoryTooBig == false) {
		//	2nd step: Pedigree, ProbabilityLocus, LociDataArray, Recomb_values
		logAvailableMemory = log(availableMemory);
		numCols = (PedigreeData.NumOfFounders + PedigreeData.NumOfNonFounders)*sizeof(Locus) + 
			sizeof(double*) + sizeof(LOCUS_DATA) + sizeof(double);
		logRequiredMemory = log(NumOfLocuses) + log(numCols);
		if (logAvailableMemory <= logRequiredMemory) {
			isMemoryTooBig = true;
		}
		else {
			availableMemory -= NumOfLocuses*numCols;
		}
	}
	
	if (isMemoryTooBig == false) {
		//	3rd step: 1 locus for running (int + 3*double) and 1 locus for storing (int + 2*double)
		logAvailableMemory = log(availableMemory)/log(2);
		numCols = 2*sizeof(int) + 5*sizeof(double);
		logRequiredMemory = PedigreeData.SpeedUpNumOfMeioses + log(numCols)/log(2);
		if (logAvailableMemory <= logRequiredMemory) {
			isMemoryTooBig = true;
		}
	}
	
	return isMemoryTooBig;
}


/******************************************************************************
*
*       Function        getMemForWindowCalc
*       Parameters	none	
*       Output           memory size in bytes available for window HMM calculation
*       Description	gets the required memory
*       Comments		none
*
******************************************************************************/
double getMemForWindowCalc(int givenMem) {
	double vectorSize, locusSize;
	double availMem;

	//	AllelesInh +  6 vectors {ProbabilityDesiase, SpairsArray, SallArray, Vector, fromLeftRes, fromRightRes}
	#if (ALLELES_INH_USE == 0)
		vectorSize = (2*(PedigreeData.NumOfNonFounders) + 1)*sizeof(int) + 6*sizeof(double);
	#else
		vectorSize = sizeof(int) + 6*sizeof(double);
	#endif
	
	//	Pedigree myLocus (for all persons) + ProbabilityLocus + lociDataArray +  Recomb_values
	locusSize = (PedigreeData.NumOfFounders + PedigreeData.NumOfNonFounders)*sizeof(Locus) + 
			sizeof(double*) + sizeof(LOCUS_DATA) + sizeof(double);
	// instead of using only available memory: availablePhysMem();
	if (givenMem == 0) {
		availMem = TOTAL_MEM_PERCENT_TO_USE * totalPhysMem(); 
	}
	else {
		availMem = givenMem * 1024.00 * 1024.00;
	}
	availMem = availMem - PedigreeData.Possibilities*vectorSize - NumOfLocuses*locusSize;

	return availMem;
}


/******************************************************************************
*
*       Function        getSizeOfRunningLocus
*       Parameters	none	
*       Output           memory size in bytes of the locus for the run of forward step:
*								back pointers (int*), ProbabilityLocus (double*), RightToLeftEndProb (double*),
*								LeftToRightEndProb (double*)
*       Description	gets the required memory
*       Comments		none
*
******************************************************************************/
double getSizeOfRunningLocus() {
	double size = PedigreeData.Possibilities * (sizeof(int) + 3*sizeof(double));
	return size;
}



/******************************************************************************
*
*       Function        initiateLociTypes
*       Parameters	managerInput	
*       Output           none
*       Description	fills in the array LociTypesArray of managerInput
*       Comments		none
*
******************************************************************************/
void initiateLociTypes(MemoryManagerCreateInput* managerInput) {
	int i;

	sl_myMalloc(managerInput->slotsInfoTypesArray, (managerInput->numSlotsInfoTypes)*sizeof(SlotInfoType));
	for(i = 0; i < managerInput->numSlotsInfoTypes; i++) {
		managerInput->slotsInfoTypesArray[i].funcDeleteSlotInfoFromRAM = free;
		switch (i) {
			case LIKELIHOOD_PROBS_IND:			managerInput->slotsInfoTypesArray[i].slotInfoSize = 
																				PedigreeData.Possibilities*sizeof(double);
																			managerInput->slotsInfoTypesArray[i].funcWriteInfoToFile = 
																				writeDoubleArrayToFile;
																			managerInput->slotsInfoTypesArray[i].funcReadInfoFromFile =
																				readDoubleArrayFromFile;
																			break;
			case LIKELIHOOD_FACTORS_IND:		managerInput->slotsInfoTypesArray[i].slotInfoSize = 
																				sizeof(double);
																			managerInput->slotsInfoTypesArray[i].funcWriteInfoToFile = 
																				writeDoubleArrayToFile;
																			managerInput->slotsInfoTypesArray[i].funcReadInfoFromFile =
																				readDoubleArrayFromFile;
																			break;
			case HAPLO_BACK_POINTERS_IND:	managerInput->slotsInfoTypesArray[i].slotInfoSize = 
																				PedigreeData.Possibilities*sizeof(int);
																			managerInput->slotsInfoTypesArray[i].funcWriteInfoToFile = 
																				writeIntArrayToFile;
																			managerInput->slotsInfoTypesArray[i].funcReadInfoFromFile =
																				readIntArrayFromFile;
																			break;
			case HAPLO_PROBS_IND:					managerInput->slotsInfoTypesArray[i].slotInfoSize = 
																				PedigreeData.Possibilities*sizeof(double);
																			managerInput->slotsInfoTypesArray[i].funcWriteInfoToFile = 
																				writeDoubleArrayToFile;
																			managerInput->slotsInfoTypesArray[i].funcReadInfoFromFile =
																				readDoubleArrayFromFile;
																			break;
			default:													break;
		}
	}
}


void* readIntArrayFromFile(char* filePath, int arraySize) {
	int numElements = arraySize / sizeof(int);
	return readArrayFromBinaryFile(sizeof(int), numElements, filePath);
}


void* readDoubleArrayFromFile(char* filePath, int arraySize) {
	int numElements = arraySize / sizeof(double);
	return readArrayFromBinaryFile(sizeof(double), numElements, filePath);
}


int writeIntArrayToFile(void* array, char* filePath, int arraySize) {
	int numElements = arraySize / sizeof(int);
	return writeArrayToBinaryFile(array, sizeof(int), numElements, filePath);
}


int writeDoubleArrayToFile(void* array, char* filePath, int arraySize) {
	int numElements = arraySize / sizeof(double);
	return writeArrayToBinaryFile(array, sizeof(double), numElements, filePath);
}


//-----------------------------------------------------------------------------------------------------------------------


//	help function for "initPedigreeAndElimOrderArray" function
void initElimOrderArray(inferenceInput* inf)
{
	variable* currVariable;
	int currElimVarIndex, regularBitIndex;

	//	initiation of variables arrays
	inf->numNotElimVars = PedigreeData.SpeedUpNumOfMeioses;
	sl_myMalloc(inf->notElimOrderArray, (inf->numNotElimVars)*sizeof(int));
	//	iteration over variables in "prod"
	currVariable = inf->productPtr->variables;
	currElimVarIndex = 0;
	for (; currElimVarIndex < inf->productPtr->numVariables; 
			currVariable++, currElimVarIndex++)
	{
		//	choosing only SELECTOR variables 
		if (currVariable->originalNode->nodeType == SELECTOR)
		{
			//	checking that current selector is regular bit in inheritance vector
			regularBitIndex = checkRegularBitIndex(currVariable->originalNode->pNum, 
												currVariable->originalNode->nodeSex);
			if (regularBitIndex >= 0)
			{
				//	current selector is regular bit in inheritance vector
				inf->notElimOrderArray[regularBitIndex] = currElimVarIndex;
			}
		}
	}
}


void initElimOrderArrayNumSelectors(inferenceInput* inf, int numSelectors)
{
	variable* currVariable;
	int currElimVarIndex, regularBitIndex;

	//	initiation of variables arrays
	inf->numNotElimVars = numSelectors;
	sl_myMalloc(inf->notElimOrderArray, (inf->numNotElimVars)*sizeof(int));
	//	iteration over variables in "prod"
	currVariable = inf->productPtr->variables;
	currElimVarIndex = 0;
	for (; currElimVarIndex < inf->productPtr->numVariables; 
			currVariable++, currElimVarIndex++)
	{
		//	choosing only SELECTOR variables 
		if (currVariable->originalNode->nodeType == SELECTOR)
		{
			//	checking that current selector is regular bit in inheritance vector
			regularBitIndex = checkRegularBitIndex(currVariable->originalNode->pNum, 
												currVariable->originalNode->nodeSex);
			if (regularBitIndex >= 0)
			{
				if (regularBitIndex < numSelectors) {
					//	current selector is 0 regular bit in inheritance vector
					inf->notElimOrderArray[regularBitIndex] = currElimVarIndex;
				}
				else {
					//	reduce current selector to single 0 value
					currVariable->numValues = 1;
					//	reduce all functions where current selector appears
					{
						int f, i, size;
						double* newTable;
						for(f = 0; f < 2; ++f) {
							double* table = currVariable->functions[f]->probabList;
							size = currVariable->functions[f]->sizeOfTable;
							sl_myMalloc(newTable, (size/2)*sizeof(double));
							for(i = 0; i < size/2; ++i) {
								newTable[i] = table[2*i];
							}
							free(table);
							currVariable->functions[f]->probabList = newTable;
							currVariable->functions[f]->sizeOfTable = size/2;
						}
					}
				}
			}
		}
	}
}



//	help function for "initElimOrderArray" function
int checkRegularBitIndex(int personNumber, NodeSex parentalMode)
{
	Pedigree* pedigreeGH;
	int regularBitIndex, personIndex;
	AlleleData* parentalBit;

	//	initiations
	regularBitIndex = -1;
	pedigreeGH = AllPedigreesList->PedigeeGraph;
	
	//	going through all persons in "pedigreeGH" 
	for(personIndex = 0; personIndex < AllPedigreesList->size; personIndex++)
	{
		//	choosing the required person (with index equal to "personNumber")
		if ( (pedigreeGH[personIndex].informative) && 
				(pedigreeGH[personIndex].index == personNumber) )
		{
			//	choosing inheritance bit (paternal or maternal) and
			//	checking if it's regular bit
			(parentalMode == PATERNAL) ? 
				(parentalBit = pedigreeGH[personIndex].paternalAllele) :
				(parentalBit = pedigreeGH[personIndex].maternalAllele);
			if (parentalBit->bitType == RegularBit)
			{
				regularBitIndex = parentalBit->number;
			}
			break;
		}
	}
	return regularBitIndex;
}


//	freeing all the structures in inference input
void destroyInferenceInput(inferenceInput* infer)
{
	destroyProduct(infer->productPtr);
	free(infer->notElimOrderArray);
	free(infer);
}

/*
void compressToFile() {
	FILE* infoFile, *compressedFile;
	int MaxPoss, i;
	double* compressedArray, value;
	int sizeCompressedArray;
	int size = 33554432;

	sl_myMalloc(compressedArray, sizeof(double)*size);
	value = 0;
	for(i = 0; i < size; i++) {
		compressedArray[i] = value;
		value += 0.01;
	}

	mkdir("examples");
	infoFile = fopen("examples/example.bin", "wb");
	fwrite(compressedArray, sizeof(double), size, infoFile);
	free(compressedArray);
	fclose(infoFile);
	
	//	open file for writing and writing
	infoFile = fopen("examples/example.bin", "rb");
	SET_BINARY_MODE(infoFile);
	compressedFile = fopen("examples/compressedExample.bin", "wb");
	SET_BINARY_MODE(compressedFile);
	//	compressing from infoFile to compressedFile
	def(infoFile, compressedFile, Z_DEFAULT_COMPRESSION);
	fclose(infoFile);
	fclose(compressedFile);
	remove("examples/example.bin");

	infoFile = fopen("examples/example.bin", "wb");
	SET_BINARY_MODE(infoFile);
	compressedFile = fopen("examples/compressedExample.bin","rb");
	SET_BINARY_MODE(compressedFile);
	inf(compressedFile, infoFile);
	fclose(infoFile);
	fclose(compressedFile);
	remove("examples/compressedExample.bin");
	
	sl_myMalloc(compressedArray, sizeof(double)*size);
	infoFile = fopen("examples/example.bin", "rb");
	fread(compressedArray, sizeof(double), size, infoFile);
	fclose(infoFile);
	remove("examples/example.bin");
	//rmdir("examples");
	printf("\n Total run time  %.3lf sec\n\n",((double)clock())/(double)CLOCKS_PER_SEC);
	exit(1);
}*/



/******************************************************************************
*
*       Function        getMemoryManagerInput
*       Parameters	none	
*       Output          MemoryManagerCreateInput
*       Description	creates struct MemoryManagerCreateInput with relevant information
*       Comments		none
*
******************************************************************************/
MemoryManagerCreateInput* getMemoryManagerInput(int givenMem, double extMem) {
	MemoryManagerCreateInput* managerInput;

	sl_myMalloc(managerInput, sizeof(MemoryManagerCreateInput));
	managerInput->availMemForWindowCalc = getMemForWindowCalc(givenMem);
	managerInput->sizeOfRunningSlotInfo = getSizeOfRunningLocus();
	managerInput->numSlots = NumOfLocuses;
	managerInput->numSlotsInfoTypes = NUM_LOCI_INFO_TYPES;
	initiateLociTypes(managerInput);
	managerInput->availExternalMemForWindowCalc = extMem;
	return managerInput;
}


/******************************************************************************
*
*       Function        deleteMemoryManagerInput
*       Parameters	managerInput	
*       Output          none
*       Description	deletes struct MemoryManagerCreateInput
*       Comments		none
*
******************************************************************************/
void deleteMemoryManagerInput(MemoryManagerCreateInput* managerInput) {
	free(managerInput->slotsInfoTypesArray);
	free(managerInput);
}



/******************************************************************************
*
*       Function        writeArrayToBinaryFile
*       Parameters	array, elementSize, numElements, filePath	
*       Output           success indication: 0 - failed, 1 - succeded
*       Description	writes array to file in filePath
*       Comments		none
*
******************************************************************************/
int writeArrayToBinaryFile(void* array, int elementSize, int numElements, char* filePath) {
	FILE* infoFile;
	int res;

	//	Open file
	infoFile = fopen(filePath, "wb");
	if (infoFile == NULL) {
		return 0;
	}
	//	Write to file
	#if (PRINT_FILES_IO_TIME == 1)
		fprintf(timesFile,"Start writing = %lf\t\t", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif
	if (fwrite(array, elementSize, numElements, infoFile) != numElements) {
		printf("\n\n!!!!!!!!!!!!!!!!!!!!!!!!!!!!!ERROR = %s\n\n",strerror(errno));
		fclose(infoFile);
		return 0;
	}
	//	Flush all unwritten data from the output buffers to the file
	fflush(infoFile);
	#if (PRINT_FILES_IO_TIME == 1)
		fprintf(timesFile,"End writing = %lf\n", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif
	fclose(infoFile);
	return 1;
}



/******************************************************************************
*
*       Function        readArrayFromBinaryFile
*       Parameters	elementSize, numElements, filePath
*       Output           pointer to the required array
*       Description	writes array to file in filePath
*       Comments		none
*
******************************************************************************/
void* readArrayFromBinaryFile(int elementSize, int numElements, char* filePath) {
	void* array;
	FILE* infoFile;

	//	Open binary file for reading
	infoFile = fopen(filePath, "rb");
	if (infoFile == NULL) {
		return NULL;
	}
	//	Allocate the array
	sl_myMalloc(array, elementSize*numElements);
	//	Read from the file into the array
	#if (PRINT_FILES_IO_TIME == 1)
		fprintf(timesFile,"Start reading = %lf\t\t", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif
	if (fread(array, elementSize, numElements, infoFile) != numElements) {
		free(array);
		fclose(infoFile);
		return NULL;
	}
	#if (PRINT_FILES_IO_TIME == 1)
		fprintf(timesFile,"End reading = %lf\n", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif
	//	Close file and return array
	fclose(infoFile);
	return array;
}












