/* $Id: gh_probability.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  probability.c											   				*/
/*  Finds probability for each inheretence vector							*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#include "fdata.h"
#include "gdata.h"
#include "allocfile.h"
#include "sl_Auxilary.h"
#include "desiase.h"
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include "vectorGraph.h"
#include "treeSearch.h"

#include "haplotypingSimple.h"

double desiaseProb(int curLocus, int curVector);


/******************************************************************************
*
*       Function        numeratedProb
*       Parameters		none
*       Output          probability of the vector (global variable)
*       Description		calculates probavility of the inheritance vector
*       Comments		none
*
******************************************************************************/
double numeratedProb(int curLocus){
	int i;
	double prob=1;
	i = 0;
	while(1){
		while(i < 2*PedigreeData.NumOfFounders){
			if(Alleles[i].if_visitedA == true || Alleles[i].if_visitedB==true
				|| Alleles[i].if_visited==true)
				i++;
			else 
				break;
		}
		if(i == 2*PedigreeData.NumOfFounders)
			break;
		prob *= ComputeComp(i++, curLocus);/* i num of founder allele */
	}

	return  prob; 
}

/******************************************************************************
*
*       Function        calculateProb
*       Parameters		none
*       Output          probability of the vector (global variable)
*       Description		calculates probavility of the inheritance vector
*       Comments		inits data structures
*
******************************************************************************/
double calculateProb(int curLocus, int curVector){
	/* for all alleles find there founders  */
	memset(NumFounderNeighbours,'\0',2*PedigreeData.NumOfFounders*sizeof(int));
	if (!connectComp(curLocus,curVector ))
			return 0;

/* For each Founder allele other founder allele connected to it */
	return 	numeratedProb(curLocus);

}



/******************************************************************************
*
*       Function        PrepareMatrix
*       Parameters		none
*       Output          none
*       Description		computes all vectors probability for all loci
*       Comments		
*
******************************************************************************/
void  PrepareMatrix(int startLocus, int endLocus){
	int i, symetryMaskZero, markerOrderIndex;
	double sum;
	int curLocus;
	Assign *savedAlleles; 

	#if (PRINT_SINGLE_LOCUS_TIME == 1)
		fprintf(singleLocusFile,  "Start writing = %lf\t\t", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif

	NonZeroMarkersList = NULL;
	NumInformativeLoci = 0;
	markerOrderIndex = 0;
	
	allocDataNew(ProbabilityLocus, startLocus, endLocus);
	
	#if DISEASE == 1
			for ( i = 0; i < PedigreeData.Possibilities; i++){	
				ProbabilityDesiase[i] =  desiaseProb(DesiaseLocus, i);
				printf("Disease vector %d (%d) = %.3lf in  %.3lf sec\n", i, PedigreeData.Possibilities,ProbabilityDesiase[i],((double)clock())/(double)CLOCKS_PER_SEC);
			}
	#endif
	
	
	/* Digonal Matrixes of probability of the observed data for sl_locus i , while given vector v */
	for (curLocus = startLocus; curLocus <= endLocus; ++curLocus){	
		int numNonZeroVectors;
		
		sum = 0;
		memset(ProbabilityLocus[curLocus],'\0',sizeof(double)*PedigreeData.Possibilities);
		memset(Alleles,'\0',(2 * PedigreeData.NumOfFounders*sizeof(Assign)));

		#if TREE_SEARCH == 1
			initAllelesGirstGen(curLocus+1);
			symetryMask = 0;	
			treeSearch(0,0,true, curLocus+1, 0);
		#endif

		#if TREE_SEARCH == 0
			//	symetryMask[curLocus] = findSimmetryMask(curLocus+1);
			//	sl_myMalloc(savedAlleles,(sizeof(Assign)*NumOfFounders*2));
			//	memcpy(savedAlleles,Alleles, (sizeof(Assign)*NumOfFounders*2));
		#endif
		
		/************************************************************************/
		/*	added by Anisenia Andrei for haplotyping calculation				*/
		numNonZeroVectors = 0;
		/************************************************************************/

		symetryMaskZero = ~symetryMask;
		for (i=0;i < PedigreeData.Possibilities;i++){
		#if TREE_SEARCH == 0
			if (!(i & symetryMask[curLocus])){
				//	memcpy(Alleles,savedAlleles, (sizeof(Assign)*NumOfFounders*2));
				memset(Alleles,'\0',(2 * PedigreeData.NumOfFounders*sizeof(Assign)));
				ProbabilityLocus[curLocus][i] =  calculateProb(curLocus+1, i);
				//printf("ProbabilityLocus = %.3lf\n", ProbabilityLocus[curLocus-1][i] );

			}
		#endif

			if (i & symetryMask) {	
				ProbabilityLocus[curLocus][i] = ProbabilityLocus[curLocus][i & symetryMaskZero];
					//PREV VERSION: ProbabilityLocus[curLocus-1][i & (PedigreeData.MaxPoss -1) & ~symetryMask[curLocus-1]];
			}

			/************************************************************************/
			/*	added by Anisenia Andrei for haplotyping calculation				*/
			if (ProbabilityLocus[curLocus][i] != 0)
			{
				numNonZeroVectors++;
			}
			/************************************************************************/

			sum+=	ProbabilityLocus[curLocus][i] ;
		}
		if (sum != 0){

			/************************************************************************/
			/*	added by Anisenia Andrei for haplotyping calculation				*/
			nonZeroMarker* tmpMarker;
			sl_myMalloc(tmpMarker,sizeof(nonZeroMarker));
			tmpMarker->next = NULL;
			tmpMarker->prev = NULL;
			tmpMarker->probabilityLocusIndex = curLocus;
			tmpMarker->numNonZeroVectors = numNonZeroVectors;
			if (NonZeroMarkersList == NULL)
			{
				NonZeroMarkersList = tmpMarker;
			}
			else
			{
				NonZeroMarkersList->next = tmpMarker;
				tmpMarker->prev = NonZeroMarkersList;
				NonZeroMarkersList = tmpMarker;
			}
			/************************************************************************/

			MarkerOrder[markerOrderIndex] = curLocus;
			markerOrderIndex++;
			NumInformativeLoci++;
		}else{
			printf("Locus #%d(%s) probability is zero. Dropped.\n", (curLocus+2), lociDataArray[curLocus].locusName);
		}
	}
	
	NumInfMarkers = NumInformativeLoci;
	NumInformativeLoci++;

	#if (PRINT_SINGLE_LOCUS_TIME == 1)
		fprintf(singleLocusFile,  "End writing = %lf\n", ((double)clock())/(double)CLOCKS_PER_SEC);	
	#endif
}



/******************************************************************************
*
*       Function        deleteProbabilityLocusForWindow
*       Parameters		startLocus, endLocus - loci to be deleted
*       Output          none
*       Description		deletes all given loci probability vectors
*       Comments		
*
******************************************************************************/
void deleteProbabilityLocusForWindow(int startLocus, int endLocus) {
	int i;
	for(i = startLocus; i <= endLocus; i++) {
		if (ProbabilityLocus[i] != NULL) {
			free(ProbabilityLocus[i]);
			ProbabilityLocus[i] = NULL;
		}
	}
}
