/* $Id: haplotypingLevelsTree.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingLevelsTree.c									   				*/
/*  Constructs levels tree for haplotyping calculation						*/
/*  Author: Anisenia Andrei													*/
/****************************************************************************/


#include "haplotypingLevelsTree.h"

levelsTree* initiateLevels(levelsTree* tree, int nextLevelVectorsNum);

void TestTree2()
{
	levelsTree* levTree;
	reversedTree* revTree;
	double* probs;

	PedigreeData.Possibilities = 8;
	sl_myMalloc(probs, PedigreeData.Possibilities*sizeof(double));
	probs[0] = 0;
	probs[1] = 1;
	probs[2] = 0;
	probs[3] = 0;
	probs[4] = 1;
	probs[5] = 1;
	probs[6] = 0;
	probs[7] = 1;
	revTree = reversedTreeConstruction(probs,3,4);
	//levTree = levelsTreeFromReversedTree(revTree);
	free(probs);
	destroyLevelsTree(levTree);
	destroyReversedTree(revTree);
}





/****************************************************************************
*
*       Function		levelsTreeConstruction  
*       Parameters	  	vectorsProbs - probabilities of all vectors
*						treeHeight - (2*n - f) : the height of the tree
*						zeroLevelVectorsNum - number of nonzero vectors
*       Output			levelsTree  
*       Description		constructs levelsTree using listLocusVectors
*       Comments		none
*
*****************************************************************************/
levelsTree* levelsTreeConstruction(double* vectorsProbs,int treeHeight,
								   int zeroLevelVectorsNum)
{
	levelsTree* tree;
	int nextLevelVectorsNum;

	/*	initializing levelsTree and level's array	*/
	sl_myMalloc(tree,sizeof(levelsTree));
	tree->numLevels = treeHeight+1;
	sl_myMalloc(tree->zeroLevel, (treeHeight+1)*sizeof(level) );
	/*	constructing zero level	*/
	tree->zeroLevel->levelNum = 0;
	tree->zeroLevel->vectorsNum = zeroLevelVectorsNum;
	nextLevelVectorsNum = zeroLevelConctruction(tree->zeroLevel,vectorsProbs);
	if (nextLevelVectorsNum == 0)
	{
		free(tree->zeroLevel);
		free(tree);
		return NULL;
	}
	/*	constructing levels [1 .. (treeHeight)]	*/
	return initiateLevels(tree,nextLevelVectorsNum);
}





/****************************************************************************
*
*       Function		zeroLevelConctruction  
*       Parameters	  	zeroLevel - pointer to zero level node
*						vectorsProbs - probabilities of all vectors
*						at current locus
*       Output			number of vector nodes in the next level  
*       Description		constructs zero level of levelsTree using vectorsProbs
*       Comments		none
*
*****************************************************************************/
int zeroLevelConctruction(level* zeroLevel,double* vectorsProbs)
{
	vectorNode* thisNode;
	int nextLevelVectorsNum, i, thisParentVector, prevParentVector, vectorsNum;

	/*	initializing zero level	node array	*/
	sl_myMalloc(zeroLevel->levelVectorsArray, 
				(zeroLevel->vectorsNum)*sizeof(vectorNode) );
	/*	going through vectorsProbs array 
		and initializing zero level array of levelsTree	*/
	vectorsNum = 0;
	prevParentVector = -1;
	nextLevelVectorsNum = 0;
	thisNode = zeroLevel->levelVectorsArray;
	for(i=0; i<PedigreeData.Possibilities; i++,vectorsProbs++)
	{
		if (*vectorsProbs > 0)
		{
			vectorsNum++;
			/*	initializing data of current node in the array	*/
			thisNode->parent = NULL;
			thisNode->value = *vectorsProbs;
			thisNode->vector = i;
			/*	counting nextLevelVectorsNum	*/
			thisParentVector = i>>1;
			if (thisParentVector != prevParentVector)
			{
				nextLevelVectorsNum++;
				prevParentVector = thisParentVector;
			}
			thisNode++;
		}
	}
	if (vectorsNum != zeroLevel->vectorsNum)
	{
		return 0;
	}
	else
	{
		return nextLevelVectorsNum;
	}
}





/****************************************************************************
*
*       Function		levelConstruction  
*       Parameters	  	curLevel - pointer to current level node
*       Output			number of vector nodes in the next level  
*       Description		constructs current level of levelsTree 
*						using previuos level of levelsTree
*       Comments		none
*
*****************************************************************************/
int levelConstruction(level* curLevel)
{
	/*	for counting nextLevelVectorsNum	*/
	int nextLevelVectorsNum, nextLevelParentPrev, nextLevelParentCur;
	/*	for stepping forward between nodes of curLevel	*/
	int i, thisParentVector, prevParentVector;
	vectorNode* prevLevelVectorNode, *curLevelVectorNode;
	level* prevLevel;

	/*	initializing current level node array	*/
	sl_myMalloc(curLevel->levelVectorsArray, 
				(curLevel->vectorsNum)*sizeof(vectorNode) );
	/*	initializing pointers to nodes that will be treated simultaneously	*/
	prevLevel = curLevel - 1;
	prevLevelVectorNode = prevLevel->levelVectorsArray;
	curLevelVectorNode = (curLevel->levelVectorsArray)-1;	/*	trick	*/
	/*	going through prevLevel nodes array 
		and initializing current level array of levelsTree	*/
	prevParentVector = -1;
	nextLevelVectorsNum = 0;
	nextLevelParentPrev = -1;
	for(i=0; i<(prevLevel->vectorsNum); i++, prevLevelVectorNode++)
	{
		/*	calculating parent vector of prevLevelVectorNode	*/
		thisParentVector = (prevLevelVectorNode->vector)>>1;
		/*	if we need to step forward to the next parent in curLevel	*/
		if (thisParentVector != prevParentVector)
		{	
			curLevelVectorNode++;
			prevParentVector = thisParentVector;
			/*	initializing data of current node in current level array	*/
			curLevelVectorNode->myType = -1;
			curLevelVectorNode->parent = NULL;
			curLevelVectorNode->value = -1.0;
			curLevelVectorNode->vector = thisParentVector;
			/*	counting nextLevelVectorsNum	*/
			nextLevelParentCur = (curLevelVectorNode->vector)>>1;
			if (nextLevelParentCur != nextLevelParentPrev)
			{
				nextLevelVectorsNum++;
				nextLevelParentPrev = nextLevelParentCur;
			}
		}
		/*	connecting prevLevelVectorNode to curLevelVectorNode	*/
		/*	as to parent											*/
		prevLevelVectorNode->parent = curLevelVectorNode;
		prevLevelVectorNode->myType = (prevLevelVectorNode->vector)%2;
	}
	return nextLevelVectorsNum;
}




/****************************************************************************
*
*       Function		destroyLevelsTree  
*       Parameters	  	tree
*       Output			none  
*       Description		destroys tree
*       Comments		none
*
*****************************************************************************/
void destroyLevelsTree(levelsTree* tree)
{
	level* curLevel;
	int i;

	for(i=0, curLevel=tree->zeroLevel; i<(tree->numLevels); i++, curLevel++)
	{
		free(curLevel->levelVectorsArray);
	}
	free(tree->zeroLevel);
	free(tree);
}




/****************************************************************************/
/*	help functions	*/
int findMaxVectorInLevelsTree(double* result,levelsTree* tree)
{
	int max,i, vectorsNum;
	vectorNode* curNode;

	curNode = tree->zeroLevel->levelVectorsArray;
	vectorsNum = tree->zeroLevel->vectorsNum;
	*result = curNode->value;
	max = curNode->vector;
	curNode++;
	/*	searching for the maximal vector in the zero level of the tree	*/
	for(i = 1; i < vectorsNum; i++, curNode++)
	{
		if (*result < curNode->value)
		{
			*result = curNode->value;
			max = curNode->vector;
		}
	}		
	return max;
}


levelsTree* initiateLevels(levelsTree* tree, int nextLevelVectorsNum)
{
	level* curLevel;
	int i;

	/*	constructing levels [1 .. (treeHeight)]	*/
	for(i = 1, curLevel = (tree->zeroLevel) + 1; i < tree->numLevels; i++, curLevel++)
	{
		curLevel->levelNum = i;
		curLevel->vectorsNum = nextLevelVectorsNum;
		nextLevelVectorsNum = levelConstruction(curLevel);
	}
	return tree;
}


/****************************************************************************/



