/* $Id: haplotypingLevelsTree.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingLevelsTree.h									   				*/
/*  Constructs levels tree for haplotyping calculation						*/
/*  Author: Anisenia Andrei													*/
/****************************************************************************/

#ifndef HAPLOTYPING_LEVELS_TREE_H
#define HAPLOTYPING_LEVELS_TREE_H


#include "gdata.h"
#include "sl_Auxilary.h"
#include "haplotypingReversedTree.h"


/****************************************/
/*	definitions for the levelsTree (T2)	*/

typedef struct vectorNode_t
{
	struct vectorNode_t* parent;
	int myType;		/* 0 or 1	*/
	int vector;
	double value;
}vectorNode;


typedef struct level_t
{
	int levelNum;
	int vectorsNum;
	vectorNode* levelVectorsArray;
}level;


typedef struct levelsTree_t
{
	int numLevels;
	level* zeroLevel;
}levelsTree;

/****************************************/



/****************************************************************************
*
*       Function		levelsTreeConstruction  
*       Parameters	  	vectorsProbs - probabilities of all vectors
*						treeHeight - (2*n - f) : the height of the tree
*						zeroLevelVectorsNum - number of vector nodes in 
*						listLocusVectors
*       Output			levelsTree  
*       Description		constructs levelsTree using listLocusVectors
*       Comments		none
*
*****************************************************************************/
levelsTree* levelsTreeConstruction(double* vectorsProbs,int treeHeight,
								   int zeroLevelVectorsNum);



/****************************************************************************
*
*       Function		zeroLevelConctruction  
*       Parameters	  	zeroLevel - pointer to zero level node
*						vectorsProbs - probabilities of all vectors
*						at current locus
*       Output			number of vector nodes in the next level  
*       Description		constructs zero level of levelsTree using vectorsProbs
*       Comments		none
*
*****************************************************************************/
int zeroLevelConctruction(level* zeroLevel,double* vectorsProbs);



/****************************************************************************
*
*       Function		levelConstruction  
*       Parameters	  	curLevel - pointer to current level node
*       Output			number of vector nodes in the next level  
*       Description		constructs current level of levelsTree 
*						using previuos level of levelsTree
*       Comments		none
*
*****************************************************************************/
int levelConstruction(level* curLevel);


/****************************************************************************
*
*       Function		destroyLevelsTree  
*       Parameters	  	tree
*       Output			none  
*       Description		destroys tree
*       Comments		none
*
*****************************************************************************/
void destroyLevelsTree(levelsTree* tree);


/*	help function	*/
int findMaxVectorInLevelsTree(double* result,levelsTree* tree);


void TestTree2();



#endif


