/* $Id: haplotypingReversedTree.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingReversedTree.c								   				*/
/*  Constructs reversed tree for haplotyping calculation					*/
/*  Author: Anisenia Andrei													*/
/****************************************************************************/


#include "haplotypingReversedTree.h"


void TestTree1()
{
	reversedTree* tree;
	double* probs;

	PedigreeData.Possibilities = 8;
	sl_myMalloc(probs, PedigreeData.Possibilities*sizeof(double));
	probs[0] = 0;
	probs[1] = 1;
	probs[2] = 0;
	probs[3] = 0;
	probs[4] = 0;
	probs[5] = 0;
	probs[6] = 1;
	probs[7] = 1;

	tree = reversedTreeConstruction(probs,3,3);
	printf("");
}




/****************************************************************************
*
*       Function		reversedTreeConstruction  
*       Parameters	  	vectorsProbs - probabilities of all vectors
*						treeHeight - (2*n - f) : the height of the tree
*						zeroLevelVectorsNum - number of nonzero vectors
*       Output			reversedTree  
*       Description		constructs reversedTree using vectorsProbs
*       Comments		none
*
*****************************************************************************/
reversedTree* reversedTreeConstruction(double* vectorsProbs,int treeHeight,
										int zeroLevelVectorsNum)
{
	reversedTree* tree;
	int i, nextLevelVectorsNum, numToExtract;
	revLevel* currRevLevel;

	/*	allocating tree and reversed levels	*/
	sl_myMalloc(tree,sizeof(reversedTree));
	tree->numLevels = treeHeight+1;
	sl_myMalloc(tree->rootLevel, (treeHeight+1)*sizeof(revLevel) );
	/*	constructing last level	of the reversed tree	*/
	numToExtract = PedigreeData.Possibilities/2;
	currRevLevel = tree->rootLevel + treeHeight;
	currRevLevel->levelNum = treeHeight;
	currRevLevel->vectorsNum = zeroLevelVectorsNum;
	nextLevelVectorsNum = lastLevelConctruction(currRevLevel,vectorsProbs,numToExtract);
	/*	constructing levels [(treeHeight-1) .. 0] of reversed tree	*/
	for(i = treeHeight-1; i >= 0; i--)
	{
		numToExtract = numToExtract/2;
		currRevLevel--;
		currRevLevel->levelNum = i;
		currRevLevel->vectorsNum = nextLevelVectorsNum;
		nextLevelVectorsNum = 
			currLevelConctruction(currRevLevel,currRevLevel+1,numToExtract);
	}	
	return tree;
}




/****************************************************************************
*
*       Function		lastLevelConctruction  
*       Parameters	  	lastLevel - pointer to last reversed level
*						vectorsProbs - probabilities of all vectors
*						at current locus
*						numToExtract - number to extract to get parent 
*       Output			number of vector nodes in the next level  
*       Description		constructs last level of reversedTree using vectorsProbs
*       Comments		none
*
*****************************************************************************/
int lastLevelConctruction(revLevel* lastLevel,double* vectorsProbs, int numToExtract)
{
	int i;
	double* currProb;
	reversedTreeNode* currNode;

	/*	allocating last level vectors array	*/
	sl_myMalloc(lastLevel->revLevelVectorsArray, 
				(lastLevel->vectorsNum)*sizeof(reversedTreeNode));
	lastLevel->currLevelArrayEnd = 
		lastLevel->revLevelVectorsArray + lastLevel->vectorsNum;
	/*	initializations	*/ 
	currProb = vectorsProbs;
	lastLevel->firstZeroVector = NULL;
	lastLevel->firstOneVector = NULL;
	currNode = lastLevel->revLevelVectorsArray;
	/*	initializing data of the nodes in the last level	*/
	for(i = 0; i < PedigreeData.Possibilities; i++, currProb++)
	{
		/*	if the current vector is nonzero vector	*/
		if (*currProb > 0)
		{
			/*	initializing data fields of current node in the last level	*/
			currNode->leftSon = NULL;
			currNode->rightSon = NULL;
//			currNode->value = *currProb;
			currNode->vector = i;
			/*	if we need to initialize firstZeroVector in the last level	*/
			if (i - numToExtract < 0)
			{
				if (lastLevel->firstZeroVector == NULL)
				{
					lastLevel->firstZeroVector = currNode;
				}
			}
			/*	if we need to initialize firstOneVector in the last level	*/
			else
			{
				if (lastLevel->firstOneVector == NULL)
				{
					lastLevel->firstOneVector = currNode;
				}
			}
			/*	making step in the last level of the nodes	*/
			currNode++;
		}
	}
	/*	getting the number of nodes in the level which follows the last level	*/
	return getNextLevelVectorsNum(lastLevel,numToExtract);
}




/****************************************************************************
*
*       Function		getNextLevelVectorsNum  
*       Parameters	  	curLevel - pointer to current reversed level
*						numToExtract - number to extract to get parent 
*       Output			number of reversed vector nodes in the next level  
*       Description		counts nextLevelVectorsNum
*       Comments		none
*
*****************************************************************************/
int getNextLevelVectorsNum(revLevel* currLevel, int numToExtract)
{
	int nextLevelVectorsNum, zeroParent, oneParent;
	reversedTreeNode* firstZeroVector;
	reversedTreeNode* firstOneVector;

	firstZeroVector = currLevel->firstZeroVector;
	firstOneVector = currLevel->firstOneVector;
	/*	if there are no left sons or no right sons to the next level parents */
	/*	then the number of parents in the next level equals to the number 	*/
	/*	of children in the current level	*/
	if ( (firstZeroVector == NULL) || (firstOneVector == NULL) )
	{
		return currLevel->vectorsNum;
	}
	/*	counting the number of parents in the next level	*/
	nextLevelVectorsNum = 0;
	while( (firstZeroVector != currLevel->firstOneVector) && 
			(firstOneVector != currLevel->currLevelArrayEnd) )
	{
		zeroParent = firstZeroVector->vector;
		oneParent = firstOneVector->vector - numToExtract;
		/*	checking the order between zeroParent and oneParent	*/
		if (zeroParent < oneParent)
		{
			firstZeroVector++;		
		}
		if (zeroParent == oneParent)
		{
			firstZeroVector++;
			firstOneVector++;
		}
		if (zeroParent > oneParent)
		{
			firstOneVector++;
		}
		nextLevelVectorsNum++;
	}
	while(firstZeroVector != currLevel->firstOneVector)
	{
		firstZeroVector++;
		nextLevelVectorsNum++;
	}
	while(firstOneVector != currLevel->currLevelArrayEnd)
	{
		firstOneVector++;
		nextLevelVectorsNum++;
	}
	return nextLevelVectorsNum;
}



/****************************************************************************
*
*       Function		currLevelConctruction  
*       Parameters	  	parentsLevel - pointer to reversed level of parents,
*						(this is the level to be constructed)
*						childrenLevel - pointer to reversed level of children
*						(this is previously constructed level)
*						numToExtract - number to extract to get parent 
*       Output			number of vector nodes in the next level  
*       Description		constructs current level of reversedTree using previous level
*       Comments		none
*
*****************************************************************************/
int currLevelConctruction(revLevel* parentsLevel, revLevel* childrenLevel, 
						  int numToExtract)
{
	reversedTreeNode* leftChild, *rightChild;
	reversedTreeNode* currParent;
	int zeroParentVector, oneParentVector, childrenExtract, i;

	/*	allocation of current level (level of parents) array	*/
	sl_myMalloc(parentsLevel->revLevelVectorsArray,
				(parentsLevel->vectorsNum)*sizeof(reversedTreeNode));
	parentsLevel->currLevelArrayEnd = 
		parentsLevel->revLevelVectorsArray + parentsLevel->vectorsNum;

	/*	going through array of children and initializing array of parents	*/
	leftChild = childrenLevel->firstZeroVector;
	rightChild = childrenLevel->firstOneVector;
	currParent = parentsLevel->revLevelVectorsArray;
	parentsLevel->firstZeroVector = NULL;
	parentsLevel->firstOneVector = NULL;
	childrenExtract = 2*numToExtract;
	if (childrenExtract == 0)
	{
		childrenExtract = 1;
	}
	/*	going through parents level and initializing all parents	*/
	for(i = 0; i < parentsLevel->vectorsNum; i++, currParent++)
	{
		/*	finding zeroParentVector and oneParentVector	*/
		zeroParentVector = PedigreeData.Possibilities;
		oneParentVector = PedigreeData.Possibilities;
		if ( (leftChild != NULL) && (leftChild != childrenLevel->firstOneVector) )
		{
			zeroParentVector = leftChild->vector;	
		}
		if ( (rightChild != NULL) && (rightChild != childrenLevel->currLevelArrayEnd) )
		{
			oneParentVector = rightChild->vector - childrenExtract;	
		}
		
		/*	choosing vector and child for current parent	*/
		if (zeroParentVector < oneParentVector)
		{
			/*	creating parent for leftChild	*/
			currParent->leftSon = leftChild;
			currParent->rightSon = NULL;
			currParent->vector = zeroParentVector;
			leftChild++;
		}
		if (zeroParentVector > oneParentVector)
		{
			/*	creating parent for rightChild	*/
			currParent->leftSon = NULL;
			currParent->rightSon = rightChild;
			currParent->vector = oneParentVector;
			rightChild++;
		}
		if (zeroParentVector == oneParentVector)
		{
			/*	creating common parent for leftChild and rightChild	*/
			currParent->leftSon = leftChild;
			currParent->rightSon = rightChild;
			currParent->vector = zeroParentVector;
			leftChild++;
			rightChild++;
		}

		/*	if we need to initialize firstZeroVector in the parents level	*/
		if (currParent->vector - numToExtract < 0)
		{
			if (parentsLevel->firstZeroVector == NULL)
			{
				parentsLevel->firstZeroVector = currParent;
			}
		}
		/*	if we need to initialize firstOneVector in the parents level	*/
		else
		{
			if (parentsLevel->firstOneVector == NULL)
			{
				parentsLevel->firstOneVector = currParent;
			}
		}	
	}
	/*	getting the number of nodes in the level	*/
	/*	which follows the present parents level		*/
	return getNextLevelVectorsNum(parentsLevel,numToExtract);
}




/****************************************************************************
*
*       Function		destroyReversedTree  
*       Parameters	  	tree
*       Output			none  
*       Description		destroys reversed tree
*       Comments		none
*
*****************************************************************************/
void destroyReversedTree(reversedTree* tree)
{
	revLevel* curLevel;
	int i;

	for(i=0, curLevel=tree->rootLevel; i<(tree->numLevels); i++, curLevel++)
	{
		free(curLevel->revLevelVectorsArray);
	}
	free(tree->rootLevel);
	free(tree);
}








