/* $Id: haplotypingReversedTree.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingReversedTree.h								   				*/
/*  Constructs reversed tree for haplotyping calculation					*/
/*  Author: Anisenia Andrei													*/
/****************************************************************************/

#ifndef HAPLOTYPING_REVERSED_TREE_H
#define HAPLOTYPING_REVERSED_TREE_H


#include "gdata.h"
#include "sl_Auxilary.h"

/********************************************/
/*	definitions for the reversedTree (T1)	*/

typedef struct reversedTreeNode_t
{
	struct reversedTreeNode_t* leftSon;
	struct reversedTreeNode_t* rightSon;
	int vector;
//	double value;
}reversedTreeNode;


typedef struct revLevel_t
{
	int levelNum;
	int vectorsNum;
	reversedTreeNode* revLevelVectorsArray;
	reversedTreeNode* firstZeroVector;
	reversedTreeNode* firstOneVector;
	reversedTreeNode* currLevelArrayEnd;
}revLevel;


typedef struct reversedTree_t
{
	int numLevels;
	revLevel* rootLevel;
}reversedTree;



/****************************************************************************
*
*       Function		reversedTreeConstruction  
*       Parameters	  	vectorsProbs - probabilities of all vectors
*						treeHeight - (2*n - f) : the height of the tree
*						zeroLevelVectorsNum - number of nonzero vectors
*       Output			reversedTree  
*       Description		constructs reversedTree using listLocusVectors
*       Comments		none
*
*****************************************************************************/
reversedTree* reversedTreeConstruction(double* vectorsProbs,int treeHeight,
										int zeroLevelVectorsNum);



/****************************************************************************
*
*       Function		lastLevelConctruction  
*       Parameters	  	lastLevel - pointer to last reversed level
*						vectorsProbs - probabilities of all vectors
*						at current locus
*						numToExtract - number to extract to get parent 
*       Output			number of vector nodes in the next level  
*       Description		constructs last level of reversedTree using vectorsProbs
*       Comments		none
*
*****************************************************************************/
int lastLevelConctruction(revLevel* lastLevel,double* vectorsProbs, int numToExtract);



/****************************************************************************
*
*       Function		getNextLevelVectorsNum  
*       Parameters	  	curLevel - pointer to current reversed level
*						numToExtract - number to extract to get parent 
*       Output			number of reversed vector nodes in the next level  
*       Description		counts nextLevelVectorsNum
*       Comments		none
*
*****************************************************************************/
int getNextLevelVectorsNum(revLevel* currLevel, int numToExtract);



/****************************************************************************
*
*       Function		currLevelConctruction  
*       Parameters	  	parentsLevel - pointer to reversed level of parents,
*						(this is the level to be constructed)
*						childrenLevel - pointer to reversed level of children
*						(this is previously constructed level)
*						numToExtract - number to extract to get parent 
*       Output			number of vector nodes in the next level  
*       Description		constructs current level of reversedTree using previous level
*       Comments		none
*
*****************************************************************************/
int currLevelConctruction(revLevel* parentsLevel, revLevel* childrenLevel, 
						  int numToExtract);



/****************************************************************************
*
*       Function		destroyReversedTree  
*       Parameters	  	tree
*       Output			none  
*       Description		destroys reversed tree
*       Comments		none
*
*****************************************************************************/
void destroyReversedTree(reversedTree* tree);



void TestTree1();


#endif


