/* $Id: haplotypingSimple.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************
 ************** haplotypingSimple.h *****************************************
 ****************************************************************************
 ************** created by Anisenia Andrei **********************************/

#ifndef HAPLOTYPING_SIMPLE
#define HAPLOTYPING_SIMPLE


#include "gdata.h"
#include "fdata.h"
#include "sl_Auxilary.h"
#include "likehood.h"
#include "initiatePedigree.h"
#include "queue.h"


/****************************************************************************/
/*	list of markers where for each marker the number of non-zero vectors	 is stored	*/

typedef struct nonZeroMarker_t
{
	struct nonZeroMarker_t* next;
	struct nonZeroMarker_t* prev;
	int probabilityLocusIndex;
	int numNonZeroVectors;
}nonZeroMarker;

nonZeroMarker* NonZeroMarkersList;
/**************************************************************************/



/***************************************************************************/
/*	array of powers of 2 just like in NPL calculation	*/
int* powers2;
int size;
/***************************************************************************/


/************************************************************************/
/* added by Anisenia Andrei												*/
/* for storing the result of haplotype calculation						*/
/* the most possible vectors path through the chromosome is stored		*/
/* the size of this vector of vectors indexes is (NumInformativeLoci-1) */
int* haplotypeVectors;
/************************************************************************/

void TestHaplotyping();


/****************************************************************************
*
*       Function		haplotypeCalculationForwardPhase  
*       Parameters	leftLocusBackPointers - back pointers at the first window locus
*								maxVectorLeftLocus - max probability Viterbi vector at the first window locus
*								rightPriorProbs - haplotyping probabilities vector at the end locus
*       Output			the probability of the most probable vectors path    
*       Description	calculates haplotype by using NumInfMarkers as
*								number of HMM nodes, PedigreeData.Possibilities as number of 
*								hidden states (=vectors), ProbabilityLocus as 
*								emissions matrix
*       Comments		implements Viterbi algorithm in O( m*(2n-f)*2^(2n-f) )
*
*****************************************************************************/
double haplotypeCalculationForwardPhase(int* leftLocusBackPointers, 
			int* maxVectorLeftLocus, double* rightPriorProbs);



/****************************************************************************
*
*       Function		deleteBackPointers  
*       Parameters	  	backPointers
*       Output			none  
*       Description		deletes backPointers
*       Comments		none
*
*****************************************************************************/
void deleteBackPointers(int** backPointers);


/****************************************************************************
*
*       Function			haplotypeCalculationBackwardPhase
*       Parameters	  	backwardPhaseIterationInput	
*       Output				none 
*       Description		calculates haplotype by using NumOfLocuses as
*									number of HMM nodes, PedigreeData.Possibilities as number of 
*									hidden states (=vectors), ProbabilityLocus as 
*									emissions matrix
*       Comments			implements only the backward phase of Viterbi algorithm
*														
*
*****************************************************************************/
void haplotypeCalculationBackwardPhase(BackwardPhaseIterationInput* backwardPhaseIterationInput);



/****************************************************************************
*
*       Function			algorithmChooser  
*       Parameters	  	backPointers, haplotypingRightPriorProbs, leftLocusBackPointers
*       Output				probabilities at the first locus    
*       Description		chooses and operates the appropriate algorithm for 
*									calculating backward step from locus = "locus" to 
*									locus = "locus"-1
*       Comments			none
*
*****************************************************************************/
double* algorithmChooser(int** backPointers, double* haplotypingRightPriorProbs, int* leftLocusBackPointers);


/****************************************************/
/*	help functions for algorithmChooser() function	*/
int findMaxVectorInArray(VECTOR prevStepProbsArray);

void deleteNonZeroMarkers();

void normalizeProbsArray(VECTOR prevStepProbsArray);

/****************************************************************************
*
*       Function		findMaxPath  
*       Parameters	  	maxVector - the result of the last step of backward phase
*									backPointers
*									maxVectorRightLocusForUpdate - vector to be updated after finding the max path
*       Output			none  
*       Description		traces back the most probable path using backPointers matrix 
*       Comments		none
*
*****************************************************************************/
void findMaxPath(int maxVector,int** backPointers, int* maxVectorRightLocusForUpdate);



/****************************************************************************
*
*       Function		haplotyping  
*       Parameters	  	none
*       Output			none  
*       Description		main function in haplotyping 
*       Comments		none
*
*****************************************************************************/
void haplotyping();




#endif


