/* $Id: haplotypingTreesAlgorithms.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingTreesAlgorithms.c							   				*/
/*  Implements 3 algorithms:												*/
/*	1. transition from reversed tree to levels tree: treeToTreeCalculation	*/
/*	2. transition from array to levels tree:		 arrayToTreeCalculation	*/
/*	3. transition from reversed tree to array:		 treeToArrayCalculation	*/
/*  Author: Anisenia Andrei													*/
/****************************************************************************/



#include "haplotypingTreesAlgorithms.h"
#include "haplotypingSimple.h"



void TestAlgorithm()
{
/*	int** backPointers;
	double* probs;
	reversedTree* revTree;
	levelsTree* levTree;

	Possibilities = 8;
	size = 3;
	powers2 = initiatePowersOf2(size+1);
	sl_myMalloc(backPointers, Possibilities*sizeof(int*));
	sl_myMalloc(probs, Possibilities*sizeof(double));


	/*	reversed tree construction	*
	probs[0] = 0;
	probs[1] = 1;
	probs[2] = 0;
	probs[3] = 0;
	probs[4] = 0;
	probs[5] = 0;
	probs[6] = 1;
	probs[7] = 1;
	revTree = reversedTreeConstruction(probs,size,3);

	probs[0] = 0;
	probs[1] = 1;
	probs[2] = 0;
	probs[3] = 0;
	probs[4] = 4;
	probs[5] = 5;
	probs[6] = 0;
	probs[7] = 7;
	
	treeToArrayCalculation(revTree,probs,backPointers,0,0.25);
/*
	treeToArrayCalculation(revTree,probs,backPointers,0.25);
	
	treeToTreeCalculation(revTree,levTree,backPointers,0.25);
	levTree = levelsTreeConstruction(probs,3,4);

	free(backPointers);
	free(probs);
	destroyReversedTree(revTree);
*/	
}








/****************************************************************************
*
*       Function		levelCalculation  
*       Parameters	  	currLevel - level to be calculated
*						comparisonBit - bit to be compared
*						theta - recombination value
*       Output			next level in current levels tree  
*       Description		computes transition from the current level 
*						in levels tree to the next level using comparisonBit
*       Comments		this function is used by treeToTreeCalculation
*						and by arrayToTreeCalculation algorithms
*
*****************************************************************************/
level* levelCalculation(level* currLevel, int comparisonBit, double theta)
{
	vectorNode* currLevNode, *prevParent, *thisParent;
	double newValue;
	int i;

	currLevNode = currLevel->levelVectorsArray;
	prevParent = NULL;
	for(i = 0; i < currLevel->vectorsNum; i++, currLevNode++)
	{
		thisParent = currLevNode->parent;
		/*	finding transition value	*/
		if (comparisonBit == currLevNode->myType)
		{
			/*	bits are the same => no recombination	*/
			newValue = (currLevNode->value) * (1 - theta);
		}
		else
		{
			/*	bits are different => recombination	*/
			newValue = (currLevNode->value) * theta;
		}
		/*	if this parent and previous parent are not the same or	*/
		/*	they are the same and current child node is maximal		*/
		/*	then we rewrite the transition information in parent	*/
		if ( (thisParent != prevParent) ||
				((thisParent == prevParent)&&(newValue > thisParent->value)) )
		{
			thisParent->value = newValue;
			thisParent->vector = currLevNode->vector;
		}
		prevParent = thisParent;
	}
	currLevel++;
	return currLevel;
}



/****************************************************************************
*
*       Function		initiateInterchanges  
*       Parameters	  	numInterchnages
*       Output			array of interchanges of size = numInterchnages  
*       Description		initiates lastInterchnages array
*       Comments		this function is used by treeToTreeCalculation
*						and by treeToArrayCalculation algorithms
*
*****************************************************************************/
lastInterchange* initiateInterchanges(int numInterchnages)
{
	lastInterchange* lastInterArray;
	int i;

	sl_myMalloc(lastInterArray, numInterchnages*sizeof(lastInterchange));
	for(i = -1; i <= numInterchnages - 2; i++)
	{
		lastInterArray[i+1].index = i;
		lastInterArray[i+1].lastInterchangeLevel = NULL;
		lastInterArray[i+1].lastInterchangeNode = NULL;
	}
	return lastInterArray;
}




/****************************************************************************
*
*       Function		treeToTreeCalculation  
*       Parameters		probsArray - array of probabilities to be filled
*						revTree - reversed tree
*						levTree - levels tree
*						backPointers
*						theta
*       Output			updated array of probabilities  
*       Description		performs transition from reversed tree to levels tree
*						and saves transition probabilities in reversed tree
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* treeToTreeCalculation(double* probsArray, reversedTree* revTree, levelsTree* levTree, 
						   int* backPointers, double theta)	
{
	reversedTreeNode* currRevNode;
	level* currLevel;
	lastInterchange* lastInterPtr, *lastInterArray;
	int comparisonBit;

	/*	initializations	*/
	currRevNode = revTree->rootLevel->revLevelVectorsArray;
	currLevel = levTree->zeroLevel;
	lastInterArray = initiateInterchanges(revTree->numLevels);
	lastInterPtr = lastInterArray;

	/*	performing transition algorithm	from reversed tree to levels tree	*/
	while(1)
	{
		/*	choosing the case to treat:	*/
		if (currRevNode->leftSon != NULL)
		{
			/*	if the treated node has left son:	*/
			if (currRevNode->rightSon != NULL)
			{
				/*	if the treated node has 2 sons: left and right:	*/
				/*	saving current tree interchange	*/
				lastInterPtr++;
				lastInterPtr->lastInterchangeNode = currRevNode;
				lastInterPtr->lastInterchangeLevel = currLevel;
			}
			/*	going to the left son	*/
			comparisonBit = 0;
			currRevNode = currRevNode->leftSon;
		}
		else
		{
			if (currRevNode->rightSon != NULL)
			{
				/*	if the treated node has only right son:	*/
				/*	going to the right son	*/
				comparisonBit = 1;
				currRevNode = currRevNode->rightSon;
			}
			else
			{
				/*	if the treated node is the leaf node: 	*/
				/*	saving transition value and vectors	*/
				probsArray[currRevNode->vector] = 
					probsArray[currRevNode->vector] * (currLevel->levelVectorsArray->value);
				backPointers[currRevNode->vector] = 
					currLevel->levelVectorsArray->vector;
				if (lastInterPtr->index == -1) 
				{
					/*	if it was the last leaf	*/
					break;
				}
				else
				{
					/*	we return to the last interchange in the reversed tree	*/
					/*	and then we will turn to the right son of currRevNode	*/
					currRevNode = lastInterPtr->lastInterchangeNode;
					currLevel = lastInterPtr->lastInterchangeLevel;
					currRevNode->leftSon = NULL;
					lastInterPtr--;
					continue;
				}
			}
		}
		/*	now we need to go through the current level nodes in levels tree	*/
		/*	and to make the transition to the next level using comparisonBit	*/
		currLevel = levelCalculation(currLevel,comparisonBit,theta);
	}
	free(lastInterArray);
	return probsArray;
}




/****************************************************************************
*
*       Function		arrayToTreeCalculation  
*       Parameters	  	probsArray - array of probabilities (size=Possibilities)
*						levTree - levels tree
*						backPointers
*						theta
*       Output			updated array of probabilities after transition  
*       Description		performs transition from array (probsArray) to levels tree
*						and saves transition probabilities in probsArray
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* arrayToTreeCalculation(double* probsArray, levelsTree* levTree, 
						   int* backPointers, double theta)
{
	vectorBit* vectorBitsArray, *vectorBitPtr;
	level* currLevel;
	int currVector, i, powerOf2, index, size, calculatedVectorsCounter;

	/*	initializations	*/
	size = levTree->numLevels-1;
	sl_myMalloc(vectorBitsArray, size*sizeof(vectorBit));
	vectorBitPtr = vectorBitsArray;
	currVector = 0;
	/*	initiating vector bits array with zero bits	*/
	/*	and calculating tree transitions using 0 as comparison bit	*/
	powerOf2 = 1;
	currLevel = levTree->zeroLevel;
	for(i = 0; i < size; i++, vectorBitPtr++)
	{
		vectorBitPtr->bit = 0;
		vectorBitPtr->powerOf2 = powerOf2;
		vectorBitPtr->rewritable = true;
		currLevel = levelCalculation(currLevel,0,theta);
		powerOf2 = powerOf2 * 2;
	}

	/*	performing transition algorithm	from array to levels tree	*/
	calculatedVectorsCounter = 0;
	index = size;
	while(calculatedVectorsCounter < PedigreeData.Possibilities)
	{
		if (index == size)
		{
			/*	currVector is calculated, transition is ready to be saved	*/
			probsArray[currVector] = 
				probsArray[currVector] * (currLevel->levelVectorsArray->value);
			backPointers[currVector] = currLevel->levelVectorsArray->vector;
			calculatedVectorsCounter++;
			/*	stepping back	*/
			index--;
			currLevel--;
			vectorBitPtr--;
			continue;
		}
		/*	if current bit in vector is 1:	*/
		if (vectorBitPtr->bit == 1)
		{
			/*	changing current bit in vector	*/
			vectorBitPtr->bit = 0;
			vectorBitPtr->rewritable = false;
			currVector -= vectorBitPtr->powerOf2;
			/*	stepping back	*/
			index--;
			currLevel--;
			vectorBitPtr--;
			continue;
		}
		/*	if current bit in vector is 0:	*/
		else
		{
			/*	if current bit 0 can be changed to 1	*/
			if (vectorBitPtr->rewritable == true)
			{
				vectorBitPtr->bit = 1;
				currVector += vectorBitPtr->powerOf2;
				currLevel = levelCalculation(currLevel,1,theta);
			}
			/*	if current bit 0 can't be changed	*/
			else
			{
				vectorBitPtr->rewritable = true;
				currLevel = levelCalculation(currLevel,0,theta);
			}
			/*	stepping forward	*/
			index++;
			vectorBitPtr++;
		}
	}
	free(vectorBitsArray);
	return probsArray;
}




/****************************************************************************
*
*       Function		arrayToArrayCalculation  
*       Parameters	  	prevStepProbs - double* vector of size=Possibilities
*						with calculation results for all vectors in prev.step
*						backPointers - int** matrix of back pointers (for forward)
*						locus - current locus to be calculated
*						theta - recombination value between "locus" and ("locus"+1)
*       Output			updated vector prevStepProbs    
*       Description		none
*       Comments		is used by haplotypeCalculation()
*
*****************************************************************************/
double* 
arrayToArrayCalculation(VECTOR prevStepProbs, int** backPointers, int locus, double theta)
{
	int *backCur, *backCurTmp, *backNext, *backTmp;
	double *numsCur, *numsNext, *numsTmp;
	int step,i;

	/*	initialization	*/
	numsCur = prevStepProbs;
	sl_myMalloc(backCur, (PedigreeData.Possibilities*sizeof(int)) );
	backCurTmp = backCur;
	for(i=0; i < PedigreeData.Possibilities; i++, backCurTmp++)
	{
		*backCurTmp = i;
	}
	backTmp = backCur;
	numsTmp = numsCur;
	sl_myMalloc(backNext, (PedigreeData.Possibilities*sizeof(int)) );
	sl_myMalloc(numsNext, (PedigreeData.Possibilities*sizeof(double)) );

	/*	the main loop: at each step the number of treated vector groups is halved	*/
	for( step=0; step <= size-1; step++)
	{
		vectorGroupsCalculation(step,theta,numsCur,numsNext,backCur,backNext);
		/*	changing places between numsCur,backCur and numsNext,backNext	*/
		/*	using numsTmp,backTmp											*/
		numsCur = numsNext;
		backCur = backNext;
		numsNext = numsTmp;
		backNext = backTmp;
		numsTmp = numsCur;
		backTmp = backCur;
	} 
	
	free(backNext);
	free(numsNext);

	backPointers[locus] = backCur;
	return numsCur;
}



/****************************************************************************
*
*       Function		vectorGroupsCalculation  
*       Parameters		step - current step in the main loop of locusCalculation
*						theta,numsCur,numsNext,backCur,backNext - 
*						were defined (or received) in locusCalculation
*       Output			none
*       Description		none
*       Comments		is used by arrayToArrayCalculation()
*
*****************************************************************************/
void vectorGroupsCalculation(int step, double theta, 
							 double* numsCur, double* numsNext,
							 int* backCur, int* backNext)
{
	int i, k, j;
	int powersStep, powersHalfStep;
	int* backCurPtr1, *backCurPtr2, *backNextPtr1, *backNextPtr2;
	double value1, value2;
	double* numsCurPtr1, *numsCurPtr2, *numsNextPtr1, *numsNextPtr2;

	powersHalfStep = powers2[step];
	powersStep = powers2[step+1];
	/*	the 1st loop: we go over all vector groups at current step	*/
	for( i=0; i <= PedigreeData.Possibilities-1; i+=powersStep)
	{
		/*	the 2nd loop: we go over all vectors in current vector group	*/
		k = i+powersHalfStep;
		numsCurPtr1 = numsCur + i;
		numsCurPtr2 = numsCurPtr1 + powersHalfStep;
		numsNextPtr1 = numsNext + i;
		numsNextPtr2 = numsNextPtr1 + powersHalfStep;
		backCurPtr1 = backCur + i;
		backCurPtr2 = backCurPtr1 + powersHalfStep;
		backNextPtr1 = backNext + i;
		backNextPtr2 = backNextPtr1 + powersHalfStep;
		for( j=i; j<k; j++)
		{
			/*	if the last digit of the vector is 0:					*/
			value1 = (*numsCurPtr1)*(1-theta);/*	passing to 0		*/
			value2 = (*numsCurPtr2)*theta;/*	passing to 1			*/
			/*	deciding which passing is better	*/
			if (value1 >= value2)
			{
				*numsNextPtr1 = value1;
				*backNextPtr1 = *backCurPtr1;
			}
			else
			{
				*numsNextPtr1 = value2;
				*backNextPtr1 = *backCurPtr2;
			}	

			/*	if the last digit of the vector is 1:					*/
			value1 = (*numsCurPtr1)*(theta);/*	passing to 0			*/
			value2 = (*numsCurPtr2)*(1-theta);/*	passing to 1		*/
			/*	deciding which passing is better	*/
			if (value1 >= value2)
			{
				*numsNextPtr2 = value1;
				*backNextPtr2 = *backCurPtr1;
			}
			else
			{
				*numsNextPtr2 = value2;
				*backNextPtr2 = *backCurPtr2;
			}
			numsCurPtr1++;
			numsCurPtr2++;
			numsNextPtr1++;
			numsNextPtr2++;
			backCurPtr1++;
			backCurPtr2++;
			backNextPtr1++;
			backNextPtr2++;
		}
	}
}





/****************************************************************************
*
*       Function		treeToArrayCalculation  
*       Parameters	  	revTree - reversed tree
*						probsArray - array of probabilities (size=Possibilities)
*						backPointers
*						locus
*						theta
*       Output			probsArray which was changed   
*       Description		performs transition from reversed tree to array probsArray
*						and saves transition probabilities in reversed tree
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* treeToArrayCalculation(reversedTree* revTree, double* probsArray, 
						   int** backPointers, int locus, double theta)
{
	int *backCur, *backCurTmp, *backNext, *backTmp;
	double *numsCur, *numsNext, *numsTmp;
	int iteration, i;
	revLevel* currLevel;
	iterationStruct* iStruct;

	/*	initialization	*/
	numsCur = probsArray;
	sl_myMalloc(backCur, (PedigreeData.Possibilities*sizeof(int)) );
	backCurTmp = backCur;
	for(i=0; i < PedigreeData.Possibilities; i++, backCurTmp++)
	{
		*backCurTmp = i;
	}
	backTmp = backCur;
	numsTmp = numsCur;
	sl_myMalloc(backNext, (PedigreeData.Possibilities*sizeof(int)) );
	sl_myMalloc(numsNext, (PedigreeData.Possibilities*sizeof(double)) );
	currLevel = revTree->rootLevel;
	sl_myMalloc(iStruct, sizeof(iterationStruct));
	iStruct->theta = theta;
	/*	the main loop: at each step the number of treated vector groups is halved	*/
	for( iteration = 0; iteration < size; iteration++)
	{
		iStruct->backCur = backCur;
		iStruct->backNext = backNext;
		iStruct->currLevel = currLevel;
		iStruct->iteration = iteration;
		iStruct->numsCur = numsCur;
		iStruct->numsNext = numsNext;
		currLevel = revLevelToArrayCalculation(iStruct);
		/*	changing places between numsCur,backCur and numsNext,backNext	*/
		/*	using numsTmp,backTmp											*/
		numsCur = numsNext;
		backCur = backNext;
		numsNext = numsTmp;
		backNext = backTmp;
		numsTmp = numsCur;
		backTmp = backCur;
	} 
	
	free(backNext);
	free(numsNext);
	free(iStruct);

	backPointers[locus] = backCur;
	return numsCur;
}



/****************************************************************************
*
*       Function		revLevelToArrayCalculation  
*       Parameters		currLevel - currently treated level in reversed tree 
*						step - current step in the main loop of locusCalculation
*						theta,numsCur,numsNext,backCur,backNext
*       Output			next level in reversed tree
*       Description		none
*       Comments		is used by treeToArrayCalculation()
*
*****************************************************************************/
revLevel* revLevelToArrayCalculation(iterationStruct* iStruct)
{
	int step, halfStep, vectorsNum, vectorsCounter, i, index;
	double value0, value1, theta;
	int* backCur, *backNext;
	double* numsCur, *numsNext;
	reversedTreeNode* currNode, *currSon;

	halfStep = powers2[iStruct->iteration];
	step = powers2[iStruct->iteration + 1];
	theta = iStruct->theta;
	backCur = iStruct->backCur;
	backNext = iStruct->backNext;
	numsCur = iStruct->numsCur;
	numsNext = iStruct->numsNext;
	vectorsNum = iStruct->currLevel->vectorsNum;

	/*	the 1st loop: going through all vectors space in steps of powersStep	*/
	for(i = 0; i < PedigreeData.Possibilities; i += step)
	{
		currNode = iStruct->currLevel->revLevelVectorsArray;
		/*	the 2nd loop:  we go over all nodes at current level of reversed tree	*/
		for(vectorsCounter = 0; vectorsCounter < vectorsNum; 
												vectorsCounter++, currNode++)
		{
			/*	treating sons	*/
			currSon = currNode->leftSon;
			if (currSon != NULL)
			{
				/*	treating left son with transition bit = 0	*/
				index = i + currSon->vector;
				value0 = numsCur[index]*(1-theta);				/*	passing to 0	*/
				value1 = numsCur[index+halfStep]*theta;		/*	passing to 1	*/
				/*	deciding which passing is better	*/
				if (value0 >= value1)
				{
					numsNext[index] = value0;
					backNext[index] = backCur[index];
				}
				else
				{
					numsNext[index] = value1;
					backNext[index] = backCur[index+halfStep];
				}	
			}
			currSon = currNode->rightSon;
			if (currSon != NULL)
			{
				/*	treating right son with transition bit = 1	*/
				index = i + currSon->vector;
				value0 = numsCur[index-halfStep]*theta;			/*	passing to 0	*/
				value1 = numsCur[index]*(1-theta);				/*	passing to 1	*/
				/*	deciding which passing is better	*/
				if (value0 >= value1)
				{
					numsNext[index] = value0;
					backNext[index] = backCur[index-halfStep];
				}
				else
				{
					numsNext[index] = value1;
					backNext[index] = backCur[index];
				}	
			}
		}
	}
	(iStruct->currLevel)++;
	return iStruct->currLevel;
}
















