/* $Id: haplotypingTreesAlgorithms.h,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************/
/*  haplotypingTreesAlgorithms.h							   				*/
/*  Implements 3 algorithms:												*/
/*	1. transition from reversed tree to levels tree							*/
/*	2. transition from array to levels tree									*/
/*	3. transition from reversed tree to array
/*  Author: Anisenia Andrei													*/
/****************************************************************************/

#ifndef HAPLOTYPING_TREES_ALGORITHMS_H
#define HAPLOTYPING_TREES_ALGORITHMS_H


#include "haplotypingLevelsTree.h"
#include "haplotypingReversedTree.h"



/****************************************************************************/
/*	Data structure for the first algorithm (tree to tree) 					*/
/*	and is also used in the third algorithm (tree to array)					*/ 
typedef struct lastInterchange_t
{
	int index;
	reversedTreeNode* lastInterchangeNode;
	void* lastInterchangeLevel;
}lastInterchange;


/*	Data structure for the second algorithm (array to tree)					*/
typedef struct vectorBit_t
{
	int bit;
	int powerOf2;
	Bool rewritable;
}vectorBit;


/*	Data structures for the third algorithm (tree to array)					*/
/*typedef struct levelDataArrays_t
{
	double* probsToRead;
	double* probsToWrite;
	int* vectorsToRead;
	int* vectorsToWrite;
	int numSteps;
}levelDataArrays;
*/
typedef struct iterationStruct_t
{
	revLevel* currLevel; 
	int iteration; 
	double theta; 
	double* numsCur; 
	double* numsNext;
	int* backCur; 
	int* backNext;
}iterationStruct;




/****************************************************************************
*
*       Function		levelCalculation  
*       Parameters	  	currLevel - level to be calculated
*						comparisonBit - bit to be compared
*						theta - recombination value
*       Output			next level in current levels tree  
*       Description		computes transition from the current level 
*						in levels tree to the next level using comparisonBit
*       Comments		this function is used by treeToTreeCalculation
*						and by arrayToTreeCalculation algorithms
*
*****************************************************************************/
level* levelCalculation(level* currLevel, int comparisonBit, double theta);



/****************************************************************************
*
*       Function		initiateInterchanges  
*       Parameters	  	numInterchnages
*       Output			array of interchanges of size = numInterchnages  
*       Description		initiates lastInterchnages array
*       Comments		this function is used by treeToTreeCalculation
*						and by treeToArrayCalculation algorithms
*
*****************************************************************************/
lastInterchange* initiateInterchanges(int numInterchnages);



/****************************************************************************
*
*       Function		treeToTreeCalculation  
*       Parameters		probsArray - array of probabilities to be filled
*						revTree - reversed tree
*						levTree - levels tree
*						backPointers
*						theta
*       Output			updated array of probabilities  
*       Description		performs transition from reversed tree to levels tree
*						and saves transition probabilities in reversed tree
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* treeToTreeCalculation(double* probsArray, reversedTree* revTree, levelsTree* levTree, 
						   int* backPointers, double theta);



/****************************************************************************
*
*       Function		arrayToTreeCalculation  
*       Parameters	  	probsArray - array of probabilities (size=Possibilities)
*						levTree - levels tree
*						backPointers
*						theta
*       Output			updated array of probabilities after transition  
*       Description		performs transition from array (probsArray) to levels tree
*						and saves transition probabilities in probsArray
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* arrayToTreeCalculation(double* probsArray, levelsTree* levTree, 
						   int* backPointers, double theta);


/****************************************************************************
*
*       Function		vectorGroupsCalculation  
*       Parameters		step - current step in the main loop of locusCalculation
*						theta,numsCur,numsNext,backCur,backNext - 
*						were defined (or received) in locusCalculation
*       Output			none
*       Description		none
*       Comments		is used by arrayToArrayCalculation()
*
*****************************************************************************/
void vectorGroupsCalculation(int step, double theta, 
							 double* numsCur, double* numsNext,
							 int* backCur, int* backNext);





/****************************************************************************
*
*       Function		arrayToArrayCalculation  
*       Parameters	  	prevStepProbs - double* vector of size=Possibilities
*						with calculation results for all vectors in prev.step
*						backPointers - int** matrix of back pointers (for forward)
*						locus - current locus to be calculated
*						theta - recombination value between "locus" and ("locus"+1)
*       Output			updated vector prevStepProbs    
*       Description		none
*       Comments		is used by haplotypeCalculation()
*
*****************************************************************************/
double* 
arrayToArrayCalculation(double* prevStepProbs, int** backPointers, int locus, double theta);



/****************************************************************************
*
*       Function		treeToArrayCalculation  
*       Parameters	  	revTree - reversed tree
*						probsArray - array of probabilities (size=Possibilities)
*						backPointers
*						locus
*						theta
*       Output			probsArray which was changed   
*       Description		performs transition from reversed tree to array probsArray
*						and saves transition probabilities in reversed tree
*						and transitions between vectors in backPointers
*       Comments		none
*
*****************************************************************************/
double* treeToArrayCalculation(reversedTree* revTree, double* probsArray, 
						   int** backPointers, int locus, double theta);

/****************************************************************************
*
*       Function		revLevelToArrayCalculation  
*       Parameters		currLevel - currently treated level in reversed tree 
*						step - current step in the main loop of locusCalculation
*						theta,numsCur,numsNext,backCur,backNext
*       Output			next level in reversed tree
*       Description		none
*       Comments		is used by treeToArrayCalculation()
*
*****************************************************************************/
revLevel* revLevelToArrayCalculation(iterationStruct* iStruct);


void TestAlgorithm();


#endif




