/* $Id: initAllelesHunter.c,v 1.2 2008/08/03 08:15:47 superlink Exp $ */
/****************************************************************************
 ************** initAllelesHunter.c  ****************************************
 ****************************************************************************
 ************** created by Anisenia Andrei **********************************
 *	initiates AllelesHunter structure and destroys it						*
 ****************************************************************************/


#include "initAllelesHunter.h"
#include "haplotypingSimple.h"


/****************************************************************************
*
*       Function		initiateAllelesHunter  
*       Parameters		pedigree,numF,numN,locus  	
*		Output			allelesHunter
*       Description		initiates AllelesHunter
*       Comments		none
*
*****************************************************************************/
allelesHunter* initiateAllelesHunter(ped_node** pedigree,int numF,int numN,int locus)
{
	int i;
	allelesHunter* AllelesHunter;
	
	/*	allocating memory for AllelesHunter	*/
	sl_myMalloc(AllelesHunter,sizeof(allelesHunter));
	sl_myMalloc(AllelesHunter->FounderAlleles, (2*numF*sizeof(fAllele)) );
	sl_myMalloc(AllelesHunter->People, ((numN+numF)*sizeof(pedigreePerson)) );
	for(i=0; i < numN+numF; i++)
	{
		AllelesHunter->People[i].used = false;
	}
	/*	initiating fields	*/
	AllelesHunter->numFounders = numF;
	AllelesHunter->numNonFounders = numN;
	AllelesHunter->locus = locus;
	AllelesHunter->vector = haplotypeVectors[locus];
	/*	initiating FounderAlleles array	*/
	for(i=0; i < 2*numF; i++)
	{
		AllelesHunter->FounderAlleles[i].includedInComponent = false;
		initComponentList(AllelesHunter,i);
		AllelesHunter->FounderAlleles[i].used = false;
		AllelesHunter->FounderAlleles[i].tryFirstAllele = 0;
		AllelesHunter->FounderAlleles[i].trySecondAllele = 0;
		AllelesHunter->FounderAlleles[i].index = i;
	}
	connectFounderAlleles(pedigree,AllelesHunter);
	connectNonFounderAlleles(pedigree,AllelesHunter);
	
	return AllelesHunter;
}



/****************************************************************************
*
*       Function		initComponentList  
*       Parameters		AllelesHunter, curAllele
*		Output			none
*       Description		initiates first component node of curAllele in FounderAlleles
*       Comments		none
*
*****************************************************************************/
void initComponentList(allelesHunter* AllelesHunter,int curAllele)
{
	componentNode* head;
	
	sl_myMalloc(head,sizeof(componentNode));
	head->next = NULL;
	head->person = -1;
	AllelesHunter->FounderAlleles[curAllele].firstComponentPerson = head;
	AllelesHunter->FounderAlleles[curAllele].lastComponentPerson = head;
	AllelesHunter->FounderAlleles[curAllele].curComponentPerson = NULL;
}




/****************************************************************************
*
*       Function		addPersonToComponentList  
*       Parameters		AllelesHunter, curAllele, Person
*		Output			none
*       Description		adds node with Person to the end of "component" list
*						of curAllele in FounderAlleles
*       Comments		none
*
*****************************************************************************/
void addPersonToComponentList(allelesHunter* AllelesHunter,int curAllele,int Person)
{
	componentNode* nodePtr;

	if (Person >= 0)
	{
		sl_myMalloc(nodePtr,sizeof(componentNode));
		nodePtr->next = NULL;
		nodePtr->person = Person;
		AllelesHunter->FounderAlleles[curAllele].lastComponentPerson->next = nodePtr;
		AllelesHunter->FounderAlleles[curAllele].lastComponentPerson = nodePtr;
	}
}





/****************************************************************************
*
*       Function		resetCurComponentPerson  
*       Parameters		AllelesHunter, curAllele
*		Output			none
*       Description		sets the curComponentPerson to point to the first 
*						person in the component list
*       Comments		none
*
*****************************************************************************/
void resetCurComponentPerson(allelesHunter* AllelesHunter,int curAllele)
{
	AllelesHunter->FounderAlleles[curAllele].curComponentPerson = 
		AllelesHunter->FounderAlleles[curAllele].firstComponentPerson->next;
}

void resetCurComponentPerson2(fAllele* curFounderAllele)
{
	curFounderAllele->curComponentPerson = curFounderAllele->firstComponentPerson->next;
}




/****************************************************************************
*
*       Function		connectFounderAlleles  
*       Parameters		pedigree - is used in order to achieve founder 
*						alleles from Pedigree
*						AllelesHunter - pointer to main algorithm structure	
*		Output			none
*       Description		fills unordered founder alleles in array "People" 
*						(allele1,allele2) using Pedigree 
*       Comments		none
*
*****************************************************************************/
void connectFounderAlleles(ped_node** pedigree,allelesHunter* AllelesHunter)
{
	int curFounder;

	/*	for each founder in the array People:	*/
	for(curFounder = 0; curFounder < AllelesHunter->numFounders; curFounder++)
	{
		/*	updating "pointers" to FounderAlleles array	*/
		AllelesHunter->People[curFounder].founderAllele1 = 2*curFounder;
		AllelesHunter->People[curFounder].founderAllele2 = 2*curFounder+1;
		updatePersonAlleles(curFounder,pedigree,AllelesHunter);
	}
}



/****************************************************************************
*
*       Function		connectNonFounderAlleles  
*       Parameters		pedigree - is used in order to achieve unordered
*						nonfounder's alleles from Pedigree
*						AllelesHunter - pointer to main algorithm structure	
*		Output			none
*       Description		fills unordered nonfounder alleles in array "People" 
*						(allele1,allele2) using Pedigree 
*       Comments		none
*
*****************************************************************************/
void connectNonFounderAlleles(ped_node** pedigree,allelesHunter* AllelesHunter)
{
	int curNonFounder,size;
	Bool* firstChild;
	int vector;

	size = AllelesHunter->numFounders + AllelesHunter->numNonFounders;
	firstChild = FirstChildren(AllelesHunter->numFounders,size);
	vector = AllelesHunter->vector;
	/*	for each nonfounder in the array People:	*/
	for(curNonFounder = AllelesHunter->numFounders; curNonFounder < size; curNonFounder++)
	{
		vector = getParentsAlleles(vector,curNonFounder,pedigree,AllelesHunter,firstChild);
		updatePersonAlleles(curNonFounder,pedigree,AllelesHunter);
	}	
}



/****************************************************************************
*
*       Function		getParentsAlleles  
*       Parameters		vector
*						curNonFounder
*						pedigree
*						AllelesHunter
*						firstChild
*		Output			vector - changed inheritance vector 
*       Description		gets specific alleles by using first bit of inheritance vector
*       Comments		none
*
*****************************************************************************/
int getParentsAlleles(int vector,int curNonFounder,ped_node** pedigree,
					  allelesHunter* AllelesHunter,Bool* firstChild)
{
	int father, mother, *founderAllele;

	father = pedigree[curNonFounder]->father;
	founderAllele = &(AllelesHunter->People[curNonFounder].founderAllele1);
	/*	if curNonFounder is the first child of his father	*/
	if (firstChild[father] == true)
	{
		firstChild[father] = false;
		*founderAllele = AllelesHunter->People[father].founderAllele1;
	}
	/*	else we should look in given inheritance vector	*/
	else
	{
		vector = getAlleleByVector(vector,father, AllelesHunter,founderAllele);
	}
	mother = pedigree[curNonFounder]->mother;
	founderAllele = &(AllelesHunter->People[curNonFounder].founderAllele2);
	/*	if curNonFounder is the first child of his mother	*/
	if (firstChild[mother] == true)
	{
		firstChild[mother] = false;
		*founderAllele = AllelesHunter->People[mother].founderAllele1;
	}
	/*	else we should look in given inheritance vector	*/
	else
	{
		vector = getAlleleByVector(vector,mother, AllelesHunter,founderAllele);
	}
	return vector;
}



/****************************************************************************
*
*       Function		getAlleleByVector  
*       Parameters		vector
*						founderAllele - pointer to founderAllele
*						parent - index of the parent in People array
*						AllelesHunter
*		Output			vector - inheritance vector without first bit 
*       Description		gets specific allele by using first bit of inheritance vector
*       Comments		none
*
*****************************************************************************/
int getAlleleByVector(int vector,int parent,
					  allelesHunter* AllelesHunter,int* founderAllele)
{
	if (vector%2 == 0)
	{
		*founderAllele = AllelesHunter->People[parent].founderAllele1;
	}
	else
	{
		*founderAllele = AllelesHunter->People[parent].founderAllele2;
	}
	vector = vector>>1;
	return vector;
}



/****************************************************************************
*
*       Function		updatePersonAlleles  
*       Parameters		curPerson - his alleles will be initiated from Pedigree
*									if they are not 0
*						pedigree
*						AllelesHunter
*		Output			none
*       Description		initiates alleles of curPerson from Pedigree  
*       Comments		if they are not 0
*
*****************************************************************************/
void updatePersonAlleles(int curPerson,ped_node** pedigree,allelesHunter* AllelesHunter)
{
	Locus* allelesPair;
	Pedigree* Ped;
	pedigreePerson* personPtr;
	int fAllele1, fAllele2;

	Ped = AllPedigreesList->PedigeeGraph;
	personPtr = &(AllelesHunter->People[curPerson]);
	/*	getting unordered pair of current founder alleles from Ped	*/
	allelesPair = 
		&(Ped[pedigree[curPerson]->PedIndex].myLocus[AllelesHunter->locus+1]);
	/*	if the alleles are informative (!=0)	*/
	if ( (allelesPair->allele1 != 0) && (allelesPair->allele2 != 0) )
	{
		/*	copying alleles from Ped to People array	*/
		personPtr->allele1 = allelesPair->allele1;
		personPtr->allele2 = allelesPair->allele2;
		/*	adding curPerson to the end of "component" list of allele1 and allele2	*/
		fAllele1 = personPtr->founderAllele1;
		fAllele2 = personPtr->founderAllele2;
		addPersonToComponentList(AllelesHunter,fAllele1,curPerson);
		addPersonToComponentList(AllelesHunter,fAllele2,curPerson);
		/*	marking that current founder's alleles are in some component	*/
		AllelesHunter->FounderAlleles[fAllele1].includedInComponent = true;
		AllelesHunter->FounderAlleles[fAllele2].includedInComponent = true;
	}
}




/****************************************************************************
*
*       Function		destroyAllelesHunter  
*       Parameters		AllelesHunter
*		Output			none
*       Description		frees all the memory occupied by the AllelesHunter
*       Comments		none
*
*****************************************************************************/
void destroyAllelesHunter(allelesHunter* AllelesHunter)
{
	int i;
	fAllele* curAllele;

	if (AllelesHunter == NULL){
		return;
	}
	free(AllelesHunter->People);
	for (i=0; i < 2*(AllelesHunter->numFounders); i++)
	{
		curAllele = &(AllelesHunter->FounderAlleles[i]);
		while(curAllele->firstComponentPerson != NULL)
		{
			resetCurComponentPerson2(curAllele);
			curAllele->firstComponentPerson->next = NULL;
			free(curAllele->firstComponentPerson);
			curAllele->firstComponentPerson = curAllele->curComponentPerson;
		}
	}
	free(AllelesHunter->FounderAlleles);
}



