/* $Id: initAllelesHunter.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************
 ************** initAllelesHunter.h  ****************************************
 ****************************************************************************
 ************** created by Anisenia Andrei **********************************
 *	initiates AllelesHunter structure and destroys it						*
 ****************************************************************************/

#ifndef INIT_ALLELES_HUNTER_H
#define INIT_ALLELES_HUNTER_H


#include "initiatePedigree.h"



/*	for creating the array of 2*f founder alleles	*/
typedef struct component_Node
{
	int person;
	struct component_Node* next;
}componentNode;


typedef struct founder_allele
{
	Bool includedInComponent;
	componentNode* firstComponentPerson;
	componentNode* lastComponentPerson;
	componentNode* curComponentPerson;
	int tryFirstAllele;
	int trySecondAllele;
	Bool used;
	int index;
}fAllele;


/*	each person have 2 of founder alleles*/
typedef struct pedigree_person
{
	/*	ordered pair	*/
	int founderAllele1;
	int founderAllele2;
	/*	measured (unordered) pair	*/
	int allele1;
	int allele2;
	Bool used;
}pedigreePerson;


/*	structure which is used in the algorithm		*/
/*	for determining particular founder's alleles	*/
/*	given particular inheritance vector				*/
typedef struct alleles_hunter
{
	fAllele* FounderAlleles;
	pedigreePerson* People;
	int numFounders;
	int numNonFounders;
	int vector;
	int locus;
}allelesHunter;


/****************************************************************************
*
*       Function		initiateAllelesHunter  
*       Parameters		pedigree,numF,numN,locus  	
*		Output			allelesHunter
*       Description		initiates AllelesHunter
*       Comments		none
*
*****************************************************************************/
allelesHunter* initiateAllelesHunter(ped_node** pedigree,int numF,int numN,int locus);



/****************************************************************************
*
*       Function		initComponentList  
*       Parameters		AllelesHunter, curAllele
*		Output			none
*       Description		initiates first component node of curAllele in FounderAlleles
*       Comments		none
*
*****************************************************************************/
void initComponentList(allelesHunter* AllelesHunter,int curAllele);


/****************************************************************************
*
*       Function		addPersonToComponentList  
*       Parameters		AllelesHunter, curAllele, Person
*		Output			none
*       Description		adds node with Person to the end of "component" list
*						of curAllele in FounderAlleles
*       Comments		none
*
*****************************************************************************/
void addPersonToComponentList(allelesHunter* AllelesHunter,int curAllele,int Person);


/****************************************************************************
*
*       Function		resetCurComponentPerson  
*       Parameters		AllelesHunter, curAllele
*		Output			none
*       Description		sets the curComponentPerson to point to the first 
*						person in the component list
*       Comments		none
*
*****************************************************************************/
void resetCurComponentPerson(allelesHunter* AllelesHunter,int curAllele);

void resetCurComponentPerson2(fAllele* curFounderAllele);

/****************************************************************************
*
*       Function		connectFounderAlleles  
*       Parameters		pedigree - is used in order to achieve founder 
*						alleles from Pedigree
*						AllelesHunter - pointer to main algorithm structure	
*		Output			none
*       Description		fills unordered founder alleles in array "People" 
*						(allele1,allele2) using Pedigree 
*       Comments		none
*
*****************************************************************************/
void connectFounderAlleles(ped_node** pedigree,allelesHunter* AllelesHunter);



/****************************************************************************
*
*       Function		connectNonFounderAlleles  
*       Parameters		pedigree - is used in order to achieve founder 
*						alleles from Pedigree
*						AllelesHunter - pointer to main algorithm structure	
*		Output			none
*       Description		fills unordered founder alleles in array "People" 
*						(allele1,allele2) using Pedigree 
*       Comments		none
*
*****************************************************************************/
void connectNonFounderAlleles(ped_node** pedigree,allelesHunter* AllelesHunter);


/****************************************************************************
*
*       Function		getAlleleByVector  
*       Parameters		vector
*						founderAllele - pointer to founderAllele
*						parent - index of the parent in People array
*						AllelesHunter
*		Output			vector - inheritance vector without first bit 
*       Description		gets specific allele by using first bit of inheritance vector
*       Comments		none
*
*****************************************************************************/
int getAlleleByVector(int vector,int parent,
					  allelesHunter* AllelesHunter,int* founderAllele);



/****************************************************************************
*
*       Function		getParentsAlleles  
*       Parameters		vector
*						curNonFounder
*						pedigree
*						AllelesHunter
*						firstChild
*		Output			vector - changed inheritance vector 
*       Description		gets specific alleles by using first bit of inheritance vector
*       Comments		none
*
*****************************************************************************/
int getParentsAlleles(int vector,int curNonFounder,ped_node** pedigree,
					  allelesHunter* AllelesHunter,Bool* firstChild);


/****************************************************************************
*
*       Function		updatePersonAlleles  
*       Parameters		curPerson - his alleles will be initiated from Pedigree
*									if they are not 0
*						pedigree
*						AllelesHunter
*		Output			none
*       Description		initiates alleles of curPerson from Pedigree  
*       Comments		if they are not 0
*
*****************************************************************************/
void updatePersonAlleles(int curPerson,ped_node** pedigree,allelesHunter* AllelesHunter);



/****************************************************************************
*
*       Function		destroyAllelesHunter  
*       Parameters		AllelesHunter
*		Output			none
*       Description		frees all the memory occupied by the AllelesHunter
*       Comments		none
*
*****************************************************************************/
void destroyAllelesHunter(allelesHunter* AllelesHunter);


#endif


