/* $Id: initProductFromPedigree.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "gdata.h"
#include "fdata.h"
#include "allocfile.h"
#include "sl_Auxilary.h"
#include "../Product.h"

//	constants for number of variables and functions 
#define VARS_PER_FOUNDER				4
#define VARS_PER_NON_FOUNDER			6
#define FUNCS_PER_FOUNDER				4
#define FUNCS_PER_FOUNDER_MALE_SEX		3
#define FUNCS_PER_NON_FOUNDER			6
#define FUNCS_PER_NON_FOUNDER_MALE_SEX	4

#define TABLE_1_INDEX				1
#define TABLE_2_INDEX				2
#define TABLE_3_INDEX				3
#define TABLE_4_INDEX				4
#define TABLE_5_INDEX				5
#define TABLE_5_LITTLE_INDEX		6
#define TABLE_2_MALE_SEX_INDEX		7
#define TABLE_5_INDEX_SEXL			8
#define TABLE_5_LITTLE_INDEX_SEXL	9

#define HEALTHY_INDEX			0
#define AFFECTED_INDEX			1

#define TABLES_14_SIZE				2
#define TABLES_23_SIZE				16
#define TABLE_5_SIZE				8
#define TABLE_5_SIZE_LITTLE			4
#define TABLE_2_MALE_SEXL_SIZE		4
#define TABLE_5_SIZE_SEXL			4
#define TABLE_5_SIZE_LITTLE_SEXL	2

#define TABLES_14_NUM_VARS		1
#define TABLE_2_NUM_VARS		3
#define TABLE_2_SEXL_NUM_VARS	2
#define TABLE_3_NUM_VARS		4
#define TABLE_5_NUM_VARS		2
#define TABLE_5_NUM_VARS_LITTLE	1

//	functions for internal use only:
void allocateVariablesAndFunctions(product* prod, AllPedigrees* Ped, Bool sexLinked);

//	variable initiation functions
int* initiateVariables(product* prod, AllPedigrees* Ped, Bool sexLinked);
int initiatePersonVariables(int variableToTreat, variable* variables,
							Pedigree* pedigreePerson, Bool sexLinked);
void initiateVariable(variable* treatedVariable, Pedigree* pedigreePerson, 
					  NodeType type, NodeSex sex);

//	tables initiation functions
double* initiateTable1(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
double* initiateTable2(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
double* initiateTable3(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
double* initiateTable4(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
double* initiateTable5(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
double* initiateTable2MaleSexLinked(LOCUS_DATA* diseaseLocusData, 
									Locus* personDiseaseLocus);
double* initiateTable5MaleSexLinked(LOCUS_DATA* diseaseLocusData, 
									Locus* personDiseaseLocus);

//	BN-functions initiation functions
void initiateFunctions(product* prod, AllPedigrees* Ped, int* personVarsBlock, 
					   LOCUS_DATA* diseaseLocusData, Bool sexLinked);

void linkPersonFuncsToTables(Bool isFounder, function* funcPtr,
					LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
void linkMaleFuncsToTables(Bool isFounder, function* funcPtr,
						LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus);
void linkFuncToTable(function* funcPtr, int tableIndex, LOCUS_DATA* diseaseLocusData, 
					 Locus* personDiseaseLocus, 
					 double* (*createTable)(LOCUS_DATA*, Locus*), sex parentalMode);

int linkPersonFuncsToVariables(AllPedigrees* Ped, int currPersonIndex, 
				int firstFuncIndexOfCurrPerson, product* prod, 
				int* personVarsBlock, Bool sexLinked);
void linkFuncToVariables(AllPedigrees* Ped, int currPersonIndex, 
				function* funcPtr, product* prod, int* personVarsBlock);
void updateFuncFields(function* funcPtr, int tableSize, int numVars);

void linkFuncType1(function* funcPtr, variable* firstVariablePtr);
void linkFuncType2(function* funcPtr, variable* firstVariablePtr);
void linkFuncType2MaleSexLinked(function* funcPtr, variable* firstVariablePtr);
void linkFuncType3(AllPedigrees* Ped, int currPersonIndex, function* funcPtr, 
				   variable* variables, int* personVarsBlock);
void linkFuncType4(function* funcPtr, variable* firstVariablePtr);
void linkFuncType5(function* funcPtr, variable* firstVariablePtr);
void linkFuncType5MaleSexLinked(function* funcPtr, variable* firstVariablePtr);
void linkFuncType5LittleMaleSexLinked(function* funcPtr, variable* firstVariablePtr);
void linkFuncType5Little(function* funcPtr, variable* firstVariablePtr);
void setFuncVariablesUsed(function* funcPtr);
void deleteUnusedVariables(product* prod);
void copyUsedVariables(product* prod, variable* usedVariables);
void updateFuncVariables(product* prod, variable* usedVariables);
void linkVarsToFuncs(product* prod);
void reduceConstSelectors(AllPedigrees* PedPtr, product* prod, int* personVarsBlock);
void reduceConstSelector(variable* reducedSelector, int selectorValue);
double* initReducedTable3(int selectorValue);
double* initReducedTable4();
void treatFemaleSexLinked(variable* reducedSelector);


/******************************************************************************
*
*       Function        initProduct: the main function of this module
*       Parameters		PedPtr - pointer to GH pedigree
*						diseaseLocusData - disease locus specific information
*       Output          common Bayes Net product  
*       Description		initiates product using PedPtr
*       Comments		output product is input to InferEngine_Ido::CInferEngine
*
******************************************************************************/
product* initProduct(AllPedigrees* PedPtr, LOCUS_DATA* diseaseLocusData, 
					 Bool sexLinked)
{	
	product* prod;
	int* personVarsBlock;

	sl_myMalloc(prod, sizeof(product));
	//	alocating variables and funcions of the product:  
	allocateVariablesAndFunctions(prod, PedPtr, sexLinked);
	//	filling in the variables of the product
	personVarsBlock = initiateVariables(prod, PedPtr, sexLinked);
	//	filling in the functions of the product: 
	initiateFunctions(prod, PedPtr, personVarsBlock, diseaseLocusData, sexLinked);
	//	link each variable to functions it appears in 
	linkVarsToFuncs(prod);
	//	special treatment for const selectors: giving them only 1 value
	reduceConstSelectors(PedPtr, prod, personVarsBlock);
	//	deleting unused variables (variables Lf and Sf of males in sex linked case)
	deleteUnusedVariables(prod);
	return prod;
}


/******************************************************************************
*
*       Function        destroyProduct: the main destroy function of this module
*       Parameters		prod - product to destroy
*       Output          none  
*       Description		frees all structures of prod
*       Comments		for external use
*
******************************************************************************/
void destroyProduct(product* prod)
{
	int varIndex, funcIndex;

	for(varIndex = 0; varIndex < prod->numVariables; varIndex++)
	{
		free(prod->variables[varIndex].functions);
		free(prod->variables[varIndex].originalNode);
	}
	for(funcIndex = 0; funcIndex < prod->numFunctions; funcIndex++)
	{
		free(prod->functions[funcIndex].probabList);
		free(prod->functions[funcIndex].variables);
	}
	free(prod->variables);
	free(prod->functions);
	free(prod);
}


/******************************************************************************
*
*       Function        linkVarsToFuncs
*       Parameters		product
*       Output          none  
*       Description		link each variable to functions it appears in 
*       Comments		none
*
******************************************************************************/
void linkVarsToFuncs(product* prod)
{	
	int varIndex, funcIndex;
	function* funcPtr;

	//	initiations and allocations for each variable
	for(varIndex = 0; varIndex < prod->numVariables; varIndex++)
	{
		if (prod->variables[
			varIndex].used == FALSE)
		{
			prod->variables[varIndex].functions = NULL;
		}
		else
		{
			prod->variables[varIndex].currValue = 0;
			sl_myMalloc(prod->variables[varIndex].functions, 
				(prod->variables[varIndex].currNumFunctions)*sizeof(function*));
		}
	}
	//	go through all functions
	funcPtr = prod->functions;
	for(funcIndex = 0; funcIndex < prod->numFunctions; funcIndex++, funcPtr++)
	{
		//	for each function: link each it's variable to the function
		int numFuncVars = funcPtr->numVariables;
		for(varIndex = 0; varIndex < numFuncVars; varIndex++)
		{
			variable* varPtr = funcPtr->variables[varIndex];
			varPtr->functions[varPtr->currValue] = funcPtr;
			(varPtr->currValue)++;
		}
	}

}



/******************************************************************************
*
*       Function        reduceConstSelectors
*       Parameters		PedPtr, product, personVarsBlock
*       Output          none  
*       Description		special treatment for const selectors (reduced bits): 
*						fixing their value and updating T3 and T4 tables
*       Comments		none
*
******************************************************************************/
void reduceConstSelectors(AllPedigrees* PedPtr, product* prod, int* personVarsBlock)
{
	int personIndex;

	//	go through all people in the pedigree
	for(personIndex = 0; personIndex < PedPtr->size; personIndex++)
	{
		Pedigree* personPtr = PedPtr->PedigeeGraph + personIndex;
		//	treat only informative non-founders
		if (	(personPtr->informative) && (personPtr->founder == false) )
		{
			//	treating paternal bit of the person
			if (personPtr->paternalAllele->bitType == ReductionBit)
			{
				//	founder reduction: selector = 0
				reduceConstSelector
					(prod->variables + personVarsBlock[personIndex] + 4, 0);
			}
			if (personPtr->paternalAllele->bitType == ConstBit)
			{
				//	constant bit reduction: special case for sex linked
				treatFemaleSexLinked(prod->variables + personVarsBlock[personIndex] + 4);
				/*reduceConstSelector(prod->variables + personVarsBlock[personIndex] + 4,
					personPtr->paternalAllele->value);
				*/
			}
			//	treating maternal bit of the person
			if (personPtr->maternalAllele->bitType == ReductionBit)
			{
				//	founder reduction: selector = 0
				reduceConstSelector
					(prod->variables + personVarsBlock[personIndex] + 5, 0);
			}
			if (personPtr->maternalAllele->bitType == ConstBit)
			{
				//	constant bit reduction
				reduceConstSelector(prod->variables + personVarsBlock[personIndex] + 5,
					personPtr->maternalAllele->value);
			}
		}
	}
}


/******************************************************************************
*
*       Function        reduceConstSelector
*       Parameters		reducedSelector - selector to be reduced,
*						selectorValue - value of the selector
*       Output          none  
*       Description		fixing selector value and updating T3 and T4 tables
*       Comments		none
*
******************************************************************************/
void reduceConstSelector(variable* reducedSelector, int selectorValue)
{
	int funcIndex;
	//	updates for const selector:
	//	1. selector has only 1 value
	reducedSelector->numValues = 1;
	//	2. updates for the tables T3 and T4 of the selector
	for(funcIndex = 0; funcIndex < reducedSelector->currNumFunctions; funcIndex++)
	{
		function* funcPtr = reducedSelector->functions[funcIndex];
		//	choosing function to treat
		switch(funcPtr->baseInd)
		{
			case TABLE_3_INDEX:		//	updating table of type 3
									free(funcPtr->probabList);
									funcPtr->sizeOfTable = 8;
									funcPtr->probabList = 
										initReducedTable3(selectorValue);
									break;
			case TABLE_4_INDEX:		//	updating table of type 4
									free(funcPtr->probabList);
									funcPtr->sizeOfTable = 1;
									funcPtr->probabList = initReducedTable4();
									break;
			default:				//	error!!!
									printf("ERROR: No such table for selector!\n");
									return;
		}
	}
}

void treatFemaleSexLinked(variable* reducedSelector)
{
	int funcIndex;
	variable** variables;
	//	updates for const selector:
	//	1. selector has only 1 value
	reducedSelector->numValues = 1;
	//	2. updates for the tables T3 and T4 of the selector
	//	2. updates for the tables T3 and T4 of the selector
	for(funcIndex = 0; funcIndex < reducedSelector->currNumFunctions; funcIndex++)
	{
		function* funcPtr = reducedSelector->functions[funcIndex];
		//	choosing function to treat
		switch(funcPtr->baseInd)
		{
			case TABLE_3_INDEX:		//	updating table of type 3
									free(funcPtr->probabList);
									funcPtr->sizeOfTable = 4;
									sl_myMalloc(funcPtr->probabList, 4*sizeof(double));
									funcPtr->probabList[0] = funcPtr->probabList[3] = 1;
									funcPtr->probabList[1] = funcPtr->probabList[2] = 0;
									sl_myMalloc(variables, 3*sizeof(variable*));
									variables[0] = funcPtr->variables[0];
									variables[1] = funcPtr->variables[1];
									variables[2] = funcPtr->variables[3];
									funcPtr->variables[2]->used = FALSE;
									free(funcPtr->variables);
									funcPtr->variables = variables;
									funcPtr->numVariables = 3;
									break;
			case TABLE_4_INDEX:		//	updating table of type 4
									free(funcPtr->probabList);
									funcPtr->sizeOfTable = 1;
									funcPtr->probabList = initReducedTable4();
									break;
			default:				//	error!!!
									printf("ERROR: No such table for selector!\n");
									return;
		}
	}
}


//	help functions for "reduceConstSelector" function
double* initReducedTable3(int selectorValue)
{
	double* table5;

	sl_myMalloc(table5, 8*sizeof(double));
	if (selectorValue == 0)
	{
		table5[0] = table5[2] = table5[5] = table5[7] = 1.0;
		table5[1] = table5[3] = table5[4] = table5[6] = 0.0;
	}
	else
	{
		table5[0] = table5[1] = table5[6] = table5[7] = 1.0;
		table5[2] = table5[3] = table5[4] = table5[5] = 0.0;
	}
	return table5;
}

double* initReducedTable4()
{
	double* table4;
	sl_myMalloc(table4, sizeof(double));
	table4[0] = 1.0;
	return table4;
}



/******************************************************************************
*
*       Function        deleteUnusedVariables
*       Parameters		prod
*       Output          none  
*       Description		deletes unused (not linked to any function) variables
*       Comments		none
*
******************************************************************************/
void deleteUnusedVariables(product* prod)
{
	int numUsedVariables, variableIndex;
	variable* usedVariables;

	//	counting used variables
	numUsedVariables = 0;
	for(variableIndex = 0; variableIndex < prod->numVariables; variableIndex++)
	{
		if (prod->variables[variableIndex].used)
		{
			numUsedVariables++;
		}
	}
	//	allocating new array for used variables
	sl_myMalloc(usedVariables, numUsedVariables*sizeof(variable));
	//	copying used variables from "prod->variables" to "usedVariables" array
	copyUsedVariables(prod, usedVariables);
	//	rearranging functions pointers to used variables array
	updateFuncVariables(prod, usedVariables);
	//	updating product to include only used variables
	//	t.m., to pointer to usedVariables array
	prod->numVariables = numUsedVariables;
	free(prod->variables);
	prod->variables = usedVariables;
}


//*****************************************************************************
//	help functions for "deleteUnusedVariables" function:

//	copying used variables from "prod->variables" to "usedVariables" array
void copyUsedVariables(product* prod, variable* usedVariables)
{
	int prodVariableIndex, usedVariableIndex;

	usedVariableIndex = 0;
	for(prodVariableIndex = 0; prodVariableIndex < prod->numVariables; 
			prodVariableIndex++)
	{
		if (prod->variables[prodVariableIndex].used)
		{
			//	copying from product variable to used variable
			usedVariables[usedVariableIndex].originalNode = 
				prod->variables[prodVariableIndex].originalNode;
			usedVariables[usedVariableIndex].numValues = 
				prod->variables[prodVariableIndex].numValues;
			usedVariables[usedVariableIndex].functions = 
				prod->variables[prodVariableIndex].functions;
			//	linking product variable to used variable (using it's index)
			prod->variables[prodVariableIndex].currValue = usedVariableIndex;
			usedVariableIndex++;
		}
		else
		{
			//	deleting originalNode of unused variable
			free(prod->variables[prodVariableIndex].originalNode);
		}
	}
}


//	rearranging functions pointers to used variables array
void updateFuncVariables(product* prod, variable* usedVariables)
{
	int funcIndex, funcVarIndex, usedVarIndex;

	//	go through all functions of the product
	for(funcIndex = 0; funcIndex < prod->numFunctions; funcIndex++)
	{
		//	go through variables of the function
		for(funcVarIndex = 0; funcVarIndex < prod->functions[funcIndex].numVariables;
				funcVarIndex++)
		{
			//	pointing variable pointer to "used" variable (from usedVariables array)
			usedVarIndex = 
				prod->functions[funcIndex].variables[funcVarIndex]->currValue;
			prod->functions[funcIndex].variables[funcVarIndex] = 
												usedVariables + usedVarIndex;
				
		}
	}
}

//*****************************************************************************




/******************************************************************************
*
*       Function        initiateFunctions
*       Parameters		prod - Bayes Net product, ped - GH pedigree
*						personVarsBlock - array with indexes of variables blocks
*						start for each informative pedigree member
*						diseaseLocusData 
*       Output          none 
*       Description		initiated prod functions by linking appropriate tables
*						and variables to each function being built
*       Comments		none
*
******************************************************************************/
void initiateFunctions(product* prod, AllPedigrees* Ped, int* personVarsBlock, 
					   LOCUS_DATA* diseaseLocusData, Bool sexLinked)
{
	int currPersonIndex, firstFuncIndexOfCurrPerson;

	//	going through all informative persons in the pedigree
	firstFuncIndexOfCurrPerson = 0;
	for(currPersonIndex = 0; currPersonIndex < Ped->size; currPersonIndex++)
	{
		if (Ped->PedigeeGraph[currPersonIndex].informative)
		{
			//	linking person functions to appropriate tables 
			//	starting from the first function of the person
			if (sexLinked && (Ped->PedigeeGraph[currPersonIndex].sex == Male)) {
				linkMaleFuncsToTables(
					Ped->PedigeeGraph[currPersonIndex].founder,
					&(prod->functions[firstFuncIndexOfCurrPerson]),
					diseaseLocusData, 
					Ped->PedigeeGraph[currPersonIndex].myLocus);
			} else{
				linkPersonFuncsToTables(
					Ped->PedigeeGraph[currPersonIndex].founder,
					&(prod->functions[firstFuncIndexOfCurrPerson]),
					diseaseLocusData, 
					Ped->PedigeeGraph[currPersonIndex].myLocus);
			}
			//	linking person functions to appropriate variables
			//	starting from the first function of the person:
			firstFuncIndexOfCurrPerson = 
				linkPersonFuncsToVariables(Ped, currPersonIndex, 
				firstFuncIndexOfCurrPerson, prod, personVarsBlock, sexLinked);
		}
	}
}




/******************************************************************************
*
*       Function        linkPersonFuncsToTables
*       Parameters		isFounder - true or false
*						funcPtr - pointer to the first function of the person
*						tables, tablesUsed
*						diseaseLocus - for the current person
*       Output          none 
*       Description		links functions to appropriate tables
*       Comments		none
*
******************************************************************************/
void linkPersonFuncsToTables(Bool isFounder, function* funcPtr,
							 LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	//	1. linking Lf and Lm functions:
	if (isFounder)
	{
		//	linking function to table 1 (Lf paternal)
		linkFuncToTable(funcPtr + 0, TABLE_1_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable1, Male);
		//	linking function to table 1 (Lm maternal)
		linkFuncToTable(funcPtr + 1, TABLE_1_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable1, Female);
	}
	else
	{
		//	linking function to table 3 (Lf_f Lm_f Lf Sf paternal)
		linkFuncToTable(funcPtr + 0, TABLE_3_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable3, Male);
		//	linking function to table 3 (Lf_m Lm_m Lm Sm maternal)
		linkFuncToTable(funcPtr + 1, TABLE_3_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable3, Female);
	}

	//	2. linking GE function:
	linkFuncToTable(funcPtr + 2, TABLE_2_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable2, Male);

	//	3. linking PH function:
	if (personDiseaseLocus->allele1 == 0)
	{
		//	unknown affected status
		linkFuncToTable(funcPtr + 3, TABLE_5_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable5, Male);
	}
	else
	{
		//	known (healthy or affected) status
		linkFuncToTable(funcPtr + 3, TABLE_5_LITTLE_INDEX, diseaseLocusData, 
			personDiseaseLocus, initiateTable5, Male);
	}

	//	4. linking Sf and Sm functions (for non-founder)
	if (!isFounder)
	{
		//	linking function to table 4 (Sf paternal)
		linkFuncToTable(funcPtr + 4, TABLE_4_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable4, Male);
		//	linking function to table 4 (Sm maternal)
		linkFuncToTable(funcPtr + 5, TABLE_4_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable4, Female);
	}
}



/******************************************************************************
*
*       Function        linkMaleFuncsToTables
*       Parameters		isFounder - true or false
*						funcPtr - pointer to the first function of the person
*						tables, tablesUsed
*						diseaseLocus - for the current person
*       Output          none 
*       Description		links functions to appropriate tables of the male
*						in the case of sex-linked
*       Comments		none
*
******************************************************************************/
void linkMaleFuncsToTables(Bool isFounder, function* funcPtr,
						LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	//	1. linking Lm function:
	if (isFounder)
	{
		//	linking function to table 1 (Lm maternal)
		linkFuncToTable(funcPtr + 0, TABLE_1_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable1, Female);
	}
	else
	{
		//	linking function to table 3 (Lf_m Lm_m Lm Sm maternal)
		linkFuncToTable(funcPtr + 0, TABLE_3_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable3, Female);
	}
	//	2. linking GE function:
	linkFuncToTable(funcPtr + 1, TABLE_2_MALE_SEX_INDEX, diseaseLocusData, 
		personDiseaseLocus, initiateTable2MaleSexLinked, Male);

	//	3. linking PH function:
	if (personDiseaseLocus->allele1 == 0)
	{
		//	unknown affected status
		linkFuncToTable(funcPtr + 2, TABLE_5_INDEX_SEXL, diseaseLocusData, 
			personDiseaseLocus, initiateTable5MaleSexLinked, Male);
	}
	else
	{
		//	known (healthy or affected) status
		linkFuncToTable(funcPtr + 2, TABLE_5_LITTLE_INDEX_SEXL, diseaseLocusData, 
			personDiseaseLocus, initiateTable5MaleSexLinked, Male);
	}
	//	4. linking Sm function (for non-founder)
	if (!isFounder)
	{
		//	linking function to table 4 (Sm maternal)
		linkFuncToTable(funcPtr + 3, TABLE_4_INDEX, diseaseLocusData, personDiseaseLocus, 
					initiateTable4, Female);
	}
}


/******************************************************************************
*
*       Function        linkFuncToTable
*       Parameters		funcPtr,tableIndex,tables,tablesUsed,parentalMode
*       Output          none 
*       Description		links function to the table with table index = tableIndex
*       Comments		none
*
******************************************************************************/
void linkFuncToTable(function* funcPtr, int tableIndex, LOCUS_DATA* diseaseLocusData, 
					 Locus* personDiseaseLocus, 
					 double* (*createTable)(LOCUS_DATA*, Locus*), sex parentalMode)
{
	//	linking function to the specified table (with tableIndex)
	funcPtr->baseInd = tableIndex;
	funcPtr->probabList = createTable(diseaseLocusData, personDiseaseLocus);
	funcPtr->parentalMode = parentalMode;
}



/******************************************************************************
*
*       Function        linkPersonFuncsToVariables
*       Parameters		Ped - GH pedigree,
*						currPersonIndex - index of the treated person
*						firstFuncIndexOfCurrPerson
*						prod - BN product
*						personVarsBlock - array with indexes of variables blocks
*						start for each informative pedigree member
*       Output          updated firstFuncIndexOfCurrPerson (that means
*						index of the first function for the next person)
*       Description		links functions to appropriate variables of the person
*       Comments		none
*
******************************************************************************/
int linkPersonFuncsToVariables(AllPedigrees* Ped, int currPersonIndex, 
				int firstFuncIndexOfCurrPerson, product* prod, 
				int* personVarsBlock, Bool sexLinked)
{
	int funcIndex, numFuncs;
	function* funcPtr;

	//	setting the number of functions of current person
	if (Ped->PedigeeGraph[currPersonIndex].founder)
	{
		if (sexLinked && (Ped->PedigeeGraph[currPersonIndex].sex == Male)){
			numFuncs = FUNCS_PER_FOUNDER_MALE_SEX;
		} else {
			numFuncs = FUNCS_PER_FOUNDER;
		}
	}
	else
	{
		if (sexLinked && (Ped->PedigeeGraph[currPersonIndex].sex == Male)){
			numFuncs = FUNCS_PER_NON_FOUNDER_MALE_SEX;
		} else {
			numFuncs = FUNCS_PER_NON_FOUNDER;
		}
	}

	//	go through all functions starting "firstFuncIndexOfCurrPerson" 
	funcIndex = firstFuncIndexOfCurrPerson;
	funcPtr = prod->functions + funcIndex;
	for(; funcIndex < firstFuncIndexOfCurrPerson + numFuncs; funcIndex++, funcPtr++)
	{
		//	link current function to appropriate person variables
		linkFuncToVariables(Ped, currPersonIndex, funcPtr, prod, personVarsBlock);
		//	set all linked variables "used"
		setFuncVariablesUsed(funcPtr);
	}
	return funcIndex;
}



/******************************************************************************
*
*       Function        linkFuncToVariables
*       Parameters		Ped - GH pedigree,
*						currPersonIndex - index of the treated person
*						funcPtr - pointer to the currently treated function
*						prod - BN product
*						personVarsBlock - array with indexes of variables blocks
*						start for each informative pedigree member
*       Output          none
*       Description		links current function to appropriate variables of the person,
*						parses due to the type of the table of current function
*       Comments		none
*
******************************************************************************/
void linkFuncToVariables(AllPedigrees* Ped, int currPersonIndex, 
				function* funcPtr, product* prod, int* personVarsBlock)
{
	//	parser: choosing the type of function using baseInd
	switch (funcPtr->baseInd)
	{
		case TABLE_1_INDEX:	linkFuncType1(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_2_INDEX:	linkFuncType2(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_2_MALE_SEX_INDEX:
							linkFuncType2MaleSexLinked(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_3_INDEX:	linkFuncType3(Ped, currPersonIndex, 
								funcPtr, prod->variables, personVarsBlock);
							break;
		case TABLE_4_INDEX:	linkFuncType4(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_5_INDEX:	linkFuncType5(funcPtr, 
									prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_5_LITTLE_INDEX:
							linkFuncType5Little(funcPtr,
									prod->variables + personVarsBlock[currPersonIndex]);
		case TABLE_5_INDEX_SEXL:
							linkFuncType5MaleSexLinked(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		case TABLE_5_LITTLE_INDEX_SEXL:
							linkFuncType5LittleMaleSexLinked(funcPtr, 
								prod->variables + personVarsBlock[currPersonIndex]);
							break;
		default:			break;
	}
}


/******************************************************************************
*
*       Function        setFuncVariablesUsed
*       Parameters		funcPtr
*       Output          none
*       Description		sets all variables of funcPtr as "used"
*       Comments		none
*
******************************************************************************/
void setFuncVariablesUsed(function* funcPtr)
{
	int varIndex;

	for(varIndex = 0; varIndex < funcPtr->numVariables; varIndex++)
	{
		funcPtr->variables[varIndex]->used = TRUE;
		(funcPtr->variables[varIndex]->currNumFunctions)++;
	}
}



/******************************************************************************/
//	help functions for "linkFuncToVariables" function

//	updating and allocating necessary fields of function:
//	"sizeOfTable", "numVariables" and "variables" array
void updateFuncFields(function* funcPtr, int tableSize, int numVars)
{
	funcPtr->sizeOfTable = tableSize;
	funcPtr->numVariables = numVars;
	sl_myMalloc(funcPtr->variables, (funcPtr->numVariables)*sizeof(variable*));
}


//	linking function of type 1 to it's variables:
void linkFuncType1(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLES_14_SIZE, TABLES_14_NUM_VARS);
	//	linking function to it's variables:
	if (funcPtr->parentalMode == Male)
	{
		//	linking function to Lf variable of the person
		funcPtr->variables[0] = firstVariablePtr;
	}
	else
	{
		//	linking function to Lm variable of the person
		funcPtr->variables[0] = firstVariablePtr + 1;
	}
}


//	linking function of type 2 to it's variables:
void linkFuncType2(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLES_23_SIZE, TABLE_2_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to X variable of the person
	funcPtr->variables[0] = firstVariablePtr + 2;
	//	2. linking function to Lm variable of the person
	funcPtr->variables[1] = firstVariablePtr + 1;
	//	3. linking function to Lf variable of the person
	funcPtr->variables[2] = firstVariablePtr + 0;
}


//	linking function of type 2 to it's variables for male in the sex linked case:
void linkFuncType2MaleSexLinked(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLE_2_MALE_SEXL_SIZE, TABLE_2_SEXL_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to X variable of the person
	funcPtr->variables[0] = firstVariablePtr + 2;
	//	2. linking function to Lm variable of the person
	funcPtr->variables[1] = firstVariablePtr + 1;
}


//	linking function of type 3 to it's variables:
void linkFuncType3(AllPedigrees* Ped, int currPersonIndex, function* funcPtr, 
				   variable* variables, int* personVarsBlock)
{
	int parentIndex, parentVariablesStartIndex; 
	int locusVariableIndex, selectorVariableIndex;
	
	updateFuncFields(funcPtr, TABLES_23_SIZE, TABLE_3_NUM_VARS);
	//	linking function to it's variables
	if (funcPtr->parentalMode == Male)
	{
		//	finding father's variables block
		parentIndex = Ped->PedigeeGraph[currPersonIndex].father - 1;
		parentVariablesStartIndex = personVarsBlock[parentIndex];
		//	finding paternal locus and selector variable indexes of the person
		locusVariableIndex = personVarsBlock[currPersonIndex];
		selectorVariableIndex = locusVariableIndex + 4;
	}
	else
	{
		//	finding mother's variables block
		parentIndex = Ped->PedigeeGraph[currPersonIndex].mother - 1;
		parentVariablesStartIndex = personVarsBlock[parentIndex];
		//	finding maternal locus and selector variable indexes of the person
		locusVariableIndex = personVarsBlock[currPersonIndex] + 1;
		selectorVariableIndex = locusVariableIndex + 4;
	}
	//	linking function to it's variables:
	//	1. linking function to Lf/Lm variable of the person (due to parental mode)
	funcPtr->variables[0] = variables + locusVariableIndex;
	//	2. linking function to Lm variable of the person's parent
	funcPtr->variables[1] = variables + parentVariablesStartIndex + 1;
	//	3. linking function to Lf variable of the person's parent
	funcPtr->variables[2] = variables + parentVariablesStartIndex;
	//	4. linking function to Sf/Sm variable of the person (due to parental mode)
	funcPtr->variables[3] = variables + selectorVariableIndex;
}


//	linking function of type 4 to it's variables:
void linkFuncType4(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLES_14_SIZE, TABLES_14_NUM_VARS);
	//	linking function to it's variables:
	if (funcPtr->parentalMode == Male)
	{
		//	linking function to Sf variable of the person (non-founder)
		funcPtr->variables[0] = firstVariablePtr + 4;
	}
	else
	{
		//	linking function to Sm variable of the person (non-founder)
		funcPtr->variables[0] = firstVariablePtr + 5;
	}
}


//	linking function of type 5 to it's variables:
void linkFuncType5(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLE_5_SIZE, TABLE_5_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to Y variable of the person
	funcPtr->variables[0] = firstVariablePtr + 3;
	//	2. linking function to X variable of the person
	funcPtr->variables[1] = firstVariablePtr + 2;
}


//	linking function of type 5 to it's variables for male in the sex linked case:
void linkFuncType5MaleSexLinked(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLE_5_SIZE_SEXL, TABLE_5_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to Y variable of the person
	funcPtr->variables[0] = firstVariablePtr + 3;
	//	2. linking function to X variable of the person
	funcPtr->variables[1] = firstVariablePtr + 2;
}

//	linking function of type 5 little to it's variables:
void linkFuncType5Little(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLE_5_SIZE_LITTLE, TABLE_5_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to Y variable of the person
	funcPtr->variables[0] = firstVariablePtr + 3;
	//	2. linking function to X variable of the person
	funcPtr->variables[1] = firstVariablePtr + 2;
}

//	linking function of type 5 little to it's variables:
void linkFuncType5LittleMaleSexLinked(function* funcPtr, variable* firstVariablePtr)
{
	updateFuncFields(funcPtr, TABLE_5_SIZE_LITTLE_SEXL, TABLE_5_NUM_VARS);
	//	linking function to it's variables:
	//	1. linking function to Y variable of the person
	funcPtr->variables[0] = firstVariablePtr + 3;
	//	2. linking function to X variable of the person
	funcPtr->variables[1] = firstVariablePtr + 2;
}

/******************************************************************************/




/******************************************************************************
*
*       Function        allocateVariablesAndFunctions
*       Parameters		product, Pedigree
*       Output          none 
*       Description		initiates numVariables and numFunctions, and allocates
*						appropriate arrays of product
*       Comments		assuming: each founder needs 4 variables (Lf, Lm, X, Y)
*						and 5 tables (table for Lf, table for Lm, table for 
*						LfLmX, table for XY and table for Y);
*						each non-founder needs 6 variables (Lf,Lm,X,Y,Sf,Sm) and
*						7 tables (table for Sf, table for Sm, table for LfLmX,
*						table for XY, table for Y, table for LffLmfLfSf and table 
*						for LfmLmmLmSm)
*
******************************************************************************/
void allocateVariablesAndFunctions(product* prod, AllPedigrees* Ped, Bool sexLinked)
{
	int numInfMaleFounders, numInfFemaleFounders;
	int numInfMaleNonFounders, numInfFemaleNonFounders, currIndex;
	int numInfFounders, numInfNonFounders;

	//	counting informative founders and non-founders (males and females)
	numInfMaleFounders = numInfFemaleFounders = 0;
	numInfMaleNonFounders = numInfFemaleNonFounders = 0;
	for(currIndex = 0; currIndex < Ped->size; currIndex++) {
		if (Ped->PedigeeGraph[currIndex].informative) {
			if (Ped->PedigeeGraph[currIndex].founder) {
				//	adding founder to count
				if (Ped->PedigeeGraph[currIndex].sex == Male) {
					//	male founder
					numInfMaleFounders++;
				}else {
					//	female founder
					numInfFemaleFounders++;
				}
			}else {
				//	adding non-founder to count
				if (Ped->PedigeeGraph[currIndex].sex == Male) {
					//	male non-founder
					numInfMaleNonFounders++;
				}else {
					//	female non-founder
					numInfFemaleNonFounders++;
				}
			}
		}
	}
	numInfFounders = numInfMaleFounders + numInfFemaleFounders;
	numInfNonFounders = numInfMaleNonFounders + numInfFemaleNonFounders;
	//	updating number of variables and functions
	prod->numVariables = 
		VARS_PER_FOUNDER*numInfFounders + VARS_PER_NON_FOUNDER*numInfNonFounders;
	if (sexLinked)
	{
		prod->numFunctions = FUNCS_PER_FOUNDER*numInfFemaleFounders + 
							FUNCS_PER_NON_FOUNDER*numInfFemaleNonFounders +
							FUNCS_PER_FOUNDER_MALE_SEX*numInfMaleFounders +
							FUNCS_PER_NON_FOUNDER_MALE_SEX*numInfMaleNonFounders;
	}
	else
	{
		prod->numFunctions = 
		FUNCS_PER_FOUNDER*numInfFounders + FUNCS_PER_NON_FOUNDER*numInfNonFounders;
	}
	//	allocating "variables" and "functions"
	sl_myMalloc(prod->variables, (prod->numVariables)*sizeof(variable));
	sl_myMalloc(prod->functions, (prod->numFunctions)*sizeof(function));
}



/******************************************************************************
*
*       Function        initiateVariables
*       Parameters		product, Pedigree
*       Output          array (of size = num persons in the Pedigree) 
*						wlth indexes of the begining of variables block for 
*						each person
*       Description		fills the array of variables : for each person in the 
*						Pedigree the block of his variables is filled
*       Comments		none
*
******************************************************************************/
int* initiateVariables(product* prod, AllPedigrees* Ped, Bool sexLinked)
{
	int *PersonVarsBlock, currIndex, variableToTreat;

	sl_myMalloc(PersonVarsBlock, (Ped->size)*sizeof(int));
	//	go through all people in the Pedigree and fill his variables
	variableToTreat = 0;
	for(currIndex = 0; currIndex < Ped->size; currIndex++)
	{
		if (!(Ped->PedigeeGraph[currIndex].informative))
		{
			continue;
		}
		//	saving index of the first variable of the person in help array
		PersonVarsBlock[currIndex] = variableToTreat;
		//	initiating all variables of the person
		variableToTreat = initiatePersonVariables(variableToTreat, 
				prod->variables, &(Ped->PedigeeGraph[currIndex]), sexLinked);		
	}
	return PersonVarsBlock;
}



/******************************************************************************
*
*       Function        initiatePersonVariables
*       Parameters		variableToTreat - index of the treated variable
*						variables - array of variables in the product
*						pedigreePerson - currently treated person in the pedigree
*       Output          index of the next variable to be treated
*       Description		fills the variables for current person in variables array
*       Comments		none
*
******************************************************************************/
int initiatePersonVariables(int variableToTreat, variable* variables,
							Pedigree* pedigreePerson, Bool sexLinked)
{
	//	initiating common variables (for founders and non-founders):
	//	1. father locus variable of the person - Lf
	initiateVariable(variables + variableToTreat, 
		pedigreePerson, LOCUS, PATERNAL);
	//	2. mother locus variable of the person - Lm
	initiateVariable(variables + variableToTreat + 1, 
		pedigreePerson, LOCUS, MATERNAL);
	//	3. genotype variable of the person - X: treatimg sex linked case
	if ( sexLinked && (pedigreePerson->sex == Male) ) {
		initiateVariable(variables + variableToTreat + 2, 
			pedigreePerson, GENOTYPE_MALE_SEXL, 0);
	} else {
		initiateVariable(variables + variableToTreat + 2, 
			pedigreePerson, GENOTYPE, 0);
	}
	//	4. phenotype variable of the person - Y
	initiateVariable(variables + variableToTreat + 3, 
		pedigreePerson, PHENOTYPE, 0);
	variableToTreat += 4;
	
	//	initiating non-founders variables-selectors:
	if (!(pedigreePerson->founder))
	{
		//	1. paternal selector variable of the person - Sf
		initiateVariable(variables + variableToTreat, 
			pedigreePerson, SELECTOR, PATERNAL);
		//	2. maternal selector variable of the person - Sm
		initiateVariable(variables + variableToTreat + 1, 
			pedigreePerson, SELECTOR, MATERNAL);
		variableToTreat += 2;
	}

	return variableToTreat;
}



/******************************************************************************
*
*       Function        initiateVariable
*       Parameters		treatedVariable - pointer to the treated variable
*						personNum - index of the person in the Pedigree
*						type, sex
*       Output          none
*       Description		initiated person variable due to the received parameters
*       Comments		none
*
******************************************************************************/
void initiateVariable(variable* treatedVariable, Pedigree* pedigreePerson, 
					  NodeType type, NodeSex sex)
{
	char numberSting[5];

	sl_myMalloc(treatedVariable->originalNode, sizeof(node));
	treatedVariable->originalNode->pNum = pedigreePerson->index;
	treatedVariable->originalNode->nodeType = type;
	treatedVariable->originalNode->nodeSex = sex;
	treatedVariable->used = FALSE;
	treatedVariable->currNumFunctions = 0;
	switch(type)
	{
		case LOCUS:		treatedVariable->numValues = 2;
						if (sex == PATERNAL)
						{
							strcpy(treatedVariable->originalNode->name, "Lf_");
						}
						else
						{
							strcpy(treatedVariable->originalNode->name, "Lm_");
						}
						break;
		case GENOTYPE:	treatedVariable->numValues = 4;
						strcpy(treatedVariable->originalNode->name, "GE_");
						break;
		case GENOTYPE_MALE_SEXL:
						treatedVariable->numValues = 2;
						strcpy(treatedVariable->originalNode->name, "GE_");
						break;
		case PHENOTYPE:	if (pedigreePerson->myLocus->allele1 == 0)
						{
							treatedVariable->numValues = 2;
						}
						else
						{
							treatedVariable->numValues = 1;
						}
						strcpy(treatedVariable->originalNode->name, "PH_");
						break;
		case SELECTOR:	treatedVariable->numValues = 2;
						if (sex == PATERNAL)
						{
							strcpy(treatedVariable->originalNode->name, "Sf_");
						}
						else
						{
							strcpy(treatedVariable->originalNode->name, "Sm_");
						}
						break;
	}
	sprintf(treatedVariable->originalNode->name, "%s%d",
		treatedVariable->originalNode->name, pedigreePerson->index);
}



//	1. initiation of table T1
double* initiateTable1(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	double* table1;
	sl_myMalloc(table1, TABLES_14_SIZE*sizeof(double));
	table1[HEALTHY_INDEX] = diseaseLocusData->frequencies[0];
	table1[AFFECTED_INDEX] = diseaseLocusData->frequencies[1];
	return table1;
}

//	2. initiation of table T2
double* initiateTable2(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	double* table2;
	sl_myMalloc(table2, TABLES_23_SIZE*sizeof(double));
	memset(table2,0,TABLES_23_SIZE*sizeof(double));
	table2[0] = table2[5] = table2[10] = table2[15] = 1;
	return table2;
}

//	initiation of table 2 for male in the case of sex linked
double* initiateTable2MaleSexLinked(LOCUS_DATA* diseaseLocusData, 
									Locus* personDiseaseLocus)
{
	double* table2;
	sl_myMalloc(table2, TABLE_2_MALE_SEXL_SIZE*sizeof(double));
	memset(table2,0,TABLE_2_MALE_SEXL_SIZE*sizeof(double));
	table2[0] = table2[3] = 1;
	return table2;
}

//	3. initiation of table T3
double* initiateTable3(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	double* table3;
	sl_myMalloc(table3, TABLES_23_SIZE*sizeof(double));
	memset(table3,0,TABLES_23_SIZE*sizeof(double));
	table3[0] = table3[1] = table3[3] = table3[4] = 1;
	table3[10] = table3[13] = table3[14] = table3[15] = 1;
	return table3;
}

//	4. initiation of table T4
double* initiateTable4(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	double* table4;
	sl_myMalloc(table4, TABLES_14_SIZE*sizeof(double));
	table4[0] = table4[1] = 0.5;
	return table4;
}

//	5. initiation of table T5
double* initiateTable5(LOCUS_DATA* diseaseLocusData, Locus* personDiseaseLocus)
{
	double* table5;
	double HH_penetrance, HA_penetrance, AH_penetrance, AA_penetrance;
	int liabClass;

	liabClass = personDiseaseLocus->allele2 - 1;
	HH_penetrance = diseaseLocusData->penetrances[0][liabClass];
	AH_penetrance = diseaseLocusData->penetrances[1][liabClass];
	HA_penetrance = diseaseLocusData->penetrances[2][liabClass];
	AA_penetrance = diseaseLocusData->penetrances[3][liabClass];
	switch (personDiseaseLocus->allele1)
	{
		case 0:		//	status = unknown
					sl_myMalloc(table5, TABLE_5_SIZE*sizeof(double));
					table5[0] = 1 - HH_penetrance;
					table5[1] = 1 - HA_penetrance;
					table5[2] = 1 - AH_penetrance;
					table5[3] = 1 - AA_penetrance;
					table5[4] = HH_penetrance;
					table5[5] = HA_penetrance;
					table5[6] = AH_penetrance;	
					table5[7] = AA_penetrance;
					break;
		case 1:		//	status = healthy
					sl_myMalloc(table5, TABLE_5_SIZE_LITTLE*sizeof(double));
					table5[0] = 1 - HH_penetrance;
					table5[1] = 1 - HA_penetrance;
					table5[2] = 1 - AH_penetrance;	
					table5[3] = 1 - AA_penetrance;
					break;
		case 2:		//	status = affected
					sl_myMalloc(table5, TABLE_5_SIZE_LITTLE*sizeof(double));
					table5[0] = HH_penetrance;
					table5[1] = HA_penetrance;
					table5[2] = AH_penetrance;
					table5[3] = AA_penetrance;
					break;
		default:	break;
	}
	return table5;
}


//	5. initiation of table T5 for male in the case of sex linked
double* initiateTable5MaleSexLinked(LOCUS_DATA* diseaseLocusData, 
									Locus* personDiseaseLocus)
{
	double* table5;
	double H_penetrance, A_penetrance;
	int liabClass;

	liabClass = personDiseaseLocus->allele2 - 1;
	H_penetrance = diseaseLocusData->malePenatrences[0][liabClass];
	A_penetrance = diseaseLocusData->malePenatrences[1][liabClass];
	switch (personDiseaseLocus->allele1) {
		case 0:		//	status = unknown
					sl_myMalloc(table5, TABLE_5_SIZE_SEXL*sizeof(double));
					table5[0] = 1 - H_penetrance;
					table5[1] = 1 - A_penetrance;
					table5[2] = H_penetrance;
					table5[3] = A_penetrance;
					break;
		case 1:		//	status = healthy
					sl_myMalloc(table5, TABLE_5_SIZE_LITTLE_SEXL*sizeof(double));
					table5[0] = 1 - H_penetrance;
					table5[1] = 1 - A_penetrance;
					break;
		case 2:		//	status = affected
					sl_myMalloc(table5, TABLE_5_SIZE_LITTLE_SEXL*sizeof(double));
					table5[0] = H_penetrance;
					table5[1] = A_penetrance;
					break;
		default:	break;					
	}
	return table5;
}


/******************************************************************************/


