/* $Id: initiatePedigree.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  initiatePedigree.h									   					*/
/*  initiates pedigree structure	 									 	*/
/*  Author: Andrei Anisenia													*/
/****************************************************************************/
#ifndef INITIATE_PEDIGREE_H
#define INITIATE_PEDIGREE_H

#include <stdlib.h>
#include "gdata.h"
#include "sl_Auxilary.h"

typedef struct locus_t
{
	int allele1;
	int paternalInheritance;
    int allele2;
	int maternalInheritance;
}ped_locus;



/* For each human in pedigree we define his personal information in ped_node*/
typedef struct ped_node
{
	struct ped_node* next_af;	/* only for affected individuals:
								   pointer to the next affected individual
								   in the pedigree							*/
	int mother;					/* mother index	(starting 0, in founder -1)	*/
	int father;					/* father index	(starting 0, in founder -1)	*/
	Bool af;					/* false - not affected; true - affected	*/
	int allele1;				/* allele1 - from father; allele2 - from mother;		*/
	int allele2;				/* two of 2*f alleles {0,1,...,2*f-1}		*/
	Bool usefull;				/* shows if the person is usefull in the computation of NPL		
								   (meaning he is affected or has affected children)	*/
	ped_locus* myAlleles;		/* is used in haplotype calculation	*/
	int PedIndex;				/* index of the person in the main pedigree starting 0	*/
}ped_node;


/*	sorted pedigree structure (parents come before their children)	*/
typedef struct SortedPedigree_t
{
	int numF;
	int numN;
	ped_node** pedigree;
}SortedPedigree;



/****************************************************************************
*
*       Function		InitSortedPedigree
*       Parameters	  	numLoci - number of locuses
*       Output			initiated sorted pedgree    
*       Description		intiates in each node in "ped" the next fields:
*						"af", "mother" and "father" according to the 
*						information in Pedigree
*       Comments		none
*
*****************************************************************************/
SortedPedigree* InitSortedPedigree(int numLoci);


/****************************************************************************
*
*       Function		destroySortedPedigree
*       Parameters	  	sortedPed - pedigree to destroy	
*       Output			none
*       Description		deletes sortedPed
*       Comments		none
*
*****************************************************************************/
void destroySortedPedigree(SortedPedigree* sortedPed);



/****************************************************************************
*
*       Function		InitPed
*       Parameters	  	f - pointer to int (number of founders)
*						n - pointer to int (number of nonfounders)
*						numLoci - number of locuses
*       Output			initiated pedgree    
*       Description		intiates in each node in "ped" the next fields:
*						"af", "mother" and "father" according to the 
*						information in Pedigree
*       Comments		none
*
*****************************************************************************/
ped_node** InitPed(int* found, int* nonfound, int numLoci);


/****************************************************************************
*
*       Function		deleteUnlinkedFounders
*       Parameters	  	none	
*       Output			none
*       Description		deletes list of childless founders
*       Comments		none
*
*****************************************************************************/
void deleteUnlinkedFounders();



/****************************************************************************
*
*       Function		destroyPedigree
*       Parameters	  	ped - pedigree to destroy	
*       Output			none
*       Description		deletes ped
*       Comments		none
*
*****************************************************************************/
void destroyPedigree(ped_node** ped,int numF, int numN);


/****************************************************************************
*
*       Function		createFirstChildren  
*       Parameters		numF,size
*		Output			array firstChildren
*       Description		creates and initiates first children 
*       Comments		none
*
*****************************************************************************/
Bool* FirstChildren(int numF, int size);



/****************************************************************************
*
*       Function		reinitFirstChildren  
*       Parameters		firstChild - array to be full
*						numF,size
*		Output			none
*       Description		reinitiates first children 
*       Comments		none
*
*****************************************************************************/
void reinitFirstChildren(Bool* firstChild, int numF, int size);




/****************************************************************************
*
*       Function		initiatePowersOf2  
*       Parameters	  	size
*       Output			initiated array of size of powers of 2    
*       Description		initiates powersOf2 array
*       Comments		none
*
*****************************************************************************/
int* initiatePowersOf2(int size);



/****************************************************************************
*
*       Function		initiateFoundersIBDalleles  
*       Parameters	  	ped
*       Output			number of IBD alleles    
*       Description		parses IBD alleles between founders
*       Comments		none
*
*****************************************************************************/
int initiateFoundersIBDalleles(SortedPedigree* sortedPed);


/****************************************************************************
*
*       Function		initiateNonFoundersAlleles  
*       Parameters	  	ped , vectorIndex
*       Output			none    
*       Description		parses founder alleles between non-founders using 
*						given inheritance vector
*       Comments		none
*
*****************************************************************************/
void initiateNonFoundersAlleles(SortedPedigree* sortedPed, int vectorIndex);

// help function of "initiateNonFoundersAlleles"
int getParentalAllele(AlleleData* parentalBit, ped_node* parent, int* vector);



/****************************************************************************
*
*       Function		getInheritanceVector  
*       Parameters	  	vectorIndex
*       Output			none    
*       Description		getting vector from vector index
*       Comments		none
*
*****************************************************************************/
int* getInheritanceVector(int vectorIndex);


/****************************************************************************
*
*       Function		setInheritanceVector  
*       Parameters	  	vectorPtr - pointer to vector, vectorIndex
*       Output			none    
*       Description		getting vector from vector index and setting it to 
*						vectorPtr
*       Comments		assuming vectorPtr array's size is equal to 
*						"PedigreeData.SpeedUpNumOfMeioses"
*
*****************************************************************************/
void setInheritanceVector(int* vectorPtr, int vectorIndex);



#endif

