/* $Id: intersection.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  intersection.c											   				*/
/*  Finds possible alleles assignments for each vertex (founder allele) 	*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/

#include "fdata.h"
#include "gdata.h"
#include "stdio.h"
#include "memory.h"

void Intersection4Allele(int);
void BasicIntersect(int* , int* , Neighbours* );


/******************************************************************************
*
*       Function        Intersection
*       Parameters		none
*       Output          none
*       Description		Compute possible values for vertexes 
*						according to closest neigbours(not all sl_component)
*       Comments		-1  = no possible value
*						 0  = any value
*						>0  = specific value
*
******************************************************************************/
void Intersection(){
	int i;
	for(i = 0; i < 2*PedigreeData.NumOfNonFounders; i++){
			Intersection4Allele(i);
			// if both values are equal - it same as was only one value
			if (Alleles[i].A==Alleles[i].B)
				Alleles[i].B=-1;
		}
}


/******************************************************************************
*
*       Function        Intersection4Allele
*       Parameters		i - founder allele id
*       Output          none
*       Description		Compute possible values for vertex i
*       Comments		none
*
******************************************************************************/
void Intersection4Allele(int i){
	Neighbours * it;
	int A,B,currNeigh;
	// not connected to anyone - nothing known about founder 
	//any possible value
//	if(!FoundersGraph[i]){
	if (NumFounderNeighbours[i]==0){
		Alleles[i].B = 0;	
		Alleles[i].A = 0;
		return;
	}
	A = 0;
	B = 0;
//	for(it = FoundersGraph[i];it!=NULL;it=it->next){
	for(currNeigh=0;currNeigh<NumFounderNeighbours[i];currNeigh++){
		it = &(FoundersGraph[i][currNeigh]);
        BasicIntersect(&A, &B, it);
		Alleles[i].A=A;
        Alleles[i].B=B;
	}       
	
}

/******************************************************************************
*
*       Function        BasicIntersect
*       Parameters		A - first possible value
*						B - second possible value
*						allele - current edge
*       Output          none
*       Description		puts in A and B possible assignments for allele
*       Comments		none
*
*****************************************************************************/
void BasicIntersect(int* A, int* B, Neighbours* allele){
	if(!(*A) && !(*B)){
		*A = allele->allele1;
		*B = allele->allele2;
		return;
	}
    // neigbour can accept any value
	if(!allele->allele1 && !allele->allele2)
		return;
    // negbour can't accept any of possible values
	if (*A != allele->allele1 && 	*A != allele->allele2 )
		*A=-1;
	
	if (*B != allele->allele1 && 	*B != allele->allele2 )
		*B=-1;
   
	//only one possible value
	if ((*A == allele->allele1  && *B != allele->allele2)
		|| (*A == allele->allele2 && *B != allele->allele1 ))
		*B=-1;
}


/******************************************************************************
*
*       Function        removeDouble
*       Parameters		none
*       Output          none
*       Description		removes edges to itself
*       Comments		none
*
*****************************************************************************/
void removeDouble(){
	int i,currNeigh,realInd;
	for(i = 0; i < 2*PedigreeData.NumOfNonFounders; i++){
		for(currNeigh=0,realInd = 0;currNeigh<NumFounderNeighbours[i];currNeigh++){
			if (FoundersGraph[i][currNeigh].id != i){
				memcpy(&(FoundersGraph[i][realInd]),&(FoundersGraph[i][currNeigh]),sizeof(Neighbours));
				realInd++;
			}
		}
	}
}
