/* $Id: likehood.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  likehood.c											   					*/
/*  Finds likelihood and Lod score										 	*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#include "likehood.h"
#include "fdata.h"
#include "gdata.h"
#include "allocfile.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <time.h>
#include <limits.h>
#include <float.h>
#include "fft.h"
#include "programSpec.h"

#include "NPLstudents.h"

double sumVector(VECTOR Vector);

double *FactorLeft;
double *FactorRight;

VECTOR* LeftToRightEndProb;     // PTPTP..
VECTOR* RightToLeftEndProb;     // ....PTP

int *SpeedToRegularArrayCreation(int);
/******************************************************************************
*
*       Function        findFactor
*       Parameters		vector - array of values
*						size - size of array
*       Output          -log(factor)
*       Description		multiplicates array by factor, to prevent underflow
*       Comments		none
*
*****************************************************************************/
double findFactor(double *vector, int size){
	int k = 0;
	double factor = 0.0;
	
	for(k = 0; k < size;  k++){
		if (factor < vector[k]){
			factor = vector[k];
		}
	}
	
	for(k = 0; k < size; k++){
		vector[k] /= (factor);
	}
	factor = -log(factor);

	//	printf("factor = %lf\n",factor);
	return factor;
}



/******************************************************************************
*
*       Function        allocArray
*       Parameters		none
*       Output          double[NumOfLocuses][Possibilities]
*       Description		Alocates array (size=NumOfLocuses) of vectors (size=NumOfMeioses) 
*       Comments		none
*
*****************************************************************************/
VECTOR *allocArray(){
	int i;
	VECTOR* v;
	sl_myMalloc(v,(NumInfMarkers*sizeof(double*)));
	for (i=0;i < NumInfMarkers;++i){
		sl_myMalloc(v[i],(PedigreeData.Possibilities*sizeof(double)));		
	}
	return v;
}




/******************************************************************************
*
*       Function        fromLeft
*       Parameters		none
*       Output          array of multiplicationg
*       Description		saves computations of P*T* .. *T*P->
*       Comments		P * T * P * T .......* T * P 
*
*****************************************************************************/
double* fromLeft(double* leftPriorProbs, double* rightEndProbs, double* priorFactorLeft){
	int i;
	VECTOR LeftToRight;

	sl_myMalloc(LeftToRight, (PedigreeData.Possibilities*sizeof(double)));
	LeftToRightEndProb = allocArray();
	if (leftPriorProbs != NULL) {
		memcpy(LeftToRightEndProb[0], leftPriorProbs, sizeof(double)*PedigreeData.Possibilities);
	}
	else {
		memcpy(LeftToRightEndProb[0],ProbabilityLocus[MarkerOrder[0]],sizeof(double)*PedigreeData.Possibilities);
	}

	for (i=0;i < NumInfMarkers;i++){
		if  (i == NumInfMarkers - 1) {
			memcpy(rightEndProbs, LeftToRightEndProb[NumInfMarkers - 1], sizeof(double)*PedigreeData.Possibilities);
		}
		if (i==0)
			FactorLeft[i]=findFactor(LeftToRightEndProb[i],PedigreeData.Possibilities) + (*priorFactorLeft);
		else
			FactorLeft[i]=findFactor(LeftToRightEndProb[i],PedigreeData.Possibilities)+FactorLeft[i-1];
		if  (i == NumInfMarkers - 1)
			continue;
		*priorFactorLeft = FactorLeft[i];
		gh_calcFFT(LeftToRight,LeftToRightEndProb[i], Recomb_values[MarkerOrder[i]],PedigreeData.Possibilities);

		MultDiagonalMatrix(LeftToRight,ProbabilityLocus[MarkerOrder[i+1]],LeftToRightEndProb[i+1]);
	}
	free(LeftToRight);
}



/******************************************************************************
*
*       Function        fromLeft
*       Parameters		none
*       Output          array of multiplicationg
*       Description		saves computations of <-P*T* .. *T*P
*       Comments		P * T * P * T .......* T * P 
*
*****************************************************************************/
void fromRight(double* rightPriorProbs, double* rightToLeftEndProbs, double priorFactorRight){
	int i;
	VECTOR RightToLeft;

	sl_myMalloc(RightToLeft, (PedigreeData.Possibilities*sizeof(double)));

	sl_myMalloc(RightToLeftEndProb,(NumInfMarkers*sizeof(double*)));
	sl_myMalloc(RightToLeftEndProb[NumInformativeLoci-2],(PedigreeData.Possibilities*sizeof(double)));
	if (rightPriorProbs != NULL) {
		memcpy(RightToLeftEndProb[NumInformativeLoci-2], rightPriorProbs, sizeof(double)*PedigreeData.Possibilities);
	}
	else {
		memcpy(RightToLeftEndProb[NumInformativeLoci-2],ProbabilityLocus[MarkerOrder[NumInformativeLoci-2]],
							sizeof(double)*PedigreeData.Possibilities);
	}
	for (i = NumInfMarkers - 1; i >= 0; i--){

		if (i == NumInfMarkers - 1)
			FactorRight[i]=findFactor(RightToLeftEndProb[i],PedigreeData.Possibilities) + priorFactorRight;
		else
			FactorRight[i]=findFactor(RightToLeftEndProb[i],PedigreeData.Possibilities)+FactorRight[i+1];

		//	Perform backward step only if this is not the first locus (i > 0)
		if (i > 0) {
			sl_myMalloc(RightToLeftEndProb[i-1],(PedigreeData.Possibilities*sizeof(double)));
			//	If this is the last step and the result vector of the step already exists 
			//	(was passed as parameter "rightToLeftEndProbs") => copy it to RightToLeftEndProb without calculation
			if ((i == 1) && (rightToLeftEndProbs != NULL)) {
				memcpy(RightToLeftEndProb[i-1], rightToLeftEndProbs, sizeof(double)*PedigreeData.Possibilities);
			}
			//  else perform backward step calculation
			else {	
				gh_calcFFT(RightToLeft, RightToLeftEndProb[i] , Recomb_values[MarkerOrder[i-1]],PedigreeData.Possibilities);
				MultDiagonalMatrix(RightToLeft, ProbabilityLocus[MarkerOrder[i-1]],RightToLeftEndProb[i-1]);
			}
		}
	}
	free(RightToLeft);
}


/******************************************************************************
*
*       Function        likehood
*       Parameters		none
*       Output          none
*       Description		compute likelihood for all points
*       Comments		prints result to stdout
*
*****************************************************************************/
double* getVectorFFT(double *result, double *input, double theta, int size) {
	double* vector;
	if (theta == 0) {
		vector = input;
	}
	else{
		gh_calcFFT(result,input,theta,size);
		vector = result;
	}
	return vector;
}



/******************************************************************************
*
*       Function        likehood
*       Parameters		none
*       Output          none
*       Description		compute likelihood for all test points (forward-backward algorithm)
*       Comments		none
*
*****************************************************************************/
double* likelihood(ForwardPhaseIterationInput* iterationInput) {
	int i;
	TestPoint *sl_node;
	VECTOR Vector;
	VECTOR fromLeftTmp;
	VECTOR fromRightTmp;
	VECTOR rightEndProbs;
	double std,logPossibities;
	double *fromLeftRes, *fromRightRes;
	double sumProbs;
	double lnLike;

	/************************************************************************/
	/*added by Anisenia Andrei for computation of NPL score					*/
	double NPLSpairs;
	double NPLSall;
	double sum;
	/************************************************************************/

	sl_myMalloc(Vector, sizeof(double) * PedigreeData.Possibilities);
	sl_myMalloc(fromLeftRes,(sizeof(double)*PedigreeData.Possibilities));
	sl_myMalloc(fromRightRes,(sizeof(double)*PedigreeData.Possibilities));
	sl_myCalloc(FactorLeft,NumInformativeLoci,sizeof(double));
	sl_myCalloc(FactorRight,NumInfMarkers,sizeof(double));

	sl_myMalloc(rightEndProbs, sizeof(double)*PedigreeData.Possibilities);
	
	fromLeft(iterationInput->leftPriorProbs, rightEndProbs, &(iterationInput->FactorLeft));
	fromRight(iterationInput->rightPriorProbs, iterationInput->rightToLeftEndProbs, iterationInput->FactorRight);
	logPossibities=log(PedigreeData.Possibilities);

	//If this is the first locus in overall calculation => calculate the likelihood for No Linkage
	if (iterationInput->leftPriorProbs == NULL) {
		sumProbs = 0;
		for (i = 0; i < PedigreeData.Possibilities; i++) {
			sumProbs += RightToLeftEndProb[0][i];
		}
		iterationInput->stdUnlinked = log(iterationInput->sumDiseaseProbs) -FactorRight[0] + log(sumProbs)+
					PedigreeData.SpeedUpNumOfMeioses*log(0.5) - logPossibities + logMultFactor + logResultFactor ;
	}
	std = iterationInput->stdUnlinked;	
	LnUnlinked = std;

	//	Calculation of the test points
	numTestPoints = 0;
	for (i=0; i < sizeTestPointsArray; i++){
		sl_node = testPoints[i];
		while(sl_node != NULL){
			if ( i == 0 ){
				fromRightTmp = getVectorFFT(fromRightRes,RightToLeftEndProb[0], sl_node->theta2,PedigreeData.Possibilities);
				lnLike =
					log(VecMultVec(fromRightTmp,ProbabilityDesiase))-FactorRight[0]-logPossibities +  logMultFactor + logResultFactor;
				
				/********** added by Anisenia Andrei ***************************************/
				/**********	computation of NPL scores Spairs  and Sall *********************/
				sum = sumVector(fromRightTmp);
				NPLSpairs = VecMultVec(fromRightTmp,SpairsArray)/sum; 
				NPLSall = VecMultVec(fromRightTmp,SallArray)/sum;
				/***************************************************************************/

			}
			else if (i >= NumInformativeLoci-1 ){
				fromLeftTmp = getVectorFFT(fromLeftRes,LeftToRightEndProb[NumInformativeLoci-2], sl_node->theta1,PedigreeData.Possibilities);
				lnLike =
					log(VecMultVec(fromLeftTmp,ProbabilityDesiase))-FactorLeft[NumInformativeLoci-2]-logPossibities+  logMultFactor + logResultFactor;

				/********** added by Anisenia Andrei ***************************************/
				/**********	computation of NPL scores Spairs  and Sall *********************/
				sum = sumVector(fromLeftTmp);
				NPLSpairs = VecMultVec(fromLeftTmp,SpairsArray)/sum;
				NPLSall = VecMultVec(fromLeftTmp,SallArray)/sum;
				/***************************************************************************/

			}else{
				fromLeftTmp = 
					getVectorFFT(fromLeftRes,LeftToRightEndProb[i-1], sl_node->theta1,PedigreeData.Possibilities);
				fromRightTmp = 
					getVectorFFT(fromRightRes,RightToLeftEndProb[i], sl_node->theta2,PedigreeData.Possibilities);
				
				MultDiagonalMatrix(fromLeftTmp,ProbabilityDesiase,Vector);
				lnLike = log(VecMultVec(Vector,fromRightTmp))-FactorRight[i]-FactorLeft[i-1]-logPossibities +  logMultFactor + logResultFactor;

				/********** added by Anisenia Andrei ***************************************/
				/**********	computation of NPL scores Spairs  and Sall *********************/
				MultDiagonalMatrix(fromLeftTmp,fromRightTmp,Vector);
				sum = sumVector(Vector);
				NPLSpairs = VecMultVec(Vector,SpairsArray)/sum;
				NPLSall = VecMultVec(Vector,SallArray)/sum;		
				/***************************************************************************/
			}
			sl_node->LOD += (lnLike - std)/log(10);
			sl_node->ln_like += lnLike;

			/********** added by Anisenia Andrei ***************************************/
			/**********	saving NPL scores Spairs  and Sall in test points*********************/
			sl_node->NPL_Spairs = NPLSpairs;
			sl_node->NPL_Sall = NPLSall;
			/*********************************************************************************/

			sl_node = sl_node->next;
			numTestPoints++;
		}
	}

	free(fromLeftRes);
	free(fromRightRes);
	free(FactorRight);
	free(FactorLeft);
	free(Vector);
	freeAll(LeftToRightEndProb,RightToLeftEndProb);

	return rightEndProbs;
}



/******************************************************************************
*
*       Function        likelihoodBackward
*       Parameters		none
*       Output          result vector of backward HMM calculation
*       Description		performs backward HMM algorithm to get the resulted left vector 
*       Comments		none
*
*****************************************************************************/
double* likelihoodBackward(double* rightPriorProbs, double* priorFactorRight) {
	VECTOR RightToLeftTmp;
	VECTOR RightToLeftResult;
	int i;

	//	Allocations and initiations
	sl_myMalloc(RightToLeftTmp, (PedigreeData.Possibilities*sizeof(double)));
	sl_myMalloc(RightToLeftResult, (PedigreeData.Possibilities*sizeof(double)));
	if (rightPriorProbs != NULL) {
		memcpy(RightToLeftResult, rightPriorProbs, sizeof(double)*PedigreeData.Possibilities);
	}
	else {
		memcpy(RightToLeftResult, ProbabilityLocus[MarkerOrder[NumInformativeLoci-2]], 
			sizeof(double)*PedigreeData.Possibilities);
	}

	//	Backward algorithm 
	for (i = NumInfMarkers - 1; i > 0; i--){
		//	Normalizing RightToLeftResult vector using division by the maximal value in the vector
		*priorFactorRight = findFactor(RightToLeftResult,PedigreeData.Possibilities) + (*priorFactorRight);		
		//	One locus backward step calculation (with storing the result to RightToLeftTmp)
		gh_calcFFT(RightToLeftTmp, RightToLeftResult , Recomb_values[MarkerOrder[i-1]],PedigreeData.Possibilities);
		//	Multiplication between backward step result and locus "i - 1" probabilities  
		//	And storing the result of multiplication in RightToLeftResult
		MultDiagonalMatrix(RightToLeftTmp, ProbabilityLocus[MarkerOrder[i-1]], RightToLeftResult);
	}

	//findFactor(RightToLeftResult,PedigreeData.Possibilities);
	free(RightToLeftTmp);
	return RightToLeftResult;
}


/******************************************************************************
*
*       Function        sumVector
*       Parameters		Vector
*       Output          sum of probabilities in Vector
*       Description		help function 
*		Comments		none
*
*****************************************************************************/
double sumVector(VECTOR Vector)
{
	double sum;
	int i;
	
	sum = 0.00;
	for (i = 0; i < PedigreeData.Possibilities; i++)
		sum += Vector[i];
	return sum;
}


/******************************************************************************
*
*       Function        performLocusTransition
*       Parameters		result - vector with resulted probabilities
*						input - vector with input probabilities
*						theta - recombination value between 2 current loci
*						numVectors - number of inh. vectors (speed up number)
*       Output          none
*       Description		help function 
*		Comments		is used instead of "gh_calcFFT" function
*
*****************************************************************************/
void performLocusTransition(double* result, double* input,  double theta, int numVectors) {
	//	variables
	double* tmpArray;
	double* fromArrayPtr;
	double* toArrayPtr;
	double* changeTmpPtr;
	double notTheta;
	int step, range, stepSize, index1, index2, numMeioses, lowerBound;

	//	allocations and initiations
	notTheta = 1 - theta;
	sl_myMalloc(tmpArray, numVectors*sizeof(double));
	numMeioses = log(numVectors)/log(2);
	if (numMeioses%2 == 0) {
		memcpy(result,input,numVectors*sizeof(double));
		fromArrayPtr = result;
		toArrayPtr = tmpArray;
	}
	else {
		memcpy(tmpArray,input,numVectors*sizeof(double));
		fromArrayPtr = tmpArray;
		toArrayPtr = result;
	}

	//	go through all vector's bits (in O(numMeioses))
	for(step = 1, stepSize = 1, range = 2; step <= numMeioses; step++, stepSize *= 2, range *= 2) {
		//	go through all inh. vectors (in O(numVectors))
		for(index1 = 0; index1 < numVectors; index1++) {
			//	get the second index (index2)
			if (index1%range == 0) {
				lowerBound = index1;
			}
			index2 = (index1 - stepSize < lowerBound) ? (index1 + stepSize) : (index1 - stepSize);
			//	updating the transition (using arrays "fromArrayPtr" and "toArrayPtr")
			toArrayPtr[index1] = notTheta*fromArrayPtr[index1] + theta*fromArrayPtr[index2];
		}
		//	exchange pointers "fromArrayPtr" and "toArrayPtr"
		changeTmpPtr = fromArrayPtr;
		fromArrayPtr = toArrayPtr;
		toArrayPtr = changeTmpPtr;
	}

	free(tmpArray);
}