/* $Id: normalize.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  normalize.c											   					*/		
/*  Prepare data structures for fast caculation								*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/

#include "fdata.h"
#include "gdata.h"
#include <memory.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include "allocfile.h"
#include "sl_Auxilary.h"
#include "limits.h"




/******************************************************************************
*
*       Function        WeightOfVector
*       Parameters		index of vector (from 0 to 2^(2n) ) - unsigned int
*       Output          weight of the given vector (number of bits=1 in the vector)
*       Description		computes number of '1' in vector
*       Comments		none
*
******************************************************************************/
unsigned int WeightOfVector(unsigned int vector){

	unsigned int weight, maxReducedVector, currReducedVector, numShiftedBits;
	
	//	initiations
	weight = 0;
	maxReducedVector = PedigreeData.Possibilities - 1;
	numShiftedBits = PedigreeData.SpeedUpNumOfMeioses;
	
	//	count the number of bits
	while(vector != 0){
		//	in each iteration the number of bits=1 in the first (2n-f) bits of the given vector is counted
		//	and then vector is shifted right in (2n-f) bits
		currReducedVector = vector & maxReducedVector;
		weight += WeightArray[currReducedVector];
		vector = vector >> numShiftedBits;
	}

	return weight;
}


/******************************************************************************
*
*       Function        createWeightArray
*       Parameters		none
*       Output          none
*       Description		computes number of '1' in each number
*       Comments		none
*
******************************************************************************/
void createWeightArray(){
	int i;

	#if (WEIGHT_CALCULATION_TYPE == 1)
	/*	sl_myCalloc(WeightArray,PedigreeData.MaxPoss, sizeof(int));
		for (i=1;i<PedigreeData.MaxPoss;i++){
			if (i%2)
				WeightArray[i] = WeightArray[i-1]+1;
			else
				WeightArray[i] = WeightArray[i/2];
		}*/
	#endif

	#if (WEIGHT_CALCULATION_TYPE == 2)
		sl_myCalloc(WeightArray,PedigreeData.Possibilities, sizeof(int));
		for (i=1;i<PedigreeData.Possibilities;i++){
			if (i%2)
				WeightArray[i] = WeightArray[i-1]+1;
			else
				WeightArray[i] = WeightArray[i/2];
		}
	#endif
}


/******************************************************************************
*
*       Function        createNoSpeedUpDummies
*       Parameters		none
*       Output          none
*       Description		creates dummy arrays for program that runs without speedup
*       Comments		not in use
*
******************************************************************************/
void createNoSpeedUpDummies(){
	int i;

	sl_myCalloc(SpeedToRegularArray,PedigreeData.Possibilities,(sizeof(int)));
	sl_myCalloc(RegularToSpeedArray,PedigreeData.Possibilities,(sizeof(int)));
	sl_myCalloc(TransformSpeedTheta,PedigreeData.Possibilities,(sizeof(int)));
	sl_myCalloc(WeightArrayTransform,PedigreeData.Possibilities,(sizeof(int)));
	for (i=0;i<PedigreeData.Possibilities;i++){
		RegularToSpeedArray[i]=i;
		SpeedToRegularArray[i]=i;
		TransformSpeedTheta[i]=i;
		WeightArrayTransform[i]=i;
	}

}





/******************************************************************************
*
*       Function        SpeedToRegularArrayCreation
*       Parameters		size - 2^( 2*nonfounders - founders)
*       Output          array represeting transformation
*       Description		Transformation from Zn to Zn-foundersCount (Pi)
*						Performed by setting  first bit corresponding to i-th
*						founder to zero for all founders
*       Comments		Kruglyak and Lander 1998 p4
*
******************************************************************************/
int *SpeedToRegularArrayCreation(int  size){
	int i,j,firstGen,bit;

	sl_myCalloc(SpeedToRegularArray,(size),(sizeof(int)));

	
	/* itterate over all vectors */
	for (i=0;i < PedigreeData.Possibilities; i++){	

		firstGen = PedigreeData.NumOfFounders - 1 - PedigreeData.ChildlessFounders;
		bit = PedigreeData.SpeedUpNumOfMeioses-1;
		//vector = 0;
		/* Itterate over all bits in vector */
		for (j = PedigreeData.NumOfNonFounders -1  ; j >= 0; j--){
			if (PedigreeGraph[PedigreeData.NonFounders[j]].maternalAllele->bitType  == ReductionBit){
				SpeedToRegularArray[i]=(SpeedToRegularArray[i]*2);//+1
			}else if(PedigreeGraph[PedigreeData.NonFounders[j]].maternalAllele->bitType  == RegularBit){
				SpeedToRegularArray[i] =
					(SpeedToRegularArray[i]*2)+((i&MaskLSB[bit])>>bit);
				bit--;
			}
			//Set first meioses from founder to 0
			if (PedigreeGraph[PedigreeData.NonFounders[j]].paternalAllele->bitType  == ReductionBit){
				SpeedToRegularArray[i]=(SpeedToRegularArray[i]*2);//+1
			}else if(PedigreeGraph[PedigreeData.NonFounders[j]].paternalAllele->bitType  == RegularBit){
				SpeedToRegularArray[i] =
					(SpeedToRegularArray[i]*2)+((i&MaskLSB[bit])>>bit);
				bit--;
			}
		}
	}

	return SpeedToRegularArray;
}


/******************************************************************************
*
*       Function        RegularToSpeedArrayCreation
*       Parameters		size - 2^( 2*nonfounders )
*       Output          array represeting transformation
*       Description		Transformation from Zn-foundersCount to Zn. Reverse transmormation
*       Comments		Kruglyak and Lander 1998 p4
*
******************************************************************************
int *RegularToSpeedArrayCreation(int  size){
	int i,j,num;

	sl_myCalloc(RegularToSpeedArray,(size),(sizeof(int)));

	/* Where no changes are needed *
	for (i = 0;i < PedigreeData.Possibilities; i++){
		num = SpeedToRegularArray[i];
		RegularToSpeedArray[num] = i;
	}

	for (i = 0; i < PedigreeData.MaxPoss; i++){
		// all founders bits are zero, already take care.
		if (((i&PedigreeData.FoundersBitOneMask) == 0))
			continue;
		/* set num to regular number *
		num=i;
		for (j=0;j<PedigreeData.NumOfFounders;j++){
			/* of founders first child bit is 1, invert all other bits of its children *
			if (i & FirstFoundersChild[j])
				num=num^FoundersChildren[j];
		}
		RegularToSpeedArray[i] = RegularToSpeedArray[num];
	
	}

	return RegularToSpeedArray;
}*/




/******************************************************************************
*
*       Function        findParity
*       Parameters		num - input number
*       Output          1 if even
*       Description		return 1 if number of 1 in binary representation of num is even 
*       Comments		none
*
******************************************************************************/
int findParity(int num){
	int count=0;
	while(num>0){
		count+=(num%2);
		num/=2;
	}
	return count%2;
}



/******************************************************************************
*
*       Function        linearSpeedTransform
*       Parameters		size - array size
*       Output          array represeting transformation
*       Description		PiWave transformation
*       Comments		Kruglyak and Lander 1998 p4
*
******************************************************************************/
int *linearSpeedTransform(int size){
	int i,j,num;
	sl_myMalloc(TransformSpeedTheta,(sizeof(int)*size));
	for (i=0;i<size;i++){
		num=SpeedToRegularArray[i];
		for (j=0;j<PedigreeData.NumOfFounders;j++){
			
			#if (WEIGHT_CALCULATION_TYPE == 1)
				if (WeightArray[SpeedToRegularArray[i] & FoundersChildren[j]] % 2)
					num=num^FirstFoundersChild[j];
			#endif

			#if (WEIGHT_CALCULATION_TYPE == 2)
				if (WeightOfVector(SpeedToRegularArray[i] & FoundersChildren[j]) % 2)
					num=num^FirstFoundersChild[j];
			#endif
		}
		TransformSpeedTheta[i]=num;	
	}
	return TransformSpeedTheta;
}


/******************************************************************************
*
*       Function        createSpeedWeightTransform
*       Parameters		size - array size
*       Output          array represeting transformation
*       Description		weight of PiWave transformed number
*       Comments		none
*
******************************************************************************/
int *createSpeedWeightTransform(int size){
	int i;

	sl_myMalloc(WeightArrayTransform,(sizeof(int)*size));
	for (i=0;i<size;i++){
		#if (WEIGHT_CALCULATION_TYPE == 1)
			WeightArrayTransform[i]=WeightArray[ TransformSpeedTheta[i]];
		#endif

		#if (WEIGHT_CALCULATION_TYPE == 2)
			WeightArrayTransform[i]=WeightOfVector(TransformSpeedTheta[i]);
		#endif
	}
	return  WeightArrayTransform;
}



/******************************************************************************
*
*       Function        normalizeInput
*       Parameters		none
*       Output          none
*       Description		calculates and indexes founders and nonfounders
*       Comments		none
*
******************************************************************************/
void normalizeInput(){

		LOCUS_DATA *locPtr, *locPtrEnd;
	double *freqPtr, *freqPtrEnd;
	//PERSON_DATA *personPtr;
	/*NumOfFounders=0;
	NumOfNonFounders=0;
	for (pedPtr = PedigreeGraph, pedPtrEnd =PedigreeGraph + PedigreeSize;pedPtr < pedPtrEnd; pedPtr++){
		if ((pedPtr->father == 0) && (pedPtr->mother == 0)){
			pedPtr->num=NumOfFounders++;
			pedPtr->founder=1;

		}
		else {
			pedPtr->num=NumOfNonFounders++;
			pedPtr->founder=0;			
		}
	}*/

	for (locPtr = lociDataArray, locPtrEnd = lociDataArray + NumOfLocuses; locPtr < locPtrEnd; locPtr++){
		if ( locPtr->alleleFreqDifFromOne == 0)
			continue;
		for ( freqPtr = locPtr->frequencies , freqPtrEnd = freqPtr + locPtr->num_of_freq;
			freqPtr < freqPtrEnd ;freqPtr++){
				
			(*freqPtr) += (locPtr->alleleFreqDifFromOne)/locPtr->num_of_freq;
				
			}
	}


}

/******************************************************************************
*
*       Function        prepareInput
*       Parameters		none
*       Output          none
*       Description		converts from file representation to internal structures
*       Comments		none
*
******************************************************************************/
void prepareInput(){
	int i, foundersCount, nonFoundersCount, firstGen, bitId;
	int child;
	int *tmpFounders;

	/*	added by Anisenia Andrei*************************************/
	UnlinkedFounder* last;
	/***************************************************************/

	int numOfFemaleFounders = 0;
	int noSpeedUpIndex = 0;
	int childlessMaleFounders =0;
	PedigreeData.FoundersBitOneMask = 0;

	/*	added by Anisenia Andrei*************************************/
	getChidrenForParents(AllPedigreesList->PedigeeGraph, AllPedigreesList->size);
	sortChildrenIncreasingIndexes();
	/*	initiating linked list of childless founders				*/ 	
	last = (UnlinkedFounder*)malloc(sizeof(UnlinkedFounder));
	last->next = NULL;
	last->founder = -1;
	unlinkedFounders = last;
	/***************************************************************/


	sl_myMalloc(tmpFounders,(sizeof(int)*PedigreeData.NumOfFounders));
	sl_myMalloc(PedigreeData.NonFounders,(sizeof(int)*PedigreeData.NumOfNonFounders));
	sl_myMalloc (PedigreeData.Founders,(sizeof(int)*PedigreeData.NumOfFounders));
	sl_myCalloc(FirstFoundersChild,PedigreeData.NumOfFounders, sizeof(int));


	/* init counters */
	foundersCount=0;
	nonFoundersCount=0;
	firstGen=0;
	PedigreeData.ChildlessFounders = 0;

	
	/* Using temp filed for marking if allready visit this person */
	for (i = 0; i < PedigreeData.PedigreeSize; i++){
		if (PedigreeGraph[i].informative == false){
			continue;
		}
		if (PedigreeGraph[i].founder){
			tmpFounders[foundersCount++]=i;
			PedigreeGraph[i].temp = true;
			if (PedigreeGraph[i].sex == Female )
				numOfFemaleFounders++;
		}else{
			if (!ProgramSpecificData.sexLinked){
				PedigreeGraph[i].paternalAllele->noSpeedUpNumber = noSpeedUpIndex++;
			}
			PedigreeGraph[i].maternalAllele->noSpeedUpNumber = noSpeedUpIndex++;

		}
	}


	nonFoundersCount =0;
	firstGen =0;


	for (i=0; i < PedigreeData.NumOfFounders; i++){
		child = PedigreeGraph[tmpFounders[i]].child;
  		while(child != 0 && PedigreeGraph[ child -1].informative == false){
			if (	PedigreeGraph[tmpFounders[i]].sex == Male){
				child = PedigreeGraph[child - 1].NeSiPa;
			}else{
				child = PedigreeGraph[child - 1].NeSiMa;
			}
		}
		if (child != 0){
			if (PedigreeGraph[tmpFounders[i]].sex == Male){
				
				if (!ProgramSpecificData.sexLinked){
					PedigreeGraph[child - 1].paternalAllele->bitType = ReductionBit;
					PedigreeGraph[child - 1].paternalAllele->value = 0;
					PedigreeGraph[child - 1].paternalAllele->number = firstGen++;

				}else{
					PedigreeGraph[tmpFounders[i]].paternalAllele->bitType = NotUsedBit;
				}
			}else{
				PedigreeGraph[child - 1].maternalAllele->bitType = ReductionBit;
				PedigreeGraph[child - 1].maternalAllele->value = 0;
				PedigreeGraph[child - 1].maternalAllele->number = firstGen++;
			}
			PedigreeGraph[tmpFounders[i]].num = i - PedigreeData.ChildlessFounders -childlessMaleFounders ;
		}
		else{
			printf ("Unconnected founder  %d\n",(tmpFounders[i]+1));

			/*	added by Anisenia Andrei*************************************/
			/*	adding index of unconnected founder to "unlinkedFounders" list  */
			last->next = (UnlinkedFounder*)malloc(sizeof(UnlinkedFounder));
			last = last->next;
			last->next = NULL;
			last->founder = tmpFounders[i];
			/****************************************************************/

			PedigreeGraph[tmpFounders[i]].num = PedigreeData.NumOfFounders - PedigreeData.ChildlessFounders - 1 - childlessMaleFounders;
			if (!ProgramSpecificData.sexLinked || PedigreeGraph[tmpFounders[i]].sex == Female)
				PedigreeData.ChildlessFounders++;
			if (ProgramSpecificData.sexLinked &&  PedigreeGraph[tmpFounders[i]].sex == Male)
				childlessMaleFounders++;
			
		}
		PedigreeData.Founders[ PedigreeGraph[tmpFounders[i]].num] = tmpFounders[i];
	}
#if DEBUG_MEMORY == 1
#ifdef WIN32
	_ASSERT(_CrtCheckMemory());
#endif
#endif

	
	PedigreeData.NumOfMeioses = (ProgramSpecificData.sexLinked )?PedigreeData.NumOfNonFounders:2*PedigreeData.NumOfNonFounders; 

	//PedigreeData.MaxPoss=1<<(PedigreeData.NumOfMeioses); 
#if (SpeedUp==0)
	PedigreeData.SpeedUpNumOfMeioses=PedigreeData.NumOfMeioses; // no speedUp 
#else //Speed Up
	PedigreeData.SpeedUpNumOfMeioses=(ProgramSpecificData.sexLinked)?
		PedigreeData.NumOfMeioses - numOfFemaleFounders + PedigreeData.ChildlessFounders 
		:PedigreeData.NumOfMeioses- PedigreeData.NumOfFounders + PedigreeData.ChildlessFounders; 

#endif

	// Find all nonFounders and numirate their bits
	if (ProgramSpecificData.sexLinked){
		for ( i = 0; i < PedigreeData.PedigreeSize; i++){
			if (!PedigreeGraph[i].founder){
				if (PedigreeGraph[i].sex == Female){
					PedigreeGraph[i].paternalAllele->bitType = ConstBit;
					PedigreeGraph[i].paternalAllele->value = 1;
					PedigreeGraph[i].paternalAllele->number = firstGen++;
				}else{
					PedigreeGraph[i].paternalAllele->bitType = NotUsedBit;
				}
			}
		}
	}
	nonFoundersCount = 0;
	bitId =0;
	
	while (nonFoundersCount < PedigreeData.NumOfNonFounders){

		for (i = 0; i < PedigreeData.PedigreeSize; i++){
			if (PedigreeGraph[i].informative == false || PedigreeGraph[i].founder 
				|| PedigreeGraph[i].temp == true){
				continue;
			}
			if ( PedigreeGraph[PedigreeGraph[i].father - 1].temp == true && 
				PedigreeGraph[PedigreeGraph[i].mother - 1].temp == true) {
					PedigreeGraph[i].num = nonFoundersCount;
					if (PedigreeGraph[i].paternalAllele->bitType == RegularBit){
						PedigreeGraph[i].paternalAllele->number = bitId++;

					}else{
						if (PedigreeGraph[i].paternalAllele->bitType == ReductionBit){
							foundersCount =  PedigreeGraph[PedigreeGraph[i].father - 1].num;
							FirstFoundersChild[foundersCount] = 1<< PedigreeGraph[i].paternalAllele->noSpeedUpNumber;
							PedigreeData.FoundersBitOneMask += FirstFoundersChild[foundersCount];
						}
					}
					if (PedigreeGraph[i].maternalAllele->bitType == RegularBit){

						PedigreeGraph[i].maternalAllele->number = bitId++;

					}else{
						foundersCount =  PedigreeGraph[PedigreeGraph[i].mother - 1].num;
						FirstFoundersChild[foundersCount] = 1<< PedigreeGraph[i].maternalAllele->noSpeedUpNumber;
						PedigreeData.FoundersBitOneMask += FirstFoundersChild[foundersCount];
					}

					PedigreeData.NonFounders[nonFoundersCount++]=i;

					PedigreeGraph[i].temp = true;
				}
		}
	}
#if DEBUG_MEMORY == 1
#ifdef WIN32
	_ASSERT(_CrtCheckMemory());
#endif
#endif


	PedigreeData.Possibilities=1<<(PedigreeData.SpeedUpNumOfMeioses);
	PedigreeData.NumFoundersAlleles = (ProgramSpecificData.sexLinked)?
		2*numOfFemaleFounders + (PedigreeData.NumOfFounders-numOfFemaleFounders):
	2*(PedigreeData.NumOfFounders);
	//2*(numOfFemaleFounders-PedigreeData.ChildlessFounders) + (PedigreeData.NumOfFounders-numOfFemaleFounders):
	//2*(PedigreeData.NumOfFounders- PedigreeData.ChildlessFounders);
#if DEBUG_MEMORY == 1
#ifdef WIN32
	_ASSERT(_CrtCheckMemory());
#endif
#endif

	PedigreeData.ChildlessFounders+= childlessMaleFounders;
	free(tmpFounders);

	/*	added by Anisenia Andrei*******************************************/
	/*  freeing the first struct in the list (helper struct for initiation)*/
	last = unlinkedFounders;
	unlinkedFounders = unlinkedFounders->next;
	last->next = NULL;
	free(last);
	/**********************************************************************/
}



void finishPrepaireInput() {
	int* foundersChidren;
	
	initMaskLSB();
	#if (ALLELES_INH_USE == 0) 
		//	initiate int** AlleleInh, initiate and fill FoundersChildren array
		initAlleles(); 
	#else 
		//	initiate AllelesInheritance* allelesInhObj
		allelesInhObj =  createAllelesInheritance(&PedigreeData, PedigreeGraph);
		//	initiate and fill FoundersChildren array
		FoundersChildren = initFoundersChildren(&PedigreeData, PedigreeGraph);		
	#endif
	
	if (SpeedUp){					 	
		SpeedToRegularArrayCreation(PedigreeData.Possibilities);
		/*#if  DISEASE == 0
			RegularToSpeedArrayCreation(PedigreeData.MaxPoss);
		#endif*/
	}
	else{
		createNoSpeedUpDummies();
	}
	
	#if DISEASE != 3
		ProbabilityDesiase=allocVector();	
	#endif
}

