/* $Id: programSpec.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  programSpec.c											   				*/		
/*  Saves programs specific information and prints output	    			*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#include "programSpec.h"
#include <string.h>
#include <stdio.h>
#include <math.h>
#include <memory.h>
#include <float.h>
#include "gdata.h"
//#include "..\Globals.h"
//#include "..\PSGraph.h"


double *calculateMarkersDist(int startPoint, int endPoint);
void createTestPointsMinusS(double sl_step, CreateTestPointsInput* input);
void createTestPointsMinusN(int numIter, CreateTestPointsInput* input);



/******************************************************************************
*
*       Function        printTestPoints
*       Parameters      none
*       Output          none
*       Description		prints LOD and likelihood for each point. finds maximum
*       Comments		none
*
******************************************************************************/
TestPoint* printTestPoints(FILE *resFile){
	int i;
	TestPoint *sl_node,*maxNode = NULL;
	
	for (i = 0; i < sizeTestPointsArray; i++){

		sl_node = testPoints[i];

		if (sl_node != NULL && i != 0 && i != NumInformativeLoci){
			printf("  %s\n",lociDataArray[MarkerOrder[i-1]+1].locusName);
			fprintf(resFile,"  %s\n",lociDataArray[MarkerOrder[i-1]+1].locusName);		
		}

		while(sl_node != NULL){
			if ( sl_node->ln_like > -DBL_MAX){
				printf("\t\t%.6lf\t%.6lf\t     %.6lf\t  ",sl_node->distFromFirstWindowMarker,sl_node->ln_like,sl_node->LOD);
				fprintf(resFile,"\t\t%.6lf\t\t  %.2lf\t\t%.6lf\t",sl_node->distFromFirstWindowMarker,sl_node->ln_like,sl_node->LOD);
			}else{
				printf("\t\t%lf\t-Infinity\t-Infinity\t",sl_node->distFromFirstWindowMarker);
				fprintf(resFile,"\t\t%lf\t\t-Infinity\t-Infinity\t",sl_node->distFromFirstWindowMarker);
			}
			/********** added by Anisenia Andrei ***************************************/
			/*  printing NPL scores Spairs and Sall	********************************/
			printf("%.6lf   ",sl_node->NPL_Spairs);
			fprintf(resFile,"\t%.6lf\t",sl_node->NPL_Spairs);
			
			printf("%.6lf\n",sl_node->NPL_Sall);
			fprintf(resFile,"%.6lf\n",sl_node->NPL_Sall);
			/***********************************************************************/
			if (maxNode == NULL){
				maxNode = sl_node;
			}else if (maxNode->LOD < sl_node->LOD){
				maxNode = sl_node;
			}
			
			sl_node = sl_node->next;
		}
	}
	return maxNode;
}
	
void printSummary(FILE *resFile, TestPoint* maxNode) {

	printf("--------------------------------------------------------------------------------\n");
	fprintf(resFile,"-------------------------------------------------------------------------------------------\n");
	if (maxNode->LOD  > 0){
		printf("  MAX\t\t%.6lf\t%.2lf\t     %.6lf\t\n",maxNode->distFromFirstWindowMarker,maxNode->ln_like,maxNode->LOD);
		fprintf(resFile,"  MAX\t\t%.6lf\t\t  %.2lf\t\t%lf\t\n",maxNode->distFromFirstWindowMarker,maxNode->ln_like,maxNode->LOD);
	}else{
		if (LnUnlinked > -DBL_MAX){
			printf("  MAX\t\tUNLINKED\t  %.2lf\t%lf\t\n",LnUnlinked,0);
			fprintf(resFile,"  MAX\t\tUNLINKED\t  %.2lf\t%lf\t\n",LnUnlinked,0.0);
		}else{
			printf("  MAX\t\tUNLINKED\t-Infinity\n");
			fprintf(resFile,"  MAX\t\tUNLINKED\t-Infinity\n");
		}

	}
	printf("--------------------------------------------------------------------------------\n");
	fprintf(resFile,"-------------------------------------------------------------------------------------------\n");	
	
	fclose(resFile);
}



FILE* printTitle() {
	FILE *resFile;

	if ((resFile = fopen(RESULTS_FILE_NAME, "w")) == NULL) {
		printf("File %s could not be opened.", RESULTS_FILE_NAME);
		exit(1);
	}
	
	printf("--------------------------------------------------------------------------------\n");
	printf("  Marker | Trait Position (cM) | LN(LIKE)  |  LOD_SCORE  |   NPL     |   NPL    \n");
	printf("   Name  |  (from marker #1)   |           |   	         |   SPAIRS  |   SALL   \n");
	printf("--------------------------------------------------------------------------------\n");
	fprintf(resFile,"-------------------------------------------------------------------------------------------\n");
	fprintf(resFile,"  Marker | Trait Position (cM) |   LN(LIKE)  | LOD_SCORE   |	NPL         |	NPL         \n");
	fprintf(resFile,"   Name  |  (from marker #1)   |             |             |	SPAIRS      |	SALL        \n");
	fprintf(resFile,"-------------------------------------------------------------------------------------------\n");

	return resFile;
}


void copyTestNode(TestPoint const* nodeFrom, TestPoint* nodeTo) {
	if ((nodeFrom != NULL) && (nodeTo != NULL)) {
		nodeTo->distFromFirstWindowMarker = nodeFrom->distFromFirstWindowMarker;
		nodeTo->ln_like = nodeFrom->ln_like;
		nodeTo->LOD = nodeFrom->LOD;
		nodeTo->next = NULL;
		nodeTo->NPL_Sall = nodeFrom->NPL_Sall;
		nodeTo->NPL_Spairs = nodeFrom->NPL_Spairs;
		nodeTo->theta1 = nodeFrom->theta1;
		nodeTo->theta2 = nodeFrom->theta2;
	}
}

/******************************************************************************
*
*       Function        createTestPoints
*       Parameters      programCode - program code
*						in - datafile
*       Output          none
*       Description		Parses input and find test points
*       Comments		none
*
******************************************************************************/
void readProgSpec( int programCode, FILE* in){
	int tmp;
	char tmpStr[10];
	char line[LineLength];

	
	
	fscanf(in, "%d", &tmp);
	if ((tmp != 1) && (tmp != 2)) {
		sl_inputErr("You can only iterate 1 or 2 sl_loci.\n");
	}
	else if (tmp == 2)  {
		sl_inputErr("You can only iterate 1 sl_locus in this version of gh.\n");
	}	
	// 4.	Read the Super-GH specific data for the iterated disease sl_loci. 


	fgets( line, LineLength, in);
	fscanf(in, "%s", tmpStr);

	if (strcmp(tmpStr, "-s") == 0) {
		fscanf(in, "%lf", &ProgramSpecificData.step);	
		ProgramSpecificData.type = 1;
	}
	else if (strcmp(tmpStr, "-n") == 0) {
		fscanf(in, "%d", &ProgramSpecificData.numIntervals);
		ProgramSpecificData.type = 0;
	}else{
		sl_inputErr("Program specific information - unknown flag.\n");
	}
	//4.2.	Read the indices of the markers delimiting the scan interval on the
	//		left and on the right. 
	fscanf(in,"%d%d",&ProgramSpecificData.start,&ProgramSpecificData.end);



	//	updateOffEndDist(scanDataPtr, sGHData->numIterLoci, numMarkers);
	while (fscanf(in,"%c",&tmpStr) == 1) {
		if (tmpStr[0] == ' ' || tmpStr[0] == '\t') {
			continue;
		}
		if (tmpStr[0] == '\n') {
			break;
		}
		if (tmpStr[0] == '-') {
			fscanf(in,"%c",&tmpStr);

			if ((tmpStr[0] == 'o') || (tmpStr[0] == 'O')) {
				fscanf(in, "%lf", &ProgramSpecificData.offMap);

				/* 1.	If the 'offEndDist' is smaller than 0.5 (--> it is recombination 
				value), turn it into cM using Haldane's (or Kosambi's) mapping sl_function. */

				if (ProgramSpecificData.offMap < 0.5) {
					ProgramSpecificData.offMap = getRecTocM_GH(ProgramSpecificData.offMap,1);
				}
			}
		}
	}
}



/******************************************************************************
*
*       Function        createTestPoints
*       Parameters      startPoint, endPoint - window boundaries
*						offMapLeftPtr, offMapRightPtr - pointers to offmap parameters
*						distFromFirstMarker 
*       Output          array of distances
*       Description		Finds positions of informative markers  in cM
*       Comments		none
*
******************************************************************************/
void createTestPoints(int startPoint, int endPoint, double* offMapLeftPtr, double* offMapRightPtr, 
										double* distFromFirstMarker){
	CreateTestPointsInput* input;

	//	TODO: check that this "if" is not nesessery
	/*if ((endPoint - startPoint + 1 > NumInfMarkers) || (endPoint < startPoint)) {
		printf("createTestPoints: window is too big!\n\n\n");
		exit(1);
	}*/
	distances = calculateMarkersDist(startPoint, endPoint);
	sl_myMalloc(testPoints,sizeof(TestPoint*)*(NumInformativeLoci+1));
	memset(testPoints,'\0',sizeof(TestPoint*)*(NumInformativeLoci+1));
	sizeTestPointsArray = 0;
	
	sl_myMalloc(input, sizeof(CreateTestPointsInput));
	input->distances = distances;
	input->distFromFirstMarker = *distFromFirstMarker;
	input->start = 1;
	input->end = endPoint - startPoint + 1;
	if (offMapLeftPtr == NULL) {
		input->offMapLeft = 0;
	}
	else {
		input->offMapLeft = *offMapLeftPtr;
	}
	if (offMapRightPtr == NULL) {
		input->offMapRight = 0;
	}
	else {
		input->offMapRight = *offMapRightPtr;
	}
	
	if (ProgramSpecificData.type == 1)
		createTestPointsMinusS(ProgramSpecificData.step, input);
	else
		createTestPointsMinusN(ProgramSpecificData.numIntervals, input);
	free(distances);
	free(input);

	if (offMapRightPtr == NULL) {
		(*distFromFirstMarker) = testPoints[sizeTestPointsArray - 1]->distFromFirstWindowMarker;
		free(testPoints[sizeTestPointsArray - 1]);
		testPoints[sizeTestPointsArray - 1] = NULL;
	}
}



/******************************************************************************
*
*       Function        reculcalateThetas
*       Parameters      none
*       Output          array of distances
*       Description		Finds positions of informative markers  in cM
*       Comments		none
*
******************************************************************************/
void reculcalateThetas(int startPoint, int endPoint){
	double *array, *ptrTheta,*ptrArray, tmp;
	int i,j;
	sl_myMalloc(array, sizeof(double)* (endPoint - startPoint + 1));
	for (ptrTheta = Recomb_values + startPoint,ptrArray = array, i = startPoint, j= 0, tmp  = 0;
		((j < NumInfMarkers ) && (i <= endPoint));  ptrTheta++,ptrArray++,i++){
			tmp += getRecTocM_GH((*ptrTheta),1);
			if (MarkerOrder[j] == i){				
				array[j] = getHaldaneCMToRec(tmp);
				tmp = 0;
				j++;
			}
		}
	memcpy(Recomb_values + startPoint, array, sizeof(double)* (endPoint - startPoint + 1));
	free(array);

}


/******************************************************************************
*
*       Function        calculateMarkersDist
*       Parameters      none
*       Output          array of distances
*       Description		Finds positions of informative markers  in cM
*       Comments		none
*
******************************************************************************/
double *calculateMarkersDist(int startPoint, int endPoint){
	double *array, *ptrArray ,*ptrTheta,tmp;
	int i, size;
	
	reculcalateThetas(startPoint, endPoint);
	
	size = endPoint - startPoint + 1;
	sl_myMalloc(array, sizeof(double) * (size + 2));
	(*array) = 0;
	*(array+1) = 0;

	for (ptrArray = array+2, ptrTheta = Recomb_values + startPoint,i = 0 ; 
		((i < size - 1) &&(i < NumInfMarkers)); i++, ptrTheta++,ptrArray++){
			tmp = getRecTocM_GH((*ptrTheta),1);
			*ptrArray = *(ptrArray-1) + tmp;
	}

	return array;
}

/******************************************************************************
*
*       Function        createTestPointsMinusS
*       Parameters      sl_step - size of sl_step (if less then 0.5 then it recombination 
*						value,otherwise its cM
*						sl_start - first marker number (starting 1)
*						end - last marker number
*						distances - array of distances of markers in cM
*       Output          none
*       Description		Finds all the requested positions of the iterated disease 
*						sl_locus in the marker map for -s option.
*       Comments		none
*
******************************************************************************/
void createTestPointsMinusS(double sl_step, CreateTestPointsInput* input){

	double cMStep, currentPos;
	int curLocus;
	TestPoint *sl_node, *lastNode;
	int sl_start;
	int end;
	double* distances;
	double offMapLeft;
	double offMapRight;

	sl_start = input->start;
	end = input->end;
	distances = input->distances;
	offMapLeft = input->offMapLeft;
	offMapRight = input->offMapRight;

	if ( sl_step < 0.5){
		cMStep = getRecTocM_GH(sl_step,1);
	}
	else{
		cMStep = sl_step;
	}

	distances[end + 1] = distances[end] + offMapRight;


	if (offMapLeft != 0 && sl_start == 1){
		curLocus = 0;
		currentPos = -offMapLeft;
	}else{
		curLocus = sl_start;
		currentPos = distances[curLocus];
		sizeTestPointsArray++;
	}
	if (offMapRight!=0 && end == NumInformativeLoci-1){
		end = NumInformativeLoci;
	}


	while (curLocus < end){
		sl_myMalloc(sl_node,sizeof(TestPoint));
		sl_node->LOD = 0;
		sl_node->ln_like = 0;
		sl_node->theta1 = getcMToRec((currentPos - distances[curLocus] ),1);
		if (curLocus != end){
			sl_node->theta2 = getcMToRec((distances[curLocus+1] - currentPos),1);
		}
		sl_node->distFromFirstWindowMarker = currentPos - distances[sl_start] + input->distFromFirstMarker;
		sl_node->next = NULL;
		if (testPoints[curLocus] == NULL) {
			testPoints[curLocus] = sl_node;
			sizeTestPointsArray++;
		}
		else { 
			lastNode->next = sl_node;
		}
		lastNode = sl_node;
		currentPos += cMStep;
		if (curLocus == end)
			break;
		while( currentPos >= distances[curLocus + 1] && curLocus < end){
			curLocus++;
		}

	}
}

/******************************************************************************
*
*       Function        createTestPointsMinusM
*       Parameters      numIter - number of itteration between two markers
*						sl_start - first marker number (starting 1)
*						end - last marker number
*						distances - array of distances of markers in cM
*       Output          none
*       Description		Finds all the requested positions of the iterated disease 
*						sl_locus in the marker map for -n option.
*       Comments		none
*
******************************************************************************/
void createTestPointsMinusN(int numIter, CreateTestPointsInput* input){
	double cMStep, currentPos;
	int curLocus, count;
	TestPoint *sl_node, *lastNode;
	int sl_start;
	int end;
	double* distances;
	double offMapLeft;
	double offMapRight;

	sl_start = input->start;
	end = input->end;
	distances = input->distances;
	offMapLeft = input->offMapLeft;
	offMapRight = input->offMapRight;

	currentPos = distances[sl_start];
	curLocus = sl_start;

	distances[end + 1] = distances[end] + offMapRight;
	if (numIter != 0 && sl_start != end + 1){
		cMStep = (distances[sl_start+1] - distances[sl_start] )/numIter;
	}else
		cMStep = 0;

	
	if (offMapLeft != 0){
		if (sl_start == 1){
			if (numIter != 0 )
				cMStep = offMapLeft / numIter;
			else
				cMStep = offMapLeft;
			currentPos = -offMapLeft;
			curLocus = 0;
		}
	}
	else {
		sizeTestPointsArray++;
	}
	if (offMapRight != 0) {
		if (end >= (NumInformativeLoci-1)){
			end = NumInformativeLoci;
		}
	}

	count = 0;
	while (curLocus <= end){
		count++;
		sl_myMalloc(sl_node,sizeof(TestPoint));
		sl_node->LOD = 0;
		sl_node->ln_like = 0;
		if (curLocus != end){
			sl_node->theta1 = getcMToRec((currentPos - distances[curLocus] ),1);
			sl_node->theta2 = getcMToRec((distances[curLocus+1] - currentPos),1);
		}else{
			sl_node->theta1 = getcMToRec(offMapRight,1);
		}
		sl_node->distFromFirstWindowMarker = currentPos - distances[sl_start] + input->distFromFirstMarker;
		sl_node->next = NULL;
		if (testPoints[curLocus] == NULL){
			testPoints[curLocus] = sl_node;
			sizeTestPointsArray++;
		}else {
			lastNode->next = sl_node;
		}
		lastNode = sl_node;
		currentPos += cMStep;

		if (curLocus == end)
			break;
		if (count == numIter) {
			count = 0;
			curLocus++;
			currentPos = distances[curLocus];
			if (numIter != 0){
				cMStep = (distances[curLocus + 1 ] - distances[curLocus ])/numIter;
			}else
				cMStep = 0;
		}
	}
}


/******************************************************************************
*
*       Function        freeTestPoints
*       Parameters      none
*       Output          none
*       Description		frees program specific information
*       Comments		none
*
******************************************************************************/
void freeTestPoints(){
	int i;
	TestPoint *sl_node,*nodeTmp;;
	for (i=0; i < sizeTestPointsArray; i++){
		sl_node = testPoints[i];
		while(sl_node != NULL){
			nodeTmp = sl_node;
			sl_node = sl_node->next;
			free(nodeTmp);
		}
	}
	free(testPoints);
}

