/* $Id: programSpec.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  programSpec.h											   				*/		
/*  Saves programs specific information and prints output	    			*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/
#ifndef PROGRAMSPEC_H
#define PROGRAMSPEC_H
#include "sl_Auxilary.h"
#include "gdata.h"


#define LineLength  30000

void freeTestPoints();
/* double definitions */
typedef char xLabel[100];	
/* Defines a 2-dimensional point. */

typedef struct point {
	double		x;
	double		y;
} point;

void createPSGraph(char *psFileName, char *graphTitle, char *yTitle, point *vals, 
				   int numPoints, xLabel *markerNames, int numXLabels, double *mapDistArr,
				   boolean	drawBezCurve);

#define getHaldaneCMToRec(dist) \
	(0.5 * (1 - exp(-2 * (dist/100))));

#define	getRecTocM_GH(rec, haldane) \
	(rec > 0.499) ? 0.5 : \
	(haldane ? (log(1 - 2 * rec) * (-50)) : (25*log((1+2*rec)/(1-2*rec))));

/* Computes the recombination fraction (probability) from the given distance 
   in Morgans, using Haldane's or Kosambi's map sl_function. */

#define getcMToRec(dist, haldane) \
	haldane ? (0.5 * (1 - exp(-0.02 * dist))) : (0.5*(exp(0.04*dist)-1)/(exp(0.04*dist)+1));


// Information for one test point
typedef struct TestPoint { 
	double  theta1;  // theta before sl_affected
	double  theta2;  // theta after sl_affected
	double  distFromFirstWindowMarker; // distance in cM from first Marker of current window
	double LOD;
	double ln_like;

	/**************************************************************/
	/* added by Anisenia Andrei									  */
	double NPL_Spairs;
	double NPL_Sall;
	/**************************************************************/

	struct TestPoint *next; // pointer to the next test point
}TestPoint;

typedef struct CreateTestPointsInput{
	int start;
	int end;
	double* distances;
	double offMapLeft;
	double offMapRight;
	double distFromFirstMarker;
}
CreateTestPointsInput;


double LnUnlinked;

TestPoint **testPoints; // holds all Test points
int sizeTestPointsArray;

double* distances; // distances of the markers;
#define RESULTS_FILE_NAME  "results.txt"
#define BITS_INFO_FILE_NAME "vectorsInfo.bin"
int numTestPoints;


void readProgSpec( int programCode, FILE* in);
void createTestPoints(int startPoint, int endPoint, double* offMapLeftPtr, double* offMapRightPtr, 
										double* distFromFirstMarker);
TestPoint* printTestPoints(FILE* resFile);
FILE* printTitle();
void printSummary(FILE* resFile, TestPoint* maxNode);
void copyTestNode(TestPoint const* nodeFrom, TestPoint* nodeTo);



#endif
