/* $Id: runAllelesHunter.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************
 ************** run AllelesHunter.c  ****************************************
 ****************************************************************************
 ************** created by Anisenia Andrei **********************************
 *	runs main algorithm using AllelesHunter structure						*
 ****************************************************************************/


#include "runAllelesHunter.h"


/****************************************************************************
*
*       Function		runAllelesHunter  
*       Parameters		pedigree
*						AllelesHunter
*		Output			none
*       Description		runs the main algorithm of findning the alleles 
*						that best match the most probable vector and
*						allele frequences, and thus updating pedigree,
*						by using structure AllelesHunter  
*       Comments		none
*
*****************************************************************************/
void runAllelesHunter(ped_node** pedigree,allelesHunter* AllelesHunter)
{
	int maxFrequentAllele, numFrequences, curAllele;
	double* curFrequences;
	ped_locus* curLocusPtr;

	curFrequences = lociDataArray[AllelesHunter->locus+1].frequencies;
	numFrequences = lociDataArray[AllelesHunter->locus+1].num_of_freq;

	maxFrequentAllele = findMaxFrequentAllele(curFrequences,numFrequences);

	for(curAllele = 0; curAllele < 2*AllelesHunter->numFounders; curAllele++)
	{
		if (AllelesHunter->FounderAlleles[curAllele].includedInComponent == false)
		{
			curLocusPtr = &(pedigree[curAllele/2]->myAlleles[AllelesHunter->locus]);
			fixAllele(curAllele,curLocusPtr,maxFrequentAllele);
		}
		else
		{
			if (AllelesHunter->FounderAlleles[curAllele].used == false)
			{
				treatComponent(pedigree,AllelesHunter,curAllele,curFrequences);
			}
		}
	}	
}



/****************************************************************************
*
*       Function		findMaxFrequentAllele  
*       Parameters		curFrequences - array of allele frequences
*						numFrequences - the number of frequences(alleles) in array
*		Output			allele wiht maximal frequency
*       Description		none
*       Comments		none
*
*****************************************************************************/
int findMaxFrequentAllele(double* Frequences,int numFrequences)
{
	int maxFrequentAllele, curAllele;
	double curFrequency;

	maxFrequentAllele = 1;
	curFrequency = Frequences[0];
	for( curAllele=2; curAllele <= numFrequences; curAllele++)
	{
		if (curFrequency < Frequences[curAllele-1])
		{
			maxFrequentAllele = curAllele;
			curFrequency = Frequences[curAllele-1];
		}
	}
	return maxFrequentAllele;
}




/****************************************************************************
*
*       Function		fixAllele  
*       Parameters		alleleToFix - allele to be fixed
*						curLocusPtr - locus of this allele in the pedigree
*						fixValue - value of alleleToFix
*		Output			none
*       Description		assignes fixValue to alleleToFix in curLocusPtr
*       Comments		none
*
*****************************************************************************/
void fixAllele(int alleleToFix, ped_locus* curLocusPtr, int fixValue)
{
	
	if (alleleToFix % 2 == 0)
	{
		curLocusPtr->allele1 = fixValue;
	}
	else
	{
		curLocusPtr->allele2 = fixValue;
	}	
}



/****************************************************************************
*
*       Function		treatFounderCellComponent  
*       Parameters		queue
*						AllelesHunter
*						first, second, curFrequences
*		Output			true - if the queue is not empty
*						false - if the queue is empty
*       Description		treats some founder cell component
*       Comments		none
*
*****************************************************************************/
Bool treatFounderCellComponent(Queue* queue,allelesHunter* AllelesHunter,
							   double* first,double* second,double* curFrequences)
{
	fAllele* currFounderCell, *fAllele1, *fAllele2, *newCell;
	pedigreePerson* person; 

	currFounderCell = (fAllele*)getCurr(queue);
	if (currFounderCell == NULL)
	{
		return true;
	}
	resetCurComponentPerson2(currFounderCell);
	while(currFounderCell->curComponentPerson != NULL)
	{
		person = &(AllelesHunter->People[currFounderCell->curComponentPerson->person]);
		if (person->used == false)
		{
			person->used = true;
			fAllele1 = &(AllelesHunter->FounderAlleles[person->founderAllele1]);
			fAllele2 = &(AllelesHunter->FounderAlleles[person->founderAllele2]);
			if (*first != 0.0)
			{
				checkAllelesCompatibility(	&(fAllele1->tryFirstAllele),
											&(fAllele2->tryFirstAllele),
											person, first, curFrequences); 
			}
			if (*second != 0.0)
			{
				checkAllelesCompatibility(	&(fAllele1->trySecondAllele),
											&(fAllele2->trySecondAllele),
											person,second,curFrequences);
			}
			newCell = (currFounderCell == fAllele1)? fAllele2 : fAllele1;
			/*	if newCell was not used before	*/
			if (newCell->used == false)
			{
				newCell->used = true;
				/*	if the component list of newCell is not empty	*/
				if (newCell->lastComponentPerson->person != -1)
				{
					/*	we need to add newCell to the end of queue	*/
					InsertToTail(queue,(void*)newCell);
				}
			}
		}
		currFounderCell->curComponentPerson = currFounderCell->curComponentPerson->next;
	}	
	return false;
}



/****************************************************************************
*
*       Function		checkAllelesCompatibility  
*       Parameters		tryAllele1, tryAllele2, person,prob, curFrequences
*		Output			none
*       Description		checks alleles compatibility
*       Comments		none
*
*****************************************************************************/
void checkAllelesCompatibility(int* tryAllele1,int* tryAllele2,pedigreePerson* person,
							   double* prob, double* curFrequences)
{
	/*	if 2 alleles are used	*/
	if ( (*tryAllele1 != 0) && (*tryAllele2 != 0) )
	{
		/*	then we check if they are not equal to person's alleles:	*/
		if	(!(	(*tryAllele1 == person->allele1 && *tryAllele2 == person->allele2) ||
				(*tryAllele1 == person->allele2 && *tryAllele2 == person->allele1)	)
			)
		{
			*prob = 0.0;
		}
	}
	/*	if allele at fAllele1 is not used	*/
	if (*tryAllele1 == 0)
	{
		if (*tryAllele2 == person->allele1)
		{
			*tryAllele1 = person->allele2;
			*prob = (*prob) * curFrequences[person->allele2-1];
		}
		if (*tryAllele2 == person->allele2)
		{
			*tryAllele1 = person->allele1;
			*prob = (*prob) * curFrequences[person->allele1-1];
		}
		if (*tryAllele2 != person->allele1 &&
			*tryAllele2 != person->allele2 )
		{
			*prob = 0.0;
		}
	}
	/*	if allele at fAllele2 is not used	*/
	if (*tryAllele2 == 0)
	{
		if (*tryAllele1 == person->allele1)
		{
			*tryAllele2 = person->allele2;
			*prob = (*prob) * curFrequences[person->allele2-1];
		}
		if (*tryAllele1 == person->allele2)
		{
			*tryAllele2 = person->allele1;
			*prob = (*prob) * curFrequences[person->allele1-1];
		}
		if (*tryAllele1 != person->allele1 &&
			*tryAllele1 != person->allele2 )
		{
			*prob = 0.0;
		}
	}
}




/****************************************************************************
*
*       Function		treatComponent  
*       Parameters		pedigree
*						AllelesHunter
*						curAllele - current allele
*						curFrequences
*		Output			true - if current treated component has some 
*						compatible assignment (1 or 2) of alleles
*						false - if no assignment is compatible (error)
*       Description		uses AllelesHunter structure
*       Comments		none
*
*****************************************************************************/
Bool treatComponent(ped_node** pedigree,allelesHunter* AllelesHunter,int curAllele,
					double* curFrequences)
{
	Queue* queue;
	double first, second;
	int person;
	Bool end;
	fAllele* currFounderCell;

	currFounderCell = &(AllelesHunter->FounderAlleles[curAllele]);
	if (currFounderCell->firstComponentPerson == NULL)
	{
		return false;
	}
	person = currFounderCell->firstComponentPerson->next->person;
	queue = CreateQueue((void*)currFounderCell);
	currFounderCell->tryFirstAllele = AllelesHunter->People[person].allele1;
	currFounderCell->trySecondAllele = AllelesHunter->People[person].allele2;
	first = curFrequences[AllelesHunter->People[person].allele1-1];
	second = curFrequences[AllelesHunter->People[person].allele2-1];
	currFounderCell->used = true;
	/*	filling 2 possible alleles assignments for current component	*/
	do
	{
		increaseCurr(queue);
		end = treatFounderCellComponent(queue,AllelesHunter,&first,&second,curFrequences);
	}
	while(end == false);
	/*	tracing back the most probable assignment	*/
	traceBackAlleles(pedigree,AllelesHunter,first,second,queue);

	destroyQueue(queue);
	return true;
}




/****************************************************************************
*
*       Function		traceBackAlleles  
*       Parameters		pedigree
*						AllelesHunter
*						first, second
*		Output			none
*       Description		traces back alleles using queue of this component
*						and writes the most probable alleles to pedigree
*       Comments		none
*
*****************************************************************************/
void traceBackAlleles(ped_node** pedigree,allelesHunter* AllelesHunter,
					  double first,double second, Queue* queue)
{
	fAllele* curAllele;
	ped_locus* curLocusPtr;
	
	if (first >= second)
	{
		while(1)
		{
			decreaseCurr(queue);
			curAllele = (fAllele*)getCurr(queue);
			if (curAllele == NULL)
			{
				break;
			}
			curLocusPtr = 
				&(pedigree[(curAllele->index)/2]->myAlleles[AllelesHunter->locus]);
			fixAllele(curAllele->index,curLocusPtr,curAllele->tryFirstAllele);
		}		
	}
	else
	{
		while(1)
		{
			decreaseCurr(queue);
			curAllele = (fAllele*)getCurr(queue);
			if (curAllele == NULL)
			{
				break;
			}
			curLocusPtr = 
				&(pedigree[(curAllele->index)/2]->myAlleles[AllelesHunter->locus]);
			fixAllele(curAllele->index,curLocusPtr,curAllele->trySecondAllele);
		}		
	}	
}



