/* $Id: runAllelesHunter.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************
 ************** run AllelesHunter.h  ****************************************
 ****************************************************************************
 ************** created by Anisenia Andrei **********************************
 *	runs main algorithm using AllelesHunter structure						*
 ****************************************************************************/

#ifndef RUN_ALLELES_HUNTER_H
#define RUN_ALLELES_HUNTER_H

#include "initAllelesHunter.h"
#include "queue.h"



/****************************************************************************
*
*       Function		runAllelesHunter  
*       Parameters		pedigree
*						AllelesHunter
*		Output			none
*       Description		runs the main algorithm of findning the alleles 
*						that best match the most probable vector and
*						allele frequences, and thus updating pedigree,
*						by using structure AllelesHunter  
*       Comments		none
*
*****************************************************************************/
void runAllelesHunter(ped_node** pedigree,allelesHunter* AllelesHunter);



/****************************************************************************
*
*       Function		findMaxFrequentAllele  
*       Parameters		curFrequences - array of allele frequences
*						numFrequences - the number of frequences(alleles) in array
*		Output			allele wiht maximal frequency
*       Description		none
*       Comments		none
*
*****************************************************************************/
int findMaxFrequentAllele(double* Frequences,int numFrequences);


/****************************************************************************
*
*       Function		fixAllele  
*       Parameters		alleleToFix - allele to be fixed
*						curLocusPtr - locus of this allele in the pedigree
*						fixValue - value of alleleToFix
*		Output			none
*       Description		assignes fixValue to alleleToFix in curLocusPtr
*       Comments		none
*
*****************************************************************************/
void fixAllele(int alleleToFix, ped_locus* curLocusPtr, int fixValue);



/****************************************************************************
*
*       Function		treatComponent  
*       Parameters		pedigree
*						AllelesHunter
*						curAllele - current allele
*						curFrequences
*		Output			true - if current treated component has some 
*						compatible assignment (1 or 2) of alleles
*						false - if no assignment is compatible (error)
*       Description		uses AllelesHunter structure
*       Comments		none
*
*****************************************************************************/
Bool treatComponent(ped_node** pedigree, allelesHunter* AllelesHunter,int curAllele,
					double* curFrequences);


/****************************************************************************
*
*       Function		checkAllelesCompatibility  
*       Parameters		tryAllele1, tryAllele2, person,prob, curFrequences
*		Output			none
*       Description		checks alleles compatibility
*       Comments		none
*
*****************************************************************************/
void checkAllelesCompatibility(int* tryAllele1,int* tryAllele2,pedigreePerson* person,
							   double* prob, double* curFrequences);


/****************************************************************************
*
*       Function		treatFounderCellComponent  
*       Parameters		queue
*						AllelesHunter
*						first, second, curFrequences
*		Output			true - if the queue is not empty
*						false - if the queue is empty
*       Description		treats some founder cell component
*       Comments		none
*
*****************************************************************************/
Bool treatFounderCellComponent(Queue* queue,allelesHunter* AllelesHunter,
							   double* first,double* second,double* curFrequences);



/****************************************************************************
*
*       Function		traceBackAlleles  
*       Parameters		pedigree
*						AllelesHunter
*						first, second
*		Output			none
*       Description		traces back alleles using queue of this component
*						and writes the most probable alleles to pedigree
*       Comments		none
*
*****************************************************************************/
void traceBackAlleles(ped_node** pedigree,allelesHunter* AllelesHunter,
					  double first,double second, Queue* queue);



#endif

