/* $Id: sequence.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  sequence.h											   					*/
/*  computes "gray code" sequence										 	*/
/*  Author: Andrei Anisenia													*/
/****************************************************************************/
#ifndef SEQUENCE_H
#define SEQUENCE_H

#include "gdata.h"

#define LIMIT 15

/* struct used by the algorithm												*/
typedef struct helper_to_algorithm
{
	int k;				
	int m;
	int n;
	unsigned long size;	/* size of the helper array of k					*/
	int* arr;			/* helper array of k								*/
	int* i;				/* array of indexes "i"								*/
	Bool flag;			/* shows when to stop (false)						*/
	int lim;			/* if this value is more than n => stop algorithm	*/
} help_alg;


/****************************************************************************/
/*	alternative algorithm structures										*/
typedef struct controlNode_t
{
	int currentBasicIndex;
	int* currentBasicNumberPtr;
	int addedNumber;
}controlNode;

typedef struct grayCodeGenerator_t
{
	int* basicArray;
	int basicSize;
	controlNode controlArray[2];
	int controlSize;
	int currentControlIndex;
	controlNode* currentControlNodePtr;
}grayCodeGenerator;

/****************************************************************************/





/*	help functions															*/

unsigned long power(int x, int y);

int* compute_strait_to_array(int k, unsigned long* size);

void compute(help_alg* ph, void (*uF)(int) );


/* the main function for computing the "gray code" sequence					*/
/* parameters:	n - the size (= maximal number) of the "gray code"			*/
/*				uF - external function which will use "gray code"			*/
/*				generated by the algorithm (this function has single 		*/
/*				parameter of type "int" - in each iteration of algorithm	*/
/*				it will get the next index in the order of "gray code")		*/
/* note:		for example, if n==3 so the gray code is (1,2,1,3,1,2,1)	*/
void compute_algorithm(int n, void (*uF)(int) );



/****************************************************************************/
/*	alternative algorithm													*/

int* getGrayCodeArray(int k, unsigned long* size);

grayCodeGenerator* initiateGrayCodeGenerator(int size);

int getNextGrayCodeNumber(grayCodeGenerator* generator);



void destroyGrayCodeGenerator(grayCodeGenerator* generator);

/****************************************************************************/



#endif
