/* $Id: sl_Auxilary.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/*****************************************************************************/
/*  Auxilary.c																 */
/*  Author: Ma'ayan Fishelson				                                 */
/*****************************************************************************/

#include "sl_Auxilary.h"


/******************************************************************************
*
*       sl_function        sl_mallocErr
*       parameters      message (the error message printed).
*       output          none
*       description     prints an error message when malloc fails to allocate
*						memory, and exits.
*       comments		none
*
******************************************************************************/
void sl_mallocErr(char *message)
{
	fprintf(stderr, "\nProblem with malloc, probably not enough space.\n");
	fprintf(stderr, "Problem occurred when allocating %s.\n", message);

	exit(EXIT_FAILURE);
}




/******************************************************************************
*
*       sl_function        sl_reallocErr
*       parameters      message (the error message printed before exiting).
*       output          none
*       description     prints an error message when sl_reallocErr fails to allocate
*						additional space, and exits.
*       comments		none
*
******************************************************************************/
void sl_reallocErr(char *message)
{
	fprintf(stderr, "\nProblem with realloc, probably not enough space.\n");
	fprintf(stderr, "Problem occurred when allocating %s.\n", message);

#if MANAGE_MEM
	freeAllBlocks();
	longjmp(jumper, 1);
#endif

	exit(EXIT_FAILURE);
}


/******************************************************************************
*
*       sl_function        sl_inputErr
*       parameters      message (the error message printed before exiting).
*       output          none
*       description     prints an error message when there's an input error,
*						and exits.
*       comments		none
*
******************************************************************************/
void sl_inputErr(char *message, ...) 
{
	va_list		ap;


	/* 1.	Set a pointer to the beginning of the argument list. */

	va_start(ap, message);

	/* 2.	Print the requested string to stderr. */

	fprintf(stderr, "\nAn input error has been found:\n");
	vfprintf(stderr, message, ap);

	/* 4.	Reset the pointer to the beginning of the argument list. */

	va_end(ap);

	/* 5.	Exit. */

	exit(EXIT_FAILURE);
}



/******************************************************************************
*
*       sl_function        sl_inputWarning
*       parameters      message (the warning message printed).
*       output          none
*       description     prints a warning message when there might be an input 
*						error.
*       comments		none
*
******************************************************************************/
void sl_inputWarning(char * message) 
{
	fprintf(stderr, "\nA possible input error has been found:\n");
	fprintf(stderr, "%s\n", message);
}


/******************************************************************************
*
*       Function        sl_P_eof
*       Parameters		FILE *f
*       Output          none
*       Description		tests for end of file. 
*       Comments		(this sl_function is also used in "unknown.c" in Fastlink)
*
******************************************************************************/
int sl_P_eof(FILE *f)
{
    register int ch;

    if (feof(f))
        return 1;
    if (f == stdin)
        return 0;    /* not safe to look-ahead on the keyboard! */
    ch = getc(f);
    if (ch == EOF)
        return 1;
    
	ungetc(ch, f);

    return 0;
}

/******************************************************************************
*
*       Function        sl_swap
*       Parameters      val1, val2
*       Output          none
*       Description		swaps between the values of 'val1' & 'val2'.
*       Comments		none
*
******************************************************************************/
void sl_swap(int *val1, int *val2)
{
	int temp = *val1;

	*val1 = *val2;
	*val2 = temp;
}

