/* $Id: sl_Settings.h,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/*****************************************************************************/
/*  Settings.h																 */
/*  Author: Ma'ayan Fishelson							                     */
/*****************************************************************************/

#ifndef _SL_SETTINGS_H_
#define _SL_SETTINGS_H_

#include <stdio.h>
#include <stdlib.h>


#define SL_INVALID		-1.0


/* Prints various statistics into the file "statistics.csv" according to the 
   level requested:
   0) No statistics are printed.
   1) Statistics regarding the elimination-order-finding-process are printed,
   2) and statistics regarding the treewidth of the sl_graph, the number 
      of edges in the sl_graph, the number of vertices, the number of constant vertices,
   3) and the minimum, average and maximum number of values of each sl_vertex, 
   4) and the minimum, average and maximum number of selectors between each 2 
      consecutive sl_loci.*/
#define		SL_PRINT_VARIOUS_STATISTICS_LEVEL		0


/* Prints messages of how many variables have been trimmed from the sl_graph, how 
   many values have been eliminated as a result of the value-elimination process,
   and how many nodes have been created. Also prints the number of nodes that are
   determined in the preprocessing phase. */
#define		SL_PRINT_ELIMINATION_STATISTICS		0

/* Prints the percentage of the total computation that has been performed already. */
#define		SL_PRINT_PROGRESS_REPORT				1


/* Prints messages regarding the optimization of the elimination order (for debugging). */
#define		SL_PRINT_DEBUG_OPTIMIZATION_ELIMINATION_ORDER	0

/* A constant which determines when it's not worth to continue searching for a 
   sl_variable to eliminate next (if the size of the combined table is equal to 
   this constant or below it, then we stop the search and return the current 
   sl_variable to be eliminated.) */
#define		SL_ELIMINATION_BOTTOM_THRESHOLD		15

/* A constant that determines when a more simple sl_function will be used for 
   multiplying several probability functions (if the sl_product of the size of the
   combined table and the number of functions that need to be combined is below
   this threshold, then the simple sl_function is used).*/
#define		SL_ELIMINATION_SIMPLE_THRESHOLD		24

/* Determines if sl_product-breaking will be performed (in case needed). */	
#define		SL_PERFORM_PRODUCT_BREAKING			1

/* Determines the maximum size of a probability table that would be processed,
   without sl_product-breaking.  If the table that would be created equals this 
   size or exceeds it, sl_product breaking shall be performed on some sl_variable 
   that hasn't been eliminated yet. */
#define		SL_BREAK_PRODUCT_BOTTOM_THRESHOLD		30000000  

/* Determines if sl_bucket-merging will be performed (in case needed). */	
#define		SL_PERFORM_BUCKET_MERGING				0

#define		SL_BUCKET_MERGING_BOTTOM_THRESHOLD		2000000

#define		SL_MIN_DBL								-DBL_MAX

/* The number of times a sl_function can be handled before it needs to be rescaled. */
#define		SL_MAX_RESCALE_UNITS					8

/* The highest complexity coefficient for which no optimization is performed. */
#define		SL_OPTIMIZATION_EO_BOTTOM_THRESHOLD	6

/* The number of iterations that are performed for each order (general-greedy &
   sl_locus-by-sl_locus) to determine which order should be searched for. */
#define		SL_NUM_TEST_ITER						3

/* The maximum number of non-founder individuals for which an order of 
   sl_locus-after-sl_locus is determined automatically (without testing which order is better). */
#define		SL_MAX_NUM_NF_LOCUS_LOCUS				15

/* The minimum number of non-founder individuals for which a general greedy order is 
   determined automatically (without testing which order is better). */
#define		SL_MIN_NUM_NF_GENERAL					30

#define		SL_EPSILON								0.000005

/* Determines whether approximate or exact calculations would be performed. */

#define		SL_PERFORM_EXACT_COMPUTATIONS			1


/* Determines whether to apply stochastic-greedy methods for finding an 
   elimination-order. */

#if	SL_PERFORM_EXACT_COMPUTATIONS
	#define		SL_APPLY_STOCH_M					1
	#define		SL_APPLY_S_G_M_C					0
#else
	#define		SL_APPLY_STOCH_M					0
	#define		SL_APPLY_S_G_M_C					0
#endif  
  





/* Determines whether all the selectors would be eliminated together (style Genehunter),
   or one by one. */

#define		SL_ELIMINATE_SELECTORS_TOGETHER		1

#define		SL_USE_FFT								0


/* Determines whether a preprocessing phase would be applied on the Bayesian network 
   when determining the elimination-order for the variables of the Bayesian network. */

#define		SL_PERFORM_PREPROCESSING				1


#endif /* _SETTINGS_H_ */
