/* $Id: treeSearch.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
#include "treeSearch.h"

double calculateProb(int curLocus, int curVector);
Bool checkEdge(int nonfounder,int curLocus,int curVector   );
Bool addEdgeToVertex(int founder, int nonfounder,  int curLocus );
/******************************************************************************
*
*       Function        getIfSymetry
*       Parameters		nonfounder - index of nonfounder (out of nonfounders)
*						paternal - true if paternal.
*       Output          true, if simetrial
*       Description		checks if vector is semetral from this point 
*       Comments		none
*
******************************************************************************/
Bool getIfSymetry(int nonfounder, Bool paternal, int curLocus){
	int personInd = PedigreeData.NonFounders[nonfounder];
	int parent;
	
	
	/* If alleles of the person are unknown and it doesn't have children */
	if ( (PedigreeGraph[personInd].myLocus[curLocus].allele1 == 0)
		&& PedigreeGraph[personInd].child == 0)
		return true;

	/* Find parent */
	if ( paternal){
		parent = PedigreeGraph[personInd].father - 1;
	}else{
		parent = PedigreeGraph[personInd].mother - 1;
	}

	if (ProgramSpecificData.sexLinked && paternal)
		return false;


	/* if it is founder - error*/
	if (parent == -1){
		return false;
	}

	/* If person is homozygote at this locus return true*/
	
	if (PedigreeGraph[parent].myLocus[curLocus].allele1 ==
		PedigreeGraph[parent].myLocus[curLocus].allele2 &&
		PedigreeGraph[parent].myLocus[curLocus].allele1 != 0)
		return true;
	return false;
}

/******************************************************************************
*
*       Function        treeSearch
*       Parameters		bitIndex - current bit
*						nonFounderIndex - non founder index (out of nonfoundes)
*						paternal - paternal or maternal bit
*       Output          none
*       Description		computes all vectors probability for locus 
*       Comments		updates symetry mask for vectors tha were not computed
*
******************************************************************************/
void treeSearch(int bitIndex, int nonFounderIndex,Bool paternal, int curLocus, int curVector){
	int savedVector;
	Pedigree *personPtr;
	int nextBitIndex;
	AlleleData *allelePtr;
	Assign *savedAlleles; 
	int alleleInd;
	int allele;
	
#if DEBUG_MEMORY == 1
#ifdef WIN32
	_ASSERT(_CrtCheckMemory());
#endif
#endif
	alleleInd = nonFounderIndex*2 + !paternal;

	/* If it is whole vector compute probability */
	if (bitIndex == PedigreeData.SpeedUpNumOfMeioses && nonFounderIndex == PedigreeData.NumOfNonFounders){
		ProbabilityLocus[curLocus-1][curVector] =  calculateProb(curLocus, curVector);
		return;
	}

	personPtr = &PedigreeGraph[PedigreeData.NonFounders[nonFounderIndex]];	
	if (paternal){
		allelePtr = personPtr->paternalAllele;
	}else{
		allelePtr = personPtr->maternalAllele;
	}

	if (ProgramSpecificData.sexLinked &&  personPtr->sex == Male && paternal){
		return treeSearch(bitIndex, nonFounderIndex + !paternal,!paternal, curLocus, curVector);
	}

	savedVector = curVector;

	/* If this tree is symetry to another one*/ 
	sl_myMalloc(savedAlleles,(sizeof(Assign)*2*PedigreeData.NumOfFounders));
	memcpy(savedAlleles,Alleles, (sizeof(Assign)*2*PedigreeData.NumOfFounders));

	/* If there is possible assignment to the founder */
	#if (ALLELES_INH_USE == 0)
		allele = AlleleInh[curVector][alleleInd];
	#else
		allele = getAllele(allelesInhObj, curVector, alleleInd);
	#endif
	if ( allelePtr->bitType  == ReductionBit || addEdgeToVertex(allele,PedigreeData.NonFounders[nonFounderIndex],curLocus)){

		if (paternal || checkEdge(PedigreeData.NonFounders[nonFounderIndex],curLocus, curVector)){
			nextBitIndex = (allelePtr->bitType  == RegularBit)?bitIndex + 1:bitIndex;
			treeSearch(nextBitIndex, nonFounderIndex + !paternal,!paternal, curLocus, curVector);
		}
	}
	
	/*Iverse bit*/
	curVector = savedVector;

	/* If SpeedUp (-f) return */
	if (allelePtr->bitType  == ReductionBit  || allelePtr->bitType == ConstBit){
		free(savedAlleles);
		return;
	}

	//printf("BIT INDEX:	%d \n", bitIndex);
	curVector |= MaskLSB[bitIndex];

	/* Check if there is a symmetry in the parent */
	if (getIfSymetry(nonFounderIndex,paternal, curLocus)){
		free(savedAlleles);
		symetryMask |= MaskLSB[bitIndex];	
		return;
	}else{
		memcpy(Alleles, savedAlleles,sizeof(Assign)*2*PedigreeData.NumOfFounders);
		free(savedAlleles);
		#if (ALLELES_INH_USE == 0)
			allele = AlleleInh[curVector][alleleInd];
		#else
			allele = getAllele(allelesInhObj, curVector, alleleInd);
		#endif
		
		if (addEdgeToVertex(allele, PedigreeData.NonFounders[nonFounderIndex],curLocus)){
			if (paternal || checkEdge(PedigreeData.NonFounders[nonFounderIndex],curLocus, curVector)){	
				nextBitIndex = (allelePtr->bitType  == RegularBit)?bitIndex + 1:bitIndex;
				treeSearch(nextBitIndex, nonFounderIndex + !paternal,!paternal, curLocus, curVector);
			}
		}
	}
}

/******************************************************************************
*
*       Function        checkEdge
*       Parameters		nonfounder - index of nonfounder (out of nonfounders)
*       Output          if component is possible
*       Description		checks if is possible to add an edge
*       Comments		none
*
******************************************************************************/
Bool checkEdge(int nonfounder, int curLocus, int curVector ){
	int allele1, allele2;
	int founder1, founder2;
	Pedigree *pedPtr;
	Assign *founderAllelesPtr1, *founderAllelesPtr2;

	pedPtr = &PedigreeGraph[nonfounder];
	/* 1.0 If nonfounder's sl_alleles are unknown */
	allele1 = pedPtr->myLocus[curLocus].allele1;
	/* If no information - no problem */
	if (allele1==0)
		return true;
	allele2 = pedPtr->myLocus[curLocus].allele2;

	#if (ALLELES_INH_USE == 0) 
		founder1 = AlleleInh[curVector][pedPtr->num*2];
		founder2 = AlleleInh[curVector][pedPtr->num*2 + 1];
	#else
		founder1 = getAllele(allelesInhObj, curVector, pedPtr->num*2);
		founder2 = getAllele(allelesInhObj, curVector, pedPtr->num*2 + 1);
	#endif
	
	if (ProgramSpecificData.sexLinked && pedPtr->sex == Male){
		founderAllelesPtr2 = &(Alleles[founder2 ]);
		if (Alleles[founder2].A ==0  ||
			allele1 == Alleles[founder2].A  || allele1 == Alleles[founder2].B)
			return true;
		return false;

	}

	/* If both alleles are inhereted fom same founder, but have different values,
	the vector is impossible */
	if (founder1 == founder2 && allele1 != allele2 ) 
		return false;
	founderAllelesPtr1 = &(Alleles[founder1 ]);
	founderAllelesPtr2 = &(Alleles[founder2 ]);
	/* Why no check for 0? */
	if ((allele1 == founderAllelesPtr1->A && allele2 == founderAllelesPtr2->A) ||
		(allele1 == founderAllelesPtr1->A && allele2 == founderAllelesPtr2->B) ||
		(allele1 == founderAllelesPtr1->B && allele2 == founderAllelesPtr2->A) ||
		(allele1 == founderAllelesPtr1->B && allele2 == founderAllelesPtr2->B) ||
		(allele1 == founderAllelesPtr2->A && allele2 == founderAllelesPtr1->A) ||
		(allele1 == founderAllelesPtr2->A && allele2 == founderAllelesPtr1->B) ||
		(allele1 == founderAllelesPtr2->B && allele2 == founderAllelesPtr1->A) ||
		(allele1 == founderAllelesPtr2->B && allele2 == founderAllelesPtr1->B))
		return true;
	return false;
}
/*****************************************************************************
*
*       Function        addEdgeToVertex
*       Parameters		founder - index of the founder allele
*						nonfounder - index of the non founderout of all people
*						
*       Output          true if assignment is possible
*       Description		Determinetes possible assignment to the founder allele
*       Comments		does NOT eliminate all unpossible assignments
*
******************************************************************************/

Bool addEdgeToVertex(int founder, int nonfounder, int curLocus ){
	int allele1, allele2;
	Assign *founderAllelesPtr;


	allele1 = PedigreeGraph[nonfounder].myLocus[curLocus].allele1;
	allele2 = PedigreeGraph[nonfounder].myLocus[curLocus].allele2;

	if (allele1 == 0 )
		return true;

	/* 2.0 If sl_alleles are same, only one possible asssignement to the founder */

	if (allele1 == allele2)
		allele2 = -1;

	founderAllelesPtr = &(Alleles[founder ]);

	/* 3.0 First assignment to the sl_vertex */
	if (founderAllelesPtr->A == 0){
		founderAllelesPtr->A = allele1;
		founderAllelesPtr->B = allele2;
		return true;
	}
	/* 3.1 If previous assignmnet is unpossible for allele A */
	if ( founderAllelesPtr->A != allele1 && founderAllelesPtr->A != allele2){
		founderAllelesPtr->A = -1;
	}

	/* 3.2 If previous assignmnet is unpossible for allele B */
	if ( founderAllelesPtr->B != allele1 && founderAllelesPtr->B != allele2){
		founderAllelesPtr->B = -1;
	}
	/*3.3  If A is impossible and B is possible swich  them */
	if (founderAllelesPtr->A == -1 && founderAllelesPtr->B != -1 ){
		founderAllelesPtr->A = founderAllelesPtr->B ;
		founderAllelesPtr->B = -1;
	}
	/* 4.0 Return true if there is possible assignment */
	if (founderAllelesPtr->A == -1)
		return false;
	return true;
}

/******************************************************************************
*
*       Function        initAllelesGirstGen
*       Parameters		none
*       Output          true if possible
*       Description		adds to graph first generation edges
*       Comments		inits data structures
*
******************************************************************************/
Bool initAllelesGirstGen(int curLocus ){
	int i,child;
	for ( i = 0; i < PedigreeData.NumOfFounders - PedigreeData.ChildlessFounders; i++){
		if (ProgramSpecificData.sexLinked && PedigreeGraph[PedigreeData.Founders[i]].sex == Male)
			continue;
		child = PedigreeGraph[PedigreeData.Founders[i]].child;
		while(child != 0 && PedigreeGraph[ child -1].informative == false){
			if (	PedigreeGraph[PedigreeData.Founders[i]].sex == Male){
				child = PedigreeGraph[child - 1].NeSiPa;
			}else{
				child = PedigreeGraph[child - 1].NeSiMa;
			}
		}
		addEdgeToVertex(PedigreeGraph[PedigreeData.Founders[i]].num*2,child - 1,curLocus);
	}
	return true;
}


/********************************************************
*
**********************************************************/
int findSimmetryMask(int curLocus){
	int i, nonfounder;
	int symmetryMask = 0;


	for (i =0; i < PedigreeData.NumOfNonFounders; i++){
		nonfounder = PedigreeData.NonFounders[i];
		if ((PedigreeGraph[nonfounder].paternalAllele->bitType == RegularBit) &&
			getIfSymetry(i,true,curLocus)){
				symmetryMask |= MaskLSB[PedigreeGraph[nonfounder].paternalAllele->number];	
			}

			if (PedigreeGraph[nonfounder].maternalAllele->bitType == RegularBit &&
				getIfSymetry(i,false,curLocus)){
					symmetryMask |= MaskLSB[PedigreeGraph[nonfounder].maternalAllele->number];	
				}
	}
	return symmetryMask;
}
