/* $Id: vectorGraph.c,v 1.2 2008/08/03 08:15:48 superlink Exp $ */
/****************************************************************************/
/*  vectorGraph.c											   				*/		
/*  Creates	graph (components) of founders alleles			    			*/
/*  Author: Anna Tzemach													*/
/****************************************************************************/

#include "vectorGraph.h"
#include "allocfile.h"
#include "sl_Auxilary.h"
/*****************************************************************************
*
*       Function        insert
*       Parameters		founder1 - founder allele index - out of all founders sl_alleles
*						nonfounder - sl_person index - out of all people
*						founder2 - founder allele index -  out of all founders sl_alleles
*       Output          true if edge is possibele
*       Description		Adds  edge to the sl_component 
*       Comments		if nonfounder is founder, add sl_alleles to the Alleles structure
******************************************************************************/
Bool insertNoTree(int founder1,int nonfounder,int founder2, int curLocus){
	Neighbours *node1,*node2,*i;
	int currNeigh;


	if (ProgramSpecificData.sexLinked && PedigreeGraph[nonfounder].sex == Male){
		return addEdgeToVertex(founder2,nonfounder, curLocus);
	}

	if (founder1 == founder2){
		if (PedigreeGraph[nonfounder].myLocus[curLocus].allele1 !=
			PedigreeGraph[nonfounder].myLocus[curLocus].allele2)
			return false;
	}

	if (!(addEdgeToVertex(founder1,nonfounder, curLocus) &&
			addEdgeToVertex(founder2,nonfounder, curLocus)))
			return false;
    	

	for (currNeigh=0;currNeigh<NumFounderNeighbours[founder1];currNeigh++){
		i=&(FoundersGraph[founder1][currNeigh]);
		if (i->id==founder2){ 
			if (((i->allele1==PedigreeGraph[nonfounder].myLocus[curLocus].allele1)
				&&  (i->allele2==PedigreeGraph[nonfounder].myLocus[curLocus].allele2))
				||
				((i->allele1==PedigreeGraph[nonfounder].myLocus[curLocus].allele2)
				&&  (i->allele2==PedigreeGraph[nonfounder].myLocus[curLocus].allele1)))
				return true;
			else
				return false;
		}

	}
	node1 = &(FoundersGraph[founder1][NumFounderNeighbours[founder1]]) ;
	node2 = &(FoundersGraph[founder2][NumFounderNeighbours[founder2]]) ;
	node1->id=founder2;
	node2->id=founder1;
	node1->allele1=PedigreeGraph[nonfounder].myLocus[curLocus].allele1;
	node2->allele1=PedigreeGraph[nonfounder].myLocus[curLocus].allele1;
	node1->allele2=PedigreeGraph[nonfounder].myLocus[curLocus].allele2;
	node2->allele2=PedigreeGraph[nonfounder].myLocus[curLocus].allele2;
	if (founder1 != founder2)
		NumFounderNeighbours[founder1]++;
	NumFounderNeighbours[founder2]++;
	return true;
}


/*****************************************************************************
*
*       Function        insert
*       Parameters		founder1 - founder allele index - out of all founders sl_alleles
*						nonfounder - sl_person index - out of all people
*						founder2 - founder allele index -  out of all founders sl_alleles
*       Output          true if edge is possibele
*       Description		Adds  edge to the sl_component 
*       Comments		if nonfounder is founder, add sl_alleles to the Alleles structure
******************************************************************************/
Bool insert(int founder1,int nonfounder,int founder2, int curLocus){
	Neighbours *node1,*node2,*i;
	int currNeigh;
/*	if (ProgramSpecificData.sexLinked && PedigreeGraph[nonfounder].sex == Male)
		return true;
*/
#if TREE_SEARCH == 0
	return insertNoTree( founder1, nonfounder, founder2, curLocus);
#endif
	/* The check performed now in connect commponent */
	/* anna 12/7
	if (PedigreeGraph[nonfounder].myLocus[curLocus].allele1==0)
	return true;
	*/

	if (ProgramSpecificData.sexLinked && PedigreeGraph[nonfounder].sex == Male){
		return addEdgeToVertex(founder2,nonfounder, curLocus);
	}


	if (founder1 == founder2){
		if (PedigreeGraph[nonfounder].myLocus[curLocus].allele1 !=
			PedigreeGraph[nonfounder].myLocus[curLocus].allele2)
			return false;
		//return addEdgeToVertex(founder2,nonfounder, curLocus);
		//return true;
	}

	if (PedigreeGraph[nonfounder].founder ){
		if (!(addEdgeToVertex(founder1,nonfounder, curLocus) &&
			addEdgeToVertex(founder2,nonfounder, curLocus)))
			return false;
	}

/*	PREVIOUS WRONG VERSION
	for (currNeigh=0;currNeigh<NumFounderNeighbours[founder1];currNeigh++){
		i=&(FoundersGraph[founder1][currNeigh]);
		//for (i=FoundersGraph[founder1];i!=NULL;i=i->next)
		if ((i->id==founder2) && (i->allele1==PedigreeGraph[nonfounder].myLocus[curLocus].allele1)
			&&  (i->allele2==PedigreeGraph[nonfounder].myLocus[curLocus].allele2))
			return true;
	}
*/
	for (currNeigh=0;currNeigh<NumFounderNeighbours[founder1];currNeigh++){
		i=&(FoundersGraph[founder1][currNeigh]);
		if (i->id==founder2){ 
			if (((i->allele1==PedigreeGraph[nonfounder].myLocus[curLocus].allele1)
				&&  (i->allele2==PedigreeGraph[nonfounder].myLocus[curLocus].allele2))
				||
				((i->allele1==PedigreeGraph[nonfounder].myLocus[curLocus].allele2)
				&&  (i->allele2==PedigreeGraph[nonfounder].myLocus[curLocus].allele1)))
				return true;
			else
				return false;
		}

	}

	node1 = &(FoundersGraph[founder1][NumFounderNeighbours[founder1]]) ;
	node2 = &(FoundersGraph[founder2][NumFounderNeighbours[founder2]]) ;
	node1->id=founder2;
	node2->id=founder1;
	node1->allele1=PedigreeGraph[nonfounder].myLocus[curLocus].allele1;
	node2->allele1=PedigreeGraph[nonfounder].myLocus[curLocus].allele1;
	node1->allele2=PedigreeGraph[nonfounder].myLocus[curLocus].allele2;
	node2->allele2=PedigreeGraph[nonfounder].myLocus[curLocus].allele2;
	if (founder1 != founder2)
		NumFounderNeighbours[founder1]++;
	NumFounderNeighbours[founder2]++;
	return true;
}


/*****************************************************************************
*
*       Function        connectComp
*       Parameters		none*						
*       Output          true if probability of the sl_component is bigger then 0
*       Description		Adds alle edges of rhe sl_component 
*       Comments		none*
******************************************************************************/
Bool connectComp(int curLocus, int curVector){

	int i;
	int founder1, founder2;
	Pedigree *pedPtr;
	for (i=0; i < PedigreeData.PedigreeSize; i++ ){
		pedPtr = PedigreeGraph + i;
		if (pedPtr->informative == false)
			continue;
		if (pedPtr->myLocus[curLocus].allele1 == 0)
			continue;
		if (pedPtr->founder){
			if (!insert(pedPtr->num*2, i ,pedPtr->num*2 + 1, curLocus))
				return false;
		}else{

			findFounders(pedPtr,&founder1,&founder2, curVector);
			if (!insert(founder1, i,founder2, curLocus))
				return false;
		}
	}
	return true;

}


