<?php
class DataBrowser
{
    /*********** property declaration ***********/
    // private $var = 'a default value';
	const TotalGraphAreaName 	= "totalGraphArea";
	const PartGraphAreaName		= "partGraphArea";
	const DataAreaName			= "dataArea";
	
	const PartGraphAreaTitle	= "partGraphTitle";
	
	const jobTypeTotal = "jobTypeTotal";
	const jobTypePart  = "jobTypePart" ;
	const jobTypeData  = "jobTypeData" ;
	
    /************ method declaration ***********/
	
	// displays the data browser for a given project
    public function displayBrowser($project, $experiment = false) {
	
		#If no SNP file exists, print error message
		$dir = USER_PROJECTS_DIR."/".$project."/raw";
		if (!is_file($dir."/".SORTED_SNP_DB_FILENAME)) {
			echo "<p style=\"color: red;\">The Data Browser can only be used for data uploaded in SNP format.</p>";
			return;
		}	

	
		?><script type="text/javascript" src="jsFunctions.js"></script>
	    <!--[if IE]><script language="javascript" type="text/javascript" src="./Flot/excanvas.min.js"></script><![endif]-->
		<script language="javascript" type="text/javascript" src="./Flot/jquery.js"></script>
		<script language="javascript" type="text/javascript" src="./Flot/jquery.flot.js"></script>
	    <script language="javascript" type="text/javascript" src="./Flot/jquery.flot.selection.js"></script>
		Data browser for project: <?php echo $project; ?> <br/> <?php
		
		self::displayExperimentsChoice($project, $experiment);
		self::createArea(self::TotalGraphAreaName, 850, 80);
		self::createArea(self::PartGraphAreaName, 850, 250);
		self::createArea(self::DataAreaName);
		
		self::initJS($project);		
    }
	
	private function createArea($name, $width = false, $height = false) {
		if (self::getAreaTitleByName($name)){?>
			<span name="<?php echo self::getAreaTitleByName($name) ?>" id="<?php echo self::getAreaTitleByName($name) ?>"></span>
		<?php }
		?><p><div name="<?php echo $name;?>" id="<?php echo $name;?>" style="<?php if ($width) echo "width:".$width."px;";?> <?php if ($height) echo "height:".$height."px;";?>"></div></p><?php
	}
	
	private function displayExperimentsChoice($project, $experiment) {
		$experiments = getFilesOfType(USER_PROJECTS_DIR."/".$project."/experiments", false);
		?>
		Choose an experiment:
		<!--<select class="experimentSelect" id="experimentSelect" onchange="this.blur();">-->
		<select class="experimentSelect" id="experimentSelect" onchange="showTotalGraph('<?php echo $project; ?>', (this.options[this.selectedIndex]).value);">
			<option value="" <?php if (!$experiment){ echo "selected"; }?> ><strong>Experiments &#9658;</strong></option>
			<?php
			foreach ($experiments as $exp) {				
				$directory 	= USER_PROJECTS_DIR."/".$project."/experiments/".$exp;
				
				#Omer 16/10/2011 - Don't show deleted experiments
				$isDeleted 	= is_file($directory."/".DELETED_FLAG);
				if ($isDeleted) {continue;}
				
				// show only the experiments that can have a data browser
				$varsTable = getVariablesTable($directory);
				if (!isset($varsTable[KEY_RUN_ANALYSIS]) || (
						  ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_MULTIPOINT_HMM) && 
						  ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_MULTIPOINT) &&
						  ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_LM_MARKERS) &&
						  ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_TWOPOINT) )) {continue;}
				
				#Omer 16/10/2011 - Don't display experiments that do not have an associated data file for at least one chromosome
				$results = file($directory."/.results", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
				$hasResultsFiles = false;
				foreach ($results as $res) {
					$res = explode(" ", $res);
					$link = $res[1];					
					if (!$link || !is_file(WEBDIR."/".$link."/collect_data.txt")) {continue;}
					$hasResultsFiles = true;
				}
				if (!$hasResultsFiles) {continue;}
				
				?><option value="<?php echo $exp?>" <?php if ($exp == $experiment) { echo "selected"; }?> ><?php echo $exp?></option><?php
			}
			?>
		</select>
		<?php
	}
	
	public function ajaxJobExecute($jobType, $parameters) {
		switch ($jobType) {
			case self::jobTypeTotal: return self::performAjaxJobTypeTotal($parameters); break;
			case self::jobTypePart:	 return self::performAjaxJobTypePart($parameters); break;
			case self::jobTypeData:	 return self::performAjaxJobTypeData($parameters); break;
			default: 				 return false; break;
		}
		return false;
	}
	
	public function getJobTypeTotal() {return self::jobTypeTotal;}
	public function getJobTypePart() {return self::jobTypePart;}
	public function getJobTypeData() {return self::jobTypeData;}
	
	
	public function getAreaTitleByName($areaName){
		switch ($areaName) {
			case self::PartGraphAreaName:  return self::PartGraphAreaTitle; break;
			default: 			return false; break;
		}
		return false;		
	}
	
	// requires $parameters['proj'], $parameters['exp']
	private function performAjaxJobTypeTotal($parameters) {
		$proj = $parameters['proj'];
		$exp = $parameters['exp'];
		$resultsArray = array();
		$directory 	= USER_PROJECTS_DIR."/".$proj."/experiments/".$exp;
		$info = getExpContentInfo($directory);
		$chromNum = 0;
		foreach ($info as $key => $table){
			$max = self::getMaxLodFor($proj, $exp, $key);
			if ($max == null) { $resultsArray = array(); $resultsArray[] = array(false); break; }
			if (preg_match("/chrom_([0-9]+|[Xx])/", $key, $num)) {
				$chromNum = (($num[1] == "X") || ($num[1] == "x")) ? 23 : $num[1];
			}
			else {$chromNum += 1;}			
			$resultsArray[] = array($chromNum, $max);
		}
		$final = array ('data' => $resultsArray);		
		$finalJson = json_encode($final, JSON_NUMERIC_CHECK);
		return $finalJson;
	}
	
	// requires $parameters['proj'], $parameters['exp'] and $parameters['chrom']
	private function performAjaxJobTypePart($parameters){
		$proj = $parameters['proj'];
		$exp = $parameters['exp'];
		$chrom = $parameters['chrom'];
		$directory 	= USER_PROJECTS_DIR."/".$proj."/experiments/".$exp;
		/*$info = getExpContentInfo($directory);
		$keys = array();
		foreach ($info as $key => $table){
			//$keys[] = "<a href=\"\" style=\"font-size: 60%\">$key</a>";
			$keys[] = "$key";
		}*/
		$results = file($directory."/.results", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$link = false;
		foreach ($results as $res) {
			$res = explode(" ", $res);
			$resFor = $res[0];
			$check = ($chrom == "23") ? "X" : (($chrom < 10) ? "0".$chrom : $chrom);
			if (preg_match("/$check/", $resFor)){
				$link = $res[1];
				break;
			}
		}
		if (!$link) { return false; }
		
		if (!is_file(WEBDIR."/".$link."/collect_data.txt")) {
			//return str_replace("\"", "", json_encode(array ('data' => array(false)))); 
			return json_encode(array('data' => array(false)), JSON_NUMERIC_CHECK);
		}
		$data = file(WEBDIR."/".$link."/collect_data.txt", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$varsTable = getVariablesTable($directory);
		if (isset($varsTable[KEY_RUN_ANALYSIS]) && ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_MULTIPOINT_HMM)) {
			$data = array_unique($data, SORT_STRING);
			self::orderData($data);
		}
		$resultsArray = array();
		$nonameLodArray = array();
		$nonameCount = 0;
		foreach ($data as $line){
			$line = explode("\t", $line);
			$name = str_replace("c", "", $line[0]);
			$lod = (($line[3] == "-Infinity") || ($line[3] == "nan")) ? -10 : $line[3];
			if ($name == "") {
				$nonameLodArray[$nonameCount] = $lod;
				$nonameCount += 1;
			}
			else {
				$nonamePart = 1/($nonameCount+1);
				for( $i = 0 ; $i < $nonameCount ; $i+=1){
					$newName = ($name - 1) + (($i + 1) * $nonamePart);
					$resultsArray[] = array($newName, $nonameLodArray[$i]);
				}
				$resultsArray[] = array($name, $lod);
				$nonameCount = 0;
				$nonameLodArray = array();
			}
		}
		
		$final = array ('data' => $resultsArray);
		//return implode(" ", $resultsArray);
		//$finalJson = str_replace("\"", "", json_encode($final));
		$finalJson = json_encode($final, JSON_NUMERIC_CHECK);
		return $finalJson;
		
	}
	
	private function performAjaxJobTypeData($parameters) {}
	
	private function initJS($project) {
		?>
		<script id="source" language="javascript" type="text/javascript">
		var dataFetched = false;
		var totalPlaceholder = $("#<?php echo self::TotalGraphAreaName; ?>");		
		
		$(function () {
		   var options = {
		        //lines: { show: true },
				grid: { hoverable: true , clickable: true , backgroundColor: { colors: ["#fff", "#eee"] }},
		        points: { show: true },
		        xaxis: { tickDecimals: 0, tickSize: 1 }
		    };
		    //var data = [];
		    //var placeholder = $("#<?php echo self::TotalGraphAreaName; ?>");
			var placeholder = totalPlaceholder;
			placeholder.unbind("plothover");
			placeholder.unbind("plotclick");
			//var placeholder = getElementById("<?php echo self::TotalGraphAreaName; ?>");
		    //$.plot(placeholder, data, options);
			
		    // fetch one series, adding to what we got
		    //var alreadyFetched = {};
		    
			function showTooltip(x, y, contents) {
				$('<div id="tooltip">' + contents + '</div>').css( {
					position: 'absolute',
					display: 'none',
					top: y + 5,
					left: x + 5,
					border: '1px solid #fdd',
					padding: '2px',
					'background-color': '#fee',
					opacity: 0.80
				}).appendTo("body").fadeIn(200);
			}

			var previousPoint = null;
			var x;
			var y;
			placeholder.bind("plothover", function (event, pos, item) {
				if (item) {
					if (previousPoint != item.datapoint) {
						previousPoint = item.datapoint;
						
						$("#tooltip").remove();
						var x = parseInt(item.datapoint[0]),
							y = item.datapoint[1].toFixed(2);
						
						showTooltip(item.pageX, item.pageY,
									"max LOD score of chromosome " + x + " is " + y);
					}
				}
				else {
					$("#tooltip").remove();
					previousPoint = null;            
				}
			});
			
			placeholder.bind("plotclick", function (event, pos, item) {
		        if (item) {
		            var x = parseInt(item.datapoint[0]);
					var project = '<?php echo $project;?>';
					var sel = document.getElementById('experimentSelect');
					var exp = (sel.options[sel.selectedIndex]).value;
					showChromGraph(project, exp, x);
		        }
		    });
			
			var project = '<?php echo $project;?>';
			var sel = document.getElementById('experimentSelect');
			var exp = (sel.options[sel.selectedIndex]).value;
			if (exp != "") {showTotalGraph(project, exp);}
			
			/*
		    $("select.experimentSelect").change(function () {
		        var select = $(this);
				var sel = this;
				if ((sel.options[sel.selectedIndex]).value == "") { return false; }

				var partArea = document.getElementById("<?php echo self::PartGraphAreaName; ?>");
				partArea.innerHTML = "";
				
				var title = document.getElementById('<?php echo self::getAreaTitleByName(self::PartGraphAreaName);?>');
				title.innerHTML = "";
				
				var Dataarea = document.getElementById("<?php echo self::DataAreaName; ?>");
				Dataarea.innerHTML = "";
				
				var area = document.getElementById("<?php echo self::TotalGraphAreaName; ?>");
				area.innerHTML = "Please wait, processing...<br/><img src=\"./images/waitspinner.gif\">";
				
		        // find the URL in the link right next to us 
		        //var dataurl = select.siblings('a').attr('href');
				var parameters = new Array();
				var proj = '<?php echo $_GET['proj'];?>';
				var exp = (sel.options[sel.selectedIndex]).value;
				var dataurl = "processAjaxRequest.php?proj="+proj+"&exp="+exp+"&<?php echo self::getJobTypeTotal(); ?>=true";
				
		        // then fetch the data with jQuery
		        function onDataReceived(series) {
					var data = [];
					//alert(series.data);
					// extract the first coordinate pair so you can see that
		            // data is now an ordinary Javascript object
		            var firstcoordinate = '(' + series.data[0][0] + ', ' + series.data[0][1] + ')';

		            //select.siblings('span').text('Fetched ' + series.label + ', first point: ' + firstcoordinate);

		            // let's add it to our current data
		            //if (!alreadyFetched[series.label]) {
		                //alreadyFetched[series.label] = true;
		                data.push(series);
		            //}
		            
		            // and plot all we got
		            $.plot(placeholder, data, options);
		         }
		        
		        $.ajax({
		            url: dataurl,
		            method: 'GET',
		            dataType: 'json',
		            success: onDataReceived,
					error:function (xhr, ajaxOptions, thrownError) {					
						area.innerHTML = "Operation could not be performed... Please contact web support.";
						//area.innerHTML += "<br>http://cbl-hap.cs.technion.ac.il/oren/webDevelopment/"+dataurl;
						//area.innerHTML += "<br>"+thrownError;
						return;					
					}   					
		        });
		    });*/
		});
		
		function showTotalGraph(project, experiment) {
			if ((experiment == "") || (project == "")) { return false; }
			var placeholder = totalPlaceholder;
			var options = {
		        //lines: { show: true },
				grid: { hoverable: true , clickable: true , backgroundColor: { colors: ["#fff", "#eee"] }},
		        points: { show: true },
		        xaxis: { tickDecimals: 0, tickSize: 1 }
		    };

			var partArea = document.getElementById("<?php echo self::PartGraphAreaName; ?>");
			partArea.innerHTML = "";
			
			var title = document.getElementById('<?php echo self::getAreaTitleByName(self::PartGraphAreaName);?>');
			title.innerHTML = "";
			
			var Dataarea = document.getElementById("<?php echo self::DataAreaName; ?>");
			Dataarea.innerHTML = "";
			
			var area = document.getElementById("<?php echo self::TotalGraphAreaName; ?>");
			area.innerHTML = "Please wait, processing...<br/><img src=\"./images/waitspinner.gif\">";
			
			// find the URL in the link right next to us 
			//var dataurl = select.siblings('a').attr('href');
			var parameters = new Array();
			var proj = project;
			var exp = experiment;
			var dataurl = "processAjaxRequest.php?proj="+proj+"&exp="+exp+"&<?php echo self::getJobTypeTotal(); ?>=true";			
			
			// then fetch the data with jQuery
			function onDataReceived(series) {				
				if (series.data[0][0] == false){
					area.innerHTML = "Operation could not be performed... Please contact web support.";
					return;
				}
				var data = [];
				//alert(series.data);
				// extract the first coordinate pair so you can see that
				// data is now an ordinary Javascript object
				var firstcoordinate = '(' + series.data[0][0] + ', ' + series.data[0][1] + ')';

				//select.siblings('span').text('Fetched ' + series.label + ', first point: ' + firstcoordinate);

				// let's add it to our current data
				//if (!alreadyFetched[series.label]) {
					//alreadyFetched[series.label] = true;
					data.push(series);
				//}
				
				// and plot all we got				
				$.plot(placeholder, data, options);
			 }
			 
			$.ajax({
				url: dataurl,
				method: 'GET',
				dataType: 'json',
				success: onDataReceived,
				error:function (xhr, ajaxOptions, thrownError) {					
					area.innerHTML = "Operation could not be performed... Please contact web support.";
					//area.innerHTML += "<br>http://cbl-hap.cs.technion.ac.il/oren/webDevelopment/"+dataurl;
					//area.innerHTML += "<br>"+thrownError;
					return;					
				}   
			});
		}
		
		function showChromGraph(project, experiment, chrom) {
		    //var data = [];
			dataFetched = false;
		    var placeholder = $("#<?php echo self::PartGraphAreaName; ?>");
			placeholder.unbind("plotselected");
			placeholder.unbind("plotunselected");
			
		    placeholder.bind("plotselected", function (event, ranges) {
		        //alert ("from: " + parseInt(ranges.xaxis.from) + " to: " + parseInt(ranges.xaxis.to));
				showData(project, experiment, chrom, parseInt(ranges.xaxis.from), parseInt(ranges.xaxis.to) );
				dataFetched = true;
		    });
			
		    placeholder.bind("plotunselected", function (event) {
				// do nothing
			});
			
			var dataarea = document.getElementById("<?php echo self::DataAreaName; ?>");
			dataarea.innerHTML = "";
			
			var area = document.getElementById("<?php echo self::PartGraphAreaName; ?>");
			area.innerHTML = "Please wait, processing...<br/><img src=\"./images/waitspinner.gif\">";
							
			var parameters = new Array();
			var dataurl = "processAjaxRequest.php?proj="+project+"&exp="+experiment+"&chrom="+chrom+"&<?php echo self::getJobTypePart(); ?>=true";
			
			// then fetch the data with jQuery
			function onDataReceived(series) {
				if (series.data[0][0] == false){
					area.innerHTML = "Operation could not be performed... Please contact web support.";
					return;
				}
				
				var ticksEvery = parseInt(((series.data).length / 30) + 0.99);
				var options = {
			        series: {
			            lines: { show: true },
			            points: { show: true }
			        },
			        legend: { noColumns: 2 },
			        xaxis: { tickDecimals: 0, tickSize: ticksEvery },
			        selection: { mode: "x" },
					grid: {
						backgroundColor: { colors: ["#fff", "#eee"] }
					}
			    };
				var data = [];
				
				var firstcoordinate = '(' + series.data[0][0] + ', ' + series.data[0][1] + ')';
				data.push(series);
				
				// set title
				var title = document.getElementById('<?php echo self::getAreaTitleByName(self::PartGraphAreaName);?>');				
				if (title != null){
					title.innerHTML = "";
					title.innerHTML = "Lod score graph for chromosome: " + chrom;
					title.innerHTML = title.innerHTML + "<br/>Select the region of interest by clicking and draging your mouse over it";
					title.innerHTML = title.innerHTML + "<br/><b>OR</b> show data between marker: ";
					dynamicAdd(title, "input", "text", "from", "10");
					title.innerHTML = title.innerHTML + " and marker: ";
					dynamicAdd(title, "input", "text", "to", "10");
					title.innerHTML = title.innerHTML + "&nbsp&nbsp";
					
					// adding the function
					var func = function() {											
											var parameters = new Array();
											parameters['proj'] = project;
											parameters['exp'] = experiment;
											parameters['chrom'] = chrom;
											parameters['from'] = document.getElementById("from").value;
											parameters['to'] = document.getElementById("to").value;
											PerformAjaxReuqest(parameters, '<?php echo self::DataAreaName; ?>', 'mapValues');
										}					
					
					dynamicAdd(title, "input", "button", null, null, "Submit", func);
				}
				
				// and plot all we got
				$.plot(placeholder, data, options);
			}
			
			$.ajax({
				url: dataurl,
				method: 'GET',
				dataType: 'json',
				success: onDataReceived,
				error:function (xhr, ajaxOptions, thrownError) {					
					area.innerHTML = "Operation could not be performed... Please contact web support.";
					//area.innerHTML += "<br>http://cbl-hap.cs.technion.ac.il/oren/webDevelopment/"+dataurl;
					//area.innerHTML += "<br>"+thrownError;
					return;					
				}   				
			});
		}
		
		function showData(project, experiment, chrom, from, to) {
			var area = document.getElementById("<?php echo self::DataAreaName; ?>");
			area.innerHTML = "Please wait, processing...<br/><img src=\"./images/waitspinner.gif\">";
			
			//var dataurl = "processAjaxRequest.php?proj="+project+"&exp="+experiment+"&chrom="+chrom+"&<?php echo self::getJobTypeData(); ?>=true";
			var parameters = new Array();
			parameters['proj'] = project;
			parameters['exp'] = experiment;
			parameters['chrom'] = chrom;
			parameters['from'] = from;
			parameters['to'] = to;
			
			PerformAjaxReuqest(parameters, '<?php echo self::DataAreaName; ?>', 'mapValues');
		}		
		
		function dynamicAdd (addTo, elem, elemType, elemId, elemSize, elemValue, elemOnclick) {
			//Create an elem type dynamically.  
			var element = document.createElement(elem);  
			
			//Assign different attributes to the element.  
			if (elemType != null) { element.type = elemType; }
			if (elemId != null) { element.id = elemId; }
			if (elemValue != null) { element.value = elemValue;}
			if (elemSize != null) { element.size = elemSize; }
			if (elemOnclick != null) { element.onclick = elemOnclick; }
			
			//Append the element in page (in span). 
			addTo.appendChild(element);
		}
		</script>
		<?php
	}
	
	public function getMaxLodFor($project, $experiment, $chrom) {
		$directory 	= USER_PROJECTS_DIR."/".$project."/experiments/".$experiment;
		if (!is_dir($directory) || (!is_file($directory."/.results"))){ return null; }
		
		$results = file($directory."/.results", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		
		foreach ($results as $res) {
			$res = explode(" ", $res);
			$resFor = $res[0];
			$link = $res[1];
			if (preg_match("/$chrom/", $resFor)) {break;}			
		}
		
		if (!is_file(WEBDIR."/".$link."/collect_data.txt")){ return null; }
		$data = file(WEBDIR."/".$link."/collect_data.txt", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$line = explode("\t", $data[0]);
		$max = (($line[3] == "-Infinity") || ($line[3] == "nan")) ? -10 : $line[3];
		foreach ($data as $line) {
			$line = explode("\t", $line);
			$name = $line[0];
			$lod = (($line[3] == "-Infinity") || ($line[3] == "nan")) ? -10 : $line[3];
			if ($lod > $max) { $max = $lod; }
		}
		return $max;
	}
	
	public function orderData(&$data) {
		$code = "\$line1 = explode(\"\t\", \$line1);
				\$line2 = explode(\"\t\", \$line2);
				return strnatcmp(\$line1[0], \$line2[0]);";
		
		usort($data, create_function('$line1,$line2', $code));
	}
	
}
?>