#!/usr/bin/perl -w
use submissionConfig;
use SharedConsts;

$config = new submissionConfig;

$binDir = "$config->{'FULL_DIR'}\/cgi-bin";
$maxLODDir = "$config->{'MAX_LODSCORE_FILE'}";

# for example : perl GlobalPost.pl 9d2bgoh0e3 /var/www/html/oren/webDevelopment/Submissions/9d2bgoh0e3
$projectName = $ARGV[0];
$projectsDir = $ARGV[1];

$graphMin = -3;
$graphMax = 4;

$max = $graphMin - 1;
	
# dvir 2011/08/03-04
my $submissionInfoFileText = `cat "$projectsDir\/$submissionConfig::SUBMISSION_INFORMATION_FILENAME"`;
# checking if this is a lm _ markers analysis
# if so than we will work with the file $SharedConsts::LOD_SCORES_DATA_FILENAME 
# that appears in each sub folder of chromozome, and contains
# col 1: the distance of the point of the lod score in CM
# col 2: the aggregate lod score for all the families at that distance
# col 3-.. : the lod score for the familiy num (which is written at the first line) for that distance.
# dvir 2011/08/23 -> now also two point works with this.
#Omer 4/9/2011 - now all graphs work with this
my @plotCommadsFilesName = ();
my %maxLodByFamilyNum = ();

# chromozomes dir are sub directories at the submission dir
my @chromDirsNamesAtSubmissionDir = `ls -d $projectsDir\/*.$projectName.*`;
for my $currChromDirName (@chromDirsNamesAtSubmissionDir) {
	chomp($currChromDirName);
	
	my $currChromLodScoresFileName = "$currChromDirName\/$SharedConsts::LOD_SCORES_DATA_FILENAME";
	my $currChromGnuPlotCommandsForShowResultsPageFileName = "$currChromDirName\/$SharedConsts::GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE_FILENAME";
	#print "$currChromDirName\n$currChromLodScoresFileName\n";
	
	if (! -e $currChromLodScoresFileName) {
		warn "file $currChromLodScoresFileName is missing";
		next;
	}
	if (! -e $currChromGnuPlotCommandsForShowResultsPageFileName) {
		warn "file $currChromGnuPlotCommandsForShowResultsPageFileName is missing";
		next;
	}
	
	push @plotCommadsFilesName, $currChromGnuPlotCommandsForShowResultsPageFileName;
			
	open LOD_SCORE_FILE, $currChromLodScoresFileName;		

	# connecting the col num to the familiy num
	# example for the headline : "postions        all     1       3       2"
	my $headline = <LOD_SCORE_FILE>;
	chomp($headline);
	# cell ~ column
	# cell 0 is "positions", cell 1 is "all" families lods, cell 2+ are lods by familiy
	my @familyNumByColNum = split $SharedConsts::LOD_SCORES_DATA_FIELD_SEPEREATOR, $headline;
	
	# checking if the max lods by family hash is empty
	if (scalar(keys %maxLodByFamilyNum) == 0) {
		for (my $currCol = 1;$currCol <= $#familyNumByColNum; ++$currCol) {
			my $currFamily = $familyNumByColNum[$currCol];
			$maxLodByFamilyNum{$currFamily} = $SharedConsts::MIN_LOD_SCORE_AT_LOD_GRAPH;
		}
	}
	
	# reading the lod scores to find out what is the max lod score for each family		
	while (my $currLine = <LOD_SCORE_FILE>) {
		chomp($currLine);
		
		my @lodScores = split $SharedConsts::LOD_SCORES_DATA_FIELD_SEPEREATOR, $currLine;
		
		# col 1 is the distance and not the lod, so it is irreleveant
		for (my $currCol = 1; $currCol <= $#lodScores; ++$currCol) {
			my $currFamilyLod = $lodScores[$currCol];
			my $currFamily = $familyNumByColNum[$currCol];
			
			# $currFamilyLod may be '' in case that we don't know what is the lod
			if ($currFamilyLod ne '' && $currFamilyLod > $maxLodByFamilyNum{$currFamily}) {
				$maxLodByFamilyNum{$currFamily} = $currFamilyLod;
			}
		}
	}
	close LOD_SCORE_FILE;
}

foreach my $currFamily (keys %maxLodByFamilyNum) {
	if ($graphMax <= $maxLodByFamilyNum{$currFamily}) {
		$maxLodByFamilyNum{$currFamily} = $graphMax-0.05;
	}
	if ($max <= $maxLodByFamilyNum{$currFamily}) {
		$max = $maxLodByFamilyNum{$currFamily};
	}
	
}
		
if (scalar(@plotCommadsFilesName) > 0) {
	# now we know what is the main lod for each family
	# we will update the gnu plot commands file
	my $plotCommandsFileNamesStr = join ' ' , @plotCommadsFilesName;
	my $commands = '';

	# concating all the commands for all the chromozomes
	for my $plotCommadsFile (@plotCommadsFilesName) {
		my $currFileCommands = `cat $plotCommadsFile`;
		my $currFileDir = `dirname $plotCommadsFile`;
		chomp($currFileDir);
		# g -> to do the substitue over all the lines
		$currFileCommands =~ s/$SharedConsts::LOD_SCORE_GRAPH_DIR_TEXT/$currFileDir/g;
		#print $commands;
		$commands.= $currFileCommands;
	}
	
	while (my ($familyNum, $maxLod) = each %maxLodByFamilyNum) {
		$str = $SharedConsts::MAX_LOD_SCORE_PER_FAMILY_STR_PREFIX.$familyNum;
		$commands =~ s/$str/$maxLod/g;
	}
	# giving the plots with the green line a new name
	$commands =~ s/$SharedConsts::SHOW_RESULTS_FILENAME_TEXT/${SharedConsts::SHOW_RESULTS_FILENAME_TEXT}_update/g;
	
	
	# comment : $SharedConsts::GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE_FILENAME
	# is now at the submission dir and not at the chromozomes dir
	$gnuplotCommandsFileName = "$projectsDir\/$SharedConsts::GNU_PLOT_COMMANDS_FOR_SHOW_RESULTS_PAGE_FILENAME";
	open COMMANDS_FILE, ">$gnuplotCommandsFileName";
	print COMMANDS_FILE $commands;
	close COMMANDS_FILE; 

	system("gnuplot $gnuplotCommandsFileName >> $projectsDir\/gnuplot_exe.out 2>> $projectsDir\/gnuplot_exe.error");
}	


$fatherMax = 0;
$ancestor = $projectName;
if (-e "$projectsDir\/$maxLODDir") {
	open MAX_FILE, "$projectsDir\/$maxLODDir";
	for $line (<MAX_FILE>) {
		if ($line =~ /fatherMax/) {
			@lineData = split /\s+/, $line;
			$fatherMax = $lineData[1];
		}
		if ($line =~ /ancestor/) {
			@lineData = split /\s+/, $line;
			$ancestor = $lineData[1];
		}
	}
	close MAX_FILE;
}

open MAX_FILE, ">>$projectsDir\/$maxLODDir";
print MAX_FILE "currentMax $max\n";
close MAX_FILE;

#Automate the file
open SUB_INFO, "$projectsDir\/submissionInformation";
@subData = <SUB_INFO>;
close SUB_INFO;
chomp($subData[1]);
if (-e "$config->{'FULL_DIR'}\/$subData[1]\/.automate") {
	system("export PERLLIB=$binDir; perl $binDir\/findSuspectAreas.pl $projectName $projectsDir $max $fatherMax $ancestor");
}