package InputValidation;
################################################################
# 		This file is the library for all functions,
#	which used in input validation process.
################################################################

use FindBin ();
use lib "$FindBin::Bin";
use Configuration;
use Processing;
use Notification;
use JavaExecutor;
use SharedConsts;

################################################################
################################################################
##			PUBLIC FUNCTIONS:
################################################################
##
sub parseRemoteInput();
sub parseLocalInput( $ );
sub printHash( $ $ \% );
sub initVariables( $ );	# For Debug purposes only - to know Hash of variables content
sub initAndCheckCommonVariables( \% \% );
sub initAndCheckVariablesFromSymplifiedFormat( \% \% );
sub initAndCheckVariablesFromSLFormat( \% \% );
sub checkPedigree( \% );
sub initTwoLocusVariables( \% \% );
sub initMultiPointVariables( \% \% );
sub convertToSLFormat( \% \% );
sub getTwoLocusUsedLoci( \% \% $ );
sub removeLociForTwoLocus( \% );
sub getUsedLoci( \% \% );
sub removeLoci( \% );
sub finishInputParsing( \% \% );
sub preparePointInput( \% );
sub executeCommand( $ \% );
##
################################################################
##			PRIVATE FUNCTIONS:
################################################################
##
sub getStartsOfChromosomes( \% );
sub updateThetas( \% \@);
sub getMOI( \% \% );
sub getTwoLocusMOI( \% \% );
sub checkFamilyID( $ \% );
sub convertToSimplyf ( $ \% );
sub convPersonSimplyf( $ );
sub checkPass( \% );
sub createDir( $ \% );
sub saveMail( $ \% );
sub createPassword( \% );
sub getAlleleNum( $ \% \% );
sub getMarkersNames( $  \% \% );
sub saveLast( \% );
sub getPedNames( \% );
sub getVariablesFromDataFile( \% );
sub isImportantFile( $ );
sub findMax( \@  );
sub getComplexity( $ $ $  \% );
sub sortArrayBySubField( \@ $ );
sub dayMonth();
sub shiftLine( \@ \% );
sub joinTwoLocusData( \% \% );
sub initTwoLocusDataFromSimplifiedFormat( \% \% );
sub initTwoLocusDataFromSLFormat( \% \% );
##
################################################################
################################################################




################################################################
################################################################
##			INPUT VALIDATION FUNCTIONS:
################################################################
################################################################



################################################################
# 		initVariables
#	Initializes hash with variables
#	For debugging and reading clearness only!!!
#	Parameters:
#					$_[0] - Running type
#	Returns:
#					Hash with variables
################################################################
sub initVariables( $ )
{
	my %hash = ();
	my $type = $_[0];
	&Processing::debug_print( "sub initVariables with type=($type)" );
	
	$hash{ email } = $Processing::UNCALCULATED_INFORMATION;					#	User's email
	$hash{ newDir } = $Processing::UNCALCULATED_INFORMATION;					# Project Directory
	$hash{ currentDay } = $Processing::UNCALCULATED_INFORMATION;			# Current Day and month
	$hash{ PROJ } = $Processing::UNCALCULATED_INFORMATION;				#	Project name
	$hash{ OUTPUT_FILE } = $Processing::OUTPUT_FILE;					#	Output file name
	$hash{ tmpPed } = $Processing::UNCALCULATED_INFORMATION;				# File with input .ped or .txt
	$hash{ runFormat } = $Processing::UNCALCULATED_INFORMATION;				# Format of input: "-simple" for copy/paste input,  "-sl" for file input 
	$hash{ pedfile } = $Processing::UNCALCULATED_INFORMATION;				# pedfile name
	$hash{ datafile } = $Processing::UNCALCULATED_INFORMATION;				#	dat file name
	$hash{ numMarkers } = $Processing::UNCALCULATED_INFORMATION;			# Number of markers in the data
	$hash{ part } = $Processing::UNCALCULATED_INFORMATION;						#	"PasteButton" for copy/paste input,  "BrowseButton" for file input
	if ( $type =~ /TwoLocus/ )
	{
		$hash{ linked } = $Processing::UNCALCULATED_INFORMATION;					# Start Marker calculation position
		
		$hash{ mutantFreq1 } = $Processing::UNCALCULATED_INFORMATION; 		# Frequency of mutant allele for first locus
		@{$hash{ usedMarkersId1 }} = ("[0]");			# Array with list of markers to use
		@{$hash{ usedMarkersNames1 }} = ("[]");			# Array with names of markers to use

		$hash{ mutantFreq2 } = $Processing::UNCALCULATED_INFORMATION; 		# Frequency of mutant allele for second locus
		@{$hash{ usedMarkersId2 }} = ("[0]");			# Array with list of markers to use
		@{$hash{ usedMarkersNames2 }} = ("[]");			# Array with names of markers to use
	}
	else
	{
		$hash{ mutantFreq } = $Processing::UNCALCULATED_INFORMATION; 		# Frequency of mutant allele
		@{$hash{ usedMarkersId }} = ("[0]");			# Array with list of markers to use
		@{$hash{ usedMarkersNames }} = ("[]");			# Array with names of markers to use
	}
	$hash{ mode } = $Processing::UNCALCULATED_INFORMATION;					# "rec" for Recessive, "dom" for Dominant, "complex" for Complex
	@{$hash{ penetrances }} = ("[]");				# Disease penetrances
	@{$hash{ alleleNum }} = ("");				# Array with lists for each marker of allele's frequences
	@{$hash{ markersNames }} = ("[]");	#Array with Markers names
	@{$hash{ pednames }} = ("[]");		#	Array with names of pedigrees ( small, smith )
	@{$hash{ chromosomes } } = ();		#	Array with first markers on each chromosome
	$hash{ program } = "2";				# Program to execute: "TwoPoint", "MultiPoint"...
	$hash{ sexLinked } = 0;				#	If SEX linked or no

	$hash{ errorExist } = $Processing::NO;			#	int value on purpose to return errors
										#	Possible values:	$Processing::NO, INPUT, SYSTEM
	$hash{ errorMsg } = "";				#	Error content

	return \%hash if ( $type =~ /TwoPoint/ );

# Multi and One Point Only

	@{$hash{ thetas }} = ("[]");			#	Array with thetas between markers
	if ( $type =~ /TwoLocus/ )
	{
#	First Locus
		$hash{ from1 } = $Processing::UNCALCULATED_INFORMATION;					# Start Marker calculation position
		$hash{ to1 } = $Processing::UNCALCULATED_INFORMATION;						# End Marker Calculation position
		$hash{ jump1 } = $Processing::UNCALCULATED_INFORMATION;					# Jump step of Calculation
		$hash{ offmap1 } = $Processing::UNCALCULATED_INFORMATION;					# Offmap calculaation distance
		$hash{ before1 } = $Processing::UNCALCULATED_INFORMATION;					# Before Marker
		$hash{ after1 } = $Processing::UNCALCULATED_INFORMATION;					# After Marker
		$hash{ settings1 } = $Processing::UNCALCULATED_INFORMATION;					# Mode of calculation: 

#	Second Locus
		$hash{ from2 } = $Processing::UNCALCULATED_INFORMATION;					# Start Marker calculation position
		$hash{ to2 } = $Processing::UNCALCULATED_INFORMATION;						# End Marker Calculation position
		$hash{ jump2 } = $Processing::UNCALCULATED_INFORMATION;					# Jump step of Calculation
		$hash{ offmap2 } = $Processing::UNCALCULATED_INFORMATION;					# Offmap calculaation distance
		$hash{ before2 } = $Processing::UNCALCULATED_INFORMATION;					# Before Marker
		$hash{ after2 } = $Processing::UNCALCULATED_INFORMATION;					# After Marker
		$hash{ settings2 } = $Processing::UNCALCULATED_INFORMATION;		# Mode of calculation: 
	}
	else
	{
		$hash{ from } = $Processing::UNCALCULATED_INFORMATION;					# Start Marker calculation position
		$hash{ to } = $Processing::UNCALCULATED_INFORMATION;						# End Marker Calculation position
		$hash{ jump } = $Processing::UNCALCULATED_INFORMATION;					# Jump step of Calculation
		$hash{ offmap } = $Processing::UNCALCULATED_INFORMATION;					# Offmap calculaation distance
		$hash{ settings } = $Processing::UNCALCULATED_INFORMATION;		# Mode of calculation: 
	}																	#	middle - middle point between "from" and "to"
																		#	each - at each marker + "jump" + "offset"
																		#	custom - each marker between "from" and "to" + "jump" + "offset"
	
	return \%hash;
}
################################################################



################################################################
#			finishInputParsing
#	Finish checking input and prepeare to execute SL
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub finishInputParsing( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "sub finishInputParsing" );

	if ( $variables{program} =~ /TwoLocus/)
	{
		$variables{ linked } = $inputForm{ "link" } if ( $variables{ part } =~ /PasteButton/ );
		@{ $variables{usedMarkersNames1} }  = ();
		for ( $i = 0; $i <= $#{$variables{usedMarkersId1out}}; $i ++ )
		{
			my $id = ${ $variables{ usedMarkersId1out } }[ $i ] - 1;
			my $name = ${ $variables{ markersNames } }[ $id ];
			push @{ $variables{usedMarkersNames1} }, $name;
		}
		@{ $variables{usedMarkersNames2} }  = ();
		for ( $i = 0; $i <= $#{$variables{usedMarkersId2out}}; $i ++ )
		{
			my $id = ${ $variables{ usedMarkersId2out } }[ $i ] - 1;
			my $name = ${ $variables{ markersNames } }[ $id ];
			push @{ $variables{usedMarkersNames2} }, $name;
		}
	}
	else
	{
		@{ $variables{usedMarkersNames} }  = ();
		for ( $i = 0; $i <= $#{$variables{usedMarkersIdout}}; $i ++ )
		{
			my $id = ${ $variables{ usedMarkersIdout } }[ $i ] - 1;
			my $name = ${ $variables{ markersNames } }[ $id ];
			push @{ $variables{usedMarkersNames} }, $name;
		}
	}

# Check the size of input pedfile
	return &Processing::makeSystemError("Failed open pedfile ($variables{ newDir }\/$variables{ pedfile } )", \%variables)
		unless ( open PED, "$variables{ newDir }\/$variables{ pedfile }" );
	my @ped = <PED>;
	close PED;
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														

	return &Processing::makeSystemError("Failed enter project directory ($variables{ newDir })", \%variables)
		unless ( chdir( $variables{ newDir } )  );

	%variables = %{ &Processing::makeLink( "$variables{ pedfile }", "pedfile.dat" , \%variables )};
	%variables = %{ &Processing::makeLink( "$variables{ datafile }", "datafile.dat", \%variables )};
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														
	$variables{ pedfile } = "pedfile.dat";
	$variables{ datafile } = "datafile.dat";

	return &Processing::makeSystemError("Failed enter project directory ($variables{ newDir })", \%variables)
		unless ( chdir( $Processing::RUN_DIR )  );

	@{$variables{ pednames } } = @{&getPedNames( \%variables )};
	&Processing::debug_print( "Pednames = @{$variables{ pednames }}" );

	&saveMail($variables{ program },\%variables);
	
	%variables = %{ &getStartsOfChromosomes( \%variables ) };
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														

	$variables{ OUTPUT_FILE } = "$variables{ PROJ }.$variables{ currentDay }.$Processing::OUTPUT_FILE";

	return \%variables;
}
################################################################


################################################################
#			getStartsOfChromosomes
#	Creates array of first markers on each chromosome
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub getStartsOfChromosomes( \% )
{
	%variables = %{ $_[0] };

	push @{$variables{ chromosomes } }, 1; 
	for ( my $i = 0; $i <= $#{ $variables{ thetas } }; $i++ )
	{
		push @{$variables{ chromosomes } }, ($i+2) if ( abs( ${ $variables{ thetas } }[$i] ) >= 0.4999 );
	}
	return \%variables;
}
################################################################



################################################################
#			convertToSLFormat
#	Converts Symplified format to SL format
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub convertToSLFormat( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	
	&Processing::debug_print("started convertToSLFormat, number of markers is ($variables{ numMarkers })");
	
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		%variables = %{ &getTwoLocusMOI( \%variables, \%inputForm ) };
	
		$variables{ mutantFreq1 } =  $inputForm{ freqA1 };
		$variables{ mutantFreq2 } =  $inputForm{ freqB1 };
	}
	else
	{
		%variables = %{ &getMOI( \%variables, \%inputForm ) };
	
		$variables{ mutantFreq } =  $inputForm{ freq };
	}
	$tempFile = "$variables{ newDir }\/tmp.dat";
  	%variables = %{ &getAlleleNum( $tempFile,  \%variables, \%inputForm ) };
  	%variables = %{ &getMarkersNames( $tempFile, \%variables, \%inputForm ) };

	my $name = "$variables{ PROJ }.sl-input.$variables{ currentDay }";
	%variables = %{ &JavaExecutor::simpleToSLFormat( "$variables{ newDir }\/$name" , \%variables )};
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														

	return &Processing::makeSystemError("Failed enter project directory ($variables{ newDir })", \%variables)
		unless ( chdir( $variables{ newDir } )  );

	$variables{ datafile } = "$name.dat";
	$variables{ pedfile } = "$name.ped";
	
	return &Processing::makeSystemError("Cannot open file $variables{ datafile }", \%variables)
		unless ( open DATA, $variables{ datafile } );
	my @data = <DATA>;
	close DATA;
	&Processing::debug_print("Created data file is: (\n@data)");
	
	return &Processing::makeSystemError("Cannot open file $variables{ pedfile }", \%variables)
		unless ( open PED, $variables{ pedfile } );
	my @ped = <PED>;
	close PED;
	&Processing::debug_print("Created ped file is: (\n@ped)");

	return &Processing::makeSystemError("Failed enter project directory ($variables{ newDir })", \%variables)
		unless ( chdir( $Processing::RUN_DIR )  );

	&Processing::debug_print("finished converting Simple to SL formats");
	return \%variables;
}
################################################################



################################################################
#			checkPedigree
#	Checks the pedigree with pedcheck program. If find any problem - sends an email
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub checkPedigree( \% )
{
	%variables = %{ $_[0] };
	
	&Processing::debug_print("Start pedcheck...");
	my $cDir = &getcwd();
	%variables = %{&Processing::makeSystemError("Failed enter project directory ($variables{ newDir })", \%variables)}
		unless ( chdir( $variables{ newDir } )  );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	my $ped = "out.ped";
	my $dat = "out.dat";
	
	%variables = %{&Processing::makeSystemError("Cannot open file $variables{ PROJ }.input.$variables{ currentDay }.txt", \%variables)}
		unless ( open INPUT, "$variables{ PROJ }.input.$variables{ currentDay }.txt" );
	%variables = %{&Processing::makeSystemError("Cannot open file ($ped)", \%variables)}
		unless ( open OUTPUT, ">$ped" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	foreach $line(<INPUT>)
	{
		if ($line !~ /^\s*$/)
		{
			$line =~ s/^\s+//sig;
			$line =~ s/\?/0/g;
			print OUTPUT $line;
		}
	}
	close INPUT;
	close OUTPUT;

	%variables = %{&Processing::makeSystemError("Cannot open file ($dat)", \%variables)}
		unless ( open DATA, ">$dat" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	close DATA;

	if ( $variables{ part } =~ /BrowseButton/ )			#superlink format
	{
		&Processing::debug_print("IN SL FORMAT<br>\n");

		return $Processing::ERROR if ( $Processing::ERROR =~ &checkFamilyID($ped, \%variables) );

		&convertToSimplyf($ped, \%variables );
		&Processing::debug_print("Failed to rename ($ped.tmp) to ($ped)") 
			unless (rename ("$ped.tmp", "$ped") );
	}
	
	if ( $variables{ sexLinked } == 0 )
	{
		my $sex = 1;
		&Processing::debug_print("CHECKING PEGIREE ... (Data is not sex linked)\n");
	
		%variables = %{&executeCommand("$Configuration::PROG_DIR\/$Configuration::PEDCHECK_PROG $dat $ped $sex",
			\%variables) };
		return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	}

	if ( -s "Out.txt" )
	{
		%variables = %{&Processing::makeSystemError("Failed working with (Out.txt) in the checkPedigree function", \%variables)}
			unless( open OUT, "Out.txt" );
		$myOut = join "\n", <OUT>;
		close OUT;

		%variables = %{&Processing::makeInputError($myOut, \%variables)};
		return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	}

	%variables = %{&Processing::makeSystemError("Failed to change directory to ($cDir)", \%variables) }
		unless ( chdir($cDir) );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	
	&Processing::debug_print("Finished checking pedigree <br>");
	return $Processing::SUCCESS;
}
################################################################



################################################################
#			initAndCheckVariablesFromSLFormat
#	Initializes and checks variables from SL format files 
#			(datafile and pefile)
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initAndCheckVariablesFromSLFormat( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print("Init And Check Variables From SL Format");

	$variables{ runFormat } = "-sl";
	%variables = %{&Processing::makeInputError("No pedfile uploaded.\n Please return and upload the missing file.", \%variables)}
		unless ( exists $inputForm{ pedfile } || defined ($inputForm{ pedfile }) || ($inputForm{ pedfile } ne "")  );
	%variables = %{&Processing::makeInputError("No datafile uploaded.\n Please return and upload the missing file.", \%variables)}
		unless ( exists $inputForm{ datafile } || defined ($inputForm{ datafile }) || ($inputForm{ datafile } ne "")  );
	return \%variables unless( $variables{ errorExist } =~ $Processing::NO );

	$variables{ pedfile } = $inputForm{ pedfile };
	$variables{ datafile } = $inputForm{ datafile };

	if ( $Processing::YES == $Configuration::ONLINE )
	{
		&Processing::saveToFile( "$variables{ newDir }\/$variables{ tmpPed }", $variables{ pedfile }, \%variables);
		&Processing::saveToFile( "$variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.ped", $variables{ pedfile },  \%variables);
		&Processing::saveToFile( "$variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.dat", $variables{ datafile }, \%variables);
	}
	else
	{
		%variables = %{&Processing::makeSystemError("Failed to find ($variables{ pedfile }) between the files (".<*>.")", \%variables )}
			unless( -e $variables{ pedfile } );

		%variables = %{&Processing::makeSystemError("Failed to copy ($variables{ pedfile }) to the file ($variables{ newDir }\/$variables{ tmpPed })", \%variables )}
			unless( link( $variables{ pedfile }, "$variables{ newDir }\/$variables{ tmpPed }" ) );
		%variables = %{&Processing::makeSystemError("Failed to copy ($variables{ pedfile }) to the file ($variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.ped)", \%variables )}
			unless( link( $variables{ pedfile }, "$variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.ped" ) );
		%variables = %{&Processing::makeSystemError("Failed to copy ($variables{ datafile }) to the file ($variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.dat)", \%variables )}
			unless( link( $variables{ datafile }, "$variables{ newDir }\/$variables{ PROJ }.sl-input.$variables{ currentDay }.dat" ) );
	}
	return \%variables unless( $variables{ errorExist } =~ $Processing::NO );

	$variables{ pedfile } = "$variables{ PROJ }.sl-input.$variables{ currentDay }.ped";
	$variables{ datafile } = "$variables{ PROJ }.sl-input.$variables{ currentDay }.dat";
	
	%variables = %{ &getVariablesFromDataFile( \%variables ) };
	return \%variables unless( $variables{ errorExist } =~ $Processing::NO );
	&Processing::debug_print( "after getVariablesFromDataFile:	size of thetas=(".($#{ $variables{ thetas } }).")");

	%variables = %{ &JavaExecutor::commentFiles( "$variables{ newDir }\/$variables{ datafile }", "$variables{ newDir }\/$variables{ pedfile }" , \%variables )};

	if ( $Processing::NO !~ $variables{ errorExist } )	
	{											#	From prev. func. we always get only SYSTEM and here it is INPUT error
		$variables{ errorExist } = $Processing::INPUT;
		return \%variables;
	}

	return \%variables;
}
################################################################



################################################################
#			initAndCheckVariablesFromSymplifiedFormat
#	Initializes and checks variables from simplified format file
#		( Text Area input parsing)
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initAndCheckVariablesFromSymplifiedFormat( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };

	&Processing::debug_print("In the initAndCheckVariablesFromSymplifiedFormat");
	$variables{ runFormat } = "-simple";
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		my @data = ();
		if ( $variables{ program } !~ /TwoPoint/ )
		{
			my @famData = split /\n/, $inputForm{pedArea};
			my @mark1Data = split /\n/, $inputForm{marker1Area};
			my @mark2Data = ();
			@mark2Data = split /\n/, $inputForm{marker2Area} if ( defined $inputForm{marker2Area} );
			while ( ($#famData>=0) && ($famData[$#famData] !~ /\S/ ) ){ pop @famData; }; 
			while ( ($#mark1Data>=0) && ($mark1Data[$#mark1Data] !~ /\S/ ) ){ pop @mark1Data; }; 
			while ( ($#mark2Data>=0) && ($mark2Data[$#mark2Data] !~ /\S/ ) ){ pop @mark2Data; };
			
			my $choped = chop $famData[$#famData];
			$famData[$#famData] = $famData[$#famData].$choped if ( $choped !~ /\s+/ );
			$choped = chop $mark1Data[$#mark1Data];
			$mark1Data[$#mark1Data] = $mark1Data[$#mark1Data].$choped if ( $choped !~ /\s+/ );
			$choped = chop $mark2Data[$#mark2Data];
			$mark2Data[$#mark2Data] = $mark2Data[$#mark2Data].$choped if ( $choped !~ /\s+/ );
			
			$famData[$#famData] = $famData[$#famData]."\n" if ( $famData[$#famData] !~ /\n/ );
			$mark1Data[$#mark1Data] = $mark1Data[$#mark1Data]."\n" if ( $mark1Data[$#mark1Data] !~ /\n/ );
			$mark2Data[$#mark2Data] = $mark2Data[$#mark2Data]."\n" if ( ( $#mark2Data >= 0 ) && ( $mark2Data[$#mark2Data] !~ /\n/ ));
			chop ( @famData, @mark1Data, @mark2Data );
			my $f = 0;
			my $m1 = 0;
			my $m2 = 0;
			my $finished = 0;
			while ( $finished eq 0 )
			{
				while ( ( ( $famData[$f] !~ /\S/ ) || ( $famData[$f] =~ /pedigree:/ ) ) && ( $f <= $#famData ) ) 
				{
					push @data, $famData[$f] if ( $famData[$f] =~ /pedigree:/ );
					$f++;
				}
				while ( ( $m1 <= $#mark1Data ) && ( ( $mark1Data[$m1] !~ /\S/ ) || ( $mark1Data[$m1] =~ /pedigree:/ ) ) ) {$m1++;}
				while ( ( $m2 <= $#mark2Data ) && ( ( $mark2Data[$m2] !~ /\S/ ) || ( $mark2Data[$m2] =~ /pedigree:/ ) ) ) {$m2++;}
				
				if ( defined $mark2Data[$m2] )
				{
					push @data, "$famData[$f] \t $mark1Data[$m1] \t $mark2Data[$m2]";
				}
				else
				{
					push @data, "$famData[$f] \t $mark1Data[$m1]";
				}
				$f++;$m1++;$m2++;
				$finished = 1 if ( ( $f eq (1+$#famData) ) || ( $m1 eq (1+$#mark1Data) ) );
				$finished = 1 if ( ( $#mark2Data >= 0 ) && ( $m2 eq (1+$#mark2Data) ) );
			}
			&Processing::debug_print("f=($f), m1=($m1), m2=($m2)");
			&Processing::debug_print("sizes of famData=($#famData), m1=($#mark1Data), m2=($#mark2Data)");
			return &Processing::makeInputError("Wrong datafile uploaded.\n".
				"Please check that each person has all markers in both windows and each marker data belongs to person.", \%variables)
				unless ( ( ( $f eq (1+$#famData) ) && ( $m1 eq (1+$#mark1Data) ) ) ||
					( ( $f eq (1+$#famData) ) && ( $m1 eq (1+$#mark1Data) ) && ( $#mark2Data >= 0 ) && ( $m2 eq (1+$#mark2Data) ) )
				);

			$variables{ numMarkers } = $inputForm{numMarkers1};
			$variables{ numMarkers1 } = $inputForm{numMarkers1} if ( $inputForm{ "link" } =~ /Unlinked/ );
			$variables{ numMarkers } += $inputForm{numMarkers2} if ( $inputForm{ "link" } =~ /Unlinked/ );
			$variables{ numMarkers2 } = $inputForm{numMarkers2} if ( $inputForm{ "link" } =~ /Unlinked/ );
		}
		else
		{
			@data = split /\n/, $inputForm{textArea};
			$variables{ numMarkers } = $inputForm{numMarkers};
		}
		$variables{ pedfile } = join "\n", @data;
	}
	else
	{
		$variables{ pedfile } = $inputForm{textArea};
		$variables{ numMarkers } = $inputForm{numMarkers};
	}
	&Processing::saveToFile( $variables{ newDir }."\/$variables{ tmpPed }",$variables{ pedfile }, \%variables );
	&Processing::debug_print("Pedfile is: ($variables{ pedfile })");
	&Processing::debug_print("Number of markers is: ($variables{ numMarkers })");

	return \%variables;
}
################################################################



################################################################
#			initAndCheckCommonVariables
#	Inititializes and checks variables received from input form
#		( email, password and etc. )
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initAndCheckCommonVariables( \% \% )
{
	%variables = %{$_[0]};
	%inputForm = %{$_[1]};
	&Processing::debug_print("Start of initAndCheckCommonVariables");

	$variables{ part } =  $inputForm{ R1 };
	$variables{ mail } = $inputForm{ email };
	$variables{ PROJ } = $inputForm{ name };
	$variables{ PROJ } =~ s/\s//sig;

	$variables{ currentDay } = &dayMonth();
	$variables{ program } = $inputForm{ program };

	if ( $Processing::YES == $Configuration::ONLINE )
	{
		$variables{ pass } = $inputForm{ pass };

		return \%variables if ( $Processing::SUCCESS !~ &checkPass( \%variables ) );
		return \%variables if ( $Processing::SUCCESS !~ &createDir( $Configuration::HOME_REAL_DIR, \%variables ) );
		return \%variables if ( $Processing::SUCCESS !~ &createPassword( \%variables ) );
	}
	else
	{
		return &Processing::makeSystemError( "Input Validation:   sub initAndCheckCommonVariables: Failed to mkdir($Configuration::RES_DIR\\$inputForm{ newDir })  ", \%variables )
			unless ( mkdir( "$Configuration::RES_DIR\/$inputForm{ newDir }", 0777));

#	Make the path to $variables{ newDir } appropriate

		$currDir = getcwd();
		&Processing::debug_print("current dir is ($currDir)");
		chdir "$Configuration::RES_DIR\/$inputForm{ newDir }";
		$dir = getcwd();
		&Processing::debug_print("dir is ($dir)");
		chdir $currDir;
		
#		@words = split '$Processing::RUN_DIR', $dir;	#	WHY ON HELL IT DOES NOT WORK???
		@words = split /cgi-bin/, $dir;	#	TEMPORARY SOLUTION
		
		&Processing::debug_print("Size of words = $#words, Word0=($words[0]), word1=($words[1]), runDir = ($Processing::RUN_DIR)");
		$variables{ newDir } = ".".$words[1];
		&Processing::debug_print("Directory NewDir = ($variables{ newDir }), Word0=($words[0]), word1=($words[1]), runDir = ($Processing::RUN_DIR)");
#
	}
	&Processing::debug_print("Moving the LOG files (errorFile + debugFile) to the execution directory ($Processing::RUN_DIR\/$variables{ newDir })");
	%variables = %{&executeCommand
	(
		"mv $Processing::DEBUG_FILE \"$Processing::RUN_DIR\/$variables{ newDir }\/$Processing::DEFAULT_DEBUG_FILE\"",
		\%variables
	)};
	%variables = %{&executeCommand
	(
		"mv $Processing::ERROR_FILE \"$Processing::RUN_DIR\/$variables{ newDir }\/$Processing::DEFAULT_ERROR_FILE\"",
		\%variables
	)} if ( -e $Processing::ERROR_FILE );
	&Processing::setLogFiles( "$Processing::RUN_DIR\/$variables{ newDir }\/" );

	&Processing::debug_print("End of directory & password creating");

	$variables{ tmpPed } =  "$variables{ PROJ }.input.ped";
	$variables{ tmpPed } =~ s/\.ped/\.$variables{ currentDay }\.txt/sig;

	return \%variables;
}
################################################################




################################################################
#			initMultiPointVariables
#	Inititializes and checks variables received from 
#	MultiPoint input form only
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initMultiPointVariables( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "Init MultiPoint and OnePoint Variables");
	
	my $runStraightFromSL = 0;
	if ( ! $runStraightFromSL )
	{
		if ( $variables{ program } =~ /OnePoint/ )
		{
			&Processing::debug_print( "Init One Point Parameters");
			$variables{ from } = $inputForm{ from };
			&Processing::debug_print( "The point is ($variables{ from })");
			return \%variables;
		}
		
		&Processing::debug_print( "Init Multi Point Parameters");
		my $settings = $inputForm{ prog };
	
		if ( $settings =~ /gh1/ )
		{
			$variables{ from } = $inputForm{ from1 };
			$variables{ to } = $inputForm{ from1 };
			$variables{ settings } = "middle";
		}
		elsif ( $settings =~ /gh2/ )
		{
			$variables{ from } = $inputForm{ from2 };
			$variables{ to } = $inputForm{ end2 };
			$variables{ settings } = "middle";
		}
		elsif ( $settings =~ /gh3/ )
		{
			$variables{ from } = 1;
			$variables{ to } = "last";
			$variables{ jump } = "zero";
			$variables{ offmap } = 0;
			$variables{ settings } = "each";
		}
		elsif ( $settings =~ /gh4/ )
		{
			$variables{ from } = 1;
			$variables{ to } = "last";
			$variables{ jump } = "one";
			$variables{ offmap } = $inputForm{ off4 };
			$variables{ settings } = "each";
		}
		elsif ( $settings =~ /gh5/ )
		{
			$variables{ from } = 1;
			$variables{ to } = "last";
			$variables{ jump } = "two";
			$variables{ offmap } = $inputForm{ off5 };
			$variables{ settings } = "each";
		}
		elsif ( $settings =~ /gh6/ )
		{
			$variables{ from } = $inputForm{ from6 };
			$variables{ to } = $inputForm{ end6 };
			$variables{ jump } = $inputForm{ jump6 };
			$variables{ offmap } = $inputForm{ off6 };
			$variables{ settings } = "custom";
		}
		else
		{
			return &Processing::makeInputError("Unknown prog option in inputForm hash!!!", \%variables);
		}
	}
	else
	{
		&Processing::debug_print( "Superlink input format");
		return &Processing::makeSystemError("Failed to open datafile ($variables{ newDir }\/$variables{ datafile })", \%variables)
			unless (open DATA, "$variables{ newDir }\/$variables{ datafile }");
		my @data = <DATA>;
		close DATA;
		my $poped = pop @data;
		while ( ( $#data >= 0 ) && ( $poped !~ /\S/ ) )
		{
			$poped = pop @data;
		}
		&Processing::debug_print( "Data line is ($poped)");

		if ( ($poped =~ /\(/) && ($poped =~ /\)/) )
		{
			&Processing::debug_print( "initMultiPointVariables: OnePoint => poped= ($poped)");
			$variables{ program } = "SingleLocus.OnePoint";
			return \%variables;
		}
		my @parts = split /\s+/, $poped;
		while ( $parts[0] !~ /\S/ )
		{
			shift @parts;
		}
		$variables{ jump } = "";

		if ( $poped =~ /-s/ )
		{
			$variables{ settings } = "custom";
			$variables{ jump } = $parts[1] if ( $parts[1] > 0 );
			$variables{ from } = $parts[2];
			$variables{ to } = $parts[3];
			$variables{ offmap } = $parts[5] 
				if ( ( $#parts == 5) && ( $parts[4] =~ /-o/ ) && ( $parts[5] >= 0 ) );
			
			return &Processing::makeInputError("Wrong datafile format - wrong command options format!!!", \%variables)
				if ( ( $variables{ from } > $variables{ to } )
				  || ( $variables{ from } < 1 ) 
				  || ( $variables{ to } < 1 ) 
				  || ( $variables{ jump } eq "" )
				 );
		}
		elsif ( $poped =~ /-n/ )
		{
			$variables{ settings } = "each";
			$variables{ jump } = "zero" if ( $parts[1] =~ /1/ );
			$variables{ jump } = "one" if ( $parts[1] =~ /2/ );
			$variables{ jump } = "two" if ( $parts[1] =~ /3/ );
			$variables{ from } = $parts[2];
			$variables{ to } = $parts[3];
			$variables{ offmap } = $parts[5] 
				if ( ( $#parts == 5) && ( $parts[4] =~ /-o/ ) && ( $parts[5] >= 0 ) );
			
			return &Processing::makeInputError("Wrong datafile format - wrong command options format!!!", \%variables)
				if ( ( $variables{ from } > $variables{ to } )
				  || ( $variables{ from } < 1 ) 
				  || ( $variables{ to } < 1 ) 
				  || ( $variables{ jump } eq "" )
				 );
		}
		else
		{
			$variables{ settings } = "jumps";
			$variables{ from } = $parts[0];
			$variables{ jump } = $parts[1];
			$variables{ to } = $parts[2];
		}
	}

	&Processing::debug_print( "settings = ($variables{ settings })");
	&Processing::debug_print( "from = ($variables{ from })");
	&Processing::debug_print( "to = ($variables{ to })");
	&Processing::debug_print( "jump = ($variables{ jump })");
	&Processing::debug_print( "offmap = ($variables{ offmap })");

	return &Processing::makeInputError("Wrong final marker: $variables{ to }. Maximal possible marker to use in calculations is: $variables{ numMarkers }", \%variables)
		unless ( ($variables{ to } =~ /\D/) || ( ($variables{ to } =~ /\d+/) && ($variables{ to } < $variables{ numMarkers }) ) );

	if ( defined $inputForm{ rec } )
	{
		&Processing::debug_print( "Simplified input format");
		@{ $variables{ thetas } } = split /,/, $inputForm{ rec };
		for( my $th = 0; $th <= $#{ $variables{ thetas } }; $th++ )
		{
			${ $variables{ thetas } }[$th] = &Processing::getThetaFromCM( ${ $variables{ thetas } }[$th] );
		}
		&Processing::debug_print( "Thetas from Simplified input format file are (@{ $variables{ thetas } })");
	}
	else
	{
		&Processing::debug_print( "Superlink input format");
		return &Processing::makeSystemError("Failed to open datafile ($variables{ newDir }\/$variables{ datafile })", \%variables)
			unless (open DATA, "$variables{ newDir }\/$variables{ datafile }");
		my @data = <DATA>;
		close DATA;
		my $poped = pop @data;
		while ( ( $#data >= 0 ) && ( $poped !~ /\S/ ) )
		{
			$poped = pop @data;
		}
		
		&Processing::debug_print( "Thetas from SL format file are (@{ $variables{ thetas } })");
		return &Processing::makeInputError("Wrong datafile format - wrong number of thetas!!!", \%variables)
			if ( $#{ $variables{ thetas } } < ( $variables{ to } - 2 ) );	# size is counted from -1 and markers from 1

		return &Processing::makeSystemError("Failed to open datafile ($variables{ newDir }\/$variables{ pedfile })", \%variables)
			unless (open PED, "$variables{ newDir }\/$variables{ pedfile }");
		my @ped = <PED>;
		close PED;
		foreach my $pedLine ( @ped )
		{
			my @words = split "<<", $pedLine;
			@words = split /\s+/, $words[0];
			return &Processing::makeInputError("Wrong pedfile format - wrong number of markers", \%variables)
				if ( $#words < (9+2*$variables{ to }) );
		}
	}
	
	return \%variables;
}
################################################################



################################################################
#			initTwoLocusVariables
#	Inititializes and checks variables received from Two Locus
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initTwoLocusVariables( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "Init Two Locus Variables");
	
	if ( ( $inputForm{ R1 } !~ /BrowseButton/ ) && ( $variables{ program } =~ /TwoPoint/ ) )
	{
		&Processing::debug_print( "Two Point format");
		if ( $inputForm{ R1 } =~ /PasteButton/ ) 
		{
			if ( $inputForm{ "link" } !~ /Unlinked/ )
			{
				@{ $variables{ thetas } } = split /,/, $inputForm{ markersDistances };
				for ( my $t = 0; $t <= $#{ $variables{ thetas } }; $t ++ )
				{
					${ $variables{ thetas } }[$t] = &Processing::getThetaFromCM( ${ $variables{ thetas } }[$t] );
				}
			}
		}
		return \%variables;
	}
	if ( ( $inputForm{ R1 } =~ /BrowseButton/ ) && ( $variables{program} =~ /TwoPoint/ ) )
	{
		%variables = %{ &initTwoLocusDataFromSLFormat( \%variables, \%inputForm ) };
	}
	else
	{
		%variables = %{ &initTwoLocusDataFromSimplifiedFormat( \%variables, \%inputForm ) };
	}

	return \%variables;
}
################################################################


		
################################################################
#			initTwoLocusDataFromSimplifiedFormat
#	Inititializes and checks variables received from Two Locus
#	MultiPoint input form only
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initTwoLocusDataFromSimplifiedFormat( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "initTwoLocusDataFromSimplifiedFormat");

	if ( $variables{ program } =~ /OnePoint/ )
	{
		&Processing::debug_print( "Init One Point Parameters");
		&Processing::debug_print( "NOT IMPLEMENTED!!!");
		return &Processing::makeSystemError("ONE POINT IS NOT IMPLEMENTED!!!", \%variables);
	}
		
	&Processing::debug_print( "Init Multi Point Parameters:");

	$variables{ settings1 } = $inputForm{ mod1 };
	&Processing::debug_print( "Init First Locus Parameters, settings1=($variables{ settings1 })");
	
	if ( $variables{ settings1 } =~ /7/ )
	{
		my @thetas1 = split /,/, $inputForm{ dist1 } if ( defined $inputForm{ dist1 } );
		for ( my $t_cnt = 0; $t_cnt <= $#thetas1; $t_cnt ++ )
		{
			$thetas1[$t_cnt] = 1000;
		}
		$inputForm{ dist1 } = join ',', @thetas1;
		$variables{ settings1 } = 3;
	}

	if ( $variables{ settings1 } =~ /0/ )
	{
		$variables{ before1 } = $inputForm{ mod1_0_Dist };
		$variables{ from1 } = $inputForm{ mod1_0_Mark1 };
		$variables{ settings1 } = "point";
	}
	elsif ( $variables{ settings1 } =~ /1/ )
	{
		$variables{ after1 } = $inputForm{ mod1_1_Dist };
		$variables{ from1 } = "last";
		$variables{ settings1 } = "point";
	}
	elsif ( $variables{ settings1 } =~ /2/ )
	{
		$variables{ from1 } = $inputForm{ mod1_2_Mark1 };
		$variables{ to1 } = $inputForm{ mod1_2_Mark2 };
		$variables{ settings1 } = "middle";
	}
	elsif ( $variables{ settings1 } =~ /3/ )
	{
		$variables{ from1 } = "first";
		$variables{ to1 } = "last";
		$variables{ jump1 } = "zero";
		$variables{ offmap1 } = 0;
		$variables{ settings1 } = "each";
	}
	elsif ( $variables{ settings1 } =~ /4/ )
	{
		$variables{ from1 } = "first";
		$variables{ to1 } = "last";
		$variables{ jump1 } = "one";
		$variables{ offmap1 } = $inputForm{ mod1_4_off };
		$variables{ settings1 } = "each";
	}
	elsif ( $variables{ settings1 } =~ /5/ )
	{
		$variables{ from1 } = "first";
		$variables{ to1 } = "last";
		$variables{ jump1 } = "two";
		$variables{ offmap1 } = $inputForm{ mod1_5_off };
		$variables{ settings1 } = "each";
	}
	elsif ( $variables{ settings1 } =~ /6/ )
	{
		$variables{ from1 } = $inputForm{ mod1_6_Mark1 };
		$variables{ to1 } = $inputForm{ mod1_6_Mark2 };
		$variables{ jump1 } = $inputForm{ mod1_6_jump };
		$variables{ offmap1 } = $inputForm{ mod1_6_off };
		$variables{ settings1 } = "custom";
	}
	else
	{
		return &Processing::makeInputError("Unknown prog option in inputForm hash!!!", \%variables);
	}
	&Processing::debug_print( "settings1 = ($variables{ settings1 })");
	&Processing::debug_print( "from1 = ($variables{ from1 })");
	&Processing::debug_print( "to1 = ($variables{ to1 })");
	&Processing::debug_print( "jump1 = ($variables{ jump1 })");
	&Processing::debug_print( "offmap1 = ($variables{ offmap1 })");
	&Processing::debug_print( "before1 = ($variables{ before1 })");
	&Processing::debug_print( "after1 = ($variables{ after1 })");

	$variables{ settings2 } = $inputForm{ mod2 };
	&Processing::debug_print( "Init Second Locus Parameters, settings2=($variables{ settings2 })");
	
	if ( $variables{ settings2 } =~ /7/ )
	{
		my @thetas2 = split /,/, $inputForm{ dist2 } if ( defined $inputForm{ dist2 } );
		for ( my $t_cnt = 0; $t_cnt <= $#thetas2; $t_cnt ++ )
		{
			$thetas2[$t_cnt] = 1000;
		}
		$inputForm{ dist2 } = join ',', @thetas2;
		$variables{ settings2 } = 3;
	}
	
	if ( $variables{ settings2 } =~ /0/ )
	{
		$variables{ before2 } = $inputForm{ mod2_0_Dist };
		$variables{ from2 } = $inputForm{ mod2_0_Mark1 };
		$variables{ settings2 } = "point";
	}
	elsif ( $variables{ settings2 } =~ /1/ )
	{
		$variables{ after2 } = $inputForm{ mod2_1_Dist };
		$variables{ from2 } = "last";
		$variables{ settings2 } = "point";
	}
	elsif ( $variables{ settings2 } =~ /2/ )
	{
		$variables{ from2 } = $inputForm{ mod2_2_Mark1 };
		$variables{ to2 } = $inputForm{ mod2_2_Mark2 };
		$variables{ settings2 } = "middle";
	}
	elsif ( $variables{ settings2 } =~ /3/ )
	{
		$variables{ from2 } = "first";
		$variables{ to2 } = "last";
		$variables{ jump2 } = "zero";
		$variables{ offmap2 } = 0;
		$variables{ settings2 } = "each";
	}
	elsif ( $variables{ settings2 } =~ /4/ )
	{
		$variables{ from2 } = "first";
		$variables{ to2 } = "last";
		$variables{ jump2 } = "one";
		$variables{ offmap2 } = $inputForm{ mod2_4_off };
		$variables{ settings2 } = "each";
	}
	elsif ( $variables{ settings2 } =~ /5/ )
	{
		$variables{ from2 } = "first";
		$variables{ to2 } = "last";
		$variables{ jump2 } = "two";
		$variables{ offmap2 } = $inputForm{ mod2_5_off };
		$variables{ settings2 } = "each";
	}
	elsif ( $variables{ settings2 } =~ /6/ )
	{
		$variables{ from2 } = $inputForm{ mod2_6_Mark1 };
		$variables{ to2 } = $inputForm{ mod2_6_Mark2 };
		$variables{ jump2 } = $inputForm{ mod2_6_jump };
		$variables{ offmap2 } = $inputForm{ mod2_6_off };
		$variables{ settings2 } = "custom";
	}
	else
	{
		return &Processing::makeInputError("Unknown prog option in inputForm hash!!!", \%variables);
	}
	&Processing::debug_print( "settings2 = ($variables{ settings2 })");
	&Processing::debug_print( "from2 = ($variables{ from2 })");
	&Processing::debug_print( "to2 = ($variables{ to2 })");
	&Processing::debug_print( "jump2 = ($variables{ jump2 })");
	&Processing::debug_print( "offmap2 = ($variables{ offmap2 })");
	&Processing::debug_print( "before2 = ($variables{ before2 })");
	&Processing::debug_print( "after2 = ($variables{ after2 })");

	%variables = %{ &joinTwoLocusData( \%variables, \%inputForm ) } unless ( $inputForm{ R1 } =~ /BrowseButton/ );

	return \%variables;
}
################################################################



################################################################
#			joinTwoLocusData
#	Joins the data from unlinked input
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub joinTwoLocusData( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "joinTwoLocusData");

	@{ $variables{ thetas } } = split /,/, $inputForm{ dist1 };
	&Processing::debug_print( "First Locus Distances are:(@{ $variables{ thetas } })");
	my @thetas2 = ();
	@thetas2 = split /,/, $inputForm{ dist2 } if ( defined $inputForm{ dist2 } );
	&Processing::debug_print( "Second Locus Distances are:(@thetas2)");

	my $th = 0;
	for( ; $th <= $#{ $variables{ thetas } }; $th++ )
	{
		${ $variables{ thetas } }[$th] = &Processing::getThetaFromCM( ${ $variables{ thetas } }[$th] );
	}
	
	${ $variables{ thetas } }[$th++] = 0.5 if ( defined $inputForm{marker2Area} );
	return \%variables if ( $#thetas2 < 0 );
	
	for( my $th2 = 0 ; $th2 <= $#thetas2; $th2++ )
	{
		${ $variables{ thetas } }[$th+$th2] = &Processing::getThetaFromCM( $thetas2[$th2] );
	}
	&Processing::debug_print( "Thetas from Simplified input format file are (@{ $variables{ thetas } })");
	
	return \%variables;
}
################################################################



################################################################
#			initTwoLocusDataFromSLFormat
#	Inititializes and checks variables received from Two Locus
#	MultiPoint input form only
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub initTwoLocusDataFromSLFormat( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	&Processing::debug_print( "initTwoLocusDataFromSLFormat");

	&Processing::debug_print( "Superlink input format");
	return &Processing::makeSystemError("Failed to open datafile ($variables{ newDir }\/$variables{ datafile })", \%variables)
		unless (open DATA, "$variables{ newDir }\/$variables{ datafile }");

	my @data = <DATA>;
	close DATA;
	my $shifted = shift @data;
	while ( ( $#data >= 0 ) && ( $shifted !~ /\S/ ) )
	{
		$shifted = shift @data;
	}
	my @words = split /\s+/, $shifted;
	$variables{ numMarkers } = $words[0] - 2;
	$variables{ numMarkers1 } = $words[0] - 2;
	$variables{ numMarkers2 } = $words[0] - 2;
	
	my $poped = pop @data;
	&Processing::debug_print( "Data line is ($poped)");
	while ( ( $#data >= 0 ) && ( $poped !~ /\S/ ) )
	{
		$poped = pop @data;
		&Processing::debug_print( "Data line is ($poped)");
	}
#	while ( ($poped !~ /-s/) && ($poped !~ /-n/) && ( ($poped !~ /\(/) && ($poped !~ /\)/) ) )
#	{
#		&Processing::makeSystemError("Wrong datafile format - no run options line at the end!!!", \%variables)
#			if ($#data < 0);
#		$poped = pop @data;
#	}

	if ( ($poped =~ /\(/) && ($poped =~ /\)/) )
	{
		&Processing::debug_print( "initTwoLocusDataFromSLFormat, OnePoint => poped= ($poped)");
		$variables{ program } = "TwoLocus.OnePoint";
		return \%variables;
	}
	else
	{
		&Processing::debug_print( "First Locus details");
		my @parts = split /\s+/, $poped;
		while ( $parts[0] !~ /\S/ )
		{
			shift @parts;
		}
			
		$variables{ jump1 } = "";
		if ( $parts[0] =~ /-s/)
		{
			$variables{ settings1 } = "custom";
			$variables{ jump1 } = $parts[1] if ( $parts[1] > 0 );
		}
		else
		{
			$variables{ settings1 } = "each";
			$variables{ jump1 } = "zero" if ( $parts[1] =~ /1/ );
			$variables{ jump1 } = "one" if ( $parts[1] =~ /2/ );
			$variables{ jump1 } = "two" if ( $parts[1] =~ /3/ );
		}
		$variables{ from1 } = $parts[2];
		$variables{ to1 } = $parts[3];
		$variables{ offmap1 } = $parts[5] 
			if ( ( $#parts == 5) && ( $parts[4] =~ /-o/ ) && ( $parts[5] >= 0 ) );
		
		return &Processing::makeInputError("Wrong datafile format - wrong command options format for First Locus!!!", \%variables)
			if ( ( $variables{ from1 } > $variables{ to1 } )
			  || ( $variables{ from1 } < 1 ) 
			  || ( $variables{ to1 } < 1 ) 
			  || ( $variables{ jump1 } eq "" )
			 );
			
		&Processing::debug_print( "Second Locus details");
		$poped = pop @data;
		while ( ( $#data >= 0 ) && ( $poped !~ /\S/ ) )
		{
			$poped = pop @data;
		}
		&Processing::debug_print( "Data line is ($poped)");
		@parts = split /\s+/, $poped;
		while ( $parts[0] !~ /\S/ )
		{
			shift @parts;
		}

		$variables{ jump2 } = "";
		if ( $parts[0] =~ /-s/)
		{
			$variables{ settings2 } = "custom";
			$variables{ jump2 } = $parts[1] if ( $parts[1] > 0 );
		}
		else
		{
			$variables{ settings2 } = "each";
			$variables{ jump2 } = "zero" if ( $parts[1] =~ /1/ );
			$variables{ jump2 } = "one" if ( $parts[1] =~ /2/ );
			$variables{ jump2 } = "two" if ( $parts[1] =~ /3/ );
		}
		$variables{ from2 } = $parts[2];
		$variables{ to2 } = $parts[3];
		$variables{ offmap2 } = $parts[5] 
			if ( ( $#parts == 5) && ( $parts[4] =~ /-o/ ) && ( $parts[5] >= 0 ) );
		
		return &Processing::makeInputError("Wrong datafile format - wrong command options format for Second Locus!!!", \%variables)
			if ( ( $variables{ from2 } > $variables{ to2 } )
			  || ( $variables{ from2 } < 1 ) 
			  || ( $variables{ to2 } < 1 ) 
			  || ( $variables{ jump2 } eq "" )
			 );
	}

	&Processing::debug_print( "Thetas from SL format file are (@{ $variables{ thetas } })");
	return &Processing::makeInputError("Wrong datafile format - wrong number of thetas!!!", \%variables)
		if ( $#{ $variables{ thetas } } < $variables{ numMarkers }-2 );	# size is counted from -1 and markers from 1
	return &Processing::makeSystemError("Failed to open datafile ($variables{ newDir }\/$variables{ pedfile })", \%variables)
	unless (open PED, "$variables{ newDir }\/$variables{ pedfile }");

	my @ped = <PED>;
	close PED;
	foreach my $pedLine ( @ped )
	{
		@words = split "<<", $pedLine;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError("Wrong pedfile format - wrong number of markers", \%variables)
			if ( $#words < (9+2*$variables{ numMarkers }) );
	}
	
	return \%variables;
}
################################################################



################################################################
# 				parseRemoteInput
#	Parses input from HTML form
#
#	Parameters:
#					NOTHING
#	Returns:
#					Hash with input from HTML Form
################################################################
sub parseRemoteInput()
{
	&Processing::debug_print("parseRemoteInput");
	%in = ();
	$ret = &UseCgiLibPl::ReadParse(*in);

	&UseCgiLibPl::CgiDie("Error in reading and parsing of CGI input")
		unless ( defined $ret );
	&Processing::debug_print("parseRemoteInput: ret=($ret), size of in=($#in)");

	&UseCgiLibPl::CgiDie("No data uploaded."," Please go back to <a href= $Configuration::HOME_WEB_DIR >$Notification::TITLE</a>.")
		unless ( $ret );
	&UseCgiLibPl::CgiDie("No data uploaded."," Please go back to <a href= $Configuration::HOME_WEB_DIR >$Notification::TITLE</a>.")
		if ( $#in < 0 );

	return \%in;
}
################################################################



################################################################
# 				parseLocalInput
#	Parses input from Input Hash file
#
#	Parameters:
#					NOTHING
#	Returns:
#					Hash with input  Input Hash file
################################################################
sub parseLocalInput( $ )
{
	my $inputHashFile = $_[0];
	my $dir = &getcwd();
	&Processing::debug_print("Looking for file: (./$inputHashFile) from directory ($dir)");

	if ( ! -e  "./$inputHashFile" )
	{
		&Processing::debug_print("Wrong file name!");
		&UseCgiLibPl::CgiDie("Wrong file name!") ;
	}

	require "./$inputHashFile";	#	Definition of Global Hash (%in)

	&UseCgiLibPl::CgiDie("Wrong file was loaded")
		unless ( defined %in );
	
	return \%in;		#Has to be defined in the:  $inputHashFile
}
################################################################



################################################################
# 				preparePointInput
#	Prepares mode file with all information about the Project
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub preparePointInput( \% )
{
	%variables = %{ $_[0] };
	my $dir = &getcwd();
	%variables = %{&Processing::makeSystemError("Failed to enter project directory ($variables{ newDir })", \%variables)}
		unless ( chdir( $variables{ newDir } ) );
	
	%variables = %{&Processing::makeSystemError("Failed to open (modefile.txt) for writing", \%variables)}
		unless ( open MODE, ">modefile.txt" );
		
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	&Processing::debug_print( "Creating Project details for Point analisys" );

		print MODE "PROJECT INFORMATION:\n====================\n";
		print MODE "Project Name: _(_[_(_PROJ=$variables{ PROJ }_)_]_)_\n";
		if (defined $variables{ method }) {
			print MODE "Data Reduction: _(_[_(_method=$variables{ method }_)_]_)_\n";
		}
		print MODE "Output File Name: _(_[_(_OUTPUT_FILE=$variables{ OUTPUT_FILE }_)_]_)_\n";
		print MODE "Program: _(_[_(_program=$variables{ program }_)_]_)_\n";
		print MODE "Date: _(_[_(_currentDay=$variables{ currentDay }_)_]_)_\n";
		print MODE "Root: _(_[_(_root=$variables{ root }_)_]_)_\n";
		print MODE "RootName: _(_[_(_rootName=$variables{ rootName }_)_]_)_\n";
		print MODE "Email: _(_[_(_mail=$variables{ mail }_)_]_)_\n";
		print MODE "Directory: _(_[_(_newDir=$variables{ newDir }_)_]_)_\n";
		print MODE "pedfile: _(_[_(_pedfile=$variables{ pedfile }_)_]_)_\n";
		print MODE "datafile: _(_[_(_datafile=$variables{ datafile }_)_]_)_\n";
		print MODE "tmpPed: _(_[_(_tmpPed=$variables{ tmpPed }_)_]_)_\n";
		print MODE "part: _(_[_(_part=$variables{ part }_)_]_)_\n";
		print MODE "Format: _(_[_(_runFormat=$variables{ runFormat }_)_]_)_\n";
		print MODE "Mode: _(_[_(_mode=$variables{ mode }_)_]_)_\n";
		print MODE "Sex LInked: _(_[_(_sexLinked=$variables{ sexLinked }_)_]_)_\n";
	#	print MODE "Complex: _(_[_(_combo=$variables{ combo }_)_]_)_\n";
		print MODE "numMarkers: _(_[_(_numMarkers=$variables{ numMarkers }_)_]_)_\n";
		
		for( $i=0; $i <= $#{$variables{ markersNames }}; $i++ )
		{
			${$variables{ markersNames }}[$i] = "[${$variables{ markersNames }}[$i]]";
		}
		print MODE "Marker names: _(_[_(_markersNames=@{$variables{ markersNames }}_)_]_)_\n";

		for( $i=0; $i <= $#{$variables{ chromosomes } }; $i++ )
		{
			${$variables{ chromosomes }}[$i] = "[${$variables{ chromosomes }}[$i]]";
		}
		print MODE "First Markers on Chromosomes: _(_[_(_chromosomes=@{$variables{ chromosomes }}_)_]_)_\n";
		
		for( $i=0; $i <= $#{$variables{ alleleNum }}; $i++ )
		{
			${$variables{ alleleNum }}[$i] = "[${$variables{ alleleNum }}[$i]]";
		}
		print MODE "Marker allels frequences: _(_[_(_alleleNum=@{$variables{ alleleNum }}_)_]_)_\n"
			if ( 0 <= $#{$variables{ alleleNum }});
		
		for( $i=0; $i <= $#{$variables{ penetrances }}; $i++ )
		{
			${$variables{ penetrances }}[$i] = "[${$variables{ penetrances }}[$i]]";
		}
		print MODE "Penetrance: _(_[_(_penetrances=@{$variables{ penetrances }}_)_]_)_\n";

		for( $i=0; $i <= $#{$variables{ pednames }}; $i++ )
		{
			${$variables{ pednames }}[$i] = "[${$variables{ pednames }}[$i]]";
		}
		print MODE "List of names of used pedigrees: _(_[_(_pednames=@{$variables{ pednames }}_)_]_)_\n";

		if ( $variables{ program } =~ /MultiPoint/ )
		{
			for( $i=0; $i <= $#{$variables{ thetas }}; $i++ )
			{
				${$variables{ thetas }}[$i] = "[${$variables{ thetas }}[$i]]";
			}
			print MODE "List of thetas: _(_[_(_thetas=@{$variables{ thetas }}_)_]_)_\n";
			if ( $variables{ program } =~ /TwoLocus/ )
			{
		#	First Locus
				if ( $variables{ from1 } =~ /\D/ )
				{
					print MODE "First Locus Start marker: _(_[_(_from1=${$variables{usedMarkersId1}}[0]_)_]_)_\n" if ( $variables{ from1 } =~ /first/ );
					print MODE "First Locus Start marker: _(_[_(_from1=".(1+${$variables{usedMarkersId1}}[$#{$variables{ usedMarkersId1 }}])."_)_]_)_\n" if ( $variables{ from1 } =~ /last/ );
				}
				else
				{
					print MODE "First Locus Start marker: _(_[_(_from1=$variables{ from1 }_)_]_)_\n";
				}
				print MODE "First Locus End marker: _(_[_(_to1=$variables{ to1 }_)_]_)_\n" if ( $variables{ to1 } !~ /last/ );
				print MODE "First Locus End marker: _(_[_(_to1=".(1+${$variables{usedMarkersId1}}[$#{$variables{ usedMarkersId1 }}])."_)_]_)_\n" if ( $variables{ to1 } =~ /last/ );

				if ( $variables{ before1 } !~ /$Processing::UNCALCULATED_INFORMATION/ )
				{
					print MODE "Second Locus Before: _(_[_(_jump1=".(-1*$variables{ before1 })."_)_]_)_\n";
				}
				elsif ( $variables{ after1 } !~ /$Processing::UNCALCULATED_INFORMATION/ )
				{
					print MODE "Second Locus After: _(_[_(_jump1=$variables{ after1 }_)_]_)_\n";
				}
				else
				{
					print MODE "Second Locus Jump step: _(_[_(_jump1=$variables{ jump1 }_)_]_)_\n";
				}
				
				print MODE "First Locus Offmap position: _(_[_(_offmap1=$variables{ offmap1 }_)_]_)_\n";
				print MODE "First Locus Mode of calculation: _(_[_(_settings1=$variables{ settings1 }_)_]_)_\n";
		#	Second Locus
				if ( $variables{ from2 } =~ /\D/ )
				{
					print MODE "Second Locus Start marker: _(_[_(_from2=${$variables{usedMarkersId2}}[0]_)_]_)_\n" if ( $variables{ from2 } =~ /first/ );
					print MODE "Second Locus Start marker: _(_[_(_from2=".(1+${$variables{ usedMarkersId2 }}[$#{$variables{ usedMarkersId2 }}])."_)_]_)_\n" if ( $variables{ from2 } =~ /last/ );
				}
				else
				{
					$variables{ from2 } += 1+$#{$variables{ usedMarkersId1 }}
						if ( $variables{ linked } =~ /Unlinked/ );
					print MODE "Second Locus Start marker: _(_[_(_from2=".($variables{ from2 })."_)_]_)_\n";
				}
				if ( $variables{ to2 } =~ /\D/ )		#	to2= last	
				{
					print MODE "Second Locus End marker: _(_[_(_to2=".(1+${$variables{ usedMarkersId2 }}[$#{$variables{ usedMarkersId2 }}])."_)_]_)_\n";
				}
				else
				{
					$variables{ to2 } += 1+$#{$variables{ usedMarkersId1 }}
						if ( $variables{ linked } =~ /Unlinked/ );
					print MODE "Second Locus End marker: _(_[_(_to2=".($variables{ to2 })."_)_]_)_\n";
				}

				if ( $variables{ before2 } !~ /$Processing::UNCALCULATED_INFORMATION/ )
				{
					print MODE "Second Locus Before: _(_[_(_jump2=".(-1*$variables{ before2 })."_)_]_)_\n";
				}
				elsif ( $variables{ after2 } !~ /$Processing::UNCALCULATED_INFORMATION/ )
				{
					print MODE "Second Locus After: _(_[_(_jump2=$variables{ after2 }_)_]_)_\n";
				}
				else
				{
					print MODE "Second Locus Jump step: _(_[_(_jump2=$variables{ jump2 }_)_]_)_\n";
				}
				print MODE "Second Locus Offmap position: _(_[_(_offmap2=$variables{ offmap2 }_)_]_)_\n";
				print MODE "Second Locus Mode of calculation: _(_[_(_settings2=$variables{ settings2 }_)_]_)_\n";
			}
			else
			{
				print MODE "Start marker: _(_[_(_from=$variables{ from }_)_]_)_\n";
				print MODE "End marker: _(_[_(_to=$variables{ to }_)_]_)_\n" if ( $variables{ to } !~ /last/ );
				print MODE "End marker: _(_[_(_to=".(1+$#{$variables{ usedMarkersId }})."_)_]_)_\n" if ( $variables{ to } =~ /last/ );
				print MODE "Jump step: _(_[_(_jump=$variables{ jump }_)_]_)_\n";
				print MODE "Offmap position: _(_[_(_offmap=$variables{ offmap }_)_]_)_\n";
				print MODE "mode of calculation: _(_[_(_settings=$variables{ settings }_)_]_)_\n";
			}
		}
		if ( $variables{ program } =~ /OnePoint/ )
		{
			print MODE "List of thetas: _(_[_(_thetas=@{$variables{ thetas }}_)_]_)_\n";
			if ( $variables{ program } =~ /TwoLocus/ )
			{
				print MODE "Start marker: _(_[_(_from1=$variables{ from1 }_)_]_)_\n";
				print MODE "Start marker: _(_[_(_from2=$variables{ from2 }_)_]_)_\n";
			}
			else
			{
				print MODE "Start marker: _(_[_(_from=$variables{ from }_)_]_)_\n";
			}
		}
		if ( $variables{ program } =~ /TwoLocus/ )
		{
			print MODE "Locuses linked: _(_[_(_linked=$variables{ linked }_)_]_)_\n";
#	First Locus
			print MODE "Mutant frequency for First Locus: _(_[_(_mutantFreq1=$variables{ mutantFreq1 }_)_]_)_\n";
			for( $i=0; $i <= $#{$variables{ usedMarkersId1 }}; $i++ )
			{
				${$variables{ usedMarkersId1 }}[$i] = "[${$variables{ usedMarkersId1 }}[$i]]";
				${$variables{ usedMarkersId1out }}[$i] = "[${$variables{ usedMarkersId1out }}[$i]]";
			}
			print MODE "For first locus list of markers to use: _(_[_(_usedMarkersId1=@{$variables{ usedMarkersId1 }}_)_]_)_\n";
			print MODE "For first locus list of markers to output: _(_[_(_usedMarkersId1out=@{$variables{ usedMarkersId1out }}_)_]_)_\n";
	
			for( $i=0; $i <= $#{$variables{ usedMarkersNames1 }}; $i++ )
			{
				${$variables{ usedMarkersNames1 }}[$i] = "[${$variables{ usedMarkersNames1 }}[$i]]";
#				${$variables{ usedMarkersNames1out }}[$i] = "[${$variables{ usedMarkersNames1out }}[$i]]";
			}
			print MODE "For first locus list of names of markers to use: _(_[_(_usedMarkersNames1=@{$variables{ usedMarkersNames1 }}_)_]_)_\n";

#	Second Locus
			print MODE "Mutant frequency for Second Locus: _(_[_(_mutantFreq2=$variables{ mutantFreq2 }_)_]_)_\n";
			for( $i=0; $i <= $#{$variables{ usedMarkersId2 }}; $i++ )
			{
				${$variables{ usedMarkersId2 }}[$i] = "[${$variables{ usedMarkersId2 }}[$i]]";
				${$variables{ usedMarkersId2out }}[$i] = "[${$variables{ usedMarkersId2out }}[$i]]";
			}
			print MODE "For second locus list of markers to use: _(_[_(_usedMarkersId2=@{$variables{ usedMarkersId2 }}_)_]_)_\n";
			print MODE "For second locus list of markers to output: _(_[_(_usedMarkersId2out=@{$variables{ usedMarkersId2out }}_)_]_)_\n";
	
			for( $i=0; $i <= $#{$variables{ usedMarkersNames2 }}; $i++ )
			{
				${$variables{ usedMarkersNames2 }}[$i] = "[${$variables{ usedMarkersNames2 }}[$i]]";
#				${$variables{ usedMarkersNames2out }}[$i] = "[${$variables{ usedMarkersNames2out }}[$i]]";
			}
			print MODE "For second locus list of names of markers to use: _(_[_(_usedMarkersNames2=@{$variables{ usedMarkersNames2 }}_)_]_)_\n";
		}
		else
		{
			print MODE "Mutant frequency: _(_[_(_mutantFreq=$variables{ mutantFreq }_)_]_)_\n";
	
			for( $i=0; $i <= $#{$variables{ usedMarkersNames }}; $i++ )
			{
				${$variables{ usedMarkersNames }}[$i] = "[${$variables{ usedMarkersNames }}[$i]]";
			}
			print MODE "List of names of markers to use: _(_[_(_usedMarkersNames=@{$variables{ usedMarkersNames }}_)_]_)_\n";
		}
		for( $i=0; $i <= $#{$variables{ usedMarkersId }}; $i++ )
		{
			${$variables{ usedMarkersId }}[$i] = "[${$variables{ usedMarkersId }}[$i]]";
			${$variables{ usedMarkersIdout }}[$i] = "[${$variables{ usedMarkersIdout }}[$i]]";
		}
		for( $i=0; $i <= $#{$variables{ positions }}; $i++ )
		{
			${$variables{ positions }}[$i] = "[${$variables{ positions }}[$i]]";
		}
		if ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/ ||
			$variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_LM_MARKERS/ ||
			$variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_GL_AUTO/ ||
			$variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HMM/) {
			print MODE "Start marker: _(_[_(_from=$variables{ from }_)_]_)_\n" if ( defined($variables{ from }) );
			print MODE "End marker: _(_[_(_to=$variables{ to }_)_]_)_\n" if ( defined($variables{ to }) );
			print MODE "Jump step: _(_[_(_jump=$variables{ jump }_)_]_)_\n" if ( defined($variables{ jump }) );
			
			#Omer 28/7/2011 - new "Morgan lm _ markers" parameters
			# Dvir 2011/10/25, added unlimited				
			my $timeToRunSingleWindow = $variables{WindowTime};
			my $maxJobsPerTask = $variables{MaxJobsPerTask};
			my $maxTimePerJob = $variables{MaxTimePerJob};
			my $minTimePerJob = $variables{MinTimePerJob};
			
			# dvir 09/11/2011, $timeToRunSingleWindow with the value 5*60*60 (5 hours)
			# & unlimited num of time per job is in order that each haplo job will be on a single window.
			if ( ! defined $timeToRunSingleWindow) { $timeToRunSingleWindow = 5*60*60; }
			if ( ! defined $maxJobsPerTask) { $maxJobsPerTask = $SharedConsts::UNLIMITED_VALUE; }
			if ( ! defined $maxTimePerJob) { $maxTimePerJob = $SharedConsts::UNLIMITED_VALUE; }
			if ( ! defined $minTimePerJob) { $minTimePerJob = $SharedConsts::UNLIMITED_VALUE; }
			print MODE "Max Jobs per Task: _(_[_(_max_jobs_per_task=${maxJobsPerTask}_)_]_)_\n";
			print MODE "Min Time per Job: _(_[_(_min_time_per_job=${minTimePerJob}_)_]_)_\n";
			print MODE "Max Time per Job: _(_[_(_max_time_per_job=${maxTimePerJob}_)_]_)_\n";
			print MODE "Min Iterations per Job: _(_[_(_$SharedConsts::MORGAN_MIN_NUM_OF_ITERATIONS=$variables{ MinNumOfIterations }_)_]_)_\n" if ( defined($variables{ MinNumOfIterations }) );
			print MODE "Max Iterations per Job: _(_[_(_$SharedConsts::MORGAN_MAX_NUM_OF_ITERATIONS=$variables{ MaxNumOfIterations }_)_]_)_\n" if ( defined($variables{ MaxNumOfIterations }) );
			print MODE "Number of Duplicates per Job: _(_[_(_num_dups_per_job=$variables{ NumDupsPerJob }_)_]_)_\n" if ( defined($variables{ NumDupsPerJob }) );
			print MODE "Running Time for Window: _(_[_(_run_time_of_window=${timeToRunSingleWindow}_)_]_)_\n";
			print MODE "Burn in iterations percentage: _(_[_(_$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_BURN_IN_PERCENTAGE=$variables{ BurnInPercentage }_)_]_)_\n" if ( defined($variables{ BurnInPercentage }) );
			print MODE "Sequential imputation realizations percentage: _(_[_(_$SharedConsts::MORGAN_LM_MARKERS_PARAM_NAME_SEQUENTIAL_IMPUTATION_REALI=$variables{ SeqImpRealizationsPercentage }_)_]_)_\n" if ( defined($variables{ SeqImpRealizationsPercentage }) );
		}
		print MODE "List of markers to use: _(_[_(_usedMarkersId=@{$variables{ usedMarkersId }}_)_]_)_\n";
		print MODE "List of markers to output: _(_[_(_usedMarkersIdout=@{$variables{ usedMarkersIdout }}_)_]_)_\n";	
		print MODE "List of markers and their positions: _(_[_(_positions=";
		for( $i=0; $i <= $#{$variables{ positions }}; $i++ ){
			print MODE ${$variables{ positions }}[$i]." ";
		}
		print MODE "_)_]_)_\n";
		print MODE "Task finish flag: _(_[_(_taskFinishFlag=$variables{ taskFinishFlag }_)_]_)_\n";
		print MODE "-------------------------\n";
		print MODE "END_OF_MODE\n";
	close MODE;
	
	%variables = %{&Processing::makeSystemError("Failed to return to directory ($dir)", \%variables)}
		unless ( chdir( $dir ) );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	return $Processing::SUCCESS;
}
################################################################



################################################################
#			removeLociForTwoLocus
#	Removes the Markers, that user don't want to calculate
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					Updated Hash with variables
################################################################
sub removeLociForTwoLocus( \% )
{
	%variables = %{ $_[0] };
	&Processing::debug_print("sub removeLociForTwoLocus ");

	my @usedArray = @{$variables{usedMarkersId1}};
	&Processing::debug_print("sub removeLociForTwoLocus: program=($variables{program}), linked=($variables{ linked })");

	if ( ( $variables{program} =~ /MultiPoint/) && ( $variables{ linked } =~ /Unlinked/ ) )
	{
		for( my $id =0; $id <= $#{$variables{usedMarkersId2}}; $id++ )
		{
			${$variables{usedMarkersId2}}[$id] += $variables{numMarkers1};
			push @usedArray, ${$variables{usedMarkersId2}}[$id];
		}
	}
	else
	{
		foreach my $id ( @{$variables{usedMarkersId2}} )
		{
			my $in = 0;
			foreach my $idIn ( @usedArray )
			{
				if ( $idIn eq $id )
				{
					$in = 1;
					last;
				}
			}
			push @usedArray, $id if ( $in eq 0 );
		}
	}

	@{$variables{usedMarkersId}} = sort { $a <=> $b } @usedArray;
	&Processing::debug_print("Used markers are: (@{$variables{usedMarkersId}})");

	return \%variables if ( $Processing::SUCCESS !~ &removeLoci( \%variables, \@{$variables{usedMarkersId}} ) );
	
	@{$variables{usedMarkersId1}} = sort { $a <=> $b } 	@{$variables{usedMarkersId1}}; 
	@{$variables{usedMarkersId2}} = sort { $a <=> $b } 	@{$variables{usedMarkersId2}};

	@{$variables{usedMarkersIdout}} = @{$variables{usedMarkersId}}; 
	@{$variables{usedMarkersId1out}} = @{$variables{usedMarkersId1}}; 
	@{$variables{usedMarkersId2out}} = @{$variables{usedMarkersId2}};

	my @convert=();
	for ( my $i = 0; $i <= $#{$variables{usedMarkersId}}; $i++ )
	{
		$convert[ ${$variables{usedMarkersId}}[$i] ] = $i + 1;
	}
	for ( my $i = 0; $i <= $#{$variables{usedMarkersId}}; $i++ )
	{
		${$variables{usedMarkersId}}[$i] = $convert[ ${$variables{usedMarkersId}}[$i] ];
	}
	for ( my $i = 0; $i <= $#{$variables{usedMarkersId1}}; $i++ )
	{
		${$variables{usedMarkersId1}}[$i] = $convert[ ${$variables{usedMarkersId1}}[$i] ];
	}
	for ( my $i = 0; $i <= $#{$variables{usedMarkersId2}}; $i++ )
	{
		${$variables{usedMarkersId2}}[$i] = $convert[ ${$variables{usedMarkersId2}}[$i] ];
	}
	
	&Processing::debug_print("Used markers for first locus now are: (@{$variables{usedMarkersId1}})");
	&Processing::debug_print("Used markers for second locus now are: (@{$variables{usedMarkersId2}})");
	&Processing::debug_print("All Used markers now are: (@{$variables{usedMarkersId}})");
	
	return \%variables;
}
################################################################



################################################################
#			getTwoLocusUsedLoci
#	Get's the id-s of markers, which user want to calculate for Two Locus
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#					$_[2] - Locus number
#	Returns:
#					Array with numbers of used markers
################################################################
sub getTwoLocusUsedLoci( \% \% $ )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	my $askedLocus = $_[2];
	my $inputLocus = $askedLocus;
	my @markersArray = ();
	
	$inputLocus = 1 if ( ( $variables{ program } !~ /TwoPoint/ ) && ($askedLocus == 2 ) && defined ($inputForm{ "link" }) && ( $inputForm{ "link" } !~ /Unlinked/ ) );
	&Processing::debug_print("sub getUsedLoci for Two Locus, Asked Locus = ($askedLocus), Input form locus number=($inputLocus)");
	unless ( (defined $inputForm{ "usedMarkers".$askedLocus }) && ($inputForm{ "usedMarkers".$askedLocus } =~ /some/ ) )
	{
		&Processing::debug_print( "All markers are used" );
		my $step = 0;
		$step = $inputForm{ numMarkers1 } if ( ( $variables{ program } !~ /TwoPoint/ ) && ( $inputLocus eq 2 ) );
		$inputForm{ "numMarkers".$inputLocus } = $inputForm{ "numMarkers" } if ( $variables{ program } =~ /TwoPoint/ );
		my $maxMarkers = $inputForm{ "numMarkers".$inputLocus };
		$maxMarkers = $variables{ "numMarkers".$inputLocus } if ( $inputForm{ R1 } =~ /BrowseButton/ );
		
		for ( $i = 1; $i <= $maxMarkers; $i ++ )
		{
			push (@markersArray, ($step + $i) );
		}
		%variables = %{&Processing::makeInputError("Wrong datafile format - wrong final marker number for $askedLocus Locus!!!".
						" Maximum final value has to be ($markersArray[ $#markersArray ]) and not".
						" (".$variables{ "to".$askedLocus }.")", \%variables)}
			if ( (defined $variables{ "to".$askedLocus } ) && ( $variables{ "to".$askedLocus } =~ /\d/ ) 
					&& ( $variables{ "to".$askedLocus } > $markersArray[ $#markersArray ] ) );
		&Processing::debug_print("Markers array is (@markersArray)");
		return \@markersArray;
	}

	&Processing::debug_print("Markers to calculate: ".$inputForm{ "usedMarkers".$inputLocus."numbers" });
	$inputForm{ "usedMarkers".$inputLocus."numbers" } = $inputForm{ "usedMarkers".$inputLocus."numbers" }." ";
   	$inputForm{ "usedMarkers".$inputLocus."numbers" } =~ s/\s//sig;
	
	unless ( $inputForm{ "usedMarkers".$inputLocus."numbers" } )
	{
  		%variables = {&Processing::makeInputError("You must specify at least one marker to calculate LOD score", \%variables)};
		return \@markersArray;
	}
    
	if ( 2 == $inputLocus )
	{
		if ( $inputForm{ usedMarkers2 } =~ /some/ )
		{
			my @markers = split /,/, $inputForm{ usedMarkers2numbers };
			for( my $m=0; $m <= $#markers; $m++ ){ 	$markers[ $m ] += $inputForm{ numMarkers1 }; }
			$inputForm{ usedMarkers2numbers } = join ",", @markers;
			&Processing::debug_print( "inputForm{ usedMarkers2numbers } updated to ($inputForm{ usedMarkers2numbers }) ");
		}
	}
	@markersArray = split(/,/,$inputForm{  "usedMarkers".$inputLocus."numbers" });
	&Processing::debug_print("Markers array is (@markersArray)");
   	for ( my $marker=0; $marker <= $#markersArray; $marker++ ) 
	{
   		$markersArray[$marker] =~ s/\s+//sig;
   		if ( ($markersArray[$marker] =~ /\D+/) 
			|| ( ($marker>0) && ($markersArray[$marker] < $markersArray[$marker-1]) ) )
		{
			my $message = "Can't understand which markers you want to use.\n";
			$message = $message."If you want to use only some markers, you should provide a list of markers ";
			$message = $message."numbers in increasing order. The numbers should be separated by coma.";
				
			%variables = %{&Processing::makeInputError($message, \%variables)};
			return \@markersArray;
   		}
   	}
	%variables = %{&Processing::makeInputError("Wrong datafile format - wrong final marker number for $askedLocus Locus!!!", \%variables)}
		if ( (defined $variables{ "to".$askedLocus } ) && ( $variables{ "to".$askedLocus } > $markersArray[ $#markersArray ] ) );		

	return \@markersArray;
}
################################################################



################################################################
#			getUsedLoci
#	Get's the id-s of markers, which user want to calculate for Two Locus
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Array with numbers of used markers
################################################################
sub getUsedLoci( \% \% )
{
	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	my @markersArray = ();
	
	&Processing::debug_print("sub getUsedLoci");
	unless ( ( (defined $inputForm{ markersNumOption }) && ($inputForm{ markersNumOption } =~ /ON/ ) )
		|| ( (defined $inputForm{ usedMarkers  }) && ($inputForm{ usedMarkers } =~ /some/ ) ) )
	{
		&Processing::debug_print( "All markers are used" );
		for ( $i = 1; $i <= $variables{ numMarkers }; $i ++ )
		{
			push (@markersArray, $i);
		}
		%variables = %{&Processing::makeInputError("Wrong datafile format - wrong final marker: to=($variables{ to }), last=($markersArray[ $#markersArray ])!!!", \%variables)}
			if ( (defined $variables{ to } ) && ( $variables{ to } =~ /\d/ )
				&& ( $variables{ to } > $markersArray[ $#markersArray ] ) );
		return \@markersArray;
	}

	my $chosenMarkers = $inputForm{ usedMarkersNumbers };
	$chosenMarkers = $inputForm{ markersNum } 
		if ( (defined $inputForm{ markersNumOption }) && ($inputForm{ markersNumOption } =~ /ON/ ) );
	&Processing::debug_print("Markers to calculate: $chosenMarkers");
	$chosenMarkers = $chosenMarkers." ";
   	$chosenMarkers =~ s/\s//sig;
	
	unless ( $chosenMarkers )
	{
  		%variables = %{&Processing::makeInputError("You must specify at least one marker to calculate LOD score", \%variables)};
		&Processing::debug_print("No used markers specified");
		return \@markersArray;
	}
    
 	@markersArray = split(/,/,$chosenMarkers);
  	&Processing::debug_print("Markers to calculate are: (@markersArray)");
  	
	%variables = %{&Processing::makeInputError("Wrong datafile format - wrong final marker: to=($variables{ to }), last=($markersArray[ $#markersArray ])!!!", \%variables)}
		if ( ( $variables{ to } =~ /\d/ ) && ( $variables{ to } > $markersArray[ $#markersArray ] ) );

	return \@markersArray;
}
################################################################




################################################################
#			removeLoci
#	Removes the Markers, that user don't want to calculate
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#				$Processing::SUCCESS		<=	permission to continiue running
#				$Processing::ERROR			<=	error in the pedfile
################################################################
sub removeLoci( \% )
{
	%variables = %{ $_[0] };
	my @markersArray = @{$variables{usedMarkersId}};
	&Processing::debug_print("sub removeLoci with used markers (@markersArray)");
	
	@{$variables{ thetas }} = @{ &updateThetas( \%variables, \@markersArray ) };
	%variables = %{ &JavaExecutor::removeSpareLoci( "$variables{ newDir }\/$variables{ datafile }", "$variables{ newDir }\/$variables{ pedfile }", \@markersArray, \%variables )};

	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	return $Processing::SUCCESS;
}
################################################################



################################################################
#			updateThetas
#	Change datafile according to symplified input
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Array with used markers id
#	Returns:
#					Hash with variables
################################################################
sub updateThetas( \% \@ )
{
 	%variables = %{ $_[0] };
	my @markersArray = @{ $_[1] };
 	&Processing::debug_print("Updating thetas for markers (@markersArray)");

	my @thetas = ();
 	unless ( ( defined $variables{ thetas } ) && ( scalar @{ $variables{ thetas } } > 0 ) )
	{
		for ( my $i=0; $i < $#markersArray; $i ++ )
		{
			push @thetas, 0.1;						#	Default thetas. Can be used in Single Locus Two Point
		}
		return \@thetas;
	}
	&Processing::debug_print("Distances before: (@{$variables{ thetas }})");

  	my $i = 1;
	for ( ; $i <= $variables{ numMarkers }; $i ++ )
	{
		if ( $i !~ $markersArray[0])
		{
			shift @{$variables{ thetas }};
		}
		else
		{
			last;
		}
	}
  	&Processing::debug_print("Distances in the middle: (@{$variables{ thetas }}), first used marker is ($i)");
	if ( $#{$variables{ thetas }} >= 0 )
	{
		my $theta = -10;
		for ( $t=0, $m=0; (($i <= $variables{ numMarkers }) && ($t<=$#{$variables{ thetas }}) ); $i++, $t ++)
		{
			if ( $i =~ $markersArray[$m])		# Always true for the first $i
			{
				push @thetas, $theta unless ( $theta eq -10);	# Always false for the first $i
				$m ++;
				$theta = ${$variables{ thetas }}[$t];
			}
			else
			{
				$theta = &Processing::getThetaFromCM( &Processing::getCMfromTheta( $theta ) + &Processing::getCMfromTheta( ${$variables{ thetas }}[$t] ) )
					if ( $t <= $#{$variables{ thetas }});
			}
		}
		push @thetas, $theta unless ( ( $theta eq -10) || ($i > $markersArray[$#markersArray]) );
	}
  	&Processing::debug_print("Distances after: (@thetas)");
	
	return \@thetas;
}
################################################################




################################################################
#			getMOI
#	Get mode of inheritance
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub getMOI( \% \% )
{
 	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	
 	&Processing::debug_print("getMOI");
  	$variables{ mode  } = $inputForm{ mode };
  	&Processing::debug_print("mode = $inputForm{ mode }");
  	
  	${ $variables{ penetrances } }[0] = $inputForm{ ComboDom } if ( $variables{ mode  } =~ /dom/);
  	${ $variables{ penetrances } }[0] = $inputForm{ ComboRec } if ($variables{ mode  } =~ /rec/);
  	${ $variables{ penetrances } }[0] = $inputForm{ ComboComplex } if ($variables{ mode  } =~ /complex/);
  	&Processing::debug_print("mod = $variables{ mode } \n pene = (@{$variables{ penetrances } })");
  	$variables{ mode } = 1 if ( $variables{ program } =~ /TwoLocus/ );	#	Number of Liability classes in input form
	  
	return \%variables;
}
################################################################



################################################################
#			getTwoLocusMOI
#	Get mode of inheritance for Two Locus calculation
#
#	Parameters:
#					$_[0] - Hash with variables
#					$_[1] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub getTwoLocusMOI( \% \% )
{
 	%variables = %{ $_[0] };
	%inputForm = %{ $_[1] };
	
 	&Processing::debug_print("getTwoLocusMOI");
  	$variables{ mode } = 1;					#	Number of Liability classes in input form
  	&Processing::debug_print("mode = $inputForm{ mode }");
  	
   	${ $variables{ penetrances } }[0] = $inputForm{ A1A1B1B1 }." ".$inputForm{ A1A1B1B2 }." ".$inputForm{ A1A1B2B2 };
   	${ $variables{ penetrances } }[1] = $inputForm{ A1A2B1B1 }." ".$inputForm{ A1A2B1B2 }." ".$inputForm{ A1A2B2B2 };
   	${ $variables{ penetrances } }[2] = $inputForm{ A2A2B1B1 }." ".$inputForm{ A2A2B1B2 }." ".$inputForm{ A2A2B2B2 };
    	
 	&Processing::debug_print("mod = $variables{ mode } \n pene = (@{$variables{ penetrances } })");
	  
	return \%variables;
}
################################################################



################################################################
#			checkFamilyID
#	Checks Pedfile syntax
#
#	Parameters:
#				$_[0]   <= input pedfile name 
#				$_[1]	<= hash with variables
#	Returns:
#				$Processing::SUCCESS		<=	permission to continiue running
#				$Processing::ERROR			<=	error in the pedfile
################################################################
sub checkFamilyID( $ \%)
{
	my $file = $_[0];
	%variables = %{$_[1]};
	
  	&Processing::debug_print("in checkFamilyID with file=($file)");

  	my ( $line, @w, @ids);
  	my $ids_size=0;

  	%variables = %{&Processing::makeSystemError("Cannot open file ($file)", \%variables)}
		unless ( open IN, "$file" );
 	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	my $line_cnt = 0;
  	while ($line = <IN>)
	{
    	$line_cnt++;
    	@w = split(/\s+/,$line);

    	if (($#w > 0) and ($#w < 9))
		{
      		my $message =  "Error in superlink format line $line_cnt: \n
							The line should contain minimum 10 fields (fam, person, father, mother ...)\n";
      		%variables = %{&Processing::makeInputError($message, \%variables)};
			return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
		}

    	$w[0] =~ s/\s+//sig;

    	if ($w[0] !~ /\b[0-9]+\b/)
		{
      		$message = "Error in superlink format line  $line_cnt: \nFamily Id should be a number\n";
      		%variables = %{&Processing::makeInputError($message, \%variables)};
			return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
    	}

    	if ($w[0] <= 0)
		{
      		$message = "Error: family id <= 0\nIn superlink format family ids should start from 1,\n
							and contain sequential numbers\n";
       		%variables = %{&Processing::makeInputError($message, \%variables)};
			return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
    	}

    	if (not $ids[$w[0]])
		{
      		$ids_size++;
    	}
    	$ids[$w[0]]=1;
  	}
  	close IN;

  	for $i (1 .. $ids_size)
	{
    	if ( not $ids[$i])
		{
 			$message = "Error: Family ids are not fitting supelink format \n
							In superlink format family ids should start from 1,\nand contain sequential numbers\n";
       		%variables = %{&Processing::makeInputError($message, \%variables)};
			return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
    	}
  	}
  	return $Processing::SUCCESS;
}
################################################################


################################################################
#			convertToSimplyf
#		Converts given file to simplyfied format
#
#	Parameters:
#				$_[0]   <= input pedfile name 
#				$_[1] - Hash with variables
#	Returns:
#				$Processing::SUCCESS		<=	permission to continiue running
#				$Processing::ERROR			<=	error in the pedfile
################################################################
sub convertToSimplyf ($ \%) 
{
	my $file = $_[0];
	%variables = %{ $_[1] };
	
	&Processing::debug_print( "Convert to Simplified Format with file=($file)");

	%variables = %{&Processing::makeSystemError("Failed opening ($file) file", \%variables)}
		unless( open IN, "$file" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
 	my @lines = <IN>;
	close IN;
	
	@lines = @{ &sortArrayBySubField( \@lines, 1 ) };
 	my @arr = @{ &sortArrayBySubField( \@lines, 0 ) };

 	my @words = split(/\s+/, $arr[0]);
  	my $famID = $words[0];

	%variables = %{&Processing::makeSystemError("Failed opening temp file ($file.tmp)", \%variables)}
		unless ( open TMP_OUT, ">$file.tmp" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

 	print TMP_OUT "pedigree:  $famID\n";

  	my $personID = 1;
  	for $i (0 .. $#arr)
	{
		@words = split(/\s+/, $arr[$i]);
    	if ($famID !~ $words[0])
		{
      		$famID = $words[0];
      		$personID = 1;
      		print TMP_OUT "\npedigree:  $famID\n";
		}

#check the persons numbers of superlink format
    	if ($personID !~ $words[1])
		{
      		my $message = "\nError in family $famID person $words[1]:\n
						In superlink format persons id's shoud start from 1 and be sequential numbers \n";
			return &Processing::makeInputError($message, \%variables);
#			$personID = $words[1];
    	}

    	print TMP_OUT &convPersonSimplyf("$arr[$i]");

    	$personID++;
  	}
  	close TMP_OUT;
  	
	return;
}
################################################################


################################################################
#			convPersonSimplyf
#	Converts given line with person info to simplyfied format
#
#	Parameters:
#				$_[0]   <= line with person details
#	Returns:
#					Person details in symplyfied format
################################################################
sub convPersonSimplyf( $ )
{
  	my $line = $_[0];
  	my @w1 = split(/<</, $line);
  	my @w = split(/\s+/, $w1[0]);
  	my ($i, $st)= (0,10);
  
  	$st = 11 if ($#w % 2 == 0);		#there is a penetrance class

  	my $result = "$w[1]\t$w[2]\t$w[3]\t$w[7]\t$w[9]\t";
  	foreach $i ($st .. $#w)
	{
    	$result = "$result $w[$i]  ";
  	}
  	return "$result\n";
}
################################################################


################################################################
#		checkPass
# Checks password
# 
#	Parameters:
#				$_[0] 	- hash with variables
#	Return:
#				$Processing::SUCCESS 	- if password is correct
#				$Processing::ERROR	- if password is incorrect
################################################################
sub checkPass( \% )
{ 
	%variables = %{ $_[0] };
	&Processing::debug_print("Check Password, pass=($variables{pass})");

  	return $Processing::SUCCESS if (( $variables{pass} eq $Configuration::PASSWORD ));

	&Processing::debug_print("Failed read from (Texts/password.txt)")
		unless ( open IN, "Texts/password.txt" );
	my @lines = <IN>;
	close IN;
	
	my $message = join "\n", @lines;

	%variables = %{&Processing::makeInputError($message, \%variables)};
	return $Processing::ERROR;														
}
################################################################


################################################################
#				createDir
# Finds temporary name for directory and create it
#
# Parameters : 
#				$_[0] - where to create directory - can be full or .. path
#				$_[1] - Hash with variables
# Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub createDir( $ \% )
{
	my $dir = $_[0];
	%variables = %{ $_[1] };

	%variables = %{&Processing::makeSystemError("Cann't change directory to ($Processing::RUN_DIR)", \%variables)}
		unless ( chdir $Processing::RUN_DIR );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	&Processing::debug_print("Start of createDir");

    use   Fcntl;
    use POSIX;
    umask 0;
    my $name;
	my $username = $variables{ mail };
	$username =~ s/@.*//sig;	
  	
  	do 
	{
		$name = tmpnam();
      	&Processing::debug_print("Temp Name is:   ($name)");
      	
      	$name = "/tmp$name"	if ( $name !~ /tmp/ );
		$name = "$name.$variables{ program }.$variables{ currentDay }.$variables{ PROJ }.$username";
      	&Processing::debug_print("Proposed directory name is:   ($name)");
      	&Processing::debug_print("Trying to create:   $dir/var$name");
    } 
	until mkdir("$dir/var$name", 0777) ;
	
	$variables{ newDir } = "$dir/var$name";
	return $Processing::SUCCESS;
}
################################################################


################################################################
#				saveMail
# Saves mail to the file
#
# Parameters:
#				$_[0] request
#       		$_[1] variables
# Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR		- if error
################################################################
sub saveMail( $ \% )
{
	my $message = $_[0];
	%variables = %{ $_[1] };

	return $Processing::SUCCESS 
		if ( ( $Processing::YES != $Configuration::ONLINE ) || ( $Processing::YES != $Configuration::EMAIL ) );

	&Processing::debug_print("Save mail for purpose: ($message)");
	
#	use strict;
  	use Fcntl qw(:flock);
  
  	my $date = localtime;
  	my $outFile = "$Configuration::TEMP_WRITE_DIR\/$Configuration::LIST_USERS";
  	my $SEMAPHORE = "$Configuration::TEMP_WRITE_DIR\/$Configuration::LOCK_FILE";
  	my $remoteHost = "";
  	$remoteHost = $ENV{ REMOTE_HOST } if ( $ENV{ REMOTE_HOST } );
  	
  	%variables = %{ &Processing::makeSystemError( "Can't open $SEMAPHORE ($!)", \%variables )}
		unless ( open(LOCK, "$SEMAPHORE") );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
		
  	flock(LOCK, LOCK_EX);

  	open(OUT ," >> $outFile") || &Processing::debug_print( "\nfailed to open listUsers\n");
  	print OUT ("$message # $variables{ mail } # $variables{ newDir } # $remoteHost # $date \n");
  	close OUT;
  	close LOCK;

 	&Processing::debug_print("Failed creating ($variables{ newDir }\/projData.txt) file")
		unless ( open OUT, ">$variables{ newDir }\/projData.txt" );

 		print OUT "email = $variables{ mail }\n";
 		print OUT "Project = $variables{ PROJ }\n";
 		print OUT "Current day = $variables{ currentDay }\n";
 		print OUT "Curent time = $date\n";
		print OUT "Program = $variables{ program }\n";
		print OUT "Markers number = $variables{ numMarkers }\n";
		print OUT "Part = $variables{ part }\n";
 		print OUT time()."\n";

 	close OUT;

  	&saveLast( \%variables );
	return $Processing::SUCCESS;
}
################################################################



################################################################
#				createPassword
# 	Creates the password to the Project directory
#
# Parameters:
#		 $_[0] - Hash with variables
# Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub createPassword( \% )
{
	%variables = %{ $_[0] };
	&Processing::debug_print( "createPassword" );
	my $username = $variables{ mail };

  	$username =~ s/@.*//sig;
  	%variables = %{&Processing::makeSystemError( "Failed to enter project directory ($variables{ newDir })", \%variables )}
		unless ( chdir( $variables{ newDir } ) );

  	%variables = %{&Processing::makeSystemError( "Failed writing to file (.htaccess)", \%variables )}
		unless ( open FH, ">.htaccess" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	print FH "AuthUserFile $Processing::RUN_DIR\/$variables{ newDir }\/.htpasswd\n".
				'AuthGroupFile /dev/null
				AuthName "Superlink-online"
				AuthType Basic

				<Limit GET POST>
				require valid-user
				</Limit>';
  	close FH;

	%variables = %{&executeCommand("/usr/bin/htpasswd -nb $username $variables{ PROJ } >.htpasswd",	\%variables) };
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

   	%variables = %{ &Processing::makeSystemError( "Failed writing to file (pass.txt)", \%variables )}
		unless ( open FH, ">pass.txt" );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

  		print FH "Your username: $username\n";
  		print FH "Your password is the project name: $variables{ PROJ }\n\n\n";
  	close FH;
  	
  	%variables = %{&Processing::makeSystemError( "Failed to enter perl directory ($Processing::RUN_DIR)", \%variables )}
		unless ( chdir("$Processing::RUN_DIR") );
	return $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														
	
	return $Processing::SUCCESS;
} 
################################################################



################################################################
#				getAlleleNum
# 	Gets allels number and frequnces of each allel
#
#	Parameters:
#					$_[0] - Output file
#					$_[1] - Hash with variables
#					$_[2] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub getAlleleNum( $ \% \% )
{
	my $file = $_[0];
	%variables = %{ $_[1] };
	%inputForm = %{ $_[2] };
	my @freq = (" ");
 	@{$variables{ alleleNum } } = @freq;
	&Processing::debug_print("getAlleleNum");
	
 	return \%variables	unless ( (defined $inputForm{ allelesOption }) && ( $inputForm{ allelesOption } =~ /ON/ ) );

	&Processing::debug_print("allelesOption is ON");   
   	@freq = split(/;/,$inputForm{ alleleNum });

   	if ($#freq != ($variables{ numMarkers }-1))
	{
		my $message = "You specified $variables{ numMarkers } markers, but frequencies only for different ";
		$message = $message.($#freq+1)." markers.\n";

		&Processing::debug_print("Failed reading from (Texts\/AlleleFreqError.txt)")
			unless ( open MES, "Texts\/AlleleFreqError.txt" );
		my $errorFile = join "\n",<MES>;
		close MES;
			
		return &Processing::makeInputError( $message.$errorFile, \%variables );
   	}
	&Processing::debug_print("Frequences are: (@freq)");   

 	%variables = %{&Processing::makeSystemError( "Failed to append to file ($file)", \%variables )}
		unless ( open ( FH, ">>$file") );
 	my $markerNum = 0;
   	foreach my $listFreq (@freq)
	{
   		$markerNum++;
   		my @alleles =  split(/,/," $listFreq ");
   		next if ($#alleles <= 0);

		print FH "-freq ";
   		print FH "$markerNum ".($#alleles+1)."\n";
  		&Processing::debug_print("====markerNum = $markerNum=========");
     
  		foreach my $allele (@alleles)
		{
			&Processing::debug_print("freq  =  $allele ");
			if ($allele =~ /^\s*$/)
			{
				my $message = "You didn't specify one or more frequencies for marker # $markerNum\n";
				open MES, "Texts\/AlleleFreqError.txt"
					||	&Processing::debug_print("Failed reading from (Texts\/AlleleFreqError.txt) file");
				my $errorFile = join "\n",<MES>;
				close MES;
				close FH;
				
				return &Processing::makeInputError( $message.$errorFile, \%variables );
   			}
			print FH " $allele";
   		}
  		print FH "\n";
   	}
	close FH;

	@{$variables{ alleleNum } } = @freq;
	return \%variables;
}
################################################################



################################################################
#				getMarkersNames
# 	Get Markers names
#
#	Parameters:
#					$_[0] - Output file
#					$_[1] - Hash with variables
#					$_[2] - Hash with input from HTML Form
#	Returns:
#					Hash with variables
################################################################
sub getMarkersNames( $ \% \% )
{
	my $outputFile = $_[0];
	my %variables = %{ $_[1] };
	%inputForm = %{ $_[2] };
	my @names = ();
	
  	&Processing::debug_print("getMarkersNames");
  	if ( ( $variables{ program } =~ /TwoLocus/ ) && ( $variables{ program } !~ /TwoPoint/ ) )
  	{
		if ( ! ( (defined $inputForm{ names1Option }) && ( $inputForm{ names1Option } =~ /ON/ ) ) )
		{
			for( $i = 1; $i <= $inputForm{ numMarkers1 }; $i ++ )
			{
				push @names, "Marker1_$i";
			}
		}
		else
		{
			$inputForm{ markersNames1 } = " ".$inputForm{ markersNames1 }." ";
			@names =  split ',', $inputForm{ markersNames1 };
			if ($#names != ($inputForm{ numMarkers1 }-1)) 
			{
				my $message = "You have specified $inputForm{ numMarkers1 } markers at first locus, but ".($#names+1)." markers names.\n
								The format is a list of names separated by commas.\n
								If you do not know one or more of the marker names,you can leave a blank 
								between the commas, in which case a generic name will be given.";
				return &Processing::makeInputError( $message, \%variables );
			}
		}
		if ( $inputForm{ "link" } =~/Unlinked/ ) 
		{
			if ( ! ( (defined $inputForm{ names2Option }) && ( $inputForm{ names2Option } =~ /ON/ ) ) )
			{
				for( $i = 1; $i <= $inputForm{ numMarkers2 }; $i ++ )
				{
					push @names, "Marker2_$i";
				}
			}
			else
			{
				$inputForm{ markersNames2 } = " ".$inputForm{ markersNames2 }." ";
				my @names2 =  split ',', $inputForm{ markersNames2 };
				if ($#names2 != ($inputForm{ numMarkers2 }-1)) 
				{
					my $message = "You have specified $inputForm{ numMarkers2 } markers at second locus, but ".($#names+1)." markers names.\n
									The format is a list of names separated by commas.\n
									If you do not know one or more of the marker names,you can leave a blank 
									between the commas, in which case a generic name will be given.";
					return &Processing::makeInputError( $message, \%variables );
				}
				push @names, @names2;
			}
		}
	}
	else
	{
		if ( ! ( (defined $inputForm{ namesOption }) && ( $inputForm{ namesOption } =~ /ON/ ) ) )
		{
			for( $i = 1; $i <= $variables{ numMarkers  }; $i ++ )
			{
				push @names, "Marker$i";
			}
		}
		else
		{
			$inputForm{ markersNames } = " ".$inputForm{ markersNames }." ";
			@names =  split ',', $inputForm{ markersNames };
			if ($#names != ($variables{ numMarkers }-1)) 
			{
				$message = "You have specified $variables{ numMarkers } markers, but $variables{ numMarkers } markers names.\n
								The format is a list of names separated by commas.\n
								If you do not know one or more of the marker names,you can leave a blank 
								between the commas, in which case a generic name will be given.";
				return &Processing::makeInputError( $message, \%variables );
			}
		}
	}
    
	%variables = %{&Processing::makeSystemError( "Failed appending to ($outputFile)", \%variables ) }
		unless ( open FH, ">>$outputFile" );

   	print FH "-names aff " if ( $variables{ program } !~ /TwoLocus/ );
   	print FH "-names aff1 aff2 " if ( $variables{ program } =~ /TwoLocus/ );
   	my $i = 0;
   	foreach my $name (@names)
	{
		$i++;
		if ($name =~ /^\s*$/) 
		{
  			$name = " Marker$i ";
			$names[ $i - 1 ] = $name;
		}
		$name =~ s/^\s+//sig;
		$name =~ s/\s+$//sig;
		$name =~ s/\s/_/sig;
		print FH " $name";
		&Processing::debug_print("$name");
   	}
   	print FH "\n";
	close FH;

	@{$variables{ markersNames }} = @names;
	return \%variables;
}
################################################################


################################################################
#				saveLast
# 	Checks, if project from Dan Geiger and sends him additional links
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					NOTHING
################################################################
sub saveLast( \% )
{
	%variables = %{ $_[0] };

	return if ( ( $Processing::YES != $Configuration::ONLINE ) || ( $Processing::YES != $Configuration::EMAIL ) );
		
  	if ($variables{ mail } =~ /dang\@cs.technion.ac.il/) 
	{

 		open LAST, "> ../last/index.html"
			|| &Processing::debug_print("Failed creating (../last/index.html) file");
 			print LAST "For segregation analysis 
			 	<a href = \"$HOME_WEB_DIR/var/$variables{ newDir }/$variables{ PROJ }.segregation-results.$variables{ currentDay }.html\">results</a><br>";
 			print LAST "For two point 
			 	<a href = \"$HOME_WEB_DIR/var/$variables{ newDir }/$variables{ PROJ }.two_point-results.$variables{ currentDay }.html\">results</a><br>";
 			print LAST "For multipint results 
			 	<a href = \"$HOME_WEB_DIR/var/$variables{ newDir }/$variables{ PROJ }.formatted_output.$variables{ currentDay }.html\">results</a><br> ";
 			print LAST "Input and output files 
			 <a href = \"$HOME_WEB_DIR/var/$variables{ newDir }/index.html\"> index.html </a><br>";
 		close LAST;
	}
}
################################################################



################################################################
#			getPedNames
#	Gets the names of the pedigrees from the SL-format file
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					Array with names
################################################################
sub getPedNames( \% )
{
	%variables = %{ $_[0] };
  
  	my $PedigreeNum = 0;
 	my @pedNames = ();
  	%variables = %{&Processing::makeSystemError("Failed to open file ($variables{ newDir }\/$variables{ pedfile })", \%variables)}
		unless ( open INPUT, "$variables{ newDir }\/$variables{ pedfile }" );
   		
  	foreach $line ( <INPUT> ) 
  	{
    	if ($line =~ /pedigree:.*person:/)
		{
      		my $tmp = $line;
      		$tmp =~ s/.*pedigree://sig;
      		$tmp =~ s/person:.*//sig;
      		$tmp =~ s/\s+//sig;
      		push @pedNames, "$tmp";
    	}
    	
		my @tmp = split(/\s+/,$line);
    	my $num = int( $tmp[0]);
    	$PedigreeNum = $num if ($PedigreeNum < $num);
   	}
  	unless ( @pedNames )
	{
		for (my $i = 1; $i <=  $PedigreeNum; $i++)
		{
			push @pedNames, "Pedigree$i";
		}
  	}
	close INPUT;
	
	return \@pedNames;
}
################################################################



################################################################
#			getVariablesFromDataFile
#	Gets all the possible variables from the datafile
#
#	Parameters:
#					$_[0] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub getVariablesFromDataFile( \% )
{
 	%variables = %{ $_[0] };
	&Processing::debug_print( "Get Variables From Data File: ($variables{ newDir }\/$variables{ datafile })");
	return &Processing::makeSystemError( "Cann't open datafile ($variables{ newDir }\/$variables{ datafile })", \%variables )
		unless ( open DATA, "$variables{ newDir }\/$variables{ datafile }" );

	my @lines = <DATA>;
	chomp @lines;
	close DATA;
	&Processing::debug_print( "Data=(\n@lines)");
	 
	$line = &shiftLine( \@lines, \%variables );		#	<< NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOC
	@words = split '<<', $line;
	@words = split /\s+/, $words[0];
	return &Processing::makeInputError( "Wrong datafile format - Wrong first line. Has to be at least 4 parameteres (default single loci claculation) and not ".(scalar @words), \%variables )
		if ( scalar @words < 4 );
	$words[4] = 0 unless ( defined $words[4] );
	my $numMarkers = $words[0] ;
	my $aff = $words[4];
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		return &Processing::makeInputError( "Wrong datafile format - Wrong affection value for Two Loci calculation. Has to be 2", \%variables ) unless ( $aff == 2 ) ;
		$variables{ numMarkers } = ( $numMarkers - 2 );
		$variables{ numMarkers1 } = $variables{ numMarkers };
		$variables{ numMarkers2 } = $variables{ numMarkers };
	}
	else
	{
		return &Processing::makeInputError( "Wrong datafile format - Wrong affection value for Single Loci calculation. Has to be 0", \%variables ) unless ( $aff == 0 ) ;
		$variables{ numMarkers } = ( $numMarkers - 1 );
	}
	&Processing::debug_print( "getVariablesFromDataFile: numMarkers=($variables{ numMarkers })");
	return &Processing::makeInputError( "Wrong datafile format - wrong number of markers", \%variables )
		unless ( $variables{ numMarkers } > 0 );

	$variables{ sexLinked } = ( $words[2] );
	&Processing::debug_print( "getVariablesFromDataFile: sexLinked=($variables{ sexLinked })");
	return &Processing::makeInputError( "Wrong datafile format - wrong sex linked type", \%variables )
		unless ( ( $variables{ sexLinked } == 0 ) || ( $variables{ sexLinked } == 1 ) );
	
	$line = &shiftLine( \@lines, \%variables );		#	<< MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)
	$line = &shiftLine( \@lines, \%variables ); 		#	Order of markers
	@words = split '<<', $line;
	@words = split /\s+/, $words[0];
	return &Processing::makeInputError( "Wrong datafile format - wrong marker order. Has to be $numMarkers markers and not ".(scalar @words), \%variables )
		unless ( $numMarkers == (scalar @words) );
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		$variables{ linked } = "Linked";
# 	First Locus
		$line = &shiftLine( \@lines, \%variables ); 		#	<<  AFFECTION, NO. OF ALLELES 
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong Status line for the first affected loci. Has to start with 4 and not with $words[0]", \%variables )
			if ( $words[0] != 4 );
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of alleles for the first affected loci. Has to be 2 alleles and not $words[1]", \%variables )
			if ( $words[1] != 2 );
		$line = &shiftLine( \@lines, \%variables ); 		#	<< GENE FREQUENCIES
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of frequencies for the first affected loci. Has to be 2 frequencies and not ".(scalar @words), \%variables )
			unless ( scalar @words == 2 );
		$variables{ mutantFreq1 } = $words[ $#words ];

# 	Second Locus
		$line = &shiftLine( \@lines, \%variables ); 		#	<<  AFFECTION, NO. OF ALLELES 
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong Status line for the second affected loci. Has to start with 4 and not with $words[0]", \%variables )
			if ( $words[0] != 4 );
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of alleles for the second affected loci. Has to be 2 alleles and not $words[1]", \%variables )
			if ( $words[1] != 2 );
		$line = &shiftLine( \@lines, \%variables ); 		#	<< GENE FREQUENCIES
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of frequencies for the second affected loci. Has to be 2 frequencies and not ".(scalar @words), \%variables )
			unless ( scalar @words == 2 );
		$variables{ mutantFreq2 } = $words[ $#words ];
	}
	else
	{
		$line = &shiftLine( \@lines, \%variables ); 		#	<<  AFFECTION, NO. OF ALLELES
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong Status line for the affected loci. Has to start with 1", \%variables )
			if ( $words[0] != 1 );
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of alleles for the second affected loci. Has to be 2 alleles", \%variables )
			if ( $words[1] != 2 );
		$line = &shiftLine( \@lines, \%variables ); 		#	<< GENE FREQUENCIES
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format - Wrong number of frequencies for the affected loci. Has to be 2 frequencies", \%variables )
			unless ( scalar @words == 2 );
		$variables{ mutantFreq } = $words[ $#words ];
	}
	$line = &shiftLine( \@lines, \%variables ); 		#	<< NO. OF LIABILITY CLASSES
	return &Processing::makeInputError( "Wrong datafile format!!! Most probable reasons - truncated file. Failed parsing line with number of liability classes", \%variables )
		unless ( $Processing::NO =~ $variables{ errorExist } );														

	@words = split '<<', $line;
	$variables{ mode } = $words[ 0 ];
	$variables{ mode } =~ s/\s+//g;
	&Processing::debug_print( "getVariablesFromDataFile: mode=($variables{ mode })");
	my $i = 0;
	for ( my $m = 0; $m < $variables{ mode }; $m ++ )
	{
		my $size = 1;
		$size = 2 if ( ( 0 == $aff ) && ( $variables{ sexLinked } == 1 ) );
		$size = 3 if ( ( 2 == $aff ) && ( $variables{ sexLinked } == 0 ) );
		$size = 5 if ( ( 2 == $aff ) && ( $variables{ sexLinked } == 1 ) );
		&Processing::debug_print( "getVariablesFromDataFile: Has to be ($size) lines for each mode");
		for ( my $p = 0; $p < $size; $p ++ )
		{
			$line = &shiftLine( \@lines, \%variables );	#	<< PENETRANCE VALUE
			chomp $line;
			@words = split '<<', $line;
			${ $variables{ penetrances } }[$i] = $words[ 0 ];
			$i ++;
		}
	}
	return &Processing::makeInputError( "Wrong datafile format!!! Most probable reasons - truncated file. ", \%variables )
		unless ( $Processing::NO =~ $variables{ errorExist } );														
	&Processing::debug_print( "getVariablesFromDataFile: All requested penetrance line received");

	$line = &shiftLine( \@lines, \%variables );	#	<< PENETRANCE VALUE
	return &Processing::makeInputError( "Wrong datafile format!!! Wrong number of penetrance lines! ", \%variables )
		unless ( $line =~ /^3\s+\d+/ );														
	&Processing::debug_print( "getVariablesFromDataFile: penetrances=(@{ $variables{ penetrances } })");
	@{$variables{ markersNames }} = ();
	my $markId = 0;
	while ( ( $line =~ /^3\s+\d+/ )	&& ( $variables{ errorExist } == $Processing::NO ) )	#	<< SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2)
	{
		$markId ++;
		my $name = "";
		return &Processing::makeInputError( "Wrong datafile format!!! Wrong name for marker $markId. You can't use # inside marker name ", \%variables )
			if ($line =~ /\#.*\#.*\#/);														
		if ($line =~ /\#.*\#/)
		{
 			$name = $&;
 			$name =~ s/#//sig;
 			&Processing::debug_print("name = $name");
   		}
		else
		{
			$name = "Marker_$markId";
			&Processing::debug_print("No name for marker $markId. Given default name ($name)");
		}
		my @words = split /\s+/, $line;
		my $markAlleles = $words[1];
		return &Processing::makeInputError( "Wrong datafile format!!! It can't be single allele for marker $markId ", \%variables )
			if ( $markAlleles == 1 );
		push @{$variables{ markersNames }}, $name;
		$line = &shiftLine( \@lines, \%variables );	#	<< GENE FREQUENCIES
		@words = split '<<', $line;
		@words = split /\s+/, $words[0];
		return &Processing::makeInputError( "Wrong datafile format!!! Wrong number of frequencies for marker $markId. There are ".(scalar @words)." frequencies instead of $markAlleles  ", \%variables )
			unless ( $markAlleles == scalar @words );
		$line = &shiftLine( \@lines, \%variables );	#	<<  ALLELE NUMBERS, NO. OF ALLELE
	}
	&Processing::debug_print( "getVariablesFromDataFile: markersNames=(@{ $variables{ markersNames } })");
	return &Processing::makeInputError( "Wrong datafile format!!! Most probable reasons - truncated file. Failed while reading INFERENCE line", \%variables )
		unless ( $Processing::NO =~ $variables{ errorExist } );														
	return &Processing::makeInputError( "Wrong datafile format!!! Wrond INFERENCE line ", \%variables )
		unless ( $line =~ /\s*\d+\s+\d+/ );														
	$line = &shiftLine( \@lines, \%variables );		#	<< RECOMBINATION VALUES
	@words = split '<<', $line;
	@words = split /\s+/, $words[0];
	return &Processing::makeInputError( "Wrong datafile format!!! Wrond recombination numbers. You defined ".(scalar @words)." of thetas instead of ".($numMarkers-1), \%variables )
		unless ( scalar @words == ($numMarkers-1) );
	@{$variables{ thetas }}  = @words;
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		shift @{$variables{ thetas }};
		shift @{$variables{ thetas }};
	}
	else
	{
		shift @{$variables{ thetas }};
	}
	for (my $th=0; $th <= $#{$variables{ thetas }}; $th ++ )
	{
		next unless ( ( ${$variables{ thetas }} [$th] < 0 ) || ( ${$variables{ thetas }} [$th] > 0.5 ) );
		my $errMsg = "Wrong datafile format!!! Wrong theta ${$variables{ thetas }} [$th] at position ";
		$th ++ if ( $variables{ program } =~ /TwoLocus/ );
		return &Processing::makeInputError( $errMsg.($th +2), \%variables );
	}
	return &Processing::makeInputError( "Wrong datafile format!!! Most probable reasons - truncated file. Failed parsing last, run parameter line(s)", \%variables )
		unless ( $Processing::NO =~ $variables{ errorExist } );														
	&Processing::debug_print( "getVariablesFromDataFile: thetas=(@{ $variables{ thetas } })");
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		$variables{ to1 } = $variables{ numMarkers };
		$variables{ from1 } = 1;
		$variables{ from2 } = 1;
		$variables{ to2 } = $variables{ numMarkers };
	}
	else
	{
		$variables{ to } = $variables{ numMarkers };
		$variables{ from } = 1;
	}
	return \%variables;
}
################################################################
################################################################
##			GLOBAL FUNCTIONS:
################################################################
################################################################



################################################################
#			isImportantFile
#	Checks, if the file is important to user
#
#	Parameters:
#					$_[0] - filename
#	Returns:
#					$Processing::YES	-	Important
#					$Processing::NO	-	Not important
################################################################
sub isImportantFile( $ )
{
	my $file = $_[0];
	
	if (   ( $file =~ /$Processing::OUTPUT_FILE/ )
		|| ( $file eq "debug.txt" )
		|| ( $file =~ /input/ )
		|| ( $file =~ /conversion-table/ )
		)
	{
		return $Processing::YES;
	}
	return $Processing::NO;
}
################################################################


################################################################
#			findMax
#	Finds Maximum value in array
#
#	Parameters:
#					$_[0] - Array of numbers
#	Returns:
#					Max value in array
################################################################
sub findMax( \@ )
{
	my @array = @{ $_[0] };
  	my $max = $array[0];
	  
  	foreach $item ( @array )
	{
    	$max = $item if ( $max < $item );
  	}
  	return $max;
}
################################################################



################################################################
#			executeCommand
#	Executes system command and checks the errors. Send an email with them
#
#	Parameters:
#					$_[0]  <= Command to execute
#					$_[1] - Hash with variables
#	Returns:
#				Hash with variables
################################################################
sub executeCommand( $ \% )
{
	my $command = $_[0];
	%variables = %{ $_[1] };

	&Processing::debug_print( $command );
	my $errorFile = "$Processing::ERROR_FILE.execCmd";
	unless ( -e $errorFile )
	{
		%variables = %{&Processing::makeSystemError("Cann't open ($errorFile) to write", \%variables)}
			unless ( open ERR, ">$errorFile");
		close ERR;
	}
	%variables = %{&Processing::makeSystemError("Cann't open ($errorFile) to write", \%variables)}
		unless ( -w $errorFile );
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														
	system( "$command 2>> $errorFile");
	if ( -s $errorFile )
	{
		my $size = -s $errorFile;
		&Processing::debug_print("error size: $size");

		%variables = %{&Processing::makeSystemError("Cann't open ($errorFile) to read", \%variables)}
			unless ( open ERROR, $errorFile );
		return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														
		
		my @error = <ERROR>;
		my $errorMsg = join "\n",@error;
		close ERROR;
		
		%variables = %{&Processing::makeSystemError($errorMsg, \%variables)};
	}
	unlink $errorFile;
	return \%variables;
}
################################################################




################################################################
#			printHash
#	Prints the given hash into output file or on the screen, if $_[1] == "STDOUT"
#
#	Parameters:
#					$_[0] - Hash Name
#					$_[1] - Output file
#					$_[2] - Hash to print
#	Returns:
#					NOTHING
################################################################
sub printHash( $ $ \% )
{
	my $hashName = $_[0];
	my $output = $_[1];
	my %hash = %{ $_[2] };
	
	if ( $output ne "STDOUT" )
	{
		open OUT, ">>$output"
			|| &Processing::debug_print( "Failed to append to file ($output)");
		print OUT "\n$hashName:\n====================\n";
		foreach $key ( keys ( %hash ) )
		{
			next if ( ! defined $hash{ $key } );

			my @array =  ( $hash{ $key } );
			if ( $#array <= 0 )
			{
				print OUT  "$key  =  $hash{ $key }\n" if ( $#array == 0);
				next;
			}
			my $message = "$key  =  \n";
			for( my $i = 0; $i <= $#array; $i ++ )
			{
				$message = $message."($i=$array[$i])";
			}
			print OUT "$message\n";
		}
		print OUT "\n-------------------------\n";
		close OUT;			
	}
	else
	{
		&Processing::debug_print( "\n$hashName:\n" );
		&Processing::debug_print( "====================\n");
		foreach $key ( keys ( %hash ) )
		{
			next if ( ! defined $hash{ $key } );

			my @array =  ( $hash{ $key } );
			if ( $#array <= 0 )
			{
				&Processing::debug_print( "$key  =  $hash{ $key } " ) if ( $#array == 0);
				next;
			}
			my $message = "$key  =  \n";
			for( my $i = 0; $i <= $#array; $i ++ )
			{
				$message = $message."($i=$array[$i])";
			}
			&Processing::debug_print( $message );
		}
		&Processing::debug_print ( "-------------------------\n" );
	}
}

################################################################




################################################################
# 		getComplexity
#	Get's the calculation complexity
#
#	Parameters:
#					$_[0] - directory
#					$_[1] - datafile
#					$_[2] - pedfile
#					$_[3] - Hash with variables
#	Returns:
#					Complexity
################################################################
sub getComplexity( $ $ $ \% )
{
	my $dir = $_[0];
	my $datafile = $_[1];
	my $pedfile = $_[2];
	%variables = %{$_[3]};
	
	$pedfile = "$dir\/$pedfile";
	$datafile = "$dir\/$datafile";
	$output = "complexity.txt";
	&Processing::debug_print("Get Complexity from dir($dir), datafile($datafile), pedfile($pedfile)" );

	&executeCommand( "$Configuration::PROG_DIR\/$Configuration::SUPERLINK_PROG $datafile $pedfile -c > $output", \%variables );

	my $maxComplexity = 0;
	%variables = %{&Processing::makeSystemError("Cannot open ($output) file for reading", \%variables)}
		unless ( open RESULT, $output );
	exit $Processing::ERROR unless ( $Processing::NO =~ $variables{ errorExist } );														

	foreach my $line ( <RESULT> )
	{
		&Processing::debug_print("The line is ($line)" );
		if ( $line =~ /The complexity coefficient of the elimination order found/) 
		{
			$line =~ s/^\s+//sig;
			$line =~ s/\s+$//sig;
			my @words = split(/\s+/,$line);
			$line = join ' ! ', @words;
			$words[$#words] =~ s/\D+//sig;
			if ($#words >= 0)
			{
				if ( $maxComplexity < $words[ $#words] ) 
				{
	    			$maxComplexity = $words[$#words];
					&Processing::debug_print("The Complexiry is ($maxComplexity)" );
				}
	  		}
		}
	}
	close RESULT;
	
	return $maxComplexity;
}
################################################################



################################################################
# 		sortArrayBySubField
#	Sorts the Array with lines by the given subfield in the line
#
#	Parameters:
#					$_[0] - Not sorted array
#					$_[1] - Subfield
#	Returns:
#					Sorted Array
################################################################
sub sortArrayBySubField( \@ $ )
{
	my @array = @{ $_[0] };
	my $field = $_[1];
	&Processing::debug_print("Sorting array by field number ($field) ");
	
	for ( my $i = 1; $i <= $#array; $i ++ )
	{
		my @words = split /\s+/, $array[$i];
		my @prev = split /\s+/, $array[$i - 1];
	
		if ( $words[ $field ] < $prev[ $field ] )
		{
			my $temp = $array[$i];
			$array[$i] = $array[$i-1];
			$array[$i-1] = $temp;
			$i-=2 if ( $i > 1 );
		}
	}
	
	return \@array;
}
################################################################



################################################################
#			PRIVATE FUNCTIONS
################################################################
################################################################



################################################################
#				dayMonth
# returns current day and month
#Variable    Values       Range
# $sec       seconds      0-60
# $min       minutes      0-59
# $hours      hours       0-23
# $mday     day of month  1-31
# $month    month of year 0-11, 0 == January
# $year     years since   1900  1-138 (or more)
# $wday     day of week   0-6, 0 == Sunday
# $yday     day of year   1-366
# $isdst    0 or 1        true if daylight savings is in effect
#
#	Parameters:
#					NOTHING
#	Returns:
#					Current Day and Month in format: (day.month)
################################################################
sub dayMonth()
{
  # using Time::tm objects
 # use Time::localtime;
     $tm = localtime;
     @date = split(/[ ]+/,$tm);
     # print ("date = @date");
     #print ("return  $date[2].'.'.$date[1]");
    return  $date[2].'.'.$date[1];
 }
################################################################



################################################################
#			shiftLine
#	Gets the first variable (line) from the array or exits if empty
#	If the program is right - never should exit.
#
#	Parameters:
#					$_[0] - Array
#					$_[1] - Hash with variables
#	Returns:
#					First variable (line)
################################################################
sub shiftLine( \@ \% )
{
	%variables = %{$_[1]};
	while ( ( ${ $_[0] }[0] !~ /\S/ ) && ( $#{ $_[0] } >= 0 ) )
	{
		shift @{ $_[0] };
	}
	%variables = %{&Processing::makeSystemError( "Empty array!!!=(@{ $_[0] })", \%variables )}
		if ( $#{ $_[0] } < 0 ); # Never should happen

	${ $_[0] }[0] =~ s/^\s+//;
	&Processing::debug_print( "The line is: (${ $_[0] }[0])"  );
	return shift @{ $_[0] };
}
################################################################
1;
