package JavaExecutor;
################################################################
# 		This file is the library for all functions,
#	which used in work with Java.
#	Java programs to be replaced:
#		mkaeped.commandLine.CheckFiles
#		mkaeped.commandLine.RemoveLociMain
#		mkaeped.commandLine.SimpleToSLFormatMain
################################################################

use Configuration;
use Processing;
use InputValidation;

################################################################
sub commentFiles( $ $ \% );
sub commentPedfile( $ \% );
sub commentDatafile( $ \% );
sub removeSpareLoci( $ $ \@ \% );
sub simpleToSLFormat( $ \% );
sub createPedfile( $ \% );
sub convertData( \@  );
sub createDatafile( \@ \@ \% );
################################################################


################################################################
# 		commentFiles
#	Instead of mkaeped.commandLine.CheckFiles
#	Adding comments
#	Parameters:
#					$_[0] - Datafile
#					$_[1] - Pedfile
#					$_[2] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub commentFiles( $ $ \% )
{
	my $datafile = $_[0];
	my $pedfile = $_[1];
	my %variables = %{ $_[2] };
	&Processing::debug_print("started JavaExecutor::commentFiles, with parameters: datafile=($datafile), pedfile=($pedfile)");

	%variables = %{ &commentPedfile($pedfile, \%variables ) };
	%variables = %{ &commentDatafile($datafile, \%variables ) };
	return \%variables;
}
################################################################

################################################################
# 		commentPedfile
#	Adding comments to pedfile. Checks it's syntax
#	Parameters:
#					$_[0] - Pedfile
#					$_[1] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub commentPedfile( $ \% )
{
	my $pedfile = $_[0];
	my %variables = %{ $_[1] };
	&Processing::debug_print("started JavaExecutor::commentPedfile, with pedfile=($pedfile)");

	%variables = %{&Processing::makeSystemError("JavaExecutor::commentPedfile: Failed opening pedfile ($pedfile) for reading", \%variables)}
		unless ( open PED, $pedfile );
	my @ped = <PED>;
	close PED;
	chomp @ped;
	%variables = %{&Processing::makeSystemError("JavaExecutor::commentPedfile: Failed opening pedfile ($pedfile) for writing", \%variables)}
		unless ( open PED, ">$pedfile" );
	for( my $p=0; $p <= $#ped; $p ++ )
	{
		$pedline = $ped[$p];
		next unless ( $pedline =~ /\S+/ );
		$pedline =~ s/\?/0/g;
		$pedline =~ s/^\s+//;
		$pedline =~ s/\s+/ /g;

		my @words = split /<</, $pedline;
		@words = split /\s+/, $words[0];
		
		return &Processing::makeInputError( "Pedfile error!!! Not enough data columns line ".($p+1).". Has to be data for ".($variables{ numMarkers })." markers (".(10 + 2 * $variables{ numMarkers })." columns). And you had specified only ".(scalar @words)." columns", \%variables ) 
			if ( $#words < (9 + 2 * $variables{ numMarkers }) );
		for ( my $d = 0; $d <= (9 + 2 * $variables{ numMarkers }); $d ++ )
		{
			return &Processing::makeInputError( "Pedfile error!!! Non digital value at column ".($d+1).",  line ".($p+1), \%variables ) 
				if( $words[$d] =~ /\D/ );
			return &Processing::makeInputError( "Pedfile error!!! Negative value at column ".($d+1).",  line ".($p+1), \%variables ) 
				if( $words[$d] < 0 );
		}

		$pedline = $pedline." << " unless ( $pedline =~ /<</ );
		print PED "$pedline\n";
	}
	close PED;
	&Processing::debug_print("finished JavaExecutor::commentPedfile");
	return \%variables;
}
################################################################

################################################################
# 		commentDatafile
#	Adding comments to pedfile. Checks it's syntax
#	Parameters:
#					$_[0] - Datafile
#					$_[1] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub commentDatafile( $ \% )
{
	my $datafile = $_[0];
	my %variables = %{ $_[1] };
	&Processing::debug_print("started JavaExecutor::commentDatafile, with datafile=($datafile)");

	%variables = %{&Processing::makeSystemError("JavaExecutor::commentDatafile: Failed opening datafile ($datafile) for reading", \%variables)}
		unless ( open DATA, $datafile );
	my @data = ();
	foreach my $dline (<DATA> )
	{
		next unless ( $dline =~ /\S+/ );
		chomp $dline;
		$dline =~ s/^\s+//;
		$dline =~ s/<<.*//;
		push @data, $dline;
	}
	close DATA;

	%variables = %{&Processing::makeSystemError("JavaExecutor::commentDatafile: Failed opening datafile ($datafile) for writing", \%variables)}
		unless ( open DATA, ">$datafile" );
	my $ind = 0;
	my @words = split /\s+/, $data[$ind];
	my $sex = $words[2];
	my $code = $words[3];
	$words[4] = 0 unless ( defined $words[4] );
	my $aff = $words[4];
	return &Processing::makeInputError( "Wrong affection value!", \%variables ) unless ( ($aff ==0) || ($aff ==2) );
	return &Processing::makeInputError( "Wrong Sex value!", \%variables ) unless ( ($sex ==0) || ($sex ==1) );
	$aff ++ if ( 0 == $aff );
	$data[$ind] = join " ", @words;

	print DATA "$data[$ind] << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI \n";$ind++;
	print DATA "$data[$ind] << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1) \n";$ind++;
	print DATA "$data[$ind] \n";$ind++;

	for ( my $a=0; $a < $aff; $a ++ )
	{
		my $name = "";
		$name = " #aff".(1+$a)."#" unless ( $data[$ind] =~ /\#.*\#/ );
		print DATA $data[$ind]." $name <<  AFFECTION, NO. OF ALLELES \n";$ind++;
		print DATA $data[$ind]."  << GENE FREQUENCIES \n";$ind++;
	}

	my $pene = $data[$ind];
	print DATA $data[$ind]." << NO. OF LIABILITY CLASSES ";$ind++;
	my $lines = 1;
	for ( my $p = 0; $p < $pene; $p++ )
	{
		$lines = 2 if ( ( 1 == $aff ) && ( 1 == $sex ) );
		$lines = 3 if ( ( 2 == $aff ) && ( 0 == $sex ) );
		$lines = 5 if ( ( 2 == $aff ) && ( 1 == $sex ) );
	
		for ( my $l = 0; $l < $lines; $l++ )
		{
			print DATA " \n".$data[$ind];$ind++;
		}
	}
	print DATA " << PENETRANCES \n";

	my $marker = 1;
	while ( $data[$ind] =~ /^3/ )
	{
		my $name = "";
		$name = " #Marker_$marker#" unless ( $data[$ind] =~ /\#.*\#/ );
		print DATA $data[$ind]." $name <<  ALLELE NUMBERS, NO. OF ALLELES \n";$ind++;
		print DATA $data[$ind]." << GENE FREQUENCIES \n";$ind++;
		$marker++;
	}

	@words = split /\s+/, $data[$ind];
	my $inf = $words[0];
	print DATA $data[$ind]."  << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2) \n";$ind++;
	if ( 0 == $inf )
	{
		print DATA $data[$ind]."  << RECOMBINATION VALUES \n";$ind++;
	}
	elsif ( 1 == $inf )
	{
		print DATA $data[$ind]."  << RECOMBINATION VALUES \n";$ind++;
		print DATA $data[$ind]."  <<  FEMALE / MALE RATIOOF GENETIC DISTANCE \n";$ind++;
	}
	elsif ( 2 == $inf )
	{
		print DATA $data[$ind]."  << MALE RECOMBINATION VALUES \n";$ind++;
		print DATA $data[$ind]."  << FEMALE RECOMBINATION VALUES \n";$ind++;
	}
	else
	{
		return &Processing::makeInputError( "Wrong datafile format!!! Wrond INFERENCE number ", \%variables )
	}

	if ( $code == 4 )
	{
		print DATA $data[$ind]."  << LOCUS VARIED, FINISHING VALUE, NUM OF EVAL \n";$ind++;
	}
	elsif ( $code == 5 )
	{
		print DATA $data[$ind]."  << REC VARIED, INCREMENT, FINISHING VALUE \n";$ind++;
	}
	elsif ( ( $code == 8 ) || ( $code == 12 ) || ( $code == 13 ) || ( $code == 6 ) )
	{
		while ( $ind <= $#data )
		{
			print DATA $data[$ind]." \n";$ind++;
		}
	}
	elsif ( $code == 9 )
	{
		print DATA $data[$ind]."  << LOCUS VARIED \n";$ind++;
	}
	else
	{
		return &Processing::makeInputError( "Wrong datafile format!!! Wrond PROGRAM CODE number ", \%variables )
	}

	close DATA;

	&Processing::debug_print("finished JavaExecutor::commentDatafile");
	return \%variables;
}
################################################################

################################################################
# 		removeSpareLoci
#	Instead of REMOVE_LOCI_PROG
#	Parameters:
#					$_[0] - Datafile
#					$_[1] - Pedfile
#					$_[2] - Markers to leave
#					$_[3] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub removeSpareLoci( $ $ \@ \% )
{
	my $datafile = $_[0];
	my $pedfile = $_[1];
	my @params = @{ $_[2] };
	my %variables = %{ $_[3] };
	my @thetas = @{$variables{ thetas }};
	&Processing::debug_print("started JavaExecutor::removeSpareLoci, with parameters: datafile=($datafile), pedfile=($pedfile), params=(@params), thetas=(@thetas) and hash of variables");

# Update pedfile
	my $last = 9;
	$last ++ if ( ( $variables{ mode } =~ /\d+/ ) && ( $variables{ mode } > 1 ) );
	my @finalP = ();
	for ( my $i = 0; $i <= $#params; $i ++ )
	{
		push @finalP, $last - 1 + 2 * $params[$i];
		push @finalP, $last + 2 * $params[$i];
	}
	return &Processing::makeSystemError("JavaExecutor::removeSpareLoci: Failed opening pedfile ($pedfile) for reading", \%variables)
		unless ( open PED, $pedfile );
	my @ped = <PED>;
	close PED;
	chomp @ped;
	return &Processing::makeSystemError("JavaExecutor::removeSpareLoci: Failed opening pedfile ($pedfile) for writing", \%variables)
		unless ( open PED, ">$pedfile" );
	foreach my $pedline ( @ped )
	{
		my @words = split /\s+/, $pedline;
		my @left = @words[0..$last, @finalP ];
		@words = split /<</, $pedline;
		push @left, "<<";
		push @left, $words[1] if ( defined $words[1] );
		$pedline = join " ", @left;
		print PED "$pedline\n";
	}
	close PED;

# Update datafile
	return &Processing::makeSystemError("JavaExecutor::removeSpareLoci: Failed opening datafile ($datafile) for reading", \%variables)
		unless ( open DATA, $datafile );
	my @data = <DATA>;
	close DATA;

	return &Processing::makeSystemError("JavaExecutor::removeSpareLoci: Failed opening datafile ($datafile) for writing", \%variables)
		unless ( open DATA, ">$datafile" );
	
	my @info = split /\s+/, $data[0];
	my $loc = $info[0];
	my $aff = $info[4];
	$aff ++ if ( $aff == 0 );
	$info[0] = $aff + scalar ( @params );
	$data[0] = join " ",@info;
	print DATA $data[0]."\n";
	print DATA $data[1];

	for( my $l = 1; $l <= $info[0]; $l ++ )			#	<< LOCI ORDER
	{
		print DATA $l." ";
	}
	print DATA "\n";

	my $ind = 3;
	while ( $data[$ind] !~ /^3\s+\d+/ )
	{
		print DATA $data[$ind];
		$ind ++;
	}

	for ( my $cnt = 1; $cnt <= ($loc-$aff); $cnt ++ )		#	<< Other lines
	{
		$ind += 2;
		next unless grep( /^$cnt$/, @params );
		print DATA $data[$ind-2];
		print DATA $data[$ind-1];
	}
	
	@words = split /\s+/, $data[$ind];
	my $inf = $words[0];
	print DATA $data[$ind];
	$ind ++;
	
	@words = split /<</, $data[$ind];		#	<< RECOMBINATION VALUES 
	@words = split /\s+/, $words[0];		
	
	for ( my $a = 0; $a < $aff; $a ++ )
	{
		print DATA "0 ";
	}
	$data[$ind] = join " ", @thetas;
	print DATA $data[$ind]."  << RECOMBINATION VALUES \n";
	$ind ++;
	
	while ( $ind <= $#data )
	{
		print DATA $data[$ind];
		$ind ++;
	}

	close DATA;

	&Processing::debug_print("finished JavaExecutor::removeSpareLoci");
	return \%variables;
}
################################################################



################################################################
# 		simpleToSLFormat
#	Instead of SIMPLE_TO_SL_PROG
#	Parameters:
#					$_[0] - Date
#					$_[1] - Hash with variables
#	Returns:
#					Hash with variables
################################################################
sub simpleToSLFormat( $ \% )
{
	my $name = $_[0];
	my %variables = %{ $_[1] };
	&Processing::debug_print("started JavaExecutor::simpleToSLFormat, with parameters: name=($name) and hash of variables");

	my ( $pedfile, $conversion, $maxVals, $vars ) = &createPedfile( $variables{pedfile}, \%variables );
	%variables = %{ $vars };
	return \%variables unless ( $Processing::NO =~ $variables{ errorExist } );														

	%variables = %{&Processing::makeSystemError("JavaExecutor::simpleToSLFormat: Failed opening datafile ($datafile) for reading", \%variables)}
		unless ( open PED, ">$name.ped" );
	foreach my $pedline ( @{ $pedfile } )
	{
		print PED $pedline."\n";
	}
	close PED; 

	my @datafile = @{ &createDatafile( $conversion, $maxVals, \%variables ) };
	%variables = %{&Processing::makeSystemError("JavaExecutor::simpleToSLFormat: Failed opening datafile ($datafile) for reading", \%variables)}
		unless ( open DATA, ">$name.dat" );
	foreach my $dataline ( @datafile )
	{
		print DATA $dataline."\n";
	}
	close DATA; 

	&Processing::debug_print("finished JavaExecutor::simpleToSLFormat");
	return \%variables;
}
################################################################



################################################################
# 		createPedfile
#	Creating pedfile
#	Parameters:
#					$_[0] - Pedigree data
#					$_[1] - Hash with variables
#	Returns:
#					Pedfile array & Markers data array & Maximum values array
################################################################
sub createPedfile( $ \% )
{
	my @ped = split /\n/, $_[0];
	my %variables = %{ $_[1] };
	my @pedfile = ();
	my @data = ();

	&Processing::debug_print("started JavaExecutor::createPedfile: started pedfile parsing");
	while ( ( $ped[0] !~ /\S+/ ) && ( $#ped >= 0 ) ) { shift @ped };
	my $pednumber = 0;
	my $person = 0;
	my $dindex = 0;
	my @total = ();
	my @pedigrees = ();
	my @names = ();
	my $pedname = "pedigree";
	for( my $p=0; $p <= $#ped; $p ++ )
	{ 
		my $pedline = $ped[$p];
		next unless ( $pedline =~ /\S+/ );
		$pedline =~ s/^\s+//g;
		$pedline =~ s/\?/0/g;
		if ( $pedline =~ /^pedigree:/ )
		{
			&Processing::debug_print("JavaExecutor::createPedfile: found pedigree line ($pedline)");
			my @words = split //, $pedline;			
			pop @words if ( $words[$#words] =~ /\s+/ );												#  ????? WHY ON HELL CHOMP does not work ??????
			$pedigrees[$pednumber] = join "", @words;
			$pedname = $pedigrees[$pednumber];
			$pednumber ++;
			$person = 0;
			next;
		}
		if ( $pednumber == 0 )		#	Single pedigree
		{
			$pednumber ++;
			$person = 0;
		}

		$person ++;
		@words = split /\s+/, $pedline;
		%variables = %{ &Processing::makeInputError( "Pedfile error!!! Not enough data columns line ".($p+1).". Has to be data for ".($variables{ numMarkers })." markers  (".(5 + 2 * $variables{ numMarkers })." columns). And you had specified only ".(scalar @words)." columns", \%variables ) }
			if ( $#words < (4 + 2 * $variables{ numMarkers }) );
		return ( \@pedfile, $conversion, $max, \%variables ) unless ( $Processing::NO =~ $variables{ errorExist } );

		my $pName = shift @words;												#Name

		if ( defined ${ $names[ $pednumber ] }{ $pName } )
		{
			%variables = %{ &Processing::makeInputError( "Pedfile error!!! Name $pName appears at least twice in $pedname", \%variables ) };
			return ( \@pedfile, $conversion, $max, \%variables );
		}
		else
		{
			${ $names[ $pednumber ] }{ $pName } = $Processing::YES;
		}

		${${ $total[ $pednumber ] }[ $person ]}{name} = $pName;					#Name

		${${ $total[ $pednumber ] }[ $person ]}{father} = shift @words;			#Father
		${${ $total[ $pednumber ] }[ $person ]}{mather} = shift @words;			#Mather

		if ( ( "0" eq ${${ $total[ $pednumber ] }[ $person ]}{father} ) && ( "0" ne ${${ $total[ $pednumber ] }[ $person ]}{mather} ) )
		{
			%variables = %{ &Processing::makeInputError( "Pedfile error!!! In $pedname person $pName has only mather!!!", \%variables ) };
			return ( \@pedfile, $conversion, $max, \%variables );
		}
		if ( ( "0" ne ${${ $total[ $pednumber ] }[ $person ]}{father} ) && ( "0" eq ${${ $total[ $pednumber ] }[ $person ]}{mather} ) )
		{
			%variables = %{ &Processing::makeInputError( "Pedfile error!!! In $pedname person $pName has only father!!!", \%variables ) };
			return ( \@pedfile, $conversion, $max, \%variables );
		}

		${${ $total[ $pednumber ] }[ $person ]}{gender} = shift @words;			#Gender
		${${ $total[ $pednumber ] }[ $person ]}{disease} = shift @words;		#Disease status
		${${ $total[ $pednumber ] }[ $person ]}{data} = join " ", @words;		#Data
		push @data, ${${ $total[ $pednumber ] }[ $person ]}{data};
		${${ $total[ $pednumber ] }[ $person ]}{dindex} = $dindex;
		$dindex ++;
		${${ $total[ $pednumber ] }[ $person ]}{child} = 0;				#First Child
		${${ $total[ $pednumber ] }[ $person ]}{nextF} = 0;				#Next father's child
		${${ $total[ $pednumber ] }[ $person ]}{nextM} = 0;				#Next mather's child

		&Processing::debug_print($pedline);
	}
	&Processing::debug_print("JavaExecutor::createPedfile: finished pedfile parsing, found ($#total) families");
	
	my ( $conversion, $max, $dataP ) = &convertData( \@data );
	@data = @{ $dataP };

	for( my $f=1; $f<= $#total; $f ++ )
	{
		&Processing::debug_print("JavaExecutor::createPedfile: family ($f), size= ($#{ $total[$f] })");
		for ( my $p=1; $p <= $#{ $total[$f] }; $p ++ )
		{
# Father			
			unless ( ${ ${ $total[$f] }[$p] }{father} eq 0 )
			{
				for ( my $pf=1; $pf <= $#{ $total[$f] }; $pf ++ )
				{
					next unless ( ${ ${ $total[$f] }[$p] }{father} eq ${ ${ $total[$f] }[$pf] }{name} );
					${ ${ $total[$f] }[$p] }{father} = $pf;
					${ ${ $total[$f] }[$pf] }{child} = $p     if ( ( ${ ${ $total[$f] }[$pf] }{child} eq 0 ) || ( ${ ${ $total[$f] }[$pf] }{child} > $p ) );
					last;
				}
			}
# Mather			
			unless ( ${ ${ $total[$f] }[$p] }{mather} eq 0 )
			{
				for ( my $pm=1; $pm <= $#{ $total[$f] }; $pm ++ )
				{
					next unless ( ${ ${ $total[$f] }[$p] }{mather} eq ${ ${ $total[$f] }[$pm] }{name} );
					${ ${ $total[$f] }[$p] }{mather} = $pm;
					${ ${ $total[$f] }[$pm] }{child} = $p     if ( ( ${ ${ $total[$f] }[$pm] }{child} eq 0 ) || ( ${ ${ $total[$f] }[$pm] }{child} > $p ) );
					last;
				}
			}
# Gender
			${ ${ $total[$f] }[$p] }{gender} = 1 if ( (${ ${ $total[$f] }[$p] }{gender} eq 1) || (${ ${ $total[$f] }[$p] }{gender} eq "m") || (${ ${ $total[$f] }[$p] }{gender} eq "male") );
			${ ${ $total[$f] }[$p] }{gender} = 2 if ( (${ ${ $total[$f] }[$p] }{gender} eq 2) || (${ ${ $total[$f] }[$p] }{gender} eq "f") || (${ ${ $total[$f] }[$p] }{gender} eq "female") );

			%variables = %{ &Processing::makeInputError( "Pedfile error!!! Wrong gender for person ".${ ${ $total[$f] }[$pm] }{name}, \%variables ) }
				unless  ( ( ${ ${ $total[$f] }[$p] }{gender} == 1 ) || ( ${ ${ $total[$f] }[$p] }{gender} == 2 ) );
# Disease
			${ ${ $total[$f] }[$p] }{disease} = 0 if ( (${ ${ $total[$f] }[$p] }{disease} eq 0) || (${ ${ $total[$f] }[$p] }{disease} eq "u") || (${ ${ $total[$f] }[$p] }{disease} eq "unknown") );
			${ ${ $total[$f] }[$p] }{disease} = 1 if ( (${ ${ $total[$f] }[$p] }{disease} eq 1) || (${ ${ $total[$f] }[$p] }{disease} eq "h") || (${ ${ $total[$f] }[$p] }{disease} eq "healthy") );
			${ ${ $total[$f] }[$p] }{disease} = 2 if ( (${ ${ $total[$f] }[$p] }{disease} eq 2) || (${ ${ $total[$f] }[$p] }{disease} eq "a") || (${ ${ $total[$f] }[$p] }{disease} eq "affected") );
			%variables = %{ &Processing::makeInputError( "Pedfile error!!! Wrong disease status for person ".${ ${ $total[$f] }[$pm] }{name}, \%variables ) }
				unless  ( ( ${ ${ $total[$f] }[$p] }{disease} == 0 ) || ( ${ ${ $total[$f] }[$p] }{disease} == 1 ) || ( ${ ${ $total[$f] }[$p] }{disease} == 2 ) );
# Data
			${ ${ $total[$f] }[$p] }{data} = join " ", $data[ ${ ${ $total[$f] }[$p] }{dindex} ];
		}
	}
	&Processing::debug_print("JavaExecutor::createPedfile: finished pedfile changing format");
	for( $f=1; $f<= $#total; $f ++ )
	{
		for ( $p=1; $p <= $#{ $total[$f] }; $p ++ )
		{
# Father			
			for ( $pf=$p+1; $pf <= $#{ $total[$f] }; $pf ++ )
			{
				next unless ( ${ ${ $total[$f] }[$p] }{father} eq ${ ${ $total[$f] }[$pf] }{father} );
				next if ( ${ ${ $total[$f] }[$p] }{father} eq 0 );
				${ ${ $total[$f] }[$p] }{nextF} = $pf;
				last;
			}
# Mather			
			for ( $pm=$p+1; $pm <= $#{ $total[$f] }; $pm ++ )
			{
				next unless ( ${ ${ $total[$f] }[$p] }{mather} eq ${ ${ $total[$f] }[$pm] }{mather} );
				next if ( ${ ${ $total[$f] }[$p] }{mather} eq 0 );
				${ ${ $total[$f] }[$p] }{nextM} = $pm;
				last;
			}
		}
	}
	&Processing::debug_print("JavaExecutor::createPedfile: finished brothers updating");
	for( $f=1; $f<= $#total; $f ++ )
	{
		my @family = @{ $total[$f] }; 
		for ( $p=1; $p <= $#family; $p ++ )
		{
			my @parts = @{ &BFSfamily( \@family ) };
			if ( scalar @parts > 1 )
			{
				my $pedname = "Pedigree";
				$pedname = $pedigrees[$f-1] if ( defined $pedigrees[$f-1] );
				my $message = "Pedfile error!!! $pedname is not fully connected.<br>\n";
				for ( my $part=0; $part <= $#parts;  )
				{	
					my $list = "";
					for ( my $l=0; $l <= $#{ $parts[$part] }; $l ++ )
					{
						$list = $list." ".${ $family[   ${ $parts[$part] }[$l]     ] }{name};
					}
					$part ++;
					$message = $message."Part $part:  $list<br>\n";
				}

				%variables = %{ &Processing::makeInputError( $message, \%variables ) };
				return ( \@pedfile, $conversion, $max, \%variables );
			}
			my %person = %{ $family[$p] };
			my @pedwords = ($f, $p, $person{father}, $person{mather}, $person{child}, $person{nextF}, $person{nextM}, $person{gender}, 0, "", $person{disease}, $person{data}, "\t<< $person{name} " );
			$pedline = join "\t", @pedwords;
			$pedline =~ s/<</<< $pedigrees[$f-1] person:/ if ( 1 == $p );
			push @pedfile, $pedline;
		}
	}
	&Processing::debug_print("JavaExecutor::createPedfile: finished creating pedlines array");

	return ( \@pedfile, $conversion, $max, \%variables );
}
################################################################

################################################################
# 		BFSfamily
#	Calculates the number of one of the connected peaces in family
#	Parameters:
#					$_[0] - Pedigree data
#	Returns:
#					List of sub families
################################################################
sub BFSfamily( \@ )
{
	my @family = @{ $_[0] };
	&Processing::debug_print("started JavaExecutor::BFSfamily: started. Family size=$#family");
	my @parts = ();
	my $currentInd = 0;
	my %list = ();
	for ( my $p=1; $p <= $#family; $p ++ )
	{
		$list{ $p } = $Processing::NO;
	}
	$list{ 0 } = $Processing::YES;
	$list{ ' ' } = $Processing::YES;
	my $done = 0;

	while ( $done < $#family )
	{
		&Processing::debug_print("JavaExecutor::BFSfamily: Starting NEW part" );
# Init
		my @block = (); 
		my @checkedBlock = (); 
		my $first = 1;
		while ( $list{ $first } == $Processing::YES )
		{
			$first ++;
		}
		push @block, $first;

# BFS
		while ( scalar @block > 0 )
		{
# pop person
			my $person = pop @block;
			next if ( $list{ $person } == $Processing::YES );
			push @checkedBlock, $person;
			$list{ $person } = $Processing::YES;
			$done ++;
# push relatives
			my %info = %{ $family[$person] };
			my @relKeys = ( "father", "mather", "child", "nextF", "nextM" );
			my @relatives = ();
			foreach my $r ( @relKeys )
			{
				push @relatives, $info{$r} if ( defined $info{$r} );
			}
			push @block, @relatives if ( scalar @relatives > 0 );
		}

		@{ $parts[ $currentInd ] } = @checkedBlock;
		$currentInd  ++;
	}

	&Processing::debug_print("finished JavaExecutor::BFSfamily: found $currentInd families" );
	return \@parts;
}


################################################################
# 		convertData
#	Converting pedlines into updated pedlines
#	Parameters:
#					$_[0] - Pedigree data
#	Returns:
#					Updated data array & max values array
################################################################
sub convertData( \@ )
{
	my @data = @{ $_[0] };

	&Processing::debug_print("started JavaExecutor::convertData: started");
	my @markers = ();
	my @max = ();
	my @words = split /\s+/, $data[0];
	for ( my $m = 0; $m < (scalar @words)/2; $m ++ )
	{
		$max[$m] = 0;
	}

	for ( my $p=0; $p<=$#data; $p ++ )
	{
		@words = split /\s+/, $data[$p];
		for ( $m = 0; $m < (scalar @words)/2; $m ++ )
		{
			unless ( $words[ 2*$m ] eq 0 )
			{
				unless( defined ${ $markers[$m] }{ $words[ 2*$m ] } )
				{
					$max[$m] ++;
					${ $markers[$m] }{ $words[ 2*$m ] } = $max[$m];
				}
				$words[ 2*$m ] = ${ $markers[$m] }{ $words[ 2*$m ] };
			}
			unless ( $words[ 2*$m+1 ] eq 0 )
			{
				unless( defined ${ $markers[$m] }{ $words[ 2*$m+1 ] } )
				{
					$max[$m] ++;
					${ $markers[$m] }{ $words[ 2*$m+1 ] } = $max[$m];
				}
				$words[ 2*$m+1 ] = ${ $markers[$m] }{ $words[ 2*$m+1 ] };
			}
		}
		$data[$p] = join " ", @words;
	}

	&Processing::debug_print("started JavaExecutor::convertData: finished");
	return (\@markers, \@max, \@data);
}
################################################################


################################################################
# 		createDatafile
#	Creating datafile
#	Parameters:
#					$_[0] - Conversion table
#					$_[1] - Max marker values
#					$_[2] - Hash with variables
#	Returns:
#					Pedfile array
################################################################
sub createDatafile( \@ \@ \% )
{
	my @conversion = @{ $_[0] };
	my @maxVals = @{ $_[1] };
	my %variables = %{ $_[2] };
	my @datafile = ();

	&Processing::debug_print("started JavaExecutor::createDatafile: ");
	my $index = 0;
	my $numMarks = 1 + scalar @conversion;
	my $code = 8;
	my $aff = 0;
	if ( $variables{ program } =~ /TwoLocus/ )
	{
		$aff = 2;
		$numMarks ++;
	}
	
	$datafile[ $index ] = "$numMarks 0 $variables{ sexLinked } $code $aff << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI "; $index++;
	$datafile[ $index ] = "0 0 0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1) "; $index++;
	
	$datafile[ $index ] = "";
	for ( my $i = 1; $i <= $numMarks; $i ++ )
	{
		$datafile[ $index ] = $datafile[ $index ]."$i ";	
	}
	$index++;											

	if ( $aff == 0 )
	{
		$datafile[ $index ] = "1 2 #aff# <<  AFFECTION, NO. OF ALLELES "; $index++;														
		$datafile[ $index ] = (1-$variables{ mutantFreq })." $variables{ mutantFreq }  << GENE FREQUENCIES "; $index++;	
	}
	elsif ( $aff == 2 )
	{
		$datafile[ $index ] = "4 2 #aff1# <<  AFFECTION, NO. OF ALLELES "; $index++;														
		$datafile[ $index ] = (1-$variables{ mutantFreq1 })." $variables{ mutantFreq1 } << GENE FREQUENCIES "; $index++;	
		$datafile[ $index ] = "4 2 #aff2# <<  AFFECTION, NO. OF ALLELES "; $index++;														
		$datafile[ $index ] = (1-$variables{ mutantFreq2 })." $variables{ mutantFreq2 } << GENE FREQUENCIES "; $index++;	
	}

	$datafile[ $index ] = "1 << NO. OF LIABILITY CLASSES "; $index++;
	for ( $i = 0; $i <= $#{ $variables{ penetrances } }; $i ++ )
	{
		$datafile[ $index ] = ${ $variables{ penetrances } }[$i]; $index++;
	}
	$datafile[ $index-1 ] = $datafile[ $index-1 ]." << PENETRANCES ";

	for ( my $m = 0; $m <= $#maxVals; $m ++ )
	{
		$datafile[ $index ] = "3 $maxVals[$m] #${ $variables{ markersNames } }[$m]# <<  ALLELE NUMBERS, NO. OF ALLELES "; $index++;
		my $freq = 1 / $maxVals[$m];
		$datafile[ $index ] = "";
		for ( my $f = 0; $f < $maxVals[$m]; $f ++ )
		{
			$datafile[ $index ] = $datafile[ $index ]."$freq ";
		}
		$datafile[ $index ] = $datafile[ $index ]." << GENE FREQUENCIES ";
		$index++;
	}
	$datafile[ $index ] = "0 0 << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2) "; $index++;	

	$datafile[ $index ] = "0 ";									#	Recombination
	$datafile[ $index ] = $datafile[ $index ]."0 " if ( 2 == $aff );
	for ( my $t = 0; $t <= $#{ $variables{ thetas } }; $t ++ )
	{
		$datafile[ $index ] = $datafile[ $index ]."${ $variables{ thetas } }[$t] ";
	}
	$datafile[ $index ] = $datafile[ $index ]." << RECOMBINATION VALUES ";
	$index++;

	if ( 0 == $aff )
	{
		$datafile[ $index ] = 1; $index++;														
		$datafile[ $index ] = "-n 3 1 ".( scalar @conversion)." -o 10"; $index++;														
	}
	elsif ( 2 == $aff )
	{
		$datafile[ $index ] = 2; $index++;														
		$datafile[ $index ] = "-n 3 1 ".( scalar @conversion)." -o 10"; $index++;														
		$datafile[ $index ] = "-n 3 1 ".( scalar @conversion)." -o 10"; $index++;														
	}

	&Processing::debug_print("finished JavaExecutor::createDatafile: ");
	return \@datafile;
}
################################################################
1;