package Notification;
################################################################
#	This is the library that works with all messages to user
################################################################

use FindBin ();
use lib "$FindBin::Bin";
use Configuration;
use InputValidation;
use Processing;
use UseCgiLibPl;
$TITLE = "SuperLink Online";

umask 0;

################################################################
################################################################
##			PUBLIC FUNCTIONS:
################################################################
##
sub printWindowHeader();
sub notifyAccepted( $ $ $ \% );
sub viewHtml( \% $ );
sub notifyResult( $ $ $ );
sub finishRun( $ );
sub notifyAdmin( $ $ $ );
sub notifyError( $ $ $ $ );
##
################################################################
##				PRIVATE FUNCTIONS
################################################################
##
sub printWindowButtom();
sub printAcceptedMessage( $ $ $ );
sub printNoResultMessage( $ );
##
################################################################
################################################################


################################################################
################################################################
# 			GLOBAL FUNCTIONS:
################################################################
################################################################



################################################################
#			finishRun
#	Finishes the program execution
#
#	Parameters:
#					$_[0] - Project directory
#	Returns:
#					Does NOT return
################################################################
sub finishRun( $ ) {
	my $newDir = $_[0];
	&Processing::debug_print("finishRun in directory ($newDir)");

	&Processing::debug_print("Failed changing to running directory ($Processing::RUN_DIR)")
		unless ( chdir $Processing::RUN_DIR );
	
	unless ( ( $Processing::YES == $Configuration::ONLINE )	|| ( $Processing::YES == $Processing::DEBUG ) ) {
		&Processing::debug_print( "Failed to enter project directory ($newDir)" )
			unless ( chdir $newDir );

		@files = <*>;
		foreach my $file ( @files )
		{
			unless ( $Processing::YES =~ &isImportantFile($file) )	
			{
				if ( -d $file )
				{										#	First directory layer
					&Processing::debug_print( "Failed to enter directory ($file)" )
						unless ( chdir $file );

					my @firstLayerFiles = <*>;
					foreach my $firstLayerFile ( @firstLayerFiles )
					{
						if ( -d $firstLayerFile )
						{								#	Second directory layer
							&Processing::debug_print( "Failed to enter directory ($firstLayerFile)" )
								unless ( chdir $firstLayerFile );
							
							my @secondLayerFiles = <*>;
							foreach my $secondLayerFile ( @secondLayerFiles )
							{
								unlink $secondLayerFile
									|| print OUT ( "Failed to remove ($secondLayerFile) from directory ($firstLayerFile) error=($!)" );
							}
							&Processing::debug_print( "Failed to enter back to directory ($file)" )
								unless ( chdir '..' );
							&Processing::debug_print( "Failed to remove directory ($firstLayerFile) error=($!)" )
								unless( rmdir $firstLayerFile );
						}
						else
						{
							&Processing::debug_print( "Failed to remove file ($firstLayerFile) error=($!)" )
								unless ( unlink $firstLayerFile );
						}
					}
					&Processing::debug_print( "Failed to enter back to directory ($newDir)" )
						unless ( chdir '..' );
					&Processing::debug_print( "Failed to remove directory ($file) error=($!)" )
						unless ( rmdir $file );
				}
				else
				{
					&Processing::debug_print( "Failed to remove file ($file) error=($!)" ) 
						unless ( unlink $file );
				}
			}
		}
	}

########################################
	&Processing::debug_print("\n\nEND OF PROGRAM!!!\n");
########################################
	exit(0);
}
################################################################



################################################################
#			viewHtml
#	Creates index.html
#
#	Parameters:
#					$_[0] - Hash with variables
# 					$_[2] - File to create
#	Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub viewHtml( \% $ )
{
	return $Processing::SUCCESS if ($Configuration::ONLINE == $Processing::NO);

	my %variables = %{ $_[0] };
	my $output = $_[1];
	
	&Processing::debug_print( "View HTML: output = ($output)" );
  	&Processing::debug_print ("openning $variables{ newDir }\/$output");
  	
  	%variables = %{ &makeSystemError("Failed writing to ($variables{ newDir }\/$output) ", \%variables )}
		unless ( open OUT, ">$variables{ newDir }\/$output" );
		
	return $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );
  	&Processing::debug_print ("openned");

		print OUT &UseCgiLibPl::HtmlTop( $TITLE );
  		print( OUT "<bgcolor=#F0FFFF>\n<p></p>\n<hr>\n<p></p>\n");
  
  		%variables = %{&makeSystemError("Failed changing to project directory ($variables{ newDir }", \%variables )}
			unless ( chdir $variables{ newDir } );
		return $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );

		my @filelist = ();
		foreach my $f ( <*> )
		{
			push @filelist, $f if ( $Processing::YES =~ &isImportantFile($f) );
		}
  		print( OUT "<H4>List of files for project: $variables{ PROJ }</H4>\n
             <p>Files are kept 48 hours starting submission time and then possibly removed depending on our resources. </p>\n");

  		foreach $datFile (@filelist) 
		{
   			print( OUT "&nbsp;&nbsp;&nbsp;&nbsp;<a href=$datFile>$datFile</a>\n<BR>\n");
  		}

  		print( OUT "<p></p>\n"); # error problem with pwd
#
#	NO SUCH INTERNAL PERL COMMAND.
#  		
	   	return $Processing::ERROR 
			if ($Processing::ERROR =~ &InputValidation::executeCommand("zip files.zip  @filelist > outzip.tmp", \%variables) );
 
  		print( OUT "Zipped version: <a href=files.zip>files.zip</a>\n<BR>\n");

  		print( OUT "<BR><BR><H5>To view files,&nbsp;  click on the link with the left mouse button.<BR>\n");
  		print( OUT "To save files,&nbsp;  click on the link with the right mouse button and then choose \"Save target as...\".<BR>\n</H5>\n");
  		print( OUT "</p>\n<p></p>\n<p></p>\n<hr>\n");
  		print( OUT "<p> Save these files before closing this window.</p>\n");
  		print( OUT "<p><a href=http://bioinfo.cs.technion.ac.il/superlink-online/>Superlink online </a></p>\n");
  		print OUT &printWindowButtom();
 	close OUT;

 	%variables = %{&makeSystemError("Failed changing to running directory ($Processing::RUN_DIR)", \%variables )}
		unless ( chdir $Processing::RUN_DIR );
	return $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );
	
	return $Processing::SUCCESS;
}
################################################################

################################################################
#			isImportantFile
#	Return if given file is important to user
#
#	Parameters:
#					$_[0] - Filename
#	Returns:
#				$Processing::YES - if file is important to user
#				$Processing::NO	- otherwise
################################################################
sub isImportantFile(  $ )
{
	my $file = $_[0];

	return $Processing::YES if ($file =~ /sl-input/ ) ;

	return $Processing::NO;
}
################################################################



################################################################
#			notifyAccepted
#	Prints the message that input was accepted. 
#	Sends the email if the EMAIL in configuration.pl is on.
#
#	Parameters:
#					$_[0] - Directory name
#					$_[1] - Project name
#					$_[2] - User email
#					$_[3] - Hash with variables
#	Returns:
#				$Processing::SUCCESS 	- if success
#				$Processing::ERROR	- if error
################################################################
sub notifyAccepted( $ $ $ \% )
{
	return $Processing::SUCCESS if ($Configuration::ONLINE == $Processing::NO);

	my $dir = $_[0];
	my $project = $_[1];
	my $email = $_[2];
	%variables = %{ $_[3] };
	
	&Processing::debug_print("Notify Accepted with dir = $dir, project = $project, email = $email");

 	&Processing::debug_print( "creating $dir/accepted.html");
	my $accepted = "";
	$accepted = $accepted.&printWindowHeader();
	$accepted = $accepted.&printAcceptedMessage( $project, $email, $dir );
	$accepted = $accepted.&printWindowButtom();
	
	unless ( -e "$dir\/$variables{ OUTPUT_FILE }")
	{
		&Processing::debug_print("creating $dir/$variables{ OUTPUT_FILE } ");
		%variables = %{&makeSystemError("Cannot create ($dir\/$variables{ OUTPUT_FILE })", \%variables)}
			unless ( open RES, ">$dir\/$variables{ OUTPUT_FILE }");
		return $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );

			print RES &UseCgiLibPl::HtmlTop( $TITLE );
			print RES &printNoResultMessage( $project );
			print RES &printWindowButtom();
		close RES;
	}
	
	print $accepted;
	
	if ( $Processing::YES == $Configuration::EMAIL )
	{
		&Processing::debug_print("Looking for email module...");
		require "./sendMail.pl";
	
		&Processing::debug_print(" sendMail (accepted, $email, $project, $dir\/, $Configuration::HOME_WEB_DIR/cgi-bin/$dir/$variables{ OUTPUT_FILE })");
		&sendMail
		(
			"accepted", 
			"$email",
			"$project", 
			"$dir/",
			"$Configuration::HOME_WEB_DIR/cgi-bin/$dir/$variables{ OUTPUT_FILE }"
		);
	}
	
	return $Processing::SUCCESS;
}
################################################################



################################################################
#			notifyError
#	Prints the message that input has an error.
#	Sends the email if the EMAIL in configuration.pl is on.
#
#	Parameters:
#					$_[0] - Directory name
#					$_[1] - Project name
#					$_[2] - User email
#					$_[3] - File with error
#	Returns:
#					NOTHING
################################################################
sub notifyError( $ $ $ $ )
{
	my $dir = $_[0];
	my $project = $_[1];
	my $email = $_[2];
	my $errorFile = $_[3];
	
	&Processing::debug_print("Notify Error with dir=($dir), project=($project), email=($email), errorFile=($errorFile)");

	chdir $Processing::RUN_DIR
		|| die("Cann't enter running directory ($Processing::RUN_DIR)");
		
	if ( $Processing::YES == $Configuration::EMAIL )
	{
		&Processing::debug_print("Looking for email module...");
		require "./sendMail.pl";
	
		&Processing::debug_print(" sendMail (error, $email, $project, $dir\/$errorFile)");
		&sendMail
		(
			"error", 
			"$email",
			"$project", 
			"$dir\/$errorFile"
		);
	}
	
	&Processing::debug_print("link (\"$dir\/$errorFile), \"$dir\/$variables{ OUTPUT_FILE }\")");
	unlink "$dir\/$variables{ OUTPUT_FILE }";
	link (  "$dir\/$errorFile", "$dir\/$variables{ OUTPUT_FILE }" ) || die( "ERROR NOTIFICATION:  $!");
}
################################################################



################################################################
#			notifyAdmin
#	Sends email to Admin
#	Parameters:
#					$_[0] - Directory name
#					$_[1] - Project name
#					$_[2] - File with error
#	Returns:
#					NOTHING
################################################################
sub notifyAdmin( $ $ $ )
{
	my $dir = $_[0];
	my $project = $_[1];
	my $errorFile = $_[2];
	
	&Processing::debug_print("Notify Admin with dir=($dir), project=($project), errorFile=($errorFile)");

	if ( $Processing::YES == $Configuration::EMAIL )
	{
		&Processing::debug_print("Looking for email module...");
		require "./sendMail.pl";
	
		&Processing::debug_print(" sendMail (error, $Configuration::ADMIN_EMAIL, $project, $errorFile)");
		&sendMail
		(
			"error", 
			"$Configuration::ADMIN_EMAIL",
			"$project", 
			"$errorFile"
		);
	}
}
################################################################



################################################################
#			notifyResult
#	Prints the message that result is ready. 
#	Sends the email if the EMAIL in configuration.pl is on.
#
#	Parameters:
#					$_[0] - Directory name
#					$_[1] - Project name
#					$_[2] - User email
#	Returns:
#					NOTHING
################################################################
sub notifyResult( $ $ $ )
{
	return if ($Configuration::ONLINE == $Processing::NO);

	my $dir = $_[0];
	my $project = $_[1];
	my $email = $_[2];
	
	&Processing::debug_print("Notify Result with dir = $dir, project = $project, email = $email");
	$dir =~ s/\.\.\///sig; 
	
	if ( $Processing::YES == $Configuration::EMAIL )
	{
		&Processing::debug_print("Looking for email module...");
		require "./sendMail.pl";
	
		&Processing::debug_print(" sendMail (result, $email, $project, ../$dir\/, $Configuration::HOME_WEB_DIR\/cgi-bin\/$dir\/$variables{ OUTPUT_FILE }, $Configuration::HOME_WEB_DIR\/cgi-bin\/..\/$dir\/index.html)");
		&sendMail
		(
			"result", 
			"$email",
			"$project",
			"../$dir\/",
			"$Configuration::HOME_WEB_DIR\/cgi-bin\/..\/$dir\/$variables{ OUTPUT_FILE }",
			"$Configuration::HOME_WEB_DIR\/cgi-bin\/..\/$dir\/index.html"
		);
	}
	print "<br>The result is ready</br>";
	print printWindowButtom();
}
################################################################



################################################################
################################################################
# 			PRIVATE FUNCTIONS:
################################################################
################################################################


################################################################
# 				printWindowHeader
#	Prints the header of the  window
#
#	Parameters:
#					NOTHING
#	Returns:
#					Window header
################################################################
sub printWindowHeader()
{
	my $result = "";
	$result = $result.&UseCgiLibPl::PrintHeader();
	$result = $result.&UseCgiLibPl::HtmlTop( $TITLE );
	return $result;
}
################################################################


################################################################
# 				printWindowButtom
#	Prints the buttom of the  window
#
#	Parameters:
#					NOTHING
#	Returns:
#					Window header
################################################################
sub printWindowButtom()
{
	return &UseCgiLibPl::HtmlBot;
}
################################################################


################################################################
#			printAcceptedMessage
#	Prints message to the user about what to do.
#
#	Parameters:
#					$_[0] - Project name
#					$_[1] - user's email
#					$_[2] - Project directory
#	Returns:
#					Body of message
################################################################
sub printAcceptedMessage( $ $ $ )
{
	my $proj = $_[0];
	my $mail = $_[1];
	my $dir  = $_[2];
	my $result = "";

	open IN, "Texts/accepted.txt"
		||	&Processing::debug_print("Failed to read from file (Texts/accepted.txt)");
	$result = join '', <IN>;
	close IN;
	
	$result = $result."<p>Your result will be available <a href=\"$dir\/$variables{ OUTPUT_FILE }\">here</p></a>\n";
	my @words = split ('\@', $mail);
	my $user = $words[0];
	$result = $result."<p>Your Username is:  $user </p>\n";
	$result = $result."<p>Your Password is:  $proj </p>\n";

	return $result;
}
################################################################



################################################################
#			printNoResultMessage
#	Prints message to the user that there is no result yet.
#
#	Parameters:
#					$_[0] - Project name
#	Returns:
#					Body of message
################################################################
sub printNoResultMessage( $ )
{
	my $proj = $_[0];
	my $result = "";
		
	$result = $result."<p>Results for project $proj are not available yet.<p>";
	$result = $result."<p>Please, refresh the window after some time<p>";
	
	return $result;
}
################################################################
1;