#!/usr/bin/perl -w
my $fullPedfile = $ARGV[0];
my $destDir = $ARGV[1];
my $ERROR_FILE = "$destDir\/.simplifiedConvertERR.txt";
my $OUTPUT_PED_FILE = "$destDir\/sl_input.ped";

system("cp $fullPedfile $destDir\/simplified_data.txt");
my $NEW_PEDIGREE_LINE_PREFIX = 'pedigree:';

@pedigrees = `cat $fullPedfile | grep $NEW_PEDIGREE_LINE_PREFIX`;
$count = 1;
$end = 0;
if ( $#pedigrees > -1 ) {
	$end = $#pedigrees;
}
open (OUT,"> $OUTPUT_PED_FILE ") || &errorLog("Could not create file $destDir\/sl_input.ped !");

for $ped (0 .. $end) {
	$transpath = "$destDir\/.translate_$ped.txt";
	$transpathMain = "$destDir\/.translateMain.txt";
	if ( $#pedigrees > -1 ) {
		my @pedLine = split /$NEW_PEDIGREE_LINE_PREFIX/, $pedigrees[ $ped ];
		$pedName = &trim($pedLine[1]);
		$pedfile = &getPedigree( $fullPedfile, $pedName ); 
		$pedName =~ s/\s+/_/g;
		
		open TMP_PED, ">$destDir\/.tmp_ped.txt";
		print TMP_PED "$pedfile";
		close TMP_PED;
		
		$uniqDataTotal =  ` awk '{print \$1}' $destDir\/.tmp_ped.txt | sort | uniq | wc -l`;
		$dataTotal =  `awk '{print \$1}' $destDir\/.tmp_ped.txt | wc -l`;
		if ($dataTotal > $uniqDataTotal) {
			&errorLog("Some of the names appear twice or more!");
		}
	}
	else {
		# dvir 2011-02-13,
		# allowing that when there is only one ped, than it may have empty lines
		# $pedfile = `cat $fullPedfile`; # the old solution
		$pedfile = `grep [a-zA-Z0-9] $fullPedfile`;
		
		$uniqDataTotal =  `grep [a-zA-Z0-9] $fullPedfile | awk '{print \$1}' | sort | uniq | wc -l`;
		$dataTotal =  `grep [a-zA-Z0-9] $fullPedfile | awk '{print \$1}' | wc -l`;
		$uniqDataTotal =~ s/\n//g;
		$dataTotal =~ s/\n//g;
		if ($dataTotal > $uniqDataTotal) {
			&errorLog("Some of the names appear twice or more! $uniqDataTotal $dataTotal");
		}
	}
	
	&checkPedLen($pedfile);
	
	$transfile = &translateNamesToIDs ( $pedfile , $ped + 1 );
	open (FH,"> $transpath") || &errorLog("Could not create file $transpath !");
	print FH $transfile;	
	close FH;
	
	open (FH_MAIN,">> $transpathMain") || &errorLog("Could not create file $transpathMain !");
	print FH_MAIN $transfile;	
	close FH_MAIN;
	
	@people = `awk ' { print \$3 } ' $transpath`; 
	for $p (0 .. $#people) {
		$person = &trim( $people[$p] );
		if ($person) {
			($fatherName, $fatherID, $motherName, $motherID, $sex, $aff) = &getPersonData($person, $ped + 1);
			($sameFather, $sameMother) = &getNextSiblingSameParents($person, $fatherName, $motherName, $ped + 1);
			print OUT $count."\t".&getIDForName($person, $ped + 1)."\t".$fatherID."\t".$motherID."\t".&getFirstChild($person, $sex, $ped + 1)."\t".
						$sameFather."\t".$sameMother."\t".$sex."\t0\t".$aff."\n";
		}
	}
	my $output = "$destDir\/sl_input.ped";
	my $connectivityInput = `grep ^$count $output`;
	@ids = `awk ' { print \$2 } ' $transpath`; 
	&checkPedConnectivity(@ids, $connectivityInput, $ped + 1);
	foreach $id (@ids) {
		&checkPredecessors(@ids, $id, $connectivityInput, $ped + 1);
	}	
	
	$count++;
}

close OUT;

system("rm -rf $destDir\/.translate_*.txt");
system("mv $transpathMain $destDir\/.translate_1.txt");

# DVIR 2010/12/26, when there isn't an error, than we will delete the file
#&errorLog(""); # Clear error log
if (-e $ERROR_FILE) {
	my $result = unlink $ERROR_FILE;
	if (!$result) {
		die $!;
	}
}
# DVIR end

exit(0);

################################################################
# 				getPedigree
#	Parameters:
#					$_[0]  - The full pedigree file
#					$_[1]  - The name of the pedigree to cut
#	Returns:
#				The content of the pedigree	
#
################################################################
sub getPedigree( $ $ ) {
	my $fullPedFile = $_[0];
	my $pedName = $_[1];
	
	open IN, $fullPedFile || &errorLog("Can't open file $fullPedFile!");
	my @data = <IN>;
	close IN;
	
	$result = "";
	my $copy = 0;
	$line = shift @data;
	while ( ( ($line !~ /$NEW_PEDIGREE_LINE_PREFIX/) || ($line !~ /$pedName/) ) && ($#data > -1) ) {
		$line = shift @data;
	}
	
	if ($#data <= -1) {
		return "";
	}
	
	$line = shift @data;  # The first line of the pedigree we were looking for
	
	while ( ($line !~ /$NEW_PEDIGREE_LINE_PREFIX/) && ($#data > -1) ) {  # Copy until next pedigree or EOF
		if ($line =~ /[a-zA-Z0-9]/) {
			$result = $result.$line;
		}
		$line = shift @data;
	}
	
	if ( ($#data == -1) && ($line) ) {
		$result = $result.$line;
	}
	
	return $result;
}

################################################################
# 				checkPedLen
#	Returns:
#				If some fields are missing an error 
#				will be reported.	
################################################################
sub checkPedLen( $ ) {
	my @data = split /\n/, $pedfile;
	
	for $i (0 .. $#data) {
		my @line = split /\s+/, $data[$i];
		if ($line[0]) {
			if ($#line < 4) {
				&errorLog("The definition for person ".$line[0]." is missing values!");
			}
		}
	}
}

################################################################
# 				translateNamesToIDs
#	Returns:
#				The data to be saved into a translation file:
#				"id = <id> name = <name>"
#				for each name that appears in $pedfile
#				
#				The result of this function is later used by 
#				all the other functions.
################################################################
sub translateNamesToIDs {
	my $ped = $_[1];
	my $result = "";
	
	my @data = split /\n/, $pedfile;
	
	my $count = 1;
	for $i (0 .. $#data) {
		my @line = split /\s+/ , $data[$i];
		if ($line[0]) {
			if ($result =~ / $line[0] /) {
				&errorLog("A person named ".$line[0]." was entered more then once in pedigree ".$ped."!");
			}
			$result = $result.$ped."\t".$count."\t".$line[0]."\n";
			$count++;
		}
	}
	
	return $result;
}

################################################################
# 				getPersonData
#	Parameters:
#					$_[0]  - The person 
#					$_[1]  - The pedigree number
#	Returns:
#				(<fatherName>, <fatherID>, <motherName>, <motherID>, 
#				 <sex>, <affection status>)	
#
#	Comments:	1. 	The is returned in this order
#				2. 	The ids are according to $transfile
#				3. 	Sex values are 1 (for male) or 2 (for female)
#				3. 	Affection status values are: 0 (for unknown),
#				 	1 (for healthy), 2 (for affected)
################################################################
sub getPersonData ( $ $ ) {
	my $person = $_[0];
	my $pedNum = $_[1];
	
	my @data = split /\n/, $pedfile;
	
	my @result = ();
	for $i (0 .. $#data) {
		my @line = split /\s+/, $data[$i];
		if ( ($line[0]) && ($line[0] eq $person) ) {
			#father
			push (@result, $line[1]);
			push (@result, &getIDForName($line[1],$pedNum));
			
			#mother
			push (@result, $line[2]);
			push (@result, &getIDForName($line[2],$pedNum));
			
			#sex
			if ( ($line[3] eq "male") || ($line[3] eq "m") || ($line[3] eq "1") ) {  
				push (@result, 1); 	
			}
			else {
				if ( ($line[3] eq "female") || ($line[3] eq "f") || ($line[3] eq "2") ) { 
					push (@result, 2); 	
				}
				else {
					&errorLog("The sex input for person ".$person." in pedigree ".$pedNum." is incorrect!");
				}
			}

			#affection status
			if ( ($line[4] eq "unknown") || ($line[4] eq "u") || ($line[4] eq "?") || ($line[4] eq "0") ) {  
				push (@result, 0); 	
			}
			else {
				if ( ($line[4] eq "healthy") || ($line[4] eq "h") || ($line[4] eq "1") ) {  
					push (@result, 1); 	
				}
				else {
					if ( ($line[4] eq "affected") || ($line[4] eq "a") || ($line[4] eq "2") ) {  
						push (@result, 2); 	
					}
					else {
						&errorLog("The affection status input for person ".$person." in pedigree ".$pedNum." is incorrect!");
					}
				}
			}
			
			last;
		}
	}
	
	for $i (0 .. $#data) {
		my @line = split /\s+/, $data[$i];
		
		#check the sex of father
		if ( ($line[0]) && ($line[0] eq $result[0]) ) {
			if ( ($line[3] ne "male") && ($line[3] ne "m") && ($line[3] ne "1") ) {
				&errorLog("The sex of the father for person ".$person." in pedigree ".$pedNum." is incorrect!");
			}
		}
		if ( ($line[0]) && ($line[0] eq $result[2]) ) {
			if ( ($line[3] ne "female") && ($line[3] ne "f") && ($line[3] ne "2") ) {
				&errorLog("The sex of the mother for person ".$person." in pedigree ".$pedNum." is incorrect!");
			}
		}
	}
		
	
	return @result;
}

################################################################
# 				getFirstChild
#	Parameters:
#					$_[0]  - The person 
#					$_[1]  - His sex (1 or 2)
#					$_[2]  - The pedigree number
#	Returns:
#				The ID of the first child (by input order)	
#
################################################################
sub getFirstChild ( $ $ $ ) {
	my $person = $_[0];
	my $sex = $_[1];
	my $pedNum = $_[2];
	
	my @data = split /\n/, $pedfile;
	
	for $i (0 .. $#data) {
		my @line = split /\s+/, $data[$i];
		if ($line[0]) {
			if ( ( ($line[1] eq $person) && ($sex == 1) ) || ( ($line[2] eq $person) && ($sex == 2) ) ) {
				return ( &getIDForName($line[0],$pedNum) );
			}
		}
	}
	
	return 0;
}

################################################################
# 				getNextSiblingSameParents
#	Parameters:
#					$_[0]  - The person 
#					$_[1]  - His father
#					$_[2]  - His mother
#					$_[3]  - The pedigree number
#	Returns:
#				1. 	The first person after $person 
#					that has the same $father
#				2. 	The first person after $person 
#					that has the same $mother	
#
#				(In this order)
################################################################
sub getNextSiblingSameParents ( $ $ $ $ ) {
	my $person = $_[0];
	my $father = $_[1];
	my $mother = $_[2];
	my $pedNum = $_[3];
	
	my @result = (0,0);
	
	if ( ($father eq "?") || ($father eq "0") ) {
		if ( ($mother eq "?") || ($mother eq "0") ) {
			push (@result, 0);
			push (@result, 0);
			return @result;
		}
		else {
			&errorLog("Only one parent specified for a person named ".$person." in pedigree ".$pedNum."!"); 
		}
	}
	
	if ( ($mother eq "?") || ($mother eq "0") ) {
		if ( ($father eq "?") || ($father eq "0") ) {
			push (@result, 0);
			push (@result, 0);
			return @result;
		}
		else {
			&errorLog("Only one parent specified for a person named ".$person." in pedigree ".$pedNum."!"); 
		}
	}
	
	my @data = split /\n/ , $pedfile;
	
	my $seenPerson = 0;
	
	for $i (0 .. $#data) {
		my @line = split /\s+/, $data[$i];
		if ($line[0]) {
			if ( $line[0] eq $person) {
				$seenPerson = 1;
			}
			if ( ($line[1] eq $father) && ($line[0] ne $person) && ($seenPerson == 1) && ($result[0] == 0) ) {
				$result[0] = &getIDForName($line[0],$pedNum);
			}
			if ( ($line[2] eq $mother) && ($line[0] ne $person) && ($seenPerson == 1) && ($result[1] == 0) ) {
				$result[1] = &getIDForName($line[0],$pedNum);
			}
			if (($result[0] != 0) && ($result[1] != 0)) { 		#Quit if you already have all the results
				return @result;
			}
		}
	}
	
	return @result;
}

################################################################
# 				getIDForName
#	Parameters:
#					$_[0]  - The name of person 
#					$_[1]  - The pedigree number
#	Returns:
#				The ID of the person according to the $transfile
#				Returns 0 for people who don't appear in $transfile
#				and for people with names that mean "unknown"
#
################################################################
sub getIDForName ( $ $ ) {
	my $name = $_[0];
	my $pedNum = $_[1];
	
	if ( ($name eq "?") || ($name eq "0") ) {
		return 0;
	}
	my @transData = split /\n/ , $transfile;
	foreach $line (@transData) {
		if ($line =~ /$pedNum\t.*\t$name$/ ) {
			my @personIDData = split /\t/ ,$line;
			return (trim($personIDData[1]));
		}
	}
	
	&errorLog("A person with the name ".$name." was not defined in pedigree ".$pedNum."!");
}

################################################################
# 				getIDForName
#	Parameters:
#					$_[0]  - The id of person 
#					$_[1]  - The pedigree number
#	Returns:
#				The ID of the person according to the $transfile
#				Returns 0 for people who don't appear in $transfile
#				and for people with names that mean "unknown"
#
################################################################
sub getNameForID ( $ $ ) {
	my $id = $_[0];
	my $pedNum = $_[1];
	
	if ( ($id eq "?") || ($id eq "0") ) {
		return 0;
	}
	my @transData = split /\n/ , $transfile;
	foreach $line (@transData) {
		if ($line =~ /$pedNum\t$id\t/ ) {
			my @personIDData = split /\t/ , $line;
			return (trim($personIDData[2]));
		}
	}
	
	&errorLog("A person with the id ".$id." was not defined in pedigree ".$pedNum."!");
}

################################################################
# 				checkPedConnectivity
#	Parameters:
#					$_[0]  - array of all the ids in the ped
#					$_[1]  - sl_input ped file
#					$_[2]  - The pedigree number
#	Returns:
#				If the pedigree isn't connected an error 
#				will be reported.
################################################################
sub checkPedConnectivity {
	my $pedNum = pop @_;
	my $SLPedFile = pop @_;
	my @peopleIDs = @_;
	
	my @data = split /\n/, $SLPedFile;
	
	my @connected = ();
	for $i (0 .. $#peopleIDs) {
		push(@connected, 0);
	}
	
	$connected[0] = 1;  # visit the first person
	my @idsToVisit = (1);
	
	while ($#idsToVisit > -1) {
		my $id = pop @idsToVisit;
		my @line = split /\s+/, $data[$id - 1];
		
		#visit the person
		$connected[ $line[1] - 1 ] = 1;
		
		#should visit father (if not already visited)
		$father = $line[2];
		if ( ( $father != 0 ) && ( $connected[ $father - 1 ] == 0 ) ){
			push (@idsToVisit, $father);
		}
		
		#should visit mother (if not already visited)
		$mother = $line[3];
		if ( ( $mother != 0 ) && ( $connected[ $mother - 1 ] == 0 ) ){
			push (@idsToVisit, $mother);
		}
		
		#should visit first son (if not already visited)
		$firstSon = $line[4];
		if ( ( $firstSon != 0 ) && ( $connected[ $firstSon - 1 ] == 0 ) ){
			push (@idsToVisit, $firstSon);
		}
		
		#should visit next sibling by father (if not already visited)
		$siblingFather = $line[5];
		if ( ( $siblingFather != 0 )  && ( $connected[ $siblingFather - 1 ] == 0 ) ){
			push (@idsToVisit, $siblingFather);
		}		

		#should visit next sibling by mother (if not already visited)
		$siblingMother = $line[6];
		if ( ( $siblingMother != 0 )  && ( $connected[ $siblingMother - 1 ] == 0 ) ){
			push (@idsToVisit, $siblingMother);
		}
		
	}
	
	for $p (0..$#connected) {
		if ($connected[ $p ] == 0) {
			# dvir , 02-09-2010, I add 1 from the person number, since that in the file, the persons numbers start from 1. and if there isn't an addition, than the user 
			# gets an alert on the wrong person
			&errorLog("There is a problem with the pedigree connectivity! Please check the input for the ".(&getNameForID($p + 1,$pedNum))." person in pedigree ".$pedNum);
		}
	}
}

################################################################
# 				checkPredecessors
#	Parameters:
#					$_[0]  - array of all the ids in the ped
#					$_[1]  - person ID
#					$_[2]  - sl_input ped file
#					$_[3]  - The pedigree number
#	Returns:
#				If the person is his own predecessor an error
#				will be reported.
################################################################
sub checkPredecessors {
	my $pedNum = pop @_;
	my $SLPedFile = pop @_;
	my $personID = pop @_;
	my @peopleIDs = @_;
	
	$personID = &trim($personID);
	
	my @data = split /\n/, $SLPedFile;
	
	my @pred = ();
	for $i (0 .. $#peopleIDs) {
		push(@pred, 0);
	}
	
	# visit the person parents
	my @parentsToVisit = ( $personID );
	
	while ($#parentsToVisit > -1) {
		my $id = pop @parentsToVisit;
		my @line = split /\s+/, $data[$id - 1];
			
		#visit father (if not already visited)
		$father = $line[2];
		if ( ( $father != 0 ) && ( $pred[ $father - 1 ] == 0 ) ){
			$pred[ $father - 1 ] = 1;
			push (@parentsToVisit, $father);
		}
		
		#visit mother (if not already visited)
		$mother = $line[3];
		if ( ( $mother != 0 ) && ( $pred[ $mother - 1 ] == 0 ) ){
			$pred[ $mother - 1 ] = 1;
			push (@parentsToVisit, $mother);
		}
	}
	
	if ($pred[ $personID - 1 ] == 1) {
		&errorLog("The person ".(&getNameForID($personID, $pedNum))." in pedigree ".$pedNum." is a predecessor of himself!" );
	}
}
	


sub trim( $ )
{
	my $string = $_[0];
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

################################################################
#		fileLog
#
#	Parameters:
#		 $_[0] - the error to be reported
#	Returns:
#		 reports an error found in the file
################################################################
sub errorLog( $ )
{
	my ($errorReport) = $_[0];
	open( FILEERRORLOG , ">$ERROR_FILE") or die "Cant report inner error!\n";
	print FILEERRORLOG "Pedigree error: $errorReport";
	close FILEERRORLOG;
	# delete the ped with the error
	# DVIR 2010/12/26,
	if (-e $OUTPUT_PED_FILE) {
		my $result = unlink $OUTPUT_PED_FILE;
		if (!$result) {
			die $!;
		}	
	}	
	exit(0);
} 
