#!/usr/bin/perl -w

use FindBin ();
use lib "$FindBin::Bin";
use submissionConfig;
use SharedConsts;

use constant EX_LOCK => 2;
use constant NB_LOCK => 4;

$config = new submissionConfig;

$lock = "$config->{'QUEUE_LOCK'}";
#$queue = "$config->{'QUEUE'}";
$queue = $ARGV[0];

@fullpath = split(/\//,$queue);
$queueFile = $fullpath[$#fullpath];
@fileName = split /\./, $fullpath[$#fullpath];
my $submissionsLogDir = "$config->{'FULL_DIR'}/Submissions/log/";
$logFile = $submissionsLogDir.$fileName[0].".log";
# dvir 10/08/2010 $currentJobStandardErrorLogFileName will contain the standard error output of the current job executed
# each queue has its own unique CurrentJobStandardErrorLogFile file
# and only when the script owns the lock to $queueFile, it will try to write to CurrentJobStandardErrorLogFile of the relevat queue
# so there will be no collisions.
my $currentJobStandardErrorLogFileName = $submissionsLogDir.$fileName[0]."CurrentJobStandardError.log"; 
#$logFile = "/tmp/queue/".$fileName[0].".log";

chdir("$config->{'FULL_DIR'}/cgi-bin");
#chdir("/tmp/queue");

open(LOCK, ">".$lock."_".$queueFile) || die();
$gotLock = flock (LOCK, EX_LOCK | NB_LOCK);		# wait until you lock is free (non-blocking)
if (!$gotLock){									# already locked
	close(LOCK);
	exit(0);
}

print LOCK "Locked by queueHandler.\n";

open(QUEUE, $queue) || die("failed opening file: $queue");

my $ret = 0;
my $job;
# dvir 2011/10/23, I have added $errorInSubmissionFullDir
my $errorInSubmissionDir;
while(my $line = <QUEUE>) {
	# removing \n from the end of the job
	chomp($line);
	my @lineArr = split ' ', $line;
	$errorInSubmissionDir = $lineArr[0];
	$job = join ' ', @lineArr[1..$#lineArr];
	
	&queueLog($job, "Sent for execution", $logFile);
	# overwriting the standard error of the previous job
	system("$job 2> $currentJobStandardErrorLogFileName");
	$ret = $? >> 8;
	$msg = "Was executed and got: $ret.\n";
	if (0 == $ret) {
		&queueLog($job, $msg, $logFile);
	}
	elsif ($ret == $SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS) {
		&queueLog($job, $msg."Executor at high load ($ret), retry all jobs in this executor's queue later", $logFile);
		last; # stop submitting since that is is likely that other attempts to submit will return in a high load
	}
	if (0 != $ret){ # Error with the job execution - not an executor failure
		&queueLog($job, $msg."Job execution failed ($ret), won't try this job again\nStandard Error:\n `cat $currentJobStandardErrorLogFileName`", $logFile);
		last;
	}
}

@jobsLeft = <QUEUE>;
close(QUEUE);

print ">".$queue."2\n	";
open(QUEUE2, ">".$queue);
if ($ret == $SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS) {
	# the job whose submision failed becasue of a high load, we will try to submit it again
	print QUEUE2 $errorInSubmissionDir.' '.$job."\n";
}
# dvir 2011/10, reporting to the user that there is a submission failure,
# and that he should contact a system admin.
elsif($ret != 0) {
	if ($errorInSubmissionDir ne 'None') {
		# when the job is on chrom, only now we discover the dir full name.
		my $command = "ls -d $config->{'FULL_DIR'}\/$errorInSubmissionDir\*";
		my $errorInSubmissionFullDir = `$command`;
		chomp($errorInSubmissionFullDir);
		# creating an indication that there was an error in the submission.
		system("touch $errorInSubmissionFullDir\/submission.error.indication");
	}
}

print QUEUE2 @jobsLeft;
close(QUEUE2);

print LOCK "queueHandler released lock.\n";	#just to indicate lock release
close(LOCK);								# release lock

exit(0);

sub queueLog ( $ $ $ ) {
	my $job = $_[0];
	my $msg = $_[1];
	my $logName = $_[2];
	
	system("echo \"*********************************************\n LOG: `date`\n job = $job\n $msg \n\" >> $logName");
}
