#!/usr/bin/perl -w
################################################################
# 		This file creates Two Point output
#	Allowed next result files in the directories:
#		- 	No File
#		- 	$Processing::ERROR_RESULT_FILE
#		- 	$LOCAL_RESULT_FILE
#		- 	$CONDOR_RESULT_FILE
#		- 	$SKIPPED_RESULT_FILE
################################################################

use FindBin ();
use lib "$FindBin::Bin";
use Processing;
use Math::Complex;
use List::Util;
use Scalar::Util;

####
# dvir 2011/08, I added the use of Plotter.pm from superlinksnp
use WebServerConfiguration;
# in order to use packages from cbl-link01 , superlinksnp , perlbin dir
use lib $WebServerConfiguration::PERL_BIN_DIR_AT_BACKEND;

# packages in use from cbl-link01 , superlinksnp , perlbin dir
use Plotter;
use Distances;
####

my $CMPLX_HEADER_FILE = "Texts\/twoPointSnpTableHeader.txt";
my $FILE_LIST_NAME = "fileList.txt";

################################################################
# 			FUNCTIONS:
################################################################
sub handleRemoteComputations( \% );
sub handleAllComputations( \% );
sub createInfoListForFamily( \% \% \% $ \% \% );
sub createLodsHash( \% \% );
sub createLocalLikelihoodsHash( $ $ $ \% );
sub getComplexitiesAndMarkerNamesHashes( $ $ \% \% \% );
sub createLodTableFile( \% \% \% \% \% \% );
sub createMarkerPositionsHash( $ );
sub createGraphs( \% \% \% );
sub addRemoteLikelihoodsToLikelihoodsHash( \% \% );
sub getCondorComputationResult( $ \% );
sub countNumberOfPreviousAndCurrentResults( $ % % );
################################################################
# 		START
################################################################
&Processing::setClusteringVariables();
if ( $#ARGV < 0 ) {$projectDir = ".";}
else {$projectDir = $ARGV[0];}
my @path = split "postTwoPoint.pl", $0;
chop $path[0];
use Cwd;
my $currDir =  &getcwd();
chdir $path[0];
&Processing::setRunDir( &getcwd() );
&Processing::setLogFiles( "$Processing::RUN_DIR\/$projectDir\/" );
chdir $currDir;

$RESULT_HEADER_FILE = "$Processing::RUN_DIR\/Texts/multiPointResultHead.txt";

&Processing::debug_print ( "================================================================" );
&Processing::debug_print ( "      POST MULTI POINT has been started in dir ($Processing::RUN_DIR)" );
&Processing::debug_print ( "================================================================" );

my %variables = %{ &Processing::initVariablesFromPrePoint( $projectDir ) };

my $LOD_HEADER_FILE = "";
if ($variables{program} =~ /MultiPoint/) {$LOD_HEADER_FILE = "Texts\/multiPointSnpTableHeader.txt";}
else {$LOD_HEADER_FILE = "Texts\/twoPointSnpTableHeader.txt";}
&Processing::setHeaderFiles( $LOD_HEADER_FILE, $CMPLX_HEADER_FILE );

$variables{ webDir } = $variables{ newDir };
$variables{ newDir } = $projectDir;

%variables = %{&Processing::makeSystemError("Failed changing to project directory ($variables{ newDir})", \%variables )}
	unless ( chdir $variables{ newDir } );
exit $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );

$projName = $variables{ PROJ };
@chromNameData = split /chrom_/, $projName;
@positions = &Processing::getBEDPositions( \%variables );
if ($#positions > -1) {
	$chrom = $chromNameData[1];
	if ($chromNameData[1] !~ /X/) {$chrom = $chromNameData[1] + 0;}
	&Processing::createBEDfile($chrom, $positions[0], $positions[1]);
}

#This is the new code to create the output tables and graphs
&handleAllComputations(\%variables);

#This is the old code to create the output tables and graphs
#&handleRemoteComputations(\%variables);

exit(0);
################################################################



#This function creates output for remote computations
sub handleRemoteComputations( \% ) {
	my %variables = %{ $_[0]};

	my @totalLOD = ();
	for( my $famCounter =  0; $famCounter <= $#{ $variables{ pednames } }; $famCounter ++ )
	{
		my $family = $variables{ pednames }[$famCounter];
		&Processing::debug_print("Working on family ($family)");
		&Processing::debug_print("Counter is ($famCounter), maximum is ($#{ $variables{ pednames } })");

		my @familyLOD = @{ &Processing::createFamilyResult( $famCounter, "$family.result", "$family.complexity", \%variables ) };
		@totalLOD = @{ &Processing::updateTotalLOD( \@familyLOD, \@totalLOD, \%variables ) };
	}

	&Processing::debug_print("Creating total result");
	&Processing::createTotalResult( \@totalLOD, "total.result", "total.complexity", \%variables );
	&Processing::makeGraphs();
	&Processing::printOutputHeader( $RESULT_HEADER_FILE, \%variables );
	&Processing::printTablesToOutput( \%variables );
	&Processing::makeOuputFrameset(  );
}


#Read the result of a computation made by Condor
sub getCondorComputationResult( $ \% ) {
	my $dataDir = $_[0];
	my %variables = %{ $_[1] };
	
	my %pointInfo = %{ &Processing::getPointInfoFromDirectory($dataDir, \%variables) };	
	return ${$pointInfo{LN}}[0];
}

#Add the results of all the remote computations that were performed to the likelihoods hash
sub addRemoteLikelihoodsToLikelihoodsHash( \% \% ) {
	my %variables = %{ $_[0] };
	my %likelihoodsHash = %{ $_[1] };

	#Open the file which states in which directory each remote computation was performed
	unless (-e "computationDirectories.txt") {
		%variables = %{&Processing::makeSystemError("Failed finding file computationDirectories.txt", \%variables )};
		exit $Processing::ERROR;
	}
	
	open COMPUTATION_DIRECTORIES_FH, "computationDirectories.txt";
	while (my $currLine = <COMPUTATION_DIRECTORIES_FH>) {
		chomp($currLine);		
		my @currLineArr = split ',', $currLine;
		my @currLineSubArr = split '\$\$\$', $currLineArr[0];
		my $markerName = $currLineSubArr[1];	#Name of the first marker in the window
		my $pedNum = $currLineSubArr[2];	#Pedigree number	
		my $firstMarkerNum = $currLineArr[1];
		my $windowSize = $currLineArr[2];
		my $distanceFromFirstMarker = $currLineArr[3];
		my $computationDir = $currLineArr[4];
		
		#Get the likelihood from the directory
		my $likelihood = &getCondorComputationResult($computationDir, \%variables);
		
		#Update the hash
		$likelihoodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker} = $likelihood;
		
		#Update the total likelihood
		if (defined $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker}) {
            my $totalLikelihood = $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
            if (!Scalar::Util::looks_like_number($totalLikelihood)) {$totalLikelihood = $likelihood;}
            elsif (Scalar::Util::looks_like_number($likelihood)) {$totalLikelihood += $likelihood;}                    
            $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $totalLikelihood;
        }			
        else {$likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $likelihood;}	
	}
	close COMPUTATION_DIRECTORIES_FH;	
	
	return %likelihoodsHash;
}


#Create a hash of marker positions, using a file created in the Pre script
sub createMarkerPositionsHash( $ ) {
	my $markerPositionsInCmFileName = $_[0];
	
	unless (-e $markerPositionsInCmFileName) {
		%variables = %{&Processing::makeSystemError("Failed finding file $markerPositionsInCmFileName", \%variables )};
		exit $Processing::ERROR;
	}	
	
	open MARKER_POSITIONS_FH, $markerPositionsInCmFileName;
	my %markerPositionsHash = ();
	#Each line looks something like: $$$57$$$1,1,1,-100000000 ,%-43.3369
	#Or: $$$57$$$1,1,1,-100000000 ,@2	(if local computations haven't finished yet)
	while (my $currLine = <MARKER_POSITIONS_FH>) {
		chomp($currLine);
		my @currLineArr = split ',', $currLine;
		my @currLineSubArr = split '\$\$\$', $currLineArr[0];
		my $markerName = $currLineSubArr[1];	#Name of the first marker in the window
		my $pedNum = $currLineSubArr[2];	#Pedigree number
		my $firstMarkerNum = $currLineArr[1];	#Number of first marker in the window, as it appears in the datfile
		my $windowSize = $currLineArr[2];		#Window size of the computation
		my $distanceFromFirstMarker = $currLineArr[3];	#Distance of the putative gene from the first marker in theta		
		my $markerPosition = $currLineArr[4];		
		
		#Create an entry in the marker positions hash
		$markerPositionsHash{$firstMarkerNum}{$distanceFromFirstMarker} = $markerPosition;
	}
	close MARKER_POSITIONS_FH;	
	return %markerPositionsHash;	
}



#Create a hash of likelihoods for all the local computations that were performed
sub createLocalLikelihoodsHash( $ $ $ \% ) {
	my $estimateLikelihoodComputationComplexityFileName = $_[0];
	my $complexityEstimationsOfLikelihoodComputationsFileName = $_[1];
	my $listOfLikelihoodResultsFileName = $_[2];
	my %variables = %{$_[3]};
	
	#Check if a sliding window was used in the analysis
	my $useSlidingWindow = 1;
	my $firstMarkerInWindow = 0;
	unless (defined($variables{jump}) && ( $variables{jump} !~ /Unknown/ ) ) {
		$useSlidingWindow = 0;
		$firstMarkerInWindow = ${$variables{usedMarkersId}}[0];
	}
	
	#Check if the computation is Multipoint or not
	my $isMultiPointComputation = 0;
	if ($variables{program} =~ /MultiPoint/) {$isMultiPointComputation = 1;}	
	
	#Initially, assume that the local computation aren't finished yet.
	#And that the complexities estimation aren't finished yet either.
	#This means that no computation was made (neither local nor remote).
	#So we just mark all computations as unfinished.
	my $fileToOpen = $estimateLikelihoodComputationComplexityFileName;
	my $localComputationsFinished = 0;
	
	#If the complexities estimaion file doesn't exist, exit with error
	unless (-e $estimateLikelihoodComputationComplexityFileName) {
		%variables = %{&Processing::makeSystemError("Failed finding file $estimateLikelihoodComputationComplexityFileName", \%variables )};
		exit $Processing::ERROR;
	}		
	
	#If the local computations finished, then read them instead
	if (-e $listOfLikelihoodResultsFileName) {
		$fileToOpen = $listOfLikelihoodResultsFileName;
		$localComputationsFinished = 1;
	}
	#If local computations aren't finished yet, but the complexities computations are finished
	#then read them (because maybe some remote computations already finished)
	elsif (-e $complexityEstimationsOfLikelihoodComputationsFileName) {
		$fileToOpen = $complexityEstimationsOfLikelihoodComputationsFileName;
	}
	
	open COMPLEXITY_RESULTS_FH, $fileToOpen;
	my %likelihoodsHash = ();
	#Each line looks something like: $$$57$$$1,1,1,-100000000 ,%-43.3369
	#Or: $$$57$$$1,1,1,-100000000 ,@2	(if local computations haven't finished yet)
	while (my $currLine = <COMPLEXITY_RESULTS_FH>) {
        chomp($currLine);
		my @currLineArr = split ',', $currLine;
		my @currLineSubArr = split '\$\$\$', $currLineArr[0];
		my $markerName = $currLineSubArr[1];	#Name of the first marker in the window
		my $pedNum = $currLineSubArr[2];	#Pedigree number
		my $firstMarkerNum = $currLineArr[1];	#Number of first marker in the window, as it appears in the datfile
		if ($isMultiPointComputation && !$useSlidingWindow) {$firstMarkerNum = $firstMarkerInWindow;}	#This is the real index of the first marker in the window
		my $windowSize = $currLineArr[2];		#Window size of the computation
		my $distanceFromFirstMarker = $currLineArr[3];	#Distance of the putative gene from the first marker in theta		
		$distanceFromFirstMarker =~ s/\s+$//;			#Remove trailing white spaces
		
		#If the local computations finished, read the likelihood from the file
		my $likelihood = $Processing::UNCALCULATED_INFORMATION;
		if ($localComputationsFinished) {
			$likelihood = $currLineArr[4];				#Likelihood computation (in units of ln)
			$likelihood =~ s{%}{};	#Remove the %sign
		}		
		
		#Create an entry in the likelihoods hash
		$likelihoodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker} = $likelihood;
		
		#Create the combined likelihood hash (sum of likelihoods for all families)
		if (defined $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker}) {
            my $totalLikelihood = $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
            if (!Scalar::Util::looks_like_number($totalLikelihood)) {$totalLikelihood = $likelihood;}
            elsif (Scalar::Util::looks_like_number($likelihood)) {$totalLikelihood += $likelihood;}                    
            $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $totalLikelihood;
        }
        else {$likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $likelihood;}
	}
	close COMPLEXITY_RESULTS_FH;
	
	return %likelihoodsHash;
}



#Create a hash of complexities for all the computations that need to be performed,
#using the output of Superlink's complexity estimation (created in the Pre script).
sub getComplexitiesAndMarkerNamesHashes( $ $ \% \% \% ) {
	my $estimateLikelihoodComputationComplexityFileName = $_[0];
	my $complexityEstimationsOfLikelihoodComputationsFileName = $_[1];
	my $complexitiesHashRef = $_[2];
	my $markerNamesHashRef = $_[3];
	my %variables = %{$_[4]};
	
	#Initially, assume that the local computation aren't finished yet.
	#And that the complexities estimation aren't finished yet either.
	#This means that no computation was made (neither local nor remote).
	#So we mark all complexities as unknown.
	my $fileNameToOpen = $estimateLikelihoodComputationComplexityFileName;
	my $complexityComputationsFinished = 0;
	
	#If the complexities computations are finished then read them
	if (-e $complexityEstimationsOfLikelihoodComputationsFileName) {
		$fileNameToOpen = $complexityEstimationsOfLikelihoodComputationsFileName;
		$complexityComputationsFinished = 1;
	}	
	
	#Check if a sliding window was used in the analysis
	my $useSlidingWindow = 1;
	my $firstMarkerInWindow = 0;
	unless (defined($variables{jump}) && ( $variables{jump} !~ /Unknown/ ) ) {
		$useSlidingWindow = 0;
		$firstMarkerInWindow = ${$variables{usedMarkersId}}[0];
	}
	
	#Check if the computation is Multipoint or not
	my $isMultiPointComputation = 0;
	if ($variables{program} =~ /MultiPoint/) {$isMultiPointComputation = 1;}	
	
	#Each line looks something like: $$$57$$$1,1,1,-100000000 ,@2
	open COMPLEXITY_FH, $fileNameToOpen;
	while (my $currLine = <COMPLEXITY_FH>) {
		chomp($currLine);
		my @currLineArr = split ',', $currLine;
		my @currLineSubArr = split '\$\$\$', $currLineArr[0];	
		my $markerName = $currLineSubArr[1];	#Name of the first marker in the window
		my $pedNum = $currLineSubArr[2];	#Pedigree number
		my $firstMarkerNum = $currLineArr[1];	#Number of first marker in the window, as it appears in the datfile
		if ($isMultiPointComputation && !$useSlidingWindow) {$firstMarkerNum = $firstMarkerInWindow;}	#This is the real index of the first marker in the window
		my $windowSize = $currLineArr[2];		#Window size of the computation
		my $distanceFromFirstMarker = $currLineArr[3];	#Distance of the putative gene from the first marker in theta		
		$distanceFromFirstMarker =~ s/\s+$//;			#Remove trailing white spaces	
		
		my $complexity = Processing::UNCALCULATED_INFORMATION;
		if ($complexityComputationsFinished) {	
			$complexity = $currLineArr[4];		#complexity computation (in units of ln)
			$complexity =~ s{@}{};				#Remove the @sign
		}
		
		$$complexitiesHashRef{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker} = $complexity;
		
		#Update marker name hash
		$$markerNamesHashRef{$firstMarkerNum}{$distanceFromFirstMarker} = $markerName;
		
		#Create the combined complexity hash (sum of complexities for all families)
		if (!defined $$complexitiesHashRef{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker}) {
			$$complexitiesHashRef{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $complexity;
		}		
		$prevComplexity = $$complexitiesHashRef{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
		if (!Scalar::Util::looks_like_number($prevComplexity)) {
			$$complexitiesHashRef{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $complexity;
		}
		elsif (Scalar::Util::looks_like_number($complexity)) {
			$$complexitiesHashRef{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = 
				List::Util::max($complexity, $prevComplexity);
		}
	}
	close COMPLEXITY_FH;
}


#Create a hash of LOD scores for all the local computations that were performed, in units of ln (and not log10)
#The function receives as a parameter a hash of likelihoods, as created by the function createLocalLikelihoodsHash
sub createLodsHash( \% \% ) {
	
	my %variables = %{$_[0] };
	my %likelihoodsHash = %{$_[1] };
	my $infinityDistance = &Distances::recToCm(-0.5);	#This is how infinity is represented in CM	
	my %lodsHash = ();
	
	for( my $famCounter =  0; $famCounter <= $#{ $variables{ pednames } }; $famCounter ++ ) {
		my $pedNum = $famCounter + 1;
		foreach my $firstMarkerNum ( keys %{$likelihoodsHash{$pedNum}} ) {			
			foreach my $distanceFromFirstMarker ( keys %{$likelihoodsHash{$pedNum}{$firstMarkerNum}} ) {
			
				#Skip over the likelihood for distance -infinity
				if ($distanceFromFirstMarker == $infinityDistance) {next;}
				
				#Check that the likelihood at infinity distance is defined
				if (!defined $likelihoodsHash{$pedNum}{$firstMarkerNum}{$infinityDistance}) {
					&Processing::debug_print ( "Post.pl, missing likelihood for ped $pedNum, marker $firstMarkerNum at distance $infinityDistance");
					%variables = %{&Processing::makeInputError("Post.pl, missing likelihood for ped $pedNum, marker $firstMarkerNum at distance $infinityDistance", \%variables )};
					exit $Processing::ERROR;
				}
				
				#Compute the lod score and put it in the hash
				my $positionLikelihood = $likelihoodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker};
				my $infinityLikelihood = $likelihoodsHash{$pedNum}{$firstMarkerNum}{$infinityDistance};
				my $lodScore;
				if (!Scalar::Util::looks_like_number($positionLikelihood)) {$lodScore = $positionLikelihood;}
				elsif (!Scalar::Util::looks_like_number($infinityLikelihood)) {$lodScore = $infinityLikelihood;}
				else {$lodScore = ($positionLikelihood - $infinityLikelihood) / ln(10);}				
				$lodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker} = $lodScore;
				
				#Update the total LOD score (sum of LOD scores for all the families)
                if (defined $lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker}) {
                    my $totalLOD = $lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
                    if (!Scalar::Util::looks_like_number($totalLOD)) {$totalLOD = $lodScore;}
                    elsif (Scalar::Util::looks_like_number($lodScore)) {$totalLOD += $lodScore;}                    
                    $lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $totalLOD;
                }			
                else {$lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker} = $lodScore;}
			}
		}
	}
	
	return %lodsHash;
}



#Create a list of information about each computation for a specific family, in the format used by the Processing module.
sub createInfoListForFamily( \% \% \% $ \% \% ) {
	my %variables = %{$_[0] };
	my %likelihoodsHash = %{$_[1] };	
	my %lodsHash = %{$_[2] };
	my $pedigree = $_[3];	
	my %markerNamesHash = %{$_[4] };
	my %complexitiesHash = %{$_[5] };
	
	my @familyPointsInfo = ();
	my $counter = 0;	
	foreach my $firstMarkerNum (sort {$a <=> $b} (keys %{$lodsHash{$pedigree}}) ) {		
		$counter++;
	
		foreach my $distanceFromFirstMarker ( sort {$a <=> $b} (keys %{$lodsHash{$pedigree}{$firstMarkerNum}}) ) {
            my $finished = 1;
			my $markerName = $markerNamesHash{$firstMarkerNum}{$distanceFromFirstMarker};			
			my $lodScore = $lodsHash{$pedigree}{$firstMarkerNum}{$distanceFromFirstMarker};
			my $lnScore = $likelihoodsHash{$pedigree}{$firstMarkerNum}{$distanceFromFirstMarker};			
			if (Scalar::Util::looks_like_number($lnScore)) {$lnScore = sprintf("%.4f",$lnScore);}
			if (Scalar::Util::looks_like_number($lodScore)) {$lodScore = sprintf("%.4f",$lodScore);}
            else {$finished = 0;}            

			#Create complexity string for computation
			my $complexity = $complexitiesHash{$pedigree}{$firstMarkerNum}{$distanceFromFirstMarker};
			if (defined $WebServerConfiguration::COMPLEXITY_TO_STRING_MAP{$complexity}) {
				$complexityString = $WebServerConfiguration::COMPLEXITY_TO_STRING_MAP{$complexity};
			}
			else {$complexityString = $Processing::UNCALCULATED_INFORMATION;}
			
			my %pointHash = ();
			$pointHash{CHROM} = $counter;
			@{$pointHash{CMPLX}} = ($complexityString);
			$pointHash{DIST} = $distanceFromFirstMarker;
			$pointHash{DIST1} = $Processing::UNCALCULATED_INFORMATION;
			$pointHash{DIST2} = $Processing::UNCALCULATED_INFORMATION;
			@{$pointHash{FINISHED}} = ($finished);
			@{$pointHash{LN}} = ($lnScore);
			@{$pointHash{LOD}} = ($lodScore);
			@{$pointHash{MULT}} = ($complexity);
			$pointHash{NAME} = $markerName;
			$pointHash{NAME1} = $Processing::UNCALCULATED_INFORMATION;
			$pointHash{NAME2} = $Processing::UNCALCULATED_INFORMATION;
			$pointHash{PROGRAM} = $variables{program};
			
			push (@familyPointsInfo, \%pointHash);			
		}
	}
	
	return @familyPointsInfo;
}


#Create a table of Lod scores and complexities
sub createLodTableFile( \% \% \% \% \% \% ) {
	my %variables = %{$_[0] };
	my %likelihoodsHash = %{$_[1] };
	my %lodsHash = %{$_[2] };
	my %markerNamesHash = %{$_[3] };
	my %markerPositionsHash = %{$_[4] };
	my %complexitiesHash = %{$_[5] };

	my $fileName = "for_dvir.txt";	
	
	open LOD_TABLE_FILE_FH, ">$fileName";
	foreach my $firstMarkerNum (sort {$a <=> $b} (keys %{$lodsHash{"TOTAL"}}) ) {		
		foreach my $distanceFromFirstMarker ( sort {$a <=> $b} (keys %{$lodsHash{"TOTAL"}{$firstMarkerNum}}) ) {
			my $markerPos = $markerPositionsHash{$firstMarkerNum}{$distanceFromFirstMarker};
			my $markerName = $markerNamesHash{$firstMarkerNum}{$distanceFromFirstMarker};
			my $lodScore = $lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
			my $lnScore = $likelihoodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
			my $complexity = $complexitiesHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};			
			
			if (defined $WebServerConfiguration::COMPLEXITY_TO_STRING_MAP{$complexity}) {
				$complexityString = $WebServerConfiguration::COMPLEXITY_TO_STRING_MAP{$complexity};
			}
			else {$complexityString = $Processing::UNCALCULATED_INFORMATION;}			
			
			print LOD_TABLE_FILE_FH "$firstMarkerNum $markerName $markerPos $lnScore $lodScore ${complexityString}\n";
		}
	}
	close LOD_TABLE_FILE_FH;
}

#Create all the neccessary graphs for the webserver
sub createGraphs( \% \% \% ) {
	my %variables = %{$_[0] };
	my %lodsHash = %{$_[1] };	
	my %markerPositionsHash = %{$_[2] };
	
	#Create an array of combined LOD scores for all pedigrees.
	#Also create (in the same pass) an array of marker positions
	my @usedPositions = ();	
	my @aggregateLodForAllFamiliesArr = ();
	foreach my $firstMarkerNum (sort {$a <=> $b} (keys %{$lodsHash{"TOTAL"}}) ) {
		foreach my $distanceFromFirstMarker ( sort {$a <=> $b} (keys %{$lodsHash{"TOTAL"}{$firstMarkerNum}}) ) {			
			my $lodScore = $lodsHash{"TOTAL"}{$firstMarkerNum}{$distanceFromFirstMarker};
			push (@aggregateLodForAllFamiliesArr, $lodScore);
			
			#Add the marker position to the hash
			push (@usedPositions, $markerPositionsHash{$firstMarkerNum}{$distanceFromFirstMarker});
		}
	}
	
	#Create a hash of LOD scores for each pedigree
	my %aggregateLodPerFamilyHash = ();
	for ( my $famCounter =  0; $famCounter <= $#{ $variables{ pednames } }; $famCounter ++ ) {		
		my $pedNum = $famCounter + 1;
		@{$aggregateLodPerFamilyHash{$pedNum}} = ();
		foreach my $firstMarkerNum (sort {$a <=> $b} (keys %{$lodsHash{$pedNum}}) ) {
			foreach my $distanceFromFirstMarker ( sort {$a <=> $b} (keys %{$lodsHash{$pedNum}{$firstMarkerNum}}) ) {			
				my $lodScore = $lodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker};
				push (@{$aggregateLodPerFamilyHash{$pedNum}}, $lodScore);
			}
		}		
	}

	#Create the graphs
	&Plotter::plotLodGraphs(\@usedPositions, \@aggregateLodForAllFamiliesArr, \%aggregateLodPerFamilyHash);
	# if the analysis is TwoPoint - create also avarage graph
	if ($variables{ program } =~ /TwoPoint/) {
		&Plotter::plotLodAvgGraphs(\@usedPositions, \@aggregateLodForAllFamiliesArr, \%aggregateLodPerFamilyHash);
	}
	
	# Mark that the graphs should be sent to the webserver by printing their names into $FILE_LIST_NAME
	my @plottingFiles = &Plotter::getPlottingFilesNamesToBeSentToWebServer();
	my $plottingFilesStr = join ("\n", @plottingFiles);
	open FILES_TO_BE_SENT_TO_WEB_SERVER, ">$FILE_LIST_NAME";
	print FILES_TO_BE_SENT_TO_WEB_SERVER "$plottingFilesStr\n";
	close FILES_TO_BE_SENT_TO_WEB_SERVER;
}


#return the number of results obtained in the last run of this script, and the number of results in the current run
sub countNumberOfPreviousAndCurrentResults( $ % % ) {
	my $numberOfResultsFileName = $_[0];
	my %likelihoodsHash = %{$_[1]};
	my %variables = %{$_[2]};
	
	#Count the number of results we have in the current run
	my $numberOfResultsInCurrentRun = 0;
	my $totalNumberOfRequiredResults = 0;
	my @familiesList = (1 .. scalar(@{$variables{pednames}}), "TOTAL");
	foreach my $pedNum (@familiesList) {		
		foreach my $firstMarkerNum ( keys %{$likelihoodsHash{$pedNum}} ) {			
			foreach my $distanceFromFirstMarker ( keys %{$likelihoodsHash{$pedNum}{$firstMarkerNum}} ) {
				my $likelihood = $likelihoodsHash{$pedNum}{$firstMarkerNum}{$distanceFromFirstMarker};
				$totalNumberOfRequiredResults++;
				if ($likelihood ne $Processing::UNCALCULATED_INFORMATION) {$numberOfResultsInCurrentRun++;}
			}
		}
	}
	
	#Find the number of results we had the last time this script ran (0 if this is the first time)	
	my $numberOfResultsInLastRun = 0;
	if (-e $numberOfResultsFileName) {$numberOfResultsInLastRun = `cat $numberOfResultsFileName`;}

	return ($numberOfResultsInLastRun, $numberOfResultsInCurrentRun, $totalNumberOfRequiredResults);
}


#This function creates output for all computations
sub handleAllComputations( \% ) {
	my %variables = %{$_[0] };
	
	#Create a hash of local likelihoods
	my $localComputationsOutputFileName = 'listOfLikelihoodComputations.txt.out';
	my %likelihoodsHash = &createLocalLikelihoodsHash('estimateLikelihoodComputationComplexity.txt',
													  'listOfLikelihoodComputations.txt',
													  $localComputationsOutputFileName,
													  \%variables);
	
	#Add the likelihoods of the remote computations to the hash
	%likelihoodsHash = &addRemoteLikelihoodsToLikelihoodsHash(\%variables, \%likelihoodsHash);
	
	#If there are no computations to make, create an empty HTML file and exit
	if (scalar(keys(%likelihoodsHash)) == 0) {
		open HTML_FILE_FH, ">$Processing::OUTPUT_RESULTS_FILE";
		print HTML_FILE_FH "No markers were available for analysis\n";
		close HTML_FILE_FH;
		&Processing::makeOuputFrameset();
		exit 0;
	}
		
	#Count the number of results we have in the current run and in the previous run of this script
	my $numberOfResultsFileName = 'number_of_results.txt';
	my ($numberOfResultsInLastRun, $numberOfResultsInCurrentRun, $totalNumberOfRequiredResults) =
		&countNumberOfPreviousAndCurrentResults($numberOfResultsFileName, \%likelihoodsHash, \%variables);	
	#If we don't have new results, return now	
	if ($numberOfResultsInCurrentRun <= $numberOfResultsInLastRun) {return 0;}
	
	#Update the number of results we have
	`echo $numberOfResultsInCurrentRun > $numberOfResultsFileName`;
	
	#create a hash of LOD scores
	my %lodsHash = &createLodsHash( \%variables, \%likelihoodsHash );
	
	#Get a hash of computation complexities and marker names (for all computations - both local and remote)
	#Using the file with the marker complexities
	my %complexitiesHash = ();
	my %markerNamesHash = ();
	&getComplexitiesAndMarkerNamesHashes(
							'estimateLikelihoodComputationComplexity.txt',
							'estimateLikelihoodComputationComplexity.txt.out',
							\%complexitiesHash, \%markerNamesHash, \%variables);
		
	#Create a list of LOD scores in the format required by the Processing Module for each family
	#And print an HTML file for each pedigree based on this list
	for ( my $famCounter =  0; $famCounter <= $#{ $variables{ pednames } }; $famCounter ++ ) {	
		
		#Create the family list
		my $pedNum = $famCounter + 1;
		my @familyPointsInfo = &createInfoListForFamily(\%variables, \%likelihoodsHash, \%lodsHash, $pedNum, \%markerNamesHash, \%complexitiesHash);
		
		#Create HTML output for this family
		my $family = $variables{ pednames }[ $famCounter ];	
		&Processing::createTotalResult( \@familyPointsInfo, "$family.result", "$family.complexity", \%variables );
	}
	
	#Create a marker positions hash
	my $markerPositionsInCmFileName = 'markerPositionsInCm.txt';
	my %markerPositionsHash = &createMarkerPositionsHash($markerPositionsInCmFileName);
	
	#Create an HTML file for all families combined
	my @totalPointsInfo = &createInfoListForFamily(\%variables, \%likelihoodsHash, \%lodsHash, "TOTAL", \%markerNamesHash, \%complexitiesHash);
	&Processing::createTotalResult( \@totalPointsInfo, "total.result", "total.complexity", \%variables );
	
	#create a text file with the LOD table
	&createLodTableFile(\%variables, \%likelihoodsHash, \%lodsHash, \%markerNamesHash, \%markerPositionsHash, \%complexitiesHash);
	
	#Create final HTML files and graphs
	&Processing::debug_print("Creating total result");		
	&createGraphs(\%variables, \%lodsHash, \%markerPositionsHash);
	&Processing::printOutputHeader( $RESULT_HEADER_FILE, \%variables );
	&Processing::printTablesToOutput( \%variables );
	
	#If we finished all computations, send the file TASK_LOG.txt to the webserver to mark that the
	#computation is finished
	if ($numberOfResultsInCurrentRun >= $totalNumberOfRequiredResults) {
		open FILES_TO_BE_SENT_TO_WEB_SERVER, ">>$FILE_LIST_NAME";
		print FILES_TO_BE_SENT_TO_WEB_SERVER $variables{taskFinishFlag}."\n";
		close FILES_TO_BE_SENT_TO_WEB_SERVER;
	}	
	
	# dvir: this line should be here , because, inside makeOuputFrameset, we are creating the file
	# formatted_output.tar.gz by the list $FILE_LIST_NAME, and we want that the files in @plottingFiles
	# will be also inside this list.
	# by the way, the png files appears twice in the list $FILE_LIST_NAME because both 
	# Plotter::getPlottingFilesNamesToBeSentToWebServer & Processing::makeOuputFrameset add them.
	&Processing::makeOuputFrameset(  );

	#Send current results to the webserver
	#Omer 15/9/2011 - Since the Post script is run inside a Condor DAG, it does not know the environment
	#variables. We set here the BIN directory manually...
	my $bashBinDir = "/scratch/gridbot/bin";
	system("$bashBinDir\/sendResultsToWebServer.sh");
}
