#!/usr/bin/perl -w
################################################################
# 		This file is the prepares files for each point
#	of Two Point Superlink calculations
################################################################
use FindBin ();
use lib "$FindBin::Bin";
use Processing;

####
# dvir 2011/08
use WebServerConfiguration;
# in order to use packages from cbl-link01 , superlinksnp , perlbin dir
use lib $WebServerConfiguration::PERL_BIN_DIR_AT_BACKEND;

# packages in use from cbl-link01 , superlinksnp , perlbin dir
use Plotter;
use DatWrapper;
use PedWrapper;
use Distances;
####
################################################################

sub handleRemoteComputations( \% );
sub calculateComplexity( $ $ $ $ $ \% $ $);
sub createLocalComputationsFile( $ $ );
sub performLocalComputations( $ $ $ \% );
sub performRemoteComputationsNew( $ \% $ $);

################################################################
#			USEFUL CONSTANTS
################################################################
# IMPORTANT : $PEDTOOL_ROOT is a environment variable of the backend cbl - link01
my $PRE_POST_SCRIPTS_DIR = "\$PEDTOOL_ROOT\/pre_post_scripts";
#my $SUPERLINK_EXECUTEABLE = "$PRE_POST_SCRIPTS_DIR\/SuperlinkwindowedHMM.64bit.linux";

#Omer - this executable contains fixes for the previous one.
my $SUPERLINK_EXECUTEABLE = "$PRE_POST_SCRIPTS_DIR\/Superlink1_8windowedSL.linux";
my $DUMMY_TAR_WITH_ONE_COMPUTATION_FILENAME = "$PRE_POST_SCRIPTS_DIR\/dummy_datafile.tar.gz";
my $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY = 7;

################################################################
# 		START
################################################################
if ( $#ARGV < 0 ) {$projectDir = ".";}
else {$projectDir = $ARGV[0];}
#my @path = split "prepareMultiPointFiles.pl", $0;
#chop $path[0];
use Cwd;
my $currDir =  &getcwd();
#chdir $path[0];
&Processing::setRunDir( &getcwd() );
&Processing::setLogFiles( "$Processing::RUN_DIR\/$projectDir\/" );
chdir $currDir;

&Processing::debug_print ( "================================================================" );
&Processing::debug_print ( "      PREPARE MULTI POINT FILES has been started with parameter ($projectDir) in dir ($Processing::RUN_DIR)" );
&Processing::debug_print ( "================================================================" );

my %variables = %{ &Processing::initVariablesFromPrePoint( $projectDir ) };
$variables{ newDir } = $projectDir;

%variables = %{&Processing::makeSystemError("Failed creating directory ($projectDir\/pedfiles)", \%variables )}
	unless (  -e  "$projectDir\/pedfiles" || mkdir "$projectDir\/pedfiles" );

%variables = %{&Processing::makeSystemError("Failed creating directory ($projectDir\/datafiles)", \%variables )}
	unless (  -e "$projectDir\/datafiles" || mkdir "$projectDir\/datafiles");

%variables = %{&Processing::makeSystemError("Failed enter to project directory ($projectDir)", \%variables )}
	unless ( chdir $projectDir );
exit $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );

# extacting a tar with only one computation in order that the framework of the condor won't
# fail at the next stages
`tar -zxf $DUMMY_TAR_WITH_ONE_COMPUTATION_FILENAME`;

#Sort the ped file according to pedigree order to prevent bugs in complexity estimation
$tmpPedName = $variables{pedfile}."_tmp";
system("sort +0n -2 -s ".$variables{pedfile}." > ".$tmpPedName);
system("cp -f ".$tmpPedName." ".$variables{pedfile});
unlink($tmpPedName);


#In case a list of markers is used, create a dat and ped file file so that will contain
# only the used markers
my $usedMarkersOnlyDatFileName = $variables{datafile};
my $usedMarkersOnlyPedFileName = $variables{pedfile};

my $datWrapper = new DatWrapper($usedMarkersOnlyDatFileName);
my $numOfMarkers = $datWrapper->getNumOfMarkers();
my $pedWrapper = new PedWrapper($usedMarkersOnlyPedFileName, $numOfMarkers);
		
#Omer 1/9/2011 - This file now handles both TwoPoint and MultiPoint computations.
#Creation of new dat and ped files is required only for Multipoint computations
if ($variables{program} =~ /MultiPoint/) {
	unless (defined($variables{ jump }) && ( $variables{ jump } !~ /Unknown/ ) ) {
		#Create new dat and ped files, with only the relevant markers		
		my @familiesList = $pedWrapper->getFamiliesNumbers();
		#Print the dat file
		$usedMarkersOnlyDatFileName = 'datfile.listOfMarkers.dat';
		open (DATFILE_FH, ">$usedMarkersOnlyDatFileName");
		$datWrapper->writeSubDatFileByListOfMarker(\*DATFILE_FH, \@{$variables{usedMarkersId}});
		close DATFILE_FH;
		#Print the ped file
		$usedMarkersOnlyPedFileName = 'pedfile.listOfMarkers.dat';
		open (PEDFILE_FH, ">$usedMarkersOnlyPedFileName");
		$pedWrapper->writeSubPedFileByListOfMarker(\*PEDFILE_FH, \@{$variables{usedMarkersId}}, \@familiesList);
		close PEDFILE_FH;
	}
}
# checking what is the complexity of each computation
# This function will create a file called ${estimateLikelihoodComputationComplexityFileName}.out,
# which will contain the complexity of each computation.
# If the computation is less than $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY
# then we will write the computation to a file with a list of computations to run locally : $listOfLikelihoodComputationsFileName
my $estimateLikelihoodComputationComplexityFileName = 'estimateLikelihoodComputationComplexity.txt';
my $markerPositionsInCmFileName = 'markerPositionsInCm.txt';
my $complexityEstimationsOfLikelihoodComputationsFileName = "$estimateLikelihoodComputationComplexityFileName".".out";
&calculateComplexity($usedMarkersOnlyDatFileName,
					 $usedMarkersOnlyPedFileName,
					 $estimateLikelihoodComputationComplexityFileName,
					 $complexityEstimationsOfLikelihoodComputationsFileName,
					 $markerPositionsInCmFileName,
					 \%variables,
					 $datWrapper,
					 $pedWrapper);

#Create a file with the computations that will be performed locally (those with low complexity).
#Also, update the variables $doWeHaveComputationsToRunLocally and $doWeHaveComputationsToRunRemotely.
my $listOfLikelihoodComputationsFileName = 'listOfLikelihoodComputations.txt';

(my $doWeHaveComputationsToRunLocally, my $doWeHaveComputationsToRunRemotely) =
&createLocalComputationsFile($complexityEstimationsOfLikelihoodComputationsFileName, 
	$listOfLikelihoodComputationsFileName);
    
#Create directories for computations that will be performed through Condor (if needed), and
#Create the file that tells in which directory each computation was made
&performRemoteComputationsNew($complexityEstimationsOfLikelihoodComputationsFileName, \%variables, $datWrapper, $pedWrapper);

#checking if there are computations to be run locally
if ($doWeHaveComputationsToRunLocally) {
	&performLocalComputations($usedMarkersOnlyDatFileName,
							  $usedMarkersOnlyPedFileName,
							  $listOfLikelihoodComputationsFileName, \%variables);
}					 

exit(0);
################################################################



################################################################
# 				calculateComplexity
#	Creates a file that contains the complexity of each computation that needs to be performed
#	and also create a file with the position (in cM) of each marker on the chromosome
#
#	Parameters:
#					$_[0]  - Name of datfile which contains only the used markers
#					$_[1]  - Name of pedfile which contains only the used markers
#					$_[2]  - Name of the input file to create
#					$_[3]  - Name of the output file that should be created
#					$_[4]  - Name of the marker positions file to create
#					$_[5]  - Hash with variables
#					$_[6]  - A DatWrapper object
#					$_[7]  - A PedWrapper object
#	Returns:
#					Nothing
################################################################
# Omer 2011/08
sub calculateComplexity( $ $ $ $ $ \% $ $) {
	my $usedMarkersOnlyDatFileName = $_[0];
	my $usedMarkersOnlyPedFileName = $_[1];
	my $estimateLikelihoodComputationComplexityFileName = $_[2];
	my $complexityEstimationsOfLikelihoodComputationsFileName = $_[3];
	my $markerPositionsInCmFileName = $_[4];
	my %variables = %{$_[5] };
	my $datWrapper = $_[6];
	my $pedWrapper = $_[7];	
	
	my $distOfFirstMarkerFromTheBeginingOfChromIn_cM = &Distances::calcDistanceOfFirstMarkerFromTheBeginingOfTheChromosomeIn_cM(\%variables);
	
	#Read the computation details from the modefile
	my $startAt = 1;
	my $endAt = $variables{ numMarkers };
	if ( defined($variables{ from }) && ( $variables{ from } !~ /Unknown/ ) ) {$startAt = $variables{ from };}
	if ( defined($variables{ to }) && ( $variables{ to } !~ /Unknown/ ) ) {$endAt = $variables{ to };}
	if ($endAt < $startAt) {die "Invalid start and end parameters";}
	my $windowSize = 1;			#Default value in case the computation is TwoPoint
	my $useSlidingWindow = 0;	#Default value in case the computation is TwoPoint
	my @markersArray = sort {$a <=> $b} @{$variables { usedMarkersId }};	#Default value in case the computation is TwoPoint
	my $isMultiPointComputation = 0;	#Default value in case the computation is TwoPoint
	
	if ($variables{program} =~ /MultiPoint/) {
		$isMultiPointComputation = 1;
	
		#In case a window was requested for the computation	
		if ( defined($variables{ jump }) && ( $variables{ jump } !~ /Unknown/ ) ) {
			$useSlidingWindow = 1;
			@markersArray = ($startAt .. $endAt);
			$windowSize = $variables{ jump };
			# If the jump value was too high we take all the markers as one group
			if ($startAt + $windowSize > $endAt) {$windowSize = $endAt - $startAt + 1;}		
			
			my $validWindow = &Processing::isValidWindow($startAt, $endAt, $windowSize, $variables{ datafile });
			if ($validWindow =~ /size/ )  {
				%variables = %{&Processing::makeInputError("Invalid parameters! ".$validWindow, \%variables )};
				exit $Processing::ERROR if ( $Processing::NO !~ $variables{ errorExist } );
			}
			
			#Create MULTIDATA file to be used later by the Post script
			open (MULTIDATA, ">>$projectDir/multiParams.dat") or die "Can't open log file 1\n";
			print MULTIDATA "$startAt $endAt $windowSize \n";
			close MULTIDATA;
		}
		
		#In case no window was requested for the computation,
		#put all markers in a single window
		else {
			$useSlidingWindow = 0;			
			$windowSize = scalar(@markersArray);
			
			#Create MULTIDATA file to be used later by the Post script
			$list = join ' ', @markersArray;
			open (MULTIDATA, ">>$projectDir/multiList.dat") or die "Can't open log file 1\n";		
			print MULTIDATA "$list \n";
			close MULTIDATA;
		}
	}
	
	#Get the list of families in the ped file
	my @familiesNumbers = $pedWrapper->getFamiliesNumbers();
	
	# creating a file with a list of all likelihood computations in order to estimate the complexity.
	# each line is a computation
	# line structre: $$$Some data you should ignore$$$<pedigree number>,<first marker number>,<number of markers to use>, teta1	
	open LIKELIHOOD_COMPUTATIONS_FH, ">$estimateLikelihoodComputationComplexityFileName";
	open MARKER_POSITIONS_FH, ">$markerPositionsInCmFileName";
	my $infinityDistance = &Distances::recToCm(-0.5);
	
	#Iterate over every computation that needs to be performed and write it to the computations file
	for my $familyNum (@familiesNumbers) {		
        my $windowCounter = 0;	
        my $firstMarkerNum = $startAt;    
		for my $markerNum (@markersArray) {
			my $markerName = $datWrapper->getMarkerName($markerNum);
			my @thetaPoints = ();
			
			#If a new window starts, compute the likelihood at infinity and reset the distance counter and the first marker
			if ($windowCounter == 0) {
				push(@thetaPoints, $infinityDistance);				
				$firstMarkerNum = $markerNum;
				#Enlarge the window if it's too small (because #markers does not divide windowsize)
				if ($firstMarkerNum + $windowSize - 1 > $endAt) {$firstMarkerNum = $endAt - $windowSize + 1;}
			}
			my $distance = $datWrapper->calcDistIn_cM_BetweenTwoMarkers($firstMarkerNum, $markerNum);
			push(@thetaPoints, $distance);
			
			#Print the relevant computations to the computations file
			#And to the marker positions file
			for my $thetaPoint (@thetaPoints) {
				my $firstMarkerNumToPrint = $firstMarkerNum;
				#In case of a Multipoint computation with single window, the first marker in the datfile is
				#always the first marker in the window
				if ($isMultiPointComputation && !$useSlidingWindow) {$firstMarkerNumToPrint = 1;}				
				print LIKELIHOOD_COMPUTATIONS_FH "\$\$\$${markerName}\$\$\$$familyNum,$firstMarkerNumToPrint,$windowSize,$thetaPoint\n";
				
				#Write the marker position to the marker positions file
				#This file will be used by the post script
				if ($thetaPoint != $infinityDistance) {
					my $markerPosition = $distOfFirstMarkerFromTheBeginingOfChromIn_cM + 
										 $datWrapper->getDistIn_cM_FromFirstMarker($markerNum);
					print MARKER_POSITIONS_FH "\$\$\$${markerName}\$\$\$$familyNum,$firstMarkerNum,$windowSize,$thetaPoint,${markerPosition}\n";
				}				
			}
			
			#Increase the window counter
			$windowCounter++;
			if ($windowCounter >= $windowSize) {$windowCounter = 0;}	#Start a new window if needed
		}
	}
	close LIKELIHOOD_COMPUTATIONS_FH;
	close MARKER_POSITIONS_FH;	
	
	# calculating the complexity of the computations
	#The output will be written to a file called ${estimateLikelihoodComputationComplexityFileName}.out	
	
	#Omer 11/10/2011 - if the complexity estimation takes more than $allowedTimeForComplexityEstimation seconds,
	#timeout and mark that all processes should run remotely, because apparently they are very hard
	my $allowedTimeForComplexityEstimation = 120;	
	my $returnVal = -1;
	my $timeOutOccurred = 0;
	my $superlinkReturnValFileName = "superlinkReturnVal.txt";
	unlink $superlinkReturnValFileName;
	
	eval {
		my $pid = fork();	
		unless (defined $pid) {exit $Processing::ERROR;}
		
		#Child process runs the Superlink executable
		if ($pid == 0) {
			setpgrp(0,0);
			my $superlinkCommand = "$SUPERLINK_EXECUTEABLE $usedMarkersOnlyDatFileName $usedMarkersOnlyPedFileName --batch $estimateLikelihoodComputationComplexityFileName --complexity -mem 500";
			my $superlinkReturnVal = system($superlinkCommand." > /dev/null");
			`echo $superlinkReturnVal > $superlinkReturnValFileName`;
			exit 0;
		}
		
		#The father process waits for $allowedTimeForComplexityEstimation seconds and then kills the child process
		else {
			local $SIG{ALRM} = sub {kill -9, $pid; $timeOutOccurred = 1; alarm 0; $returnVal = 0;};
			alarm $allowedTimeForComplexityEstimation;
			waitpid	$pid, 0;
			alarm 0;
		}
	};
	
	#If we timed out, create a fake output file that says all computations are hard	
	if ($timeOutOccurred) {
		open LIKELIHOOD_COMPUTATIONS_FH, "$estimateLikelihoodComputationComplexityFileName";
		open OUTPUT_COMPLEXITY_FILE_FH, ">$complexityEstimationsOfLikelihoodComputationsFileName";
		my $fakeHardComplexity = $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY * 2;		
		while (my $currLine = <LIKELIHOOD_COMPUTATIONS_FH>) {			
			chomp($currLine);			
			$currLine .= (",@".$fakeHardComplexity."\n");			
			print OUTPUT_COMPLEXITY_FILE_FH $currLine;
		}
		close LIKELIHOOD_COMPUTATIONS_FH;
		close OUTPUT_COMPLEXITY_FILE_FH;
		$returnVal = 0;
	}
	else {$returnVal = `cat $superlinkReturnValFileName`;}
	
	if ($returnVal != 0) {
		&Processing::debug_print("Superlink failed, please contact system administrators.");
		%variables = %{&Processing::makeInputError("Superlink failed, please contact system administrators.", \%variables )};
		exit $Processing::ERROR;
	}	
}

################################################################
# 				createLocalComputationsFile
#	Creates a file that contains the computations that will be perfomed locally
#	(those with low complexity). Note that function updates the global variable $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY.
#
#	Parameters:
#					$_[0]  - Name of file with complexities
#					$_[1]  - Name of the file to create	
#	Returns:
#					the variables $doWeHaveComputationsToRunLocally
#					and $doWeHaveComputationsToRunRemotely.
################################################################
# dvir & omer 2011/08
sub createLocalComputationsFile( $ $ ) {
	my $complexityEstimationsOfLikelihoodComputationsFileName = $_[0];
	my $listOfLikelihoodComputationsFileName = $_[1];
	my $doWeHaveComputationsToRunLocally = 0;
	my $doWeHaveComputationsToRunRemotely = 0;    
    my $totalLocalComplexity = 0;	
	
    #Read the computation complexities and calculate the sum of the complexities of the local computations
	# line structre in $complexityEstimationsOfLikelihoodComputationsFileName : 
	# $$$ asbsadbabsdfb$$$1, 23,4,0.01,@COMPLEXITY    
	open COMPLEXITY_RESULTS_FH, $complexityEstimationsOfLikelihoodComputationsFileName;
	while (my $currLine = <COMPLEXITY_RESULTS_FH>) {        
		@currLineArr = split '@', $currLine;
		my $complexity = $currLineArr[1];        
        chomp($complexity);
		if ($complexity <= $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY) {$totalLocalComplexity += 10**$complexity};
	}		
	close COMPLEXITY_RESULTS_FH;

    #Change the complexity threshold for local computations
    if ($totalLocalComplexity > 0) {
        $totalLocalComplexity = log($totalLocalComplexity) / log(10);
        if ($totalLocalComplexity > $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY+4) {$MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY = 0;}    
    }
    
    #Now read the complexities files again and create a file of local computations 
    open COMPUTATIONS_FH, ">$listOfLikelihoodComputationsFileName";
	open COMPLEXITY_RESULTS_FH, $complexityEstimationsOfLikelihoodComputationsFileName;
	while (my $currLine = <COMPLEXITY_RESULTS_FH>) {
		@currLineArr = split '@', $currLine;
		my $complexity = $currLineArr[1];
        chomp($complexity);
		if ($complexity <= $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY) {
			print COMPUTATIONS_FH $currLine;			
			$doWeHaveComputationsToRunLocally = 1;  #marking that we have computations to run locally                      
		}
		else {$doWeHaveComputationsToRunRemotely = 1;} #marking that we have computations not to run locally 
	}	
	close COMPUTATIONS_FH;
	close COMPLEXITY_RESULTS_FH;
	
	return ($doWeHaveComputationsToRunLocally, $doWeHaveComputationsToRunRemotely);
}


################################################################
# 				performLocalComputations
#	Runs the executable that performs all the computations that are supposed to run locally,
#	And marks that it was ran
#
#	Parameters:
# 					$_[0]  - Name of dat file containing only the markers used in the analysis
# 					$_[1]  - Name of ped file containing only the markers used in the analysis
#					$_[2]  - Name of file with computations to perform
#					$_[3]  - Hash with variables
#	Returns:
#					Nothing
################################################################
# dvir & omer 2011/08
sub performLocalComputations( $ $ $ \% ) {
	my $usedMarkersOnlyDatFileName = $_[0];
	my $usedMarkersOnlyPedFileName = $_[1];
	my $listOfLikelihoodComputationsFileName = $_[2];
	my %variables = %{$_[3] };

	# running those computations, a results.txt file will also be created at the cwd
	my $superlinkCommand = "$SUPERLINK_EXECUTEABLE $usedMarkersOnlyDatFileName $usedMarkersOnlyPedFileName --batch $listOfLikelihoodComputationsFileName -mem 500 --lod";	
	my $returnVal = system($superlinkCommand." > /dev/null");
	if ($returnVal != 0) {
		&Processing::debug_print("Superlink failed, please contact system administrators.");
		%variables = %{&Processing::makeInputError("Superlink failed, please contact system administrators.", \%variables )};
		exit $Processing::ERROR;
	}
}


################################################################
# 				performRemoteComputationsNew
#	Creates directories and files needed to run remote computations. The Conder DAG will recognize these directories
#	And perform remote computations on them. 
#
#	Parameters:
#					$_[0]  - Name of file with computation complexities 
#					$_[1]  - Hash with variables
#	Returns:
#					Nothing
################################################################
sub performRemoteComputationsNew( $ \% $ $ ) {
	my $complexityEstimationsOfLikelihoodComputationsFileName = $_[0];
	my %variables = %{$_[1] };	
	my $datWrapper = $_[2];
	my $pedWrapper = $_[3];
	
	&Processing::debug_print("Creating directories" );
	
	#Create the array that maps thetas to computation numbers.
	#This will affect the directory structure.
	#The computation of marker $m at theta $t will be created in the directory
	#called "datafile_$m_$markerToComputationNumHash{$m}+1"	
	my %markerToComputationNumHash = ();
	
	#Open the file which states in which directory each remote computation will be performed
	open COMPUTATION_DIRECTORIES_FH, ">computationDirectories.txt";

	#Check if a sliding window is used
	my $useSlidingWindow = 1;
	my $firstMarkerInWindow = 0;
	unless (defined($variables{jump}) && ( $variables{jump} !~ /Unknown/ ) ) {
		$useSlidingWindow = 0;
		$firstMarkerInWindow = ${$variables{usedMarkersId}}[0];
	}
	
	#Check if the computation is Multipoint or not
	my $isMultiPointComputation = 0;
	if ($variables{program} =~ /MultiPoint/) {$isMultiPointComputation = 1;}
	
	my @familiesList = $pedWrapper->getFamiliesNumbers();	
	
	# line structre in $complexityEstimationsOfLikelihoodComputationsFileName : 
	# $$$ asbsadbabsdfb$$$1, 23,4,0.01,@COMPLEXITY
	# COMPLEXITY is a number with the complexity	
	open COMPLEXITY_RESULTS_FH, $complexityEstimationsOfLikelihoodComputationsFileName;
	while (my $currLine = <COMPLEXITY_RESULTS_FH>) {
		#Read the line information
		@currLineArr = split '@', $currLine;
		my $complexity = $currLineArr[1];		
		@currLineArr = split ',', $currLine;
		@currLineSubArr = split '\$\$\$', $currLineArr[0];	
		my $markerName = $currLineSubArr[1];	#Name of the first marker in the window
		my $pedNum = $currLineSubArr[2];	#Pedigree number
		my $firstMarkerNum = $currLineArr[1];	#Number of first marker in the window, as it appears in the datfile
		if ($isMultiPointComputation && !$useSlidingWindow) {$firstMarkerNum = $firstMarkerInWindow;}	#This is the real index of the first marker in the window
		my $windowSize = $currLineArr[2];		#Window size of the computation
		my $distanceFromFirstMarker = $currLineArr[3];	#Distance of the putative gene from the first marker in theta		
		$distanceFromFirstMarker =~ s/\s+$//;			#Remove trailing white spaces
		
		#If the computation will be done locally, proceed to the next computation		
		if ($complexity <= $MAX_COMPLEXITY_TO_BE_COMPUTE_LOCALLY) {next;}		
		
		#Determine the computation number based on the identity of the first marker in the window		
		unless (defined $markerToComputationNumHash{$firstMarkerNum}) {$markerToComputationNumHash{$firstMarkerNum} = 0;}
		my $computationNum = $markerToComputationNumHash{$firstMarkerNum};
		$markerToComputationNumHash{$firstMarkerNum}++;
		
		#Create a directory for the dat and ped files
		my $datafileDir = "datafile_".$firstMarkerNum."_".$computationNum;			
		&Processing::debug_print ( "Creating directory: $datafileDir" );
		if (!-e $datafileDir && !mkdir $datafileDir) {%variables = %{&Processing::makeSystemError("Failed creating directory $datafileDir", \%variables )};}
		my $pedfileDir = "pedfile_".($pedNum-1);
		my $computationDir = $datafileDir."/".$pedfileDir;
		if (!-e $computationDir && !mkdir $computationDir) {%variables = %{&Processing::makeSystemError("Failed creating directory $computationDir", \%variables )};}
		
		#Get strings with the contents of the dat and ped file that need to be created for this computation
		my @markersArray = ($firstMarkerNum .. $firstMarkerNum + $windowSize - 1);
		
		#Create the dat and ped files		
		my $programOptions = "1 1\n(".$distanceFromFirstMarker.")\n";
		$datWrapper->setProgramCode(6, $programOptions);
		open MARKER_DAT_FILE_FH, ">${computationDir}/datafile.dat";		
		$datWrapper->writeSubDatFileByListOfMarker(\*MARKER_DAT_FILE_FH, \@markersArray);
		close MARKER_DAT_FILE_FH;
		open MARKER_PED_FILE_FH, ">${computationDir}/pedfile.dat";		
		$pedWrapper->writeSubPedFileByListOfMarker(\*MARKER_PED_FILE_FH, \@markersArray, \@familiesList);
		close MARKER_PED_FILE_FH;

		#Write the computation details to COMPUTATION_DIRECTORIES_FH
		print COMPUTATION_DIRECTORIES_FH "\$\$\$$markerName\$\$\$$pedNum,$firstMarkerNum,$windowSize,$distanceFromFirstMarker,${computationDir}\n";
	}	
	close COMPLEXITY_RESULTS_FH;
	close COMPUTATION_DIRECTORIES_FH;
}

################################################################
1;
