#!/usr/bin/perl -w

use FindBin ();
use lib "$FindBin::Bin";
use submissionConfig;

use constant EX_LOCK => 2;

$config = new submissionConfig;

$tool = 0;
$task = 0;
$parent = 0;
$expName = 0;
$files = 0;
$enlarge = 0;
$freq = 0;
$MOI = 0;
$max = 0;

$clean = 0;
$from = 0;
$to = 0;
$window = 0;
$automate = 0;

for $i (0..$#ARGV){
		if ($ARGV[$i] eq "-tool"){
			$tool = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-sourceTask"){
			$task = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-parent"){
			$parent = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-expName"){
			$expName = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-files"){
			$files = $ARGV[$i+1];			# separated by '$'
		}
		
		if ($ARGV[$i] eq "-enlarge"){
			$enlarge = "true";
		}
		
		if ($ARGV[$i] eq "-freq"){
			$freq = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-MOI"){
			$MOI = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-max"){
			$max = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-from"){
			$from = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-to"){
			$to = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-window"){
			$window = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-clean"){
			$clean = $ARGV[$i+1];
		}
		
		if ($ARGV[$i] eq "-automate"){
			$automate = $ARGV[$i+1];
		}
}

if ( (!$tool) or (!$task) or (!$parent) or (!$expName) or (!$files and !$enlarge) or (!$freq) or (!$MOI) or (!$max) ){
	print "Needed pamameters are:\n";
	print "\t-tool <tool>\t\t\t: TwoPointAnalysis, MultiPointAnalysis\n";
	print "\t-sourceTask <file source task name>\n";
	print "\t-parent <father task name>\n";
	print "\t-expName <new exp name>\n";
	print "\t-files <file names>\t\t: file names should be separated by '|'\n";
	print "\t-enlarge\t\t\t: use instead of 'files' in order to use enlarged area from the original snp\n";
	print "\t-freq <frequency>\n";
	print "\t-MOI <MOI>\t\t\t: format: Rec_<penetrance> / Dom_<penetrance1>_<penetrance2>\n";
	print "\t-max <max>\t\t\t: max lod score of father experiment.\n";
	print "Optional:\n";
	print "\t-from <from marker>\n";
	print "\t-to <to marker>\n";
	print "\t-window <window size>\n";
	print "\t-clean <cleaning threshold>\n";
	print "\t-automate <true/false>\n";
	exit -1;
}

$mode = 0;
$penetrance = 0;
@MOI_arr = split(/[_]/, $MOI);
if ($MOI_arr[0] eq "Rec"){
	$mode = "Recessive";
	$penetrance = "0+".$MOI_arr[1];
}
else{
	if ($MOI_arr[0] eq "Dom"){
		$mode = "Dominant";
		$penetrance = $MOI_arr[1]."+".$MOI_arr[2];
	}
	else{
		print "Error on MOI, please check your input.\n";
		exit -1;
	}
}

$range = "";
if ($from and $to){		# in case one of them was not set
	$range = " InputRadio=Range StartMarker=$from EndMarker=$to";
	if ($window){
		$range = $range." WindowBox=on WindowSize=$window";
	}
}

$cleaning = "";
if ($clean){
	$cleaning = "CleanThreshold=$clean";
}

$fileChoice = "enlarge=true";			# in case -enlarge defined
if ($files){					# in case -files were defined
	$fileChoice = "FileChoice[]=$files";
}

if ($automate){
	$automate = "automate=true";
}
else {
	$automate = "automate=false";
}

#system("php opTool.php?...");

$fatherExpDir = &getDirByTask( $task );
$ancestor = &getAncestor( $parent );
#$max = &getMaxOfTask( $task );

@temp = split(/\//, $fatherExpDir);
$fatherExpName 	= $temp[$#temp];
$projectName 	= $temp[$#temp-2];
$user 			= $temp[7];
$temp[$#temp] 	= $expName;
$newExpDir 		= join("/", @temp);

#&setNewExpParams($task, $user, $tool, $fatherExpName, $projectName, $files, $expName, $freq, $mode, $penetrance, $from, $to, $window);

print "Project: ".$projectName."\n";

chdir "$config->{'FULL_DIR'}";
#system("php $config->{'FULL_DIR'}/opTool_offline.php?user=$user\&toolsChoice=$tool\&parent=$fatherExpName\&ProjectChoice=$projectName\&ExperimentChoice=$fatherExpName\&FileChoice[]=@$files&experimentName=$expName\&frequency=$freq\&penetrance=$mode\&penetranceChoiceRec=$penetrance".$range);
$ret = 0;
system("php opTool_offline.php user=$user toolsChoice=$tool parent=$fatherExpName ProjectChoice=$projectName ExperimentChoice=$fatherExpName $fileChoice experimentName=$expName frequency=$freq penetrance=$mode penetranceChoiceRec=$penetrance ".$range." ".$cleaning." ".$automate);
$ret = $? >> 8;
#if ($ret > 200) { $ret = $ret - 256; }
print "return value: ".$ret;
if ($ret == 1) { die("Could not create experiment, failed on creation"); }
elsif ($ret == 2) { die("Failed, experiment with the chosen name already exists."); }
elsif ($ret == 3) { die("Failed due to an input error"); }
elsif ($ret == 4) { die("Failed, no tool was chosen"); }


open(PROCESS, "$newExpDir/.processID.txt") || die("Cannot open info file");
@process = <PROCESS>;
close(PROCESS);

$newExpTask = $process[0];
chomp($newExpTask);

system("echo \"fatherMax $max\" >> $config->{'FULL_DIR'}/Submissions/$newExpTask/$config->{'MAX_LODSCORE_FILE'}");
system("echo \"ancestor $ancestor\" >> $config->{'FULL_DIR'}/Submissions/$newExpTask/$config->{'MAX_LODSCORE_FILE'}");

exit 0;

sub getDirByTask( $ ){
	my $task = $_[0];

	my $dir = "$config->{'FULL_DIR'}/Submissions/$task";
	
	open(INFO, "$dir/submissionInformation") || die("Cannot open info file");
	@details = <INFO>;
	close(INFO);
	
	my $expDir = "$config->{'FULL_DIR'}/".$details[1];
	chomp($expDir);
	return $expDir;
}

sub getMaxOfTask( $ ){
	my $task = $_[0];

	my $dir = "$config->{'FULL_DIR'}/Submissions/$task";
	
	open(MAXLOD, "$dir/$config->{'MAX_LODSCORE_FILE'}") || die("Cannot open info file");
	@maxlod = <MAXLOD>;
	close(MAXLOD);
	
	$max = 0;
	for $i (0..$#maxlod){
		chomp($maxlod[$i]);
		@keyVal = split(/[ ]/, $maxlod[$i]);
		if ($keyVal[0] eq "currentMax"){
			$max = $keyVal[1];
		}
	}
	return $max;
}

sub setNewExpParams( $ $ $ $ $ $ $ $ $ $ $ $ $ ){
	my $task = $_[0];
	my $user = $_[1];
	my $tool = $_[2];
	my $fatherExpName = $_[3];
	my $projectName = $_[4];
	my $files = $_[5];
	my $expName = $_[6];
	my $freq = $_[7];
	my $mode = $_[8];
	my $penetrance = $_[9];
	my $from = $_[10];
	my $to = $_[11];
	my $window = $_[12];
	
	my $dir = "$config->{'FULL_DIR'}/Submissions/$task";
	
	open(PARAMS, ">$dir/params.txt") || die("Cannot open info file");
	print PARAMS "user = $user\n";
	print PARAMS "toolsChoice = $tool\n" ;
	print PARAMS "parent = $fatherExpName\n";
	print PARAMS "ProjectChoice = $projectName\n";
	print PARAMS "ExperimentChoice = $fatherExpName\n";
	print PARAMS "FileChoice[] = $files\n";
	print PARAMS "experimentName = $expName\n";
	print PARAMS "frequency = $freq\n";
	print PARAMS "penetrance = $mode\n";
	print PARAMS "penetranceChoiceRec = $penetrance\n";
	print PARAMS "from = $from\n";
	print PARAMS "to = $to\n";
	print PARAMS "window = $window\n";
	
	close(PARAMS);
}

sub getAncestor( $ ){
	my $task = $_[0];
	my $ancestor = $task;
	
	if (!(-e "$config->{'FULL_DIR'}/Submissions/$task/$config->{'MAX_LODSCORE_FILE'}")) { return $ancestor; }
	
	open (MAXFILE, "$config->{'FULL_DIR'}/Submissions/$task/$config->{'MAX_LODSCORE_FILE'}");
	@lines = <MAXFILE>;
	close(MAXFILE);
	
	foreach $line (@lines){
		chomp($line);
		@line = split (/[ ]/, $line);
		if ($line[0] eq "ancestor"){
			#$ancestor = &getAncestor( $line[1] );
			$ancestor = $line[1];
			last;
		}
	}
	
	return $ancestor;
}