#!/usr/bin/perl -w

use strict;
use submissionConfig;

my $config = new submissionConfig;

my $mode = 0; # mode is 0 if apache calls this script (not a must). use mode 1 if others with ~/.ssh folder call this script.  specify by -mode.

# the name of the ped that will be send to exec server.
my $PED_FOR_DRAWING_FILENAME = 'pedForDrawing.ped';

my $destDir;
my $pedFileName;
my $transFile;
for my $i (0..$#ARGV){
	# this ped can be simplified or with marker data
	if ($ARGV[$i] eq "-ped"){
		$pedFileName = "$config->{'FULL_DIR'}\/$ARGV[$i+1]";
	}
	if ($ARGV[$i] eq "-destDir"){
		$destDir = "$config->{'FULL_DIR'}\/$ARGV[$i+1]";
	}
	if ($ARGV[$i] eq "-translate"){
		$transFile = "$config->{'FULL_DIR'}\/$ARGV[$i+1]";
	}
	if ($ARGV[$i] eq "-mode"){
		$mode = $ARGV[$i+1];
	}
}
# checks that 
if (!defined $destDir) {
	die "Please specify the destination directory: -destDir <path>\n";
}
if (!-d $destDir) {
	die "given dest dir isn't a dir\n";
}
if (!defined $pedFileName) {
	die "Ped file name argument missing. Please specify: -ped <pedFileName>\n";
}
if (!-e $pedFileName) {
	die "ped File doesn't exsits\n";
}

my @path = split /\//, $destDir;

my $painter_parameters = "DestDir $destDir\nPedNameForDrawing $PED_FOR_DRAWING_FILENAME\n";
my $tarList = "painter_parameters.txt\n$PED_FOR_DRAWING_FILENAME\n";
	
# dvir 2010/11/29
# awk works on $pedFileName with fields seperated by tab \ space.
# I'm using awk in order :
# 1. not to copy a large file with all the allels of the people to the tasks server
# 2. To use less space in the current dir: $destDir
# $1 -> ped num
# $2 -> person in family num
# $3 -> father num
# $4 -> mother num
# $8 -> sex
# $10 -> affection status
system("awk ' { print \$1, \$2, \$3, \$4, \$8, \$10 }' $pedFileName > $destDir\/$PED_FOR_DRAWING_FILENAME");

if (defined($transFile)) {
	system("cp $transFile $destDir\/translate.txt");
	$painter_parameters.= "Translate translate.txt\n";
	$tarList .= "translate.txt\n";
}
# The parameter PainterData in the params file, is used to mark to the tasks server to 
# draw with HaploPainter
$painter_parameters.= "PainterData\n";
# The parameter MadelineData in the params file, is used to mark to the tasks server to 
# draw with madeline
$painter_parameters.= "MadelineData\n";

open VAR, ">$destDir\/painter_parameters.txt";
print VAR $painter_parameters;
close VAR;

open LIST, ">$destDir\/tarList.txt";
print LIST $tarList;
close LIST;

open DETAILS, ">$destDir\/details.txt";
print DETAILS $tarList;
close DETAILS;
	
system("tar -C $destDir -T $destDir\/tarList.txt -z -h -c -f  $destDir\/Files.tar.gz");
system("tar -C $destDir -z -h -c -f $destDir\/Submission.tar.gz details.txt Files.tar.gz");

my $sshCmd;
if ($mode == 1){
	$sshCmd = "$config->{'SSHCMD'}";
}
else{
	$sshCmd = "$config->{'SSHCMD_NOT_APACHE'}";
}

my @destDirData = split /\//, $destDir;
my $project = $destDirData[$#destDirData - 1];
my $user = $destDirData[$#destDirData - 3];

my $tm = localtime;
my @date = split(/[ \:]+/,$tm);
my $date = join('.',@date);
my $projName = $path[$#path] . '_' . $date."_".$user."_".$project;

my $ret = system("cat $destDir\/Submission.tar.gz | $sshCmd $config->{'REMOTE_USER'}" . "@" . "$config->{'GW'} $config->{'REMOTE_SUBMIT_SCRIPT'} $projName"); #>> $config->{'ERROR_LOG'}");

$ret = $? >> 8;
exit $ret;