#!/usr/bin/perl -w

use FindBin ();
use lib "$FindBin::Bin";
use InputValidation;
use Processing;
use submissionConfig;
use SharedConsts;
use WebServerConfiguration;
use strict;

sub generateError(\%);
sub createMarkersPositionsForDat( $ $ );
sub setParameters( $ $ );
sub reportErrorAndExit ( $ $ \% );

use constant EX_LOCK => 2;

my $config = new submissionConfig;

#Create a hash of parameters from the parameters file
my $runDir = $ARGV[0];
my $jobPath = $ARGV[1];
my %parametersHash = setParameters($runDir, $jobPath);

#Define the run mode of the analysis
my $runMode = "SingleLocus.TwoPoint";
if (defined($parametersHash{"Analysis"})) {
	$runMode	= "SingleLocus.$parametersHash{Analysis}";
	if (defined($parametersHash{"SegregationType"})) {$runMode = $parametersHash{"SegregationType"}.".".$parametersHash{"Analysis"};}
}

#define the input type of the analysis
my $inputType = "range";
if (defined($parametersHash{"MarkersList"})) {$inputType = "list";}

my $amount = &getMarkersAmount($parametersHash{"datFile"});
my $startMarker = 1;
my $endMarker = $amount;
my $markersList = undef;
if ($inputType eq "range") {
    $startMarker = $parametersHash{"FromMarker"} if (defined($parametersHash{"FromMarker"}));
    $endMarker = $parametersHash{"ToMarker"} if (defined($parametersHash{"ToMarker"}));
    if ($endMarker > $amount) {$endMarker = $amount;}
    $markersList = join ',', ($startMarker .. $endMarker);
}
elsif ($inputType eq "list") {
    $markersList	= 	$parametersHash{"MarkersList"} if (defined($parametersHash{"MarkersList"}));
    $markersList	= 	&validateAndFixList($markersList, &getMarkersAmount($parametersHash{"datFile"}));
}


my @dir		= split(/[\/]/, $parametersHash{"pedFile"});
$dir[$#dir] = $parametersHash{chrom} . ".map";
my $mapFile 	= join('/', @dir);
my $projDir = "$config->{'FULL_DIR'}/var/tmp/$parametersHash{projName}"; 

system("mkdir $config->{'FULL_DIR'}/Submissions/$runDir/$parametersHash{projName}");
system("chmod 777 $config->{'FULL_DIR'}/Submissions/$runDir/$parametersHash{projName}");
symlink ("$config->{'FULL_DIR'}/Submissions/$runDir/$parametersHash{projName}", "$projDir");

system("cp $config->{'FULL_DIR'}/$parametersHash{pedFile} $projDir/pedfile.dat");
system("cp $config->{'FULL_DIR'}/$parametersHash{datFile} $projDir/datafile.dat");
if (-e "$config->{'FULL_DIR'}/$mapFile") {system("cp $config->{'FULL_DIR'}/$mapFile $projDir/mapfile.dat");}
else {system("touch $projDir/mapfile.dat");}

# in case of haplotyping/HMM and a simplified format provided (translate file exists) - copy the translate file as well.
# if ($runMode =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/ || $runMode =~ /MultiPointHMM/) {
if ($runMode =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/ || $runMode =~ /$SharedConsts::ANALYSIS_NAME_HMM/) {
	my @rawDir = split(/[\/]/, $parametersHash{resultsDir});
	$rawDir[$#rawDir] = ".pedigree_translate_1.txt";
	$rawDir[$#rawDir-1] ="raw";
	my $translateFile = join('/', @rawDir);
	if (-e "$config->{'FULL_DIR'}/$translateFile") {system("cp $config->{'FULL_DIR'}/$translateFile $projDir/");}
}

#Omer 31/7/2011 - Copy "Morgan lm _markers" Parameters files if it exists
if (-e "$config->{'FULL_DIR'}/Submissions/$runDir/$SharedConsts::MORGAN_MORE_PARAMS_FILENAME") {
    system("cp $config->{'FULL_DIR'}/Submissions/$runDir/$SharedConsts::MORGAN_MORE_PARAMS_FILENAME $config->{'FULL_DIR'}/Submissions/$runDir/$parametersHash{projName}/");
}

&Processing::setLogFiles("$projDir/Log.");

#########################################################################
#Create the variables hash
my %variables = %{&InputValidation::initVariables($runMode)};
my $tm = localtime;
my @date = split(/[ ]+/,$tm);
my $pName = $parametersHash{chrom};
if (-e "$config->{'FULL_DIR'}/Submissions/$runDir/clustering.flag") {$variables{method} = "cluster";}
$variables{currentDay} = $date[2].'.'.$date[1];
$variables{PROJ} = $pName;
$variables{program} = $runMode;
$variables{root} = &getRootTask($runDir);
$variables{rootName} = &getRootFileName($runDir, $pName);
$variables{mail} = $parametersHash{"mail"};
$variables{newDir} = $projDir;
$variables{datafile} = "datafile.dat";
$variables{pedfile} = "pedfile.dat";
$variables{OUTPUT_FILE} = $variables{PROJ} . '.' . "output.html";
$variables{sexLinked} = 0;
if ($parametersHash{sexLinked}) {$variables{sexLinked} = 1;}
$variables{mode} = "rec";
$variables{mutantFreq} = $parametersHash{sick};
my @penetrances = ();
push @penetrances, $parametersHash{HH};
push @penetrances, $parametersHash{HD};
if ($parametersHash{HD} != "0") {$variables{mode} = "dom";}
push @penetrances, $parametersHash{DD};
$variables{penetrances} = [ @penetrances ];
$variables{chromosomes} = [ (1) ];
$variables{numMarkers} = &getMarkersAmount($parametersHash{"datFile"});

my $DBFile = "$config->{'FULL_DIR'}/".$parametersHash{"resultsDir"}."/../../raw/.final_.sortedDB.snp";
if ( $variables{ program } !~ /$SharedConsts::ANALYSIS_NAME_SEGREGATION/ ) {
	$variables{positions} = [ createMarkersPositionsForDat("$projDir/datafile.dat", $DBFile) ];
}

my @names = (1 .. $variables{numMarkers});
my @usedNames = @names;
if (defined($markersList)) {@usedNames = split(/[,]/,$markersList);}

$variables{markersNames} = [ @names ];
$variables{usedMarkersId} = [ @usedNames ];
$variables{usedMarkersIdout} = [ @usedNames ];
$variables{usedMarkersNames} = [ @usedNames ];
$variables{ part } = "BrowseButton";
$variables{ taskFinishFlag } = $config->{'TASK_FINISH_FLAG'};
#########################################################################

#Get the pedigree numbers
my $pedFileName = "$projDir/pedfile.dat";
my $familiesNumbersStr = `awk '{ print \$1 }' $pedFileName | sort | uniq  | grep -v \"\^\$\"`;
my @pedNames = split /\s+/,$familiesNumbersStr;
$variables{pednames} = [ @pedNames ];

#Get the range parameters
$variables{from}	= $startMarker 	if (defined($startMarker));
$variables{to}		= $endMarker  	if (defined($endMarker));
$variables{jump}	= $parametersHash{"WindowSize"} if (defined($parametersHash{"WindowSize"}));

#Add extra parameters (e.g. lm_markers and gl_auto parameters)
my @extraParameters = ("MaxJobsPerTask", "MaxTimePerJob", "MinTimePerJob", "NumDupsPerJob", "WindowTime",
				"BurnInPercentage", "SeqImpRealizationsPercentage", "MinNumOfIterations", "MaxNumOfIterations");
foreach my $keyName (@extraParameters) {
	$variables{$keyName} = $parametersHash{$keyName} if (defined($parametersHash{$keyName}));
}

symlink ("$projDir/pedfile.dat" , "$projDir/$variables{ PROJ }.input.$variables{ currentDay }.txt");

if ( $variables{ program } !~ /$SharedConsts::ANALYSIS_NAME_SEGREGATION/ ) {
	%variables = %{ &JavaExecutor::commentFiles( "$variables{ newDir }\/$variables{ datafile }", "$variables{ newDir }\/$variables{ pedfile }" , \%variables )};
}

if ( $Processing::NO !~ $variables{ errorExist } )  {$variables{ errorExist } = $Processing::INPUT;}
if (&InputValidation::preparePointInput( \%variables ) !~ $Processing::SUCCESS) {&generateError( \%variables);}

open LIST, ">$variables{ newDir }\/$Configuration::PARAMETERS_FILE";
	print LIST "modefile.txt\n";
	print LIST "$variables{ datafile }\n";
	print LIST "$variables{ pedfile }\n";
	print LIST "mapfile.dat\n";	
close LIST;
	
my $ret = $Processing::ERROR;

# project dir should be an absolute path from /var:
# 
my @path=split '/', $variables{ newDir };

my $params = "";
$params = $params." $Configuration::PARAMETERS_FILE";
$params = $params." $projDir";
$params = $params." $variables{ mail }";
$params = $params." $variables{ PROJ }";
$params = $params." $variables{ OUTPUT_FILE }";

# dvir 2011/08/23 I added those files in order that the post & pre scripts
# will be able to use code which is in the perl bin dir of superlink snp
my $lib = "'./Processing.pm ./WebServerEnvironment.pm ./WebServerConfiguration.pm'";

#Omer 1/9/2011 - TwoPoint computation is now a specific case of MultiPoint computations (with a window of size 1)
if (($variables{program} =~ /MultiPoint/ && $variables{program} !~ /$SharedConsts::ANALYSIS_NAME_HMM/) || $variables{program} =~ /TwoPoint/) {
	system("./run_condor.pl ./prepareMultiPointFiles.pl ./postSnpMultiPoint.pl $lib ./Texts $params >$projDir/condor.submission.log");
#} elsif ($variables{ program } =~ /MultiPointHMM/ || $variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/ || $variables{ program } =~ /Segregation/ 
} elsif ($variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HMM/ || $variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_HAPLOTYPING/ || $variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_SEGREGATION/ 
			|| $variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_LM_MARKERS/ || 
			$variables{ program } =~ /$SharedConsts::ANALYSIS_NAME_GL_AUTO/) {
	system("perl ./send_jobs.pl -dir $projDir >$projDir/submission.log");
} else { &reportErrorAndExit ( $runDir, \$config, \%parametersHash ); }

my $retV = $? >> 8;
#system("echo \"pre.pl got: $retV for: $pName\" >> /tmp/log.log");
# deal with ERROR from Marik's script ir 0!=$ret
# return values:
# 0 - success
# $SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS - highload
# 1   - general error 
# 13 - no login
if ($retV == $SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS) {
	$variables{ errorExist } = $Processing::INPUT;
	$variables{ errorMsg } = "Failed Sending to CONDOR. Fatal System Error. Try later, please";
	submissionConfig::Log($runDir, "got an error sending task, value returned: $retV / $?", "$config->{'FULL_DIR'}/Submissions/$runDir/log.log");

	exit($SharedConsts::ACCEPT_SUBMISSION_SCRIPT_TOO_MANY_SUBMITTED_TASKS);
}

if ($retV != 0) {
	submissionConfig::Log($runDir, "got error sending task, value returned: $retV / $?", "$config->{'FULL_DIR'}/Submissions/$runDir/log.log");
	submissionConfig::Log($runDir, "Superlink failed ($retV), won't try $runDir - $parametersHash{projName} again", "$config->{'FULL_DIR'}/Submissions/$runDir/log.log");
	&reportErrorAndExit ( $runDir, \$config, \%parametersHash );
}

# success
# notifySuccess
my $link = "/var/tmp/$parametersHash{projName}";

open(RESULTS_LOCK, "> $config->{'FULL_DIR'}/$parametersHash{resultsDir}/.resultsLock");
flock (RESULTS_LOCK, EX_LOCK);
system("echo \"$parametersHash{chrom} $link\" >> $config->{'FULL_DIR'}/$parametersHash{resultsDir}/.results");		# show result to user
close(RESULTS_LOCK);

exit(0);










sub generateError( \% ) {
	my %variables = %{ $_[0] };
	&Processing::debug_print("generateError, error=($variables{ errorMsg }), errorType = ($variables{ errorExist })" );
	print &Notification::printWindowHeader() if ( $Processing::NO == $Processing::DEBUG );
	
	if ( ( $Processing::NO eq $variables{ errorExist } ) && ( -s "$Processing::ERROR_FILE" ) )
	{
		&Processing::debug_print("generateError, No error left, but file exist" );
		open ERR, "$Processing::ERROR_FILE";
		my @errorFileContent = <ERR>;
		$variables{ errorExist } = shift @errorFileContent;
		$variables{ errorMsg } = join "\n", @errorFileContent;
		close ERR;
	}
	elsif ( $Processing::NO eq $variables{ errorExist } )
	{
		&Processing::debug_print("generateError, No error left, AND FILE ($Processing::ERROR_FILE) DOES NOT EXIST!!!" );
		exit(9);
		$variables{ errorExist } =~ $Processing::SYSTEM;
		$variables{ errorMsg } = "UNKNOWN PROBLEM";
	}
	
	&Processing::debug_print("generateError, now type error=($variables{ errorExist })" );
	if ( $variables{ errorExist } =~ /$Processing::INPUT/ )
	{
		print "<h1>ERROR!!!</h1>";
		print $variables{ errorMsg };
	}
	else	# if ( $variables{ errorExist } =~ /$Processing::SYSTEM/ )
	{
		my $message = "<h1>FATAL ERROR</h1>";
		open ERR, "$Configuration::HOME_FULL_DIR\/Submissions\/.submissions.error";
		my @errMsg = <ERR>;
		close ERR;
		$message = join "\n", @errMsg if ( $#errMsg >= 0);		
		
		print $message;
		
		open ERR, ">$Processing::ERROR_FILE";
		print ERR $variables{ errorMsg };
		close ERR;
		&Notification::notifyAdmin( $variables{ newDir }, $variables{ PROJ }, $Processing::ERROR_FILE );
	}
	&Notification::finishRun($variables{ newDir }) if ( $variables{ newDir } !~ /Unknown/);

	exit($Processing::ERROR);
}

sub createMarkersPositionsForDat( $ $ ) {
	my $datFile = $_[0];
	my $DBfile = $_[1];
	unless (-e "$datFile") {return -1;}
	unless (-e "$DBfile") {return ();}
	
	my @markersLines = `grep "3.*#.*#" $projDir/datafile.dat`;			# used to get the markers' names
	my @markerToPosition = ();
	
	open (DB, "$DBfile");
	my @DB = <DB>;
	close(DB);
	my $i = 0;
	my @item = split(/[\t]/, $DB[$i]);
	
	for my $line (@markersLines) {		
		$line =~ /#(.*)#/;
		my @markersInLine = split(/[ ]/, $1);
		
		for my $marker (@markersInLine) {
			if ($marker =~ /(.*)\([0-9]+\)/) {$marker = $1;}			
			
			while ($marker ne $item[0]){			# assuming that the markers on the snp and dat apear on the same order
				$i++;
				@item = split(/[\t]/, $DB[$i]);				
			}
			if ($marker eq $item[0]) {push (@markerToPosition, "$marker : ".$item[2]);}			
		}
	}
	
	return @markerToPosition;
}


sub getRootTask( $ ) {
	my $task = $_[0];
	
	open(INF,"$config->{'FULL_DIR'}/Submissions/$task/submissionInformation");
	my @inf = <INF>;
	close(INF);
	
	my $dir = $inf[1];
	chomp($dir);
	open(PROCESS, "$config->{'FULL_DIR'}/$dir/../Experiment1/.processID.txt");
	my @proc = <PROCESS>;
	close(PROCESS);
	
	my $process = $proc[0];
	chomp($process);
	return $process;
}


sub getRootFileName( $ $ ) {
	my $task = $_[0];
	my $currName = $_[1];
	
	open(INF,"$config->{'FULL_DIR'}/Submissions/$task/submissionInformation");
	my @inf = <INF>;
	close(INF);
	
	my $dir = $inf[1];
	chomp($dir);
	open(CONTENT, "$config->{'FULL_DIR'}/$dir/../Experiment1/.contentInfo");
	my @content = <CONTENT>;
	close(CONTENT);
	
	my $name = 0;
	for my $line (@content) {
		chomp($line);
		$line =~ s/\[//g;
		$line =~ s/\]//g;
		if ($currName =~ /.*$line.*/) {
			$name = $line;
			last;
		}
	}

	return $name;
}

sub setParameters( $ $ ) {

	#Define job details
	my $runDir = $_[0];
	my $jobPathStr = $_[1];
	my @jobPath = split(/\//,$jobPathStr);
	
	#Read the information file into a list
	open(INF,"$config->{'FULL_DIR'}/Submissions/$runDir/submissionInformation");
	my @inf = ();
	while (my $currLine = <INF>) {
		chomp($currLine);
		push @inf, $currLine;
	}	
	close(INF);
	
	#Create the parameters Hash
	my %parametersHash = ();
	
	#The first lines in the information file always have the same meaning		
	#### dvir 2011/10/23
	$parametersHash{"mail"} = &submissionConfig::getMailOfUserBySubmissionDir($runDir);
	#$parametersHash{"mail"} =  $inf[0];
	####
	$parametersHash{"resultsDir"} = $inf[1];
	$parametersHash{"sexLinked"} = $inf[2];
	$parametersHash{"healthy"} = $inf[3];
	$parametersHash{"sick"} = $inf[4];
	$parametersHash{"HH"} = $inf[5];
	$parametersHash{"HD"} = $inf[6];
	$parametersHash{"DD"} = $inf[7];
	$parametersHash{"maleH"} = $inf[8];
	$parametersHash{"maleD"} = $inf[9];
	$parametersHash{"chrom"} = $jobPath[$#jobPath];
	$parametersHash{"runDir"} = $runDir;

	#Define the dat and ped files	
	$parametersHash{"pedFile"} = $jobPathStr.".ped";
	$parametersHash{"datFile"} = $jobPathStr.".dat";	
	
	#Define the project name
	my $mail = $parametersHash{"mail"};
	my $chrom = $jobPath[$#jobPath];
	my $tm = localtime;
	my @dateList = split(/[ \:]+/,$tm);
	my $date = join('.',@dateList);
	#### dvir 2011/10/23
	my @mailList = split(/[@]/,$mail);
	#$parametersHash{"projName"} = $mailList[0] . '.' . $chrom . '.' . $runDir . '.' . $date;
	$parametersHash{"projName"} = &submissionConfig::getChromSubDirInitial_AtSubmissionDir_BySubmissionDir($runDir, $chrom, $mail). $date;
	####
	#Iterate over the other lines of the inf file and add key/value pairs
	for my $line (@inf) {
		chomp($line);
		my @lineList = split(/[ \t]/, $line);
		if (scalar(@lineList) != 2) {next;}
		my $key = $lineList[0];
		my $value = $lineList[1];
		$parametersHash{$key} = $value;
	}
	
	return %parametersHash;
}


sub getMarkersAmount( $ ) {
	my $datfile = $config->{'FULL_DIR'}."/".$_[0];
	my $amount = `head -1 $datfile | cut -d\" \" -f1`;
	$amount--;
	return $amount;
}

sub validateAndFixList( $ $ ) {
	my $listStr = $_[0];
	my $max = $_[1];
	
	my @list = split(/[,]/, $listStr);
	my @newList = ();
	foreach my $num (@list) {
		if (($num >= 0) and ($num <= $max)) {push(@newList,$num);}
	}
	
	return join(',', @newList);
}

sub reportErrorAndExit ( $ $ \% ) {
	my $runDir = $_[0];
	my $configRef = $_[1];
	my %parametersHash = %{$_[2]};
	
	my $message = "General error while running this file on Superlink Online. Please contact an administrator.";
	$message =~ s/ /~~/g;
	my $chr_name = $parametersHash{chrom};
	$chr_name =~ s/resultFile\.//g;
	submissionConfig::Log($runDir, "perl reportResults.pl error -r $runDir -chr $chr_name -parameter analysis -msg $message", "$$configRef->{'FULL_DIR'}/Submissions/$runDir/log.log");
	system("perl reportResults.pl error -r $runDir -chr $chr_name -parameter analysis -msg $message");
	exit (1);
}