#!/usr/bin/perl -w

use FindBin ();
use lib "$FindBin::Bin";
use submissionConfig;

use constant EX_LOCK => 2;
use constant NB_LOCK => 4;
use constant LIST => "TasksWaitingList";

$config = new submissionConfig;
#$cgibinDir = $config->{'FULL_DIR'}."/cgi-bin"; # dvir 2011/08/03, unused


$lock = "$config->{'QUEUE_LOCK'}";
$queueFile = LIST;
$queue = "$config->{'QUEUE_DIR'}/$queueFile";
$logFile = "$config->{'FULL_DIR'}/Submissions/log/".$queueFile.".log";

chdir("$config->{'FULL_DIR'}/cgi-bin");

open(LOCK, ">".$lock."_".$queueFile) || die();
$gotLock = flock (LOCK, EX_LOCK | NB_LOCK);		# wait until you lock is free (non-blocking)
if (!$gotLock){									# already locked
	close(LOCK);
	exit(0);
}

print LOCK "Locked by tasksMonitor.\n";

# dvir 2011/12/05 : I'm preforming sort & uniq in order to avoid cases when a task appears twice or more at tasks Monitor file
# can happen when preforming reportResults.pl for each chromosome.
open(TASKS, "cat $queue|sort|uniq|");
@tasks = <TASKS>;
close(TASKS);
system("echo -n '' > ".$queue);		# clean the list
print LOCK "tasksMonitor released lock.\n";	#just to indicate lock release
close(LOCK);

my $ret = 0;
my @notFinished;
foreach $taskLine (@tasks){
	my @taskArr = split ' ', $taskLine;
	$errorInSubmissionDir = $taskArr[0];
	$task = join ' ', @taskArr[1..$#taskArr];
	
	chomp($task);
	if (!(-d "$config->{'FULL_DIR'}/Submissions/$task") or ($task eq "")) { next; }
	if (&isFinished($task)){
		&queueLog($task, "running GlobalPost on $task.", $logFile);
		&runPostGlobal($task);
		#&collectTaskResults($task);
	}
	else{
		if (!(-e "$config->{'FULL_DIR'}/Submissions/$task/error")){
			push (@notFinished, $errorInSubmissionDir.' '.$task."\n");
		}
	}
}

open(LOCK, ">".$lock."_".$queueFile) || die();
flock (LOCK, EX_LOCK);		# wait until lock is free (blocking)

print LOCK "Locked by tasksMonitor.\n";

open(TASKS, ">>".$queue);		# append list
print TASKS @notFinished;
close(TASKS);

print LOCK "tasksMonitor released lock.\n";	#just to indicate lock release
close(LOCK);									# release lock

exit(0);

sub queueLog ( $ $ $ ) {
	my $task = $_[0];
	my $msg = $_[1];
	my $logName = $_[2];
	
	system("echo \"*********************************************\n LOG: `date`\n task = $task\n $msg \n\" >> $logName");
}

sub isFinished ( $ ) {
	my $task = $_[0];
	my $dir = "$config->{'FULL_DIR'}/Submissions/$task";
	
	open(DETAILS, "$dir/details.txt") || die("Cannot open details file");
	@details = <DETAILS>;
	close(DETAILS);
	
	opendir(TASKDIR, "$dir") || die("Cannot open directory");
	@taskContent = readdir(TASKDIR);
	closedir(TASKDIR);

	foreach $job (@details){
		chomp($job);
		if ($job =~ /(.*)\.dat$/){
			my $finished = 0;
			
			@folders = grep (/$1/, @taskContent);		# getting only the folders of that job
			foreach $folder (@folders){
				if ( ($folder ne ".") and ($folder ne "..") and (-d $dir."/".$folder) ){
					if (-e "$dir/$folder/".$config->{'TASK_FINISH_FLAG'}){
						$finished = 1;					# found that the job was finished
					}
				}
			}
			
			if ($finished == 0){						# if job is not finished yet, return false
				return 0;
			}
		}
	}
	
	return 1;		# found FINISH_FLAG for all jobs on details.txt of that task
	# search for finish flags...
}

sub runPostGlobal( $ ){
	my $task = $_[0];
	
	system("perl GlobalPost.pl $task $config->{'FULL_DIR'}/Submissions/$task");
	# run post global on task
}

sub collectTaskResults( $ ){
	$task = $_[0];
	$dir = "$config->{'FULL_DIR'}/Submissions/$task";
	
	open(INFO, "$dir/submissionInformation") || die("Cannot open info file");
	@details = <INFO>;
	close(INFO);
	
	chomp($details[1]);
	my $expDir = "$config->{'FULL_DIR'}/".$details[1];
	
	open(RESULTS_LOCK, ">$expDir/.resultsLock") || die("Cannot open results lock file");
	flock (RESULTS_LOCK, EX_LOCK);		# wait until lock is free (blocking)
	
	open(RESULTS, "$expDir/.results") || die("Cannot open results file");
	@results = <RESULTS>;
	close(RESULTS);
	
	if (!(-d "$expDir/Results")){
		mkdir "$expDir/Results";
	}
	
	for $i (0..$#results){
		$result = $results[$i];
		chomp ($result);
		@res = split (/[ ]/, $result);
		if ($#res > 1) { next; }
		$name = $res[0];
		$link = $res[1];
		$resultsDir = "$expDir/Results/$name";
		mkdir $resultsDir;
		system("cp $config->{'FULL_DIR'}/$link/*.html $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/*.png $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/custom_tracks.txt $resultsDir");
		
		system("cp $config->{'FULL_DIR'}/$link/debug.txt $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/suspect_areas.txt $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/high_points.txt $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/condor.submission.log $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/Log.debug.txt $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/TASK_LOG.txt $resultsDir");
		system("cp $config->{'FULL_DIR'}/$link/$config->{'TASK_FINISH_FLAG'} $resultsDir");
		
		$newLink = $details[1]."/Results/$name";
		$results[$i] = $name." ".$newLink."\n";
	}

	open(RESULTS, ">$expDir/.results") || die("Cannot open results file");
	print RESULTS @results;
	close(RESULTS);
	
	close(RESULTS_LOCK);
}