<?php

require_once ("exceptions.php");

/*
* checks a dat file.
* in case of a failure, an exception will be thrown.
* if the function finished it's running succesfully - the dat file passed all the tests.
* input:
*        $datFile - the path and name of the dat file to check.
*/
function checkDatFile($datFile) {
    if (!is_file($datFile)) {throw new GeneralException("Dat file ".$datFile." does not exist");}
    if (!($datLinesArray = file($datFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES ))) {
		throw new GeneralException("Cannot open file ".$datFile);
	}	
    $datFileLink = '<a href="http://bioinfo.cs.technion.ac.il/superlink/Locus_File.html" target="_blank">dat file format document</a>';
    $newDatLines = Array();
   
    #Check line 1 (should be: #markers 0 (0 or 1) 8 0)
	if (!isSet($datLinesArray[0])) {throw new InputException("File is empty");}
    $lineArray = explode("<<", $datLinesArray[0]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces
    $matches = Array();	
    if (!preg_match("/^(\d+) (\d+) (\d+) (\d+) (\d+)$/", $str, $matches)) {
        $errorMessage = "First line is incorrect. Please refer to the ".$datFileLink." for more information.";
        throw new InputException($errorMessage);
    }
    $numOfMarkers = intval($matches[1])-1;
    if ($numOfMarkers < 1) {throw new InputException("First field of first line (#markers) must be greater than 2");}
    if ($matches[2] != '0') {throw new InputException("Second field of first line must be 0");}
    if ($matches[3] != '0' and $matches[2] != '1') {
		throw new InputException("Third field of first line (sex-linked field) must be 0 or 1");
	}
    if ($matches[4] != '8') {throw new InputException("Fourth field of first line must be 8");}
    if ($matches[5] != '0') {
		throw new InputException("Fifth field of first line must be 0. Superlink-Online SNP currently does not support two-loci analysis.");
	}
    $newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
   
    #Check line 2 (should be: 0 0 0 0)
    if (!isSet($datLinesArray[1])) {throw new InputException("Line 2 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[1]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces
    if (!preg_match("/^0 0 0 0$/", $str, $matches)) {
		throw new InputException("Second line of dat file must contain four zeros");
	}
    $newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
   
    #Check line 3 (should be: "1 2 3 .. numofmarkers")
    if (!isSet($datLinesArray[2])) {throw new InputException("Line 3 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[2]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces
    $expectedThirdLine = implode(" ",range(1, $numOfMarkers + 1));
    if (!preg_match("/^".$expectedThirdLine."$/", $str, $matches)) {
        throw new InputException("Line 3 of dat file must contain the following text: ".$expectedThirdLine);
    }
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	
    #Check line 4 (should be: "1 2")
    if (!isSet($datLinesArray[3])) {throw new InputException("Line 4 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[3]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces    
    if (!preg_match("/^1 2$/", $str, $matches)) {
        throw new InputException("Line 4 of dat file must contain the following text: 1 2");
    }
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	
    #Check line 5 (should be: "freq 1-freq")
    if (!isSet($datLinesArray[4])) {throw new InputException("Line 5 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[4]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces    
    if (!preg_match("/^(\S+) (\S+)$/", $str, $matches)
	||  !is_numeric($matches[1]) || !is_numeric($matches[2])
	||	$matches[1] < 0 || $matches[1] > 1 ||	$matches[2] < 0 || $matches[2] > 1 
	||	($matches[1]+$matches[2] != 1)) {
        throw new InputException("Line 5 of dat file must contain two non-negative numbers summing up to one");
    }
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	
    #Check line 6 (should be: "1")
    if (!isSet($datLinesArray[5])) {throw new InputException("Line 6 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[5]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces    
    if (!preg_match("/^1$/", $str, $matches)) {
        throw new InputException("Line 6 of dat file must be \"1\" (Superlink-Online SNP only supports one liability class)");
    }
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	
    #Check line 7 (should be p(A|hh), p(A|ha), p(A|aa))	
    if (!isSet($datLinesArray[6])) {throw new InputException("Line 7 is missing from the file");}    
    $lineArray = explode("<<", $datLinesArray[6]);
    $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces    
    if (!preg_match("/^(\S+) (\S+) (\S+)$/", $str, $matches)
	||  !is_numeric($matches[1]) || !is_numeric($matches[2]) || !is_numeric($matches[3])
	||	$matches[1] < 0 || $matches[1] > 1 ||	$matches[2] < 0 || $matches[2] > 1 ||	$matches[3] < 0 || $matches[3] > 1) {
        throw new InputException("Line 7 of dat file must contain three numbers between 0 and 1");
    }
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	
	//start checking each marker line
	$currLine = 7;	
	$markerNames = Array();
	for ($markerNum = 1; $markerNum <= $numOfMarkers; $markerNum++) {
	
		#Check marker first line
		if (!isSet($datLinesArray[$currLine])) {throw new InputException("Data for marker number ".$markerNum." is missing");}
		$lineArray = explode("<<", $datLinesArray[$currLine]);
		$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces		
		if (!preg_match("/^3 (\d+) \#([\w\-]+)\#$/", $str, $matches)) {
			$nextLine = $currLine+1;
			$errorMessage = "Line ".$nextLine." of dat file does not contain valid marker data. Please verify that the marker has a valid name (only letters, digits ,underscores and dashes, surrounded by #)";
			throw new InputException($errorMessage);
		}
		$markerName = $matches[2];
		if (in_array($markerName, $markerNames)) {
			throw new InputException("There are two markers with the name ".$markerName);
		}
		$markerNames[] = $markerName;
		$numAlleles = $matches[1];
		if ($numAlleles < 0) {throw new InputException("The number of alleles for marker ".$markerNum." must be positive");}
		$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
		
		#Check marker second line
		$currLine ++;
		if (!isSet($datLinesArray[$currLine])) {throw new InputException("Data for marker number ".$markerNum." is missing");}
		$lineArray = explode("<<", $datLinesArray[$currLine]);
		$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces
		$expectedLineRegExp = trim(str_repeat("(\S+) ", $numAlleles));
		if (!preg_match("/^".$expectedLineRegExp."$/", $str, $matches)) {
			throw new InputException("Invalid allele frquency lines for marker number ".$markerNum.". Please verify that this marker has ".$numAlleles." frequencies");
		}
		#Verify that the marker allele frequencies sum up to one
		$freqSum = 0;
		for ($freqNum = 1; $freqNum <= $numAlleles; $freqNum++) {
			if (!is_numeric($matches[$freqNum])) {
				throw new InputException("Invalid allele frequencies for marker number ".$markerNum);
			}
			if ($matches[$freqNum] <= 0 || $matches[$freqNum] > 1) {
				throw new InputException("Allele frequencies for marker number ".$markerNum." must be between 0 and 1");
			}
			$freqSum += $matches[$freqNum];
		}
		if (abs($freqSum - 1) > 0.000000001) {
			$errorMessage = "Allele frequencies for marker number ".$markerNum." must sum up to one (current sum is ".$freqSum.")";
			throw new InputException($errorMessage);
		}
		$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
		$currLine ++;
	}
	
	#Check fourth line from last (must be 0 0)
	if (!isSet($datLinesArray[$currLine])) {throw new InputException("File is missing last four lines");}
	$lineArray = explode("<<", $datLinesArray[$currLine]);
	$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces	
	if (!preg_match("/^0 0$/", $str, $matches)) {
		throw new InputException("Sex-difference and interference line (line 4 from the bottom) must contain 0 0. Superlink-Online SNP currently does not support other modes");
	}
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	$currLine ++;
	
	#Check third line from last (recombination frequencies line)
	$nextLine = $currLine+1;
	if (!isSet($datLinesArray[$currLine])) {throw new InputException("File is missing last three lines");}
	$lineArray = explode("<<", $datLinesArray[$currLine]);
	$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces
	$expectedLineRegExp = trim(str_repeat("(\S+) ", $numOfMarkers));
	if (!preg_match("/^".$expectedLineRegExp."$/", $str, $matches)) {
		throw new InputException("Line ".$nextLine." must contain exactly ".$numOfMarkers." recombination frequencies");
	}	
	for ($recNum = 1; $recNum <= $numOfMarkers; $recNum++) {
		if (!is_numeric($matches[$recNum])) {			
			throw new InputException("Recombination frequency number ".$recNum." in line ".$nextLine." is invalid");
		}
		if ($matches[$recNum] < 0 || $matches[$recNum] > 0.5) {
			throw new InputException("Recombination frequency number ".$recNum." in line ".$nextLine." must be between 0 and 0.5");
		}
	}
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";	
	$currLine ++;
	
	#Check second line from last (must be 1)
	if (!isSet($datLinesArray[$currLine])) {throw new InputException("File is missing last two lines");}
	$lineArray = explode("<<", $datLinesArray[$currLine]);
	$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces	
	if (!preg_match("/^1$/", $str, $matches)) {throw new InputException("Second to last line must contain the number 1");}
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	$currLine ++;
	
	#Check last line (must be -n x x x)
	if (!isSet($datLinesArray[$currLine])) {throw new InputException("File is missing last line");}
	$lineArray = explode("<<", $datLinesArray[$currLine]);
	$str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces	
	if (!preg_match("/^-n (\d+) (\d+) (\d+)$/", $str, $matches)) {
		throw new InputException("Last dat line must be -n <num_iter> start_pos end_pos");
	}
	$newDatLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
	$currLine ++;
	
	#Check that there are no more lines
	if (isSet($datLinesArray[$currLine])) {throw new InputException("file has too many lines");}
	
	#Write the new lines to the dat file
	$datFileHandler = fopen($datFile, "w");	
	foreach ($newDatLines as &$line) {fwrite($datFileHandler, $line);}
	fclose($datFileHandler);
}


/*
 * checks a ped file.
 * in case of a failure, an exception will be thrown.
 * if the function finished it's running succesfully - the ped file passed all the tests.
 * input:
 *    $pedFile - the path and name of the ped file to check
 *    $containAlelleData - true if the file must contain alellee data, false otherwise (for example,
 *               in case of a snp path, ped file contains only data on the pedigrees).
*/
function checkPedFile($pedFile, $containAlelleData = true) {
    if (!is_file($pedFile)) {throw new GeneralException("Ped file ".$pedFile." does not exist");}
	
	#Sort the ped file according to pedigree order
	$tmpPedName = $pedFile."_tmp";
	system("sort +0n -2 -s ".$pedFile." > ".$tmpPedName);
    if (!($pedLinesArray = file($tmpPedName, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES ))) {throw new GeneralException("Cannot open file ".$pedFile);}
	unlink($tmpPedName);
    $pedFileLink = '<a href="http://bioinfo.cs.technion.ac.il/superlink/Pedigree_File.html" target="_blank">ped file format document</a>';
    $newPedLines = Array();

    #Find the name of the matching datfile
    $pathParts = pathinfo($pedFile);
    $datFile = $pathParts['dirname']."/".$pathParts['filename'].".dat";
    
    #Verify that all lines in the ped file have the correct number of columns and are valid, etc.
    $numOfRequiredOfColumns = 10;
    if ($containAlelleData) {
		#Find the number of markers written in the dat file
		$markersAmount = explode(" ",exec("head -1 $datFile"));
		$markersAmount = $markersAmount[0] - 1;		
		$numOfRequiredOfColumns = 2*$markersAmount + 10;		
	}
    $lineCount = 1;
    foreach ($pedLinesArray as &$line) {
        $lineArray = explode("<<", $line);
        $str = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces		
        $numColumns = count(explode(" ",$str));
        if ($numColumns != $numOfRequiredOfColumns) {
            throw new InputException("Line ".$lineCount." should have exactly ".$numOfRequiredOfColumns." columns");
        }
        $strArray = explode(" ", $str);
        $curColumn = 1;
        foreach ($strArray as &$curVal) {
            if (!is_numeric($curVal)) {throw new InputException("Non-numeric value in line ".$lineCount.", column ".$curColumn);}
            if ($curColumn == 8 and $curVal != "0" and $curVal != "1" and $curVal != "2") {
                throw new InputException("Sex information for person in line ".$lineCount. " (column 8) must be either 0,1 or 2");
            }
            if ($curColumn == 9 and $curVal != "0") {throw new InputException("The data in line ".$lineCount. ", column 9 must be 0");}
            if ($curColumn == 10 and $curVal != "0" and $curVal != "1" and $curVal != "2") {
                throw new InputException("Phonotypic information for person in line ".$lineCount. " (column 10) must be either 0,1 or 2");
            }
            $curColumn++;
        }
        
        $newPedLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
        $lineCount++;
    }
    
	#Write the new lines to the ped file
	$pedFileHandler = fopen($pedFile, "w");	
	foreach ($newPedLines as &$line) {fwrite($pedFileHandler, $line);}
	fclose($pedFileHandler);    

    #Create an array of strings representing the pedigree in simplified format    
    exec("cut -d \" \" -f 1 $pedFile | sort | uniq | grep -v \"\^\$\"", $pedFamilies);
    $simplifiedPedigreeTextArray = Array();
    foreach ($pedFamilies as &$pedName) {
        unset($familyLines);
        exec("grep \"^".$pedName."\" ".$pedFile." | cut -d\" \" -f2,3,4,8,10", $familyLines);        
        $simplifiedPedigreeTextArray[] = "pedigree: ".$pedName;        
        foreach ($familyLines as &$familyLine) {$simplifiedPedigreeTextArray[] = $familyLine;}
    }

    #Send the simplified pedigree to the perl script that checks its topology    
    $testProjectName = $_SESSION["newProjectName"]."_ped_check";
    $tempDir = "/tmp/".$testProjectName;
    exec("rm -rf ".$tempDir);
    mkdir($tempDir, 0775, true);
    $simplifiedPedigreeText = implode("\\n", $simplifiedPedigreeTextArray)."\\n";
    $simplifiedFile = $tempDir."/".$testProjectName;
    exec("echo -e \"".$simplifiedPedigreeText."\" > ".$simplifiedFile);    
    exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/convertSimplifiedToSL.pl $simplifiedFile ".$tempDir);
    
    #If the pedigree topology is wrong, throw an exception
    if (is_file($tempDir."/".SIMPLIFIED_CONVERT_ERROR)) {
        $errorMessage = implode("<br>",file($tempDir."/".SIMPLIFIED_CONVERT_ERROR));
        throw new InputException($errorMessage);
    }
    
    #Update columns 1-10 of the ped file according to the script output
    $scriptOutputFile = $tempDir."/sl_input.ped";
    if (!is_file($scriptOutputFile)) {throw new GeneralException("Ped file ".$scriptOutputFile." does not exist");}
    if (!($simplifiedPedLinesArray = file($scriptOutputFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES ))) {
        throw new GeneralException("Cannot open file ".$scriptOutputFile);
    }
    $pedLinesArray = file($pedFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES );
    $numLines = count($pedLinesArray);
    $newPedLines = Array();
    for ($lineNum = 0; $lineNum < $numLines; $lineNum++) {
        $lineArray = explode("<<", $pedLinesArray[$lineNum]);
        $pedStr = trim(preg_replace('/\s+/', ' ', $lineArray[0]));  #Remove extra whitespaces        
        $pedArray = explode(" ", $pedStr);
        $simplifiedPedArray = explode("\t", $simplifiedPedLinesArray[$lineNum]);
        for ($i = 0; $i < 10; $i++) {$pedArray[$i] = $simplifiedPedArray[$i];}
        $str = implode(" ", $pedArray);
        $newPedLines[] = $str.((isSet($lineArray[1])) ? (" <<".$lineArray[1]) : "")."\n";
    }
    
	#Write the new lines to the ped file (again)
	$pedFileHandler = fopen($pedFile, "w");	
	foreach ($newPedLines as &$line) {fwrite($pedFileHandler, $line);}
	fclose($pedFileHandler);
    
    #Verify that no marker has more alleles than stated in the dat file
	if ($containAlelleData) {
		if (!($datLinesArray = file($datFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES ))) {throw new GeneralException("Cannot open file ".$datFile);}
		for ($currMarker = 1; $currMarker <= $markersAmount; $currMarker++) {
			$datLineNumber = 7 + 2*($currMarker-1);
			$pedColumn1 = 11 + 2*($currMarker-1);
			$pedColumn2 = $pedColumn1+1;
			
			$numAllelesInDat = explode(" ",$datLinesArray[$datLineNumber]);        
			$numAllelesInDat = $numAllelesInDat[1];

			unset($pedAllelesArray);
			exec ("cut -d\" \" -f".$pedColumn1.",".$pedColumn2." ".$pedFile, $pedAllelesArray);        
			$personCount = 1;
			foreach ($pedAllelesArray as &$alleles) {            
				$allelesArray = explode(" ", $alleles);
				$allele1 = $allelesArray[0];
				$allele2 = $allelesArray[1];
				if (!is_numeric($allele1) || $allele1 < 0 || !is_numeric($allele2) || $allele2 < 0) {
					throw new InputException("The data for person ".$personCount.", marker ".$currMarker." is invalid");
				}
				if ($allele1 > $numAllelesInDat || $allele2 > $numAllelesInDat) {
					throw new InputException("The data for person ".$personCount.", marker ".$currMarker." is illegal - according to the dat file
					there are only ".$numAllelesInDat." alleles for this marker");
				}
                
                if (($allele1 == 0 && $allele2 != 0) || ($allele2 == 0 && $allele1 != 0)) {
                    throw new InputException("The data for person ".$personCount.", marker ".$currMarker." is invalid. Either none or both alleles can be unknown.");
                }
				$personCount++;
			}
		}
	}
}

/*
 * checks a snp file.
 * in case of a failure, an exception will be thrown.
 * if the function finished it's running succesfully - the snp file passed all the tests.
 * input:
 *    $pedFile - the path and name of the ped file to check
 */
function checkSnpFile($snpFile) {
	#For now there is nothing to check here...
	
}

?>