<?php
	/* a group of functions for creating superlink ped files. */
	require_once("snpManipulation.php");
	require_once("config.php");
	require_once("debug.php");
	require_once("formulas.php");
	require_once("exceptions.php");

	function interpretCall($call) {
		$returnCall = "";
		if (preg_match(MATCHES_FOR_00 ,$call)) {$returnCall = "0 0";}		
		else {
			//$calls = split("", $call);
			// treat the call string as 2 letters and match a number to each letter
			for ($i = 0; $i < 2; $i += 1) {
				if (preg_match(MATCHES_FOR_1 ,$call[$i])) {$returnCall = $returnCall . "1";}									
				elseif (preg_match(MATCHES_FOR_2 ,$call[$i])) {$returnCall = $returnCall . "2";}				
				else {
					$returnCall = "0 0";
					break;
				} //unknown pattern
				if ($i == 0) {$returnCall = $returnCall . " ";}				
			}
		}
		return $returnCall;
	}
	
	/*
	 * mapping file must be in the directory $dir.
	 * will not overwrite existing ped file unless $overwrite is specified with true.
	 * $origPed - orininal ped file
	 * $snpFileList - an array of the snp files to take data from.
	 * $chromNum - number of the chromosome the ped is for.
	 * $chromColMap - hash of each snpFile in the List to the column number of the chromosome in this file. if no mapping specified default value will be the second column.
	 */
	function createPedBy($dir, $DBfile, $origPed, $destDir, $chromNum, $overwrite = false) {
		$zero = ( ($chromNum != "X") && ($chromNum < 10)) ? "0" : "";
		if ( (!(is_dir($dir))) || ((is_file($destDir."/chrom_".$zero.$chromNum.".ped")) && !($overwrite)) ||
			 (!(is_dir($destDir))) || (!(is_file($dir."/".$origPed))) ){
			log_error("general error on function createPedBy");
			return false;
		}
		if (!($DB = fopen($dir."/".$DBfile, "r"))) {
			log_error("can't open $dir/$DBfile");
			return false;
		}
		$chromExist = false;
		do {
			$line = explode("\t", rtrim(fgets($DB)));
			if (!isset($line[DB_CHROM_COL])) {continue;} //skip emply lines
			if (strcmp($line[DB_CHROM_COL],$chromNum) != 0) {continue;} //if not the needed chromosome
			$chromExist = true;
			for ($person_num = 1; $person_num + DB_POS_COL < sizeof($line) ; $person_num++) {								
				$call = $line[$person_num + DB_POS_COL];				
				$call = interpretCall($call);
				if (!isset($pedNumToCalls[$person_num-1])) {$pedNumToCalls[$person_num-1] = "";}
				$pedNumToCalls[$person_num-1] = $pedNumToCalls[$person_num-1]." ".$call;
			}
		} while (!(feof($DB)));		
		fclose($DB);
		//If the SNP file doesn't contain markers from the needed chromosome, exit
		if (!$chromExist) {return false;}
		$pedPath_n_Name = $destDir."/chrom_".$zero.$chromNum.".ped";
		$originalPed = $dir."/".$origPed;
		return createPed($originalPed, $pedNumToCalls, $pedPath_n_Name);
	}
	
	/*
	 * create .ped file
	 * input:
	 * 		$originalPed - path and name of the original ped file (as generated by the perl convertSimplifiedToSL.pl script
	 *		$alleleData - array of the data on the allele. each person has a entey in the array (1st person in the ped
	 *				file is in $alleleData[0] and so on...). each entry is a string as followed:
	 * 				allele1_marker1 allele2_marker1 allele1_marker2 allele2_marker2 ... allele1_markerN allele2_markerN
	 *		$pedPath_n_Name - the path and the name of the new ped file. NOTE: the old file (if exists) will be override
	 */
	function createPed ($originalPed, &$alleleData, $pedPath_n_Name) {
		if (!is_array($alleleData)) {
			log_error("error in createPed: alleleData is not an array");
			return false;
		}
		if (!($newPed = fopen($pedPath_n_Name, "w"))) {
			log_error("cant create $pedPath_n_Name");
			return false;
		}
		$oldPed = file($originalPed, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		for ($person_num = 0 ; $person_num < sizeof($oldPed) ; $person_num++) {
			$line = $oldPed[$person_num];
			$line = trim(str_replace("\t", " ", $line));	//replace tabs with spaces
			if (strlen($line) == 0) {continue;}		//If the line is empty, skip it
            
            //Omer - keep only the first 10 words of the ped file (the words of the person data without the alleles)
            $lineArray = explode(" ", $line);
            if (count($lineArray) > 10) {
                array_splice($lineArray, 10);
                $line = implode(" ", $lineArray);
            }
            
            //Create the new line and print it to the file
			$newLine = $line." ".$alleleData[$person_num]."\n";	
			fwrite($newPed, $newLine);			
		}
		fclose($newPed);
		return true;
	}
	
	/*
	 * takes a .snp file, and create dat file (for a certain chromozon) out of it.
	 * $parameters is a hash whith the keys: KEY_SEX_LINKED, KEY_FREQUENCY, KEY_PENETRANCE, 
	 */
	function createDatBy($dir, $DBfile, $destDir, $chromNum, $parameters, $overwrite = false) {
		//verify data
		$zero = ( ($chromNum != "X") && ($chromNum < 10)) ? "0" : "";
		if ( (!(is_dir($dir))) || ((is_file($dir."/chrom_".$zero.$chromNum.".dat")) && !($overwrite)) ||
			 (!(is_dir($destDir))) ) {
			log_error("general error on function createDatBy");
			return false;
		}
		if (!($DB = fopen($dir."/".$DBfile, "r"))) {
			log_error("can't open $dir/$DBfile");
			return false;
		}
		$sexLinked = isset($parameters[KEY_SEX_LINKED]) ? ($parameters[KEY_SEX_LINKED] == "1" ? true : false) : false;
		$frequency = 0.01;
		if (isset($parameters[KEY_FREQUENCY])) { //$parameters[KEY_FREQUENCY] is in format "healthy% sick%"
			$freqArr = explode(" ", $parameters[KEY_FREQUENCY]);
			$frequency = $freqArr[1];
		}
		$penetrance = isset($parameters[KEY_PENETRANCE]) ? "0 ".$parameters[KEY_PENETRANCE] : "0 0 0.99";
		$datName = "chrom_".$zero.$chromNum.".dat";
		$newFilePath = $destDir."/".$datName;
		//collecting data on the markers out of the snp file
		$markerIndex = 0;
		$markers = array();
		while (!(feof($DB))) {
			$line = fgets($DB);										//read a line
			if (preg_match("/^[ \t]*[\n]?$/", $line)) {continue;}	//Skip empty lines
			$lineArr = explode("\t", rtrim($line));					//create a list of line values	
			//skip markers that are not on the current chromosome
			if (strcmp($lineArr[DB_CHROM_COL], $chromNum) != 0) {continue;}
			//temporary ignoring empty named markers!!!!!
			if ($lineArr[DB_MARKERNAME_COL] == "") {continue;}
			$markers[$markerIndex] = array();
			$markers[$markerIndex]["name"] = trim($lineArr[DB_MARKERNAME_COL]);
			$markers[$markerIndex]["distance"] = trim($lineArr[DB_POS_COL]);
			$markers[$markerIndex]["distance"] = $markers[$markerIndex]["distance"] / 1000000;
			$markers[$markerIndex]["numOfAlleles"] = 2;
			$markerIndex++;            
		}
		fclose($DB);
		if ($markerIndex == 0) {return false;}
		try {createDatFile($markers, $penetrance, $newFilePath, $frequency, $sexLinked);}
		catch(Exception $e) {return false;}
		return true;
	}
	
	/*
	* create .dat file
	* input:
	*	$markers - array of markers, each cell in the array is an array as followed:
	*		(the indexes of the array are integers from 0 to $markers.length-1).
	* 		$markers[$i]["name"] - 	the name of the i'th marker.
	*		$markers[$i]["distance"] - 	the distance of the i'th marker frome the (i-1)th. $markers[0]["distance"] = 0
	*		$markers[$i]["numOfAlleles"] -	the number of alleles avalible in this marker (over all the data,
	*							not only from one pedigree).
	*	$penetrance - in the format: "num1 num2 num3" when num1 is the chance of beeing sick with no bad genes,
	*						num2 - the cance of beeing sick with 1 bad gene and num3 - the cannce of beeing sick with 2 bad genes.
	*	$datPath_n_Name - the path and the name of the new dat file. NOTE: the old file (if exists) will be override
	*	$deseaseFrequency - the chance to pick a sick person from the entire population (a number between 0 and 1)
	*	$isSexLinxed - true if this is a sex chromoson, false otherwise
	*/
	function createDatFile(&$markers, $penetrance, $datPath_n_Name, $deseaseFrequency, $isSexLinxed=false) {
		// input verification
		if (!isSet($deseaseFrequency) || $deseaseFrequency==null) {
			throw new GeneralException("problem with deseaseFrequency in function createDatFile in datPedCreator.php");
		}
		if (is_NaN($deseaseFrequency) || $deseaseFrequency<0 || $deseaseFrequency>1) {
			throw new InputException("Disease mutant gene frequency: $deseaseFrequency is incorrect");
		}
		if (!is_array($markers)) {
			throw new GeneralException("problem with markers in function createDatFile in datPedCreator.php");
		}
		for ($i=0; $i<sizeOf($markers); $i++) {
			if (!isSet($markers[$i]) || !isSet($markers[$i]["name"]) || !isSet($markers[$i]["distance"]) || !isSet($markers[$i]["numOfAlleles"]) || is_NaN($markers[$i]["numOfAlleles"])) {
				throw new GeneralException("problem with markers array at index $i in function createDatFile in datPedCreator.php");
			} else if (is_NaN($markers[$i]["distance"]) || $markers[$i]["distance"]<0) {
				throw new InputException("The distance: ".$markers[$i]["distance"]." at index $i is incorrect");
			} else if (is_NaN($markers[$i]["numOfAlleles"]) || $markers[$i]["numOfAlleles"]<0) {
				throw new GeneralException("problem with numOfAlleles in markers array at index $i in function createDatFile in datPedCreator.php");
			}
		}
		//open the new file
		if (!($newDat = fopen($datPath_n_Name, "w"))) {throw new GeneralException("can't open file: $datPath_n_Name");}
		//Write line 1 of dat file
		$sexLinked = $isSexLinxed ? "1" : "0";
		fwrite($newDat, ((sizeOf($markers))+1)." 0 ".$sexLinked." 8 0 << num of markers, ignored, sexLinked, superlink program, one/two locus disease\n");
		fwrite($newDat, "0 0 0 0 << ignored\n");	//Write line2 of dat file
		for ($i=1; $i<=sizeOf($markers)+1; $i++) {fwrite($newDat, $i." ");}	//line 3
		fwrite($newDat, "<< markers order\n");		//line 3 part 2
		fwrite($newDat, "1 2 << disease marker\n");	//Line 4
		fwrite($newDat, (1-$deseaseFrequency)." ".$deseaseFrequency." << disease frequency\n");	//line 5
		fwrite($newDat, "1 << number of liability classes\n");	//line 6
		//line 7
		fwrite($newDat, "$penetrance << penetrance classes\n");
		//line 8 (in case of a sex-linked trait only). Currently, we use the same penetrance
		//values for both males and females.
		if ($sexLinked == 1) {fwrite($newDat, "$penetrance << penetrance classes\n");}
		//markers lines
		for ($i=0; $i<sizeOf($markers); $i++) {
			$numOfAlleles = $markers[$i]["numOfAlleles"];
			if ($numOfAlleles == 0) { //writes that there are 2 allales instead of 0. the cleaning proccess will delete this marker anywhay.
				fwrite($newDat, "3 2 #".$markers[$i]["name"]."# << non informative marker ".($i+1)."\n");
				fwrite($newDat, "0.5 0.5 << marker frequency\n");
			}
			else {
				fwrite($newDat, "3 ".$markers[$i]["numOfAlleles"]." #".$markers[$i]["name"]."# << marker ".($i+1)."\n");
				$alleleFrequancy = 1/$numOfAlleles;
				for ($j=0; $j<$numOfAlleles; $j++) {
					fwrite($newDat, $alleleFrequancy." ");
				}
				fwrite($newDat, "<< marker frequency\n");
			}
		}
		fwrite($newDat, "0 0 << ignored\n");	//ignored line
		//count distances in theta
		$distLine="0 ";
		for ($i=1; $i<sizeOf($markers); $i++) {
			$distanceInCM = ($markers[$i]["distance"] - $markers[$i-1]["distance"]);
			$distanceInTheta = convertCMtoTheta($distanceInCM);			
			$distLine = $distLine.sprintf("%.16f", $distanceInTheta)." ";
		}
		//print all the distances
		fwrite($newDat, $distLine." << distance between markers\n");	//distance between markers
		//print last two lines
		fwrite($newDat, "1 << number of disease loci\n");
		fwrite($newDat, "-n 2 1 ".sizeOf($markers)." << iteration over the chromosome\n");
		
		//close the file and exit
		fclose($newDat);
	}
	
	// $dir / $DB - the database and name mapping location is	(.snp file)
	// $destDir - where to put the output files
	// $chromNum which chromosome to make files for - should be string
	function makePedAndDatForChrom($dir, $DB, $origPed, $destDir, $chromNum, $parameters) {
		if ( (!(is_dir($dir))) || (!(is_dir($destDir))) || (!(is_file($dir."/".$DB))) ){
			log_error("can't open directory/database/destination folder, $dir / $DB / $destDir");
			return false;
		}
		if (createPedBy($dir, $DB, $origPed, $destDir, $chromNum)) {return createDatBy($dir, $DB, $destDir, $chromNum, $parameters);}
		return false; //becouse createPedBy failed
	}

	// creates a zoomed pair in the given $dir.
	// the $fileHead.dat/ped must exist in $dir.
	// returns false on illegal input like (file does not exist, folder does not exist, $start >= $end, $start or $end are not positive integers.)
	function zoomDatAndPed($dir, $fileHead, $start, $end, $rangeType = "numbers") {
		
		$datFile = $dir."/".$fileHead.".dat";
		
		if ($rangeType == "strings"){
			$start 	= findMarkerInDat($datFile, $start);
			$end 	= findMarkerInDat($datFile, $end  );
		}

        //Omer 8/8/2011 - add support for zooming by position
		if ($rangeType == "positions"){
            $start = findPositionInDat($datFile, $start, true);
            $end = findPositionInDat($datFile, $end, false);
		}
        
        if ($start < 0 or $end < 0) {return false;}		
		$zoomDatFile = zoomOnDat($dir, $fileHead, $start, $end);		// zoomOnDat is safe from invalid inputs				
		if (!$zoomDatFile) {return false;}								// return false upon failure
			
		$zoomPedFile = zoomOnPed($dir, $fileHead, $start, $end);		// zoomOnPed is safe from invalid inputs				
		$files['zoom_dat'] = $zoomDatFile;
		$files['zoom_ped'] = $zoomPedFile;
		return $files;
	}
	
	// zooms on dat file - returns the name of the zoomed file
	function zoomOnDat($dir, $fileHead, $start, $end) {
		$datFile = $dir."/".$fileHead.".dat";
		
		$zoomDatFile = $dir."/zoom_".$start."_".$end."_".$fileHead.".dat";
        $start = max($start,1);
				
		if ( !is_dir($dir) || !is_file($datFile) || ($start >= $end) || ($start <= 0) || ($end <= 0) ) {return false;}
		
		if (!($Dat = fopen($datFile, "r"))){
			log_error("can't open $dir/$datFile");
			return false;
		}
		
		if (!($zoomDat = fopen($zoomDatFile, "w"))){
			log_error("can't open $dir/$zoomDatFile");
			return false;
		}
        		
		// fixing first line
		$line = preg_split("/[ \t]/", rtrim(fgets($Dat)));
		if (feof($Dat)){return false;}
		$markersAmount = $line[0] - 1;	// save original markers amount (minus the disease marker)
        if ($start > $markersAmount) {return false;}        
		$end = min($end, $markersAmount);
		$newAmount = $end - $start + 1;
		$line[0] = $newAmount + 1;		// +1 for the disease marker
		$line = implode(" ", $line);
		fwrite($zoomDat, $line."\n");
		// second line
		fwrite($zoomDat, fgets($Dat));
		// fixing third line
		$line = rtrim(fgets($Dat));
		$line = "1";
		for ($i = 2 ; $i <= $newAmount + 1 ; $i += 1){		// +1 for the disease marker
			$line = $line." ".$i;
		}
		fwrite($zoomDat, $line." << markers order\n");
		
		// 4th, 5th, 6th and 7th lines
		for ($i = 0 ; $i < 4 ; $i += 1){
			$line = fgets($Dat);
			if (feof($Dat)){return false;}
			fwrite($zoomDat, $line);
		}
		
		// zooming on markers
		$line = fgets($Dat);
		$markersPassed = 1;
		while (!(feof($Dat))) {
			if ($markersPassed > $markersAmount) {break;}
			if ( ($markersPassed >= $start) && ($markersPassed <= $end) ){			// first marker line
				$line = explode("<<",rtrim($line));
				$newMarkerNum = $markersPassed - $start + 1;
				$line[1] = " << marker ".$newMarkerNum;				// commenting the number of the marker
				fwrite($zoomDat, implode("",$line)."\n");
			}
			$line = fgets($Dat);            
			if (feof($Dat)) {return false;}			// problem
			if ( ($markersPassed >= $start) && ($markersPassed <= $end) ){			// need to write 2 lines of the marker
				fwrite($zoomDat, $line);
			}			
			$line = fgets($Dat);
			$markersPassed += 1;			
		}		
		if (feof($Dat)) {return false;}	// problem		
		
		// last 4 lines
		// first line
		fwrite($zoomDat, $line);		// $line has the first line after the markers
		// fixing second line
		$physical = preg_split("/[ \t]/", rtrim(fgets($Dat)));
		if (feof($Dat)){return false;}
		$line = "0.0000000000000000";		
		for ($j = $start ; $j < $end ; $j += 1){
			$line = $line." ".$physical[$j];
		}
		fwrite($zoomDat, $line."\n");
		// 3rd line
		fwrite($zoomDat, fgets($Dat));
		if (feof($Dat)){return false;}
		// fixing fourth line
		$lastline = explode("<<", rtrim(fgets($Dat)));
		$line = preg_split("/[ \t]/", rtrim($lastline[0]));
		$comment = (sizeof($lastline) == 2) ? " << ".$lastline[1] : "";
		if (sizeof($line) != 4){return false;}
		$line[3] = $newAmount;
		fwrite($zoomDat, implode(" ", $line).$comment."\n");
		fclose($zoomDat);
		fclose($Dat);
		return $zoomDatFile;
	}

	// zooms on ped file - returns the name of the zoomed file
	function zoomOnPed($dir, $fileHead, $start, $end) {
		$pedFile = $dir."/".$fileHead.".ped";		
		$start = max($start,1);		
		if ( !is_dir($dir) || !is_file($pedFile) || ($start >= $end) || ($start <= 0) || ($end <= 0) ){
			return false;
		}
		
		if (!($Ped = fopen($pedFile, "r"))) {
			log_error("can't open $dir/$pedFile");
			return false;
		}
		// checking markers amount
		$line = preg_split("/[ \t]/", rtrim(fgets($Ped)));
		if (feof($Ped)){return false;}
        $markersAmount = (sizeof($line) - 10) / 2;	// we don't count the first 10 cells (person information), and each marker holds 2 data cells.
        $end = min($end, $markersAmount);
		$newAmount = $end - $start + 1;		
		
		$start 	= (($start * 2) - 1) + 10;			// start point ignores first 10 columns, and counts the odd column from the first pair
		$end 	= ($end * 2) + 10;					// end point ignores first 10 columns and counts the even column from the last pair
		
		$pedInfoCols = PEDFILE_COLUMNS;
		$zoomPedFile = $dir."/zoom_".$start."_".$end."_".$fileHead.".ped";
		exec("awk '{ for(i=1;i<=$end;i++) { if ((i <= $pedInfoCols) || (i >= $start)) { printf \"%s \",\$i } } ;print \"\" }' $pedFile >> $zoomPedFile");		// magical awk command does the job :P
		if (!is_file($zoomPedFile)){return false;}
        
        #Remove whitespaces from the created file		
        exec("sed -i 's/^[ \t]*//;s/[ \t]*$//' ".$zoomPedFile);
		return $zoomPedFile;
	}
	
	// filters dat and ped files by given cM.
	// returns true upon success and false upon failure.
	function filterDatPedByCM($dir, $fileHead, $cM) {
		
		$chosenMarkers = array();
		$filterDatFile = filterDatByCM($dir, $fileHead, $cM, $chosenMarkers);		// filterDatByCM is safe from invalid inputs
		if (!$filterDatFile) {return false;}										// return false upon failure
			
		$filterPedFile = filterPedByBitMap($dir, $fileHead, $cM, $chosenMarkers);	// filterPedByBitMap is safe from invalid inputs
		if (!$filterPedFile) {return false;}										// return false upon failure		
		$files['filter_dat'] = $filterDatFile;
		$files['filter_ped'] = $filterPedFile;
		return $files;
	}
	
	// filters markers from a Dat file which their distance is lower than given $cM
	// on success: returns the name of the filtered file.
	// on failure: returns false.
	// $markerChosen array will contain a bitmap array. An array cell contains 1 if the (i-1)'s marker stays, and 0 if its filtered.
	function filterDatByCM($dir, $fileHead, $cMToFilter, &$markerChosen) {
		$datFile = $dir."/".$fileHead.".dat";		$filteredDatFile = $dir."/filter_".$cMToFilter."cM_".$fileHead.".dat";		
		if ( !is_dir($dir) || !is_file($datFile) || ($cMToFilter <= 0) ) {return false;}		

		#Read recombinations line		
		$markersTheta = explode(" ",exec("tail -3 $datFile | head -1"));		// getting the line of thetas
		$markersAmount = explode(" ",exec("head -1 $datFile"));
		$markersAmount = $markersAmount[0] - 1;
		if ($markersAmount < 1) {return false;}

		#Find the position of each markers in cM
		$markersPositionsInCm = Array();
		$lastMarkerPos = 0;
		foreach ($markersTheta as &$theta) {
			$lastMarkerPos += convertThetaToCM($theta);
			$markersPositionsInCm[] = $lastMarkerPos;			
		}
		
		#Initialize variables		
		$markersCount = 0;				// number of markers that were not removed during filtering		
		$segmentLengthInCm = 0;		
		$firstMarkerInCurrentSegment = 1;
		$currMarker = 1;		
		$chosenMarkers = Array();
		$newMarkersThetaArray = Array();
		$lastSegmentEndPosInCm = 0;
		
		#Choose markers that will be kept		
		while ($currMarker <= $markersAmount) {
			#Add the current marker data to the current segment
			$newLength = $markersPositionsInCm[$currMarker-1] - $lastSegmentEndPosInCm;			
			$addedMarkerToSegment = false;
			if ($newLength <= $cMToFilter) {
				$segmentLengthInCm = $newLength;
				if ($currMarker < $markersAmount) {$currMarker++;};
				$markerChosen[] = 0;
				$addedMarkerToSegment = true;
			}
			
			#At the end of the segment, choose one marker in it for the filtered dat file
			if (!$addedMarkerToSegment || $currMarker >= $markersAmount) {				
				$chosenMarker = rand($firstMarkerInCurrentSegment, max($currMarker-1,$firstMarkerInCurrentSegment));
				$markerChosen[$chosenMarker-1] = 1;
				$chosenMarkers[] = $chosenMarker;				

				#Find the recombination frequency between this marker and the last one that was chosen
				if ($markersCount == 0) {$newMarkersThetaArray[] = "0";}
				else {
					$c1 = $chosenMarkers[$markersCount-1];
					$c2 = $chosenMarkers[$markersCount];					
					$distanceInCm = $markersPositionsInCm[$c2-1] - $markersPositionsInCm[$c1-1];					
					$distanceInTheta = convertCMtoTheta($distanceInCm);
					$newMarkersThetaArray[] = $distanceInTheta;
				}
				$markersCount++;
				
				#Prepare to start a new segment
				$segmentLengthInCm = 0;
				$firstMarkerInCurrentSegment = $currMarker;				
				$lastSegmentEndPosInCm = $markersPositionsInCm[$currMarker-1];
				
				if ($chosenMarker == $markersAmount) {break;}
			}
		}
		
		//Create the lines array of the filtered data file
		if (!($Dat = fopen($datFile, "r"))) {
			log_error("can't open $dir/$datFile");
			return false;
		}
		//keeping first 7 lines
		$newDatLines = Array();
		for ($i = 0 ; $i < 7 ; $i++) {$newDatLines[] = fgets($Dat);}		
		#Fixing first line
		$firstLine = explode(" ", $newDatLines[0]);
		$firstLine[0] = $markersCount + 1;
		$newDatLines[0] = implode(" ",$firstLine);				
		$newDatLines[2] = implode(" ",range(1, $markersCount + 1))."\n";	#Fixing third line
		
		#Add markers that were not filtered to filtered dat file
		$markersAddedToFilteredFile = 0;
		$currMarkerNumber = 0;
		$nextIndexOfFilteredMarkerToAdd = 0;
		while ($markersAddedToFilteredFile < $markersCount) {
			$line1 = fgets($Dat);
			$line2 = fgets($Dat);
			$currMarkerNumber++;
			if ($currMarkerNumber == $chosenMarkers[$nextIndexOfFilteredMarkerToAdd]) {
				$nextIndexOfFilteredMarkerToAdd++;
				$markersAddedToFilteredFile++;
				$newDatLines[] = $line1;
				$newDatLines[] = $line2;
			}
		}
		fclose($Dat);

		// last 4 lines		
		$newDatLines[] = "0 0 << sex difference, interference\n";
		$newDatLines[] = implode(" ", $newMarkersThetaArray)."\n";
		$newDatLines[] = "1 << number of disease loci \n";
		$newDatLines[] = "-n 2 1 ".$markersCount."	 << REC VARIED, INCREMENT, FINISHING VALUE \n";
		
		if (!($filteredDat = fopen($filteredDatFile, "w"))) {
			log_error("can't open $dir/$filteredDatFile");
			return false;
		}			
		foreach ($newDatLines as &$line) {fwrite($filteredDat, $line);}
		fclose($filteredDat);
		
		return $filteredDatFile;		
	}
	

	// filter on ped file - returns the name of the filtered file
	// $cM is needed in order to stay consistant with the file names (will be similar to the dat file name)
	function filterPedByBitMap($dir, $fileHead, $cM, $bitMap) {
		$pedFile = $dir."/".$fileHead.".ped";
		if (!is_dir($dir) || !is_file($pedFile) || ($bitMap == null)) {return false;}		
		if (!($Ped = fopen($pedFile, "r"))){
			log_error("can't open $dir/$pedFile");
			return false;
		}
		
		// checking markers amount
		$line = preg_split("/[ \t]/", rtrim(fgets($Ped)));
		if (feof($Ped)) {return false;}			
		
		$newAmount = sizeof($bitMap);
		$markersAmount = (sizeof($line) - 10) / 2;			// we don't count the first 10 cells (person information), and each marker holds 2 data cells.
		if ($newAmount > $markersAmount) {return false;}
		
		$cols = "";
		for ($i = 1 ; $i <= PEDFILE_COLUMNS ; $i += 1) {				// first 10 columns (person's info)
			$cols = $cols."\$".$i."\" \"";
		}		
		for ($i = 0 ; $i < $newAmount ; $i += 1){					// chosen columns
			if ($bitMap[$i] == 1) {
				$cols = $cols."\$".((2*$i) + 1 + PEDFILE_COLUMNS)."\" \""."\$".((2*$i + 1) + 1 + PEDFILE_COLUMNS)."\" \"";	// need a pair of data for each marker.
			}
		}		
		
		$filteredPedFile = $dir."/filter_".$cM."cM_".$fileHead.".ped";
		exec("awk '{ print $cols }' $pedFile >> $filteredPedFile");		// awk command does the job :P
		if (!is_file($filteredPedFile)) {return false;}		
		return $filteredPedFile;
	}
	
	
	/* finds a marker in dat file by a String and returns its place in the file. (1 means its the first marker, 2 the second, etc...)
	*   returns -1 on errors. (not found, file unopenable...)
	*/
	function findMarkerInDat($datFile, $marker) {
		$retVal = -1;
		if ( !is_file($datFile) ){return $retVal;}
		
		if (!($dat = fopen($datFile, "r"))){
			log_error("can't open $dir/$datFile");
			return $retVal;
		}
		
		for ($j = 0 ; $j < 7 ; $j += 1){ // we don't count the first 7 lines of the dat file
			$line = fgets($dat);
			if (feof($dat)) {
				fclose($dat);
				return $retVal;
			}
		}
		
		$line = fgets($dat);
		$markerNum = 1;
		while (!feof($dat)) {
			if (preg_match("/$marker/", $line)) {
				$retVal = $markerNum;
				break;
			}
			$markerNum += 1;
			$line = fgets($dat);			// jump over frequency line
			if (feof($dat)) {break;}
			$line = fgets($dat);
		}		
		fclose($dat);		
		return $retVal;
	}
    
    
	/* Returns the name of a certain marker in the .dat according to its number
	*   returns -1 on errors. (not found, file unopenable...)
	*/
	function getMarkerNameByNumber($datFile, $markerNum) {
		$retVal = -1;
		$name = "";
		if ( !is_file($datFile) ){return $retVal;}
		
		if (!($dat = fopen($datFile, "r"))){
			log_error("can't open $dir/$datFile");
			return $retVal;
		}
		
		for ($j = 0 ; $j < 7 ; $j += 1) {			// we don't count the first 7 lines of the dat file
			$line = fgets($dat);
			if (feof($dat)){
				fclose($dat);
				return $retVal;
			}
		}
		
		$line = fgets($dat);
		$i = 1;
		while (!feof($dat)) {            
            if (preg_match("/#(.*)#/", $line)) {
                $name = explode("#", $line);
                $name = $name[1];
            }			
            if ($i == $markerNum) {
                if ($name != "") {$retVal = $name;}				
                break;                    
            }
			$i += 1;
			$line = fgets($dat);			// jump over frequency line
			if (feof($dat)) {               //If we've reached the end of the file, take the last marker name
                $retVal = $name;
                break;
            }            
			$line = fgets($dat);            //Read the next marker line
            if (feof($dat)) {$retVal = $name;}
		}
		if (feof($dat)) {$retVal = $name;}
		
		fclose($dat);				
		return $retVal;
	}
    
    
	/* finds a marker in dat file by a position and returns its place in the file. (1 means its the first marker, 2 the second, etc...)
	*   returns -1 on errors. (not found, file unopenable...)
	*/
	function findPositionInDat($datFile, $position, $before=false) {
		$retVal = -1;
		if ( !is_file($datFile) ){return $retVal;}
        
        if ($position <= 0) {return 1;}
		
        exec("tail -3 $datFile | head -1", $line);
		$line = $line[0];
		$line = explode("<<", $line);
		if (!isset($line[0])) { return $retVal; }
        $distances = explode(" ", $line[0]);
        
        $curMarker = 1;
        $curDistance = 0;		
		
        
        while ($curDistance < $position and $curMarker < sizeof($distances)) {
            $curMarker += 1;
            $curDistance += convertThetaToCM($distances[$curMarker-1]);            
        }
        //If we've reached the last marker, this is the one		
        if ($curMarker == sizeof($distances)) {return $curMarker;}
        
        if ($curDistance == $position) {return $curMarker;}        
        elseif ($before) {return $curMarker-1;}
        else {return $curMarker;}
	}
    
	/* Returns the name of a certain marker in the .dat according to its position
	*   returns -1 on errors. (not found, file unopenable...)
	*/
	function getMarkerNameByPosition($datFile, $position, $before=false) {
        $markerNum = findPositionInDat($datFile, $position, $before);
        if ($markerNum < 1) {return $markerNum;}    //In case of error
        $markerName = getMarkerNameByNumber($datFile, $markerNum);
        return $markerName;
	}      
    

?>