<?php
	require_once("config.php");

	// removes a directory and all its content. give $DeleteMe false if the directory itself should not be deleted.
	function rmdir_r ($dir, $DeleteMe = TRUE) {
	
		$dir = realpath($dir);	#Fix directory structure		
		if ($DeleteMe) {system("rm -rf ".$dir);}
		else {system("rm -rf ".$dir."/*");}
	}
	
	/*
	* counts the number of files in a given $dir
	* $type = "all" will count all files.
	* $type = "visible" will count non-hidden files.
	* specify $extension to count files with that specific extension.
	*/
	function countFilesInDir($dir, $type = "all", $extension = "all") {
		if (!is_dir($dir)) {return 0;}
		if ( (!($handle = opendir($dir))) ) {return 0;}
		
		$count = 0;		
		while ( false !== ($obj = readdir($handle)) ) {
			if (is_file($dir."/".$obj)) {
				if ( ($type == "all") || (($type == "visible") && (preg_match("/^[^\.].*$/", $obj))) ) {
					if ( ($extension == "all") || (($extension != "all") && (preg_match("/^.*\.$extension$/", $obj))) ){
						$count += 1;
					}
				}
			}
		}		
		return $count;
	}
	
	/*
	* returns true if a folder has no visible files / folders. false otherwise.
	*/
	function folderEmpty($dir) {
		if (!is_dir($dir)) {return true;}
		if ( (!($handle = opendir($dir))) ) {return true;}
		
		while ( false !== ($obj = readdir($handle)) ) {
			if (preg_match("/^[^\.].*$/", $obj)) {return false;}			
		}
		
		return true;
	}

	// returns true if a visible file of type < ... >.$fileType exists in dir
	// returns false otherwise
	function fileOfTypeExistsInDir($dir, $fileType) {
		if (!is_dir($dir)) {return false;}
		if ( (!($handle = opendir($dir))) ) {return false;}
		
		while ( false !== ($obj = readdir($handle)) ) {
			if ( (is_file($dir."/".$obj)) && (preg_match("/^[^\.].*\.$fileType$/", $obj)) ) {
				return true;
			}
		}
		return false;
	}
	
	// returns an array with all visible file of type $fileType in $dir
	// if fileType is false or undefined - returns all visible directories
	// returns false on errors
	function getFilesOfType($dir, $fileType = false) {
		if (!is_dir($dir)){ return false; }
		
		if ( (!($handle = opendir($dir))) ) { return false;	}
		
		$files = array();
		while ( false !== ($obj = readdir($handle)) ){
			if ( (!$fileType) && is_dir($dir."/".$obj) && (preg_match("/^[^\.]/", $obj)) ){
				$files[] = $obj;
			}
			else {
				if ( (is_file($dir."/".$obj)) && (preg_match("/^[^\.].*\.$fileType$/", $obj)) ){
					$files[] = $obj;
				}
			}
		}
		
		return $files;
	}
	
	// this function sorts given $data of arrays by a given $field (assuming the content of the $field is an existing string)
	function orderBy(&$data, $field){
		$code = "return strnatcmp(\$a['$field'], \$b['$field']);";
		usort($data, create_function('$a,$b', $code));
	}
		
	/* generates a random name for a folder. this name does not already exist.
	gets the user's directory to generate, and the type (Project, Experiment, and so on...)
	*/
	function generateName($directory, $type) {
		if ( (!($handle = opendir($directory))) ) {return "-1";}			
		
		$i = 1;
		while ( false !== ($obj = readdir($handle)) ){
			if ( (is_dir($directory."/".$obj)) && (preg_match("/^".$type."([0-9]+)$/", $obj, $matches)) ){
				if ($matches[1] >= $i) {$i = $matches[1] + 1;}
			}
		} 
		closedir($handle);
		$randName = $type.$i;
		return $randName;
	} 
	
	// does not allow "$", "/" in folder name, at all.  does not allow "." in the beginning of a folder name
	// will not create a folder which already exists
	function createFolder($directory, $name, $special = false) {
		if ( ($name == NULL) || ($name == "") || ($directory == NULL) || ( strpos($name,"/") !== false )  ||
			 (preg_match("/^[ \t\r\n]+.*$/", $name)) ){
			return false; //error
		}
		
		if ( $special || ((!(strpos($name, ".") === 0)) && (!strstr($name,"$"))) ){
			if ($handle = opendir($directory)) { // look for a project directory with the same name
			    while (false !== ($file = readdir($handle))) {
			        if ( (is_dir($directory."/".$file)) && ($file == $name) ){ // a directory with that name already exists...
						closedir($handle);
						return false;
			        }
			    }
			    closedir($handle);
			}
			if (mkdir($directory."/".$name, 0757) == true){  // everything is ok
				//mkdir doesn't apply 757 to the directory from an unknown reason
				############   better to add the user that runs all the receiving  (from superlink) scripts to the apache group ?!!!! ###########
				$dir = $directory."/".$name;
				exec("chmod 757 $dir");
				##############################################################################
				return true;
			}
			else {return false;}			
		}
		else {return false;}		
	}
	
	// creates a project folder under the specified directory, with raw and experiments sub-folders.
	// returns false on failure.
	function createProjectFolder($directory, $name) {
		if (createFolder($directory, $name)) {
			if ( (!createFolder($directory."/".$name, "raw")) || (!createFolder($directory."/".$name, "experiments")) ){
				rmdir_r($directory."/".$name);  // if there's an error deletes everything created in the process
				return false;
			}
		}
		else {return false;}		
		return true;
	}
	
	function logExists($directory, $file, $type){
		if (!is_file($directory."/".PROGRESS_LOGS_FILENAME) || ($file == false)) {return false;}
		if (is_file($directory."/".".".$file."_".$type."_log.txt")) {return true;}
		return false;
	}
    
	
	// $projectDir is the project's directory. a segregation folder will be created under the experiments directory of this project.
	function createAndRunSegregation($projectDir, $snpSource = false) {
		$rawFilesDir	= $projectDir ."/raw";
		$experimentsDir = $projectDir ."/experiments";
		$segregationDir = $projectDir ."/experiments/Segregation";
		if (is_dir($segregationDir)) { return false; }
		createExperimentFolder($experimentsDir, "Segregation", false, false, true);
		markSegregationFolder($segregationDir);
		$flags = SEND_TO_SL_FLAG;
		$varsTable[KEY_RUN_ANALYSIS] = VAL_RUN_ANALYSIS_SEGREGATION;
		$varsTable[KEY_SEGREGATION_PARAMETER] = VAL_SEGREGATION_DEFAULT;
		if ($snpSource) { $varsTable[SNP_SOURCE_EXPERIMENT] = "true"; }
		
		createVariablesFile($segregationDir, $varsTable);		
		
	    $handler = opendir($rawFilesDir); 
	    while(false !== ( $file = readdir($handler)) ) { 
	        if (( $file != '.' ) && ( $file != '..' )) {
				$info = pathinfo($rawFilesDir."/".$file);
	            if ( is_file($rawFilesDir."/".$file) && ( $info['extension'] == "ped") ) { 
					$fileName = substr($file, 0, -4);
	                //copy($rawFilesDir.'/'.$file, $segregationDir.'/'.$file);
					//copy($rawFilesDir.'/'.$fileName.".ped", $segregationDir.'/'.$fileName.".ped");
					file_put_contents($segregationDir.'/'.$fileName.".ped", implode("\n", getPedigreeFromPedfile($rawFilesDir."/".$file)));
					createArbitraryDat($segregationDir, $fileName.".dat");
					$table['header'] = $fileName;
					$table['datfile'] = $segregationDir.'/'.$fileName.".dat";
					$table['pedfile'] = $segregationDir.'/'.$fileName.".ped";
					updateExpContentInfo($segregationDir, $table);
					
					//initProgressForFile($segregationDir, $fileName, array("cleaning"));
					break;
	            }
	        }
	    }
	    closedir($handler); 
		
		sendExperiment($segregationDir, $flags);
	}
    
    
    function createAndRunLmMarkers($projectDir, $filesLocation, $cleanThreshold, $snpSource = false) {
		$experimentsDir = $projectDir ."/experiments";
		$LmMarkersDir = $experimentsDir."/Morgan_lm_markers";
		
        //Create a directory and set options for the moreVariables.txt file
		if (is_dir($LmMarkersDir)) { return false; }		
		createExperimentFolder($experimentsDir, "Morgan_lm_markers", true, false, true);
		$flags = CLEANING_FLAG." ".SEND_TO_SL_FLAG;
		$varsTable[KEY_CLEANING_THRESHOLD] = $cleanThreshold;
		$varsTable[KEY_RUN_MODE] = VAL_RUN_MODE_CLEAN_ONLY;
		$varsTable[KEY_RUN_ANALYSIS] = VAL_RUN_ANALYSIS_LM_MARKERS;
		if ($snpSource) { $varsTable[SNP_SOURCE_EXPERIMENT] = "true"; }
        
        //Morgan specific options        
        $varsTable[KEY_MAX_JOBS_PER_TASK] 		 	= MAX_JOBS_PER_TASK;
        $varsTable[KEY_MAX_TIME_PER_JOB] 		 	= MAX_TIME_PER_JOB;
        $varsTable[KEY_MIN_DESIRED_TIME_PER_JOB] 	= MIN_DESIRED_TIME_PER_JOB;
        $varsTable[KEY_NUMBER_DUPLICATIONS_PER_JOB] = NUMBER_DUPLICATIONS_PER_JOB;
        $varsTable[KEY_TIME_OF_WINDOW_IN_SECONDS] 	= TIME_OF_WINDOW_IN_SECONDS;

        //Create the moreVariables.txt file
		createVariablesFile($LmMarkersDir, $varsTable);
		
        //Create an analysis folder for each dat file
	    $handler = opendir($filesLocation);
	    while(false !== ( $file = readdir($handler)) ) {
	        if (( $file == '.' ) or ( $file == '..' )) {continue;}  //ignore non-files
            $info = pathinfo($filesLocation."/".$file);
            if ( !is_file($filesLocation."/".$file) or ($info['extension'] != "dat" and  $info['extension'] != "ped")) {continue;}            
            copy($filesLocation.'/'.$file, $LmMarkersDir.'/'.$file);
            if ($info['extension'] != "dat") {continue;}
            $fileName = substr($file, 0, -4);
            $table['header'] = $fileName;
            $table['datfile'] = $LmMarkersDir.'/clean/resultFile.'.$fileName.".dat";
            $table['pedfile'] = $LmMarkersDir.'/clean/resultFile.'.$fileName.".ped";
            $table['clean_datfile'] = $LmMarkersDir.'/clean/resultFile.'.$fileName.".dat";
            $table['clean_pedfile'] = $LmMarkersDir.'/clean/resultFile.'.$fileName.".ped";
            updateExpContentInfo($LmMarkersDir, $table);
            initProgressForFile($LmMarkersDir, $fileName, array("cleaning"));
            unset($table);            
	    }
	    closedir($handler);		
		sendExperiment($LmMarkersDir, $flags);            
    }

	// $projectDir is the experiments' directory. an hmm multipoint folder will be created there.
	// $expName is the name given to this HMM experiment
	// $parentDir is the experiment from which the HMM was run 
	// $multiDir id the directory in which the regular multipoint will run - assuming it has the files already
	function runSubHmmMultiPoint($experimentsDir, $expName, $parentDir, $multiDir){
		$HmmDir = $experimentsDir."/".$expName;
		
		//Update parent table
		$parentTable = getVariablesTable($parentDir);
		$parentTable[KEY_DERIVED_EXPERIMENTS] = (isset($parentTable[KEY_DERIVED_EXPERIMENTS]) ? $parentTable[KEY_DERIVED_EXPERIMENTS]."," : "").$expName;
		createVariablesFile($parentDir, $parentTable);
		
		if (is_dir($HmmDir)) { return false; }
		//if ( !testHMM($parentDir) ) { return false; }
		
		createExperimentFolder($experimentsDir, $expName, false, false, true);
		$multiTable = getVariablesTable($multiDir);								// get normal multipoint experiment's table
		$flags = SEND_TO_SL_FLAG;
		$varsTable[KEY_RUN_ANALYSIS] 	= VAL_RUN_ANALYSIS_MULTIPOINT_HMM;
		if (isset($multiTable[KEY_START_MARKER]) && isset($multiTable[KEY_END_MARKER])){
			$varsTable[KEY_START_MARKER] 	= $multiTable[KEY_START_MARKER];
			$varsTable[KEY_END_MARKER] 		= $multiTable[KEY_END_MARKER];
			$varsTable[KEY_WINDOW_SIZE] 	= $multiTable[KEY_END_MARKER] - $multiTable[KEY_START_MARKER] + 1;
		}
		elseif (isset($multiTable[KEY_MARKERS_LIST])){
			$varsTable[KEY_MARKERS_LIST] 	= $multiTable[KEY_MARKERS_LIST];
		}
		else {
			return false;
		}
		
		createVariablesFile($HmmDir, $varsTable);
		
	    $handler = opendir($multiDir);
	    while(false !== ( $file = readdir($handler)) ) { 
	        if (( $file != '.' ) && ( $file != '..' )) {
				$info = pathinfo($multiDir."/".$file);
	            if ( is_file($multiDir."/".$file) &&
					 ( ($info['extension'] == "dat") ||
					   ($info['extension'] == "ped") ||
					   ($info['extension'] == "map")) ) { 
	                copy($multiDir.'/'.$file, $HmmDir.'/'.$file);
					if ($info['extension'] == "dat"){
						$fileName = substr($file, 0, -4);
						$table['header'] = $fileName;
						$table['datfile'] = $HmmDir.'/'.$fileName.".dat";
						$table['pedfile'] = $HmmDir.'/'.$fileName.".ped";
						$table['mapfile'] = $HmmDir.'/'.$fileName.".map";
						updateExpContentInfo($HmmDir, $table);
						//initProgressForFile($HmmDir, $fileName, array("cleaning"));
						unset($table);
					}
	            }
	        }
	    }
	    closedir($handler);
		
		return sendExperiment($HmmDir, $flags);
	}
	
	// tests if all dat/ped files in $filesLocation support HMM analysis
	function testHMM_Directory($filesLocation) {
		$handler = opendir($filesLocation);
	    while(false !== ( $file = readdir($handler)) ) { 
	        if (( $file != '.' ) && ( $file != '..' )) {
				$info = pathinfo($filesLocation."/".$file);
	            if ( is_file($filesLocation."/".$file) &&
					 ( ($info['extension'] == "dat") ||
					   ($info['extension'] == "ped") ) ) { 
					if ($info['extension'] == "dat"){
						$fileName = substr($file, 0, -4);
						$dat = $filesLocation."/".$fileName.".dat";
						$ped = $filesLocation."/".$fileName.".ped";
						if (!runSuperlinkLocally($dat, $ped, SUPERLINK_PARAMETER_HMM." ".SUPERLINK_PARAMETER_M800." ".SUPERLINK_PARAMETER_HMM_TEST_ONLY)) {
							log_error("HMM Test on $fileName in $filesLocation failed.");
							return false;
						}
					}
	            }
	        }
	    }
	    closedir($handler);
		
		log_error("HMM Test on all files in $filesLocation completed successfully.");
		return true;
	}

	function testHMM($dir, $dat, $ped) {
		$datInfo = pathinfo($dir."/".$dat);
		$pedInfo = pathinfo($dir."/".$ped);
		if ( is_file($dir."/".$dat) && is_file($dir."/".$ped) && ($datInfo['extension'] == "dat") && ($pedInfo['extension'] == "ped") ) { 
			if (!runSuperlinkLocally($dir."/".$dat, $dir."/".$ped, SUPERLINK_PARAMETER_HMM." ".SUPERLINK_PARAMETER_M800." ".SUPERLINK_PARAMETER_HMM_TEST_ONLY)) {
				log_error("HMM Test on $dat and $ped in $dir failed.");
					return false;
			}
		}
		
		log_error("HMM Test on $dat and $ped in $dir completed successfully.");
		return true;
	}
	
	function createArbitraryDat($dir, $dat) {
		if (!is_dir($dir) || is_file($dir."/".$dat)){ return false; }
		$content = 	"2 0 0 5 0 << NO. OF LOCI, RISK LOCUS, SEXLINKED (IF 1) PROGRAM, # COMPLEX LOCI\n".
					"0 0.0 0.0 0 << MUT LOCUS, MUT RATE, HAPLOTYPE FREQUENCIES (IF 1)\n".
					"1 2 \n".
					"1 2 << AFFECTION, NO. OF ALLELES\n".
					"0.99 0.01 << GENE FREQUENCIES\n".
					"1 << NO. OF LIABILITY CLASSES \n".
					"0 0 0.99 << PENETRANCES \n".
					"3 2 #m1# << ALLELE NUMBERS, NO. OF ALLELES \n".
					"0.5 0.5  << GENE FREQUENCIES \n".
					"0 0 << SEX DIFFERENCE, INTERFERENCE (IF 1 OR 2) \n".
					"0.5 << RECOMBINATION VALUES \n".
					"1 0.1 0.45  << REC VARIED, INCREMENT, FINISHING VALUE \n";
		system("echo '$content' > ".$dir."/".$dat);
		return true;
	}
	
	function getPedigreeFromPedfile($pedfile) {
		if (!is_file($pedfile)) { return null; }
		$ped = array();
		exec("cut -d \" \" -f 1,2,3,4,5,6,7,8,9,10 $pedfile", $ped);
		return $ped;
	}
	
	// runs Superlink locally on given $dat and $ped files input in $dir, with given $parameters
	// returns 1 on success and 0 on failure or any other value if didnt want success or failure.
	function runSuperlinkLocally($dat, $ped, $parameters) {
		system(LOCAL_SUPERLINK_EXEC . " $dat $ped $parameters > /dev/null", $ret);
		//$ret = exec(LOCAL_SUPERLINK_EXEC . " $dat $ped $parameters > /dev/null; echo $?");
		switch ($ret){
			case SUPERLINK_RETURN_SUCCESS : return true; break;
			case SUPERLINK_RETURN_FAILURE : return false; break;
			default:						return false; break;
		}
	}
	
	/*
	* helper function for showContent
	* gets the percentages from the progressFile which should be in the format:
	* chrom_<num> cleaning <percent>%
	* chrom_<num> clustering <percent>%
	* chrom_<num> analysis <percent>%
	*/
	function getProgressPercents($directory, $file, &$percents) {
		if (!is_file($directory."/".PROGRESS_FILENAME) || ($file == false)) {return false;}

		###
		# dvir 2011/10/17, showing an error in submitting
		$IsThereIsAnErrorAtSubmission = checkIfThereIsAnErrorAtSubmission($directory, $file);
		if ($IsThereIsAnErrorAtSubmission != "") {
			$percents[0] = $IsThereIsAnErrorAtSubmission;
			$percents[1] = $IsThereIsAnErrorAtSubmission;
			$percents[2] = "";
			return true;
		}
		###
		
		$percents[0] = (isConversionFolder($directory) ? "conversion 0%": "cleaning 0%");
		$percents[1] = "clustering 0%";
		$percents[2] = "analisys 0%";
		
		$progress = file($directory."/".PROGRESS_FILENAME);
		for ($i = 0 ; $i < sizeof($progress) ; $i += 1) {
			$line = explode(" ", $progress[$i]);
			if (!isset($line[0]) || !isset($line[1]) || !isset($line[2])) { continue; }
			$fName = trim($line[0]);
			$parameter = trim($line[1]);
			$percent = trim($line[2]);
			$link = logExists($directory, $fName, $parameter) ? "viewReport.php?file=".base64_encode($directory."/.".$fName."_".$parameter."_log.txt") : false;
			if ($file != $fName) {continue;}
			switch ($parameter) {
				case "cleaning": $percents[0] = (isConversionFolder($directory) ? "conversion" : $parameter)." ".$percent;
									if ($link){
										$percents[0] = "<a href=\"$link\" target=_blank>".$percents[0]."</a>";										
									}
									break;
				case "clustering": $percents[1] = $parameter." ".$percent;
									if ($link){
										$percents[1] = "<a href=\"$link\" target=_blank>".$percents[1]."</a>";										
									}
									break;
				case "analysis": $percents[2] = $parameter." ".$percent;
									if ($link){
										$percents[2] = "<a href=\"$link\" target=_blank>".$percents[2]."</a>";										
									}
									break;
				default: continue;
			}
		}
		
		return true;
	}
	
	function isErrorFor($directory, $fileHead, $errorIn) {
		if (!is_file($directory."/".PROGRESS_FILENAME) || ($fileHead == false)) {return false;}
		$progress = file($directory."/".PROGRESS_FILENAME);
		for ($i = 0 ; $i < sizeof($progress) ; $i += 1) {
			$line = explode(" ", $progress[$i]);
			if (!isset($line[0]) || !isset($line[1]) || !isset($line[2])) { continue; }
			$fName = trim($line[0]);
			$parameter = trim($line[1]);
			$percent = trim($line[2]);
			if (($fileHead == $fName) && ($parameter == $errorIn) && ($percent == "Error")) {return true;}
		}
		return false;
	}

	// for directories give $type = "dir",  for files give $type = "file", to see all $type="all"
	//spaces in file or directory names are changed to $ in the checkbox value and in the action value.
	// for non-files ONLY: $actName is the action to perform upon clicking a non-file. will apear as: runningScript.php?$actName=. default value is "".
	// if type = "file",  $actName is not needed at all.
	// if can't open directory or didn't write anything returns false
	// every shown item has a checkbox next to it. in a form all items are in checkboxes[] array.
	// give encode true if you want the folder links to be encoded with 64base. leave it for no encoding.
	// give linkSubDir to be added before directories links
	// each file/directory shown is in a <div> tag with id of the file name, like so: <div id="file1"> file1 </div>
	function showContent($directory, $type, $actName = "", $showBox = true, $encode = false, $linkSubDir = "") {
		$wroteSomething = false;
		if ( (is_dir($directory)) && ($handle = opendir($directory))) {
			while (($file = readdir($handle)) != false) {
				//if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
				if (strpos($file,".") === 0) { continue; }
				$contentArray[] = $file;
			}
			if ( (!isset($contentArray)) || (sizeof($contentArray) == 0) ) {
				closedir($handle);
				return $wroteSomething;
			}
			sort($contentArray);
			
			?><!--<b class="b1f"></b><b class="b2f"></b><b class="b3f"></b><b class="b4f"></b> <div class="contentf"> <div>-->
			<?php
			
			echo "<table style=\"width: 99%;\">";
			foreach($contentArray as $file) {
				$runningScript = $_SERVER["SCRIPT_NAME"];  // php file running name
				//$currentlyRunning = curPageURL();   // with the actions
				echo "<tr>";
				if ((($type == "dir") || ($type == "all")) && (is_dir($directory."/".$file))){
					echo "<td>";
					$wroteSomething = true;
					$file_nospaces = str_replace(" ", "$", $file);
					//echo "<div id=\"$file\">";
					if ($showBox){ 
						echo "<input type=\"checkbox\" name=\"checkboxes[]\" value=\"".$file_nospaces."\" />";
					}
					if ($actName != ""){
						if (strcmp($linkSubDir, "") != 0){
							$file_nospaces = $linkSubDir."/".$file_nospaces;
						}
						if ($encode){
							$file_nospaces = base64_encode($file_nospaces);
						}
						echo "<a href=".$runningScript."?".$actName."=".$file_nospaces.">".$file."</a><br/>";
					}
					else {
						echo $file."<br/>";
					}
					//echo "</div>";
					echo "</td>";
				}
				elseif ((($type == "file") || ($type == "all")) && (is_file($directory."/".$file))){
					echo "<td bgcolor=\"#E8EEF7\">"; //#f0ffff\">"; 
					$wroteSomething = true;
					$file_nospaces = str_replace(" ", "$", $file);
					//echo "<div id=\"$file\">";
					if ($showBox){ 
						echo "<input type=\"checkbox\" name=\"checkboxes[]\" value=\"".$file_nospaces."\" />";
					}
					echo "<a href=\"download.php?file=".base64_encode($directory."/".$file)."\">".$file."</a><br/>";
					//echo "</div>";
					echo "</td>";
					
					$Percents = array();
					if (is_file($directory."/".PROGRESS_FILENAME)){
						$Percents[0] = "none";
						$Percents[1] = "none";
						$Percents[2] = "none";
						
						/**************/
						$chrom = false;
						if (preg_match("/^.*(chrom_[0-9X][0-9]?).*$/", $file, $chr)){
							$chrom = $chr[1];
						}
						/*************/
						$fileHead = false;
						if (preg_match("/(.*)\..*$/", $file, $head)){
							$fileHead = $head[1];
						}
						
						if (getProgressPercents($directory, $fileHead, $Percents)){
							//for ($i = 0; $i < sizeof($Percents) ; $i += 1){			// use this line when more than cleaning and clustering can be updated.
							for ($i = 0; $i < 2 ; $i += 1){
								echo "<td style=\"width: 15%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
								if  (preg_match("/.*\.dat$/", $file)){
									$method = ($i == 0) ? "cleaning" : "clustering";
									echo "<span id=\"OLD_$fileHead"."_$method\">".$Percents[$i]."</span>";
									/*** setup ajax job this spot *****/
										$ajaxJobs[] = "'".$fileHead."*".$method."'";
									/**********************************/
								}
								echo "</td>";
							}
						}
					}
					
					if (is_file($directory."/".RESULTS_FILENAME)){
						//echo "<td style=\"width: 15%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
						$res = false;
						$cell1 = "analysis waiting";
						$cell2 = "No results yet";
						/* checking for error in analysis */
						if (preg_match("/(.*)\.dat*$/", $file, $head)){
							$fileHead = $head[1];
							if (isErrorFor($directory, $fileHead, "analysis")){
								$link = "viewReport.php?file=".base64_encode($directory."/.".$fileHead."_analysis_log.txt");
								$cell1 = "<a href=$link target=\"_blank\">analysis error</a>";
							}
						}
						/*       *       *         *       *        */
						
						$results = file($directory."/".RESULTS_FILENAME);
						for ($i = 0 ; $i < sizeof($results) ; $i += 1){
							$line = explode(" ", $results[$i]);
							if (!isset($line[0]) || !isset($line[1])){
								continue;
							}
							$resultForHeader = trim($line[0]);
							$resultFolder = trim($line[1]);
							if (preg_match("/^(.*)\.dat$/", $file, $thisFileHead)){
								$thisFileHead = $thisFileHead[1];
								if (preg_match("/$thisFileHead$/", $resultForHeader)){
									$fileDir = $resultFolder;
									if (preg_match("/^.*\/(var\/tmp\/.*)$/",$resultFolder, $fileDir)){
										$fileDir = $fileDir[1];
									}
									$statusPage  = WEBDIR."/".$fileDir."/".SL_STATUS_PAGE;
									$resultsPage = WEBDIR."/".$fileDir."/".SL_RESULTS_PAGE;
									if (is_file($statusPage)){
										$cell1 = "<a href=\"$statusPage\" target=\"_blank\">sent to analysis</a>";
									}
									if (is_file($resultsPage)){
										$cell2 = "<a href=\"$resultsPage\" target=\"_blank\"><strong>$thisFileHead Results</strong></a>";
										if (is_file($statusPage)){
											$cell1 = "<a href=\"$statusPage\" target=\"_blank\">analysis done</a>";
										}
										else{
											$cell1 = "<b>analysis done</b>";
										}
									}
									/*else{		# if a message was put
										$cell2 = "<div style=\"color: red\"><strong>Error on $thisFileHead</strong></div>";
									}*/
								}
							}
						}
						echo "<td style=\"width: 15%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
						if (preg_match("/^(.*)\.dat$/", $file)){
							echo $cell1;
						}
						echo "</td>";
						echo "<td style=\"width: 15%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
						if (preg_match("/^(.*)\.dat$/", $file)){
							echo $cell2;
						}
						echo "</td>";
					}					
				}
				echo "</tr>";
			}
			echo "</table>";
			closedir($handle);
			
			$splitted = explode("/",$directory);	// ./users/<name>/projects/<proj>/experiments/<exp>
												// ./users/<name>/projects/<proj>/formatted
			$proj 	= $splitted[4];
			$exp 	= $splitted[sizeof($splitted)-1];
			
			return $wroteSomething;
		}
		else {return $wroteSomething;}
	}
	

	// $types is an array of the file extensions to display (for now dat only)
	// spaces in file or directory names are changed to $ in the checkbox value and in the action value.
	// if can't open directory or didn't write anything returns false
	// every shown item has a checkbox next to it if $showBos = true. in a form all items are in checkboxes[] array.
	// give encode false if you dont want the links to be encoded with 64base. leave it for encoding.
	function showExperimentContent($directory, $types, $showBox = true, $encode = true) {	
		$wroteSomething = false;		

		if (!is_dir($directory) ) {return false;}

		$table = getExpContentInfo($directory);

		if (!$table) { $table = array(); }
		ksort($table,SORT_STRING);

		$cleanFilesCol = false;
		$cleanReportCol = false;
		$clusterFilesCol = false;
		$clusterReportCol = false;
		$analysisCols = false;

		echo "<table style=\"width: 99%;\">";
		$varsTable = getVariablesTable($directory);
		// building the table header line
		echo "<tr bgcolor=\"#E8EEF7\">";
		$tableHeader = "<th>Name</th>";
		$usedFilesColStyle = "style=\"cursor: help; color: green;\" title=\"The files on this column are used for the analysis\"";
		$grayColStyle = "style=\"color: gray;\"";
		if (isConversionFolder($directory)) {
			$tableHeader = $tableHeader . "<th>Files</th><th>Conversion Report</th>";
			$cleanReportCol = true;
		} else {
			$tableHeaderNormalSource 	= $tableHeader . "<th>Source Files</th>";
			$tableHeader = $tableHeader . "<th ".$usedFilesColStyle.">Source Files</th>";                
			if (isset($varsTable[KEY_RUN_MODE]) && ($varsTable[KEY_RUN_MODE] == VAL_RUN_MODE_CLEAN_ONLY) && 
				($varsTable[KEY_CLEANING_THRESHOLD] != "Weak" &&
				 $varsTable[KEY_CLEANING_THRESHOLD] != "REMOVE_mendelian_infinity___DO_NOT_REMOVE_uninformative")){
				$tableHeader = $tableHeaderNormalSource . "<th ".$usedFilesColStyle.">Clean Files</th><th>Cleaning Report</th>";
				$cleanFilesCol = true;
				$cleanReportCol = true;
			}
			if (isset($varsTable[KEY_RUN_MODE]) && ($varsTable[KEY_RUN_MODE] == VAL_RUN_MODE_CLEAN_AND_CLUSTER)){
				$tableHeader = $tableHeaderNormalSource . "<th ".$usedFilesColStyle.">Clustered Files</th><th>Cleaning Report</th><th>Clustering Report</th>";
				$cleanReportCol = true;
				$clusterFilesCol = true;
				$clusterReportCol = true;
			}
			if (isset($varsTable[KEY_RUN_MODE]) && ($varsTable[KEY_RUN_MODE] == VAL_RUN_MODE_CLUSTER_ONLY)){
				$tableHeader = $tableHeaderNormalSource . "<th ".$usedFilesColStyle.">Clustered Files</th><th>Clustering Report</th>";

				$clusterFilesCol = true;
				$clusterReportCol = true;
			}
			if (isset($varsTable[KEY_RUN_ANALYSIS]) && ($varsTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_FALSE)){
				//$tableHeader = $tableHeader . "<th>Analysis Status</th><th>Analysis Results</th>";
				//$tableHeader = $tableHeader . "<th>Analysis Results</th>";
				$analysisCols = true;
			}
			if (isset($varsTable[KEY_RUN_MODE]) && ($varsTable[KEY_RUN_MODE] == VAL_RUN_MODE_CLEAN_ONLY) && isZoomingFolder($directory)) {
				$cleanReportCol = true;				
			}
			$tableHeader =  ( ( ($clusterFilesCol == true) || ($cleanFilesCol == true) ) ? $tableHeaderNormalSource : $tableHeader) . 
							("<th " . ($cleanFilesCol == true ? $usedFilesColStyle : $grayColStyle). ">Clean Files</th>") .
							("<th " . ($clusterFilesCol == true ? $usedFilesColStyle : $grayColStyle). ">Clustered Files</th>") .
							("<th " . ($cleanReportCol == true ? "" : $grayColStyle). ">Cleaning Report</th>") .
							("<th " . ($clusterReportCol == true ? "" : $grayColStyle). ">Clustering Report</th>") .
							("<th " . ($analysisCols == true ? "" : $grayColStyle). ">Analysis Results</th>");
		}
		echo $tableHeader."</tr>";					
		
		foreach ($table as $key => $subTable) {
			$fileName = $key;

			echo "<tr>";
			$wroteSomething = true;
			$file_nospaces = str_replace(" ", "$", $fileName);
			?><td style="padding-left: 10px; padding-right: 10px;" width="5%" bgcolor="#E8EEF7"><?php
				echo $fileName;
			?></td><?php
			echo "<td style=\"width: 18%; text-align: center; background-color: #E8EEF7\">"; //#f0ffff\">"; 
			echo "<span id=\"$fileName"."_Fs\">";
			echo getFilesLinks($directory, $fileName);
			AjaxJobs::$jobs[] = "'".$fileName."*Fs'";
			echo "</span>";
			echo "</td>";

			$Percents = array();

			if (!isConversionFolder($directory)) {
				echo "<td style=\"width: 18%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
				if ( ($cleanFilesCol) || ($clusterFilesCol) ) {
					$method = ($clusterFilesCol) ? "cluster" : "clean";		// cr = clustering  , cn = cleaning
					$method_short = ($clusterFilesCol) ? "cr" : "cn";		// cr = clustering  , cn = cleaning
					if ($clusterFilesCol){
						echo "</td>";
						echo "<td style=\"width: 18%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
					}
					echo "<span id=\"$fileName"."_$method_short"."Fs\">";
					echo getFilesLinks($directory, $fileName, $method);
					AjaxJobs::$jobs[] = "'".$fileName."*".$method_short."Fs'";
					echo "</span>";
					if ($cleanFilesCol) {
						echo "</td>";
						echo "<td style=\"width: 18%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
					}
				}
				else {
					echo "</td>";
					echo "<td style=\"width: 18%; text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
				}
				echo "</td>";
			}
			
			if (is_file($directory."/".PROGRESS_FILENAME)) {
				$Percents[0] = "none";
				$Percents[1] = "none";
				$Percents[2] = "none";
								
				if (getProgressPercents($directory, $fileName, $Percents)) {
					echo "<td style=\"text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">";                        
					if ($cleanReportCol){                            
						echo "<span id=\"$fileName"."_cn\">".$Percents[0]."</span>";                            
						echo "</td>";

						/*** setup ajax job this spot *****/							
							AjaxJobs::$jobs[] = "'".$fileName."*cn'";
						/**********************************/
					}
					else { echo "</td>"; }
					if (!isConversionFolder($directory)) {
						echo "<td style=\"text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 						
						if ($clusterReportCol) {
							echo "<span id=\"$fileName"."_cr\">".$Percents[1]."</span>";
							echo "</td>";
							
							/*** setup ajax job this spot *****/								
								AjaxJobs::$jobs[] = "'".$fileName."*cr'";
							/**********************************/							
						}
						else { echo "</td>"; }
					}
				}
			}
			else {
				echo "<td style=\"text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
				echo "</td>";
				echo "<td style=\"text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
				echo "</td>";
			}
			
			if (!isConversionFolder($directory)) {
				echo "<td style=\"text-align: center; background-color: #E8EEF7\">"; // ##f0ffff\">"; 
				echo "<span id=\"$fileName"."_aStts\">";
				echo getAnalysisStatus($directory, $fileName);
				AjaxJobs::$jobs[] = "'".$fileName."*aStts'";
				echo "</span>";
				echo "</td>";
			}
			
			echo "</tr>";
		}
		echo "</table>";
		
		if (sizeof(AjaxJobs::$jobs) > 0) {
			$splitted = explode("/",$directory);	// ./users/<name>/projects/<proj>/experiments/<exp>
												// ./users/<name>/projects/<proj>/formatted
			$proj 	= $splitted[4];
			$exp 	= $splitted[sizeof($splitted)-1];
			?> 
				<script language="JavaScript">					
					setInterval( "doWork('<?php echo $proj;?>', '<?php echo $exp;?>', new Array(<?php echo implode(",",AjaxJobs::$jobs);?>));", 10000 );  ///////// 10 seconds
				</script>
			<?php
		}		
		
		return $wroteSomething;
	}

	function HomozigosityExperimentContent($project) {
			$directory = USER_PROJECTS_DIR."/".$project."/experiments/".HOMOZIGOSITY_FOLDER_NAME;
			?><!--<table style="width: 99%;">
			<tr bgcolor="#E8EEF7">
				<td>-->
					<?php 
					if ( is_dir($directory) ){
						$table = getExpContentInfo($directory);
						if (!$table) { $table = array(); }
						ksort($table,SORT_STRING);
						foreach ($table as $key => $subTable){
							if (is_file($directory."/results_$key".".txt")){
								create_row($key, $project);		//$directory."/results_$key".".txt");
							}
						}
					}
					?>
				<!--</td>
			</tr>
			</table>-->
			<?php
	}
	
	/*
	* shows a content of a folder nicely. (mail like)
	* $actName - get action for main folder
	* $subFolderAct - get action for sub folders
	*/
	function showProjectsMain($directory, $actName = "", $subFolderAct = "", $page = 1) {
		$wroteSomething = false;
		if ( (!is_dir($directory)) || (!($handle = opendir($directory)))) {return $wroteSomething;}			
		
		
		while (($file = readdir($handle)) != false) {
			if (strpos($file,".") === 0){  //if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
				continue;
			}
			$contentArray[] = $file;
		}
		closedir($handle);
		
		?>
		<h1 class="b1f"></h1><b class="b2f"></b><b class="b3f"></b><b class="b4f"></b><div class="contentf">
		<table style="width: 100%; word-break: break-all;">
		<tr>
		<td style="text-align: center;"><div style="float: right; font-size: 95%; padding-right: 10px;">modified</div>
		<div style="float: left; padding-left: 20px"><strong>Project Name</strong></div></td>
		<!--<td style="text-align: center; width: 37%;"><strong>Uploaded files</strong></td>-->
		<td style="text-align: center; width: 15%;"><strong>Experiments</strong></td>
		</tr>
		
		<?php
		if ( (isset($contentArray)) && (sizeof($contentArray) != 0) ) {
			sort($contentArray);
			
			$max	=  ROWS_PER_PAGE * $page;
			$min	= (ROWS_PER_PAGE * ($page - 1)) + 1;
			$count	= 0;
			$contentArrayFiltered = array();
			
			foreach($contentArray as $element) {
				$count += 1;
				if ($count < $min) { continue; }
				if ($count > $max) { break; }
				$contentArrayFiltered[] = $element;
			}
			
			foreach($contentArrayFiltered as $file) {			
				
				$runningScript = $_SERVER["SCRIPT_NAME"];  // php file running name
				//$currentlyRunning = curPageURL();   // with the actions
				echo "<tr>";
				if (is_dir($directory."/".$file)){
					echo "<td bgcolor=\"#E8EEF7\" width=\"40%\">"; //#E8EEF7\">"; 
					$wroteSomething = true;
					$file_nospaces = str_replace(" ", "$", $file);

					?> <div style="float: left"> <?php
						echo "<input type=\"checkbox\" name=\"checkboxes[]\" value=\"".$file_nospaces."\" />";
						echo "<a href=".$runningScript."?".$actName."=".$file.">".$file."</a>";
						if (isset($_GET['demo']) && $_GET['demo'] && isDemoProject($file_nospaces)) {
							?><img src="images/hand.png"/><?php							
						 }
					?> </div>
					<div style="float: right; padding-right: 5px; font-size: 95%"> <?php
						$dateDir = $directory."/".$file;
						if (is_dir($dateDir."/experiments")) { $dateDir = $dateDir."/experiments"; }
						echo date("j\/m\/Y", filemtime($dateDir));
					?> </div> <?php
					
					
					echo "</td>";
					for ($i = 1 ; $i < 2 ; $i += 1) {		// change from 0 to have uploaded files preview as well
						echo "<td bgcolor=\"#E8EEF7\" style=\"text-indent: 5px;\">"; //#E8EEF7\">"; 
						$action = "";
						switch ($i){
							case 0: $subDir = $directory."/".$file."/raw"; 
									$action = "proj=$file_nospaces&input=true"; break;
							case 1: $subDir = $directory."/".$file."/experiments";
									$action = "proj=$file_nospaces&exp="; break;
							default: $subDir = $directory."/".$file."/raw";
									 $action = ""; break;
						}
						
						$preview = "";
						if ( (is_dir($subDir)) && ($handle = opendir($subDir))) {
							if (folderEmpty($subDir)) {
								//continue;
							}
							$lengthLimit = 40;
							//$experimentsLine = "";
							$experimentsOptions = "<select style=\"width: 300px\" onChange=\"if (this.options[this.selectedIndex].value != 0){
														window.top.location.href = this.options[this.selectedIndex].value;
													}\">";
							$experimentsOptions = $experimentsOptions."<option value=\"0\">--- Project Experiments ---</option>";
							while (($subContent = readdir($handle)) != false){
								if (strpos($subContent,".") === 0){  //if file/dir is hidden  *** can use strstr($subContent,".") if "." apears anywhere ***
									continue;
								}
								$preview = $preview." ".$subContent.",";

								if (strlen($preview) > $lengthLimit){
									$preview = substr($preview, 0, $lengthLimit);
									if ($i == 0){ break; }
								}
								
								if ($i == 1){	// showing experiments
									//$experimentsLine = $experimentsLine."<a href=".$runningScript."?".$action.$subContent.">".$subContent."</a>, ";
									$experimentsOptions = $experimentsOptions."<option value=\"$runningScript?$action$subContent\">$subContent</option>";
								}
							}
							
							$preview = substr($preview, 0, strlen($preview)-1)."...";
							//$experimentsLine = substr($experimentsLine, 0, strlen($experimentsLine)-2)."...";
							$experimentsOptions = $experimentsOptions."</select>";

							closedir($handle);
							
							if ( ($actName != "") && ($subFolderAct != "") ){
								switch ($i){
									case 0: echo "<a href=".$runningScript."?".$action.">".$preview."</a><br/>"; break;
									case 1: echo $experimentsOptions; break;
									default: echo ""; break;
								}
							}
							else{ echo $preview; }
						}
						echo "</td>";
					}
				}
			}
		}
		echo "</table>";
		?></div><b class="b4f"></b><b class="b3f"></b><b class="b2f"></b><b class="b1f"></b><?php
		return $wroteSomething;		
	}
	
	function showProject($project) {
		$dir = USER_PROJECTS_DIR."/".$project;
		$comments = is_file($dir."/".COMMENTS_FILE) ? file_get_contents($dir."/".COMMENTS_FILE) : "";
		$disabled = readOnlyProject($project) ? "disabled" : "";
		?>
		<div id="projectComments">
            Project Comments:<br/>
			<textarea rows="3" cols="80" id="comments" wrap="off" name="comments"><?php echo $comments; ?></textarea>
			<input id="saveComments" name="saveComments" type="hidden"/>
			<br/><input class="expBtn" type="submit" value="Save comment changes" <?php echo $disabled; ?> onclick="document.toolBar.method = 'POST';
																											  document.toolBar.submit();
																											  return false;"/>
		</div>
		<br/>
		<?php
		$runningScript = $_SERVER["SCRIPT_NAME"];  // php file running name
		echo "<a href=".$runningScript."?proj=".$project."&input=true>Uploaded Data files</a><br/>";
		echo "<a href=".$runningScript."?proj=".$project."&exp=.formatted>Formatted Data files (dat, ped files)</a><br/><br/>";
		echo "Experiments:<br/><br/>";
        
        #Show select buttons
        $formName = "toolBar";
        ?>
        <div style="padding-left: 5px; padding-right: 5px;">
            <?php showSelectButtonsDiv($formName); ?>
        </div>
        <?php        
        
		showExperimentsTable($project);
	}
	
	function projectsCount() {
		$directory = USER_PROJECTS_DIR;
		if ( (!is_dir($directory)) || (!($handle = opendir($directory)))) {return 0;}

		$count = 0;	
		while (($file = readdir($handle)) != false){
			if (strpos($file,".") === 0) {continue;}  //if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
			$count += 1;
		}
		closedir($handle);
		return $count;
	}
	
	function getUserProjects() {
		$directory = USER_PROJECTS_DIR;
		$projectsArray = array();
		if ( (!is_dir($directory)) || (!($handle = opendir($directory)))) {
			return $projectsArray;
		}

		while (($file = readdir($handle)) != false){
			if (strpos($file,".") === 0){  //if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
				continue;
			}
			$projectsArray[] = $file;
		}
		closedir($handle);
		return $projectsArray;	
	}
	
	/* shows the experiment tool bar. set deleteOnly to true if the toolbar should not contain the tools boxes */
	function showToolBar($projName = false, $expName = false, $subexpName = false, $projectInput = false) {
		$stopped = is_file(USER_PROJECTS_DIR."/".$projName."/experiments/".$expName."/".EXP_STOPPED_FILE);
		$disabled = ($projName && (readOnlyProject($projName))) ? "disabled" : "";
		if ($expName && isReadOnlyExperiment($projName, $expName)){
			$disabled = "disabled";
		}
		$help = new Help();
		$formName = "toolBar";
		?>
		<p>
		<table style="width: 99%;">
		<tr>
		<td>
			<div style="padding-left: 5px">
			<input type="hidden" name="project" value="<?php echo $projName;?>"/>
			<input type="hidden" name="experiment" value="<?php echo $expName;?>"/>
			<input type="hidden" name="subexp" value="<?php echo $subexpName;?>"/>
			<input type="hidden" name="input" value="<?php echo $projectInput;?>"/>
			
			<?php
				if (!$projName || $projectInput || $subexpName){ ?>
					<input class="expBtn" type="submit" name="delete" value="Delete" <?php echo $disabled; ?> onclick="return confirm('This will DELETE the selected items permanently.\nAre you sure?');"/>
			<?php }
				if ( $projName && $projectInput ){ ?>
					<input class="expBtn" type="submit" name="mapping" value="Mapping" />
			<?php }
				if ( ($projName != false) && (!$expName) && (!$projectInput) ) { /*if it is the main page of a project*/ ?>

				<?php
					//if (is_file(USER_PROJECTS_DIR."/".$projName."/.inWizard") || is_file(USER_PROJECTS_DIR."/".$projName."/.inWizardDat")){
					?>
						<!--<input class="expBtnSpecial" type="submit" name="wizardReturn" value="Return to The Wizard"/>-->
					<?php //}
					?>
					<input class="expBtn" type="submit" name="pedigree" value="Show Pedigree"/>&nbsp
					<input class="expBtn" type="submit" name="dataBrowser" value="DataBrowser" <?php echo $disabled; ?>/>&nbsp
					<input class="expBtn" type="submit" name="archive" value="Download"/>&nbsp
					<input class="expBtn" type="submit" name="delete" value="Delete Selected" <?php echo $disabled; ?> onclick="return confirm('This will delete the selected items.\nExperiments bellow deleted items will be highlighted.\nAre you sure?');"/>&nbsp
					<input class="expBtn" type="submit" name="restore" value="Restore Deleted" <?php echo $disabled; ?>/>&nbsp
					<input class="expBtn" type="submit" name="copy" value="Copy"/> <!--onclick="return confirm('This will create a new project with the same input.\nAre you sure?');"-->&nbsp					
					
			<?php }
				if ( ($projName != false) && ($expName != false) && (!$projectInput) && (!$subexpName) ) {
					$formName = "getElementById('expAction').form";
					?>
					<!--<input class="expBtn" type="submit" <?php echo $stopped ? "disabled" : ""; ?> name="stop" value="Stop" onClick="return confirm('This will stop the running of the experiment.\nAre you sure?');"/>
					<input class="expBtn" type="submit" name="archive" value="Archive"/>
					&nbsp;&nbsp;&nbsp;&nbsp;-->
					<select style="border: 1px solid;" id="expAction" name="expAction" <?php echo $disabled; ?>>
						<option value="" selected><strong>Tools &#9658;</strong></option>
						<optgroup label="Exact Analysis">
							<option value="twoPoint">Two-point analysis</option>
							<option value="multiPoint">Multi-point analysis</option>
							<option value="haplotyping">Haplotyping</option>
							<!--<option value="segregation">Segregation</option>-->
						</optgroup>
						<optgroup label="Approximate Analysis">
							<option value="lm_markers">Morgan lm_linkage (Approximate Multi-point analysis)</option>
							<!-- <option value="gl_auto">Morgan gl_auto (Approximate Haplotyping analysis)</option> -->
						</optgroup>
						<optgroup label="Data Reduction">
							<option value="clustering">Cluster markers</option>
							<option value="filtering">Filter markers</option>
							<option value="cleaning">Remove erroneous markers</option>							
						</optgroup>
						<optgroup label="Zooming">
							<option value="simpleZoom">Zoom on markers</option>
						</optgroup>						
					</select>
					<input class="expBtn" type="submit" name="go" value="Go" <?php echo $disabled; ?> onclick="return legalToolBarOperation(this.form, getElementById('expAction').value);"/>
					<?php $help->displayHelpLink("projects", "ToolBar"); ?>
				<?php } ?>
			</div>
		</td>
		</tr>
		<tr>
		<td>
            <?php
            if (!$projName) {			// means we're in the main projects page
                ?>
                <div style="padding-left: 5px; padding-right: 5px;">
                    <?php showSelectButtonsDiv($formName); ?>
                    <div style="float: right;"><?php
                            showPaging(projectsCount());
                            ?>
                    </div>
                </div>
            <?php
            }
            ?>
		</td>
		</tr>
		</table>
		</p>
		<div style="clear: both;"></div>
		<?php		
	}
    
    
    function showSelectButtonsDiv($formName) {        
        ?>
        <div style="float: center;">
        Select:            
        <a href="projects.php" onClick="switchCheckAll(document.<?php echo $formName;?>, true); return false;">All</a>,            
        <a href="projects.php" onClick="switchCheckAll(document.<?php echo $formName;?>, false); return false;">None</a>
        </div>
        <?php        
    }
	
	// shows paging according to the number of give $totalCount. put in Div and place where ever u want.
	// $totalCount is the amount of total elements in the list to page.
	function showPaging($totalCount) {
		$page 	= isset($_GET['page']) ? $_GET['page'] : 1;
		$pages	= ceil($totalCount / ROWS_PER_PAGE);
		$page	= ($page > $pages) ? $pages : $page;
		$nextPage	= ($page+1 > $pages) ? $pages : $page+1;
		$prevPage	= ($page-1 < 1) ? 1 : $page-1;
		?>
		
			page: 
			<select onChange="window.top.location.href = 
					'<?php echo $_SERVER["SCRIPT_NAME"];?>?page=' + this.options[this.selectedIndex].value;">
				<?php
				for ($i = 1 ; $i <= $pages ; $i += 1){
					$sel = ($page == $i) ? "selected " : "";
					echo "<option value=\"$i\" $sel>$i</option>";
				} 
				?>
			</select>
			 out of <?php echo $pages;?>&nbsp;&nbsp;&nbsp;
			<a style="text-decoration: none;" href="<?php echo $_SERVER["SCRIPT_NAME"];?>?page=<?php echo $prevPage;?>">&lt; prev </a>
			<a style="text-decoration: none;" href="<?php echo $_SERVER["SCRIPT_NAME"];?>?page=<?php echo $nextPage;?>"> next &gt;</a>
		<?php
	}
	
	// shows the content of a columns file.
	// requires the structure of spaces/tabs separating columns
	// $amount is the amount of colums to show, if not given, will show all
	function showColsFile($file, $amount = false) {
		if (!is_file($file) || (!$content = fopen($file, "r"))){
			return;
		}
		
		echo "<table style=\"text-align: right;\">";
		while (!feof($content)){
			$line = rtrim(fgets($content));
			if ($line == "") { continue; }
			$fcontent = preg_split("/[ \t]/", $line);
			echo "<tr>";
			$colsToShow = ($amount) ? $amount : sizeof($fcontent);
			for ($i = 0; $i < $colsToShow ; $i += 1){
				echo "<td>&nbsp;".$fcontent[$i]."&nbsp;</td>";
			}
		}
		echo "</table>";
	}    
    
    //Omer 31/7/2011 - create a parameters file for Morgan lm_markers
    function createLMMarkersParametersFile($dir) {
		if (!($paramsFile = fopen($dir."/".MORGAN_PARAMETERS_FILE,"w"))) { return false;} //error...			
		
        if (isset($_GET['MorganAdvancedOptions']) and $_GET['MorganAdvancedOptions']) {
            if (isset($_GET['LSamplerProbability'])) {
                fwrite($paramsFile, "set L-sampler probability  ".$_GET['LSamplerProbability']."\n");
            }
            if (isset($_GET['MeiosesSampler'])) {
                fwrite($paramsFile, "use ".$_GET['MeiosesSampler']." meiosis sampler\n");
            }
			else {fwrite($paramsFile, "use multiple meiosis sampler\n");}
            if (isset($_GET['MeiosesInitialization'])) {
                fwrite($paramsFile, "use ".$_GET['MeiosesInitialization']." for setup\n");
            }            
		}
		
		#Default lm_markers options
		else {			
			fwrite($paramsFile, "use multiple meiosis sampler\n");
			fwrite($paramsFile, "use locus-by-locus sampling for setup\n");
			fwrite($paramsFile, "set L-sampler probability 0.5\n");
		}
		
        fclose($paramsFile);
		system("chmod 777 $dir/".MORGAN_PARAMETERS_FILE);
		return true;        
    }
	
	// creates a variables file, deletes an existing one.
	function createVariablesFile($dir, &$variablesTable) {
		//$oldTable = getVariablesTable($dir);
		if (!($varsFile = fopen($dir."/".VARIABLES_FILE_NAME,"w"))){
			//error...
			return false;
		}        
		foreach ($variablesTable as $key=>$value) {fwrite($varsFile, $key." ".$value."\n");}		
		
		fclose($varsFile);
		system("chmod 777 $dir/".VARIABLES_FILE_NAME);
		return true;
	}
	
	function getVariablesTable($dir) {				// change input to: experiment name ?
		if ( !is_dir($dir) || (!is_file($dir."/".VARIABLES_FILE_NAME)) ||
			 !($linesArray = file($dir."/".VARIABLES_FILE_NAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) ){			
			return false;	//error...
		}
		
		$varsTable = array();
		for ($i = 0; $i < sizeof($linesArray); $i += 1) {
			$varsArray = explode(" ", $linesArray[$i]);
			if ( isset($varsArray[0]) && isset($varsArray[1]) ) {
				if (!isset($varsTable[$varsArray[0]])) {
					$varsTable[$varsArray[0]] = $varsArray[1];
					for ($j=2; $j<sizeof($varsArray); $j++) {
						$varsTable[$varsArray[0]] = $varsTable[$varsArray[0]]." ".$varsArray[$j];
					}
				}
			}
		}
		
		return $varsTable;
	}
	
	/*
	 * creates a file with parameters about the project that will be 
	 * displayed to the useer
	 * input:
	 *	$dir - the project directory
	 *	$guiVarTable - the values to write, when the key is the title
	 *					and it's value is the value will be writen
	 */
	function createGuiVarFile($dir, &$guiVarTable) {
		if (!($varsFile = fopen($dir."/".VARS_FOR_GUI,"w"))) { return false; }        
		foreach ($guiVarTable as $key=>$value) {fwrite($varsFile, $key." ".$value."\n");}		
		fclose($varsFile);
		system("chmod 777 $dir/".VARS_FOR_GUI);
		
		return true;
	}
	
	/*
	 * read the file with parameters about the project that will be displayed to the useer
	 * input:
	 *	$dir - the project directory
	 */
	function getGuiVarFile($dir) {
		if ( !is_dir($dir) || (!is_file($dir."/".VARS_FOR_GUI)) ||
			 !($linesArray = file($dir."/".VARS_FOR_GUI, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) ) {			
			return false;
		}
		
		$varsTable = array();
		for ($i = 0; $i < sizeof($linesArray); $i += 1){
			$varsArray = explode(" ", $linesArray[$i]);
			if ( isset($varsArray[0]) && isset($varsArray[1]) ){
				if (!isset($varsTable[$varsArray[0]])) {$varsTable[$varsArray[0]] = $varsArray[1];}				
			}
		}
		
		return $varsTable;
	}
	
	function markExperimentFolder($dir) {
		if (!is_dir($dir) || (!($p = fopen($dir."/.experimentMark",'w')))) {return false;}		
		fclose($p);
		return true;
	}
	
	function isExperimentFolder($dir) {
		if (is_file($dir."/.experimentMark")) {return true;}
		return false;
	}
	
	function markConversionFolder($dir) {
		if (!is_dir($dir) || (!($p = fopen($dir."/.conversionMark",'w')))) {return false;}
		fclose($p);
		return true;
	}
	
	function isConversionFolder($dir) {
		if (is_file($dir."/.conversionMark")) {return true;}
		return false;
	}
	
	function markSegregationFolder($dir) {
		if (!is_dir($dir) || (!($p = fopen($dir."/.segregationMark",'w')))) {return false;}
		fclose($p);
		return true;
	}	
	
	function isSegregationFolder($dir) {
		if (is_file($dir."/.segregationMark")) {return true;}
		return false;
	}	
	
	function markZoomingFolder($dir) {
		if (!is_dir($dir) || (!($p = fopen($dir."/.ZoomingMark",'w')))) {return false;}
		fclose($p);
		return true;
	}
	
	function isZoomingFolder($dir) {
		if (is_file($dir."/.ZoomingMark")) {return true;}
		return false;
	}		
	
	
	function isLogicallyDeletedFolder($dir) {
		if (!is_dir($dir) || is_file($dir."/".DELETED_FLAG)) {return true;}
		return false;
	}
	
	function readOnlyProject($project) {
		$dir = USER_PROJECTS_DIR."/".$project;
		if (is_dir($dir) && is_file($dir."/.readOnly")) {return true;}
		return false;
	}
	
	function isDemoProject($project) {
		$dir = USER_PROJECTS_DIR."/".$project;
		if (is_dir($dir) && is_file($dir."/.demoProject")) {return true;}
		return false;
	}	

	function isReadOnlyExperiment($project, $experiment) {
		$dir = USER_PROJECTS_DIR."/$project/experiments/$experiment";
		if (is_dir($dir) && is_file($dir."/.readOnly")) {return true;}
		return false;
	}
	
	function isArchived($directory) {
		if (is_file($directory."/.archive") && is_file($directory."/.readOnly")) {
			$archive = file($directory."/.archive", FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
			if (is_file($archive[0])) {return true;}
		}
		return false;
	}
	
	function markAccess($dir) {
		if (is_file($dir."/.accessed")) {chmod($dir."/.accessed", 0776);}
		if (!is_dir($dir) || (!($p = fopen($dir."/.accessed",'w')))) {return false;}
		fclose($p);
		chmod($dir."/.accessed", 0776);
		return true;
	}
	
	/* This function creates an experiment folder named given $expName under the given $dir
	* provide the flags according to the experiment you're creating.
	* A full experiment (clustering+analysis) is already set as default for this function (cleanOnly = false, clustering = true, analysis = true)
	* sub-folders for this experiment will be created according to the given flags.
	* if $dir does not exist or the experiment already exists, the function will return false. otherwise - true.
	*/
	function createExperimentFolder($dir, $expName, $cleanFolderFlag = false, $clusteringFlag = true, $analysisFlag = true, $specialName = false) {
		if (!is_dir($dir) || is_dir($dir."/".$expName)) {return false;}
		
		if (!createFolder($dir, $expName, $specialName)) { return false; }
		exec("chmod 777 $dir/$expName");
		markExperimentFolder($dir."/".$expName);		
		
		if ($cleanFolderFlag) {			createFolder($dir."/".$expName, "clean");}
		if ($clusteringFlag) {createFolder($dir."/".$expName, "clustered");}		
		if ($analysisFlag) {			
			$resFile = $dir."/".$expName."/".RESULTS_FILENAME;
			system("touch ".$resFile);
			exec("echo \"Results are not yet ready for this experiment.\" >> $resFile");		
			exec("chmod 776 $resFile");
		}		
		return true;
	}
	
	
	/*
	 * shows the info of the experiment in the given directory
	 */
	function showExpInfo($project, $experiment) {
		$dir = USER_PROJECTS_DIR."/".$project."/experiments/".$experiment;
		if (!isExperimentFolder($dir)) {return;}
		
		$expInfoTable[0][0] = "";
		$expInfoTable[0][1] = "";
		$expInfoTable[1][0] = "";
		$expInfoTable[1][1] = "";

		$variablesTable = getVariablesTable($dir);
		if (!($variablesTable)) { return; }

		$doClean = false;
		$doCluster = false;
		$doAnalyze = false;
		$analysisType = false;
		$doZooming = false;
		$doFiltering = false;
		$tool = "";

		if (isset($variablesTable[KEY_RUN_MODE])) { //cleaning, clustering or filtering
			switch ($variablesTable[KEY_RUN_MODE]) {
				case VAL_RUN_MODE_CLEAN_ONLY:
					if (isset($variablesTable[KEY_CLEANING_THRESHOLD]) && $variablesTable[KEY_CLEANING_THRESHOLD] == "Strong") {
						$doClean = true;
					}
					break;
				case VAL_RUN_MODE_CLUSTER_ONLY:			$doClean = false; $doCluster = true; break;
				case VAL_RUN_MODE_CLEAN_AND_CLUSTER:	$doClean = true; $doCluster = true; break;
				case VAL_RUN_MODE_FILTERING:			$doFiltering = true; break;
			}
		}

		if (isset($variablesTable[KEY_RUN_ANALYSIS])) { //other analysis than cleaning\clustering
			if ($variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_FALSE) { $doAnalyze = false; }
			else {
				$tool = $variablesTable[KEY_RUN_ANALYSIS];
				$doAnalyze = true;
			}
		}

		if (isZoomingFolder($dir)) {$doZooming = true;}

		$clean_thresh = isset($variablesTable[KEY_CLEANING_THRESHOLD])  ? $variablesTable[KEY_CLEANING_THRESHOLD]	: 	"none";
		$cluster_dist = (isset($variablesTable[KEY_CLUSTERING_DISTANCE]) ? $variablesTable[KEY_CLUSTERING_DISTANCE]	:	"(default) 10") . " cM";

		$file = false;
		$foundDat = false;
		$penetrance = "";
		$frequency = "";
		if ($handle = opendir($dir)) {
			while (($file = readdir($handle)) != false) {
				//if file is hidden or its not a file
				if ( (strpos($file,".") === 0) || (!is_file($dir."/".$file)) ) { continue; }
				if (preg_match("/^.*\.dat$/",$file)) {
					$foundDat = true;
					break;		// break on that directory
				}
			}
		}
		if (!$foundDat) {
			if (isset($variablesTable[KEY_PENETRANCE])){
				$penet = explode("_",$variablesTable[KEY_PENETRANCE]);
				$penetrance = "(default) 0.99, 0.99";
				$MOI = ($penet[0] == 0) ? "recessive" : "dominant";
				$penetrance = $MOI." ".$penet[0]." ".$penet[1];
			}
			
			if (isset($variablesTable[KEY_FREQUENCY])) { $frequency = $variablesTable[KEY_FREQUENCY]; }
		}
		else {
			$MOI = getMOIFromDat($dir."/".$file);
			$penetrance = floatval($MOI[0])." ".floatval($MOI[1])." ".floatval($MOI[2]);
			$frequency = floatval(getFrequencyFromDat($dir."/".$file));
		}
		
		$expInfoTable[1][0] = isSegregationFolder($dir) ? "" : "Disease Frequency: ".$frequency."<br/>MOI: ".$penetrance."<br/>";		
		if ($doClean == true) {
			$expInfoTable[0][1] = "Cleaning";
			$expInfoTable[1][1] = "Removing:<br/>".(($clean_thresh == "Strong") ? "- Mendelain errors<br/>- Unlikely markers" : "- Mendelian errors")."<br/>";
		}
		if ($doCluster == true){
			$expInfoTable[0][1] = "Clustering";
			$expInfoTable[1][1] = "Distance: ".$cluster_dist."<br/>";
		}
		if ($doAnalyze == true) {
			$expInfoTable[0][1] = toolDisplayName($tool);
		}
		if ($doFiltering == true) { $expInfoTable[0][1] = "Filtering"; }
		if ($doZooming == true) { $expInfoTable[0][1] = "Zooming"; }
		$expInfoTable[1][1] = $expInfoTable[1][1].moreToolDetail($variablesTable, $dir);
		
		
		$status  = getExperimentStatus($project, $experiment);
		$expStatus = "";
		switch ($status){
			case EXP_STATUS_STOPPED: 	$expStatus = "<span id=\"stts\" style=\"color: red;\">Stopped</span>"; break;
			case EXP_STATUS_RUNNING: 	$expStatus = "<span id=\"stts\" style=\"color: green;\">Running...<img src=\"images/waitspinner.gif\"/ border=\"0\"></span>"; break;
			case EXP_STATUS_FINISHED: 	$expStatus = "<span id=\"stts\" style=\"color: black;\">Finished</span>"; break;
			default: 					$expStatus = "<span id=\"stts\" style=\"color: green;\">Running...</span>"; break;
		}

		/*** setup ajax job this spot *****/
		AjaxJobs::$jobs[] = "'*stts'";
		/**********************************/

		$stopped = ($status != EXP_STATUS_RUNNING);
		$resultsAvailable = false;
		if ( ($tool == VAL_RUN_ANALYSIS_TWOPOINT) || ($tool == VAL_RUN_ANALYSIS_MULTIPOINT) ||
				($tool == VAL_RUN_ANALYSIS_MULTIPOINT_HMM) || ($tool == VAL_RUN_ANALYSIS_LM_MARKERS)) {
			$resultsAvailable = true;
		}
		?>
		<div style="border: 0px; float: left;">
			<h3>Experiment name: <?php echo $experiment;?></h3>
			<b>Status: </b><?php echo $expStatus;?><br/><br/>
				<input type="hidden" name="project" value="<?php echo $project;?>"/>
				<input type="hidden" name="experiment" value="<?php echo $experiment;?>"/>
				<input <?php echo $resultsAvailable ? "" : "disabled"; ?> class="expBtn" type="button" name="viewResults" value="View Results"
				       onclick="window.open('viewExpGraphs.php?project=<?php echo $project;?>&experiment=<?php echo $experiment;?>','_blank'); return false;"/>&nbsp
				<input class="expBtn" type="submit" name="archive" value="Download"/>&nbsp
				<input <?php echo ($status == EXP_STATUS_RUNNING) ? "disabled" : ""; ?> class="expBtn" type="submit" name="dataBrowser" value="DataBrowser"/>&nbsp
				<input class="expBtn" type="submit" name="stop" value="Stop" <?php echo $stopped ? "disabled" : ""; ?> onClick="return confirm('This will stop the experiment.\nAre you sure?');"/>&nbsp
			<p>
			Experiment details:
			<br/>
			<table class="experiment">
			<tr>
				<?php for ($i = 0 ; $i <= 1 ; $i += 1) { ?><th><?php echo $expInfoTable[0][$i]; ?></th><?php } ?>
			</tr>
			<tr style="text-align: left;">
				<?php for ($i = 0 ; $i <= 1 ; $i += 1) { ?><td><?php echo $expInfoTable[1][$i]; ?></td><?php } ?>
			</tr>
			</table>
			</p>
		</div>			
		
		<div style="clear: both;"></div>
		<?php
	}
	
	/*
	 * this function fills the table in the project display window in case that the experiment is
	 * an analysis.
	 */
	function moreToolDetail (&$variablesTable, $expDir) {
		$br = "<br \/>";
		$analysisDetails = "";
		//display data about the markers
		if (isSet($variablesTable[KEY_MARKERS_LIST])) {
			$analysisDetails = $analysisDetails."Analysis markers: ".$variablesTable[KEY_MARKERS_LIST].$br;
		}
		elseif (isSet($variablesTable[KEY_START_MARKER])  && isSet($variablesTable[KEY_END_MARKER])) {
			if ($variablesTable[KEY_START_MARKER] == "" && $variablesTable[KEY_END_MARKER] == "") { 
				$analysisDetails = $analysisDetails."Analysis range: all markers".$br;
			} elseif ($variablesTable[KEY_START_MARKER] != "" && $variablesTable[KEY_END_MARKER] == "") {
				$analysisDetails = $analysisDetails."Analysis range: marker ".$variablesTable[KEY_START_MARKER]." to last marker".$br;
			} elseif ($variablesTable[KEY_START_MARKER] == "" && $variablesTable[KEY_END_MARKER] != "") {
				$analysisDetails = $analysisDetails."Analysis range: marker 1 to marker ".$variablesTable[KEY_END_MARKER].$br;
			} else { //if ($variablesTable[KEY_START_MARKER] != "" && $variablesTable[KEY_END_MARKER] != "")
				$analysisDetails = $analysisDetails."Analysis range: marker ".$variablesTable[KEY_START_MARKER]." to marker ".$variablesTable[KEY_END_MARKER].$br;
			}
		} else {
			if ( (isSet($variablesTable[KEY_RUN_ANALYSIS]) && ($variablesTable[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_SEGREGATION))
					|| (isSet($variablesTable[KEY_RUN_MODE]) && ($variablesTable[KEY_RUN_MODE] != VAL_RUN_MODE_ZOOM)) ) {
				$analysisDetails = $analysisDetails."Analysis range: all markers".$br;
			}
		}

		//display data about the number of iterations
		if (isSet($variablesTable[KEY_MIN_NUM_ITERATIONS]) && isSet($variablesTable[KEY_MAX_NUM_ITERATIONS])) {
			//if the limit is at the number of iterations
			if ($variablesTable[KEY_MIN_NUM_ITERATIONS] == $variablesTable[KEY_MAX_NUM_ITERATIONS]) {
				$analysisDetails = $analysisDetails."Number of iterations: ".$variablesTable[KEY_MIN_NUM_ITERATIONS].$br;
			} else { //the limit on tha number of iteration is by time
				$time = $variablesTable[KEY_TIME_OF_WINDOW_IN_SECONDS]; // time to run analysis in secounds
				$days = floor($time / 86400); // 86400 - num of secounds in day
				$time = $time % 86400; // secounds left
				$hours = floor($time / 3600);
				$time = $time % 3600; // secounds left
				$minutes = floor($time / 60);
				$secounds = $time % 60; // secounds left
				$analysisDetails = $analysisDetails."Analysis run time (d h:m:s): ".$days." ".$hours.":".$minutes.":".$secounds.$br;
			}
		}
		
		//display data about the size of the window
		if (isSet($variablesTable[KEY_WINDOW_SIZE]) && isSet($variablesTable[KEY_RUN_ANALYSIS])
				&& ($variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_MULTIPOINT
				|| $variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_MULTIPOINT_HMM
				|| $variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_LM_MARKERS
				|| $variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_GL_AUTO
				|| $variablesTable[KEY_RUN_ANALYSIS] == VAL_RUN_ANALYSIS_HAPLOTYPING)) {
			$analysisDetails = $analysisDetails."Window size: ".$variablesTable[KEY_WINDOW_SIZE].$br;
		}
		
		// try to read the gui parameters file
		$guiVars = getGuiVarFile($expDir);
		if ($guiVars != false) {
			if (isSet($variablesTable[KEY_RUN_MODE]) && ($variablesTable[KEY_RUN_MODE] == VAL_RUN_MODE_ZOOM)) { //zooming has spetial parameters
				if ($guiVars[ZOOMING_RANGE_TYPE] == "strings" || $guiVars[ZOOMING_RANGE_TYPE] == "numbers") {
					$msg = "Zooming from marker ".$guiVars[ZOOMING_START]." to marker ".$guiVars[ZOOMING_END];
				} else { // $guiVars[ZOOMING_RANGE_TYPE] == "positions"
					$msg = "Zooming from position ".$guiVars[ZOOMING_START]." to position ".$guiVars[ZOOMING_END];
				}
				$msg = $msg.$br;
				if (isSet($variablesTable[ZOOMING_EXPAND_REGION])) {
					$msg = $msg."More markers were added in this region".$br;
				}
				$analysisDetails = $analysisDetails.$msg;
			}
		
			//spetial  filtering options
			if (isSet($guiVars[FILTERING_DISTANCE])) {
				$analysisDetails = $analysisDetails."Filtering distance: ".$guiVars[FILTERING_DISTANCE]."cM".$br;
			}
			//spetial morgan options
			if (isSet($guiVars[MORGAN_ADVANCED_OPTIONS])) {
				if ($guiVars[MORGAN_ADVANCED_OPTIONS] == "0") { //used default options
					$analysisDetails = $analysisDetails."Morgan options: default".$br;
				} else {
					$analysisDetails = $analysisDetails."Percentage of burn-in iterations: ".$guiVars[MORGAN_BURN_IN_ITERATIONS].$br;
					$analysisDetails = $analysisDetails."L-sampler probability: ".$guiVars[MORGAN_L_SAMPLER_PROBABILITY].$br;
					$analysisDetails = $analysisDetails."Meioses indicators initialization mode: ".$guiVars[MORGAN_MEIOSES_INITIALAZATION].$br;
					if (isSet($guiVars[MORGAN_IMPUTATION_REALIZATIONS])) {
						$analysisDetails = $analysisDetails."Percentage of sequential imputation realizations for setup: ".$guiVars[MORGAN_IMPUTATION_REALIZATIONS].$br;
					}
					$analysisDetails = $analysisDetails."Meiosis sampler mode: ".$guiVars[MORGAN_MEIOSES_SAMPLER].$br;
				}
			}
		}
		
		return $analysisDetails;
	}
	
	// shows the experiments table in a given project (of a specific user name in the session)
	function showExperimentsTable($project) {
		$dir = USER_PROJECTS_DIR."/".$project."/experiments";
		$runningScript = $_SERVER["SCRIPT_NAME"];
		
		?>
		<p>
		<table style="width: 99%; text-indent: 5px; ">
		<tr style="font-weight: bold;">
			<td></td><td bgcolor="#E8EEF7" width="20%">Experiment Results</td><td bgcolor="#E8EEF7" width="10%">Modified</td><td bgcolor="#E8EEF7">Operations</td><td bgcolor="#E8EEF7">Status</td>
		</tr>
		
		<?php
		if ( (!is_dir($dir)) || (!($handle = opendir($dir)))) {
			echo "</table></p>";
			return false;
		}
		$contentArray = array();
		while (($file = readdir($handle)) != false) {
			//if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
			if (strpos($file,".") === 0) { continue; }
			//$contentArray[] = $file;
			$inf['expName'] = $file;
			$inf['depth'] = 0;
			$inf['parent'] = ".formatted";
			$contentArray[] = $inf;            
		}
		closedir($handle);
		
		if ( (isset($contentArray)) && (sizeof($contentArray) != 0) ) {
			//sort($contentArray);
			$contentArrayDerive = sortExperimentsByDerive($project);
			if ($contentArrayDerive){ $contentArray = $contentArrayDerive; }
			else { orderBy($contentArray, 'expName'); }				
			
			foreach($contentArray as $experiment) {
				$exp    = $experiment['expName'];                
				$dot	= isLogicallyDeletedFolder($dir."/".$experiment['parent']) ? "&bull;" : "&nbsp;";
				$indent = str_repeat("&nbsp;", 5*$experiment['depth']);
				if (is_dir($dir."/".$exp) && isExperimentFolder($dir."/".$exp)){
					$opsArr = getExpOperations($dir."/".$exp);
					$ops = array();
					$params = array();
					foreach ($opsArr as $val => $key) {
						$ops[] = $val;
						$params[] = $key;
					}
					$ops 	= implode(" | ", $ops);
					$params = implode("&#10;", $params);
					$variablesTable = getVariablesTable($dir."/".$exp);
					$title = "Parameters:&#10;".$params;
					
					$expStatus = "";
					switch (getExperimentStatus($project, $exp)) {
						case EXP_STATUS_STOPPED: 	$expStatus = "<span id=\"".$exp."_projstts\" style=\"color: red;\">Stopped</span>"; break;
						case EXP_STATUS_RUNNING: 	$expStatus = "<span id=\"".$exp."_projstts\" style=\"color: green;\">Running...</span>"; break;
						case EXP_STATUS_FINISHED: 	$expStatus = "<span id=\"".$exp."_projstts\" style=\"color: black;\">Finished</span>"; break;
						default: 					$expStatus = "<span id=\"".$exp."_projstts\" style=\"color: green;\">Running</span>"; break;
					}					
					
					if (!isLogicallyDeletedFolder($dir."/".$exp)) {
						AjaxJobs::$jobs[] = "'".$exp."*".$exp."_projstts'";
						?>
						<tr style="background-color: #E8EEF7;">
							<td style="width: 10px;"><input type="checkbox" name="checkboxes[]" value="<?php echo $exp;?>"/></td>
							<td><?php echo $indent.$dot; ?><a href="<?php echo $runningScript;?>?proj=<?php echo $project;?>&exp=<?php echo $exp;?>"><?php echo $exp; ?></a></td>
							<td><?php echo date("j\/m\/Y", filemtime($dir."/".$exp)); ?></td>
							<td><p style="cursor: help;" title="<?php echo $title;?>" class="noDecoration" onMouseOver="self.status=''; return true;"><?php echo $ops;?></p>
							<?php if (is_file($dir."/".$exp."/".AUTORUN_SUMMARY_FILE)) { ?>
								&nbsp;&nbsp;&nbsp;(<a href="<?php echo "$dir/$exp/".AUTORUN_SUMMARY_FILE;?>">autorun summary</a>)
							<?php } ?>
							</td>
							<td><?php echo $expStatus;?></td>
						</tr>
						<?php
					}
				}
			}
		}
		?>
		
		</table>
		</p>
		<?php
		
		if (sizeof(AjaxJobs::$jobs) > 0) {
			$exp 	= "";			
			?> 
				<script language="JavaScript">					
					setInterval( "doWork('<?php echo $project;?>', '<?php echo $exp;?>', new Array(<?php echo implode(",",AjaxJobs::$jobs);?>));", 10000 );  ///////// 10 seconds
				</script>
			<?php
		}
		
		return true;
	}
	
	function showArchiveNotice($project, $experiment) {
		$dir = USER_PROJECTS_DIR."/".$project."/experiments/".$experiment;
		if (isArchived($dir) && isReadOnlyExperiment($project, $experiment)){
			?>
			<p>
			<b>Notice</b>: This experiment has been automatically archived due to lack of usage.<br/>
			Get full experiment access back, by clicking 
			<a href="toolBar.php?project=<?php echo $project;?>&experiment=<?php echo $experiment;?>&unarchive=true">Here</a>.
			</p>
			<?php
		}	
	}
	
	// $project is the project name, and $experiments is the array of experiments of this project.
	// $project is the project name, and $experiments is the array of experiments of this project.
	function sortExperimentsByDerive($project) {
		$dir = USER_PROJECTS_DIR."/".$project."/experiments";
		if (!is_dir($dir)) { return false; }
		
		$table = getVariablesTable($dir."/.formatted");
		if (!isset($table[KEY_DERIVED_EXPERIMENTS])) { return false; }
		//$node['expName'] = "root";
		$children = isset($table[KEY_DERIVED_EXPERIMENTS]) ? explode(",",$table[KEY_DERIVED_EXPERIMENTS]) : array();
		$sortedArray = array();
		sortExperimentsByDeriveFull($project, ".formatted", $children, $sortedArray, 0);
		return $sortedArray;
	}
	
	function sortExperimentsByDeriveFull($project, $experiment, $children, &$sortedArray, $depth) {
		if (sizeof($children) == 0) { return; }
		$dir = USER_PROJECTS_DIR."/".$project."/experiments";
		if (!is_dir($dir)) { return; }		
		
		for ($i = 0 ; $i < sizeof($children) ; $i += 1){
			$exp = $children[$i];
			$node['expName'] = $exp;
			$node['depth']	 = $depth;
			$node['parent']	 = $experiment;
			$sortedArray[sizeof($sortedArray)] = $node;			
			$table = getVariablesTable($dir."/".$exp);
			$ExpChildren = isset($table[KEY_DERIVED_EXPERIMENTS]) ? explode(",",$table[KEY_DERIVED_EXPERIMENTS]) : array();
			sortExperimentsByDeriveFull($project, $exp, $ExpChildren, $sortedArray, $depth+1);
		}
	}
	
	// the $project to get its experiments
	function getExperimentsTree($project) {
		$dir = USER_PROJECTS_DIR."/".$project."/experiments";
		if (!is_dir($dir)) { return false; }
		
		$table = getVariablesTable($dir."/.formatted");
		$tree['expName'] = "root";
		$children = isset($table[KEY_DERIVED_EXPERIMENTS]) ? explode(",",$table[KEY_DERIVED_EXPERIMENTS]) : array();
		$tree['child'] = getExperimentsTreeFull($project, ".formatted", $children);
		return $tree;
	}
	
	function getExperimentsTreeFull($project, $experiment, $children) {
		if (sizeof($children) == 0) { return array(); }
		
		$dir = USER_PROJECTS_DIR."/".$project."/experiments";
		if (!is_dir($dir)) { return array(); }		
		
		$tree = array();
		for ($i = 0 ; $i < sizeof($children) ; $i += 1){
			$exp = $children[$i];
			$table = getVariablesTable($dir."/".$exp);
			$tree['expName'] = $exp;
			$children = isset($table[KEY_DERIVED_EXPERIMENTS]) ? explode(",",$table[KEY_DERIVED_EXPERIMENTS]) : array();
			$tree['child'] = getExperimentsTreeFull($project, $exp, $children);
			$tree['next'] = isset($children[$i+1]) ? $children[$i+1] : array();
		}	
	}
		
function getExpOperations($dir) {
	if (!is_dir($dir) || !isExperimentFolder($dir) || !is_file($dir."/".VARIABLES_FILE_NAME)) {
		return false;
	}
	$ops = array();
	$table = getVariablesTable($dir);
	if (isset($table[KEY_RUN_MODE])) {
		switch ($table[KEY_RUN_MODE]) {
			case VAL_RUN_MODE_CLEAN_AND_CLUSTER:
				if (isset($table[KEY_CLEANING_THRESHOLD]) && ($table[KEY_CLEANING_THRESHOLD] != "Weak") ) {
					$ops["Cleaning"] = "Clean: Unlikely markers";
				}
				$ops["Clustering"] = "Clustering distance: ".(isset($table[KEY_CLUSTERING_DISTANCE]) ? $table[KEY_CLUSTERING_DISTANCE] : "10")." cM";
				break;
			case VAL_RUN_MODE_CLEAN_ONLY:			
				if (isset($table[KEY_CLEANING_THRESHOLD]) && ($table[KEY_CLEANING_THRESHOLD] != "Weak") ) {
					$ops["Cleaning"] = "Clean: Unlikely markers";
				}
				break;
			case VAL_RUN_MODE_CLUSTER_ONLY: 		
				$ops["Clustering"] = "Clustering distance: ".(isset($table[KEY_CLUSTERING_DISTANCE]) ? $table[KEY_CLUSTERING_DISTANCE] : "10")." cM";
				break;
			case VAL_RUN_MODE_FILTERING:			$ops["Filtering"] = "Filtering"; break;
			case VAL_RUN_MODE_ZOOM:					$ops["Zooming"] = "Zooming"; break;
			default: break;
		}
	}
	
	if (isset($table[KEY_RUN_ANALYSIS])) {
		if ( ($table[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_FALSE) &&
			 ($table[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_TRUE) ) {
			$ops[toolDisplayName($table[KEY_RUN_ANALYSIS])." analysis"] = "Analysis: ".toolDisplayName($table[KEY_RUN_ANALYSIS]);
		}
	}
	
	//zooming have spetial behaiveor so its treated spetially
	if (isZoomingFolder($dir)) { $ops["Zooming"] = "Zooming"; }
	
	return $ops;
}
	
	// updates the information of an ini file with a given table.
	function AppendIniWithTable($dir, $iniFile, $table) {
		if (!is_dir($dir)) {return false;}		
		
		$totalTable = is_file($dir."/".$iniFile) ? parse_ini_file($dir."/".$iniFile, true) : array();		
		foreach ($table as $key => $subTable) {$totalTable[$key] = $subTable;}	// adds and updates tables
		
		if (!($infoFile = fopen($dir."/".$iniFile, "w"))) {return false;} 	//error...
		
		foreach ($totalTable as $name => $table){
			fwrite($infoFile, "[".$name."]\n");
			foreach ($table as $key => $val){
				if (is_array($val)) {
					foreach ($val as $value) {fwrite($infoFile, $key."[] = ".$value."\n");}
				}
				else {fwrite($infoFile, $key." = ".$val."\n");}
			}
		}
		return true;
	}
	
	// no $value means to remove the key without checking the value
	// if no removeKey set, the whole section will be removed
	function removeFromIni($dir, $iniFile, $section, $removeKey = false, $removeValue = false) {
		if (!is_dir($dir) || !is_file($dir."/".$iniFile)){
			return false;
		}

		$totalTable = parse_ini_file($dir."/".$iniFile, true);
		
		if (isset($totalTable[$section])) {
			if (!$removeKey){unset($totalTable[$section]);}
			else {
				$table = $totalTable[$section];
				if (isset($table[$removeKey])){
					if (!$removeValue || (!is_array($table[$removeKey]) && ($table[$removeKey] == $removeValue))) {
						unset($table[$removeKey]);
					}
					else {
						if (is_array($table[$removeKey])){
							for($i = 0 ; $i < sizeof($table[$removeKey]); $i+= 1){
								if ($table[$removeKey][$i] == $removeValue){ unset($table[$removeKey][$i]); }
							}
						}
					}
				}
				$totalTable[$section] = $table;
			}
			AppendIniWithTable($dir, $iniFile, $totalTable);
		}
		return true;
	}
	
	// updates the information of experiment content for a single table.
	function updateExpContentInfo($dir, $table, $keepCurrentContent = true) {
		if (!is_dir($dir) || !isset($table['header'])) {return false;}
		if ($keepCurrentContent) {$totalTable = getExpContentInfo($dir);}	// getting all content info
		else {$totalTable = Array();}
		if (!($infoFile = fopen($dir."/".CONTENT_INFO, "w"))) {return false;}	//error...
		$totalTable[$table['header']] = $table;
		foreach ($totalTable as $name => $table) {
			if (!isset($table['header'])) { continue; }
			fwrite($infoFile, "[".$table['header']."]\n");
			foreach ($table as $key => $val) {
				if ($key != 'header') {fwrite($infoFile, $key." = ".$val."\n");}				
			}
		}
		fclose($infoFile);
		return true;
	}
	
	// gives a table on info about given $header.
	// if $header not given or is false, will return a table of tables, each sub-table contains info about $table[name]
	// on general error returns false.
	// if the requested $header doesnt exists, returns an empty table.
	function getExpContentInfo($dir, $header = false) {
		if ( !is_dir($dir) || (!is_file($dir."/".CONTENT_INFO)) ||
			 !($linesArray = file($dir."/".CONTENT_INFO, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) ) {
			//error...
			return false;
		}
		
		$table = array();
		$finalTable = array();
		$alreadyFound = false;
		for ($i = 0 ; $i < sizeof($linesArray) ; $i += 1) {
			$line = $linesArray[$i];
			if (preg_match("/^\[(.*)\]$/", $line, $headName)) {// && (!$header || ($header == $headName[1])) ){
				if ($alreadyFound && $header) { break; }			// if we search for specific header and its already found, break the loop
				if ($header && (!(preg_match("/$header/", $headName[1])))) { continue; }		// BUG since the needed info is overwritten, because we won't get here for non-[...] lines!!! TODO: fix!!!
				if (!$header && isset($table['header'])){
					$finalTable[$table['header']] = $table;
					unset($table);
				}
				$table['header'] = $headName[1];
				$alreadyFound = true;
			}
			if (isset($table['header'])){
				$splitted = explode("=", $line);
				if (sizeof($splitted) == 2) {$table[trim($splitted[0])] = trim($splitted[1]);}				
			}
		}
		
		if (!$header && isset($table['header'])){
			$finalTable[$table['header']] = $table;		// inserting last sub table into the total table
			unset($table);
		}
		if ($alreadyFound && $header){ $finalTable = $table; }			// if we search for specific header and its already found, break the loop - in case of last lines (no [...])
		
		return $finalTable;
	}
	
	function getExperimentStatus($project, $exp) {
	
		$dir = USER_PROJECTS_DIR."/$project/experiments/$exp";
		if (!isExperimentFolder($dir)) { return false; }
		$vars = getVariablesTable($dir);
		
		if (is_file($dir."/".EXP_STOPPED_FILE)){return EXP_STATUS_STOPPED;}
		if (is_file($dir."/".INIT_EXPERIMENT_FLAG)){return EXP_STATUS_RUNNING;}
		if (!is_file($dir."/".PROCESS_ID_FILE)) {
			if (isset($vars[KEY_RUN_MODE]) && ( ($vars[KEY_RUN_MODE] == "filter") || ($vars[KEY_RUN_MODE] == "zoom") ) ){
				return EXP_STATUS_FINISHED;
			}
			return EXP_STATUS_RUNNING;
		}
		$procID = file($dir."/".PROCESS_ID_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$procID = $procID[0];		
		
		$cleanFile = false;
		$cleanPercents = false;
		$clusterFile = false;
		$analysis = false;
		if (isset($vars[KEY_RUN_MODE])) {
			switch($vars[KEY_RUN_MODE]) {
				case VAL_RUN_MODE_CLEAN_ONLY: 			$cleanPercents = true; $cleanFile = true; break;
				case VAL_RUN_MODE_CLEAN_AND_CLUSTER: 	$cleanPercents = true; $clusterFile = true; break;
				case VAL_RUN_MODE_CLUSTER_ONLY:			$clusterFile = true; break;
				default:								break;
			}
		}
		if (isset($vars[KEY_RUN_ANALYSIS]) && ($vars[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_FALSE) && is_file($dir."/".RESULTS_FILENAME)){
			$analysis = true;
		}
		
		$percents = array();
		$info	  = getExpContentInfo($dir);
		if (!$info) {return EXP_STATUS_FINISHED;}
		
		foreach ($info as $key => $subTable) {			
			getProgressPercents($dir, $key, $percents);
			if ($analysis && (isErrorFor($dir, $key, "analysis"))) {continue;}
			if ($cleanPercents) {				
				if (preg_match("/.*Error.*/", $percents[0])) {continue;}
				if (!preg_match("/.*100%.*/", $percents[0])) {return EXP_STATUS_RUNNING;}
			}
			if ($cleanFile && !isReadOnlyExperiment($project, $exp)) {				
				if (!is_file($subTable['clean_datfile']) || !is_file($subTable['clean_pedfile'])) {
					return EXP_STATUS_RUNNING;
				}
			}
			if ($clusterFile && !isReadOnlyExperiment($project, $exp)) {
				if (preg_match("/.*Error.*/", $percents[1])) { continue; }
				if (!preg_match("/.*100%.*/", $percents[1]) || !is_file($subTable['cluster_datfile']) || !is_file($subTable['cluster_pedfile'])) {
					return EXP_STATUS_RUNNING;
				}
			}
			if ($analysis) {
				if (isErrorFor($dir, $key, "analysis")) { continue; }
				$results = file($dir."/".RESULTS_FILENAME);
				$found = false;
				for ($i = 1 ; $i < sizeof($results) ; $i += 1){
					$line = explode(" ", $results[$i]);
					$resultForHeader = trim($line[0]);
					//$key = str_replace("-", "\-", $key);
					$resultFolder = trim($line[1]);
					if (preg_match("/$key/", $resultForHeader)){
						$found = true;						
						if (isErrorFor($dir, $resultForHeader, "analysis")) { continue; }
						if (preg_match("/^.*\/(var\/tmp\/.*)$/",$resultFolder, $fileDir)){
							$fileDir = $fileDir[1];
							if (!is_file(WEBDIR."/".$fileDir."/TASK_LOG.txt")) {								
								return EXP_STATUS_RUNNING; 
							}
						}
					}
				}
				if (!$found) { return EXP_STATUS_RUNNING; }
			}
		}
		return EXP_STATUS_FINISHED;
	}
	
	function getTaskPlaceInQueue($queue, $processID, $fileName = "") {
		if (!$processID) { return false; }
		$queue = WEBDIR. "/Submissions/". $queue;
		
		$line = shell_exec("grep -n '$processID.*$fileName' $queue");
		if ($line == "") { return false; }
		$place = explode(":", $line);
		return $place[0];
	}
	
	# dvir 2011/10/23, relevant for analysis which have the results file( and not for cleaning / clustering)
	# the file contain at which dirs there is the results file.
	# $directory - for example : /var/www/html/oren/webDevelopment/users/guest/projects/111/experiments/Experiment1
	#              the dir in which the RESULTS_FILENAME resides.
	# $fileName - for example : resultFile.chrom_06
	# example for a row at the RESULTS_FILENAME : 
	# "resultFile.chrom_06 /var/tmp/guestguesthopenomail.resultFile.chrom_06.n6v0r3s0uf.Thu.Aug.4.11.10.02.2011"
	function getChromResultsDirByResultsFile($directory, $fileName) {
		$resultsFileFullPath = $directory."/".RESULTS_FILENAME;
		if (is_file($resultsFileFullPath)) {
			$results = file($resultsFileFullPath);
			for ($i = 1 ; $i < sizeof($results) ; $i += 1) {
				$line = explode(" ", $results[$i]);
				$resultForHeader = trim($line[0]);
				if (preg_match("/$fileName$/", $resultForHeader)) {
					$resultFolder = trim($line[1]);
					return WEBDIR."/".$resultFolder;
				}
			}
		}
		return "";
	}
	
	# dvir 2011/10/23
	# returning the submission Dir by the expirement dir
	# $directory - for example : /var/www/html/oren/webDevelopment/users/guest/projects/111/experiments/Experiment1
	# return for example : "/var/www/html/oren/webDevelopment/Submissions/n6v0r3s0uf"
	# according to the content of PROCESS_ID_FILE at $directory, which contain : n6v0r3s0uf
	function getSubmissionDirByExpirementDir ($directory) {
		if (is_file($directory."/".PROCESS_ID_FILE)){
			$procID = file($directory."/".PROCESS_ID_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
			$procID = $procID[0];
			
			$submissionDir = WEBDIR. "/Submissions/". $procID;
			return $submissionDir;
		}
		return "";
	}

	# dvir 2011/10/23
	# checking if there was an error at the submission from hap to link01
	# if so, returns a string of the analysis status : ERROR_IN_SUBMISSION_ANALYSIS_STATUS.
	# else, returns ""
	function checkIfThereIsAnErrorAtSubmission($directory, $fileName) {
		$chromResultsDir = getChromResultsDirByResultsFile($directory, $fileName);
		if ($chromResultsDir != "" && 
		    is_file($chromResultsDir."/".ERROR_IN_SUBMISSION_FILE_INDICATION)) {
			return ERROR_IN_SUBMISSION_ANALYSIS_STATUS;
		}
		$submissionDir = getSubmissionDirByExpirementDir($directory);
		if ($submissionDir != "" && 
		    is_file($submissionDir."/".ERROR_IN_SUBMISSION_FILE_INDICATION)) {
			return ERROR_IN_SUBMISSION_ANALYSIS_STATUS;
		}
		return "";
	}
	
	function getAnalysisStatus($directory, $fileName) {
		$analysisStatus = "";
		if (!is_file($directory."/".RESULTS_FILENAME)) {return "";}
		
		###
		# dvir 2011/10/17, showing an error in submitting
		$IsThereIsAnErrorAtSubmission = checkIfThereIsAnErrorAtSubmission($directory, $fileName);
		if ($IsThereIsAnErrorAtSubmission != "") {
			return $IsThereIsAnErrorAtSubmission;
		}
		###
				
		$procID = false;
		if (is_file($directory."/".PROCESS_ID_FILE)){
			$procID = file($directory."/".PROCESS_ID_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
			$procID = $procID[0];
		}
		
		$res = false;
		# $analysisStatus = "analysis waiting $directory $fileName";
		$analysisStatus = "analysis waiting";
		$place = getTaskPlaceInQueue(FRONTEND_JOBSQ, $procID, $fileName);
		$place = $place ? $place : getTaskPlaceInQueue(REMOTE_JOBSQ, $procID, $fileName);
		if ($place) {$analysisStatus = $analysisStatus . "<br/><span style=\"color: gray\">place in queue: ".$place."</span>";}
		else {$analysisStatus = $analysisStatus . "<br/><span style=\"color: gray\">preparing...</span>";}
		/* checking for error in cleaning */
		if (isErrorFor($directory, $fileName, "cleaning")){
			$link = "viewReport.php?file=".base64_encode($directory."/.".$fileName."_cleaning_log.txt");
			$analysisStatus = "<a href=$link target=\"_blank\">error</a>";
		}
		/* checking for error in analysis */
		if (isErrorFor($directory, $fileName, "analysis")){
			$path = $directory."/.".(is_file($directory."/.".$fileName."_analysis_log.txt") ? "" : "resultFile.").$fileName."_analysis_log.txt";
			$link = "viewReport.php?file=".base64_encode($path);
			$analysisStatus = "<a href=$link target=\"_blank\">analysis error</a>";
		}
		
		$results = file($directory."/".RESULTS_FILENAME);
		for ($i = 1 ; $i < sizeof($results) ; $i += 1) {
			$line = explode(" ", $results[$i]);
			$resultForHeader = trim($line[0]);
			$resultFolder = trim($line[1]);
			if (!preg_match("/$fileName$/", $resultForHeader)) {continue;}
			
			$fileDir = $resultFolder;
			if (preg_match("/^.*\/(var\/tmp\/.*)$/",$resultFolder, $folder)) {$fileDir = $folder[1];}
			$statusPage  = WEBDIR."/".$fileDir."/".SL_STATUS_PAGE;
			$resultsPage = WEBDIR."/".$fileDir."/".SL_RESULTS_PAGE;
			
			if (is_file($statusPage)) {
				$message = "sent to analysis";
				$content = file_get_contents($statusPage);
				if (preg_match("/.*failed.*/",strtolower($content))) {$message = "analysis failed";}
				if (preg_match("/.*removed.*/",strtolower($content))) {$message = "stopped";}					
				$analysisStatus = "<a href=\"$statusPage\" target=\"_blank\">$message</a>";
			}
			
			if (is_file($resultsPage)) {
				$sl_folder = substr($_SERVER['REQUEST_URI'], 0 , strrpos($_SERVER['REQUEST_URI'], "/"));
				$file = is_file(WEBDIR."/".$fileDir."/res_output.html") ? "res_output.html" : "formatted_output.html";
				$file = is_file(WEBDIR."/".$fileDir."/res_output_pdf.html") ? "res_output_pdf.html" : $file;
				$url = "http://".$_SERVER['SERVER_NAME']."$sl_folder/$fileDir/$file";
				$pdf = "<a href=\"http://pdfmyurl.com?url=$url\"><img src=\"images/pdf_icon.png\" border=\"0\"/></a>";
				$analysisStatus = "<a href=\"$resultsPage\" target=\"_blank\"><strong>$fileName Results</strong></a><br/>$pdf";
			}
		}
		
		return $analysisStatus;		
	}
	
	function getFilesLinks($directory, $fileName, $method = "") {
		if (isArchived($directory)) {return "<span style=\"color: gray\">Files Archived</span>";}			
		
		$table = getExpContentInfo($directory);
		$vars = getVariablesTable($directory);
		$subTable = $table[$fileName];
		$links = "";
		$file_nospaces = str_replace(" ", "$", $fileName);
		$path = "http://" . $_SERVER['HTTP_HOST'] . substr($_SERVER['REQUEST_URI'] , 0, strrpos($_SERVER['REQUEST_URI'], "/")+1);
		
		$underscore = "";
		if ($method != "") { $underscore = "_"; }
		$dat = isset($subTable[$method.$underscore.'datfile']) ? $subTable[$method.$underscore.'datfile'] : false;
		$ped = isset($subTable[$method.$underscore.'pedfile']) ? $subTable[$method.$underscore.'pedfile'] : false;
		if ($dat && $ped) {
			if (!isset($vars[KEY_RUN_ANALYSIS]) || ($vars[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_SEGREGATION)){		
				$links = (is_file($dat) && is_file($ped)) ? "<input value=\"$method"."@$file_nospaces\" name=\"checkboxes[]\" type=\"checkbox\"/>&nbsp;&nbsp;&nbsp;" : "";
				// show box only if both files are ready! and its not segregation
				$links = $links .((is_file($dat)) ?
					"<a href=\"".$path."download.php?file=".base64_encode($dat)."\">dat file</a>" : "working...");
				$links = $links . " | ";
			}
			$links = $links . ((is_file($ped)) ?
				"<a href=\"".$path."download.php?file=".base64_encode($ped)."\">ped file</a>" : "working...");
			if (!isset($vars[KEY_RUN_ANALYSIS]) || ($vars[KEY_RUN_ANALYSIS] != VAL_RUN_ANALYSIS_SEGREGATION)){
				$links = $links . ((is_file($dat) && is_file($ped)) ? "<br/>(<a href=\"showMarkers.php?proj=".$_GET['proj']."&file=".base64_encode($dat)."\" target=\"_blank\">see markers</a>)" : "");
			}
		}
		return $links;
	}
	
	function initProgressForFile($expDir, $fileName, $parameters, $initialPercents = 0) {
		if (!is_dir($expDir)) {return false;}
		$progressFile = $expDir."/".PROGRESS_FILENAME;
		
		for ($i = 0 ; $i < sizeof($parameters) ; $i += 1){
			exec("echo \"".$fileName." ".$parameters[$i]." ".$initialPercents."%\" >> $progressFile");
		}
		
		exec("chmod 776 $progressFile");
		return true;
	}
	
	//function getExperimentFileHeads($userName, $projName, $expName){
	function getFileHeadsFromDir($dir, $extension = false) {		
		if ( (!is_dir($dir)) || (!($handle = opendir($dir)))) {return false;}		
		$heads = array();
		while (($file = readdir($handle)) != false) {
			if ((strpos($file,".") === 0) || (!is_file($dir."/".$file))){  //if file/dir is hidden  *** can use strstr($file,".") if "." apears anywhere ***
				continue;
			}
			$info = pathinfo($dir."/".$file);
			if ($extension) {
				if ($info['extension'] == $extension) {$heads[] = substr($file, 0, -4);}				
			}
			else {$heads[] = substr($file, 0, -4);}				
			
		}
		closedir($handle);
		
		return $heads;
	}

	function collectResults($project, $experiment) {
		$dir = USER_PROJECTS_DIR ."/".$project."/experiments";
		if ( ($experiment != null) && ($experiment != "") ){
			$dir = $dir . "/" . $experiment;
			collectExperimentResults($project, $experiment);
		}
		else{
			if ($handle = opendir($dir)){						// collecting results from all experiments under this project
				while (($exp = readdir($handle)) != false){
					if ( (strpos($exp,".") === 0) || (!is_dir($dir."/".$exp)) || (!isExperimentFolder($dir."/".$exp)) ){  //if experiment is hidden or its not an experiment directory
						continue;
					}
					collectExperimentResults($project, $exp);
				}
			}
		}
	}
	
	function collectExperimentResults($project, $experiment) {
		$dir = USER_PROJECTS_DIR ."/".$project."/experiments/".$experiment;
		if (!is_dir($dir)){ return false; }
		$allResults = file($dir."/".RESULTS_FILENAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$procID = file($dir."/".PROCESS_ID_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$procID = $procID[0];
		
		if (is_dir($dir."/Results")){ return; }
		createFolder($dir, "Results");
		for ($i = 1 ; $i < sizeof($allResults) ; $i += 1){
			$nameLink = explode(" ", $allResults[$i]);
			if (sizeof($nameLink) != 2) { continue; }
			$name = $nameLink[0];
			$link = $nameLink[1];
			$taskDir = WEBDIR . "/Submissions/$procID".strrchr($link, "/");
			if (is_dir($taskDir)){
				createFolder($dir."/Results", $name);
				if (is_file($taskDir."/".SL_RESULTS_PAGE)){			
					copy($taskDir."/".SL_RESULTS_PAGE, $dir."/Results/".$name."/".SL_RESULTS_PAGE);
				}
				if (is_file($taskDir."/nav_output.html")){			
					copy($taskDir."/nav_output.html", $dir."/Results/".$name."/nav_output.html");
				}
				if (is_file($taskDir."/res_output.html")){			
					copy($taskDir."/res_output.html", $dir."/Results/".$name."/res_output.html");
				}
				if (is_file($taskDir."/".CUSTOM_TRACKS_FILE)){
					copy($taskDir."/".CUSTOM_TRACKS_FILE, $dir."/Results/".$name."/".CUSTOM_TRACKS_FILE);
					// fix customTracks folder in the html files
				}
				system("cp $taskDir/*.png $dir/Results/$name/");
				
				foreach (getAdditionalStyleFolderNames() as $folderName) {
					if (is_dir($taskDir."/".$folderName)){
						createFolder($dir."/Results/$name", $folderName);
						$directory = dir( $taskDir."/".$folderName );
						while ( false !== ( $readdirectory = $directory->read() ) ) {
							if ( $readdirectory == '.' || $readdirectory == '..' ) {
								continue;
							}
							$PathDir = $taskDir."/".$folderName."/".$readdirectory; 
							if ( is_dir( $PathDir ) ) {
								//copy_directory( $PathDir, $destination . '/' . $readdirectory );
								continue;
							}
							copy( $PathDir, $dir."/Results/$name/".$folderName."/".$readdirectory );
						}
					}
				}
				
				//$results[$name] = array($taskDir."/".SL_RESULTS_PAGE , $taskDir."/nav_output.html", $taskDir."/res_output.html", $taskDir."/*.png");
				//system ("echo '$name -> ".implode(" ", $results[$name])."' >> /tmp/orentest.log");
			}
		}
	}
	
	function collectResultsPaths($project, $experiment) {
		$resultsPath = array();
		$dir = USER_PROJECTS_DIR ."/".$project."/experiments";
		if ( ($experiment != null) && ($experiment != "") ){
			$dir = $dir . "/" . $experiment;
			$expRes = collectExperimentResultsPath($project, $experiment);
			if ($expRes) {$resultsPath[$experiment] = $expRes;}			
		}
		else {
			if ($handle = opendir($dir)){						// collecting results from all experiments under this project
				while (($exp = readdir($handle)) != false){
					if ( (strpos($exp,".") === 0) || (!is_dir($dir."/".$exp)) || (!isExperimentFolder($dir."/".$exp)) ){  //if experiment is hidden or its not an experiment directory
						continue;
					}
					$expRes = collectExperimentResultsPath($project, $exp);
					if ($expRes) {
						$resultsPath[$exp] = $expRes;
					}
				}
			}
		}
		
		return $resultsPath;
	}
	
	function collectExperimentResultsPath($project, $experiment) {
		$dir = USER_PROJECTS_DIR ."/".$project."/experiments/".$experiment;
		if (!is_dir($dir)){ return false; }
		$allResults = file($dir."/".RESULTS_FILENAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$procID = file($dir."/".PROCESS_ID_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$procID = $procID[0];
		
		$results = array();		
		
		for ($i = 1 ; $i < sizeof($allResults) ; $i += 1){
			$nameLink = explode(" ", $allResults[$i]);
			if (sizeof($nameLink) != 2) { continue; }
			$name = $nameLink[0];
			$link = $nameLink[1];
			//if (!isset($info[$nameLink[0]])) { continue; }
			$taskDir = WEBDIR . "/Submissions/$procID".strrchr($link, "/");
			if (is_dir($taskDir)){
				$results[$name] = array($taskDir."/".SL_RESULTS_PAGE , $taskDir."/nav_output.html", $taskDir."/res_output.html", $taskDir."/*.png");
				//system ("echo '$name -> ".implode(" ", $results[$name])."' >> /tmp/orentest.log");
			}
		}
		
		return $results;
	}
	
	/*
	 * prepare and send the experiment to running
	 * input:
	 *	$expDir - the experiment directory
	 *	$flags - the flage of the experiment
	 *	$runExperiment - true if the experiment will running immediatley, false if the experiment is waiting to another experiment to finish
	 */
	function sendExperiment($expDir, $flags, $runExperiment = true) {
		$runName = 1;
		$flagArray = explode(" ", $flags);
		
		if (!preg_match("/^.*\/users\/(.*)/", $expDir, $userDir)) {return $runName;}		
		$dirSplitted = explode("/", $userDir[1]);		# needed in order to get the user name
		$userMail = getUserMail($dirSplitted[0]);		# getting the user mail address
		
		# can return 1 on failure
		$runName = shell_exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/prepareTask.pl -dir ".(substr($expDir,2))." -mail $userMail -flag ".$flags); // export so apache can read the pm files needed
		
		// runExperiment is false in case the task is wait to other task - in that case dont run the experiment
		if (!$runExperiment) {return 0;}
		
		$userName 	= $dirSplitted[0];						//     /userName/projects/projectName/experiments/ExperimentName/files...
		$projName	= $dirSplitted[2];
		$runName	= trim($runName);
		
		$queue 	= getQueueForExperiment($expDir);
		$script = getRunScriptForExperiment($expDir);			
		
		if ($script	== "submitTask.pl") {
			$parameter1 = "-task";
			$parameter2 = $runName;
			shell_exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/enqueueTask.pl $queue Submissions/$runName perl $script $parameter1 $parameter2"); // export so apache can read the pm files needed
			log_error("Task $runName from project: $projName, for user: $userName, has entered the $queue by $script.");
			if (in_array(SEND_TO_SL_FLAG, $flagArray)) {
				shell_exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/enqueueTask.pl ".WAIT_FINISH_TASKS_LIST." None $runName"); 			// enqueue for GlobalPost
			}
		}
		else {
			$files = getFileHeadsFromDir($expDir, "dat");
			$expDir = substr($expDir,2); //remove the "./" from "./expDir"
			foreach ($files as $fileName) {					
				$command = "export PERLLIB=".CGI_BIN_DIR ."; perl -e 'use submissionConfig; print &submissionConfig::getMailOfUserBySubmissionDir(\"$runName\");'";
				$userMail = `$command`;
				$command = "export PERLLIB=".CGI_BIN_DIR ."; perl -e \"use submissionConfig; print &submissionConfig::getChromSubDirInitial_AtSubmissionDir_BySubmissionDir('$runName','$fileName','$userMail');\"";
				$errorInSubmissionFullDir = `$command`;
				
				$errorInSubmissionFullDir = "Submissions/$runName/$errorInSubmissionFullDir";
				shell_exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/enqueueTask.pl $queue $errorInSubmissionFullDir perl $script $runName $expDir/$fileName"); // export so apache can read the pm files needed
			}					
			shell_exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/enqueueTask.pl ".WAIT_FINISH_TASKS_LIST." None $runName"); 			// enqueue for GlobalPost
			log_error("Task $runName from project: $projName, for user: $userName, has entered the $queue by $script.");
		}

		//the shell command returns the random name given to the job.
		if ( ($runName == 1) || ($runName == -1) ) {
			$errorDir = $userDir[1];
			log_error("submit $errorDir failed: $runName");
			// report to user !!!
		}		
		return $runName;
	}
	
	function getQueueForExperiment($expDir) {
		$varsTable = getVariablesTable($expDir);
		if ( isset($varsTable['Analysis']) && ($varsTable['Analysis'] == "TwoPoint" || $varsTable['Analysis'] == "MultiPoint" ) && (!isset($varsTable[KEY_RUN_MODE])) ) {
			return FRONTEND_JOBSQ;
		}
		return REMOTE_JOBSQ;
	}

	function getRunScriptForExperiment($expDir) {
		$varsTable = getVariablesTable($expDir);
		if ( isset($varsTable[KEY_RUN_MODE]) ) {return "submitTask.pl";}
		return "submitAnalysis.pl";
	}	
	
	function getFrequencyFromDat($datFile) {
		if (!is_file($datFile)) {return false;}
		
		$file = fopen($datFile,"r");
		for ($i = 0 ; $i < 5 ; $i += 1){
			if (feof($file)) {return false;}			
			$line = fgets($file);
		}
		$line = preg_split("/([ \t]|<<)/",$line);
		if ($line[1] < 0 || $line[1]>1) {return false;}
		return (float)$line[1];	#Casting removes trailing zeros
		
	}
	
	function getMOIFromDat($datFile) {
		if (!is_file($datFile)) {return false;}
		
        #Read seven first lines of dat file
		$file = fopen($datFile,"r");
		for ($i = 0 ; $i < 7 ; $i += 1){
			if (feof($file)) {return false;}
			$line = fgets($file);
		}
		$line = preg_split("/([ \t]|<<)/",$line);
		$MOI[0] = ($line[1] == 0) ? "Recessive" : "Dominant";
		if ($line[1] > 1 || $line[1] < 0 || $line[2] > 1 || $line[2] < 0) {return false;}
        $MOI[1] = (float)$line[1];	#Casting removes trailing zeros
        $MOI[2] = (float)$line[2];	#Casting removes trailing zeros
		return $MOI;
	}
	
	function getPenetranceFromInput() {
		if (!isset($_GET['penetrance'])) {return false;}
		if ( (!isset($_GET['penetranceChoiceRec'])) && (!isset($_GET['penetranceChoiceDom'])) ) {return false;}

		$val[0] = $_GET['penetrance'];
		$splitted = isset($_GET['penetranceChoiceRec']) ? explode(" ", $_GET['penetranceChoiceRec']) : explode(" ", $_GET['penetranceChoiceDom']);
		$val[1] = $splitted[0];
		$val[2] = $splitted[1];
		
		return $val;
	}
	
	function getFrequencyFromInput() {
		if (!isset($_GET['frequency'])) {return false;}		
		return $_GET['frequency'];
	}
	
	function updateMOIandFrequency($datFile, $MOI, $frequency) {
		if (!is_file($datFile)) {return false;}		
		$file = file($datFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		if (sizeof($file) < 6) {return false;}
		
		// updating frequency
		$line = explode(" ", $file[4]);
		$line[1] = $frequency;
		$line[0] = 1-$frequency;
		$file[4] = implode(" ", $line);
		
		// updating MOI
		$line = explode(" ", $file[6]);
		$line[0] = 0;
		$line[1] = $MOI[1];
		$line[2] = $MOI[2];
		$file[6] = implode(" ", $line);
		
		return file_put_contents($datFile, implode("\n", $file), LOCK_EX);
	}
	
	// shows a page with links and image to the pedigree drawing for a given project
	// returns -1 on failure
	function showPedigree($project) {
		$dir = USER_PROJECTS_DIR."/".$project;
		//$pedFileDir  = $dir."/experiments/Experiment1";
		$pedigreeDir = $dir."/pedigree";
		
		if (!is_dir($pedigreeDir)) {
			if (drawPedigree($dir) == -1) {return -1;}	// creating a folder for the pedigree images			
		}
		# dvir 2011/08/04, report on error
		# the reason for the error can be found at the condor dag log at the backend
		if (is_file($dir."/pedigree/error")) {
			?>Error occurred in the creation of the pedigree, please contact the system administrator.</br><?php
		}
		# dvir 2011/11/10, report on error at the submission
		# the reason for the error can be found at the submission log RemoteJobQ.log
		if (is_file($dir."/pedigree/".ERROR_IN_SUBMISSION_FILE_INDICATION)) {
			?>Error occurred in the submission of the pedigree creation task, please contact the system administrator.</br><?php
		}		
		?>
		<br/>
		<table><tr><td style="vertical-align: top;">
		<b>Madeline pedigree</b><br/>
		<?php
		$i = 1;
		for ($i = 1 ; $i < 5 ; $i += 1){
			$pdfFile 	= "$pedigreeDir/pedigree.$i.pdf";
			
			if (is_file($pdfFile)){
				$j = 0;
				$jpegFile[$j] 	= "$pedigreeDir/pedigree.$i-$j.jpeg";
				$pngFile = "$pedigreeDir/pedigree.$i-$j.png";
				if (!is_file($jpegFile[$j]) && is_file($pngFile)){ png2jpg($pngFile, $jpegFile[$j]); }
				$thumbnail = array();
				while (is_file($jpegFile[$j])){
					$thumbnail[$j] = $pedigreeDir."/".resizeJpeg($jpegFile[$j], $pedigreeDir);
					$j += 1;
					$jpegFile[$j] = "$pedigreeDir/pedigree.$i-$j.jpeg";
					$pngFile = "$pedigreeDir/pedigree.$i-$j.png";
					if (!is_file($jpegFile[$j]) && is_file($pngFile)){ png2jpg($pngFile, $jpegFile[$j]); }
				}
				?>
				Pedigree drawing for family <b><?php echo $i;?></b>:<br/>
				<a href="download.php?file=<?php echo base64_encode($pdfFile);?>">Download as PDF</a><br/>

				<!--<a href="download.php?file=<?php echo base64_encode($jpegFile);?>">Download as Image</a><br/>-->
				<p>
				<?php for ($count = 0 ; $count < $j ; $count += 1){ ?>
					<a href="<?php echo $jpegFile[$count];?>" target="_blank"><img src="<?php echo $thumbnail[$count];?>" height="30%"/></a>
				<?php } ?>
				<br/>
				Click on the thumbnail to view full size.
				</p>
				<?php
			}
			else { break ;}
		}
		
		if ($i == 1) {
			?>Please wait...creating pedigree <img src="images/waitspinner.gif" border="0"/><br/>
			(Please try again in 5 minutes)<br/><?php
		}
		
		?></td>
		<td style="padding-left: 80px; vertical-align: top;">
		<?php
		showHaploPainterPedigree($pedigreeDir);
		?>
		</td></tr></table>
		<?php
		
		return 0;
	}
	
	// creates a pedigree folder and draws a pedigree using 2 different programs. ($dir is the project dir of the pedigree)
	function drawPedigree($dir) {
		$pedFileDir  = $dir."/raw";
		$pedFile = "";

		$pedFound = false;
		if ($handle = opendir($pedFileDir)){			// finding a ped file
			while (($file = readdir($handle)) != false){
				if ( (strpos($file,".") === 0) || (!is_file($pedFileDir."/".$file)) ){  //if file is hidden or its not a file
					continue;
				}
				if (preg_match("/^.*\.ped$/",$file)){
					$pedFile = $file;
					$pedFound = true;
					break;		// break on that file
				}
			}
		}
		
		if (!$pedFound) {
			error_msg("Could not find any ped file in your uploaded files.");
			return -1;
		}	
	
		preg_match("/^.*(users.*)$/",$dir,$subDir);
		$pedFile = $subDir[1]."/raw/".$pedFile;
		$outputDir = $subDir[1]."/pedigree";
		
		createFolder($dir, "pedigree");
		$translate = "";
		if (is_file($subDir[1]."/raw/".SIMPLIFIED_PED_MAP_FILE)) {
			$translate = " -translate ".$subDir[1]."/raw/".SIMPLIFIED_PED_MAP_FILE;
		}
		
		
		$file = (is_file($subDir[1]."/raw/".SIMPLIFIED_DATA_FILE)) ? $subDir[1]."/raw/".SIMPLIFIED_DATA_FILE : $pedFile;
		$sl = (is_file($subDir[1]."/raw/".SIMPLIFIED_DATA_FILE)) ? "" : "-sl";

		//$simplified = (is_file($subDir[1]."/raw/".SIMPLIFIED_DATA_FILE)) ? "-simplified ".$subDir[1]."/raw/".SIMPLIFIED_DATA_FILE : "";
		$cmd = "export PERLLIB=".CGI_BIN_DIR."; perl ".CGI_BIN_DIR."/enqueueTask.pl ".REMOTE_JOBSQ." $outputDir perl send_drawing_job.pl -ped $pedFile $translate -destDir $outputDir";
		log_error($cmd);
		exec($cmd);		// run the pedigree creator (creates pdf and jpg)		
		// return error on failure
	}
	
	// creates a thumbnail.$imageFile in given $outputDir
	function resizeJpeg($imageFile, $outputDir) {
		if (!is_file($imageFile)) {return false;}		
		
		$fileName = explode("/", $imageFile);
		$fileName = $fileName[sizeof($fileName)-1];
		
		$new_img = imagecreatefromjpeg($imageFile);
		
		//list the width and height and keep the height ratio.
		list($width, $height) = getimagesize($imageFile);
		//calculate the image ratio
		$thumbWidth 	= 100;
		$imgRatio = $width/$thumbWidth;
		
		if ($imgRatio > 1) {$thumbHeight 	= $height / $imgRatio;}
		else {$thumbHeight 	= $height * $imgRatio;}
		
		$resized_img = ImageCreateTrueColor($thumbWidth, $thumbHeight);
		imagecopyresized($resized_img, $new_img, 0, 0, 0, 0, $thumbWidth, $thumbHeight, $width, $height);
		ImageJpeg ($resized_img,"$outputDir/thumbnail.$fileName");
		ImageDestroy ($resized_img);
		ImageDestroy ($new_img);
		return "thumbnail.$fileName";
	}
	
	function png2jpg($originalFile, $outputFile, $quality = 75) { 
		$image = imagecreatefrompng($originalFile); 
		imagejpeg($image, $outputFile, $quality); 
		imagedestroy($image); 
	} 	
	
	function pdf2jpg($originalFile, $outputFile){
		exec("convert -trim -density 100 $originalFile $outputFile");
	}
	
	function showHaplopainterPedigree($pedigreeDir) {
		?><b>Haplopainter pedigree</b><br/><?php
		$i = 1;
		$pdfFile = "$pedigreeDir/painter_out_$i.pdf";
		$pngFile = "$pedigreeDir/painter_out_$i.png";
		$jpgFile = "$pedigreeDir/painter_out_$i.jpg";
		if (is_file($pdfFile) && is_file($pngFile)){
			while (is_file($pdfFile) && is_file($pngFile)){ ?>
				Pedigree drawing for family <b><?php echo $i; ?></b>:<br/>
				<a href="download.php?file=<?php echo base64_encode($pdfFile);?>">Download as PDF</a><br/>
				<?php
				if (!is_file($jpgFile)){
					pdf2jpg($pdfFile, $jpgFile);
				}
				$thumbnail = $pedigreeDir."/".resizeJpeg($jpgFile, $pedigreeDir);
				?>

				<!--<a href="download.php?file=<?php echo base64_encode($jpegFile);?>">Download as Image</a><br/>-->
				<p>
				<a href="<?php echo $pngFile;?>" target="_blank"><img src="<?php echo $thumbnail;?>"/> <!--style="filter: Invert;"  width="20%"--></a>
				<br/>
				Click on the thumbnail to view full size.
				</p>
				<?php
				
				$i += 1;
				$pdfFile = "$pedigreeDir/painter_out_$i.pdf";
				$pngFile = "$pedigreeDir/painter_out_$i.png";
				$jpgFile = "$pedigreeDir/painter_out_$i.jpg";
			}
		}
		else{
			if (is_file($pedigreeDir."/".PAINTER_ERROR_FILE)){
				$error = file($pedigreeDir."/".PAINTER_ERROR_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
				for ($i = 0 ; $i < sizeof($error) ; $i++){
					error_msg($error[$i]);
				}
			}
			else{
				?>Please wait...creating pedigree <img src="images/waitspinner.gif" border="0"/><br/>
				(Please try again in 5 minutes)<br/><?php
			}
		}
	}
	
	function showDataBrowser($project, $experiment = false) {
		$dataBrowser = new DataBrowser();
		$dataBrowser->displayBrowser($project, $experiment);
	}
	
	function convertSimplifiedToPed($projectName, $simplifiedText, $name = false) {
		$name = isset($_SESSION['name']) ? $_SESSION['name'] : $name;
		if (!$name) { return; }
		$fileName = $name."_".$projectName."_simplified.txt";
		$file = "/tmp/".$fileName;

		$res = exec("echo \"".$simplifiedText."\" > $file");
		exec("chmod 775 $file");
		if (!is_dir(USER_PROJECTS_DIR."/".$projectName)) {createProjectFolder(USER_PROJECTS_DIR, $projectName);}
		$targetDir = USER_PROJECTS_DIR."/".$projectName."/raw";

		//if a ped file exists - remove it!
		if (fileOfTypeExistsInDir($targetDir, "ped") == true) {exec("rm -f $targetDir/*.ped");}

		exec("export PERLLIB=".CGI_BIN_DIR ."; perl ".CGI_BIN_DIR ."/convertSimplifiedToSL.pl $file ".$targetDir);
		exec("echo 'simplified' > ".$targetDir."/".SIMPLIFIED_FLAG);
		unlink($file);

		if (is_file($targetDir."/".MAP_FILE_NAME)){
			copy($targetDir."/".MAP_FILE_NAME, $targetDir."/".SIMPLIFIED_PED_MAP_FILE);
			unlink($targetDir."/".MAP_FILE_NAME);
			simplifiedMapToPedMap($targetDir."/".SIMPLIFIED_PED_MAP_FILE, $targetDir."/".MAP_FILE_NAME);
		}

	}

	function simplifiedMapToPedMap($input, $output) {
		$simplifiedMap = file($input, FILE_SKIP_EMPTY_LINES);
		$out = fopen($output, 'w');
		
		foreach ($simplifiedMap as $line) {
			$line = explode("\t", $line);
			$line[1] = $line[0].".".$line[1];
			unset($line[0]);
			fwrite($out, implode("\t", $line));
		}
		
		fclose($out);
	}
	
	function getDiskSpaceUsed() {
		$space = explode( "." ,  exec("du -s ".SESSION_USER_DIR));
		return trim($space[0]);
	}
	
	function hasDiskSpace() {
		if ( ($_SESSION['name'] == "guest") || ($_SESSION['name'] == "tester") || ($_SESSION['name'] == "dang") || $_SESSION['name'] == "user014"){ return true; }			// testing users always have sapce. (user014= Omer (Dan's student))
		if (getDiskSpaceUsed() < MAX_SPACE_ALLOWED) {return true;}		
		return false;
	}
	
	function getDatMarkersAmount($file) {
		if (!is_file($file) || !($dat = fopen($file, "r"))) {
			log_error("can't open $file");
			return false;
		}			
		$line = explode(" ", rtrim(fgets($dat)));
		$amount = $line[0];
		if ($line[4] == "0") {return $amount-1;}	#One disease locus
		else {return $amount - 2;}				#Two disease loci
	}
    
	function getPedFromInput($dir, $input) {
		$input = explode("@", $input);
		if (sizeof($input) != 2){ $input[1] = $input; $input[0] = ""; }		// in case there's no '@'
		$file	= $input[1];
		$method	= $input[0];
		$us		= ($method == "") ? "" : "_";		
		$info = getExpContentInfo($dir, $file);
		return $info[$method.$us."pedfile"];
	}
    
    function getPedFamiliesAmount($file) {
        if (!is_file($file)) {
			log_error("can't open $file");
			return false;        
        }
		exec("cut -d \" \" -f 1 $file | sort | uniq | grep -v \"\^\$\"", $families);
		$amount = count($families);
		return $amount;
    }
    
	
	function showAutomateOption() {
		return; // when the automation will return to work - just need to remove the return in order to run it.
		?>
		<input type="checkbox" name="automate" id="automate" onclick="  if (!this.checked) {
																			hideElement(getElement('automateOptions'));
																			hideElement(getElement('advanced'));
																		}
																		else {
																			showElement(getElement('advanced'));
																		}"/>Expand analysis automatically<br/>
		<div id="advanced" style="display:none;">
			<a href="" onclick="switchElement('automateOptions'); return false;">Advanced automate options >></a><br/>
			<div id="automateOptions" name="automateOptions" style="display:none; /*border-collapse:collapse;*/">
				<table>
				<tr><td>MaxVar</td><td><input class="shortInput" type="text" id="MaxVar" name="MaxVar" value="0.67"/></td><td>automatic experiments will run on areas with LOD score higher than MaxVar*(maximal LOD score)</td>
				<tr><td>MinVar</td><td><input class="shortInput" type="text" id="MinVar" name="MinVar" value="1.01"/></td><td>minimal expected growth from the previous max LOD score in order to continue to a larger window. </td>
				<tr><td>MinAreaSize</td><td><input class="shortInput" type="text" id="MinAreaSize" name="MinAreaSize" value="5"/>cM</td><td>the minimal size of a high LOD score area to be considered for further analysis</td>
				<tr><td>LowRate</td><td><input class="shortInput" type="text" id="LowRate" name="LowRate" value="0.33"/></td><td>the percentage of markers allowed to have a LOD score lower than MaxVar*(maximal LOD score) in the suspect area</td>
				<tr><td>MaxLowArea</td><td><input class="shortInput" type="text" id="MaxLowArea" name="MaxLowArea" value="1"/>cM</td><td>the maximal size of a low LOD score area allowed in a continuous high LOD score area</td>
				<tr><td>MaxThresh</td><td><input class="shortInput" type="text" id="MaxThresh" name="MaxThresh" value="2"/></td><td>the lowest maximal LOD score for further analysis</td> 
				</table>
			</div>
		</div>
		<?php
	}
	
	function showSnpMapping($proj) {
		$dir = USER_PROJECTS_DIR."/".$proj."/raw";
		
		if (!is_file($dir."/".MAP_FILE_NAME)){
			echo "No mapping found for this project.";
			return;
		}
		
		$map = file($dir."/".MAP_FILE_NAME, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
		$simplifiedFile = false;
		$simplifiedMap = array();
		if (is_file($dir."/".SIMPLIFIED_PED_MAP_FILE)){
			$simplifiedFile = file($dir."/".SIMPLIFIED_PED_MAP_FILE, FILE_SKIP_EMPTY_LINES | FILE_IGNORE_NEW_LINES);
			foreach ($simplifiedFile as $simpMap){
				$simpMap = explode("\t", $simpMap);
				$simplifiedMap[$simpMap[0]."_".$simpMap[1]] = $simpMap[2];
			}
		}
		
		
		echo "Mapping between the pedigree people to the SNP names for project $proj:<br/><br/>";
		echo "<table><tr><th style=\"padding-right: 5px;\">Pedigree</th><th style=\"padding-right: 5px;\">Person</th><th style=\"padding-right: 5px;\">SNP Name</th><th>Simplified Format Name</th></tr>";
		foreach ($map as $line){
			$line = explode("\t", $line);
			$first = explode(".", $line[0]);
			$pedigree = $first[0];
			$person = $first[1];
			
			$name = $line[1];
			$simplifiedName = isset($simplifiedMap[$pedigree."_".$person]) ? $simplifiedMap[$pedigree."_".$person] : "";
		
			echo "<tr><td>$pedigree</td><td>$person</td><td/>$name</td><td>$simplifiedName</td></tr>";
		}
		echo "</table>";
	}
	
/*
 * return the display name of a tool
 */
function toolDisplayName ($tool) {
	switch ($tool) {
		case VAL_RUN_MODE_CLEAN_AND_CLUSTER:	return "cleaning and clustering"; break;
		case VAL_RUN_MODE_CLEAN_ONLY:			return "cleaning"; break;
		case VAL_RUN_MODE_CLUSTER_ONLY:			return "clustering"; break;
		case VAL_RUN_MODE_FILTERING:			return "filtering"; break;
		case VAL_RUN_MODE_ZOOM:					return "zooming"; break;
		case VAL_RUN_ANALYSIS_MULTIPOINT:		return "Multipoint"; break;
		case VAL_RUN_ANALYSIS_MULTIPOINT_HMM:	return "MultiPointHMM"; break;
		case VAL_RUN_ANALYSIS_TWOPOINT:			return "TwoPoint"; break;
		case VAL_RUN_ANALYSIS_LM_MARKERS:		return "Morgan lm_linkage"; break;
		case VAL_RUN_ANALYSIS_GL_AUTO:			return "Morgan gl_auto"; break;
		case VAL_RUN_ANALYSIS_HAPLOTYPING:		return "Haplotyping"; break;
		case VAL_RUN_ANALYSIS_SEGREGATION:		return "Segregation"; break;
		default:								return "";
	}
}
?>
