<?php

function convertThetaToCM($theta) {
    $sign = ($theta < 0) ? (-1) :  (1);
    $theta = $theta * $sign;
    $dist = (($theta >= 0) && ($theta <= 0.5)) ? (log(1 - 2*$theta) * (-50)) : (0.5);
    $dist = $dist * $sign;
    return $dist;
}

function convertCMtoTheta($dist) {
    $sign = ($dist < 0) ? (-1) :  (1);
    $dist = $dist * $sign;
    $theta = 0.5 * (1 - exp($dist/(-50))); 
    $theta = $theta * $sign;
    return $theta;
}

function binomial_coefficient_gamma($n, $k) {
  if ($n < 0 || $k < 0) { return 0;}

  return
		($k > $n) ? 0 :
		(($k == 0 || $k == $n) ? 1 :
		(($k == 1 || $k == $n - 1) ? $n :
		(exp(gamma_log($n + 1) - gamma_log($k + 1) - gamma_log($n - $k + 1)))));
}

function gamma_log ( $x )

//****************************************************************************
//
//  Purpose:
//
//    GAMMA_LOG calculates the natural logarithm of GAMMA ( X ) for positive X.
//
//  Discussion:
//
//    Computation is based on an algorithm outlined in references 1 and 2.
//    The program uses rational functions that theoretically approximate
//    LOG(GAMMA(X)) to at least 18 significant decimal digits.  The
//    approximation for 12 < X is from reference 3, while approximations
//    for X < 12.0 are similar to those in reference 1, but are unpublished.
//
//    The accuracy achieved depends on the arithmetic system, the compiler,
//    intrinsic functions, and proper selection of the machine-dependent
//    constants.
//
//  Modified:
//
//    28 May 2003
//
//  Author:
//
//    William Cody, Laura Stoltz
//    Argonne National Laboratory
//
//  Reference:
//
//    William Cody, Kenneth Hillstrom,
//    Chebyshev Approximations for the Natural Logarithm of the Gamma Function,
//    Mathematics of Computation,
//    Volume 21, Number 98, April 1967, pages 198-203.
//
//    Kenneth Hillstrom,
//    ANL/AMD Program ANLC366S, DGAMMA/DLGAMA,
//    May 1969.
//
//    John Hart, Ward Cheney, Charles Lawson, Hans Maely, Charles Mesztenyi,
//    John Rice, Henry Thatcher, Christop Witzgall,
//    Computer Approximations,
//    Wiley, 1968,
//    LC: QA297.C64.
//
//  Parameters:
//
//    Input, double X, the argument of the Gamma function.  X must be positive.
//
//    Output, double GAMMA_LOG, the logarithm of the Gamma function of X.
//    If X <= 0.0, or if overflow would occur, the program returns the
//    value XINF, the largest representable double precision number.
//
//
//  Explanation of machine-dependent constants
//
//  BETA   - radix for the real number representation.
//
//  MAXEXP - the smallest positive power of BETA that overflows.
//
//  XBIG   - largest argument for which LN(GAMMA(X)) is representable
//           in the machine, i.e., the solution to the equation
//             LN(GAMMA(XBIG)) = BETA**MAXEXP.
//
//  FRTBIG - Rough estimate of the fourth root of XBIG
//
//
//  Approximate values for some important machines are:
//
//                            BETA      MAXEXP         XBIG
//
//  CRAY-1        (S.P.)        2        8191       9.62E+2461
//  Cyber 180/855
//    under NOS   (S.P.)        2        1070       1.72E+319
//  IEEE (IBM/XT,
//    SUN, etc.)  (S.P.)        2         128       4.08E+36
//  IEEE (IBM/XT,
//    SUN, etc.)  (D.P.)        2        1024       2.55D+305
//  IBM 3033      (D.P.)       16          63       4.29D+73
//  VAX D-Format  (D.P.)        2         127       2.05D+36
//  VAX G-Format  (D.P.)        2        1023       1.28D+305
//
//
//                           FRTBIG
//
//  CRAY-1        (S.P.)   3.13E+615
//  Cyber 180/855
//    under NOS   (S.P.)   6.44E+79
//  IEEE (IBM/XT,
//    SUN, etc.)  (S.P.)   1.42E+9
//  IEEE (IBM/XT,
//    SUN, etc.)  (D.P.)   2.25D+76
//  IBM 3033      (D.P.)   2.56D+18
//  VAX D-Format  (D.P.)   1.20D+9
//  VAX G-Format  (D.P.)   1.89D+76
//
{
	$c = array(
    -1.910444077728E-03,
     8.4171387781295E-04,
    -5.952379913043012E-04,
     7.93650793500350248E-04,
    -2.777777777777681622553E-03,
     8.333333333333333331554247E-02,
     5.7083835261E-03 );
  $corr;
  $d1 = - 5.772156649015328605195174E-01;
  $d2 =   4.227843350984671393993777E-01;
  $d4 =   1.791759469228055000094023E+00;
  $frtbig = 1.42E+09;
  $i;
  $p1 = array(
    4.945235359296727046734888E+00,
    2.018112620856775083915565E+02,
    2.290838373831346393026739E+03,
    1.131967205903380828685045E+04,
    2.855724635671635335736389E+04,
    3.848496228443793359990269E+04,
    2.637748787624195437963534E+04,
    7.225813979700288197698961E+03 );
  $p2 = array(
    4.974607845568932035012064E+00,
    5.424138599891070494101986E+02,
    1.550693864978364947665077E+04,
    1.847932904445632425417223E+05,
    1.088204769468828767498470E+06,
    3.338152967987029735917223E+06,
    5.106661678927352456275255E+06,
    3.074109054850539556250927E+06 );
  $p4 = array(
    1.474502166059939948905062E+04,
    2.426813369486704502836312E+06,
    1.214755574045093227939592E+08,
    2.663432449630976949898078E+09,
    2.940378956634553899906876E+010,
    1.702665737765398868392998E+011,
    4.926125793377430887588120E+011,
    5.606251856223951465078242E+011 );
  $pnt68 = 0.6796875E+00;
  $q1 = array(
    6.748212550303777196073036E+01,
    1.113332393857199323513008E+03,
    7.738757056935398733233834E+03,
    2.763987074403340708898585E+04,
    5.499310206226157329794414E+04,
    6.161122180066002127833352E+04,
    3.635127591501940507276287E+04,
    8.785536302431013170870835E+03 );
  $q2 = array(
    1.830328399370592604055942E+02,
    7.765049321445005871323047E+03,
    1.331903827966074194402448E+05,
    1.136705821321969608938755E+06,
    5.267964117437946917577538E+06,
    1.346701454311101692290052E+07,
    1.782736530353274213975932E+07,
    9.533095591844353613395747E+06 );
  $q4 = array(
    2.690530175870899333379843E+03,
    6.393885654300092398984238E+05,
    4.135599930241388052042842E+07,
    1.120872109616147941376570E+09,
    1.488613728678813811542398E+010,
    1.016803586272438228077304E+011,
    3.417476345507377132798597E+011,
    4.463158187419713286462081E+011 );
  $res;
  $sqrtpi = 0.9189385332046727417803297E+00;
  $xbig = 4.08E+36;
  $xden;
  $xm1;
  $xm2;
  $xm4;
  $xnum;
  $xsq;
//
//  Return immediately if the argument is out of range.
//
  if ( $x <= 0.0 || $xbig < $x )
  {
    return r8_huge ( );
  }

  if ( $x <= r8_epsilon ( ) )
  {
    $res = -log ( $x );
  }
  else if ( $x <= 1.5 )
  {
    if ( $x < $pnt68 )
    {
      $corr = - log ($x );
      $xm1 = $x;
    }
    else
    {
      $corr = 0.0;
      $xm1 = ( $x - 0.5 ) - 0.5;
    }

    if ( $x <= 0.5 || $pnt68 <= $x )
    {
      $xden = 1.0;
      $xnum = 0.0;

      for ( $i = 0; $i < 8; $i++ )
      {
        $xnum = $xnum * $xm1 + $p1[$i];
        $xden = $xden * $xm1 + $q1[$i];
      }

      $res = $corr + ( $xm1 * ( $d1 + $xm1 * ( $xnum / $xden ) ) );
    }
    else
    {
      $xm2 = ( $x - 0.5 ) - 0.5;
      $xden = 1.0;
      $xnum = 0.0;
      for ( $i = 0; $i < 8; $i++ )
      {
        $xnum = $xnum * $xm2 + $p2[$i];
        $xden = $xden * $xm2 + $q2[$i];
      }

      $res = $corr + $xm2 * ( $d2 + $xm2 * ( $xnum / $xden ) );

    }
  }
  else if ( $x <= 4.0 )
  {
    $xm2 = $x - 2.0;
    $xden = 1.0;
    $xnum = 0.0;
    for ( $i = 0; $i < 8; $i++ )
    {
      $xnum = $xnum * $xm2 + $p2[$i];
      $xden = $xden * $xm2 + $q2[$i];
    }

    $res = $xm2 * ( $d2 + $xm2 * ( $xnum / $xden ) );
  }
  else if ( $x <= 12.0 )
  {
    $xm4 = $x - 4.0;
    $xden = - 1.0;
    $xnum = 0.0;
    for ( $i = 0; $i < 8; $i++ )
    {
      $xnum = $xnum * $xm4 + $p4[$i];
      $xden = $xden * $xm4 + $q4[$i];
    }

    $res = $d4 + $xm4 * ( $xnum / $xden );
  }
  else
  {
    $res = 0.0;

    if ( $x <= $frtbig )
    {

      $res = $c[6];
      $xsq = $x * $x;

      for ( $i = 0; $i < 6; $i++ )
      {
        $res = $res / $xsq + $c[$i];
      }

    }

    $res = $res / $x;
    $corr = log ( $x );
    $res = $res + $sqrtpi - 0.5 * $corr;
    $res = $res + $x * ( $corr - 1.0 );

  }

  return $res;
}

function r8_epsilon ( )

//****************************************************************************
//
//  Purpose:
//    R8_EPSILON returns the R8 round off unit.
//
//  Discussion:
//    R8_EPSILON is a number R which is a power of 2 with the property that,
//    to the precision of the computer's arithmetic,
//      1 < 1 + R
//    but
//      1 = ( 1 + R / 2 )
//
//  Modified:
//    06 May 2003
//
//  Parameters:
//    Output, double R8_EPSILON, the round-off unit.
//
{
  $r = 1.0;

  while ( 1.0 < ( 1.0 + $r )  )
  {
    $r = $r / 2.0;
  }

  return ( 2.0 * $r );
}

function r8_huge ( )

//****************************************************************************
//
//  Purpose:
//    R8_HUGE returns a "huge" R8.
//
//  Discussion:
//    HUGE_VAL is the largest (?) representable legal real number

{
	$HUGE_VAL = 1.8e308;
	return $HUGE_VAL;
}
//****************************************************************************
define("MAX_MARKERS_IN_SNP", "25000");
	
//****************************************************************************
// Given $n and $total size, returns which indexes to take from these $n
// $totalSize - the size of the whole input
// $n - the size of a small part in the whole input, which from that whole size k should be taken.
// Returns an array in the size of $n, with "1" where the item in that index should be taken
//	and "0" where the item in that index should not.
// Returns false if the calculation cann't be done due to wrong input or numbers too high to represent
//****************************************************************************
function getChosenIndexes($n, $totalSize){
	//echo "n: $n, total: $totalSize<br/><br/>";
	
	$ratio = ($totalSize > $n) ? $n / $totalSize : 1;
	$newTotalSize = ($totalSize > MAX_MARKERS_IN_SNP) ? MAX_MARKERS_IN_SNP : $totalSize;
	//echo "ratio: $ratio<br/>";
	$k = ceil($newTotalSize * $ratio);
	if ($k > $n){ return false; }
	//echo "k: $k<br/>";
	$BCG = binomial_coefficient_gamma($n, $k);
	if ($BCG > mt_getrandmax()) { return false; }
	$psi = mt_rand(0, $BCG-1);
	//echo "psi: $psi<br/>";
	return getChosenIndexes_t($n, $k, $psi);
}

// chooses $k from $n with a random parameter $psi where 0 <= $psi < ($n C $k)
// the result is returned as a $indexesArray parameter, its size is $n
/*****************************************************
 Orinial Algorithm:
	if($k == 0) return str_repeat("0", $n);
	
	for( $i = 1 ; $i <= $n - $k +1; $i += 1){
		$BCG = binomial_coefficient_gamma($n-$i, $k-1);
		if ($psi >= $BCG){
			$psi = $psi - $BCG;
		}
		else{
			return str_repeat("0", $i-1) . "1" . getChosenIndexes($n - $i , $k - 1, $psi);
		}
	}
**********************************************************/
function getChosenIndexes_t($n, $k, $psi){
	$chosenIndexes = array();
	if ($k == 0) return array_pad($chosenIndexes, $n, 0);
	
	for( $i = 1 ; $i <= $n - $k +1; $i += 1){
		$BCG = binomial_coefficient_gamma($n-$i, $k-1);
		if ($psi >= $BCG){
			$psi = $psi - $BCG;
			$chosenIndexes[$i-1] = 0;
		}
		else{
			$chosenIndexes[$i-1] = 1;
			return array_merge($chosenIndexes, getChosenIndexes_t($n - $i , $k - 1, $psi));
		}
	}
	
	return $chosenIndexes;		// not really needed
}

//****************************************************************************
// Given $n and $total size, returns which indexes to take from these $n
// $totalSize - the size of the whole input
// $n - the size of a small part in the whole input, which from that whole size k should be taken.
// Returns an array in the size of $n, with "1" where the item in that index should be taken
//	and "0" where the item in that index should not.
// Returns false if the calculation cann't be done due to wrong input or numbers too high to represent
//****************************************************************************
function ChooseIndexes($n, $totalSize, $maxSize = MAX_MARKERS_IN_SNP) {
	//echo "n: $n, total: $totalSize<br/><br/>";
	
	$ratio = ($totalSize > $n) ? $n / $totalSize : 1;
	$newTotalSize = ($totalSize > $maxSize) ? $maxSize : $totalSize;
	//echo "ratio: $ratio<br/>";
	$k = ceil($newTotalSize * $ratio);
	if ($k > $n){ return false; }
	//echo "k: $k<br/>";
	
	$helpArray = array();
	$chosenInexes = array();
	for ($i = 0 ; $i < $n ; $i += 1){						// initiation
		$helpArray[$i] = $i;
		$chosenInexes[$i] = 0;
	}

	for ($i = 0 ; $i < $k ; $i += 1){						// choose $k indexes
		$choose = mt_rand(0, sizeof($helpArray)-1);			// random index to choose
		$chosenInexes[$helpArray[$choose]] = 1;
		$helpArray[$choose] = $helpArray[sizeof($helpArray)-1];		// put last index in the chosen's index spot and remove last spot
		unset($helpArray[sizeof($helpArray)-1]);
	}
	
	return $chosenInexes;
}

function bigNumbersRandom($number){
	$maxDigits = 9;
	$zeros = 0;
	if (strrpos($number, "E")){
		$zeroes = substr($number, strrpos($number, 'E')+1);
		$number = substr($number, 0, strrpos($number, 'E'));
		$number = str_replace(".", "", $number);
		$zeros = $zeroes - (strlen($number) - 1);
		$number = $number.str_repeat("0", $zeros);
	}
	$temp1 = $number;
	$temp2 = $number;
	$randVal = "";
	echo "<br/><br/>=============== BigRandom Start<br/><br/>";
	while($temp1 > mt_getrandmax()){
		$temp2 = substr($temp1 , 0, $maxDigits);
		$randVal = $randVal.mt_rand(0,$temp2);
		echo "Temp1: $temp1, Temp2: $temp2, randVal: $randVal<br/>";
		$temp1 = substr($temp1 , $maxDigits+1);
		if (($temp1 != "") && ($temp1 == 0)) { $temp1 = "1".$temp1; }
	}
	echo "Last temp1: $temp1<br/>";
	if ($temp1 != ""){
		$randVal = $randVal.mt_rand(0,$temp1);
	}
	echo "Final randVal: $randVal<br/>";
	echo "<br/>=============== BigRandom End<br/><br/>";
	return $randVal;
}

function getChromosomePartOfDNA($chrom){	
	switch ($chrom) {
		case 1: return 8.02; break;
		case 2: return 7.86; break;
		case 3: return 6.46; break;
		case 4: return 6.2; break;
		case 5: return 5.88; break;
		case 6: return 5.55; break;
		case 7: return 5.16; break;
		case 8: return 4.74; break;
		case 9: return 4.55; break;
		case 10: return 4.38; break;
		case 11: return 4.35; break;
		case 12: return 4.29; break;
		case 13: return 3.7; break;
		case 14: return 3.44; break;
		case 15: return 3.25; break;
		case 16: return 2.89; break;
		case 17: return 2.56; break;
		case 18: return 2.47; break;
		case 19: return 2.05; break;
		case 20: return 2.01; break;
		case 21: return 1.53; break;
		case 22: return 1.62; break;
		case "X": return 5.03; break;
		case "x": return 5.03; break;
		default: return -1; break;
	}
	return -1;
}

?>