#!/bin/bash
# this is a submission script for running 
basepath=`dirname $0`
source ${basepath}/CONFIG.SH
source ${basepath}/Utils.sh
lockfile=$NO_CONNECT_LOCKFILE


printIncludeFile(){
	echo "$1" >> $REMOTE_STAT_INCLUDE_FILE 
}
printlnIncludeFile(){
	echo "$1 <br>" >> $REMOTE_STAT_INCLUDE_FILE 
}

generateStat(){
	stat_file=$1
	[ -s $stat_file ] || {
		return;
	}

	line=`tail -1 $stat_file`
	error=`echo $line | grep ERROR`
	[ -z "$error" ] || {
		printlnIncludeFile "<b><font color=\"#FF0000\">System is temporary unavailable</font></b>" 
		Error "System is unavailable due to error in condor_q"
		exit;	
	}
	tasks=`echo $line | awk '{match($0,/TASKS=([[:digit:]]+)/,tasks);print tasks[1]}'`
	[ -z "$tasks" ] && return
	
	idle=`echo $line  | awk '{match($0,/IDLE=([[:digit:]]+)/,k); print k[1]}'`
	busy=`echo $line  | awk '{match($0,/BUSY=([[:digit:]]+)/,k); print k[1]}'`
	
	[ ! -z "$idle" -a $idle  -gt 0 ] &&  let idle_count+="$idle"
	[ ! -z "$busy" -a $busy  -gt 0 ] &&  let busy_count+="$busy"
	[ ! -z "$tasks" -a $tasks  -gt 0 ] &&  let task_count+="$tasks"

}

createQueueStat(){
	
	overload=$1
	shift
	pools=$*

	busy_count=0
	idle_count=0
	task_count=0

	for i in $pools; do
		gw=`echo $i | cut -f1 -d":"`
		host=`echo $gw | cut -f2 -d"_"`
		lockfile=${NO_CONNECT_LOCKFILE}_$gw
		if [ -e $lockfile ]; then
			printlnIncludeFile "<b><font color=\"#FF0000\">The system is temporary unavailable.</font></b>"
			exit;
		else
			generateStat "$i.${REMOTE_STAT_FILE_PREFIX}"
		fi
	done

	if [ $task_count -eq 0 ]; then
	 	printIncludeFile "No tasks running"
	else
		printIncludeFile "running $task_count tasks, using ${busy_count} computers, ${idle_count} subtasks waiting for execution"
		[ $idle_count -gt $overload ] && printlnIncludeFile "<br><b>Load is quite high. New tasks may be rejected or may not start immediately</b>"
	fi
	printlnIncludeFile " "

	busy_count=0
	idle_count=0
	task_count=0
}

# prevent any output
exec 1>> ${GLOBAL_LOG}_submission_tester
exec 2>> ${GLOBAL_LOG}_submission_tester
rm ${GLOBAL_LOG}_submission_tester
touch ${GLOBAL_LOG}_submission_tester
set -x
###

STAT=${LOG}/STAT
rm -rf ${STAT}
mkdir -p ${STAT}

for i in ${STAT_GATEWAYS[*]}; do
	gw=`echo $i | cut -f1 -d":"`
	lockfile=${NO_CONNECT_LOCKFILE}_$gw
	rpath=`echo $i| cut -f2 -d":"`
	username=`echo $gw | cut -f1 -d"_"`
	hostname=`echo $gw | cut -f2 -d"_"`
	export SSH_ASKPASS=${ASKPASS_SCRIPTS}/$gw
 	${scpcmd} ${username}@${hostname}:${rpath}/* ${STAT}/ >> ${GLOBAL_LOG}_submission_tester 2>&1

	[ ! -z "${rpath}" ] || continue;
	 ${sshcmd} ${username}@${hostname}  "rm -rf $rpath/*.${REMOTE_STAT_FILE_PREFIX} " >>${GLOBAL_LOG}_submission_tester 
	ret=$?
# ret=0
	if [ $ret -ne 0 ]; then
		# we don't want to send this message many times. So the first error will trigger it
		[ -e ${lockfile} ] && continue;
		echo "The system is unable to connect to execution server  ${gw}" | ${ADMIN_MAIL_SENDER} 
		Error "Cannot connect to ${gw}"
		touch ${lockfile}
	else
		[ -e ${lockfile} ] && { 
				echo "Connection with the execution server  ${gw} successfully established" | ${ADMIN_MAIL_SENDER} > /dev/null 2>&1
				rm -f $lockfile 
				Log "Connection restored to $gw";
		}
	fi
done
########################## analyze utilization here ##############
cwd=`pwd`
cd ${STAT}
rm -f $REMOTE_STAT_INCLUDE_FILE
printlnIncludeFile "Last update at: `date`"
printlnIncludeFile " "
printIncludeFile "Short jobs queue load: "
createQueueStat $NUM_JOBS_OVERLOAD_FF ${FF_POOLS[*]}
printIncludeFile "Long jobs queue load: "
createQueueStat  $NUM_JOBS_OVERLOAD_BF ${BF_POOLS[*]}
printlnIncludeFile " "
# create index file for all html files

cat>index.html <<EOF
<html>
<head><title>Statistics for all execution pools</title></head>
<body>
<h3> All pools used by superlink-online </h3>
EOF

for i in *.html; do
	[ $i = "index.html" -o $i = "help.html" ] ||  echo "<a href=\"$i\">$i</a><br>" >> index.html
done

echo "</body>" >> index.html
echo "</html>" >> index.html

mv *.html $REMOTE_HTML_STAT_LOCATION

for i in *.${REMOTE_STAT_FILE_PREFIX}; do
	cat $i >> ${LOG}/$i
done

