#!/bin/bash
# this is a submission script for running 
set -x
basepath=`dirname $0`
source ${basepath}/CONFIG.SH
source ${basepath}/Utils.sh

Die_Mail(){
	
	echo "Failure The system experiences internal failure. The error is reported to the administrator. Please try again later." | ./${MAIL_SENDER} 1
	echo "Failure: Bug found in submission: $1." | ${ADMIN_MAIL_SENDER}; 
	Die $1;	
}

Log "Starting submission with parameters $*"
[ $# -ge 10 -a $# -le 11 ] || Die_Mail "Wrong invocation parameters. Expected: PRE_SCRIPT POST_SCRIPT LIB_FILE ERROR_MSG_DIR LIST_OF_FILES PORTAL_RUNDIR EMAIL PROJECT_NAME OUTPUT_FILE [LINKAGE|LINKAGE_SERIAL|HAPLO] [optional params: BEST_EFFORT | SPECIFIC_POINT ]. Example: /var/portal/XXXX preTwoPoint.pl postTwoPoint.pl lib.pl Errors filelist.txt /var/portal/XXX/ marks@tx.mail.com TEST /var/portal/XXXX/output.html LINKAGE"

PRE_SCRIPT=$1
shift
POST_SCRIPT=$1
shift
LIB_FILE=$1
shift 
TEXT_DIR=$1

shift
LIST_OF_FILES=$1

TASK_PORTAL_RUNDIR=$2
TASK_EMAIL=$3
TASK_PROJECT_NAME=$4
TASK_OUTPUT_FILE=$5
TASK_TYPE=$6
[ "$TASK_TYPE" = "LINKAGE" -o "$TASK_TYPE" = "LINKAGE_SERIAL" -o "$TASK_TYPE" = "HAPLO" ] || Die_Mail "Wrong type specified: $TASK_TYPE. Supported only LINKAGE, LINKAGE_SERIAL, HAPLO"

#############HARD CODE BEST EFFORT########
TASK_BEST_EFFORT=1
##########################################
if [ $# -eq 7 -a "$7" = "BEST_EFFORT" ] ; then 
	TASK_BEST_EFFORT=1
	TASK_SPECIFIC_POINT=""
else if [ $# -eq 7 -a "$7" = "SPECIFIC_POINT" ]; then
	TASK_SPECIFIC_POINT=1 ; TASK_BEST_EFFORT="";
 fi
fi

TASK_PORTAL_HOST=${PORTAL_HOST}
TASK_UID=`basename $TASK_PORTAL_RUNDIR`
cd ${TASK_PORTAL_RUNDIR} || Die_Mail "Cannot chdir to ${TASK_PORTAL_RUNDIR}"
# copy and modify mail_sender
html_root=`echo $PORTAL_HTML_ROOT | sed 's/\//\\\\\\//g'`
html_root=${html_root}\\/${TASK_UID}

cwd=`pwd | sed 's/\//\\\\\\//g'`

part_results=`basename ${TASK_OUTPUT_FILE}`
cat ${TMPL}/${MAIL_SENDER} | sed "s/PEDTOOL_SERVER_DIR/${html_root}/" | sed "s/PEDTOOL_PORTAL/${PORTAL_HOST}/" | sed "s/PEDTOOL_USER_MAIL/${TASK_EMAIL}/" | sed "s/PEDTOOL_PROJECT_NAME/${TASK_PROJECT_NAME}/" | sed "s/PEDTOOL_SMTP_SERVER/${SMTP_SERVER}/" | sed "s/PEDTOOL_SERVER_OUTPUT_FILE/${part_results}/" | sed "s/PEDTOOL_SERVER_STATUS_FILE/${STATUS_FILE}/" | sed "s/PEDTOOL_FULL_DIR/$cwd/" | sed "s/PEDTOOL_EMAIL/${PORTAL_EMAIL}/" > ${MAIL_SENDER}   || Die_Mail "Failed to copy $MAIL_SENDER"
chmod +x ${MAIL_SENDER}
[ -e ${TASK_PORTAL_RUNDIR} ] || Die_Mail "Cannot find $TASK_PORTAL_RUNDIR"


cat > Details.txt <<EOF
TASK_PORTAL_RUNDIR=$TASK_PORTAL_RUNDIR
TASK_EMAIL=$TASK_EMAIL
TASK_PROJECT_NAME=$TASK_PROJECT_NAME
TASK_OUTPUT_FILE=$TASK_OUTPUT_FILE
TASK_PORTAL_HOST=${PORTAL_HOST}
TASK_SPECIFIC_POINT=$TASK_SPECIFIC_POINT
TASK_BEST_EFFORT=$TASK_BEST_EFFORT
TASK_TYPE=$TASK_TYPE
EOF

CheckJobSubmitDetails || Die_Mail "Some submission parameters are missing"

[ -r pedfile.dat ] || Die_Mail "Cannot find pedfile.dat file"
[ -r datafile.dat ] || Die_Mail "Cannot find datafile.dat file"
[ -r Details.txt ] || Die_Mail "Cannot find Details.txt file"

######
# we may want to create various files with submit status and remove - it should be here
# create status file

remove_script=`echo $REMOVE_CGI_SCRIPT | sed 's/\//\\\\\\//g'`
cat ${TMPL}/${REMOVE_FILE}.tmpl | sed "s/PEDTOOL_PROJECT_NAME/${TASK_PROJECT_NAME}/g" | sed "s/PEDTOOL_UID/${TASK_UID}/g" | sed "s/PEDTOOL_REMOVE_SCRIPT/${remove_script}/"> ${REMOVE_FILE}

# copy STATUS_UPDATER and the templates for result update
ln -sf $STATUS_UPDATER . || Die_Mail "Failed to copy $STATUS_UPDATER"

cat ${TMPL}/${STATUS_FILE}.begin | sed "s/PEDTOOL_PROJECT_NAME/${TASK_PROJECT_NAME}/g" > ${STATUS_FILE}.begin
cat ${TMPL}/${STATUS_FILE}.end | sed "s/PEDTOOL_RESULTS_REF/${part_results}/g" > ${STATUS_FILE}.end
echo "Pending execution" | ./status_updater.sh

######

echo "Starting submission"
#we first of all check if connection can be established

[ -e $NO_CONNECT_LOCKFILE ] && {

	echo "Failure The system is unable to connect to execution server. Your task cannot be executed now, please try again later." | ./${MAIL_SENDER} 1
	echo "Failure: The system is unable to connect to execution server due to  $NO_CONNECT_LOCKFILE." | ${ADMIN_MAIL_SENDER}; 
	exit -1;
}

# tar files from the $CGI_BIN dir 

set -x

cp $CGI_BIN/$PRE_SCRIPT Pre.pl || Die_Mail "Failed to create Pre.pl"
cp $CGI_BIN/$POST_SCRIPT Post.pl || Die_Mail "Failed to create Post.pl"
rm to_pack
echo Pre.pl >> to_pack
echo Post.pl >> to_pack

# dvir 2011-08-23, "--dereference" in order to archive the target of the soft link and not the soft link imself.
# ($LIB_FILE may include a .pm lib which is in fact a soft link, for example : WebServerEnvironment.pm -> Environments/Development.pm).
tar -C $CGI_BIN -c -v --exclude CVS --dereference -f all_execs.tar $LIB_FILE $TEXT_DIR || Die_Mail "Failed to create all_execs.tar"
echo all_execs.tar >> to_pack

[ -e $LIST_OF_FILES ] || Die_Mail "File $LIST_OF_FILES Not found!"
cat $LIST_OF_FILES >> to_pack

echo datafile.dat >> to_pack
echo pedfile.dat >> to_pack
echo Details.txt >> to_pack 
[ -e to_pack ] || Die_Mail "Failed to create to_pack file"

tar -T to_pack -z -h -c -f Submission.tar.gz || Die_Mail "Failed to create tar" 
[ -s Submission.tar.gz ] || Die_Mail "Failed to create tar"
size=`stat -c%s Submission.tar.gz`
# dvir, omer 2011/08/17, we have raised the max file size of the Submission.tar.gz
[ $size -gt 2000000 ] && Die_Mail "submissions of 2MB not allowed"
#[ $size -gt 500000 ] && Die_Mail "submissions of 0.5MB not allowed"

Log "current user: $USER current dir: $cwd"
Log "${sshcmd} ${REMOTE_USER}@${CONDOR_GW} $REMOTE_SUBMIT_SCRIPT $TASK_UID"
cat Submission.tar.gz |  ${sshcmd} ${REMOTE_USER}@${CONDOR_GW} $REMOTE_SUBMIT_SCRIPT $TASK_UID
ret=$?

LOAD_TOO_HIGH=10
NO_LOGIN_DETECTED=13
ALREADY_SUBMITTED=100

Log "$REMOTE_SUBMIT_SCRIPT returned $ret" 
cp -f ${TMPL}/EmptyResults.html $TASK_OUTPUT_FILE
chmod 666  $TASK_OUTPUT_FILE
[ $ret -eq 0 ] && { echo "Submission successfull";  exit 0; }

echo "Submission failed"
case "$ret" in
 ${LOAD_TOO_HIGH})
	 echo "Rejected The system is under high load. Your task cannot be executed now, please try again later."
	# | ./${MAIL_SENDER} 1
	 echo "Rejected The system is under high load. " 
	#| ${ADMIN_MAIL_SENDER}
;;
 ${NO_LOGIN_DETECTED})
	echo "Rejected The system is experiencing major failure and cannot execute your task. Please try again later."
	# | ./${MAIL_SENDER} 1
	echo "Rejected The system is experiencing major failure."
	# | ${ADMIN_MAIL_SENDER}
;;
 $ALREADY_SUBMITTED)
	Log "Got already submitted task twice"
	exit 0 
;;
*)
	# we don't know the reason here
	echo "Failure The system is unable to connect to execution server. Your task cannot be executed now, please try again later."# | ./${MAIL_SENDER} 1
	echo "Failure: The system is unable to connect to execution server."# | ${ADMIN_MAIL_SENDER}
;;
esac
exit $ret;
