#!/bin/bash
# this is a submission script for running 
basepath=`dirname $0`
source ${basepath}/CONFIG.SH
source ${basepath}/Utils.sh
lockfile=$NO_CONNECT_LOCKFILE

printIncludeFile(){
	echo "$1 <br>" >> $REMOTE_STAT_INCLUDE_FILE 
}
generateStat(){
	output_msg=$1
	stat_file=$2
	[ -e $stat_file ] || return

	line=`tail -1 $stat_file`
	[ -z "$line" ] && return
	error=`echo $line | grep ERROR`
	[ -z "$error" ] || {
		printIncludeFile "$output_msg <b><font color=\"#FF0000\">Condor failure</font></b>" 
		return;
	}
	tasks=`echo $line | awk '{match($0,/TASKS=([[:digit:]]+)/,tasks);print tasks[1]}'`
	[ -z "$tasks" ] && return
	
	idle=`echo $line  | awk '{match($0,/IDLE=([[:digit:]]+)/,k); print k[1]}'`
	busy=`echo $line  | awk '{match($0,/BUSY=([[:digit:]]+)/,k); print k[1]}'`

	[ $tasks -eq 0 ] && {
		printIncludeFile "$output_msg No tasks running";
		return;
	}

	printIncludeFile "$output_msg running $tasks tasks, using ${busy} computers, ${idle} subtasks waiting for execution"
	[ $idle -gt $NUM_JOBS_OVERLOAD_FF ] && printIncludeFile "Load is quite high. Computations may be slow."
}

# prevent any output
#exec 2>/dev/null
#exec 1>/dev/null
###
${sshcmd} ${REMOTE_USER}@${CONDOR_GW} "for i in \`ls $REMOTE_UTILIZATION_STATUS_LOCATION/*.${REMOTE_STAT_FILE_PREFIX}\`; do basename \$i; cat \$i; rm \$i; done " 2>${GLOBAL_LOG}_submission_tester >$LOG/Utilization.log  
# ${sshcmd} ${REMOTE_USER}@${CONDOR_GW} "for i in \`ls $REMOTE_UTILIZATION_STATUS_LOCATION/*.${REMOTE_STAT_FILE_PREFIX}\`; do basename \$i; cat \$i; done " 2>${GLOBAL_LOG}_submission_tester >$LOG/Utilization.log  
ret=$?
[ $ret -ne 0 ] && {
	# we don't want to send this message many times. So the first error will trigger it
	[ -e $lockfile ] && exit
	echo "The system is unable to connect to execution server  ${REMOTE_USER}@${CONDOR_GW}" | ${ADMIN_MAIL_SENDER} > /dev/null 2>&1
	touch $lockfile
	echo "<b><font color=\"#FF0000\">Communication failure. Please try again later.</font></b>" > $REMOTE_STAT_INCLUDE_FILE
	exit -1;
}
[ -e $lockfile ] && echo "Connection with the execution server  ${REMOTE_USER}@${CONDOR_GW} successfully established" | ${ADMIN_MAIL_SENDER} > /dev/null 2>&1
rm -f $lockfile 
########################## analyze utilization here ##############
cwd=`pwd`
cd ${LOG}
created_files=(`cat ${LOG}/Utilization.log  | awk '{ if (($0!~/DATE/) && ($0!~/[[:space:]]+/)) {file=$0;print file} else {print $0>>file} }'`)
echo $created_files
frontfill="Short jobs queue load: "
frontfill_stat_file="${REMOTE_USER}_${CONDOR_GW}.${REMOTE_STAT_FILE_PREFIX}"
backfill="Heavy jobs queue load: "
rm -f $REMOTE_STAT_INCLUDE_FILE

# nothing created
[ ${#created_files[*]} -eq 0 ] && {
	printIncludeFile "$frontfill No data available"
	printIncludeFile "$backfill No data available"
}

[ ${#created_files[*]} -eq 1 ] && {
	if [ "${created_files[0]}" = "$frontfill_stat_file" ]; then
		generateStat "$frontfill" "${created_files[0]}"
		printIncludeFile "$backfill No data available"
	else
		printIncludeFile "$frontfill No data available"
		generateStat "$backfill" "${created_files[0]}"
	fi;		
}

[ ${#created_files[*]} -eq 2 ] && {
		[ "${created_files[0]}" = "$frontfill_stat_file" ] && bf=1 || bf=0
		generateStat "$frontfill" "${frontfill_stat_file}"
		generateStat "$backfill" "${created_files[$bf]}"
}

stat_files=(`ls *.${REMOTE_STAT_FILE_PREFIX}`)
if [ ${#stat_files[*]} -eq 2 ]; then
	[ "${stat_files[0]}" = "${frontfill_stat_file}" ] && backfill_file="${stat_files[1]}" || backfill_file="${stat_files[0]}";
else
	[ -e ${frontfill_stat_file} ] && backfill_file="NO_FILE" 
fi

$BIN/UpdateStatFiles.pl $frontfill_stat_file $backfill_file $GRAPH_TASKS_FILE $GRAPH_CONDOR_FILE
cp $TMPL/Week.html $GRAPH_LOCATION

printIncludeFile "<p><p>Press the <a href=usage-stat/Week.html>link</a> to view usage statistics for the last week."
