<?php
	/* a group of functions for creating superlink ped files. */
	require_once("snpManipulation.php");
	require_once("config.php");

	function getPersonFileMap($dir, $mappingFile){
		if (!($mapFile = fopen($dir."/".$mappingFile, "r"))){
			echo "error while opening mapping file.";
			return false;
		}
		
		$line = fgets($mapFile);
		while (!feof($mapFile)){		// getting to the first line after the namesLine
			$Line = split("[\t\n]", $line);
			$map[$Line[1]] = $Line[2];           // line is:   pedNum	PersonID	SnpFile
			$line = fgets($mapFile);
		}
		
		fclose($mapFile);
		return $map;
	}
	
	function getNumToPersonMap($dir, $mappingFile){
		if (!($mapFile = fopen($dir."/".$mappingFile, "r"))){
			echo "error while opening mapping file.";
			return false;
		}
		
		$line = fgets($mapFile);
		while (!feof($mapFile)){		// getting to the first line after the namesLine
			$Line = split("[\t\n]", $line);
			$map[$Line[0]] = $Line[1];           // line is:   pedNum	PersonID	SnpFile
			$line = fgets($mapFile);
		}
		
		fclose($mapFile);
		return $map;
	}
		
	/*$dir - files directory.
	* $snpFileList - list of snp files to read from.
	* $IdToFileHashMap - mapping between ID's to snp file with the most updated info for him. 
	* $chrom is the number of chromosome to make the DB for.
	* $chromCol is the column where the chromosome is written.
	* returns a HaspMap of ID's to lines of calls from the snpFiles.
	*/
	function createInfoDB($dir, $snpFileList, $IdToFileHashMap, $chrom ,$chromCol){
		if ( (!(is_dir($dir))) || ($IdToFileHashMap == null) ){
			return false;
		}
		
		// $DB = null;
		$DB = "";
		
		foreach ($snpFileList as $snp){
			$nLine = getNamesLineFromSnp($dir, $snp);
			if (!($nLine)){
				continue;                // move to next snp file on the list
			}
			$namesLine = split("\t",$nLine);
			
			$nonNameRegExp = NON_NAME_REG_EXP;
			$nameRegExp = NAME_REG_EXP;
			$selFileInitial = SELECTION_FILE_INITIAL;
			
			$selectionFile = file($dir."/".SELECTION_FILE_INITIAL.$snp);
			$selection = $selectionFile[0];
			$selection = split(" ",$selection);
			
			if (!($snpFile = fopen($dir."/".SORTED_SNP_INITIAL.$snp, "r"))){
				return false;
			}
			$Line = "";
			while ((!feof($snpFile)) && ($nLine != $Line)){		// getting to the first line after the namesLine
				$Line = fgets($snpFile);
			}
			$Line = fgets($snpFile);
			
			while ((!feof($snpFile))){
				$Line = split("\t", $Line);
				if ($Line[$chromCol] != $chrom){
					$Line = fgets($snpFile);
					continue;
				}
			/*	else{
					if ($Line[$chromCol] > $chrom){       // assuming the snpfile is sorted by chromosome   *** and the first if should be with "<" ***
						break;
					}
				} */
				//foreach ($namesLine as $name){
				$i = 0;
				$DB['markers'] = 0;
				for ($i = 0 ; $i < sizeof($namesLine) ; $i = $i+1 ){
					if ($selection[$i] == 0){   // if column not selected for useage
						continue;
					}
					$name = $namesLine[$i];
					if ( (!preg_match($nonNameRegExp,$name)) && (preg_match($nameRegExp,$name)) ){       // if its a person's column
						if ( (isset($IdToFileHashMap[$name])) && ($IdToFileHashMap[$name] == $snp) ){         // if this person's info should be taken from this $snp file
							$call = $Line[$i];
							if (preg_match(MATCHES_FOR_11 ,$call)){
								$call = "1 1";
							}
							elseif (preg_match(MATCHES_FOR_12 ,$call)){
								$call = "1 2";
							}
							elseif (preg_match(MATCHES_FOR_21 ,$call)){
								$call = "2 1";
							}
							elseif (preg_match(MATCHES_FOR_22 ,$call)){
								$call = "2 2";
							}
							elseif (preg_match(MATCHES_FOR_00 ,$call)){
								$call = "0 0";
							}
							else { $call = "0 0"; } //unknown pattern

							if (!isset($DB[$name])){
								$DB[$name] = "";
							}
							$DB[$name] = $DB[$name]." ".$call;
						}
					}
				}
				
				$Line = fgets($snpFile);
			}
			
			fclose($snpFile);
		}
		
		return $DB;
	}
	
	/* mapping file must be in the directory $dir.
	* will not overwrite existing ped file unless $overwrite is specified with true.
	* $origPed - orininal ped file
	* $snpFileList - an array of the snp files to take data from.
	* $chromNum - number of the chromosome the ped is for.
	* $chromColMap - hash of each snpFile in the List to the column number of the chromosome in this file. if no mapping specified default value will be the second column.
	*/
	function createPedBy($dir, $origPed, $snpFileList, $mappingFile, $chromNum, $chromColMap, $overwrite = false){
		if ( (!(is_file($dir."/".$mappingFile))) || ((is_file($dir."/chrom_".$chromNum.".ped")) && !($overwrite)) ){
			return false;
		}
		if (!($newPed = fopen($dir."/chrom_".$chromNum.".ped", "w"))){
			echo "error opening new ped file.";
		}
		//$mapping = file($dir."/".$mappingFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		//foreach ($snpFileList as $snpFile){
			//$temp = fopen($dir."/".".Chr13b.snp", "w");
			//fclose($temp);
			$IdToFileHashMap = getPersonFileMap($dir, $mappingFile);
			$DB = createInfoDB($dir, $snpFileList, $IdToFileHashMap, $chromNum ,1);
		//}
		
	//	$test = fopen($dir."/.test", "w");
	//	fwrite($dir."/".$test, $DB['EJ1']);
		
		$map = getNumToPersonMap($dir, $mappingFile);
		$oldPed = file($dir."/".$origPed, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		foreach ($oldPed as $line){
			$line = trim($line);
			$tmp = split(" ", $line);
			$data = "";
			if (isset($map[$tmp[1]])){
				$personID = $map[$tmp[1]];
				$data = $DB[$personID];         // space included on the beginning
			}
			fwrite($newPed, $line.$data."\n");
		}
		
		//fwrite($newPed, "P03001 YAY: ".$DB['P03001']);
		fclose($newPed);
	}



































?>