<?php
	require_once("config.php");
	require_once("formulas.php");
	require_once("exceptions.php");

	define("SHOW_LINES",5);
	
	// chooses marker name, chromosome, and people columns from given snp $file.
	// returns an array of the columns numbers
	// returns false in can't open the file or wrong format
	// creates a file named "SELECTION_FILE_INITIAL$file" with the selection
	function snpSelectColumns($dir, $file, &$error) {
		$error = false;
		if (!($snp = fopen($dir."/".$file,"r"))){
			//error...
			$error = "can't open snp file";
			return false;						// ERROR: can't open snp file
		}

		$prevLine = fgets($snp);				// in case there are less than 25 lines, remember last line every time.
		$prevLine = trim(preg_replace('/\s+/', "\t", $prevLine));
		if (feof($snp)){
			$error = "snp file empty";
			return false;						// if first line resulted feof - return false		ERROR: snp file empty
		}
		for ($i=0 ; $i<25 ; $i += 1){			// try getting the 25th line
			$line = fgets($snp);
			$line = trim(preg_replace('/\s+/', "\t", $line));
			if (feof($snp)) { break; }
			$prevLine = $line;
		}
		$line = $prevLine;						// use the last line which is not feof
		
		fclose($snp);
		$line = explode("\t", $line);
		if (sizeof($line) < 4){
			$error = "not enough columns in snp file";
			return false;  // wrong format - need at least: chrom, marker, position, one person		ERROR: not enough columns in snp file
		}
		//$colRegExp = "/^(([1-9])|([1-2][0-9])|(XY?)|(rs[0-9]+)|([AB][AB])|([01][01])|0|NoCall)$/";     // update for more types of columns to auto-select
		$markerSeen = false;
		$positionSeen = false;
		$chromSeen = false;
		$i = 0;
		
		$chromCol = -1;
		$markerCol = -1;
		$positionCol = -1;
		
		foreach ($line as $val) {
			$selection[$i] = 0;
/*			if (preg_match($colRegExp,$val)){
				$selection[$i] = 1;
			} */
			if (preg_match(CHROM_REGEXP,$val)){
				if (!($chromSeen)){
					$selection[$i] = 1;
					$chromCol = $i;
					$chromSeen = true;
				}
			}
			elseif (preg_match(MARKER_REGEXP,$val)){
				if (!($markerSeen)){
					$selection[$i] = 1;
					$markerCol = $i;
					$markerSeen = true;
				}
			}
			elseif (preg_match(POSITION_REGEXP,$val)){
					if (!($positionSeen)){
						$selection[$i] = 1;
						$positionCol = $i;
						$positionSeen = true;
					}
			}
			if (preg_match(CALL_REGEXP,$val)){			// might also occur if recognized as chromosome column
				$selection[$i] = 1;
			}
			$i = $i + 1;
		}
		$selection[$i] = 0;
		
		if ( ($chromCol == -1) || ($markerCol == -1) || ($positionCol == -1) ){
			$error = "";
			if ($chromCol == -1){
				$error .= "Chromosome column is missing or has the wrong format<br/>";
			}
			if ($markerCol == -1){
				$error .= "Marker ID column is missing or has the wrong format<br/>";
			}
			if ($positionCol == -1){
				$error .= "Physical Position column is missing or has the wrong format<br/>";
			}
			return false;		// ERROR: one of the following columns is missing or has the wrong format: Marker ID, Chromosome, Physical position
		}
		
		//preg_match("/(.*)\/([^\/.$]+.snp)$/",$file, $matches);
		//$filename = $matches[2];
		$f = fopen($dir."/".SELECTION_FILE_INITIAL.$file, 'w') or (print("can't open file!"));
		fwrite($f, implode(" ", $selection)."\n");
		fwrite($f, "chromCol ".$chromCol."\n");
		fwrite($f, "markerCol ".$markerCol."\n");
		fwrite($f, "positionCol ".$positionCol);
		fclose($f);
		return $selection;
	}

	// shows a snp file's content.
	// in order to see all of the columns, give only the snp file.
	// in order to see the file content and a selection headers give the snp file and "true" in the second parameter. (put inside a form to get the selection: colSelections. item structure: snpFileName\tcolNumber\ttype
	// in order to see the file content according to a selection file. (has 1 in the chosen columns places - separated by space (" ") give snp file, false, selectionFile
	function snpShowByColumns($dir, $snpFile, $showChoice = false) {
		if (!is_dir($dir)){ return false; }
		if (!($snp = fopen($dir."/".$snpFile,"r"))){
			//error...
			return false;
		}
		
		$selectionFile = $dir."/".SELECTION_FILE_INITIAL.$snpFile;
		$selFile = false;
		$selection = array();
		
		if (is_file($selectionFile)){
			$selFile = fopen($selectionFile,"r");
			
			$selection = explode(" ", trim(fgets($selFile)));     // getting selection
			$tmp = explode(" ", trim(fgets($selFile)));         // getting chromCol
			$chromCol = $tmp[1];
			$tmp = explode(" ", trim(fgets($selFile)));       // getting markerCol
			$markerCol = $tmp[1];
			$tmp = explode(" ", trim(fgets($selFile)));       // getting positionCol
			$positionCol = $tmp[1];
		}
		if ($showChoice) {
			$line = fgets($snp);
			$line = trim(preg_replace('/\s+/', "\t", $line));
			$line = explode("\t", $line);
			for ($k = 0 ; $k < sizeof($line) ; $k += 1){
				//$selection[$k] = 1;
			}
		}
		
		echo "<table border=\"1\">";
		for ($i = 0; $i < SHOW_LINES+1 ; $i += 1){
			echo "<tr>";
			if (feof($snp)){
				break;						// if EOF before the SHOW_LINES
			}
			$line = fgets($snp);
			$line = trim(preg_replace('/\s+/', "\t", $line));
			if (preg_match("/^[ \t\n\r]*$/", $line)){
				$i = $i-1;
				continue;
			}
			$line = explode("\t", $line);
			//$lineEmpty = true;
			for ($j = 0; $j < sizeof($line) ; $j += 1){
			/*	if ( ($line[$j] != "") && ($line[$j] != null) ){
					$lineEmpty = false;
				}  */
				if ($i == 0){
					//if (!$showChoice && ($selection[$j] == 1)){
					if (!$showChoice){
						if ($selection[$j] == 1){
							echo "<td>";
							switch ($j){
								case $chromCol: echo "Chrom</td>"; break;
								case $markerCol: echo "Marker ID</td>"; break;
								case $positionCol: echo "Position</td>"; break;
								default: echo "Snp</td>"; break;
							}
						}
						else {
							echo "<td style=\"background-color: #FFC0CB;\">Unused</td>";
						}
					}
					if ($showChoice){
						$personCol = $selFile && ($j != $chromCol) && ($j != $markerCol) && ($j != $positionCol) && ($selection[$j] == 1);
						?>
						<td><select name="colSelections[]" style="font-size:0.7em">
						<option <?php echo (!$selFile || ($selection[$j] == 0)) ? "selected" : "";?> value="<?php echo $j."\t"."none"."\t".$snp;?>">Non			<!-- value is:  columnNumer\tcolumnName\tsnpFileName -->
						<option <?php echo ($selFile && ($j == $markerCol)) ? "selected" : "";?> value="<?php echo $j."\t"."Marker"."\t".$snp;?>">Mrk		<!-- value is:  columnNumer\tcolumnName\tsnpFileName -->
						<option <?php echo ($selFile && ($j == $chromCol)) ? "selected" : "";?> value="<?php echo $j."\t"."Chromosome"."\t".$snp;?>">Chr	<!-- value is:  columnNumer\tcolumnName\tsnpFileName -->
						<option <?php echo ($selFile && ($j == $positionCol)) ? "selected" : "";?> value="<?php echo $j."\t"."Position"."\t".$snp;?>">Pos		<!-- value is:  columnNumer\tcolumnName\tsnpFileName -->
						<option <?php echo ($personCol ? "selected" : "");?> value="<?php echo $j."\t"."Person"."\t".$snp;?>">Snp		<!-- value is:  columnNumer\tcolumnName\tsnpFileName -->
						</select></td>
						<?php
					}
				}
				elseif (($showChoice) || ($selection[$j] == 1)){
					echo "<td>".$line[$j]."</td>";
				}
				else{
					echo "<td style=\"background-color: #FFC0CB;\">".$line[$j]."</td>";
				}
			}
			//$i -= ($lineEmpty) ? 1 : 0;
			echo "</tr>";
		}
		echo "</table>";
	}

	// array is in the form of: columnNumer\tcolumnName\tsnpFileName
	function snpManualSelection($dir, $file, $selectionArray) {
		if (!($snp = fopen($dir."/".$file,"r"))){
			//error...
			return false;
		}
		
		$line = fgets($snp);
		fclose($snp);
		$length = sizeof(explode("/t", $line));
		/* so there won't be any chance for them not to be set */
		$markerCol = DB_MARKERNAME_COL;
		$chromCol = DB_CHROM_COL;
		$positionCol = DB_POS_COL;
		/*	-	-	-	-	-	-	-	*/
		
		foreach ($selectionArray as $sel){
			$sel = explode("\t", $sel);
			$selection[$sel[0]] = (strcmp($sel[1], "none") == 0) ? 0 : 1;
			switch ($sel[1]){
				case "Marker": $markerCol = $sel[0]; break;
				case "Chromosome": $chromCol = $sel[0]; break;
				case "Position": $positionCol = $sel[0]; break;
				default: break;
			}
		}
		
		for ($i = 0 ; $i < $length ; $i += 1){
			if (!(isset($selection[$i]))){
				$selection[$i] = 0;
			}
		}
		
		$f = fopen($dir."/".SELECTION_FILE_INITIAL.$file, 'w');
		fwrite($f, implode(" ", $selection)."\n");
		fwrite($f, "chromCol ".$chromCol."\n");
		fwrite($f, "markerCol ".$markerCol."\n");
		fwrite($f, "positionCol ".$positionCol);
		fclose($f);
		return $selection;
	}
	
	function mappingExists($directory, $mapfile) {
		if (is_file($directory."/".$mapfile)) {return true;}
		return false;
	}

	// returns false on failure to open file, or file is empty.
	function getNamesLineFromSnp($dir, $snp) {
		if (!($snpFile = fopen($dir."/".$snp, "r"))) {return false;}		
		$namesLine = "";
		while (preg_match("/^[ \t\n\r]*$/", $namesLine)){   // getting first line which is not empty (hopefully contains the names)
			if (feof($snpFile)){
				return false;
			}
			$namesLine = fgets($snpFile);
			$namesLine = trim(preg_replace('/\s+/', "\t", $namesLine));
		}
		fclose($snpFile);
		return rtrim($namesLine);
	}

	// gives selection menues for mapping between numbers in second column in the $ped, to the names in the first line of each $snpList files in $directory.
	// if surrounded by <form>, selections will be in the array $mapSelections[].
	// $directory is the location of the ped and snp files
	// $ped is the ped file's name
	// $snpList is an array containing all the snp files' names for analysis
	// $mode as "all" will give all files and their name mapping.   as "one" will give only the first snp file's name mapping.
	
	/*function choose_mapping_onePedigreeOnly($directory, $ped, $snpList ,$mode = "all"){
		if (($ped == null) || ($directory == null) || ($snpList == null) || (($mode != "all") && ($mode != "one"))){
			echo "Input error in map choosing";
			return false;
		}
		$selectedValues = array();
		
		$pedLines = file($directory."/".$ped, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$amount = count($pedLines);
		$nonNameRegExp = NON_NAME_REG_EXP;
		$nameRegExp = NAME_REG_EXP;
		echo "<table>";
		echo "<tr><td>Pedigree&nbsp;&nbsp;</td>";//<td>Person&nbsp;&nbsp;</td>";
		foreach ($snpList as $snp){
			echo "<td>".$snp."&nbsp;&nbsp;</td>";
			if ($mode == "one"){
				break;
			}
		}
		
		// collecting the mapping - if exists
		$mapping = false;
		$names = false;
		if ( is_file($directory."/".MAP_FILE_NAME) && ($mFile = file($directory."/".MAP_FILE_NAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))){
			foreach ($mFile as $mapItem) {				
				if (!(preg_match("/^([1-9][0-9]*)\t(.*)\t(.*)$/", $mapItem, $trio)) && !(preg_match("/^([1-9][0-9]*)\t(.*)$/", $mapItem, $trio))){
					continue;
				}
												// the line is of the type:    pedNum	Name	    File    OR     pedNum	Name
				$mapping[trim($trio[1])] = trim($trio[2]);
			}
		}
		if ( is_file($directory."/".SIMPLIFIED_DATA_FILE) && ($sFile = file($directory."/".SIMPLIFIED_DATA_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))){
			for ($i = 0 ; $i < sizeof($sFile) ; $i += 1){
				$simplifiedItem = explode("\t", $sFile[$i]);
				if (sizeof($simplifiedItem) < 5){
					continue;
				}
				$names[] = trim($simplifiedItem[0]);
			}
		}
		
		for ($i = 0; $i < $amount ; $i += 1){
			$line = preg_split("/[ \t]/", $pedLines[$i]);
			$pedName = $line[1];
			//$pedigreeNumber = $line[0];
			if (is_file($directory."/".SIMPLIFIED_FLAG)){
				$pedName = $names[$i];
			}
			echo "<tr><td>".$pedName."</td>";       // show number from ped file line         -  add for pedigree number: <td>".$pedigreeNumber."</td>
			$emptySnp = false;
			foreach ($snpList as $snp){
				// $namesLine = file($directory."/".$snp, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);    // get array of the file lines (hopefully first line contains the names)
				$namesLine = rtrim(getNamesLineFromSnp($directory, $snp));
				if (!($namesLine)){            // if the line is empty, or file unopenable
					continue;
				}
				
				$selFile = fopen($directory."/".SELECTION_FILE_INITIAL.$snp,"r");
				$selection = split(" ", trim(fgets($selFile)));     // getting selection
				
				$namesLine = split("\t","none\t".$namesLine);
				
				$chromCol = 0; $markerCol = 0; $positionCol = 0;						// getting non-names columns
				getColsOf($directory, $snp, $chromCol, $markerCol, $positionCol);
				
				echo "<td><select name=\"mapSelections[]\" id=\"mapSelections$i\"". (($mode == "all") ? " disabled " : "") .">";
				$j = -1;
				$gotSelection = false;
				foreach ($namesLine as $name){
					$j = $j + 1;
					if ( (($j != 0) && ($selection[$j-1] == 0)) ||
						 ($j-1 == $chromCol) || ($j-1 == $markerCol) || ($j-1 == $positionCol) ){			// checks if column $j (we added "none") is selected and its not the first (none)
						continue;
					}
					if ( (!preg_match($nonNameRegExp,$name)) &&
						  (preg_match($nameRegExp,$name)) ){
						$namesList[] = $name;
						$selected = ($mapping && isset($mapping[$line[1]]) && ($mapping[$line[1]] == $name)) ? "selected" : "";
						echo "<option $selected value=\"".$line[1]."\t".$name."\t".$snp."\">".$name."&nbsp;&nbsp;";     // value is:  numInPed\tPersonID\tsnpFileName
						if ($selected == "selected"){
							$selectedValues[] = $line[1]."\t".$name."\t".$snp;
							$gotSelection = true;
						}
					}
				}
				if (!$gotSelection){
					$selectedValues[] = $line[1]."\tnone\t".$snp;
				}
				echo '</select></td>'; 
				if ($mode == "one"){
					break;
				}
			} 
			echo "</tr>";
		}
		echo "</table>";
		return $selectedValues;
	}*/
	
	function choose_mapping($directory, $ped, $snpList ,$mode = "all") {
		if (($ped == null) || ($directory == null) || ($snpList == null) || (($mode != "all") && ($mode != "one"))){
			echo "Input error in map choosing";
			return false;
		}
		$selectedValues = array();
		
		$pedLines = file($directory."/".$ped, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
		$amount = count($pedLines);
		$nonNameRegExp = NON_NAME_REG_EXP;
		$nameRegExp = NAME_REG_EXP;
		echo "<table>";
		echo "<tr><td>Pedigree&nbsp;&nbsp;</td><td>Person&nbsp;&nbsp;</td>";
		foreach ($snpList as $snp){
			echo "<td>".$snp."&nbsp;&nbsp;</td>";
			if ($mode == "one"){
				break;
			}
		}
		
		// collecting the mapping - if exists
		$mapping = false;
		$names = false;
		if ( is_file($directory."/".MAP_FILE_NAME) && ($mFile = file($directory."/".MAP_FILE_NAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))){
			foreach ($mFile as $mapItem) {				
				if (!(preg_match("/^([1-9][0-9]*\.[1-9][0-9]*)\t(.*)\t(.*)$/", $mapItem, $trio)) && !(preg_match("/^([1-9][0-9]*\.[1-9][0-9]*)\t(.*)$/", $mapItem, $trio))){
					continue;
				}
												// the line is of the type:    pedigreeNum.pedNum	Name	    File    OR     pedNum	Name
				//$mapping[trim($trio[1])] = trim($trio[2]);
				$mapping[$trio[1]] = $trio[2];
			}
		}
		$names = array();
		if (is_file($directory."/".SIMPLIFIED_PED_MAP_FILE) && ($sFile = file($directory."/".SIMPLIFIED_PED_MAP_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))){
			for ($i = 0 ; $i < sizeof($sFile) ; $i += 1){
				$simplifiedItem = explode("\t", $sFile[$i]);
				$names[$simplifiedItem[0].".".$simplifiedItem[1]] = trim($simplifiedItem[2]);		// $names[pedigree.person] = person's name
			}
		}
		
		for ($i = 0; $i < $amount ; $i += 1){
			$line = preg_split("/[ \t]/", $pedLines[$i]);
			$pedName = $line[1];
			$showName = $pedName;
			$pedigreeNumber = $line[0];
			if (is_file($directory."/".SIMPLIFIED_FLAG)){
				if (isset($names[$pedigreeNumber.".".$pedName])){
					$showName = $names[$pedigreeNumber.".".$pedName];
				}
			}
			echo "<tr><td>".$pedigreeNumber."</td><td>".$showName."</td>";       // show number from ped file line
			$emptySnp = false;
			foreach ($snpList as $snp){
				// $namesLine = file($directory."/".$snp, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);    // get array of the file lines (hopefully first line contains the names)
				$namesLine = rtrim(getNamesLineFromSnp($directory, $snp));
				if (!($namesLine)){            // if the line is empty, or file unopenable
					continue;
				}
				
				$selFile = fopen($directory."/".SELECTION_FILE_INITIAL.$snp,"r");
				$selection = explode(" ", trim(fgets($selFile)));     // getting selection
				
				$namesLine = explode("\t","none\t".$namesLine);
				
				$chromCol = 0; $markerCol = 0; $positionCol = 0;						// getting non-names columns
				getColsOf($directory, $snp, $chromCol, $markerCol, $positionCol);
				
				echo "<td><select name=\"mapSelections[]\" id=\"mapSelections$i\"". (($mode == "all") ? " disabled " : "") .">";
				$j = -1;
				$gotSelection = false;
				foreach ($namesLine as $name){
					$j = $j + 1;
					if ( (($j != 0) && ($selection[$j-1] == 0)) ||
						 ($j-1 == $chromCol) || ($j-1 == $markerCol) || ($j-1 == $positionCol) ){			// checks if column $j (we added "none") is selected and its not the first (none)
						continue;
					}
					if ( (!preg_match($nonNameRegExp,$name)) &&
						  (preg_match($nameRegExp,$name)) ){
						$namesList[] = $name;
						$selected = ($mapping && isset($mapping[$pedigreeNumber.".".$pedName]) && ($mapping[$pedigreeNumber.".".$pedName] == $name)) ? "selected" : "";
						echo "<option $selected value=\"".$pedigreeNumber.".".$pedName."\t".$name."\t".$snp."\">".$name."&nbsp;&nbsp;";     // value is:  numInPed\tPersonID\tsnpFileName
						if ($selected == "selected"){
							$selectedValues[] = $pedigreeNumber.".".$pedName."\t".$name."\t".$snp;
							$gotSelection = true;
						}
					}
				}
				if (!$gotSelection){
					$selectedValues[] = $pedigreeNumber.".".$pedName."\tnone\t".$snp;
				}
				echo '</select></td>'; 
				if ($mode == "one"){
					break;
				}
			} 
			echo "</tr>";
		}
		echo "</table>";
		return $selectedValues;
	}
	
	/* gets the $directory where the mapping should be saved.
	    $mappingArray is an array of selections of the type: number_name   so number is mapped to name.
	    several names may be mapped to the same num.
	    returns false if fails opening.
	    returns -1 if mapping file exists.
	    returns 1 on success.
	*/
	function collectMapping($directory, $mappingFile, $mappingArray, $overwite = false) {
		if ( ($directory == null) || ($mappingArray == null) ){
			return 0;
		}
		
		if ( (!($overwite)) && (is_file($directory."/".$mappingFile))){
			return -1;
		}
		
		if (!($mapFile = fopen($directory."/".$mappingFile,"w"))){
			return 0;
		}
		
		$empty = true;
		$lineRegExp = "/^([1-9][0-9]*\.[1-9][0-9]*)\t(.*)\t(.*)$/";       //  pedNumber\tID\tsnpFileName
		
		foreach ($mappingArray as $mapItem){
			if (!( preg_match($lineRegExp, $mapItem, $trio))){
				continue;
			}
			if ($trio[2] != "none"){
				fwrite($mapFile, $trio[1]."\t".$trio[2]."\t".$trio[3]."\n");
				$empty = false;
			}
		}
		
		fclose($mapFile);
		if ($empty){
			unlink($directory."/".$mappingFile);
		}
		return 1;
	}
	
	// returns false on failure
	function showMapping($directory, $mapFile) {
		if ( ($directory == null) || ($mapFile == null) || (!(is_file($directory."/".$mapFile))) ||
			  (!($mapping = file($directory."/".$mapFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			return false;
		}
		
		$lastNum = -1;
		$curr = 1;
		echo "<table>";
		foreach ($mapping as $mapItem) {
			if (!(preg_match("/^([1-9][0-9]*\.[1-9][0-9]*)\t(.*)\t(.*)$/", $mapItem, $trio)) && !(preg_match("/^([1-9][0-9]*\.[1-9][0-9]*)\t(.*)$/", $mapItem, $trio))){
				continue;
			}
											// the line is of the type:    pedNum	Name	    File    OR     pedNum	Name
			$curr = $trio[1];
			if ($lastNum == $curr){         // several names for the same number
				echo ", ".$trio[2];
			}
			else {
				echo "<tr><td>".$curr.": </td><td>".$trio[2]."</td>";
			}
			$lastNum = $trio[1];
		}
		echo "</tr></table>";
		
		return true;
	}
	
	// will return an array of mapped names
	function getTypedNames($project) {
		$dir = USER_PROJECTS_DIR."/".$project."/raw";
		$snpregexp = "/^[^\.]+\.snp$/";
		$snpFile = false;
		
		$directory = opendir($dir);
		while ( ($item = readdir($directory)) !== false ){
				if ( (strrpos($item,".") === 0) || (is_dir($item)) ){
					continue;
				}
				if (preg_match($snpregexp,$item)){
					$snpFile = $item;
				}
		}
		$snpNames = getNamesFromSnp($dir, $snpFile);
		
		$typed = array();
		if (is_dir($dir) && ($map = file($dir."/".MAP_FILE_NAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))){
			foreach ($map as $mapItem) {
				$mapItem = explode("\t", $mapItem);
				$name = $mapItem[1];
				$pedigree = explode(".", $mapItem[0]);
				$pedigree = $pedigree[0];
				if (in_array($name, $snpNames)){
					$typed[] = $pedigree.".".$name;
				}
			}			
		}
		
		return $typed;
	}
	
	// will return an array of unmapped names
	function getUntypedNames($project) {
		$dir = USER_PROJECTS_DIR."/".$project."/raw";
		$untyped = array();
		$snpregexp = "/^[^\.]+\.snp$/";
		$snpFile = false;
		
		$directory = opendir($dir);
		while ( ($item = readdir($directory)) !== false ){
				if ( (strrpos($item,".") === 0) || (is_dir($item)) ){
					continue;
				}
				if (preg_match($snpregexp,$item)){
					$snpFile = $item;
				}
		}
			
		//if (is_dir($dir) && ($simpl = file($dir."/".SIMPLIFIED_DATA_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) && $snpFile){
		if (is_dir($dir) && ($simpl = file($dir."/".SIMPLIFIED_PED_MAP_FILE, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) &&
							($map 	= file($dir."/".MAP_FILE_NAME, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES)) ){
			
			$typed = array();
			foreach ($map as $mapItem) {
				$mapItem = explode("\t", $mapItem);
				$typed[] = trim($mapItem[0]);
			}			
			
			foreach ($simpl as $mapItem) {
				$mapItem = explode("\t", $mapItem);
				if ( !in_array($mapItem[0].".".$mapItem[1], $typed) ){
					$untyped[] = $mapItem[0].".".$mapItem[2];
				}				
			}
		}
		
		return $untyped;
	}	

	function getNamesFromSnp($dir, $snp) {
		$names = array();
		if (is_file($dir."/".$snp)){
			$selFile = fopen($dir."/".SELECTION_FILE_INITIAL.$snp,"r");
			$selection = explode(" ", trim(fgets($selFile)));     // getting selection
			
			$namesLine = rtrim(getNamesLineFromSnp($dir, $snp));
			$namesLine = explode("\t",$namesLine);
			
			$chromCol = 0; $markerCol = 0; $positionCol = 0;						// getting non-names columns
			getColsOf($dir, $snp, $chromCol, $markerCol, $positionCol);
			$namesList = array();
			
			$j = -1;
			foreach ($namesLine as $name){
				$j = $j + 1;
				if ( ($selection[$j] == 0) ||
					 ($j == $chromCol) || ($j == $markerCol) || ($j == $positionCol) ){			// checks if column $j (we added "none") is selected and its not the first (none)
					continue;
				}
				if ( (!preg_match(NON_NAME_REG_EXP,$name)) &&
					  (preg_match(NAME_REG_EXP,$name)) ){
					$names[] = $name;
				}
			}
		}
		
		return $names;
	}
	
	// will use a perl code to sort the given snp file.
	function sortSnp($dir, $fileName, $sortedFileName, $initalCol, $secondaryCol) {
		if (!shell_exec("perl ".WEBDIR."/snpSort.pl $dir $fileName $sortedFileName $initalCol $secondaryCol")){
			//echo "error on sorting snp file";
			//return false;
		}
		return true;
	}
	
	// returns the pedegree number mapped to $id.
	// returns false if there's not mapping for this person, or one of the files does not exist
	function getPedNumberByID($dir, $mapFile, $id) {
		if ( (!(is_dir($dir))) || (!($map = file($dir."/".$mapFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			return false;
		}
		
		foreach ($map as $line){
			$line = explode("\t", $line);
			if ((isset($line[1])) && ($line[1] == $id)){
				return $line[0];
			}
		}
		return false;
	}
	
	// returns the pedegree number mapped to $id.
	// returns false if there's not mapping for this person, or one of the files does not exist
	function getIDsByPedNumber($dir, $mapFile, $pedNum) {
		if ( (!(is_dir($dir))) || (!($map = file($dir."/".$mapFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			return false;
		}
		$namesList = false;
		
		foreach ($map as $line){ //TODO: return $line[1]; if the if succeeds and not in the end of the loop.
			$line = explode("\t", $line);
			if ((isset($line[1])) && (strcmp($line[0],$pedNum) == 0)){
				//$namesList[] = trim($line[1]);
				$namesList[] = $line[1];
			}
		}
		return $namesList;
	}

	/*
	* finds a marker's location in $DB, by $chrom and $location. (line number)
	* returns true if it already exists, and false if it does not exist
	* $loc is passed by reference
	* so if the marker exists in $DB - it will hold the line number of the marker in the database.
	* if the marker does not exist in $DB - $loc the line number where it should be
	*/
	/*function findMarkerLocInDB(&$DB, $chrom, $position, &$loc){
		if ( ($DB == "") || ($DB == null) || (sizeof($DB) == 0) ){
			$loc = 0;
			return false;
		}
		//echo "AAAAAAAAAAAAAA<br/>";
		for($i = 0; $i < sizeof($DB) ; $i += 1){
			$loc = $i;
			//echo "Got: $chrom, seen: ".$DB[$i][1]."<br/>";
			if ( ((preg_match("/^[xX]$/", $chrom)) && !(preg_match("/^[xX]$/", $DB[$i][1]))) ||
					((int)$chrom > (int)$DB[$i][1]) ){
				continue;
			}
			if ( (!(preg_match("/^[xX]$/", $chrom)) && (preg_match("/^[xX]$/", $DB[$i][1]))) ||
				 ((int)$chrom < (int)$DB[$i][1]) ){   // just passed the place
				return false;
			}
			if ((int)$chrom == (int)$DB[$i][1]){
				if ((int)$position > (int)$DB[$i][2]){
					continue;
				}
				if ((int)$position == (int)$DB[$i][2]){
					return true;
				}
				if ((int)$position < (int)$DB[$i][2]){     // just passed the place where the marker should have been
					return false;
				}
			}
		}
		$loc = $i;
		return false;     // reached the end of the DB without finding the line
	}*/
	
	/*
	* returns 1 if first pair is bigger, -1 if second pair is bigger, and 0 if they are equal.
	* may get strings or integers
	*/
	function compareSnpLines($chrom1, $position1, $chrom2, $position2) {
		if ( ((preg_match("/^[xX]$/", $chrom1)) && !(preg_match("/^[xX]$/", $chrom2))) ||
				((int)$chrom1 > (int)$chrom2) ){
			return 1;
		}
		if ( (!(preg_match("/^[xX]$/", $chrom1)) && (preg_match("/^[xX]$/", $chrom2))) ||
			 ((int)$chrom1 < (int)$chrom2) ){   // just passed the place
			return -1;
		}
		if ((int)$chrom1 == (int)$chrom2){
			if ((int)$position1 > (int)$position2){
				return 1;
			}
			if ((int)$position1 == (int)$position2){
				return 0;
			}
			if ((int)$position1 < (int)$position2){     // just passed the place where the marker should have been
				return -1;
			}
		}
	}
	
	/*
	* returns the line position in $DB according to chrom and position.
	* line number where the line should be is in $loc.
	* returns true if line found elsewise returns false.
	*/
	function binarySearchMarkerInDB(&$DB, $chrom, $position, &$loc) {
		if ( ($DB == "") || ($DB == null) || (sizeof($DB) == 0) ){
			$loc = 0;
			return false;
		}
		$low = 0;
		$current = (int)(sizeof($DB) / 2);
		$high = sizeof($DB);
		while ($low < ($high - 1)){
			if (compareSnpLines($chrom, $position, $DB[$current][1],$DB[$current][2]) == 0){
				$loc = $current;
				return true;
			}
			if (compareSnpLines($chrom, $position, $DB[$current][1],$DB[$current][2]) == 1){
				$low = $current;
				$current = (int)(($current + $high)/2);
				continue;
			}
			if (compareSnpLines($chrom, $position, $DB[$current][1],$DB[$current][2]) == -1){
				$high = $current;
				$current = (int)(($current + $low)/2);
				continue;
			}
		}
		$loc = $current;
		if (compareSnpLines($chrom, $position, $DB[$current][1],$DB[$current][2]) == 0){				
			return true;
		}
		else {
			$loc = $high;
			return false;
		}
	}
	
	/*
	* returns in the reference parameters the column number of the markersID/chromosome/position, in the file $snp in $dir.
	* returns true upon success.
	* returns false upon failure (one of them is not found in the selection file, selection file or $dir or $snp does not exist, file unopenable...)
	*/
	function getColsOf($dir, $snpFile, &$chromCol, &$markerCol, &$positionCol) {
		if ( (!(is_dir($dir))) || (!($selFile = file($dir."/".SELECTION_FILE_INITIAL.$snpFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			return false;
		}
		if (!( isset($selFile[1]) && isset($selFile[2]) && isset($selFile[3]) )){
			return false;
		}

		$tmp = explode(" ", $selFile[1]);
		$chromCol = (int)$tmp[1];
		$tmp = explode(" ", $selFile[2]);
		$markerCol = (int)$tmp[1];
		$tmp = explode(" ", $selFile[3]);
		$positionCol = (int)$tmp[1];
		
		return true;
	}
	
	// returns an array of the selection. the array[i] is true if the i column selected and false otherwise.
	function getSelection($dir, $snpFile) {
		if ( (!(is_dir($dir))) || (!($selFile = file($dir."/".SELECTION_FILE_INITIAL.$snpFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			return false;
		}
		if (!( isset($selFile[0])) ){
			return false;
		}
		
		$selection = explode(" ", rtrim($selFile[0]));
		return $selection;
	}
	
	// $line1/2 are not splitted
	function areLinesSimilar($line1, $line2, &$updatedLine, &$conflict) {
		$conflict = false;
		$line1 = explode("\t", rtrim($line1));
		$line2 = explode("\t", rtrim($line2));
		
		if ( ($line1[DB_CHROM_COL] == $line2[DB_CHROM_COL]) && ($line1[DB_POS_COL] == $line2[DB_POS_COL]) ){
			for ($i = DB_POS_COL+1 ; $i < sizeof($line1) ; $i += 1){
				if ( (preg_match(MATCHES_FOR_00, $line1[$i])) && (!(preg_match(MATCHES_FOR_00, $line2[$i]))) ){
					$updatedLine = implode("\t", $line2);
					return true;		// similar
				}
				if ( (!(preg_match(MATCHES_FOR_00, $line1[$i]))) && (preg_match(MATCHES_FOR_00, $line2[$i])) ){
					$updatedLine = implode("\t", $line1);
					return true;		// similar
				}
				if ( ( (preg_match(MATCHES_FOR_11, $line1[$i])) && (!(preg_match(MATCHES_FOR_11, $line2[$i]))) ) ||
					 ( (preg_match(MATCHES_FOR_12, $line1[$i])) && (!(preg_match(MATCHES_FOR_12, $line2[$i]))) ) ||
					 ( (preg_match(MATCHES_FOR_21, $line1[$i])) && (!(preg_match(MATCHES_FOR_21, $line2[$i]))) ) ||
					 ( (preg_match(MATCHES_FOR_22, $line1[$i])) && (!(preg_match(MATCHES_FOR_22, $line2[$i]))) ) ) {
					$conflict = true;
					return false; 		// not similar - with conflict
				}
			}
			$updatedLine = implode("\t", $line1);
			return true;				// totally similar
		}
		return false;	// not similar - ok
	}
	
	// return false on failure to open dir, or given file
	// returns false when 2 lines with the same chrom number and position had confliction information
	function removeSnpDoubleLines($dir, $sortedSnp) {
		if ( (!(is_dir($dir))) || (!is_file($dir."/".$sortedSnp)) ){
			log_error("$dir or $sortedSnp does not exist.");
			return false;
		}
		
		if (!($snpFile = fopen($dir."/".$sortedSnp,"r"))) {            // inf not sorting files
			log_error("can't open ".$dir."/".$sortedSnp." file.");
			return false;     // can't open ped file
		}
		
		$finalFileName = SORTED_SNP_INITIAL."final_".$sortedSnp;
		$final = fopen($dir."/".$finalFileName, "w");
		
		$same = false;
		$updatedLine;
		$conflict = false;
		$forbidden = array();		// markers with conflicts
		
		$line1 = fgets($snpFile);	//Read first line
		$line1 = trim(preg_replace('/\s+/', "\t", $line1));
		while (!feof($snpFile)) {
			$line2 = fgets($snpFile);
			$line2 = trim(preg_replace('/\s+/', "\t", $line2));
			$line2split = explode("\t", rtrim($line2));
			if (!isSet($line2split[DB_CHROM_COL]) || !isSet($line2split[DB_POS_COL])) {continue;}
			if ( in_array($line2split[DB_CHROM_COL]."-".$line2split[DB_POS_COL], $forbidden) ) {continue;}	# known conflict, ignore line
			
			if (areLinesSimilar($line1, $line2, $updatedLine, $conflict)) {$line1 = $updatedLine;}							
			else {
				if ($conflict) {
					$line1split = explode("\t", rtrim($line1));
					log_error("conflict information on: $dir / $sortedSnp . in chromosome: ".$line1split[DB_CHROM_COL]." on marker position: ".$line1split[DB_POS_COL]);
					$forbidden[] = $line1split[DB_CHROM_COL]."-".$line1split[DB_POS_COL];
					continue; // ignore the conflicting line
				}
				else {
					$line1 = rtrim($line1);
					fwrite($final, $line1."\n");
					$line1 = $line2;
				}
			}			
		}
		
		fwrite($final, $line1."\n");
		
		fclose($final);
		return SORTED_SNP_INITIAL."final_".$sortedSnp;
	}
	
	/* creates a full data base, from all snp's in $snpFileList. sorted initially by chromosome and secondary by position.
	* make sure that selection files for each snp file in the list exists in $dir.
	* make sure that $mapFile exists in $dir.
	* the DB returned's structure:
	* DB[i][0] - markers
	* DB[i][DB_CHROM_COL] - chromosome
	* DB[i][DB_POS_COL] - marker's position on the chromosome
	* DB[i][j] - for j>DB_POS_COL: people's dna calls   j = pedegreeNum +DB_POS_COL
	*
	* returns false on failure (log file updated)
	* returns the DB file name if successful
	*/
	function createDB($dir, $snpFileList, $pedFile, $mapFile, $performFiltering = true) {
		if ( (!(is_dir($dir))) || (!($map = file($dir."/".$mapFile))) || (!($ped = file($dir."/".$pedFile, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES))) ){
			log_error("can't open directory/mapFile/pedFile, $dir / $mapFile / $ped");
			return false;
		}
		
		$pplAmount = sizeof($ped);
		for($k = 0 ; $k < $pplAmount ; $k += 1){
			$pedLine = preg_split("/[ \t]/",$ped[$k]);
			$pedNum = $pedLine[0].".".$pedLine[1];
			$nameList = getIDsByPedNumber($dir, $mapFile, $pedNum);
			$pedNumToNamesMap[$pedNum] = $nameList;
			//log_error("ped number: ".$pedNum." set to: ".implode(", ",$pedNumToNamesMap[$pedNum]));
		}
		
		/* temp unsorted database */
		$DB = fopen($dir."/.unsortedDB.snp","w");        
		/***********/
		
		foreach ($snpFileList as $snp) {
			$snp = str_replace("\*", " ", $snp);
			if (!(is_file($dir."/".$snp))) {
				log_error("file ".$dir."/".$snp." does not exist.");
				fclose($DB);
				return false;    // snp file does not exist
			}
			if (!(getColsOf($dir, $snp, $chromCol, $markerCol, $positionCol))) {
				log_error("error in selection file.");
				fclose($DB);
				return false;   // error in selection file
			}
			
			//if (!($snpFile = fopen($dir."/".SORTED_SNP_INITIAL.$snp,"r"))){               // if sorting before calling this function
			if (!($snpFile = fopen($dir."/".$snp,"r"))){            // inf not sorting files
				log_error("can't open ".$dir."/".$snp." file.");
				fclose($DB);
				return false;     // can't open ped file
			}
			
			if (!($selection = getSelection($dir, $snp))) {            // inf not sorting files
				log_error("can't open ped file.");
				fclose($DB);
				return false;     // can't open ped file
			}
			
			if (!($names = getNamesLineFromSnp($dir, $snp))) {
				log_error("can't open $snp file.");
				fclose($DB);
				return false;	// can't open $snp or its empty
			}
			$names = explode("\t", $names);
			
			for($k = 0 ; $k < sizeof($names) ; $k += 1) {
				$ID = $names[$k];
				if ( ($selection[$k]) && (!preg_match(NON_NAME_REG_EXP,$ID)) && (preg_match(NAME_REG_EXP,$ID)) &&
					 ($k != $markerCol) && ($k != $chromCol) && ($k != $positionCol) && (getPedNumberByID($dir, $mapFile, $ID)) ){
					$pedNum = getPedNumberByID($dir, $mapFile, $ID);
					$nameToPedMap[$ID] = $pedNum;   //Omer - this seems unused....
					//log_error("ID: ".$ID." set to: ".$nameToPedMap[$ID]);
				}
			}
            
            
            #Omer 7/8/2011 - new code for creating the #DB file
            #Choose the order of individuals to be written to the file            
            $snpPeople = array();
            for($k = 0 ; $k < sizeof($ped) ; $k += 1) {
                $pedLine = preg_split("/[ \t]/",$ped[$k]);
                $pedNum = $pedLine[0].".".$pedLine[1];
				if (!$pedNumToNamesMap[$pedNum]) {
					$snpPeople[] = -1;
					continue;
				}
				for ($j = 0 ; $j < sizeof($names) ; $j += 1){
					if (!in_array($names[$j], $pedNumToNamesMap[$pedNum]) && ($selection[$j])) {continue;}	// not found the col number of the ped person in the snp file
					$snpPeople[] = $j;
					break;
				}
            }
            
            #Create the file
			$markerNamesArr = Array();
            while ((!feof($snpFile))) {
				$Line = fgets($snpFile);
				$Line = trim(preg_replace('/\s+/', "\t", $Line));
				$Line = explode("\t", rtrim($Line));
				if (!isSet($Line[$chromCol]) || (!preg_match(CHROM_REGEXP, $Line[$chromCol])) || ($Line[$markerCol] == "") ){  // find the first line which seems to have needed info
					continue;
				}
				$markerName = trim($Line[$markerCol]);
				$markerNamesArr[] = $markerName;
                $newLine = $markerName;
                $newLine .= ("\t".trim($Line[$chromCol]));
                $newLine .= ("\t".trim($Line[$positionCol]));
                foreach ($snpPeople as $personPos) {
                    if ($personPos == -1) {$newLine.= "\t0";}
                    else {$newLine.= ("\t".$Line[$personPos]);}
                }
                fwrite($DB, $newLine."\n");
            }
            fclose($snpFile);
        }
		
		#Check that there are no duplicate marker names
		$uniqueNamesArr = array_unique($markerNamesArr);
		if (count($uniqueNamesArr) != count($markerNamesArr)) {
			$diffkeys   = array_diff_key($markerNamesArr, $uniqueNamesArr);
			$duplicates = array_unique($diffkeys);
			$errorMessage = "The following marker names appear more than once in the file: ";
			$errorMessage .= implode(", ", $duplicates);
			throw new InputException($errorMessage);
		}
		

		
		fclose($DB);        				
		sortSnp($dir, ".unsortedDB.snp", SORTED_SNP_DB_FILENAME, DB_CHROM_COL, DB_POS_COL);   // chromCol = 1,   positionCol = 2		
		if (is_file($dir."/".SORTED_SNP_DB_FILENAME)){
			unlink($dir."/.unsortedDB.snp");
		}		
		$finalDB = removeSnpDoubleLines($dir, SORTED_SNP_DB_FILENAME);		
		if ($finalDB == false) {
			log_error("remove double lines in ".SORTED_SNP_DB_FILENAME." failed");
			return false;
		}
		
		log_error("createDB from $dir/".SORTED_SNP_DB_FILENAME." completed successfully.");		
		if ($performFiltering) {
			$filteredFinalDB = filterSnp($dir, $finalDB, 10);
			if ($filteredFinalDB){
				if ($filteredFinalDB != $finalDB){
					log_error("filterSnp on $filteredFinalDB completed successfully.");
					$finalDB = $filteredFinalDB;
				}
			}
			else {log_error("filterSnp on $finalDB failed. Will work on non-filtered DB.");}
		}		
		return $finalDB;
	}
	
	
	
	// filters markers from an Snp file which their distance is lower than given $cM
	function filterSnp($dir, $DBfile, $cM) {
		if ( (!(is_dir($dir))) || (!is_file($dir."/".$DBfile)) ) {
			log_error("$dir or $DBfile does not exist.");
			return false;
		}
		
		if (!($snpOrigFile = fopen($dir."/".$DBfile,"r"))) {            // inf not sorting files
			log_error("can't open ".$dir."/".$DBfile." file.");
			return false;     // can't open ped file
		}
		
		exec("awk '{print \$2}' $dir/$DBfile | uniq", $chrom_vals);		
		$chroms_part = array_map("getChromosomePartOfDNA", $chrom_vals);
		
		$max_markers = 0;
		
		for ($i = 0; $i < sizeof($chrom_vals); $i += 1) {
			$max_markers += $chroms_part[$i];
		}
		
		$max_markers = floor(($max_markers/100)*MAX_MARKERS_IN_SNP);
		
		$linesCount = exec("cat $dir/$DBfile | wc -l");
		if ($linesCount <= $max_markers) { return $DBfile; }
		
		$filteredFile = "filtered_".$DBfile;
		$filtered = fopen($dir."/".$filteredFile,"w");
		$debug = fopen($dir."/.debug","w");
		//$linesCount = count(file($dir."/".$DBfile));
		fwrite($debug, "***** max_markers = ".$max_markers."****\n");
		
		$start = 0;
		$currCM = 0;
		$currChrom = 1;
		$line = fgets($snpOrigFile);        
		$line = trim(preg_replace('/\s+/', "\t", $line));
		$line = explode("\t", $line);
		while (!feof($snpOrigFile)) {
			$usedLines = array();
			fwrite($debug, "***** in while ******\n".$currCM." ".$start." ".$cM." ".$currChrom);            
            if (preg_match("/^[ \t]*\n$/", implode("\t",$line))) {
                $line = fgets($snpOrigFile);
                $line = trim(preg_replace('/\s+/', "\t", $line));
                $line = explode("\t", $line);
                continue;
            }			
			fwrite($debug, "#### Line is ####".$line[DB_CHROM_COL]." ".$line[DB_POS_COL]."\n");
			while ((!feof($snpOrigFile)) && ($currCM - $start < $cM) && ($line[DB_CHROM_COL] == $currChrom)) {
				$currCM = $line[DB_POS_COL]/1000000;
				fwrite($debug, $currCM." ".$start." ".$cM." ".$currChrom);
				//echo "place: ".$currCM." ".$start." ".$cM." ".$currChrom." ".sizeof($usedLines)."<br/>";
				$usedLines[] = $line;
				$line = fgets($snpOrigFile);
				$line = trim(preg_replace('/\s+/', "\t", $line));
				if (sizeof($usedLines) > 400) {$line = explode("\t", $line); break;}
				if (preg_match("/^[ \t]*$/", $line)) {
					$line = fgets($snpOrigFile);
					$line = trim(preg_replace('/\s+/', "\t", $line));
                    $line = explode("\t", $line);
					continue;
				}
				$line = explode("\t", $line);
			}
			//echo "chrom: ".$currChrom;
			if (isset($line[DB_POS_COL])) {$start = $line[DB_POS_COL]/1000000;}							
			else {continue;}
			fwrite($debug, "\n");
			if (count($usedLines) > 0) {
				fwrite($debug,count($usedLines)." ".$linesCount."\n");
				$chosen = ChooseIndexes(count($usedLines), $linesCount, $max_markers);
				fwrite($debug, "\n********** CHOSEN SIZE is ".sizeof($chosen)." ***********\n");
				for ($i = 0; $i < sizeof($chosen); $i += 1) {
					fwrite($debug, $chosen[$i]." ");					
					if ($chosen[$i] == 1) {						
                        fwrite($filtered, implode("\t", $usedLines[$i])."\n");						
					}
				}
				fwrite($debug, "\n*********************\n");
			}
			
			// Init variables for next window
			//$currCM = 0;
			if ( (!feof($snpOrigFile)) && ($line[DB_CHROM_COL] != $currChrom) ){
				$currChrom = $line[DB_CHROM_COL];
				$currCM = $start;
			}
			unset($usedLines);
		}
		
		fclose($filtered);
		fclose($debug);
		
		return "filtered_".$DBfile;
	}
	
	function getSnpLineNumbersOf($dir, $snpFile, $fromMarker, $toMarker, &$from, &$to) {
		if (!is_file($dir."/".$snpFile)) { return false; }
		$line = "";
		exec("grep -n \"".$fromMarker."[^0-9]\" $dir/$snpFile", $line);
		$line = $line[0];
		$line = explode(":", $line);
		if (!isset($line[1])) { return false; }
		$from = $line[0];
		unset($line);
		
		exec("grep -n \"".$toMarker."[^0-9]\" $dir/$snpFile", $line);
		$line = $line[0];
		$line = explode(":", $line);
		if (!isset($line[1])) { return false; }
		$to = $line[0];
	}

    //We assume that the snpFile is already sorted
    function getSnpPart($dir, $snpFile, $chrom, $start, $end, $rangeType, $targetDir, $partFile) {
        
        if ($rangeType != "positions" and $rangeType != "strings") {return false;} //invalid rangeType
        if (!($inputFile = fopen($dir."/".$snpFile,"r"))){return false;}	// ERROR: can't open snp file
        if (!($outputFile = fopen($targetDir."/".$partFile,"w"))){return false;}	// ERROR: can't open partial SNP file
		
        $inRange = false;        
        while (!feof($inputFile)){
            $Line = fgets($inputFile);
			$Line = trim(preg_replace('/\s+/', "\t", $Line));
			if ($Line == '') {continue;}	//Ignore empty lines
			
			$LineArray = explode("\t", rtrim($Line));
            $markerName = $LineArray[DB_MARKERNAME_COL];
            $markerChrom = $LineArray[DB_CHROM_COL];
            $markerPos = $LineArray[DB_POS_COL];
            
            if (!$inRange) {
                if ($markerChrom != $chrom) {continue;}                
                if (($rangeType == "positions" and $markerPos >= $start) or 
					(($rangeType == "strings" and $markerName == $start))) {
                    $inRange = true;
                }
            }
            if ($inRange) {
                if ($markerChrom != $chrom) {break;}
                if ($rangeType == "positions" and $markerPos > $end) {break;}
                fwrite($outputFile, $Line."\n");
                if ($rangeType == "strings" and $markerName == $end) {break;}
            }            
        }
        
        fclose($inputFile);
        fclose($outputFile);
        return true;
    }
?>
